/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
#include "dpaa.h"
#include "services/data_pool_mng_ext.h"
#include <sys/mman.h>
#include <fcntl.h>
#include "e500v2_ext.h"

uint32_t func_depth_count=0; /*This is used for logging*/
static dpaa_system_t *dpaa=NULL;

/*Define our global log level variable here*/
uint32_t dpaa_log_level=LOG_NORMAL;

/*  Configuration Function Table (CFT), a list of pointers to callback handler function */
/*This list is arranged exactly in order of enumeration of configuration codes(see dpaa_exp.h). 
 * Please donot change the order here without modifying the codes, else behavior would be quite 
 * surprising*/
static int (* cft[DPAA_CFG_CODE_MAX-DPAA_CFG_CODE_MIN-1])(dpaa_system_t *dpaa, dpaa_cfg_t *cfg) = {\
        dpaa_conf_init_qman_swportal,   /*DPAA_CFG_CODE_INIT_QMAN_SWPORTAL*/
        dpaa_conf_init_fqid,            /*DPAA_CFG_CODE_INIT_FQ*/
        dpaa_conf_init_bman_swportal,   /*DPAA_CFG_CODE_INIT_BMAN_SWPORTAL*/
        dpaa_conf_init_bpool,            /*DPAA_CFG_CODE_INIT_BPID*/
        dpaa_conf_config_port           /*DPAA_CFG_CODE_CONFIG_PORT*/
    };

/*  Runtime Function Table (RFT), a list of pointers to Runtime callback handler function */
/*This list is arranged exactly in order of enumeration of runtime codes(see dpaa_exp.h). 
 * Please donot change the order here without modifying the codes, else behavior would be quite 
 * surprising*/
static int (* rft[DPAA_RUNTIME_CODE_MAX-DPAA_RUNTIME_CODE_MIN-1])(dpaa_system_t *dpaa, dpaa_runtime_t *rtm) = {\
        dpaa_runtime_enable_port,       /*DPAA_RUNTIME_CODE_ENABLE_PORT*/
        dpaa_runtime_disable_port,      /*DPAA_RUNTIME_CODE_DISABLE_PORT*/
        dpaa_runtime_handler,           /*DPAA_RUNTIME_CODE_ATTACH_IRQ_QMSWPORTAL*/
        dpaa_runtime_handler,           /*DPAA_RUNTIME_CODE_ATTACH_IRQ_BMSWPORTAL*/
        dpaa_runtime_handler,           /*DPAA_RUNTIME_CODE_DETACH_IRQ_QMSWPORTAL*/
        dpaa_runtime_handler,           /*DPAA_RUNTIME_CODE_DETACH_IRQ_BMSWPORTAL*/
        dpaa_runtime_queue_enqueue,      /*DPAA_RUNTIME_CODE_FAST_ENQUEUE*/
        dpaa_runtime_queue_dequeue,      /*DPAA_RUNTIME_CODE_FAST_DEQUEUE*/
        dpaa_runtime_buf_manage,       	/*DPAA_RUNTIME_CODE_BUF_RELEASE*/
        dpaa_runtime_buf_manage,        /*DPAA_RUNTIME_CODE_BUF_ACQUIRE*/
        dpaa_runtime_buf_manage,        /*DPAA_RUNTIME_CODE_BUF_FLUSH*/
        dpaa_runtime_clear_irq,         /*DPAA_RUNTIME_CODE_CLEAR_BMAN_IRQ_CAUSE*/
        dpaa_runtime_clear_irq,         /*DPAA_RUNTIME_CODE_CLEAR_QMAN_IRQ_CAUSE*/
        dpaa_runtime_clear_irq,         /*DPAA_RUNTIME_CODE_CLEAR_FMAN_IRQ_CAUSE*/
        dpaa_runtime_mac_config,        /*DPAA_RUNTIME_CODE_SET_MAC_CONFIG*/
        dpaa_runtime_get_mac_stats,     /*DPAA_RUNTIME_CODE_GET_MAC_STATS*/
		dpaa_runtime_mii_read_write,	/*DPAA_RUNTIME_CODE_MII_READ */
		dpaa_runtime_mii_read_write,	/*DPAA_RUNTIME_CODE_MII_WRITE */
		dpaa_runtime_pcd_handle,		/*DPAA_RUNTIME_CODE_ENABLE_PCD */
		dpaa_runtime_pcd_handle,		/*DPAA_RUNTIME_CODE_DISABLE_PCD*/
		dpaa_runtime_dump_regs,			/*DPAA_RUNTIME_CODE_DUMP_REGS */
		dpaa_runtime_set_promiscuous,     /*DPAA_RUNTIME_CODE_SET_PROMISCUOUS*/
		dpaa_runtime_add_hashmacaddr,     /*DPAA_RUNTIME_CODE_ADD_HASHMACADDR*/
		dpaa_runtime_del_hashmacaddr,     /*DPAA_RUNTIME_CODE_DEL_HASHMACADDR*/
    };

/*Currently handles map to system object directly, but in future, we may 
 * maintain an array here which basically maps several handles into a single 
 * system object multiple handles to mutiple objects.*/
#define MAP_HANDLE_TO_SYSPTR(token)   ((dpaa_system_t *)token) 
#define MAP_SYSPTR_TO_HANDLE(token)   ((dpaa_handle_t )token) 
static  sem_t *master_sem;
static  sem_t *portal_sem[DPAA_MAX_SW_PORTAL];

/*****************************************************************************/
/*  Function Name   : dpaa_system_init
 *  Input           : DPAA configuration parameters as filled by caller 
 *  Description     : This function would initialize entire DPAA system. Each
 *                    Application using DPAA services needs to call this.
 *  Returns         : A valid DPAA handle to use further for making dpaa_* calls.
 *  Note            : 
 */
/*****************************************************************************/
dpaa_handle_t 
dpaa_system_init(const dpaa_startup_params_t *dpaa_cfg){

    int rc=-1;
    int i=0;
    //int count=0;
    int masterMode=FALSE;
    //char sem_name[50];

    /*We do change our log level only if its above Normal*/
    if(dpaa_cfg->verbose>=LOG_NORMAL)
        dpaa_log_level= dpaa_cfg->verbose;
    /*Above statement needs to be there before Function entry log, because its value
     * decides whether entry log is going to be printed or not :-)*/
    LOG_FUNCTION_ENTRY;

    for(i=0;i<DPAA_MAX_SW_PORTAL;i++){
        portal_sem[i]=NULL;
    }

    /*First step is to initialize and allocate the memory for our base pointer dpaa*/
    /*Only a single instance of DPAA can exist in multithreaded environment*/
    if(dpaa!=NULL){
        /*Oops, someone has been here before us ?*/
        /*Throw an error and simply return with error*/
        DPAA_LOG(LOG_RUNTIME_FAULT, "DPAA system already initialized in multithreaded environment!");

        LOG_FUNCTION_EXIT;
        /*Return existing pointer here and see if user is happy*/
        return MAP_SYSPTR_TO_HANDLE(dpaa);
    }

    if((dpaa = malloc(sizeof(dpaa_system_t)))== NULL){
        DPAA_LOG(LOG_CRIT_ERROR, "Can not allocate memory to dpaa_system_t");
        dpaa=NULL;
        rc=-1;
        goto exit_cleanup;
    }
    /*Fill in dpaa_system_t with passed parameters*/

    /*Check Configuration function table size*/
    DPAA_ASSERT( (sizeof(cft)/sizeof(cft[0]))== (DPAA_CFG_CODE_MAX-DPAA_CFG_CODE_MIN-1));

    /*Check Runtime function table size*/
    DPAA_ASSERT( sizeof(rft)/sizeof(rft[0])== (DPAA_RUNTIME_CODE_MAX-DPAA_RUNTIME_CODE_MIN-1));

    /*Try taking the named semaphore to check presence of master configuration*/
    master_sem = 
    sem_open(DPAA_SYSTEM_MASTER_SEM_NAME ,O_CREAT | O_EXCL, DPAA_SEM_DEF_PERMISSION, DPAA_SYSTEM_MASTER_SEM_INIT_VAL );
    if(master_sem==SEM_FAILED){
        if(errno==EEXIST){
            masterMode=FALSE;   
            DPAA_LOG(LOG_DEBUG_CONFIG,"Master configuration found, attaching in runtime mode !");
        }/*End of handling of existing Master configuration*/
        else{
                DPAA_LOG(LOG_SYS_FAILURE, "Master semaphore is non existent and can not be created exclusively");
                master_sem=NULL;
                goto exit_cleanup;
        }
    }/*End of Master semaphore exlusive open failure handling*/
    else{
        masterMode=TRUE;    
        DPAA_LOG(LOG_NORMAL, "No previous master configuration exists, attaching in Master mode");
    }
    

    dpaa->imEn= dpaa_cfg->imode;
    dpaa->masterMode= masterMode;

    dpaa->master_sem= master_sem;

    
    if(!masterMode){
        DPAA_LOG(LOG_DEBUG_CONFIG, "Skipping master mode intialization");
        goto exit_ok;
    }
#if 0
	E500_DCacheFlush();
	E500_DCacheDisable();
	E500_DCacheFlush();
	CORE_MemoryBarrier();
#endif
	if((dpaa->h_Platform = PLATFORM_Init(dpaa_cfg->immr, dpaa_cfg->bman_portal_base, dpaa_cfg->qman_portal_base, dpaa_cfg->pixis_base))==NULL){
         DPAA_LOG(LOG_CRIT_ERROR, "Can not initialize platform");
         goto exit_err;

	}


    /*One time Intialization of the All components manager*/
    if(bman_init(dpaa,dpaa_cfg)==0)  {
        if( qman_init(dpaa,dpaa_cfg)==0){
            if(fman_init(dpaa,dpaa_cfg)==0){
                goto exit_ok;
            }
            else{
                DPAA_LOG(LOG_CRIT_ERROR, "fman_init() failed ");
            }
        }
        else{
            DPAA_LOG(LOG_CRIT_ERROR, "qman_init() Failed") ;
        }
    }
    else{
        DPAA_LOG(LOG_CRIT_ERROR, "bman_init() Failed");
    }

exit_cleanup:
    if(dpaa!=NULL)
    dpaa_system_shutdown(MAP_SYSPTR_TO_HANDLE(dpaa), 0) ;

exit_err:
	dpaa=NULL;

exit_ok:
    
    LOG_FUNCTION_EXIT;
    return MAP_SYSPTR_TO_HANDLE(dpaa); 

    
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
int 
dpaa_system_config(dpaa_handle_t hdl, dpaa_cfg_t *cfg){
    int rc=0;
    LOG_FUNCTION_ENTRY;

    DPAA_ASSERT((cfg->code > DPAA_CFG_CODE_MIN) && (cfg->code < DPAA_CFG_CODE_MAX));
    rc=cft[cfg->code - DPAA_CFG_CODE_MIN-1]((MAP_HANDLE_TO_SYSPTR(hdl)),cfg);
    
    LOG_FUNCTION_EXIT;
    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
int 
dpaa_system_runtime(dpaa_handle_t hdl, dpaa_runtime_t *rtm) {
    int rc=0;
//    LOG_FUNCTION_ENTRY;
	
    DPAA_ASSERT((rtm->code > DPAA_RUNTIME_CODE_MIN) && (rtm->code < DPAA_RUNTIME_CODE_MAX));
    rc=rft[rtm->code - DPAA_RUNTIME_CODE_MIN-1]((MAP_HANDLE_TO_SYSPTR(hdl)),rtm);
    
//    LOG_FUNCTION_EXIT;
    return rc;

}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
int 
dpaa_system_shutdown(dpaa_handle_t hdl, int mode) {
    dpaa_system_t * dpaa = MAP_HANDLE_TO_SYSPTR(hdl);
    
    LOG_FUNCTION_ENTRY;

    /*If master is dying, it should delete the master semaphore so that a new master 
     * configuration can be applied*/
    if(dpaa->masterMode){
        sem_close(dpaa->master_sem);
        sem_unlink(DPAA_SYSTEM_MASTER_SEM_NAME);
    }

    free(dpaa);

    dpaa=NULL;
    
    LOG_FUNCTION_EXIT;

    return 0;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
int 
dpaa_system_ioctl(dpaa_handle_t h_dpaa, dpaa_ioctl_action_t* action){
	dpaa_system_t *dpaa = (dpaa_system_t *)h_dpaa;
	int error = EOPNOTSUPP;
    LOG_FUNCTION_ENTRY;

	switch(action->code) {
    case SIOCDPAAQM: {
		error = 0;
		action->value = qman_get_counter(dpaa->qman, action->idx);
		break;
		}
    case SIOCDPAABM: {
		error = 0;
		action->value = bman_get_counter(dpaa->bman, action->idx);
		break;
		}
    case SIOCDPAABMPOOL: {
		error = 0;
		action->value = bman_pool_get_counter(dpaa->bman, action->inst, action->idx);
		break;
		}
    case SIOCDPAAFM: {
		error = 0;
		action->value = fman_get_counter(dpaa->fman, action->iface/5, action->idx);
		break;
		}
    case SIOCDPAAFMPORT: {
		error = 0;
		action->value = fman_port_get_counter(dpaa->fman, action->iface, action->inst, action->idx);
		break;
		}
    case SIOCDPAAFMPCD: {
		error = 0;
		action->value = fman_pcd_get_counter(dpaa->fman, action->iface, action->inst, action->idx);
		break;
		}
    case SIOCDPAAQRYFQ_P: {
		if(qman_query_fq_p(dpaa->qman, action->iface, action->value, action->nresbuf, action->resbuf) == E_OK)
			error = 0;
		else
			error = ENOENT;
		break;
		}
    case SIOCDPAAQRYFQ_NP: {
		if(qman_query_fq_np(dpaa->qman, action->iface, action->value, action->nresbuf, action->resbuf) == E_OK)
			error = 0;
		else
			error = ENOENT;
		break;
		}
    case SIOCDPAAQRYWQ: {
		if(qman_query_wq(dpaa->qman, action->iface, action->value, action->nresbuf, action->resbuf) == E_OK)
			error = 0;
		else
			error = ENOENT;
		break;
		}
	}
    LOG_FUNCTION_EXIT;
    return error;
}


/*****************************************************************************/
/*  Function Name   : vtophys
 *  Input           : Virtual address in 32 bits
 *  Description     : converts 32 bit vaddr to 64 bit PADDR 
 *  Returns         : PADDR if successful, -1 otherwise. 
 *  Note            : Physical Addresses in Corenet are 48bit long. Upper 16 
 *                    Bits are mostly ZERO but not guaranteed. 
 */
/*****************************************************************************/
uint64_t 
dpaa_vtophys (void *addr) {
    off64_t  offset;

    if (mem_offset64 (addr, NOFD, 1, &offset, 0) == -1) {
        return (-1);
    }
    return (offset);
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_conf_init_bman_swportal (dpaa_system_t *dpaa, dpaa_cfg_t *cfg){
    
    init_bman_portal_t *req=NULL;
    init_bman_portal_resp_t *resp=NULL;
    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

    req= &cfg->init_bman_portal;
    resp= &cfg->init_bman_portal_resp;
    
    if((rc=bman_portal_init(dpaa->bman,req, resp))!=EOK){
		DPAA_LOG_CRIT_ERROR("%s(): Cannot initialize BMAN Portal", __FUNCTION__);
    }
    resp->result= rc;
    LOG_FUNCTION_EXIT;
    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_conf_init_qman_swportal (dpaa_system_t *dpaa, dpaa_cfg_t *cfg){
    
    init_qman_portal_t *req=NULL;
    init_qman_portal_resp_t *resp=NULL;
    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

    req= &cfg->init_qman_portal;
    resp= &cfg->init_qman_portal_resp;

    DPAA_ASSERT(req->portal <DPAA_MAX_SW_PORTAL);

    if((rc=qman_portal_init(dpaa->qman,req, resp))!=EOK){
		DPAA_LOG_CRIT_ERROR("%s(): Cannot initialize QMAN Portal", __FUNCTION__);
    }
    resp->result= rc;
    LOG_FUNCTION_EXIT;
    return rc;
}


static int 
dpaa_conf_init_fqid (dpaa_system_t *dpaa, dpaa_cfg_t *cfg){

    init_fq_t *req=NULL;
    init_fq_resp_t *resp=NULL;
    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

    req= &cfg->init_fq;
    resp= &cfg->init_fq_resp;

    DPAA_ASSERT(req->portal <DPAA_MAX_SW_PORTAL);

    if(req->channel==CHANNELID_DEFAULT){
        /*Application asked us to assign default channel.
         * Allocate a Dedicated channel for this portal*/
        req->channel= MAP_PORTAL_TO_CHID(req->portal);  
    }
    if((rc=qman_portal_init_fqid(dpaa->qman,req, resp))!=EOK){
        DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot initialize FQID inside QMAN portal ", __FUNCTION__);
        resp->fqid= FQID_INVALID;
        resp->chid= CHANNELID_INVALID;
		resp->handle=NULL;

    }
    resp->result= rc;
    LOG_FUNCTION_EXIT;
    return rc;

}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_conf_init_bpool (dpaa_system_t *dpaa, dpaa_cfg_t *cfg){
    
    init_bpool_t *req=NULL;
    init_bpool_resp_t *resp=NULL;
    int32_t rc=0;

    req= &cfg->init_bpool;
    resp= &cfg->init_bpool_resp;

    DPAA_ASSERT(req->portal <DPAA_MAX_SW_PORTAL);
    if((rc=bman_bpool_init(dpaa->bman,req, resp))!=EOK){
        DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot initialize Buffer Pool ", __FUNCTION__);
    }
    resp->result= rc;
    return rc;
}


/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_conf_config_port (dpaa_system_t *dpaa, dpaa_cfg_t *cfg){
    uint32_t portal_id=0;
    int32_t rc=0;
    config_port_t *config_port_req=NULL;
    config_port_resp_t *config_port_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    config_port_req= &cfg->config_port;
    config_port_resp= &cfg->config_port_resp;

    portal_id = config_port_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_config_port(dpaa->fman,config_port_req, config_port_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot config port");
    }
    config_port_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
  
}







/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_handler(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    DPAA_LOG(LOG_DEBUG_CONFIG, "%s(): invoked with handle 0x%p code %d", __FUNCTION__,dpaa,rtm->code);
    return 0;
}


/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_clear_irq(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    uint32_t portal_id=0;
	clear_irq_cause_t *req=NULL;
	clear_irq_cause_resp_t *resp=NULL;
	int fm_id = 0;
    portal_id= rtm->clear_irq_cause.portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

	req= &rtm->clear_irq_cause;
	resp= &rtm->clear_irq_cause_resp;
	
    portal_id= rtm->clear_irq_cause.portal;

	switch(rtm->code){
		case DPAA_RUNTIME_CODE_CLEAR_BMAN_IRQ_CAUSE:
			bman_handle_err_interrupt(dpaa->bman,req);
		break;

		case DPAA_RUNTIME_CODE_CLEAR_QMAN_IRQ_CAUSE:
			qman_handle_err_interrupt(dpaa->qman,req);
		break;

		case DPAA_RUNTIME_CODE_CLEAR_FMAN_IRQ_CAUSE:
			fm_id = rtm->clear_irq_cause.id; /* Reusing the mask_irq_cause structure */
			fman_handle_event_irq(dpaa->fman, fm_id);
			fman_handle_error_irq(dpaa->fman, fm_id);

		break;
	
		default:
		break;
	}

    return 0;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_mac_config(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    int rc=EOK;

    
    rc=fman_runtime(dpaa->fman,rtm);
    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_get_mac_stats(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    int rc=EOK;
    
    rc=fman_runtime(dpaa->fman,rtm);
    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/

static int 
dpaa_runtime_mii_read_write(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    uint32_t portal_id=0;
    uint32_t port=0;
    uint32_t mac_index=0;
    uint32_t fm_index=0;
    uint32_t index=0;
	e_SysModule module;
	static uint8_t muxed=-1;

    int rc=EOK;

    portal_id= rtm->mii_read.portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);
  
  	mac_index= rtm->mii_write.port_id% MAX_MACDEV_PER_FMAN;
	fm_index= rtm->mii_write.port_id /MAX_MACDEV_PER_FMAN;


	port= rtm->mii_write.port_id %MAX_MACDEV_PER_FMAN;
	if(mac_index==MAX_MACDEV_PER_FMAN-1 ){
		/*If port is 10Gig of either FM0 or FM1*/
		rtm->mii_write.port_id= mac_index;
		index= fm_index;
		module= e_SYS_SUBMODULE_FM_PORT_10GMAC;
    	rc=fman_runtime(dpaa->fman,rtm);
	}
	else{
		/*If port is 1Gig of either FM0 or FM1*/
		rtm->mii_write.port_id= 0;
		index= fm_index*4+mac_index;
		module= e_SYS_SUBMODULE_FM_PORT_1GMAC;

		if(fm_index==1 && muxed!=1 ){
			PlatformSetIoPinForEthernet(dpaa->h_Platform,module,index,TRUE);
			muxed=1;
    		rc=fman_runtime(dpaa->fman,rtm);
			PlatformClearIoPinForEthernet(dpaa->h_Platform,module,index);
		}
		else if (fm_index==0 && muxed!=0 ){
			PlatformSetIoPinForEthernet(dpaa->h_Platform,module,index,TRUE);
			muxed=0;
    		rc=fman_runtime(dpaa->fman,rtm);
			PlatformClearIoPinForEthernet(dpaa->h_Platform,module,index);
		}
		else{
    		rc=fman_runtime(dpaa->fman,rtm);
		}
	}

    return rc;
}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_queue_dequeue(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    
    pkt_dequeue_t *deq_req=NULL;
    pkt_dequeue_resp_t *deq_resp=NULL;
    uint32_t portal_id=0;

    int32_t rc=0;
//    LOG_FUNCTION_ENTRY;

	deq_req= &rtm->pkt_dequeue;
	deq_resp= &rtm->pkt_dequeue_resp;
	portal_id = deq_req->portal;
	if(portal_id>=DPAA_MAX_SW_PORTAL){
			DPAA_LOG(LOG_CRIT_ERROR, "%s(): Incorrect Portal id %d a", __FUNCTION__,portal_id);
			LOG_FUNCTION_EXIT;
			return EINVAL;
	}
	if((rc=qman_frame_dequeue(dpaa->qman,deq_req, deq_resp))!=EOK){
			DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot Dequeue Frames from Frame queue ", __FUNCTION__);
	}
	deq_resp->result= rc;
    /* Release lock of BMAN SW portal */
//    LOG_FUNCTION_EXIT;

    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_queue_enqueue(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    
    pkt_enqueue_t *enq_req=NULL;
    pkt_enqueue_resp_t *enq_resp=NULL;
    uint32_t portal_id=0;

    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

	enq_req= &rtm->pkt_enqueue;
	enq_resp= &rtm->pkt_enqueue_resp;
	portal_id = enq_req->portal;
	if(portal_id>=DPAA_MAX_SW_PORTAL){
			DPAA_LOG(LOG_CRIT_ERROR, "%s(): Incorrect Portal id %d a", __FUNCTION__,portal_id);
			LOG_FUNCTION_EXIT;
			return EINVAL;
	}
	if((rc=qman_frame_enqueue(dpaa->qman,enq_req, enq_resp))!=EOK){
			//DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot Enqueue Frame for Frame Queue, portal %d, rc 0x%x", __FUNCTION__, portal_id, rc);
	}
	enq_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_enable_port(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    
    uint32_t portal_id=0;
    int32_t rc=0;
    enable_port_t *enable_port_req=NULL;
    enable_port_resp_t *enable_port_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    enable_port_req= &rtm->enable_port;
    enable_port_resp= &rtm->enable_port_resp;

    portal_id = enable_port_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_enable_port(dpaa->fman,enable_port_req, enable_port_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot enable port");
    }
    enable_port_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_disable_port(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    uint32_t portal_id=0;
    int32_t rc=0;
    disable_port_t *disable_port_req=NULL;
    disable_port_resp_t *disable_port_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    disable_port_req= &rtm->disable_port;
    disable_port_resp= &rtm->disable_port_resp;

    portal_id = disable_port_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_disable_port(dpaa->fman,disable_port_req, disable_port_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot disable port");
    }
    disable_port_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_dump_regs(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){

    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

    if((rc=fman_dump_regs(dpaa->fman))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot dump FMan registers");
    }

    LOG_FUNCTION_EXIT;

    return rc;
}
/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_pcd_handle(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){

    uint32_t portal_id=0;
    int32_t rc=0;
    enable_pcd_t *enable_pcd_req=NULL;
    enable_pcd_resp_t *enable_pcd_resp=NULL;

    LOG_FUNCTION_ENTRY;
        
    enable_pcd_req= &rtm->enable_pcd;
    enable_pcd_resp= &rtm->enable_pcd_resp;

    portal_id = enable_pcd_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_enable_pcd(dpaa->fman,enable_pcd_req,enable_pcd_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot enable PCD");
    }
    enable_pcd_resp->result= rc;

    LOG_FUNCTION_EXIT;

    return rc;
}



/*****************************************************************************/
/*  Function Name   : 
 *  Input           : 
 *                    
 *  Description     :  
 *  Returns         :  
 *  Note            :               
 */
/*****************************************************************************/
static int 
dpaa_runtime_buf_manage(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){
    
    buf_release_t *rel_req=NULL;
    buf_release_resp_t *rel_resp=NULL;
    buf_acquire_t *acq_req=NULL;
    buf_acquire_resp_t *acq_resp=NULL;
    uint32_t portal_id=0;
    int32_t rc=0;
    LOG_FUNCTION_ENTRY;

        
    /* Take a lock of BMAN SW portal */
    //bman_portal_sync(&dpaa->bman,req->portal);
    if(rtm->code ==DPAA_RUNTIME_CODE_BUF_RELEASE){
        rel_req= &rtm->buf_release;
        rel_resp= &rtm->buf_release_resp;
        portal_id = rel_req->portal;
        if((rc=bman_buf_release(dpaa->bman,rel_req, rel_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot release buffer in pool ", __FUNCTION__);
        }
        rel_resp->result= rc;
    }
    else if(rtm->code ==DPAA_RUNTIME_CODE_BUF_ACQUIRE){
        acq_req= &rtm->buf_acquire;
        acq_resp= &rtm->buf_acquire_resp;
        portal_id = acq_req->portal;
        if((rc=bman_buf_acquire(dpaa->bman,acq_req, acq_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot acquire buffer from pool ", __FUNCTION__);
        }
        acq_resp->result= rc;
    }
    else if(rtm->code ==DPAA_RUNTIME_CODE_BUF_FLUSH){
        acq_req= &rtm->buf_acquire;
        acq_resp= &rtm->buf_acquire_resp;
        portal_id = acq_req->portal;
        if((rc=bman_bufs_flush(dpaa->bman,acq_req, acq_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "%s(): Cannot flush buffers from pool ", __FUNCTION__);
        }
        acq_resp->result= rc;
    }
    else{
        DPAA_LOG(LOG_CRIT_ERROR, "%s(): Unhandled Code %d", __FUNCTION__,rtm->code);
    }
    /* Release lock of BMAN SW portal */
   LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : dpaa_runtime_set_promiscuous
 *  Input           : dpaa_system_t *dpaa
 * 					                   
 *  Description     :  set promiscuous mode
 *  Returns         :  dpaa_runtime_t *rtm
 *  Note            :               
 */
/*****************************************************************************/
static int 
 dpaa_runtime_set_promiscuous(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){

    uint32_t portal_id=0;
    int32_t rc=0;
    set_promiscuous_t *set_promiscuous_req=NULL;
    set_promiscuous_resp_t *set_promiscuous_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    set_promiscuous_req= &rtm->set_promiscuous;
    set_promiscuous_resp= &rtm->set_promiscuous_resp;

    portal_id = set_promiscuous_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_set_promiscuous(dpaa->fman,set_promiscuous_req, set_promiscuous_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot set promiscuous on.");
    }
     
    set_promiscuous_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : dpaa_runtime_add_hashmacaddr
 *  Input           : dpaa_system_t *dpaa
 * 					                   
 *  Description     :  add hash MAC address
 *  Returns         :  dpaa_runtime_t *rtm
 *  Note            :               
 */
/*****************************************************************************/
static int 
 dpaa_runtime_add_hashmacaddr(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){

    uint32_t portal_id=0;
    int32_t rc=0;
    hashmacaddr_t *hashmacaddr_req=NULL;
    hashmacaddr_resp_t *hashmacaddr_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    hashmacaddr_req= &rtm->hashmacaddr;
    hashmacaddr_resp= &rtm->hashmacaddr_resp;

    portal_id = hashmacaddr_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_add_hashmacaddr(dpaa->fman,hashmacaddr_req, hashmacaddr_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot add Hash MAC address .");
    }
     
    hashmacaddr_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

/*****************************************************************************/
/*  Function Name   : dpaa_runtime_del_hashmacaddr
 *  Input           : dpaa_system_t *dpaa
 * 					                   
 *  Description     :  deldd hash MAC address
 *  Returns         :  dpaa_runtime_t *rtm
 *  Note            :               
 */
/*****************************************************************************/
static int 
 dpaa_runtime_del_hashmacaddr(dpaa_system_t *dpaa, dpaa_runtime_t *rtm){

    uint32_t portal_id=0;
    int32_t rc=0;
    hashmacaddr_t *hashmacaddr_req=NULL;
    hashmacaddr_resp_t *hashmacaddr_resp=NULL;
    LOG_FUNCTION_ENTRY;
        
    hashmacaddr_req= &rtm->hashmacaddr;
    hashmacaddr_resp= &rtm->hashmacaddr_resp;

    portal_id = hashmacaddr_req->portal;
    DPAA_ASSERT(portal_id <DPAA_MAX_SW_PORTAL);

    if((rc=fman_del_hashmacaddr(dpaa->fman,hashmacaddr_req, hashmacaddr_resp))!=EOK){
            DPAA_LOG(LOG_CRIT_ERROR, "Cannot del Hash MAC address .");
    }
     
    hashmacaddr_resp->result= rc;
    LOG_FUNCTION_EXIT;

    return rc;
}

void * mmap_aligned( void * addr, size_t len, uint32_t alignment, int prot, int flags, int fildes,off_t off )
{
	void *first_map=NULL;
	off64_t offset=0;
    LOG_FUNCTION_ENTRY;

	
	if(alignment & (alignment -1)){
		DPAA_LOG(LOG_CRIT_ERROR, "%s(): alignment size %x not power of 2 !", __FUNCTION__,alignment);
	  	goto err_exit;
	}
	if((first_map=mmap( 0, len+alignment+off,prot, MAP_PRIVATE|MAP_PHYS|MAP_ANON,NOFD,0 ))==NULL){
		  DPAA_LOG(LOG_CRIT_ERROR, "%s(): first mmap failed !", __FUNCTION__);
	  goto err_exit;
	}
	if(mem_offset64(first_map, NOFD, 1, &offset, 0) == -1) {
		  DPAA_LOG(LOG_CRIT_ERROR, "%s(): mem_offset64 failed !", __FUNCTION__);
	  goto err_exit;
	}
	
	DPAA_LOG(LOG_DEBUG_CONFIG,"Result of first MMAP PADDR=0x%llx VADDR=0x%p" , offset,first_map);
	
	/*Adjust the pointer*/
	first_map  =(uint8_t *)first_map+ (alignment- (offset & (alignment-1)));
	DPAA_LOG(LOG_DEBUG_CONFIG,"%s(): Modified VADDRE offset 0x%p PADDR =0x%llx" , __FUNCTION__,first_map, (dpaa_vtophys(first_map)));
	
err_exit:
   LOG_FUNCTION_EXIT;

	return first_map;

}

#ifdef TGEC_LINK_STATUS_SUPPORT
bool dpaa_tgec_link_status(int idx) {
    return SYS_GetEthernetLinkStatus(e_SYS_SUBMODULE_FM_PORT_10GMAC, idx);
}

bool dpaa_dtsec_link_status(int idx) {
    return SYS_GetEthernetLinkStatus(e_SYS_SUBMODULE_FM_PORT_1GMAC, idx);
}
#endif

__SRCVERSION( "$URL: $ $Rev: $" );
