/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"

#include <net/ifdrvcom.h>
#include <sys/sockio.h>


/*****************************************************************************/
/*  Function Name	: p40xx_get_stats
 *	Input			: Pointer to device structure
 *					  Pointer to datastorage
 *	Description		: gets stats for interface				
 *	Returns			: void 
 *	Note			: 
 */
/*****************************************************************************/
static void
p40xx_get_stats(p40xx_dev_t *p40xx, void *data)
{
	nic_stats_t				*stats = data;

	// read nic hardware registers into p40xx data struct stats 
	p40xx_update_stats(p40xx);

	// copy it to the user buffer
	memcpy(stats, &p40xx->stats, sizeof(p40xx->stats));
}

/*****************************************************************************/
/* */
/*****************************************************************************/
static int enable_promiscuous (p40xx_dev_t *p40xx, int set_promisc)

{
	dpaa_runtime_t req;
	int32_t rc=0;
	
	req.code=DPAA_RUNTIME_CODE_SET_PROMISCUOUS;
	INIT_FLAG(req.set_promiscuous.flags);
	req.set_promiscuous.portal = p40xx->portal[0];
	req.set_promiscuous.promisc = set_promisc;
	req.set_promiscuous.port_id = p40xx->idx;
	
	if((rc=dpaa_system_runtime(p40xx->dpaa, &req))!=EOK){
			log(LOG_ERR, "%s():dpaa_system_runtime DPAA_RUNTIME_CODE_SET_PROMISCUOUS failed",__FUNCTION__);
			log(LOG_ERR, "Error Cause %d\n",req.set_promiscuous_resp.result);
			return -1;
	}
	return 0;
}

/*****************************************************************************/
/* */
/*****************************************************************************/
static int set_hashmacaddr (p40xx_dev_t *p40xx, uint8_t *addrlo, int delflag)
{
	dpaa_runtime_t req;
	int32_t rc=0;
		
	if(delflag ==0){
		req.code=DPAA_RUNTIME_CODE_ADD_HASHMACADDR;
	}
	else{
		req.code=DPAA_RUNTIME_CODE_DEL_HASHMACADDR;
	} 
	
	INIT_FLAG(req.hashmacaddr.flags);
	req.hashmacaddr.portal = p40xx->portal[0];
	req.hashmacaddr.port_id = p40xx->idx;
	memcpy(req.hashmacaddr.ethaddr, addrlo, ETHER_ADDR_LEN);
	
	if((rc=dpaa_system_runtime(p40xx->dpaa, &req))!=EOK){
			log(LOG_ERR, "%s():dpaa_system_runtime DPAA_RUNTIME_CODE_ADD/DEL_HASHMACADDR failed",__FUNCTION__);
			log(LOG_ERR, "Error Cause %d\n",req.hashmacaddr_resp.result);
			return -1;
	}
	return 0;
}

/*****************************************************************************/
/* */
/*****************************************************************************/
static int check_rangeaddr (p40xx_dev_t *p40xx )
{
struct ethercom		*ec;
struct ether_multi	*enm;
struct ether_multistep step;

	ec = &p40xx->ecom;
	
	ETHER_FIRST_MULTI (step, ec, enm);
	while (enm != NULL) {
		if (memcmp (enm->enm_addrlo, enm->enm_addrhi, ETHER_ADDR_LEN)) {
			/*A range of multi-cast addresses in the io-pkt multicast list. */
			return -1;
		}
		ETHER_NEXT_MULTI (step, enm);
	}
	return 0;	
}

/*****************************************************************************/
/*                                                                           */
/*****************************************************************************/

int p40xx_filter (p40xx_dev_t *p40xx, uint8_t oper, uint8_t *addrlo, uint8_t *addrhi)

{
struct ethercom		*ec;
struct ifnet			*ifp;
int allmulti_changed = 0;
	
	ec = &p40xx->ecom;
	ifp = &ec->ec_if;

	/*Check the Multicast mode */
	if (ifp->if_flags & IFF_MULTICAST) {
		/* Maintain the netcomm hash linked list */
		if (oper != 0){ 
			if ((memcmp (addrlo, addrhi, ETHER_ADDR_LEN)==0) ){
				/* Add or Remove the single multicast address */
				switch (oper){
					case 1: /*add*/
						set_hashmacaddr (p40xx, addrlo, 0);
						break;
					case 2: /*del*/
						set_hashmacaddr (p40xx, addrlo, 1);
						break;
					default:
						break;
				}			
			} else {
				switch(oper){ 
					case 1: { /*add range*/
						/* For a range of multicast addresses, just accept all multicasts*/
						ifp->if_flags |= IFF_ALLMULTI;
						allmulti_changed++;
						break;
					}
					case 2:{ /*del range*/
						/* For now a range of multicast addresses, first, 
					 	check if there are still other range of multi-cast addresses 
					 	in the io-pkt multicast list. If No, disable ALLMULTI. */ 
	  					if( (ifp->if_flags & IFF_ALLMULTI) && check_rangeaddr (p40xx) == 0) {
							ifp->if_flags &= ~IFF_ALLMULTI;
							allmulti_changed++;
						}
						break;
					}
					default:
						break;
				}
			}		
		}
	}
		
	/* Check the promiscus mode */
	if (ifp->if_flags & IFF_PROMISC) {
		p40xx->cfg.flags |= NIC_FLAG_PROMISCUOUS;
		enable_promiscuous (p40xx, SET_PROMISCUOUS_ALL);
		return 0;
	}

	/*
	 * We need to update promiscous if we are exiting PROMISCUOUS mode, IFF_ALLMULTI has changed, 
	 * or if this is an initialization call (oper==0).
	 */
	if ((p40xx->cfg.flags & NIC_FLAG_PROMISCUOUS) || allmulti_changed || oper == 0) {
		p40xx->cfg.flags &= ~NIC_FLAG_PROMISCUOUS;
		/*Check the Multicast mode */
		if ((ifp->if_flags & IFF_MULTICAST) && (ifp->if_flags & IFF_ALLMULTI))
			enable_promiscuous (p40xx, SET_PROMISCUOUS_MULTI);
		else
			enable_promiscuous (p40xx, SET_PROMISCUOUS_OFF);
	}

	return 0;
}


int
p40xx_ioctl(struct ifnet *ifp, unsigned long cmd, caddr_t data)
{
	p40xx_dev_t			*p40xx = ifp->if_softc;
	int				error = 0;
	struct ifdrv_com		*ifdc;
	struct drvcom_config		*dcfgp;
	struct drvcom_stats		*dstp;
	uint8_t addrlo[ETHER_ADDR_LEN];
	uint8_t addrhi[ETHER_ADDR_LEN];
		
	switch (cmd) {
	case SIOCGDRVCOM:
		ifdc = (struct ifdrv_com *)data;
		switch (ifdc->ifdc_cmd) {
		case DRVCOM_CONFIG:
			dcfgp = (struct drvcom_config *)ifdc;

			if (ifdc->ifdc_len != sizeof(nic_config_t)) {
				error = EINVAL;
				break;
			}
			memcpy(&dcfgp->dcom_config, &p40xx->cfg, sizeof(p40xx->cfg));
			break;

		case DRVCOM_STATS:
			dstp = (struct drvcom_stats *)ifdc;

			if (ifdc->ifdc_len != sizeof(nic_stats_t)) {
				error = EINVAL;
				break;
			}
			p40xx_get_stats(p40xx, &dstp->dcom_stats);
			break;

		default:
			error = EOPNOTSUPP;
			break;

		}
		break;


	case SIOCSDRVSPEC:
	case SIOCGDRVSPEC: {
        struct ifdrv *ifd = (struct ifdrv *)data;
		struct dpaa_cmd {
			struct ifdrv ifd;
			int args[2];
		} *dcom = (struct dpaa_cmd *)data;
		dpaa_ioctl_action_t action;

		memset((void*)&action, 0, sizeof(action));
		action.iface = p40xx->idx;
		action.code = ifd->ifd_cmd;
		if(action.code >= SIOCDPAAQRYFQ_P && action.code <= SIOCDPAAQRYWQ) {
			action.value   = dcom->args[0];
			action.nresbuf = IOCPARM_LEN(ifd->ifd_cmd) - sizeof(*ifd);
			action.resbuf  = &dcom->args[0];
			//XX_Print("%s: SIOCDPAAFQD: fqid %d nbuf %d buf %p dcom %p ndrvspec %d", __FUNCTION__, action.value, action.nresbuf, action.resbuf, dcom, IOCPARM_LEN(SIOCGDRVSPEC));
			error = dpaa_system_ioctl(p40xx->dpaa, &action);
		} else {
			action.idx  = dcom->args[0];
			action.inst = dcom->args[1];
			error = dpaa_system_ioctl(p40xx->dpaa, &action);
			dcom->args[0] = action.value;
		}
	    break;
	    }

    case SIOCSIFMEDIA:
    case SIOCGIFMEDIA: {
        struct ifreq *ifr = (struct ifreq *)data;

        error = ifmedia_ioctl(ifp, ifr, &p40xx->bsd_mii.mii_media, cmd);
        break;
        }

	default:
		error = ether_ioctl(ifp, cmd, data);
		if (error == ENETRESET) {
			//
			// Multicast list has changed; set the
			// hardware filter accordingly.
			//
			if ((ifp->if_flags_tx & IFF_RUNNING) == 0) {
				//
				// interface is currently down: p40xx_init() 
				// will call p40xx_set_multicast() so
				// nothing to do
				//
			} else {
				//
				// interface is up, recalc and hit gaddrs
				//
			        struct ifreq *ifr = (struct ifreq *)data;
	 				/* As ether_ioctl() returns ENETRESET, the following ether_multiaddr() shouldn't fail.*/  
	 				ether_multiaddr(ifreq_getaddr(cmd, ifr), addrlo, addrhi);
					if(cmd== SIOCADDMULTI) {
						/*Will add multicast address in the netcomm hash linked list.*/
						p40xx_filter(p40xx, 1, addrlo, addrhi);
					}
					if(cmd==SIOCDELMULTI){
						/*Will remove multicast address from the netcomm hash linked list.*/
						p40xx_filter(p40xx, 2, addrlo, addrhi);
					}
			}
			error = 0;
			break;
		}
		break;
	}
	return (error);

}



__SRCVERSION( "$URL: http://community.qnx.com/svn/repos/core_networking/trunk/sys/dev_qnx/p40xx/devctl.c $ $Rev: 904 $" );
