/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "p40xx.h"

#define MIIMADD 0x8
#define MIIMCOM 0x4
#define MIIMCON 0xC
#define MIIMSTAT 0x10
#define MIIMIND 0x14


#define MIIMCOM_READ 1
#define MIIMIND_BUSY 1
#define MIIMIND_NOT_VALID 4
#define MIIMCFG_RESET 1<<31

// PR 62747 - multiple interfaces requires mii access locking
static pthread_mutex_t  mii_mutex = PTHREAD_MUTEX_INITIALIZER;


//
// drvr lib callback to read phy register
//
uint16_t    
p40xx_mii_read (void *handle, uint8_t phy_add, uint8_t reg_add)
{
    p40xx_dev_t       *p40xx = (p40xx_dev_t *) handle;
	dpaa_handle_t dpaa=p40xx->dpaa;
	dpaa_runtime_t req;
	int rc = 0;

	if (p40xx->emu_phy != -1 && p40xx->emu_phy == p40xx->cfg.device_index) {
		return (emu_phy_read (p40xx, phy_add, reg_add));
		}

	req.code= DPAA_RUNTIME_CODE_MII_READ;
	req.mii_read.portal = p40xx->portal[0];
	req.mii_read.flags = 0;
	req.mii_read.owner = p40xx;
	req.mii_read.port_id = p40xx->idx;
	req.mii_read.phy_add = phy_add;
	req.mii_read.reg_add = reg_add;	

	_mutex_lock(&mii_mutex);

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime MII_READ  failed for id%d: return code %d",
						__FUNCTION__,	p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d\n",req.def_resp.result);
	} else {

	}
	_mutex_unlock(&mii_mutex);
	//log(LOG_INFO, "%s(): 0x%X from idx %d phy %d reg %d\n", __FUNCTION__, req.mii_read_resp.reg_value, p40xx->idx,phy_add, reg_add);
    return req.mii_read_resp.reg_value;
}


//
// drvr lib callback to write phy register
//
void    
p40xx_mii_write (void *handle, uint8_t phy_add, uint8_t reg_add, uint16_t data)

{
    p40xx_dev_t       *p40xx = (p40xx_dev_t *) handle;
	dpaa_handle_t dpaa=p40xx->dpaa;
	dpaa_runtime_t req;
	int rc = 0;

	if (p40xx->emu_phy != -1 && p40xx->emu_phy == p40xx->cfg.device_index) {
		return;
		}

	_mutex_lock(&mii_mutex);

	req.code= DPAA_RUNTIME_CODE_MII_WRITE;
	req.mii_write.portal = p40xx->portal[0];
	req.mii_write.flags = 0;
	req.mii_write.owner = p40xx;
	req.mii_write.port_id = p40xx->idx;
	req.mii_write.phy_add = phy_add;
	req.mii_write.reg_add = reg_add;
	req.mii_write.data = data;


	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime MII_WRITE failed for id%d: return code %d",
						__FUNCTION__,	p40xx->idx,rc);
		log(LOG_ERR, "Error Cause %d\n",req.def_resp.result);
	} else {

	}
	_mutex_unlock(&mii_mutex);
	//log(LOG_INFO, "%s(): 0x%X to phy %d reg %d\n", __FUNCTION__, data, phy_add, reg_add);
}


//
// drvr lib callback when PHY link state changes
//
void
p40xx_mii_callback(void *handle, uchar_t phy, uchar_t newstate)
{
	p40xx_dev_t		*p40xx = handle;
	nic_config_t        *cfg  = &p40xx->cfg;
	char				*s;
	int					i;
	int					mode;
	struct ifnet 		*ifp = &p40xx->ecom.ec_if;
	log(LOG_INFO, "%s() %s: Invoked phy=%d, newstate=%d (PHY %d, lan %d, idx %d)", __FUNCTION__, ifp->if_xname, phy, newstate, cfg->phy_addr, cfg->lan, cfg->device_index);

	switch(newstate) {
	case MDI_LINK_UP:
		if ((i = MDI_GetActiveMedia(p40xx->mdi, cfg->phy_addr, &mode)) != MDI_LINK_UP) {
			log(LOG_INFO, "%s() %s: MDI_GetActiveMedia() failed: p40xx->mdi=%p, cfg->phy_addr=0x%x", 
			__FUNCTION__, ifp->if_xname, p40xx->mdi, cfg->phy_addr);
			mode = 0;  // force default case below - all MDI_ macros are non-zero
		}

		switch(mode) {
		case MDI_10bTFD:
			s = "10 BaseT Full Duplex";
			cfg->duplex = 1;
			cfg->media_rate = 10 * 1000L;
			break;
		case MDI_10bT:
			s = "10 BaseT Half Duplex";
			cfg->duplex = 0;
			cfg->media_rate = 10 * 1000L;
			break;
		case MDI_100bTFD:
			s = "100 BaseT Full Duplex";
			cfg->duplex = 1;
			cfg->media_rate = 100 * 1000L;
			break;
		case MDI_100bT:
			s = "100 BaseT Half Duplex";
			cfg->duplex = 0;
			cfg->media_rate = 100 * 1000L;
			break;
		case MDI_100bT4:
			s = "100 BaseT4";
			cfg->duplex = 0;
			cfg->media_rate = 100 * 1000L;
			break;
        case MDI_1000bT:
			s = "1000 BaseT Half Duplex";
            cfg->duplex = 0;
            cfg->media_rate = 1000 * 1000L;
            break;
        case MDI_1000bTFD:
			if(IS_MAC_10GIG(p40xx->idx)) {
				s = "10000 BaseT Full Duplex";
				cfg->media_rate = 10000 * 1000L;
			} else {
				s = "1000 BaseT Full Duplex";
				cfg->media_rate = 1000 * 1000L;
			}
			cfg->duplex = 1;
            break;
		default:
			log(LOG_WARNING,"%s() %s: unknown link mode 0x%X",__FUNCTION__, ifp->if_xname, mode);
			s = "Unknown";
			cfg->duplex = 0;
			cfg->media_rate = 0L;
			break;
		}
	
		if (p40xx->emu_phy != -1 && p40xx->emu_phy == p40xx->cfg.device_index ) {
			/*Emu phy is ON. Dont call the speed duplex.*/
		}
		else{
			p40xx_speeduplex(p40xx);
		}

		// immediately set new speed and duplex in nic config registers

		if (cfg->media_rate) {  
		    cfg->flags &= ~NIC_FLAG_LINK_DOWN;
			if (cfg->verbose) {
				log(LOG_INFO, "%s() %s: link up lan %d idx %d PHY %d (%s)", 
				  __FUNCTION__,  ifp->if_xname, cfg->lan, cfg->device_index, cfg->phy_addr, s);
			}
			if_link_state_change(ifp, LINK_STATE_UP);
			break;
		} else {
			// fall through to link down handling
		}

	case MDI_LINK_DOWN:
		cfg->media_rate = cfg->duplex = -1;
		cfg->flags |= NIC_FLAG_LINK_DOWN;

		if (cfg->verbose) {
			log(LOG_INFO, 
			  "%s() %s: Link down lan %d idx %d PHY %d, calling MDI_AutoNegotiate()", 
			  __FUNCTION__, ifp->if_xname, cfg->lan, cfg->device_index, cfg->phy_addr);
		}
		MDI_AutoNegotiate(p40xx->mdi, cfg->phy_addr, NoWait);
		if_link_state_change(ifp, LINK_STATE_DOWN);
		break;

	default:
		break;
	}
}


//
// periodically called by stack to probe phy state
// and to clean out tx descriptor ring
//
void
p40xx_MDI_MonitorPhy (void *arg)
{
	p40xx_dev_t		*p40xx	= arg;
	nic_config_t		*cfg  		= &p40xx->cfg;


	//
	// we will probe the PHY if:
	//   the user has forced it from the cmd line, or
	//   we have not rxd any packets since the last time we ran, or
	//   the link is considered down
	//
	if (p40xx->probe_phy || !p40xx->rxd_pkts|| cfg->media_rate <= 0 ||   cfg->flags & NIC_FLAG_LINK_DOWN) {
//		if (cfg->verbose >= LOG_DEBUG_FUNC_CALL) {
//			log(LOG_ERR, "%s(): calling MDI_MonitorPhy() PHY address %d (lan %d idx %d)",  __FUNCTION__,cfg->phy_addr, cfg->lan, cfg->device_index);
//		}
		//
		// directly call drvr lib to probe PHY link state which in turn
		// will call p40xx_mii_callback() above if link state changes
		//
		MDI_MonitorPhy(p40xx->mdi);  

	} else {
//		if (cfg->verbose >= LOG_DEBUG_FUNC_CALL) {
//			log(LOG_ERR, "%s(): NOT calling MDI_MonitorPhy() PHY address %d (lan %d idx %d)",  __FUNCTION__,cfg->phy_addr, cfg->lan, cfg->device_index);
//		}
	}
	p40xx->rxd_pkts = 0;  // reset for next time we are called


	// restart timer to call us again in 2 seconds
	callout_msec(&p40xx->mii_callout, 2 * 1000, p40xx_MDI_MonitorPhy, p40xx);
}


//
// called from p40xx_init()
//
int
p40xx_init_phy(p40xx_dev_t *p40xx)
{
    nic_config_t        *cfg 		= &p40xx->cfg;
    int                 phy_idx 	= cfg->phy_addr;
    //int                 timeout;
    int                 an_capable;
    uint16_t            reg, mcsr, mcsr1;
	int					status;
    int                 i;


    if (cfg->verbose>=LOG_DEBUG_CONFIG) {
		log(LOG_ERR, "%s(): media_rate: %d, duplex: %d, PHY: %d, lan: %d, idx: %d", 
		  __FUNCTION__, cfg->media_rate, cfg->duplex, phy_idx, cfg->lan, cfg->device_index);
    }

	if (p40xx->mdi) {
		MDI_DeRegister(&p40xx->mdi);   // repetitively called by if_init
	}

	status = MDI_Register_Extended (p40xx, p40xx_mii_write, p40xx_mii_read,
	  p40xx_mii_callback, (mdi_t **)&p40xx->mdi, NULL, 0, 0);
	if (status != MDI_SUCCESS) {
		log(LOG_ERR, "%s(): MDI_Register_Extended() failed: %d", __FUNCTION__ ,status);
		p40xx->mdi = NULL;
		return -1;
	}

	callout_init(&p40xx->mii_callout);

	if (p40xx->emu_phy != -1 && p40xx->emu_phy == p40xx->cfg.device_index ) {
		emu_phy_init (p40xx);
		}
		/* The Phy regs is being reset by NetCommSW along with the Port initialization */
#if 0
    //TBD implement this *(base + MPC_TBIPA) = 0x1f;
	CCSR_SET32(p40xx->phy_regs, MIIMCON, MIIMCFG_RESET, 0);
	__asm__ __volatile__ ("sync");

    nanospin_ns(100000);
	/*Set the MDIO clock frequency*/
	CCSR_SET32(p40xx->phy_regs, MIIMCON, 0x4, 0);

	timeout = MPC_TIMEOUT;
    while ((CCSR_GET32(p40xx->phy_regs, MIIMIND,0)   & MIIMIND_BUSY)) {
        nanospin_ns(1000);
        if (! --timeout) {
            log(LOG_ERR, "%s(): MII timeout.  Continuing anyway...", __FUNCTION__);
            break;
        }
    }
#endif
    if ((i = MDI_FindPhy(p40xx->mdi, phy_idx)) == MDI_SUCCESS) {
        if ((i = MDI_InitPhy(p40xx->mdi, phy_idx)) == MDI_SUCCESS) {
               log(LOG_ERR, "%s(): PHY found at address %d", __FUNCTION__, phy_idx);

            MDI_ResetPhy(p40xx->mdi, phy_idx, WaitBusy);

            if ((PPC_GET_FAM_MEMBER(get_pvr()) == PPC_E500) && PPC_GET_REVISION(get_pvr()) < 2) {
                /* reset errata */
                if (cfg->verbose) {
                    log(LOG_ERR, "%s(): Reset PHY errata being run", __FUNCTION__);
                }
                p40xx_mii_write(p40xx, phy_idx, 0x1d, 0x001f);
                p40xx_mii_write(p40xx, phy_idx, 0x1e, 0x200c);
                p40xx_mii_write(p40xx, phy_idx, 0x1d, 0x0005);
                p40xx_mii_write(p40xx, phy_idx, 0x1e, 0x0000);
                p40xx_mii_write(p40xx, phy_idx, 0x1e, 0x0100);

            }
        } else {
            log(LOG_ERR, "%s(): MDI_InitPhy() failed for phy %d: %d", __FUNCTION__, phy_idx, i);
			return -1;
        }
    } else {
        log(LOG_ERR, "%s(): MDI_FindPhy() failed for phy %d: %d", __FUNCTION__, phy_idx, i);
		return -1;
    }

	if (p40xx->force_advertise == 0) {	// the user has forced the link down
		MDI_IsolatePhy(p40xx->mdi, phy_idx);				
		MDI_DisableMonitor(p40xx->mdi);  // neuter MDI_MonitorPhy()
		return phy_idx;
	} else {  // forced or auto-neg  
		// in case the user previously forced the link
		// down, bring it back up again
		MDI_DeIsolatePhy(p40xx->mdi, phy_idx);				
		delay(10);
	}

    an_capable = p40xx_mii_read(p40xx, phy_idx, MDI_BMSR) & 8;

	//
	// if the user has specified the speed or duplex
	// or if the phy cannot auto-negotiate ...
	//
    if (p40xx->force_advertise != -1 || !an_capable) {
        reg = p40xx_mii_read(p40xx, phy_idx, MDI_BMCR);
 
        reg &= ~(BMCR_RESTART_AN | BMCR_SPEED_100 | BMCR_SPEED_1000 | BMCR_FULL_DUPLEX);
 
        if (an_capable && p40xx->force_advertise != 0) {
            /*
             * If we force the speed, but the link partner
             * is autonegotiating, there is a greater chance
             * that everything will work if we advertise with
             * the speed that we are forcing to.
             */
            MDI_SetAdvert(p40xx->mdi, phy_idx, p40xx->force_advertise);
 
            reg |= BMCR_RESTART_AN | BMCR_AN_ENABLE;
 
            if (cfg->verbose) {
                log(LOG_ERR, "%s(): restricted autonegotiate (%dMbps only)", 
				  __FUNCTION__, cfg->media_rate / 1000);
            }
        } else {
            reg &= ~BMCR_AN_ENABLE;
 
            if (cfg->verbose) {
                log(LOG_ERR, "%s(): forcing the link", __FUNCTION__);
            }
        }
 
        mcsr = 0;
        if (cfg->duplex > 0) {
            reg |= BMCR_FULL_DUPLEX;
        }
        if (cfg->media_rate == 100 * 1000) {
            reg |= BMCR_SPEED_100;
            an_capable = (cfg->duplex > 0) ? ANAR_100bTFD : ANAR_100bT;
        } else {
            if (cfg->media_rate == 1000 * 1000) {
                reg |= BMCR_SPEED_1000;
                an_capable = 0;
                mcsr = (cfg->duplex > 0) ? MSCR_ADV_1000bTFD : MSCR_ADV_1000bT;
            } else {
                an_capable = (cfg->duplex > 0) ? ANAR_10bTFD : ANAR_10bT;
            }
        }
 
        p40xx_mii_write(p40xx, phy_idx, MDI_ANAR, an_capable);
        mcsr1 = p40xx_mii_read(p40xx, phy_idx, 9);
        mcsr1 &= ~0x300;
        mcsr1 |= mcsr;
        p40xx_mii_write(p40xx, phy_idx, MDI_MSCR, mcsr1);
        /* This PHY has to be reset for the advertisement to take effect */
        reg |= BMCR_RESET;
        p40xx_mii_write(p40xx, phy_idx, MDI_BMCR, reg);
        delay (10);
 
      
        if (reg & BMCR_AN_ENABLE) {
	        if ((i = MDI_EnableMonitor(p40xx->mdi, 1)) != MDI_SUCCESS) {
		        log(LOG_ERR, "%s(): MDI_EnableMonitor(forced) failed for phy %d: %d", __FUNCTION__, phy_idx, i);
	        }
        }

    } else {  // normal auto-negotiation mode
		if(cfg->verbose>=LOG_DEBUG_CONFIG)
		log(LOG_INFO, "%s(): Phy %d: has autonegotiation capability", __FUNCTION__, phy_idx);

        cfg->flags |= NIC_FLAG_LINK_DOWN;
        if ((i = MDI_AutoNegotiate(p40xx->mdi, phy_idx, NoWait)) != MDI_SUCCESS) {
	        log(LOG_ERR, "%s(): MDI_AutoNegotiate() failed for phy %d: %d", __FUNCTION__, phy_idx, i);
        }

        if ((i = MDI_EnableMonitor(p40xx->mdi, 1)) != MDI_SUCCESS) {
	        log(LOG_ERR, "%s(): MDI_EnableMonitor(auto) failed for phy %d: %d", __FUNCTION__, phy_idx, i);
        }
	}

    cfg->connector = NIC_CONNECTOR_MII;
	// ask for call for detecting PHY link state changes
	callout_msec(&p40xx->mii_callout, 2 * 1000, p40xx_MDI_MonitorPhy, p40xx);

	return phy_idx;
}



__SRCVERSION( "$URL: $Rev: $" );
