/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "p40xx.h"

/*********************** Local Datastorage for PADDR-VADDR cache ************/
#define PV_PAGESIZE		4096			// 4k page size
#define PV_HASH_LEN		(PV_PAGESIZE / (sizeof(struct pv_element *)))
#define PV_HASH_MASK 	(PV_HASH_LEN-1)
#define	PV_2K_SHIFT		(11)			// eleven bit shift for 2k buffers

// create hash index by taking bottom 4 bytes
// of paddr, shifting it right eleven bits (2k)
// then taking the bottom ten bits of that (1024) 

#define PV_MAKE_HASH(paddr) \
			(( ((uint64_t)paddr) >> PV_2K_SHIFT) & PV_HASH_MASK)

#define	PV_SIG			0xACBACBAB		// at start of cookie

//
// Naming convention: internal data structs start with pv_
//
//
// This struct stores an individual paddr/vaddr
// mapping in a singly-linked list.  This structure
// really should be evenly divisible into a page.
//
// For example, this struct is currently 8 bytes,
// and with a 4k page we have 4096/8=512 elements
// per allocated page
//
struct pv_element {
	struct pv_element	*next;
	void				*vaddr;	
	uint64_t		 	paddr;
};
//
// this struct stores an mmapd page
// in a singly-linked list
//
struct pv_page {
	struct pv_page		*next;
	void				*page;	
};

//
// this struct stores all the information 
// relating to a particular instance 
//
struct pv_cookie {
	unsigned			sig;			// should be PV_SIG
	pthread_mutex_t 	mutex;			// for access to data below
	struct pv_element	**hash;			// ptr to array of ptrs of hashed linked lists of pv_element
	struct pv_element	*free;			// ptr to singly-linked free list of pv_element
	struct pv_page		*page;			// ptr to singly-linked list of allocated pages 
};


/*****************************************************************************/
/*  Function Name	: p40xx_release_buf
 *	Input			: DPAA handle
 *					  Buffer Pool ID
 *					  Portal ID
 *					  Virtual address which needs to be seeded
 *	Description		: This function is responsible for releasing single/multiple buffer
 *					  into BMAN pool specified by Buffer Pool ID. 
 *	Returns			: ZERO if ok, else nonZERO  
 *	Note			:
 */
/*****************************************************************************/
int 
p40xx_release_buf (dpaa_handle_t dpaa,uint32_t bpid,uint32_t portal, void *vaddr){
	dpaa_runtime_t req;
	int rc=0;

	req.code=DPAA_RUNTIME_CODE_BUF_RELEASE ;
	
	INIT_FLAG(req.buf_release.flags);
	SET_FLAG(req.buf_release.flags,FLAG_PORTAL_LOCK);
	req.buf_release.bpid =bpid;
	req.buf_release.portal =portal ;
	req.buf_release.addr = vaddr;
	
	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_BUF_RELEASE failed for bpid%d: return code %d",
						__FUNCTION__,bpid,rc);
		log(LOG_ERR, "Error Cause %d",req.buf_release_resp.result);
		return ENOMEM;
	}
#if 0
	fd_t fd;
	paddr= XX_VirtToPhys(vaddr);
	/*Packet should now be linear into memory. Create an enqueue request*/
	fd.dd= 0;
	fd.pid= 10;
	fd.addr_low = paddr & 0xffffffff;
	fd.addr_high = (paddr >>32) & 0xffff ;
	fd.format = 0;
	fd.offset = 0; 
	fd.length20 = 64;
	fd.cmd= 0x80000000;
	fd.bpid=bpid;
	
	req.code= DPAA_RUNTIME_CODE_FAST_ENQUEUE;

	INIT_FLAG(req.pkt_enqueue.flags);
	SET_FLAG(req.pkt_enqueue.flags,FLAG_PORTAL_LOCK);

	req.pkt_enqueue.fqid= 0;
	req.pkt_enqueue.tx_handle=NULL;
	/*Here we shall try to sequentially spread the load of enqueueing over all FQIDs.*/
	req.pkt_enqueue.tx_fqid_offset= 0;

	/*FD contents shall be copied by netcomm, no need to panic*/
	req.pkt_enqueue.fd_ptr= &fd;
	req.pkt_enqueue.portal= portal;

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		nanospin_ns(10000); /*Busy wait for 10us*/

		if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){

			log(LOG_ERR, "%s():dpaa_system_runtime CODE_BUF_RELEASE failed for bpid%d: return code %d",
						__FUNCTION__,bpid,rc);
			log(LOG_ERR, "Error Cause %d",req.pkt_enqueue_resp.result);
			return ENOMEM;
		}
	}

#endif

	return 0;
}

/*****************************************************************************/
/*  Function Name	: p40xx_acquire_buf
 *	Input			: DPAA handle
 *					  Buffer Pool ID
 *					  Portal ID
 *					  Pointer of Virtual address where acquired buffer needs to be stored
 *	Description		: 
 *					   
 *	Returns			: ZERO if ok, else nonZERO  
 *	Note			:
 */
/*****************************************************************************/
int 
p40xx_acquire_buf (dpaa_handle_t dpaa,uint32_t bpid,uint32_t portal, void *vaddr){
	dpaa_runtime_t req;
	int rc=0;

	req.code=DPAA_RUNTIME_CODE_BUF_ACQUIRE ;

	INIT_FLAG(req.buf_acquire.flags);
	SET_FLAG(req.buf_acquire.flags,FLAG_PORTAL_LOCK);

	req.buf_acquire.bpid =bpid;
	req.buf_acquire.portal =portal ;
	req.buf_acquire.num =1 ;

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_BUF_ACQUIRE failed for bpid%d: return code %d",
						__FUNCTION__,bpid,rc);
		log(LOG_ERR, "Error Cause %d",req.buf_acquire_resp.result);
		return ENOMEM;
	}
	*((uint32_t*)vaddr)=req.buf_acquire_resp.addr[0];
	return 0;
}

/*****************************************************************************/
/*  Function Name	: p40xx_flush_bufs
 *	Input			: DPAA handle
 *					  Buffer Pool ID
 *					  Portal ID
 *					  Count of max buffers to flush or 0 for all in pool.
 *	Description		: 
 *					   
 *	Returns			: Count of buffers actually flushed.
 *	Note			:
 */
/*****************************************************************************/
int 
p40xx_flush_bufs (dpaa_handle_t dpaa,uint32_t bpid,uint32_t portal, uint32_t nbuf){
	dpaa_runtime_t req;
	int rc=0;

	req.code=DPAA_RUNTIME_CODE_BUF_FLUSH ;

	INIT_FLAG(req.buf_acquire.flags);
	SET_FLAG(req.buf_acquire.flags,FLAG_PORTAL_LOCK);

	req.buf_acquire.bpid =bpid;
	req.buf_acquire.portal =portal ;
	req.buf_acquire.num = nbuf ;

	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_BUF_FLUSH failed for bpid%d: return code %d",
						__FUNCTION__,bpid,rc);
		log(LOG_ERR, "Error Cause %d",req.buf_acquire_resp.result);
		return 0;
	}
	return req.buf_acquire_resp.count;
}


/**************************************************************************************/
/*                        Cache lookup routines
 *************************************************************************************/
//
// This function asks procnto to map in a 
// 4K page of memory 
//
static void *
pv_mmap_page (void)
{
	void *page;

	// ask procnto for a 4k page
	page = mmap(0, PV_PAGESIZE, PROT_READ | PROT_WRITE, 
	  MAP_PRIVATE | MAP_ANON, NOFD, 0);

	if (page == MAP_FAILED) {
		page = NULL;
	}

	return page;
}

//
// This function releases back to procnto a previously
// mapped-in a 4K page of memory 
//
static void 
pv_munmap_page (void *page)
{
	munmap(page, PV_PAGESIZE);
}

//
// This function allocates a pages worth of 
// struct pv_element, links them up, add adds 
// them to the free list of the given pvc
//
static int
pv_alloc_pve (struct pv_cookie *pvc) 
{
	struct pv_element	*new;
	struct pv_page		*page;
	int					 num_elements;
	int					 I;

	// get a new page for carving
	new = pv_mmap_page();
	if (!new) {
		return ENOMEM;
	}

	//
	// So we can free this page during the destroy
	// remember that we allocated this page, we
	// maintaining a linked list of allocated pages
	//
	page = malloc(sizeof(*page),M_DEVBUF,M_WAITOK|M_ZERO);
	if (!page) {
		pv_munmap_page(new);
		return ENOMEM;
	}
	page->page = new;
	page->next = pvc->page;
	pvc->page  = page;

	// calculate how many elements fit into a page
	num_elements = PV_PAGESIZE / sizeof(*new);
	
	// link new elements together
	for(I=0; I<(num_elements-1); I++) {
		new[I].next = &new[I+1];
	}
	// null terminate singly-linked list
	new[num_elements-1].next = NULL;

	// finally, hook new list onto pvc free queue
	pvc->free = new;

	return EOK;
}

//
// this function frees ALL allocated element 
// pages belonging to the passed cookie
//
static void
pv_free_pve (struct pv_cookie *pvc) 
{
	struct pv_page		*this_page = pvc->page;
	struct pv_page		*next_page;

	while (this_page) {
		next_page = this_page->next;

		pv_munmap_page(this_page->page);
		free(this_page,M_DEVBUF);

		this_page = next_page;
	}

	pvc->page = NULL;
}



//
// Naming convention: external functions start with paddr_cache_
//


//
// This function is called once before any
// store/lookups to allocate internal resources
//
// Returns a void * cookie which is passed to
// the store, lookup and destroy functions below
//
// Upon failure, returns NULL pointer
//


/*Gives a handle to the database*/
cache_handle_t cache_map_init(void){

	struct pv_cookie 	*pvc;

	log(LOG_INFO, "%s():Allocating a new map",__FUNCTION__);

	pvc = malloc(sizeof(*pvc),M_DEVBUF, M_WAITOK|M_ZERO);
	if (!pvc) {
		goto err1;
	}
	memset(pvc, 0, sizeof(*pvc));
	if (pthread_mutex_init(&pvc->mutex, NULL) != EOK) {
		goto err2;
	}

	// allocate and init a page for the hashed array of linked lists
	pvc->hash = pv_mmap_page();
	if (!pvc->hash) {
		goto err3;
	}

	memset(pvc->hash, 0, PV_PAGESIZE);

	// allocate and init a page for the free elements
	if (pv_alloc_pve(pvc) != EOK) {
		goto err4;

	}

	// finally, put our signature at the front of the cookie 
	pvc->sig = PV_SIG;

	return((cache_handle_t )pvc);

	// error handling from above
err4:
	pv_munmap_page(pvc->hash);	
err3:
	pthread_mutex_destroy(&pvc->mutex);
err2:
	free(pvc,M_DEVBUF);
err1:
	return NULL;	
}

/*Store entry into hash*/
int cache_map_store(uint64_t paddr, void *vaddr, cache_handle_t hndl){

	register struct pv_cookie 	*pvc = (struct pv_cookie *) hndl;
	register struct pv_element 	*new;
	register unsigned			hash_idx = PV_MAKE_HASH(paddr);
	register int				retval = EOK;
	

	if (!pvc) {	
		retval = EINVAL;
		goto err_no_mutex;
	}	

	if (pvc->sig != PV_SIG) {
		retval = EINVAL;
		goto err_no_mutex;
	}	
	

	_mutex_lock(&pvc->mutex);

	// unlink the first element in the free queue
	new = pvc->free;
	if (!new) {		
		// put some more elements on the free list
		if (pv_alloc_pve(pvc) != EOK) {
			retval = ENOMEM;
			goto err;		
		}
		new = pvc->free;
	}
	pvc->free = new->next;

	// save vaddr/paddr pair into new element
	new->vaddr = vaddr;
	new->paddr = paddr;
	
	//log(LOG_INFO, "%s():Adding PADDR %llx VADDR %x to cache handle %x idx %d",__FUNCTION__,paddr,vaddr,hndl,hash_idx);
	// link new element to front of linked list: LIFO
	new->next = pvc->hash[hash_idx];
	pvc->hash[hash_idx] = new; 

err:
	_mutex_unlock(&pvc->mutex);
err_no_mutex:
	return retval;

}
/*Delete entry from hash*/
int cache_map_delete(uint64_t paddr, void *vaddr, cache_handle_t hndl){

	return 0;
}

/*Looks up given paddr*/
void* cache_paddr_lookup(uint64_t paddr, cache_handle_t hndl){
	
	register struct pv_cookie 	*pvc = (struct pv_cookie *) hndl;
	register struct pv_element 	*this=NULL;
	register struct pv_element 	*prev=NULL;
	register unsigned			 hash_idx = PV_MAKE_HASH(paddr);
	register void				*vaddr = NULL;
	
	//log(LOG_INFO, "%s():Looking up PADDR %llx In cache handle %x ",__FUNCTION__,paddr,hndl);

	if (!pvc) {
		log(LOG_ERR, "%s():Not valid handle",__FUNCTION__);
		goto err_no_mutex;
	}	

	if (pvc->sig != PV_SIG) {
		log(LOG_ERR, "%s():Not valid signature",__FUNCTION__);
		goto err_no_mutex;
	}	
	
	_mutex_lock(&pvc->mutex);

	// set up for scan loop
	this = pvc->hash[hash_idx];
	prev = NULL;

	// scan singly-linked list, looking for paddr match
	while (this && (this->paddr != paddr)) {
		prev = this;
		this = this->next;
	}

	if (!this) {
		log(LOG_ERR, "%s(): PADDR %llx not found.",__FUNCTION__,paddr);
		this = pvc->hash[hash_idx];
		log(LOG_ERR, "Dumping Hash Index %d contents",hash_idx);
		while (this) {
			log(LOG_ERR, "PADDR %llx VADDR %p",this->paddr, this->vaddr);
			this = this->next;
		}
		goto err;  // paddr not found, no soup for you
	}

	// unlink this element with matching paddr
	if (prev) {
		prev->next = this->next;
	} else {
		pvc->hash[hash_idx] = this->next;
	}

	// finally, retrieve its vaddr, which is kind of the point
	vaddr = this->vaddr;

	// free this element - we are done with it
	this->next = pvc->free;
	pvc->free = this;

err:
	_mutex_unlock(&pvc->mutex);
err_no_mutex:
	return vaddr;
}
/*Looks up given vaddr*/


uint64_t cache_vaddr_lookup(void* vaddr, cache_handle_t hndl){
	
	return vtophys(vaddr); 
}


/*Destroy cache*/
int cache_map_destroy(cache_handle_t hndl){
	struct pv_cookie 	*pvc;

	pvc = (struct pv_cookie *) hndl;
	if (!pvc) {	
		return ENOMEM ;
	}	

	if (pvc->sig != PV_SIG) {
		return -1;
	}	

	// hope we dont block forever here.  The alternative
	// is to ignore the mutex and just crash on through
	// freeing everything, but that might result in another
	// blocked thread which holds the mutex faulting when
	// it wakes up (ie from its blocking mmap call to proc)
	// which is not a great choice, either

	_mutex_lock(&pvc->mutex);

	// first invalidate this cookie to make any future
	// calls using it, to immediately fail
	pvc->sig = 0;

	_mutex_unlock(&pvc->mutex);

	// free all previously allocated pages of elements
	pv_free_pve(pvc); 

	// free array of hashed linked lists
	pv_munmap_page(pvc->hash);	

	pthread_mutex_destroy(&pvc->mutex);

	free(pvc,M_DEVBUF);
	
	return EOK;
}




