/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
#include "bpfilter.h"
#include "p40xx.h"

#if NBPFILTER > 0
#include <net/bpf.h>
#include <net/bpfdesc.h>
#endif




#define PREPARE_MBUF \
			MGETHDR(m, M_NOWAIT, MT_DATA);\
			if(!m){   \
				log(LOG_ERR, "%s(): Cannot acquire empty mbuf ",__FUNCTION__); \
				return; \
			}\
			prefix=DPAA_GET_PREFIX(frame_vaddr);\
			INIT_FRAME_PREFIX( prefix);\
			ref_location= (uint32_t*)(&prefix->ref_count);\
			MEXTADD(m, frame_content, frame_len ,M_DEVBUF, p40xx_mbuf_release, arg, (uint32_t *)(ref_location)); \
			m->m_pkthdr.rcvif = ifp; \
			m->m_flags &= ~M_HASFCS;  \
			m->m_next = NULL; \
			m->m_ext.ext_phys = frame_paddr+frame_offset; \
			m->m_pkthdr.len = m->m_len= frame_len; \
			m->m_flags |= M_EXT_RW;  



/*****************************************************************************/
/*  Function Name	: p40xx_mbuf_release
 *	Input			: 
 *					  
 *	Description		: This function would release the mbuf when stack is done with it.
 *					  The prime objective is to seed it back in the BMAN as fast as 
 *					  possible. 
 *	Returns			: void 
 *	Note			: Since BMAN can accept 8 mbuf releases at a time, it might make sense 
 *					  to accumulate the mbufs till 8 calls and then release them 
 *					  simultaneously. This is called stockpiling inside BMAN.
 */
/*****************************************************************************/
void 
p40xx_mbuf_release(struct mbuf *m, caddr_t buf, size_t size, void *arg){
	int rc=0;
	int i=0;
	p40xx_token_t * token=arg;
	p40xx_dev_t *p40xx =(p40xx_dev_t *)token->p40xx;
	uint32_t bpid=0;
	void *vaddr =NULL;

#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,400, "->p40xx_mbuf_release" );
#endif

	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_INFO, " %s() Freeing mbuf %p with datap %p storage %p ",__FUNCTION__,m,m->m_data,buf);
	}
	
	/*Ox40 is the offset put by FMAN before each frame content. Try to match a signature there*/
	dpaa_frame_prefix_t *prefix= DPAA_GET_PREFIX((uint8_t*)buf -p40xx->rx_buffer_prefix_size);
	
	if(!(MATCH_SIGNATURE(prefix))){
		/*mbuf chain elements do not have an offset field by FMAN. */
		/*In this case look directly above the contents*/
		prefix= DPAA_GET_PREFIX((uint8_t*)buf);
		if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
			log(LOG_INFO, " %s() mbuf %p with datap %p storage %p is likely a fragment ",__FUNCTION__,m,m->m_data,buf);
		}
	}
	
	if(MATCH_SIGNATURE(prefix)){
		
		/*We dont want to handle this frame anymore in case some other fragments are still pending
		 * with stack. Erase signature to ensure we dont reseed*/
		ERASE_SIGNATURE(prefix);
		for(i=0;i<prefix->entry_cnt;i++){

			bpid=BUF_INFO_GET_BPID(prefix->info[i]);
			vaddr=(void*)(uintptr_t)(BUF_INFO_GET_VADDR(prefix->info[i]));

			if(!(bpid & BPID_HW_RELEASE)){
	
				if((rc=p40xx_release_buf(p40xx->dpaa,bpid,p40xx->portal[token->tid],vaddr))!=EOK){
					log(LOG_ERR, "%s(): Unable to release VADDR 0x%p to pool %d", 
									__FUNCTION__,vaddr, bpid);			
				}
				else{
					if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
						log(LOG_INFO,"VADDR %p Released to BPID %d Successfully via portal %d",vaddr, bpid,p40xx->portal[token->tid]);
					}
				}
			}
			else{
				if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
					log(LOG_INFO,"VADDR %p Release to BPID %d skipped !",vaddr, bpid);
				}
			}
		}
	}
	else{
		log(LOG_ERR, "mbuf %p Frame signature not found. Skipping release",m);
		log(LOG_ERR, "datap %p storage %p ",m->m_data,buf);

	}

	m_free_extfree(m);
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,400, "<-p40xx_mbuf_release" );
#endif
}





void p40xx_rx(void* arg, uint32_t fqid,fd_t *fd_ptr){
	struct mbuf		*m=NULL;
	p40xx_token_t * token=arg;
	p40xx_dev_t *p40xx= (p40xx_dev_t *)token->p40xx;
	struct ifnet	*ifp = &p40xx->ecom.ec_if;
	uint64_t		frame_paddr=0;	
	uint32_t		frame_offset=0;
	void*		frame_vaddr=NULL;
	void*		sgt_vaddr=NULL;
	void*		frame_content=NULL;
	uint32_t		frame_len=0;
	uint32_t		frame_bpid=0;
	uint32_t		fdstatus=0;
	uint32_t		sgt_bpid=0;
	uint32_t		total_length=0;
	dpaa_frame_prefix_t *prefix=NULL;
	uint32_t *ref_location=NULL;

#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1002, "->p40xx_rx" );
#endif

	
	assert(fd_ptr!=NULL);

	frame_paddr= ((uint64_t)fd_ptr->addr_high <<32) | (uint64_t)fd_ptr->addr_low;	
	frame_offset= fd_ptr->offset;
	frame_vaddr= XX_PhysToVirt(frame_paddr);
	frame_content= frame_vaddr + frame_offset;
	frame_len=fd_ptr->length20 & 0x3fffffff;
	frame_bpid= fd_ptr->bpid;
	fdstatus= fd_ptr->status;

	++p40xx->rx_counter;
	
	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_INFO, "%s():for count [%lld] BPID %d, FQID %d,Offset %d, Length %d, PADDR 0x%llx, status 0x%x Portal Id %d", 
			__FUNCTION__,p40xx->rx_counter,frame_bpid, fqid,frame_offset, frame_len,frame_paddr,fdstatus,p40xx->portal[token->tid]);
	}

	/*Check if the frame is not a scatter gather type*/
	if(fd_ptr->format==0){

		PREPARE_MBUF;

		BUF_INFO_ADD(prefix,frame_bpid,(uintptr_t)frame_vaddr);
		assert(prefix->entry_cnt<MAX_SGT_ENTRIES );

		if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
			log(LOG_INFO, "passing mbuf %p with EXTBUF VADDR %p",m,frame_vaddr);
		}
	}
	else {
		/*Scatter gather frame.*/
		/*Parse the SGT first*/
		sgt_entry_t *entry= NULL;
		sgt_vaddr= frame_vaddr;
		sgt_bpid= frame_bpid;

		struct mbuf *last_ptr=NULL;
		struct mbuf *first_ptr=NULL;
		entry= (sgt_entry_t*)frame_content;
		total_length=0;

		for( ; ; entry++){ 
		
			frame_paddr= ((uint64_t)entry->addrh <<32 )| ((uint64_t)entry->addrl );	
			frame_offset= entry->offset & 0x1fff;
			frame_vaddr= (uint32_t* )XX_PhysToVirt(frame_paddr);
			frame_content= (uint32_t* )((uint8_t*)frame_vaddr + frame_offset);
			frame_len= entry->length &  0x3fffffff;
			frame_bpid= (entry->offset &  0xff0000) >> 16;
			total_length+=frame_len ;


			PREPARE_MBUF;


			if(first_ptr==NULL){
				first_ptr= m; /*Save the lead mbuf*/

				/*SGT itself is made of a RX buffer. This buffer info is stored along with first mbuf entry.*/
				BUF_INFO_ADD(prefix,sgt_bpid,(uintptr_t)sgt_vaddr);
			}
			if(last_ptr!=NULL){
				last_ptr->m_next= m;
			}
			last_ptr= m;

			BUF_INFO_ADD(prefix,frame_bpid,(uintptr_t)frame_vaddr);
			
			if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
				log(LOG_INFO, "RX SGT Entry mbuf %p PADDR=0x%llx OFFSET=%x VADDR=0x%p CONTENT=0x%p LENGTH=%d BPID=%d",
					m,frame_paddr,frame_offset,frame_vaddr,frame_content,frame_len,frame_bpid);
			}

			if((entry->length & 0x40000000) ){
				break; /*This was the last mbuf*/
			}
			assert(prefix->entry_cnt<=MAX_SGT_ENTRIES );
		}

		m=first_ptr;
		m->m_pkthdr.len = total_length;
		if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
			log(LOG_INFO, "passing chained mbuf %p",m);
		}


	}

	if(fdstatus & 0x4) /* L4CV - Checksum Validated */
	    m->m_pkthdr.csum_flags |= M_CSUM_IPv4|M_CSUM_TCPv4|M_CSUM_UDPv4;	
	//else if((fdstatus & 0x10000000) == 0) /* DCL4C - If set, did not check layer 4 checksum */
	//    m->m_pkthdr.csum_flags |= M_CSUM_IPv4|M_CSUM_TCPv4|M_CSUM_UDPv4|M_CSUM_IPv4_BAD|M_CSUM_TCP_UDP_BAD;	

#if NBPFILTER > 0
	/* Pass this up to any BPF listeners. */
	if (ifp->if_bpf) {
			bpf_mtap(ifp->if_bpf, m);
	}
#endif
	// finally, pass this received pkt up
	ifp->if_ipackets++;
	(*ifp->if_input)(ifp, m);

#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1002, "<-p40xx_rx" );
#endif


}



void p40xx_loop(void* arg, uint32_t fqid,fd_t *fd_ptr){

	dpaa_runtime_t req;
	p40xx_token_t *token=arg; 
	p40xx_dev_t *p40xx= (p40xx_dev_t *)token->p40xx;
	uint32_t portal=0;

	uint32_t bpid=fd_ptr->bpid;
	uint32_t offset=fd_ptr->offset;
	uint32_t length=fd_ptr->length20;
	uint64_t paddr= ((uint64_t)fd_ptr->addr_high <<32) | (uint64_t)fd_ptr->addr_low;


#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1002, "->p40xx_loop" );
#endif
	p40xx->rx_counter++;

	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_INFO, "%s():for count [%lld] BPID %d, FQID %d,Offset %d, Length %d, PADDR 0x%llx", 
			__FUNCTION__,p40xx->rx_counter,bpid, fqid,offset, length,paddr);
	}

	p40xx->tx_counter++;
#ifdef MULTI_PORTAL
	portal = p40xx->portal[p40xx->tx_counter% p40xx->num_of_portals];
#else
	portal = p40xx->portal[token->tid];
#endif
	req.code= DPAA_RUNTIME_CODE_FAST_ENQUEUE;
	req.pkt_enqueue.flags= 0;
	req.pkt_enqueue.fqid= p40xx->tx_fqid;
	req.pkt_enqueue.tx_handle= p40xx->tx_handle;
	req.pkt_enqueue.tx_fqid_offset= (p40xx->tx_counter% p40xx->tx_fq_num);

	/*Since FD ptr is internally copied by API, it is safe to pass the received pointer only.*/
	req.pkt_enqueue.fd_ptr= fd_ptr;
	req.pkt_enqueue.portal= portal;

	if(dpaa_system_runtime(p40xx->dpaa, &req)!=EOK){
			/*Enqueue of the packet failed*/	
			log(LOG_ERR, "Packet Loopback Failed for if %d  FQ %dwith error code%d Portal %d",
				p40xx->idx,(p40xx->tx_fqid+req.pkt_enqueue.tx_fqid_offset),req.pkt_enqueue_resp.result,portal);
			
			void* vaddr= XX_PhysToVirt(paddr);
			if(p40xx_release_buf(p40xx->dpaa,bpid,portal,vaddr)!=EOK){
				log(LOG_ERR, "%s(): Unable to release VADDR 0x%p to pool %d", 
									__FUNCTION__,vaddr, bpid);			
			}

	}
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1002, "<-p40xx_loop" );
#endif

}


void p40xx_tx_complete(void* arg,uint32_t fqid, fd_t *fd_ptr){
	
	p40xx_token_t *token= arg;
	uint32_t bpid=fd_ptr->bpid;
	uint32_t offset=fd_ptr->offset;
	uint32_t length=fd_ptr->length20;
	uint64_t paddr= ((uint64_t)fd_ptr->addr_high <<32) | (uint64_t)fd_ptr->addr_low;
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1005, "->p40xx_tx_complete" );
#endif

	p40xx_dev_t *p40xx= (p40xx_dev_t *)token->p40xx;

	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_INFO, "%s():Invoked for BPID %d, FQID %d Offset %d, Length %d, PADDR 0x%llx", 
			__FUNCTION__,bpid,fqid,offset, length,paddr);
	}
	/*The sent frame by stack has been transmitted successfully and mbuf can be freed.*/
	if(p40xx_transmit_complete (p40xx,paddr+offset)!=EOK){
		log(LOG_ERR, "%s(): Unable to complete transmission of PADDR 0x%llx", 
									__FUNCTION__,paddr);			
		dump_fd(fd_ptr);
	}
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1005, "<-p40xx_tx_complete" );
#endif

}




void p40xx_rx_err(void* arg, uint32_t fqid,fd_t *fd_ptr){
	p40xx_token_t *token= arg;
	p40xx_dev_t *p40xx= (p40xx_dev_t *)token->p40xx;
	struct ifnet	*ifp = &p40xx->ecom.ec_if;
	int i=0;
	uint32_t bpid=fd_ptr->bpid;
	uint32_t offset=fd_ptr->offset;
	uint32_t length=fd_ptr->length20;
	uint32_t fdstatus=fd_ptr->status;
	uint64_t paddr= ((uint64_t)fd_ptr->addr_high <<32) | (uint64_t)fd_ptr->addr_low;
	void* buf= XX_PhysToVirt(paddr);

	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_ERR, "%s():Invoked for BPID %d, FQID %d, Offset %d, Length %d, PADDR 0x%llx, status 0x%x", 
			__FUNCTION__, bpid, fqid, offset, length, paddr, fdstatus);
		log(LOG_ERR, "Beginning Content dumps");
		char *data = (char*)buf+ offset;
		for(i=0;i<(length-8) && i<504; i+=8){
			log(LOG_ERR, "%02x%02x%02x%02x%02x%02x%02x%02x",
			data[i],data[i+1],data[i+2],data[i+3],data[i+4],data[i+5],data[i+6],data[i+7]);
		}
	}
	ifp->if_ierrors++;	
	p40xx_release_buf(p40xx->dpaa,bpid,p40xx->portal[token->tid],buf);
}


/*****************************************************************************/
/*  Function Name	: p40xx_receive
 *	Input			: Pointer to device struct
 *					  nw_work_thread
 *	Description		: stops transmission and reception on interface gracefully				
 *	Returns			: void 
 *	Note			: 
 */
/*****************************************************************************/
void 
p40xx_receive (p40xx_token_t *token) {
	
	p40xx_dev_t *p40xx=token->p40xx;
	int i = token->tid;
	dpaa_runtime_t  req;
	dpaa_handle_t   dpaa= p40xx->dpaa;
	int32_t 		rc=0;
	uint32_t 		idx=p40xx->idx;
	p40xx->rxd_pkts = 1;  
	
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1001, "->p40xx_receive" );
#endif
	req.code=DPAA_RUNTIME_CODE_FAST_DEQUEUE;
	req.pkt_dequeue.flags=0;
	req.pkt_dequeue.portal=p40xx->portal[i];
		
	/*Give the storage area*/
	req.pkt_dequeue_resp.fd_ptr=NULL;
		
	if((rc=dpaa_system_runtime(dpaa, &req))!=EOK){
		log(LOG_ERR, "%s():dpaa_system_runtime CODE_FAST_DEQUEUE failed for id%d: return code %d",
					__FUNCTION__,idx,rc);
		log(LOG_ERR, "Error Cause %d",req.pkt_dequeue_resp.result);
	}
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,1001, "<-p40xx_receive" );	
#endif

	return ;
} 


