/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */



#include "bpfilter.h"

#include "p40xx.h"

#if NBPFILTER > 0
#include <net/bpf.h>
#include <net/bpfdesc.h>
#endif
static uint64_t p40xx_create_sgt(p40xx_dev_t *p40xx,  struct mbuf *m , uint32_t num_frag, uint32_t * length, uint32_t *sgt_bpid);
static int p40xx_fd_enqueue(p40xx_dev_t *p40xx, fd_t *fd);
static void p40xx_recovery(p40xx_dev_t *p40xx, struct mbuf *m);

void
m_freemctx(struct mbuf * m) {
	struct mbuf *m_sgt;

	m_sgt = M_GETCTX(m, struct mbuf *);
	if(m_sgt) m_free(m_sgt);
	m_freem(m);
}

/*****************************************************************************/
/*  Function Name	: p40xx_defrag
 *	Input			: Pointer to defragmented mbuf
 *	Description		: Defragment the tx packet because DPAA doesnt uses Scatter gather 
 *					  list as of yet.
 *	Returns			: mbuf*
 *	Note			: this function is called only if the packet is ridiculously
 *					  fragmented.  If we are lucky, the entire packet will fit into
 *					  one cluster so we manually defrag.  However, if this is a jumbo
 *					  packet, and io-pkt is running without large values for mclbytes
 *					  and pagesize, we are left with no choice but to dup the packet
 */
/*****************************************************************************/
static struct mbuf * 
p40xx_defrag(struct mbuf *m) {

	struct mbuf *m2;
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "->p40xx_defrag" );
#endif

	if (m->m_pkthdr.len > MCLBYTES) {
		//
		// this is a jumbo packet which just happens to
		// be larger than a single cluster, which means
		// that the normal stuff below wont work.  So, 
		// we try a "deep" copy of this packet, which is
		// time-consuming, but is probably better than		
		// just dropping the packet
		//
		int num_frag;

		m2 = m_dup(m, 0, m->m_pkthdr.len, M_DONTWAIT);
		if (!m2) {
			m_freemctx(m);
			goto err_exit;
		}
		M_SETCTX(m2, NULL);
		m_freemctx(m);
		m = m2;  // m is now the defragged packet

		// ensure that dupd packet is not excessively fragmented
		for (num_frag=0, m2=m; m2; num_frag++) {
			m2 = m2->m_next;
		}
		if (num_frag > FM_NUM_OF_SG_TABLE_ENTRY) {
			// this should never happen
			m_freemctx(m);
			goto err_exit;
		}
	#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "<-p40xx_defrag OK Prev" );
	#endif

		return m;
	}

	// the entire packet should fit into one cluster

    MGET(m2, M_DONTWAIT, MT_DATA);
    if (m2 == NULL) {
		m_freemctx(m);
		goto err_exit;
    }

	M_COPY_PKTHDR(m2, m);
	M_SETCTX(m2, NULL);

    MCLGET(m2, M_DONTWAIT);
    if ((m2->m_flags & M_EXT) == 0) {
		m_freemctx(m);
        m_freemctx(m2);
		goto err_exit;
    }

	// this is NOT paranoid - this can happen with jumbo packets bigger 
	// than a single cluster - should be handled above
	if (m->m_pkthdr.len > m2->m_ext.ext_size) {
		m_freemctx(m);
        m_freemctx(m2);
		goto err_exit;
	}

    m_copydata(m, 0, m->m_pkthdr.len, mtod(m2, caddr_t));
    m2->m_pkthdr.len = m2->m_len = m->m_pkthdr.len;

	m_freemctx(m);
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "<-p40xx_defrag OK" );
#endif

    return m2;

	err_exit:
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "<-p40xx_defrag ERROR" );
#endif
		return NULL;

}



/*****************************************************************************/
/*  Function Name	: p40xx_start
 *	Input			: Pointer to ifnet for the interface
 *	Description		: This function is responsible for taking mbuf out of send queue
 *					  and putting them for transmission to HW. in this driver,
 *					  This function needs to take the mbuf out, get the physical 
 *					  address of the data, and call Enqueue frame API of the DPAA. 
 *	Returns			: void
 *	Note			: Called with ifp->if_snd_mutex held and IFF_OACTIVE not
 *					  yet set in ifp->if_flags_tx.
 */
/*****************************************************************************/
void p40xx_start(struct ifnet *ifp) {
	p40xx_dev_t	*p40xx = ifp->if_softc;
	struct mbuf	*m=NULL;
	struct mbuf	*m2=NULL;
	int32_t		num_frag=0;
	int32_t		num_frag_zero_len=0;
	uint64_t  	mbuf_phys_addr=0;
	uint64_t  	align=0;
	uint32_t 	bpid=0;
	uint32_t 	length=0;
	uint32_t 	format=0;
	fd_t 		fd;
	int 		sgt_enable=0;
	int32_t 	buf_hw_release=0;
	uint32_t 	offset=0;
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,100, "->p40xx_start" );
#endif

	/*Do not send anything if we are not up yet*/
	if ((ifp->if_flags_tx & (IFF_RUNNING|IFF_OACTIVE)) != IFF_RUNNING) {
		NW_SIGUNLOCK(&ifp->if_snd_ex, p40xx->iopkt);
#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,100, "<-p40xx_start" );
#endif
		return;
	}

	do{
		num_frag=0;
		num_frag_zero_len=0;
		mbuf_phys_addr=0;
		bpid=DPAA_BMAN_IGNORED_BPID;
		length=0;
		format=0;
		sgt_enable=0;
		buf_hw_release=0;
		offset=0;

		IFQ_DEQUEUE(&ifp->if_snd, m);

		if (!m) {
			break;
		}
		M_SETCTX(m, NULL);
		
		/* count how many fragements here */
		for (num_frag=0, num_frag_zero_len=0, m2=m; m2; num_frag++){
			if (m2->m_len == 0) {
				num_frag_zero_len++;
			}
			m2 = m2->m_next;
		}
		num_frag -= num_frag_zero_len;

			/* At this point we either need a defrag or a Scatter Gather table to transmit this frame.
		 	* Both of these options have their own pros and cons. While defrag should be avoided for 
		 	* larger packets, it may be faster than creating a SGT for 2-3 fragged packets.*/
		if((num_frag>1) && ( num_frag <=FM_NUM_OF_SG_TABLE_ENTRY )){
			/*****************************************************************************************
			 *    SGT Section
			 ******************************************************************************************/
				/*Create a SGT*/
				if((mbuf_phys_addr= p40xx_create_sgt(p40xx, m, num_frag, &length, &bpid))==0){
					log(LOG_ERR, "%s(): p40xx_create_sgt failed", __FUNCTION__);
					p40xx->stats.tx_failed_allocs++;
					ifp->if_oerrors++;
					m_freemctx(m);
					break;
				}
				sgt_enable=1;
				format=0x4;
				goto transmit_frame;

		}
		else if (num_frag >FM_NUM_OF_SG_TABLE_ENTRY ){
				if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
					log(LOG_INFO, "Transmitting a %d Fragged packet! ",num_frag);
				}
			
				/*Currently for both types above we need to resolve to defrag, because SGT doesn't support
			 	* more than 16 entries (our MBUF sgt's only support 14).
				*/
				if ((m2 = p40xx_defrag(m)) == NULL) {
					log(LOG_ERR, "%s(): p40xx_defrag() failed", __FUNCTION__);
					p40xx->stats.tx_failed_allocs++;
					ifp->if_oerrors++;
					break;
				}
				/*Use the defragmented packet*/
				m = m2;
		}

		mbuf_phys_addr= mbuf_phys(m);
		length= m->m_len;
		

		/*Check if it being forwarded from the Rx path directly. Or it is actually a forwarded packet
		 * getting defragged above*/
		if((m->m_flags & M_EXT)  && (m->m_ext.ext_free ==p40xx_mbuf_release)){

/*This is the fast forwarding section which should be enabled when we do fast forwarding of traffic
 * Here we dont get a confirmation but we tell HW to release the buffers. */
#if 0
			dpaa_frame_prefix_t *prefix=DPAA_GET_PREFIX((uint8_t*)m->m_ext.ext_buf - p40xx->rx_buffer_prefix_size);
			if(MATCH_SIGNATURE(prefix)){
				bpid= BUF_INFO_GET_BPID(prefix->info[0]) ;
				/*Tell the HW to free it*/
				buf_hw_release=1;
				if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
					log(LOG_INFO, "Fast forward BPID %d",bpid);
				}	
				/*Modify the BPID so that we dont release it in BMAN again*/
				prefix->info[0]|= (uint64_t) (BPID_HW_RELEASE << 48)  ;
				mbuf_phys_addr= m->m_ext.ext_phys- p40xx->rx_buffer_prefix_size ;
				offset= mbuf_phys(m)-mbuf_phys_addr;
			}
			else{
				log(LOG_ERR, "Single Frame pointer %p, fast Forward, No signature? Is chained mbuf forwarded standalone?",m);
				/*Single frame with our Free function and no signature ? Something is really bad.*/
				ifp->if_oerrors++;
				m_freemctx(m);
				continue;
			}
#endif

		}

transmit_frame:
		/**/
		/*Packet should now be linear into memory. Create an enqueue request*/
		align= mbuf_phys_addr%4;
		if(align){
			mbuf_phys_addr-= align;
			offset+=align;
		}


		fd.dd= 0;
		fd.pid= 10;
		fd.addr_low = mbuf_phys_addr & 0xffffffff;
		fd.addr_high = (mbuf_phys_addr >>32) & 0xffff ;
		fd.format =  format;
		fd.offset = offset;
		fd.length20 = length;
		if(buf_hw_release==1)
			fd.cmd= 0x80000000;
		else
			fd.cmd= 0x0;
		fd.bpid=bpid;

		/*Add the mbuf to PADDR combination into our hash here. 
		 Dont be scared to add it even before transmission because Transmit Confimation is 
		 so fast that we can have RX interrupt before we have come out of enqueue
		function. In that case we end up seeing a lookup failure in RX path and loose this
		mbuf. */

#ifndef NO_TX_CONFIRMATION
		if(buf_hw_release!=1){

#ifdef 	ENABLE_INSTR
			TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,101, "add_hash_entry start" );
#endif

			cache_map_store(mbuf_phys_addr+offset,m,p40xx->cache_handle);
#ifdef 	ENABLE_INSTR
			TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,101, "add_hash_entry finish" );
#endif

		}
#endif


		if(p40xx_fd_enqueue(p40xx, &fd)==EOK){
			if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
					log(LOG_INFO, "Packet Enqueue successful [%lld] for IF %d PADDR 0x%llx mbuf %p FQID %d",
				p40xx->tx_counter, p40xx->idx,mbuf_phys_addr,m,p40xx->tx_fqid);
			}

#if NBPFILTER > 0
				// Pass the packet to any BPF listeners
				if (ifp->if_bpf) {
					bpf_mtap(ifp->if_bpf, m);
				}
#endif

			if(buf_hw_release==1){
				ifp->if_opackets++;
				m_freemctx(m);
			}
#ifdef NO_TX_CONFIRMATION
			else{
				ifp->if_opackets++;
				p40xx_mbuf_list_manage(p40xx, m);
			}
#endif

	}
	/*Handling Enqueue Failure*/
	else{
			ifp->if_oerrors++;
			if(buf_hw_release!=1){
#ifndef NO_TX_CONFIRMATION
				/*Perform a lookup to remove entries*/
				cache_paddr_lookup(mbuf_phys_addr+offset,p40xx->cache_handle);
#endif
			}
			else{
				/*Ok, here is a big problem that we modified the BPID in the frame to tell the SW not to release this in BMAN
		 		* because HW is gonna do it. Now HW cheated us and this buffer is going to be lost forever, 
				* if we dont recover it!!!!!*/
				p40xx_recovery(p40xx, m);
			}
			m_freemctx(m);
	}

	}while(1);

#ifdef NO_TX_CONFIRMATION_FLSHBUF
	(void)p40xx_flush_bufs(p40xx->dpaa, DPAA_BMAN_IGNORED_BPID, p40xx->portal[0], 0);
#endif

	ifp->if_flags_tx &= ~IFF_OACTIVE;

	NW_SIGUNLOCK(&ifp->if_snd_ex, p40xx->iopkt);
#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,100, "<-p40xx_start" );
#endif	
	return;
}

/*****************************************************************************/
/*  Function Name	: p40xx_transmit_complete
 *	Input			: 
 *	Description		:  
 *	Returns			: void
 *	Note			: Called from receive.c
 */
/*****************************************************************************/
int p40xx_transmit_complete (p40xx_dev_t *p40xx,uint64_t paddr ){

#ifndef NO_TX_CONFIRMATION
	struct mbuf *m=NULL ;
	struct ifnet *ifp = &p40xx->ecom.ec_if;
	
	/*Check here for any error indication in transmit frame */
	#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "->p40xx_transmit_complete" );
	#endif

	m = (struct mbuf *)cache_paddr_lookup(paddr,p40xx->cache_handle);
	if(m==NULL){
		/**/
		log(LOG_ERR, "%s() Can not find PADDR %llx in mbuf hash",__FUNCTION__,paddr);
	#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "<-p40xx_transmit_complete failed PADDR lookup" );
	#endif
		return -1;
	}
	else{
	#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,201, "lookup finished" );
	#endif

		ifp->if_opackets++;
		m_freemctx(m);

	#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,200, "<-p40xx_transmit_complete" );
	#endif

	}
#endif
	return EOK;
}




static uint64_t p40xx_create_sgt(p40xx_dev_t *p40xx,  struct mbuf *m , uint32_t num_frag, uint32_t * length, uint32_t *p_sgt_bpid){

	int i=0;
	struct mbuf *m2=NULL;
	uint64_t phys_addr;
	dpaa_sgt_t *sgt=NULL;
	void * vaddr= NULL;
	struct mbuf *m_sgt;

	*length=0;
	/* Acquire an mbuf buffer for the scatter gather table */
	*p_sgt_bpid = DPAA_BMAN_IGNORED_BPID;
	MGET(m_sgt, M_DONTWAIT, MT_DATA);
	if(m_sgt == NULL) {
		log(LOG_INFO, "Unable to acquire MBUF for SGT creation");
		return 0;
	}
	M_ALIGN(m_sgt, FM_NUM_OF_SG_TABLE_ENTRY*sizeof(sgt_entry_t));
	M_SETCTX(m, m_sgt);
	vaddr = mtod(m_sgt, void *);
	sgt= (dpaa_sgt_t* )vaddr;
	
	//memset((void*)sgt,0,sizeof(dpaa_sgt_t));

	for(i=0,m2=m;(i<FM_NUM_OF_SG_TABLE_ENTRY) && (i<num_frag) && (m2);m2=m2->m_next){
		
		if(m2->m_len == 0) {
			/*We donot want to waste SGT entry with a zero size packet*/ 
			if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
				log(LOG_INFO, "Null length packet at Index %d pkthadr %d",i,m->m_pkthdr.len);
			}
			continue;
		}
		if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
			log(LOG_INFO, "SGT [%d] Prepared from MBUF %p FLAGS %x Freefunction %p",i,m2,m2->m_flags, m2->m_ext.ext_free);
		}
		sgt->entry[i].length= m2->m_len; 
		*length+= m2->m_len;
		
		if((m2->m_flags & M_EXT) && (m2->m_ext.ext_free ==p40xx_mbuf_release)){
			phys_addr= m2->m_ext.ext_phys - p40xx->rx_buffer_prefix_size ;
			sgt->entry[i].offset= mbuf_phys(m2) - phys_addr;	
		}
		else{
			phys_addr= mbuf_phys(m2) ;
			sgt->entry[i].offset= 0;
		}
		sgt->entry[i].addrh= (uint32_t)(phys_addr>>32) ;
		sgt->entry[i].addrl= (uint32_t)phys_addr ;
		/*Note: We do not support HW buffer release in Scatter Gather Table*/
		sgt->entry[i].offset|= (DPAA_BMAN_IGNORED_BPID & 0xffff) <<16;
		i++;

	}
	/*Set the final entry. Note: Do not hit i before here.*/
	sgt->entry[i-1].length|= 1<<30;
				
	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		for(i=0;(i<FM_NUM_OF_SG_TABLE_ENTRY) &&( i<num_frag);i++){
			log(LOG_INFO, "SGT [%d] addrh=%2x addrl=%8x  length=%8x  offset=%8x ",i,
			sgt->entry[i].addrh,sgt->entry[i].addrl,sgt->entry[i].length,sgt->entry[i].offset);
		}
	}
	return mbuf_phys(m_sgt);
}

static int p40xx_fd_enqueue(p40xx_dev_t *p40xx, fd_t *fd){
	uint32_t portal=0;
	dpaa_runtime_t 		req;
	uint32_t fqid= p40xx->tx_fqid; 
	static int busy_retries[10];

	p40xx->tx_counter++;

	/*Here we shall try to sequentially spread the load of enqueueing over all portals.*/
	portal=  p40xx->portal[p40xx->tx_counter% p40xx->num_of_portals];

	req.code= DPAA_RUNTIME_CODE_FAST_ENQUEUE;
	INIT_FLAG(req.pkt_enqueue.flags);
	SET_FLAG(req.pkt_enqueue.flags,FLAG_PORTAL_LOCK);
	req.pkt_enqueue.fqid= fqid;
	req.pkt_enqueue.tx_handle= p40xx->tx_handle;
	/*Here we shall try to sequentially spread the load of enqueueing over all FQIDs.*/
	req.pkt_enqueue.tx_fqid_offset= (p40xx->tx_counter% p40xx->tx_fq_num);

	/*FD contents shall be copied by netcomm, no need to panic*/
	req.pkt_enqueue.fd_ptr= fd;
	req.pkt_enqueue.portal= portal;

#ifdef 	ENABLE_INSTR
	TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,2001, "enqueue start" );
#endif
	if(dpaa_system_runtime(p40xx->dpaa, &req)!=EOK){
		if(req.pkt_enqueue_resp.result== DPAA_ERR_EBUSY){
			//log(LOG_WARNING, "Packet Enqueue Busy: tx_counter %lld num_of_portals %d portal %d tx_fq_num %d tx_fqid_offset %d",
			//		p40xx->tx_counter, p40xx->num_of_portals, portal, p40xx->tx_fq_num, (uint32_t)(p40xx->tx_counter% p40xx->tx_fq_num));

			/*retry enqueue with a cooloff or another portal*/
			if(p40xx->num_of_portals>1){
				portal= (portal+1) % p40xx->num_of_portals;
			}
			//nanospin_ns(10000); /*Busy wait for 10us*/
			req.pkt_enqueue.portal= portal;
			busy_retries[portal]++;
#if 0
			if(dpaa_qman_low_portal_eqcr_validate(p40xx->dpaa, portal, FALSE)) {
				//dpaa_qman_low_portal_dump(p40xx->dpaa, portal);
				dpaa_qman_low_portal_eqcr_validate(p40xx->dpaa, portal, TRUE);
			}
#endif
			if(dpaa_system_runtime(p40xx->dpaa, &req)!=EOK){
				//dpaa_qman_low_portal_dump(p40xx->dpaa, portal);

				log(LOG_ERR, "Packet Enqueue Failed for if %d with error code 0x%x Portal %d (retries %d)",
						p40xx->idx,req.pkt_enqueue_resp.result,portal, busy_retries[portal]);
				busy_retries[portal] = 0;
				log(LOG_ERR, "retry error: tx_counter %lld num_of_portals %d portal %d tx_fq_num %d tx_fqid_offset %d",
					p40xx->tx_counter, p40xx->num_of_portals, portal, p40xx->tx_fq_num, (uint32_t)(p40xx->tx_counter% p40xx->tx_fq_num));

				//dpaa_qman_low_portal_eqcr_validate(p40xx->dpaa, portal, TRUE);
#ifdef 	ENABLE_INSTR
				TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,2001, "enqueue failed retry" );
#endif

				return -1;

			}
		}
		else{
			log(LOG_ERR, "Packet Enqueue Failed for if %d with error code 0x%x Portal %d (retries %d)",
					p40xx->idx,req.pkt_enqueue_resp.result,portal, busy_retries[portal]);
			busy_retries[portal] = 0;
			log(LOG_ERR, "send error: tx_counter %lld num_of_portals %d portal %d tx_fq_num %d tx_fqid_offset %d",
					p40xx->tx_counter, p40xx->num_of_portals, portal, p40xx->tx_fq_num, (uint32_t)(p40xx->tx_counter% p40xx->tx_fq_num));
#ifdef 	ENABLE_INSTR
			TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,2001, "enqueue failed" );
#endif
			return -1;
		}
	}
#ifdef 	ENABLE_INSTR
		TraceEvent(_NTO_TRACE_INSERTUSRSTREVENT,2001, "enqueue complete" );
#endif
	return EOK;
}



static void p40xx_recovery(p40xx_dev_t *p40xx, struct mbuf *m){
	struct mbuf* m2=NULL;
	int i=0;
	uint32_t bpid=0;
	for (m2=m; m2; m2=m2->m_next){

	if((m2->m_flags & M_EXT) && (m2->m_ext.ext_free ==p40xx_mbuf_release)){

			dpaa_frame_prefix_t *prefix=DPAA_GET_PREFIX((uint8_t*)m2->m_ext.ext_buf - p40xx->rx_buffer_prefix_size) ;
			if(MATCH_SIGNATURE(prefix)){
				for(i=0;i<prefix->entry_cnt;i++){	
					bpid=	BUF_INFO_GET_BPID(prefix->info[i]);
					prefix->info[i]&= (uint64_t)((~((uint64_t)BPID_HW_RELEASE<< 48)))  ;

					if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
						log(LOG_INFO, "Recovering mbuf chain %d with BPID %d",i,bpid);
					}
				}
			}
		}
		
	}

}

void p40xx_mbuf_list_init(p40xx_dev_t *p40xx){
	int i=0;
	int rc=EOK;
	p40xx_mlist_element *mlist_cur=NULL;
	struct mbuf *m=NULL;

	p40xx->mlist_start= NULL;
	p40xx->mlist_end= NULL;

	for(i=0;i<MBUF_LIST_ELEMENTS+1;i++){
		 	
		MGETHDR(m, M_NOWAIT, MT_DATA);
		if(!m){
			log(LOG_ERR, "%s(): Cannot acquire empty mbuf ",__FUNCTION__);
			rc=-1;
			break;
		}
		M_SETCTX(m, NULL);

		mlist_cur = malloc(sizeof(p40xx_mlist_element),M_DEVBUF,M_NOWAIT);
		if(mlist_cur == NULL){
			log(LOG_ERR, "%s(): Cannot acquire empty mlist element ",__FUNCTION__);
			m_freemctx(m);
			rc=-1;
			break;
		}
		MLIST_INIT(mlist_cur);

		if(p40xx->mlist_start == NULL) {
		   	/*First Element*/
			mlist_cur->m_ptr = m;
			p40xx->mlist_start= mlist_cur;
			p40xx->mlist_end= mlist_cur;
		} else
			MLIST_ADD_ELEM(p40xx->mlist_end,mlist_cur,m);
	}

	if(rc!=EOK)
		p40xx_mbuf_list_finish(p40xx);
}

void p40xx_mbuf_list_finish(p40xx_dev_t *p40xx){
	p40xx_mlist_element *mlist_cur=NULL;

	if(p40xx->mlist_start != NULL) {
		while(p40xx->mlist_start->next != NULL) {
			MLIST_REM_ELEM(p40xx->mlist_start,mlist_cur);
			m_freemctx(mlist_cur->m_ptr);
			free(mlist_cur,M_DEVBUF);
		}
		m_freemctx(p40xx->mlist_start->m_ptr);
		free(p40xx->mlist_start,M_DEVBUF);
	}
	p40xx->mlist_start= NULL;
	p40xx->mlist_end= NULL;
}

void p40xx_mbuf_list_manage(p40xx_dev_t *p40xx, struct mbuf *mptr){
	p40xx_mlist_element *mlist_cur=NULL;

	/*Operation is to add this mptr to the end of the queue and free from 
	 * the head of the queue. There shouldn't be any malloc or free operation.*/
	/*Take the element from the start of the queue*/
	MLIST_REM_ELEM(p40xx->mlist_start,mlist_cur);
	if(p40xx->verbose>=LOG_DEBUG_DATA_FLOW){
		log(LOG_ERR, "%s(): Add >>>>>>> %p   Remove <<<<<<<<< %p",__FUNCTION__,mptr,mlist_cur->m_ptr);

	}
	m_freemctx(mlist_cur->m_ptr);
	MLIST_ADD_ELEM(p40xx->mlist_end,mlist_cur,mptr);
}
