/*
 * $QNXLicenseC:
 * Copyright 2011 QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "ppc_espi.h"


int ppc_espi_cfg(void *hdl, spi_cfg_t *cfg)
{
    ppc_espi_dev_t  *ppc_espi = hdl;
    uint32_t        spimode, prescale, len;

    if (cfg == NULL) {
        return 0;
    }

    spimode = 0;
    len = cfg->mode & SPI_MODE_CHAR_LEN_MASK;

    /* check the character length, support 4-16 bit format */
    if ((len > 16) || (len < 4)) {
        return 0;
    }

    /*
     * for MSB mode, the interface only support 8 or 16 bits character
     * for LSB mode, the interface support 4 - 16 bits character
     */
    if (cfg->mode & SPI_MODE_BODER_MSB) {
        if ((8 == len) || (16 == len)) {
            spimode |= PPC_ESPI_SPMODE_REV;
        } else {
            return 0;
        }
    }

    /* configure data length*/
    -- len;
    spimode |= len << 16;

    /*
     * SPI baudrate configuration
     * system clock is platform clock divide by 2
     */
    if (ppc_espi->clock < (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV)) {
        /* baudrate out of range, set to MAX support value */
        spimode &= (~PPC_ESPI_SPMODE_DIV16);        // system clock is the input to the SPI baudrate generator
        prescale =  0;                              // prescale is 1,  MAX SPI baud rate
        fprintf(stderr, "spi-ppc_espi: SPI baudrate out of range, set it to the MAX support value\n");
    }
    else if (ppc_espi->clock < (cfg->clock_rate * PPC_ESPI_PM_MAX * PPC_ESPI_CLK_DIV16 * PPC_ESPI_SYSCLK_DIV)) {
        /* Normal supported baudrate range */
        if (ppc_espi->clock <= (cfg->clock_rate * PPC_ESPI_PM_MAX * PPC_ESPI_SYSCLK_DIV)) {
            /* system clock is the input to the SPI baud rate generator */
            spimode &= (~PPC_ESPI_SPMODE_DIV16);

            /* make sure the real baudrate is less equal to the configuration value */
            if (ppc_espi->clock % (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV)) {
                prescale = (ppc_espi->clock / (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV));
            } else {
                prescale = (ppc_espi->clock / (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV)) - 1;
            }
        }
        else {
            /* system clock/16 is the input to the SPI baud rate generator */
            spimode |= PPC_ESPI_SPMODE_DIV16;
            
            /* the real baudrate is less equal to the configuration value */
            if (ppc_espi->clock % (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV * PPC_ESPI_CLK_DIV16)) {
                prescale = (ppc_espi->clock / (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV * PPC_ESPI_CLK_DIV16));
            }
            else {
                prescale = (ppc_espi->clock / (cfg->clock_rate * PPC_ESPI_SYSCLK_DIV * PPC_ESPI_CLK_DIV16)) - 1;
            }                           
        }
    }
    else {
        /* baudrate out of range, set to MIN support value */
        spimode |= PPC_ESPI_SPMODE_DIV16;           // system clock/16 is the input to the SPI baud rate generator
        prescale = PPC_ESPI_PM_MAX - 1;             // prescale is 15,  MIN SPI baud rate
        fprintf(stderr, "spi-ppc_espi: The SPI baudrate out of range, Set it to the Minimal support value\n");
    }

    if ((prescale > 15) || (prescale < 0)) {
        return 0;
    }
    spimode |= prescale << 24;

    /* set CS Polarity */
    spimode |= 1 << 20;

    if ((cfg->mode & SPI_MODE_CKPOL_HIGH)) {
        spimode |= PPC_ESPI_SPMODE_CI;              // Active low polarity
    }

    if (cfg->mode & SPI_MODE_CKPHASE_HALF) {
        spimode &= ~PPC_ESPI_SPMODE_CP;             // SPICLK starts toggling at the middle of the data transfer
    }

    return  spimode;
}

__SRCVERSION( "$URL$ $Rev$" );
