/*
 * $QNXLicenseC:
 * Copyright 2011 QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "ppc_espi.h"


static char *ppc_espi_opts[] = {
    "base",             // base address for this eSPI controller
    "irq",              // IRQ for this eSPI interface
    "clock",            // eSPI clock
    "loopback",         // loopback Control, for testing
    NULL
};

spi_funcs_t spi_drv_entry = {
    sizeof(spi_funcs_t),
    ppc_espi_init,      // init()
    ppc_espi_dinit,     // fini()
    ppc_espi_drvinfo,   // drvinfo()
    ppc_espi_devinfo,   // devinfo()
    ppc_espi_setcfg,    // setcfg()
    ppc_espi_xfer,      // xfer()
    NULL                // dma_xfer(), not support
};

/*
 * Note:
 * The devices listed are just examples, users should change
 * this according to their own hardware specification.
 */
static spi_devinfo_t devlist[4] = {
    {
        0x00,                       // device ID
        "SPI-DEV1",                 // description
        {
            8 | SPI_MODE_BODER_MSB, // data length 8 bit, MSB
            50000000                // clock rate
        },
    },
    {
        0x01,                       // device ID
        "SPI-DEV2",                 // description
        {
            8 | SPI_MODE_BODER_MSB, // data length 8 bit, MSB
            50000000                // clock rate
        },
    },
    {
        0x02,                       // device ID
        "SPI-DEV3",                 // description
        {
            8 | SPI_MODE_BODER_MSB, // data length 8 bit, MSB
            50000000                // clock rate
        },
    },
    {   
        0x03,                       // device ID
        "SPI-DEV4",                 // description
        {
            8 | SPI_MODE_BODER_MSB, // data length 8 bit, MSB
            50000000                // clock rate
        },
    },
};

uint32_t SPMODE[4] = {0x020, 0x024 ,0x028, 0x02C};


static int ppc_espi_get_uint32(const char *name, char *value, uint32_t *vout)
{
    uint32_t    v;
    char        *c;

    if (value == NULL ) {
        fprintf(stderr, "spi-ppc_espi: %s need input a 32bits integer.\n", name);
        return -1;
    }

    v = strtoul(value, &c, 0);

    if (c == value || *c != '\0') {
        fprintf(stderr, "spi-ppc_espi: %s - bad argument; "
                                    "expecting an integer.\n", name);
        return -1;
    }

    if (vout) {
        *vout = v;
    }

    return 0;
}

static int ppc_espi_get_uint64(const char *name, char *value, uint64_t *vout)
{
    uint64_t    v;
    char        *c;

    if (value == NULL ) {
        fprintf(stderr, "spi-ppc_espi: %s need input a 64bits integer.\n", name);
        return -1;
    }

    v = strtoull(value, &c, 0);

    if (c == value || *c != '\0') {
        fprintf(stderr, "spi-ppc_espi: %s - bad argument; "
                                    "expecting an integer.\n", name);
        return -1;
    }

    if (vout) {
        *vout = v;
    }

    return 0;
}

static int ppc_espi_options(ppc_espi_dev_t *dev, char *optstring)
{
    int     rc = 0, opt;
    char    *options, *freeptr, *c, *value;

    if (optstring == NULL) {
        return 0;
    }

    freeptr = options = strdup(optstring);

    while (options && *options != '\0') {

        c = options;    // save option from getsubopt
        opt = getsubopt(&options, ppc_espi_opts, &value);

        switch (opt) {
            case 0:     // base, 64 bits physical base address
                rc = ppc_espi_get_uint64("base", value, &dev->pbase);
                break;

            case 1:     // irq
                rc = ppc_espi_get_uint32("irq", value, &dev->irq);
                break;

            case 2:     // eSPI source clock
                rc = ppc_espi_get_uint32("clock", value, &dev->clock);
                break;

            case 3:     // loopback mode for SPI test
                dev->loopback = (uint32_t)PPC_ESPI_SPMODE_LOOP;
                break;

            default:
                fprintf(stderr, "spi-ppc_espi: unknown option %s\n", c);
                rc = -1;
                break;
        }
    }

    free(freeptr);
    return rc;
}

/*******************************************************************************
* hwiscan()
* 
* Scan the hwinfo section of the syspage for the specific SPI bus instance
* identified in <dev>->unit and fill in the <dev> attributes based on the
* provided information.
* 
*******************************************************************************/
static void hwiscan(ppc_espi_dev_t *dev)
{
    unsigned spibus;
    hwiattr_spi_t spiattr;

    spibus = hwi_find_bus(HWI_ITEM_BUS_SPI, dev->unit);
    
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u hwiscan ... %s", dev->unit,
                (spibus == HWI_NULL_OFF) ? "no SPI bus entry" : "");

    if (spibus == HWI_NULL_OFF) return;

    /* get the SPI bus attributes */
    if (hwiattr_get_spi(spibus, &spiattr) == EOK)
    {
        hwi_tag *spitag;

        /* register location and length */
        dev->pbase = spiattr.common.location.base;
        dev->len = spiattr.common.location.len;
        
        /* get the interrupt vector */
        dev->irq = hwitag_find_ivec(spibus, NULL);
        if (spiattr.common.num_irq > 1) {
            SPI_SLOGF(_SLOG_WARNING, "SPI unit %u hwiscan reports %u interrupt vectors",
                        dev->unit, spiattr.common.num_irq);
        }
        
        /* get clock source information */
        if (spiattr.num_clks > 0)
        {
            /* currently expect only 1 clock source */
            spitag = hwi_tag_find(spibus, HWI_TAG_NAME_inputclk, NULL);
            if (spitag != NULL) dev->clock = spitag->inputclk.clk;
            if (spiattr.num_clks > 1) {
                SPI_SLOGF(_SLOG_WARNING, "SPI unit %u hwiscan reports %u clock sources",
                            dev->unit, spiattr.num_clks);
            }
        }
    }
}

/*******************************************************************************
* check_params()
* 
* Ensure that the required parameters have been set for the device
* 
* Returns: EOK on success, otherwise EINVAL
* 
*******************************************************************************/
static int check_params(ppc_espi_dev_t *dev)
{
    if (dev->unit < 0) {
        SPI_SLOGF(_SLOG_ERROR, "SPI unit %u params check: invalid unit", dev->unit);
        return EINVAL;
    }
    if (dev->pbase == 0) {
        SPI_SLOGF(_SLOG_ERROR, "SPI unit %u params check: no register base address", dev->unit);
        return EINVAL;
    }
    if (dev->len == 0) {
        SPI_SLOGF(_SLOG_ERROR, "SPI unit %u params check: no register map length", dev->unit);
        return EINVAL;
    }
    if (dev->irq == HWI_ILLEGAL_VECTOR) {
        SPI_SLOGF(_SLOG_ERROR, "SPI unit %u params check: invalid interrupt vector %u", dev->unit, dev->irq);
        return EINVAL;
    }
    if (dev->clock == 0) {
        SPI_SLOGF(_SLOG_ERROR, "SPI unit %u params check: invalid clock rate %d", dev->unit, dev->clock);
        return EINVAL;
    }

    /* guess we're good */
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u params check: ok", dev->unit);
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u params check: register base address is 0x%llx", dev->unit, dev->pbase);
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u params check: register map length is %d", dev->unit, dev->len);
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u params check: interrupt vector is %d", dev->unit, dev->irq);
    SPI_SLOGF(_SLOG_INFO, "SPI unit %u params check: source clock is %d", dev->unit, dev->clock);
    return EOK;
}

void *ppc_espi_init(void *hdl, char *options)
{
    ppc_espi_dev_t  *dev;
    uintptr_t       base;
    uint32_t        spimode, i;
    /*
     * FIX ME - the init function should take a device instance or unit number as
     * an argument since we cannot "see" into <hdl> because spi_dev_t is not a
     * public interface. There is a field in there, devnum we could use for this
     * purpose but for now I will have to keep a local static variable. We need
     * the unit number in order to locate the proper instance of the device in
     * the hwinfo section.
    */
    static unsigned unit_num = 0;

    dev = calloc(1, sizeof(ppc_espi_dev_t));

    if (dev == NULL) {
        return NULL;
    }

    /* initial value for HWI */
    dev->unit  = unit_num++;
    dev->len   = PPC_ESPI_SIZE;
    dev->pbase = PPC_ESPI_BASE_INVALID;
    dev->irq   = HWI_ILLEGAL_VECTOR;
    dev->clock = PPC_ESPI_CLOCK_INVALID;

    /* retrieve HWI information from syspage */
    hwiscan(dev);

    /* parse command line option */
    if (ppc_espi_options(dev, options)) {
        goto fail0;
    }

    /*
     * do a sanity check of <dev> to make sure that the required parameters
     * are set. If not, this is considered an error
     */
    if (check_params(dev) != EOK) {
        SPI_SLOGF(_SLOG_INFO, "SPI check_params failed");
        goto fail0;
    }

    /*
     * Map in SPI register
     */
    if ((base = mmap_device_io(dev->len, dev->pbase)) == (uintptr_t)MAP_FAILED) {
        goto fail0;
    }
    dev->vbase = base;

    /*
     * Set default data format for CSx Mode Register 
     */
    for (i=0; i<4; i++) {
        spimode = ppc_espi_cfg(dev, & devlist[i].cfg);
        if (spimode == 0) {
            goto fail0;
        }
        out32(base + SPMODE[i], spimode);
    }

    /*
     * Attach SPI interrupt
     */
    if (ppc_espi_attach_intr(dev)) {
        goto fail1;
    }

    dev->spi.hdl = hdl;

    return dev;

fail1:
    munmap_device_io(dev->vbase, dev->len);
fail0:
    free(dev);
    SPI_SLOGF(_SLOG_INFO, "SPI ppc_espi_init failed");
    return NULL;
}

void ppc_espi_dinit(void *hdl)
{
    ppc_espi_dev_t  *dev = hdl;

    /*
     * unmap the register, detach the interrupt
     */
    InterruptDetach(dev->iid);
    ConnectDetach(dev->coid);
    ChannelDestroy(dev->chid);

    /*
     * disable eSPI
     */
    out32(dev->vbase + PPC_ESPI_MODE, 0);
    munmap_device_io(dev->vbase, dev->len);

    free(hdl);
}

int ppc_espi_drvinfo(void *hdl, spi_drvinfo_t *info)
{
    info->version = (SPI_VERSION_MAJOR << SPI_VERMAJOR_SHIFT) | (SPI_VERSION_MINOR << SPI_VERMINOR_SHIFT) | (SPI_REVISION << SPI_VERREV_SHIFT);
    strcpy(info->name, "PowerPC Enhanced SPI");
    info->feature = 0;

    return (EOK);
}

int ppc_espi_setcfg(void *hdl, uint16_t device, spi_cfg_t *cfg)
{
    ppc_espi_dev_t  *dev = hdl;
    uint32_t        spimode = 0;
    int             id = device & SPI_DEV_ID_MASK;

    memcpy(& devlist[id].cfg, cfg, sizeof(spi_cfg_t));

    spimode = ppc_espi_cfg(hdl, & devlist[id].cfg);
    if (spimode == 0) {
        return (EINVAL);
    }

    out32(dev->vbase + SPMODE[id], spimode);
    return (EOK);
}

int ppc_espi_devinfo(void *hdl, uint32_t device, spi_devinfo_t *info)
{
    int dev = device & SPI_DEV_ID_MASK;

    if (!(device & SPI_DEV_DEFAULT)) {
        if (dev != SPI_DEV_ID_NONE) {
            dev++;
        } else {
            dev = 0;
        }
    }

    if (dev > 3) {
        return (EINVAL);
    }

    memcpy(info, &devlist[dev], sizeof(spi_devinfo_t));

    return (EOK);
}

void *ppc_espi_xfer(void *hdl, uint32_t device, uint8_t *buf, int *len)
{
    ppc_espi_dev_t  *dev = hdl;
    uintptr_t       base = dev->vbase;
    uint32_t        id, spicmd;

    id = device & SPI_DEV_ID_MASK;

    /* check the id number, support id 0,1,2 and 3 */
    if (id > 3) {
        *len = -1;
        return buf;
    }

    dev->rlen  = 0;
    dev->tlen  = 0;
    dev->xlen  = *len;
    dev->pbuf  = buf;
    dev->dlen  = ((devlist[id].cfg.mode & 0xFF) + 7) >> 3;
    dev->dtime = 1 + (8 * 1000 * 1000 / devlist[id].cfg.clock_rate);

    /* check the character length, support 4-16 bit format */
    if ((dev->dlen > 2) || (dev->dlen < 1)) {
        *len = -1;
        return buf;
    }

    /*
     * disable eSPI
     */
    out32(base + PPC_ESPI_MODE, 0);

    /*
     * clear any previous events
     */
    out32(base + PPC_ESPI_INT_EVENT, 0xFFFFFFFF);

    /*
     * enable eSPI
     */
    out32(base + PPC_ESPI_MODE, PPC_ESPI_SPMODE_EN | PPC_ESPI_SPMODE_INIT_VAL | dev->loopback );

    /*
     * fill as more as transmit character to TxFIFO
     */
    while ((dev->tlen < dev->xlen) && (dev->tlen < PPC_ESPI_FIFO_LEN)) {
        out32(base + PPC_ESPI_TXD, *(uint32_t *)(&buf[dev->tlen]));
        dev->tlen += 4;
    }

    /* set device specific SPCOM register */
    spicmd = in32(base + PPC_ESPI_COMMAND);
    if (1 == dev->dlen) {       // for 4-8 word length
        spicmd |= (dev->xlen & 0xFFFF) - 1;
    } else {                    // for 9-16 word length
        spicmd |= ((dev->xlen >> 1) & 0xFFFF) - 1;
    }
    spicmd |= (id << 30);
    out32(base + PPC_ESPI_COMMAND, spicmd);

    /*
     * enable SPI receive interrupt 
     */
    out32(base + PPC_ESPI_MASK, PPC_ESPI_SPIM_LT | PPC_ESPI_SPIM_RXT);

    /*
     * wait for exchange to finish with timeout
     */
    if (ppc_espi_wait(dev, dev->xlen)) {
         dev->rlen = -1;
    }

    /*
     * disable eSPI
     */
    out32(dev->vbase + PPC_ESPI_MODE, 0);
    *len = dev->rlen;
    
    return buf;
}

__SRCVERSION( "$URL$ $Rev$" );
