/*
 * $QNXLicenseC:
 * Copyright 2011 QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#ifndef _PPC_ESPI_H_INCLUDED
#define _PPC_ESPI_H_INCLUDED

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <pthread.h>
#include <unistd.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/neutrino.h>
#include <sys/slog.h>
#include <sys/hwinfo.h>
#include <hw/inout.h>
#include <hw/spi-master.h>
#include <drvr/hwinfo.h>


#ifndef _SLOGC_SPI
#define _SLOGC_SPI  99
#endif
#define SPI_SLOGF(_lvl_, ...) \
        do { \
            slogf(_SLOGC_SPI, (_lvl_), __VA_ARGS__); \
        } while(0)

#define PPC_ESPI_PRIORITY           21
#define PPC_ESPI_EVENT              1
#define PPC_ESPI_PM_MAX             16
#define PPC_ESPI_CLK_DIV16          16
#define PPC_ESPI_SYSCLK_DIV         2

#define PPC_ESPI_SIZE               0x1000
#define PPC_ESPI_BASE_INVALID       0
#define PPC_ESPI_CLOCK_INVALID      0

#define PPC_ESPI_FIFO_LEN           32
#define PPC_ESPI_FIFO_THR           (PPC_ESPI_FIFO_LEN >> 1)

#define PPC_ESPI_SPIMODE_TXTHR(x)   ((x) << 8)
#define PPC_ESPI_SPIMODE_RXTHR(x)   ((x) << 0)
#define PPC_ESPI_SPMODE_INIT_VAL    (PPC_ESPI_SPIMODE_TXTHR(PPC_ESPI_FIFO_THR) \
                                    | PPC_ESPI_SPIMODE_RXTHR(PPC_ESPI_FIFO_THR - 1))

/*
 * SPI registers, Offset from SPI base
 */
#define PPC_ESPI_MODE               0x00        // eSPI Mode Register
    #define PPC_ESPI_SPMODE_EN      (1<<31)
    #define PPC_ESPI_SPMODE_LOOP    (1 << 30)

    #define PPC_ESPI_SPMODE_CI      (1<<31)
    #define PPC_ESPI_SPMODE_CP      (1<<30)
    #define PPC_ESPI_SPMODE_REV     (1<<29)
    #define PPC_ESPI_SPMODE_DIV16   (1<<28)
    #define PPC_ESPI_SPMODE_ODD     (1<<23)
    #define PPC_ESPI_SPMODE_POL     (1<<20)
    
#define PPC_ESPI_INT_EVENT          0x004       // eSPI Interrupt Event Register
    #define PPC_ESPI_SPIE_LT        (1<<14)
    #define PPC_ESPI_SPIE_NE        (1<<9)
    #define PPC_ESPI_SPIE_NF        (1<<8)
    #define PPC_ESPI_SPIE_RXT       (1<<13)
    
#define PPC_ESPI_MASK               0x008       // eSPI mask Register
    #define PPC_ESPI_SPIM_LT        (1<<14)
    #define PPC_ESPI_SPIM_NE        (1<<9)
    #define PPC_ESPI_SPIM_NF        (1<<8)
    #define PPC_ESPI_SPIM_TXT       (1<<11)
    #define PPC_ESPI_SPIM_RXT       (1<<13)
    #define PPC_ESPI_SPIM_TXE       (1<<15)
    
#define PPC_ESPI_COMMAND            0x00C       // eSPI command Register
#define PPC_ESPI_TXD                0x010       // SPI transmit Register
#define PPC_ESPI_RXD                0x014       // SPI receive Register

#define SPI_MODE_CS_CFG_MASK        0x3E0000    // bit numbers 17 - 21
#define SPI_MODE_CS_CFG_BIT_SHIFT   17          // bit numbers 17 - 21
#define SPI_MODE_CS_AFT_MASK        0x3C00000   // bit numbers 22 - 25
#define SPI_MODE_CS_AFT_BIT_SHIFT   22          // bit numbers 22 - 25
#define SPI_MODE_CS_BEF_MASK        0x3C000000  // bit numbers 26 - 29
#define SPI_MODE_CS_BEF_BIT_SHIFT   26          // bit numbers 26 - 29

#define SPI_CMD_RXSKIP_MASK         (0xFF)      // RXSKIP mask
#define SPI_CMD_RXDLY               (1 << 8)
#define SPI_CMD_DO                  (1 << 9)
#define SPI_CMD_TO                  (1 << 10)
#define SPI_CMD_HLD                 (1 << 11)

typedef struct {
    SPIDEV      spi;                // This has to be the first element

    int         unit;               // device instance number

    uint64_t    pbase;              // memory mapping parameters
    uintptr_t   vbase;
    size_t      len;

    uint32_t    irq;                // interrupt vector to use in InterruptAttach() calls
    int         iid;                // 'id' returned from InterruptAttach()
    int         chid;               // 'id' returned from ChannelCreate()
    int         coid;               // 'id' returned from ConnectAttach()

    uint32_t    clock;              // source clock for the baudrate generator

    uint8_t     *pbuf;              // data buffer and data transaction parameters
    int         xlen, tlen, rlen;
    int         dlen;
    int         dtime;              // uSec per data, for time out use

    uint32_t    loopback;           // loopback mode

    struct sigevent spievent;
} ppc_espi_dev_t;

    
extern void *ppc_espi_init(void *hdl, char *options);
extern void ppc_espi_dinit(void *hdl);
extern void *ppc_espi_xfer(void *hdl, uint32_t device, uint8_t *buf, int *len);
extern int ppc_espi_setcfg(void *hdl, uint16_t device, spi_cfg_t *cfg);

extern int ppc_espi_devinfo(void *hdl, uint32_t device, spi_devinfo_t *info);
extern int ppc_espi_drvinfo(void *hdl, spi_drvinfo_t *info);

extern int ppc_espi_attach_intr(ppc_espi_dev_t *ppc_espi);
extern int ppc_espi_wait(ppc_espi_dev_t *dev, int len);

extern int ppc_espi_cfg(void *hdl, spi_cfg_t *cfg);

#endif

__SRCVERSION( "$URL$ $Rev$" );
