/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"
#include <ppc/p4080cpu.h>
#include <pic_p40xx.h>
#include <hw/hwinfo_p40xx.h>
#include <drvr/hwinfo.h>

#ifndef NDEBUG
static void dump_vec_tbl(vector_tbl_t *rv);
#define DUMP_VEC_TBL(x)		dump_vec_tbl(x)
#else	/* NDEBUG */
#define DUMP_VEC_TBL(x)
#endif	/* NDEBUG */

/*
 * the size of the offset and configuration tables is based on the number of
 * interrupt vectors configured. This will not likely be known the first time
 * you write this file. If not, put a large value (like 256) for the size of
 * these 2 arrays and then execute startup with -vv so that it will output
 * the number of vectors that were initialized. This value is used for these
 * 2 arrays and for the values in vec_cfg_table.c. Note that if there will be
 * more than 1 startup_intrinfo entry for the same interrupt controller, then
 * the array size must be the sum of all of the vectors for each entry
*/
static reg_off_t offset_table[98];
static cfg_flags_t config_flags_table[98];

/* use a different patch data structure for each startup_intrinfo entry as required */
static struct
{
	paddr_t			*immr;
	vector_tbl_t	reg_vectors;
} patch_data = {&immr_paddr, {0, 0, 0, NULL, NULL}};

static struct startup_intrinfo intrs[] =
{
	{
		.vector_base = PPC_INTR_CLASS_85xx,
		.num_vectors = 0,	/* will be filled in by p4080_config_pic() */
		.cascade_vector = _NTO_INTR_SPARE,
		.cpu_intr_base = PPCBKE_SPR_IVOR4,
		.cpu_intr_stride = 0,
		.flags = 0,
		.id = {.genflags = 0, .size = 0, .rtn = &interrupt_id_ppc85xx},
		.eoi = {.genflags = 0, .size = 0, .rtn = &interrupt_eoi_ppc85xx},
		.mask = &interrupt_mask_ppc85xx,
		.unmask = &interrupt_unmask_ppc85xx,
		.config = &interrupt_config_ppc85xx,
		.patch_data = &patch_data,
	},
	{
		.vector_base = PPC_INTR_DECREMENTER,
		.num_vectors = 1,
		.cascade_vector = _NTO_INTR_SPARE,
		.cpu_intr_base = PPCBKE_SPR_IVOR10,
		.cpu_intr_stride = 0,
		.flags = 0,
		.id = {.genflags = INTR_GENFLAG_NOGLITCH, .size = 0, .rtn = &interrupt_id_booke_dec},
		.eoi = {.genflags = INTR_GENFLAG_LOAD_INTRMASK, .size = 0, .rtn = &interrupt_eoi_booke_dec},
		.mask = &interrupt_mask_booke_dec,
		.unmask = &interrupt_unmask_booke_dec,
		.config = NULL,
		.patch_data = NULL,
	},
	{
		.vector_base = PPC_INTR_FIT,
		.num_vectors = 1,
		.cascade_vector = _NTO_INTR_SPARE,
		.cpu_intr_base = PPCBKE_SPR_IVOR11,
		.cpu_intr_stride = 0,
		.flags = 0,
		.id = {.genflags = INTR_GENFLAG_NOGLITCH, .size = 0, .rtn = &interrupt_id_booke_fit},
		.eoi = {.genflags = INTR_GENFLAG_LOAD_INTRMASK, .size = 0, .rtn = &interrupt_eoi_booke_fit},
		.mask = &interrupt_mask_booke_fit,
		.unmask = &interrupt_unmask_booke_fit,
		.config = NULL,
		.patch_data = NULL,
	},
	{
		.vector_base = PPC_INTR_WATCHDOG,
		.num_vectors = 1,
		.cascade_vector = _NTO_INTR_SPARE,
		.cpu_intr_base = PPCBKE_SPR_IVOR12,
		.cpu_intr_stride = 0,
		.flags = PPC_INTR_FLAG_CI,
		.id = {.genflags = INTR_GENFLAG_NOGLITCH, .size = 0, .rtn = &interrupt_id_booke_watchdog},
		.eoi = {.genflags = INTR_GENFLAG_LOAD_INTRMASK, .size = 0, .rtn = &interrupt_eoi_booke_watchdog},
		.mask = &interrupt_mask_booke_watchdog,
		.unmask = &interrupt_unmask_booke_watchdog,
		.config = NULL,
		.patch_data = NULL,
	},
};


/*******************************************************************************
* init_intrinfo()
*
* Configure the interrupt controller for the board and set the interrupt vector
* attribute for any devices added in init_hwinfo().
*
* init_hwinfo() should be called BEFORE init_intrinfo() so that as interrupt
* vectors are assigned to the devices, the corresponding IRQ tags for the device
* can be filled in with the hwitag_set_ivec() call (see pic_p4080.c for an example)
*
*/
void init_intrinfo()
{
	extern long vec_regoffset_table_size;
	extern long vec_config_flags_table_size;
	unsigned total_vectors = 0;
	unsigned i;

	patch_data.reg_vectors.offset = &offset_table[0];
	patch_data.reg_vectors.config_flags = &config_flags_table[0];
	patch_data.reg_vectors.dst_tbl_off = 0;

	/* by default, disallow attachment to any vectors until they are explitly setup */
	for (i=0; i<(sizeof(config_flags_table)/sizeof(config_flags_table[0])); i++)
		set_config_flags(&patch_data.reg_vectors, i, INTR_CONFIG_FLAG_DISALLOWED);

	intrs[0].num_vectors = p4080_config_pic(intrs[0].vector_base, intrs[0].vector_base, &patch_data.reg_vectors);
	total_vectors += intrs[0].num_vectors;

	/* sanity check the dst_vectbl_off and vector assignments. The current callouts
	 * (and instructions) assume a signed 16 bit positive value (ie 0 -> 32767) for
	 * the vector table location and min/max vectors limited to 16 bits */
	ASSERT((patch_data.reg_vectors.dst_tbl_off & ~32767) == 0);
	ASSERT((patch_data.reg_vectors.num + patch_data.reg_vectors.start) <= 65536);

	if (debug_flag >= 2)
	{
		kprintf("Initialized %d vectors starting @ %d\n", intrs[0].num_vectors, intrs[0].vector_base);
		if (debug_flag >= 3) DUMP_VEC_TBL(&patch_data.reg_vectors);
	}

	/*
	 * At this point, all P4080 interrupting devices have been assigned vectors.
	 * If there are any cascaded interrupt controllers, they can be dealt with now.
	 * Simply define or initialize another 'struct startup_intrinfo' and initialize
	 * it appropriately. Knowing the hwi device name (defined in hw/hwinfo_p40xx.h) to
	 * which the interrupt to be cascaded was assigned, the .cascade_vector field
	 * for this additional 'struct startup_intrinfo' entry can be set as follows ...
	 * 		extra_intrs[0].cascade_vector = hwi_find_ivec(hwi_find_device("device", unit), NULL);
	 *
	 * For example, if there was an interrupting device attached to GPIO pin 0 then
	 * then the .cascade_vector would be set with
	 * 		extra_intrs[0].cascade_vector = hwi_find_ivec(hwi_find_device("gpio", 0), NULL);
	 * because we know that there is only 1 P4080 GPIO device ('0' param to hwi_find_device())
	 * and only 1 GPIO device interrupt ('NULL' param to hwi_find_ivec()). Of course you will
	 * need ensure that hwi_find_ivec() does not return HWI_ILLEGAL_VECTOR and add the callouts
	 * that will read the GPIO registers within the P4080 and deal with them.
	 *
	 * Also, if there are any interrupt vectors that need to be initialized for on board devices
	 * that are not part of the p4080_config_pic() initialization, then do them now
	*/

	/* PCIe x (INT B,C,D Signals) uses same interrupt vectors as EXT_IRQ */
	{
		unsigned irq, irq_tmp;
		unsigned extirq_dev;
		unsigned ivec;

		extirq_dev = hwi_find_device(PPCP40xx_HWI_EXT_IRQ, 0);

		/*
		 * The PCIe INTB-INTD interrupts are the last 3 irqs for each group of 4 in EXT_IRQ
		 * for the 3 possible PCIe buses
		 */
		for (irq=0; irq<12; irq++)
		{
			irq_tmp = irq;
			if ((irq%4) != 0) {
				ivec = hwitag_find_ivec(extirq_dev, &irq_tmp);
				hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_PCIe, irq/4), (irq%4), ivec);
			}
		}
	}

	/* sanity check the size of the vector register offset table with the
	 * number of calculated vectors and adjust the vec_regoffset_table_size
	 * as required. Same for the config flags table */
	ASSERT((sizeof(offset_table)/sizeof(offset_table[0])) == total_vectors);
	ASSERT(vec_regoffset_table_size == sizeof(offset_table));
	ASSERT(vec_regoffset_table_size == vec_config_flags_table_size);

	/* add the interrupt info section */
	add_interrupt_array(intrs, sizeof(intrs));
}

#ifndef NDEBUG
static void dump_vec_tbl(vector_tbl_t *rv)
{
	unsigned i;

	kprintf("\nVector/Offset & Config Flags tables (n=%d, s=%d, dvo=%d)\n"
			  "-------------\n", rv->num, rv->start, rv->dst_tbl_off);
	for (i=0; i<rv->num; i++)
		kprintf("[%d] %d  off=0x%x, cf=0x%x\n", i, i+rv->start, rv->offset[i], rv->config_flags ? rv->config_flags[i] : 0);
	kprintf("\n");
}
#endif	/* NDEBUG */

__SRCVERSION( "$URL: $ $Rev: $" );
