/*
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not
 * reproduce, modify or distribute this software except in compliance with the
 * License. You may obtain a copy of the License at:
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as contributors under
 * the License or as licensors under other terms.  Please review this entire
 * file for other proprietary rights or license notices, as well as the QNX
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/
 * for other information.
 */

/*
 * init_raminfo.c
 * Tell syspage about our RAM configuration
 */
#include "startup.h"
#include "hwinfo_p40xx_proto.h"
#include <ppc/p4080cpu.h>

// The second memory controller on the P4080 is at offset 0x9000
#define PPC4080_CCSR_OFF_DDR_MCR_2      0x09000
#define CS0_BNDS						0x00
#define CS1_BNDS						0x08
#define CS0_CONFIG						0x80
#define CS1_CONFIG						0x84
#define CSn_EN_BIT						31
#define INTLV_EN						0x20000000

static uint32_t	ddr_interleaving = 0; // Flag to check whether DDR interleaving is enabled or not

uint32_t is_ddr_interleaved()
{
	return ddr_interleaving;
}

void init_raminfo()
{
	uintptr_t	ccsr;
	ccsr = startup_io_map(MEG(1), immr_paddr);
	if(in32(ccsr + PPCP4080_CCSR_OFF_CSn_CONFIG(0)) & INTLV_EN){
		ddr_interleaving = 1;
	}

	kprintf("\nDDR Interleave = %d\n", ddr_interleaving);

#ifndef ALLOW_SDRAM_IO_OVERLAP
	if(!ddr_interleaving){
	
		/* The U-Boot is setting up the second 2GB RAM at the location 0x080000000. Since the 2GB space from 0x080000000 - 0x0FFFFFFFF
	 	* is used by the CCSR space, Peripherals, NOR, PCI etc. the second 2GB RAM has been moved to the location 0x100000000.
	 	* So, thereby we are configuring the DDR Controller Chip selects to take the address 0x100000000*/
	 
		/* Disabling the DDR Controller 2's Chip Selects 0, 1 which are configured earlier by U-Boot */ 
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS0_CONFIG), in32(ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS0_CONFIG) & ~(1 << CSn_EN_BIT));
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS1_CONFIG), in32(ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS1_CONFIG) & ~(1 << CSn_EN_BIT));	
	
		/* Configuring the addresses for the DDR Controller 2's Chip Select Memory Bounds Registers 0, 1 */
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS0_BNDS), 0x0100013f);
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS1_BNDS), 0x0140017f);

		/* Enabling the DDR Controller 2's Chip Selects 0, 1 with the changed addresses */ 	
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS0_CONFIG), in32(ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS0_CONFIG) | (1 << CSn_EN_BIT));
		out32((ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS1_CONFIG), in32(ccsr + PPC4080_CCSR_OFF_DDR_MCR_2 + CS1_CONFIG) | (1 << CSn_EN_BIT));
	
	}
#endif

	startup_io_unmap(ccsr);	
	/*
	 * Collect information on all free RAM in the system associated to the
     * primary controller.
	 */
	ppcp40xx_init_raminfo_extended(PPCP4080_CCSR_OFF_CSn_BNDS(0));

	/*
	 * Collect information on all free RAM in the system associated to the
     * secondary controller. No need in the case of DDR interleaved U-boot, as the second controller's Config registers are not initialised
	 */
	if(!ddr_interleaving && ppcp4080_law_find_target(ppcp40xx_law_target_id_SDRAM_2, NULL) != -1){
		ppcp40xx_init_raminfo_extended(PPC4080_CCSR_OFF_DDR_MCR_2);
	}
	kprintf("\n");

#ifdef ALLOW_SDRAM_IO_OVERLAP
	if(find_ram_in_range(immr_paddr, immr_paddr+MEG(16), MEG(16), 1, 0, 0) != NULL_PADDR)
	    kprintf("Removing IO shadowed SDRAM from available memory: %P-%P (IMMR/CCSR)\n\n", immr_paddr, immr_paddr+MEG(16)-1);
    alloc_ram(immr_paddr, MEG(16), 1); /* remove IMMR region from memory pool if it overlaps SDRAM */
#endif
}

__SRCVERSION( "$URL: $ $Rev: $" );
