#
# Copyright 2007, 2008, QNX Software Systems. 
# 
# Licensed under the Apache License, Version 2.0 (the "License"). You 
# may not reproduce, modify or distribute this software except in 
# compliance with the License. You may obtain a copy of the License 
# at: http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OF ANY KIND, either express or implied.
#
# This file may contain contributions from others, either as 
# contributors under the License or as licensors under other terms.  
# Please review this entire file for other proprietary rights or license 
# notices, as well as the QNX Development Suite License Guide at 
# http://licensing.qnx.com/license-guide/ for other information.
#


#
#
# An entry point for a hardware kernel callout.
# It may be called by an interrupt handler.
# This code MUST be position independant.
#

	.include "callout.ah"
#
# Interrupts are always off in this code.
#
# Note that interrupt_id_* and interrupt_eoi_* are not actually
# called by the kernel. For performance reasons, they are instead
# copied into place and intermixed with other kernel code. As such,
# they don't follow normal calling conventions. They should just fall out 
# their bottoms when done rather than attempting to do a return 
# instruction. The 'id' routine should return the interrupt level in
# the r14 register. The 'eoi' routine will find it still there. 
# By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
# the following values can be loaded before entry to this code.
# 
#		cpu number			=> r15
#		syspage_ptr			=> r16
#		intrinfo_entry		=> r17
#		intr mask count		=> r18 (only available in EOI routine)
#
# If the INTR_FLAG_CPU_FAULT bit is on, the eoi routine should set
# the R3 register to the following value to indicate the exception
# that has occurred:
# 		R3 = (fault_number << 16) | (signal_code << 8) | (signal_number)
# Set R3 to zero if no exception has occured. If the exception is memory
# related, set R4 to referenced address that caused the exception, zero
# if there is no address.
#

# callouts for systems with legacy 8259 interrupt controllers which don't 
# have a dedicated PCI_INTACK register for getting the vector. The ID 
# callout polls the master and slave for the vector. The 'legacy_io_base'
# used below in the patcher routine should be passed in the init_intrinfo
# routine of the startup code. 


.set ISA_8259M0_OFF,		0x20
.set ISA_8259M1_OFF,		0x21
.set ISA_8259S0_OFF,		0xa0
.set ISA_8259S1_OFF,		0xa1
.set ISA_MAPPING_SIZE,		0x100

.set ISA_8259M_VEC,           0X08
.set ISA_8259S_VEC,           0X80
.set ISA_VEC_MASK,            0X07

# R3: syspage paddr
# R4: syspage vaddr
# R5: rtn offset
# R6: r/w offset
# R7: patch data
# R8: rtn src

patcher:
	stwu	%r1,-32(%r1)
	mflr	%r0
	stw		%r31,28(%r1)
	stw		%r0,36(%r1)
	
	add		%r31,%r3,%r5
	mr		%r4,%r7
	loadi	%r3,ISA_MAPPING_SIZE
	bl		callout_io_map_indirect

	lhz		%r4,6(%r31)
	lhz		%r5,2(%r31)
	slwi	%r5,%r5,16
	or		%r4,%r4,%r5
	add		%r3,%r3,%r4
	sth		%r3,6(%r31)
	srwi	%r3,%r3,16
	sth		%r3,2(%r31)
	
	lwz		%r0,36(%r1)
	mtlr	%r0
	lwz		%r31,28(%r1)
	addi	%r1,%r1,32
	blr

.macro DEVBASE reg,base_paddr
	# the instruction immediate values are patched by the above routine
	lis	&reg,(&base_paddr) >> 16
	ori	&reg,&reg,(&base_paddr) & 0xffff
.endm
		
# 
# int interrupt_id(void)
#
# Returns: interrupt level, masks it off
#
CALLOUT_START	interrupt_id_isa8259_poll, 0, patcher
	DEVBASE	%r5, 0

    # poll master
    li      %r0,0x0C
    stb     %r0,ISA_8259M0_OFF(%r5)
    eieio
    lbz     %r0,ISA_8259M0_OFF(%r5)

	li		%r3,0

	andi.	%r6,%r0,ISA_VEC_MASK
	cmplwi	%r6,2
	beq		1f		# interrupt on slave

	addi	%r5,%r5,ISA_8259M1_OFF	#put M1 addr in r5 for masking

#test glitch
	andi.	%r6,%r0,ISA_VEC_MASK
	cmplwi	%r6,7
	bne		2f
	li		%r14, -1
	b		3f # glitch


1:  # 8259-2

    # poll slave
    li      %r0,0x0C
    stb     %r0,ISA_8259S0_OFF(%r5)
    eieio
    lbz     %r0,ISA_8259S0_OFF(%r5)
	
	addi	%r5,%r5,ISA_8259S1_OFF	#put S1 addr in r5 for masking
    li      %r3,8
2:
	andi.   %r0,%r0,ISA_VEC_MASK
    add     %r3,%r0,%r3

    li      %r6,1
    slw     %r0,%r6,%r0
    lbz     %r6,0(%r5)
    eieio
    or      %r6,%r0,%r6
    stb     %r6,0(%r5)          # mask int
    eieio
    mr      %r14,%r3
3:
	eieio
CALLOUT_END	interrupt_id_isa8259_poll
		

# 
# void interrupt_eoi(struct syspage_entry *sysp, int intr)
#
# Send an end-of-interrupt, then unmask the level (interrupt_id masked it)
#
CALLOUT_START	interrupt_eoi_isa8259_poll, 0, patcher
	DEVBASE %r5, 0

    #eoi master
    li  %r6,0x20
    stb %r6,ISA_8259M0_OFF(%r5)       # eoi
    eieio

    cmplwi  %r14,8
    bge 1f

	mr	%r0,%r14
	addi	%r5,%r5,ISA_8259M1_OFF	# put M1 addr in r5 for unmasking
	b	2f
1:
	subi	%r0,%r14,8

    #eoi slave
    li  %r6,0x20
    stb %r6,ISA_8259S0_OFF(%r5)      # eoi
    eieio
	addi	%r5,%r5,ISA_8259S1_OFF	# put S1 addr in r5 for unmasking

2:
    cmplwi  %r18,0
    bgt 3f

    li  %r6,1
    slw %r0,%r6,%r0
    lbz %r6,0(%r5)
    eieio
    andc    %r6,%r6,%r0
    stb %r6,0(%r5)          # unmask
    eieio
3:
CALLOUT_END	interrupt_eoi_isa8259_poll


#
# int interrupt_unmask(struct syspage_entry *sysp, int intr)
#
# Unmask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_unmask_isa8259, 0, patcher
	DEVBASE %r5, ISA_8259M1_OFF

	cmplwi 	%r4,8
	blt		1f
	subi	%r4,%r4,8	
	addi	%r5,%r5,ISA_8259S1_OFF-ISA_8259M1_OFF	# get 8259-2 OCW1 base
1:
	li		%r6,1
	slw		%r6,%r6,%r4
	lbz		%r0,0(%r5)
	eieio
	andc	%r0,%r0,%r6
	stb		%r0,0(%r5)		
	eieio
	
	li		%r3,0
	blr
CALLOUT_END	interrupt_unmask_isa8259
	
#
# void interrupt_mask(struct syspage_entry *sysp, int intr)
#
# Mask the passed hardware interrupt number.
#
CALLOUT_START	interrupt_mask_isa8259, 0, patcher
	DEVBASE %r5, ISA_8259M1_OFF

	cmplwi 	%r4,8
	blt		1f
	subi	%r4,%r4,8	
	addi	%r5,%r5,ISA_8259S1_OFF-ISA_8259M1_OFF	# get 8259-2 OCW1 base
1:
	li		%r6,1
	slw		%r6,%r6,%r4
	lbz		%r0,0(%r5)
	eieio
	or		%r0,%r0,%r6
	stb		%r0,0(%r5)		
	eieio
	
	li		%r3,0
	blr
CALLOUT_END	interrupt_mask_isa8259
