/*
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not
 * reproduce, modify or distribute this software except in compliance with the
 * License. You may obtain a copy of the License at:
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as contributors under
 * the License or as licensors under other terms.  Please review this entire
 * file for other proprietary rights or license notices, as well as the QNX
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/
 * for other information.
 */

/*
 *       RETRIEVE CONFIGURATION DATA FROM EEPROM ON FREESCALE BOARDS
 * 
 * There is not 1 routine to read the entire EEPROM. This is intentional.
 * Instead, routines exist for the specific data which is desired. this keeps
 * things much cleaner and allows EEPROM data format changes (which can only be
 * expected from Freescale) to be contained to this one file.
 * 
 * The reading of the EEPROM assumes connection on an I2C bus. The I2C bus
 * callouts are defined in 'i2c_funcs'. A default I2C driver (in mpc8xxx_i2c.c)
 * should be able to handle most Freescale boards however if you need to tweak
 * it or implement a new one then you can copy lib/ppc/mpc8xxx_i2c.c into the
 * BSP directory and those callouts will be linked in instead.
 * 
 */
#include "startup.h"
#include <drvr/hwinfo.h>
#include <errno.h>

/* EEPROM content layout. This is taken from the MPC8536 document */
typedef struct
{
	uint8_t		id[4];		/* offset 0x0000 - 0x0003 */
	uint8_t		sn[12];		/* offset 0x0004 - 0x000F */
	uint8_t		errata[5];	/* offset 0x0010 - 0x0014 */
	uint8_t		date[6];	/* offset 0x0015 - 0x001a */
	uint8_t		res_0;		/* offset 0x001b, reserved */
	uint8_t		version[4];	/* offset 0x001c - 0x001f */
	uint8_t		res_1[32];	/* offset 0x0020 - 0x003f */
	uint8_t		num_macs;	/* offset 0x0040 */
	uint8_t		mac_flag;	/* offset 0x0041 */
	uint8_t		mac[8][6];	/* offset 0x0042 - 0x0071 */
	uint8_t		crc[4];		/* offset 0x0072 crc32 checksum */
} eeprom_data_t;


static struct
{
	unsigned	initialized;	// 0 == false, 1 == true
	unsigned	unit;
	uint8_t		i2c_addr;
	size_t		size;
} eeprom_i2c_params = {.initialized = 0};	// just to be explicit

/*
 * i2c_funcs
 * 
 * This allows the BSP to hook in the proper I2C driver overriding the default
*/
extern struct
{
	void (*init)(uint64_t addr, unsigned divider);
	void (*done)(void);
	int (*xfer)(uint8_t i2c_addr, uint8_t *send_data, size_t send_length, uint8_t *recv_buffer, size_t recv_length);
} *i2c_funcs;

#define I2C_INIT(a, b)				do {if (i2c_funcs != NULL) {i2c_funcs->init((a), (b));}} while(0)
#define I2C_DONE()					do {if (i2c_funcs != NULL) {i2c_funcs->done();}} while(0)
#define I2C_XFER(a, b, c, d, e)		((i2c_funcs != NULL) ? i2c_funcs->xfer((a), (b), (c), (d), (e)) : -1)


/* protos for support functions */
static eeprom_data_t *read_eeprom(void);
static unsigned is_valid_mac(uint8_t mac[][6], unsigned num_macs);

/*
 * =============================================================================
 *                                  A P I's
 * 
 *         The following section contains the API's available to the BSP.
 *                       Add to it as the need arises
 * =============================================================================
*/

/*******************************************************************************
 * init_i2c_eeprom_read
 * 
 * Initialize parameters for reading of an EEPROM on an I2C bus.
 * The required parameters are
 * 		- I2C bus unit
 * 		- EEPROM I2C bus address
 * 		- EEPROM size
 * 
 * Returns: EOK if initialization was successful, otherwise EINVAL
*/
int init_i2c_eeprom_read(unsigned unit, uint8_t i2c_addr, size_t size)
{
	if (unit >= hwi_find_num_units(HWI_ITEM_BUS_I2C)) return EINVAL;
	if (size == 0) return EINVAL;
// Any illegal I2C addresses ?	if (i2c_addr == ??) return EINVAL;
	
	eeprom_i2c_params.unit = unit;
	eeprom_i2c_params.i2c_addr = i2c_addr;
	eeprom_i2c_params.size = size;
	eeprom_i2c_params.initialized = 1;
	
	return EOK;
}

/*******************************************************************************
 * read_eeprom_mac
 * 
 * Read the ethernet MAC addresses from the board EEPROM
 * The caller must provide enough storage for <num> 6 byte MAC addresses.
 * 
 * Returns: the number of MAC addresses found.
 * 			As long as the return value is >= to <num>, <mac> will be filled
 * 			in with the correct data. A value returned that is < <num> indicates
 * 			that fewer than the requested number were available.
*/
int read_eeprom_mac(uint8_t mac[][6], unsigned num)
{
	eeprom_data_t *epd = read_eeprom();

	if (epd != NULL)
	{
		unsigned i, n = min(num, min(epd->num_macs, 8));
		for (i=0; i<n; i++) {
			memcpy(mac[i], epd->mac[i], sizeof(epd->mac[0]));
		}
		return i;
	}
	return 0;
}


/*
 * =============================================================================
 *                    S U P P O R T   F U N C T I O N S
 * =============================================================================
*/
static unsigned crc32(unsigned crc, uint8_t *buffer, size_t size);

/*******************************************************************************
 * read_eeprom
 * 
 * Read the board EEPROM on the I2C bus.
 * 
 * Returns: a pointer to an eeprom_data_t on success or NULL on error
*/
static eeprom_data_t *read_eeprom(void)
{
	static eeprom_data_t eeprom_data = {.id={0}};
		
	/* if we already read it, give back the pointer */
	if (memcmp(eeprom_data.id, "NXID", sizeof(eeprom_data.id)) == 0) {
		return &eeprom_data;
	}
	else
	{
		/* have not successfully read out the EEPROM yet. Let's try */
		unsigned hwi_off;
		eeprom_data_t *ret = NULL;

		if (!eeprom_i2c_params.initialized)
		{
			kprintf("\n** You must call init_i2c_eeprom_read() first **\n");
			ASSERT(eeprom_i2c_params.initialized);
		}

		if ((hwi_off = hwi_find_bus(HWI_ITEM_BUS_I2C, eeprom_i2c_params.unit)) != HWI_NULL_OFF)
		{
			hwi_tag *addr_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_location, NULL);
			hwi_tag *clk_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_inputclk, NULL);
			
			if ((addr_tag == NULL) || (clk_tag == NULL)) return NULL;
			else
			{
				int r;
				uint32_t eeprom_address = 0;		// we always read the entire eeprom
				size_t eeprom_address_size = 1 +
											 (((eeprom_i2c_params.size - 1) & 0xFF00U) ? 1 : 0) +
											 (((eeprom_i2c_params.size - 1) & 0xFF0000U) ? 1 : 0) +
											 (((eeprom_i2c_params.size - 1) & 0xFF000000U) ? 1 : 0);

				/* initialize access to the controllers registers. We will target a 100Kb/s bus rate */
				if (debug_flag > 1) kprintf("i2c_init(%x, %d)\n", (uint32_t)addr_tag->location.base, clk_tag->inputclk.clk / 100000);
				I2C_INIT(addr_tag->location.base, clk_tag->inputclk.clk / 100000); 

				if (debug_flag) {
					kprintf("Looking for Config EEPROM on %s,%d @ I2C address 0x%x ... ",
								HWI_ITEM_BUS_I2C, eeprom_i2c_params.unit, eeprom_i2c_params.i2c_addr);
				}
				//kprintf("eeprom_address_size = %d\n", eeprom_address_size);

				/* read the EEPROM device at the specified address */
				r = I2C_XFER(eeprom_i2c_params.i2c_addr, (uint8_t *)&eeprom_address, eeprom_address_size,
								(uint8_t*)&eeprom_data, sizeof(eeprom_data));
				if (r != sizeof(eeprom_data))
				{
					if (debug_flag) kprintf("\n\tEEPROM read failure %d\n", r);
					memset(&eeprom_data, 0, sizeof(eeprom_data));	// make sure to erase any garbage
				}
				else
				{
					if (debug_flag) kprintf("found\nValidating contents ... ");

					/* optionally dump the EEPROM contents */
					if (debug_flag > 1)
					{
						unsigned i;
						for (i=0; i<sizeof(eeprom_data); i++)
						{
							if ((i % 8) == 0) kprintf("\n[%d]: ", i);
							kprintf("0x%x ", ((uint8_t *)&eeprom_data)[i]);
						}
						kprintf("\n\n");
					}

					/* The first 4 bytes should be the "NXID" tag */
					if (memcmp(eeprom_data.id, "NXID", sizeof("NXID")-1) == 0)
					{
						/* found the EEPROM data */
						unsigned int  calc_crc;

						/* make sure the CRC checks out */
						calc_crc =  crc32(0xffffffff, (uint8_t *)&eeprom_data, sizeof(eeprom_data) - sizeof(eeprom_data.crc));
						
#if 0	/* FIX ME - see comment below for crc32 */
						if ((calc_crc != *((uint32_t *)eeprom_data.crc)) && (is_valid_mac(eprom_data.mac, eprom_data.num_macs)))
#else
						if (debug_flag) kprintf("NOTE: CRC check disabled ... ");
						if (is_valid_mac(eeprom_data.mac, eeprom_data.num_macs))
#endif
						{

							if (debug_flag > 1) {
								void *temp;
								temp = (void *)eeprom_data.crc;
								kprintf("calc_crc = %x, eeprom crc: %x\n",  calc_crc, *(uint32_t *)temp);
							}
							if (debug_flag) kprintf("CRC or MAC validation failure\n");
							ret = NULL;
						}
						else
						{
							/* EEPROM checks out, so return a pointer to the data */
							if (debug_flag) kprintf("Ok\n");
							ret = &eeprom_data;
						}
					}
					else if (debug_flag) kprintf("Error, no signature\n");
				}
				I2C_DONE();
			}
		}
		return ret;
	}
}

/*******************************************************************************
 * is_valid_mac
 * 
 * validate whether the MAC addresses appear to be valid values.
 * Either <num_macs> will be valid or they will all be considered invalid.
 * 
 * Returns: true (1) or false (0) depending on whether <num_macs> are valid.
 *
*/
static unsigned is_valid_mac(uint8_t mac[][6], unsigned num_macs)
{
#define CHECK_VAL(_m_, _v_) \
		((_m_)[0] == (_v_) && (_m_)[1] == (_v_) && (_m_)[2] == (_v_) && \
		 (_m_)[3] == (_v_) && (_m_)[4] == (_v_) && (_m_)[5] == (_v_))

	unsigned i;

	if (num_macs > 8) return 0;	// invalid

	for (i=0; i<num_macs; i++)
	{
		/* byte 0, bit 0 must be 0 */
		if (mac[i][0] & 0x1) return 0;
		/* first 3 octets must not all be 0 */
		if ((mac[i][0] == 0x0) && (mac[i][0] == 0x0) && (mac[i][0] == 0x0)) return 0;
		/* all 0xFF (ie. unprogrammed flash value) is invalid */
		if (CHECK_VAL(mac[i], 0xff)) return 0;
		/* all 0x00 is invalid */
		if (CHECK_VAL(mac[i], 0x00)) return 0;
		/* if there are any duplicates, this will be considered invalid */
		/* FIX ME - deciding not to enforce this as perhaps someone wants
		 * to put the same MAC on a different network
		if (i > 0) && (CHECK_DUP(mac[i], mac[i-1], i)) return 0;
		*/
	}
	return 1;
}

/*******************************************************************************
 * crc32
 * 
 * Calculate a 32bit CRC across <size> bytes of <buffer>
 * 
 * FIX ME
 * 
 * Below is the algorithm that exists in the mpc8347-ge init_hwinfo.c file for
 * calculating a 32bit crc. It does not work for the 8536. Also, the 8347-ge
 * retrieve_mac_addr() routine uses a hardcoded buffer size of 128 bytes which
 * does not line up with the EEPROM data definition in the 8536/8572 manuals
 * (which you would think would be consistent across all freescale boards) which
 * is 114 bytes + 4 byte CRC. If the 8347 really has the same EEPROM content
 * definition as the 8536/8572 then I don't know how that board even works for
 * retrieving the MAC address from EEPROM. Regardless, even the standard crc32
 * algorithm (which uses the crctab32[]) does not generate the same CRC as U-BOOT
 * so I have know idea what U-BOOT is doing. I therefore have no other choice
 * but to ignore the CRC check if MAC addresses are indicated to be present. I
 * will rely soley on the is_valid_mac() call to determine whether we will use
 * them or not until this can be sorted out.   
*/
#if 0
static unsigned crc32(unsigned crc, uint8_t *buffer, size_t size)
{
    unsigned data;
    unsigned carry;
    int j;  

    while(size--)
    {
        data = *buffer++;
        for (j = 0; j < 8; j++)
        {
            carry = ((data ^ crc) & 0x01);
            crc  >>= 1; 
            data >>= 1; 
            if (carry) crc = crc ^ 0xedb88320UL;
        }
    }
    return crc;
}
#else
// Table from 1003.2-1992 algorithm
static const unsigned crctab32[] = {
	0x0,
	0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b,
	0x1a864db2, 0x1e475005, 0x2608edb8, 0x22c9f00f, 0x2f8ad6d6,
	0x2b4bcb61, 0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd,
	0x4c11db70, 0x48d0c6c7, 0x4593e01e, 0x4152fda9, 0x5f15adac,
	0x5bd4b01b, 0x569796c2, 0x52568b75, 0x6a1936c8, 0x6ed82b7f,
	0x639b0da6, 0x675a1011, 0x791d4014, 0x7ddc5da3, 0x709f7b7a,
	0x745e66cd, 0x9823b6e0, 0x9ce2ab57, 0x91a18d8e, 0x95609039,
	0x8b27c03c, 0x8fe6dd8b, 0x82a5fb52, 0x8664e6e5, 0xbe2b5b58,
	0xbaea46ef, 0xb7a96036, 0xb3687d81, 0xad2f2d84, 0xa9ee3033,
	0xa4ad16ea, 0xa06c0b5d, 0xd4326d90, 0xd0f37027, 0xddb056fe,
	0xd9714b49, 0xc7361b4c, 0xc3f706fb, 0xceb42022, 0xca753d95,
	0xf23a8028, 0xf6fb9d9f, 0xfbb8bb46, 0xff79a6f1, 0xe13ef6f4,
	0xe5ffeb43, 0xe8bccd9a, 0xec7dd02d, 0x34867077, 0x30476dc0,
	0x3d044b19, 0x39c556ae, 0x278206ab, 0x23431b1c, 0x2e003dc5,
	0x2ac12072, 0x128e9dcf, 0x164f8078, 0x1b0ca6a1, 0x1fcdbb16,
	0x018aeb13, 0x054bf6a4, 0x0808d07d, 0x0cc9cdca, 0x7897ab07,
	0x7c56b6b0, 0x71159069, 0x75d48dde, 0x6b93dddb, 0x6f52c06c,
	0x6211e6b5, 0x66d0fb02, 0x5e9f46bf, 0x5a5e5b08, 0x571d7dd1,
	0x53dc6066, 0x4d9b3063, 0x495a2dd4, 0x44190b0d, 0x40d816ba,
	0xaca5c697, 0xa864db20, 0xa527fdf9, 0xa1e6e04e, 0xbfa1b04b,
	0xbb60adfc, 0xb6238b25, 0xb2e29692, 0x8aad2b2f, 0x8e6c3698,
	0x832f1041, 0x87ee0df6, 0x99a95df3, 0x9d684044, 0x902b669d,
	0x94ea7b2a, 0xe0b41de7, 0xe4750050, 0xe9362689, 0xedf73b3e,
	0xf3b06b3b, 0xf771768c, 0xfa325055, 0xfef34de2, 0xc6bcf05f,
	0xc27dede8, 0xcf3ecb31, 0xcbffd686, 0xd5b88683, 0xd1799b34,
	0xdc3abded, 0xd8fba05a, 0x690ce0ee, 0x6dcdfd59, 0x608edb80,
	0x644fc637, 0x7a089632, 0x7ec98b85, 0x738aad5c, 0x774bb0eb,
	0x4f040d56, 0x4bc510e1, 0x46863638, 0x42472b8f, 0x5c007b8a,
	0x58c1663d, 0x558240e4, 0x51435d53, 0x251d3b9e, 0x21dc2629,
	0x2c9f00f0, 0x285e1d47, 0x36194d42, 0x32d850f5, 0x3f9b762c,
	0x3b5a6b9b, 0x0315d626, 0x07d4cb91, 0x0a97ed48, 0x0e56f0ff,
	0x1011a0fa, 0x14d0bd4d, 0x19939b94, 0x1d528623, 0xf12f560e,
	0xf5ee4bb9, 0xf8ad6d60, 0xfc6c70d7, 0xe22b20d2, 0xe6ea3d65,
	0xeba91bbc, 0xef68060b, 0xd727bbb6, 0xd3e6a601, 0xdea580d8,
	0xda649d6f, 0xc423cd6a, 0xc0e2d0dd, 0xcda1f604, 0xc960ebb3,
	0xbd3e8d7e, 0xb9ff90c9, 0xb4bcb610, 0xb07daba7, 0xae3afba2,
	0xaafbe615, 0xa7b8c0cc, 0xa379dd7b, 0x9b3660c6, 0x9ff77d71,
	0x92b45ba8, 0x9675461f, 0x8832161a, 0x8cf30bad, 0x81b02d74,
	0x857130c3, 0x5d8a9099, 0x594b8d2e, 0x5408abf7, 0x50c9b640,
	0x4e8ee645, 0x4a4ffbf2, 0x470cdd2b, 0x43cdc09c, 0x7b827d21,
	0x7f436096, 0x7200464f, 0x76c15bf8, 0x68860bfd, 0x6c47164a,
	0x61043093, 0x65c52d24, 0x119b4be9, 0x155a565e, 0x18197087,
	0x1cd86d30, 0x029f3d35, 0x065e2082, 0x0b1d065b, 0x0fdc1bec,
	0x3793a651, 0x3352bbe6, 0x3e119d3f, 0x3ad08088, 0x2497d08d,
	0x2056cd3a, 0x2d15ebe3, 0x29d4f654, 0xc5a92679, 0xc1683bce,
	0xcc2b1d17, 0xc8ea00a0, 0xd6ad50a5, 0xd26c4d12, 0xdf2f6bcb,
	0xdbee767c, 0xe3a1cbc1, 0xe760d676, 0xea23f0af, 0xeee2ed18,
	0xf0a5bd1d, 0xf464a0aa, 0xf9278673, 0xfde69bc4, 0x89b8fd09,
	0x8d79e0be, 0x803ac667, 0x84fbdbd0, 0x9abc8bd5, 0x9e7d9662,
	0x933eb0bb, 0x97ffad0c, 0xafb010b1, 0xab710d06, 0xa6322bdf,
	0xa2f33668, 0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4
};

static unsigned crc32(unsigned dummy, uint8_t *buf, size_t len)
{
	unsigned	crc = 0;
	uint8_t		*end;

	end = buf + len;
	while(buf < end) {
		crc = (crc<<8) ^ crctab32[(crc>>24) ^ ((unsigned)(*buf++))];
	}

	return(crc);
}
#endif

__SRCVERSION("$URL$ $Rev$");
