/*
 * Copyright 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not
 * reproduce, modify or distribute this software except in compliance with the
 * License. You may obtain a copy of the License at:
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as contributors under
 * the License or as licensors under other terms.  Please review this entire
 * file for other proprietary rights or license notices, as well as the QNX
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/
 * for other information.
 */

/*
 *       RETRIEVE CONFIGURATION DATA FROM RTC ON FREESCALE BOARDS
 * 
 * The reading of the RTC assumes connection on an I2C bus. The I2C bus
 * callouts are defined in 'i2c_funcs'. A default I2C driver (in mpc8xxx_i2c.c)
 * should be able to handle most Freescale boards however if you need to tweak
 * it or implement a new one then you can copy lib/ppc/mpc8xxx_i2c.c into the
 * BSP directory and those callouts will be linked in instead.
 * 
 */
#include "startup.h"
#include <errno.h>
#include <time.h>
#include <drvr/hwinfo.h>

/* RTC (out of 256 bytes, 20 bytes are for RTC and the remaining 236 bytes are SRAM) content layout. This is taken from the D3232 datasheet */
typedef struct
{
	uint8_t		seconds;/* Bits 0-3 represent seconds, Bits 4-6 represent 10 seconds */
	uint8_t		minutes;/* Bits 0-3 represent minutes, Bits 4-6 represent 10 minutes */
	uint8_t		hours;	/* Bits 0-3 represent hours, Bits 4-5 represent 10 hours, Bit 5 gives AM/PM, Bit 6 gives 12/24 hour format */
	uint8_t		day;	/* Bits 0-3 represent day of the week */
	uint8_t		date;	/* Bits 0-3 represent date, Bits 4-5 represent 10 date  */
	uint8_t		month;	/* Bits 0-3 represent months, Bit 4 represents 10 month, Bit 7 represents century */
	uint8_t		year;	/* Bits 0-3 represent years, Bits 4-7 represent 10 years */
	uint8_t     reserved;
} rtc_data_t;


static struct
{
	unsigned	initialized;	// 0 == false, 1 == true
	unsigned	unit;
	uint8_t		i2c_addr;
	size_t		size;
} rtc_i2c_params = {.initialized = 0};	// just to be explicit

/*
 * i2c_funcs
 * 
 * This allows the BSP to hook in the proper I2C driver overriding the default
*/
extern struct
{
	void (*init)(uint64_t addr, unsigned divider);
	void (*done)(void);
	int (*xfer)(uint8_t i2c_addr, uint8_t *send_data, size_t send_length, uint8_t *recv_buffer, size_t recv_length);
} *i2c_funcs;

#define I2C_INIT(a, b)				do {if (i2c_funcs != NULL) {i2c_funcs->init((a), (b));}} while(0)
#define I2C_DONE()					do {if (i2c_funcs != NULL) {i2c_funcs->done();}} while(0)
#define I2C_XFER(a, b, c, d, e)		((i2c_funcs != NULL) ? i2c_funcs->xfer((a), (b), (c), (d), (e)) : -1)

/*
 * =============================================================================
 *                                  A P I's
 * 
 *         The following section contains the API's available to the BSP.
 *                       Add to it as the need arises
 * =============================================================================
*/

/*******************************************************************************
 * init_i2c_rtc_read
 * 
 * Initialize parameters for reading of RTC on an I2C bus.
 * The required parameters are
 * 		- I2C bus unit
 * 		- RTC I2C bus address
 * 		- RTC size
 * 
 * Returns: EOK if initialization was successful, otherwise EINVAL
*/
int init_i2c_rtc_read(unsigned unit, uint8_t i2c_addr, size_t size)
{
	if (unit >= hwi_find_num_units(HWI_ITEM_BUS_I2C)) return EINVAL;
	if (size == 0) return EINVAL;
// Any illegal I2C addresses ?	if (i2c_addr == ??) return EINVAL;
	
	rtc_i2c_params.unit = unit;
	rtc_i2c_params.i2c_addr = i2c_addr;
	rtc_i2c_params.size = size;
	rtc_i2c_params.initialized = 1;
	
	return EOK;
}

unsigned long
rtc_time_ds3232() {
	struct tm	tm;
	rtc_data_t rtc_data;
	unsigned hwi_off;

	if (!rtc_i2c_params.initialized)
	{
		kprintf("\n** You must call init_i2c_rtc_read() first **\n");
		ASSERT(rtc_i2c_params.initialized);
	}

	if ((hwi_off = hwi_find_bus(HWI_ITEM_BUS_I2C, rtc_i2c_params.unit)) != HWI_NULL_OFF)
	{
		hwi_tag *addr_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_location, NULL);
		hwi_tag *clk_tag = hwi_tag_find(hwi_off, HWI_TAG_NAME_inputclk, NULL);
		
		if ((addr_tag == NULL) || (clk_tag == NULL)) return 0;
		else
		{
			int r;
			uint32_t rtc_address = 0;
			size_t rtc_address_size = 1;

			/* initialize access to the controllers registers. We will target a 100Kb/s bus rate */
			if (debug_flag > 1) kprintf("i2c_init(%x, %d)\n", (uint32_t)addr_tag->location.base, clk_tag->inputclk.clk / 100000);
			I2C_INIT(addr_tag->location.base, clk_tag->inputclk.clk / 100000); 

			if (debug_flag) {
				kprintf("Looking for RTC on %s,%d @ I2C address 0x%x ... ",
							HWI_ITEM_BUS_I2C, rtc_i2c_params.unit, rtc_i2c_params.i2c_addr);
			}

			/* read the RTC device at the specified address */
			r = I2C_XFER(rtc_i2c_params.i2c_addr, (uint8_t *)&rtc_address, rtc_address_size,
							(uint8_t*)&rtc_data, sizeof(rtc_data));
			if (r != sizeof(rtc_data))
			{
				if (debug_flag) kprintf("\n\tRTC read failure %d\n", r);
				memset(&rtc_data, 0, sizeof(rtc_data));	// make sure to erase any garbage
				I2C_DONE();
			}
			else
			{
#if 0
				if (debug_flag) kprintf("found\nValidating contents ... ");

				/* optionally dump the RTC contents */
				if (debug_flag > 1)
				{
					unsigned i;
					for (i=0; i<sizeof(rtc_data); i++)
					{
						if ((i % 8) == 0) kprintf("\n[%d]: ", i);
						kprintf("0x%x ", ((uint8_t *)&rtc_data)[i]);
					}
					kprintf("\n\n");
				}
				if (debug_flag) kprintf("Ok\n");
				
				kprintf("\n Seconds: 0x%x \n",     ((uint8_t *)&rtc_data)[0]);
				kprintf("\n Minutes: 0x%x \n",     ((uint8_t *)&rtc_data)[1]);
				kprintf("\n Hours 0x%x \n",        ((uint8_t *)&rtc_data)[2]);
				kprintf("\n Day of Week: 0x%x \n", ((uint8_t *)&rtc_data)[3]);
				kprintf("\n Date of Mon: 0x%x \n", ((uint8_t *)&rtc_data)[4]);
				kprintf("\n Month & Cen: 0x%x \n", ((uint8_t *)&rtc_data)[5]);
				kprintf("\n Year: 0x%x \n",        ((uint8_t *)&rtc_data)[6]);
#endif					
				tm.tm_sec  = bcd2bin(((uint8_t *)&rtc_data)[0] & 0x7f);
				tm.tm_min  = bcd2bin(((uint8_t *)&rtc_data)[1] & 0x7f);
				
				if (((uint8_t *)&rtc_data)[2] & 0x40) {
				/* the rtc is in 12 hour mode */ 
				int hour = bcd2bin(((uint8_t *)&rtc_data)[2] & 0x1f);

				if (((uint8_t *)&rtc_data)[2] & 0x20)  
				tm.tm_hour = (hour == 12) ? 12 : hour + 12; /* pm */
				else
				tm.tm_hour = (hour == 12) ? 0 : hour;  	 /* am */
				} 
				else { 
				/* rejoice! the rtc is in 24 hour mode */ 
				tm.tm_hour = bcd2bin(((uint8_t *)&rtc_data)[2] & 0x3f);
				}
				
				tm.tm_mday = bcd2bin(((uint8_t *)&rtc_data)[4] & 0x3f);
				tm.tm_mon  = bcd2bin(((uint8_t *)&rtc_data)[5] & 0x1f) - 1;
				tm.tm_year = bcd2bin(((uint8_t *)&rtc_data)[6] & 0xff);

				if(tm.tm_year < 70) tm.tm_year += 100;

				tm.tm_wday = bcd2bin(((uint8_t *)&rtc_data)[3] & 0x7) - 1;
				I2C_DONE();
				return(calc_time_t(&tm));
			}
		}
	}
	return 0;
}

__SRCVERSION("$URL:$ $Rev:$");
