/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"
#include "hwinfo_private.h"
#include "ppc/hwinfo_85xx_proto.h" /* Pull in the 85xx protos */
#include "ppc/hwinfo_p20xx_proto.h"
#include <hw/hwinfo_p20xx.h>
#include <drvr/hwinfo.h>		// for hwi support routines in libdrvr

/*
 * Add P2020 specific devices to the hardware info section of the syspage
*/

extern uint32_t platform_clock;

void hwi_p2020(hwi85xx_flags_t flags)
{
	unsigned hwi_bus_internal;

	/* The platform clock is set in ppc85xx_set_get_clock_freqs.
		Since we use this for calculations here, assert to ensure it is set.
	*/
	ASSERT(platform_clock != 0);

	/* add an internal bus to which all SOC devices will be attached. We handle
	 * being called multiple times and will only add 1 instance */
	if ((hwi_bus_internal = hwi_find_bus(HWI_ITEM_BUS_INTERNAL, 0)) == HWI_NULL_OFF)
	{
		hwi_bus_internal = hwibus_add(HWI_ITEM_BUS_INTERNAL, HWI_NULL_OFF);
		ASSERT(hwi_find_unit(hwi_bus_internal) == 0);
	}

	/* add I2C (unless directed not to) */
	if ((flags & hwi85xx_flags_t_I2C_NONE) == 0)
	{
		unsigned hwi_off;
		hwiattr_i2c_t attr = HWIATTR_I2C_T_INITIALIZER;
		// I2C source clock is 1/2 the CCB (ie. plaform_clock)
		struct hwi_inputclk clksrc = {.clk = platform_clock / 2, .div = 1};
		struct hwi_busattr busattr = {.speed = 100000, .addr = 0, .width = 0, .flags = 0};

		HWIATTR_I2C_SET_NUM_IRQ(&attr, 1);
		HWIATTR_I2C_SET_NUM_CLK(&attr, 1);
		HWIATTR_I2C_SET_NUM_BUSATTR(&attr, 1);

		/* create i2c0 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x3000, 0x18, 0, hwi_find_as(immr_paddr + 0x3000, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* create i2c1 and set the clock source */
		HWIATTR_I2C_SET_LOCATION(&attr, immr_paddr + 0x3100, 0x18, 0, hwi_find_as(immr_paddr + 0x3100, 1));
		hwi_off = hwibus_add_i2c(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);
		hwitag_set_busattr(hwi_off, 0, &busattr);

		/* we are done if the hwi85xx_flags_t_I2C_ONLY flag is set */
		if (flags & hwi85xx_flags_t_I2C_ONLY) return;
	}

	/* add 3 PCI ports */
	{
#define SET_BUSATTR_PCI_MEMTYPE(f, mt)	((f) = (((f) & ~hwi_busattr_PCI_memtype_mask) | (mt)))

		unsigned hwi_off;
		hwiattr_pci_t attr = HWIATTR_PCI_T_INITIALIZER;
		/* Since the PCIe ports can be configured for x1, x2 or x4, the .width parameter
		 * must be set in the board specific code */
		struct hwi_busattr busattr = {.speed = 2500000000ULL, .addr = 0, .width=0, .flags = hwi_busattr_isPCIe};

		HWIATTR_PCI_SET_NUM_IRQ(&attr, 5);	// PCIe has 1 controller interrupt and 4 for A/B/C/D when in root complex mode
		HWIATTR_PCI_SET_DLL(&attr, "p2020_pci");
		HWIATTR_PCI_SET_NUM_BUSATTR(&attr, 2);	// memory and I/O base addresses

		/* create PCIe device 0 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0xA000, 0xE38, 0, AS_NULL_OFF);
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 1 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x9000, 0xE38, 0, hwi_find_as(immr_paddr + 0x9000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);

		/* create PCIe device 2 */
		HWIATTR_PCI_SET_LOCATION(&attr, immr_paddr + 0x8000, 0xE38, 0, hwi_find_as(immr_paddr + 0x8000, 1));
		hwi_off = hwibus_add_pci(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		/* the memory and I/O address are set in the BSP. We set the first entry as memory, the 2nd as I/O */
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_MEM);
		hwitag_set_busattr(hwi_off, 0, &busattr);
		SET_BUSATTR_PCI_MEMTYPE(busattr.flags, hwi_busattr_PCI_memtype_IO);
		hwitag_set_busattr(hwi_off, 1, &busattr);
	}

	/* add 2 UART's */
	{
		unsigned hwi_off;
		hwiattr_uart_t attr = HWIATTR_UART_T_INITIALIZER;
		// UART source clock is the CCB (ie. plaform_clock)
		// FIX ME - not sure why we just don't do the division first and make div = 1?
		struct hwi_inputclk clksrc = {.clk = platform_clock, .div = 16};

		HWIATTR_UART_SET_NUM_IRQ(&attr, 1);
		HWIATTR_UART_SET_NUM_CLK(&attr, 1);

		/* create uart0 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x4500, 0x14, 0, hwi_find_as(immr_paddr + 0x4500, 1));
		hwi_off = hwidev_add_uart(PPCP20xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* create uart1 and set the clock source */
		HWIATTR_UART_SET_LOCATION(&attr, immr_paddr + 0x4600, 0x14, 0, hwi_find_as(immr_paddr + 0x4600, 1));
		hwi_off = hwidev_add_uart(PPCP20xx_HWI_UART, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		hwitag_set_inputclk(hwi_off, 0, &clksrc);

		/* historically the UART's were called '8250' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPCP20xx_HWI_UART, 0), "8250");
		hwi_add_synonym(hwi_find_device(PPCP20xx_HWI_UART, 1), "8250");
	}

	/* add the 3 eTSEC devices */
	{
		unsigned hwi_off;
		hwiattr_enet_t attr = HWIATTR_ENET_T_INITIALIZER;
		uint8_t mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0xFF};	// a non zero mac

		/* we configure for 4 interrupts, one each for Tx, Rx, Err and 1 PHY */
		if (flags & hwi85xx_flags_t_USE_MAC) HWIATTR_ENET_SET_MAC(&attr, mac);
		HWIATTR_ENET_SET_NUM_IRQ(&attr, 4);
		HWIATTR_ENET_SET_NUM_PHY(&attr, 1);
		HWIATTR_ENET_SET_DLL(&attr, "p2020_tsec");

		/* create eTSEC0 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, immr_paddr + 0x24000, 0xEB0, 0, hwi_find_as(immr_paddr + 0x24000, 1));
		hwi_off = hwidev_add_enet(PPCP20xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 0);

		/* create eTSEC1 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, immr_paddr + 0x25000, 0xEB0, 0, hwi_find_as(immr_paddr + 0x25000, 1));
		hwi_off = hwidev_add_enet(PPCP20xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 1);

		/* create eTSEC2 and set the PHY address */
		HWIATTR_ENET_SET_LOCATION(&attr, immr_paddr + 0x26000, 0xEB0, 0, hwi_find_as(immr_paddr + 0x26000, 1));
		hwi_off = hwidev_add_enet(PPCP20xx_HWI_eTSEC, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 2);
		/* By default we assume PHY's are connected in numeric order. If this is not
		 * the case, then correct it in the board specific init_hwinfo() */
		hwitag_set_phyaddr(hwi_off, 0, 2);

		/* historically the eTSEC's were called 'tsec' so add these synonyms */
		hwi_add_synonym(hwi_find_device(PPCP20xx_HWI_eTSEC, 0), PPCP20xx_HWI_LEGACY_TSEC);
		hwi_add_synonym(hwi_find_device(PPCP20xx_HWI_eTSEC, 1), PPCP20xx_HWI_LEGACY_TSEC);
		hwi_add_synonym(hwi_find_device(PPCP20xx_HWI_eTSEC, 2), PPCP20xx_HWI_LEGACY_TSEC);
	}
	/* add USB controller */
	{
		unsigned hwi_off;
		hwiattr_usb_t attr = HWIATTR_USB_T_INITIALIZER;

		HWIATTR_USB_SET_NUM_IRQ(&attr, 1);
		HWIATTR_USB_SET_DLL(&attr, "ehci-p2020");

		/* create usb */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x22000, 0x504, 0, hwi_find_as(immr_paddr + 0x22000, 1));
		hwi_off = hwibus_add_usb(hwi_bus_internal, &attr);
		ASSERT(hwi_find_unit(hwi_off) == 0);
	}
	/* add 2 dma controllers (4 channels each) */
	{
		unsigned hwi_off;
		hwiattr_dma_t attr = HWIATTR_DMA_T_INITIALIZER;

		HWIATTR_DMA_SET_NUM_IRQ(&attr, 4);	// each of 4 channels can interrupt

		/* create DMA controller 0 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0x21000, 0x304, 0, hwi_find_as(immr_paddr + 0x21000, 1));
		hwi_off = hwidev_add_dma(PPCP20xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 0);

		/* create DMA controller 1 */
		HWIATTR_USB_SET_LOCATION(&attr, immr_paddr + 0xC000, 0x304, 0, hwi_find_as(immr_paddr + 0xC000, 1));
		hwi_off = hwidev_add_dma(PPCP20xx_HWI_DMA, &attr, hwi_bus_internal);
		ASSERT(hwi_find_unit(hwi_off) == 1);
	}

	/* add performance monitor */
	hwidev_add_85xx_pmon(PPCP20xx_HWI_PMON, hwi_bus_internal, immr_paddr + 0xE1000);

	/* add security device */
	hwidev_add_85xx_security(PPCP20xx_HWI_SECURITY, hwi_bus_internal, immr_paddr + 0x30000);

	/* add GPIO */
	hwidev_add_85xx_gpio(PPCP20xx_HWI_GPIO, hwi_bus_internal, immr_paddr + 0xFC00, 1);

	/* add SDHC */
	hwidev_add_85xx_sdhc(PPCP20xx_HWI_SDHC, hwi_bus_internal, immr_paddr + 0x2E000);

	/* add Serial RapidIO devices - FIX ME - what about the architecturally defined registers at 0xC0000 */
	{
		unsigned hwi_off;

		hwi_off = hwidev_add(PPCP20xx_HWI_SRIO_EWP, 0, hwi_bus_internal);
		hwitag_add_irq(hwi_off, HWI_ILLEGAL_VECTOR);
		hwidev_add_85xx_srio_doorbell(PPCP20xx_HWI_SRIO_DOORBELL, hwi_bus_internal, immr_paddr + 0xD3400);
		hwidev_add_85xx_srio_msgunit(PPCP20xx_HWI_SRIO_MSGUNIT, hwi_bus_internal, immr_paddr + 0xD3000);
		hwidev_add_85xx_srio_msgunit(PPCP20xx_HWI_SRIO_MSGUNIT, hwi_bus_internal, immr_paddr + 0xD3100);
	}

	/* add group A and B timers (4 timers per group) */
	hwidev_add_85xx_timer_group(PPCP20xx_HWI_GTIMER, hwi_bus_internal, immr_paddr + 0x410F0, 4);
	hwidev_add_85xx_timer_group(PPCP20xx_HWI_GTIMER, hwi_bus_internal, immr_paddr + 0x420F0, 4);

	/* add 8 message devices */
	hwidev_add_p20xx_message(PPCP20xx_HWI_MSG, hwi_bus_internal, immr_paddr + 0x41400, 8);

	/* add 8 shared message devices */
	hwidev_add_p20xx_shared_message(PPCP20xx_HWI_SMSG, hwi_bus_internal, immr_paddr + 0x41600, 8);

	/* add 12 external interrupt devices (board specific) */
	hwidev_add_85xx_ext_irq(PPCP20xx_HWI_EXT_IRQ, hwi_bus_internal, 12);
}

__SRCVERSION("$URL$ $Rev$");
