/*
 * $QNXLicenseC: 
 * Copyright 2007, 2008, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */

#include "startup.h"

/* mpc85xx I2C register offsets */
#define I2CADR		0x00
#define	I2CFDR		0x04
#define	I2CCR		0x08
#define I2CSR		0x0C
#define I2CDR		0x10
#define I2CDFSRR	0x14

#define I2C_regmap_size	0x18	// I2CDFSRR + sizeof(I2CDFSRR)

#ifndef NELEMENTS
#define NELEMENTS(x)	(sizeof((x)) / sizeof((x)[0]))
#endif	/* NELEMENTS */


#define chip_write8(off, val)	i2c_base[(off)] = (val)
#define chip_read8(off)			i2c_base[(off)]

#if 0
#define DUMP_REGS \
		do { \
			kprintf("REGS:\n"); \
			kprintf("\tADR: 0x%x\n", chip_read8(I2CADR)); \
			kprintf("\tFDR: 0x%x\n", chip_read8(I2CFDR)); \
			kprintf("\tCR: 0x%x\n", chip_read8(I2CCR)); \
			kprintf("\tSR: 0x%x\n", chip_read8(I2CSR)); \
			kprintf("\tDR: 0x%x\n", chip_read8(I2CDR)); \
			kprintf("\tDFSRR: 0x%x\n", chip_read8(I2CDFSRR)); \
		} while(0)
#else
#define DUMP_REGS	do {} while(0)
#endif

//extern unsigned csb_clk;
extern unsigned platform_clock;
static volatile uint8_t *i2c_base; 


static int i2c_wait4bus(void);
static int i2c_wait(int write);
static int i2c_write_addr(uint8_t address, uint8_t read, unsigned rsta);
static int __i2c_write(uint8_t *data, size_t length);
static unsigned __i2c_read(uint8_t *data, size_t length);
static unsigned find_divider(unsigned divider);
static void usleep(unsigned u);



/*******************************************************************************
 * i2c_init
 * 
 * Initialize the I2C controller. Note that a bunch of assumptions are made in
 * this simplified initialization
 * 
 * Returns: n/a
*/
static void i2c_init(uint64_t addr, unsigned divider)
{
	/* map in the device registers */
	i2c_base = startup_memory_map(I2C_regmap_size, addr, PROT_READ|PROT_WRITE|PROT_NOCACHE);

	/* stop/disable controller */
	chip_write8(I2CCR, 0x00);

	/* set clock divider */
	chip_write8(I2CFDR, ~0xC0 & find_divider(divider));

	/* clear status register */
	chip_write8(I2CSR, 0x00);

	/* set default filter */
	chip_write8(I2CDFSRR, 0x10);	// FIX ME - where does this magic value come from ?

	/* write slave address of 0x7f (something that can't be matched by the target) */
	chip_write8(I2CADR, 0x7F);

	/* enable the controller */
//	chip_write8(I2CCR, 0x80);

	DUMP_REGS;

	return;
}

/*******************************************************************************
 * i2c_done
 * 
 * Stop/Disable the controller and unmap the registers
 * 
 * Returns: n/a
*/
static void i2c_done(void)
{
	/* stop/disable I2C controller */
    chip_write8(I2CCR, 0x00);
    /* unmap device registers */
	startup_memory_unmap((void *)i2c_base); 
}

/*******************************************************************************
 * i2c_xfer
 * 
 * Initiate a transfer to I2C address <i2c_addr>. <send_length> bytes are written
 * from <send_data> followed by a read of <recv_length> bytes into <recv_buffer>.
 * 
 * Returns: the number of bytes received or -1 on error
*/
static int i2c_xfer(uint8_t i2c_addr, uint8_t *send_data, size_t send_length,
					uint8_t *recv_buffer, size_t recv_length)
{
	unsigned rlen = 0;

	ASSERT(i2c_addr != 0x7f);	// cannot send to ourself */

	chip_write8(I2CCR, 0x20);
	chip_write8(I2CCR, 0xA0);
	chip_read8(I2CDR);
	chip_write8(I2CCR, 0x80);
	chip_write8(I2CSR, 0);
	chip_write8(I2CCR, 0x00);

	if (i2c_wait4bus() < 0) {
		if(debug_flag) kprintf("bus wait error\n");
		goto done;
	}
	DUMP_REGS;
	if (i2c_write_addr(i2c_addr, 0, 0) < 0) {
		if(debug_flag) kprintf("address write error 1\n");
		goto done;
	}
	DUMP_REGS;
	if (__i2c_write(send_data, send_length) != send_length) {
		if(debug_flag) kprintf("write data error\n");
		goto done;
	}
	DUMP_REGS;

	if (i2c_write_addr(i2c_addr, 1, 1) < 0) {
		if(debug_flag) kprintf("address write error 2\n");
		goto done;
	}
	DUMP_REGS;
	if ((rlen = __i2c_read(recv_buffer, recv_length)) != recv_length) {
		kprintf("read error: received %d of %d bytes\n", rlen, recv_length);
	}
	DUMP_REGS;

done:
	chip_write8(I2CCR, 0x80);	// Stop
	usleep(100);
	chip_write8(I2CCR, 0x00);	// Disable
	DUMP_REGS;
	return (rlen == recv_length) ? recv_length : -1;
}

/*
 * i2c_funcs
 * 
 * Initialize the driver entry table with the callin's. This can be overridden
 * with a similar function in the BSP directory (startup/boards/<board>)
*/
typedef struct
{
	void (*init)(uint64_t addr, unsigned divider);
	void (*done)(void);
	int (*xfer)(uint8_t i2c_addr, uint8_t *send_data, size_t send_length, uint8_t *recv_buffer, size_t recv_length);
} i2c_funcs_t;
static i2c_funcs_t _i2c_funcs = {i2c_init, i2c_done, i2c_xfer};
i2c_funcs_t *i2c_funcs = &_i2c_funcs;


/*
 * =============================================================================
 *                       S U P P O R T   F U N C T I O N S
 * =============================================================================
*/
static int i2c_wait4bus(void)
{
	unsigned timeout = 0;

	/* while the bus is busy ... */
	while (chip_read8(I2CSR) & 0x20)
	{
		if (++timeout > 500) return -1;
		usleep(1);
	}
	return 0;
}

static int i2c_wait(int write)
{
	unsigned timeout = 0;
	uint8_t	csr;

	while (++timeout < 500)
	{
		csr = chip_read8(I2CSR);
		
		/* docs say to check the MIF bit rather that the MCF bit */
		if (csr & 0x2)
			goto okay;

		usleep(1);
	}
//	kprintf("failed after %d tries, csr = %x\n", timeout, csr);
	return -4; 

okay:
	chip_write8(I2CSR, 0);

//	kprintf("csr = %x, ", csr);

	if (csr & 0x10)
		return -1;    /* lost arbitration */

	if (!(csr & 0x80)) 
		return -2; 

	if (write == 1 && (csr & 0x01))
		return -3;

	return 0;
}

static int i2c_write_addr(uint8_t address, uint8_t read, unsigned repeated_start)
{
	chip_write8(I2CCR, (0x80 | 0x20 | 0x10 | (repeated_start ? 0x04 : 0)));

	chip_write8(I2CDR, (address << 1) | read);

	if (i2c_wait(1) < 0)
		return -1;

	return 0;
}

static int __i2c_write(uint8_t *data, size_t length)
{
	unsigned i;

	for (i=0; i<length; i++)
	{
		int rc;
		chip_write8(I2CDR, data[i]);

		if ((rc = i2c_wait(1)) < 0)
			return rc;
	}
	return i;
}

static unsigned __i2c_read(uint8_t *data, const size_t length)
{
	unsigned num = 0;

	chip_write8(I2CCR, 0x80 | 0x20);	// receive mode

//	if (i2c_wait(0) < 0) return 0;
	i2c_wait(0);	// MIF bit isn't set before dummy read so don't fail the wait
	chip_read8(I2CDR);	// dummy read

	while (num < length)
	{
		if (i2c_wait(0) < 0) break;
		if (num >= (length - 1)) {
			/* last or 2nd last byte so set appropriate CR value */
			uint8_t cr_xtra = (num == length) ? 0 : 0x8;	// set TXAK on 2nd last byte 
			chip_write8(I2CCR, 0x80 | 0x20 | cr_xtra);
		}
		*data++ = chip_read8(I2CDR);
//	kprintf("data = %x\n", *(data - 1));
		++num;
	}
	return num;
}

static void usleep(unsigned u)
{
	unsigned t1, cycles;

	t1 = get_spr(PPC_SPR_DEC);
	cycles = u * ((platform_clock/*csb_clk*/ >> 2) / 1000000);

	while ((t1 - get_spr(PPC_SPR_DEC)) < cycles)
		;
}
 
/* given <divider> find the best match which is greater than (so the bus speed
 * is no higher than desired) and return the table index value */
static unsigned find_divider(unsigned divider)
{
	static const unsigned divider_ratio[] =
	{
		[0x00] =   384, [0x01] =   416, [0x02] =   480, [0x03] =   576,
		[0x04] =   640, [0x05] =   704, [0x06] =   832, [0x07] =  1024,
		[0x08] =  1152, [0x09] =  1280, [0x0a] =  1536, [0x0b] =  1920,
		[0x0c] =  2304, [0x0d] =  2560, [0x0e] =  3072, [0x0f] =  3840,
		[0x10] =  4608, [0x11] =  5120, [0x12] =  6144, [0x13] =  7680,
		[0x14] =  9216, [0x15] = 10240, [0x16] = 12288, [0x17] = 15360,
		[0x18] = 18432, [0x19] = 20480, [0x1a] = 24576, [0x1b] = 30720,
		[0x1c] = 36864, [0x1d] = 40960, [0x1e] = 49152, [0x1f] = 61440,
		[0x20] =   256, [0x21] =   288, [0x22] =   320, [0x23] =   352,
		[0x24] =   384, [0x25] =   448, [0x26] =   512, [0x27] =   576,
		[0x28] =   640, [0x29] =   768, [0x2a] =   896, [0x2b] =  1024,
		[0x2c] =  1280, [0x2d] =  1536, [0x2e] =  1792, [0x2f] =  2048,
		[0x30] =  2560, [0x31] =  3072, [0x32] =  3584, [0x33] =  4096,
		[0x34] =  5120, [0x35] =  6144, [0x36] =  7168, [0x37] =  8192,
		[0x38] = 10240, [0x39] = 12288, [0x3a] = 14336, [0x3b] = 16384,
		[0x3c] = 20480, [0x3d] = 24576, [0x3e] = 28672, [0x3f] = 32768
	};
	unsigned i;
	unsigned best_idx = 0;
	unsigned last_best_diff = UINT_MAX;

	for (i=0; i<NELEMENTS(divider_ratio); i++)
	{
		if (divider_ratio[i] == divider) return i;
		else if (divider_ratio[i] > divider)
		{
			unsigned diff = (divider_ratio[i] - divider);
			if (diff < last_best_diff)
			{
				last_best_diff = diff;
				best_idx = i;
			}
		}
	}
	if (debug_flag > 1) kprintf("divider ratio %d @ idx %d\n", divider_ratio[best_idx], best_idx);
	return best_idx;
}

__SRCVERSION("$URL$ $Rev$");
