/*
 * $QNXLicenseC: 
 * Copyright 2009, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

/*******************************************************************************
 * support routines for accessing MPC85xx and MPC86xx LAW registers
 * 
 * This file is only compiled when included from an mpc85xx or mpc86xx specific
 * implementation (see mpc8544_law.c, mpc8572_law.c, etc) which set up the
 * appropriate defines
*/

#ifdef	LAW_FUNC


extern paddr_t immr_paddr;
paddr_t syscfg_base = 0;
#define MAP_REG \
		do { \
			if (syscfg_base == 0) { \
				syscfg_base = (paddr_t)startup_io_map(MEG(1), immr_paddr); \
			} \
		} while(0)

#define IN32(_x_)			in32(syscfg_base + (_x_))
#define OUT32(_x_, _v_)		out32(syscfg_base + (_x_), (_v_))

#define MPC85xx_REGOFF_LAWBAR(n)	(0xC08U + (0x20 * (n % MPC85XX_NUM_LAW_REGS)))
#define MPC85xx_REGOFF_LAWAR(n)		(0xC10U + (0x20 * (n % MPC85XX_NUM_LAW_REGS)))


static unsigned overlap_check(uint64_t start, uint64_t end, int exclude_law_id);
static int find_law(uint64_t start, size_t size, int law_id, int target_id);


/*
 * =============================================================================
 *                             G E T   A P I's
 * =============================================================================
*/

/*******************************************************************************
 * MPC85XX_law_numreg
 * 
 * Returns: the number of LAW registers on the device
*/
unsigned LAW_FUNC(law_numreg)(void)
{
	return MPC85XX_NUM_LAW_REGS;
}

/*******************************************************************************
 * MPC85XX_law_is_enabled
 * 
 * Returns: whether or not the LAW register <law_id> is in use or not
*/
unsigned LAW_FUNC(law_is_enabled)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return 0;
	reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
	return ((reg & MPC85XX_LAWAR_ENABLE_MASK) == MPC85XX_LAWAR_ENABLE_MASK);
}

/*******************************************************************************
 * MPC85XX_law_get_target
 * 
 * Returns: the target id for the LAW register <law_id> or -1 on error
*/
int LAW_FUNC(law_get_target)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return -1;
	reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
	return (reg & MPC85XX_LAWAR_TGTID_MASK) >> 20;
}

/*******************************************************************************
 * MPC85XX_law_get_size
 * 
 * Returns: the window size for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
*/
size_t LAW_FUNC(law_get_size)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return 0;
	reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
	return 1U << ((reg & MPC85XX_LAWAR_SIZE_MASK) + 1);
}

/*******************************************************************************
 * MPC85XX_law_get_addr
 * 
 * Returns: the window base address for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
 *
 * IMPORTANT 
 * Since the smallest window size is 4KB, the least significant 12 bits are fixed.
 * The LAWBAR contains the most significant 24 bits which together represent the
 * 36 bit internal address. This is why we have a 12 bit shift 
*/
uint64_t LAW_FUNC(law_get_addr)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return 0;
	reg = IN32(MPC85xx_REGOFF_LAWBAR(law_id));
	return (uint64_t)((reg & MPC85XX_LAWBAR_ADDR_MASK) << 12);
}

/*******************************************************************************
 * MPC85XX_law_find_target
 * 
 * Give <target_id>, find the LAW register window that has been configured to
 * map that target. The API allows for a law_id pointer to be provided which
 * controls where the search. If this pointer is NULL, this function will return
 * the first occurrence of a matching <target_id>. If it is not NULL, a pointer
 * to a previous match should be provided such that the search does not return
 * the same value for multiple calls.
 * 
 * There are MPC85XX_NUM_LAW_REGS LAW registers in this device
 * 
 * Returns: the LAW register id or -1 on error
*/
int LAW_FUNC(law_find_target)(unsigned target_id, unsigned *law_id_p)
{
	unsigned i = ((law_id_p != NULL) && (*law_id_p < MPC85XX_NUM_LAW_REGS)) ? *law_id_p + 1 : 0;
	
	for (; i<MPC85XX_NUM_LAW_REGS; i++)
	{
		if (LAW_FUNC(law_get_target)(i) == target_id) break;
	}
	return ((i < MPC85XX_NUM_LAW_REGS) && LAW_FUNC(law_is_enabled)(i)) ? i : -1;
}

/*******************************************************************************
 * MPC85XX_law_alloc
 * 
 * Allocate a free (disabled) LAW register
 * 
 * Returns: the LAW register id to use or -1 if there is no free LAW register
*/
int LAW_FUNC(law_alloc)(void)
{
	unsigned i;
	
	for (i=0; i<MPC85XX_NUM_LAW_REGS; i++) {
		if (!LAW_FUNC(law_is_enabled)(i)) break;
	}
	return (i < MPC85XX_NUM_LAW_REGS) ? i : -1;
}

/*
 * =============================================================================
 *                             S E T   A P I's
 * =============================================================================
*/

/*******************************************************************************
 * MPC85XX_law_enable
 * 
 * Enable LAW register <law_id>. 
 * Returns: 0 on success, -1 on error
*/
int LAW_FUNC(law_enable)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return -1;
	ASSERT(!overlap_check(LAW_FUNC(law_get_addr)(law_id), LAW_FUNC(law_get_addr)(law_id) + LAW_FUNC(law_get_size)(law_id), law_id));
	reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
	OUT32(MPC85xx_REGOFF_LAWAR(law_id), reg | MPC85XX_LAWAR_ENABLE_MASK);
	return 0;
}

/*******************************************************************************
 * MPC85XX_law_disable
 * 
 * Disable LAW register <law_id>. 
 * Returns: 0 on success, -1 on error
*/
int LAW_FUNC(law_disable)(unsigned law_id)
{
	volatile uint32_t reg;
	MAP_REG;

	if (law_id >= MPC85XX_NUM_LAW_REGS) return -1;
	reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
	OUT32(MPC85xx_REGOFF_LAWAR(law_id), reg & ~MPC85XX_LAWAR_ENABLE_MASK);
	return 0;
}

/*******************************************************************************
 * MPC85XX_law_set_addr
 * 
 * Set the address of the LAW register identified by <law_id> to <addr>.
 * 
 * Returns: the previous window base address for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
 * 
 * IMPORTANT 
 * Since the smallest window size is 4KB, the least significant 12 bits are fixed.
 * The LAWBAR contains the most significant 24 bits which together represent the
 * 36 bit internal address. This is why we have a 12 bit shift 
*/
uint64_t LAW_FUNC(law_set_addr)(unsigned law_id, uint64_t addr)
{
	uint64_t prev_addr = LAW_FUNC(law_get_addr)(law_id);

	if (law_id >= MPC85XX_NUM_LAW_REGS) return 0;
	if (prev_addr != addr)
	{
		unsigned is_enabled = LAW_FUNC(law_is_enabled)(law_id);

		LAW_FUNC(law_disable)(law_id);
		OUT32(MPC85xx_REGOFF_LAWBAR(law_id), ((addr >> 12) & MPC85XX_LAWBAR_ADDR_MASK));
		if (is_enabled) LAW_FUNC(law_enable)(law_id);
	}
	return prev_addr;
}

/*******************************************************************************
 * MPC85XX_law_set_size
 * 
 * Set the size of the LAW register identified by <law_id> to <size>.
 * 
 * Returns: the previous window size for the LAW register <law_id>
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
*/
size_t LAW_FUNC(law_set_size)(unsigned law_id, size_t size)
{
	size_t prev_size = LAW_FUNC(law_get_size)(law_id);
	
	if ((law_id >= MPC85XX_NUM_LAW_REGS) || (size < 4096)) return 0;
	if (prev_size != size)
	{
		unsigned is_enabled = LAW_FUNC(law_is_enabled)(law_id);
		volatile uint32_t reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
		unsigned shift = 0;
		size_t size_cp = size;
		while((size_cp >>= 1) != 0) ++shift;

		LAW_FUNC(law_disable)(law_id);
		OUT32(MPC85xx_REGOFF_LAWAR(law_id), (reg & ~MPC85XX_LAWAR_SIZE_MASK) | ((shift - 1) & MPC85XX_LAWAR_SIZE_MASK));
		if (is_enabled) LAW_FUNC(law_enable)(law_id);
	}
	return prev_size;
}

/*******************************************************************************
 * MPC85XX_law_set_target
 * 
 * Set the target id of the LAW register identified by <law_id> to <target_id>.
 * 
 * Returns: the previous target id for the LAW register <law_id> or -1 on error
 * 
 * Note that this function will return 0 if an invalid LAW register id is
 * provided. The caller should ensure that <law_id> is correct using one of the
 * other provided functions
*/
int LAW_FUNC(law_set_target)(unsigned law_id, unsigned target_id)
{
	size_t prev_target = LAW_FUNC(law_get_target)(law_id);

	if (law_id >= MPC85XX_NUM_LAW_REGS) return -1;
	if (prev_target != target_id)
	{
		unsigned is_enabled = LAW_FUNC(law_is_enabled)(law_id);
		volatile uint32_t reg = IN32(MPC85xx_REGOFF_LAWAR(law_id));
		
		LAW_FUNC(law_disable)(law_id);
		OUT32(MPC85xx_REGOFF_LAWAR(law_id), (reg & ~MPC85XX_LAWAR_TGTID_MASK) | ((target_id << 20) & MPC85XX_LAWAR_TGTID_MASK));
		if (is_enabled) LAW_FUNC(law_enable)(law_id);
	}
	return prev_target;
}


/*
 * =============================================================================
 *                             M I S C   A P I's
 * =============================================================================
*/
void LAW_FUNC(law_dump)(int _law_id)
{
	unsigned law_id_end; 
	unsigned law_id = (_law_id < 0) ? 0 : _law_id;
	unsigned overlap_count = 0;

	law_id_end = (_law_id < 0) ? MPC85XX_NUM_LAW_REGS : law_id + 1;
	while (law_id < law_id_end)
	{
		uint64_t law_addr = LAW_FUNC(law_get_addr)(law_id);
		
		kprintf("[%u] en:%u, tgt: %x, addr: %x %x, sz: %x (%x)\n", law_id, LAW_FUNC(law_is_enabled)(law_id),
				LAW_FUNC(law_get_target)(law_id), (uint32_t)(law_addr >> 32), (uint32_t)law_addr,
				LAW_FUNC(law_get_size)(law_id), IN32(MPC85xx_REGOFF_LAWAR(law_id)) & MPC85XX_LAWAR_SIZE_MASK);
		++law_id;
	}
	
	/* check for overlapping LAW registers */
	for (law_id=0; law_id < MPC85XX_NUM_LAW_REGS; ++law_id)
	{
		int overlap_id = law_id;	// we only need to look forward (avoids duplicate output)
		while(1)
		{
			overlap_id = find_law(LAW_FUNC(law_get_addr)(law_id), LAW_FUNC(law_get_size)(law_id), overlap_id + 1, -1);
			if (overlap_id < 0) break;
			if (overlap_id != law_id)
			{
				++overlap_count;
				kprintf("[%s] LAW register %u overlaps %u with %s target id's\n",
						(LAW_FUNC(law_is_enabled)(law_id) && LAW_FUNC(law_is_enabled)(overlap_id)) ? "ERROR" : "CAUTION",
						law_id, overlap_id,
						LAW_FUNC(law_get_target)(law_id) == LAW_FUNC(law_get_target)(overlap_id) ? "same" : "different");
			}
		}
	}
	kprintf("%u LAW register %s detected\n", overlap_count, overlap_count > 1 ? "overlaps" : "overlap");
}


/*
 * =============================================================================
 *                      S U P P O R T   F U N C T I O N S
 * =============================================================================
*/

/*******************************************************************************
 * find_law
 * 
 * This function will search for the law register which "matches" the address
 * range from <start> to <start> + <size> - 1. By match, we mean the LAW register
 * that would "hit" the given address range and therefore if any part of the
 * address range falls within the range for the LAW register, a match exists.
 * The search is started from <law_id>. An optional <target_id> can be provided
 * which will filter the search to a specific target. A value of -1 means any
 * target.
 * The search does not consider whether the LAW register is enabled. The caller
 * can test for that if desired.
 * Multiple matches can be discovered by repeatedly calling this function with
 * the non-negative return value from the previous call + 1 as the <law_id>
 * (pass a <law_id> == 0 for the first call).
 * 
 * Returns: the law id which matches the address range from <start> to
 * 			<start> + <size> - 1 or -1 if no match exists
*/
static int find_law(uint64_t start, size_t size, int law_id, int target_id)
{
	law_id = (law_id < 0) ? 0 : law_id;

	for (; law_id < MPC85XX_NUM_LAW_REGS; ++law_id)
	{
		uint64_t law_addr = LAW_FUNC(law_get_addr)(law_id);
		size_t law_size = LAW_FUNC(law_get_size)(law_id);
		
		if ((start + size - 1) < law_addr) continue;				// no match
		if (start > (law_addr + law_size - 1)) continue;			// no match
		
		/* have a range match. Check for target_id as required */
		if ((target_id < 0) || (target_id == LAW_FUNC(law_get_target)(law_id)))
			return law_id;
	}
	return -1;
}

/*******************************************************************************
 * overlap_check
 * 
 * Returns: true (!0) or false (0) depending on whether the address range
 * 			between <start> and <end> inclusive overlaps an existing enabled LAW
 * 			register that is not <exclude_law_id>
*/
static unsigned overlap_check(uint64_t start, uint64_t end, int exclude_law_id)
{
	int overlap_id = -1;
	
	ASSERT(end > start);
	while(1)
	{
		overlap_id = find_law(start, (size_t)(end - start), overlap_id + 1, -1);
		if (overlap_id < 0) return 0;	// no overlap
		if ((overlap_id != exclude_law_id) && LAW_FUNC(law_is_enabled)(overlap_id)) return 1;	// overlap
	}
	return 0;
}


#endif	/* LAW_FUNC */
