/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "startup.h"
#include <ppc/p2020cpu.h>
#include <hw/hwinfo_p20xx.h>
#include <pic_85xx.h>
#include <drvr/hwinfo.h>


/*
 * =============================================================================
 *                                M A C R O S
 * =============================================================================
*/

/*
 * =============================================================================
 *                   F O R W A R D   D E C L A R A T I O N S
 * =============================================================================
*/

/*
 * =============================================================================
 *                               A P I 's
 * =============================================================================
*/

/*******************************************************************************
 * p2020_config_pic
 * 
 * This routine will initialize the PIC (Programmable Interrupt Controller) on
 * the P2020 processor.
 * 
 * Each interruptable device listed in the P2020 manual will have its interrupt
 * attributes (vector, priority, polarity, sense, etc) initialized to known
 * values. In order to uniquely identify each interrupting device, a unique
 * vector is assigned to each device starting at <vector_start>. For the P2020
 * PIC, it does not matter which vector is assigned to which device so the
 * vectors are simply assigned in linearly increasing order (starting at
 * <pic_vector_start>) for each interrupting device. The same is done for the
 * OS assigned vector numbers (the vectors that are used in InterruptAttach()
 * calls and that can be obtained from the hwinfo section of the syspage).
 * 
 * The reason why this separation of PIC and OS vectors is done is for flexibility
 * and to specifically allow a priority value to be encoded into the
 * pic_vector_offset parameter. If the priority field is not set, then a value
 * of 1 will be used. If you wish to assign different priorities to different
 * devices you can copy this function to the BSP directory and modify it to only
 * do a subset of the devices. The other devices can be done with a similar call
 * and different parameters (remember not to do the reset twice)
 * 
 * As each vector is assigned, the address of the mask register is entered into
 * the <reg_vectors>->offset[] table. This table is patched into the callouts
 * using the r/w offset callout parameter and allows a quick lookup during
 * mask and unmask operations using the assigned vector normalized to 0.
 * 
 * Similarly, any flags that you would like returned in the config() routine
 * can also be assigned if reg_vectors.config_flags != NULL (see below for the
 * IPI interrupts)
 * 
 * If an offset for a vector is not to be used (essentially invalidating the
 * vector), the offset should be set to -1 (0xFFFFFFFFU). This allows holes to
 * be left in the vector map. Leaving holes in the vector map is not encouraged
 * as it wastes vectors, vector table register offset memory and makes the mask
 * and unmask callouts more difficult
 * 
 * To facilitate a portable method for device driver writers obtaining the
 * correct vector to use in InterruptAttach() calls, if the device has been
 * added to the hwinfo section of the syspage (during init_hwinfo()) then the
 * assigned vector(s) for that device will be inserted for that device item
 * (see note above for hwitag_set_avail_ivec() for details on this).
 * 
 * Returns: the number of interrupt vectors assigned
 * 
 * Implementation Note:
 * Because the way in which the P2020 family has evolved, groups of similar
 * interruptable devices have been scattered throughout the PIC register map.
 * This routine initializes devices with similar functionality as a group. This
 * is just a choice and is irrelevant from an interface standpoint.
 * 
 * If you wish to control the assigned PIC vector values and/or the assigned
 * OS vector ranges, you can copy this function and break up the initializations
 * as you see fit
*/
uint32_t p2020_config_pic(uint32_t os_vector_base, uint32_t pic_vector_base, vector_tbl_t *reg_vectors)
{
	uintptr_t base;
	unsigned i;
	uint32_t priority = pic_vector_base & PPCP2020_xIVPR_PRIORITY_MASK;	// extract the priority field

	pic_vector_base &= PPCP2020_xIVPR_VECTOR_MASK;	// make sure the base vector number is limited to 16 bits

	reg_vectors->start = pic_vector_base;
	reg_vectors->num = 0;
	
	/* if no priority is specified, use 1 (the lowest non disabling priority) */
	if (priority == 0)
		priority = (1 << PPCP2020_xIVPR_PRIORITY_SHIFT);

	base = startup_io_map(MEG(1), immr_paddr);

	/* reset the PIC to known values */
	PIC_OUT(base, GCR,PPCP2020_GCR_RST);
	do { // Wait for the reset to complete
	} while(PIC_IN(base, GCR) & PPCP2020_GCR_RST);


	/* set the per cpu current task priority registers to 0 */
	PIC_OUT(base, GCR,PPCP2020_GCR_M);
	for(i = 0; i < lsp.syspage.p->num_cpu; ++i) {
		out32(base + (i*0x1000) + PPC85xx_CTPR_PCPU_0, 0);
	}

	/*
	 * NOTE: we are programming the interrupts in linearly increasing vector
	 * order to minimize the number of vectors. We do not program a vector for
	 * a non existent device
	*/

	/* L2 cache (internal interrupt number 0) */
	PIC_OUT(base, IIVPR(0), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(0), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(0));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_L2CACHE, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* ECM (internal interrupt number 1) */
	PIC_OUT(base, IIVPR(1), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(1), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(1));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_ECM, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DRAM controllers (internal interrupt number 2) */
	PIC_OUT(base, IIVPR(2), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(2), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(2));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DRAM, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* LBC (internal interrupt number 3) */
	PIC_OUT(base, IIVPR(3), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(3), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(3));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_LBC, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DMA 1, Channel 0 (internal interrupt number 4) */
	PIC_OUT(base, IIVPR(4), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(4), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(4));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DMA 1, Channel 1 (internal interrupt number 5) */
	PIC_OUT(base, IIVPR(5), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(5), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(5));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 0), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 1, Channel 2 (internal interrupt number 6) */
	PIC_OUT(base, IIVPR(6), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(6), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(6));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 0), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 1, Channel 3 (internal interrupt number 7) */
	PIC_OUT(base, IIVPR(7), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(7), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(7));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 0), 3, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 2, Channel 0 (internal interrupt number 60) */
	PIC_OUT(base, IIVPR(60), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(60), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(60));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;;	
	
	/* DMA 2, Channel 1 (internal interrupt number 61) */
	PIC_OUT(base, IIVPR(61), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(61), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(61));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 1), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;;	

	/* DMA 2, Channel 2 (internal interrupt number 62) */
	PIC_OUT(base, IIVPR(62), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(62), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(62));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 1), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 2, Channel 3 (internal interrupt number 63) */
	PIC_OUT(base, IIVPR(63), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(63), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(63));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_DMA, 1), 3, reg_vectors->num + os_vector_base);
	++reg_vectors->num;;

	/* PCIe Port 1, (internal interrupt number 10) */
	PIC_OUT(base, IIVPR(10), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(10), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(10));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_PCIe, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* PCIe Port 2, (internal interrupt number 9) */
	PIC_OUT(base, IIVPR(9), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(9), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(9));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_PCIe, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* PCIe Port 3, (internal interrupt number 8) */
	PIC_OUT(base, IIVPR(8), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(8), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(8));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_PCIe, 2), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* USB controller , (internal interrupt number 12) */
	PIC_OUT(base, IIVPR(12), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(12), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(12));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_USB, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC1 Tx, (internal interrupt number 13) */
	PIC_OUT(base, IIVPR(13), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(13), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(13));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC1 Rx, (internal interrupt number 14) */
	PIC_OUT(base, IIVPR(14), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(14), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(14));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 0), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* eTSEC1 Err, (internal interrupt number 18) */
	PIC_OUT(base, IIVPR(18), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(18), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(18));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 0), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC2 Tx, (internal interrupt number 19) */
	PIC_OUT(base, IIVPR(19), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(19), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(19));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC2 Rx, (internal interrupt number 20) */
	PIC_OUT(base, IIVPR(20), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(20), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(20));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 1), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* eTSEC2 Err, (internal interrupt number 24) */
	PIC_OUT(base, IIVPR(24), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(24), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(24));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 1), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC3 Tx, (internal interrupt number 15) */
	PIC_OUT(base, IIVPR(15), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(15), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(15));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 2), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* eTSEC3 Rx, (internal interrupt number 16) */
	PIC_OUT(base, IIVPR(16), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(16), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(16));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 2), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* eTSEC3 Err, (internal interrupt number 17) */
	PIC_OUT(base, IIVPR(17), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(17), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(17));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_eTSEC, 2), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DUART, (internal interrupt number 26) */
	PIC_OUT(base, IIVPR(26), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(26), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(26));
	/* There are 2 UART's connected to the same interrupt. Tell the driver */
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_UART, 0), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_UART, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* I2C, (internal interrupt number 27) */
	PIC_OUT(base, IIVPR(27), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(27), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(27));
	/* There are 2 I2C devices connected to the same interrupt. Tell the driver */
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_I2C, 0), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_bus(PPCP20xx_HWI_I2C, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* PerfMon, (internal interrupt number 28) */
	PIC_OUT(base, IIVPR(28), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(28), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(28));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_PMON, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* Security 1, (internal interrupt number 29) */
	PIC_OUT(base, IIVPR(29), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(29), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(29));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SECURITY, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* Security 2, (internal interrupt number 42) */
	PIC_OUT(base, IIVPR(42), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(42), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(42));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SECURITY, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* GPIO, (internal interrupt number 31) */
	PIC_OUT(base, IIVPR(31), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(31), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(31));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_GPIO, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* SRIO Err/Write Port, (internal interrupt number 32) */
	PIC_OUT(base, IIVPR(32), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(32), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(32));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_EWP, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* SRIO Outbound Doorbell, (internal interrupt number 33) */
	PIC_OUT(base, IIVPR(33), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(33), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(33));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_DOORBELL, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* SRIO Inbound Doorbell, (internal interrupt number 34) */
	PIC_OUT(base, IIVPR(34), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(34), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(34));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_DOORBELL, 0), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* SRIO Outbound Message Unit 1, (internal interrupt number 37) */
	PIC_OUT(base, IIVPR(37), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(37), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(37));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_MSGUNIT, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* SRIO Inbound Message Unit 1, (internal interrupt number 38) */
	PIC_OUT(base, IIVPR(38), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(38), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(38));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_MSGUNIT, 0), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	
	
	/* SRIO Outbound Message Unit 2, (internal interrupt number 39) */
	PIC_OUT(base, IIVPR(39), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(39), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(39));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_MSGUNIT, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* SRIO Inbound Message Unit 2, (internal interrupt number 40) */
	PIC_OUT(base, IIVPR(40), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(40), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(40));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SRIO_MSGUNIT, 1), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* IEEE588 TSEC 1, (internal interrupt number 52) */
	PIC_OUT(base, IIVPR(52), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(52), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(52));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_IEEE588_TSEC, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* IEEE588 TSEC 2, (internal interrupt number 53) */
	PIC_OUT(base, IIVPR(53), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(53), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(53));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_IEEE588_TSEC, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	
	
	/* IEEE588 TSEC 3, (internal interrupt number 54) */
	PIC_OUT(base, IIVPR(54), (PPCP2020_xIVPR_MSK|PPCP2020_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(54), PPCP2020_xIDR_P0);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IIVPR(54));
	hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_IEEE588_TSEC, 2), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* 4 IPI's */ 
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, IPIVPR(i), PPCP2020_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_IPIVPR(i));
		/* preattach these interrupts and don't allow further attaches */
		set_config_flags(reg_vectors, reg_vectors->num, INTR_CONFIG_FLAG_IPI | INTR_CONFIG_FLAG_DISALLOWED | INTR_CONFIG_FLAG_PREATTACH);
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_IPI, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	
	/* Global Timer Group A (4 timers) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, GTVPR(i), PPCP2020_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, GTDR(i), PPCP2020_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_GTVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_GTIMER, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	/* Global Timer Group B (4 timers) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, GTVPR(i) + 0x1000, PPCP2020_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, GTDR(i) + 0x1000, PPCP2020_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_GTVPR(i) + 0x1000);
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_GTIMER, 1), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/* 8 Message Interrupts */
	for(i=0; i<8; i++)
	{
		PIC_OUT(base, MIVPR(i), PPCP2020_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MIDR(i), PPCP2020_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_MIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_MSG, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/* 8 Message shared interrupts */	
	for(i=0; i<8; i++)
	{
		PIC_OUT(base, MSIVPR(i), PPCP2020_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MSIDR(i), PPCP2020_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_MSIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_SMSG, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/* 12 External interrupts (IRQ0 - IRQ11) */
	/*IRQ0 to IRQ6 are externally Pulled high as per schematics*/
	for(i=0; i<7; i++)
	{
			/* In p2020DS, IRQ4 & IRQ0 are connected with ngPixis FPGA which sends Falling edge sensitive interrupts.
			 * It is coming from the 8259 INTR output from ULI device. Lets configure it correctly*/
		if(i==4 || i==0){
				PIC_OUT(base, EIVPR(i), PPCP2020_xIVPR_MSK| priority | (reg_vectors->num + pic_vector_base));
		}
		else{
				/*Set rest all the Level Sensitive Active LOW*/
				PIC_OUT(base, EIVPR(i), (PPCP2020_xIVPR_MSK | PPCP2020_xIVPR_S) | priority | \
				(reg_vectors->num + pic_vector_base));
		}
		PIC_OUT(base, EIDR(i), PPCP2020_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_EIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_EXT_IRQ, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/*IRQ7 to IRQ11 are Not connected at Board*/
	for(i=7; i<12; i++)
	{
			/*Set IRQ7 to IRQ11 to Level Sensitive Active HIGH*/
			PIC_OUT(base, EIVPR(i), (PPCP2020_xIVPR_MSK | PPCP2020_xIVPR_S|PPCP2020_xIVPR_P) | priority | \
							(reg_vectors->num + pic_vector_base));
			PIC_OUT(base, EIDR(i), PPCP2020_xIDR_P0);
			set_regoffset(reg_vectors, reg_vectors->num, PPCP2020_CCSR_OFF_EIVPR(i));
			hwitag_set_avail_ivec(hwi_find_device(PPCP20xx_HWI_EXT_IRQ, 0), i, reg_vectors->num + os_vector_base);
			++reg_vectors->num;
	}

	startup_io_unmap(base);
	
	return reg_vectors->num;
}


/*
 * =============================================================================
 *                     S U P P O R T   R O U T I N E S
 * =============================================================================
*/




__SRCVERSION("pic_p2020.c $Rev: 170330 $");
