/*
 * $QNXLicenseC:
 * Copyright 2009, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "startup.h"
#include <ppc/p4080cpu.h>
#include <hw/hwinfo_p40xx.h>
#include <pic_p40xx.h>
#include <drvr/hwinfo.h>


/*
 * =============================================================================
 *                                M A C R O S
 * =============================================================================
*/

/*
 * =============================================================================
 *                   F O R W A R D   D E C L A R A T I O N S
 * =============================================================================
*/

/*
 * =============================================================================
 *                               A P I 's
 * =============================================================================
*/

/*******************************************************************************
 * p4080_config_pic
 * 
 * This routine will initialize the MPIC (Multi-Core Programmable Interrupt Controller) 
 * on the P4080 processor.
 * 
 * Each interruptable device listed in the P4080 manual will have its interrupt
 * attributes (vector, priority, polarity, sense, etc) initialized to known
 * values. In order to uniquely identify each interrupting device, a unique
 * vector is assigned to each device starting at <vector_start>. For the P4080
 * MPIC, it does not matter which vector is assigned to which device so the
 * vectors are simply assigned in linearly increasing order (starting at
 * <pic_vector_start>) for each interrupting device. The same is done for the
 * OS assigned vector numbers (the vectors that are used in InterruptAttach()
 * calls and that can be obtained from the hwinfo section of the syspage).
 * 
 * The reason why this separation of PIC and OS vectors is done is for flexibility
 * and to specifically allow a priority value to be encoded into the
 * pic_vector_offset parameter. If the priority field is not set, then a value
 * of 1 will be used. If you wish to assign different priorities to different
 * devices you can copy this function to the BSP directory and modify it to only
 * do a subset of the devices. The other devices can be done with a similar call
 * and different parameters (remember not to do the reset twice)
 * 
 * As each vector is assigned, the address of the mask register is entered into
 * the <reg_vectors>->offset[] table. This table is patched into the callouts
 * using the r/w offset callout parameter and allows a quick lookup during
 * mask and unmask operations using the assigned vector normalized to 0.
 * 
 * Similarly, any flags that you would like returned in the config() routine
 * can also be assigned if reg_vectors.config_flags != NULL (see below for the
 * IPI interrupts)
 * 
 * If an offset for a vector is not to be used (essentially invalidating the
 * vector), the offset should be set to -1 (0xFFFFFFFFU). This allows holes to
 * be left in the vector map. Leaving holes in the vector map is not encouraged
 * as it wastes vectors, vector table register offset memory and makes the mask
 * and unmask callouts more difficult
 * 
 * To facilitate a portable method for device driver writers obtaining the
 * correct vector to use in InterruptAttach() calls, if the device has been
 * added to the hwinfo section of the syspage (during init_hwinfo()) then the
 * assigned vector(s) for that device will be inserted for that device item
 * (see note above for hwitag_set_avail_ivec() for details on this).
 * 
 * Returns: the number of interrupt vectors assigned
 * 
 * Implementation Note:
 * Because the way in which the P40xx family has evolved, groups of similar
 * interruptable devices have been scattered throughout the PIC register map.
 * This routine initializes devices with similar functionality as a group. This
 * is just a choice and is irrelevant from an interface standpoint.
 * 
 * If you wish to control the assigned PIC vector values and/or the assigned
 * OS vector ranges, you can copy this function and break up the initializations
 * as you see fit
*/
uint32_t p4080_config_pic(uint32_t os_vector_base, uint32_t pic_vector_base, vector_tbl_t *reg_vectors)
{
	uintptr_t base;
	unsigned i;
	uint32_t priority = pic_vector_base & PPCP4080_xIVPR_PRIORITY_MASK;	// extract the priority field

	pic_vector_base &= PPCP4080_xIVPR_VECTOR_MASK;	// make sure the base vector number is limited to 16 bits

	reg_vectors->start = pic_vector_base;
	reg_vectors->num = 0;
	
	/* if no priority is specified, use 1 (the lowest non disabling priority) */
	if (priority == 0)
		priority = (1 << PPCP4080_xIVPR_PRIORITY_SHIFT);

	base = startup_io_map(MEG(1), immr_paddr);

	/* reset the PIC to known values */
	PIC_OUT(base, GCR,PPCP4080_GCR_RST);
	do { // Wait for the reset to complete
	} while(PIC_IN(base, GCR) & PPCP4080_GCR_RST);

	/* Programming the MPIC to the Mixed Mode */
	PIC_OUT(base, GCR,PPCP4080_GCR_M);
	
	/* set the per cpu current task priority registers to 0 */
	for(i = 0; i < lsp.syspage.p->num_cpu; ++i) {
		out32(base + (i*0x1000) + PPCP4080_CCSR_OFF_PC_CTPTR, 0);
	}

	/*
	 * NOTE: we are programming the interrupts in linearly increasing vector
	 * order to minimize the number of vectors. We do not program a vector for
	 * a non existent device
	*/
	/* LBC (internal interrupt number 9) */
	PIC_OUT(base, IIVPR(9), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(9), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(9), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(9));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_LBC, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DMA 1, Channel 0 (internal interrupt number 12) */
	PIC_OUT(base, IIVPR(12), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(12), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(12), DEFAULT_INT_TARGET);	
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(12));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DMA 1, Channel 1 (internal interrupt number 13) */
	PIC_OUT(base, IIVPR(13), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(13), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(13), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(13));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 0), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 1, Channel 2 (internal interrupt number 14) */
	PIC_OUT(base, IIVPR(14), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(14), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(14), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(14));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 0), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 1, Channel 3 (internal interrupt number 15) */
	PIC_OUT(base, IIVPR(15), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(15), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(15), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(15));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 0), 3, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 2, Channel 0 (internal interrupt number 16) */
	PIC_OUT(base, IIVPR(16), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(16), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(16), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(16));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	
	
	/* DMA 2, Channel 1 (internal interrupt number 17) */
	PIC_OUT(base, IIVPR(17), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(17), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(17), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(17));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 1), 1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 2, Channel 2 (internal interrupt number 18) */
	PIC_OUT(base, IIVPR(18), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(18), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(18), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(18));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 1), 2, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DMA 2, Channel 3 (internal interrupt number 19) */
	PIC_OUT(base, IIVPR(19), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(19), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(19), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(19));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_DMA, 1), 3, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* PCIe 1 INTA, (internal interrupt number 24) */
	PIC_OUT(base, IIVPR(24), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(24), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(24), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(24));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_PCIe, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* PCIe 2 INTA, (internal interrupt number 25) */
	PIC_OUT(base, IIVPR(25), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(25), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(25), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(25));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_PCIe, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* PCIe 3 INTA, (internal interrupt number 26) */
	PIC_OUT(base, IIVPR(26), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(26), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(26), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(26));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_PCIe, 2), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* USB controller 1, (internal interrupt number 28) */
	PIC_OUT(base, IIVPR(28), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(28), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(28), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(28));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_USB, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* USB controller 2, (internal interrupt number 29) */
	PIC_OUT(base, IIVPR(29), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(29), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(29), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(29));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_USB, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* DUART1, (internal interrupt number 20) */
	PIC_OUT(base, IIVPR(20), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(20), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(20), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(20));
	/* There are 2 UART's connected to the same interrupt. Tell the driver */
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_UART, 0), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_UART, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* DUART2, (internal interrupt number 21) */
	PIC_OUT(base, IIVPR(21), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(21), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(21), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(21));
	/* There are 2 UART's connected to the same interrupt. Tell the driver */
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_UART, 2), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_UART, 3), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* I2C, (internal interrupt number 22) */
	PIC_OUT(base, IIVPR(22), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(22), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(22), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(22));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_I2C, 0), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_I2C, 1), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;
	
	/* I2C, (internal interrupt number 23) */
	PIC_OUT(base, IIVPR(23), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(23), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(23), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(23));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_I2C, 2), -1, reg_vectors->num + os_vector_base);
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_I2C, 3), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* QMAN Registers */
	for(i=0;i<10;i++){
		PIC_OUT(base, IIVPR(88 + i*2), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, IIDR(88 + i*2), PPCP4080_xIDR_P0);
		PIC_OUT(base, IILR(88 + i*2), DEFAULT_INT_TARGET);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(88 + i*2));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_QMAN, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;	
	}

	/* BMAN Registers */
	for(i=0;i<10;i++){
		PIC_OUT(base, IIVPR(89 + i*2), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, IIDR(89 + i*2), PPCP4080_xIDR_P0);
		PIC_OUT(base, IILR(89 + i*2), DEFAULT_INT_TARGET);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(89 + i*2));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_BMAN, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;	
	}
	
	/*FMAN1 Interrupts 80 */
	PIC_OUT(base, IIVPR(80), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(80), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(80), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(80));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_FMAN, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	

	/*FMAN2 Interrupts 81 */
	PIC_OUT(base, IIVPR(81), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(81), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(81), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(81));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_FMAN, 1), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	
	
	/*Common Error interrupt 0*/
	PIC_OUT(base, IIVPR(0), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(0), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(0), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(0));
	hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_CERR, 0), 0, reg_vectors->num + os_vector_base);
	++reg_vectors->num;	
		
	/* SPI Interrupt 37 */
	PIC_OUT(base, IIVPR(37), (PPCP4080_xIVPR_MSK|PPCP4080_xIVPR_P) | priority | (reg_vectors->num + pic_vector_base));
	PIC_OUT(base, IIDR(37), PPCP4080_xIDR_P0);
	PIC_OUT(base, IILR(37), DEFAULT_INT_TARGET);
	set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IIVPR(37));
	hwitag_set_avail_ivec(hwi_find_bus(PPCP40xx_HWI_SPI, 0), -1, reg_vectors->num + os_vector_base);
	++reg_vectors->num;

	/* 4 IPI's */ 
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, IPIVPR(i), PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_IPIVPR(i));
		/* preattach these interrupts and don't allow further attaches */
		set_config_flags(reg_vectors, reg_vectors->num, INTR_CONFIG_FLAG_IPI | INTR_CONFIG_FLAG_DISALLOWED | INTR_CONFIG_FLAG_PREATTACH);
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_IPI, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	
	/* Global Timer Group A (4 timer interrupts) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, GTVPR(i), PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, GTDR(i), PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_GTVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_GTIMER, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	/* Global Timer Group B (4 timer interrupts) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, GTVPR(i) + 0x1000, PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, GTDR(i) + 0x1000, PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_GTVPR(i) + 0x1000);
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_GTIMER, 1), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/* Group A Message Interrupts (4 Message interrupts) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, MIVPR(i), PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MIDR(i), PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_MIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_MSG, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	
	/* Group B Message Interrupts (4 Message interrupts) */
	for(i=0; i<4; i++)
	{
		PIC_OUT(base, MIVPR(i) + 0x80, PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MIDR(i) + 0x80, PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_MIVPR(i) + 0x80);
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_MSG, 1), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}	

	/* Bank A Message shared interrupts (8 Message Shared interrupts) */	
	for(i=0; i<8; i++)
	{
		PIC_OUT(base, MSIVPR(i), PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MSIDR(i), PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_MSIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_SMSG, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}
	
	/* Bank B Message shared interrupts (8 Message Shared interrupts) */
	for(i=0; i<8; i++)
	{
		PIC_OUT(base, MSIVPR(i) + 0x100, PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MSIDR(i) + 0x100, PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_MSIVPR(i) + 0x100);
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_SMSG, 1), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	/* Bank C Message shared interrupts (8 Message Shared interrupts) */
	for(i=0; i<8; i++)
	{
		PIC_OUT(base, MSIVPR(i) + 0x200, PPCP4080_xIVPR_MSK | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, MSIDR(i) + 0x200, PPCP4080_xIDR_P0);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_MSIVPR(i) + 0x200);
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_SMSG, 2), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}	

	/* 12 External interrupts (IRQ0 - IRQ11) */
	/* We are configuring as level sensitive */
	for(i=0; i<12; i++)
	{
		PIC_OUT(base, EIVPR(i), (PPCP4080_xIVPR_MSK | PPCP4080_xIVPR_S) | priority | (reg_vectors->num + pic_vector_base));
		PIC_OUT(base, EIDR(i), PPCP4080_xIDR_P0);
		PIC_OUT(base, EILR(i), DEFAULT_INT_TARGET);
		set_regoffset(reg_vectors, reg_vectors->num, PPCP4080_CCSR_OFF_EIVPR(i));
		hwitag_set_avail_ivec(hwi_find_device(PPCP40xx_HWI_EXT_IRQ, 0), i, reg_vectors->num + os_vector_base);
		++reg_vectors->num;
	}

	startup_io_unmap(base);
	
	return reg_vectors->num;
}


/*
 * =============================================================================
 *                     S U P P O R T   R O U T I N E S
 * =============================================================================
*/

__SRCVERSION("pic_p4080.c $Rev: $");
