/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "std_ext.h"
#include "error_ext.h"
#include "debug_ext.h"
#include "xx_ext.h"
#include "string_ext.h"
#include "gpio.h"



static void     GpioFreeResources(t_Gpio *p_Gpio);

static t_Error  GpioConfigurePin(t_Handle h_Gpio, int ioPin);

static uint8_t  GpioReadPinInput(t_Handle h_Gpio, int ioPin);

static t_Error  GpioSetPinOutput(t_Handle h_Gpio, int ioPin, uint8_t output);

static t_Error  GpioSetPinIntrMode(t_Handle         h_Gpio,
                                   int              ioPin,
                                   e_IoPinIntrMode  intrMode);

static void     GpioIntrHandler(t_Handle h_Gpio);


/*****************************************************************************/
t_Handle GPIO_ConfigAndInit(t_GpioParam *p_GpioParam)
{
    t_Gpio      *p_Gpio;
    uint32_t    validPinMasks[] = GPIO_VALID_PIN_MASKS;
    uint32_t    validIntrMasks[] = GPIO_VALID_INTR_MASKS;
    int         i, j, numOfIntrPins;

    /* If we have more than 4 ports, interrupt handling must be modified.
       Currently we pass portId or'ed with the GPIO handle */
    ASSERT_COND(GPIO_NUM_OF_PORTS <= 4);

    p_Gpio = (t_Gpio *)XX_Malloc(sizeof(t_Gpio));
    if (!p_Gpio)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("GPIO driver structure"));
        return NULL;
    }
    /* See assertion above regarding interrupt handling */
    ASSERT_COND(((uint32_t)p_Gpio & 0x00000003) == 0);
    memset(p_Gpio, 0, sizeof(t_Gpio));

    /* Initialize GPIO structure */
    p_Gpio->p_MemMap = (t_GpioMemMap *)p_GpioParam->baseAddr;

    p_Gpio->ioPortModule.p_Ports = (t_IoPort *)
        XX_Malloc(GPIO_NUM_OF_PORTS * sizeof(t_IoPort));
    if (!p_Gpio->ioPortModule.p_Ports)
    {
        GpioFreeResources(p_Gpio);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("I/O ports structures"));
        return NULL;
    }
    memset(p_Gpio->ioPortModule.p_Ports, 0, GPIO_NUM_OF_PORTS * sizeof(t_IoPort));

    numOfIntrPins = (GPIO_NUM_OF_PORTS * NUM_OF_PINS_PER_PORT);

    p_Gpio->ioPortModule.p_IntrInfo = (t_IoPinIntrInfo *)
        XX_Malloc(numOfIntrPins * sizeof(t_IoPinIntrInfo));
    if (!p_Gpio->ioPortModule.p_IntrInfo)
    {
        GpioFreeResources(p_Gpio);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Interrupt information structure"));
        return NULL;
    }
    memset(p_Gpio->ioPortModule.p_IntrInfo, 0, numOfIntrPins * sizeof(t_IoPinIntrInfo));

    /* Initialize generic I/O ports structure */
    p_Gpio->ioPortModule.f_ConfigurePin    = GpioConfigurePin;
    p_Gpio->ioPortModule.f_ReadPinInput    = GpioReadPinInput;
    p_Gpio->ioPortModule.f_SetPinOutput    = GpioSetPinOutput;
    p_Gpio->ioPortModule.f_SetPinIntrMode  = GpioSetPinIntrMode;
    p_Gpio->ioPortModule.numOfPorts        = GPIO_NUM_OF_PORTS;

    for (i = 0; i < GPIO_NUM_OF_PORTS; i++)
    {
        p_Gpio->ioPortModule.p_Ports[i].validPinsMask = validPinMasks[i];
        p_Gpio->ioPortModule.p_Ports[i].validIntrMask = validIntrMasks[i];

        for (j = 0; j < NUM_OF_PINS_PER_PORT; j++)
        {
            if (validIntrMasks[i] & (PIN_0_MASK >> j))
            {
                /* Initialize interrupt information */
                p_Gpio->ioPortModule.p_Ports[i].pins[j].intrId =
                    (uint8_t)((i * NUM_OF_PINS_PER_PORT) + j);
            }
            else
            {
                p_Gpio->ioPortModule.p_Ports[i].pins[j].intrId = NO_INTR;
            }
        }

        /* Clear event bits in the event register */
        WRITE_UINT32(p_Gpio->p_MemMap->io_regs[i].gpier, 0xFFFFFFFF);

        p_Gpio->irq[i] = p_GpioParam->irq[i];

        if (p_Gpio->irq[i] != -1)
        {
            /* Register and enable GPIO interrupts */
            XX_SetIntr(p_Gpio->irq[i], GpioIntrHandler, (t_Handle)((uint32_t)p_Gpio | i));
            XX_EnableIntr(p_Gpio->irq[i]);
        }
    }

    return p_Gpio;
}


/*****************************************************************************/
t_Error GPIO_Free(t_Handle h_Gpio)
{
    SANITY_CHECK_RETURN_ERROR(h_Gpio, E_INVALID_HANDLE);

    GpioFreeResources((t_Gpio *)h_Gpio);

    XX_Free(h_Gpio);
    return E_OK;
}


#if (DEBUG_ERRORS > 0)

/*****************************************************************************/
t_Error GPIO_DumpRegs(t_Handle h_Gpio)
{
    t_Gpio  *p_Gpio = (t_Gpio *)h_Gpio;
    int     i;

    DECLARE_DUMP;

    SANITY_CHECK_RETURN_ERROR(p_Gpio, E_INVALID_HANDLE);

    DUMP_TITLE(p_Gpio->p_MemMap, ("GPIO Registers"));

    DUMP_SUBSTRUCT_ARRAY(i, GPIO_NUM_OF_PORTS)
    {
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpdir);
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpodr);
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpdat);
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpier);
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpimr);
        DUMP_VAR(p_Gpio->p_MemMap, io_regs[i].gpicr);
    }

    return E_OK;
}

#endif /* DEBUG_ERRORS > 0 */


/*****************************************************************************/
static void GpioFreeResources(t_Gpio *p_Gpio)
{
    int i;

    for (i = 0; i < GPIO_NUM_OF_PORTS; i++)
    {
        if (p_Gpio->irq[i] != -1)
        {
            XX_DisableIntr(p_Gpio->irq[i]);
            XX_FreeIntr(p_Gpio->irq[i]);
        }
    }
    if (p_Gpio->ioPortModule.p_IntrInfo)
    {
        XX_Free(p_Gpio->ioPortModule.p_IntrInfo);
        p_Gpio->ioPortModule.p_IntrInfo = NULL;
    }
    if (p_Gpio->ioPortModule.p_Ports)
    {
        XX_Free(p_Gpio->ioPortModule.p_Ports);
        p_Gpio->ioPortModule.p_Ports = NULL;
    }
}


/*****************************************************************************/
static t_Error GpioConfigurePin(t_Handle h_Gpio, int ioPin)
{
    t_Gpio              *p_Gpio = (t_Gpio *)h_Gpio;
    uint8_t             portId = IO_PIN_PORT_ID(ioPin);
    uint8_t             pinId = IO_PIN_ID(ioPin);
    e_IoPinDirection    direction = IO_PIN_DIRECTION(ioPin);
    e_IoPinAssignment   assignment = IO_PIN_ASSIGNMENT(ioPin);
    uint32_t            gpdir, gpodr;
    uint32_t            pinMask;

    SANITY_CHECK_RETURN_ERROR(p_Gpio, E_INVALID_HANDLE);

    if (assignment != e_IO_PIN_ASSIGN_GPIO)
    {
        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("Pin assignment"));
    }

    /* Calculate pin location */
    pinMask = (uint32_t)(PIN_0_MASK >> pinId);

    /* Set direction and open drain registers */
    gpodr = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpodr);
    gpodr &= ~(pinMask);

    gpdir = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpdir);
    gpdir &= ~(pinMask);

    switch (direction)
    {
        case e_IO_PIN_DIR_OPEN_DRAIN_OUTPUT:
            gpodr |= pinMask;
            /* Fall-through */
        case e_IO_PIN_DIR_OUTPUT:
            gpdir |= pinMask;
            break;
        case e_IO_PIN_DIR_INPUT:
            /* No action, we already cleared the bit */
            break;
        case e_IO_PIN_DIR_IN_OUT:
            RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("In/Out direction"));
            break;
        default:
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Pin direction"));
    }

    WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpodr, gpodr);
    WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpdir, gpdir);

    if (p_Gpio->ioPortModule.p_Ports[portId].pins[pinId].intrId != NO_INTR)
    {
        /* Disable interrupt on the pin */
        GpioSetPinIntrMode(p_Gpio, ioPin, e_IO_PIN_INTR_DISABLED);
    }

    return E_OK;
}


/*****************************************************************************/
static uint8_t GpioReadPinInput(t_Handle h_Gpio, int ioPin)
{
    t_Gpio      *p_Gpio = (t_Gpio *)h_Gpio;
    uint8_t     portId = IO_PIN_PORT_ID(ioPin);
    uint32_t    pinMask, gpdat;

    ASSERT_COND(p_Gpio);

    /* Calculate pin location */
    pinMask = (uint32_t)(PIN_0_MASK >> IO_PIN_ID(ioPin));

    gpdat = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpdat);

    return (uint8_t)(!!(gpdat & pinMask));
}


/*****************************************************************************/
static t_Error GpioSetPinOutput(t_Handle h_Gpio, int ioPin, uint8_t output)
{
    t_Gpio      *p_Gpio = (t_Gpio *)h_Gpio;
    uint8_t     portId = IO_PIN_PORT_ID(ioPin);
    uint32_t    pinMask, gpdat;

    ASSERT_COND(p_Gpio);

    /* Calculate pin location */
    pinMask = (uint32_t)(PIN_0_MASK >> IO_PIN_ID(ioPin));

    gpdat = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpdat);

    if (output)
        gpdat |= pinMask;
    else
        gpdat &= ~(pinMask);

    WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpdat, gpdat);

    return E_OK;
}


/*****************************************************************************/
static t_Error GpioSetPinIntrMode(t_Handle          h_Gpio,
                                   int              ioPin,
                                   e_IoPinIntrMode  intrMode)
{
    t_Gpio      *p_Gpio = (t_Gpio *)h_Gpio;
    uint8_t     portId = IO_PIN_PORT_ID(ioPin);
    uint8_t     pinId = IO_PIN_ID(ioPin);
    uint32_t    pinMask, gpimr, gpicr;

    ASSERT_COND(p_Gpio);
    ASSERT_COND(p_Gpio->ioPortModule.p_Ports[portId].pins[pinId].intrId != NO_INTR);

    /* Calculate pin location */
    pinMask = (uint32_t)(PIN_0_MASK >> pinId);

    gpimr = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpimr);
    gpicr = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpicr);

    switch (intrMode)
    {
        case e_IO_PIN_INTR_DISABLED:
            WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpimr, (gpimr & ~(pinMask)));
            break;
        case e_IO_PIN_INTR_ANY_CHANGE:
            WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpimr, (gpimr | pinMask));
            WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpicr, (gpicr & ~(pinMask)));
            break;
        case e_IO_PIN_INTR_HIGH_TO_LOW:
            WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpimr, (gpimr | pinMask));
            WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpicr, (gpicr | pinMask));
            break;
        case e_IO_PIN_INTR_LOW_TO_HIGH:
            RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("Interrupt mode"));
        default:
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Interrupt mode"));
    }

    return E_OK;
}


/*****************************************************************************/
static void GpioIntrHandler(t_Handle h_Gpio)
{
    t_Gpio          *p_Gpio = (t_Gpio *)((uint32_t)h_Gpio & 0xFFFFFFFC);
    uint8_t         portId = (uint8_t)((uint32_t)h_Gpio & 0x00000003);
    uint32_t        events, mask;
    uint8_t         pinId = 0, intrId;
    t_IoPinIntrInfo *p_IntrInfo;

    events = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpier);
    mask = GET_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpimr);
    events &= mask;

    /* Clear event bits in gpimr */
    WRITE_UINT32(p_Gpio->p_MemMap->io_regs[portId].gpier, events);

    intrId = (uint8_t)(portId * NUM_OF_PINS_PER_PORT);

    while (events)
    {
        if (events & PIN_0_MASK)
        {
            p_IntrInfo = &(p_Gpio->ioPortModule.p_IntrInfo[intrId]);
            /* Callback must be available for non-masked events */
            ASSERT_COND(p_IntrInfo->f_IntrCallback);
            p_IntrInfo->f_IntrCallback(p_IntrInfo->h_App, portId, pinId);
        }
        events <<= 1;
        intrId++;
        pinId++;
    }
}


