/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          io_port.c

 @Description   Implementation of generic I/O port module.
*//***************************************************************************/
#include "std_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "io_port.h"


/*****************************************************************************/
t_Error IO_PORT_ConfigurePin(t_Handle h_IoPortModule, int ioPin)
{
    t_IoPortModule  *p_IoModule = (t_IoPortModule *)h_IoPortModule;
    int             curPinConfig;
    t_Error         errCode;
    uint8_t         portId = IO_PIN_PORT_ID(ioPin);
    uint8_t         pinId = IO_PIN_ID(ioPin);

    /* Check parameters validity */
    if (portId >= p_IoModule->numOfPorts)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Port ID %d (should be 0-%d)", portId, p_IoModule->numOfPorts - 1));
    }
    if (pinId >= NUM_OF_PINS_PER_PORT)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Pin ID %d (should be 0-%d)", pinId, NUM_OF_PINS_PER_PORT - 1));
    }
    if (!(p_IoModule->p_Ports[portId].validPinsMask & (PIN_0_MASK >> pinId)))
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Pin %d in port %d", pinId, portId));
    }

    /* Check if pin is busy */
    if (p_IoModule->p_Ports[portId].pins[pinId].numOfUsers)
    {
        /* For shared pin, check if new configuration matches the current one */
        curPinConfig = (int)p_IoModule->p_Ports[portId].pins[pinId].configuration;

        if (!(curPinConfig & IO_PIN_SHARED_MASK) ||
            (curPinConfig != (ioPin & IO_PIN_CONFIG_MASK)))
        {
            RETURN_ERROR(MAJOR, E_BUSY, ("Pin %d in port %d", pinId, portId));
        }
    }
    else
    {
        /* Configure the pin in hardware */
        errCode = p_IoModule->f_ConfigurePin(h_IoPortModule, ioPin);
        if (errCode != E_OK)
        {
            RETURN_ERROR(MAJOR, errCode, ("Pin %d in port %d", pinId, portId));
        }

        /* Store pin configuration */
        p_IoModule->p_Ports[portId].pins[pinId].configuration =
            (uint8_t)(ioPin & IO_PIN_CONFIG_MASK);
    }

    /* Increment the number of users */
    p_IoModule->p_Ports[portId].pins[pinId].numOfUsers++;

    return E_OK;
}


/*****************************************************************************/
t_Error IO_PORT_ReleasePin(t_Handle h_IoPortModule, int ioPin)
{
    t_IoPortModule  *p_IoModule = (t_IoPortModule *)h_IoPortModule;
    uint8_t         portId = IO_PIN_PORT_ID(ioPin);
    uint8_t         pinId = IO_PIN_ID(ioPin);
    uint8_t         intrId;

    /* Check parameters validity */
    if (portId >= p_IoModule->numOfPorts)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Port ID %d (should be 0-%d)", portId, p_IoModule->numOfPorts - 1));
    }
    if (pinId >= NUM_OF_PINS_PER_PORT)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Pin ID %d (should be 0-%d)", pinId, NUM_OF_PINS_PER_PORT - 1));
    }
    if (!(p_IoModule->p_Ports[portId].validPinsMask & (PIN_0_MASK >> pinId)))
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Pin %d in port %d", pinId, portId));
    }

    /* Check if pin is indeed busy */
    if (p_IoModule->p_Ports[portId].pins[pinId].numOfUsers == 0)
    {
        RETURN_ERROR(MAJOR, E_ALREADY_FREE, ("Pin %d in port %d", pinId, portId));
    }

    if (p_IoModule->p_Ports[portId].pins[pinId].configuration != (ioPin & IO_PIN_CONFIG_MASK))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d was not configured for released function", pinId, portId));
    }

    /* Decrement the number of users */
    p_IoModule->p_Ports[portId].pins[pinId].numOfUsers--;

    if ((p_IoModule->p_Ports[portId].pins[pinId].numOfUsers == 0) &&
        (p_IoModule->p_Ports[portId].validIntrMask & (PIN_0_MASK >> pinId)))
    {
        /* Disable I/O pin interrupt */
        p_IoModule->f_SetPinIntrMode(h_IoPortModule, ioPin, e_IO_PIN_INTR_DISABLED);

        intrId = p_IoModule->p_Ports[portId].pins[pinId].intrId;

        p_IoModule->p_IntrInfo[intrId].f_IntrCallback = NULL;
        p_IoModule->p_IntrInfo[intrId].h_App = NULL;
    }

    return E_OK;
}


/*****************************************************************************/
uint8_t IO_PORT_ReadPinInput(t_Handle h_IoPortModule, int ioPin)
{
    t_IoPortModule      *p_IoModule = (t_IoPortModule *)h_IoPortModule;
    e_IoPinDirection    pinDir;
    uint8_t             portId = IO_PIN_PORT_ID(ioPin);
    uint8_t             pinId = IO_PIN_ID(ioPin);

    /* Check parameters validity */
    if (portId >= p_IoModule->numOfPorts)
    {
        REPORT_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Port ID %d (should be 0-%d)", portId, p_IoModule->numOfPorts - 1));
        return 0;
    }
    if (pinId >= NUM_OF_PINS_PER_PORT)
    {
        REPORT_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Pin ID %d (should be 0-%d)", pinId, NUM_OF_PINS_PER_PORT - 1));
        return 0;
    }
    if (!(p_IoModule->p_Ports[portId].validPinsMask & (PIN_0_MASK >> pinId)))
    {
        REPORT_ERROR(MAJOR, E_NOT_AVAILABLE, ("Pin %d in port %d", pinId, portId));
        return 0;
    }

    if ((p_IoModule->p_Ports[portId].pins[pinId].numOfUsers == 0) ||
        (p_IoModule->p_Ports[portId].pins[pinId].configuration != (ioPin & IO_PIN_CONFIG_MASK)))
    {
        REPORT_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d was not configured for requested function", pinId, portId));
        return 0;
    }

    pinDir = IO_PIN_DIRECTION(ioPin);

    /* Check if pin is configured as input pin */
    if ((pinDir != e_IO_PIN_DIR_INPUT) && (pinDir != e_IO_PIN_DIR_IN_OUT))
    {
        REPORT_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d is not an input pin", pinId, portId));
        return 0;
    }

    return p_IoModule->f_ReadPinInput(h_IoPortModule, ioPin);
}


/*****************************************************************************/
t_Error IO_PORT_SetPinOutput(t_Handle h_IoPortModule, int ioPin, uint8_t output)
{
    t_IoPortModule      *p_IoModule = (t_IoPortModule *)h_IoPortModule;
    e_IoPinDirection    pinDir;
    uint8_t             portId = IO_PIN_PORT_ID(ioPin);
    uint8_t             pinId = IO_PIN_ID(ioPin);

    /* Check parameters validity */
    if (portId >= p_IoModule->numOfPorts)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Port ID %d (should be 0-%d)", portId, p_IoModule->numOfPorts - 1));
    }
    if (pinId >= NUM_OF_PINS_PER_PORT)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Pin ID %d (should be 0-%d)", pinId, NUM_OF_PINS_PER_PORT - 1));
    }
    if (!(p_IoModule->p_Ports[portId].validPinsMask & (PIN_0_MASK >> pinId)))
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Pin %d in port %d", pinId, portId));
    }

    if ((p_IoModule->p_Ports[portId].pins[pinId].numOfUsers == 0) ||
        (p_IoModule->p_Ports[portId].pins[pinId].configuration != (ioPin & IO_PIN_CONFIG_MASK)))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d was not configured for requested function", pinId, portId));
    }

    pinDir = IO_PIN_DIRECTION(p_IoModule->p_Ports[portId].pins[pinId].configuration);

    /* Check if pin is configured as output pin */
    if (pinDir != e_IO_PIN_DIR_OUTPUT)
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d is not an output pin", pinId, portId));
    }

    if (output > 1)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Output data (should be 0 or 1)"));
    }

    return p_IoModule->f_SetPinOutput(h_IoPortModule, ioPin, output);
}


/*****************************************************************************/
t_Error IO_PORT_SetIntrCallback(t_Handle            h_IoPortModule,
                                int                 ioPin,
                                e_IoPinIntrMode     intrMode,
                                t_IoPinIntrCallback *f_IntrCallback,
                                t_Handle            h_App)
{
    t_IoPortModule      *p_IoModule = (t_IoPortModule *)h_IoPortModule;
    e_IoPinDirection    pinDir;
    t_Error             errCode;
    uint8_t             intrId;
    uint8_t             portId = IO_PIN_PORT_ID(ioPin);
    uint8_t             pinId = IO_PIN_ID(ioPin);

    /* Check parameters validity */
    if (portId >= p_IoModule->numOfPorts)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Port ID %d (should be 0-%d)", portId, p_IoModule->numOfPorts - 1));
    }
    if (pinId >= NUM_OF_PINS_PER_PORT)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Pin ID %d (should be 0-%d)", pinId, NUM_OF_PINS_PER_PORT - 1));
    }
    if (!(p_IoModule->p_Ports[portId].validPinsMask & (PIN_0_MASK >> pinId)))
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Pin %d in port %d", pinId, portId));
    }
    if (!(p_IoModule->p_Ports[portId].validIntrMask & (PIN_0_MASK >> pinId)))
    {
        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("Interrupt on Pin %d in port %d", pinId, portId));
    }

    if ((p_IoModule->p_Ports[portId].pins[pinId].numOfUsers == 0) ||
        (p_IoModule->p_Ports[portId].pins[pinId].configuration != (ioPin & IO_PIN_CONFIG_MASK)))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d was not configured for requested function", pinId, portId));
    }

    pinDir = IO_PIN_DIRECTION(ioPin);

    /* Check if pin is configured as input pin */
    if ((pinDir != e_IO_PIN_DIR_INPUT) && (pinDir != e_IO_PIN_DIR_IN_OUT))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("Pin %d in port %d is not an input pin", pinId, portId));
    }

    /* This field must be initialized specifically by each I/O module */
    intrId = p_IoModule->p_Ports[portId].pins[pinId].intrId;

    /* Set interrupt mode in hardware */
    errCode = p_IoModule->f_SetPinIntrMode(h_IoPortModule, ioPin, intrMode);
    if (errCode != E_OK)
    {
        RETURN_ERROR(MAJOR, errCode, ("Pin %d in port %d", pinId, portId));
    }

    /* Update internal variables */
    if (intrMode == e_IO_PIN_INTR_DISABLED)
    {
        p_IoModule->p_IntrInfo[intrId].f_IntrCallback = NULL;
        p_IoModule->p_IntrInfo[intrId].h_App = NULL;
    }
    else
    {
        p_IoModule->p_IntrInfo[intrId].f_IntrCallback = f_IntrCallback;
        p_IoModule->p_IntrInfo[intrId].h_App = h_App;
    }

    return E_OK;
}


