/******************************************************************************

 © 1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          qm.c

 @Description   QM & Portal implementation
*//***************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "string_ext.h"
#include "sprint_ext.h"
#include "mm_ext.h"
#include "core_ext.h"
#include "debug_ext.h"

#include "qm.h"


static volatile bool blockingFlag = FALSE;
static void QmIpcMsgCompletionCB(t_Handle   h_Module,
                                 uint8_t    *p_Msg,
                                 uint8_t    *p_Reply,
                                 uint32_t   replyLength,
                                 t_Error    status)
{
    SANITY_CHECK_RETURN(h_Module, E_INVALID_HANDLE);
	UNUSED(h_Module);

    UNUSED(p_Msg);UNUSED(p_Reply);UNUSED(replyLength);UNUSED(status);
    blockingFlag = FALSE;
}

static t_Error QmHandleIpcMsgCB(t_Handle  h_Qm,
                                uint8_t   *p_Msg,
                                uint32_t  msgLength,
                                uint8_t   *p_Reply,
                                uint32_t  *p_ReplyLength)
{
    t_Qm                    *p_Qm       = (t_Qm*)h_Qm;
    t_QmIpcMsg              *p_IpcMsg   = (t_QmIpcMsg*)p_Msg;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((msgLength >= sizeof(uint32_t)), E_INVALID_VALUE);

    UNUSED(msgLength);

    switch(p_IpcMsg->msgId)
    {
        case (QM_MASTER_IS_ALIVE):
            ((uint8_t*)p_Reply)[0] = 1;
            *p_ReplyLength = sizeof(uint8_t);
            return E_OK;
        case (QM_FORCE_FQID):
        {
            t_QmIpcFqidParams *p_IpcFqid = (t_QmIpcFqidParams *)(p_IpcMsg->msgBody);

            *p_ReplyLength = 0;
            return QmFqidGet(p_Qm, 1, 1, TRUE, p_IpcFqid->fqid);
        }
        case (QM_PUT_FQID):
        {
            t_QmIpcFqidParams *p_IpcFqid = (t_QmIpcFqidParams *)(p_IpcMsg->msgBody);

            *p_ReplyLength = 0;
            return QmFqidPut(p_Qm, p_IpcFqid->fqid);
        }
        case (QM_GET_COUNTER):
        {
            t_QmIpcGetCounter *p_IpcCounter = (t_QmIpcGetCounter *)(p_IpcMsg->msgBody);
            *p_ReplyLength = sizeof(uint32_t);
            *(uint32_t *)p_Reply =
                QmGetCounter(p_Qm, (e_QmInterModuleCounters)p_IpcCounter->enumId);
            return E_OK;
        }
        case (QM_GET_SET_PORTAL_PARAMS):
        {
            *p_ReplyLength = 0;
            return QmGetSetPortalParams(p_Qm, (t_QmInterModulePortalInitParams *)(p_IpcMsg->msgBody));
        }
        case (QM_GET_REVISION):
        {
            *p_ReplyLength = sizeof(t_QmRevisionInfo);
            return QmGetRevision(p_Qm, (t_QmRevisionInfo *)p_Reply);
        }
        default:
            *p_ReplyLength = 0;
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("command not found!!!"));
    }
    return E_OK;
}

static t_Error QmInitPfdr(t_Qm *p_Qm, uint32_t pfdr_start, uint32_t num)
{
    uint8_t rslt;

    ASSERT_COND(p_Qm);

    rslt = MCR_get_rslt(GET_UINT32(p_Qm->p_QmRegs->mcr));

    ASSERT_COND(pfdr_start && !(pfdr_start & 7) && !(num & 7) && num);
    /* Make sure te command interface is 'idle' */
    if(!MCR_rslt_idle(rslt))
        RETURN_ERROR(CRITICAL,E_INVALID_STATE,("QMAN_MCR isn't idle"));

    /* Write the MCR command params then the verb */
    WRITE_UINT32(p_Qm->p_QmRegs->mcp0, pfdr_start);
    /* TODO: remove this - it's a workaround for a model bug that is
     * corrected in more recent versions. We use the workaround until
     * everyone has upgraded. */
    WRITE_UINT32(p_Qm->p_QmRegs->mcp1, (pfdr_start + num - 16));
    WRITE_UINT32(p_Qm->p_QmRegs->mcp1, (pfdr_start + num - 1));

    lwsync();
    WRITE_UINT32(p_Qm->p_QmRegs->mcr, MCR_INIT_PFDR);

    /* Poll for the result */
    do {
        rslt = MCR_get_rslt(GET_UINT32(p_Qm->p_QmRegs->mcr));
    } while(!MCR_rslt_idle(rslt));
    if (MCR_rslt_ok(rslt))
        return 0;
    if (MCR_rslt_eaccess(rslt))
        return ERROR_CODE(E_INVALID_STATE);
    if (MCR_rslt_inval(rslt))
        return ERROR_CODE(E_INVALID_VALUE);
    XX_Print("Unexpected result from MCR_INIT_PFDR: %02x\n", rslt);
    return ERROR_CODE(E_INVALID_STATE);
}

static __inline__ void QmSetWqScheduling(t_Qm *p_Qm,
                                         e_QmWqClass wqClass,
                                         uint8_t csElev,
                                         uint8_t csw2,
                                         uint8_t csw3,
                                         uint8_t csw4,
                                         uint8_t csw5,
                                         uint8_t csw6,
                                         uint8_t csw7)
{
    ASSERT_COND(p_Qm);

    WRITE_UINT32(p_Qm->p_QmRegs->wq_cs_cfg[wqClass],
                 (uint32_t)(((csElev & 0xff) << 24) |
                 ((csw2 & 0x7) << 20) |
                 ((csw3 & 0x7) << 16) |
                 ((csw4 & 0x7) << 12) |
                 ((csw5 & 0x7) << 8) |
                 ((csw6 & 0x7) << 4) |
                 (csw7 & 0x7)));
}


/****************************************/
/*       Inter-Module functions         */
/****************************************/

uint32_t QmGetCounter(t_Handle h_Qm, e_QmInterModuleCounters counter)
{
    t_Qm *p_Qm = (t_Qm*)h_Qm;

    SANITY_CHECK_RETURN_VALUE(p_Qm, E_INVALID_HANDLE, 0);

    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        switch(counter)
        {
            case(e_QM_IM_COUNTERS_SFDR_IN_USE):
                return GET_UINT32(p_Qm->p_QmRegs->sfdr_in_use);
            case(e_QM_IM_COUNTERS_PFDR_IN_USE):
                return (p_Qm->numOfPfdr - GET_UINT32(p_Qm->p_QmRegs->pfdr_fpc));
            case(e_QM_IM_COUNTERS_PFDR_FREE_POOL):
                return GET_UINT32(p_Qm->p_QmRegs->pfdr_fpc);
            default:
                break;
        }
        /* should never get here */
        ASSERT_COND(FALSE);
    }
    else if (p_Qm->h_Session)
    {
        t_QmIpcMsg              msg;
        t_QmIpcGetCounter       ipcCounter;
        uint32_t                replyLength = sizeof(t_QmIpcGetCounter);
        uint32_t                reply=0;
        t_Error                 errCode;

        memset(&msg, 0, sizeof(t_QmIpcMsg));
        ipcCounter.enumId       = (uint32_t)counter;
        msg.msgId               = QM_GET_COUNTER;
        memcpy(msg.msgBody, &ipcCounter, sizeof(t_QmIpcGetCounter));
        if ((errCode = XX_IpcSendMessage(p_Qm->h_Session,
                                         (uint8_t*)&msg,
                                         sizeof(msg),
                                         (uint8_t*)&reply,
                                         &replyLength,
                                         NULL,
                                         NULL)) != E_OK)
            REPORT_ERROR(MAJOR, errCode, NO_MSG);
        return reply;
    }
    else
        REPORT_ERROR(WARNING, E_NOT_SUPPORTED, ("IPC"));

    return 0;
}

t_Error QmGetRevision(t_Handle h_Qm, t_QmRevisionInfo *p_QmRevisionInfo)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;
    uint32_t    tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmRevisionInfo, E_NULL_POINTER);

    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        /* read revision register 1 */
        tmpReg = GET_UINT32(p_Qm->p_QmRegs->ip_rev_1);
        p_QmRevisionInfo->majorRev = (uint8_t)((tmpReg & REV1_MAJOR_MASK) >> REV1_MAJOR_SHIFT);
        p_QmRevisionInfo->minorRev = (uint8_t)((tmpReg & REV1_MINOR_MASK) >> REV1_MINOR_SHIFT);
    }
    else if (p_Qm->h_Session)
    {
        t_QmIpcMsg              msg;
        uint32_t                replyLength = sizeof(t_QmRevisionInfo);
        t_Error                 errCode = E_OK;

        memset(&msg, 0, sizeof(t_QmIpcMsg));
        msg.msgId           = QM_GET_REVISION;
        if ((errCode = XX_IpcSendMessage(p_Qm->h_Session,
                                         (uint8_t*)&msg,
                                         sizeof(msg),
                                         (uint8_t*)p_QmRevisionInfo,
                                         &replyLength,
                                         NULL,
                                         NULL)) != E_OK)
            RETURN_ERROR(MAJOR, errCode, NO_MSG);
    }
    else
        RETURN_ERROR(WARNING, E_NOT_SUPPORTED, ("IPC"));

    return E_OK;
}

t_Error QmGetSetPortalParams(t_Handle h_Qm, t_QmInterModulePortalInitParams *p_PortalParams)
{
    t_Qm                *p_Qm = (t_Qm *)h_Qm;
    t_QmRevisionInfo    revInfo;
    uint32_t            lioReg,ioReg;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_PortalParams, E_NULL_POINTER);

    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        QmGetRevision(p_Qm, &revInfo);

        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
        {
            lioReg  = (uint32_t)(p_PortalParams->stashDestQueue << 24) |
                      (p_PortalParams->liodn << 16) |
                      (p_PortalParams->dqrrLiodn);
            ioReg   = (p_PortalParams->fdFqLiodn);
        }
        else
        {
            lioReg  = (uint32_t)(p_PortalParams->liodn << 16) |
                      (p_PortalParams->dqrrLiodn);
            ioReg   = (uint32_t)(p_PortalParams->stashDestQueue << 16) |
                      (p_PortalParams->fdFqLiodn);
        }

        WRITE_UINT32(p_Qm->p_QmRegs->swpConfRegs[p_PortalParams->portalId].lio_cfg, lioReg);
        WRITE_UINT32(p_Qm->p_QmRegs->swpConfRegs[p_PortalParams->portalId].io_cfg, ioReg);
    }
    else if (p_Qm->h_Session)
    {
        t_QmIpcMsg                  msg;
        t_QmIpcPortalInitParams     portalParams;
        t_Error                     errCode;

        memset(&msg, 0, sizeof(t_QmIpcMsg));
        portalParams.portalId       = p_PortalParams->portalId;
        portalParams.stashDestQueue = p_PortalParams->stashDestQueue;
        portalParams.liodn          = p_PortalParams->liodn;
        portalParams.dqrrLiodn      = p_PortalParams->dqrrLiodn;
        portalParams.fdFqLiodn      = p_PortalParams->fdFqLiodn;
        msg.msgId           = QM_GET_SET_PORTAL_PARAMS;
        memcpy(msg.msgBody, &portalParams, sizeof(t_QmIpcPortalInitParams));
        XX_LockSpinlock(p_Qm->lock);
        if ((errCode = XX_IpcSendMessage(p_Qm->h_Session,
                                         (uint8_t*)&msg,
                                         sizeof(msg),
                                         NULL,
                                         NULL,
                                         NULL,
                                         NULL)) != E_OK)
        {
            XX_UnlockSpinlock(p_Qm->lock);
            RETURN_ERROR(MAJOR, errCode, NO_MSG);
        }
        XX_UnlockSpinlock(p_Qm->lock);
    }
    else
        DBG(WARNING, ("Can't set portal parameters (e.g. liodns). " \
                      "probably QM is running in guest-mode with no IPC!"));

    return E_OK;
}


/****************************************/
/*       API Init unit functions        */
/****************************************/

t_Handle QM_Config(t_QmParam *p_QmParam)
{
    t_Qm    *p_Qm;

    SANITY_CHECK_RETURN_VALUE(p_QmParam, E_INVALID_HANDLE, NULL);

    p_Qm = (t_Qm *)XX_Malloc(sizeof(t_Qm));
    if (!p_Qm)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("BM obj!!!"));
        return NULL;
    }
    memset(p_Qm, 0, sizeof(t_Qm));
    p_Qm->p_QmDriverParams = (t_QmDriverParams *)XX_Malloc(sizeof(t_QmDriverParams));
    if (!p_Qm->p_QmDriverParams)
    {
        XX_Free(p_Qm);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Qm driver parameters"));
        return NULL;
    }
    memset(p_Qm->p_QmDriverParams, 0, sizeof(t_QmDriverParams));

    p_Qm->guestId                           = p_QmParam->guestId;
    p_Qm->p_QmDriverParams->partFqidBase    = p_QmParam->partFqidBase;
    p_Qm->p_QmDriverParams->partNumOfFqids  = p_QmParam->partNumOfFqids;
    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        p_Qm->p_QmRegs          = CAST_UINT64_TO_POINTER(p_QmParam->baseAddress);
        p_Qm->exceptions        = DEFAULT_exceptions;
        p_Qm->f_Exception       = p_QmParam->f_Exception;
        p_Qm->h_App             = p_QmParam->h_App;
        p_Qm->errIrq            = p_QmParam->errIrq;
        p_Qm->p_QmDriverParams->liodn                   = p_QmParam->liodn;
        p_Qm->p_QmDriverParams->rtFramesDepth           = DEFAULT_rtFramesDepth;
        p_Qm->p_QmDriverParams->fqdMemPartitionId       = p_QmParam->fqdMemPartitionId;
        p_Qm->p_QmDriverParams->pfdrMemPartitionId      = p_QmParam->pfdrMemPartitionId;
        p_Qm->p_QmDriverParams->swPortalsBaseAddress    = p_QmParam->swPortalsBaseAddress;
        p_Qm->p_QmDriverParams->totalNumOfFqids         = p_QmParam->totalNumOfFqids;
        p_Qm->p_QmDriverParams->pfdrThreshold           = DEFAULT_pfdrThreshold;
        p_Qm->p_QmDriverParams->sfdrThreshold           = DEFAULT_sfdrThreshold;
        p_Qm->p_QmDriverParams->pfdrBaseConstant        = DEFAULT_pfdrBaseConstant;

#ifdef QMAN_SFDR_LEAK_ERRATA_QMAN5
        {
#define WORKAROUND_TMP_VAL  0x00000003
        t_QmRevisionInfo revInfo;
        QmGetRevision(p_Qm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
        {
            uint32_t *tmp = CAST_UINT64_TO_POINTER(p_QmParam->baseAddress + 0xbf0);
            uint32_t tmpReg = WORKAROUND_TMP_VAL;
            WRITE_UINT32(*tmp, tmpReg);
            while ((tmpReg = GET_UINT32(*tmp)) != WORKAROUND_TMP_VAL) ;
        }
        }
#endif /* QMAN_SFDR_LEAK_ERRATA_QMAN5 */
    }

    /* build the QM partition IPC address */
    memset(p_Qm->moduleName, 0, MODULE_NAME_SIZE);
    if(Sprint (p_Qm->moduleName, "QM_0_%d",p_Qm->guestId) != (p_Qm->guestId<10 ? 6:7))
    {
        XX_Free(p_Qm->p_QmDriverParams);
        XX_Free(p_Qm);
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));
        return NULL;
    }

    return p_Qm;
}

t_Error QM_Init(t_Handle h_Qm)
{
    t_Qm                *p_Qm = (t_Qm *)h_Qm;
    t_QmDriverParams    *p_QmDriverParams;
    t_Error             err;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    p_QmDriverParams        = p_Qm->p_QmDriverParams;

    if (p_QmDriverParams->partNumOfFqids)
    {
        if (MM_Init(&p_Qm->h_FqidMm, p_QmDriverParams->partFqidBase, p_QmDriverParams->partNumOfFqids) != E_OK)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("QM-FQIDS-MEM partition!!!"));
        }
        if (MM_Init(&p_Qm->h_RsrvFqidMm, p_QmDriverParams->partFqidBase, p_QmDriverParams->partNumOfFqids) != E_OK)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("QM-Reserve-FQIDS-MEM partition!!!"));
        }
    }
    if (p_QmDriverParams->partNumOfCgrs)
        if (MM_Init(&p_Qm->h_CgrsMm, p_QmDriverParams->partCgridBase, p_QmDriverParams->partNumOfCgrs) != E_OK)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("QM-CGRS-MEM partition!!!"));
        }

    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        uint64_t            phyAddr;
        t_QmRevisionInfo    revInfo;
        uint32_t            dsSize, exp;

        QmGetRevision(p_Qm, &revInfo);
        DBG(TRACE, ("Qman ver:%02x,%02x", revInfo.majorRev, revInfo.minorRev));
        XX_Print("%s: Qman ver:%02x,%02x", __FUNCTION__, revInfo.majorRev, revInfo.minorRev);

        phyAddr = XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_QmDriverParams->swPortalsBaseAddress));
        if(phyAddr & 0x001fffff)
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("sw portals base address isn't properly aligned"));
        WRITE_UINT32(p_Qm->p_QmRegs->qcsp_bare, (uint16_t)((phyAddr >> 32) & 0x00ff));
        WRITE_UINT32(p_Qm->p_QmRegs->qcsp_bar, (uint32_t)phyAddr);

        WRITE_UINT32(p_Qm->p_QmRegs->liodnr, (uint16_t)p_QmDriverParams->liodn);

        /* FQD memory */
        dsSize = (uint32_t)(((p_QmDriverParams->totalNumOfFqids * 12)/10) * FQD_ENTRY_SIZE);
        LOG2(dsSize, exp);
        if (!POWER_OF_2(dsSize)) (exp++);
        dsSize = (uint32_t)(1 << exp);

        if (!((dsSize >= 4*KILOBYTE) && (dsSize <= 1*GIGABYTE)))
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("FQD size %d (%d Fqids) not in range (4K - 1G)!!!",dsSize, p_QmDriverParams->totalNumOfFqids));
        }

        p_Qm->p_FqdBase = XX_MallocSmart(dsSize, (int)p_QmDriverParams->fqdMemPartitionId, dsSize);
        if (!p_Qm->p_FqdBase)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("FQD obj!!!"));
        }
        memset(p_Qm->p_FqdBase, 0, dsSize);
        phyAddr = XX_VirtToPhys(p_Qm->p_FqdBase);
		XX_Print("%s: FQD dsSize %d (%d entries) physAddr %llx", __FUNCTION__, dsSize, dsSize/FQD_ENTRY_SIZE, phyAddr);
        WRITE_UINT32(p_Qm->p_QmRegs->fqd_bare, ((uint32_t)(phyAddr >> 32) & 0x00ff));
        WRITE_UINT32(p_Qm->p_QmRegs->fqd_bar, (uint32_t)phyAddr);
        WRITE_UINT32(p_Qm->p_QmRegs->fqd_ar, AR_ENABLE | (exp - 1));

        /* PFDR memory */
        dsSize = (uint32_t)(((p_QmDriverParams->rtFramesDepth * 12)/30) * PFDR_ENTRY_SIZE);
        LOG2(dsSize, exp);
        if (!POWER_OF_2(dsSize)) (exp++);
        dsSize = (uint32_t)(1 << exp);

        if (!((dsSize >= 4*KILOBYTE) && (dsSize <= 1*GIGABYTE)))
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("PFDR size not in range (4K - 1G)!!!"));
        }

        p_Qm->p_PfdrBase = XX_MallocSmart(dsSize, (int)p_QmDriverParams->pfdrMemPartitionId, dsSize);
        if (!p_Qm->p_PfdrBase)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("PFDR obj!!!"));
        }
        phyAddr = XX_VirtToPhys(p_Qm->p_PfdrBase);
		XX_Print("%s: PFDR dsSize %d (%d entries) physAddr %llx", __FUNCTION__, dsSize, dsSize/PFDR_ENTRY_SIZE, phyAddr);
        WRITE_UINT32(p_Qm->p_QmRegs->pfdr_bare, ((uint32_t)(phyAddr >> 32) & 0x00ff));
        WRITE_UINT32(p_Qm->p_QmRegs->pfdr_bar, (uint32_t)phyAddr);
        WRITE_UINT32(p_Qm->p_QmRegs->pfdr_ar, AR_ENABLE | (exp - 1));

        QmInitPfdr(p_Qm, 8, (dsSize / PFDR_ENTRY_SIZE) - 8);
        /* thresholds */
        WRITE_UINT32(p_Qm->p_QmRegs->pfdr_fp_lwit, (p_Qm->p_QmDriverParams->pfdrThreshold & 0xffffff));
        WRITE_UINT32(p_Qm->p_QmRegs->pfdr_cfg, p_Qm->p_QmDriverParams->pfdrBaseConstant);
        WRITE_UINT32(p_Qm->p_QmRegs->sfdr_cfg, (p_Qm->p_QmDriverParams->sfdrThreshold & 0x3ff));

        p_Qm->numOfPfdr = QmGetCounter(p_Qm, e_QM_IM_COUNTERS_PFDR_FREE_POOL);
		XX_Print("%s: num of PFDR %d", __FUNCTION__, p_Qm->numOfPfdr);

        /* corenet initiator settings */
        WRITE_UINT32(p_Qm->p_QmRegs->ci_sched_cfg,
                     (CI_SCHED_CFG_EN |
                     (DEFAULT_initiatorSrcciv << CI_SCHED_CFG_SRCCIV_SHIFT) |
                     (DEFAULT_initiatorSrqW << CI_SCHED_CFG_SRQ_W_SHIFT) |
                     (DEFAULT_initiatorRwW << CI_SCHED_CFG_RW_W_SHIFT) |
                     (DEFAULT_initiatorBmanW << CI_SCHED_CFG_BMAN_W_SHIFT)));

        /* HID settings */
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            //offset 0x0bf0
            WRITE_UINT32(p_Qm->p_QmRegs->res23[144], 0x3);
        else
            WRITE_UINT32(p_Qm->p_QmRegs->res23[144], 0x0);

#ifdef QMAN_WQ_CS_CFG_ERRATA_QMAN4
        {
            t_QmRevisionInfo revInfo;
            QmGetRevision(p_Qm, &revInfo);
            if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            {
                QmSetWqScheduling(p_Qm, e_QM_WQ_SW_PORTALS,0,1,1,1,1,1,1);
                QmSetWqScheduling(p_Qm, e_QM_WQ_POOLS,0,1,1,1,1,1,1);
                QmSetWqScheduling(p_Qm, e_QM_WQ_DCP0,0,1,1,1,1,1,1);
                QmSetWqScheduling(p_Qm, e_QM_WQ_DCP1,0,1,1,1,1,1,1);
                QmSetWqScheduling(p_Qm, e_QM_WQ_DCP2,0,1,1,1,1,1,1);
                QmSetWqScheduling(p_Qm, e_QM_WQ_DCP3,0,1,1,1,1,1,1);
            }
        }
#endif /* QMAN_WQ_CS_CFG_ERRATA_QMAN4 */

        WRITE_UINT32(p_Qm->p_QmRegs->err_isr, p_Qm->exceptions);
        WRITE_UINT32(p_Qm->p_QmRegs->err_ier, p_Qm->exceptions);
        WRITE_UINT32(p_Qm->p_QmRegs->err_isdr, 0x0);
        if (p_Qm->errIrq != NO_IRQ)
        {
            XX_SetIntr(p_Qm->errIrq, QM_ErrorIsr, p_Qm);
            XX_EnableIntr(p_Qm->errIrq);
        }
        if ((err = XX_IpcRegisterMsgHandler(p_Qm->moduleName, QmHandleIpcMsgCB, p_Qm, QM_IPC_MAX_REPLY_SIZE)) != E_OK)
        {
            QM_Free(p_Qm);
            RETURN_ERROR(MAJOR, err, NO_MSG);
        }
    }
    else // guest mode
    {
        char                    masterModuleName[MODULE_NAME_SIZE];

        memset(masterModuleName, 0, MODULE_NAME_SIZE);
        if(Sprint (masterModuleName, "QM_0_%d", NCSW_MASTER_ID) != (NCSW_MASTER_ID<10 ? 6:7))
        {
            XX_Free(p_Qm);
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));
        }

        p_Qm->h_Session     = XX_IpcInitSession(masterModuleName, p_Qm->moduleName);
        if (p_Qm->h_Session)
        {
            t_QmIpcMsg              msg;
            uint8_t                 isMasterAlive = 0;
            uint32_t                replyLength;

            memset(&msg, 0, sizeof(t_QmIpcMsg));
            msg.msgId           = QM_MASTER_IS_ALIVE;

            do
            {
                blockingFlag = TRUE;
                replyLength = sizeof(isMasterAlive);
                if ((err = XX_IpcSendMessage(p_Qm->h_Session,
                                             (uint8_t*)&msg,
                                             sizeof(msg.msgId),
                                             (uint8_t*)&isMasterAlive,
                                             &replyLength,
                                             QmIpcMsgCompletionCB,
                                             p_Qm)) != E_OK)
                    REPORT_ERROR(MAJOR, err, NO_MSG);
                while(blockingFlag) ;
            } while (!isMasterAlive);
        }
    }

p_Qm->lock = XX_InitSpinlock();
    XX_Free(p_Qm->p_QmDriverParams);
    p_Qm->p_QmDriverParams = NULL;

    return E_OK;
}

void QM_Free(t_Handle h_Qm)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;

    if (!p_Qm)
        return;

    if (p_Qm->guestId == NCSW_MASTER_ID)
    {
        XX_IpcUnregisterMsgHandler(p_Qm->moduleName);
        if (p_Qm->errIrq  != NO_IRQ)
        {
            XX_DisableIntr(p_Qm->errIrq);
            XX_FreeIntr(p_Qm->errIrq);
        }
        if (p_Qm->p_FqdBase)
            XX_FreeSmart(p_Qm->p_FqdBase);
        if (p_Qm->p_PfdrBase)
            XX_FreeSmart(p_Qm->p_PfdrBase);
    }
    if (p_Qm->h_Session)
        XX_IpcFreeSession(p_Qm->h_Session);
    if (p_Qm->h_RsrvFqidMm)
        MM_Free(p_Qm->h_RsrvFqidMm);
    if (p_Qm->h_FqidMm)
        MM_Free(p_Qm->h_FqidMm);
    if (p_Qm->h_CgrsMm)
        MM_Free(p_Qm->h_CgrsMm);
    if (p_Qm->p_QmDriverParams)
        XX_Free(p_Qm->p_QmDriverParams);
    XX_Free(p_Qm);
}

t_Error QM_ConfigRTFramesDepth(t_Handle h_Qm, uint32_t rtFramesDepth)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    p_Qm->p_QmDriverParams->rtFramesDepth = rtFramesDepth;

    return E_OK;
}

t_Error QM_ConfigPfdrThreshold(t_Handle h_Qm, uint32_t threshold)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    p_Qm->p_QmDriverParams->pfdrThreshold = threshold;

    return E_OK;
}

t_Error QM_ConfigSfdrReservationThreshold(t_Handle h_Qm, uint32_t threshold)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    p_Qm->p_QmDriverParams->sfdrThreshold = threshold;

    return E_OK;
}


t_Error QM_Poll(t_Handle h_Qm, e_QmPortalPollSource source)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;
    t_QmPortal  *p_QmPortal;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    p_QmPortal = QmGetPortalHandle(p_Qm);
    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);

    return QM_PORTAL_Poll(p_QmPortal, source);
}

uint32_t QM_GetCounter(t_Handle h_Qm, e_QmCounters counter)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;

    SANITY_CHECK_RETURN_VALUE(p_Qm, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(!p_Qm->p_QmDriverParams, E_INVALID_STATE, 0);
	UNUSED(p_Qm);

    switch(counter)
    {
        case(e_QM_COUNTERS_SFDR_IN_USE):
            return QmGetCounter(h_Qm, e_QM_IM_COUNTERS_SFDR_IN_USE);
        case(e_QM_COUNTERS_PFDR_IN_USE):
            return QmGetCounter(h_Qm, e_QM_IM_COUNTERS_PFDR_IN_USE);
        case(e_QM_COUNTERS_PFDR_FREE_POOL):
            return QmGetCounter(h_Qm, e_QM_IM_COUNTERS_PFDR_FREE_POOL);
        default:
            break;
    }
    /* should never get here */
    ASSERT_COND(FALSE);

    return 0;
}

void QM_ErrorIsr(t_Handle h_Qm)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;
    uint32_t    tmpReg;

    SANITY_CHECK_RETURN(p_Qm, E_INVALID_HANDLE);

    if (p_Qm->guestId != NCSW_MASTER_ID)
    {
        REPORT_ERROR(WARNING, E_INVALID_OPERATION, ("Master Only"));
        return;
    }

    tmpReg = GET_UINT32(p_Qm->p_QmRegs->err_isr);
    tmpReg &= GET_UINT32(p_Qm->p_QmRegs->err_ier);
    WRITE_UINT32(p_Qm->p_QmRegs->err_isr, tmpReg);

    if (tmpReg & QM_EX_CORENET_INITIATOR_DATA)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_CORENET_INITIATOR_DATA);
    if (tmpReg & QM_EX_CORENET_TARGET_DATA)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_CORENET_TARGET_DATA);
    if (tmpReg & QM_EX_CORENET_INVALID_TARGET_TRANSACTION)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_CORENET_INVALID_TARGET_TRANSACTION);
    if (tmpReg & QM_EX_PFDR_THRESHOLD)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_PFDR_THRESHOLD);
    if (tmpReg & QM_EX_MULTI_ECC)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_MULTI_ECC);
    if (tmpReg & QM_EX_SINGLE_ECC)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_SINGLE_ECC);
    if (tmpReg & QM_EX_PFDR_ENQUEUE_BLOCKED)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_PFDR_ENQUEUE_BLOCKED);
    if (tmpReg & QM_EX_INVALID_COMMAND)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_INVALID_COMMAND);
    if (tmpReg & QM_EX_DEQUEUE_DCP)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_DEQUEUE_DCP);
    if (tmpReg & QM_EX_DEQUEUE_FQ)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_DEQUEUE_FQ);
    if (tmpReg & QM_EX_DEQUEUE_SOURCE)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_DEQUEUE_SOURCE);
    if (tmpReg & QM_EX_DEQUEUE_QUEUE)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_DEQUEUE_QUEUE);
    if (tmpReg & QM_EX_ENQUEUE_OVERFLOW)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_ENQUEUE_OVERFLOW);
    if (tmpReg & QM_EX_ENQUEUE_STATE)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_ENQUEUE_STATE);
    if (tmpReg & QM_EX_ENQUEUE_CHANNEL)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_ENQUEUE_CHANNEL);
    if (tmpReg & QM_EX_ENQUEUE_QUEUE)
        p_Qm->f_Exception(p_Qm->h_App, e_QM_EX_ENQUEUE_QUEUE);
}

t_Error QM_SetException(t_Handle h_Qm, e_QmExceptions exception, bool enable)
{
    t_Qm                *p_Qm = (t_Qm*)h_Qm;
    t_Error             err;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    err = SetException(p_Qm, exception, enable);
    WRITE_UINT32(p_Qm->p_QmRegs->err_ier, p_Qm->exceptions);

    return E_OK;
}

t_Error QM_GetRevision(t_Handle h_Qm, t_QmRevisionInfo *p_QmRevisionInfo)
{
    t_Qm        *p_Qm = (t_Qm*)h_Qm;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmRevisionInfo, E_NULL_POINTER);

    return QmGetRevision(p_Qm, p_QmRevisionInfo);
}

t_Error QM_ReserveQueues(t_Handle h_Qm, t_QmRsrvFqrParams *p_QmFqrParams, uint32_t  *p_BaseFqid)
{
    t_Qm                *p_Qm = (t_Qm*)h_Qm;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Qm->p_QmDriverParams, E_INVALID_HANDLE);

    *p_BaseFqid = QmFqidRsrv(p_Qm,
                            (uint32_t)(p_QmFqrParams->useForce ?
                                       1 : p_QmFqrParams->qs.nonFrcQs.numOfFqids),
                            p_QmFqrParams->qs.nonFrcQs.align,
                            p_QmFqrParams->useForce,
                            p_QmFqrParams->qs.frcQ.fqid);
    if (*p_BaseFqid == ILLEGAL_BASE)
        RETURN_ERROR(CRITICAL,E_INVALID_STATE,("can't allocate a fqid"));

    return E_OK;
}

t_Error QM_GetErrorInformation(t_Handle h_Qm, t_QmErrorInfo *p_errInfo)
{
    uint32_t            ecsr, ecir;
    t_Qm                *p_Qm = (t_Qm*)h_Qm;
    t_Error             err = E_OK;

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Qm->p_QmDriverParams, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_errInfo, E_NULL_POINTER);

    ecsr = GET_UINT32(p_Qm->p_QmRegs->ecsr);
    ecir = GET_UINT32(p_Qm->p_QmRegs->ecir);
    if ((ecsr & QM_EX_MULTI_ECC) ||
        (ecsr & QM_EX_SINGLE_ECC))
    {
        err = E_NOT_SUPPORTED;
        REPORT_ERROR(INFO, E_NOT_SUPPORTED, ("single and multi ecc, use QM_DumpRegs"));
    }
    if ((ecsr & QM_EX_ENQUEUE_QUEUE)    ||
        (ecsr & QM_EX_ENQUEUE_STATE)    ||
        (ecsr & QM_EX_ENQUEUE_OVERFLOW) ||
        (ecsr & QM_EX_DEQUEUE_DCP)      ||
        (ecsr & QM_EX_DEQUEUE_FQ)       ||
        (ecsr & QM_EX_DEQUEUE_QUEUE)    ||
        (ecsr & QM_EX_DEQUEUE_SOURCE)   ||
        (ecsr & QM_EX_INVALID_COMMAND))
    {
        p_errInfo->portalValid = TRUE;
        p_errInfo->hwPortal = (bool)(ecir & ECIR_PORTAL_TYPE);
        if (p_errInfo->hwPortal)
            p_errInfo->dcpId = (e_DpaaDcp)((ecir & ECIR_PORTAL_MASK) >> ECIR_PORTAL_SHIFT);
        else
            p_errInfo->swPortalId = (e_DpaaSwPortal)((ecir & ECIR_PORTAL_MASK) >> ECIR_PORTAL_SHIFT);
    }

    if ((ecsr & QM_EX_ENQUEUE_QUEUE)    ||
        (ecsr & QM_EX_ENQUEUE_STATE)    ||
        (ecsr & QM_EX_ENQUEUE_OVERFLOW) ||
        (ecsr & QM_EX_ENQUEUE_CHANNEL)  ||
        (ecsr & QM_EX_DEQUEUE_QUEUE)    ||
        (ecsr & QM_EX_DEQUEUE_FQ))
    {
        p_errInfo->fqidValid = TRUE;
        p_errInfo->fqid = ((ecir & ECIR_FQID_MASK) >> ECIR_FQID_SHIFT);
    }

    WRITE_UINT32(p_Qm->p_QmRegs->ecsr, ecsr);

    return ERROR_CODE(err);
}


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
t_Error QM_DumpRegs(t_Handle h_Qm)
{
    t_Qm        *p_Qm = (t_Qm *)h_Qm;
    uint8_t     i = 0;

    DECLARE_DUMP;

    if (p_Qm->guestId != NCSW_MASTER_ID)
        RETURN_ERROR(WARNING, E_INVALID_OPERATION, ("Master Only"));

    SANITY_CHECK_RETURN_ERROR(p_Qm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Qm->p_QmDriverParams, E_INVALID_STATE);

    DUMP_SUBTITLE(("\n"));
    DUMP_TITLE(p_Qm->p_QmRegs, ("QmRegs Regs"));

    DUMP_SUBSTRUCT_ARRAY(i, QM_NUM_OF_SWP)
    {
        DUMP_VAR(&p_Qm->p_QmRegs->swpConfRegs[i], lio_cfg);
        DUMP_VAR(&p_Qm->p_QmRegs->swpConfRegs[i], io_cfg);
        DUMP_VAR(&p_Qm->p_QmRegs->swpConfRegs[i], dd_cfg);
    }
    DUMP_VAR(p_Qm->p_QmRegs, qman_dd_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, qcsp_dd_ihrsr);
    DUMP_VAR(p_Qm->p_QmRegs, qcsp_dd_ihrfr);
    DUMP_VAR(p_Qm->p_QmRegs, qcsp_dd_hasr);
    DUMP_VAR(p_Qm->p_QmRegs, dcp_dd_ihrsr);
    DUMP_VAR(p_Qm->p_QmRegs, dcp_dd_ihrfr);
    DUMP_VAR(p_Qm->p_QmRegs, dcp_dd_hasr);
    DUMP_SUBSTRUCT_ARRAY(i, QM_NUM_OF_DCP)
    {
        DUMP_VAR(&p_Qm->p_QmRegs->dcpConfRegs[i], cfg);
        DUMP_VAR(&p_Qm->p_QmRegs->dcpConfRegs[i], dd_cfg);
        DUMP_VAR(&p_Qm->p_QmRegs->dcpConfRegs[i], dlm_cfg);
        DUMP_VAR(&p_Qm->p_QmRegs->dcpConfRegs[i], dlm_avr);
    }
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_fpc);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_fp_head);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_fp_tail);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_fp_lwit);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, sfdr_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, sfdr_in_use);
    DUMP_ARR(p_Qm->p_QmRegs, wq_cs_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, wq_def_enq_wqid);
    DUMP_ARR(p_Qm->p_QmRegs, wq_sc_dd_cfg);
    DUMP_ARR(p_Qm->p_QmRegs, wq_pc_dd_cs_cfg);
    DUMP_ARR(p_Qm->p_QmRegs, wq_dc0_dd_cs_cfg);
    DUMP_ARR(p_Qm->p_QmRegs, wq_dc1_dd_cs_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, wq_dc2_dd_cs_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, wq_dc3_dd_cs_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, cm_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, ecsr);
    DUMP_VAR(p_Qm->p_QmRegs, ecir);
    DUMP_VAR(p_Qm->p_QmRegs, eadr);
    DUMP_ARR(p_Qm->p_QmRegs, edata);
    DUMP_VAR(p_Qm->p_QmRegs, sbet);
    DUMP_ARR(p_Qm->p_QmRegs, sbec);
    DUMP_VAR(p_Qm->p_QmRegs, mcr);
    DUMP_VAR(p_Qm->p_QmRegs, mcp0);
    DUMP_VAR(p_Qm->p_QmRegs, mcp1);
    DUMP_ARR(p_Qm->p_QmRegs, mr);
    DUMP_VAR(p_Qm->p_QmRegs, idle_stat);
    DUMP_VAR(p_Qm->p_QmRegs, ip_rev_1);
    DUMP_VAR(p_Qm->p_QmRegs, ip_rev_2);
    DUMP_VAR(p_Qm->p_QmRegs, fqd_bare);
    DUMP_VAR(p_Qm->p_QmRegs, fqd_bar);
    DUMP_VAR(p_Qm->p_QmRegs, fqd_ar);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_bare);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_bar);
    DUMP_VAR(p_Qm->p_QmRegs, pfdr_ar);
    DUMP_VAR(p_Qm->p_QmRegs, qcsp_bare);
    DUMP_VAR(p_Qm->p_QmRegs, qcsp_bar);
    DUMP_VAR(p_Qm->p_QmRegs, ci_sched_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, srcidr);
    DUMP_VAR(p_Qm->p_QmRegs, liodnr);
    DUMP_VAR(p_Qm->p_QmRegs, ci_rlm_cfg);
    DUMP_VAR(p_Qm->p_QmRegs, ci_rlm_avg);
    DUMP_VAR(p_Qm->p_QmRegs, err_isr);
    DUMP_VAR(p_Qm->p_QmRegs, err_ier);
    DUMP_VAR(p_Qm->p_QmRegs, err_isdr);
    DUMP_VAR(p_Qm->p_QmRegs, err_iir);
    DUMP_VAR(p_Qm->p_QmRegs, err_her);

    return E_OK;
}
#endif /* (defined(DEBUG_ERRORS) && ... */
