/******************************************************************************

 © 1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          qm.c

 @Description   QM & Portal implementation
*//***************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "string_ext.h"
#include "mm_ext.h"
#include "qm.h"
#include "qman_low.h"


/****************************************/
/*       static functions               */
/****************************************/


#define SLOW_POLL_IDLE   1000
#define SLOW_POLL_BUSY   10
static uint32_t LoopMessageRing(t_QmPortal *p_QmPortal, uint32_t is);
static void     LoopDequeueRing(t_QmPortal *p_QmPortal);

/* Portal interrupt handler */
static void portal_isr(void *ptr)
{
    t_QmPortal  *p_QmPortal = ptr;
    uint32_t    event = 0;

    DBG(TRACE, ("software-portal %d got interrupt", p_QmPortal->p_LowQmPortal->config.cpu));

    event |= (qm_isr_status_read(p_QmPortal->p_LowQmPortal) &
              qm_isr_enable_read(p_QmPortal->p_LowQmPortal));
    while (event)
    {
        qm_isr_status_clear(p_QmPortal->p_LowQmPortal, event);

        /* Only do fast-path handling if it's required */
        if (/*(event & QM_PIRQ_DQRI) &&*/
            (p_QmPortal->options & QMAN_PORTAL_FLAG_IRQ_FAST))
        {
            event &= ~QM_PIRQ_DQRI;
            LoopDequeueRing(p_QmPortal);
        }

        if ((event & (QM_PIRQ_CSCI | QM_PIRQ_EQCI | QM_PIRQ_EQRI | QM_PIRQ_MRI)) &&
            (p_QmPortal->options & QMAN_PORTAL_FLAG_IRQ_SLOW))
            event &= ~(LoopMessageRing(p_QmPortal, event));
        event |= (qm_isr_status_read(p_QmPortal->p_LowQmPortal) &
                  qm_isr_enable_read(p_QmPortal->p_LowQmPortal));
    }
}

static const char *mcr_result_str(uint8_t result)
{
    switch (result) {
    case QM_MCR_RESULT_NULL:
        return "QM_MCR_RESULT_NULL";
    case QM_MCR_RESULT_OK:
        return "QM_MCR_RESULT_OK";
    case QM_MCR_RESULT_ERR_FQID:
        return "QM_MCR_RESULT_ERR_FQID";
    case QM_MCR_RESULT_ERR_FQSTATE:
        return "QM_MCR_RESULT_ERR_FQSTATE";
    case QM_MCR_RESULT_ERR_NOTEMPTY:
        return "QM_MCR_RESULT_ERR_NOTEMPTY";
    case QM_MCR_RESULT_PENDING:
        return "QM_MCR_RESULT_PENDING";
    }
    return "<unknown MCR result>";
}

static void cb_ern_dcErn_fqs(t_Handle                   h_App,
                             t_Handle                   h_QmPortal,
                             struct qman_fq             *fq,
                             const struct qm_mr_entry   *msg)
{
    UNUSED(fq);
    UNUSED(msg);
    UNUSED(h_App);
    UNUSED(h_QmPortal);

    XX_Print("cb_ern_dcErn_fqs() unimplemented");
}

static void null_cb_mr(t_Handle                   h_App,
                       t_Handle                   h_QmPortal,
                       struct qman_fq             *fq,
                       const struct qm_mr_entry   *msg)
{
    t_QmPortal      *p_QmPortal = (t_QmPortal *)h_QmPortal;

    UNUSED(fq);UNUSED(h_App);

    XX_Print("Ignoring unowned MR frame on cpu %d, verb 0x%02x.\n",
             p_QmPortal->p_LowQmPortal->config.cpu,msg->verb);
}

static t_Error qman_create_fq(t_QmPortal *p_QmPortal, uint32_t fqid, uint32_t flags,
                struct qman_fq *fq)
{
    struct qm_fqd fqd;
    struct qm_mcr_queryfq_np np;
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;

    fq->fqid = fqid;
    fq->flags = flags;
    fq->state = qman_fq_state_oos;
    fq->cgr_groupid = 0;
    if (!(flags & QMAN_FQ_FLAG_RECOVER) ||
            (flags & QMAN_FQ_FLAG_NO_MODIFY))
        return E_OK;
    /* Everything else is RECOVER support */
    PLOCK(p_QmPortal);
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->queryfq.fqid = fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYFQ);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal))) ;
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCC_VERB_QUERYFQ);
    if (mcr->result != QM_MCR_RESULT_OK) {
        XX_Print("QUERYFQ failed: %s\n",
            mcr_result_str(mcr->result));
        goto err;
    }
    fqd = mcr->queryfq.fqd;
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->queryfq_np.fqid = fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYFQ_NP);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal))) ;
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCC_VERB_QUERYFQ_NP);
    if (mcr->result != QM_MCR_RESULT_OK) {
        XX_Print("QUERYFQ_NP failed: %s\n",
            mcr_result_str(mcr->result));
        goto err;
    }
    np = mcr->queryfq_np;
    /* Phew, have queryfq and queryfq_np results, stitch together
     * the FQ object from those. */
    fq->cgr_groupid = fqd.cgid;
    switch (np.state & QM_MCR_NP_STATE_MASK) {
    case QM_MCR_NP_STATE_OOS:
        break;
    case QM_MCR_NP_STATE_RETIRED:
        fq->state = qman_fq_state_retired;
        if (np.frm_cnt)
            fq->flags |= QMAN_FQ_STATE_NE;
        break;
    case QM_MCR_NP_STATE_TEN_SCHED:
    case QM_MCR_NP_STATE_TRU_SCHED:
    case QM_MCR_NP_STATE_ACTIVE:
        fq->state = qman_fq_state_sched;
        if (np.state & QM_MCR_NP_STATE_R)
            fq->flags |= QMAN_FQ_STATE_CHANGING;
        break;
    case QM_MCR_NP_STATE_PARKED:
        fq->state = qman_fq_state_parked;
        break;
    default:
        ASSERT_COND(NULL == "invalid FQ state");
    }
    if (fqd.fq_ctrl & QM_FQCTRL_CGE)
        fq->state |= QMAN_FQ_STATE_CGR_EN;
    PUNLOCK(p_QmPortal);

    return E_OK;
err:
    PUNLOCK(p_QmPortal);

    return ERROR_CODE(E_INVALID_STATE);
}

static void qman_destroy_fq(struct qman_fq *fq, uint32_t flags)
{
    /* We don't need to lock the FQ as it is a pre-condition that the FQ be
     * quiesced. Instead, run some checks. */
    UNUSED(flags);
    switch (fq->state) {
    case qman_fq_state_parked:
        ASSERT_COND(flags & QMAN_FQ_DESTROY_PARKED);
    case qman_fq_state_oos:
        return;
    default:
        break;
    }
    ASSERT_COND(NULL == "qman_free_fq() on unquiesced FQ!");
}

static t_Error qman_init_fq(t_QmPortal *p_QmPortal, struct qman_fq *fq, uint32_t flags,
        struct qm_mcc_initfq *opts)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    uint8_t res, myverb = (uint8_t)((flags & QMAN_INITFQ_FLAG_SCHED) ?
        QM_MCC_VERB_INITFQ_SCHED : QM_MCC_VERB_INITFQ_PARKED);

    SANITY_CHECK_RETURN_ERROR((fq->state == qman_fq_state_oos) ||
                              (fq->state == qman_fq_state_parked),
                              E_INVALID_STATE);

    if (fq->flags & QMAN_FQ_FLAG_NO_MODIFY)
        return ERROR_CODE(E_INVALID_VALUE);
    /* Issue an INITFQ_[PARKED|SCHED] management command */
    PLOCK(p_QmPortal);
    FQLOCK(fq);
    if ((fq->flags & QMAN_FQ_STATE_CHANGING) ||
            ((fq->state != qman_fq_state_oos) &&
                (fq->state != qman_fq_state_parked))) {
        FQUNLOCK(fq);
        PUNLOCK(p_QmPortal);
        return ERROR_CODE(E_BUSY);
    }
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    Mem2IOCpy32((void*)&mcc->initfq, opts, sizeof(struct qm_mcc_initfq));
    qm_mc_commit(p_QmPortal->p_LowQmPortal, myverb);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal))) ;
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == myverb);
    res = mcr->result;
    if (res != QM_MCR_RESULT_OK) {
        FQUNLOCK(fq);
        PUNLOCK(p_QmPortal);
        RETURN_ERROR(MINOR, E_INVALID_STATE,("INITFQ failed: %s", mcr_result_str(res)));
    }
    if (mcc->initfq.we_mask & QM_INITFQ_WE_FQCTRL) {
        if (mcc->initfq.fqd.fq_ctrl & QM_FQCTRL_CGE)
            fq->flags |= QMAN_FQ_STATE_CGR_EN;
        else
            fq->flags &= ~QMAN_FQ_STATE_CGR_EN;
    }
    if (mcc->initfq.we_mask & QM_INITFQ_WE_CGID)
        fq->cgr_groupid = mcc->initfq.fqd.cgid;
    fq->state = (flags & QMAN_INITFQ_FLAG_SCHED) ?
            qman_fq_state_sched : qman_fq_state_parked;
    FQUNLOCK(fq);
    PUNLOCK(p_QmPortal);
    return E_OK;
}

static int qman_retire_fq(t_QmPortal *p_QmPortal, struct qman_fq *fq, uint32_t *flags)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    int rval = 0;
    uint8_t res;

    SANITY_CHECK_RETURN_ERROR((fq->state == qman_fq_state_parked) ||
                              (fq->state == qman_fq_state_sched),
                              E_INVALID_STATE);

    if (fq->flags & QMAN_FQ_FLAG_NO_MODIFY)
        return -E_INVALID_VALUE;
    PLOCK(p_QmPortal);
    FQLOCK(fq);
    if ((fq->flags & QMAN_FQ_STATE_CHANGING) ||
            (fq->state == qman_fq_state_retired) ||
                (fq->state == qman_fq_state_oos)) {
        rval = -E_BUSY;
        goto out;
    }
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->alterfq.fqid = fq->fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_ALTER_RETIRE);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal))) ;
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_ALTER_RETIRE);
    res = mcr->result;
    if (res == QM_MCR_RESULT_OK)
    {
        rval = 0;
        /* Process 'fq' right away, we'll ignore FQRNI */
        if (mcr->alterfq.fqs & QM_MCR_FQS_NOTEMPTY)
            fq->flags |= QMAN_FQ_STATE_NE;
        if (mcr->alterfq.fqs & QM_MCR_FQS_ORLPRESENT)
            fq->flags |= QMAN_FQ_STATE_ORL;
        if (flags)
            *flags = fq->flags;
        fq->state = qman_fq_state_retired;
    }
    else if (res == QM_MCR_RESULT_PENDING)
    {
        rval = 1;
        fq->flags |= QMAN_FQ_STATE_CHANGING;
    }
    else {
        XX_Print("ALTER_RETIRE failed: %s\n",
                mcr_result_str(res));
        rval = -E_INVALID_STATE;
    }
out:
    FQUNLOCK(fq);
    PUNLOCK(p_QmPortal);
    return rval;
}

static t_Error qman_oos_fq(t_QmPortal *p_QmPortal, struct qman_fq *fq)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    t_Error ret = E_OK;
    uint8_t res;

    ASSERT_COND(fq->state == qman_fq_state_retired);
    if (fq->flags & QMAN_FQ_FLAG_NO_MODIFY)
        return ERROR_CODE(E_INVALID_VALUE);
    PLOCK(p_QmPortal);
    FQLOCK(fq);
    if ((fq->flags & QMAN_FQ_STATE_BLOCKOOS) ||
            (fq->state != qman_fq_state_retired)) {
        ret = E_BUSY;
        goto out;
    }
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->alterfq.fqid = fq->fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_ALTER_OOS);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
        cpu_relax();
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_ALTER_OOS);
    res = mcr->result;
    if (res != QM_MCR_RESULT_OK) {
        XX_Print("ALTER_OOS failed: %s\n", mcr_result_str(res));
        ret = E_INVALID_STATE;
        goto out;
    }
    fq->state = qman_fq_state_oos;
out:
    FQUNLOCK(fq);
    PUNLOCK(p_QmPortal);
    return ret;
}

static t_Error qman_schedule_fq(t_QmPortal *p_QmPortal, struct qman_fq *fq)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    t_Error ret = E_OK;
    uint8_t res;

    ASSERT_COND(fq->state == qman_fq_state_parked);
    if (fq->flags & QMAN_FQ_FLAG_NO_MODIFY)
        return ERROR_CODE(E_INVALID_VALUE);
    /* Issue a ALTERFQ_SCHED management command */
    PLOCK(p_QmPortal);
    FQLOCK(fq);
    if ((fq->flags & QMAN_FQ_STATE_CHANGING) ||
            (fq->state != qman_fq_state_parked)) {
        ret = E_BUSY;
        goto out;
    }
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->alterfq.fqid = fq->fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_ALTER_SCHED);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
        cpu_relax();

    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_ALTER_SCHED);
    res = mcr->result;
    if (res != QM_MCR_RESULT_OK) {
        XX_Print("ALTER_SCHED failed: %s\n", mcr_result_str(res));
        ret = E_INVALID_STATE;
        goto out;
    }
    fq->state = qman_fq_state_sched;
out:
    FQUNLOCK(fq);
    PUNLOCK(p_QmPortal);
    return ret;
}

static t_Error qm_query_fq_p(t_QmPortal *p_QmPortal, struct qman_fq *fq, struct qm_mcr_queryfq *p)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    uint8_t res;

    PLOCK(p_QmPortal);
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->queryfq.fqid = fq->fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYFQ);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
        cpu_relax();
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_QUERYFQ);
    res = mcr->result;
    if (res == QM_MCR_RESULT_OK)
        *p = mcr->queryfq;
    PUNLOCK(p_QmPortal);
    if (res != QM_MCR_RESULT_OK) {
        XX_Print("QUERYFQ failed: %s\n", mcr_result_str(res));
        return ERROR_CODE(E_INVALID_STATE);
    }
    return E_OK;
}

static t_Error qm_query_fq_np(t_QmPortal *p_QmPortal, struct qman_fq *fq, struct qm_mcr_queryfq_np *np)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    uint8_t res;

    PLOCK(p_QmPortal);
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->queryfq_np.fqid = fq->fqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYFQ_NP);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
        cpu_relax();
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_QUERYFQ_NP);
    res = mcr->result;
    if (res == QM_MCR_RESULT_OK)
        *np = mcr->queryfq_np;
    PUNLOCK(p_QmPortal);
    if (res != QM_MCR_RESULT_OK) {
        XX_Print("QUERYFQ_NP failed: %s\n", mcr_result_str(res));
        return ERROR_CODE(E_INVALID_STATE);
    }
    return E_OK;
}

static t_Error qm_query_wq(t_QmPortal *p_QmPortal, uint16_t wqid, struct qm_mcr_querywq *p)
{
    struct qm_mc_command *mcc;
    struct qm_mc_result *mcr;
    uint8_t res;

    PLOCK(p_QmPortal);
    mcc = qm_mc_start(p_QmPortal->p_LowQmPortal);
    mcc->querywq.channel.id = wqid;
    qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYWQ);
    while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
        cpu_relax();
    ASSERT_COND((mcr->verb & QM_MCR_VERB_MASK) == QM_MCR_VERB_QUERYWQ);
    res = mcr->result;
    if (res == QM_MCR_RESULT_OK)
        *p = mcr->querywq;
    PUNLOCK(p_QmPortal);
    if (res == QM_MCR_RESULT_ERR_BADCHANNEL) {
        return ERROR_CODE(E_NOT_FOUND);
    }
    if (res != QM_MCR_RESULT_OK) {
        XX_Print("QUERYWQ failed: %s\n", mcr_result_str(res));
        return ERROR_CODE(E_INVALID_STATE);
    }
    return E_OK;
}

static t_Error qm_new_fq(t_QmPortal     *p_QmPortal,
                         uint32_t       fqid,
                         uint32_t       fqidOffset,
                         uint32_t       channel,
                         uint32_t       wqid,
                         uint16_t       count,
                         uint32_t       flags,
                         t_QmContextA   *p_ContextA,
                         t_QmContextB   *p_ContextB,
                         t_Handle       h_QmFqr,
                         struct qman_fq **p_Fqs)
{
    struct qman_fq          *fq = NULL;
    struct qm_mcc_initfq    fq_opts;
    uint32_t                i;
    t_Error                 err = E_OK;

    ASSERT_COND(p_QmPortal);
    ASSERT_COND(count);

    for(i=0;i<count;i++)
    {
        fq = (struct qman_fq *)XX_MallocSmart(sizeof(struct qman_fq), 0, 256);
        if (!fq)
            RETURN_ERROR(MAJOR, E_NO_MEMORY, ("FQ obj!!!"));

        fq->cb.dqrr     = p_QmPortal->f_DfltFrame;
        fq->cb.ern      = fq->cb.dc_ern = fq->cb.fqs = cb_ern_dcErn_fqs;
        fq->h_App       = p_QmPortal->h_App;
        fq->h_QmFqr     = h_QmFqr;
        fq->fqidOffset  = fqidOffset;
        p_Fqs[i] = fq;
        if ((err = qman_create_fq(p_QmPortal,(uint32_t)(fqid + i), 0, p_Fqs[i])) != E_OK)
            break;
    }

    if (err != E_OK)
    {
        for(i=0;i<count;i++)
            if (p_Fqs[i])
                XX_FreeSmart(p_Fqs[i]);
        RETURN_ERROR(MINOR, err, ("Failed to create Fqs"));
    }

    memset(&fq_opts,0,sizeof(fq_opts));
    fq_opts.fqid = fqid;
    fq_opts.count = (uint16_t)(count-1);
    fq_opts.we_mask |= QM_INITFQ_WE_DESTWQ;
    fq_opts.fqd.dest.channel = channel;
    fq_opts.fqd.dest.wq = wqid;
    fq_opts.we_mask |= QM_INITFQ_WE_FQCTRL;
    fq_opts.fqd.fq_ctrl = (uint16_t)flags;

    if (p_ContextA)
    {
        fq_opts.we_mask |= QM_INITFQ_WE_CONTEXTA;
        if ((flags & QM_FQCTRL_CTXASTASHING) &&
                ((u_QmFqdContextA*)p_ContextA)->stashing.context_cl)
        {
            ((u_QmFqdContextA*)p_ContextA)->context_hi =
                (uint8_t)((CAST_POINTER_TO_UINT64(p_Fqs[0]) >> 32) & 0xff);
            ((u_QmFqdContextA*)p_ContextA)->context_lo = (uint32_t)CAST_POINTER_TO_UINT64(p_Fqs[0]);
        }
        memcpy((void*)&fq_opts.fqd.context_a, p_ContextA, sizeof(t_QmContextA));
    }
    /* If this FQ will not be used for tx, we can use contextB field */
    if (fq_opts.fqd.dest.channel < e_QM_FQ_CHANNEL_FMAN0_SP0)
    {
        if (sizeof(p_Fqs[0]) <= sizeof(fq_opts.fqd.context_b))
        {
            fq_opts.we_mask |= QM_INITFQ_WE_CONTEXTB;
            fq_opts.fqd.context_b = CAST_POINTER_TO_UINT32(p_Fqs[0]);
        }
        else
            RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("64 bit pointer (virtual) not supported yet!!!"));
    }
    else if (p_ContextB) /* Tx-Queue */
    {
        fq_opts.we_mask |= QM_INITFQ_WE_CONTEXTB;
        memcpy((void*)&fq_opts.fqd.context_b, p_ContextB, sizeof(t_QmContextB));
    }

    if ((err = qman_init_fq(p_QmPortal, p_Fqs[0], QMAN_INITFQ_FLAG_SCHED, &fq_opts)) != E_OK)
    {
        for(i=0;i<count;i++)
            if (p_Fqs[i])
                XX_FreeSmart(p_Fqs[i]);
        RETURN_ERROR(MINOR, err, ("Failed to init Fqs"));
    }

    for(i=1;i<count;i++)
    {
        memcpy(p_Fqs[i], p_Fqs[0], sizeof(struct qman_fq));
        p_Fqs[i]->fqid += i;
    }

    return err;
}

static t_Error qm_free_fq(t_QmPortal *p_QmPortal, struct qman_fq *fq)
{
    uint32_t res, flags=0;

    res = (uint32_t)qman_retire_fq(p_QmPortal, fq, &flags);
    if (res < 0) {
        XX_Print("qm_free_fq: qman_retire_fq() failed\n");
        return ERROR_CODE(E_INVALID_STATE);
    } else if (res > 0) {
        XX_Print("qm_free_fq: qman_retire_fq() failed, ");
        XX_Print("Frame Queue Not Empty, Need to dequeue\n");
        return ERROR_CODE(E_INVALID_STATE);
    }
    if (flags & QMAN_FQ_STATE_BLOCKOOS) {
        XX_Print("qm_free_fq: leaking frames\n");
        return ERROR_CODE(E_INVALID_STATE);
    }

    if (qman_oos_fq(p_QmPortal, fq) != E_OK) {
        XX_Print("qm_free_fq: qman_oos_fq() failed\n");
        return ERROR_CODE(E_INVALID_STATE);
    }

    qman_destroy_fq(fq,0);

    return E_OK;
}

/* Inline helper to reduce nesting in LoopMessageRing() */
static __inline__ void fq_state_change(struct qman_fq *fq,
                                       struct qm_mr_entry *msg,
                                       uint8_t verb)
{
    FQLOCK(fq);
    switch(verb) {
    case QM_MR_VERB_FQRL:
        ASSERT_COND(fq->flags & QMAN_FQ_STATE_ORL);
        fq->flags &= ~QMAN_FQ_STATE_ORL;
        break;
    case QM_MR_VERB_FQRN:
        ASSERT_COND((fq->state == qman_fq_state_parked) ||
            (fq->state == qman_fq_state_sched));
        ASSERT_COND(fq->flags & QMAN_FQ_STATE_CHANGING);
        fq->flags &= ~QMAN_FQ_STATE_CHANGING;
        if (msg->fq.fqs & QM_MR_FQS_NOTEMPTY)
            fq->flags |= QMAN_FQ_STATE_NE;
        if (msg->fq.fqs & QM_MR_FQS_ORLPRESENT)
            fq->flags |= QMAN_FQ_STATE_ORL;
        fq->state = qman_fq_state_retired;
        break;
    case QM_MR_VERB_FQPN:
        ASSERT_COND(fq->state == qman_fq_state_sched);
        ASSERT_COND(fq->flags & QMAN_FQ_STATE_CHANGING);
        fq->state = qman_fq_state_parked;
    }
    FQUNLOCK(fq);
}

static void qman_disable_portal(t_QmPortal *p_QmPortal)
{
    PLOCK(p_QmPortal);
    if (!(p_QmPortal->disable_count++))
        qm_dqrr_set_maxfill(p_QmPortal->p_LowQmPortal, 0);
    PUNLOCK(p_QmPortal);
}


static int int_dqrr_mr_empty(t_QmPortal *p_QmPortal, int can_wait)
{
    int ret;
    ret = (qm_dqrr_current(p_QmPortal->p_LowQmPortal) == NULL) &&
        (qm_mr_current(p_QmPortal->p_LowQmPortal) == NULL);
    if (ret && can_wait) {
        /* Stall and recheck to be sure it has quiesced. */
/* TODO - check it */
        ret = (qm_dqrr_current(p_QmPortal->p_LowQmPortal) == NULL) &&
            (qm_mr_current(p_QmPortal->p_LowQmPortal) == NULL);
    }
    return ret;
}

static t_Error qman_create_portal(t_QmPortal *p_QmPortal,
                                   uint32_t flags,
                                   uint32_t sdqcrFlags,
                                   struct qman_fq_cb *null_cb)
{
    const struct qm_portal_config   *config = &(p_QmPortal->p_LowQmPortal->config);
    int                             ret;
    t_Error                         err;
    uint32_t                        isdr;

    if ((err = qm_eqcr_init(p_QmPortal->p_LowQmPortal, e_QmPortalPVB, e_QmPortalEqcrCCE)) != E_OK)
        RETURN_ERROR(MINOR, err, ("Qman EQCR initialization failed\n"));

    if (qm_dqrr_init(p_QmPortal->p_LowQmPortal,
                     e_QmPortalDequeuePushMode,
                     e_QmPortalPVB,
                     (flags & QMAN_PORTAL_FLAG_DCA) ? e_QmPortalDqrrDCA : e_QmPortalDqrrCCI,
                     DQRR_MAXFILL,
                     (flags & QMAN_PORTAL_FLAG_RSTASH) ? 1 : 0,
                     (flags & QMAN_PORTAL_FLAG_DSTASH) ? 1 : 0)) {
        XX_Print("Qman DQRR initialization failed\n");
        goto fail_dqrr;
    }

    if (qm_mr_init(p_QmPortal->p_LowQmPortal, e_QmPortalPVB, e_QmPortalMrCCI)) {
        XX_Print("Qman MR initialization failed\n");
        goto fail_mr;
    }
    if (qm_mc_init(p_QmPortal->p_LowQmPortal)) {
        XX_Print("Qman MC initialization failed\n");
        goto fail_mc;
    }
    if (qm_isr_init(p_QmPortal->p_LowQmPortal)) {
        XX_Print("Qman ISR initialization failed\n");
        goto fail_isr;
    }
    /* static interrupt-gating controls */
    qm_dqrr_set_ithresh(p_QmPortal->p_LowQmPortal, 12);
    qm_mr_set_ithresh(p_QmPortal->p_LowQmPortal, 4);
    qm_isr_set_iperiod(p_QmPortal->p_LowQmPortal, 1);
    memset(&p_QmPortal->cgrs, 0, sizeof(p_QmPortal->cgrs));
    p_QmPortal->options = flags;
    p_QmPortal->bits = 0;
    p_QmPortal->slowpoll = 0;
    p_QmPortal->eqProd = p_QmPortal->eqCons = 0;
    p_QmPortal->disable_count = 0;
    if (null_cb)
        p_QmPortal->p_NullCB = null_cb;
    else
        p_QmPortal->p_NullCB = NULL;
    isdr = 0xffffffff;
    qm_isr_status_clear(p_QmPortal->p_LowQmPortal, 0xffffffff);
    qm_isr_enable_write(p_QmPortal->p_LowQmPortal, DEFAULT_portalExceptions);
    qm_isr_disable_write(p_QmPortal->p_LowQmPortal, isdr);
    if (flags & QMAN_PORTAL_FLAG_IRQ)
    {
        if (XX_SetIntr(config->irq, portal_isr, p_QmPortal) ||
            XX_EnableIntr(config->irq))
        {
            XX_Print("request_irq() failed\n");
            goto fail_irq;
        }
        qm_isr_uninhibit(p_QmPortal->p_LowQmPortal);
    } else
        /* without IRQ, we can't block */
        flags &= ~QMAN_PORTAL_FLAG_WAIT;
    /* Need EQCR to be empty before continuing */
    isdr ^= QM_PIRQ_EQCI;
    qm_isr_disable_write(p_QmPortal->p_LowQmPortal, isdr);
    if (!(flags & QMAN_PORTAL_FLAG_RECOVER) ||
            !(flags & QMAN_PORTAL_FLAG_WAIT))
        ret = qm_eqcr_get_fill(p_QmPortal->p_LowQmPortal);
    if (ret) {
        XX_Print("Qman EQCR unclean, need recovery\n");
        goto fail_eqcr_empty;
    }
    /* Check DQRR and MR are empty too, subject to RECOVERY logic */
    if (flags & QMAN_PORTAL_FLAG_RECOVER)
        p_QmPortal->bits |= PORTAL_BITS_RECOVER;
    isdr ^= (QM_PIRQ_DQRI | QM_PIRQ_MRI);
    qm_isr_disable_write(p_QmPortal->p_LowQmPortal, isdr);
    if (!(flags & QMAN_PORTAL_FLAG_RECOVER))
        ret = !int_dqrr_mr_empty(p_QmPortal, 0);
    else {
        if (!(flags & QMAN_PORTAL_FLAG_WAIT))
            ret = !int_dqrr_mr_empty(p_QmPortal, 0);
        qman_disable_portal(p_QmPortal);
        p_QmPortal->bits ^= PORTAL_BITS_RECOVER;
    }
    if (ret) {
        if (flags & QMAN_PORTAL_FLAG_RECOVER)
            XX_Print("Qman DQRR/MR unclean, "
                "recovery failed\n");
        else
            XX_Print("Qman DQRR/MR unclean, "
                "need recovery\n");
        goto fail_dqrr_mr_empty;
    }
    qm_isr_disable_write(p_QmPortal->p_LowQmPortal, 0);
    /* Write a sane SDQCR */
    qm_dqrr_sdqcr_set(p_QmPortal->p_LowQmPortal, sdqcrFlags);
    return E_OK;
fail_dqrr_mr_empty:
fail_eqcr_empty:
fail_irq:
    if (p_QmPortal->cgrs)
        XX_Free(p_QmPortal->cgrs);
    qm_isr_finish(p_QmPortal->p_LowQmPortal);
fail_isr:
    qm_mc_finish(p_QmPortal->p_LowQmPortal);
fail_mc:
    qm_mr_finish(p_QmPortal->p_LowQmPortal);
fail_mr:
    qm_dqrr_finish(p_QmPortal->p_LowQmPortal);
fail_dqrr:
    qm_eqcr_finish(p_QmPortal->p_LowQmPortal);
    return ERROR_CODE(E_INVALID_STATE);
}

static void qman_destroy_portal(t_QmPortal *p_QmPortal)
{
    /* NB we do this to "quiesce" EQCR. If we add enqueue-completions or
     * something related to QM_PIRQ_EQCI, this may need fixing. */
    qmPortalEqcrCceUpdate(p_QmPortal->p_LowQmPortal);
    if (p_QmPortal->options & QMAN_PORTAL_FLAG_IRQ)
    {
        XX_DisableIntr(p_QmPortal->p_LowQmPortal->config.irq);
        XX_FreeIntr(p_QmPortal->p_LowQmPortal->config.irq);
    }
    qm_isr_finish(p_QmPortal->p_LowQmPortal);
    qm_mc_finish(p_QmPortal->p_LowQmPortal);
    qm_mr_finish(p_QmPortal->p_LowQmPortal);
    qm_dqrr_finish(p_QmPortal->p_LowQmPortal);
    qm_eqcr_finish(p_QmPortal->p_LowQmPortal);
}

static uint32_t LoopMessageRing(t_QmPortal *p_QmPortal, uint32_t is)
{
    struct qm_mr_entry          *msg;
    uint32_t                    flags;

    flags = XX_LockIntrSpinlock(p_QmPortal->pollLock);
    if (p_QmPortal->inPoll)
    {
        XX_UnlockIntrSpinlock(p_QmPortal->pollLock, flags);
        return 0;
    }
    p_QmPortal->inPoll = TRUE;
    XX_UnlockIntrSpinlock(p_QmPortal->pollLock, flags);

    qmPortalMrPvbPrefetch(p_QmPortal->p_LowQmPortal);
    if (is & QM_PIRQ_CSCI) {
        struct qm_mc_result *mcr;
        unsigned int i;
        PLOCK(p_QmPortal);
        qm_mc_start(p_QmPortal->p_LowQmPortal);
        qm_mc_commit(p_QmPortal->p_LowQmPortal, QM_MCC_VERB_QUERYCONGESTION);
        while (!(mcr = qm_mc_result(p_QmPortal->p_LowQmPortal)))
            cpu_relax();
        p_QmPortal->cgrs[1].q = mcr->querycongestion.state;
        PUNLOCK(p_QmPortal);
        for (i = 0; i < 8; i++)
            p_QmPortal->cgrs[1].q.__state[i] &= p_QmPortal->cgrs[0].q.__state[i];
    }


    if (is & QM_PIRQ_EQRI) {
        PLOCK(p_QmPortal);
        p_QmPortal->eqCons += qmPortalEqcrCceUpdate(p_QmPortal->p_LowQmPortal);
        qm_eqcr_set_ithresh(p_QmPortal->p_LowQmPortal, 0);
        PUNLOCK(p_QmPortal);
    }

    if (is & QM_PIRQ_MRI) {
        uint8_t         num = 0;
mr_loop:
        if (qmPortalMrPvbUpdate(p_QmPortal->p_LowQmPortal))
            qmPortalMrPvbPrefetch(p_QmPortal->p_LowQmPortal);
        msg = qm_mr_current(p_QmPortal->p_LowQmPortal);
        if (msg) {
            struct qman_fq  *fqFqs  = (void *)msg->fq.contextB;
            struct qman_fq  *fqErn  = (void *)msg->ern.tag;
            uint8_t         verb    =(uint8_t)(msg->verb & QM_MR_VERB_TYPE_MASK);
            qm_mr_next(p_QmPortal->p_LowQmPortal);
            if (!(verb & QM_MR_VERB_DC_ERN))
            {
                if ((p_QmPortal->bits & PORTAL_BITS_RECOVER) ||
                    !fqErn)
                    p_QmPortal->p_NullCB->ern(NULL, p_QmPortal, NULL, msg);
                else
                    fqErn->cb.ern(fqErn->h_App, p_QmPortal, fqErn, msg);
            } else if (verb == QM_MR_VERB_DC_ERN)
            {
                if ((p_QmPortal->bits & PORTAL_BITS_RECOVER) ||
                    !fqErn)
                    p_QmPortal->p_NullCB->dc_ern(NULL, p_QmPortal, NULL, msg);
                else
                    fqErn->cb.dc_ern(fqErn->h_App, p_QmPortal, fqErn, msg);
            } else
            {
                if (verb == QM_MR_VERB_FQRNI)
                    ; /* we drop FQRNIs on the floor */
                else if ((p_QmPortal->bits & PORTAL_BITS_RECOVER) ||
                           !fqFqs)
                            p_QmPortal->p_NullCB->fqs(NULL, p_QmPortal, NULL, msg);
                else if ((verb == QM_MR_VERB_FQRN) ||
                         (verb == QM_MR_VERB_FQRL) ||
                         (verb == QM_MR_VERB_FQPN))
                {
                    fq_state_change(fqFqs, msg, verb);
                    fqFqs->cb.fqs(fqFqs->h_App, p_QmPortal, fqFqs, msg);
                }
            }
            num++;
            goto mr_loop;
        }
        qmPortalMrCciConsume(p_QmPortal->p_LowQmPortal, num);
    }

    p_QmPortal->inPoll = FALSE;

    return is & (QM_PIRQ_CSCI | QM_PIRQ_EQCI | QM_PIRQ_EQRI | QM_PIRQ_MRI);
}

static void LoopDequeueRing(t_QmPortal *p_QmPortal)
{
    struct qm_dqrr_entry        *dq;
    struct qman_fq              *fq;
    enum qman_cb_dqrr_result    res = qman_cb_dqrr_consume;
    e_RxStoreResponse           tmpRes;
    int                         prefetch = !(p_QmPortal->options & QMAN_PORTAL_FLAG_RSTASH);
/*
    uint32_t                    flags;

    flags = XX_LockIntrSpinlock(p_QmPortal->pollLock);
    if (p_QmPortal->inPoll)
    {
        XX_UnlockIntrSpinlock(p_QmPortal->pollLock, flags);
        return;
    }
    p_QmPortal->inPoll = TRUE;
    XX_UnlockIntrSpinlock(p_QmPortal->pollLock, flags);
*/

    while (res != qman_cb_dqrr_pause)
    {
        if (qmPortalDqrrPvbUpdate(p_QmPortal->p_LowQmPortal) && prefetch)
            qmPortalDqrrPvbPrefetch(p_QmPortal->p_LowQmPortal);
        dq = qm_dqrr_current(p_QmPortal->p_LowQmPortal);
        if (!dq)
            break;
        fq = (void *)dq->contextB;
        /* Interpret 'dq' from the owner's perspective. */
        if ((p_QmPortal->bits & PORTAL_BITS_RECOVER) || !fq) {
            /* use portal default handlers */
            ASSERT_COND(dq->fqid);
            tmpRes = fq->cb.dqrr(fq->h_App, fq->h_QmFqr, p_QmPortal, fq->fqidOffset, (t_DpaaFD*)&dq->fd);
            if (tmpRes == e_RX_STORE_RESPONSE_PAUSE)
                res = qman_cb_dqrr_pause;
            else if (fq->state == qman_fq_state_waiting_parked)
                res = qman_cb_dqrr_park;
            ASSERT_COND(res == qman_cb_dqrr_consume);
        } else {
            if (dq->stat & QM_DQRR_STAT_FQ_EMPTY)
                fq->flags &= ~QMAN_FQ_STATE_NE;
            /* Now let the callback do its stuff */
            ASSERT_COND((dq->stat & QM_DQRR_STAT_FD_VALID) ||
                        (dq->stat & QM_DQRR_STAT_FQ_EMPTY));
            ASSERT_COND(dq->fqid);
            tmpRes = fq->cb.dqrr(fq->h_App, fq->h_QmFqr, p_QmPortal, fq->fqidOffset, (t_DpaaFD*)&dq->fd);
            if (tmpRes == e_RX_STORE_RESPONSE_PAUSE)
                res = qman_cb_dqrr_pause;
            else if (fq->state == qman_fq_state_waiting_parked)
                res = qman_cb_dqrr_park;
        }
        /* Interpret 'dq' from a driver perspective. */
        if ((dq->stat & VDQCR_DONE) == VDQCR_DONE)
            PORTAL_BITS_INC_V(p_QmPortal);

        /* Parking isn't possible unless HELDACTIVE was set. NB,
         * FORCEELIGIBLE implies HELDACTIVE, so we only need to
         * check for HELDACTIVE to cover both. */
        ASSERT_COND((dq->stat & QM_DQRR_STAT_FQ_HELDACTIVE) ||
                    (res != qman_cb_dqrr_park));
        if (p_QmPortal->options & QMAN_PORTAL_FLAG_DCA) {
            /* Defer just means "skip it, I'll consume it myself later on" */
            if (res != qman_cb_dqrr_defer)
                qmPortalDqrrDcaConsume1ptr(p_QmPortal->p_LowQmPortal,
                                           dq,
                                           (res == qman_cb_dqrr_park));
            qm_dqrr_next(p_QmPortal->p_LowQmPortal);
        } else {
            if (res == qman_cb_dqrr_park)
                /* The only thing to do for non-DCA is the park-request */
                qm_dqrr_park_ci(p_QmPortal->p_LowQmPortal);
            qm_dqrr_next(p_QmPortal->p_LowQmPortal);
            qmPortalDqrrCciConsume(p_QmPortal->p_LowQmPortal, 1);
        }
    }

    if (prefetch)
        qmPortalDqrrPvbPrefetch(p_QmPortal->p_LowQmPortal);
    //p_QmPortal->inPoll = FALSE;
}

#ifndef DISABLE_UNUSED_CODE
static void qman_poll(t_QmPortal *p_QmPortal, e_QmPortalPollSource source)
{
    if (((source == e_QM_PORTAL_POLL_SOURCE_CONTROL_FRAMES) ||
         (source == e_QM_PORTAL_POLL_SOURCE_BOTH)) &&
        !(p_QmPortal->slowpoll--))
    {
        uint32_t is = qm_isr_status_read(p_QmPortal->p_LowQmPortal);
        uint32_t active = LoopMessageRing(p_QmPortal, is);
        if (active) {
            qm_isr_status_clear(p_QmPortal->p_LowQmPortal, active);
            p_QmPortal->slowpoll = SLOW_POLL_BUSY;
        } else
            p_QmPortal->slowpoll = SLOW_POLL_IDLE;
    }

    if (source == e_QM_PORTAL_POLL_SOURCE_DATA_FRAMES ||
        source == e_QM_PORTAL_POLL_SOURCE_BOTH)
        LoopDequeueRing(p_QmPortal);

}
#endif

static __inline__ void eqcr_set_thresh(t_QmPortal *p_QmPortal, int check)
{
    if (!check || !qm_eqcr_get_ithresh(p_QmPortal->p_LowQmPortal))
        qm_eqcr_set_ithresh(p_QmPortal->p_LowQmPortal, EQCR_ITHRESH);
}

static struct qm_eqcr_entry *try_eq_start(t_QmPortal *p_QmPortal)
{
    struct qm_eqcr_entry    *eq;
    uint8_t                 avail;

    PLOCK(p_QmPortal);
    avail = qm_eqcr_get_avail(p_QmPortal->p_LowQmPortal);
    if (avail == EQCR_THRESH)
        qmPortalEqcrCcePrefetch(p_QmPortal->p_LowQmPortal);
    else if (avail < EQCR_THRESH)
            p_QmPortal->eqCons += qmPortalEqcrCceUpdate(p_QmPortal->p_LowQmPortal);
    eq = qm_eqcr_start(p_QmPortal->p_LowQmPortal);
    if (!eq) {
        eqcr_set_thresh(p_QmPortal, 1);
        PUNLOCK(p_QmPortal);
    }
    return eq;
}

static int eqcr_completed(t_QmPortal *p_QmPortal, uint32_t eq_poll)
{
    uint32_t tr_cons = p_QmPortal->eqCons;
    if (eq_poll & 0xc0000000) {
        eq_poll &= 0x7fffffff;
        tr_cons ^= 0x80000000;
    }
    if (tr_cons >= eq_poll)
        return 1;
    if ((eq_poll - tr_cons) > QM_EQCR_SIZE)
        return 1;
    if (!qm_eqcr_get_fill(p_QmPortal->p_LowQmPortal))
        /* If EQCR is empty, we must have completed */
        return 1;
    eqcr_set_thresh(p_QmPortal, 0);
    return 0;
}

static void eqcr_commit(t_QmPortal *p_QmPortal, uint32_t flags, int orp)
{
    uint32_t eq_poll;

    qmPortalEqcrPvbCommit(p_QmPortal->p_LowQmPortal,
                          (uint8_t)(QM_EQCR_VERB_CMD_ENQUEUE |
                                   (flags & (QM_EQCR_VERB_COLOUR_MASK | QM_EQCR_VERB_INTERRUPT)) |
                                   (orp ? QM_EQCR_VERB_ORP : 0)));
    /* increment the producer count and capture it for SYNC */
    eq_poll = ++p_QmPortal->eqProd;
    if ((flags & QMAN_ENQUEUE_FLAG_WAIT_SYNC) ==
            QMAN_ENQUEUE_FLAG_WAIT_SYNC)
        eqcr_set_thresh(p_QmPortal, 1);
    PUNLOCK(p_QmPortal);
    if ((flags & QMAN_ENQUEUE_FLAG_WAIT_SYNC) ==
        QMAN_ENQUEUE_FLAG_WAIT_SYNC)
        return;

    eqcr_completed(p_QmPortal, eq_poll);
}

static t_Error qman_enqueue(t_QmPortal *p_QmPortal,struct qman_fq *fq, const struct qm_fd *fd,uint32_t flags)
{
    struct qm_eqcr_entry    *eq;
    struct qman_fq          *orp_fq = NULL;
    uint32_t                *p_Dst, *p_Src;
    uint16_t                orp_seqnum = 0;
    int                     orp = 0;

    if (fq->flags & QMAN_FQ_FLAG_NO_ENQUEUE)
        RETURN_ERROR(MINOR, E_INVALID_VALUE, NO_MSG);
    if ((!(fq->flags & QMAN_FQ_FLAG_NO_MODIFY)) && ((fq->state == qman_fq_state_retired) ||
            (fq->state == qman_fq_state_oos)))
        return ERROR_CODE(E_BUSY);

    eq = try_eq_start(p_QmPortal);
    if (!eq)
        return ERROR_CODE(E_BUSY);

    if (flags & QMAN_ENQUEUE_FLAG_DCA) {
        uint8_t dca = QM_EQCR_DCA_ENABLE;
        if (flags & QMAN_ENQUEUE_FLAG_DCA_PARK)
            dca |= QM_EQCR_DCA_PARK;
        dca |= ((flags >> 8) & QM_EQCR_DCA_IDXMASK);
        eq->dca = dca;
    }
    if (orp) {
        if (flags & QMAN_ENQUEUE_FLAG_NLIS)
            orp_seqnum |= QM_EQCR_SEQNUM_NLIS;
        else {
            orp_seqnum &= ~QM_EQCR_SEQNUM_NLIS;
            if (flags & QMAN_ENQUEUE_FLAG_NESN)
                orp_seqnum |= QM_EQCR_SEQNUM_NESN;
            else
                /* No need to check 4 QMAN_ENQUEUE_FLAG_HOLE */
                orp_seqnum &= ~QM_EQCR_SEQNUM_NESN;
        }
        eq->seqnum = orp_seqnum;
        eq->orp = orp_fq->fqid;
    }

    eq->fqid = fq->fqid;
    eq->tag = (uint32_t)fq;

    /* gcc does a dreadful job of the following;
     *  eq->fd = *fd;
     * It causes the entire function to save/restore a wider range of
     * registers, and comes up with instruction-waste galore. This will do
     * until we can rework the function for better code-generation. */
    p_Dst = (uint32_t *)&eq->fd;
    p_Src = (uint32_t *)fd;
    p_Dst[0] = p_Src[0];
    p_Dst[1] = p_Src[1];
    p_Dst[2] = p_Src[2];
    p_Dst[3] = p_Src[3];

    /* Issue the enqueue command, and wait for sync if requested.
     * NB: design choice - the commit can't fail, only waiting can. Don't
     * Propagate any failure if a signal arrives. Otherwise the caller can't
     * distinguish whether the enqueue was issued or not. Code for
     * user-space can check signal_pending() after we return. */
    eqcr_commit(p_QmPortal, flags, orp);

    return E_OK;
}

static __inline__ t_Error CheckStashParams(t_QmFqrParams *p_QmFqrParams)
{
    ASSERT_COND(p_QmFqrParams);

    if (p_QmFqrParams->stashingParams.frameAnnotationSize > QM_CONTEXTA_MAX_STASH_SIZE)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Frame Annotation Size Exceeded Max Stash Size(%d)", QM_CONTEXTA_MAX_STASH_SIZE));
    if (p_QmFqrParams->stashingParams.frameDataSize > QM_CONTEXTA_MAX_STASH_SIZE)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Frame Data Size Exceeded Max Stash Size(%d)", QM_CONTEXTA_MAX_STASH_SIZE));
    if (p_QmFqrParams->stashingParams.fqContextSize > QM_CONTEXTA_MAX_STASH_SIZE)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Frame Context Size Exceeded Max Stash Size(%d)", QM_CONTEXTA_MAX_STASH_SIZE));
    if (p_QmFqrParams->stashingParams.fqContextSize > QM_CONTEXTA_MAX_STASH_SIZE)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("FQ Context Size Exceeded Max Stash Size(%d)", QM_CONTEXTA_MAX_STASH_SIZE));
    if (!IS_ALIGNED(p_QmFqrParams->stashingParams.fqContextAddr, CACHELINE_SIZE))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("FQ Context Address Must be aligned to %d", CACHELINE_SIZE));
    if (p_QmFqrParams->stashingParams.fqContextAddr & 0xffffff0000000000LL)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("FQ Context Address May be up to 40 bit"));

    return E_OK;
}



/****************************************/
/*       API Init unit functions        */
/****************************************/
t_Handle QM_PORTAL_Config(t_QmPortalParam *p_QmPortalParam)
{
    t_QmPortal          *p_QmPortal;
    uint32_t            i;

    SANITY_CHECK_RETURN_VALUE(p_QmPortalParam, E_INVALID_HANDLE, NULL);
    SANITY_CHECK_RETURN_VALUE(p_QmPortalParam->swPortalId < DPAA_MAX_NUM_OF_SW_PORTALS, E_INVALID_VALUE, 0);

    p_QmPortal = (t_QmPortal *)XX_Malloc(sizeof(t_QmPortal));
    if (!p_QmPortal)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Qm Portal obj!!!"));
        return NULL;
    }
    memset(p_QmPortal, 0, sizeof(t_QmPortal));

    p_QmPortal->p_LowQmPortal = (struct qm_portal *)XX_Malloc(sizeof(struct qm_portal));
    if (!p_QmPortal->p_LowQmPortal)
    {
        XX_Free(p_QmPortal);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Low qm p_QmPortal obj!!!"));
        return NULL;
    }
    memset(p_QmPortal->p_LowQmPortal, 0, sizeof(struct qm_portal));

    p_QmPortal->p_QmPortalDriverParams = (t_QmPortalDriverParams *)XX_Malloc(sizeof(t_QmPortalDriverParams));
    if (!p_QmPortal->p_QmPortalDriverParams)
    {
        XX_Free(p_QmPortal);
        XX_Free(p_QmPortal->p_LowQmPortal);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Qm Portal driver parameters"));
        return NULL;
    }
    memset(p_QmPortal->p_QmPortalDriverParams, 0, sizeof(t_QmPortalDriverParams));

    p_QmPortal->p_LowQmPortal->addr.addr_ce = CAST_UINT64_TO_POINTER(p_QmPortalParam->ceBaseAddress);
    p_QmPortal->p_LowQmPortal->addr.addr_ci = CAST_UINT64_TO_POINTER(p_QmPortalParam->ciBaseAddress);
    p_QmPortal->p_LowQmPortal->config.irq = p_QmPortalParam->irq;
    p_QmPortal->p_LowQmPortal->config.bound = 0;
    p_QmPortal->p_LowQmPortal->config.cpu = (int)p_QmPortalParam->swPortalId;
    p_QmPortal->p_LowQmPortal->config.channel = (e_QmFQChannel)(e_QM_FQ_CHANNEL_SWPORTAL0 + p_QmPortalParam->swPortalId);
    p_QmPortal->p_LowQmPortal->bind_lock = XX_InitSpinlock();

    p_QmPortal->h_Qm                = p_QmPortalParam->h_Qm;
    p_QmPortal->f_DfltFrame         = p_QmPortalParam->f_DfltFrame;
    p_QmPortal->h_App               = p_QmPortalParam->h_App;
    p_QmPortal->inPoll              = FALSE;
    p_QmPortal->pollLock            = XX_InitSpinlock();

    p_QmPortal->p_QmPortalDriverParams->fdLiodn                 = p_QmPortalParam->fdLiodn;
    p_QmPortal->p_QmPortalDriverParams->dequeueDcaMode          = DEFAULT_dequeueDcaMode;
    p_QmPortal->p_QmPortalDriverParams->dequeueUpToThreeFrames  = DEFAULT_dequeueUpToThreeFrames;
    p_QmPortal->p_QmPortalDriverParams->commandType             = DEFAULT_dequeueCommandType;
    p_QmPortal->p_QmPortalDriverParams->userToken               = DEFAULT_dequeueUserToken;
    p_QmPortal->p_QmPortalDriverParams->specifiedWq             = DEFAULT_dequeueSpecifiedWq;
    p_QmPortal->p_QmPortalDriverParams->dedicatedChannel        = DEFAULT_dequeueDedicatedChannel;
    p_QmPortal->p_QmPortalDriverParams->dedicatedChannelHasPrecedenceOverPoolChannels = DEFAULT_dequeueDedicatedChannelHasPrecedenceOverPoolChannels;
    p_QmPortal->p_QmPortalDriverParams->poolChannelId           = DEFAULT_dequeuePoolChannelId;
    p_QmPortal->p_QmPortalDriverParams->wqId                    = DEFAULT_dequeueWqId;
    for (i=0;i<QM_MAX_NUM_OF_POOL_CHANNELS;i++)
        p_QmPortal->p_QmPortalDriverParams->poolChannels[i] = FALSE;

    return p_QmPortal;
}

t_Error QM_PORTAL_Init(t_Handle h_QmPortal)
{
    t_QmPortal                          *p_QmPortal = (t_QmPortal *)h_QmPortal;
    uint32_t                            i, flags=0, sdqcrFlags=0;
    t_Error                             err;
    struct qman_fq_cb                   *p_NullCb;
    t_QmInterModulePortalInitParams     qmParams;

    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmPortal->p_QmPortalDriverParams, E_INVALID_HANDLE);

    memset(&qmParams, 0, sizeof(qmParams));
    qmParams.portalId       = (uint8_t)p_QmPortal->p_LowQmPortal->config.cpu;
    qmParams.liodn          = p_QmPortal->p_QmPortalDriverParams->fdLiodn;
    qmParams.dqrrLiodn      = p_QmPortal->p_QmPortalDriverParams->dqrrLiodn;
    qmParams.fdFqLiodn      = p_QmPortal->p_QmPortalDriverParams->fdFqLiodn;
    qmParams.stashDestQueue = p_QmPortal->p_QmPortalDriverParams->stashDestQueue;
    if ((err = QmGetSetPortalParams(p_QmPortal->h_Qm, &qmParams)) != E_OK)
        RETURN_ERROR(MAJOR, err, NO_MSG);

    flags = (uint32_t)(((p_QmPortal->p_LowQmPortal->config.irq == NO_IRQ) ?
            0 :
            (QMAN_PORTAL_FLAG_IRQ |
             QMAN_PORTAL_FLAG_IRQ_FAST |
             QMAN_PORTAL_FLAG_IRQ_SLOW)));
    flags |= ((p_QmPortal->p_QmPortalDriverParams->dequeueDcaMode) ? QMAN_PORTAL_FLAG_DCA : 0);

    flags |= (p_QmPortal->p_QmPortalDriverParams->dqrr)?QMAN_PORTAL_FLAG_RSTASH:0;
    flags |= (p_QmPortal->p_QmPortalDriverParams->fdFq)?QMAN_PORTAL_FLAG_DSTASH:0;

    sdqcrFlags |= (p_QmPortal->p_QmPortalDriverParams->dequeueUpToThreeFrames) ? QM_SDQCR_COUNT_UPTO3 : QM_SDQCR_COUNT_EXACT1;
    sdqcrFlags |= QM_SDQCR_TOKEN_SET(p_QmPortal->p_QmPortalDriverParams->userToken);
    sdqcrFlags |= QM_SDQCR_TYPE_SET(p_QmPortal->p_QmPortalDriverParams->commandType);
    if (!p_QmPortal->p_QmPortalDriverParams->specifiedWq)
    {
        /*sdqcrFlags |= QM_SDQCR_SOURCE_CHANNELS;*/ /* removed as the macro is '0' */
        sdqcrFlags |= (p_QmPortal->p_QmPortalDriverParams->dedicatedChannelHasPrecedenceOverPoolChannels) ? QM_SDQCR_DEDICATED_PRECEDENCE : 0;
        sdqcrFlags |= (p_QmPortal->p_QmPortalDriverParams->dedicatedChannel) ? QM_SDQCR_CHANNELS_DEDICATED : 0;
        for (i=0;i<QM_MAX_NUM_OF_POOL_CHANNELS;i++)
            sdqcrFlags |= ((p_QmPortal->p_QmPortalDriverParams->poolChannels[i]) ?
                 QM_SDQCR_CHANNELS_POOL(i+1) : 0);
    }
    else
    {
        sdqcrFlags |= QM_SDQCR_SOURCE_SPECIFICWQ;
        sdqcrFlags |= (p_QmPortal->p_QmPortalDriverParams->dedicatedChannel) ?
                        QM_SDQCR_SPECIFICWQ_DEDICATED : QM_SDQCR_SPECIFICWQ_POOL(p_QmPortal->p_QmPortalDriverParams->poolChannelId);
        sdqcrFlags |= QM_SDQCR_SPECIFICWQ_WQ(p_QmPortal->p_QmPortalDriverParams->wqId);
    }

    p_NullCb = (struct qman_fq_cb *)XX_Malloc(sizeof(struct qman_fq_cb));
    if (!p_NullCb)
    {
        QM_PORTAL_Free(p_QmPortal);
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("FQ Null CB obj!!!"));
    }

    p_NullCb->dqrr = p_QmPortal->f_DfltFrame;
    p_NullCb->ern = p_NullCb->dc_ern = p_NullCb->fqs = null_cb_mr;

    if (qman_create_portal(p_QmPortal, flags, sdqcrFlags, p_NullCb) != E_OK)
    {
        XX_Free(p_NullCb);
        QM_PORTAL_Free(p_QmPortal);
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("create portal failed"));
    }

    QmSetPortalHandle(p_QmPortal->h_Qm, (t_Handle)p_QmPortal, (e_DpaaSwPortal)p_QmPortal->p_LowQmPortal->config.cpu);
    XX_Free(p_QmPortal->p_QmPortalDriverParams);
    p_QmPortal->p_QmPortalDriverParams = NULL;

    DBG(TRACE,("Qman portal at %p:%p (%d)",
               p_QmPortal->p_LowQmPortal->addr.addr_ce,
               p_QmPortal->p_LowQmPortal->addr.addr_ci,
               p_QmPortal->p_LowQmPortal->config.cpu));

    return E_OK;
}

t_Error QM_PORTAL_Free(t_Handle h_QmPortal)
{
    t_QmPortal  *p_QmPortal = (t_QmPortal *)h_QmPortal;

    if (!p_QmPortal)
       return ERROR_CODE(E_INVALID_HANDLE);

    QmSetPortalHandle(p_QmPortal->h_Qm, NULL, (e_DpaaSwPortal)p_QmPortal->p_LowQmPortal->config.cpu);
    qman_destroy_portal(p_QmPortal);
    if (p_QmPortal->p_NullCB)
        XX_Free(p_QmPortal->p_NullCB);

    if (p_QmPortal->pollLock)
        XX_FreeSpinlock(p_QmPortal->pollLock);

    if (p_QmPortal->p_LowQmPortal)
    {
        if (p_QmPortal->p_LowQmPortal->bind_lock)
            XX_FreeSpinlock(p_QmPortal->p_LowQmPortal->bind_lock);
        XX_Free(p_QmPortal->p_LowQmPortal);
    }

    XX_Free(p_QmPortal);
    return E_OK;
}

t_Error QM_PORTAL_ConfigDcaMode(t_Handle h_QmPortal, bool enable)
{
    t_QmPortal  *p_QmPortal = (t_QmPortal *)h_QmPortal;

    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmPortal->p_QmPortalDriverParams, E_INVALID_HANDLE);

    p_QmPortal->p_QmPortalDriverParams->dequeueDcaMode = enable;

    return E_OK;
}

t_Error QM_PORTAL_ConfigStash(t_Handle h_QmPortal, t_QmPortalStashParam *p_StashParams)
{
    t_QmPortal  *p_QmPortal = (t_QmPortal *)h_QmPortal;

    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_QmPortal->p_QmPortalDriverParams, E_NULL_POINTER);
    SANITY_CHECK_RETURN_ERROR(p_StashParams, E_NULL_POINTER);

    p_QmPortal->p_QmPortalDriverParams->stashDestQueue = p_StashParams->stashDestQueue;
    p_QmPortal->p_QmPortalDriverParams->dqrrLiodn = p_StashParams->dqrrLiodn;
    p_QmPortal->p_QmPortalDriverParams->fdFqLiodn = p_StashParams->fdFqLiodn;
    p_QmPortal->p_QmPortalDriverParams->eqcr = p_StashParams->eqcr;
    p_QmPortal->p_QmPortalDriverParams->eqcrHighPri = p_StashParams->eqcrHighPri;
    p_QmPortal->p_QmPortalDriverParams->dqrr = p_StashParams->dqrr;
    p_QmPortal->p_QmPortalDriverParams->dqrrHighPri = p_StashParams->dqrrHighPri;
    p_QmPortal->p_QmPortalDriverParams->fdFq = p_StashParams->fdFq;
    p_QmPortal->p_QmPortalDriverParams->fdFqHighPri = p_StashParams->fdFqHighPri;
    p_QmPortal->p_QmPortalDriverParams->fdFqDrop = p_StashParams->fdFqDrop;

    return E_OK;
}

t_Error QM_PORTAL_AddPoolChannel(t_Handle h_QmPortal, uint8_t poolChannelId)
{
    t_QmPortal  *p_QmPortal = (t_QmPortal *)h_QmPortal;
    uint32_t    sdqcrFlags;

    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((poolChannelId < QM_MAX_NUM_OF_POOL_CHANNELS), E_INVALID_VALUE);

    sdqcrFlags = qm_dqrr_sdqcr_get(p_QmPortal->p_LowQmPortal);
    sdqcrFlags |= QM_SDQCR_CHANNELS_POOL(poolChannelId+1);
    qm_dqrr_sdqcr_set(p_QmPortal->p_LowQmPortal, sdqcrFlags);

    return E_OK;
}

t_Error QM_PORTAL_Poll(t_Handle h_QmPortal, e_QmPortalPollSource source)
{
    t_QmPortal  *p_QmPortal = (t_QmPortal *)h_QmPortal;
    t_Error     err = E_OK;

    SANITY_CHECK_RETURN_ERROR(p_QmPortal, E_INVALID_HANDLE);
#if defined (__QNXNTO__)
   portal_isr(p_QmPortal);
#else
    qman_poll(p_QmPortal, source);
#endif

    return err;
}

t_Handle QM_FQR_Create(t_QmFqrParams *p_QmFqrParams)
{
    t_QmFqr         *p_QmFqr;
    uint32_t        i, flags = 0;
    u_QmFqdContextA cnxtA;

    SANITY_CHECK_RETURN_VALUE(p_QmFqrParams, E_INVALID_HANDLE, NULL);
    SANITY_CHECK_RETURN_VALUE(p_QmFqrParams->h_Qm, E_INVALID_HANDLE, NULL);

    p_QmFqr = (t_QmFqr *)XX_Malloc(sizeof(t_QmFqr));
    if (!p_QmFqr)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("QM FQR obj!!!"));
        return NULL;
    }
    memset(p_QmFqr, 0, sizeof(p_QmFqr));

    p_QmFqr->h_Qm       = p_QmFqrParams->h_Qm;
    p_QmFqr->h_QmPortal = p_QmFqrParams->h_QmPortal;
    p_QmFqr->channel    = p_QmFqrParams->channel;
    p_QmFqr->workQueue  = p_QmFqrParams->wq;
    p_QmFqr->numOfFqids = p_QmFqrParams->useForce ?
                            1 : p_QmFqrParams->qs.nonFrcQs.numOfFqids;

    if (!p_QmFqr->h_QmPortal)
    {
        SANITY_CHECK_RETURN_VALUE(p_QmFqr->h_Qm, E_INVALID_HANDLE, NULL);
        p_QmFqr->h_QmPortal = QmGetPortalHandle(p_QmFqr->h_Qm);
        SANITY_CHECK_RETURN_VALUE(p_QmFqr->h_QmPortal, E_INVALID_HANDLE, NULL);
    }

    p_QmFqr->fqidBase = QmFqidGet(p_QmFqr->h_Qm,
                                 p_QmFqr->numOfFqids,
                                 p_QmFqrParams->qs.nonFrcQs.align,
                                 p_QmFqrParams->useForce,
                                 p_QmFqrParams->qs.frcQ.fqid);
    if (p_QmFqr->fqidBase == ILLEGAL_BASE)
    {
        REPORT_ERROR(CRITICAL,E_INVALID_STATE,("can't allocate a fqid"));
        QM_FQR_Free(p_QmFqr);
        return NULL;
    }

    p_QmFqr->p_Fqs = (struct qman_fq **)XX_Malloc(sizeof(struct qman_fq *) * p_QmFqr->numOfFqids);
    if (!p_QmFqr->p_Fqs)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("QM FQs obj!!!"));
        QM_FQR_Free(p_QmFqr);
        return NULL;
    }
    memset(p_QmFqr->p_Fqs, 0, sizeof(struct qman_fq *) * p_QmFqr->numOfFqids);
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_CGE : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_TDE : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_ORP : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_CPCSTASH : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_FORCESFDR : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_AVOIDBLOCK : 0;
    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_HOLDACTIVE : 0;
//    flags |= (p_QmFqrParams->canBeParked) ? QM_FQCTRL_LOCKINCACHE : 0;

    if (p_QmFqrParams->useContextAForStash)
    {
        if (CheckStashParams(p_QmFqrParams) != E_OK)
        {
            QM_FQR_Free(p_QmFqr);
            return NULL;
        }

        memset(&cnxtA, 0, sizeof(cnxtA));
        cnxtA.stashing.annotation_cl = DIV_CEIL(p_QmFqrParams->stashingParams.frameAnnotationSize, CACHELINE_SIZE);
        cnxtA.stashing.data_cl = DIV_CEIL(p_QmFqrParams->stashingParams.frameDataSize, CACHELINE_SIZE);
        cnxtA.stashing.context_cl = DIV_CEIL(p_QmFqrParams->stashingParams.fqContextSize, CACHELINE_SIZE);
        cnxtA.context_hi = (uint8_t)((p_QmFqrParams->stashingParams.fqContextAddr >> 32) & 0xff);
        cnxtA.context_lo = (uint32_t)(p_QmFqrParams->stashingParams.fqContextAddr);
        flags |= QM_FQCTRL_CTXASTASHING;
    }

    for(i=0;i<p_QmFqr->numOfFqids;i++)
    if (qm_new_fq(p_QmFqr->h_QmPortal,
                  p_QmFqr->fqidBase+i,
                  i,
                  p_QmFqr->channel,
                  p_QmFqr->workQueue,
                  1/*p_QmFqr->numOfFqids*/,
                  flags,
                  p_QmFqrParams->useContextAForStash ?
                      (t_QmContextA *)&cnxtA :
                  p_QmFqrParams->p_ContextA,
                  p_QmFqrParams->p_ContextB,
                  p_QmFqr,
                  &p_QmFqr->p_Fqs[i]) != E_OK)
    {
        QM_FQR_Free(p_QmFqr);
        return NULL;
    }

    return p_QmFqr;
}

t_Error QM_FQR_Free(t_Handle h_QmFqr)
{
    t_QmFqr     *p_QmFqr    = (t_QmFqr *)h_QmFqr;
    uint32_t    i;

    if (!p_QmFqr)
        return ERROR_CODE(E_INVALID_HANDLE);

    if (p_QmFqr->p_Fqs)
    {
        for (i=0;i<p_QmFqr->numOfFqids;i++)
        {
            qm_free_fq(p_QmFqr->h_QmPortal, p_QmFqr->p_Fqs[i]);
            XX_FreeSmart(p_QmFqr->p_Fqs[i]);
        }
        XX_FreeSmart(p_QmFqr->p_Fqs);
    }

    if (p_QmFqr->fqidBase)
        QmFqidPut(p_QmFqr->h_Qm, p_QmFqr->fqidBase);

    XX_Free(p_QmFqr);

    return E_OK;
}

t_Error QM_FQR_RegisterCB(t_Handle h_QmFqr, t_QmReceivedFrameCallback *f_CallBack, t_Handle h_App)
{
    t_QmFqr     *p_QmFqr = (t_QmFqr *)h_QmFqr;
    int         i;

    SANITY_CHECK_RETURN_VALUE(p_QmFqr, E_INVALID_HANDLE, 0);

    for (i=0;i<p_QmFqr->numOfFqids;i++)
    {
        p_QmFqr->p_Fqs[i]->cb.dqrr = f_CallBack;
        p_QmFqr->p_Fqs[i]->h_App   = h_App;
    }

    return E_OK;
}

t_Error QM_FQR_Enqueue(t_Handle h_QmFqr, t_Handle h_QmPortal, uint32_t fqidOffset, t_DpaaFD *p_Frame)
{
    t_QmFqr     *p_QmFqr = (t_QmFqr *)h_QmFqr;

    SANITY_CHECK_RETURN_ERROR(p_QmFqr, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((fqidOffset < p_QmFqr->numOfFqids), E_INVALID_STATE);

    if (!h_QmPortal)
    {
        SANITY_CHECK_RETURN_ERROR(p_QmFqr->h_Qm, E_INVALID_HANDLE);
        h_QmPortal = QmGetPortalHandle(p_QmFqr->h_Qm);
        SANITY_CHECK_RETURN_ERROR(h_QmPortal, E_INVALID_HANDLE);
    }
        return  qman_enqueue(h_QmPortal,
                             p_QmFqr->p_Fqs[fqidOffset],
                             (struct qm_fd *)p_Frame, 0);
}

t_Error QM_FQR_Resume(t_Handle h_QmFqr, t_Handle h_QmPortal, uint32_t fqidOffset)
{
    t_QmFqr     *p_QmFqr = (t_QmFqr *)h_QmFqr;

    SANITY_CHECK_RETURN_ERROR(p_QmFqr, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((fqidOffset < p_QmFqr->numOfFqids), E_INVALID_STATE);

    if (!h_QmPortal)
    {
        SANITY_CHECK_RETURN_ERROR(p_QmFqr->h_Qm, E_INVALID_HANDLE);
        h_QmPortal = QmGetPortalHandle(p_QmFqr->h_Qm);
        SANITY_CHECK_RETURN_ERROR(h_QmPortal, E_INVALID_HANDLE);
    }
    return qman_schedule_fq(h_QmPortal, p_QmFqr->p_Fqs[fqidOffset]);
}

t_Error  QM_FQR_Suspend(t_Handle h_QmFqr, t_Handle h_QmPortal, uint32_t fqidOffset)
{
    t_QmFqr     *p_QmFqr = (t_QmFqr *)h_QmFqr;

    SANITY_CHECK_RETURN_ERROR(p_QmFqr, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((fqidOffset < p_QmFqr->numOfFqids), E_INVALID_STATE);

    UNUSED(h_QmPortal);
    p_QmFqr->p_Fqs[fqidOffset]->state = qman_fq_state_waiting_parked;

    return E_OK;
}

uint32_t QM_FQR_GetFqid(t_Handle h_QmFqr)
{
    t_QmFqr *p_QmFqr = (t_QmFqr *)h_QmFqr;

    return p_QmFqr->fqidBase;
}

t_Error QM_FQR_QueryFq_P(t_Handle h_QmPortal, uint32_t fqid, void *buf)
{
    t_QmPortal *p_QmPortal = (t_QmPortal *)h_QmPortal;
	struct qman_fq fq;

	fq.fqid = fqid;
	return qm_query_fq_p(p_QmPortal, &fq, (struct qm_mcr_queryfq *)buf);
}

t_Error QM_FQR_QueryFq_NP(t_Handle h_QmPortal, uint32_t fqid, void *buf)
{
    t_QmPortal *p_QmPortal = (t_QmPortal *)h_QmPortal;
	struct qman_fq fq;

	fq.fqid = fqid;
	return qm_query_fq_np(p_QmPortal, &fq, (struct qm_mcr_queryfq_np *)buf);
}

t_Error QM_FQR_QueryWq(t_Handle h_QmPortal, uint16_t wqid, void *buf)
{
    t_QmPortal *p_QmPortal = (t_QmPortal *)h_QmPortal;

	return qm_query_wq(p_QmPortal, wqid, (struct qm_mcr_querywq *)buf);
}
