/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          dec.c

 @Author        Shlomi Gridish (gridish@freescale.com)

 @Description   Routines for initialize the decrementer interrupt.
*//***************************************************************************/
#include "string_ext.h"
#include "error_ext.h"
#include "std_ext.h"
#include "xx_ext.h"
#include "core_ext.h"
#include "e500v2_ext.h"
#include "e500v2_asm_ext.h"

#include "dec.h"


static t_Dec *p_Decrementer[INTG_MAX_NUM_OF_CORES] = { 0 };


t_Handle DEC_ConfigAndInit(t_DecParam *p_DecParam)
{
    t_Dec   *p_Dec;

    SANITY_CHECK_RETURN_VALUE(p_DecParam, E_INVALID_HANDLE, NULL);

    p_Dec = (t_Dec *)XX_Malloc(sizeof(t_Dec));
    if (!p_Dec)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("DEC object!"));
        return NULL;
    }
    memset(p_Dec, 0, sizeof(t_Dec));

    p_Dec->sysClk = p_DecParam->sysClk;

    SetDec(0);

    p_Decrementer[CORE_GetId()] = p_Dec;
    return p_Dec;
}

t_Error DEC_Free(t_Handle h_Dec)
{
    t_Dec   *p_Dec = (t_Dec *)h_Dec;

    SANITY_CHECK_RETURN_ERROR(p_Dec, E_INVALID_HANDLE);

    DEC_Stop(h_Dec, 0);

    XX_Free(p_Dec);
    p_Decrementer[CORE_GetId()] = NULL;

    return E_OK;
}

t_Error DEC_Start(t_Handle      h_Dec,
                  uint8_t       id,
                  void          (*f_ExpiredCallback)(t_Handle h_App, uint8_t id),
                  t_Handle      h_App,
                  uint64_t      period,
                  e_TimerMode   mode)
{
    t_Dec   *p_Dec = (t_Dec *)h_Dec;

    SANITY_CHECK_RETURN_ERROR(p_Dec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((id == 0), E_INVALID_VALUE);
    SANITY_CHECK_RETURN_ERROR(f_ExpiredCallback, E_INVALID_HANDLE);

    p_Dec->f_ExpiredCallback = f_ExpiredCallback;
    p_Dec->h_App = h_App;
    p_Dec->mode = mode;
    p_Dec->isRunning = TRUE;

    if (p_Dec->mode == e_TIMER_MODE_PERIODIC)
    {
        if (period > MAX_PERIOD)
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("In periodic mode, the period cannot exceed %d", MAX_PERIOD));
        p_Dec->origReqPeriod = (uint32_t)period;
        p_Dec->curValue = USECS_TO_TICKS(p_Dec->origReqPeriod);

        SetDec(p_Dec->curValue);
    }
    else
        DEC_SetTime(p_Dec, id, period);

    EnableDec(p_Dec);

    return E_OK;
}

t_Error DEC_Stop(t_Handle          h_Dec,
                 uint8_t           id)
{
    t_Dec   *p_Dec = (t_Dec *)h_Dec;

#ifdef DISABLE_SANITY_CHECKS
    UNUSED(id);
#endif /* DISABLE_SANITY_CHECKS */
    SANITY_CHECK_RETURN_ERROR(p_Dec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((id == 0), E_INVALID_VALUE);

    DisableDec();
    SetDec(0);

    p_Dec->isRunning = FALSE;
    p_Dec->h_App = NULL;
    p_Dec->f_ExpiredCallback = NULL;
    p_Dec->mode = e_TIMER_MODE_INVALID;

    return E_OK;
}

t_Error DEC_SetTime(t_Handle          h_Dec,
                    uint8_t           id,
                    uint64_t          period)
{
    t_Dec   *p_Dec = (t_Dec *)h_Dec;
    uint32_t nextTimeInterval;
    uint32_t intFlags;

#ifdef DISABLE_SANITY_CHECKS
    UNUSED(id);
#endif /* DISABLE_SANITY_CHECKS */
    SANITY_CHECK_RETURN_ERROR(p_Dec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((id == 0), E_INVALID_VALUE);

    /* If the DEC is disabled, we can't change its values */
    if (!p_Dec->isRunning)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);

    p_Dec->origReqPeriod = (uint32_t)period;

    intFlags = XX_DisableAllIntr();
    if (p_Dec->origReqPeriod > MAX_PERIOD)
    {
        nextTimeInterval = MAX_PERIOD;
        p_Dec->timeLeft = p_Dec->origReqPeriod - MAX_PERIOD;
    }
    else
    {
        nextTimeInterval = p_Dec->origReqPeriod;
        p_Dec->timeLeft = 0;
    }

    /* Calculate the new decrementer value according to the
       required period */
    p_Dec->curValue = USECS_TO_TICKS(nextTimeInterval);

    /* Initialize the decrementer with the required value */
    /* If the request time has already passed, we are setting the dec to minimum
       so it will imidiatlly expire. */
    if ((int32_t)(p_Dec->curValue + (int32_t)GetDec()) <= 0)
        SetDec(MIN_DEC_VALUE);
    /* If the dec has already expired, we need to decrement the time that was passed. */
    else if ((int32_t)GetDec() < 0)
        SetDec((uint32_t)(p_Dec->curValue + (int32_t)GetDec()));
    else
        SetDec(p_Dec->curValue);
    XX_RestoreAllIntr(intFlags);

    return E_OK;
}

t_Error DEC_GetTime(t_Handle          h_Dec,
                    uint8_t           id,
                    uint64_t          *p_Period)
{
    t_Dec   *p_Dec = (t_Dec *)h_Dec;

#ifdef DISABLE_SANITY_CHECKS
    UNUSED(id);
#endif /* DISABLE_SANITY_CHECKS */
    SANITY_CHECK_RETURN_ERROR(p_Dec, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((id == 0), E_INVALID_VALUE);

    /* If the DEC is disabled, we can't get any value */
    if (!p_Dec->isRunning)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);

    *p_Period = ((p_Dec->origReqPeriod > MAX_PERIOD) ?
                  ((p_Dec->origReqPeriod - p_Dec->timeLeft - TICKS_TO_USECS(p_Dec->curValue)) +
                   TICKS_TO_USECS(p_Dec->curValue - (int32_t)GetDec())) :
                 TICKS_TO_USECS(p_Dec->curValue - (int32_t)GetDec()));

    return E_OK;
}

void DEC_Intr(void)
{
    t_Dec    *p_Dec = p_Decrementer[CORE_GetId()];
    uint32_t tmpTsr;

    SANITY_CHECK_RETURN(p_Dec, E_INVALID_HANDLE);

    if (!p_Dec || !p_Dec->isRunning)
        return;

    if (p_Dec->timeLeft == 0)
    {
        if (p_Dec->f_ExpiredCallback)
        {
            /* Call to the user callback routine */
            p_Dec->f_ExpiredCallback(p_Dec->h_App, 0);
            if (p_Dec->mode == e_TIMER_MODE_SINGLE)
                DEC_Stop(p_Dec, 0);
        }
    }
    else
    {
        uint32_t nextTimeInterval;

        if (p_Dec->timeLeft > MAX_PERIOD)
        {
            nextTimeInterval = MAX_PERIOD;
            p_Dec->timeLeft -= MAX_PERIOD;
        }
        else
        {
            nextTimeInterval = p_Dec->timeLeft;
            p_Dec->timeLeft = 0;
        }

        p_Dec->curValue = USECS_TO_TICKS(nextTimeInterval);

        /* Initialize the decrementer with the required value. */
        /* If the request time has already passed, we are setting the dec to minimum
           so it will imidiatlly expire. */
        if ((int32_t)(p_Dec->curValue + (int32_t)GetDec()) <= 0)
            SetDec(MIN_DEC_VALUE);
        /* If the dec has already expired, we need to decrement the time that was passed. */
        else if ((int32_t)GetDec() < 0)
            SetDec((uint32_t)(p_Dec->curValue + (int32_t)GetDec()));
        else
            SetDec(p_Dec->curValue);
    }

    tmpTsr = E500_GetSPR_TSR();
    tmpTsr |= TSR_DIS;          /* Re enable Interrupt */
    E500_SetSPR_TSR(tmpTsr);
}
