/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          dec.h

 @Author        Shlomi Gridish (gridish@freescale.com)

 @Description   Header file for Decrementer.
*//***************************************************************************/
#ifndef __DEC_H
#define __DEC_H

#include "error_ext.h"
#include "std_ext.h"
#include "e500v2_asm_ext.h"
#include "dec_ext.h"


#define __ERR_MODULE__  MODULE_UNKNOWN


#define TICKS_TO_USECS(ticks)   (uint32_t)(((ticks)<<3)/p_Dec->sysClk)
#define USECS_TO_TICKS(usecs)   (uint32_t)((p_Dec->sysClk>>3)*(usecs))

#define MAX_DEC_VALUE 0x0fffffff
#define MIN_DEC_VALUE 0x00000010

/* calculate the minimum period between two decrementer
   interrupts according to SYSTEM_CLOCK.
   The calculation is:
   the decrementer is decremented once every eight ccb_clock
   therefore - the time for decrement one from the DEC is 1/((CCB_CLOCK/8)*10^6(mega))
*/
#define MIN_PERIOD              MAX(TICKS_TO_USECS(MIN_DEC_VALUE), 1)
/* calculate the maximum period between two decrementer
   interrupts according to SYSTEM_CLOCK.
*/
#define MAX_PERIOD              TICKS_TO_USECS(MAX_DEC_VALUE)


typedef struct t_Dec
{
    t_Handle    h_App;
    void        (*f_ExpiredCallback)(t_Handle h_App, uint8_t id);
    uint32_t    sysClk;
    e_TimerMode mode;
    bool        isRunning;
    uint32_t    timeLeft;
    uint32_t    curValue;
    uint32_t    origReqPeriod;
} t_Dec;


static __inline__ void SetDec(uint32_t val)
{
    E500_SetSPR_DEC(val);
    E500_SetSPR_DECAR(val);
}

static __inline__ void EnableDec(t_Dec *p_Dec)
{
    uint32_t tmpTcr;
    uint32_t tmpTsr;
#ifndef CORE_E500MC
    uint32_t tmpHid0;
#endif /* CORE_E500MC */

    tmpTsr = E500_GetSPR_TSR();
    tmpTsr |= TSR_DIS;
    E500_SetSPR_TSR(tmpTsr);

    tmpTcr = E500_GetSPR_TCR();
    tmpTcr |= TCR_DIE;
    if (p_Dec->mode == e_TIMER_MODE_PERIODIC)
        tmpTcr |= TCR_ARE;
    E500_SetSPR_TCR(tmpTcr);

#ifndef CORE_E500MC
    tmpHid0 = E500_GetSPR_HID0();
    tmpHid0 |= HID0_TBEN;
    tmpHid0 &=  ~HID0_SEL_TBCLK;                    /* CLK source = 1/8 of CCB Clock */
    E500_SetSPR_HID0(tmpHid0);
#endif /* CORE_E500MC */
}

static __inline__ void DisableDec()
{
    uint32_t tmpTcr;
#ifndef CORE_E500MC
    uint32_t tmpHid0;
#endif /* CORE_E500MC */

    tmpTcr = E500_GetSPR_TCR() & ~TCR_DIE;
    E500_SetSPR_TCR(tmpTcr);

#ifndef CORE_E500MC
    tmpHid0 = E500_GetSPR_HID0() & ~HID0_TBEN;
    E500_SetSPR_HID0(tmpHid0);
#endif /* CORE_E500MC */
}


/**************************************************************************//**
 @Function      DEC_Intr

 @Description   Set the decrementer value and call to the user call back routiune.

                The routine set the decrementer value according to
                the required period between two decrementer interrupts
                and call to the user callback routine.

                NOTE: The period is a global value

@Cautions       The routine is called in case of decrementer interrupt.
*//***************************************************************************/
void DEC_Intr(void);


#endif /* __DEC_H */
