/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*
 *
 * @File          e500v2_intr.c
 *
 * @Description   This is an E500 specific implementation.
 *                This file contain the PPC general interrup routines.
 *                The functions in this file export a general INT Service routine.
 *
 *
 * @Cautions
 *
 *
 * Author: Donio Ron .
 *
 */

#include "std_ext.h"
#include "error_ext.h"
#include "string_ext.h"
#include "xx_ext.h"
#include "e500v2_ext.h"
#include "dec_ext.h"
#include "e500v2_asm_ext.h"

/*-----------------------------------------------------
*
* Local definitions from vector to index xx00->xx
*------------------------------------------------------*/
#define CRITICAL_INTR_INDEX             ((CRITICAL_INTR&0xFF00)>>8)
#define MACHINE_CHECK_INTR_INDEX        ((MACHINE_CHECK_INTR&0xFF00)>>8)
#define DATA_STORAGE_INTR_INDEX         ((DATA_STORAGE_INTR&0xFF00)>>8)
#define INSTRUCTION_STORAGE_INTR_INDEX  ((INSTRUCTION_STORAGE_INTR&0xFF00)>>8)
#define EXTERNAL_INTR_INDEX             ((EXTERNAL_INTR&0xFF00)>>8)
#define ALIGNMENT_INTR_INDEX            ((ALIGNMENT_INTR&0xFF00)>>8)
#define PROGRAM_INTR_INDEX              ((PROGRAM_INTR&0xFF00)>>8)
#define FLOATING_POINT_INTR_INDEX       ((FLOATING_POINT_INTR&0xFF00)>>8)
#define SYSTEM_CALL_INTR_INDEX          ((SYSTEM_CALL_INTR&0xFF00)>>8)
#define AUXILIARY_INTR_INDEX            ((AUXILIARY_INTR&0xFF00)>>8)
#define DECREMENTER_INTR_INDEX          ((DECREMENTER_INTR&0xFF00)>>8)
#define FIXED_INTERVAL_INTR_INDEX       ((FIXED_INTERVAL_INTR&0xFF00)>>8)
#define WATCHDOG_TIMER_INTR_INDEX       ((WATCHDOG_TIMER_INTR&0xFF00)>>8)
#define D_TLB_ERROR_INTR_INDEX          ((D_TLB_ERROR_INTR&0xFF00)>>8)
#define I_TLB_ERROR_INTR_INDEX          ((I_TLB_ERROR_INTR&0xFF00)>>8)
#define DEBUG_INTR_INDEX                ((DEBUG_INTR&0xFF00)>>8)
#define SPE_APU_INTR_INDEX              ((SPE_APU_INTR&0xFF00)>>8)
#define SPE_FLT_DATA_INTR_INDEX         ((SPE_FLT_DATA_INTR&0xFF00)>>8)
#define SPE_FLT_ROUND_INTR_INDEX        ((SPE_FLT_ROUND_INTR&0xFF00)>>8)
#define PERF_MONITOR_INTR_INDEX         ((PERF_MONITOR_INTR&0xFF00)>>8)

/******************************************************
*   Structure:      t_E500IntrTable
*
*   Description:    Hold all E500 callback functions
*******************************************************/
typedef struct
{
    void            (* f_Isr)(t_Handle);
                                /**< Interupt handler */
    t_Handle        srcHandle;  /**< Parameter to pass to interupt handler */
    bool            active;     /* True if f_Isr active */
} t_E500IntrSrcTable;


void E500_InitInterruptVector(void);

static t_E500IntrSrcTable t_E500IntrTable[LAST_INTR_INDEX];


/*---------------------------------------------------------*/
/*  Install A PPC Interrupt vector routine                 */
/*---------------------------------------------------------*/
t_Error E500_SetIntr(int ppcIntrSrc, void (* Isr)(t_Handle Handle), t_Handle Handle )
{
    int index = (ppcIntrSrc & 0xFF00) >> 8; /* Numbered from 0x100-->0x1900 */

    t_E500IntrTable[index].f_Isr =  Isr;
    t_E500IntrTable[index].srcHandle = Handle;
    t_E500IntrTable[index].active = TRUE;

    return (E_OK);
}

/*---------------------------------------------------------*/
/*  Clear A PPC Interrupt vector routine                   */
/*---------------------------------------------------------*/
t_Error E500_ClearIntr(int ppcIntrSrc)
{
    int index = (ppcIntrSrc & 0xFF00) >> 8; /* Numbered from 0x100-->0x1900 */

    t_E500IntrTable[index].active = FALSE;
    return (E_OK);
}


/*****************************************************************/
/* routine:       E500_GenericIntrInit                           */
/*                                                               */
/* description:                                                  */
/*         On the E500 the Ivor's need to initialize to point to */
/*         right address.                                        */
/* arguments:                                                    */
/*    None.                                                      */
/*                                                               */
/*****************************************************************/
void E500_GenericIntrInit(void)
{
    memset(t_E500IntrTable, 0, sizeof(t_E500IntrTable));

    E500_InitInterruptVector();
}


/*****************************************************************/
/* routine:       CriticalIntr                                        */
/*                                                               */
/* description:                                                  */
/*    Internal routine, called by the critical interrupt handler */
/*    to indicate the ocurrance of an critical INT when it is not*/
/*    handled by the QE directly                                 */
/*                                                               */
/* arguments:                                                    */
/*    intrEntry (In) - The interrupt handler original address.   */
/*                                                               */
/*****************************************************************/
void CriticalIntr(uint32_t intrEntry)
{
    if (intrEntry != CRITICAL_INTR)
    {
        XX_Print("Core Int: Wrong call to CRITICAL Int. Handler\n");
        return;
    }

    if (t_E500IntrTable[CRITICAL_INTR_INDEX].active)
        t_E500IntrTable[CRITICAL_INTR_INDEX].f_Isr(t_E500IntrTable[CRITICAL_INTR_INDEX].srcHandle);
    else
        XX_Print("Core Int: CRITICAL\n");

}

/*****************************************************************/
/* routine:       E500_GenericIntrHandler                          */
/*                                                               */
/* description:                                                  */
/*    Internal routine, called by the main interrupt handler     */
/*    to indicate the ocurrance of an INT.                       */
/*                                                               */
/* arguments:                                                    */
/*    intrEntry (In) - The interrupt handler original address.   */
/*                                                               */
/*****************************************************************/
void E500_GenericIntrHandler(uint32_t intrEntry)
{
    switch(intrEntry)
    {
    case(CRITICAL_INTR):                                /* IVOR0 100 */
        if (t_E500IntrTable[CRITICAL_INTR_INDEX].active)
            t_E500IntrTable[CRITICAL_INTR_INDEX].f_Isr(t_E500IntrTable[CRITICAL_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: CRITICAL\n");
        break;
    case(MACHINE_CHECK_INTR):                           /* IVOR1 200 */
        if (t_E500IntrTable[MACHINE_CHECK_INTR_INDEX].active)
            t_E500IntrTable[MACHINE_CHECK_INTR_INDEX].f_Isr(t_E500IntrTable[MACHINE_CHECK_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: MACHINE_CHECK\n");
        break;
    case(DATA_STORAGE_INTR):                            /* IVOR2 300 */
        if (t_E500IntrTable[DATA_STORAGE_INTR_INDEX].active)
            t_E500IntrTable[DATA_STORAGE_INTR_INDEX].f_Isr(t_E500IntrTable[DATA_STORAGE_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: DATA_STORAGE\n");
        break;
    case(INSTRUCTION_STORAGE_INTR):                     /* IVOR3 400 */
        if (t_E500IntrTable[INSTRUCTION_STORAGE_INTR_INDEX].active)
            t_E500IntrTable[INSTRUCTION_STORAGE_INTR_INDEX].f_Isr(t_E500IntrTable[INSTRUCTION_STORAGE_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: INSTRUCTION_STORAGE\n");
        break;
    case(EXTERNAL_INTR):                                /* IVOR4 500 */
        if (t_E500IntrTable[EXTERNAL_INTR_INDEX].active)
            t_E500IntrTable[EXTERNAL_INTR_INDEX].f_Isr(t_E500IntrTable[EXTERNAL_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: EXTERNAL\n");
        break;
    case(ALIGNMENT_INTR):                               /* IVOR5 600 */
        if (t_E500IntrTable[ALIGNMENT_INTR_INDEX].active)
            t_E500IntrTable[ALIGNMENT_INTR_INDEX].f_Isr(t_E500IntrTable[ALIGNMENT_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: ALIGNMENT\n");
        break;
    case(PROGRAM_INTR):                                 /* IVOR6 700 */
        if (t_E500IntrTable[PROGRAM_INTR_INDEX].active)
            t_E500IntrTable[PROGRAM_INTR_INDEX].f_Isr(t_E500IntrTable[PROGRAM_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: PROGRAM\n");
        break;
    case(FLOATING_POINT_INTR):                          /* IVOR7 800 */
        if (t_E500IntrTable[FLOATING_POINT_INTR_INDEX].active)
            t_E500IntrTable[FLOATING_POINT_INTR_INDEX].f_Isr(t_E500IntrTable[FLOATING_POINT_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: FLOATING_POINT\n");
        break;
    case(SYSTEM_CALL_INTR):                             /* IVOR8 900 */
        if (t_E500IntrTable[SYSTEM_CALL_INTR_INDEX].active)
            t_E500IntrTable[SYSTEM_CALL_INTR_INDEX].f_Isr(t_E500IntrTable[SYSTEM_CALL_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: SYSTEM CALL\n");
        break;
    case(AUXILIARY_INTR):                               /* IVOR9 A00 */
        if (t_E500IntrTable[AUXILIARY_INTR_INDEX].active)
            t_E500IntrTable[AUXILIARY_INTR_INDEX].f_Isr(t_E500IntrTable[AUXILIARY_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: Auxiliary processor unavailable\n");
        break;
    case(DECREMENTER_INTR):                             /* IVOR10 1000 */
        if (t_E500IntrTable[DECREMENTER_INTR_INDEX].active)
            t_E500IntrTable[DECREMENTER_INTR_INDEX].f_Isr(t_E500IntrTable[DECREMENTER_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: DECREMENTER\n");
        break;
    case(FIXED_INTERVAL_INTR):                          /* IVOR11 1100 */
        if (t_E500IntrTable[FIXED_INTERVAL_INTR_INDEX].active)
            t_E500IntrTable[FIXED_INTERVAL_INTR_INDEX].f_Isr(t_E500IntrTable[FIXED_INTERVAL_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: fixed-interval timer\n");
        break;
    case(WATCHDOG_TIMER_INTR):                          /* IVOR12 1200 */
        if (t_E500IntrTable[WATCHDOG_TIMER_INTR_INDEX].active)
            t_E500IntrTable[WATCHDOG_TIMER_INTR_INDEX].f_Isr(t_E500IntrTable[WATCHDOG_TIMER_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: watchdog timer\n");
        break;
    case(D_TLB_ERROR_INTR):                             /* IVOR13 1300 */
        if (t_E500IntrTable[D_TLB_ERROR_INTR_INDEX].active)
            t_E500IntrTable[D_TLB_ERROR_INTR_INDEX].f_Isr(t_E500IntrTable[D_TLB_ERROR_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: data TLB error\n");
        break;
    case(I_TLB_ERROR_INTR):                             /* IVOR14 1400 */
        if (t_E500IntrTable[I_TLB_ERROR_INTR_INDEX].active)
            t_E500IntrTable[I_TLB_ERROR_INTR_INDEX].f_Isr(t_E500IntrTable[I_TLB_ERROR_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: instruction TLB error\n");
        break;
    case(DEBUG_INTR):                                   /* IVOR15 1500 */
        if (t_E500IntrTable[DEBUG_INTR_INDEX].active)
            t_E500IntrTable[DEBUG_INTR_INDEX].f_Isr(t_E500IntrTable[DEBUG_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: debug\n");
        break;
    case(SPE_APU_INTR):                                 /* IVOR32 1600 */
        if (t_E500IntrTable[SPE_APU_INTR_INDEX].active)
            t_E500IntrTable[SPE_APU_INTR_INDEX].f_Isr(t_E500IntrTable[SPE_APU_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: SPE-APU unavailable\n");
        break;
    case(SPE_FLT_DATA_INTR):                            /* IVOR33 1700 */
        if (t_E500IntrTable[SPE_FLT_DATA_INTR_INDEX].active)
            t_E500IntrTable[SPE_FLT_DATA_INTR_INDEX].f_Isr(t_E500IntrTable[SPE_FLT_DATA_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: SPE-floating point data\n");
        break;
    case(SPE_FLT_ROUND_INTR):                           /* IVOR34 1800 */
        if (t_E500IntrTable[SPE_FLT_ROUND_INTR_INDEX].active)
            t_E500IntrTable[SPE_FLT_ROUND_INTR_INDEX].f_Isr(t_E500IntrTable[SPE_FLT_ROUND_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: SPE-floating point round\n");
        break;
    case(PERF_MONITOR_INTR):                            /* IVOR35 1900 */
        if (t_E500IntrTable[PERF_MONITOR_INTR_INDEX].active)
            t_E500IntrTable[PERF_MONITOR_INTR_INDEX].f_Isr(t_E500IntrTable[PERF_MONITOR_INTR_INDEX].srcHandle);
        else
            XX_Print("Core Int: performance monitor\n");
        break;
    default:
        XX_Print("e500 undefined Interrupt #%x\n", intrEntry);
        break;
    }
}
