/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "part_ext.h"
#include "e500v2_L2_ext.h"
#include "e500v2_l2.h"
#include "e500v2_asm_ext.h"
#include "xx_ext.h"
#include "string_ext.h"


/* Forward Functions */
static bool IS_CACHE_MODE(e_L2Mode l2Mode, e_L2Stash   l2Stash);
static bool IS_SRAM_MODE(e_L2Mode l2Mode);

static void L2_Intr(t_Handle h_L2);


/*------------------------------*/
/* initilization unit routines  */
/*------------------------------*/

/******************************************************
*   Routine:        L2_Config
*
*   Description:    Creation of L2 object and Configuration
*                   of L2 registers. This routine must be
*                   called befor any other L2 routine may be
*                   called.
*
*
*
*   Arguments:      p_L2Param - (In) A structure of parameters for the L2, as
*                               defined at e500v1_L2_ext.h.
*
*                   baseAddr -  (In) Base Address of L2 memory mapped registers.
*
*   Return Value    Id handle to the initialized L2 object.
********************************************************/
t_Handle L2_Config(t_L2Param *p_L2Param)
{
    t_L2    *p_L2;
    int     i;

    SANITY_CHECK_RETURN_VALUE(p_L2Param, E_NULL_POINTER, NULL);

    /* initialize L2 object */
    p_L2 = (t_L2 *)XX_Malloc(sizeof(t_L2));
    if (!p_L2)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("L2_CACHE driver structure"));
        return NULL;
    }
    memset(p_L2, 0, sizeof(t_L2));

    /* allocate memory for the parameter structures */
    p_L2->p_DriverParam = (t_L2DriverParam *)XX_Malloc(sizeof(t_L2DriverParam));
    if (!p_L2->p_DriverParam)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("L2 driver parameters"));
        XX_Free(p_L2);
        return NULL;
    }
    memset(p_L2->p_DriverParam, 0, sizeof(t_L2DriverParam));

    for (i=0; i < NUMBER_STASH_REGIONS; i++)
        p_L2->L2RegionAllocated[i] = FALSE;

    p_L2->p_MemMap          = (t_L2MemMap *)p_L2Param->baseAddress;
    p_L2->irq               = p_L2Param->irq;
    p_L2->cacheRegionCurr   = 0;

    /* Copy paramters */
    p_L2->p_DriverParam->l2Active                   = p_L2Param->l2Active;
    p_L2->p_DriverParam->l2Mode                     = p_L2Param->l2Mode;
    p_L2->p_DriverParam->l2Stash                    = p_L2Param->l2Stash;
    p_L2->p_DriverParam->dataOnly                   = p_L2Param->dataOnly;
    p_L2->p_DriverParam->instructionOnly            = p_L2Param->instructionOnly;
    p_L2->p_DriverParam->cacheInterventionDisable   = p_L2Param->cacheInterventionDisable;
    p_L2->p_DriverParam->sramBaseAddrBlock0         = p_L2Param->sramBaseAddrBlock0;
    p_L2->p_DriverParam->sramBaseAddrBlock1         = p_L2Param->sramBaseAddrBlock1;

    return p_L2;
}


t_Error L2_Init(t_Handle h_L2)
{
    t_L2            *p_L2 = (t_L2 *)h_L2;
    t_L2DriverParam *p_DriverParam;
    t_L2MemMap      *p_MemMap;
    uint32_t        tmpL2ctl, l2Size=0, sramBlockSize=0;
    int             i;
    bool            validBase1 = FALSE;

    SANITY_CHECK_RETURN_ERROR(p_L2, E_INVALID_HANDLE);

    p_DriverParam = p_L2->p_DriverParam;      /* Get Parameter pointer */
    p_MemMap      = p_L2->p_MemMap;         /* Get MemMap pointer */

    /* Disable L2 SRAM/Cache and Invalidate it */
    tmpL2ctl = GET_UINT32(p_MemMap->l2ctl);
    tmpL2ctl &= L2CTL_L2SIZ_MASK;             /* Mask only Size part */
    tmpL2ctl  |= L2CTL_L2I;                   /* Flash Invalidate Bit */
    tmpL2ctl = L1CtlWrite((uint32_t)&(p_MemMap->l2ctl), tmpL2ctl);

    do
    {
       tmpL2ctl = GET_UINT32(p_MemMap->l2ctl);
    } while (tmpL2ctl & L2CTL_L2I);

    if (!p_DriverParam->l2Active) goto l2_done;     /* L2 CACHE/SRAM not Active */

    tmpL2ctl = GET_UINT32(p_MemMap->l2ctl);
    tmpL2ctl &= L2CTL_L2SIZ_MASK;             /* Mask only Size part */

    if (tmpL2ctl == L2CTL_L2BLKSZ_1024)
        l2Size = 0x100000;
    else if (tmpL2ctl == L2CTL_L2BLKSZ_512)
        l2Size = 0x80000;
    else if (tmpL2ctl == L2CTL_L2BLKSZ_256)
        l2Size = 0x40000;

    tmpL2ctl |= (p_DriverParam->l2Mode << L2CTL_MODE_SHIFT);

    if (p_DriverParam->l2Stash != e_L2_L2CTL_NO_STASH)
    {
        tmpL2ctl |= (p_DriverParam->l2Stash << L2CTL_STASH_SHIFT);
        tmpL2ctl &= ~L2CTL_L2STASHDIS;
    }

    /* If at least one block is cache, cache parameters must be configured */
    if (IS_CACHE_MODE(p_DriverParam->l2Mode, p_DriverParam->l2Stash))
    {
        if (p_DriverParam->dataOnly)
            tmpL2ctl |= L2CTL_L2DO;
        if (p_DriverParam->instructionOnly)
            tmpL2ctl |= L2CTL_L2IO;
        if (p_DriverParam->cacheInterventionDisable)
            tmpL2ctl |= L2CTL_L2INTDIS;
    }

    for (i=0; i<4; i++)
        WRITE_UINT32(p_MemMap->l2_ext_write_regs[i].l2cewcr, 0);

    /* If at least one block is SRAM, SRAM parameters must be configured. */
    if (IS_SRAM_MODE(p_DriverParam->l2Mode))
    {
        switch (p_DriverParam->l2Mode){
	        case( e_L2_L2CTL_NO_SRAM):
                sramBlockSize = 0;
                break;
            case(e_L2_L2CTL_FULL_SRAM):
                sramBlockSize = l2Size;
                break;

            case(e_L2_L2CTL_2_HALVES_SRAM):
                validBase1 = TRUE;
            case(e_L2_L2CTL_1_HALF_SRAM):
                sramBlockSize = (l2Size / 2);
                break;

            case(e_L2_L2CTL_2_QUARTERS_SRAM):
                validBase1 = TRUE;
            case(e_L2_L2CTL_1_QUARTER_SRAM):
                sramBlockSize = (l2Size / 4);
                break;

            case(e_L2_L2CTL_2_EIGHTHS_SRAM):
                validBase1 = TRUE;
            case(e_L2_L2CTL_1_EIGHTH_SRAM):
                sramBlockSize = (l2Size / 8);
                break;
        }
    }

    if (p_DriverParam->sramBaseAddrBlock0 & (sramBlockSize - 1))
    {
        RETURN_ERROR(MAJOR, E_NOT_ALIGNED, ("SRAM base address is unaligned to %dKB block size", (sramBlockSize/1024)));
    }
    if (validBase1)
    {
        if (p_DriverParam->sramBaseAddrBlock1 & (sramBlockSize - 1))
        {
            RETURN_ERROR(MAJOR, E_NOT_ALIGNED, ("SRAM base address is unaligned to %dKB block size", (sramBlockSize/1024)));
        }
        WRITE_UINT32(p_MemMap->l2srbar1,   (uint32_t)(p_DriverParam->sramBaseAddrBlock1  & LL_ADDRESS_LOW_32_BIT_MASK));
        WRITE_UINT32(p_MemMap->l2srbarea1, (uint32_t)((p_DriverParam->sramBaseAddrBlock1 & LL_ADDRESS_EXT_4_BIT_MASK)>>32));
    }
    WRITE_UINT32(p_MemMap->l2srbar0,   (uint32_t)(p_DriverParam->sramBaseAddrBlock0  & LL_ADDRESS_LOW_32_BIT_MASK));
    WRITE_UINT32(p_MemMap->l2srbarea0, (uint32_t)((p_DriverParam->sramBaseAddrBlock0 & LL_ADDRESS_EXT_4_BIT_MASK)>>32));

    /* Enable all interrupts */
    WRITE_UINT32(p_MemMap->l2errdet, L2ERRDET_ALL);
    WRITE_UINT32(p_MemMap->l2errinten, L2ERRINTEN_ALL);

    tmpL2ctl  |= L2CTL_L2E;                 /* Set Enable Bit */

    tmpL2ctl = L1CtlWrite((uint32_t)&(p_MemMap->l2ctl), tmpL2ctl);

    /* It is mandatory to write to SRAM region prior to use. */
    if (IS_SRAM_MODE(p_DriverParam->l2Mode))
    {
        /* Disable ECC errors check prior to SRAM configuration */
        WRITE_UINT32(p_MemMap->l2errdis, 0xC);

        WRITE_BLOCK(CAST_UINT64_TO_POINTER(p_DriverParam->sramBaseAddrBlock0), 0, sramBlockSize);
        if (validBase1)
            WRITE_BLOCK(CAST_UINT64_TO_POINTER(p_DriverParam->sramBaseAddrBlock1), 0, sramBlockSize);
    }

    /* Now ECC errors check can be enabled. */
    WRITE_UINT32(p_MemMap->l2errdis, 0);

    /* Register and enable L2 interrupts on all error types */
    XX_SetIntr(p_L2->irq, L2_Intr, p_L2);
    XX_EnableIntr(p_L2->irq);

l2_done:
    /* Wrap Up */
    XX_Free(p_L2->p_DriverParam);
    p_L2->p_DriverParam = NULL;

    return E_OK;
}


t_Error L2_Free(t_Handle h_L2)
{
    t_L2             *p_L2 = (t_L2 *)h_L2;
    t_L2MemMap       *p_MemMap;
    uint32_t         tmpL2Ctl;

    SANITY_CHECK_RETURN_ERROR(p_L2, E_INVALID_HANDLE);

    p_MemMap      = p_L2->p_MemMap;         /* Get MemMap pointer */

     /* disable L2 interrupts on all error types */
    XX_DisableIntr(p_L2->irq);

     /* Set controller as not active */
    tmpL2Ctl = ~L2CTL_L2E & GET_UINT32(p_MemMap->l2ctl);
    tmpL2Ctl = L1CtlWrite((uint32_t)&(p_MemMap->l2ctl), tmpL2Ctl );

    XX_FreeIntr(p_L2->irq);

    XX_Free(p_L2);

    return E_OK;
}

/*------------------------------------------------------------------------*/
/*                          Error Init and Control routines               */
/*------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*                          Cache Lock/Unlock/Invalidate Routines               */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------*/
/*                          Exception unit                                */
/*------------------------------------------------------------------------*/
static void L2_Intr(t_Handle h_L2)
{
    t_L2        *p_L2 = (t_L2 *)h_L2;
    t_L2MemMap  *p_MemMap;
    uint32_t    tmpErrDetect;

    p_MemMap = p_L2->p_MemMap;

    /* check if error occured */
    tmpErrDetect = GET_UINT32(p_MemMap->l2errdet);

    /* clear events */
    WRITE_UINT32(p_L2->p_MemMap->l2errdet, tmpErrDetect);
}


/*------------------------------------------------------------------------*/
/*                  Local Functions                                       */
/*------------------------------------------------------------------------*/
static bool IS_CACHE_MODE(e_L2Mode l2Mode, e_L2Stash l2Stash)
{
    uint32_t    nonCacheEighths = 0;

    switch (l2Mode)
    {
        case e_L2_L2CTL_NO_SRAM:         nonCacheEighths  = 0; break;
        case e_L2_L2CTL_1_HALF_SRAM:     nonCacheEighths  = 4; break;
        case e_L2_L2CTL_1_QUARTER_SRAM:  nonCacheEighths  = 1; break;
        case e_L2_L2CTL_2_QUARTERS_SRAM: nonCacheEighths  = 2; break;
        case e_L2_L2CTL_1_EIGHTH_SRAM:   nonCacheEighths  = 1; break;
        case e_L2_L2CTL_2_EIGHTHS_SRAM:  nonCacheEighths  = 2; break;
        case e_L2_L2CTL_FULL_SRAM:       nonCacheEighths  = 8; break;
        case e_L2_L2CTL_2_HALVES_SRAM:   nonCacheEighths  = 8; break;
    }

    switch (l2Stash)
    {
        case e_L2_L2CTL_NO_STASH:                              break;
        case e_L2_L2CTL_HALF_STASH:      nonCacheEighths += 4; break;
        case e_L2_L2CTL_QUARTER_STASH:   nonCacheEighths += 2; break;
        case e_L2_L2CTL_EIGHTH_STASH:    nonCacheEighths += 1; break;
    }

    return (bool)(nonCacheEighths < 8);
}

static bool IS_SRAM_MODE(e_L2Mode l2Mode)
{
    return (bool)(l2Mode != e_L2_L2CTL_NO_SRAM);
}




t_Error L2_MapStashRegion(t_Handle      h_L2,
                          uint8_t       regionId,
                          uint32_t      baseAddr,
                          e_L2StashSize size,
                          bool          lock)
{
    t_L2        *p_L2 = (t_L2 *)h_L2;
    t_L2MemMap  *p_MemMap;
    uint32_t    tmpL2cewcr;
    uint32_t    intFlags;

    SANITY_CHECK_RETURN_ERROR(p_L2, E_INVALID_HANDLE);

    p_MemMap = p_L2->p_MemMap;


    /* check that Address is aligned to region size */
    if (baseAddr & (!((uint32_t)size << 4)))
    {
        REPORT_ERROR(MAJOR, E_INVALID_ADDRESS, ("L2_CacheRegionConfig: Address is not aligned to region size."));
        return 0;
    }

    if (lock)
        tmpL2cewcr =  L2CEWCR_LOCK;
    else
        tmpL2cewcr =  0;

    intFlags = XX_DisableAllIntr();

    /* configure region Address */
    WRITE_UINT32(p_L2->p_MemMap->l2_ext_write_regs[regionId].l2cewar, baseAddr);

    /* configure region control register */
    WRITE_UINT32(p_L2->p_MemMap->l2_ext_write_regs[regionId].l2cewcr, size | tmpL2cewcr | L2CEWCR_EN);

    XX_RestoreAllIntr(intFlags);

    return E_OK;
}

