/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*******************************************************************
*
*
* Original Author: Ganit Prisant
* re written for E500V2 : Donio Ron
*
* The e500 core has the following features:
* Please read PowerPC e500 Core Family Reference Manual, Rev. 1
*
* Memory Management Units are orgenized in types of TLBs:
*
* 16 TLB1 4KB->256 MB (V1) 4GB (v2) pages.
* 256 or 512 TLB0 of 4-Kbyte pages.
* The level 2 MMU has the following features:
*
*
* Current implementation only TLB1 is fully tested.
* In Embedded virtual address = physical addressapplication 16 TLB's should satisfy the requirment.
*
*   Initialization unit routines:
                    MMU_Identify
*                   MMU_TlbEntryWrite
*                   MMU_Tlb1EntryRead
*                   MMU_Tlb0EntryRead Eliminted for now
*                   MMU_Tlb1EntryPrint
*                   MMU_Tlb1EntryPrint
*
*************************************************************************************/

#include "error_ext.h"
#include "std_ext.h"
#include "e500v2_mmu_ext.h"
#include "e500v2_mmu.h"
#include "xx_ext.h"
#include "string_ext.h"
#include "sprint_ext.h"
#include "e500v2_ext.h"
#include "e500v2_asm_ext.h"


static char *MMU_Page2Size(e_MmuPageSize mmupage);


/*------------------------------------------
 * Global Functions
 *------------------------------------------
 */


/************************************************************************************
*   Routine:        MMU_Identify
*
*   Description:    Read and return a structure with MMU Identification information.
*
*   Arguments       p_MmuIdent - (Out) a structure of parameters for the
*                                       identification of the MMU
*                                       as described in mmu_ext.h.
*   Return          None
*
**************************************************************************************
*/
void MMU_Identify(t_MmuIdent *p_MmuIdent)
{
uint32_t mmucfg, tlb0cfg, tlb1cfg;

    mmucfg  = E500_GetSPR_MMUCFG();
    tlb0cfg = E500_GetSPR_TLB0CFG();
    tlb1cfg = E500_GetSPR_TLB1CFG();

    p_MmuIdent->nPids       = (uint8_t)((mmucfg & 0x00007800) >>11);
    p_MmuIdent->pidSize     = (uint8_t)((mmucfg & 0x000007C0) >>6);
    p_MmuIdent->nTlbs       = (uint8_t)(((mmucfg & 0x0000000C) >>2) + 1);
    p_MmuIdent->mAvn        = (uint8_t) (mmucfg & 0x00000003);

    p_MmuIdent->tlb0aSsoc   = (uint8_t)((tlb0cfg & 0xFF000000) >> 24);
    p_MmuIdent->tlb0nEntry  = (uint16_t)(tlb0cfg & 0x00000FFF);
    p_MmuIdent->tlb0minSize = (uint8_t)((tlb0cfg & 0x00F00000) >> 20);
    p_MmuIdent->tlb0maxSize = (uint8_t)((tlb0cfg & 0x000F0000) >> 16);

    p_MmuIdent->tlb1aSsoc   = (uint8_t)((tlb1cfg & 0xFF000000) >> 24);
    p_MmuIdent->tlb1nEntry  = (uint16_t)(tlb1cfg & 0x00000FFF);
    p_MmuIdent->tlb1minSize = (uint8_t)((tlb1cfg & 0x00F00000) >> 20);
    p_MmuIdent->tlb1maxSize = (uint8_t)((tlb1cfg & 0x000F0000) >> 16);
}

/************************************************************************************
*   Routine:        MMU_TlbEntryWrite                                               *
*                                                                                   *
*   Description:    Configure a TLB entry - new or overwrite an existing one.       *
*                                                                                   *
*   Arguments       t_MmuTlbEntryParam - (In) a structure of parameters for the     *
*                                       configuration of the specific TLB entry     *
*                                       as described in mmu_ext.h.                  *
*   Return          Error Code                                                      *
*                                                                                   *
*************************************************************************************/
t_Error MMU_TlbEntryWrite(t_MmuTlbEntryParam *p_MmuTlbEntryParam)
{
    uint32_t    tmpMas[9];
    uint32_t    tmpWay, pageSize, tmpEntry, maxNumEntries;
    int32_t     pgSize;
    t_MmuIdent  p_MmuIdent;

/*
 * Read MMU Status registers so we can know if it is V1 or V2.
 *
 */
    MMU_Identify(&p_MmuIdent);             /* Read MMU version information */

    if (((p_MmuIdent.tlb1maxSize != 0x9)  && (p_MmuIdent.tlb1maxSize != 0xb)) ||
        ((p_MmuIdent.tlb1nEntry  != 0x10) && (p_MmuIdent.tlb1nEntry  != 0x40)) ||
        ((p_MmuIdent.tlb0aSsoc != 0x2)    && (p_MmuIdent.tlb0aSsoc != 0x4)) ||
        ((p_MmuIdent.tlb0nEntry != 0x100) && (p_MmuIdent.tlb0nEntry != 0x200)))
    {
       REPORT_ERROR(CRITICAL, E_NO_DEVICE, ("Wrong MMU Device"));
       return(E_NO_DEVICE);
    }

    pgSize = (p_MmuTlbEntryParam->pageSize & MAS1_TSIZE_MASK) >> MAS1_TSIZE_SHIFT;

    /* Verify the page is in Range */
    if ((pgSize > (e_MMU_MAS1_TLB1_TSIZE_4G & MAS1_TSIZE_MASK) >> MAS1_TSIZE_SHIFT) || (pgSize == e_MMU_MAS1_TLB1_TSIZE_WRONG))
    {
        REPORT_ERROR(CRITICAL, E_NOT_IN_RANGE, ("Wrong Page size Entry"));
        return(E_INVALID_ADDRESS);
    }

    pageSize = PAGE_SIZE(pgSize);

    /* Verify that virtualBaseAddr aligned to page size */
    if (((pageSize-1) & p_MmuTlbEntryParam->virtualBaseAddr) != 0)
    {
        REPORT_ERROR(CRITICAL, E_NOT_ALIGNED, ("Page virtual address must be aligned to page size"));
        return(E_INVALID_ADDRESS);
    }

    /* Verify that physicalBaseAddr aligned to page size */
    if (((pageSize-1) & p_MmuTlbEntryParam->physicalBaseAddr) != 0)
    {
        REPORT_ERROR(CRITICAL, E_NOT_ALIGNED, ("Page virtual address must be aligned to page size"));
        return(E_INVALID_ADDRESS);
    }

    memset(tmpMas, 0, sizeof(tmpMas));
    tmpMas[0] |= p_MmuTlbEntryParam->tlbSel;
    if (p_MmuTlbEntryParam->tlbSel == TLB1)
    {
        /* TLB1 number of entries selected by ESEL filed in MAS0 */
        if (p_MmuTlbEntryParam->entrySel > TLB1_NUM_OF_ENTRIES)
        {
            REPORT_ERROR(CRITICAL, E_NOT_IN_RANGE, ("entrySel(%d) must be 0-%d",
                                                    p_MmuTlbEntryParam->entrySel, TLB1_NUM_OF_ENTRIES - 1));
            return(E_NOT_IN_RANGE);
        }
        tmpMas[0] |= (uint32_t)p_MmuTlbEntryParam->entrySel << MAS0_TLB_ESEL_SHIFT;
        if (p_MmuTlbEntryParam->invalidateProtect)
            tmpMas[1] |= MAS1_IPROT;
    }
    else
    {
    /* TLB0 has TLB0_NUM_OF_ENTRIES(512 on V2)
     * Way seleceted by ESEL 0:1 for way0, way1, way2, way3
     *
     */
        if (p_MmuIdent.tlb1maxSize == 0x9)          /* e500v1 */
            maxNumEntries = TLB0_NUM_OF_ENTRIES_V1;
        else                                        /* e500v2 */
            maxNumEntries = TLB0_NUM_OF_ENTRIES_V2;

        if (p_MmuTlbEntryParam->entrySel > maxNumEntries)
        {
            REPORT_ERROR(CRITICAL, E_NOT_IN_RANGE, ("entrySel must be 0-511"));
            return(E_NOT_IN_RANGE);
        }
        tmpWay   = (uint32_t)p_MmuTlbEntryParam->entrySel / (maxNumEntries /TLB0_NUM_OF_WAYS);
        tmpEntry = (uint32_t)p_MmuTlbEntryParam->entrySel % (maxNumEntries /TLB0_NUM_OF_WAYS);
        tmpMas[0] |= tmpWay << MAS0_TLB_ESEL_SHIFT;
        tmpMas[2] |= tmpEntry << MAS2_EPN_SHIFT;
    }


    if (p_MmuTlbEntryParam->tlbValid)
        tmpMas[1] |= MAS1_TLB_VALID;
    else
        tmpMas[1] &= ~MAS1_TLB_VALID;
    tmpMas[1] |=  p_MmuTlbEntryParam->transId |
                  p_MmuTlbEntryParam->pageSize;

    tmpMas[2] = (uint32_t)(p_MmuTlbEntryParam->virtualBaseAddr & MAS2_EPN_MASK) |
                           p_MmuTlbEntryParam->endianess;

    if(p_MmuTlbEntryParam->x0)
        tmpMas[2] |= MAS2_X0;
    if(p_MmuTlbEntryParam->x1)
        tmpMas[2] |= MAS2_X1;
    if(p_MmuTlbEntryParam->writeThrough)
        tmpMas[2] |= MAS2_W;
    if(p_MmuTlbEntryParam->cacheInhibited)
        tmpMas[2] |= MAS2_I;
    if(p_MmuTlbEntryParam->memCoherencyReq)
        tmpMas[2] |= MAS2_M;
    if(p_MmuTlbEntryParam->guarded)
        tmpMas[2] |= MAS2_G;

    tmpMas[3] = (uint32_t)(p_MmuTlbEntryParam->physicalBaseAddr & MAS3_RPN_MASK) | p_MmuTlbEntryParam->permission;
    /* On e500v2 size can be 16*4GB=64GB (36Bit Addr). RPN Holds the upper 4 bits */
    tmpMas[7]  = (uint32_t)((p_MmuTlbEntryParam->physicalBaseAddr & 0x0000000F00000000LL) >> 32);

    if (0)      /* Good for debug */
       XX_Print("MAS0=%x\nMAS1=%x\nMAS2=%x\nMAS3=%x\nMAS4=%x\nMAS7=%x\n",tmpMas[0], tmpMas[1], tmpMas[2], tmpMas[3], tmpMas[7]);

#ifdef CORE_E500MC
        E500MC_TlbWrite(tmpMas);   /* e500mc */
#else
    if (p_MmuIdent.tlb1maxSize == 0x9)
        E500_TlbWrite(tmpMas[0], tmpMas[1], tmpMas[2], tmpMas[3]);                    /* e500v1 */
    else
        E500_TlbWrite5(tmpMas[0], tmpMas[1], tmpMas[2], tmpMas[3], tmpMas[7]);        /* e500v2 */
#endif /* CORE_E500MC */

    return (E_OK);
}

/******************************************************************************************
*   Routine:        MMU_Tlb1EntryRead                                                     *
*                                                                                         *
*   Description:    Get an initialized TLB entry specification from the MMU.              *
*                                                                                         *
*   Arguments       entrySel -      (in) entry number 0-15.                               *
*                   virtualAddr -   (in) virtual address of the memory page defined       *
*                                   by this entry.                                        *
*                   p_MmuTlbEntry - (out) a structure of parameters defining the          *
*                                   requested TLB entry as described in mmu_ext.h.        *
*   Return          Error Code                                                            *
*                                                                                         *
*                                                                                         *
*******************************************************************************************/
t_Error MMU_Tlb1EntryRead(uint8_t entrySel, uint32_t virtualAddr, t_MmuTlbEntry *p_MmuTlbEntry)
{
    uint32_t masArray[5], tmpMas0, tmpMas2;

    /* TLB1 number of entries selected by ESEL filed in MAS0 */
    if (entrySel > TLB1_NUM_OF_ENTRIES)
        {
            REPORT_ERROR(CRITICAL, E_NOT_IN_RANGE, ("entrySel(%d) must be 0-%d", entrySel, TLB1_NUM_OF_ENTRIES - 1));
            return(E_NOT_IN_RANGE);
        }

    tmpMas0 = (uint32_t)(TLB1 | (entrySel << MAS0_TLB_ESEL_SHIFT));
    tmpMas2 = virtualAddr & MAS2_EPN_MASK;

    /* Use MAS0 and MAS2 to read the TLB entry */
    E500_TlbRead5(masArray, tmpMas0, tmpMas2 );


    /* Check if TLB entry Valid */
    if (!(masArray[1] & MAS1_TLB_VALID))
        return E_NOT_AVAILABLE;

    /* Decode  */
    if (masArray[0] & MAS0_NV)
        p_MmuTlbEntry->nextVictim = TRUE;
    else
        p_MmuTlbEntry->nextVictim = FALSE;

    if (masArray[1] & MAS1_IPROT)
        p_MmuTlbEntry->invalidateProtect = TRUE;
    else
        p_MmuTlbEntry->invalidateProtect = FALSE;

    if (masArray[1] & MAS1_TS)
        p_MmuTlbEntry->transSpace = TRUE;
    else
        p_MmuTlbEntry->transSpace = FALSE;

    p_MmuTlbEntry->transId = (uint8_t)((masArray[1] & MAS1_TID_MASK) >> MAS1_TID_SHIFT);

    p_MmuTlbEntry->pageSize = (e_MmuPageSize) (masArray[1] & MAS1_TSIZE_MASK);

    if (masArray[2] & MAS2_W)
        p_MmuTlbEntry->writeThrough = TRUE;
    else
        p_MmuTlbEntry->writeThrough = FALSE;

    if (!(masArray[2] & MAS2_I))
        p_MmuTlbEntry->cacheEnable = TRUE;
    else
        p_MmuTlbEntry->cacheEnable = FALSE;

    if (masArray[2] & MAS2_M)
        p_MmuTlbEntry->memCoherencyReq = TRUE;
    else
        p_MmuTlbEntry->memCoherencyReq = FALSE;

    if (masArray[2] & MAS2_G)
        p_MmuTlbEntry->guarded = TRUE;
    else
        p_MmuTlbEntry->guarded = FALSE;

    if (masArray[2] & MAS2_X0)
        p_MmuTlbEntry->x0 = TRUE;
    else
        p_MmuTlbEntry->x0 = FALSE;

    if (masArray[2] & MAS2_X1)
        p_MmuTlbEntry->x1 = TRUE;
    else
        p_MmuTlbEntry->x1 = FALSE;

    p_MmuTlbEntry->endianess = (e_MmuEndianess)(masArray[2] & MAS2_E);

    p_MmuTlbEntry->physicalBaseAddr = masArray[3] & MAS3_RPN_MASK;

    /* On e500v2 size can be 16*4GB=64GB (36Bit Addr). RPN Holds the upper 4 bits */
    p_MmuTlbEntry->physicalBaseAddr |= (uint64_t)((uint64_t)(masArray[4] & 0xF) << 32);

    p_MmuTlbEntry->virtualBaseAddr = masArray[2] & MAS2_EPN_MASK;
    p_MmuTlbEntry->permission   = (e_MmuPermission)(masArray[3] & MAS3_PERMIS_MASK);

    return E_OK;
}

/*************************************************************************************
*   Routine:        e_MmuPageSize
*
*   Description:    Convert Phisycal size of 4KB-4GB into TLB1 definition.
*
*   Arguments       pageSize -      (in)Page Size
*
*
*   Return          e_MmuPageSize as define in e500v2_mmu_ext.h
*
*
*************************************************************************************/
e_MmuPageSize MMU_Tlb1Size2Page(uint64_t pageSize)
{
    switch (pageSize)
    {
        case 0x0000000000001000LL: return(e_MMU_MAS1_TLB1_TSIZE_4K);
        case 0x0000000000004000LL: return(e_MMU_MAS1_TLB1_TSIZE_16K);
        case 0x0000000000010000LL: return(e_MMU_MAS1_TLB1_TSIZE_64K);
        case 0x0000000000040000LL: return(e_MMU_MAS1_TLB1_TSIZE_256K);
        case 0x0000000000100000LL: return(e_MMU_MAS1_TLB1_TSIZE_1M);
        case 0x0000000000400000LL: return(e_MMU_MAS1_TLB1_TSIZE_4M);
        case 0x0000000001000000LL: return(e_MMU_MAS1_TLB1_TSIZE_16M);
        case 0x0000000004000000LL: return(e_MMU_MAS1_TLB1_TSIZE_64M);
        case 0x0000000010000000LL: return(e_MMU_MAS1_TLB1_TSIZE_256M);
        case 0x0000000040000000LL: return(e_MMU_MAS1_TLB1_TSIZE_1G);
        case 0x0000000100000000LL: return(e_MMU_MAS1_TLB1_TSIZE_4G);
        default:
            return(e_MMU_MAS1_TLB1_TSIZE_WRONG);
    }
}
/*************************************************************************************
*   Routine:        e_MmuPageSize
*
*   Description:    Convert Phisycal size of 4KB-4GB into TLB1 definition.
*
*   Arguments       pageSize -      (in)Page Size
*
*
*   Return          e_MmuPageSize as define in e500v2_mmu_ext.h
*
*
*************************************************************************************/
static uint64_t MMU_Tlb1Page2Size(e_MmuPageSize mmupg)
{
    switch (mmupg)
    {
        case e_MMU_MAS1_TLB1_TSIZE_4K:            return(0x0000000000001000LL);
        case e_MMU_MAS1_TLB1_TSIZE_16K:           return(0x0000000000004000LL);
        case e_MMU_MAS1_TLB1_TSIZE_64K:           return(0x0000000000010000LL);
        case e_MMU_MAS1_TLB1_TSIZE_256K:          return(0x0000000000040000LL);
        case e_MMU_MAS1_TLB1_TSIZE_1M:            return(0x0000000000100000LL);
        case e_MMU_MAS1_TLB1_TSIZE_4M:            return(0x0000000000400000LL);
        case e_MMU_MAS1_TLB1_TSIZE_16M:           return(0x0000000001000000LL);
        case e_MMU_MAS1_TLB1_TSIZE_64M:           return(0x0000000004000000LL);
        case e_MMU_MAS1_TLB1_TSIZE_256M:          return(0x0000000010000000LL);
        case e_MMU_MAS1_TLB1_TSIZE_1G:            return(0x0000000040000000LL);
        case e_MMU_MAS1_TLB1_TSIZE_4G:            return(0x0000000100000000LL);
        default:
            return(0);
    }
}

/************************************************************************************
*   Routine:        MMU_Tlb1EntryPrint                                              *
*                                                                                   *
*   Description:    Print all entries for debug.                                    *
*                                                                                   *
*   Arguments       mmupage -       (in) e_MmuPageSize                              *
*                                                                                   *
*                                                                                   *
*   Return          None                                                            *
*                                                                                   *
*                                                                                   *
*************************************************************************************/
void MMU_Tlb1EntryPrint(void)
{
uint32_t tmpspr;
t_MmuIdent p_MmuIdent;
t_Error p_Error;
t_MmuTlbEntry p_MmuTlbEntry;
uint8_t i, count;
char buf[1024];
uint32_t *tmp;

    count = 0;
    memset(buf, 0, sizeof(buf));
    tmpspr=E500_GetSPR_MMUCFG();

    count += Sprint((char *)&buf[count],"MMUcfg: %x ", tmpspr);
    tmpspr=E500_GetSPR_TLB0CFG();
    count += Sprint((char *)&buf[count],"TLB0cfg: %x ", tmpspr);
    tmpspr=E500_GetSPR_TLB1CFG();
    count += Sprint((char *)&buf[count],"TLB1cfg: %x \n", tmpspr);
    XX_Print(buf);
    count = 0;
    memset(buf, 0, sizeof(buf));

    MMU_Identify(&p_MmuIdent);
    count += Sprint((char *)&buf[count],"MMU: Version: %x PidSize: %x #PIDS %d #TLBS %d\n", p_MmuIdent.mAvn , p_MmuIdent.pidSize, p_MmuIdent.nPids, p_MmuIdent.nTlbs);
    count += Sprint((char *)&buf[count],"MMU: Tlb0Assoc: %d #Tlb0Entry %d\n", p_MmuIdent.tlb0aSsoc , p_MmuIdent.tlb0nEntry);
    count += Sprint((char *)&buf[count],"MMU: Tlb1Assoc: %d #Tlb1Entry %d\n", p_MmuIdent.tlb1aSsoc , p_MmuIdent.tlb1nEntry);
    count += Sprint((char *)&buf[count],"MMU: Tlb1Entry: U=Used F=Free\n");
    XX_Print(buf);
    count = 0;
    memset(buf, 0, sizeof(buf));

    for (i=0; i < p_MmuIdent.tlb1nEntry; i++)
    {
      p_Error = MMU_Tlb1EntryRead(i, 0x70000000, &p_MmuTlbEntry);
      if (p_Error != E_OK)
         XX_Print("%d:F\n", i);
      else
      {
         count = 0;
         memset(buf, 0, sizeof(buf));
         count += Sprint((char *)&buf[count],"Tlb1Entry#%d ", i);
         tmp = (uint32_t*)(&p_MmuTlbEntry.physicalBaseAddr);
         count += Sprint((char *)&buf[count],"BaseAddr:0x%08X%08X ",tmp[0],tmp[1]);
         count += Sprint((char *)&buf[count],"pgSize:%-6s",MMU_Page2Size((e_MmuPageSize)p_MmuTlbEntry.pageSize));
         if (p_MmuTlbEntry.cacheEnable)
            count += Sprint((char *)&buf[count],"cacheEna. ");
         else
            count += Sprint((char *)&buf[count],"cacheDis. ");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_USER_X) count += Sprint((char *)&buf[count],"UX:");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_USER_R) count += Sprint((char *)&buf[count],"UR:");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_USER_W) count += Sprint((char *)&buf[count],"UW:");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_SUPERVISOR_X) count += Sprint((char *)&buf[count],"SX:");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_SUPERVISOR_R) count += Sprint((char *)&buf[count],"SR:");
         if (p_MmuTlbEntry.permission&e_MMU_MAS3_SUPERVISOR_W) count += Sprint((char *)&buf[count],"SW:");
         if (p_MmuTlbEntry.x0) count += Sprint((char *)&buf[count]," X0");
         if (p_MmuTlbEntry.x1) count += Sprint((char *)&buf[count]," X1");
         count += Sprint((char *)&buf[count],"\n");

         XX_Print(buf);
      }
    }
}

void MMU_Tlb1ClearAllExceptRunning(uint8_t protect1, uint8_t protect2)
{
    t_MmuIdent          p_MmuIdent;
    t_Error             p_Error;
    t_MmuTlbEntry       p_MmuTlbEntry;
    t_MmuTlbEntryParam  MmuTlbEntryParam;
    uint8_t             i;

    MMU_Identify(&p_MmuIdent);

    for (i=0; i < p_MmuIdent.tlb1nEntry; i++)
    {
        if ((i == protect1) || (i == protect2)) continue;     /* Skip up to 2 protected TLB's */

        p_Error = MMU_Tlb1EntryRead(i, 0x70000000, &p_MmuTlbEntry);
        if (p_Error == E_NOT_AVAILABLE) continue;
        else if (p_Error != E_OK)
        {
            REPORT_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);
            return;
        }

        MmuTlbEntryParam.tlbSel               = e_MMU_MAS0_TLBSEL_TLB1;
        MmuTlbEntryParam.entrySel             = i;

        MmuTlbEntryParam.invalidateProtect    = FALSE;                /* Not Protected */
        MmuTlbEntryParam.tlbValid             = FALSE;                /* Not Valid */
        MmuTlbEntryParam.transId              = 0;
        MmuTlbEntryParam.pageSize             = p_MmuTlbEntry.pageSize;
        MmuTlbEntryParam.virtualBaseAddr      = p_MmuTlbEntry.virtualBaseAddr;
        MmuTlbEntryParam.physicalBaseAddr     = p_MmuTlbEntry.physicalBaseAddr;
        MmuTlbEntryParam.x0                   = FALSE;
        MmuTlbEntryParam.x1                   = FALSE;
        MmuTlbEntryParam.writeThrough         = p_MmuTlbEntry.writeThrough;              /* Copy Back */
        MmuTlbEntryParam.cacheInhibited       = p_MmuTlbEntry.cacheEnable;             /* Disable Cache */
        MmuTlbEntryParam.memCoherencyReq      = p_MmuTlbEntry.memCoherencyReq;
        MmuTlbEntryParam.guarded              = FALSE;
        MmuTlbEntryParam.endianess            = p_MmuTlbEntry.endianess;
        MmuTlbEntryParam.permission           = p_MmuTlbEntry.permission;

        MMU_TlbEntryWrite(&MmuTlbEntryParam);
    }
}

/*********************************************************************************
*   Routine:        MMU_Tlb1ClearAll                                             *
*                                                                                *
*   Description:    Clear all entries except protected ones.                     *
*                                                                                *
*                                                                                *
*                                                                                *
*   Return          None                                                         *
*                                                                                *
*                                                                                *
**********************************************************************************/
void MMU_Tlb1ClearAll(void)
{
    t_MmuIdent          p_MmuIdent;
    t_Error             errCode;
    t_MmuTlbEntry       mmuTlbEntry;
    t_MmuTlbEntryParam  mmuTlbEntryParam;
    uint8_t             i;

    MMU_Identify(&p_MmuIdent);

    for (i=0; i < p_MmuIdent.tlb1nEntry; i++)
    {
        errCode = MMU_Tlb1EntryRead(i, 0x70000000, &mmuTlbEntry);

        /* clear the entries that are not protected */
        if (!mmuTlbEntry.invalidateProtect)
        {
            mmuTlbEntryParam.tlbSel               = e_MMU_MAS0_TLBSEL_TLB1;
            mmuTlbEntryParam.entrySel             = i;

            mmuTlbEntryParam.invalidateProtect    = FALSE;                /* Not Protected */
            mmuTlbEntryParam.tlbValid             = FALSE;                /* Not Valid */
            mmuTlbEntryParam.transId              = 0;
            mmuTlbEntryParam.pageSize             = mmuTlbEntry.pageSize;
            mmuTlbEntryParam.virtualBaseAddr      = mmuTlbEntry.virtualBaseAddr;
            mmuTlbEntryParam.physicalBaseAddr     = mmuTlbEntry.physicalBaseAddr;
            mmuTlbEntryParam.x0                   = FALSE;
            mmuTlbEntryParam.x1                   = FALSE;
            mmuTlbEntryParam.writeThrough         = mmuTlbEntry.writeThrough;              /* Copy Back */
            mmuTlbEntryParam.cacheInhibited       = mmuTlbEntry.cacheEnable;             /* Disable Cache */
            mmuTlbEntryParam.memCoherencyReq      = mmuTlbEntry.memCoherencyReq;
            mmuTlbEntryParam.guarded              = FALSE;
            mmuTlbEntryParam.endianess            = mmuTlbEntry.endianess;
            mmuTlbEntryParam.permission           = mmuTlbEntry.permission;

            MMU_TlbEntryWrite(&mmuTlbEntryParam);
        }
    }
}


/*********************************************************************************
*   Routine:        MMU_Tlb1FindTlb                                              *
*                                                                                *
*   Description:    Go Over TLB1 List and return the TLB that feat to the IN     *
*                   Arguments                                                    *
*                                                                                *
*                                                                                *
*   Arguments       uint64_t -      (in) startAddress                            *
*   Arguments       uint64_t -      (in) size                                    *
*                                                                                *
*   Return          uint8_t -       (out) TLB1 Entry number                      *
*                                                                                *
*                                                                                *
**********************************************************************************/
uint8_t MMU_Tlb1FindTlb(uint64_t startAddr, uint64_t size)
{
    t_MmuIdent          p_MmuIdent;
    t_Error             errCode;
    t_MmuTlbEntry       mmuTlbEntry;
    t_MmuTlbEntryParam  mmuTlbEntryParam;
    uint8_t i;
    uint8_t             runTlb;             /* Return Value */
    uint64_t            sAddr, eAddr;


    MMU_Identify(&p_MmuIdent);
    sAddr = startAddr;
    eAddr = startAddr + size;
    runTlb = 0xFF;

    for (i=0; i < p_MmuIdent.tlb1nEntry; i++)
    {
      errCode = MMU_Tlb1EntryRead(i, 0x70000000, &mmuTlbEntry);
      mmuTlbEntryParam.tlbSel = e_MMU_MAS0_TLBSEL_TLB1;
      mmuTlbEntryParam.entrySel = i;

      if  ((sAddr == mmuTlbEntry.virtualBaseAddr) &&
           (eAddr <= mmuTlbEntry.virtualBaseAddr+MMU_Tlb1Page2Size(mmuTlbEntry.pageSize)))
      {
          mmuTlbEntryParam.tlbValid = TRUE;
          mmuTlbEntryParam.invalidateProtect = TRUE;
          runTlb = i;                                /* Mark the TLB only */
          break;
      }
    }
    return runTlb;
}

/*********************************************************************************
*   Routine:        MMU_Tlb1ProtectTlb                                           *
*                                                                                *
*   Description:    Protect an entry .                                           *
*                                                                                *
*   Arguments       uint8_t -       (in) entry                                   *
*                                                                                *
*   Return          None                                                         *
*                                                                                *
**********************************************************************************/
void MMU_Tlb1ProtectTlb(uint8_t entry)
{
t_MmuIdent          p_MmuIdent;
t_Error             errCode;
t_MmuTlbEntry       mmuTlbEntry;
t_MmuTlbEntryParam  mmuTlbEntryParam;


    MMU_Identify(&p_MmuIdent);

    errCode = MMU_Tlb1EntryRead(entry, 0x70000000, &mmuTlbEntry);
    mmuTlbEntryParam.tlbSel = e_MMU_MAS0_TLBSEL_TLB1;
    mmuTlbEntryParam.entrySel = entry;
    mmuTlbEntryParam.invalidateProtect = TRUE;
    mmuTlbEntryParam.tlbValid          = TRUE;
    mmuTlbEntryParam.transId           = mmuTlbEntry.transId;
    mmuTlbEntryParam.pageSize          = mmuTlbEntry.pageSize;
    mmuTlbEntryParam.virtualBaseAddr   = mmuTlbEntry.virtualBaseAddr;
    mmuTlbEntryParam.physicalBaseAddr  = mmuTlbEntry.physicalBaseAddr;
    mmuTlbEntryParam.x0                = TRUE;                                 /* User Define Marked  */
    mmuTlbEntryParam.x1                = TRUE;
    mmuTlbEntryParam.writeThrough      = mmuTlbEntry.writeThrough;            /* Copy Back */
    mmuTlbEntryParam.cacheInhibited    = !mmuTlbEntry.cacheEnable;
    mmuTlbEntryParam.memCoherencyReq   = mmuTlbEntry.memCoherencyReq;
    mmuTlbEntryParam.guarded           = mmuTlbEntry.guarded;
    mmuTlbEntryParam.endianess         = mmuTlbEntry.endianess;
    mmuTlbEntryParam.permission        = mmuTlbEntry.permission;

    MMU_TlbEntryWrite(&mmuTlbEntryParam);
}


/*************************************************************************************/
uint8_t MMU_Tlb1FindUnusedTlb(void)
{
    t_MmuIdent          mmuIdent;
    t_MmuTlbEntry       mmuTlbEntry;
    uint8_t             i, freeTlb;
    t_Error             errCode;

    MMU_Identify(&mmuIdent);

    freeTlb = 0xFF;
    for (i=1; i < mmuIdent.tlb1nEntry; i++)
    {
        errCode = MMU_Tlb1EntryRead(i, 0x70000000, &mmuTlbEntry);

        if (errCode == E_NOT_AVAILABLE)
        {
            freeTlb = i;
            break;
        }
    }

    return freeTlb;
}


/*------------------------------------------
 * Local Functions
 *------------------------------------------
 */

/*************************************************************************************
*   Routine:        MMU_Page2Size Local function.
*
*   Description:    Convert TLB1 size definition into text for printout
*
*   Arguments       mmupage -       (in) e_MmuPageSize
*
*
*   Return      String definition of size
*
*
*************************************************************************************/
static char *MMU_Page2Size(e_MmuPageSize mmupage)
{

    switch (mmupage)
    {
        case e_MMU_MAS1_TLB1_TSIZE_4K:      return("4KB");
        case e_MMU_MAS1_TLB1_TSIZE_16K:     return("16KB");
        case e_MMU_MAS1_TLB1_TSIZE_64K:     return("64KB");
        case e_MMU_MAS1_TLB1_TSIZE_256K:    return("256KB");
        case e_MMU_MAS1_TLB1_TSIZE_1M:      return("1MB");
        case e_MMU_MAS1_TLB1_TSIZE_4M:      return("4MB");
        case e_MMU_MAS1_TLB1_TSIZE_16M:     return("16MB");
        case e_MMU_MAS1_TLB1_TSIZE_64M:     return("64MB");
        case e_MMU_MAS1_TLB1_TSIZE_256M:    return("256MB");
        case e_MMU_MAS1_TLB1_TSIZE_1G:      return("1GB");
        case e_MMU_MAS1_TLB1_TSIZE_4G:      return("4GB");
        default:
            return(0x0);
    }
}
