/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          e500_pm.c

 @Description   core performance routines
*//***************************************************************************/

#include "error_ext.h"
#include "e500v2_pm.h"
#include "e500v2_ext.h"
#include "e500v2_asm_ext.h"


/* Local and forward functions
 *============================
 */
static uint32_t overflow[E500_PM_MAX_COUNTER];
static void E500_PM_Intr( t_Handle   handle);
static uint32_t GetPmc(uint8_t index);
static void SetPmc(uint8_t counter, uint32_t value);

/*****************************************************************************/
uint32_t E500_PM_GetNumCounters(void)
{
    return E500_PM_MAX_COUNTER;
}

/*****************************************************************************/
t_Error E500_PM_SetupCounter(uint8_t counter, e_E500PmEvent event, e_E500PmThreshMul thresMul, uint8_t thresHold)
{
    uint32_t tempReg1, tempReg2;

    if (counter >= E500_PM_MAX_COUNTER)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE,("PM counter value (should not exceed %d)",E500_PM_MAX_COUNTER-1));
    }

    if (thresHold > E500_PM_MAX_THRESHOLD)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE,("PM max threshold (should not exceed %d)",E500_PM_MAX_THRESHOLD));
    }

    if (thresMul > E500_PM_MAX_THRESHMUL)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE,("PM max threshmul (should not exceed %d)",E500_PM_MAX_THRESHMUL));
    }

    /* Thresholding is not supported for counter 2 or 3 */
    if ((counter == 2 || counter == 3) && (thresHold || thresMul))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,("Thresholding cannot be applied for counter 2 or 3"));
    }

    tempReg1 = (uint32_t)event << E500_PM_PMLCA_EVENT_SHIFT;
    tempReg1 |= E500_PM_PMLCA_CE;      /* Enable Overflow Interrupts */

    tempReg2 = thresHold || (thresMul << E500_PM_THRESHMUL_SHIFT);

    switch (counter)
    {
        case 0:
            E500_SetPMLCa0(tempReg1);
            E500_SetPMLCb0(tempReg2);
            E500_SetPMC0(0);
            break;
        case 1:
            E500_SetPMLCa1(tempReg1);
            E500_SetPMLCb1(tempReg2);
            E500_SetPMC1(0);
            break;
        case 2:
            E500_SetPMLCa2(tempReg1);
            E500_SetPMC2(0);
            break;
        case 3:
            E500_SetPMLCa3(tempReg1);
            E500_SetPMC3(0);
            break;
    }

    return (E_OK);
}

/*****************************************************************************/
void E500_PM_Start(void)
{
    uint32_t    tempReg;
    t_Error     error;
    int         i;

    for (i=0; i<E500_PM_MAX_COUNTER; i++)
    {
        overflow[i] = 0;
        SetPmc((uint8_t)i, 0);
    }

    tempReg = E500_GetPMGC0();
    tempReg &= ~E500_PM_PMGC0_FAC;           /* Start Counters run */
    tempReg |= E500_PM_PMGC0_PMIE;           /* Enable Interrupts */
    error = E500_SetIntr(PERF_MONITOR_INTR, E500_PM_Intr, NULL);

    E500_SetPMGC0(tempReg);
    MSR_EnablePMM();
}

/*****************************************************************************/
void E500_PM_Stop(void)
{
    uint32_t    tempReg;

    MSR_DisablePMM();
    tempReg = E500_GetPMGC0();
    tempReg |= E500_PM_PMGC0_FAC;            /* Stop Counters run */
    tempReg &= ~E500_PM_PMGC0_PMIE;          /* Disable Interrupts */
    E500_SetPMGC0(tempReg);

    E500_ClearIntr(PERF_MONITOR_INTR);
}

/*****************************************************************************/
uint64_t E500_PM_GetCounter(uint8_t counter)
{
    uint32_t tempH, tempL;

    if (counter > E500_PM_MAX_COUNTER)
        return 0;

    tempH = overflow[counter];
    tempL = GetPmc(counter);
    /* Check if was an overflow interrupt in the middle */
    if (tempH != overflow[counter])
    {
        tempL = GetPmc(counter);
    }
    /* Since we are incrementing overflow every 31 bit combine the 64 bit counter based on it */
    return ((((uint64_t)overflow[counter]) << 31) + tempL);

}

/*****************************************************************************/
static void E500_PM_Intr( t_Handle   handle)
{
    uint32_t tempReg,tempReg1;
    uint8_t i;
    UNUSED (handle);

    for (i=0; i<E500_PM_MAX_COUNTER; i++)
    {
        tempReg = GetPmc(i);
        if (E500_PM_PMC_OVERFLOW & tempReg)               /* Overflow based on high 32 bit */
        {
            overflow[i]++;
            tempReg1 = GetPmc(i);
            tempReg1 &= (~E500_PM_PMC_OVERFLOW);
            SetPmc(i, tempReg1);
        }
    }
}

/*************************************************************************************
* Internal Routine return a counter based on index
*************************************************************************************/
static uint32_t GetPmc(uint8_t counter)
{
    switch (counter)
    {
        case 0: return  E500_GetPMC0();
        case 1: return  E500_GetPMC1();
        case 2: return  E500_GetPMC2();
        case 3: return  E500_GetPMC3();
    }
    return 0;
}

/*************************************************************************************
* Internal Routine Set a counter based on index
*************************************************************************************/
static void SetPmc(uint8_t counter, uint32_t value)
{
    switch (counter)
    {
        case 0: E500_SetPMC0(value); break;
        case 1: E500_SetPMC1(value); break;
        case 2: E500_SetPMC2(value); break;
        case 3: E500_SetPMC3(value); break;
    }
}
