/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//*

 @File          cq.c

 @Description   Source file for the Circular Queue object.

                Elements are put in and taken out of the queue in FIFO order.
                In addition any position in the queue may be read (without
                affecting the contents of the queue). The size of each item
                in the queue is set when the queue is initialized.

 @Cautions      None.
 *//***************************************************************************/


#include "std_ext.h"

#include "cq.h"
#include "cq_ext.h"
#include "xx_ext.h"
#include "error_ext.h"


/*****************************************************************************/
t_Handle CQ_New(uint16_t maxItems, uint16_t itemSize)
{
    t_Cq *p_Cq;

    ASSERT_COND(maxItems);
    ASSERT_COND(itemSize);

    p_Cq = (t_Cq *)XX_Malloc((uint32_t)(sizeof(t_Cq) + (maxItems * itemSize)));
    if (p_Cq)
    {
        p_Cq->maxItems = maxItems;
        p_Cq->itemSize = itemSize;
        p_Cq->itemsInQueue = 0;
        p_Cq->first = 0;

#ifdef DEBUG
        p_Cq->putFailures = 0;
        p_Cq->maxIn = 0;
#endif /* DEBUG */
    }

    return p_Cq;
}


/*****************************************************************************/
void CQ_Delete(t_Handle h_Cq)
{
    ASSERT_COND(h_Cq);

    XX_Free(h_Cq);
}


/*****************************************************************************/
int CQ_Flush (t_Handle h_Cq)
{
    t_Cq    *p_Cq = (t_Cq *)h_Cq;
    int     itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    p_Cq->first = 0;
    p_Cq->itemsInQueue = 0;

    XX_RestoreAllIntr(intFlags);

    return itemsInQueue;
}


/*****************************************************************************/
int CQ_Put(t_Handle h_Cq, void *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    register int    itemSize;
    register int    i;
    uint8_t         *p_Byte = p_Item;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);

    intFlags = XX_DisableAllIntr();

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;
    itemSize = p_Cq->itemSize;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = ((first + itemsInQueue) * itemSize);
    }
    else
    {
        putPosition = ((first + itemsInQueue - maxItems) * itemSize);
    }

    /* add element to queue */
    for (i=0; i < itemSize; i++)
    {
        p_Cq->items[putPosition + i] = *(p_Byte + i);
    }

    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_PutUnsafe(t_Handle h_Cq, void *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    register int    itemSize;
    register int    i;
    uint8_t         *p_Byte = p_Item;

    ASSERT_COND(h_Cq);

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;
    itemSize = p_Cq->itemSize;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = ((first + itemsInQueue) * itemSize);
    }
    else
    {
        putPosition = ((first + itemsInQueue - maxItems) * itemSize);
    }

    /* add element to queue */
    for (i=0; i < itemSize; i++)
    {
        p_Cq->items[putPosition + i] = *(p_Byte + i);
    }

    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_Put1Byte(t_Handle h_Cq, uint8_t item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Cq->itemSize == 1);

    intFlags = XX_DisableAllIntr();

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = first + itemsInQueue;
    }
    else
    {
        putPosition = first + itemsInQueue - maxItems;
    }

    /* add element to queue */
    p_Cq->items[putPosition] = item;
    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */
    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_Put2Bytes(t_Handle h_Cq, uint16_t item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Cq->itemSize == 2);

    intFlags = XX_DisableAllIntr();

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */

        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = first + itemsInQueue;
    }
    else
    {
        putPosition = first + itemsInQueue - maxItems;
    }

    /* add element to queue */
    *((uint16_t*)p_Cq->items + putPosition) = item;
    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_Put4Bytes(t_Handle h_Cq, uint32_t item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Cq->itemSize == 4);

    intFlags = XX_DisableAllIntr();

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = first + itemsInQueue;
    }
    else
    {
        putPosition = first + itemsInQueue - maxItems;
    }

    /* add element to queue */
    *((uint32_t*)p_Cq->items + putPosition) = item;
    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_Put8Bytes(t_Handle h_Cq, uint64_t item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    putPosition;
    register int    maxItems;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Cq->itemSize == 8);

    intFlags = XX_DisableAllIntr();

    first = p_Cq->first;
    maxItems = p_Cq->maxItems;
    itemsInQueue = p_Cq->itemsInQueue;

    /* Check if queue is full */
    if (itemsInQueue == maxItems)
    {
#ifdef DEBUG
        p_Cq->putFailures++;
#endif /* DEBUG */
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    if ((first + itemsInQueue) < maxItems)
    {
        putPosition = first + itemsInQueue;
    }
    else
    {
        putPosition = first + itemsInQueue - maxItems;
    }

    /* add element to queue */
    *((uint64_t*)p_Cq->items + putPosition) = item;
    p_Cq->itemsInQueue++;

#ifdef DEBUG
    if ((itemsInQueue + 1) > p_Cq->maxIn)
    {
        p_Cq->maxIn = (uint16_t)(itemsInQueue + 1);
    }
#endif /* DEBUG */

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue + 1);
}


/*****************************************************************************/
int CQ_Get(t_Handle h_Cq, void *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    getPosition;
    register int    itemsInQueue;
    register int    itemSize;
    register int    first;
    register int    i;
    uint8_t         *p_Byte = p_Item;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    first = p_Cq->first;
    itemSize = p_Cq->itemSize;
    getPosition = first * itemSize;

    /* Get item from queue */
    for (i=0; i < itemSize; i++)
    {
        *((p_Byte) + i) = (p_Cq->items[i + getPosition]);
    }
    p_Cq->itemsInQueue--;

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_GetUnsafe(t_Handle h_Cq, void *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    getPosition;
    register int    itemsInQueue;
    register int    itemSize;
    register int    first;
    register int    i;
    uint8_t         *p_Byte = p_Item;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        return -1;
    }

    first = p_Cq->first;
    itemSize = p_Cq->itemSize;
    getPosition = first * itemSize;

    /* Get item from queue */
    for (i=0; i < itemSize; i++)
    {
        *((p_Byte) + i) = (p_Cq->items[i + getPosition]);
    }
    p_Cq->itemsInQueue--;

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_Get1Byte(t_Handle h_Cq, uint8_t *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);
    ASSERT_COND(p_Cq->itemSize == 1);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    first = p_Cq->first;

    /* Get item from queue */
    *p_Item = (p_Cq->items[first]);
    p_Cq->itemsInQueue--;

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_Get2Bytes(t_Handle h_Cq, uint16_t *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);
    ASSERT_COND(p_Cq->itemSize == 2);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    first = p_Cq->first;

    /* Get item from queue */
    p_Cq->itemsInQueue--;
    *p_Item = *((uint16_t*)p_Cq->items + first);

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_Get4Bytes(t_Handle h_Cq, uint32_t *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);
    ASSERT_COND(p_Cq->itemSize == 4);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    first = p_Cq->first;

    /* Get item from queue */
    p_Cq->itemsInQueue--;
    *p_Item = *((uint32_t*)p_Cq->items + first);

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_Get8Bytes(t_Handle h_Cq, uint64_t *p_Item)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    first;
    register int    itemsInQueue;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_Item);
    ASSERT_COND(p_Cq->itemSize == 8);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty */
    if (itemsInQueue == 0)
    {
        XX_RestoreAllIntr(intFlags);
        return -1;
    }

    first = p_Cq->first;

    /* Get item from queue */
    p_Cq->itemsInQueue--;
    *p_Item = *((uint64_t*)p_Cq->items + first);

    if (++first >= p_Cq->maxItems)
    {
        first = 0;
    }
    p_Cq->first = (uint16_t)first;

    XX_RestoreAllIntr(intFlags);

    return (itemsInQueue - 1);
}


/*****************************************************************************/
int CQ_ItemsInQueue(t_Handle h_Cq)
{
    ASSERT_COND(h_Cq);

    return ((t_Cq *)h_Cq)->itemsInQueue;
}


/*****************************************************************************/
int CQ_Read(t_Handle h_Cq, void *p_ReadItem, uint16_t position)
{
    t_Cq            *p_Cq = (t_Cq *)h_Cq;
    register int    itemSize;
    register int    first;
    register int    itemsInQueue;
    register int    bytePosition;
    register int    i;
    uint8_t         *p_ReadByte = p_ReadItem;
    uint32_t        intFlags;

    ASSERT_COND(h_Cq);
    ASSERT_COND(p_ReadItem);

    intFlags = XX_DisableAllIntr();

    itemsInQueue = p_Cq->itemsInQueue;
    /* Check if queue is empty and if position is valid */
    if ((itemsInQueue == 0) || (position > itemsInQueue))
    {
        return -1;
    }

    itemSize = p_Cq->itemSize;
    first = p_Cq->first;

    /* Find byte position */
    if (first + position - 1 > p_Cq->maxItems )
    {
        bytePosition = (first + position - 1 - p_Cq->maxItems) * itemSize;
    }
    else
    {
        bytePosition = (first + position - 1) * itemSize;
    }

    /*read from queue*/
    for (i=0; i < itemSize; i++)
    {
        *(p_ReadByte + i) = (p_Cq->items[bytePosition + i]);
    }

    XX_RestoreAllIntr(intFlags);

    return itemsInQueue;
}

