/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//*

 @File          regsolver.c

 @Description   Source file for the Register Solver object.

*//***************************************************************************/

#include "std_ext.h"
#include "string_ext.h"
#include "xx_ext.h"
#include "error_ext.h"
#include "regsolver.h"


/*****************************************************************************/
t_Handle REGSOLVER_New(uint8_t maxUserCount, uint32_t resetValue)
{
    t_RegSolver *p_RegSolver;
    uint32_t    allocSize;

    if (maxUserCount == 0)
    {
        REPORT_ERROR(MAJOR, E_NOT_IN_RANGE, ("Maximum user count"));
        return NULL;
    }

    /* Allocate space for internal structure including required lock masks */
    allocSize = (uint32_t)
        (sizeof(t_RegSolver) + ((maxUserCount - 1) * sizeof(uint32_t)));

    p_RegSolver = (t_RegSolver *)XX_Malloc(allocSize);
    if (!p_RegSolver)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("RegSolver"));
        return NULL;
    }
    memset(p_RegSolver, 0, allocSize);

    p_RegSolver->maxUserCount = maxUserCount;
    p_RegSolver->curMask = resetValue;

    return p_RegSolver;
}


/*****************************************************************************/
t_Error REGSOLVER_Delete(t_Handle h_RegSolver)
{
    SANITY_CHECK_RETURN_ERROR(h_RegSolver, E_INVALID_HANDLE);

    if (h_RegSolver)
    {
        XX_Free(h_RegSolver);
    }

    return E_OK;
}


/*****************************************************************************/
uint32_t REGSOLVER_GetValue(t_Handle h_RegSolver)
{
    t_RegSolver *p_RegSolver = (t_RegSolver *)h_RegSolver;

    SANITY_CHECK_RETURN_ERROR(p_RegSolver, E_INVALID_HANDLE);

    return p_RegSolver->curMask;
}


/*****************************************************************************/
bool REGSOLVER_LockBits(t_Handle h_RegSolver, uint32_t value, uint32_t mask)
{
    t_RegSolver *p_RegSolver = (t_RegSolver *)h_RegSolver;
    uint32_t    intFlags, maxUsers, tempMask, lockMask = mask;
    int         i = 0;

    SANITY_CHECK_RETURN_VALUE(p_RegSolver, E_INVALID_HANDLE, FALSE);

    maxUsers = p_RegSolver->maxUserCount;

    intFlags = XX_DisableAllIntr();

    if ((p_RegSolver->userLocks[0] & mask & p_RegSolver->curMask) !=
        (p_RegSolver->userLocks[0] & mask & value))
    {
        /* New state conflicts with current state */
        XX_RestoreAllIntr(intFlags);
        return FALSE;
    }

    if (p_RegSolver->userLocks[maxUsers - 1] & mask)
    {
        /* Reached users limit */
        XX_RestoreAllIntr(intFlags);
        return FALSE;
    }

    /* Update user locks */
    do
    {
        ASSERT_COND(i < maxUsers);  /* Guaranteed by algorithm */
        tempMask = (p_RegSolver->userLocks[i] & lockMask);
        p_RegSolver->userLocks[i] |= lockMask;
        lockMask &= tempMask;
        i++;
    } while (lockMask);

    /* Update current mask */
    p_RegSolver->curMask |= (value & mask);
    p_RegSolver->curMask &= ~(~(value) & mask);

    XX_RestoreAllIntr(intFlags);

    return TRUE;
}


/*****************************************************************************/
bool REGSOLVER_UnlockBits(t_Handle h_RegSolver, uint32_t mask)
{
    t_RegSolver *p_RegSolver = (t_RegSolver *)h_RegSolver;
    uint32_t    intFlags, tempMask, lockMask = mask;
    int         i;

    SANITY_CHECK_RETURN_VALUE(p_RegSolver, E_INVALID_HANDLE, FALSE);

    i = (int)(p_RegSolver->maxUserCount - 1);

    intFlags = XX_DisableAllIntr();

    if ((p_RegSolver->userLocks[0] & mask) != mask)
    {
        /* Trying to release bits that are already released */
        XX_RestoreAllIntr(intFlags);
        return FALSE;
    }

    /* Update user locks */
    do
    {
        ASSERT_COND(i >= 0);  /* Guaranteed by algorithm */
        tempMask = (p_RegSolver->userLocks[i] & lockMask);
        p_RegSolver->userLocks[i] &= ~(lockMask);
        lockMask &= ~(tempMask);
        i--;
    } while (lockMask);

    XX_RestoreAllIntr(intFlags);

    return TRUE;
}


/*****************************************************************************/
bool REGSOLVER_CheckLock(t_Handle h_RegSolver, uint32_t mask)
{
    t_RegSolver *p_RegSolver = (t_RegSolver *)h_RegSolver;
    uint32_t    intFlags;
    bool        ret;

    SANITY_CHECK_RETURN_VALUE(p_RegSolver, E_INVALID_HANDLE, FALSE);

    intFlags = XX_DisableAllIntr();

    ret = (bool)((p_RegSolver->userLocks[0] & mask) == mask);

    XX_RestoreAllIntr(intFlags);

    return ret;
}

