/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          epic_ext.h

 @Description   Prototypes, externals and typedefs for system-supplied
                 (external) e500 Open PIC Interrupt controler
*//***************************************************************************/

#ifndef __EPIC_EXT_H
#define __EPIC_EXT_H

#include "std_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         epic_grp  EPIC Application Programming Interface

 @Description   Prototypes, externals and typedefs for EPIC Routines.
 @{
*//***************************************************************************/

#define EPIC_NUM_OF_IPI_EVENTS      4

#define EPIC_INTR_BASE_TIMER        0
#define EPIC_INTR_BASE_EXTERNAL     (EPIC_INTR_BASE_TIMER    + EPIC_NUM_OF_TIMERS)
#define EPIC_INTR_BASE_INTERNAL     (EPIC_INTR_BASE_EXTERNAL + EPIC_NUM_OF_EXT_INTRS)
#define EPIC_INTR_BASE_MSG          (EPIC_INTR_BASE_INTERNAL + EPIC_NUM_OF_INT_INTRS)
#define EPIC_INTR_BASE_IPI          (EPIC_INTR_BASE_MSG      + EPIC_NUM_OF_MSG_INTRS)
#define EPIC_INTR_BASE_SHARED_MSG   (EPIC_INTR_BASE_IPI      + EPIC_NUM_OF_IPI_EVENTS)

#define EPIC_NUM_OF_INTR_SOURCES    (EPIC_NUM_OF_MSG_INTRS    + \
                                     EPIC_NUM_OF_TIMERS       + \
                                     EPIC_NUM_OF_IPI_EVENTS   + \
                                     EPIC_NUM_OF_EXT_INTRS    + \
                                     EPIC_NUM_OF_INT_INTRS    + \
                                     EPIC_NUM_OF_SMSG_INTRS)


#define MAX_DEVICE_PER_INT          8 /* Some INT like DUART have more than one device on IRQ line */


/**************************************************************************//**
 @Description   EPIC Fields defintions for source table
*//***************************************************************************/
#define     MASKED              1       /**< Interrupt source is masked */
#define     UNMASKED            0       /**< Interrupt source is masked  */
#define     P_ACTIVE_LOW        0       /**< Polarity High to Low */
#define     P_ACTIVE_HIGH       1       /**< Polarity Low to High */
#define     SENSE_EDGE          0       /**< Interrupt is SENSE EDGE */
#define     SENSE_LEVEL         1       /**< Interrupt is SENSE LEVEL */
#define     IN_INT              0       /**< Internal Interrupt */
#define     EP_INT              1       /**< External Interrupt */
#define     CI_INT              2       /**< Critical Interrupt */
#define     NA                  0xF     /**< Not Appliceble */


/**************************************************************************//**
 @Description   Interrupt table definition .
*//***************************************************************************/
typedef struct t_EpicIntrInfo
{
    int             intrSrc;        /**< Interrupt source (interrupt vector) */
    uint8_t         polarity;       /**< Interrupt signal polarity */
    uint8_t         sense;          /**< Level or Edge */
    uint8_t         priority;       /**< Priority: 00 masked, 01 lower, ... 15 highest */
    uint8_t         ep_ci;          /**< EP or CI Bits */
    uint8_t         multihandle;    /**< 0,1 Single Handle 2 or more for multi interrupt devices per single interrupt source */
} t_EpicIntrInfo;


/**************************************************************************//**
 @Group         epic_init_grp  EPIC Init Functions

 @Description   Prototypes, for system-supplied (external) EPIC Config and Init Routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      EPIC_Config

 @Description   General initialization of the EPIC interrupt controller.

 @Param[in]     baseAddress     - Base address of EPIC memory mapped registers.
 @Param[in]     p_EpicParam     - Pointer to the HW Init table.
 @Param[in]     epicParamSize   - Number of params in HW Init table.
 @Param[in]     spuriousIrq     - The id of the spurious irq to be configured by the EPIC driver.

 @Return        Handle to the initialized EPIC object.
*//***************************************************************************/
t_Handle EPIC_Config(uint32_t       baseAddress,
                     t_EpicIntrInfo *p_EpicParam,
                     uint32_t       epicParamSize,
                     uint32_t       spuriousIrq);

/**************************************************************************//**
 @Function      EPIC_ConfigGuestId

 @Description   Configure guest ID for EPIC interrupt controller.

                This function is used in multi-partition AMP
                environment.

 @Param[in]     h_Epic  - Handle of the EPIC object.
 @Param[in]     guestId - Guest ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error EPIC_ConfigGuestId(t_Handle h_Epic, uint8_t guestId);

/**************************************************************************//**
 @Function      EPIC_Init

 @Description   Unmasks a specific interupt source to enable interrupts to
                this source.

 @Param[in]     h_Epic - Handle of the EPIC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error EPIC_Init(t_Handle h_Epic);

/**************************************************************************//**
 @Function      EPIC_Free

 @Description   Epic free Routine release all resources and close Interrupt.

 @Param[in]     h_Epic - Handle of the EPIC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  EPIC_Free(t_Handle h_Epic);

/**************************************************************************//**
 @Function      EPIC_SetIntr

 @Description   Registration of an interrupt source.

 @Param[in]     h_Epic      - Handle of the EPIC object.
 @Param[in]     epicIntrSrc - EPIC interrupt source
 @Param[in]     f_Isr       - Callback routine to be called when interrupt occurs;
 @Param[in]     handle      - Handle of the object handling the interrupt.
                              This handle will be passed in the f_Isr callback.

 @Return        E_OK on success; Error code otherwise.
*//****************************************************************************/
t_Error EPIC_SetIntr(t_Handle   h_Epic,
                     int        epicIntrSrc,
                     t_Isr      *f_Isr,
                     t_Handle   handle);

/**************************************************************************//**
 @Function      EPIC_FreeIntr

 @Description   Deregistration of an interrupt source.

 @Param[in]     h_Epic      - Handle of the EPIC object.
 @Param[in]     epicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error EPIC_FreeIntr(t_Handle h_Epic, int epicIntrSrc);

/** @} *//* End of epic_init_grp */


/**************************************************************************//**
 @Group         epic_ctrl  EPIC Control Functions

 @Description   Prototypes, for system-supplied (external) EPIC Set/Clear Routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      EPIC_EnableIntr

 @Description   Unmask (enable) a specific interupt source.

 @Param[in]     h_Epic      - Handle of the EPIC object.
 @Param[in]     epicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error EPIC_EnableIntr(t_Handle h_Epic, int epicIntrSrc);

/**************************************************************************//**
 @Function      EPIC_DisableIntr

 @Description   Mask (disable) a specific interupt source.

 @Param[in]     h_Epic      - Handle of the EPIC object.
 @Param[in]     epicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error EPIC_DisableIntr(t_Handle h_Epic, int epicIntrSrc);


/**************************************************************************//**
 @Function      EPIC_Reset

 @Description   Reset Epic Controler.

 @Param[in]     h_Epic - Handle of the EPIC object.

 @Return        None.
*//***************************************************************************/
void EPIC_Reset(t_Handle h_Epic);

/**************************************************************************//**
 @Function      EPIC_Intr

 @Description   Epic interrupt Routine called from PPC External Interrupt handler.

 @Param[in]     handle - Unused.

 @Return        None.
*//***************************************************************************/
void EPIC_Intr(t_Handle handle);

/** @} *//* End of epic_ctrl */
/** @} *//* End of epic_grp */


#endif /* __EPIC_EXT_H */
