/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          ipic_ext.h

 @Description   IPIC Application Programming Interface
*//***************************************************************************/
#ifndef __IPIC_EXT_H
#define __IPIC_EXT_H

#include "std_ext.h"

/**************************************************************************//**
 @Group         ipic_id IPIC Application Programming Interface

 @Description   IPIC API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ipic_init_grp IPIC Initialization Unit

 @Description   IPIC initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   IPIC interrupt signal types
*//***************************************************************************/
typedef enum e_IpicIntrSignal
{
    e_IPIC_INT_SIGNAL   = 0,    /**< IPIC int signal */
    e_IPIC_SMI_SIGNAL   = 1,    /**< IPIC smi signal */
    e_IPIC_CINT_SIGNAL  = 2     /**< IPIC cint signal */
} e_IpicIntrSignal;


/**************************************************************************//**
 @Description   IPIC interrupt priority groups
*//***************************************************************************/
typedef enum e_IpicGrpId
{
    e_IPIC_INT_GRP_A = 0,   /**< IPIC internal interrupts priority group A */
    e_IPIC_INT_GRP_B,       /**< IPIC internal interrupts priority group B */
    e_IPIC_INT_GRP_C,       /**< IPIC internal interrupts priority group C */
    e_IPIC_INT_GRP_D,       /**< IPIC internal interrupts priority group D */
    e_IPIC_MIX_GRP_A,       /**< IPIC mixed interrupts priority group A */
    e_IPIC_MIX_GRP_B        /**< IPIC mixed interrupts priority group B */
} e_IpicGrpId;

/**************************************************************************//**
 @Description   IPIC interrupt priority groups
*//***************************************************************************/
typedef enum e_IpicIrqMode
{
    e_IPIC_IRQ_MODE_ACTIVE_LOW = 0,     /**< External IRQ is level-sensitive, active-low */
    e_IPIC_IRQ_MODE_HIGH_TO_LOW_CHANGE, /**< External IRQ is edge-sensitive, triggered on high-to-low change */
    e_IPIC_IRQ_MODE_ACTIVE_HIGH,        /**< External IRQ is level-sensitive, active-high */
    e_IPIC_IRQ_MODE_LOW_TO_HIGH_CHANGE  /**< External IRQ is edge-sensitive, triggered on low-to-high change */
} e_IpicIrqMode;



/**************************************************************************//**
 @Description   QE interrupt controller configuration structure.
 *//***************************************************************************/
typedef struct t_IpicParam
{
    uint32_t    baseAddress;    /**< Base address of IPIC registers in memory. */

} t_IpicParam;


/**************************************************************************//**
 @Function      IPIC_Config

 @Description   Configuration of the IPIC interrupt controller.

 @Param[in]     p_IpicParam - Pointer to IPIC configuration parameters.

 @Return        Handle to the initialized IPIC object; NULL on failure.
*//***************************************************************************/
t_Handle IPIC_Config(t_IpicParam *p_IpicParam);


/**************************************************************************//**
 @Group         ipic_advanced_init_grp IPIC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      IPIC_ConfigSpreadMode

 @Description   Configure spread mode for a specified interrupt group (default is grouped)

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicGroupId - Group Id.
 @Param[in]     enable      - TRUE to enable spread mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ConfigSpreadMode(t_Handle h_Ipic, e_IpicGrpId ipicGroupId, bool enable);


/**************************************************************************//**
 @Function      IPIC_ConfigHighestPriorityDestSignal

 @Description   Configure destination signal for one of the 2 top priorities,
                for a specified interrupt group (default is e_IPIC_INT_SIGNAL for all)


 @Param[in]     h_Ipic          - Handle of the IPIC object.
 @Param[in]     ipicDestSignal  - IPIC signal type.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ConfigHighestPriorityDestSignal(t_Handle           h_Ipic,
                                             e_IpicIntrSignal   ipicDestSignal);

/**************************************************************************//**
 @Function      IPIC_ConfigDestSignal

 @Description   Optional, used to change default.

                The highest 2 priorities in each priority may be directed to
                any of the 2 optional interrupt signals. This routine
                configures a specific group priority (0 or 1) to be directed
                to a specific signal. If this routine is not called, default
                signal is used which is e_IPIC_INT_SIGNAL.

 @Param[in]     h_Ipic          - Handle of the IPIC object.
 @Param[in]     ipicGrpId       - One of:
                                  e_IPIC_INT_GRP_A
                                  e_IPIC_INT_GRP_D
                                  e_IPIC_MIX_GRP_A
                                  e_IPIC_MIX_GRP_B
 @Param[in]     intrSrcPriority - 0 (highest);
                                  1 (second).
 @Param[in]     ipicDestSignal  - One of:
                                  e_IPIC_INT_SIGNAL
                                  e_IPIC_SMI_SIGNAL
                                  e_IPIC_CINT_SIGNAL

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ConfigDestSignal(t_Handle          h_Ipic,
                              e_IpicGrpId       ipicGrpId,
                              int               intrSrcPriority,
                              e_IpicIntrSignal  ipicDestSignal);

/**************************************************************************//**
 @Function      IPIC_ConfigMachineCheckCallback

 @Description   Configure an optional callback function for handling machine
                check events.

                The single argument of the callback function holds the machine
                check events that generated the interrupt.

 @Param[in]     h_Ipic          - Handle of the IPIC object.
 @Param[in]     f_MachineCheck  - Machine check callback function.

 @Return        E_OK on success; Error code otherwise.

 @See           @ref ipic_mcpbm "IPIC Machine Check Events Bit Mask".
*//***************************************************************************/
t_Error IPIC_ConfigMachineCheckCallback(t_Handle    h_Ipic,
                                        void        (*f_MachineCheck)(uint32_t events));

/**************************************************************************//**
 @Function      IPIC_ConfuigMultiHandler

 @Description   Configure multi handlers per one Interrupt vector.


 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.

 *//***************************************************************************/
t_Error IPIC_ConfigMultiHandler(t_Handle h_Ipic, int ipicIntrSrc );


/** @} */ /* end of ipic_advanced_init_grp group */

/**************************************************************************//**
 @Function      IPIC_Init

 @Description   General initialization of the IPIC interrupt controller.
                It includes configuration of general registers, and driver's
                interrupt table initialization.

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        E_OK when successful, other value otherwise.
*//***************************************************************************/
t_Error IPIC_Init(t_Handle h_Ipic);

/**************************************************************************//**
 @Function      IPIC_Free

 @Description   Frees all memory associated with
                the SIU Interrupt controller.

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        E_OK on success; Error code otherwise.
*//****************************************************************************/
t_Error IPIC_Free(t_Handle h_Ipic);

/**************************************************************************//**
 @Function      IPIC_SetIntr

 @Description   Registration of an interrupt source.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - IPIC interrupt source
 @Param[in]     f_Isr       - Callback routine to be called when interrupt occurs;
 @Param[in]     handle      - Handle of the object handling the interrupt.
                              This handle will be passed in the f_Isr callback.

 @Return        E_OK on success; Error code otherwise.
*//****************************************************************************/
t_Error IPIC_SetIntr(t_Handle   h_Ipic,
                     int        ipicIntrSrc,
                     t_Isr      *f_Isr,
                     t_Handle   handle);

/**************************************************************************//**
 @Function      IPIC_FreeIntr

 @Description   Deregistration of an interrupt source.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_FreeIntr(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_ModifyExternalIrqMode

 @Description   This routine modifies the edge sensitivity (and also polarity in
                devices that support this option) of a specific external IRQ signal.

 @Param[in]     h_Ipic  - Handle of the IPIC object.
 @Param[in]     irqId   - External IRQ number.
 @Param[in]     irqMode - Requested mode for external IRQ.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ModifyExternalIrqMode(t_Handle h_Ipic, uint8_t irqId, e_IpicIrqMode irqMode);

/**************************************************************************//**
 @Function      IPIC_ModifyIrq0Mcp

 @Description   This routine modifies IRQ0 to be used as an MCP request rather
                than a regular interrupt request which is the default.

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        None
*//***************************************************************************/
void IPIC_ModifyIrq0Mcp(t_Handle h_Ipic);

/**************************************************************************//**
 @Function      IPIC_CoreDisableMcpOut

 @Description   This routine should be called by the user
                If core is disabled and MCP should be routed to
                MCP_OUT signal.

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        None
*//***************************************************************************/
void IPIC_CoreDisableMcpOut(t_Handle h_Ipic);

/** @} */ /* end of ipic_init_grp group */


/**************************************************************************//**
 @Group         ipic_control_grp IPIC Control Unit

 @Description   IPIC control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      IPIC_EnableIntr

 @Description   Unmasks a specific interupt source to
                enable interrupts to this source.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_EnableIntr(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_DisableIntr

 @Description   Masks a specific interupt source to
                enable interrupts to this source.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_DisableIntr(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_EnableMcp

 @Description   Unmasks a specific interupt source to enable MCP interrupts to
                this source.

                All MCP's are enabled by default.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_EnableMcp(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_DisableMcp

 @Description   Masks a specific interrupt source to enable interrupts to this source.

                All MCP's are enabled by default.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_DisableMcp(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_ModifyHighestPriority

 @Description   This routine defines a single interrupt source to be highest
                priority.

                It may be called at any stage, thus enabling dynamic change of
                the highest priority interrupt. In default, Highest priority
                is Mixed group A highest priority interrupt source, going
                through !int signal.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ModifyHighestPriority(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_ModifyPriority

 @Description   Optional routine that may be called at run time to manipulate
                interrupt priorities.

                This routine is called to reorganize a specific group. The user
                should pass a list of interrupt sources (of type int)
                in order of priority. The list must include all and only sources
                of the specified group.

 @Param[in]     h_Ipic          - Handle of the IPIC object.
 @Param[in]     ipicGrpId       - One of:
                                  e_IPIC_INT_GRP_A
                                  e_IPIC_INT_GRP_D
                                  e_IPIC_MIX_GRP_A
                                  e_IPIC_MIX_GRP_B
 @Param[in]     pri0            - Interrupt source with priority 0
 @Param[in]     pri1            - Interrupt source with priority 1
 @Param[in]     pri2            - Interrupt source with priority 2
 @Param[in]     pri3            - Interrupt source with priority 3
 @Param[in]     pri4            - Interrupt source with priority 4
 @Param[in]     pri5            - Interrupt source with priority 5
 @Param[in]     pri6            - Interrupt source with priority 6
 @Param[in]     pri7            - Interrupt source with priority 7

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ModifyPriority(t_Handle    h_Ipic,
                            e_IpicGrpId ipicGrpId,
                            int         pri0,
                            int         pri1,
                            int         pri2,
                            int         pri3,
                            int         pri4,
                            int         pri5,
                            int         pri6,
                            int         pri7);

/**************************************************************************//**
 @Function      IPIC_EnableWatchDog

 @Description   Enables software watchdog timer

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        None
*//***************************************************************************/
void IPIC_EnableWatchDog(t_Handle h_Ipic);

/**************************************************************************//**
 @Function      IPIC_DisableWatchDog

 @Description   Watchdog timer is enabled by default, this
                routine disables the software watchdog timer.

 @Param[in]     h_Ipic - Handle of the IPIC object.

 @Return        None
*//***************************************************************************/
void IPIC_DisableWatchDog(t_Handle h_Ipic);

/** @} */ /* end of ipic_control_grp group */

/**************************************************************************//**
 @Group         ipic_exception_grp IPIC Exception Unit

 @Description   IPIC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      IPIC_Intr

 @Description   This routine is the main interrupt handler of IPIC.

 @Param[in]     handle - Unused.

 @Return        None
*//***************************************************************************/
void IPIC_Intr(t_Handle handle);

/**************************************************************************//**
 @Function      IPIC_MachineCheckIntr

 @Description   This routine is the machine check interrupt handler of IPIC.

 @Param[in]     handle - Unused.

 @Return        None
*//***************************************************************************/
void IPIC_MachineCheckIntr(t_Handle handle);


/** @} */ /* end of ipic_exception_grp group */
/** @} */ /* end of ipic_id_grp group */


#ifdef DEBUG

/**************************************************************************//**
 @Function      IPIC_GenerateIntr

 @Description   This routine should be called by the user to generate a specific
                interrupt (usually used for debug).

                This action will provoke a level interrupt that must be cleared
                by calling IPIC_ClearGeneratedIntr().

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_GenerateIntr(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_GenerateMcp

 @Description   This routine should be called by the user to generate
                a specific MCP (usually used for debug). This action will
                provoke a level interrupt that must be cleared by calling
                IPIC_ClearGeneratedMcp.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_GenerateMcp(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_ClearGeneratedIntr

 @Description   This routine should be called by the user after generating an
                interrupt in order to clear the event.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPIC_ClearGeneratedIntr(t_Handle h_Ipic, int ipicIntrSrc);

/**************************************************************************//**
 @Function      IPIC_GetForceRegStatus

 @Description   This routine reads the pending force register.

 @Param[in]     h_Ipic      - Handle of the IPIC object.
 @Param[in]     ipicIntrSrc - Interrupt source Id.

 @Return        The value of the register.
*//***************************************************************************/
uint32_t IPIC_GetForceRegStatus(t_Handle h_Ipic, int ipicIntrSrc);

#endif /* DEBUG */


#endif /* __IPIC_EXT_H */
