/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          qe_ic_ext.h

 @Description   QE interrupt controller external definitions and structures.
*//***************************************************************************/

#ifndef __QE_IC_EXT_H
#define __QE_IC_EXT_H

#include "std_ext.h"


/**************************************************************************//**
 @Group         qe_ic_id QE Interrupt Controller Application Programming Interface

 @Description   QE interrupt controller functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qe_ic_init_grp QE Interrupt Controller Initialization Unit

 @Description   QE interrupt controller initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   QE interrupt sources groups
*//*************************************************************************/
typedef enum e_QeIcGrpId
{
    e_QE_IC_GRP_W,      /**< QE interrupt controller group W */
    e_QE_IC_GRP_X,      /**< QE interrupt controller group X */
    e_QE_IC_GRP_Y,      /**< QE interrupt controller group Y */
    e_QE_IC_GRP_Z,      /**< QE interrupt controller group Z */
    e_QE_IC_GRP_RISCA,  /**< QE interrupt controller RISC group A */
    e_QE_IC_GRP_RISCB   /**< QE interrupt controller RISC group B */
} e_QeIcGrpId;

/**************************************************************************//**
 @Description   QE interrupt controller signal types
*//*************************************************************************/
typedef enum e_QeIcIntrSignal
{
    e_QE_IC_QE_LOW_SIGNAL   = 0,    /**< Low QE Signal */
    e_QE_IC_QE_HIGH_SIGNAL  = 2     /**< High QE Signal */
} e_QeIcIntrSignal;

/**************************************************************************//**
 @Description   QE interrupt controller configuration structure.
 *//***************************************************************************/
typedef struct t_QeIcParam
{
    uint32_t    baseAddress;    /**< Base address of QE IC registers in memory. */
    int         irqLow;         /**< QE IC low interrupt ID (system-specific number) */
    int         irqHigh;        /**< QE IC high interrupt ID (system-specific number) */

} t_QeIcParam;


/**************************************************************************//**
 @Function      QE_IC_Config

 @Description   Configuration of QE interrupt controller.

                This routine must be called prior to any other QE_IC routine.
                It effects software only and creates a QE_IC object.

 @Param[in]     p_QeIcParam - Pointer to QE IC configuration parameters.

 @Return        Handle to the QE_IC object; NULL on failure;
*//*************************************************************************/
t_Handle QE_IC_Config(t_QeIcParam *p_QeIcParam);

/**************************************************************************//**
 @Function      QE_IC_Init

 @Description   Initialization of QE interrupt controller.

                This routine must be called after all configuration routines
                and before other QE_IC routines. It includes configuration of
                general registers, and driver's interrupt table initialization.

 @Param[in]     h_QeIc - Handle to the QE_IC object.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_Init(t_Handle h_QeIc);

/**************************************************************************//**
 @Function      QE_IC_Free

 @Description   Frees all resources associated with the QE Interrupt controller.

 @Param[in]     h_QeIc - Handle to the QE_IC object.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_Free(t_Handle h_QeIc);

/**************************************************************************//**
 @Function      QE_IC_SetIntr

 @Description   Registration of an interrupt source.

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcIntrSrc - QE IC interrupt source
 @Param[in]     f_Isr       - Callback routine to be called when interrupt occurs;
 @Param[in]     handle      - Handle of the object handling the interrupt.
                              This handle will be passed in the f_Isr callback.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_SetIntr(t_Handle  h_QeIc,
                      int       qeIcIntrSrc,
                      t_Isr     *f_Isr,
                      t_Handle  handle);

/**************************************************************************//**
 @Function      QE_IC_FreeIntr

 @Description   Un-registration of an interrupt source.

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_FreeIntr(t_Handle h_QeIc, int qeIcIntrSrc);


/**************************************************************************//**
 @Group         qe_ic_advanced_init_grp QE Interrupt Controller Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_IC_ConfigSpreadMode

 @Description   Configure spread mode for a specified interrupt group (default is grouped)

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcGroupId - Group Id.
 @Param[in]     enable      - TRUE to enable spread mode.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_ConfigSpreadMode(t_Handle h_QeIc, e_QeIcGrpId qeIcGroupId, bool enable);

/**************************************************************************//**
 @Function      QE_IC_ConfigHighestPriorityDestSignal

 @Description   Configure destination signal (QE high/low) for one of the 2 top
                top priorities, for a specified interrupt group (default is QE low for all)

 @Param[in]     h_QeIc          - Handle to the QE_IC object.
 @Param[in]     qeIcDestSignal  - QE high or low signal.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_ConfigHighestPriorityDestSignal(t_Handle          h_QeIc,
                                              e_QeIcIntrSignal  qeIcDestSignal);


/**************************************************************************//**
 @Function      QE_IC_ConfigDestSignal

 @Description   This routine configures a specific group priority (0 or 1) to
                be directed to a specific signal.

                The highest 2 priorities in each priority may be directed to
                any of the 2 optional interrupt signals. If this routine is
                not called, default signal is used which is QE low.

 @Param[in]     h_QeIc          - Handle to the QE_IC object.
 @Param[in]     qeIcGrpId       - One of:
                                  e_QE_IC_GRP_W
                                  e_QE_IC_GRP_X
                                  e_QE_IC_GRP_Y
                                  e_QE_IC_GRP_Z
                                  e_QE_IC_GRP_RISCA
                                  e_QE_IC_GRP_RISCB
 @Param[in]     intrSrcPriority - 0 (highest), or 1 (second).
 @Param[in]     qeIcDestSignal  - One of:
                                  e_QE_IC_QE_LOW_SIGNAL
                                  e_QE_IC_QE_HIGH_SIGNAL

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_IC_ConfigDestSignal(t_Handle         h_QeIc,
                               e_QeIcGrpId      qeIcGrpId,
                               int              intrSrcPriority,
                               e_QeIcIntrSignal qeIcDestSignal);

/** @} */ /* end of qe_ic_advanced_init_grp group */
/** @} */ /* end of qe_ic_init_grp group */


/**************************************************************************//**
 @Group         qe_ic_control_grp QE Interrupt Controller Control Unit

 @Description   QE interrupt controller control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_IC_EnableIntr

 @Description   Unmasks a specific interupt source to enable interrupts to
                this source.

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_EnableIntr(t_Handle h_QeIc, int qeIcIntrSrc);

/**************************************************************************//**
 @Function      QE_IC_DisableIntr

 @Description   Masks a specific interupt source to disable interrupts to
                this source.

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_DisableIntr(t_Handle h_QeIc, int qeIcIntrSrc);

/**************************************************************************//**
 @Function      QE_IC_ModifyHighestPriority

 @Description   Defines a single interrupt source to be highest priority (optional).

                This routine may be called at any stage, thus enabling dynamic
                change of the highest priority interrupt.

                The default highest priority is XCC1 highest priority interrupt
                source.

 @Param[in]     h_QeIc      - Handle to the QE_IC object.
 @Param[in]     qeIcIntrSrc - Interrupt source Id.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_ModifyHighestPriority(t_Handle h_QeIc, int qeIcIntrSrc);

/**************************************************************************//**
 @Function      QE_IC_ModifyPriority

 @Description   Optional routine that may be called at run time to manipulate
                interrupt priorities.

                This routine is called to reorganize a specific group. The user
                should pass a list of interrupt sources (of type int)
                in order of priority. The list must include all and only sources
                of the specified group.

 @Param[in]     h_QeIc          - Handle to the QE_IC object.
 @Param[in]     qeIcGroupId     - One of:
                                  e_QE_IC_GRP_W
                                  e_QE_IC_GRP_X
                                  e_QE_IC_GRP_Y
                                  e_QE_IC_GRP_Z
                                  e_QE_IC_GRP_RISCA
                                  e_QE_IC_GRP_RISCB
 @Param[in]     pri0            - Interrupt source with priority 0
 @Param[in]     pri1            - Interrupt source with priority 1
 @Param[in]     pri2            - Interrupt source with priority 2
 @Param[in]     pri3            - Interrupt source with priority 3
 @Param[in]     pri4            - Interrupt source with priority 4
 @Param[in]     pri5            - Interrupt source with priority 5
 @Param[in]     pri6            - Interrupt source with priority 6
 @Param[in]     pri7            - Interrupt source with priority 7

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_ModifyPriority(t_Handle       h_QeIc,
                             e_QeIcGrpId    qeIcGroupId,
                             int            pri0,
                             int            pri1,
                             int            pri2,
                             int            pri3,
                             int            pri4,
                             int            pri5,
                             int            pri6,
                             int            pri7);

#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      QE_IC_DumpRegs

 @Description   Dumps all registers of the QE IC module.

 @Param[in]     h_QeIc - Handle to the QE_IC object.

 @Return        E_OK on success; Error code otherwise.
*//*************************************************************************/
t_Error QE_IC_DumpRegs(t_Handle h_QeIc);

#endif /* DEBUG_ERRORS > 0 */

/** @} */ /* end of qe_ic_control_grp group */
/** @} */ /* end of qe_ic_id group */

#endif /* __QE_IC_EXT_H */

