/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          cpc_ext.h

 @Description   CPC CACHE/SRAM external structures and definitions.
*//***************************************************************************/
#ifndef __CPC_EXT_H
#define __CPC_EXT_H

#include "std_ext.h"

/**************************************************************************//**
 @Group         cpc_grp CPC Application Programming Interface

 @Description   CPC API functions, definitions structures and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         cpc_init_grp CPC Initialization Unit

 @Description   CPC initialization functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Description   CPC write policy
*//***************************************************************************/
typedef enum e_CpcWritePolicy
{
    e_CPC_WRITE_BACK_MODE    = 0x0,  /**< CPC is operating in write-back mode.   */
    e_CPC_WRITE_THROUGH_MODE = 0x1   /**< CPC is operating in write-through mode.*/
} e_CpcWritePolicy;

/**************************************************************************//**
 @Description   CPC Lock Overflow Policy
*//***************************************************************************/
typedef enum e_CpcLockOverflowPolicy
{
    e_CPC_NOT_REPLACE_LOCKED_GRANULE    = 0x0,  /**< Locked granule won't be replaced in
                                                     case of overflow.   */
    e_CPC_REPLACE_LOCKED_GRANULE        = 0x1   /**< Locked granule will be replaced in
                                                     case of overflow.*/
} e_CpcLockOverflowPolicy;

/**************************************************************************//**
 @Description   SRAM region attributes.
*//***************************************************************************/
typedef struct t_CpcSramRegionAttr
{
    uint64_t            baseAddress;            /**< SRAM base address */
    uint32_t            size;                   /**< SRAM size in bytes */
    bool                interleaving;           /**< TRUE for interleaving mode */
    uint8_t             interleavingFactor;     /**< Interleaving factor */
} t_CpcSramRegionAttr;

/**************************************************************************//**
 @Description   DRAM interleaving mode indication
 *//***************************************************************************/
typedef enum e_CpcDdrInterleavingMode
{
    e_CPC_DDR_NO_INTERLEAVING,  /**< DDR controllers are not interleaved */
    e_CPC_DDR_INTERLEAVING      /**< DDR controllers are interleaved */
} e_CpcDdrInterleavingMode;

/**************************************************************************//**
 @Description   CPC line replacement algorithm.
*//***************************************************************************/
typedef enum e_CpcReplacementAlg
{
    e_CPC_ALG_SPLRU_WITH_AGING  = 0x0,  /**< Streaming Pseudo Least Recently Used with Aging */
    e_CPC_ALG_FIFO              = 0x1,  /**< First In, First Out */
    e_CPC_ALG_SPLRU             = 0x2,  /**< Streaming Pseudo Least Recently Used */
    e_CPC_ALG_PLRU              = 0x3   /**< Pseudo Least Recently Used */
} e_CpcReplacementAlg;

struct t_CpcErr;

/**************************************************************************//**
 @Description   CPC Input Parameter Structure.
 *//***************************************************************************/
typedef struct t_CpcParam
{
    uint64_t    baseAddress;        /**< Base address of CPC registers in memory. */
    int         irq;                /**< CPC interrupt ID (system-specific number) */
    t_Error     (*f_Exceptions)(t_Handle h_App, struct t_CpcErr *p_CpcErr);
                                    /**< Exceptions callback */
} t_CpcParam;

/**************************************************************************//**
 @Function      CPC_Config

 @Description   Creates CPC descriptor according to user parameters.

                User parameters are the minimum necessary for CPC_Init.
 
 @Param[in]     p_CpcParam - Pointer to CPC configuration parameters.

 @Return        CPC module descriptor. NULL if failed.
*//***************************************************************************/
t_Handle CPC_Config(t_CpcParam *p_CpcParam);

/**************************************************************************//**
 @Function      CPC_Init

 @Description   Initialize the CPC hardware block.

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error CPC_Init(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_Free

 @Description   Disable and Frees all resources used by CPC module.

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_Free(t_Handle h_Cpc);


/**************************************************************************//**
 @Group         ddr_adv_init_grp DDR Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      CPC_ConfigSramRegion

 @Description   Config a new SRAM region

 @Param[in]     h_Cpc           - Handle to CPC object.
 @Param[in]     p_Attributes    - Attributes of the requested SRAM region

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigSramRegion(t_Handle h_Cpc, t_CpcSramRegionAttr *p_Attributes);

/**************************************************************************//**
 @Function      CPC_ConfigWritePolicy

 @Description   Set the requested Write policy

 @Param[in]     h_Cpc       - Handle to CPC object.
 @Param[in]     writePolicy - The requested write policy

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigWritePolicy(t_Handle h_Cpc, e_CpcWritePolicy writePolicy);

/**************************************************************************//**
 @Function      CPC_ConfigReplacementAlg

 @Description   Setting replacement algorithm

 @Param[in]     h_Cpc   - Handle to CPC object.
 @Param[in]     alg     - The requested replecement algorithm

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigReplacementAlg(t_Handle h_Cpc, e_CpcReplacementAlg alg);

/**************************************************************************//**
 @Function      CPC_ConfigErrorCheckEn

 @Description   Enables the CPC parity/ECC error checking

 @Param[in]     h_Cpc           - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigErrorCheckEn(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_ConfigLockOverflowMode

 @Description   Controls the behavior of a transaction that wishes to establish
                a coherency granule in the locked state.

                Determines whether to replace or not replace an existing locked
                granule with the requested granule using the selected replacement
                algorithm.

 @Param[in]     h_Cpc               - Handle to CPC object.
 @Param[in]     lockOverflowMode    - Requested mode

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigLockOverflowMode(t_Handle h_Cpc, e_CpcLockOverflowPolicy lockOverflowMode);

/**************************************************************************//**
 @Function      CPC_ConfigFlushErrataWorkaround

 @Description   Configures the backing DDR memory base and size in order to handle
                the hardware flush errata.

 @Param[in]     h_Cpc       - Handle to CPC object.
 @Param[in]     ddrBaseAddr - DDR base address
 @Param[in]     ddrSize     - DDR size

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ConfigFlushErrataWorkaround(t_Handle h_Cpc, uint64_t ddrBaseAddr, uint64_t ddrSize);

/** @} */ /* end of cpc_adv_init_grp */
/** @} */ /* end of cpc_init_grp */

/**************************************************************************//**
 @Group         cpc_control_grp CPC Control Unit

 @Description   CPC control functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Description   Available CPC stash region sizes
 *//***************************************************************************/
typedef enum e_CpcStashSize
{
    e_CPC_STASH_SIZE_64B,       /**< Stash region size - 64 bytes   */
    e_CPC_STASH_SIZE_128B,      /**< Stash region size - 128 bytes  */
    e_CPC_STASH_SIZE_256B,      /**< Stash region size - 256 bytes  */
    e_CPC_STASH_SIZE_512B,      /**< Stash region size - 512 bytes  */
    e_CPC_STASH_SIZE_1KB,       /**< Stash region size - 1 KB       */
    e_CPC_STASH_SIZE_2KB,       /**< Stash region size - 2 KB       */
    e_CPC_STASH_SIZE_4KB,       /**< Stash region size - 4 KB       */
    e_CPC_STASH_SIZE_8KB,       /**< Stash region size - 8 KB       */
    e_CPC_STASH_SIZE_16KB,      /**< Stash region size - 16 KB      */
    e_CPC_STASH_SIZE_32KB,      /**< Stash region size - 32 KB      */
    e_CPC_STASH_SIZE_64KB,      /**< Stash region size - 64 KB      */
    e_CPC_STASH_SIZE_128KB,     /**< Stash region size - 128 KB     */
    e_CPC_STASH_SIZE_256KB,     /**< Stash region size - 256 KB     */
    e_CPC_STASH_SIZE_512KB,     /**< Stash region size - 512 KB     */
    e_CPC_STASH_SIZE_1MB,       /**< Stash region size - 1 MB       */
    e_CPC_STASH_SIZE_2MB        /**< Stash region size - 2 MB       */
} e_CpcStashSize;

/*************************************************************************//**
 @Function      CPC_CreateStashRegion

 @Description   Define a region of memory address as stashed area.

 @Param[in]     h_Cpc       - Handle to CPC object.
 @Param[in]     baseAddr    - Start address of the area to be stashed.
 @Param[in]     size        - Size of the area to be stashed.
 @Param[in]     lock        - Lock or don't lock the data in the CPC
 @Param[out]    p_RegionId  - Returns the selected region ID; this ID should
                              be used when calling CPC_FreeStashRegion().

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_CreateStashRegion(t_Handle          h_Cpc,
                              uint64_t          baseAddr,
                              e_CpcStashSize    size,
                              bool              lock,
                              uint8_t           *p_RegionId);

/**************************************************************************//**
 @Function      CPC_FreeStashRegion

 @Description   Free stashed region

 @Param[in]     h_Cpc       - Handle to CPC object.
 @Param[in]     regionId    - Region ID of the stashed area, as returned by the
                              CPC_CreateStashRegion() function.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_FreeStashRegion(t_Handle h_Cpc, uint8_t regionId);

/**************************************************************************//**
 @Function      CPC_CacheInvalidate

 @Description   Invalidate all lines in the CPC that are designated as cache
                (that is, not SRAM).

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_CacheInvalidate(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_Flush

 @Description   Flush all dirty lines in a given address range to the backing memory.

 @Param[in]     h_Cpc           - Handle to CPC object.
 @Param[in]     startAddress    - The start address (virtual) for flushing.
 @Param[in]     size            - The address space size for flushing.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_Flush(t_Handle h_Cpc, uint64_t startAddress, uint32_t size);

/**************************************************************************//**
 @Function      CPC_LockFlashClear

 @Description   Clear the locks from all lines in the CPC that are designated
                as cache (that is, not SRAM).

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_LockFlashClear(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_CheckLockOverflow

 @Description   Check whether a lock overflow event occurred

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
bool CPC_CheckLockOverflow(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_SetPartitioning

 @Description   Sets partition for the specified snoop ID

 @Param[in]     h_Cpc           - Handle to CPC object.
 @Param[in]     snoopId         - Requested snoop ID
 @Param[in]     allocationMask  - CPC partitioning control
 @Param[in]     wayMask         - CPC Partitioning Way

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_SetPartitioning(t_Handle    h_Cpc,
                            uint8_t     snoopId,
                            uint32_t    allocationMask,
                            uint32_t    wayMask);

/**************************************************************************//**
 @Function      CPC_FreePartition

 @Description   Free all partitions

 @Param[in]     h_Cpc - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ClearPartitioning(t_Handle h_Cpc);

/** @} */ /* end of cpc_control_grp */

/**************************************************************************//**
 @Group         cpc_exception_grp CPC Exception Unit

 @Description   CPC exception functions, definitions structures and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Description   Error types
 *//***************************************************************************/
typedef enum e_CpcErrType
{
    e_CPC_TAG_MULTU_WAT_HIT_ERR  = 0x00000080,   /**< Tag multi-way hit error.   */
    e_CPC_TAG_MULTI_BIT_ECC_ERR  = 0x00000040,   /**< Tag multiple-bit ECC error.*/
    e_CPC_TAG_SINGLE_BIT_ECC_ERR = 0x00000020,   /**< Tag single-bit ECC error   */
    e_CPC_DATA_MULT_BIT_ERR      = 0x00000008,   /**< Data multiple-bit ECC      */
    e_CPC_DATA_SINGLE_BIT_ERR    = 0x00000004,   /**< Data single-bit ECC error  */
    e_CPC_CONFIG_ERROR           = 0x00000001,   /**< CPC configuration error    */
    e_CPC_ALL_ERRORS             = 0x000000ED    /**< CPC all errors             */
} e_CpcErrType;

/**************************************************************************//**
 @Description   Error Policy
 *//***************************************************************************/
typedef enum
{
    e_ERR_DISABLE,              /**< Disable capturing errors */
    e_ERR_ENABLE_CAPTURE,       /**< Enable capturing errors  */
    e_ERR_ENABLE_INTERRUPT,     /**< Enable error interrupts  */
} e_ErrPolicy;

/**************************************************************************//**
 @Description   Data structure to report error attributes to the user.

                The ErrReport routine fills a user structure with the
                current values.
 *//***************************************************************************/
typedef struct t_CpcErr
{
    bool            multErr;            /**< If TRUE, multiple errors of the same type
                                             detected. Otherwise - single error.  */
    bool            tagMultiWayHitErr;  /**< Tag multi-way hit error.             */
    bool            tagMultiBitEccErr;  /**< Tag or status multiple-bit ECC error.*/
    bool            tagSingleBitEccErr; /**< Tag or status single-bit ECC error.  */
    bool            dataMultBitErr;     /**< Data multiple-bit ECC error.         */
    bool            dataSingleBitErr;   /**< Data single-bit ECC error.           */
    bool            configErr;          /**< CPC configuration error.             */

    uint64_t        transAddr;          /**< Address of the transaction associated
                                             with a captured error.               */
    uint64_t        dataCaptured;       /**< Memory data path error captured.     */

    uint8_t         eccSyndrome;        /**< Calculated ECC syndrome of the captured error.*/
    uint8_t         eccChksum;          /**< Stored ECC syndrome of the captured error.    */

} t_CpcErr;

/**************************************************************************//**
 @Function      CPC_ErrorPolicy

 @Description   Set an error policy for specific errors

 @Param[in]     h_Cpc               - Handle to CPC object.
 @Param[in]     errType             - The requested errors
 @Param[in]     errPolicy           - The requested policy
 @Param[in]     singleBitTrigger    - The ECC single-bit error threshold.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ErrorPolicy(t_Handle        h_Cpc,
                        e_ErrPolicy     errPolicy,
                        e_CpcErrType    errType,
                        uint8_t         singleBitTrigger);

/**************************************************************************//**
 @Function      CPC_SetErrorInjectionMask

 @Description   Setting the injecting errors mask into the data, data ECC, tag,
                tag ECC, status and status ECC.

                This may be used to test error recovery software by deterministically
                creating error scenarios.

 @Param[in]     h_Cpc                   - Handle to CPC object.
 @Param[in]     dataErrorMask           - Error injection mask for the data.
 @Param[in]     eccErrorMask            - Error injection mask for the ECC bits.
 @Param[in]     eccMirrorByte           - ECC mirrored byte of the most significant data path enable.
 @Param[in]     tagErrorInjection       - Tag error injection enable.
 @Param[in]     statusErrorInjection    - Status error injection enable

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_SetErrorInjectionMask(t_Handle h_Cpc,
                                  uint64_t dataErrorMask,
                                  uint8_t  eccErrorMask,
                                  bool     eccMirrorByte,
                                  bool     tagErrorInjection,
                                  bool     statusErrorInjection);

/**************************************************************************//**
 @Function      CPC_ErrorInjectionEnable

 @Description   Enable error injection

 @Param[in]     h_Cpc           - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ErrorInjectionEnable(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_ErrorInjectionDisable

 @Description   Disable error injection

 @Param[in]     h_Cpc           - Handle to CPC object.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
t_Error CPC_ErrorInjectionDisable(t_Handle h_Cpc);

/**************************************************************************//**
 @Function      CPC_ErrorReport

 @Description   Error report function, for polling CPC exceptions.

                When interrupt mode is used, errors are reported via
                f_Exceptions callback.

 @Param[in]     h_Cpc       - Handle to CPC object.
 @Param[out]    p_CpcErr    - Captured error's Attributes.

 @Return        E_OK on success; Error code otherwise.
**//***************************************************************************/
bool CPC_ErrorReport(t_Handle h_Cpc, t_CpcErr *p_CpcErr);

/** @} */ /* cpc_exception_grp */

/** @} */ /* end of cpc_grp Programming Interface group */

#endif /* __CPC_EXT_H */
