/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          ecm_v2_ext.h

 @Description   External API for e500 Coherency Module.
*//***************************************************************************/

#ifndef __ECM_V2_EXT_H
#define __ECM_V2_EXT_H


/**************************************************************************//**
 @Group         ecm_grp  ECM Application Programming Interface

 @Description   External prototypes and definitons for e500 Coherency Module.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Maximum number of streamed transactions

                Specifies the maximum number of transactions that any master
                can stream (issue sequentially without preemption) on the CCB
                following an initial transaction.
*//***************************************************************************/
typedef enum e_EcmStreamingMode
{
    e_ECM_STREAM_DISABLED               = 0x00000000,
                                        /**< Streaming is disabled */
    e_ECM_STREAM_MAX_1_ADDITIONAL_TRANS = 0x00000001,
                                        /**< One transaction can be streamed with
                                             the initial transaction */
    e_ECM_STREAM_MAX_2_ADDITIONAL_TRANS = 0x00000002,
                                        /**< Two transactions can be streamed with
                                             the initial transaction */
    e_ECM_STREAM_MAX_3_ADDITIONAL_TRANS = 0x00000003
                                        /**< Three transactions can be streamed with
                                             the initial transaction; this is the
                                             default mode. */
} e_EcmStreamingMode;

/**************************************************************************//**
 @Description   CPU Priority.

                Specifies the priority level of the e500 core (CPU) port. This
                priority level is used to determine whether a particular ports
                bus request can cause the CCB arbiter to terminate another
                ports streaming of address tenures.
*//***************************************************************************/
typedef enum e_EcmCpuPriority
{
    e_ECM_CPU_PRIORITY_LOW  = 0x00000000,   /**< CPU lowest priority */
    e_ECM_CPU_PRIORITY_MID  = 0x00000001,   /**< CPU middle priority */
    e_ECM_CPU_PRIORITY_HIGH = 0x00000002    /**< CPU high priority */
} e_EcmCpuPriority;

/**************************************************************************//**
 @Description   CPU Read Queue Type.

                Identifies which read queue of DDR targets is assigned to the
                e500 core (CPU) read transactions (in understressed system).
*//***************************************************************************/
typedef enum e_EcmCpuReadQueue
{
    e_ECM_CPU_READ_QUEUE_HIGH   = 0x00000000,   /**< High bandwidth CPU read queue */
    e_ECM_CPU_READ_QUEUE_LOW    = 0x00000004    /**< Low bandwidth CPU read queue */
} e_EcmCpuReadQueue;

/**************************************************************************//**
 @Description   ECM transaction type - for error management.
*//***************************************************************************/
typedef enum e_EcmTransType
{
    e_ECM_TRANS_TYPE_WR           = 0,  /**< Transaction type: write */
    e_ECM_TRANS_TYPE_WR_ALLOC     = 2,  /**< Transaction type: write allocate */
    e_ECM_TRANS_TYPE_WR_ALLOC_LCK = 3,  /**< Transaction type: write allocate lock */
    e_ECM_TRANS_TYPE_ADDR_ONLY    = 4,  /**< Transaction type: address */
    e_ECM_TRANS_TYPE_RD           = 8,  /**< Transaction type: read */
    e_ECM_TRANS_TYPE_RD_UNLCK     = 9,  /**< Transaction type: read and unlock */
    e_ECM_TRANS_TYPE_RD_CLR       = 12, /**< Transaction type: read and clear */
    e_ECM_TRANS_TYPE_RD_SET       = 13, /**< Transaction type: read and set */
    e_ECM_TRANS_TYPE_RD_DEC       = 14, /**< Transaction type: read and decrease */
    e_ECM_TRANS_TYPE_RD_INC       = 15  /**< Transaction type: read and increase */
} e_EcmTransType;

/**************************************************************************//**
 @Description   ECM revision.
*//***************************************************************************/
typedef struct t_EcmRevision
{
    uint16_t    ipId;           /**< IP block ID */
    uint8_t     revMajor;       /**< Major revision */
    uint8_t     revMinor;       /**< Minor revision */
    uint8_t     intOptions;     /**< IP block integration options */
    uint8_t     cfgOptions;     /**< IP block configuration options */
} t_EcmRevision;

/**************************************************************************//**
 @Description   ECM error report structure.

                The exceptions callback routine fills this structure with the
                current error indications. The transaction source, type and
                size are only valid when the "valid" field is TRUE.
*//***************************************************************************/
typedef struct t_EcmErrorReport
{
    bool            valid;          /**< TRUE is result are valid */
    bool            multipleErrors; /**< if TRUE, multiple errors of the same type. */
                                    /**< detected. Otherwise - single error. */
    uint64_t        transAddress;   /**< Address of the transaction. */
    e_TransSrc      transSrc;       /**< ID of the device mastering the transaction. */
    e_EcmTransType  transType;      /**< Transaction type. */
    uint8_t         transSize;      /**< Size (in bytes) of the transaction. */

} t_EcmErrorReport;

/**************************************************************************//**
 @Description   ECM Configuration Parameters.
*//***************************************************************************/
typedef struct t_EcmParam
{
    uint32_t    baseAddress;    /**< Base address of memory mapped ECM registers */
    int         irq;            /**< ECM interrupt ID (system-specific number) */
    t_Handle    h_App;          /**< Handle to application (shall passed back to the
                                     application in the exceptions callback function) */
    void        (*f_Exceptions)(t_Handle h_App, t_EcmErrorReport *p_EcmErrorReport);
                                /**< Pointer to exceptions callback routine */
} t_EcmParam;



/**************************************************************************//**
 @Function      ECM_Config

 @Description   Allocates the ECM object with default configuration.

 @Param[in]     p_EcmParam - Pointer to ECM configuration parameters.

 @Return        Handle to ECM object on success; NULL on failure.
*//***************************************************************************/
t_Handle ECM_Config(t_EcmParam *p_EcmParam);

/**************************************************************************//**
 @Function      ECM_ConfigStreamingMode

 @Description   Configures the maximum number of transactions that may be
                streamed together.
                [e_ECM_STREAM_MAX_3_ADDITIONAL_TRANS]

                The streaming mode specifies the maximum number of transactions
                that any master can stream (issue sequentially without
                preemption) on the CCB following an initial transaction.

 @Param[in]     h_Ecm           - Handle to ECM object.
 @Param[in]     streamingMode   - Requested streaming mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ECM_ConfigStreamingMode(t_Handle            h_Ecm,
                                e_EcmStreamingMode  streamingMode);

/**************************************************************************//**
 @Function      ECM_ConfigCoreStreaming

 @Description   Controls whether the e500 core can stream commands onto the CCB.
                [TRUE]

                Note that ECM streaming must also be enabled in order to allow
                core streaming (refer to ECM_ConfigStreamingMode() routine).
                Note also that HID1[ASTME] bit must also be set to allow
                streaming.

 @Param[in]     h_Ecm   - Handle to ECM object.
 @Param[in]     enabled - TRUE to enable core streaming; FALSE to disable it.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ECM_ConfigCoreStreaming(t_Handle h_Ecm, bool enabled);

/**************************************************************************//**
 @Function      ECM_ConfigCpuPriority

 @Description   Configures the e500 priority over other masters.
                [e_ECM_CPU_PRIORITY_LOW]

                Specifies the priority level of the e500 core (CPU) port. This
                priority level is used to determine whether a particular ports
                bus request can cause the CCB arbiter to terminate another
                ports streaming of address tenures.

 @Param[in]     h_Ecm       - Handle to ECM object
 @Param[in]     cpuPriority - New e500 priority on OCeaN

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ECM_ConfigCpuPriority(t_Handle h_Ecm, e_EcmCpuPriority cpuPriority);

/**************************************************************************//**
 @Function      ECM_ConfigCpuReadQueue

 @Description   Configures which read queue is assigned to CPU read transactions.
                [e_ECM_CPU_READ_QUEUE_HIGH]

                This routine configures which read queue of DDR targets is
                assigned to the e500 core read transactions (in understressed system).

 @Param[in]     h_Ecm           - Handle to ECM object.
 @Param[in]     cpuReadQueue    - New read queue assignment

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ECM_ConfigCpuReadQueue(t_Handle h_Ecm, e_EcmCpuReadQueue cpuReadQueue);

/**************************************************************************//**
 @Function      ECM_Init

 @Description   Initialize ECM.

 @Param[in]     h_Ecm - Handle to ECM object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ECM_Init(t_Handle h_Ecm);

/**************************************************************************//**
 @Function      ECM_Free

 @Description   Close ECM.

 @Param[in]     h_Ecm - Handle to ECM object.

 @Return        None.
*//***************************************************************************/
void ECM_Free(t_Handle h_Ecm);

/**************************************************************************//**
 @Function      ECM_ErrIntrEnable

 @Description   Enable ECM Error interrupts.

 @Param[in]     h_Ecm - Handle to ECM object.

 @Return        None.
*//***************************************************************************/
void ECM_ErrIntrEnable(t_Handle h_Ecm);

/**************************************************************************//**
 @Function      ECM_ErrIntrDisable

 @Description   Disable ECM Error interrupts.

 @Param[in]     h_Ecm - Handle to ECM object.

 @Return        None.
*//***************************************************************************/
void ECM_ErrIntrDisable(t_Handle h_Ecm);

/**************************************************************************//**
 @Function      ECM_GetRevision

 @Description   Get ECM Revision.

 @Param[in]     h_Ecm           - Handle to ECM object.
 @Param[out]    p_EcmRevision   - Pointer to ECM revision structure.

 @Return        None.
*//***************************************************************************/
void ECM_GetRevision(t_Handle h_Ecm, t_EcmRevision *p_EcmRevision);


/** @} */ /* end of ecm_grp */


#endif /* __ECM_V2_EXT_H */

