/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          la83_ext.h

 @Description   Local Access external definitions and structures.
*//***************************************************************************/
#ifndef __LA83_EXT_H
#define __LA83_EXT_H


#include "std_ext.h"


#define LA83_NUM_OF_WINDOWS         10      /**< Number of local access windows */

#define LA83_ILLEGAL_LAW            LA83_NUM_OF_WINDOWS
                                            /**< Illegal window ID, for allocation failure */

#define LA83_ID_PCI					0       /**< PCI target interface ID */
#define LA83_ID_LBIU				1       /**< Local bus target interface ID */
#define LA83_ID_DDR					2       /**< DDR target interface ID */
#define LA83_ID_SECONDARY_DDR		3       /**< Secondary DDR target interface ID */
#define LA83_ID_PCIE1               3       /**< PCI Express 1 target interface ID */
#define LA83_ID_PCIE2               4       /**< PCI Express 2 target interface ID */
#define LA83_ID_MMAP				8       /**< Memory map target interface ID */

#define LA83_ID_NONE				0xFF    /**< Error code */


/**************************************************************************//**
 @Description   Local Access Window Attributes
*//***************************************************************************/
typedef struct t_LawAttr
{
    uint8_t   lawNum;   /**< Number of window */
    uint32_t  id;       /**< Target interface ID */
    uint32_t  baseAddr; /**< Base address of the window */
    uint32_t  endAddr;  /**< End Address of the window */
} t_LawAttr;

/**************************************************************************//**
 @Description   Local Access Window Attributes Report Structure
*//***************************************************************************/
typedef t_LawAttr t_LawReport[LA83_NUM_OF_WINDOWS];


/**************************************************************************//*
 @Routine       LA83_ConfigAndInit

 @Description   Creation of LA83 object.

 @Param[in]     baseAddr - Base address of Local Access memory mapped registers.

 @Return        Handle to the initialized LA83 object.
*//***************************************************************************/
t_Handle LA83_ConfigAndInit(uint32_t baseAddr);

/**************************************************************************//*
 @Routine       LA83_WinGet

 @Description   Allocation of a local access window.

                The routine allocates memory space window according to the
                requested size and owner.

 @Param[in]     h_La83  - Handle to the initialized LA83 object.
 @Param[in]     id      - Target interface ID.
 @Param[in]     size    - Size of window; must be a power of 2.
 @Param[out]    p_Addr  - returns base address allocated for the window.

 @Return        Number of allocated window or LA83_ILLEGAL_LAW if size is illegal,
                no window is available or no free memory of required size.
*//***************************************************************************/
uint8_t LA83_WinGet(t_Handle h_La83, uint32_t id, uint32_t size, uint32_t *p_Addr);

/**************************************************************************//*
 @Routine       LA83_WinForce

 @Description   Allocation of a local access window in a specific address.

                The routine allocates memory space window at the requested
                address, according to the requested size and owner.

 @Param[in]     h_La83  - Handle to the initialized LA83 object.
 @Param[in]     id      - Target interface ID.
 @Param[in]     size    - Size of window; must be a power of 2.
 @Param[in]     address - Base address for the window.

 @Return        Number of allocated window or LA83_ILLEGAL_LAW if size or address
                are illegal, no window is available or failed to allocate a
                block of the requested size.
*//***************************************************************************/
uint8_t LA83_WinForce(t_Handle h_La83, uint32_t id, uint32_t size, uint32_t address);

/**************************************************************************//*
 @Routine       LA83_WinFree

 @Description   Release a local access window.

                The routine releases the memory block associated with a specific
                local access window, and disables this window.

                On termination, the routine modifies the current available window
                index to point to the freed window. This is useful in case all
                windows are busy. It may be of no significance or a little
                inefficient in other cases.

 @Param[in]     h_La83  - Handle to the initialized LA83 object.
 @Param[in]     id      - Target interface ID, for verifying ownership.
 @Param[in]     lawNum  - Window number as returned by the window allocation routine.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA83_WinFree(t_Handle h_La83, uint32_t id, uint8_t lawNum);

/**************************************************************************//*
 @Routine       LA83_WinReport

 @Description   Reports the current configuration of the memory space, as
                reflected by the local access memory windows.

                An Array of LA83_NUM_OF_WINDOWS structures of type t_LawAttr is
                allocated and filled with attributes of current status of memory
                space.

                Note: the reported addresses are physical addresses, and not
                virtual addresses.

 @Param[in]     h_La83      - Handle to the initialized LA83 object.
 @Param[in]     p_LawReport - A pointer to an empty structure which is an array
                              of LA83_NUM_OF_WINDOWS attributes structures each
                              holding attributes of a single window. Note that
                              the structures are ordered according to their
                              location at the memory space. Unused (disabled)
                              windows are listed at the end of the array and
                              marked with lawNum LA83_ILLEGAL_LAW.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA83_WinReport(t_Handle h_La83, t_LawReport *p_LawReport);

/**************************************************************************//*
 @Routine       LA83_WinFindId

 @Description   The routine checks for a specific address, which window it falls
                into.

                It does that by going through all windows and comparing the address
                to their start address and size.

 @Param[in]     h_La83      - Handle to the initialized LA83 object.
 @Param[in]     address     - address for testing.
 @Param[out]    p_LawNum    - Returns the window index within all of the found
                              target's windows.

 @Return        Target ID code or LA83_ID_NONE if no match is found.
*//***************************************************************************/
uint8_t LA83_WinFindId(t_Handle h_La83, uint32_t address, uint8_t *p_LawNum);

/**************************************************************************//*
 @Routine       LA83_SetProtect

 @Description   Set or remove protection for each local access window.

                A protected LAW will not be released using the LA83_FreeAll routine

 @Param[in]     h_La83  - Handle to the initialized LA83 object.
 @Param[in]     id      - Target interface ID, for verifying ownership.
 @Param[in]     lawNum  - Window number as returned by the window allocation routine.
 @Param[in]     set     - protection setting.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA83_SetProtect(t_Handle h_La83, uint32_t id, uint8_t lawNum, bool set);

/**************************************************************************//*
 @Routine       LA83_Free

 @Description   Free all LA83 resources.

 @Param[in]     h_La83 - Handle to the initialized LA83 object.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA83_Free(t_Handle h_La83);

/**************************************************************************//*
 @Routine       LA83_WinFreeAll

 @Description   Free all LAWS except the protected LWAS.

 @Param[in]     h_La83 - Handle to the initialized LA object.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA83_WinFreeAll(t_Handle h_La83);



#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      LA83_DumpRegs

 @Description   Prints all LA83 registers.

 @Param[in]     h_La83 - Handle to the initialized LA83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error LA83_DumpRegs(t_Handle h_La83);

#endif /* DEBUG_ERRORS > 0 */

#endif /* __LA83_EXT_H */
