/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          la_ext.h

 @Description   Local Access external definitions and structures.
*//***************************************************************************/
#ifndef __LA_EXT_H
#define __LA_EXT_H


#include "std_ext.h"
#include "part_ext.h"


#define LA_ILLEGAL_LAW          LA_NUM_OF_WINDOWS       /**< Illegal window ID,
                                                             used for allocation failure */

#define LA_MIN_WINDOW_SIZE      0x0000000000001000LL    /**< Minimum window size 4KB */
#define LA_MAX_WINDOW_SIZE      0x0000001000000000LL    /**< Maximum window size 32GB */


/**************************************************************************//**
 @Description   Local Access Window Attributes
*//***************************************************************************/
typedef struct t_LawAttr
{
    uint8_t     lawNum;     /**< Window number */
    uint32_t    id;         /**< Target interface ID */
    uint64_t    baseAddr;   /**< Base address of the window */
    uint64_t    endAddr;    /**< End address of the window */
} t_LawAttr;

/**************************************************************************//**
 @Description   Local Access Window Attributes Report Structure
*//***************************************************************************/
typedef t_LawAttr t_LawReport[LA_NUM_OF_WINDOWS];


/**************************************************************************//*
 @Routine       LA_ConfigAndInit

 @Description   Creation of LA object.

 @Param[in]     baseAddr - Base address of Local Access memory mapped registers.

 @Return        Handle to the initialized LA object.
*//***************************************************************************/
t_Handle LA_ConfigAndInit(uint64_t addr, uint64_t ccsraddr);


/**************************************************************************//*
 @Routine       LA_WinSet

 @Description   Allocation of a local access window in a specific address.

                The routine allocates memory space window at the requested address,
                according to the requested size and owner.

 @Param[in]     h_La    - Handle to the initialized LA object.
 @Param[in]     id      - Target interface ID.
 @Param[in]     size    - Size of window; must be a power of 2.
 @Param[in]     address - Base address for the window.
 @Param[in]     entry   - Entry number to use.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA_WinSet(t_Handle  h_La,
                  e_LaTgtId id,
                  uint64_t  size,
                  uint64_t  address,
                  uint8_t   entry);

/**************************************************************************//*
 @Routine       LA_WinFree

 @Description   Release a local access window.

                The routine releases the memory block associated with a specific
                local access window, and disables this window.

                On termination, the routine modifies the current available window
                index to point to the freed window. This is useful in case all
                windows are busy. It may be of no significance or a little
                inefficient in other cases.

 @Param[in]     h_La    - Handle to the initialized LA object.
 @Param[in]     id      - Target interface ID, for verifying ownership.
 @Param[in]     lawNum  - Window number as returned by the window allocation routine.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA_WinFree(t_Handle h_La, uint32_t id, uint8_t lawNum);

/**************************************************************************//*
 @Routine       LA_WinClear

 @Description   Clears a local access window entry.

 @Param[in]     h_La    - Handle to the initialized LA object.
 @Param[in]     entry   - Local access window entry.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA_WinClear(t_Handle h_La, uint8_t entry);

/**************************************************************************//*
 @Routine       LA_WinReport

 @Description   Reports the current configuration of the memory space, as
                reflected by the local access memory windows.

                An Array of LA_NUM_OF_WINDOWS structures of type t_LawAttr is
                allocated and filled with attributes of current status of memory
                space.

                Note: the reported addresses are physical addresses, and not
                virtual addresses.

 @Param[in]     h_La        - Handle to the initialized LA object.
 @Param[in]     p_LawReport - A pointer to an empty structure which is an array
                              of LA_NUM_OF_WINDOWS attributes structures each
                              holding attributes of a single window. Note that
                              the structures are ordered according to their
                              location at the memory space. Unused (disabled)
                              windows are listed at the end of the array and
                              marked with lawNum LA_ILLEGAL_LAW.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error LA_WinReport(t_Handle h_La, t_LawReport *p_LawReport);

/**************************************************************************//*
 @Routine       LA_WinFindId

 @Description   The routine checks for a specific address, which window it falls
                into.

                It does that by going through all windows and comparing the address
                to their start address and size.

 @Param[in]     h_La        - Handle to the initialized LA object.
 @Param[in]     address     - address for testing.
 @Param[out]    p_LawNum    - Returns the window index within all of the found
                              target's windows.

 @Return        Target ID code or LA_ID_NONE if no match is found.
*//***************************************************************************/
e_LaTgtId LA_WinFindId(t_Handle h_La, uint64_t address, uint32_t *p_LawNum);

/**************************************************************************//*
 @Routine       LA_Free

 @Description   Free all LA resources.

 @Param[in]     h_La - Handle to the initialized LA object.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error  LA_Free(t_Handle h_La);


#endif /* __LA_EXT_H */
