/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          law_ext.h

 @Description   Local Access Windows driver API.
*//***************************************************************************/
#ifndef __LAW_EXT_H
#define __LAW_EXT_H


#include "std_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         law_grp LAW Application Programming Interface

 @Description   Local Access Windows (LAW) driver API .

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         law_init_grp    LAW Initialization Unit

 @Description   LAW configuration and initialization API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function       LAW_ConfigAndInit

 @Description   Create the LAW object.

 @Param[in]     baseAddress - Base address of Local Access Windows registers.

 @Return        Handle to the initialized LA object.
*//***************************************************************************/
t_Handle LAW_ConfigAndInit(uint64_t baseAddress);

/**************************************************************************//**
 @Function       LAW_Free

 @Description   Free the LAW object.

 @Param[in]     h_Law - Handle to LAW object.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error LAW_Free(t_Handle h_Law);

/** @} */ /* end of law_init_grp */

/**************************************************************************//**
 @Group         law_ctrl_grp    LAW Control Unit

 @Description   LAW control API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Local Access Window Attributes
*//***************************************************************************/
typedef struct t_LawAttributes
{
    physAddress_t   physAddress;    /**< Base (physical) address of the window  */
    uint64_t        size;           /**< Size of window; must be a power of 2   */
    int             targetId;       /**< Target ID, as defined in the SoC       */
#ifndef LAW_ARCH_CCB
    uint8_t         snoopId;        /**< Coherence subdomain ID for this window */
#endif /* not LAW_ARCH_CCB */
} t_LawAttributes;


/**************************************************************************//**
 @Function       LAW_Set

 @Description   Set a specified LAW entry with given attributes.

 @Param[in]     h_Law           - Handle to LAW object.
 @Param[in]     entry           - Local access window entry to set.
 @Param[in]     p_LawAttributes - Pointer to LAW attributes structure.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error LAW_Set(t_Handle h_Law, uint8_t entry, t_LawAttributes *p_LawAttributes);

/**************************************************************************//**
 @Function       LAW_Clear

 @Description   Clears a local access window entry.

 @Param[in]     h_Law   - Handle to LAW object.
 @Param[in]     entry   - Local access window entry to clear.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error LAW_Clear(t_Handle h_Law, uint8_t entry);

/**************************************************************************//**
 @Function       LAW_Find

 @Description   Try to find an existing LAW matching a specific physical address.
 
                The user is allowed to pass a NULL pointer in p_Entry or p_TargetId
                parameters if any of these values is not required.

 @Param[in]     h_Law       - Handle to LAW object.
 @Param[in]     physAddress - Physical address for matching.
 @Param[out]    p_Entry     - If not NULL, returns the LAW entry that matched the address.
 @Param[out]    p_TargetId  - If not NULL, returns the target ID that matched the address.

 @Retval        E_OK        - Address successfully matched to the returned LAW.
 @Retval        E_NOT_FOUND - Address did not match any existing LAW.
*//***************************************************************************/
t_Error LAW_Find(t_Handle h_Law, physAddress_t physAddress, uint8_t *p_Entry, int *p_TargetId);

/**************************************************************************//**
 @Function       LAW_IsEnabled

 @Description   Check if a given LAW entry is enabled.

 @Param[in]     h_Law   - Handle to LAW object.
 @Param[in]     entry   - Local access window entry.

 @Return        TRUE if the specified LAW entry is enabled; FALSE otherwise.
*//***************************************************************************/
bool LAW_IsEnabled(t_Handle h_Law, uint8_t entry);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function       LAW_DumpRegs

 @Description   Prints the contents of LAW registers.
 
 @Param[in]     h_Law - Handle to LAW object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error LAW_DumpRegs(t_Handle h_Law);
#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of law_ctrl_grp */
/** @} */ /* end of law_grp */


#endif /* __LAW_EXT_H */

