/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          lbc_ext.h

 @Description   Local Bus Controller Application Programming Interface
*//***************************************************************************/
#ifndef __LBC_EXT_H
#define __LBC_EXT_H


#include "part_ext.h"


/**************************************************************************//**
 @Group         lbc_grp LBC Application Programming Interface

 @Description   LBC API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         lbc_init_grp LBC Initialization Unit

 @Description   LBC initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Bank port size
*//***************************************************************************/
typedef enum e_LbcPortSize
{
    e_LBC_PORT_SIZE_8_BIT   = 0x00000800,   /**< 8-bit port size */
    e_LBC_PORT_SIZE_16_BIT  = 0x00001000    /**< 16-bit port size */
#ifdef LBC_32BIT_PORT_SIZE_SUPPORT
   ,e_LBC_PORT_SIZE_32_BIT  = 0x00001800    /**< 32-bit port size */
#endif /* LBC_32BIT_PORT_SIZE_SUPPORT */
} e_LbcPortSize;


#ifdef LBC_ATOMIC_OPERATION_SUPPORT
/**************************************************************************//**
 @Description   Atomic operation selection
*//***************************************************************************/
typedef enum e_LbcAtomicOperation
{
    e_LBC_ATOMIC_OP_NONE                = 0x00000000,
                                          /**< No atomic operation */
    e_LBC_ATOMIC_OP_READ_AFTER_WRITE    = 0x00000004,
                                          /**< Read-after-write atomic operation */
    e_LBC_ATOMIC_OP_WRITE_AFTER_READ    = 0x00000008
                                          /**< Write-after-read atomic operation */
} e_LbcAtomicOperation;
#endif /* LBC_ATOMIC_OPERATION_SUPPORT */

#ifdef LBC_ADDRESS_HOLD_TIME_CTRL
/**************************************************************************//**
 @Description   Address hold time mode

                Configuration of the address hold time mode. This mode selects
                when to negate the LALE signal during address phases on the
                local bus.

                The value should be passed as the parameter of the
                LBC_ConfigAddressHoldTimeMode() routine.
*//***************************************************************************/
typedef enum e_LbcAddrHoldTime
{
    e_LBC_ADDR_HOLD_TIME_NORMAL = 0x00000000,   /**< Normal address hold time; the LALE signal negates
                                                     two platform clock periods prior to the address
                                                     being invalidated; At 666 MHz, this provides 3 ns
                                                     of additional address hold time at the external
                                                     address latch. */
    e_LBC_ADDR_HOLD_TIME_HALF   = 0x00200000    /**< Half address hold time; the LALE signal negates
                                                     one platform clock period prior to the address
                                                     being invalidated; This halves the address hold time,
                                                     but extends the latch enable duration; This may be
                                                     necessary for very high frequency designs. */
} e_LbcAddrHoldTime;
#endif /* LBC_ADDRESS_HOLD_TIME_CTRL */

#ifdef LBC_PARITY_SUPPORT
/**************************************************************************//**
 @Description   Parity Mode
*//***************************************************************************/
typedef enum e_LbcParityMode
{
    e_LBC_PARITY_ODD    = 0x00000000,   /**< Odd parity; Normal, odd-parity ECC */
    e_LBC_PARITY_EVEN   = 0x00010000    /**< Even parity; Inverted, even-parity ECC */
} e_LbcParityMode;

/**************************************************************************//**
 @Description   Data parity errors checking
*//***************************************************************************/
typedef enum e_LbcParityCheck
{
    e_LBC_PARITY_CHECK_OFF                  = 0x00000000,
                                              /**< Disable parity check */
    e_LBC_PARITY_CHECK_ON                   = 0x00000200,
                                              /**< Enable parity check */
    e_LBC_PARITY_CHECK_AND_GENERATE_FOR_RMW = 0x00000400
                                              /**< Enable parity check and
                                                   read-modify-write parity generation
                                                   (for 32-bit port size only) */
} e_LbcParityCheck;
#endif /* LBC_PARITY_SUPPORT */

/**************************************************************************//**
 @Description   Control signal functions
*//***************************************************************************/
typedef enum e_LbcCtlSignal
{
    e_LBC_USE_LBCTL_AS_W_NOT_R  = 0x00000000,   /**< Use LBCTL as write-not-read (GPCM/UPM) */
    e_LBC_USE_LBCTL_AS_NOT_LOE  = 0x00400000,   /**< Use LBCTL as not LOE (GPCM only) */
    e_LBC_USE_LBCTL_AS_NOT_LWE  = 0x00800000    /**< Use LBCTL as not LWE (GPCM only) */
} e_LbcCtlSignal;

/**************************************************************************//**
 @Description   Address to chip select setup (+ extra bit).

                Refer to the hardware spec for valid settings of these bits.
*//***************************************************************************/
typedef enum e_LbcAddrToCsSetup
{
    e_LBC_ACS_0 = 0x00000000,   /**< ACS = 00, XACS = 0 */
    e_LBC_ACS_1 = 0x00000100,   /**< ACS = 00, XACS = 1 */
    e_LBC_ACS_2 = 0x00000200,   /**< ACS = 01, XACS = 0 */
    e_LBC_ACS_3 = 0x00000300,   /**< ACS = 01, XACS = 1 */
    e_LBC_ACS_4 = 0x00000400,   /**< ACS = 10, XACS = 0 */
    e_LBC_ACS_5 = 0x00000500,   /**< ACS = 10, XACS = 1 */
    e_LBC_ACS_6 = 0x00000600,   /**< ACS = 11, XACS = 0 */
    e_LBC_ACS_7 = 0x00000700    /**< ACS = 11, XACS = 1 */
} e_LbcAddrToCsSetup;

/**************************************************************************//**
 @Description   LBC transaction type - for error management.
*//***************************************************************************/
typedef enum e_LbcTransType
{
    e_LBC_TRANS_TYPE_WRITE  = 0,    /**< Transaction type: write */
    e_LBC_TRANS_TYPE_READ   = 1     /**< Transaction type: read */
} e_LbcTransType;

/**************************************************************************//**
 @Description   UPM ID
*//***************************************************************************/
typedef enum e_LbcUpmId
{
    e_LBC_UPM_A = 0,    /**< UPMA machine */
    e_LBC_UPM_B,        /**< UPMB machine */
    e_LBC_UPM_C         /**< UPMC machine */
} e_LbcUpmId;

/**************************************************************************//**
 @Description   UPM wait signal polarity

                The value should be passed as the parameter of the
                LBC_ConfigUpmWaitSignalPolarity() routine.
*//***************************************************************************/
typedef enum e_LbcUpmWaitPolarity
{
    e_LBC_UPM_WAIT_ACTIVE_HIGH  = 0x00000000,   /**< Wait signal is active-high */
    e_LBC_UPM_WAIT_ACTIVE_LOW   = 0x08000000    /**< Wait signal is active-low */
} e_LbcUpmWaitPolarity;

/**************************************************************************//**
 @Description   UPM address multiplexing.

                Determines how the address of the current memory cycle can
                be output on the address pins. This setting is needed when
                interfacing with devices requiring row and column addresses
                multiplexed on the same pins.

                The value should be passed as the parameter of the
                LBC_ConfigUpmAddressMultiplexing() routine.
*//***************************************************************************/
typedef enum e_LbcUpmAddrMux
{
    e_LBC_UPM_ADDR_MUX_A8_A23   = 0x00000000,   /**< Multiplex address bits A8-A23 */
    e_LBC_UPM_ADDR_MUX_A7_A22   = 0x01000000,   /**< Multiplex address bits A7-A22 */
    e_LBC_UPM_ADDR_MUX_A6_A21   = 0x02000000,   /**< Multiplex address bits A6-A21 */
    e_LBC_UPM_ADDR_MUX_A5_A20   = 0x03000000,   /**< Multiplex address bits A5-A20 */
    e_LBC_UPM_ADDR_MUX_A4_A19   = 0x04000000,   /**< Multiplex address bits A4-A19 */
    e_LBC_UPM_ADDR_MUX_A3_A18   = 0x05000000    /**< Multiplex address bits A3-A18 */
} e_LbcUpmAddrMux;

/**************************************************************************//**
 @Description   UPM address line used for memory access control signal (LGPL0).

                Determines which logical address line can be output to the LGPL0
                pin when the given UPM is selected to control the memory access.

                The value should be passed as the parameter of the
                LBC_ConfigUpmCtlAddrLine() routine.
*//***************************************************************************/
typedef enum e_LbcUpmCtlAddrLine
{
    e_LBC_UPM_CTL_LINE_A12  = 0x00000000,   /**< LGPL0 control line is A12 */
    e_LBC_UPM_CTL_LINE_A11  = 0x00080000,   /**< LGPL0 control line is A11 */
    e_LBC_UPM_CTL_LINE_A10  = 0x00100000,   /**< LGPL0 control line is A10 */
    e_LBC_UPM_CTL_LINE_A9   = 0x00180000,   /**< LGPL0 control line is A9 */
    e_LBC_UPM_CTL_LINE_A8   = 0x00200000,   /**< LGPL0 control line is A8 */
    e_LBC_UPM_CTL_LINE_A7   = 0x00280000,   /**< LGPL0 control line is A7 */
    e_LBC_UPM_CTL_LINE_A6   = 0x00300000,   /**< LGPL0 control line is A6 */
    e_LBC_UPM_CTL_LINE_A5   = 0x00380000    /**< LGPL0 control line is A5 */
} e_LbcUpmCtlAddrLine;

/**************************************************************************//**
 @Description   UPM RAM loop-counter type.

                Defines types of RAM loop-counters for various UPM operations.
                These loop-counters can be set individually using the
                LBC_SetUpmRamLoopCounter() routine.
*//***************************************************************************/
typedef enum e_LbcUpmLoopCounterType
{
    e_LBC_UPM_LOOP_CNT_READ_OR_RUN, /**< Read loop-counter; determines the
                                         number of times a loop defined in
                                         the UPM RAM will be executed for a
                                         burst- or single-beat read pattern,
                                         or for a RUN command. */
    e_LBC_UPM_LOOP_CNT_WRITE,       /**< Write loop-counter; determines the
                                         number of times a loop defined in
                                         the UPM RAM will be executed for a
                                         burst- or single-beat write pattern. */
    e_LBC_UPM_LOOP_CNT_REFRESH      /**< Refresh loop-counter; determines the
                                         number of times a loop defined in
                                         the UPM RAM will be executed for a
                                         refresh service pattern. */
} e_LbcUpmLoopCounterType;


#ifdef LBC_FCM_AVAILABLE

/**************************************************************************//**
 @Description   FCM page size modes
*//***************************************************************************/
typedef enum e_LbcFcmPageSize
{
    e_LBC_FCM_PAGE_SIZE_SMALL   = 0x00000000,
                                  /**< Page size of 512 main area bytes plus 16
                                       spare area bytes (small page devices);
                                       FCM RAM buffers are 1 Kbyte each;
                                       Flash block size of 16 Kbytes. */
    e_LBC_FCM_PAGE_SIZE_LARGE   = 0x00000400
                                  /**< Page size of 2048 main area bytes plus 64
                                       spare area bytes (large page devices);
                                       FCM RAM buffers are 4 Kbytes each;
                                       Flash block size of 128 Kbytes. */
} e_LbcFcmPageSize;


/**************************************************************************//**
 @Description   Position of ECC block (for FCM).

                When hardware checking and/or generation of error correcting
                codes (ECC) is enabled in an FCM bank, and full page transfers
                are specified, sets the ECC block size and position of the ECC
                code word(s) in the NAND flash spare region.

                ECC is checked/calculated over 512-Byte blocks.

                The value should be passed as the parameter of the
                LBC_ConfigFcmEccPosition() routine.
*//***************************************************************************/
typedef enum e_LbcFcmEccPosition
{
    e_LBC_FCM_ECC_POSITION_Nx16_PLUS_6  = 0x00000000,
                                          /**< A 24-bit ECC is assigned to spare
                                               region bytes at offsets ((N*16)+6)
                                               through ((N*16)+8) for spare region
                                               N, N = 0-3. */
    e_LBC_FCM_ECC_POSITION_Nx16_PLUS_8  = 0x00000100
                                          /**< A 24-bit ECC is assigned to spare
                                               region bytes at offsets ((N*16)+8)
                                               through ((N*16)+10) for spare region
                                               N, N = 0-3. */
} e_LbcFcmEccPosition;


/**************************************************************************//**
 @Description   ECC modes (for FCM banks)
*//***************************************************************************/
typedef enum e_LbcFcmEccMode
{
    e_LBC_FCM_ECC_DISABLED              = 0x00000000,
                                          /**< ECC is disabled for FCM. */
    e_LBC_FCM_ECC_CHECK                 = 0x00000200,
                                          /**< Enable ECC checking for FCM on
                                               full-page transfers. */
    e_LBC_FCM_ECC_GENERATE_AND_CHECK    = 0x00000400
                                          /**< Enable ECC checking and generation
                                               for FCM on full-page transfers. */
} e_LbcFcmEccMode;


/**************************************************************************//**
 @Description   FCM command wait timeout.

                For FCM commands that wait on LFRB being sampled high (CW0, CW1,
                RBW and RSW), FCM pauses execution of the instruction sequence
                until either LFRB is sampled high, or a timer expires, whichever
                occurs first. The given values specify the wait timeout in local
                bus clock cycles.

                The value should be passed as the parameter of the
                LBC_ConfigFcmCommandWaitTimeout() routine.
*//***************************************************************************/
typedef enum e_LbcFcmCmdWaitTimeout
{
    e_LBC_FCM_CW_TIMEOUT_256_CYCLES     = 0x00000000,   /**< 256 cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_512_CYCLES     = 0x00001000,   /**< 512 cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_1K_CYCLES      = 0x00002000,   /**< 1K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_2K_CYCLES      = 0x00003000,   /**< 2K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_4K_CYCLES      = 0x00004000,   /**< 4K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_8K_CYCLES      = 0x00005000,   /**< 8K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_16K_CYCLES     = 0x00006000,   /**< 16K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_32K_CYCLES     = 0x00007000,   /**< 32K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_64K_CYCLES     = 0x00008000,   /**< 64K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_128K_CYCLES    = 0x00009000,   /**< 128K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_256K_CYCLES    = 0x0000A000,   /**< 256K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_512K_CYCLES    = 0x0000B000,   /**< 512K cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_1M_CYCLES      = 0x0000C000,   /**< 1M cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_2M_CYCLES      = 0x0000D000,   /**< 2M cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_4M_CYCLES      = 0x0000E000,   /**< 4M cycles of local bus clock */
    e_LBC_FCM_CW_TIMEOUT_8M_CYCLES      = 0x0000F000    /**< 8M cycles of local bus clock */

} e_LbcFcmCmdWaitTimeout;


/**************************************************************************//**
 @Description   FCM instructions

                The FCM can execute a sequence of flash instructions (up to
                eight instructions) on any FCM bank, one instruction at a time,
                starting from FCM instruction 0 and continuing with adjacent
                instructions, until only NOP opcodes remain.

                The instructions sequence can be loaded to FCM using the
                LBC_SetFcmInstructions() routine, and then executed by calling
                the LBC_RunFcm() routine.

                The CMn instructions source their command bytes from FCR[CMDn],
                therefore up to four different NAND commands can be issued in
                any FCM instruction sequence. These instructions write a single
                command byte by asserting LFCLE and LFWE, while driving an 8-bit
                command onto LAD[0:7].

                The CWn instructions first poll the LFRB pin, waiting for it to
                go high, before writing a single command byte onto LAD[0:7].
                The command byte for instruction CWn is sourced from FCR[CMDn].
                It is necessary to use these instructions whenever the device
                is expected to be in a busy state (such as following a page read,
                block erase, or program operation) and therefore initially
                unresponsive to commands.

                NAND commands (FCR[CMD0-3]) should be loaded into FCR using the
                LBC_SetFcmNandCommands() routine. The flash device manufacturers
                datasheet should be consulted to determine values for programming
                into FCR, and whether a given command in the sequence is expected
                to initiate busy device behavior.

                The UA, WS, RS and RSW instructions use MDR (FCM data register)
                to transfer data, address or status to/from the flash device.
                MDR can hold up to four bytes, that are read/written starting
                from the MSB. The MDR bytes for writing can be set using the
                cmdData parameter of LBC_RunFcm() function. The user can get MDR
                bytes that were read during the FCM instructions sequence using
                the LBC_GetFcmCmdData() function.

                The MDR read and write pointers are reset before an instructions
                sequence is executed. Subsequent MDR-related instructions in
                the same FCM sequence advance the corresponding write/read
                pointer one byte at a time. The MDR write pointer, used by UA
                and WS instructions, is independent of the MDR read pointer,
                used by RS and RSW instructions.
*//***************************************************************************/
typedef enum e_LbcFcmInstruction
{
    e_LBC_FCM_INST_NOP  = 0x0,  /**< No operation, and end of operation sequence;
                                     A NOP instruction that appears ahead of the last
                                     instruction is executed with the timing of a regular
                                     command instruction, but neither LFCLE nor LFWE are
                                     asserted, thus it may be used to insert a pause matching
                                     the time taken for a regular command write. */
    e_LBC_FCM_INST_CA   = 0x1,  /**< Issue a column address, comprising one byte (small page) or
                                     two bytes (large page), locating the starting byte or word
                                     to be transferred in the next page read or write sequence;
                                     The column index is valid only when byteCount is non-zero;
                                     In the case that byteCount is zero, a column index of zero
                                     is issued to the device. */
    e_LBC_FCM_INST_PA   = 0x2,  /**< Issue current page address, comprising of 2, 3 or 4 bytes,
                                     as configured by LBC_ConfigFcmAddresLength(), locating the
                                     data page in the flash address space. */
    e_LBC_FCM_INST_UA   = 0x3,  /**< Write a user-defined address byte from next AS field in MDR,
                                     starting at MDR[AS0]; Each subsequent UA instruction reads
                                     an adjacent AS field in MDR, until all four AS bytes have
                                     been sent; a fifth and any following UA instructions send
                                     zero as the address byte. */
    e_LBC_FCM_INST_CM0  = 0x4,  /**< Issue NAND command 0 (from FCR[CMD0]) immediately. */
    e_LBC_FCM_INST_CM1  = 0x5,  /**< Issue NAND command 1 (from FCR[CMD1]) immediately. */
    e_LBC_FCM_INST_CM2  = 0x6,  /**< Issue NAND command 2 (from FCR[CMD2]) immediately. */
    e_LBC_FCM_INST_CM3  = 0x7,  /**< Issue NAND command 3 (from FCR[CMD3]) immediately. */
    e_LBC_FCM_INST_WB   = 0x8,  /**< Write byteCount bytes of data from FCM buffer to the
                                     selected location (page & column) in the flash device;
                                     If byteCount is zero, the entire page (including the
                                     spare region) is transferred in a burst, starting at
                                     the page boundary, and the ECC calculation is stored
                                     in the spare region. */
    e_LBC_FCM_INST_WS   = 0x9,  /**< Write one byte/two bytes (8b/16b port) of data/status from
                                     next AS field of MDR to flash device; This instruction
                                     asserts LFWE exactly once; Attempts to write beyond four
                                     bytes of MDR has the effect of writing zeros. */
    e_LBC_FCM_INST_RB   = 0xA,  /**< Read byteCount bytes of data from the selected location
                                     (page & column) in the flash device to FCM buffer;
                                     If byteCount is zero, the entire page (including the
                                     spare region) is transferred in a burst, starting at
                                     the page boundary, and the ECC calculation is stored
                                     in the spare region (correctable ECC errors are corrected
                                     silently, other errors may cause an interrupt if enabled). */
    e_LBC_FCM_INST_RS   = 0xB,  /**< Read one byte/two bytes (8b/16b port) of data/status from
                                     flash device to next AS field of MDR; This instruction
                                     asserts LFRE exactly once; Reads beyond the fourth byte of
                                     MDR are discarded. */
    e_LBC_FCM_INST_CW0  = 0xC,  /**< Wait for LFRB to go high (or timeout), then issue NAND
                                     command 0 (from FCR[CMD0]). */
    e_LBC_FCM_INST_CW1  = 0xD,  /**< Wait for LFRB to go high (or timeout), then issue NAND
                                     command 1 (from FCR[CMD1]). */
    e_LBC_FCM_INST_RBW  = 0xE,  /**< Wait for LFRB to go high (or timeout), then proceed with a
                                     read as described for RB instruction; Sampling and timeout
                                     for polling LFRB follow the behavior of CW0/CW1 instructions. */
    e_LBC_FCM_INST_RSW  = 0xF   /**< Wait for LFRB to go high (or timeout), then proceed with
                                     a read as described for RS instruction; Sampling and timeout
                                     for polling LFRB follow the behavior of CW0/CW1 instructions. */
} e_LbcFcmInstruction;

#endif /* LBC_FCM_AVAILABLE */

#ifdef LBC_SDRAM_MACHINE_AVAILABLE

/**************************************************************************//**
 @Description   SDRAM page mode.

                Selects page mode for the SDRAM connected to the memory bank.
*//***************************************************************************/
typedef enum e_LbcSdramPageMode
{
    e_LBC_SDRAM_PAGE_MODE_BACK_TO_BACK  = 0x00000000,
                                        /**< Back-to-back page mode (normal operation).
                                             Page is closed when the bus becomes idle. */
    e_LBC_SDRAM_PAGE_MODE_KEEP_OPEN     = 0x00000020
                                        /**< Page is kept open until a page miss
                                             or refresh occurs. */

} e_LbcSdramPageMode;

#endif /* LBC_SDRAM_MACHINE_AVAILABLE */

/** @} */ /* end of lbc_init_grp group */


/**************************************************************************//**
 @Group         lbc_exception_grp LBC Exception Unit

 @Description   LBC Exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Data structure for reporting error attributes to the user.

                The driver fills this structure with current valid error
                attributes and passes the structure in the exceptions callback
                routine.

 @See           @ref lbc_exbm "LBC Errors Bit Mask".
*//***************************************************************************/
typedef struct t_LbcErrorReport
{
    uint32_t        errorsMask;     /**< Valid errors mask */
    bool            validInfo;      /**< TRUE if all error information is valid,
                                         FALSE if errors mask information is valid,
                                         but other error attributes are invalid. */
    uint32_t        transAddr;      /**< Address of the transaction */
    uint8_t         transSize;      /**< Transaction size (in double words). */
    e_LbcTransType  transType;      /**< Transaction type for the error. */
    bool            burst;          /**< TRUE if error was on burst transaction.*/
    e_TransSrc      transSrc;       /**< Id of the transaction source for
                                         the error. */
    uint8_t         parityLocation; /**< Indicates in which byte lane was a
                                         parity error (0-3). */
    uint8_t         bank;           /**< Indicates in which memory controller
                                         bank was the error (0-7). */
} t_LbcErrorReport;


/** @} */ /* end of lbc_exception_grp group */


/**************************************************************************//**
 @Group         lbc_init_grp LBC Initialization Unit

 @Description   LBC initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   LBC exceptions callback function type.
*//***************************************************************************/
typedef void (t_LbcExceptionsCallback)(t_Handle h_App, t_LbcErrorReport *p_ErrorReport);


/**************************************************************************//**
 @Description   Parameters structure - for initialization
*//***************************************************************************/
typedef struct t_LbcParam
{
    uint32_t                baseAddress;    /**< Base address of LBC registers in memory. */
    int                     irq;            /**< LBC interrupt ID (system-specific number). */
    bool                    loadCurrentSettings;
                                            /**< TRUE to load current settings from the registers;
                                                 FALSE to overwrite the current settings. */
    uint16_t                sysClk;         /**< System bus clock frequency (MHz) */
    uint16_t                localBusClk;    /**< Requested local bus clock frequency (MHz);
                                                 If loadCurrentSettings is TRUE, this parameter is
                                                 ignored and the bus clock is calculated internally
                                                 using the given system clock and the LBC registers;
                                                 The resulting frequency can be retrieved later
                                                 using the LBC_GetClockFrequency() routine. */
    t_Handle                h_App;          /**< Handle to an application entity, which will
                                                 be passed as the first parameter of the
                                                 exceptions callback. */
    t_LbcExceptionsCallback *f_Exceptions;  /**< User's callback routine for LBC interrupts. */

} t_LbcParam;


/**************************************************************************//**
 @Description   Structure for configuring a memory bank for UPM
*//***************************************************************************/
typedef struct t_LbcUpmBankParam
{
    uint8_t                 bankId;             /**< Memory bank (chip select) ID. */
    e_LbcUpmId              upmId;              /**< Requested UPM ID. */
    uint64_t                baseAddr;           /**< Base address of chip select. */
    uint64_t                addrMask;           /**< Memory address mask, which defines (by
                                                     masking the base address bits) the
                                                     address ranges handled by this bank;
                                                     For contiguous address range, this value
                                                     can be achieved from the
                                                     LBC_GetAddrMaskFromBankSize() routine. */
    e_LbcPortSize           portSize;           /**< Port size. */
#ifdef LBC_PARITY_SUPPORT
    e_LbcParityCheck        parityCheck;        /**< Method for parity checking. */
#endif /* LBC_PARITY_SUPPORT */
#ifdef LBC_ATOMIC_OPERATION_SUPPORT
    e_LbcAtomicOperation    atomicOp;           /**< Atomic operation */
#endif /* LBC_ATOMIC_OPERATION_SUPPORT */
    bool                    writeProtect;       /**< TRUE to deny write access to this bank. */
    bool                    bufferCtlDisable;   /**< TRUE to disable LBCTL assertion;
                                                     FALSE for normal assertion of LBCTL. */
    bool                    relaxedTiming;      /**< TRUE for relaxed UPM timing (see description
                                                     of extendReadHoldTime);
                                                     FALSE for normal UPM timing. */
    bool                    burstInhibit;       /**< TRUE to perform any burst accesses on this bank
                                                     as a series of single accesses;
                                                     FALSE to allow burst accesses on this bank.  */
    bool                    extendAddrDelay;    /**< TRUE to add extra clock cycle/s when using LALE;
                                                     FALSE for normal assertion of LALE (1 cycle). */
    bool                    extendReadHoldTime; /**< Extended hold time between read accesses:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, no additional clock
                                                     cycles are inserted between read accesses;
                                                     If relaxedTiming is TRUE, 4 idle clock cycles
                                                     are inserted between read accesses;
                                                     TRUE for extended hold time -
                                                     If relaxedTiming is FALSE, 1 idle clock cycle
                                                     is inserted between read accesses;
                                                     If relaxedTiming is TRUE, 8 idle clock cycles
                                                     are inserted between read accesses. */
 } t_LbcUpmBankParam;


/**************************************************************************//**
 @Description   Structure for configuring a memory bank for GPCM
*//***************************************************************************/
typedef struct t_LbcGpcmBankParam
{
    uint8_t                 bankId;             /**< Memory bank (chip select) ID. */
    uint64_t                baseAddr;           /**< Base address of chip select. */
    uint64_t                addrMask;           /**< Memory address mask, which defines (by
                                                     masking the base address bits) the
                                                     address ranges handled by this bank;
                                                     For contiguous address range, this value
                                                     can be achieved from the
                                                     LBC_GetAddrMaskFromBankSize() routine. */
    e_LbcPortSize           portSize;           /**< Port size. */
#ifdef LBC_PARITY_SUPPORT
    e_LbcParityCheck        parityCheck;        /**< Method for parity checking. */
#endif /* LBC_PARITY_SUPPORT */
#ifdef LBC_ATOMIC_OPERATION_SUPPORT
    e_LbcAtomicOperation    atomicOp;           /**< Atomic operation. */
#endif /* LBC_ATOMIC_OPERATION_SUPPORT */
    e_LbcAddrToCsSetup      addrToCsSetup;      /**< Address-to-chip-select setup bits (see spec) */
    uint8_t                 cycleWaitStates;    /**< Number of wait states in a bus cycle (0-15);
                                                     This number is doubled if relaxedTiming is TRUE. */
    bool                    writeProtect;       /**< TRUE to deny write access to this bank. */
    bool                    bufferCtlDisable;   /**< TRUE to disable LBCTL assertion;
                                                     FALSE for normal assertion of LBCTL. */
    bool                    relaxedTiming;      /**< TRUE for relaxed GPCM timing (see spec);
                                                     FALSE for normal GPCM timing. */
    bool                    earlyNegationTime;  /**< TRUE for early negation time of LCS and LWE;
                                                     FALSE for normal negation time. */
    bool                    externalAccessTerm; /**< TRUE for external termination of transactions;
                                                     FALSE for internal termination of transactions. */
    bool                    extendAddrDelay;    /**< TRUE to add extra clock cycle/s when using LALE;
                                                     FALSE for normal assertion of LALE (1 cycle). */
    bool                    extendReadHoldTime; /**< Extended hold time between read accesses:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, no additional clock
                                                     cycles are inserted between read accesses;
                                                     If relaxedTiming is TRUE, 4 idle clock cycles
                                                     are inserted between read accesses;
                                                     TRUE for extended hold time -
                                                     If relaxedTiming is FALSE, 1 idle clock cycle
                                                     is inserted between read accesses;
                                                     If relaxedTiming is TRUE, 8 idle clock cycles
                                                     are inserted between read accesses. */
} t_LbcGpcmBankParam;


#ifdef LBC_FCM_AVAILABLE

/**************************************************************************//**
 @Description   Structure for configuring a memory bank for FCM
*//***************************************************************************/
typedef struct t_LbcFcmBankParam
{
    uint8_t                 bankId;             /**< Memory bank (chip select) ID. */
    uint64_t                baseAddr;           /**< Base address of chip select. */
    uint64_t                addrMask;           /**< Memory address mask, which defines (by
                                                     masking the base address bits) the
                                                     address ranges handled by this bank;
                                                     For contiguous address range, this value
                                                     can be achieved from the
                                                     LBC_GetAddrMaskFromBankSize() routine. */
    e_LbcPortSize           portSize;           /**< Port size. */
    e_LbcFcmPageSize        pageSize;           /**< Page size mode. */
    e_LbcFcmEccMode         eccMode;            /**< ECC generation and checking mode. */
#ifdef LBC_ATOMIC_OPERATION_SUPPORT
    e_LbcAtomicOperation    atomicOp;           /**< Atomic operation. */
#endif /* LBC_ATOMIC_OPERATION_SUPPORT */
    uint8_t                 addrLength;         /**< Address length - sets the number of address
                                                     bytes issued during page address (PA) operations. */
    uint8_t                 cycleWaitStates;    /**< Number of wait states in a bus cycle (0-7);
                                                     This number is doubled if relaxedTiming is TRUE. */
    bool                    writeProtect;       /**< TRUE to deny write access to this bank. */
    bool                    bufferCtlDisable;   /**< TRUE to disable LBCTL assertion;
                                                     FALSE for normal assertion of LBCTL. */
    bool                    relaxedTiming;      /**< TRUE for relaxed FCM timing (see spec);
                                                     FALSE for normal FCM timing. */
    bool                    extendCsToCmdTime;  /**< Extended chip-select to command time:
                                                     FALSE to assert chip-select one clock cycle before
                                                     any command;
                                                     TRUE to assert chip-select four clock cycles before
                                                     any command;
                                                     These numbers are doubled if relaxedTiming is TRUE. */
    bool                    extendCmdSetupTime; /**< Extended command setup time:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, write-enable is asserted
                                                     coincident with any command;
                                                     If relaxedTiming is TRUE, write-enable is asserted
                                                     1/2 clock cycles after any command, address, or data;
                                                     TRUE for extended setup time -
                                                     If relaxedTiming is FALSE, write-enable is asserted
                                                     1/4 clock cycles after any command, address, or data;
                                                     If relaxedTiming is TRUE, write-enable is asserted
                                                     1 clock cycle after any command, address, or data. */
    bool                    extendCmdHoldTime;  /**< Extended command hold time:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, write-enable is negated 0.5
                                                     clock cycles before any command, address, or data change;
                                                     If relaxedTiming is TRUE, write-enable is negated 1.5
                                                     clock cycles before any command, address, or data change;
                                                     TRUE for extended hold time -
                                                     If relaxedTiming is FALSE, write-enable is negated 1.0
                                                     clock cycles before any command, address, or data change;
                                                     If relaxedTiming is TRUE, write-enable is negated 2.0
                                                     clock cycles before any command, address, or data change. */
    bool                    extendReadSetupTime;/**< Extended read setup time:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, read-enable is asserted
                                                     3/4 clock cycles prior to any wait states;
                                                     If relaxedTiming is TRUE, read-enable is asserted
                                                     1/2 clock cycles prior to any wait states;
                                                     TRUE for extended setup time -
                                                     Regardless of the value of relaxedTiming, read-enable
                                                     is asserted 1 clock cycle prior to any wait states. */
    bool                    extendReadHoldTime; /**< Extended hold time between read accesses:
                                                     FALSE for normal timing -
                                                     If relaxedTiming is FALSE, 1 idle clock cycle
                                                     is inserted between read accesses;
                                                     If relaxedTiming is TRUE, 4 idle clock cycles
                                                     are inserted between read accesses;
                                                     TRUE for extended hold time -
                                                     If relaxedTiming is FALSE, 2 idle clock cycles
                                                     are inserted between read accesses;
                                                     If relaxedTiming is TRUE, 8 idle clock cycles
                                                     are inserted between read accesses. */
} t_LbcFcmBankParam;

#endif /* LBC_FCM_AVAILABLE */

#ifdef LBC_SDRAM_MACHINE_AVAILABLE

/**************************************************************************//**
 @Description   Structure for configuring a memory bank for SDRAM
*//***************************************************************************/
typedef struct t_LbcSdramBankParam
{
    uint8_t                 bankId;             /**< Memory bank (chip select) ID. */
    uint64_t                baseAddr;           /**< Base address of chip select. */
    uint64_t                addrMask;           /**< Memory address mask, which defines (by
                                                     masking the base address bits) the
                                                     address ranges handled by this bank;
                                                     For contiguous address range, this value
                                                     can be achieved from the
                                                     LBC_GetAddrMaskFromBankSize() routine. */
#ifdef LBC_PARITY_SUPPORT
    e_LbcParityCheck        parityCheck;        /**< Method for parity checking. */
#endif /* LBC_PARITY_SUPPORT */
#ifdef LBC_ATOMIC_OPERATION_SUPPORT
    e_LbcAtomicOperation    atomicOp;           /**< Atomic operation. */
#endif /* LBC_ATOMIC_OPERATION_SUPPORT */
    bool                    writeProtect;       /**< TRUE to deny write access to this bank. */
    bool                    extendAddrDelay;    /**< TRUE to add extra clock cycle/s when using LALE;
                                                     FALSE for normal assertion of LALE (1 cycle). */
    e_LbcSdramPageMode      pageMode;           /**< Page mode selection. */
    uint8_t                 numOfColumns;       /**< Number of columns. */
    uint8_t                 numOfRows;          /**< Number of rows. */

} t_LbcSdramBankParam;


/**************************************************************************//**
 @Description   SDRAM standard timing parameters.

                These parameters match the JEDEC standard, and should be set
                according to the SDRAM device's data sheet.
*//*************************************************************************/
typedef struct t_LbcSdramTimingParam
{
    uint32_t    tRFC;   /**< Refresh recovery time (picosec);
                             Defines the earliest timing for an ACTIVATE or REFRESH
                             command after a REFRESH command to the same SDRAM device. */
    uint32_t    tRP;    /**< Precharge to activate interval (picosec);
                             Defines the earliest timing for an ACTIVATE or REFRESH
                             command after a PRECHARGE command to the same SDRAM bank. */
    uint32_t    tRCD;   /**< Activate to read/write interval (picosec);
                             Defines the earliest timing for a READ/WRITE command after
                             an ACTIVATE command to the same SDRAM bank. */
    uint32_t    tWR;    /**< Write recovery time - last data to precharge interval (picosec);
                             Defines the earliest timing for a PRECHARGE command after
                             the last data was written to the SDRAM. */
} t_LbcSdramTimingParam;


/**************************************************************************//**
 @Description   Address lines used for SDRAM bank selection

                Bank select multiplexed address line. Selects which address
                signals serve as the 2-bit bank-select address for SDRAM.
                Note that only 4-bank SDRAMs are supported.
*//*************************************************************************/
typedef enum e_LbcSdramBankSelAddr
{
    e_LBC_SDRAM_BANK_SEL_ADDR_12_13 = 0x00000000,   /**< Bank selection using LA[12:13] */
    e_LBC_SDRAM_BANK_SEL_ADDR_13_14 = 0x00200000,   /**< Bank selection using LA[13:14] */
    e_LBC_SDRAM_BANK_SEL_ADDR_14_15 = 0x00400000,   /**< Bank selection using LA[14:15] */
    e_LBC_SDRAM_BANK_SEL_ADDR_15_16 = 0x00600000,   /**< Bank selection using LA[15:16] */
    e_LBC_SDRAM_BANK_SEL_ADDR_16_17 = 0x00800000,   /**< Bank selection using LA[16:17] */
    e_LBC_SDRAM_BANK_SEL_ADDR_17_18 = 0x00A00000,   /**< Bank selection using LA[17:18] */
    e_LBC_SDRAM_BANK_SEL_ADDR_18_19 = 0x00C00000,   /**< Bank selection using LA[18:19] */
    e_LBC_SDRAM_BANK_SEL_ADDR_19_20 = 0x00E00000    /**< Bank selection using LA[19:20] */

} e_LbcSdramBankSelAddr;

#endif /* LBC_SDRAM_MACHINE_AVAILABLE */


/**************************************************************************//**
 @Function      LBC_Config

 @Description   Configures the LBC object.

                This routine must be called before any other LBC routine may
                be called.

                The input for this routine are the main configuration
                parameters for the LBC module. If the loadCurrentSettings
                parameter is set to FALSE, the driver shall assign default
                values to the remaining parameters.

                Assuming that the LBC registers may have been initialized at
                the boot sequence, the user may choose to load the existing
                configuration from the LBC registers into the ELBC software
                module (by setting the loadCurrentSettings parameter to TRUE).

                In both cases (either loading current configuration or setting
                default values), The configuration can be further modified using
                the advanced configuration routines.

                Once configuration is done, the user should call the LBC_Init()
                routine to initialize the LBC registers.

 @Param[in]     p_LbcParam - The configuration parameters for LBC.

 @Return        Handle to the created LBC object.
*//***************************************************************************/
t_Handle LBC_Config(t_LbcParam *p_LbcParam);

/**************************************************************************//**
 @Function      LBC_Init

 @Description   Initializes the driver and hardware of LBC.

                This routine initializes the LBC registers according to the
                selected configuration.

 @Param[in]     h_Lbc   - Handle to the LBC object, as returned by the
                          LBC_Config() routine.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error LBC_Init(t_Handle h_Lbc);

/**************************************************************************//**
 @Function      LBC_Free

 @Description   Frees all memory associated with the LBC object.

 @Param[in]     h_Lbc   - Handle to the LBC object, as returned by the
                          LBC_Config() routine.

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error LBC_Free(t_Handle h_Lbc);

/**************************************************************************//**
 @Function      LBC_GetAddrMaskFromBankSize

 @Description   Calculates the bank address mask for a requested bank
                (chip-select) size.

                Bank size should be 32KB - 4GB, and a power of two.

 @Param[in]     bankSize - Requested bank size.

 @Return        Bank address mask, or zero if bank size is illegal.
 *//***************************************************************************/
uint64_t LBC_GetAddrMaskFromBankSize(uint64_t bankSize);


/**************************************************************************//**
 @Group         lbc_adv_init_grp LBC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    General Configuration Functions

                These functions are not related to a specific machine type,
                although some of them may not apply to all machine types.
 @{
*//***************************************************************************/

#ifndef LBC_LOCK_LOOP_UNAVAILABLE
/**************************************************************************//**
 @Function      LBC_ConfigLockLoopBypass

 @Description   Enables/disables PLL bypass mode.

                This mode should be enabled when using low bus clock frequencies
                if the lock loop is unable to lock. When in bypass mode, incoming
                data is captured in the middle of the bus clock cycle.

                At high frequencies, the local bus PLL is used to provide improved
                hold times at external receivers, and ease set-up margins for read
                data captured by LBC.

                Deafault value - TRUE when frequency is under 80 MHz and FALSE on higher frequency.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     enabled - TRUE to enable lock loop bypass mode;
                          FALSE to disable lock loop bypass mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigLockLoopBypass(t_Handle h_Lbc, bool enabled);
#endif /* not LBC_LOCK_LOOP_UNAVAILABLE */

/**************************************************************************//**
 @Function      LBC_ConfigBusMonitorTimeout

 @Description   Configures the bus monitor timeout, in local bus clock cycles.
                [0]

                The bus monitor is provided to ensure that each bus cycle is
                terminated within a reasonable period. When a transaction starts,
                the bus monitor starts counting down from the timeout value
                until a data beat is acknowledged on the bus. It then reloads
                the timeout value and resumes the countdown until the data
                tenure completes and then idles if there is no pending
                transaction.

                The bus monitor can generate an exception if the
                #LBC_ERR_BUS_MONITOR interrupt is enabled. It is very important
                to ensure that the timeout value is not set too low; otherwise
                spurious bus timeouts may occur during normal operation,
                resulting in incomplete data transfers.

 @Param[in]     h_Lbc               - Handle to the LBC object.
 @Param[in]     busMonitorTimeout   - Timeout value in local bus clock cycles.
                                      Pass zero to configure the maximum
                                      possible timeout.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigBusMonitorTimeout(t_Handle h_Lbc, uint32_t busMonitorTimeout);

/**************************************************************************//**
 @Function      LBC_ConfigCtlSignalMode

 @Description   Configures the use of the local bus control signal.
                [e_LBC_USE_LBCTL_AS_W_NOT_R]

                The LBC provides a data buffer control signal for the local
                bus (LBCTL). This signal is activated when a GPCM, FCM, or UPM
                controlled bank is accessed. LBCTL can be disabled by setting
                bufCtlDisable in the bank parameters.

                This routine can configure LBCTL to act as an extra LWE or an
                extra LOE signal when in GPCM mode. If LBCTL is configured as a
                data buffer control (W_NOT_R), the signal is asserted (high) on
                the rising edge of the bus clock on the first cycle of the memory
                controller operation, coincident with LALE. If the access is a
                write, LBCTL remains high for the whole duration. However, if the
                access is a read, LBCTL is negated (low) with the negation of LALE
                so that the memory device is able to drive the bus.

                If back-to-back read accesses are pending, LBCTL is asserted
                (high) one bus clock cycle before the next transaction starts
                (that is, one bus clock cycle before LALE) to allow a whole bus
                cycle for the bus to turn around before the next address is driven.

                If an external bus transceiver is used, LBCTL should be used to
                signify the write direction when high. Note that the default
                (reset and bus idle) value of LBCTL is also high.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     ctlSignalMode   - Local bus control signal (LBCTL) mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigCtlSignalMode(t_Handle h_Lbc, e_LbcCtlSignal ctlSignalMode);

#ifdef LBC_ADDRESS_HOLD_TIME_CTRL
/**************************************************************************//**
 @Function      LBC_ConfigAddressHoldTimeMode

 @Description   Configures the address hold time mode.
                [e_LBC_ADDR_HOLD_TIME_NORMAL]

                The user can select to remove half of the hold time for LAD
                with respect to LALE, in order to lengthen the LALE pulse during
                address phases on the local bus. This may be necessary for very
                high frequency designs.

                This routine can be used to configure the address hold time to
                normal/half mode.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     addrHoldTime    - Address hold time mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigAddressHoldTimeMode(t_Handle h_Lbc, e_LbcAddrHoldTime addrHoldTime);
#endif /* LBC_ADDRESS_HOLD_TIME_CTRL */

/**************************************************************************//**
 @Function      LBC_ConfigExtendedAddressDelayCycles

 @Description   Configures the number of extended cycles for the assertion of
                LALE signal during address phase.
                [4]

                This address delay cycles value is relevant only for those banks
                which have the extendAddrDelay parameter set to TRUE in the bank
                parameters. Otherwise, LALE is asserted for one bus clock cycle.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     addrDelayCycles - Extended address delay cycles (1-4).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigExtendedAddressDelayCycles(t_Handle   h_Lbc,
                                             uint8_t    addrDelayCycles);

#ifdef LBC_PARITY_SUPPORT
/**************************************************************************//**
 @Function      LBC_ConfigParityMode

 @Description   Configures the parity mode (odd/even).
                [e_LBC_PARITY_ODD]

                The parity can be either odd (normal) or even (inverted).
                This mode affects the parity checking done by the local bus
                controller.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     parityMode  - Requested parity mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigParityMode(t_Handle h_Lbc, e_LbcParityMode parityMode);

/**************************************************************************//**
 @Function      LBC_ConfigParityByteSelect

 @Description   Configures the parity byte-select functionality on the
                LGTA/LFRB/LGPL4/LUPWAIT/LPBSE signal.
                [FALSE]

                This mode is provided mainly for systems that use
                read-modify-write parity, as they require an additional memory
                device and must generate a byte-select like a normal data device.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     enabled - FALSE: Parity byte-select is disabled. The LGTA/LGPL4/LPBSE
                                 signal is available for memory control as LGPL4
                                 (output) or LGTA/LFRB/LUPWAIT (input).
                          TRUE:  Parity byte-select is enabled. LPBSE signal is
                                 dedicated as the parity byte select output, and
                                 LGTA/LFRB/LUPWAIT is disabled.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigParityByteSelect(t_Handle h_Lbc, bool enabled);
#endif /* LBC_PARITY_SUPPORT */

#ifdef LBC_ADDRESS_SHIFT_SUPPORT
/**************************************************************************//**
 @Function      LBC_ConfigAddressShift

 @Description   Configures the address shift functionality.
                [FALSE]

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     enabled - FALSE: Address shift is disabled.
                          TRUE:  Address shift is enabled.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigAddressShift(t_Handle h_Lbc, bool enabled);
#endif /* LBC_ADDRESS_SHIFT_SUPPORT */

#ifdef LBC_ADDRESS_BYTE_SWAP_SUPPORT
/**************************************************************************//**
 @Function      LBC_ConfigAddressByteSwap

 @Description   Configures the address byte swap functionality.

                Address byte swap for non 32 bit port size. It swaps the LSB of
                address on the MSB od al bus.
                [FALSE]

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     enabled - FALSE: Address shift is disabled.
                          TRUE:  Address shift is enabled.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
                Mutually exclusive from address shift functionality. If both are
                enabled, the address shift functionality takes the priority.
*//***************************************************************************/
t_Error LBC_ConfigAddressByteSwap(t_Handle h_Lbc, bool enabled);
#endif /* LBC_ADDRESS_BYTE_SWAP_SUPPORT */

/**************************************************************************//**
 @Function      LBC_ConfigRefreshTimerPrescaler

 @Description   Sets the prescaler that affects the refresh timer periods.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     prescaler   - Refresh timer prescaler (allowed values are 1-256).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigRefreshTimerPrescaler(t_Handle h_Lbc, uint16_t prescaler);


/* @} */

/**************************************************************************//**
 @Collection    UPM Advanced Configuration

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      LBC_ConfigUpmRefresh

 @Description   Configures refresh services for a given UPM.
                [FALSE]

                Use the LBC_SetUpmRefreshTimerDividers() routine to set the
                refresh timer that generates the UPM refresh requests.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     upmId   - UPM ID.
 @Param[in]     enabled - TRUE to enable UPM refresh services for selected UPM;
                          FALSE if refresh services are not required.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmRefresh(t_Handle h_Lbc, e_LbcUpmId upmId, bool enabled);

/**************************************************************************//**
 @Function      LBC_ConfigUpmAccessDelayCycles

 @Description   Configures number of access delay cycles for a given UPM.
                [1]

                The UPM provides a disable timer, which guarantees a minimum time
                between accesses to the same memory bank controlled by that UPM.
                The disable timer is turned on by the TODT bit in the RAM array
                word, and when expired, the UPM allows the machine access to
                handle a memory pattern to the same bank. Accesses to a different
                bank by the same UPM are also allowed.

                To avoid conflicts between successive accesses to different banks,
                the minimum pattern in the RAM array for a request serviced,
                should not be shorter than the access delay period.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     upmId       - UPM ID.
 @Param[in]     delayCycles - Number of bus clock cycles.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmAccessDelayCycles(t_Handle     h_Lbc,
                                       e_LbcUpmId   upmId,
                                       uint8_t      delayCycles);

/**************************************************************************//**
 @Function      LBC_ConfigUpmCtlAddrLine

 @Description   Determines which logical address line can be output to the LGPL0
                pin when the given UPM is selected to control the memory access.
                [e_LBC_UPM_CTL_LINE_A12]

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     upmId       - UPM ID.
 @Param[in]     ctlAddrLine - Logical address line to be output on LGPL0.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmCtlAddrLine(t_Handle               h_Lbc,
                                 e_LbcUpmId             upmId,
                                 e_LbcUpmCtlAddrLine    ctlAddrLine);

/**************************************************************************//**
 @Function      LBC_ConfigUpmAddressMultiplexing

 @Description   Determines how the address of the current memory cycle can be
                output on the address pins.
                [e_LBC_UPM_ADDR_MUX_A8_A23]

                This setting is needed when interfacing with devices requiring
                row and column addresses multiplexed on the same pins.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     upmId   - UPM ID.
 @Param[in]     addrMux - Address multiplexing mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmAddressMultiplexing(t_Handle           h_Lbc,
                                         e_LbcUpmId         upmId,
                                         e_LbcUpmAddrMux    addrMux);

/**************************************************************************//**
 @Function      LBC_ConfigUpmWaitSignalPolarity

 @Description   Configures the polarity (active-high or active-low) of the
                LUPWAIT pin in UPM mode.
                [e_LBC_UPM_WAIT_ACTIVE_HIGH]

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     upmId           - UPM ID.
 @Param[in]     waitPolarity    - LUPWAIT signal polarity.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmWaitSignalPolarity(t_Handle                h_Lbc,
                                        e_LbcUpmId              upmId,
                                        e_LbcUpmWaitPolarity    waitPolarity);

/**************************************************************************//**
 @Function      LBC_ConfigUpmWaitInputOnGPL4

 @Description   Determines how the LGPL4/LUPWAIT pin is controlled by the
                corresponding bits in the UPM array.
                [FALSE]

                When this setting is FALSE:
                - The LGPL4/LUPWAIT line functions as LGPL4 (output).
                - The G4T1/DLT3 bit of UPM RAM words is interpreted as G4T1.
                - The G4T3/WAEN bit of UPM RAM words is interpreted as G4T3.

                When this setting is TRUE:
                - The LGPL4/LUPWAIT line functions as LUPWAIT (input).
                - The G4T1/DLT3 bit of UPM RAM words is interpreted as DLT3.
                - The G4T3/WAEN bit of UPM RAM words is interpreted as WAEN.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     upmId           - UPM ID.
 @Param[in]     waitInputOnGpl4 - TRUE to use LGPL4/LUPWAIT signal as LUPWAIT;
                                  FALSE to use LGPL4/LUPWAIT signal as LGPL4.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigUpmWaitInputOnGPL4(t_Handle   h_Lbc,
                                     e_LbcUpmId upmId,
                                     bool       waitInputOnGpl4);

/* @} */

#ifdef LBC_FCM_AVAILABLE

/**************************************************************************//**
 @Collection    FCM Advanced Configuration

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      LBC_ConfigFcmEccPosition

 @Description   Configures the position of ECC information in the NAND flash
                spare region (for FCM only).
                [e_LBC_FCM_ECC_POSITION_Nx16_PLUS_6]

                When hardware checking and/or generation of error correcting
                codes (ECC) is enabled in an FCM bank, and full page transfers
                are specified, sets the ECC block size and position of the ECC
                code word(s) in the NAND flash spare region.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     eccPosition - ECC position.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigFcmEccPosition(t_Handle               h_Lbc,
                                 e_LbcFcmEccPosition    eccPosition);

/**************************************************************************//**
 @Function      LBC_ConfigFcmCommandWaitTimeout

 @Description   Configures the timeout for FCM wait commands.
                [e_LBC_FCM_CW_TIMEOUT_8M_CYCLES]

                For FCM commands that wait on LFRB being sampled high (CW0, CW1,
                RBW and RSW), FCM pauses execution of the instruction sequence
                until either LFRB is sampled high, or a timer expires, whichever
                occurs first. The given values specify the wait timeout in local
                bus clock cycles.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     timeout - Timeout selection for FCM wait commands, local bus
                          clock cycles.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigFcmCommandWaitTimeout(t_Handle                h_Lbc,
                                        e_LbcFcmCmdWaitTimeout  timeout);

/* @} */

#endif /* LBC_FCM_AVAILABLE */

#ifdef LBC_SDRAM_MACHINE_AVAILABLE

/**************************************************************************//**
 @Collection    SDRAM Advanced Configuration

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      LBC_ConfigSdramPortSize

 @Description   Configures the port size for all SDRAM banks.
                [e_LBC_PORT_SIZE_32_BIT]

                Note that all SDRAM devices that reside on the bus should have
                the same port size and timing parameters.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     portSize    - SDRAM port size (common to all banks).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramPortSize(t_Handle h_Lbc, e_LbcPortSize portSize);

/**************************************************************************//**
 @Function      LBC_ConfigSdramRefresh

 @Description   Configures refresh service for all SDRAM banks.
                [TRUE][0x100]

 @Description   Sets the divider that determines the SDRAM refresh timer period.
                Each time the timer expires, all qualifying banks generate a
                bank staggering auto-refresh request using the SDRAM machine.

                The timer period is calculated according to the following formula:

                TimerPeriod[usec] = SystemClockFreq[MHz] / (divider * prescaler)

                The prescaler value can be configured using the
                LBC_ConfigRefreshTimerPrescaler() routine.

                The memory controller supplies AUTO-REFRESH commands to any
                connected SDRAM device according to the calculated interval.
                The refresh interval should allow for a potential collision
                between memory accesses and refresh cycles. The refresh interval
                period must be greater than the access time to ensure that read
                and write operations complete successfully.

                There are two levels of refresh request prioritylow and high.
                The low priority request is generated as soon as the refresh timer
                expires; this request is granted only if no other requests to the
                memory controller are pending. If the request is not granted
                (memory controller is busy) and the refresh timer expires two more
                times, the request becomes high priority and is served when the
                current memory controller operation finishes.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     enabled - TRUE to enable SDRAM refresh; FALSE to disable it.
 @Param[in]     divider - Refresh divider (allowed values are 1-256).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramRefresh(t_Handle h_Lbc,
                               bool     enabled,
                               uint16_t divider);

/**************************************************************************//**
 @Function      LBC_ConfigSdramTiming

 @Description   Configures the SDRAM timing values for all SDRAM banks.

                See the documentation of #t_LbcSdramTimingParam for defined
                default timing values.

                The timing parameters include the following:
                - Precharge to activate interval
                - Activate to read/write interval
                - Write recovery, last data in to precharge
                - Refresh recovery interval

                In addition, the LBC hardware ensures a default activate to
                precharge interval of 10 bus cycles.

                Note that all SDRAM devices that reside on the bus should have
                the same port size and timing parameters.

 @Param[in]     h_Lbc               - Handle to the LBC object.
 @Param[in]     p_SdramTimingParam  - Pointer to SDRAM timing parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramTiming(t_Handle              h_Lbc,
                              t_LbcSdramTimingParam *p_SdramTimingParam);

/**************************************************************************//**
 @Function      LBC_ConfigSdramBankSelectAddrLines

 @Description   Selects which address signals serve as the SDRAM bank-select.
                [e_LBC_SDRAM_BANK_SEL_ADDR_12_13]

                Note that only 4-bank SDRAMs are supported.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     bankSelAddr - Bank select address lines.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramBankSelectAddrLines(t_Handle                 h_Lbc,
                                           e_LbcSdramBankSelAddr    bankSelAddr);

/**************************************************************************//**
 @Function      LBC_ConfigSdramExtraCtlDelayCycles

 @Description   Configures additional delay cycles for SDRAM control signals.
                [0]

                This value defines the number of cycles to be added for each
                SDRAM command. If external buffers are placed on the control
                lines going to both the SDRAM and address lines, selecting a
                non-zero value of extraCtlDelayCycles causes all SDRAM control
                lines except LCSn, LCKE, LALE and LSDDQM[0:3] to be asserted
                for the given number of cycles.

                If extraCtlDelayCycles is set to zero (default mode), the control
                lines shall have normal timing (i.e. one cycle).

 @Param[in]     h_Lbc               - Handle to the LBC object.
 @Param[in]     extraCtlDelayCycles - Additional delay cycles for control signals.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramExtraCtlDelayCycles(t_Handle h_Lbc,
                                           uint8_t  extraCtlDelayCycles);

/**************************************************************************//**
 @Function      LBC_ConfigSdramCasLatency

 @Description   Configures the CAS latency value for all SDRAM banks.
                [4]

                This value defines the timing for first read data after SDRAM
                samples a column address..

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     casLatency  - CAS latency.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Config() and before LBC_Init().
*//***************************************************************************/
t_Error LBC_ConfigSdramCasLatency(t_Handle h_Lbc, uint8_t casLatency);

/* @} */

#endif /* LBC_SDRAM_MACHINE_AVAILABLE */

/** @} */ /* end of lbc_adv_init_grp */
/** @} */ /* end of lbc_init_grp */


/**************************************************************************//**
 @Group         lbc_ctrl_grp LBC Control Unit

 @Description   LBC control unit API functions, definitions and enums

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      LBC_SetUpmRefreshTimerDivider

 @Description   Sets the divider that determines the UPM refresh timer period.

                The timer period is calculated according to the following formula:

                TimerPeriod[usec] = SystemClockFreq[MHz] / (divider * prescaler)

                The prescaler value can be configured using the
                LBC_ConfigRefreshTimerPrescaler() routine.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     divider - Refresh divider (allowed values are 1-256).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetUpmRefreshTimerDivider(t_Handle h_Lbc, uint16_t divider);

/**************************************************************************//**
 @Function      LBC_SetUpmOutputMemoryAddress

 @Description   Sets the UPM memory address for output (MAR).

                The given address can be output to the address signals under
                control of the AMX bits in the UPM RAM word.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     addr    - Memory address.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetUpmOutputMemoryAddress(t_Handle h_Lbc, uint32_t addr);

/**************************************************************************//**
 @Function      LBC_SetUpmRamLoopCounter

 @Description   Sets loop-counters for various UPM operations.

                The UPM defines three types of RAM loop-counters, which can be
                set individually using this routine.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     upmId           - UPM ID.
 @Param[in]     loopCounterType - UPM loop-counter type.
 @Param[in]     loopCount       - Loop count value (indicates the number of
                                  times to execute the loop).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetUpmRamLoopCounter(t_Handle                   h_Lbc,
                                 e_LbcUpmId                 upmId,
                                 e_LbcUpmLoopCounterType    loopCounterType,
                                 uint16_t                   loopCount);

/**************************************************************************//**
 @Function      LBC_SetGpcmBank

 @Description   Configure of a specific memory bank (chip select) to operate
                with the GPCM.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     p_GpcmBankParam - GPCM bank (chip select) parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetGpcmBank(t_Handle h_Lbc, t_LbcGpcmBankParam *p_GpcmBankParam);

/**************************************************************************//**
 @Function      LBC_SetUpmBank

 @Description   Configure of a specific memory bank (chip select) to operate
                with one of the UPMs.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     p_UpmBankParam  - UPM bank (chip select) parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetUpmBank(t_Handle h_Lbc, t_LbcUpmBankParam *p_UpmBankParam);

#ifdef LBC_FCM_AVAILABLE

/**************************************************************************//**
 @Function      LBC_SetFcmBank

 @Description   Configure of a specific memory bank (chip select) to operate
                with the FCM.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     p_FcmBankParam  - FCM bank (chip select) parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetFcmBank(t_Handle h_Lbc, t_LbcFcmBankParam *p_FcmBankParam);

#endif /* LBC_FCM_AVAILABLE */

#ifdef LBC_SDRAM_MACHINE_AVAILABLE

/**************************************************************************//**
 @Function      LBC_SetSdramBank

 @Description   Configure of a specific memory bank (chip select) to operate
                with the SDRAM machine.

 @Param[in]     h_Lbc               - Handle to the LBC object.
 @Param[in]     p_SdramBankParam    - SDRAM bank (chip select) parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetSdramBank(t_Handle h_Lbc, t_LbcSdramBankParam *p_SdramBankParam);

#endif /* LBC_SDRAM_MACHINE_AVAILABLE */

/**************************************************************************//**
 @Function      LBC_SetBankWriteProtection

 @Description   Disables a specific memory bank (chip select).

                The user must invoke either LBC_SetGpcmBank(), LBC_SetUpmBank()
                or LBC_SetFcmBank() to re-enable a disabled bank.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     bankId  - Memory bank (chip select) ID.
 @Param[in]     enabled - TRUE: enable write protection for the given bank;
                          FALSE: disable write protection for the given bank.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetBankWriteProtection(t_Handle h_Lbc, uint8_t bankId, bool enabled);


/**************************************************************************//**
 @Anchor        lbc_upmrw

 @Collection    LBC UPM RAM Words

                These values can be combined (bitwise or) to form UPM RAM words.
                RAM words can be written to the UPM RAM using LBC_WriteUpmRam()
                routine, and read from the UPM RAM using the LBC_ReadUpmRam()
                routine.

                The RAM array for each UPM is 64 locations deep and 32 bits wide.
                The RAM word is a 32-bit microinstruction stored in one of 64
                locations in the RAM array. It specifies timing for external
                signals controlled by the UPM.

                When the system clock divider (see LBC_ConfigSystemClockDivider())
                is 4 or up, the CSTn and BSTn bits determine the state of UPM
                signals LCSn and LBS[0:3] at each quarter phase of the bus clock.

                When the system clock divider is 2, CST2 and CST4 are ignored
                and the external has the values defined by CST1 and CST3 but
                extended to half the clock cycle in duration. The same
                interpretation occurs for the BSTn bits.
 @{
*//***************************************************************************/
#define LBC_UPMRW_CST1          0x80000000
                                /**< Chip select timing 1 - Defines the state (0 or 1) of LCSn
                                     during bus clock quarter phase 1 if LCRR[CLKDIV] = 4 or 8;
                                     Defines the state (0 or 1) of LCSn during bus clock half
                                     phase 1 if LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_CST2          0x40000000
                                /**< Chip select timing 2 - Defines the state (0 or 1) of LCSn
                                     during bus clock quarter phase 2 if LCRR[CLKDIV] = 4 or 8;
                                     Ignored when LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_CST3          0x20000000
                                /**< Chip select timing 3 - Defines the state (0 or 1) of LCSn
                                     during bus clock quarter phase 3 if LCRR[CLKDIV] = 4 or 8;
                                     Defines the state (0 or 1) of LCSn during bus clock half
                                     phase 2 if LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_CST4          0x10000000
                                /**< Chip select timing 4 - Defines the state (0 or 1) of LCSn
                                     during bus clock quarter phase 4 if LCRR[CLKDIV] = 4 or 8;
                                     Ignored when LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_BST1          0x08000000
                                /**< Byte select timing1 - Defines the state (0 or 1) of LBS
                                     during bus clock quarter phase 1 (LCRR[CLKDIV] = 4 or 8)
                                     or bus clock half phase 1 (LCRR[CLKDIV] = 2), in conjunction
                                     with BRn[PS] and LA[24:25]. */
#define LBC_UPMRW_BST2          0x04000000
                                /**< Byte select timing 2 - Defines the state (0 or 1) of LBS
                                     during bus clock quarter phase 2 (LCRR[CLKDIV] = 4 or 8),
                                     in conjunction with BRn[PS] and LA[24:25]; Ignored when
                                     LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_BST3          0x02000000
                                /**< Byte select timing 3 - Defines the state (0 or 1) of LBS
                                     during bus clock quarter phase 3 (LCRR[CLKDIV] = 4 or 8)
                                     or bus clock half phase 2 (LCRR[CLKDIV] = 2), in conjunction
                                     with BRn[PS] and LA[24:25]. */
#define LBC_UPMRW_BST4          0x01000000
                                /**< Byte select timing 4 - Defines the state (0 or 1) of LBS
                                     during bus clock quarter phase 4 (LCRR[CLKDIV] = 4 or 8),
                                     in conjunction with BRn[PS] and LA[24:25]; Ignored when
                                     LCRR[CLKDIV] = 2. */
#define LBC_UPMRW_G0L_0         0x00800000
                                /**< General purpose line 0 lower (LGPL0 during the bus clock
                                     first half phase) is 0 */
#define LBC_UPMRW_G0L_1         0x00C00000
                                /**< General purpose line 0 lower (LGPL0 during the bus clock
                                     first half phase) is 1 */
#define LBC_UPMRW_G0L_G0CL      0x00000000
                                /**< General purpose line 0 lower (LGPL0 during the bus clock
                                     first half phase) is defined by MxMR[G0CL]. */
#define LBC_UPMRW_G0H_0         0x00200000
                                /**< General purpose line 0 higher (LGPL0 during the bus clock
                                     second half phase) is 0 */
#define LBC_UPMRW_G0H_1         0x00300000
                                /**< General purpose line 0 higher (LGPL0 during the bus clock
                                     second half phase) is 1 */
#define LBC_UPMRW_G0H_G0CL      0x00000000
                                /**< General purpose line 0 higher (LGPL0 during the bus clock
                                     second half phase) is defined by MxMR[G0CL]. */
#define LBC_UPMRW_G1T1          0x00080000
                                /**< General purpose line 1 timing 1 - Defines the state (0 or 1)
                                     of LGPL1 during first half phase. */
#define LBC_UPMRW_G1T3          0x00040000
                                /**< General purpose line 1 timing 3 - Defines the state (0 or 1)
                                     of LGPL1 during second half phase. */
#define LBC_UPMRW_G2T1          0x00020000
                                /**< General purpose line 2 timing 1 - Defines the state (0 or 1)
                                     of LGPL2 during first half phase. */
#define LBC_UPMRW_G2T3          0x00010000
                                /**< General purpose line 2 timing 3 - Defines the state (0 or 1)
                                     of LGPL2 during second half phase. */
#define LBC_UPMRW_G3T1          0x00008000
                                /**< General purpose line 3 timing 1 - Defines the state (0 or 1)
                                     of LGPL3 during first half phase. */
#define LBC_UPMRW_G3T3          0x00004000
                                /**< General purpose line 3 timing 3 - Defines the state (0 or 1)
                                     of LGPL3 during second half phase. */
#define LBC_UPMRW_G4T1_DLT3     0x00002000
                                /**< General purpose line 4 timing 1/delay time 3 - The function of
                                     this bit is determined by MxMR[GPL4]; If MxMR[GPL4] = 0 and
                                     LGPL4/LUPWAIT pin functions as an output (LGPL4), G4T1/DLT3
                                     defines the state (0 or 1) of LGPL4 during first half phase;
                                     If MxMR[GPL4] = 1 and LGPL4/LUPWAIT functions as an input
                                     (LUPWAIT), if a read burst or single read is executed, G4T1/DLT3
                                     defines the sampling of the data bus as follows:
                                     0 - In the current word, the data bus should be sampled at the
                                     start of bus clock quarter phase 1 of the next bus clock cycle;
                                     1 - In the current word, the data bus should be sampled at the
                                     start of bus clock quarter phase 3 of the current bus clock cycle. */
#define LBC_UPMRW_G4T3_WAEN     0x00001000
                                /**< General purpose line 4 timing 3/wait enable - Bit function is
                                     determined by MxMR[GPL4]; If MxMR[GPL4] = 0 and LGPL4/LUPWAIT
                                     pin functions as an output (LGPL4), G4T3/WAEN defines the state
                                     (0 or 1) of LGPL4 during second half phase; If MxMR[GPL4] = 1
                                     and LGPL4/LUPWAIT functions as an input (LUPWAIT), G4T3/WAEN is
                                     used to enable the wait mechanism:
                                     0 - LUPWAIT detection is disabled;
                                     1 - LUPWAIT is enabled;
                                     If LUPWAIT is detected as being asserted, a freeze in the external
                                     signals logical values occurs until LUPWAIT is detected as being
                                     negated. */
#define LBC_UPMRW_G5T1          0x00000800
                                /**< General purpose line 5 timing 1 - Defines the state (0 or 1) of
                                     LGPL5 during first half phase. */
#define LBC_UPMRW_G5T3          0x00000400
                                /**< General purpose line 5 timing 3 - Defines the state (0 or 1) of
                                     LGPL5 during second half phase. */
#define LBC_UPMRW_REDO_1        0x00000000
                                /**< Redo current RAM word once (normal operation). */
#define LBC_UPMRW_REDO_2        0x00000100
                                /**< Redo current RAM word twice. */
#define LBC_UPMRW_REDO_3        0x00000200
                                /**< Redo current RAM word three times. */
#define LBC_UPMRW_REDO_4        0x00000300
                                /**< Redo current RAM word four times. */
#define LBC_UPMRW_LOOP          0x00000080
                                /**< Loop start/end - The first RAM word in the RAM array where LOOP
                                     is 1 is recognized as the loop start word; The next RAM word where
                                     LOOP is 1 is the loop end word; RAM words between, and including
                                     the start and end words, are defined as part of the loop;
                                     The number of times the UPM executes this loop is defined in the
                                     corresponding loop fields of the MxMR. */
#define LBC_UPMRW_EXEN          0x00000040
                                /**< Exception enable - Allows branching to an exception pattern at
                                     the exception start address (EXS); When an internal bus monitor
                                     time-out exception is recognized and EXEN in the RAM word is set,
                                     the UPM branches to the special exception start address (EXS) and
                                     begins operating as the pattern defined there specifies;
                                     The user should provide an exception pattern to negate signals
                                     controlled by the UPM in a controlled fashion; For DRAM control,
                                     a handler should negate RAS and CAS to prevent data corruption;
                                     If EXEN = 0, exceptions are ignored by UPM (but not by Local Bus)
                                     and execution continues; After the UPM branches to the exception
                                     start address, it continues reading until the LAST bit is set in
                                     the RAM word. */
#define LBC_UPMRW_AMX_NONE      0x00000000
                                /**< Address multiplexing - Determines the source of LAD[0:15] during
                                     a LALE phase (and in conjunction with LA[16:25]) is the
                                     non-multiplexed address; For example, column address. */
#define LBC_UPMRW_AMX_AM        0x00000020
                                /**< Address multiplexing - Determines the source of LAD[0:15] during
                                     a LALE phase (and in conjunction with LA[16:25]) is the address
                                     multiplexed according to MxMR[AM]; For example, row address. */
#define LBC_UPMRW_AMX_MAR       0x00000030
                                /**< Address multiplexing - Determines the source of LAD[0:15] during
                                     a LALE phase (and in conjunction with LA[16:25]) the contents of
                                     MAR; Used, for example, to initialize a mode. */
#define LBC_UPMRW_NA            0x00000008
                                /**< Next burst address - Determines when the address is incremented
                                     during a burst access:
                                     0 - The address increment function is disabled;
                                     1 - The address is incremented in the next cycle;
                                     In conjunction with the BRn[PS], the increment value of LA[21:25]
                                     is 1 or 2 for port sizes of 8 and 16 bits, respectively. */
#define LBC_UPMRW_UTA           0x00000004
                                /**< UPM transfer acknowledge - Indicates if transfer acknowledge is
                                     asserted in the current cycle. */
#define LBC_UPMRW_TODT          0x00000002
                                /**< Turn-on disable timer - The disable timer associated with each
                                     UPM allows a minimum time to be guaranteed between two successive
                                     accesses to the same memory bank; This feature is critical when
                                     DRAM requires a RAS precharge time; TODT turns the timer on to
                                     prevent another UPM access to the same bank until the timer expires;
                                     The disable timer period is determined in MxMR[DSn]; The disable
                                     timer does not affect memory accesses to different banks; Note that
                                     TODT must be set together with LAST, otherwise it is ignored. */
#define LBC_UPMRW_LAST          0x00000001
                                /**< Last word - When LAST is read in a RAM word, the current UPM pattern
                                     terminates and control signal timing set in the RAM word is applied
                                     to the current (and last) cycle; However, if the disable timer is
                                     activated and the next access is to the same bank, execution of the
                                     next UPM pattern is held off and the control signal values specified
                                     in the last word are extended in duration for the number of clock
                                     cycles specified in MxMR[DSn]. */
/* @} */


/**************************************************************************//**
 @Function      LBC_WriteUpmRam

 @Description   Writes one or more words to the UPM RAM.

                This routine allows the user to write any number (up to 64) of
                words to the UPM RAM. Each RAM word is a 32-bit microinstruction
                that specifies timing for external signals controlled by the UPM.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     upmId       - UPM ID.
 @Param[in]     ramIndex    - Start index (RAM word index) for writing.
 @Param[in]     numOfWords  - Number of RAM words to write.
 @Param[in]     p_Words     - Array of RAM words (array size must be numOfWords).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().

 @See           @ref lbc_upmrw "LBC UPM RAM Words".
*//***************************************************************************/
t_Error LBC_WriteUpmRam(t_Handle    h_Lbc,
                        e_LbcUpmId  upmId,
                        uint8_t     ramIndex,
                        uint8_t     numOfWords,
                        uint32_t    *p_Words);

/**************************************************************************//**
 @Function      LBC_ReadUpmRam

 @Description   Reads one or more words from the UPM RAM.

                This routine allows the user to read any number (up to 64) of
                words from the UPM RAM. Each RAM word is a 32-bit microinstruction
                that specifies timing for external signals controlled by the UPM.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     upmId       - UPM ID.
 @Param[in]     ramIndex    - Start index (RAM word index) for reading.
 @Param[in]     numOfWords  - Number of RAM words to read.
 @Param[out]    p_Words     - Array to fill with RAM words (array size must be
                              numOfWords).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().

 @See           @ref lbc_upmrw "LBC UPM RAM Words".
*//***************************************************************************/
t_Error LBC_ReadUpmRam(t_Handle     h_Lbc,
                       e_LbcUpmId   upmId,
                       uint8_t      ramIndex,
                       uint8_t      numOfWords,
                       uint32_t     *p_Words);

/**************************************************************************//**
 @Function      LBC_RunUpmPattern

 @Description   Executes a UPM RAM pattern (set of instructions, starting from
                a known RAM index) on a given memory bank.

                Software can start a request to the UPM by issuing a RUN command
                to the UPM. Some memory devices have their own signal handshaking
                protocol to put them into special modes, such as self-refresh mode.

                For these special cycles, the user creates a special RAM pattern
                (using the LBC_WriteUpmRam() routine) that can be stored at any
                unused area in the UPM RAM. Then a RUN command is used (by calling
                this routine) to run the cycle. The UPM runs the pattern beginning
                at the specified RAM location until it encounters a RAM word with
                its LAST bit set.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     bankId      - Memory bank (chip select) ID.
 @Param[in]     ramIndex    - Index (RAM word index) of the pattern to run.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().

 @See           @ref lbc_upmrw "LBC UPM RAM Words".
*//***************************************************************************/
t_Error LBC_RunUpmPattern(t_Handle h_Lbc, uint8_t bankId, uint8_t ramIndex);

#ifdef LBC_FCM_AVAILABLE

/**************************************************************************//**
 @Function      LBC_SetFcmNandCommands

 @Description   Loads NAND commands to the FCM.

                This routine allows loading up to four NAND commands into the
                flash commands register (FCR[CMD0-3]). FCM instructions may
                then write any of these commands by asserting LFCLE and LFWE,
                while driving the 8-bit command onto LAD[0:7].

                The flash device manufacturers datasheet should be consulted to
                determine the values for these commands, and whether a given
                command in the sequence is expected to initiate busy device
                behavior.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     nandCmd0    - NAND command 0 (FCR[CMD0]).
 @Param[in]     nandCmd1    - NAND command 1 (FCR[CMD1]).
 @Param[in]     nandCmd2    - NAND command 2 (FCR[CMD2]).
 @Param[in]     nandCmd3    - NAND command 3 (FCR[CMD3]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetFcmNandCommands(t_Handle h_Lbc,
                               uint8_t  nandCmd0,
                               uint8_t  nandCmd1,
                               uint8_t  nandCmd2,
                               uint8_t  nandCmd3);

/**************************************************************************//**
 @Function      LBC_SetFcmInstructions

 @Description   Loads instructions into the FCM.

                This routine allows loading up to eight FCM instructions into
                the FCM instructions register (FIR[OP0-7]).

                When the user invokes LBC_RunFcm(), FCM executes the loaded
                instructions on the specified bank, one instruction at a time,
                starting from inst0 and continuing with adjacent instructions,
                until only NOP opcodes remain.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     inst0   - FCM instruction 0.
 @Param[in]     inst1   - FCM instruction 1.
 @Param[in]     inst2   - FCM instruction 2.
 @Param[in]     inst3   - FCM instruction 3.
 @Param[in]     inst4   - FCM instruction 4.
 @Param[in]     inst5   - FCM instruction 5.
 @Param[in]     inst6   - FCM instruction 6.
 @Param[in]     inst7   - FCM instruction 7.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetFcmInstructions(t_Handle             h_Lbc,
                               e_LbcFcmInstruction  inst0,
                               e_LbcFcmInstruction  inst1,
                               e_LbcFcmInstruction  inst2,
                               e_LbcFcmInstruction  inst3,
                               e_LbcFcmInstruction  inst4,
                               e_LbcFcmInstruction  inst5,
                               e_LbcFcmInstruction  inst6,
                               e_LbcFcmInstruction  inst7);

/**************************************************************************//**
 @Function      LBC_RunFcm

 @Description   Executes FCM instructions sequence.

                This routine causes the FCM to execute the previously loaded
                instructions on the specified bank, one instruction at a time,
                starting from FCM instruction 0 and continuing with adjacent
                instructions, until only NOP opcodes remain.

                The user must first program up to eight FCM instructions using
                the LBC_SetFcmInstructions() rotine, and possibly load NAND
                commands using the LBC_SetFcmNandCommands() routine.

                The user may request to transfer data to/from the main region
                or the spare region of the specified NAND flash page, according
                to the value of spareRegion. The byteCount parameter indicates
                the amount of bytes to transfer. Use byteCount of zero to
                indicate a full-page transfer, including the spare region.
                In this case, the spareRegion parameter is ignored.

                The column parameter indexes the first byte to transfer to/from
                the main or spare region of the NAND flash page and corresponding
                transfer buffer. If byteCount is zero (to indicate a full-page
                transfer), column is treated as zero.

                Valid column ranges for small-page NAND devices are (0-511) for
                main region and (0-15) for spare region.
                Valid column ranges for large-page NAND devices are (0-2047) for
                main region and (0-63) for spare region.

                The UA, WS instructions use MDR (FCM data register) to transfer
                data, address or status to the flash device. MDR can hold up to
                four bytes, that are written starting from the MSB. MDR bytes for
                writing should be set using the cmdData parameter before issuing
                any of the above instructions.

                The RS and RSW instructions use MDR to transfer data, address or
                status from the flash device. The four MDR bytes are read starting
                from the MSB. The user can get MDR bytes that were read during the
                instructions sequence using the LBC_GetFcmCmdData() function.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     bankId          - Memory bank (chip select) to act upon.
 @Param[in]     page            - NAND flash page to access.
 @Param[in]     spareRegion     - TRUE: Access the spare region of the flash page;
                                  FALSE: Access the main region of the flash page.
 @Param[in]     column          - Column (byte) index to access in the flash page.
 @Param[in]     byteCount       - Number of bytes to write/read in the transaction.
 @Param[in]     cmdData         - Data for writing in the sequence (command argument).
 @Param[in]     writeProtection - Indicates whether to execute the instruction
                                  sequence on the specified bank with write
                                  protection enabled, or not.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_RunFcm(t_Handle h_Lbc,
                   uint8_t  bankId,
                   uint16_t page,
                   bool     spareRegion,
                   uint16_t column,
                   uint16_t byteCount,
                   uint32_t cmdData,
                   bool     writeProtection);

/**************************************************************************//**
 @Function      LBC_GetFcmCmdData

 @Description   Read the contents of FCM data register (MDR), following the
                execution of FCM instructions sequence.

                This routine should be called only after the LBC_RunFcm() has
                been invoked to execute a command which reads data into the
                FCM data register (MDR).

                The RS and RSW instructions use MDR (FCM data register) to
                transfer data, address or status from the flash device. MDR can
                hold up to four bytes, that are read starting from the MSB.
                This routine allows the user to get MDR bytes that were read
                during the instructions sequence.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[out]    p_CmdData   - Gets the current value of MDR.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_GetFcmCmdData(t_Handle h_Lbc, uint32_t *p_CmdData);

/**************************************************************************//**
 @Function      LBC_SimulateFcmBootLoading

 @Description   Simulates boot-block loading from the specified memory bank.

                This routine simulates the power-on-reset boot block loading
                procedure, carried out by FCM, outside of reset. The port size,
                NAND flash attributes, and timing parameters used for the
                simulated boot are those of the specified memory bank, although
                in practice only bank 0 can be used for actual power-on-reset
                boot block loading. The specified memory bank must be configured
                before this routine is called.

                During boot loading, FCM performs a sequence of random-access
                page reads, reading entire pages from the boot block until 4
                Kbytes have been saved to the FCM buffer RAM. The ECC of each
                512-byte region is verified and single-bit errors are corrected
                if possible. following the boot block load, the FCM buffer RAM
                contains the 4-Kbyte boot block mapped contiguously.

                The user must call the LBC_ExitFromFcmBootLoading() routine
                after the boot block has been utilized, to resume normal operation
                of the FCM.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     bankId      - Memory bank (chip select) to "boot" from.
 @Param[in]     bootBlock   - NAND flash block, from which FCM starts searching
                              for a valid boot block to load (usually 0).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SimulateFcmBootLoading(t_Handle h_Lbc,
                                   uint8_t  bankId,
                                   uint16_t bootBlock);

/**************************************************************************//**
 @Function      LBC_ExitFromFcmBootLoading

 @Description   Tells FCM to exit from the boot-block loading state, and return
                to normal operation.

                The user must call this routine following boot block loading,
                after the boot block has been utilized, to resume normal
                operation of the FCM.

                Refer to LBC_SimulateFcmBootLoading() routine for more details.

 @Param[in]     h_Lbc - Handle to the LBC object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_ExitFromFcmBootLoading(t_Handle h_Lbc);

#endif /* LBC_FCM_AVAILABLE */

#ifdef LBC_SDRAM_MACHINE_AVAILABLE

/**************************************************************************//**
 @Function      LBC_EnableSdram

 @Description   Enables an SDRAM device on a selected bank.

                This routine should be called following the LBC initialization,
                for each of the SDRAM banks. The routine  executes the following
                initialization sequence on the selected SDRAM bank:
                - Issue a PRECHARGE-ALL-BANKS command
                - Issue eight AUTO-REFRESH commands
                - Issue a MODE-SET command to initialize the mode register

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     bankId  - Memory bank (chip select) ID.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().

 @Cautions      The application should ensure that no memory operations begin
                on SDRAM before this routine is called.
*//***************************************************************************/
t_Error LBC_EnableSdram(t_Handle h_Lbc, uint8_t bankId);

#endif /* LBC_SDRAM_MACHINE_AVAILABLE */


/**************************************************************************//**
 @Function      LBC_EnableErrorChecking

 @Description   Enables error checking for one or more error types.

                This routine does not impact error types that are not included
                in the given errors mask.

                Note that the driver's default setting is to check all errors
                and enable interrupts generation when they occur.

 @Param[in]     h_Lbc           - Handle to the LBC object.
 @Param[in]     errorsMask      - Indicates which error types to enable.
                                  This is a bit mask combination of error
                                  type definitions.
 @Param[in]     enableInterrupt - TRUE to generate an interrupt when one of
                                  the given error types occur;
                                  FALSE to report the given error types
                                  without generating an interrupt.

 @Return        E_OK on success; Error code otherwise.

 @See           @ref lbc_exbm "LBC Errors Bit Mask".
*//***************************************************************************/
t_Error LBC_EnableErrorChecking(t_Handle    h_Lbc,
                                uint32_t    errorsMask,
                                bool        enableInterrupt);

/**************************************************************************//**
 @Function      LBC_DisableErrorChecking

 @Description   Disables error checking for one or more error types.

                This routine does not impact error types that are not included
                in the given errors mask.

                Note that the driver's default setting is to check all errors
                and enable interrupts generation when they occur.

 @Param[in]     h_Lbc       - Handle to the LBC object.
 @Param[in]     errorsMask  - Indicates which error types to disable. This is
                              a bit mask combination of error type definitions.

 @Return        E_OK on success; Error code otherwise.

 @See           @ref lbc_exbm "LBC Errors Bit Mask".
*//***************************************************************************/
t_Error LBC_DisableErrorChecking(t_Handle h_Lbc, uint32_t errorsMask);

/**************************************************************************//**
 @Function      LBC_GetClockFrequency

 @Description   Returns the local bus clock frequency (MHz).

                This function may be called at any time following LBC_Config().

 @Param[in]     h_Lbc   - Handle to the LBC object.

 @Return        The local bus clock frequency (MHz).
*//***************************************************************************/
uint32_t LBC_GetClockFrequency(t_Handle h_Lbc);

/**************************************************************************//**
 @Function      LBC_SetClockFrequency

 @Description   Changes the local bus clock frequency.

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     clkFreq - New clock frequency (in MHz).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_SetClockFrequency(t_Handle h_Lbc, uint16_t clkFreq);

/**************************************************************************//**
 @Function      LBC_InRange

 @Description   Tests whether a given address is in the address range of a
                specified memory bank (chip select).

 @Param[in]     h_Lbc   - Handle to the LBC object.
 @Param[in]     bankId  - Memory bank (chip select) ID.
 @Param[in]     address - Address to be checked.

 @Return        TRUE if address was matched to the bank, FALSE otherwise.
*//***************************************************************************/
bool LBC_InRange(t_Handle h_Lbc, uint8_t bankId, uint64_t address);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      LBC_DumpRegs

 @Description   Prints all LBC registers.

 @Param[in]     h_Lbc   - Handle to the LBC object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following LBC_Init().
*//***************************************************************************/
t_Error LBC_DumpRegs(t_Handle h_Lbc);

#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of lbc_ctrl_grp group */
/** @} */ /* end of lbc_grp group */

#endif /* __LBC_EXT_H */


