/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          pamu_ext.h

 @Description   Peripheral Access Management Unit (PAMU)
*//***************************************************************************/
#ifndef __PAMU_EXT_H
#define __PAMU_EXT_H

#include "std_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         pamu_grp PAMU Programming Interface

 @Description   PAMU API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pamu_init_grp PAMU Initialization Unit

 @Description   PAMU initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PAMU window/Sub window operation type
*//***************************************************************************/
typedef enum e_PamuOperationType{
    e_PAMU_OPERATION_NONE            = 0,  /**< No special operation for this entry */
    e_PAMU_OPERATION_QMAN_PORTAL     = 1,  /**< QMAN portals operations             */
    e_PAMU_OPERATION_FMAN            = 2,  /**< FMAN operations             */
    e_PAMU_OPERATION_QMAN_GENERAL    = 3,  /**< QMAN general operations             */
    e_PAMU_OPERATION_CAAM            = 4,  /**< CAAAM operations                */
}e_PamuOperationType;


//exceptions callback??


/**************************************************************************//**
 @Description   Parameters structure - for initialization of the PAMU
*//***************************************************************************/
typedef struct
{
    t_Handle        h_App;              /**< Handle to an application entity                  */
    uint64_t        baseAddress;        /**< Base address (virtual) of the PAMU block         */
    uint32_t        memPartitionId;     /**< Mem partition id used for APMU tables allocation */
    int             irq;                /**< Pamu access violation interrupt vector           */
} t_PamuParam;

/**************************************************************************//**
 @Description   Structure for configuring of a PAMU window/sub window
*//***************************************************************************/
typedef struct
{
    uint64_t            windowPhysAddr;   /**< Physical base address                    */
    uint64_t            windowSize;       /**< Size in bytes                            */
    uint64_t            windowVirtAddr;   /**< Virtual base address(translated address) */
    bool                addrXlate;        /**< Address translation                      */
    int                 stashId;          /**< Stashing ID                              */
    int                 snoopId;          /**< Snoop group id                           */
    e_PamuOperationType opertionType;     /**< Operation types supported by this enrty  */
    bool                coherence;        /**< Coherence - snooping enable              */
} t_PamuWin;

/**************************************************************************//**
 @Description   Structure for configuring an entry for a liodn in the PAMU
*//***************************************************************************/
typedef struct
{
    t_PamuWin   *pamuWindows;           /**< Pointer to array of pamu sub windows, the size of the array
                                             should be equal to windowCount                                 */
    int         windowCount;            /**< number of sub windows                                          */
} t_PamuLiodnParam;


/**************************************************************************//**
 @Function      P4080_PAMU_ConfigAndInit

 @Description   Creation and initialization of PAMU object.

                This routine must be called before any other PAMU routine may
                be called.

                The input for this routine are the main configuration
                parameters of the PAMU module.

 @Param[in]     p_PamuParam - The configuration parameters for PAMU.

 @Return        Handle to the created PAMU object.

*//***************************************************************************/
t_Handle PAMU_ConfigAndInit(t_PamuParam     *p_PamuParam);

/**************************************************************************//**
 @Function      P4080_PAMU_Free

 @Description   Frees all memory associated with the PAMU object, And any
                other resources allocated by the driver such as spinlock,interrupts...

 @Param[in]      h_Pamu   - Handle to the PAMU object, as returned by the
                            PAMU_ConfigAndInit() routine.


 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error PAMU_Free(t_Handle  h_Pamu);

/** @} */ /* end of pamu_init_grp */


/**************************************************************************//**
 @Group         pamu_ctrl_grp PAMU Control Unit

 @Description   PAMU control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PAMU_AddLiodnEntry

 @Description   Configuration of all the windows for a specific Liodn

 @Param[in]     h_Pamu   - Handle to the PAMU object, as returned by the
                            PAMU_ConfigAndInit() routine.

 @Param[in]     liodn    - Liodn number

@Param[in]      p_LiodnParam - Liodn winows parameters.
                Each liodn can be granted access to multiple windows.The number of windows is passed
                as parameter - windowCount.
                According to this number the user should allocate an array of structures of the type t_PamuWin, The size of the array
                should eqaul windowCount. The parameter pamuWindows will be update with the pointer to the array.
                Each Window is defined by one entry of the array, all parameters are defined in t_PamuWin structure.

                Important:

                1.Each window size should be in the range of 4K - 16E.

                2.Window adrees should be aligned with the window size.

                3.According to the PAMU block requierments, When more than one window is configured.
                the base adress of each window needs to be a  multiplication of the largest window size,
                even that the memory used is much smaller.Memory map need to be planed accordingly.

                For example if we have 3 windows :

                1st adress 0x000000000 size 0x40000000 (1 GIGA)

                2nd window 0x400000000 size 0x10000000 (256 Mega)

                3rd window 0x800000000 size 0x10000000 (256 Mega)

                Window 3 could have been alocated in lower address but since the 1st windows takes a giga,
                both other windows needs to start at the same offset difference.


 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following P4080_PAMU_ConfigAndInit().
*//***************************************************************************/
t_Error PAMU_AddLiodnEntry(t_Handle     h_Pamu, uint32_t    liodn, t_PamuLiodnParam     *p_LiodnParam);

/** @} */ /* end of pamu_ctrl_grp group */
/** @} */ /* end of pamu_grp group */


#endif /* __PAMU_EXT_H */
