/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          aes_ext.h

 @Description   structures and definitions for operating AES algorithm.

*//***************************************************************************/

#ifndef __AES_EXT_H
#define __AES_EXT_H

#define ENCRYPT  0x00100000
#define AES_CBC  0x00200000
#define AES_CTR  0x00600000
#define AES_CCM  0x0b000000
#define AES_RDK  0x00800000


/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_aes SEC AES

 @Description   SEC AES functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_aes_init SEC AES Initialization Unit

 @Description   SEC AES initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   enum of the different Aes algorithm types
 
                different modes of AES algorithm
*//***************************************************************************/
typedef enum {
		 e_ECB_MODE,
		 e_CBC_MODE,
         e_CTR_MODE,
         e_CCM_MODE
}e_AesMode;

/* accessory routine */
/**************************************************************************//**
 @Function      aes_alloc_ctx

 @Description   Allocate memory for the aes ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
 
*//***************************************************************************/
t_Handle aes_alloc_ctx();

/* accessory routine */
/**************************************************************************//**
 @Function      aes_set_mode

 @Description   The routine sets the required AES operation mode.

				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         mode -   (In) Aes operation mode - CBC,ECB,CTR,CCM.

 @Return        None.
 
*//***************************************************************************/
void     aes_set_mode (void *ctx,e_AesMode mode);

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      aes_setkey

 @Description   initialize the Key that is required for AES algorithm

                Initialize the key for the aes calculation.
                The routine sets the specific key and its length in the
                private context structure.The key is required to operate
                the algorithm.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         key -    (In) handler to the key.
 @Param         keylen - (In) the length of the key handler.
 @Param         flags -  (In) Not relevant.

 @Return        return status.

 @Cautions      The AES key length is limited (between 16 to 32 bytes).
*//***************************************************************************/
int  aes_setkey (void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      aes_setiv

 @Description   Initialize the initialization vector in the ctx strucutre.

				The function sets the iv pointer in the context
                structure. The need of initialization vector (iv) depends
			    on the AES algorithm mode.
				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src - 	 (In) handler to the initialization vector.
 @Param         len -    (In) the length of the initialization vector.

 @Return        None.

 @Cautions      The AES iv length is limited (0 or 16 bytes).
 
*//***************************************************************************/
void     aes_setiv (void *ctx, void *src, unsigned int len);

/**************************************************************************//**
 @Function      aes_set_outiv

 @Description   Initialize the output initialization vector in the ctx strucutre.

				The function sets the output iv pointer in the context
                structure.
				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         outsrc - (In) handler to the output initialization vector.
 @Param         outlen - (In) the length of the output initialization vector.

 @Return        None.
 
*//***************************************************************************/
void     aes_set_outiv (void *ctx, void *outsrc, unsigned int outlen);

/**************************************************************************//**
 @Function      aes_set_srclen

 @Description   The routine sets the data source length in the ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -    (In) the length of the input.

 @Return        None.
 
*//***************************************************************************/
void     aes_set_srclen (void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      aes_set_dstlen

 @Description   The routine sets the data destination length in the ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -    (In) the length of the output.

 @Return        None.
 
*//***************************************************************************/
void     aes_set_dstlen (void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      aes_set_rdk

 @Description   The routine sets the restore decrypt key option.

				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
 
*//***************************************************************************/
void     aes_set_rdk (void *ctx);

#ifdef SEC2
/**************************************************************************//**
 @Function      aes_set_aad

 @Description   The routine sets the required aad - additional authentication data pointer and its length.

				The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -    (In) aad length.
 @Param         src -    (In) aad pointer.

 @Return        None.
 
*//***************************************************************************/
void aes_set_aad (void *ctx, unsigned int len, void *src);

#endif /* SEC2 */

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_aes_runtime SEC AES Runtime Unit

 @Description   SEC AES Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      aes_encrypt

 @Description   Initialize and send the AES request to the crypto co-processor.

				Initialize and send the aes request to the Crypto hardware.
 				This routine initializes the aes request structure
				according to the fields in the aes ctx and then call to
				the Ioctl function in order to process the request.
				The routine is a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -    (In) the output destination handler.
 @Param         src - 	 (In) the input source handler.

 @Return        None.
 
*//***************************************************************************/
void aes_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      aes_decrypt

 @Description   Initialize and send the AES request to the crypto co-processor.

				Initialize and send the aes request to the Crypto hardware.
 				This routine initializes the aes request structure
				according to the fields in the aes ctx and then call to
				the Ioctl function in order to process the request.
				The routine is a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -    (In) the output destination handler.
 @Param         src - 	 (In) the input source handler.

 @Return        None.
 
*//***************************************************************************/
void aes_decrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec aes */
/** @} */ /* end of group sec */


#endif
