/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          apm_ext.h

 @Description   advanced power management external structures and definitions.
*//***************************************************************************/
#ifndef __APM_EXT_H
#define __APM_EXT_H

#include "apm_integration_ext.h"

/**************************************************************************//**
 @Group         apm_grp Advanced Power Management Programming Interface

 @Description   Advanced Power Management API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         apm_init_grp Advanced Power Management Initialization Unit


 @Description   Advanced Power Management initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description    APM initialization structure, used when calling APM_Config().
 *//***************************************************************************/
typedef struct t_ApmParam
{
    uint32_t    baseAddress;        /**< Base address of APM registers in memory */
#ifdef APM_JOG_MODE
	uint32_t    ccbClock;           /**< Platform Core Complex Bus Clock */
#endif
#ifdef APM_DEEP_SLEEP_MODE
	uint32_t    bptrAddress;        /**< Boot Pointer Translation Register Address (from CCSR) */
#endif
} t_ApmParam;


/**************************************************************************//**
 @Function      APM_ConfigAndInit
 
 @Description   Creates a APM descriptor.

 @Param[in]     p_ApmParam - Pointer to the configuration parameters structure;

 @Return        Handle to a APM descriptor on success, NULL on error.
*//***************************************************************************/
t_Handle APM_ConfigAndInit(t_ApmParam *p_ApmParam);


/**************************************************************************//**
 @Function      APM_Free

 @Description   Frees all resources used by APM.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error APM_Free(t_Handle h_Apm);

/** @} */ /* end of apm_init_grp group */


/**************************************************************************//**
 @Group         apm_control Advanced Power Management Control Unit

 @Description   Advanced Power Management Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   WakeUp Interrupts
*//***************************************************************************/
typedef enum e_ApmWakeUpInterrupt
{
    e_APM_WAKEUP_INTERRUPT_CORE_IN     = 0x01,     /**< Core interrupt input. */
    e_APM_WAKEUP_INTERRUPT_CORE_CIN    = 0x02,     /**< Core critical interrupt input */
    e_APM_WAKEUP_INTERRUPT_CORE_UDE    = 0x04,     /**< Core unconditional debug event */
    e_APM_WAKEUP_INTERRUPT_CORE_MCP    = 0x08      /**< Core machine check processor */
} e_ApmWakeUpInterrupt;

/**************************************************************************//**
 @Description   APM Request
 
                These flags can be passed to APM_RequestMode or APM_GetStatus.
 
                Can't be selected more then one mode.
*//***************************************************************************/

typedef enum e_ApmRequestMode
{
    e_APM_REQUEST_CORE_DOZE         = 0x0       /**< Core doze mode */
    , e_APM_REQUEST_CORE_NAP                    /**< Core nap mode */
    , e_APM_REQUEST_SLEEP                       /**< Sleep mode */
#ifdef APM_DEEP_SLEEP_MODE
    , e_APM_REQUEST_DEEP_SLEEP                  /**< Deep Sleep mode */
#endif
#ifdef APM_JOG_MODE
    , e_APM_REQUEST_JOG                         /**< Jog request */
#endif /* APM_JOG_MODE */
#ifdef APM_QUIESCE_MODE
    , e_APM_REQUEST_QUIESCE                     /**< Quiesce mode */
#endif /* APM_QUIESCE_MODE */
}e_ApmRequestMode;

#ifdef APM_JOG_MODE

/**************************************************************************//**
 @Description   Core Speed in Jog Mode
*//***************************************************************************/
typedef enum e_ApmJogCoreSpeedMode
{
    e_APM_JOG_CORE_SPEED_MODE_SLOW          = APM_JOG_CORE_SPEED_MODE_SLOW,
    e_APM_JOG_CORE_SPEED_MODE_HIGH          = APM_JOG_CORE_SPEED_MODE_HIGH
}e_ApmJogCoreSpeedMode;

/**************************************************************************//**
 @Function      APM_SetJogCoreClock

 @Description   This routine is called to set the new core clock.
                This value takes effect when the system wakes from deep sleep or a jog
                mode request is completed.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     coreId - selected core to update core clock.
 @Param[in]     newVal - New core clock in MHz.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetJogCoreClock (t_Handle h_Apm, uint8_t coreId, uint32_t newVal);

/**************************************************************************//**
 @Function      APM_SetJogCoreSpeedMode

 @Description   This routine is called to configure Jog core speed mode.
                This value takes effect when the system wakes from deep sleep or a jog
                mode request is completed.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     coreId - Selected core to update clock interval.
 @Param[in]     newVal - New core clock interval.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetJogCoreSpeedMode (t_Handle h_Apm, uint8_t coreId, e_ApmJogCoreSpeedMode newVal);

/**************************************************************************//**
 @Function      APM_SetJogTimer

 @Description   This routine is called to configure Jog Counter.
                When the system reaches SLEEP mode, which is a state during jog,
                the timer begins decrementing. When the timer reaches 0 it implies that
                the Cores PLLs are locked with new values.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     newVal - time to wait usec.


 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetJogTimer (t_Handle h_Apm, uint64_t newVal);

#endif          /* APM_JOG_MODE */
/**************************************************************************//**
 @Function      APM_EnableWakeUpInterrupt

 @Description   This routine is called to enable wake-up interrupts.
                If interrupts are masked as a wake-up condition the device remains in a
                low-power state despite the presence of an interrupt request.
                By default all interrupts are enabled.
 
 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     coreId - Selected core enable interrupts.
 @Param[in]     intr   - Selected interrupts to be enabled from e_ApmWakeUpInterrupt enum.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_EnableWakeUpInterrupt (t_Handle h_Apm, uint8_t coreId, uint32_t intr);

/**************************************************************************//**
 @Function      APM_DisableWakeUpInterrupt

 @Description   This routine is called to disable wake-up interrupts.
                If interrupts are masked as a wake-up condition the device remains in a
                low-power state despite the presence of an interrupt request.
                By default all interrupts are enabled.
 
 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     coreId - Selected core disable interrupts.
 @Param[in]     intr   - Selected interrupts to be disabled from e_ApmWakeUpInterrupt enum.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_DisableWakeUpInterrupt (t_Handle h_Apm, uint8_t coreId, uint32_t intr);

/**************************************************************************//**
 @Function      APM_EnableLossLessMode

 @Description   This routine is called to enable receiving Ethernet packets in
                sleep or deep sleep mode.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     newVal - Enable/disable receiving Ethernet packets in deep sleep mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetLossLessMode (t_Handle h_Apm, bool newVal);

#ifdef APM_QUIESCE_MODE
/**************************************************************************//**
 @Function      APM_SetQuiesceMode

 @Description   This routine is called to enable/disable Quiesce Mode.
				Instruction fetching is halted in the core, and all interfaces
				except the system access port (SAP), cease responding to
				externally-initiated transactions.

 @Param[in]     h_Apm   - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     newVal  - Enable/disable Quiesce Mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetQuiesceMode (t_Handle h_Apm, bool newVal);

#endif          /* APM_QUIESCE_MODE */


#ifdef APM_VOLTAGE_RAMPUP_COUNTER
/**************************************************************************//**
 @Function      APM_SetVoltageRampUpTimer

 @Description   This routine is called to configure Voltage Ramp-up Counter.
                This counter establishes the duration for VDD to become stable
                after POWER_OK is asserted before enabling the Core PLL.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     newVal - time to wait usec.


 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetVoltageRampUpTimer (t_Handle h_Apm, uint64_t newVal);

#endif          /* APM_VOLTAGE_RAMPUP_COUNTER */

#ifdef APM_POWER_DOWN_COUNTER
/**************************************************************************//**
 @Function      APM_SetPowerDownTimer

 @Description   This routine is called to configure Power Down Count.
                This counter establishes a minimum time for which power can be removed
                to the VDD supply during deep sleep

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     newVal - time to wait usec.


 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetPowerDownTimer (t_Handle h_Apm, uint64_t newVal);

#endif          /* APM_POWER_DOWN_COUNTER */

/**************************************************************************//**
 @Function      APM_EnableBlockClock

 @Description   This routine is called to enable clock for various functional blocks
                in sleep power states.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     block  - the block to enable/disable clock.
 @Param[in]     newVal - Enable/disable block's clock.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_EnableBlockClock (t_Handle h_Apm, e_ApmClockEnable block, bool newVal);

/**************************************************************************//**
 @Function      APM_SetLowPowerMode

 @Description   This routine is called for placing the device into a low power mode.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     mode   - requested mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_SetLowPowerMode(t_Handle h_Apm, e_ApmRequestMode mode);


/**************************************************************************//**
 @Function      APM_Recover

 @Description   This routine is called for recovering the device from a low power mode.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_Recover(t_Handle h_Apm);

/**************************************************************************//**
 @Function      APM_GetStatus

 @Description   This routine is called to get status of a specific low power mode.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().
 @Param[in]     mode   - requested mode.
 @Param[out]    value  - True if the device is to be placed in the requested mode, False if it is not.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error APM_GetStatusMode(t_Handle h_Apm, e_ApmRequestMode mode, bool *value);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      APM_DumpRegs

 @Description   Dumps APM registers.

 @Param[in]     h_Apm  - Handle to the APM descriptor returned by APM_ConfigAndInit().

 @Return        None.

 @Cautions      h_Apm must have been previously created using APM_ConfigAndInit().
*//***************************************************************************/
void APM_DumpRegs(t_Handle h_Apm);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of apm_control */
/** @} */ /* end of apm_grp */


#endif /* __APM_EXT_H */
