/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          arbiter_ext.h

 @Description   Prototypes, externals and typedefs for the Arbiter Module
*//***************************************************************************/

#ifndef __ARBITER_EXT_H
#define __ARBITER_EXT_H

#include "part_ext.h"
#include "part_integration_ext.h"

/**************************************************************************//**
 @Group         arbiter_grp  Arbiter Application Programming Interface

 @Description   Arbiter external API.
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         arbiter_exception_grp Arbiter Exception Unit

 @Description   Arbiter exception functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Transfre size
 
                Transfer size of the reported event which cuase the error
*//***************************************************************************/
typedef enum e_ArbiterTransferSize
{
    e_ARBITER_TRANSFER_SIZE_1_BYTE          = 0x00000001,  /**< 1 byte transfer size    */
    e_ARBITER_TRANSFER_SIZE_2_BYTES         = 0x00000002,  /**< 2 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_3_BYTES         = 0x00000003,  /**< 3 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_4_BYTES         = 0x00000004,  /**< 4 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_5_BYTES         = 0x00000005,  /**< 5 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_6_BYTES         = 0x00000006,  /**< 6 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_7_BYTES         = 0x00000007,  /**< 7 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_8_BYTES         = 0x00000008,  /**< 8 bytes transfer size   */
    e_ARBITER_TRANSFER_SIZE_16_BYTES        = 0x00000009,  /**< 16 bytes transfer size  */
    e_ARBITER_TRANSFER_SIZE_24_BYTES        = 0x0000000A,  /**< 24 bytes transfer size  */
    e_ARBITER_TRANSFER_SIZE_32_BYTES        = 0x0000000B   /**< 32 bytes transfer size  */

} e_ArbiterTransferSize;

/**************************************************************************//**
 @Description   Transfer burst
 
                Transaction type; burst or single beat
*//***************************************************************************/
typedef enum e_ArbiterTransferBurst
{
    e_ARBITER_BURST_TRANSACTION                 = 0x00000000, /**< Burst transaction       */
    e_ARBITER_SIGNAL_BEAT_TRANSACTION           = 0x00000001  /**< Single beat transaction */

} e_ArbiterTransferBurst;

/**************************************************************************//**
 @Description   Event type
 
                The type of the event which cuase the first error
*//***************************************************************************/
typedef enum e_ArbiterEventType
{
    e_ARBITER_EVENT_ADDR_TIMEOUT                    = 0x00000000, /**< Address time out                             */
    e_ARBITER_EVENT_DATA_TIMEOUT                    = 0x00000001, /**< Data time out                                */
    e_ARBITER_EVENT_ADDR_ONLY_TRANSFER              = 0x00000002, /**< Address only transfer type                   */
    e_ARBITER_EVENT_EXTERNAL_CONTROL_WORD_TRANSFER  = 0x00000003, /**< External control word transfer type          */
    e_ARBITER_EVENT_RESERVED_TRANSFER               = 0x00000004, /**< Reserved transfer type                       */
    e_ARBITER_EVENT_SLAVE_DETECTED_TRANSFER_ERROR   = 0x00000005  /**< Transfer error detected by one of the slaves */
 
} e_ArbiterEventType;

/**************************************************************************//**
 @Description   Arbiter error attributes
*//***************************************************************************/
typedef struct t_ArbiterErrorReport
{
    e_ArbiterEventType      errorType;      /**< The event type of the reported event which cuase the error */
    uint32_t                errAddr;        /**< Address of the reported event which cuase the error        */
    e_TransSrc              masterId;       /**< The master is of the reported event which cuase the error  */
    e_ArbiterTransferBurst  transferBurst;  /**< Transaction type; burst or single beat                     */
    e_ArbiterTransferSize   transferSize;   /**< Transfer size of the reported event which cuase the error  */
    uint32_t                transferType;   /**< Transfer type of the reported event which cuase the error  */
 
} t_ArbiterErrorReport;

/** @} */ /* end of arbiter_exception_grp group */

/**************************************************************************//**
 @Group         arbiter_init_grp Arbiter Initialization Unit

 @Description   Arbiter initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   User's callback routine for reporting arbiter errors.
 
 @Param[in]     h_App       - Handle to the application's object; this handle
                              will be passed as the first argument of the
                              callback routine.
 @Param[in]     p_ErrorAttr - Pointer to the error attributes structure.
*//***************************************************************************/
typedef void (t_ArbiterException)(t_Handle h_App, t_ArbiterErrorReport* p_ErrorAttr);

/**************************************************************************//**
 @Description   Arbiter initialization structure, used when calling ARBITER_Config()
 *//***************************************************************************/
typedef struct t_ArbiterParam
{
    uint32_t    baseAddress; /**< Base address of memory mapped Arbiter registers */
    int         irq;         /**< Arbiter interrupt ID (system-specific number) */
    t_Handle    h_App;       /**< Handle to application (shall passed back to the
                                  application in the exceptions callback function) */
    void        (*f_Exception)(t_Handle h_App, t_ArbiterErrorReport *p_ArbiterErrorReport);
                                /**< Pointer to exceptions callback routine */
} t_ArbiterParam;

/**************************************************************************//**
 @Function      Arbiter_Config

 @Description   Creates a Arbiter descriptor according to user parameters.
 
                User parameters are the minimum necessary to specify arbiter
                behavior. Additional parameters can be specified later using
                Arbiter_Config<Property> functions.

 @Param[in]     p_ArbiterParam  - Pointer to structure of user-specified parameters
 
 @Return        Handle to a Arbiter descriptor on success, NULL on error
*//***************************************************************************/
t_Handle ARBITER_Config(t_ArbiterParam *p_ArbiterParam);

/**************************************************************************//**
 @Function      ARBITER_Init

 @Description   Fully Arbiter initialization.

 @Param[in]     h_Arbiter - Handle to the Arbiter descriptor returned by Arbiter_Config()

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error ARBITER_Init(t_Handle h_Arbiter);

/**************************************************************************//**
 @Function      ARBITER_Free

 @Description   Free Arbiter object.

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by ARBITER_Config()
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error ARBITER_Free(t_Handle h_Arbiter);


/**************************************************************************//**
 @Group         arbiter_advanced_init_grp  Arbiter Advanced Configuration Unit

 @Description   Configuration functions used to change default values.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Event Handling Mode
*//***************************************************************************/
typedef enum e_ArbiterEventMode
{
    e_ARBITER_INTERRUPT_AND_RESET_DISABLE,      /**< Disable reset and interrupt    */
    e_ARBITER_INTERRUPT_REGULAR,                /**< Regular interrupt type         */
    e_ARBITER_INTERRUPT_MCP,                    /**< MCP interrupt type             */
    e_ARBITER_RESET                             /**< Reset on event                 */

} e_ArbiterEventMode;

/**************************************************************************//**
 @Description   Arbiter pipeline depth
 
                number of outstanding transactions 1-4.
*//***************************************************************************/
typedef enum e_ArbiterPipelineDepth
{
    e_ARBITER_PIPELINE_DEPTH_1   = 0x00000000,  /**< Pipeline depth 1 (1 outstanding transaction) */
    e_ARBITER_PIPELINE_DEPTH_2   = 0x00010000,  /**< Pipeline depth 2 (2 outstanding transaction) */
    e_ARBITER_PIPELINE_DEPTH_3   = 0x00020000,  /**< Pipeline depth 3 (3 outstanding transaction) */
    e_ARBITER_PIPELINE_DEPTH_4   = 0x00030000   /**< Pipeline depth 4 (4 outstanding transaction) */
} e_ArbiterPipelineDepth;

/**************************************************************************//**
 @Description   Arbiter pipeline depth
 
                number of outstanding transactions 1-4.
*//***************************************************************************/
typedef enum e_ArbiterPciRepeatCnt
{
    e_ARBITER_1_PCI_PEPEAT_CNT_TRANS  = 0x00000000,    /**< 1 consecutive transactions */
    e_ARBITER_2_PCI_PEPEAT_CNT_TRANS  = 0x00001000,    /**< 2 consecutive transactions */
    e_ARBITER_3_PCI_PEPEAT_CNT_TRANS  = 0x00002000,    /**< 3 consecutive transactions */
    e_ARBITER_4_PCI_PEPEAT_CNT_TRANS  = 0x00003000,    /**< 4 consecutive transactions */
    e_ARBITER_5_PCI_PEPEAT_CNT_TRANS  = 0x00004000,    /**< 5 consecutive transactions */
    e_ARBITER_6_PCI_PEPEAT_CNT_TRANS  = 0x00005000,    /**< 6 consecutive transactions */
    e_ARBITER_7_PCI_PEPEAT_CNT_TRANS  = 0x00006000,    /**< 7 consecutive transactions */
    e_ARBITER_8_PCI_PEPEAT_CNT_TRANS  = 0x00007000     /**< 8 consecutive transactions */
 
} e_ArbiterPciRepeatCnt;

/**************************************************************************//**
 @Description   Arbiter repeat count
 
                Specifies the maximum number of consecutive transactions, that
                any master (except PCI) can perform, using REPEAT request mode.
*//***************************************************************************/
typedef enum e_ArbiterRepeatCount
{
    e_ARBITER_1_PEPEAT_CNT_TRANS  = 0x00000000,    /**< 1 consecutive transactions */
    e_ARBITER_2_PEPEAT_CNT_TRANS  = 0x00000100,    /**< 2 consecutive transactions */
    e_ARBITER_3_PEPEAT_CNT_TRANS  = 0x00000200,    /**< 3 consecutive transactions */
    e_ARBITER_4_PEPEAT_CNT_TRANS  = 0x00000300,    /**< 4 consecutive transactions */
    e_ARBITER_5_PEPEAT_CNT_TRANS  = 0x00000400,    /**< 5 consecutive transactions */
    e_ARBITER_6_PEPEAT_CNT_TRANS  = 0x00000500,    /**< 6 consecutive transactions */
    e_ARBITER_7_PEPEAT_CNT_TRANS  = 0x00000600,    /**< 7 consecutive transactions */
    e_ARBITER_8_PEPEAT_CNT_TRANS  = 0x00000700     /**< 8 consecutive transactions */

} e_ArbiterRepeatCount;

/**************************************************************************//**
 @Description   Address parking
 
                Specifies arbiter bus parking mode.
*//***************************************************************************/
typedef enum e_ArbiterAddressParking
{
    e_ARBITER_PARK_TO_MASTER        = 0x00000000,   /**< Arbiter parks the address bus to the master                    */
    e_ARBITER_PARK_TO_LAST_OWNER    = 0x00000010,   /**< Arbiter parks the address bus to last bus owner                */
    e_ARBITER_PARKING_DISABLE       = 0x00000020    /**< Arbiter does not assert BG to any master, if no BR is present  */
 
} e_ArbiterAddressParking;

/**************************************************************************//**
 @Function      ARBITER_ConfigPipelineDepth

 @Description   Change the default value of the pipeline depth(number of
                outstanding transactions). [e_ARBITER_PIPELINE_DEPTH_4]

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by ARBITER_Config()
 @Param[in]     depth       - Pipeline depth
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error ARBITER_ConfigPipelineDepth(t_Handle h_Arbiter, e_ArbiterPipelineDepth depth);

/**************************************************************************//**
 @Function      ARBITER_ConfigRepeatCount

 @Description   Change the default value of the maximum number of consecutive
                transactions, that any master (except PCI) can perform.
                [e_ARBITER_4_PEPEAT_CNT_TRANS]

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by Arbiter_Config()
 @Param[in]     count       - Repeat count
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error ARBITER_ConfigRepeatCount(t_Handle h_Arbiter, e_ArbiterRepeatCount count);

/**************************************************************************//**
 @Function      ARBITER_ConfigPciRepeatCount

 @Description   Change the default value of the maximum number of consecutive
                transactions, that PCI master can perform.
                [e_ARBITER_4_PCI_PEPEAT_CNT_TRANS]

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by Arbiter_Config()
 @Param[in]     count       - Pci Repeat count
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error ARBITER_ConfigPciRepeatCount(t_Handle h_Arbiter, e_ArbiterPciRepeatCnt count);

/**************************************************************************//**
 @Function      ARBITER_ConfigAddressParking

 @Description   Change the default value of the parking mode. [e_ARBITER_PARK_TO_MASTER]

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by Arbiter_Config()
 @Param[in]     parkMode    - Arbiter's bus address parking mode
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error ARBITER_ConfigAddressParking(t_Handle h_Arbiter, e_ArbiterAddressParking parkMode);

/**************************************************************************//**
 @Function      ARBITER_ConfigParkingMaster

 @Description   Change the default value of the parking master.
                [QE if available, otherwise e300 Core]

 @Param[in]     h_Arbiter       - Handle to the Arbiter descriptor returned by Arbiter_Config()
 @Param[in]     parkingMaster   - Arbiter's bus parking master
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error ARBITER_ConfigParkingMaster(t_Handle h_Arbiter, e_ArbiterParkingMaster parkingMaster);

/**************************************************************************//**
 @Function      ARBITER_ConfigEventMode

 @Description   Enable the Generation of an MCP or regular interrupt when certain event occur
 
 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by Arbiter_Config()
 @Param[in]     event       - The event which cuase the interrupt
 @Param[in]     type        - The type of the interrupt (regular or MCP)
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error ARBITER_ConfigEventMode (t_Handle h_Arbiter, e_ArbiterEventType event, e_ArbiterEventMode type);

/** @} */ /* end of arbiter_advanced_init_grp */
/** @} */ /* end of arbiter_init_grp */

/**************************************************************************//**
 @Group         arbiter_control_grp Arbiter Control Unit

 @Description   Arbiter Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function       ARBITER_SetDataTimeout

 @Description    Set the time-out period for the data tenure.
                 The granularity of this field is ARBITER_TIMEOUT_GRANULARITY bus clocks.
                 Data time_out occurs if the data tenure does not end before the specified
                 time-out periodtimer expires between the assertion of DBB until the
                 assertion of last TA.

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by ARBITER_Config()
 @Param[in]     cycles      - Clock cycles
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error ARBITER_SetDataTimeout(t_Handle h_Arbiter, uint32_t cycles);

/**************************************************************************//**
 @Function      ARBITER_SetAddressTimeout

 @Description   Specifies the time-out period for the address tenure.
                The granularity of this field is ARBITER_TIMEOUT_GRANULARITY bus clocks.
                Address time-out occurs if the address tenure did not end before
                the specified time-out periodtimer expires between assertion of TS
                signal until the assertion of AACK signal.

 @Param[in]     h_Arbiter   - Handle to the Arbiter descriptor returned by ARBITER_Config()
 @Param[in]     cycles      - Clock cycles
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error ARBITER_SetAddressTimeout(t_Handle h_Arbiter, uint32_t cycles);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      ARBITER_DumpRegs

 @Description   Dumps the memory of ARBITER module

 @Param[in]     h_Arbiter - ARBITER channel descriptor returned by ARBITER_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ARBITER_DumpRegs(t_Handle h_Arbiter);

#endif


/** @} */ /* end of arbiter_control_grp */
/** @} */ /* end of arbiter_grp */


#endif /* __ARBITER_EXT_H */
