/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          des_ext.h

 @Description   structures and definitions for operating the DES algorithm.

*//***************************************************************************/
#ifndef __DES_EXT_H
#define __DES_EXT_H

#define DES3_EDE 0x00200000
#define ENCRYPT  0x00100000
#define DES_CBC  0x00400000

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_des SEC DES/3DES

 @Description   SEC DES functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_des_init SEC DES/3DES Initialization Unit

 @Description   SEC DES/3DES initialization functions.

 @{
*//***************************************************************************/

/* Accessories routine */
/**************************************************************************//**
 @Function      des_alloc_ctx

 @Description   Allocate memory for the des_ctx structure.
 
 @Return        ctx -  ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle des_alloc_ctx();

/* Crypto Api standard routine */
/**************************************************************************//**
 @Function      des_setkey

 @Description   initialize the Key that is required for DES algorithm

                Initialize the key for the des calculation.
                The routine sets the specific key and its length in the
                private context structure.The key is required to operate
                the algorithm.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         key -    (In) handler to the key.
 @Param         keylen - (In) the length of the key handler.
 @Param         flags -  (In) Not relevant.

 @Return        return status.

 @Cautions      The DES key length is limited (8 bytes).
*//***************************************************************************/
int  des_setkey (void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/* Accessories routine */
/**************************************************************************//**
 @Function      des_setiv

 @Description   The function sets the initialization vector in the des_ctx structure.

                The initialization vector (iv) is important in case
                of different modes like CBC calculation mode.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) handler to the initialization vector.
 @Param         len -  (In) the length of the initialization vector.

 @Return        None.
*//***************************************************************************/
void     des_setiv (void *ctx, void *src, unsigned int len);

/* Accessories routine */
/**************************************************************************//**
 @Function      des_set_outiv

 @Description   The function sets the output initialization vector in the des_ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         outsrc - (In) handler to the output initialization vector.
 @Param         outlen - (In) the length of the output initialization vector.

 @Return        None.
*//***************************************************************************/
void     des_set_outiv (void *ctx, void *outsrc, unsigned int outlen);

/* Accessories routine */
/**************************************************************************//**
 @Function      des_set_srclen

 @Description   The routine sets the data source length in the des_ctx structure.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -  (In) the length of the input.

 @Return        None.
*//***************************************************************************/
void     des_set_srclen (void *ctx, unsigned int len);

/* Accessories routine */
/**************************************************************************//**
 @Function      des3_ede_alloc_ctx

 @Description   Allocate memory for the des3_ede_ctx structure.
 
 @Return        ctx -  ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle des3_ede_alloc_ctx();

/* Crypto Api standard routine */
/**************************************************************************//**
 @Function      des3_ede_setkey

 @Description   Initialize the key for the 3des calculation.

                The routine sets the specific key and its length in the
                private context structure.The key is required to operate
                the algorithm.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         key -    (In) handler to the key.
 @Param         keylen - (In) the length of the key handler.
 @Param         flags  - (In) Not relevant.
 
 @Return        return status.

 @Cautions      The 3DES key length is limited (between 8 to 24 bytes).
*//***************************************************************************/
int  des3_ede_setkey (void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/* Accessories routine */
/**************************************************************************//**
 @Function      des3_ede_setiv

 @Description   The function sets the initialization vector in the des3_ede_ctx structure.

                The initialization vector (iv) is important
                in case of algorithms that use the CBC calculation mode.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) handler to the initialization vector.
 @Param         len -  (In) the length of the initialization vector.

 @Return        None.
*//***************************************************************************/
void     des3_ede_setiv (void *ctx, void *src, unsigned int len);

/* Accessories routine */
/**************************************************************************//**
 @Function      des3_ede_set_outiv

 @Description   The function sets the output initialization vector in the des3_ede_ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         outsrc - (In) handler to the output initialization vector.
 @Param         outlen - (In) the length of the output initialization vector.

 @Return        None.
*//***************************************************************************/
void     des3_ede_set_outiv (void *ctx, void *outsrc, unsigned int outlen);

/* Accessories routine */
/**************************************************************************//**
 @Function      des3_ede_set_srclen

 @Description   The routine sets the data source length in the des3_ede_ctx structure.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -  (In) the length of the input.

 @Return        None.
*//***************************************************************************/
void     des3_ede_set_srclen (void *ctx, unsigned int len);

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_des_runtime SEC DES/3DES  Runtime Unit

 @Description   SEC DES/3DES Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Crypto Api standard routines                                              */
/*****************************************************************************/

/**************************************************************************//**
 @Function      des_encrypt

 @Description   initialize and send the des request to the Crypto hardware.

                This routine initializes the des request structure
                by the fields in the des_ctx and then call to the Ioctl
                function in order to process the request for encryption.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the output destination handler.
 @Param         src -  (In) the input source handler.

 @Return        None.
*//***************************************************************************/
void des_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      des_decrypt

 @Description   initialize and send the des request to the Crypto hardware.

                This routine initializes the des request structure
                by the fields in the des_ctx and then call to the Ioctl
                function in order to process the request for decryption.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the output destination handler.
 @Param         src -  (In) the input source handler.

 @Return        None.
*//***************************************************************************/
void des_decrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      des3_ede_encrypt

 @Description   initialize and send the 3des request to the Crypto hardware.

                This routine initializes the 3des request structure
                by the fields in the des3_ede_ctx and then call to the Ioctl
                function in order to process the request for encryption.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the output destination handler.
 @Param         src -  (In) the input source handler.

 @Return        None.
*//***************************************************************************/
void des3_ede_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      des3_ede_decrypt

 @Description   initialize and send the 3des request to the Crypto hardware.

                This routine initializes the 3des request structure
                by the fields in the des3_ede_ctx and then call to the Ioctl
                function in order to process the request for decryption.
                The routine is a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the output destination handler.
 @Param         src -  (In) the input source handler.

 @Return        None.
*//***************************************************************************/
void des3_ede_decrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/** @} */ /* end of group runtime */
/** @} */ /* end of group des/3des */
/** @} */ /* end of group sec */
#endif
