/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/

/**

 @File          diu_ext.h

 @Description   DIU external structures and definitions.
*//***************************************************************************/
#ifndef __DIU_EXT_H
#define __DIU_EXT_H

/**************************************************************************//**
 @Group         diu_grp DIU Application Programming Interface

 @Description   DIU API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         diu_init DIU Initialization Unit

 @Description   DIU initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description    DIU data buffers alignment
 *//***************************************************************************/

#define DIU_DATA_BUFFERS_ALIGN          32

/**************************************************************************//**
 @Description    DIU operation mode.
 *//***************************************************************************/
typedef enum e_DiuMode
{
    e_DIU_MODE_OFF = 0,             /**< Mode 0: DIU off. */
    e_DIU_MODE_DISPLAY_ALL,         /**< Mode 1: All three planes output to display. */
    e_DIU_MODE_DISPLAY_P1,          /**< Mode 2: Plane 1 to display, Planes 2 and 3
                                    written back to memory. */
    e_DIU_MODE_WRITE_BACK_ALL,      /**< Mode 3: All three planes written back to memory. */
    e_DIU_MODE_GENERATE_COLOR_BAR   /**< Mode 4: Color bar generation. */

} e_DiuMode;

/**************************************************************************//**
 @Description    DIU planes.
 *//***************************************************************************/
typedef enum e_DiuPlane
{
    e_DIU_PLANE_1 = 0x00,       /**< Plane 1 */
    e_DIU_PLANE_2,              /**< Plane 2 */
    e_DIU_PLANE_3               /**< Plane 3 */

} e_DiuPlane;

/**************************************************************************//**
 @Description    DIU Colorbar
 *//***************************************************************************/
typedef enum e_DiuColorbar
{
    e_DIU_COLORBAR_BLACK = 0,       /**< Black colorbar */
    e_DIU_COLORBAR_BLUE,            /**< Blue colorbar */
    e_DIU_COLORBAR_CYAN,            /**< Cyan colorbar */
    e_DIU_COLORBAR_GREEN,           /**< Green colorbar */
    e_DIU_COLORBAR_YELLOW,          /**< Yellow colorbar */
    e_DIU_COLORBAR_RED,             /**< Red colorbar */
    e_DIU_COLORBAR_PURPLE,          /**< Purple colorbar */
    e_DIU_COLORBAR_WHITE            /**< White colorbar */

} e_DiuColorbar;

/**************************************************************************//**
 @Description   Type definition for user callback function used by the driver for
                reporting interrupts.

 @Param         h_AppId - (In)  Handle passed to the application level interrupt
                routine,
 @Param         interrupt - (Out) DIU interrupts identifier.
 
 @Return        N/A
 *//***************************************************************************/
typedef void (t_DiuIntrCallBack)(t_Handle h_AppId, uint32_t interrupt);

/**************************************************************************//**
 @Description    DIU configuration parameters structure.
 *//***************************************************************************/
typedef struct t_DiuParam
{
    uint32_t        xres;           /**< Horizontal resolution (in pixels) */
    uint32_t        xfp;            /**< Horizontal front-porch pulse width (in pixels) */
    uint32_t        xpw;            /**< Horizontal active pulse width (in pixels) */
    uint32_t        xbp;            /**< Horizontal back-porch pulse width (in pixels) */
    uint32_t        yres;           /**< Vertical resolution (in lines) */
    uint32_t        yfp;            /**< Vertical front-porch pulse width (in lines) */
    uint32_t        ypw;            /**< Vertical active pulse width (in lines) */
    uint32_t        ybp;            /**< Vertical back-porch pulse width (in lines) */
    uint32_t        refresh;        /**< Refresh rate */

    uint32_t        baseAddress;    /**< Base address of DIU registers */
    int             irq;            /**< DIU interrupt ID (system-specific number) */
    t_Handle        h_AppId;        /**< Handle to the upper layer */
    t_DiuIntrCallBack   *f_IntCb;   /**< User-provided callback for DIU specific
                                    interrupts reporting */
} t_DiuParam;

/**************************************************************************//**
 @Description    Position structure.
 *//***************************************************************************/
typedef struct t_DiuPosition
{
    uint32_t        x;              /**< Horizontal position (in pixels) */
    uint32_t        y;              /**< Verical position (in lines) */
 
} t_DiuPosition;

/**************************************************************************//**
 @Description    Color structure.
 *//***************************************************************************/
typedef struct t_DiuColor
{
    uint8_t         red;            /**< Red component */
    uint8_t         green;          /**< Green component */
    uint8_t         blue;           /**< Blue component */
 
} t_DiuColor;

/**************************************************************************//**
 @Description    DIU Pixel Byte Flip.
 *//***************************************************************************/
typedef enum e_DiuPixelByteFlip
{
    e_DIU_PIXEL_BYTE_FLIP_ENABLE  = 0,  /**< Bytes are flipped */
    e_DIU_PIXEL_BYTE_FLIP_DISABLE = 1   /**< Bytes are not flipped (recommended) */

} e_DiuPixelByteFlip;

/**************************************************************************//**
 @Description    DIU Alpha Component.
 *//***************************************************************************/
typedef enum e_DiuAlphaComponent
{
    e_DIU_ALPHA_COMPONENT_0 = 0,        /**< Component 0 */
    e_DIU_ALPHA_COMPONENT_1,            /**< Component 1 */
    e_DIU_ALPHA_COMPONENT_2,            /**< Component 2 */
    e_DIU_ALPHA_COMPONENT_3,            /**< Component 3 */
    e_DIU_ALPHA_GLOBAL                  /**< Global Alpha */

} e_DiuAlphaComponent;

/**************************************************************************//**
 @Description    DIU Pixel Component.
 *//***************************************************************************/
typedef enum e_DiuPixelComponent
{
    e_DIU_PIXEL_COMPONENT_0 = 0,        /**< Component 0 */
    e_DIU_PIXEL_COMPONENT_1,            /**< Component 1 */
    e_DIU_PIXEL_COMPONENT_2,            /**< Component 2 */
    e_DIU_PIXEL_COMPONENT_3             /**< Component 3 */

} e_DiuPixelComponent;

/**************************************************************************//**
 @Description    DIU Pixel Palette.
 *//***************************************************************************/
typedef enum e_DiuPixelPalette
{
    e_DIU_PIXEL_PALETTE_DISABLE = 0,    /**< Palette disabled */
    e_DIU_PIXEL_PALETTE_ENABLE          /**< Palette enabled */

} e_DiuPixelPalette;

/**************************************************************************//**
 @Description    DIU Pixel Size.
 *//***************************************************************************/
typedef enum e_DiuPixelSize
{
    e_DIU_PIXEL_SIZE_1 = 0,             /**< 1 Byte per pixel */
    e_DIU_PIXEL_SIZE_2,                 /**< 2 Bytes per pixel */
    e_DIU_PIXEL_SIZE_3,                 /**< 3 Bytes per pixel */
    e_DIU_PIXEL_SIZE_4                  /**< 4 Bytes per pixel */

} e_DiuPixelSize;

/**************************************************************************//**
 @Description    DIU Pixel Component Bits Number.
 *//***************************************************************************/
typedef enum e_DiuPixelCompBits
{
    e_DIU_PIXEL_COMP_BITS_0 = 0,        /**< 0 Bits in component */
    e_DIU_PIXEL_COMP_BITS_1,            /**< 1 Bit in component */
    e_DIU_PIXEL_COMP_BITS_2,            /**< 2 Bits in component */
    e_DIU_PIXEL_COMP_BITS_3,            /**< 3 Bits in component */
    e_DIU_PIXEL_COMP_BITS_4,            /**< 4 Bits in component */
    e_DIU_PIXEL_COMP_BITS_5,            /**< 5 Bits in component */
    e_DIU_PIXEL_COMP_BITS_6,            /**< 6 Bits in component */
    e_DIU_PIXEL_COMP_BITS_7,            /**< 7 Bits in component */
    e_DIU_PIXEL_COMP_BITS_8             /**< 8 Bits in component */

} e_DiuPixelCompBits;

/**************************************************************************//**
 @Description    DIU Bitmap Flip Image
 *//***************************************************************************/
typedef enum e_DiuBitmapFlip
{
    e_DIU_BMP_FLIP_NORMAL = 0,      /**< Normal */
    e_DIU_BMP_FLIP_HORIZONTALLY,    /**< Flip image horizontally (about the y-axis) */
    e_DIU_BMP_FLIP_VERTICALLY       /**< Flip image vertically (about the x-axis) */

} e_DiuBitmapFlip;

/**************************************************************************//**
 @Description    DIU Bitmap Pixel Format structure.
 *//***************************************************************************/
typedef struct t_DiuPixelFormat
{
    e_DiuPixelByteFlip  byte_f;     /**< Byte flip disable */
    e_DiuAlphaComponent alpha_c;    /**< Alpha component assignment */
    e_DiuPixelComponent blue_c;     /**< Blue component assignment */
    e_DiuPixelComponent green_c;    /**< Green component assignment */
    e_DiuPixelComponent red_c;      /**< Red component assignment */
    e_DiuPixelPalette   palette;    /**< Palette mode */
    e_DiuPixelSize      pixel_s;    /**< Pixel size */
    e_DiuPixelCompBits  comp_3;     /**< Number of bits for component 3 */
    e_DiuPixelCompBits  comp_2;     /**< Number of bits for component 2 */
    e_DiuPixelCompBits  comp_1;     /**< Number of bits for component 1 */
    e_DiuPixelCompBits  comp_0;     /**< Number of bits for component 0 */
} t_DiuPixelFormat;

/**************************************************************************//**
 @Description    Bitmap data structure.
 *//***************************************************************************/
typedef struct t_DiuBitmapData
{
    uint8_t             *p_Data;        /**< Pointer to bitmap data */
 
    t_DiuPixelFormat    pixelFormat;    /**< Input bitmap pixel format */
    uint8_t             gAlpha;         /**< Global Alpha */
    uint32_t            width;          /**< Horizontal size (in pixels) */
    uint32_t            height;         /**< Vertical size (in pixels) */
 
} t_DiuBitmapData;

/**************************************************************************//**
 @Description    Bitmap clipping rectangle parameters structure.
 *//***************************************************************************/
typedef struct t_DiuBmpClip
{
    uint32_t        x;          /**< Horizontal offset (in pixels) */
    uint32_t        y;          /**< Vertical offset (in pixels) */
    uint32_t        width;      /**< Horizontalal size (in pixels) */
    uint32_t        height;     /**< Vertical size (in pixels) */
 
} t_DiuBmpClip;

/**************************************************************************//**
 @Description    Bitmap clipping rectangle drawing parameters structure.
 *//***************************************************************************/
typedef struct t_DiuBmpDraw
{
    uint32_t        x;              /**< Horizontal display position (in pixels) */
    uint32_t        y;              /**< Vertical display position (in pixels) */
    e_DiuBitmapFlip flip;           /**< Clipping rectangle image flip */
    t_DiuColor      chromaKeyMax;   /**< Chroma key maximum */
    t_DiuColor      chromaKeyMin;   /**< Chroma key minimum */
} t_DiuBmpDraw;

/**************************************************************************//**
 @Description    DIU Area Descriptor Parameters structure.
 *//***************************************************************************/
typedef struct t_DiuAreaDescriptorParam
{
    t_DiuBitmapData     bmpData;         /**< Input bitmap data */
    t_DiuBmpClip        bmpClip;         /**< Clipping rectangle */
    t_DiuBmpDraw        bmpDraw;         /**< Drawing rectangle */

} t_DiuAreaDescriptorParam;

/**************************************************************************//**
 @Description    Filling structure.
 *//***************************************************************************/
typedef struct t_DiuFilling
{
    uint8_t         out_buff;       /**< Filling status of the output buffer
                                    (in pixels, 24 bits per pixel). */
    uint8_t         wb_buff;        /**< Filling status of the write back pixel
                                    buffer (number of filled buffers out of the
                                    8 256-byte buffers). */
    uint8_t         p3_buff;        /**< Filling status of Plane 3 input pixel
                                    buffer (number of filled buffers out of the
                                    8 256-byte buffers). */
    uint8_t         p2_buff;        /**< Filling status of Plane 2 input pixel
                                    buffer (number of filled buffers out of the
                                    8 256-byte buffers). */
    uint8_t         p1_buff;        /**< Filling status of Plane 1 input pixel
                                    buffer (number of filled buffers out of the
                                    8 256-byte buffers). */
 
} t_DiuFilling;

/**************************************************************************//**
 @Function      DIU_Config

 @Description   Creates a DIU driver handler based on the configuration
                parameters.
 
 @Param[in]     p_DiuParam - Pointer to configuration parameters structure.
                See the t_DiuParam structure definition for a description of
                the requireq parameters.
 
 @Return        Handle to a DIU descriptor on success, NULL on error
*//***************************************************************************/
t_Handle DIU_Config(t_DiuParam *p_DiuParam);

/**************************************************************************//**
 @Function      DIU_Init

 @Description   Initialize DIU driver. After initialization DIU controller is
                disabled. One must call the DIU_Enable function to start DIU
                controller in a specified operating mode.
 
 @Param[in]     h_Diu - DIU driver handle returned by DIU_Config()

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_Init(t_Handle h_Diu);

/**************************************************************************//**
 @Function      DIU_Free

 @Description   Frees all resources used by DIU driver and stops DIU controller
                operation.

 @Param[in]     h_Diu - DIU driver handler

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error DIU_Free(t_Handle h_Diu);

/**************************************************************************//**
 @Group         diu_advanced_init_grp  DIU Controller Advanced Configuration Unit

 @Description   DIU advanced configuration functions and enumerations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description    DIU interrupts
 *//***************************************************************************/
typedef enum e_DiuInterrupt
{
    e_DIU_INTERRUPT_VSYNC              = 0x01, /**< Vertical synchronization interrupt. */
    e_DIU_INTERRUPT_VSYNC_WB           = 0x02, /**< Vertical synchronization interrupt for
                                            write back operation. */
    e_DIU_INTERRUPT_UNDER_RUN          = 0x04, /**< Under run exception interrupt. */
    e_DIU_INTERRUPT_PARM_ERROR         = 0x08, /**< Display parameters error interrupt. */
    e_DIU_INTERRUPT_LINES_BEFORE_VSYNC = 0x10, /**< Lines before VSync interrupt. */
    e_DIU_INTERRUPT_WB_PENDING         = 0x20  /**< Write Back Pending Interrupt. */

} e_DiuInterrupt;

/**************************************************************************//**
 @Description    DIU synchronization signals mode
 *//***************************************************************************/
typedef enum e_DiuSyncMode
{
    e_DIU_SYNC_MODE_INV_HS = 1,         /**< Invert Horizontal Synchronize Signal */
    e_DIU_SYNC_MODE_INV_VS = 2,         /**< Invert Vertical Synchronize Signal */
    e_DIU_SYNC_MODE_INV_CS = 4,         /**< Invert Composite Synchronize Signal */
    e_DIU_SYNC_MODE_CSYNC_NOT_HS = 8,   /**< Composite Synchronize bypass Horizontal Synchronize */
    e_DIU_SYNC_MODE_CSYNC_NOT_VS = 16   /**< Composite Synchronize bypass Vertical Synchronize */

} e_DiuSyncMode;

/**************************************************************************//**
 @Description    DIU Priority
 *//***************************************************************************/
typedef enum e_DiuPriority
{
    e_DIU_PRIORITY_LOW           = 0x00000000,      /**< Low bandwidth DIU */
    e_DIU_PRIORITY_LOW_MEDIUM    = 0x000FF5F6,      /**< Low-medium bandwidth DIU */
    e_DIU_PRIORITY_MEDIUM_HIGH_1 = 0x00111F66,      /**< Medium-high bandwidth DIU */
    e_DIU_PRIORITY_MEDIUM_HIGH_2 = 0x001F5F66,      /**< Medium-high bandwidth DIU */
    e_DIU_PRIORITY_HIGH_1        = 0x0111F666,      /**< High bandwidth DIU */
    e_DIU_PRIORITY_HIGH_2        = 0x01F5F666       /**< High bandwidth DIU */

} e_DiuPriority;

/**************************************************************************//**
 @Function      DIU_ConfigInterrupts

 @Description   Configures DIU driver enabled interrupts. By default the driver
                enables all DIU interrupts. If an application does not use one
                or more interrupts it may call this function to enable only the
                the desired interrupts.
                [\]

 @Param[in]     h_Diu - DIU driver handler

 @Param[in]     intMask - Combination of e_DiuInterrupt values for the interrupts
                to enable

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following DIU_Config() and before DIU_Init().
*//***************************************************************************/
t_Error DIU_ConfigInterrupts(t_Handle h_Diu, uint32_t intMask);

/**************************************************************************//**
 @Function      DIU_ConfigPriority

 @Description   Configures the DIU priority when accessing the system memory. By
                default the driver configures the maximum priority. Call this
                function to set the priority function of the application
                requirements.
                [e_DIU_PRIORITY_HIGH_2]

 @Param[in]     h_Diu - DIU driver handler

 @Param[in]     priority - DIU DRAM accessing priority

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following DIU_Config() and before DIU_Init().
*//***************************************************************************/
t_Error DIU_ConfigPriority(t_Handle h_Diu, uint32_t priority);

/**************************************************************************//**
 @Function      DIU_ConfigSyncSignalsUsage

 @Description   Configures the usage of the synchronization signals. By default
                the driver configures the synchronization signals for the most
                usually cases : separate vertical and horizontal synchronization
                signal with normal polarity (not inverted)
                [0]

 @Param[in]     h_Diu - DIU driver handler

 @Param[in]     usage - Combination of e_DiuSyncMode values

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following DIU_Config() and before DIU_Init().
*//***************************************************************************/
t_Error DIU_ConfigSyncSignalsUsage(t_Handle h_Diu, uint32_t usage);

/**************************************************************************//**
 @Function      DIU_ConfigAreaDescriptors

 @Description   Configures maximum number of area descriptors an application may
                use and theirs memory partition type. By default the DIU driver
                allocates a pool of 3 (3) area descriptors in the
                1 (Primary DDR cacheable ) memory partition.
                [3, 1]

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     cnt - Number of area descriptors
 @Param[in]     memId - Memory partition ID.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following DIU_Config() and before DIU_Init().
*//***************************************************************************/
t_Error DIU_ConfigAreaDescriptors(t_Handle h_Diu, uint32_t cnt, uint8_t memId);

/**************************************************************************//**
 @Function      DIU_ConfigDisableDynamicPlanes

 @Description   Disables the dynamic change of the image displayed on planes. By
                default this driver implemented operation mode is enabled. In
                the initalization phase of the driver it allocs and configures
                three default area descriptors (one for each DIU plane). If an
                application does not intend to change dinamically the displayed
                bitmap images it may deactivate this mode to save some memory
                space.
                [TRUE]

 @Param[in]     h_Diu - DIU driver handler
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Allowed only following DIU_Config() and before DIU_Init().
*//***************************************************************************/
t_Error DIU_ConfigDisableDynamicPlanes(t_Handle h_Diu);

/** @} */ /* end of diu_advanced_init_grp */
/** @} */ /* end of diu_init */

/**************************************************************************//**
 @Group         diu_control DIU Control Unit

 @Description   DIU Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DIU_Enable

 @Description   Enable DIU controller operation in a specified operation mode.
                Before issuing the command the application must configure and
                set the information required for the intended operating mode.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     mode - DIU operating mode

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_Enable(t_Handle h_Diu, e_DiuMode mode);

/**************************************************************************//**
 @Function      DIU_Disable

 @Description   Disables DIU controller operation by setting it the operating
                mode 0 (DIU OFF).
                DIU driver and DIU controller are re-initialized : all driver
                structures and all DIU controller registers are set in the same
                state as after initialization.

 @Param[in]     h_Diu - DIU driver handler

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_Disable(t_Handle h_Diu);

/**************************************************************************//**
 @Function      DIU_SetThreshold

 @Description   Set a threshold value. There are two threshold values that may
                be changed :
                    - Lines before Vertical Synchoronization interupt threshold.
                    that sets the number of lines ahead of vertical front porch
                    (FP_V) when the interrupt will be generated. By default this
                    threshold is 0.
                    - Output buffer filling low threshold (in pixels), This is
                    used to generate the buffer under run exception. An under
                    run exception is generated if display needs data and output
                    buffer filling is lower than or equal to the OUT_BUF_LOW
                    threshold. By default this threshold is 0.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     val - Thresold value to set
 @Param[in]     isLbvsTh - TRUE to set the Lines Before VSync threshold, FALSE
                to set the Output Buffer threshold

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetThreshold(t_Handle h_Diu, uint32_t val, bool isLbvsTh);

/**************************************************************************//**
 @Function      DIU_ModifyPriority

 @Description   Changes the DIU priority when accessing the system memory.

 @Param[in]     h_Diu - DIU driver handler

 @Param[in]     priority - DIU DRAM accessing priority

 @Return        E_OK on success; Error code otherwise.
 
*//***************************************************************************/
t_Error DIU_ModifyPriority(t_Handle h_Diu, uint32_t priority);

/**************************************************************************//**
 @Function      DIU_SetWriteBackPlaneResolution

 @Description   Set Write Back plane resolution. The function may be called
                before enabling a write back operating mode. The function must
                be called if the resolution of the written back bitmap image
                changes while DIU is working in Mode #2 or #3. Host application
                must change, before changing the resolution, the write back
                memory buffer size, to fit the new resolution
                Note : The resolution of the bitmap image may increase when new
                added source bitmap images are placed outside of the current
                writeback plane display resolution.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     xres - Horizontal resolution (in pixels)
 @Param[in]     yres - Vertical resolution (in lines)

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetWriteBackPlaneResolution(
                                t_Handle h_Diu, uint32_t xres, uint32_t yres);

/**************************************************************************//**
 @Function      DIU_SetWriteBackPlaneAddress

 @Description   Set Write Back plane memory address. The function may be called
                before enabling e_DIU_MODE_WRITE_BACK_ALL operating mode. The
                function must be called if the resolution of written back bitmap
                image changes while DIU is working in Mode #2 or #3. The size
                of the buffer must fit the resolution of the bitmap image.
                Notes :
                    The resolution of the bitmap image may increase when new
                added source bitmap images are placed outside of the current
                writeback plane display resolution.
                    If the size of the memory buffer is not large enought to
                fit the written back bitmap image unpredictable errors may
                occur.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_MemAddr - Pointer to memory zone where the bitmap image is
                returned
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetWriteBackPlaneAddress(t_Handle h_Diu, uint8_t *p_MemAddr);

/**************************************************************************//**
 @Function      DIU_SetWriteBackPlane

 @Description   Set Write Back plane address and resolution. The function must
                be called before enabling the e_DIU_MODE_WRITE_BACK_ALL opera-
                ting mode. The function may be called before enabling the
                e_DIU_MODE_DISPLAY_P1 operating mode. If the size of the memory
                buffer is not large enought to fit the written back bitmap image
                unpredictable errors may occur.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     xres - Horizontal resolution (in pixels)
 @Param[in]     yres - Vertical resolution (in lines)
 @Param[in]     p_MemAddr - Pointer to memory zone where the bitmap image is
                returned
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetWriteBackPlane(t_Handle h_Diu, uint8_t *p_MemAddr,
                                                uint32_t xres, uint32_t yres);

/**************************************************************************//**
 @Function      DIU_SetBackgroundColor

 @Description   Set Plane 1 or Write Back Plane background color.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     color - Color to set in the RGB format
 @Param[in]     forWbPlane - TRUE to set the Write Back plane background color,
                FALSE to set the Plane 1 background color
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetBackgroundColor(t_Handle h_Diu, uint32_t color, bool forWbPlane);

/**************************************************************************//**
 @Function      DIU_SetCursorPosition

 @Description   Set Cursor position.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     x - Horizontal position
 @Param[in]     y - Vertical position

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetCursorPosition(t_Handle h_Diu, uint32_t x, uint32_t y);

/**************************************************************************//**
 @Function      DIU_GetCursorPosition

 @Description   Get the position of the cursor

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Pos - Pointer to t_DiuPosition structure

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_GetCursorPosition(t_Handle h_Diu, t_DiuPosition *p_Pos);

/**************************************************************************//**
 @Function      DIU_ShowCursor

 @Description   Show or hide the cursor

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     bShow - TRUE to show the cursor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_ShowCursor(t_Handle h_Diu, bool bShow);

/**************************************************************************//**
 @Function      DIU_FillingStatus

 @Description   Get filling status of the DIU buffers (for debug purpose)

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Fill - Pointer to t_DiuFilling structure

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_GetFillingStatus(t_Handle h_Diu, t_DiuFilling *p_Fill);

/**************************************************************************//**
 @Function      DIU_SetColorbarColor

 @Description   Set colors in colorbars. Call this function to change the default
                colors in the colorbars registers. The default colors are Black,
                Blue, Cyan, Green, Yellow, Red, Purple and White. The colorbars
                register are used only in the DIU operation mode 4 (Color Bar
                Generation)

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     color - Color to set in the colorbar register (in the RGB format)
 @Param[in]     reg - Colorbar register
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetColorbarColor(t_Handle h_Diu, uint32_t color, e_DiuColorbar reg);

/**************************************************************************//**
 @Function      DIU_AreaDescriptorConfigAndInit

 @Description   Configures and initializes an Area Descriptor based on input
                parameters. An initialized area descriptor may be added to one
                or more planes. Same area descriptor may be added only once to
                a certain plane.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Desc - Pointer to area descriptor parameters

 @Return        Handle of the Area Descriptor
*//***************************************************************************/
t_Handle DIU_AreaDescriptorConfigAndInit(t_Handle h_Diu,
                                             t_DiuAreaDescriptorParam *p_Desc);
 
/**************************************************************************//**
 @Function      DIU_AddAreaDescriptorToPlane

 @Description   Add an Area Descriptor to the list of bitmaps' area descriptors
                to be displayed on a specified plane. The same area descriptor
                may be added to any plane, but only once on a certain plane. The
                same area descriptor may be added to each plane.
                At runtime, the change becomes effective only after a call of the
                DIU_RefreshPlane function.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     plane - A plane

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_AddAreaDescriptorToPlane(t_Handle h_Diu,
                                              t_Handle h_Ad, e_DiuPlane plane);

/**************************************************************************//**
 @Function      DIU_RemoveAreaDescriptorFromPlane

 @Description   Remove an area descriptor from the plane's area descriptors list.
                At runtime, the change becomes effective only after a call of the
                DIU_RefreshPlane function. One cannot remove the last descriptor,
                if the DIU controller is on. One cannot remove a descriptor if a
                plane is enabled and the DIU controller is off.
 
 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     plane - A plane

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_RemoveAreaDescriptorFromPlane(t_Handle h_Diu,
                                              t_Handle h_Ad, e_DiuPlane plane);

/**************************************************************************//**
 @Function      DIU_AreaDescriptorModifyBitmapAddress

 @Description   Change input bitmap address of an Area Descriptor. One needn't
                to refresh the plane(s) on which the descriptor is added.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     p_Bmp - Pointer to the input bitmap data

 @Return        Handle of the Area Descriptor
 
 @Cautions      The new input bitmap must match the Area Descriptor parameters

*//***************************************************************************/
t_Error DIU_AreaDescriptorModifyBitmapAddress(t_Handle h_Diu,
                                                t_Handle h_Ad, uint8_t *p_Bmp);
 
/**************************************************************************//**
 @Function      DIU_AreaDescriptorModifyClippingRectangleSize

 @Description   Change the size of the input bitmap clipping rectangle. To make
                this change to take effect on a plane, the plane must be refreshed.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     width - Clipping rectangle width (in pixels)
 @Param[in]     height - Clipping rectangle height (in pixels)

 @Return        Handle of the Area Descriptor
 
 @Cautions      The clipping rectangle must be entirely inside the input bitmap.

*//***************************************************************************/
t_Error DIU_AreaDescriptorModifyClippingRectangleSize(t_Handle h_Diu,
                               t_Handle h_Ad, uint32_t width, uint32_t height);
 
/**************************************************************************//**
 @Function      DIU_AreaDescriptorModifyClippingRectangleOffset

 @Description   Change the position of the input bitmap clipping rectangle. To make
                this change to take effect on a plane, the plane must be refreshed.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     x - Clipping rectangle horizontal position (in pixels)
 @Param[in]     y - Clipping rectangle vertical position (in pixels)

 @Return        Handle of the Area Descriptor
 
 @Cautions      The clipping rectangle must be entirely inside the input bitmap.

*//***************************************************************************/
t_Error DIU_AreaDescriptorModifyClippingRectangleOffset(t_Handle h_Diu,
                                        t_Handle h_Ad, uint32_t x, uint32_t y);
 
/**************************************************************************//**
 @Function      DIU_AreaDescriptorModifyClippingRectanglePosition

 @Description   Change the position on display of the input bitmap clipping
                rectangle. To make this change to take effect on a plane, the
                plane must be refreshed.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     h_Ad - Area descriptor handler
 @Param[in]     x - Clipping rectangle horizontal position on display (in pixels)
 @Param[in]     y - Clipping rectangle vertical position on display (in pixels)

 @Return        Handle of the Area Descriptor
 
 @Cautions      The clipping rectangle must be entirely inside the display.

*//***************************************************************************/
t_Error DIU_AreaDescriptorModifyClippingRectanglePosition(t_Handle h_Diu,
                                        t_Handle h_Ad, uint32_t x, uint32_t y);
 
/**************************************************************************//**
 @Function      DIU_SetPaletteTable

 @Description   Set a Palette table.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Palette - Pointer to palette table data. This must be aligned
                to a 32 bytes boundary.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetPaletteTable(t_Handle h_Diu, uint8_t *p_Palette);

/**************************************************************************//**
 @Function      DIU_SetCursorBitmap

 @Description   Set a Cursor bitmap.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Cursor - Pointer to cursor bitmap data. This must be aligned to
                a 32 bytes boundary.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetCursorBitmap(t_Handle h_Diu, uint8_t *p_Cursor);

/**************************************************************************//**
 @Function      DIU_SetGammaCorrectionTable

 @Description   Set a Gamma correction table.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     p_Gamma - Pointer to gamma correction table data. This must be
                aligned to a 32 bytes boundary.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_SetGammaCorrectionTable(t_Handle h_Diu, uint8_t *p_Gamma);

/**************************************************************************//**
 @Function      DIU_RefreshPlane

 @Description   Initiates DIU plane refresh. The function may be called from
                the host application in a time window delimited by :
                    - the Lines Before Vertical Synchronization interrupt if DIU
                is in Mode #1 (e_DIU_MODE_DISPLAY_ALL) or #2 (e_DIU_MODE_DISPLAY_P1)
                    - the Vertical Synchronization for Write Back interrupt if DIU
                is in Mode #3 (e_DIU_MODE_WRITE_BACK_ALL).
                The intended plane is refreshed on the next frame rendering. If
                one refreshes the write back plane, the refreshing process ending
                is signaled by a Vertical Synchronization for Write Back interrupt.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     wbPlane - TRUE if one refreshes the write back plane
 @Param[in]     param - If one refreshes displayed plane the parameter contains
                        the ID of the plane (of e_DiuPlane type)
                      - If one refreshes the write back plane the parameter contains
                      the address of the write back plane memory buffer. If param = 0
                      the driver uses the address previously set by a call of the
                      SetWriteBackPlane or DIU_SetWriteBackPlaneAddress function.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_RefreshPlane(t_Handle h_Diu, bool wbPlane, uint32_t param);

/**************************************************************************//**
 @Function      DIU_EnablePlane

 @Description   Enable / Disable a DIU plane. By default all DIU planes are disabled.
                Before enabling a plane one must prepare the list of the plane
                displayed bitmap images. When a plane is "disabled" it displays an
                invisible driver defined bitmap.
                At runtime, the function may be called from the host application in
                a time window delimited by :
                    - the Lines Before Vertical Synchronization interrupt if DIU
                is in Mode #1 (e_DIU_MODE_DISPLAY_ALL) or #2 (e_DIU_MODE_DISPLAY_P1)
                    - the Vertical Synchronization for Write Back interrupt if DIU
                is in Mode #3 (e_DIU_MODE_WRITE_BACK_ALL).
                The intended plane is enabled/disabled on the next frame rendering.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     bEnable - TRUE to enable a plane
 @Param[in]     plane - The plane to be enabled / disabled.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_EnablePlane(t_Handle h_Diu, bool bEnable, e_DiuPlane plane);

/**************************************************************************//**
 @Function      DIU_StoreContext

 @Description   Store DIU driver context. The function is called from an APM
                host application when the system is about to enter the Deep-Sleep
                mode.

 @Param[in]     h_Diu - DIU driver handler
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_StoreContext(t_Handle h_Diu);

/**************************************************************************//**
 @Function      DIU_RestoreContext

 @Description   Restore DIU driver context. The function is called from an APM
                host application after the system exits the Deep-Sleep mode.

 @Param[in]     h_Diu - DIU driver handler

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DIU_RestoreContext(t_Handle h_Diu);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      DIU_DumpRegs

 @Description   Dumps DIU registers.

 @Param[in]     h_Diu - DIU driver handler

 @Return        None.

 @Cautions      h_Diu must have been previously created using DIU_Config.
*//***************************************************************************/
void DIU_DumpRegs(t_Handle h_Diu);

/**************************************************************************//**
 @Function      DIU_DumpAreaDescriptors

 @Description   Dumps Area Descriptors of the specified plane.

 @Param[in]     h_Diu - DIU driver handler
 @Param[in]     plane - The plane to dump.

 @Return        None.

 @Cautions      h_Diu must have been previously created using DIU_Config.
*//***************************************************************************/
void DIU_DumpAreaDescriptors(t_Handle h_Diu, e_DiuPlane plane);

#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of diu_control */
/** @} */ /* end of diu_grp */


#endif /* __DIU_EXT_H */
