/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __DMA83XX_EXT_H
#define __DMA83XX_EXT_H

/**************************************************************************//**
 @File          dma83xx_ext.h

 @Description   Prototypes, externals and typedefs for the DMA Module
*//***************************************************************************/

#include "dma_std_ext.h"
/**************************************************************************//**
 @Group         dma_grp  DMA Application Programming Interface

 @Description   Prototypes, externals and typedefs for system-supplied
     			(external) DMA Routines.
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         dma_init DMA Initialization Unit


 @Description   DMA initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    General DMA controller definitions

 @{
*//***************************************************************************/
#define DMA_NUM_OF_CHANNELS    4       /**< Number of DMA channels in the device */
/* @} */

/**************************************************************************//**
 @Description   DMA Bandwidth/Pause control options
 
                The field determines how many cache lines a given channel is
                allowed to transfer before it must release the IOS interface
                to the next channel. This allows the user to prioritize the DMA
                channels.
                Only applies when multiple channels are executing transfers
                concurrently.
*//***************************************************************************/
typedef enum e_DmaBwcSize
{
	e_DMA_BWC_1_CACHE_LINE   = 0x00000000,  /**< A channel must release the IOS interface after transferring 1 cache line */
	e_DMA_BWC_2_CACHE_LINES  = 0x00200000,  /**< A channel must release the IOS interface after transferring 2 cache lines */
	e_DMA_BWC_4_CACHE_LINES  = 0x00400000,  /**< A channel must release the IOS interface after transferring 4 cache lines */
	e_DMA_BWC_8_CACHE_LINES  = 0x00600000,  /**< A channel must release the IOS interface after transferring 8 cache lines */
	e_DMA_BWC_16_CACHE_LINES = 0x00800000,  /**< A channel must release the IOS interface after transferring 16 cache lines */
	e_DMA_BWC_NO_OVERRIDE    = 0xFFFFFFFF   /**< Driver's default bandwidth value will not be overridden */
} e_DmaBwcSize;

/**************************************************************************//**
 @Description   DMA request count
 
                Specifies the number of cache lines transfered per DMA request
                assertion. Valid only when external control is enabled for the
                relevant channel.
*//***************************************************************************/
typedef enum e_DmaRequestCount
{
	e_DMA_DRC_1_CACHE_LINE   = 0x05000000,  /**< A channel may transmit 1 cache line before DMA request is asserted   */
	e_DMA_DRC_2_CACHE_LINES  = 0x06000000,  /**< A channel may transmit 2 cache lines before DMA request is asserted  */
	e_DMA_DRC_4_CACHE_LINES  = 0x07000000,  /**< A channel may transmit 4 cache lines before DMA request is asserted  */
	e_DMA_DRC_8_CACHE_LINES  = 0x08000000,  /**< A channel may transmit 8 cache lines before DMA request is asserted  */
	e_DMA_DRC_16_CACHE_LINES = 0x09000000,  /**< A channel may transmit 16 cache lines before DMA request is asserted */
	e_DMA_DRC_32_CACHE_LINES = 0x0A000000   /**< A channel may transmit 32 cache lines before DMA request is asserted */
} e_DmaRequestCount;

/**************************************************************************//**
 @Description   DMA initialization structure, used when calling DMA_Config()
 *//***************************************************************************/
typedef struct t_DmaParam
{
    uint32_t    baseAddress;    /**< Base address of DMA registers in memory. */
    int     irq;                /**< DMA interrut ID (system-specific number) */
 
    bool        haltOnTransErr; /**< Determines if DMA will halt when a transfer error occurs */
    struct
    {
        bool    preAllocated;   /**< TRUE if channel is pre-allocated and will be
                                     controlled by an external master (will not be
                                     available in channels pool) */
    } channelsParam[DMA_NUM_OF_CHANNELS];
} t_DmaParam;

/**************************************************************************//**
 @Description   Error Callback Type definition
*//***************************************************************************/
typedef enum e_DmaError
{
	e_DMA_INT_CH_OK = 0,        /**< A transfer was successfully completed */
	e_DMA_INT_TRANSFER_ERROR    /**< An error condition during the DMA transfer has occurred */
} e_DmaError;

/**************************************************************************//**
 @Description   End of Transfer Callback Type definition
*//***************************************************************************/
typedef enum e_DmaTransferAck
{
	e_DMA_INT_END_OF_SEGMENT,   /**< A segment of data was successfully completed */
	e_DMA_INT_END_OF_TRANSFER   /**< The last DMA transfer is finished, either in chaining or direct mode */
} e_DmaTransferAck;


/**************************************************************************//**
 @Description   User callback function called by driver for event handling;
                User provides this function; Driver invokes it.

 @Param[in]     h_AppId     - User-defined data
 @Param[in]     transferAck - End of transfer type, describes the cause for the
                              current interrupt
 *//***************************************************************************/
typedef void (t_DmaTransferCallback) (t_Handle h_AppId, e_DmaTransferAck transferAck);

/**************************************************************************//**
 @Description   User callback function called by driver for error handling;
                User provides this function; Driver invokes it.

 @Param[in]     h_AppId  - User-defined data
 @Param[in]     dmaError - Error type, describes the error that caused this
                           interrupt
 *//***************************************************************************/
typedef void (t_DmaErrorCallback) (t_Handle h_AppId, e_DmaError dmaError);

/**************************************************************************//**
 @Function      DMA_Config

 @Description   Creates a DMA descriptor according to user parameters.
 
                User parameters are the minimum necessary to specify dma
                behavior. Additional parameters can be specified later using
                DMA_Config<Property> functions.

 @Param[in]     p_DmaParam   - Pointer to structure of user-specified parameters
 
 @Return        Handle to a DMA descriptor on success, NULL on error
*//***************************************************************************/
t_Handle DMA_Config(t_DmaParam *p_DmaParam);

/**************************************************************************//**
 @Function      DMA_Init

 @Description   Fully initializes DMA and enables it.

 @Param[in]     h_Dma - Handle to the DMA descriptor returned by DMA_Config()

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_Init (t_Handle h_Dma);

/** @} */ /* end of dma_init */

/**************************************************************************//**
 @Group         DMA_Runtime DMA Runtime Unit

 @Description   DMA Runtime routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   DMA Chain general parameters
*//***************************************************************************/
typedef struct t_DmaChainParam
{
    uint8_t       srcHoldSize;   /**< 0 = NoHold or 1,2,4,8 byte hold for Source       */
    uint8_t       dstHoldSize;   /**< 0 = NoHold or 1,2,4,8 byte hold for Destrination */
    e_DmaBwcSize  bandwidth;     /**< Bandwidth Control, overides channel default      */
} t_DmaChainParam;

/**************************************************************************//**
 @Function      DMA_CH_Transfer

 @Description   Simple DMA transfer from one address to another.

 @Param[in]     h_Channel     - Handle to the channel
 @Param[in]     p_TransParams - Input parameters for the transfer
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_Transfer(t_Handle           h_Channel,
                        t_DmaTransferParam *p_TransParams);

/**************************************************************************//**
 @Function      DMA_CH_TransferExtended

 @Description   Extended DMA transfer from one address to another.

 @Param[in]     h_Channel       - Handle to the channel
 @Param[in]     p_TransferParam - Input parameters for the transfer
 @Param[in]     p_DmaChainParam - DMA chain general parameters, valid for all transfers of the chain
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_TransferExtended(t_Handle             h_Channel,
                                t_DmaTransferParam   *p_TransferParam,
                                t_DmaChainParam      *p_DmaChainParam);

/**************************************************************************//**
 @Function      DMA_CH_TransferChain

 @Description   Complex DMA transfer from a link of chained transfers.

 @Param[in]     h_Channel   - Handle to the channel from which to transfer
 @Param[in]     h_DmaChain  - Handle to the chain of transfers
 @Param[in]     intrEnable  - Enable interrupt on completion of the transfer of the chain
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_TransferChain(t_Handle   h_Channel,
                             t_Handle   h_DmaChain,
                             bool       intrEnable);
 
/** @} */ /* end of dma_runtime */

/**************************************************************************//**
 @Group         dma_control DMA Control Unit

 @Description   DMA Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   DMA PCI read command type
*//***************************************************************************/
typedef enum e_PciReadCmd
{
	e_DMA_PCI_READ          = 0x00000000,   /**< PCI Read command will be used */
	e_DMA_PCI_READ_LINE     = 0x00000400,   /**< PCI Read Line command will be used */
	e_DMA_PCI_READ_MULTIPLE = 0x00000800    /**< PCI Read Multiple command will be used */
} e_PciReadCmd;

/**************************************************************************//**
 @Description   DMA initialization structure, used when calling DMA_CH_Config()
 *//***************************************************************************/
typedef struct t_DmaChannelParam
{
    t_DmaTransferCallback   *f_TransferCallback;  /**< Transfer Callback, invoked by driver on end of transfer events */
    t_DmaErrorCallback      *f_Exceptions;        /**< Callback invoked by driver on error events */
    t_Handle                h_App;                /**< User-defined data that will be passed to the callbacks  */
    e_DmaBwcSize            bandwidth;            /**< default Bandwith control for channel */
} t_DmaChannelParam;

/**************************************************************************//**
 @Function      DMA_AllocateChannel

 @Description   Allocate a DMA channel automaticly by system.

 @Param[in]     h_Dma - Handle to the DMA descriptor returned by DMA_Config()
 
  @Return        Handle to a DMA channel descriptor on success, NULL on error
*//***************************************************************************/
t_Handle DMA_CH_Allocate(t_Handle h_Dma);

/**************************************************************************//**
 @Function      DMA_CH_AllocateForce

 @Description   Allocate a DMA channel with a specific channel number.

 @Param[in]     h_Dma      - Handle to the DMA descriptor returned by DMA_Config()
 @Param[in]     channelNum - Input to a channel number to force allocation for
 
 @Return        Handle to a DMA channel descriptor on success, NULL on error
*//***************************************************************************/
t_Handle DMA_CH_AllocateForce(t_Handle h_Dma, uint8_t channelNum);

/**************************************************************************//**
 @Function      DMA_CH_Config

 @Description   Set working parameters for an allocated DMA channel.

 @Param[in]     h_Channel         - Handle to the DMA channel
 @Param[in]     p_DmaChannelParam - Pointer to structure of user-specified
                                    parameters for the configured channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_Config(t_Handle h_Channel, t_DmaChannelParam *p_DmaChannelParam);

/**************************************************************************//**
 @Function      DMA_CH_Free

 @Description   Free a DMA channel.

 @Param[in]     h_Channel - Handle to the DMA channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_Free(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_SetPciReadCommand

 @Description   Sets the type of PCI read command to use.

 @Param[in]     h_Channel  - Handle to the DMA channel
 @Param[in]     pciReadCmd - PCI read command
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_SetPciReadCommand(t_Handle h_Channel, e_PciReadCmd pciReadCmd);

/**************************************************************************//**
 @Function      DMA_CH_SetIntrSteerToPci

 @Description   Enables routing of all DMA interrupts to the PCI bus through INTA.

 @Param[in]     h_Channel - Handle to the DMA channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_SetIntrSteerToPci(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_ClearIntrSteerToPci

 @Description   Disables routing of all DMA interrupts to the PCI bus; All
                interrupts will be routed to the on-chip interrupt controller.
 
 @Param[in]     h_Channel - Handle to the channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_ClearIntrSteerToPci(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_SetDmaExtRequestCount

 @Description   Changes the default number of transfered cache lines per
                external request assertion; Valid only if the specified
                channel is under external master control;
                [e_DMA_DRC_32_CACHE_LINES]
 
 @Param[in]     h_Channel       - Handle to the channel
 @Param[in]     dmaRequestCount - Number of transfered cache lines per request assertion
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_SetDmaExtRequestCount(t_Handle          h_Channel,
                                     e_DmaRequestCount dmaRequestCount);

/**************************************************************************//**
 @Function      DMA_IsChannelBusy

 @Description   Returns if the specified channel is busy or not; Can be used by
                the application to poll for end of transfer.

 @Param[in]     h_Channel - Handle to the channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
bool    DMA_IsChannelBusy(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_GetLastTransferStatus

 @Description   Returns the status of the last DMA transfer.

 @Param[in]     h_Channel        - Handle to the channel
 @Param[out]    p_DmaError       - Pointer to DMA error report
 @Param[out]    p_DmaTransferAck - Pointer to DMA transfer report, only valid when no Error
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_GetLastTransferStatus(t_Handle         h_Channel,
                                  e_DmaError       *p_DmaError,
                                  e_DmaTransferAck *p_DmaTransferAck);

/**************************************************************************//**
 @Function      DMA_CH_HaltTransfer

 @Description   Halts current DMA transfer for the specified channel.
 
                This routine stops a DMA transfer. The transfer is not canceled
                and the channel status remains BUSY.
                The transfer can be resumed by invoking the resume transfer
                routine or the user can begin a new transfer.

 @Param[in]     h_Channel - Handle to the appropraite channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_HaltTransfer(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_ResumeTransfer.

 @Description   Resume current DMA transfer for the specified channel.
 
                If this routine invoke right after a DMA transfer was halted
                on the specified channel, that transfer is resumed from the
                point at which it was halted. The transfer is not restarted
                from the beginning.

 @Param[in]     h_Channel - Handle to the appropraite channel
 
 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_ResumeTransfer(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_BuildChain

 @Description   Builds a chain of descriptors for transfer from an input array.

 @Param[in]     p_TransferList    - Array of input parameters for each transfer
 @Param[in]     transferListCount - Number of input parameters sets in p_TransferList
 @Param[in]     memId             - Memory partition ID for allocating internal chain structures
 @Param[in]     p_DmaChainParam   - DMA chain general parameters, valid for all transfers of the chain

 @Return        Handle to the built chain on success, NULL on error
*//***************************************************************************/
t_Handle DMA_BuildChain(t_DmaTransferParam *p_TransferList,
                        uint32_t           transferListCount,
                        uint8_t            memId,
                        t_DmaChainParam    *p_DmaChainParam);

/**************************************************************************//**
 @Function      DMA_UpdateChainEntry

 @Description   Update basic parameters of one entry in a chain of descriptors.

 @Param[in]     h_DmaChain          - Handle to the chain of transfers
 @Param[in]     transferEntry       - Which entry to update
 @Param[in]     p_DmaTransParam     - Input parameters for the transfer
 

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error DMA_UpdateChainEntry(t_Handle            h_DmaChain,
                             uint32_t            transferEntry,
                             t_DmaTransferParam *p_DmaTransParam);

/**************************************************************************//**
 @Function      DMA_FreeChain

 @Description   Free Chain list of transfer.

 @Param[in]     h_DmaChain - Handle to the chain of transfers

 @Retval        None
 *//***************************************************************************/
void DMA_FreeChain(t_Handle h_DmaChain);

/**************************************************************************//**
 @Function      DMA_Free

 @Description   Frees all resources used by DMA.

 @Param[in]     h_Dma - Handle to the DMA descriptor returned by DMA_Config()

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error DMA_Free(t_Handle h_Dma);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      DMA_DumpRegs

 @Description   Dumps DMA registers.

 @Param[in]     h_Dma - Handle to the DMA descriptor returned by DMA_Config()

 @Return        None.

 @Cautions      h_Dma must have been previously created using DMA_Config()
                and initialized using DMA_Init().
*//***************************************************************************/
void DMA_DumpRegs(t_Handle h_Dma);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of dma_control */
/** @} */ /* end of dma_grp */


#endif /* __DMA83XX_EXT_H */
