/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __DMA85XX_EXT_H
#define __DMA85XX_EXT_H

/**************************************************************************//**
 @File          dma85xx_ext.h

 @Description   Prototypes, externals and typedefs for the DMA Module.
*//***************************************************************************/

#include "dma_std_ext.h"

/**************************************************************************//**
 @Group         dma_grp  DMA Application Programming Interface

 @Description   Prototypes, externals and typedefs for system-supplied
                (external) DMA Routines.
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         dma_init DMA Initialization Unit

 @Description   DMA initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    General DMA controller definitions

 @{
*//***************************************************************************/
#define DMA_NUM_OF_CHANNELS     4   /**< Number of DMA channels in the device */
/* @} */

/**************************************************************************//**
 @Description   DMA initialization structure, used when calling DMA_Config()
 *//***************************************************************************/
typedef struct t_DmaParam
{
    uint32_t    baseAddress;    /**< Base address of DMA registers in memory. */

    struct
    {
        int     irq;            /**< DMA channel interrut ID (system-specific number) */
        bool    preAllocated;   /**< TRUE if channel is pre-allocated and will be
                                     controlled by an external master (will not be
                                     available in channels pool) */
    } channelsParam[DMA_NUM_OF_CHANNELS];
} t_DmaParam;

/**************************************************************************//**
 @Description   DMA Bandwidth/Pause control options.

                If multiple channels are executing transfers concurrently, this
                value determines how many bytes a given channel is allowed to
                transfer before the DMA engine pauses it and switches to the next
                channel.
                If only one channel is executing transfers, this value dictates
                how many bytes are transferred before the DMA engine pauses the
                channel. The operation will be resumed after DMA request is
                asserted.
*//***************************************************************************/
typedef enum e_DmaBwcSize
{
    e_DMA_BWC_1_BYTE      = 0x00000000,     /**< A channel is paused after transferring 1 byte      */
    e_DMA_BWC_2_BYTES     = 0x01000000,     /**< A channel is paused after transferring 2 bytes     */
    e_DMA_BWC_4_BYTES     = 0x02000000,     /**< A channel is paused after transferring 4 bytes     */
    e_DMA_BWC_8_BYTES     = 0x03000000,     /**< A channel is paused after transferring 8 bytes     */
    e_DMA_BWC_16_BYTES    = 0x04000000,     /**< A channel is paused after transferring 16 bytes    */
    e_DMA_BWC_32_BYTES    = 0x05000000,     /**< A channel is paused after transferring 32 bytes    */
    e_DMA_BWC_64_BYTES    = 0x06000000,     /**< A channel is paused after transferring 64 bytes    */
    e_DMA_BWC_128_BYTES   = 0x07000000,     /**< A channel is paused after transferring 128 bytes   */
    e_DMA_BWC_256_BYTES   = 0x08000000,     /**< A channel is paused after transferring 256 bytes   */
    e_DMA_BWC_512_BYTES   = 0x09000000,     /**< A channel is paused after transferring 512 bytes   */
    e_DMA_BWC_1024_BYTES  = 0x0A000000,     /**< A channel is paused after transferring 1024 bytes  */
    e_DMA_BWC_NO_SHARE    = 0x0F000000,     /**< Disables bandwidth sharing to allow uninterrupted transfers from each channel */
    e_DMA_BWC_NO_OVERRIDE = 0xFFFFFFFF      /**< Driver's default bandwidth value will not be overridden */
} e_DmaBwcSize;

/**************************************************************************//**
 @Description   Error Callback Type definition
*//***************************************************************************/
typedef enum e_DmaError
{
    e_DMA_INT_CH_OK = 0,        /**< A transfer was successfully completed */
    e_DMA_INT_CH_HALTED,        /**< A transfer was halted before the tranmission had been completed */
    e_DMA_INT_PROGRAM_ERROR,    /**< A programming error was detected that prevents the DMA transfer from occurring */
    e_DMA_INT_TRANSFER_ERROR    /**< An error condition during the DMA transfer has occurred */
} e_DmaError;

/**************************************************************************//**
 @Description   End of Transfer Callback Type definition
*//***************************************************************************/
typedef enum e_DmaTransferAck
{
    e_DMA_INT_END_OF_SEGMENT,   /**< A segment of data was successfully completed */
    e_DMA_INT_END_OF_TRANSFER   /**< The last DMA transfer is finished, either in chaining or direct mode */
} e_DmaTransferAck;

/**************************************************************************//**
 @Description   User callback function called by driver for event handling;
                User provides this function; Driver invokes it.

 @Param[in]     h_App       - User-defined data
 @Param[in]     transferAck - End of transfer type, describes the cause for the
                              current interrupt

 @Return        None.
 *//***************************************************************************/
typedef void (t_DmaTransferCallback)(t_Handle h_App, e_DmaTransferAck transferAck);

/**************************************************************************//**
 @Description   User callback function called by driver for error handling;
                User provides this function; Driver invokes it.

 @Param[in]     h_App    - User-defined data
 @Param[in]     dmaError - Error type, describes the  error that caused this
                           interrupt

 @Return        None
 *//***************************************************************************/
typedef void (t_DmaErrorCallback)(t_Handle h_App, e_DmaError dmaError);

/**************************************************************************//**
 @Function      DMA_Config

 @Description   Creates DMA channel descriptor according to user parameters.

                User parameters are the minimum necessary to specify dma
                behavior. Additional parameters can be specified later using
                DMA_Config<Property> functions.

 @Param[in]     p_DmaParam  - pointer to structure of user-specified parameters for the DMA

 @Return        Handle to a DMA descriptor on success, NULL on error
*//***************************************************************************/
t_Handle DMA_Config(t_DmaParam *p_DmaParam);

/**************************************************************************//**
 @Function      DMA_Init

 @Description   Fully initializes DMA and enables it.

 @Param[in]     h_Dma - DMA descriptor returned by DMA_Config()

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_Init(t_Handle h_Dma);

/**************************************************************************//**
 @Function      DMA_Free

 @Description   Frees all resources used by DMA.

 @Param[in]     h_Dma - DMA descriptor returned by DMA_Config()

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error DMA_Free(t_Handle h_Dma);


/** @} */ /* end of dma_init */

/**************************************************************************//**
 @Group         dma_runtime DMA Runtime Unit

 @Description   DMA Runtime routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   DMA Single Transfer chain/list mode Advanced Input Param
*//***************************************************************************/
typedef struct t_DmaTransferAdvancedParam
{
    uint32_t    srcStepSize;    /**< 0=not used or Source Transfer Step Size */
    uint32_t    srcStepJump;    /**< 0=not used or Source jump for next Step */
    uint32_t    dstStepSize;    /**< 0=not used or Destination Transfer Step Size */
    uint32_t    dstStepJump;    /**< 0=not used or Destination jump for next Step */
} t_DmaTransferAdvancedParam;



/**************************************************************************//**
 @Description   DMA Single Transfer chain/list/extended mode Input Param
*//***************************************************************************/
typedef struct t_DmaExtTransParam
{
    t_DmaTransferParam          dmaTransferParam;  /**< mandatory basic parameters for transfer operation */

    /* Optional Advanced parameters */
    bool                        advancedParamOn;    /**< Advanced parameters option */
    t_DmaTransferAdvancedParam  advancedParam;      /**< Advanced parameters structure */
} t_DmaExtTransParam;

/**************************************************************************//**
 @Description   DMA Chain general parameters
*//***************************************************************************/
typedef struct t_DmaChainParam
{
    uint8_t         srcHoldSize;    /**< 0=NoHold or 1,2,4,8 byte hold for Source */
    uint8_t         dstHoldSize;    /**< 0=NoHold or 1,2,4,8 byte hold for DEstrination */
    e_DmaBwcSize    bandwidth;      /**< Bandwidth Control, overides channel default */
} t_DmaChainParam;

/**************************************************************************//**
 @Function      DMA_Transfer

 @Description   Simple DMA copy from one address to another.

 @Param[in]     h_Channel     - Handle to the channel
 @Param[in]     p_TransParams - Transfer parameters

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error DMA_CH_Transfer(t_Handle            h_Channel,
                        t_DmaTransferParam  *p_TransParams);

/**************************************************************************//**
 @Function      DMA_CH_TransferExtended

 @Description   Extended DMA copy from one address to another.

 @Param[in]     h_Channel       - Handle to the channel
 @Param[in]     p_TransferParam - Input parameters for the transfer
 @Param[in]     p_DmaChainParam - DMA chain general parameters, valid for all transfers of the chain

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_TransferExtended(t_Handle             h_Channel,
                                t_DmaExtTransParam   *p_TransferParam,
                                t_DmaChainParam      *p_DmaChainParam);

/**************************************************************************//**
 @Function      DMA_CH_TransferChain

 @Description   Complex DMA transfer from a link of chained transfers.

 @Param[in]     h_Channel     - Handle to the channel from which to transfer
 @Param[in]     h_DmaChain    - Handle to the chain of transfers
 @Param[in]     intrEnable    - Enable interrupt on completion of the transfer
                                of the chain

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_TransferChain(t_Handle h_Channel,
                             t_Handle h_DmaChain,
                             bool     intrEnable);

/** @} */ /* end of dma_runtime */

/**************************************************************************//**
 @Group         dma_control DMA Control Unit

 @Description   DMA Runtime Control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   DMA initialization structure, used when calling DMA_CH_Config()
 *//***************************************************************************/
typedef struct t_DmaChannelParam
{
    e_DmaBwcSize            bandwidth;              /**< Bandwith control for channel */
    t_Handle                h_App;                  /**< Private data that will be passed to the callbacks  */
    t_DmaTransferCallback   *f_TransferCallback;    /**< Transfer Callback invoked by driver on end of transfer events */
    t_DmaErrorCallback      *f_Exceptions;          /**< Callback invoked by driver on error events */
} t_DmaChannelParam;

/**************************************************************************//**
 @Function      DMA_AllocateChannel

 @Description   Allocate a DMA channel automaticly by system.

 @Param[in]     h_Dma - DMA descriptor returned by DMA_Config()

 @Return        Handle to a DMA channel descriptor on success, NULL on error
 *//***************************************************************************/
t_Handle DMA_CH_Allocate(t_Handle h_Dma);

/**************************************************************************//**
 @Function      DMA_CH_AllocateForce

 @Description   Allocate a DMA channel with a specific channel number.

 @Param[in]     h_Dma      - DMA descriptor returned by DMA_Config()
 @Param[in]     channelNum - Input to a channel number to force allocation for

 @Return        Handle to a DMA channel descriptor on success, NULL on error
 *//***************************************************************************/
t_Handle DMA_CH_AllocateForce(t_Handle h_Dma, uint8_t channelNum);

/**************************************************************************//**
 @Function      DMA_CH_Config

 @Description   Set working parameters for an allocated DMA channel.

 @Param[in]     h_Channel         - Handle to a DMA channel
 @Param[in]     p_DmaChannelParam - Pointer to structure of user-specified
                                    parameters for the configured channel

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_Config(t_Handle h_Channel, t_DmaChannelParam *p_DmaChannelParam);

/**************************************************************************//**
 @Function      DMA_CH_Free

 @Description   Free a DMA channel and set it's working parameters.

 @Param[in]     h_Channel - Handle to the DMA channel

 @Retval        E_OK      : On success
 @Retval        Error code: Otherwise
 *//***************************************************************************/
t_Error DMA_CH_Free(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_SetExternalPause

 @Description   Enables / disables external master pause.

                When enabled, the channel will pause after transmitting the
                number of bytes that equals to the bandwidth control value, as
                defined by the user;
                [TRUE]

 @Param[in]     h_Channel           - Handle to the dma channel
 @Param[in]     externalPauseEnable - Enables/disables external pause

 @Retval        E_OK      : On success
 @Retval        Error code: Otherwise
 *//***************************************************************************/
t_Error DMA_CH_SetExternalPause(t_Handle  h_Channel,
                                bool      externalPauseEnable);

/**************************************************************************//**
 @Function      DMA_IsChannelBusy

 @Description   Returns if the specified channel is busy or not; Can be used by
                the application to poll for end of transfer.

 @Param[in]     h_Channel - Handle to the channel

 @Retval        TRUE  : If the channel is busy
 @Retval        FALSE : Otherwise
*//***************************************************************************/
bool DMA_IsChannelBusy(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_GetLastTransferStatus

 @Description   Returns the status of the last DMA transfer.

 @Param[in]     h_Channel        - Handle to the channel
 @Param[out]    p_DmaError       - Pointer to DMA error report
 @Param[out]    p_DmaTransferAck - Pointer to DMA transfer report; Only valid when no Error

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_GetLastTransferStatus(t_Handle         h_Channel,
                                  e_DmaError       *p_DmaError,
                                  e_DmaTransferAck *p_DmaTransferAck);

/**************************************************************************//**
 @Function      DMA_CH_AbortTransfer

 @Description   Aborts current DMA transfer for the specified channel.

                This routine aborts a DMA transfer. The transfer cannot be
                resumed  - it is canceled and the channel status becomes NOT
                BUSY.

 @Param[in]     h_Channel - Handle to the appropriate channel

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_AbortTransfer(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_HaltTransfer

 @Description   Halts current DMA transfer for the specified channel.

                This routine stops a DMA transfer, but the transfer is not canceled
                and the channel status remains BUSY.
                The transfer can be resumed by invoking the resume transfer
                routine or the user can begin a new transfer.

 @Param[in]     h_Channel - Handle to the appropriate channel

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_HaltTransfer(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_CH_ResumeTransfer.

 @Description   Resume current DMA transfer for the specified channel.

                If this routine is invoked right after a DMA transfer was halted
                on the specified channel, that transfer is resumed from the
                point at which it was halted. The transfer is not restarted
                from the beginning.

 @Param[in]     h_Channel - Handle to the appropriate channel

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
*//***************************************************************************/
t_Error DMA_CH_ResumeTransfer(t_Handle h_Channel);

/**************************************************************************//**
 @Function      DMA_BuildChain

 @Description   Builds a chain of descriptors for transfer from an input array.

 @Param[in]     p_TransferList    - Array of input parameters for each transfer
 @Param[in]     transferListCount - Number of input parameters sets in p_TransferList
 @Param[in]     memId             - Memory partition ID for allocating internal chain structures
 @Param[in]     p_DmaChainParam   - DMA chain general parameters, valid for all transfers

 @Return        Handle to the built chain on success, NULL on error
*//***************************************************************************/
t_Handle DMA_BuildChain(t_DmaExtTransParam *p_TransferList,
                        uint32_t           transferListCount,
                        uint8_t            memId,
                        t_DmaChainParam    *p_DmaChainParam);

/**************************************************************************//**
 @Function      DMA_UpdateChainEntry

 @Description   Update basic parameters of one entry in a chain of descriptors.

 @Param[in]     h_DmaChain    - Handle to the chain of transfers
 @Param[in]     transferEntry - Which entry to update
 @Param[in]     p_TransParams - New transfer parameters

 @Retval        E_OK       : On success
 @Retval        Error code : Otherwise
 *//***************************************************************************/
t_Error DMA_UpdateChainEntry(t_Handle            h_DmaChain,
                             uint32_t            transferEntry,
                             t_DmaTransferParam  *p_TransParams);

/**************************************************************************//**
 @Function      DMA_FreeChain

 @Description   Free a chain list of transfer.

 @Param[in]     h_DmaChain - Handle to the chain of transfers

 @Retval        None
 *//***************************************************************************/
void DMA_FreeChain(t_Handle h_DmaChain);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      DMA_DumpRegs

 @Description   Dumps DMA registers.

 @Param[in]     h_Dma - Handle to the DMA descriptor returned by DMA_Config()

 @Return        None.

 @Cautions      h_Dma must have been previously created using DMA_Config()
                and initialized using DMA_Init().
*//***************************************************************************/
void DMA_DumpRegs(t_Handle h_Dma);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of dma_control */
/** @} */ /* end of dma_grp */


#endif /* __DMA85XX_EXT_H */

