/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          dpa_port_ext.h

 @Description   DPA-Port Application Programming Interface.
*//***************************************************************************/
#ifndef __DPA_PORT_EXT_H
#define __DPA_PORT_EXT_H

#include "std_ext.h"
#include "error_ext.h"
#include "fm_ext.h"



/**************************************************************************//**
 @Group         DPA_port_grp DPAA Port API

 @Description   DPA Port API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    General DPA Port defines
*//***************************************************************************/
#define DPAPORT_MAX_NUM_OF_BM_POOLS     4
#define DPAPORT_BM_POOL_ANY             0xff
#define DPAPORT_MAX_NUM_OF_TX_QUEUES    20
/* @} */

/**************************************************************************//**
 @Collection    DPA port standard queues definitations
*//***************************************************************************/
#define DPAPORT_RX_BASE_Q            0              /**< Base FQID */
#define DPAPORT_RX_ERR_Q             0xffffffff     /**< Rx error FQID */
#define DPAPORT_RX_DFLT_Q            0xfffffffe     /**< Default Rx FQID */
#define DPAPORT_TX_ERR_Q             0xfffffffd     /**< Tx error FQID */
#define DPAPORT_TX_CONF_Q            0xfffffffc     /**< Default Tx confirmation FQID */

#define DPAPORT_DEDICATED_Q          0x80000000     /**< Dedicated FQID (instead Default Tx-conf/OP-Rx-dflt FQID) */
#define DPAPORT_IS_DEDICATED_Q(__q)  (__q & DPAPORT_DEDICATED_Q)
#define DPAPORT_SET_DEDICATED_Q(__q) (__q | DPAPORT_DEDICATED_Q)
#define DPAPORT_GET_DEDICATED_Q(__q) (__q & ~DPAPORT_DEDICATED_Q)
/* @} */

/**************************************************************************//**
 @Description   A structure of parameters for describing and
                accessing buffer context.
*//***************************************************************************/
typedef struct t_DpaPortBuffContext
{
    t_Handle        h_UserPriv;         /**< A user defined handle. */
    t_Handle        h_QmPortal;         /**< The QM portal to be used when calling any QM routines */
    t_Handle        h_BmPortal;         /**< The BM portal to be used when calling any BM routines */
    t_Handle        h_BmPool;           /**< A handle to the BM data buffers pool */
    uint16_t        dataOffset;         /**< new data-offset; This parameter is valid only for FWD queues */
    uint16_t        length;             /**< new length for frame; This parameter is valid only for FWD queues */
    uint64_t        *p_FmTimeStamp;     /**< The timestamp of the frames reception/transmission as
                                             read from the buffer internal context, valid only if
                                             this port was configured to copy the timestamp to/from
                                             the IC from/to the buffer prefix. */
    t_FmPrsResult   *p_FmPrsResult;     /**< A pointer to a structure of the parser result in the
                                             buffer internal context, valid only if this port was configured
                                             to copy the parser result to/from the IC from/to the buffer
                                             prefix. */
    uint8_t         *p_FmKgHashResult;  /**< A pointer to the Keygen hash result in the buffer internal
                                             context, valid only if this port was configured to copy the
                                             hash result to/from the IC from/to the buffer prefix. */
    uint8_t         *p_FmIntContext;    /**< A pointer to the beginning of the buffer internal context,
                                             valid only if this port was configured to copy this
                                             information to/from the IC from/to the buffer prefix. */
#ifdef DEBUG
    uint8_t         *p_FmDebugInfo;     /**< A pointer to the beginning of the debug information in the
                                             buffer internal context, valid only if this port was configured
                                             to copy this information to/from the IC from/to the
                                             buffer prefix. */
#endif /* DEBUG */
} t_DpaPortBuffContext;


/**************************************************************************//**
 @Description   This callback type is used to get the sw-portal-handle.

                User provides this function. Driver invokes it.

 @Param[in]     h_App       - User's application descriptor.

 @Retval        software-portal-handle.
*//***************************************************************************/
typedef t_Error (t_DpaPortFillBmSwPortalCallback)(t_Handle h_App, t_DpaPortBuffContext *p_DpaContext);

/**************************************************************************//**
 @Description   User callback function called by driver with recieved data.

                User provides this function. Driver invokes it.

 @Param[in]     h_App           Application's handle originally specified to
                                the API Config function
 @Param[in]     queueId         The FQID on which frame was received.
 @Param[in]     p_Data          A pointer to data received
 @Param[in]     length          length of received data
 @Param[in]     status          receive status and errors
 @Param[in]     position        position of buffer in frame
 @Param[in]     h_BufContext    A handle of the user accosted with this buffer.
 @Param[in]     p_DpaContext    A handle to the DPA buffer context related to the data recieved
                                Note that in S/G, this handle will appear on the last buffer of a frame.
 *//***************************************************************************/
typedef void (t_DpaPortRxStoreCallback) (t_Handle               h_App,
                                         uint32_t               queueId,
                                         uint8_t                *p_Data,
                                         uint16_t               length,
                                         uint32_t               status,
                                         uint8_t                position,
                                         t_Handle               h_BufContext,
                                         t_DpaPortBuffContext   *p_DpaContext);

/**************************************************************************//**
 @Description   User callback function called by driver when transmit completed.

                User provides this function. Driver invokes it.

 @Param[in]     h_App           Application's handle originally specified to
                                the API Config function
 @Param[in]     p_Data          A pointer to data received
 @Param[in]     status          transmit status and errors
 @Param[in]     p_DpaContext    A handle to the DPA buffer context related to the data recieved
 *//***************************************************************************/
typedef void (t_DpaPortTxConfCallback) (t_Handle                h_App,
                                        uint8_t                 *p_Data,
                                        uint32_t                status,
                                        t_DpaPortBuffContext    *p_DpaContext);


/**************************************************************************//**
 @Group         DPA_port_init_grp DPA-Port Initialization Unit

 @Description   DPA-Port Initialization Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   DPA port types
*//***************************************************************************/
typedef enum e_DpaPortType
{
    e_DPA_PORT_T_VIRTUAL = 0,       /**< NOTE: This port type is temporary. Please, don't use it;
                                         It will be deprecated! Virtual port; A port without any
                                         physical MAC or physical FM-Port (only queues) */
    e_DPA_PORT_T_RX_TX,             /**< Standard FM port; A port with physical MAC and physical Rx
                                         and Tx FM-Ports */
    e_DPA_PORT_T_IM_RX_TX,          /**< Independent FM port; A port with physical MAC and physical
                                         Rx and Tx FM-Ports. however, the FM operates in IM */
    e_DPA_PORT_T_OP,                /**< Offline-Parsing port; A port without physical MAC but with
                                         physical FM-Port (OP) */
    e_DPA_PORT_T_SEC                /**< SEC port; Note that this is only "virtual-port" i.e. only
                                         input and output queues */
} e_DpaPortType;

/**************************************************************************//**
 @Description   structure representing DPA-Port initialization parameters
*//***************************************************************************/
typedef struct t_DpaPortParams
{
    e_DpaPortType                   type;               /**< DPA Port type */
    t_Handle                        h_Qm;               /**< A handle of the QM */
    uint8_t                         numOfQmTxQueues;    /**< The number of Tx queues required by this port */
    uint32_t                        qmTxChannel;        /**< The QM Tx Channel used by this port */
    uint32_t                        qmRxChannel;        /**< The QM Rx Channel used by this port */
    union {
        t_Handle                    h_FmMacDev;         /**< A handle to the FM MAC - valid only for FM-RXTX or FM-IM ports. */
        t_Handle                    h_Sec;              /**< A handle to the SEC module - valid only for sec ports */
    };
    t_Handle                        h_Bm;               /**< A handle of the BM that will be used for allocating pools */
    uint8_t                         numOfBmPools;       /**< The number of BM pools required for this port */
    bool                            userBmPools;        /**< TRUE for user defined buffer pools */
    union {
        uint32_t                    size;               /**< valid if userBmPools=FALSE: size of the BM pool */
        t_Handle                    h_BmPool;           /**< valid if userBmPools=TRUE: user defined pool handle */
    } bmPoolsParams[DPAPORT_MAX_NUM_OF_BM_POOLS];       /**< An array of numOfBmPools entries of BM pool
                                                             sizes/handles - Depending on 'userBmPools'. */
    t_DpaPortRxStoreCallback        *f_RxStore;         /**< Mandatory callback to handle received buffer.
                                                             If noAutoBmFill is set, received buffers belong to a
                                                             BM pool and must be release to the appropriate one.
                                                             Otherwise,  BM pool is handled by driver and the
                                                             received buffers belong to the appliction. */
    t_DpaPortTxConfCallback         *f_TxConf;          /**< Optional callback. When defined, will be called
                                                             for each transmitted buffer. If NULL, no confirmation
                                                             will be accepted on transmitted buffers and Tx routine
                                                             will behave differently - see DPAPORT_Tx() routine description. */
    t_Handle                        h_App;              /**< A handle to the application layer module */
} t_DpaPortParams;


/**************************************************************************//**
 @Function      DPAPORT_Config

 @Description   Creates descriptor for the DPA Port module.

 @Param[in]     p_DpaPortParams   - Pointer to data structure of parameters

 @Retval        Handle to DPA PORT object, or NULL for Failure.
*//***************************************************************************/
t_Handle    DPAPORT_Config(t_DpaPortParams *p_DpaPortParams);

/**************************************************************************//**
 @Function      DPAPORT_Init

 @Description   Initializes the DPA Port module

 @Param[in]     h_DpaPort - A handle to the DPA Port module

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     DPAPORT_Init(t_Handle h_DpaPort);

/**************************************************************************//**
 @Function      DPAPORT_Free

 @Description   Frees all resources that were assigned to DPA PORT module.

                Calling this routine invalidates the descriptor.

 @Param[in]     h_DpaPort - DPA PORT module descriptor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     DPAPORT_Free(t_Handle h_DpaPort);


/**************************************************************************//**
 @Group         DPA_port_advanced_init_grp    DPA-Port (common) Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Tx-Queue Special parameters
*//***************************************************************************/
typedef struct t_DpaPortQmTxParams
{
    bool                valid;              /**< boolean indicates whether this entry is valid or not */
    union
    {
        struct
        {
            bool        setFlow;            /**< boolean indicates whether flowId field is valid or not */
            uint16_t    flowId;             /**< The flow-id to be used in the FM */
            bool        dedicatedRxQ;       /**< boolean indicates whether to override the default-rx-queue
                                                 for this tx-queue of this port. */
        } fmOpParams;                       /**< FM-OP port parameters */
        struct
        {
            bool        macsecControlled;   /**< boolean indicates whether this queue will be transmitted using MACSEC */
            bool        dedicatedTxConfQ;   /**< boolean indicates whether to override the
                                                 default-tx-confirmation-queue for this tx-queue of this port. */
        } fmRxTxParams;                     /**< FM-RXTX and FM-IM-RXTX ports parameters. */
        struct
        {
            uint8_t     *p_SharedDesc;      /**< A pointer a shared-descriptor that was created by SEC-DCL driver */
            uint16_t    sharedDescSize;     /**< The shared-descriptor size (in bytes). */
            uint8_t     bmPoolIndx;         /**< DPA-port relative BM-pool index */
        } secParams;                        /**< SEC port parameters */
    };
} t_DpaPortQmTxQueueParams;



/**************************************************************************//**
 @Function      DPAPORT_ConfigQmTxQueueParams

 @Description   Set Special parameters for the Tx-Queues.

 @Param[in]     h_DpaPort           - DPA PORT module descriptor
 @Param[in]     p_QmTxQueuesParams  - pointer to the struct

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigQmTxQueueParams(t_Handle h_DpaPort, t_DpaPortQmTxQueueParams *p_QmTxQueuesParams);

/**************************************************************************//**
 @Function      DPAPORT_ConfigConsecutiveQmTxQueues

 @Description   Configure the dpa-port to allocate all the QM tx queues consecutive.
                The default is to allocate the tx queues separately so each queue
                will have its own work-queue. This feature may be use also if user
                want to do some kind of "hw-forward/echo"; I.e. the tx queues of
                the dpa-port can be used as the rx-pcd queues of another dpa-port.

 @Param[in]     h_DpaPort           - DPA PORT module descriptor
 @Param[in]     val                 - boolean indicates whether to allocate the queues
                                      consecutive or not.
 @Param[in]     align               - Queue's Alignment.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigConsecutiveQmTxQueues(t_Handle h_DpaPort, bool val, uint8_t align);

/**************************************************************************//**
 @Function      DPAPORT_ConfigFillBmSwPortalCallBack

 @Description   Config the callback that fill the Bm Portal for access Bm Pool routines
                in run-time;
                NOTE : If the callback not configured, the driver will assume affinity.

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     f_FillBmPortal  - pointer to the callback

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigFillBmSwPortalCallBack(t_Handle h_DpaPort, t_DpaPortFillBmSwPortalCallback *f_FillBmPortal);

/**************************************************************************//**
 @Function      DPAPORT_ConfigQmPortal

 @Description   Config the Qm Portal handle to be used only on init and free routines;
                NOTE : If the Qm Handle not configured, the driver will assume affinity.

 @Param[in]     h_DpaPort   - DPA PORT module descriptor
 @Param[in]     h_QmPortal  - A Handle of the Qm Portal

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigQmPortal(t_Handle h_DpaPort, t_Handle h_QmPortal);

/**************************************************************************//**
 @Function      DPAPORT_ConfigNoAutoBmFill

 @Description   Change the Bm fill action from its default
                configuration [TRUE].

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     noAutoBmFill    - FALSE for the driver to take care of filling BM pool
                                  after a buffer was used by FM. When TRUE it is
                                  application's responsibility.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigNoAutoBmFill(t_Handle h_DpaPort, bool noAutoBmFill);

/**************************************************************************//**
 @Function      DPAPORT_ConfigAutoBmAlloc

 @Description   Configure the dpa-port driver to automatic allocate BM buffer
                when transmited buffer was not allocated from BM; In this case,
                the drive will automatically allocated new buffer from one of its
                BM-pools and will COPY the data into the new bm-buffer; Calling
                this routine will change the driver from its default [FALSE].

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     en              - boolean indicates whether to enable auto-allocating

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigAutoBmAlloc(t_Handle h_DpaPort, bool en);

/**************************************************************************//**
 @Function      DPAPORT_ConfigStashingEn

 @Description   Change the stashing-mode from its default
                configuration [FALSE].

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     en              - boolean indicates whether to enable stashing
                                  for this port or not.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_Config() and before DPAPORT_Init().
*//***************************************************************************/
t_Error  DPAPORT_ConfigStashingEn(t_Handle h_DpaPort, bool en);

/** @} */ /* end of DPA_port_advanced_init_grp group */
/** @} */ /* end of DPA_port_init_grp group */


/**************************************************************************//**
 @Group         DPA_port_runtime_control_grp DPA-Port Runtime Control Unit

 @Description   DPA-Port Runtime control unit API functions, definitions and enums.
                The DPA-Port driver provides a set of control routines.
                These routines may only be called after the module was fully
                initialized (both configuration and initialization routines were
                called). They are typically used to get information from hardware
                (status, counters/statistics, revision etc.), to modify a current
                state or to force/enable a required action. Run-time control may
                be called whenever necessary and as many times as needed.
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   structure for defining BM pools sizes
*//***************************************************************************/
typedef struct t_DpaPortBmPoolParams
{
    uint8_t     poolId;         /**< id of the pool */
    uint16_t    bufferSize;     /**< Size of buffers in this pool */
} t_DpaPortBmPoolParams;

/**************************************************************************//**
 @Description   structure for returning to user DPA PORT parameters
*//***************************************************************************/
typedef struct t_DpaPortInternalParams
{
    uint32_t                txFqids[DPAPORT_MAX_NUM_OF_TX_QUEUES];
                                                    /**< An array of all the FQIDs of the Tx;
                                                         Note that the number-of-queues will be
                                                         according to what was passed by the user
                                                         in the DPAPORT_ConfigAndInit() */
    uint32_t                txErrFqid;              /**< Tx error FQID */
    uint32_t                txConfFqid;             /**< Default Tx confirmation FQID */
    uint32_t                rxErrFqid;              /**< Rx error FQID */
    uint32_t                rxDfltFqid;             /**< Default Rx FQID */
    t_DpaPortBmPoolParams   bmPoolsParams[DPAPORT_MAX_NUM_OF_BM_POOLS];
                                                    /**< An array of BM pool params; Note that the
                                                         number-of-pools will be according to what
                                                         was passed by the user in the
                                                         DPAPORT_ConfigAndInit() */
    uint8_t                 txPrivDataSize;         /**< Area that user may save for his own need (E.g. save the SKB) */
    uint8_t                 rxPrivDataSize;         /**< Area that user may save for his own need (E.g. save the SKB) */
} t_DpaPortInternalParams;

/**************************************************************************//**
 @Description   structure for defining FM parameters
*//***************************************************************************/
typedef struct t_DpaPortFmParams
{
    union {
        struct {
            t_Handle        h_TxFmPortDev;          /**< A handle to the FM Tx Port */
            t_Handle        h_RxFmPortDev;          /**< A handle to the FM Rx Port - not valid for FM-OP ports. */
        };                                          /**< This structure should be used for FM-RXTX, FM-IM-RXTX and FM-OP ports */
        struct {
            uint32_t    txDataOffset;
            uint32_t    txParserResultsOffset;
            uint32_t    rxParserResultsOffset;
            uint32_t    txTimeStampOffset;
            uint32_t    rxTimeStampOffset;
        } virt;                                     /**< This parameter is temporary - will be deprecated, don't use this! */
    };
} t_DpaPortFmParams;

/**************************************************************************//**
 @Description   structure for HW-FWD parameters
*//***************************************************************************/
typedef struct t_DpaPortHwFwdParams
{
    uint32_t                  rxBaseQueueId;/**< The ingress base queue-id on which to forward
                                                 traffic from; The rxBaseQueueId is port relative. */
    uint16_t                  numOfQueues;  /**< num of queues to fwd */
    t_Handle                  h_FwdDpaPort; /**< DPA PORT module descriptor to forward the traffic */
    uint32_t                  txBaseQueueId;/**< The egress base queue-id on which to forward
                                                 traffic to; The txBaseQueueId is port relative. */
} t_DpaPortHwFwdParams;

/**************************************************************************//**
 @Description   structure for FWD parameters
*//***************************************************************************/
typedef struct t_DpaPortFwdParams
{
    bool                      hwFwd;        /**< boolean indicates whether this forward entry represent
                                                 some HW forward; I.e. tx-queues of the Fwd-port will
                                                 be the same as the rx-queues of this dpa-port */
    uint32_t                  rxBaseQueueId;/**< The ingress base queue-id on which to forward
                                                 traffic from; The rxBaseQueueId is port relative. */
    uint16_t                  numOfQueues;  /**< num of queues to fwd */
    t_Handle                  h_FwdDpaPort; /**< DPA PORT module descriptor to forward the traffic */
    uint32_t                  txBaseQueueId;/**< The egress base queue-id on which to forward
                                                 traffic to; The txBaseQueueId is port relative. */
    t_DpaPortRxStoreCallback  *f_RxPreFwd;  /**< (Optional) callback to handle received buffer;
                                                 I.e. if callback is passed here, all data on the
                                                 configured rx-queues will be passed to user through
                                                 this callback; This routine is invoked by the driver
                                                 before the forward operation;
                                                 NOTE that for all rx-fwd-queues, the default
                                                 f_RxStore() routine is not being used.
                                                 Also note that this parameter may be used only if
                                                 hwFwd is FALSE */
    t_Handle                  h_App;        /**< A handle to the application layer module */
} t_DpaPortFwdParams;


/**************************************************************************//**
 @Function      DPAPORT_GetDpaParams

 @Description   This routine may be called by the user in order to read
                some DPA port parameters.

 @Param[in]     h_DpaPort - DPA PORT module descriptor
 @Param[out]    p_Params  - A structure of DPA Port parameters

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     DPAPORT_GetDpaParams(t_Handle h_DpaPort, t_DpaPortInternalParams *p_Params);

/**************************************************************************//**
 @Function      DPAPORT_SetFmDevHandle

 @Description   This routine should be called by the user after FM ports and MAC
                initialization in order to pass their handles to the DPA Port.

 @Param[in]     h_DpaPort - DPA PORT module descriptor
 @Param[in]     p_Params  - A structure of FM parameters

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     DPAPORT_SetFmDevHandle(t_Handle h_DpaPort, t_DpaPortFmParams *p_Params);

/**************************************************************************//**
 @Function      DPAPORT_GetBufferDataOffset

 @Description   Returns the data offset from the begining of the data buffer

 @Param[in]     h_DpaPort   - DPA PORT module descriptor
 @Param[in]     tx          - boolean indicates whether this routine will be called
                              for transmit or recieve side.

 @Return        data offset.

 @Cautions      Allowed only following DPAPORT_SetFmDevHandle() for FM ports
                or after DPAPORT_Init() for SEC ports.
*//***************************************************************************/
uint32_t     DPAPORT_GetBufferDataOffset(t_Handle h_DpaPort, bool tx);

/**************************************************************************//**
 @Function      DPAPORT_Enable

 @Description   A runtime routine provided to allow enable of dpa-port.

 @Param[in]     h_DpaPort - DPA PORT module descriptor

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_SetFmDevHandle().
*//***************************************************************************/
t_Error     DPAPORT_Enable(t_Handle h_DpaPort);

/**************************************************************************//**
 @Function      DPAPORT_Disable

 @Description   A runtime routine provided to allow disable of dpa-port.

 @Param[in]     h_DpaPort - DPA PORT module descriptor

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_SetFmDevHandle().
*//***************************************************************************/
t_Error     DPAPORT_Disable(t_Handle h_DpaPort);

/**************************************************************************//**
 @Function      DPAPORT_AllocRxQueues

 @Description   This routine should be called by the user to allocate Rx FQID's.

 @Param[in]     h_DpaPort   - DPA PORT module descriptor
 @Param[in]     numOfQueues - Number of queues required
 @Param[in]     align       - Alignment of first queue

 @Return        rx-base-fqid on success; '0' code otherwise.

 @Cautions      Allowed only following DPAPORT_Init(). Also, note that this
                routine may be called once.
*//***************************************************************************/
uint32_t    DPAPORT_AllocRxQueues(t_Handle h_DpaPort, uint16_t numOfQueues, uint8_t align);

/**************************************************************************//**
 @Function      DPAPORT_AllocHwForwardQueues

 @Description   This routine should be called by the user to allocate Rx FQID's to be used for hw-fwd.

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     p_Params        - A structure of hw-fwd parameters.

 @Return        hw-fwd-base-fqid on success; '0' code otherwise.

 @Cautions      This routine must be call ONLY after DPAPORT_Init() and before
                DPAPORT_GetDpaParams().
*//***************************************************************************/
uint32_t  DPAPORT_AllocHwForwardQueues(t_Handle                  h_DpaPort,
                                       t_DpaPortHwFwdParams      *p_Params);

/**************************************************************************//**
 @Function      DPAPORT_AddForwardQueues

 @Description   This routine should be called by the user to allocate Rx FQID's to be used for fwd.

 @Param[in]     h_DpaPort       - DPA PORT module descriptor
 @Param[in]     p_Params        - A structure of fwd parameters..

 @Return        E_OK on success; Error code otherwise.

 @Cautions      When passing f_RxPreFwd() user may not release the buffer as the
                driver is using it for forward.
                This routine must be called only for allocated Rx and Tx queues.
*//***************************************************************************/
t_Error  DPAPORT_AddForwardQueues(t_Handle                  h_DpaPort,
                                  t_DpaPortFwdParams        *p_Params);

/** @} */ /* end of DPA_port_runtime_control_grp group */


/**************************************************************************//**
 @Group         DPA_port_runtime_data_grp DPA-Port Runtime Data Unit

 @Description   DPA-Port Runtime data unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DPAPORT_Tx

 @Description   Tx function, called to transmit a data buffer on the port.
                Note: If f_TxConf() was set as NULL, transmitted buffer will be
                copied to a new BM buffer that will be released back to BM pool
                by the FM.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.
 @Param[in]     queueId         relative Tx queue id.
 @Param[in]     p_Data          A pointer to a data buffer.
 @Param[in]     length          Size of data for transmission.
 @Param[in]     lastBuffer      Buffer position - TRUE for the last buffer
                                of a frame, including a single buffer frame
 @Param[in]     p_Context       A handle to the DPA buffer context related to the data transmitted

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_ConfigAndInit().
*//***************************************************************************/
t_Error  DPAPORT_Tx ( t_Handle              h_DpaPort,
                      uint32_t              queueId,
                      uint8_t               *p_Data,
                      uint16_t              length,
                      bool                  lastBuffer,
                      t_DpaPortBuffContext  *p_Context);

/**************************************************************************//**
 @Function      DPAPORT_TxConf

 @Description   Tx port confirmation routine, optional, may be called to verify
                transmission of all frames. The procedure performed by this
                routine will be performed automatically on next buffer transmission,
                but if desired, calling this routine will invoke this action on
                demand.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.

 @Cautions      Allowed only following DPAPORT_ConfigAndInit().
*//***************************************************************************/
void DPAPORT_TxConf(t_Handle h_DpaPort);

/**************************************************************************//**
 @Function      DPAPORT_Rx

 @Description   Rx function, may be called to poll for received buffers.
                Normally, Rx process is invoked by the driver on Rx interrupt.
                Alternatively, this routine may be called on demand.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following DPAPORT_ConfigAndInit().
*//***************************************************************************/
t_Error  DPAPORT_Rx(t_Handle h_DpaPort);

/**************************************************************************//**
 @Function      DPAPORT_ImRxStoreCB

 @Description   This is a routine to deal with received frames. It should be used as
                the f_RxStore callback for the FM IM Port initialization.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.
 @Param[in]     p_Data          A pointer to data received
 @Param[in]     length          length of received data
 @Param[in]     status          receive status and errors
 @Param[in]     position        position of buffer in frame
 @Param[in]     h_BufContext    A handle of the user associated with this buffer

 @Retval        e_RX_STORE_RESPONSE_CONTINUE - order the driver to continue Rx
                                               operation for all ready data.
 @Retval        e_RX_STORE_RESPONSE_PAUSE    - order the driver to stop Rx operation.
*//***************************************************************************/
e_RxStoreResponse DPAPORT_ImRxStoreCB (t_Handle h_DpaPort,
                                       uint8_t  *p_Data,
                                       uint16_t length,
                                       uint16_t status,
                                       uint8_t  position,
                                       t_Handle h_BufContext);

/**************************************************************************//**
 @Function      DPAPORT_ImTxConfCB

 @Description   This is a routine to deal with confirmed transmitted frames.
                It should be used as the f_TxConf callback for the FM IM Port initialization.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.
 @Param[in]     p_Data          A pointer to data received
 @Param[in]     status          transmit status and errors
 @Param[in]     h_BufContext    A handle of the user associated with this buffer

*//***************************************************************************/
void DPAPORT_ImTxConfCB (t_Handle   h_DpaPort,
                         uint8_t    *p_Data,
                         uint16_t   status,
                         t_Handle   h_BufContext);

/**************************************************************************//**
 @Function      DPAPORT_ExceptionCB

 @Description   This is a routine to deal with exceptions. It should be used as
                the f_Exception callback for the FM Port initialization.

 @Param[in]     h_DpaPort       A handle to a DPA-Port module.
 @Param[in]     ex              The exception that occurred.

*//***************************************************************************/
void DPAPORT_ExceptionCB (t_Handle   h_DpaPort, uint32_t ex);

/** @} */ /* end of DPA_port_runtime_data_grp group */
/** @} */ /* end of DPA_port_grp group */


#endif /* __DPA_PORT_EXT_H */
