/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          duart_ext.h

 @Description   DUART UART external structures and definitions.
*//***************************************************************************/

#ifndef __DUART_EXT_H
#define __DUART_EXT_H

/**************************************************************************//**
 @Group         duart_id DUART Application Programming Interface

 @Description   DUART functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         duart_init DUART Initialization Unit

 @Description   DUART initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   User callback function, called if driver Rx buffer is full.
                The user provides this function and the driver invokes it.

 @Param[in]     h_App - Application handle given by the user at driver configuration.
 *//***************************************************************************/
typedef void (t_DuartBufferCallBack)(t_Handle h_App);

/**************************************************************************//**
 @Description   User callback function, called after all bytes are transferred.
                The user provides this function and the driver invokes it.
                If the function exists, it is called after all the bytes in
                UART Tx buffer are tranferred.

 @Param[in]     h_App       - Application handle given by the user at driver
                              configuration.
 @Param[in]     p_Buffer    - Pointer to the UART internal Tx buffer.
*//***************************************************************************/
typedef void (t_DuartTxConf)(t_Handle h_App, uint8_t *p_Buffer);

/**************************************************************************//**
 @Description   User callback function, called by driver for event handling.
                The user provides this function and the driver invokes it.

 @Param[in]     h_App   - Application handle given by the user at driver
                          configuration.
 @Param[in]     errors  - Events that caused this routine to be
                          invoked by the driver.
 @Param[in]     flags   - Protocol dependant indicator (unused).

 *//***************************************************************************/
typedef void (t_DuartException)(t_Handle h_App, uint32_t errors, uint32_t flags);

/**************************************************************************//**
 @Collection   Baud rate dividor edge values
 *//***************************************************************************/
#define BAUD_RATE_MIN_VAL       600
#define BAUD_RATE_MAX_VAL       512000
/* @} */

/**************************************************************************//**
 @Description   Data bits
 *//***************************************************************************/
typedef enum e_DuartUartDataBits
{
    e_DUART_DATA_BITS_5 = 0x00, /**< 5 bit word */
    e_DUART_DATA_BITS_6 = 0x01, /**< 6 bit word */
    e_DUART_DATA_BITS_7 = 0x02, /**< 7 bit word */
    e_DUART_DATA_BITS_8 = 0x03  /**< 8 bit word */
} e_DuartUartDataBits;

/**************************************************************************//**
 @Description   Stop bits
 *//***************************************************************************/
typedef enum e_DuartUartStopBits
{
    e_DUART_STOP_BITS_1 = 0x00, /**< 1 stop bit      */
    e_DUART_STOP_BITS_2 = 0x04  /**< 1.5/2 stop bits */
} e_DuartUartStopBits;

/**************************************************************************//**
 @Description    Parity
 *//***************************************************************************/
typedef enum e_DuartUartParity
{
    e_DUART_PARITY_ODD,     /**< Odd parity   */
    e_DUART_PARITY_EVEN,    /**< Even parity  */
    e_DUART_PARITY_SPACE,   /**< Space parity */
    e_DUART_PARITY_MARK,    /**< Mark parity  */
    e_DUART_PARITY_NONE     /**< No parity    */
} e_DuartUartParity;

/**************************************************************************//**
 @Description    Flow Control
 *//***************************************************************************/
typedef enum e_DuartFlowControl
{
    e_DUART_HW_FLOW_CONTROL,    /**< Hardware flow control mode */
    e_DUART_NO_FLOW_CONTROL     /**< No flow control mode       */
} e_DuartFlowControl;

/**************************************************************************//**
 @Description    Size of UART fifo in DUART
 *//***************************************************************************/
typedef enum e_DuartUartFifoSize
{
    e_DUART_FIFO_01 = 0x00,     /**< Rx FIFO interrupt trigger level 1 */
    e_DUART_FIFO_04 = 0x40,     /**< Rx FIFO interrupt trigger level 4 */
    e_DUART_FIFO_08 = 0x80,     /**< Rx FIFO interrupt trigger level 8 */
    e_DUART_FIFO_14 = 0xc0      /**< Rx FIFO interrupt trigger level 14 */
} e_DuartUartFifoSize;

/**************************************************************************//**
@Description   Duart error type.

               The error type is sent as a parameter in the f_Exceptions
               callback function.
*//***************************************************************************/
typedef enum e_DuartUartErrorType
{
    e_DUART_RECEIVER_FIFO_ERROR = 0x80, /**< One the characters in the received
                                             FIFO encountered an error */
    e_DUART_FRAMING_ERROR       = 0x08, /**< Invalid Stop Bit for received data*/
    e_DUART_PARITY_ERROR        = 0x04, /**< Unexpected parity value received*/
    e_DUART_OVERRUN_ERROR       = 0x02  /**< Received character was overrun
                                             before it was read*/
} e_DuartUartErrorType;

/**************************************************************************//**
 @Description   DUART protocol initialization structure.

                Structure is used when calling DUART_UART_Config
 *//***************************************************************************/
typedef struct t_DuartUartParam
{
    /* UART basic parameters */
    uint32_t                baudRate;           /**< Baud-rate to use - same for Tx and Rx  */
    e_DuartUartParity       parity;             /**< Parity - same for Tx and Rx */
    e_DuartUartDataBits     dataBits;           /**< Data bits per char */
    e_DuartUartStopBits     stopBits;           /**< Stop bits per char */
    e_DuartFlowControl      flowControl;        /**< Flow control of the channel */

    /* System integration parameters */
    uint32_t                baseAddress;        /**< Base address of DUART registers in memory. */
    int                     irq;                /**< DUART interrupt ID (system-specific number) */
    uint32_t                systemClockMhz;     /**< System clock frequency */

    /* Application interface */
    t_Handle                h_App;              /**< Handle to the upper layer handle */
    t_DuartBufferCallBack   *f_LowSpaceAlert;   /**< Overflow alert user callback function */
    t_DuartTxConf           *f_TxConf;          /**< Transmit finished user callback function */
    t_DuartException        *f_Exceptions;      /**< Callback invoked by driver on exception events;
                                                     If NULL, no exceptions are reported */
} t_DuartUartParam;

/**************************************************************************//**
 @Function      DUART_Config

 @Description   Creates DUART channel descriptor according to user parameters.

                User parameters are the minimum necessary to specify duart channel
                behavior. Additional parameters can be specified later using
                DUART advanced configuartion functions.

 @Param[in]     p_DuartUartParam - Pointer to structure of user-specified parameters.

 @Return        DUART channel descriptor. NULL if failed.
*//***************************************************************************/
t_Handle DUART_Config(t_DuartUartParam *p_DuartUartParam);

/**************************************************************************//**
 @Function      DUART_Init

 @Description   Fully initializes DUART channel and enables it.

 @Param[in]     h_Uart - DUART channel descriptor returned by DUART_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_Init(t_Handle h_Uart);

/**************************************************************************//**
 @Function      DUART_Free

 @Description   Frees all resources used by DUART.

 @Param[in]     h_Uart - DUART descriptor returned by DUART_Config().

 @Return        None.

 @Cautions      Does not disable DUART.
*//***************************************************************************/
void DUART_Free(t_Handle h_Uart);


/**************************************************************************//**
 @Group         duart_advanced_init_grp DUART Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

                These functions can be called only after DUART_Config() and
                before DUART_Init().
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DUART_ConfigRxBufferSize

 @Description   Configures the Rx Buffer Size.

                Default value is [1024].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - New value for rx BufferSize (in bytes).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigRxBufferSize(t_Handle h_Uart, uint32_t newVal);

/**************************************************************************//**
 @Function      DUART_ConfigRxTimeout

 @Description   Configures the Rx Timeout.

                If timeout is reached, the driver will return from the Rx routine.
                Default value is [1000000].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - New value for Rx Timeout. The value sets how many
                          times the driver will check for new chars before all
                          bytes are received.
                          - If 0, the driver won't wait for chars to be received.
                          - If -1, the driver will wait infinitely until all chars
                            are received.
                          - For any other value, the number of times to check for
                            each new char equals the new value.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigRxTimeout(t_Handle h_Uart, int32_t newVal);

/**************************************************************************//**
 @Function      DUART_ConfigEnModemStatIntr

 @Description   Enables or disables the Modem Status Interrupt.

                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables the interrupt.
                          If FALSE, disables the interrupt.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigEnModemStatIntr(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigEnRecLineStatIntr

 @Description   Enable or disables the Receiver Line Status Interrupt.

                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables the interrupt.
                          If FALSE, disables the interrupt.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigEnRecLineStatIntr(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigEnTransHoldRegEmptyIntr

 @Description   Enables or disables the Transmitter Holding Register Empty Interrupt.

                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables the interrupt.
                          If FALSE, disables the interrupt.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigEnTransHoldRegEmptyIntr(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigEnRecDataAvailIntr

 @Description   Enables or disables the Receiver Data Available Interrupt.

                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables the interrupt.
                          If FALSE, disables the interrupt.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigEnRecDataAvailIntr(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigRecTriggerLevel

 @Description   Configures the number of chars in FIFO before triggering
                an interrupt.

                Default value is [e_DUART_FIFO_01].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - Number of chars before triggering the interrupt.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigRecTriggerLevel(t_Handle h_Uart, e_DuartUartFifoSize newVal);

/**************************************************************************//**
 @Function      DUART_ConfigDmaModeSelect

 @Description   Enables or disables DMA mode in FIFO mode.

                In DMA mode the transmitter is ready only when the FIFO is full
                and the receiver is ready only when the trigger level has been
                reached.
                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables DMA mode.
                          If FALSE, disables DMA mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigDmaModeSelect(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigEnableFifo

 @Description   Enables or disables the FIFO.

                Default value is [TRUE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables FIFO mode.
                          If FALSE, disables FIFO mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigEnableFifo(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigLoopBackMode

 @Description   Enables or disables loopback mode.

                In loopback mode all chars sent from each UART are transferred
                back to the same UART.

                Default value is [FALSE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, enables loopback mode.
                          If FALSE, disables loopback mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigLoopBackMode(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigPollMode

 @Description   Enables or disables poll mode. If poll mode is disabled then
                the driver works on interrupt mode.

                Default value is [FALSE].

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, system operates in poll mode.
                          If FALSE, system operates in interrupt mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigPollMode(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigPollLf2CrLf

 @Description   Enables or disables Lf2CrLf mode.

                Default value is [FALSE].

                When enabled in Poll mode, converts LF to CR LF.

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal  - If TRUE, system converts LF to CR LF.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigPollLf2CrLf(t_Handle h_Uart, bool newVal);

/**************************************************************************//**
 @Function      DUART_ConfigControlChar

 @Description   Configures the control characters.

                When a control char is received Rx stops receiving bytes. Every
                control char can be received with the data before it or rejected
                from the buffer.
                Default values are ["\r\3\0\0\0\0\0\0"], ["\0\1\0\0\0\0\0\0"],
                [2].

 @Param[in]     h_Uart              - DUART channel descriptor returned by DUART_Config().
 @Param[in]     *controlChar        - The 8 control characters.
 @Param[in]     *controlCharReject  - 8 Control Char Reject values. If a value
                                      is zero then the control char in the same
                                      index will not be rejected from the buffer.
                                      If the value is non-zero, the control char
                                      will be rejected.
 @Param[in]     numCCEntries        - Number of control chars. Must be <= 8.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_ConfigControlChar(t_Handle h_Uart, char *controlChar, bool *controlCharReject, uint8_t numCCEntries);

/** @} */ /* end of duart_advanced_init_grp group */

/** @} */ /* end of duart_init group */

/**************************************************************************//**
 @Group         duart_runtime DUART Runtime Unit

 @Description   DUART Run Time functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DUART_Tx

 @Description   Sends a buffer through the specified DUART channel.

                This functions is called when the user wishes to transmit data.

 @Param[in]     h_Uart  - DUART channel descriptor returned by DUART_Config().
 @Param[in]     p_Data  - Pointer to a transmit buffer.
 @Param[in]     size    - Number of characters to send.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_Tx(t_Handle h_Uart, uint8_t *p_Data, uint32_t size);

/**************************************************************************//**
 @Function      DUART_Rx

 @Description   Collects the received buffer.

                This function is called by the user to collect data which has
                been received by the UART.

                The function will return when one of the following occurs:
                - The number of characters set by the user has been received.
                - RxTimeout has been reached.
                - A control character has been received.

 @Param[in]     h_Uart      - DUART channel descriptor returned by DUART_Config().
 @Param[in]     p_Buffer    - Points to a buffer which will store the received data.
 @Param[in]     size        - Number of charcters to receive.

 @Return        Size of received buffer.
*//***************************************************************************/
uint32_t DUART_Rx(t_Handle h_Uart, uint8_t *p_Buffer, uint32_t size);

/** @} */ /* end of duart Runtime group */

/**************************************************************************//**
 @Group         duart_control DUART Control Unit

 @Description   DUART Control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      DUART_GetNumOfRxCharInBuffer

 @Description   Returns the number of chars in the received buffer.

                This function can be useful to know what value to set
                in the 'size' parameter of the DUART_Rx() routine.

 @Param[in]     h_Uart - DUART channel descriptor returned by DUART_Config().

 @Return        Number of chars in the received buffer.
*//***************************************************************************/
uint32_t DUART_GetNumOfRxCharInBuffer(t_Handle h_Uart);

/**************************************************************************//**
 @Function      DUART_SetLowSpaceAlert

 @Description   Sets the triggel level for the overflow alert callback function.
                Default value is [128].

 @Param[in]     h_Uart - DUART channel descriptor returned by DUART_Config().
 @Param[in]     newVal - The number of empty bytes in the Rx Buffer which will
                         cause an overflow. Once the Rx buffer has this number
                         of empty bytes or less the overflow alert callback
                         funciton is called.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_SetLowSpaceAlert(t_Handle h_Uart, uint32_t newVal);


/**************************************************************************//**
 @Function      DUART_SetBreakSignal

 @Description   Sets the break signal.

 @Param[in]     h_Uart - DUART channel descriptor returned by DUART_Config().
 @Param[in]     breakSignal - indication if to set or clear the break signal option.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_SetBreakSignal (t_Handle h_Uart, bool breakSignal);

/**************************************************************************//**
 @Function      DUART_SetBaudRate

 @Description   Sets a new value of the baud rate divisor.

 @Param[in]     h_Uart   - DUART channel descriptor returned by DUART_Config().
 @Param[in]     baudRate - New baud rate value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      To ensure proper operation, the baud rate can only be modified
                when no data transfer or data reception are in progress.
*//***************************************************************************/
t_Error DUART_SetBaudRate (t_Handle h_Uart, uint32_t baudRate);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      DUART_DumpRegs

 @Description   Dumps the memory of DUART module

 @Param[in]     h_Uart - DUART channel descriptor returned by DUART_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error DUART_DumpRegs(t_Handle h_Uart);

#endif

/** @} */ /* end of Control group */
/** @} */ /* end of Duart Programming Interface group */


#endif /* __DUART_EXT_H */
