/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          espi_ctrl_ext.h

 @Description   External definitions and API of the eSPI controller driver.

 @Cautions      None.
*//***************************************************************************/

#ifndef __ESPI_CTRL_EXT_H
#define __ESPI_CTRL_EXT_H


#include "part_ext.h"
#include "spi_std_ext.h"


/**************************************************************************//**
 @Group         espi_ctrl_grp   eSPI Controller Application Programming Interface

 @Description   eSPI controller functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         espi_ctrl_init_grp   eSPI Controller Initialization Unit

 @Description   eSPI controller initialization functions and definitions.

 @{
*//***************************************************************************/

#define ESPI_NUM_OF_HW_CHIP_SELECTS     4   /**< Number of chip selects supported */


/**************************************************************************//**
 @Description   Transfer completion callback function - called at end of any
                Tx/Rx transfer.

 @Param[in]     h_App       - Handle to the user's object, as provided in the
                              t_EspiCtrlParam configuration parameters structure.
 @Param[in]     h_Context   - Handle to the user's context for the specific transfer,
                              as provided in the ESPI_CTRL_TxRx() routine.
 @Param[in]     status      - Transfer completion status: zero if transfer was
                              completed successfully.

 @Return        None.
*//***************************************************************************/
typedef void (t_EspiCtrlTransferDoneCb)(t_Handle h_App, t_Handle h_Context, uint32_t status);

/**************************************************************************//**
 @Description   eSPI controller driver configuration parameters.
*//***************************************************************************/
typedef struct t_EspiCtrlParam
{
    /* System integration parameters */
    uint32_t                    baseAddress;        /**< Base address of eSPI controller registers in memory. */
    uint32_t                    sourceClockFreqHz;  /**< The source clock frequency to the eSPI baud rate generator [Hz] */
    int                         irq;                /**< eSPI controller interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle                    h_App;              /**< User's handle - will be passed in the
                                                         transfer completion callback without change */
    t_EspiCtrlTransferDoneCb    *f_TransferDone;    /**< Transfer complete callback routine */

} t_EspiCtrlParam;


/**************************************************************************//**
 @Function      ESPI_CTRL_Config

 @Description   Configure the eSPI controller driver according to the user's
                configuration parameters structure.

                The driver decides on a few default values for the advanced
                configuration options. These default values may be changed using
                the set of advanced configuration routines.

                The driver does not initialize any of the controller registers
                at this phase.

                The returned handle must be passed to any consequent eSPI routine.

 @Param[in]     p_EspiCtrlParam - Pointer to eSPI controller configuration parameters.

 @Return        Handle to the eSPI controller; NULL if configuration failed.
*//***************************************************************************/
t_Handle ESPI_CTRL_Config(t_EspiCtrlParam *p_EspiCtrlParam);

/**************************************************************************//**
 @Function      ESPI_CTRL_Init

 @Description   Initialize the eSPI controller registers according to previous
                configuration.

 @Param[in]     h_EspiCtrl - Handle to the eSPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_Init(t_Handle h_EspiCtrl);

/**************************************************************************//**
 @Function      ESPI_CTRL_Free

 @Description   Free the eSPI controller driver object.

                This routine may be called at any time after ESPI_CTRL_Config(),
                to release the eSPI driver object and make it as unusable.

                The routine disables the eSPI controller before freeing the
                driver.

 @Param[in]     h_EspiCtrl - Handle to the eSPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_Free(t_Handle h_EspiCtrl);


/**************************************************************************//**
 @Group         espi_ctrl_advanced_init_grp  eSPI Controller Advanced Configuration Unit

 @Description   eSPI controller advanced configuration functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ESPI_CTRL_ConfigOpenDrain

 @Description   Configures the open-drain mode of the eSPI controller pins.
                [FALSE]

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     openDrain   - FALSE - All pins are actively driven;
                              TRUE  - All pins are set to open-drain mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_ConfigOpenDrain(t_Handle h_EspiCtrl, bool openDrain);

/**************************************************************************//**
 @Function      ESPI_CTRL_ConfigPollingMode

 @Description   Configures the eSPI controller to operate in interrupts or
                polling mode. [FALSE]

                In polling mode, the driver disables the controller interrupts
                and calls to ESPI_CTRL_TxRx() are blocking until the transfer
                is completed.

                In interrupts mode, the driver operates in non-blocking mode.

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     polling     - FALSE for interrupt mode; TRUE for polling mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_ConfigPollingMode(t_Handle h_EspiCtrl, bool polling);

/**************************************************************************//**
 @Function      ESPI_CTRL_ConfigDataOutputDelay

 @Description   Configures the data output hold time, which may be useful for
                communicating with devices in RapidS mode. [0]

 @Param[in]     h_EspiCtrl      - Handle to the eSPI controller driver object.
 @Param[in]     dataOutputDelay - Data output hold time, in eSPI clock cycles (0-7).

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_ConfigDataOutputDelay(t_Handle h_EspiCtrl, uint8_t dataOutputDelay);

/**************************************************************************//**
 @Function      ESPI_CTRL_ConfigChipSelectTiming

 @Description   Configures additional delays (pre-frame setup time, post-frame
                hold time and inter-frame gap) for the given chip select signal.
                [Pre-frame setup time: 0;
                 Post-frame hold time: 0;
                 Inter-frame gap: 1]

 @Param[in]     h_EspiCtrl          - Handle to the eSPI controller driver object.
 @Param[in]     chipSelect          - Chosen chip select.
 @Param[in]     preFrameSetupTime   - Chip select assertion time in bits before frame start (0-15).
 @Param[in]     postFrameHoldTime   - Chip select assertion time in bits after frame end (0-15).
 @Param[in]     interFrameGap       - Chip select negation time in bits between frames (1-32).

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_ConfigChipSelectTiming(t_Handle   h_EspiCtrl,
                                         uint8_t    chipSelect,
                                         uint8_t    preFrameSetupTime,
                                         uint8_t    postFrameHoldTime,
                                         uint8_t    interFrameGap);

/**************************************************************************//**
 @Function      ESPI_CTRL_ConfigChipSelectPolarity

 @Description   Configures the polarity of the given chip select signal
                (use TRUE for active-high mode). [FALSE]

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     chipSelect  - Chosen chip select.
 @Param[in]     activeHigh  - FALSE for active-low mode (default);
                              TRUE for active-high mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_ConfigChipSelectPolarity(t_Handle h_EspiCtrl,
                                           uint8_t  chipSelect,
                                           bool     activeHigh);

/** @} */ /* end of espi_ctrl_advanced_init_grp */
/** @} */ /* end of espi_ctrl_init_grp */


/**************************************************************************//**
 @Group         espi_ctrl_runtime_grp    eSPI Controller Runtime Unit

 @Description   eSPI controller runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ESPI_CTRL_TxRx

 @Description   Initiates a transfer on the SPI bus.

                A typical transfer consists of both Tx and Rx data buffers, and
                an identical number of bytes to send and to read. However, the
                user may also request a Tx-only transfer, for which the p_RxData
                pointer should be set to NULL. The user may also request to skip
                a few Rx bytes at the start of the transfer. This mode is useful
                for reads of SPI flash memories, where the first valid data is
                received several characters after the transmission begins (after
                the instruction opcode and memory address have been transmitted).

                In polling mode, this routine is blocking until the transfer is
                completed. In interrupt mode, this routine is non-blocking, and
                the transfer progress is managed by the driver's interrupt handler.
                In both modes, transfer completion is reported to the user through
                the transfer completion callback function (if it is not NULL).

                The Tx data should contain byte-aligned consecutive words in memory.
                If the word size is 4-8 bits, each word should be right-aligned in
                a single byte. If the word size is 9-16 bits, each word should be
                right-aligned in two bytes. The same format is applied to received
                data.

                The user may provide a private context for each transfer. This
                context will be returned in the transfer completion callback
                without change.

 @Param[in]     h_EspiCtrl      - Handle to the eSPI controller driver object.
 @Param[in]     p_TxData        - Pointer to data for transmission.
 @Param[in]     p_RxData        - Pointer to a data buffer to fill with read data;
                                  The buffer size must be at least rxBytesToRead bytes.
 @Param[in]     txBytesToSend   - Number of bytes to transmit.
 @Param[in]     rxBytesToSkip   - Number of received bytes to skip (discard).
 @Param[in]     rxBytesToRead   - Number of received bytes to put in the Rx data buffer.
 @Param[in]     h_Context       - User's context for this transfer - will be passed
                                  in the transfer completion callback without change.

 @Return        E_OK if successful; Error code otherwise.

 @Cautions      The user must not call this routine to initiate a new transfer
                before the previous transfer was completed.
*//***************************************************************************/
t_Error ESPI_CTRL_TxRx(t_Handle h_EspiCtrl,
                       uint8_t  *p_TxData,
                       uint8_t  *p_RxData,
                       uint16_t txBytesToSend,
                       uint8_t  rxBytesToSkip,
                       uint16_t rxBytesToRead,
                       t_Handle h_Context);

/** @} */ /* end of espi_ctrl_runtime_grp */


/**************************************************************************//**
 @Group         espi_ctrl_control_grp    eSPI Controller Control Unit

 @Description   eSPI controller control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ESPI_CTRL_Enable

 @Description   Enable the eSPI controller to allow transfers on the bus.

                Normally, the user should not call this routine, because the
                controller is automatically enabled when ESPI_CTRL_TxRx() is
                called.

 @Param[in]     h_EspiCtrl - Handle to the eSPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_Enable(t_Handle h_EspiCtrl);

/**************************************************************************//**
 @Function      ESPI_CTRL_Disable

 @Description   Disable the eSPI controller.

                The user may use this routine to move the controller into
                idle, power-saving mode. If a transfer is active when this
                routine is called, it will be terminated. Thus, it is not
                recommended to call this routine before the current transfer is
                completed.

 @Param[in]     h_EspiCtrl - Handle to the eSPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_Disable(t_Handle h_EspiCtrl);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetActiveChipSelect

 @Description   Set the active chip select of the eSPI controller.

                The eSPI controller may issue a single transfer at a time,
                thus only one chip select can be active at a time. This routine
                determines the chip select for the next transfer.

                Consequent calls to any of the ESPI_CTRL_Set<Property> routines
                shall set the properties of the active chip select only.

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     chipSelect  - The chip select to set as active.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetActiveChipSelect(t_Handle h_EspiCtrl, uint8_t chipSelect);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetBusFrequency

 @Description   Set the SPI bus frequency for the active chip select.

                The new bus frequency will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

                The exact frequency may not be reached due to divisor limitations.
                The routine will set the closest possible frequency and return it
                in the p_ActualFreq parameter.

 @Param[in]     h_EspiCtrl      - Handle to the eSPI controller driver object.
 @Param[in]     busFrequencyHz  - The required SPI bus frequency [Hz].
 @Param[out]    p_ActualFreq    - Returns the actual bus clock frequency that
                                  was set by the driver.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetBusFrequency(t_Handle  h_EspiCtrl,
                                  uint32_t  busFrequencyHz,
                                  uint32_t  *p_ActualFreq);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetBitOrder

 @Description   Set the SPI bit ordering mode for the active chip select.

                The new bit ordering mode will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     bitOrder    - The selected bit order mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetBitOrder(t_Handle h_EspiCtrl, e_SpiBitOrder bitOrder);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetWordSize

 @Description   Set the SPI word size for the active chip select.

                The new word size will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     bitsInWord  - The SPI word size in bits (4-16 bits).

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetWordSize(t_Handle h_EspiCtrl, uint8_t bitsInWord);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetClockPolarity

 @Description   Set the SPI clock polarity for the active chip select.

                The new clock polarity will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     polarity    - The selected clock polarity.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetClockPolarity(t_Handle h_EspiCtrl, e_SpiClockPolarity polarity);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetClockPhase

 @Description   Set the SPI clock phase for the active chip select.

                The new clock phase will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     phase       - The selected clock phase.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetClockPhase(t_Handle h_EspiCtrl, e_SpiClockPhase phase);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetFullCycleRapidS

 @Description   Set the full-cycle mode for the active chip select, which is
                useful for communicating with devices in RapidS mode.

                The full-cycle mode will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl      - Handle to the eSPI controller driver object.
 @Param[in]     fullCycleRapidS - FALSE for normal SPI operation;
                                  TRUE for full-cycle SPI operation (RapidS mode).

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetFullCycleRapidS(t_Handle h_EspiCtrl, bool fullCycleRapidS);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetDualOutputRead

 @Description   Set the dual-output read mode for the active chip select.

                The dual-output read mode will be effective only in the next transfer.

                This routine affects only the active chip select, as was set by
                the last call to ESPI_CTRL_SetActiveChipSelect().

 @Param[in]     h_EspiCtrl      - Handle to the eSPI controller driver object.
 @Param[in]     dualOutputRead  - FALSE for normal SPI operation;
                                  TRUE for using dual-output read mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetDualOutputRead(t_Handle h_EspiCtrl, bool dualOutputRead);

/**************************************************************************//**
 @Function      ESPI_CTRL_SetLoopbackMode

 @Description   Set the eSPI controller loopback mode, which may be useful in
                debug phase.

                The loopback mode affects all chip selects, but will be effective
                only in the next transfer. If the user wants to use loopback mode
                for a subset of the chip selects, this mode must be disabled
                when other chip selects are set active.

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.
 @Param[in]     loopEnabled - FALSE for normal SPI operation;
                              TRUE to enable controller loopback mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_SetLoopbackMode(t_Handle h_EspiCtrl, bool loopEnabled);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      ESPI_CTRL_DumpRegs

 @Description   Prints the current values of all eSPI controller registers.

 @Param[in]     h_EspiCtrl  - Handle to the eSPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error ESPI_CTRL_DumpRegs(t_Handle h_EspiCtrl);
#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of espi_ctrl_control_grp */
/** @} */ /* end of espi_ctrl_grp */


#endif /* __ESPI_CTRL_EXT_H */

