/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          etsec_ext.h

 @Description   eTSEC Application Programming Interface
*//***************************************************************************/
#ifndef __ETSEC_EXT_H
#define __ETSEC_EXT_H

#include "enet_ext.h"
#include "xx_ext.h"
#include "error_ext.h"

#define ETSEC_NUM_QUEUES_TX 8
#define ETSEC_NUM_QUEUES_RX 8

/**************************************************************************//**
 @Group         etsec_grp eTSEC Application Programming Interface

 @Description   eTSEC API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         etsec_init_grp eTSEC Initialization Unit

 @Description   eTSEC initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   eTSEC port number
*//***************************************************************************/
typedef enum e_Etsec
{
    e_ETSEC_0   = 0,    /**< eTSEC port 0 */
    e_ETSEC_1,          /**< eTSEC port 1 */
    e_ETSEC_2,          /**< eTSEC port 2 */
    e_ETSEC_3,          /**< eTSEC port 3 */
    e_ETSEC_DUMMY_LAST
} e_Etsec;

#ifdef ETSEC_REV2
/**************************************************************************//**
 @Description   User's callback routine events handling (invoked by the driver).

 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to ETSEC_Config().
 @Param[in]     events  - Events that caused this routine to be invoked by
                          the driver.
 @Param[in]     flags   - Protocol dependent indicator, indicates halted queues
                          when used with one of Tx Error events (IMASK_EBERREN,
                          IMASK_LCEN, IMASK_CRLEN or IMASK_XFUNEN).

 @Return        None.
 *//***************************************************************************/
#else
/**************************************************************************//**
 @Description   User's callback routine events handling (invoked by the driver).

 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to ETSEC_Config().
 @Param[in]     events  - Events that caused this routine to be invoked by
                          the driver.
 @Param[in]     flags   - Protocol dependent indicator, indicates halted queues
                          when used with Tx Error event (IMASK_TXEEN).

 @Return        None.
 *//***************************************************************************/
#endif  /* ETSEC_REV2 */
typedef void (t_EtsecExceptionCallback)(t_Handle h_App, uint32_t events, uint32_t flags);


/*********************************************************************************//**
 @Description   eTSEC initialization structure, used when calling ETSEC_Config().
*//**********************************************************************************/
typedef struct t_EtsecParam
{
    /* Ethernet protocol parameters */
    t_EnetAddr                  macAddress;         /**< MAC address of device; First octet is sent first */
    e_EnetMode                  enetMode;           /**< Ethernet physical interface */
    bool                        halfDuplex;         /**< TRUE for half duplex; FALSE for full duplex */

    /* System integration parameters */
    uint32_t                    baseAddress;        /**< Base address of eTSEC registers in memory;
                                                         In multicore processors this is group 0 address. */
    e_Etsec                     etsecId;            /**< eTSEC port number */
    int                         rxIrq;              /**< eTSEC Rx interrupt ID (system-specific number) */
    int                         errorIrq;           /**< eTSEC error interrupt ID (system-specific number) */
    t_BufferPoolInfo            bufferPoolInfo;     /**< Data buffer pool information */
    uint32_t                    dataMemAttributes;  /**< Memory attributes mask for data buffers */

    /* Application interface */
    t_Handle                    h_App;              /**< Handle to the upper layer handler. */
    t_RxStoreFunction           *f_RxStore;         /**< Pointer to user callback called by
                                                         driver with received data */
    t_TxConfFunction            *f_TxConf;          /**< Pointer to user callback function called
                                                         by driver to confirm a transmitted buffer
                                                         to the application. */
    t_EtsecExceptionCallback    *f_Exceptions;      /**< Pointer to the application's callback routine
                                                         for handling eTSEC exceptions; This routine is
                                                         invoked by the driver when eTSEC events occur;
                                                         If NULL,  exceptions are not reported */
} t_EtsecParam;


/**************************************************************************//**
 @Function      ETSEC_Config
 
 @Description   Creates and configures the eTSEC object.
 
                This routine must be called prior to any other eTSEC routine.
                Some parameters are set implicitly by the driver to their
                default values.

 @Param[in]     p_EtsecParam - Pointer to the configuration parameters structure;

 @Return        Handle to the initialized eTSEC object.
                This handle uniquely identifies the eTSEC and shall be used
                when calling other eTSEC API routines.
*//***************************************************************************/
t_Handle ETSEC_Config(t_EtsecParam *p_EtsecParam);

/**************************************************************************//**
 @Function      ETSEC_Init
 
 @Description   Initialization of eTSEC unit.
 
                This routine initializes the Ethernet Controller according
                to the parameters provided previously by the configurations
                routines.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
*//***************************************************************************/
t_Error ETSEC_Init(t_Handle h_Etsec);

/**************************************************************************//**
 @Function      ETSEC_Free

 @Description   This routine is called to shut down the Ethernet Controller.
                It disables the transmission and reception of frames and
                releases all previously allocated resources.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_Free (t_Handle h_Etsec);


/**************************************************************************//**
 @Group         etsec_advanced_init_grp ETSEC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

#ifdef ETSEC_L2_CACHE_SUPPORT
/**************************************************************************//**
 @Description   L2 cache write type
*//***************************************************************************/
typedef enum e_EtsecL2CacheWriteType
{
    e_ETSEC_NO_ALLOC       = 0x0, /**< No allocation performed */
    e_ETSEC_ALLOC          = 0x2, /**< Allocate L2 cache line  */
    e_ETSEC_ALLOC_AND_LOCK = 0x3  /**< Allocate and lock L2 cache line  */
} e_EtsecL2CacheWriteType;

#endif /* ETSEC_L2_CACHE_SUPPORT */

/**************************************************************************//**
 @Description   eTSEC Tx scheduling policy
*//***************************************************************************/
typedef enum e_EtsecSched
{
    e_ETSEC_SCHED_SINGLE    = 0,    /**< Single polled ring mode -
                                         only Tx queue 0 is polled */
    e_ETSEC_SCHED_PRIORITY  = 1,    /**< Priority queue mode - Tx queue with
                                         lower index has higher priority */
    e_ETSEC_SCHED_WRR       = 2     /**< Weighted round robin */
} e_EtsecSched;

/**************************************************************************//**
 @Description   eTSEC Rx parsing level
*//***************************************************************************/
typedef enum e_EtsecPrsdep
{
    e_ETSEC_PRSDEP_DIS    = 0x0,    /**< Parsing is disabled */
    e_ETSEC_PRSDEP_L2     = 0x1,    /**< Parsing is enabled for layer 2 */
    e_ETSEC_PRSDEP_L2L3   = 0x2,    /**< Parsing is enabled for layers 2 and 3 */
    e_ETSEC_PRSDEP_L2L3L4 = 0x3     /**< Parsing is enabled for layers 2, 3 and 4 */
} e_EtsecPrsdep;

/**************************************************************************//**
 @Description   eTSEC interrupt coalescing parameters
*//***************************************************************************/
typedef struct t_EtsecIntrCoalescingParam
{
    bool        enable;             /**< Enable or disable interrupt coalescing */
    uint8_t     countThreshold;     /**< Number of frames to receive prior to raising
                                         an interrupt */
    uint16_t    timerThreshold;     /**< Maximum amount of time after receiving a frame
                                         and before raising an interrupt - if frame has
                                         been received but frame count threshold has not
                                         been met, an interrupt is raised when the threshold
                                         timer reaches zero; the timer threshold value is
                                         represented in units equal to 64 periods of eTSEC
                                         Rx interface clock */
} t_EtsecIntrCoalescingParam;


/**************************************************************************//**
 @Function      ETSEC_ConfigNumQueuesTx

 @Description   This routine configures amount of Tx queues.
                [1].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Number of Tx queues

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigNumQueuesTx(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigNumQueuesRx

 @Description   This routine configures amount of Rx queues.
                [1].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Number of Rx queues

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigNumQueuesRx(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigQueueBdRingLenTx

 @Description   Configures BD length for specified Tx queue.
                [64].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New length for Tx BD ring.
 @Param[in]     index   - BD ring index.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigQueueBdRingLenTx(t_Handle h_Etsec, uint16_t newVal, uint8_t index);

/**************************************************************************//**
 @Function      ETSEC_ConfigQueueBdRingLenRx

 @Description   Configures BD length for specified Rx queue.
                [64].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New length for Rx BD ring
 @Param[in]     index   - BD ring index

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigQueueBdRingLenRx(t_Handle h_Etsec, uint16_t newVal, uint8_t index);

/**************************************************************************//**
 @Function      ETSEC_ConfigRxInterruptEnable

 @Description   This routine configures Rx interrupt mode for
                all Rx queues.
                [TRUE].

 @Param[in]     h_Etsec             - Handle to the eTSEC object.
 @Param[in]     interruptEnabled    - Enable/disable Rx interrupts

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigRxInterruptEnable(t_Handle h_Etsec, bool interruptEnabled);

/**************************************************************************//**
 @Function      ETSEC_ConfigIntrManagerId

 @Description   Configures Rx queue to operate in interrupt manager mode.
                [0].
 
                Interrupt manager mode can be used only when operating
                in interrupt mode (see ETSEC_ConfigRxInterruptEnable()).

 @Param[in]     h_Etsec         - Handle to the eTSEC object.
 @Param[in]     intrManagerId   - The interrupt manager ID.
 @Param[in]     queue           - Index of Rx queue to configure

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigIntrManagerId(t_Handle h_Etsec, uint32_t intrManagerId, uint8_t queue);

#ifdef ETSEC_REV2
/**************************************************************************//**
 @Function      ESTEC_ConfigRxInterruptCoalescing

 @Description   This routine configures interrupt coalescing for
                specified Rx queue.
                [FALSE].
 
                Interrupt coalescing allows to raise interrupt to the core
                after receiving specified number of frames (as opposed
                to interrupting on every frame received).

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     p_Param     - Interrupt coalescing parameters
 @Param[in]     queueIndex  - Rx queue index

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ESTEC_ConfigRxInterruptCoalescing(t_Handle                      h_Etsec,
                                          t_EtsecIntrCoalescingParam    *p_Param,
                                          uint8_t                       queueIndex);

#else   /* ETSEC_REV2 */
/**************************************************************************//**
 @Function      ESTEC_ConfigRxInterruptCoalescing

 @Description   This routine configures interrupt coalescing for
                specified Rx queue.
                [FALSE].

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     p_Param     - Interrupt coalescing parameters

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ESTEC_ConfigRxInterruptCoalescing(t_Handle                      h_Etsec,
                                          t_EtsecIntrCoalescingParam    *p_Param);
#endif  /* ETSEC_REV2 */

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanTagInsert

 @Description   This routine enables VLAN tag insertion during transmitting a frame.
                [FALSE].
 
                The actual tag to be inserted is specified
                - either by frame control block (FCB) attached to Tx frame with valid
                VLAN control word bit set;
                - or if the latter is not present, by default VLAN tag (see
                ETSEC_ConfigVlanPri(), ETSEC_ConfigVlanCfi(), ETSEC_ConfigVlanId()).

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable VLAN tag insertion

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanTagInsert(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanId

 @Description   This routine configures default VLAN tag ID value to be
                inserted on Tx.
                [0x0].
 
                See also ETSEC_ConfigVlanTagInsert() description.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     vlanId  - The VLAN tag value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanId(t_Handle h_Etsec, uint16_t vlanId);

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanPri

 @Description   This routine configures default VLAN priority to be
                inserted on Tx.
                [0x0].

                See also ETSEC_ConfigVlanTagInsert() description.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - VLAN priority

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanPri(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanCfi

 @Description   This routine configures default VLAN canonical format indicator
                to be added to Tx frames.
                [0x1].
 
                See also ETSEC_ConfigVlanTagInsert() description.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Canonical frame indicator

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanCfi(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanType

 @Description   This routine configures the default ethertype used to tag VLAN frames.
                [0x8100].
 
                This ethertype is added prior to VLAN tag on transmit and
                used to match VLAN frames on receive.
 
 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - The default ethertype to tag VLAN frames

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanType(t_Handle h_Etsec, uint16_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigVlanTagExtraction

 @Description   This routine configures Rx VLAN tag extraction and deletion.
                [FALSE].
 
 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable VLAN tag extraction

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigVlanTagExtraction(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigIpCheckSumCheck

 @Description   This routine configures the IPv4 header checksum check on Rx.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling checksum checking.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigIpCheckSumCheck(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigIpCheckSumGenerate

 @Description   This routine configures the IPv4 header checksum generation on Tx.
                [FALSE].
 
                Tx frame control block (FCB) should be attached to frame with
                checksum IP header enable (CIP) bit set. IP/IP6 and layer 3
                offset values should be also set.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling checksum generation.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigIpCheckSumGenerate(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigTcpUdpCheckSumCheck

 @Description   This routine configures the TCP/UDP header checksum check on Rx.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling checksum checking.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigTcpUdpCheckSumCheck(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigTcpUdpCheckSumGenerate

 @Description   This routine configures the TCP/UDP header checksum generation on Tx.
                [FALSE].

                Tx frame control block (FCB) should be attached to frame with
                checksum TCP/UDP header enable (CTU) bit set. Layer 3 and 4 header
                and offset bits should be also set.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling checksum generation.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigTcpUdpCheckSumGenerate(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigExactMatch

 @Description   This routine configures the exact MAC address matching option.
                [TRUE].
 
                This function enables additional MAC addresses to be
                recognized by eTSEC receiver. Address recognition
                is performed by eTSEC receiver HW in the same way as
                station MAC address provided in t_EtsecParam structure.
 
                Up to 15 additional MAC addresses can be added/removed
                through ETSEC_StationAddressAdd() and ETSEC_StationAddressClear()
                functions.

                See also ETSEC_ConfigMultiIndivAddr() and ETSEC_ConfigMultiGrpAddr()
                for other MAC address recognition options.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE to enable additional exact match MAC addresses

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigExactMatch(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigMultiIndivAddr

 @Description   This routine configures multiple individual addresses.
                [FALSE].
 
                This function enables MAC address recognition of individual
                (vs. group) MAC addresses through HW hash table match. The
                driver complements hash result by checking full MAC address.
 
                Up to 256 MAC addresses can be added/removed through ETSEC_AddAddr()
                and ETSEC_RemoveAddr() control functions.
 
                See also ETSEC_ConfigExactMatch() and ETSEC_ConfigMultiGrpAddr()
                for other MAC address recognition options.
 
                Both exact match and multiple individual MAC address options
                allow for individual/non-multicast MAC address recognition by
                eTSEC receiver. In both cases Rx frame is forwarded to application
                only on full MAC address match. However, when using exact match option,
                the entire MAC address is verified by the HW while in hash option, HW
                checks address only partially (hashing) and exact MAC is verified
                by the SW driver. This makes the first (HW/exact match) option faster and
                preferable where possible.
 
                Note that multiple individual addresses can't be used if
                additional group addresses have been enabled through
                ETSEC_ConfigGroupHashExtend() routine.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling multiple addresses

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMultiIndivAddr(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigMultiGrpAddr

 @Description   This routine configures multiple group/multicast addresses.
                [FALSE].

                This function enables MAC address recognition of group
                (vs. individual) MAC addresses through HW hash table match. The
                driver complements hash result by checking full MAC address.

                Up to 256 or 512 multicast MAC addresses can be added/removed
                through ETSEC_AddAddr() and ETSEC_RemoveAddr() control functions.
                512 addresses option should be enabled through ETSEC_ConfigGroupHashExtend()
                routine.
 
                See also ETSEC_ConfigExactMatch() and ETSEC_ConfigMultiIndivAddr()
                for other MAC address recognition options.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE/FALSE for enabling/disabling multiple addresses

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMultiGrpAddr(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigGroupHashExtend

 @Description   This routine extends multicast addresses recognition support.
                [FALSE].
 
                This function extends number of group/multicast MAC addresses
                that can be recognized by eTSEC receiver from 256 to 512
                addresses.
 
                Group address recognition option should be enabled through
                ETSEC_ConfigMultiGrpAddr() routine.
 
                Note that when group hash extension option is used individual
                hash MAC addresses are not available (see ETSEC_ConfigMultiIndivAddr).
                However, exact match addresses still can be used (see ETSEC_ConfigExactMatch()).

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable group hash extension

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigGroupHashExtend(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigPromiscuous

 @Description   This routine configures eTSEC Rx controller to
                operate in promiscuous mode.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable promiscuous mode

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigPromiscuous(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigBroadcastAddressReject

 @Description   This routine configures Rx broadcast address policy.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable broadcast address rejection

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBroadcastAddressReject(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigTxSched

 @Description   This routine configures Tx scheduler mode.
                [e_ETSEC_SCHED_SINGLE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     txSched - Tx scheduling mode

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigTxSched(t_Handle h_Etsec, e_EtsecSched txSched);

/**************************************************************************//**
 @Function      ETSEC_ConfigWeightFactor

 @Description   This routine configures weight factor for Tx WRR queuing policy.
                [0].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     weight  - Queue weight
 @Param[in]     index   - Queue index

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigWeightFactor(t_Handle h_Etsec, uint8_t weight, uint8_t index);

/**************************************************************************//**
 @Function      ETSEC_ConfigExceptions

 @Description   This routine configures the interrupt mask for eTSEC.
                [uint32_t].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New interrupt mask

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().

 @See           @ref etsec_exbm "eTSEC Exceptions Bit Mask".
*//***************************************************************************/
t_Error ETSEC_ConfigExceptions(t_Handle h_Etsec, uint32_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigTbiPhysicalAddress

 @Description   This routine configures TBI phy address.
                [5].
 
                This phy is used internally by eTSEC in TBI/RTBI/SGMII
                mode. It is managed through MII MNG interface of eTSEC
                port represented by h_Etsec handle. Since MII MNG registers
                of eTSEC-1 port are used for configuration of all eTSEC
                external phys, TBI phy address of eTSEC-1 port must
                differ from addresses of eTSEC external phys.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New TBI phy address

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigTbiPhysicalAddress(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigBdMemory

 @Description   Configures memory partition and attributes for buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE].
 
                Note that memory partition and attributes for data buffers
                are provided as part of t_EtsecParam structure.

 @Param[in]     h_Etsec         - Handle to the eTSEC object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBdMemory(t_Handle h_Etsec, uint8_t memId, uint32_t memAttributes);

/**************************************************************************//**
 @Function      ETSEC_ConfigInitTx

 @Description   This routine configures whether Tx is enabled or not.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable Tx init

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigInitTx(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigInitRx

 @Description   This routine configures whether Rx is enabled or not.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable Rx init

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigInitRx(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigLoopbackMode

 @Description   This routine configures loopback mode.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable loopback

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigLoopbackMode(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigFiler

 @Description   This routine configures the filer's operation.
                [FALSE].
 
                Filer distributes Rx frames according to rules
                defined by filer table - please refer to
                @ref etsec_filer_grp "eTSEC Filer API" documentation.
 
                When filer is disabled, no search of filer table
                is performed and all frames are filed to Rx queue 0.
 
 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable filer

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigFiler(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigFilerSingleQ

 @Description   This routine configures single queue filer operation.
                [FALSE].
 
                In single queue operation, the filer always attempts to file
                a frame to queue 0 regardless of actual queue index specified
                in the matching filer queue entry. This mode is intended for
                operating the filer as a packet classification engine.
 
                Single queue mode settings is ignored when filer operation
                is disabled either by ETSEC_ConfigFiler() or ETSEC_FilerDisable()
                routine.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable single queue mode filer operation

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigFilerSingleQ(t_Handle h_Etsec, bool enable);

#ifdef ETSEC_REV2
/**************************************************************************//**
 @Function      ETSEC_ConfigFilerQRoundRobin

 @Description   This routine configures queue mapping operation of filer.
                [FALSE].
 
                Rx queue selection on filer match event is driven either by
                - Queue index specified in the matching filer table entry, or
                - One of queue/ring mapping registers (RIRs) set by
                ETSEC_FILER_QueueMapping().
 
                In the latter case, this function specifies if queue mapping registers
                map Rx queue round robin counter or filer hash result to physical
                Rx queue.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE to enable queue round robin mapping by RIRs registers,
                          FALSE to hash result mapping by RIR registers.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigFilerQRoundRobin(t_Handle h_Etsec, bool enable);
#endif  /* ETSEC_REV2 */

/**************************************************************************//**
 @Function      ETSEC_ConfigParserControl

 @Description   This routine configures the parser layer.
                [e_ETSEC_PRSDEP_DIS].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Parsing level

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigParserControl(t_Handle h_Etsec, e_EtsecPrsdep newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigStatisticsEnable

 @Description   This routine enables statistics gathering for eTSEC.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE for enabling eTSEC statistics counters

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigStatisticsEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigAutoZeroCounters

 @Description   This routine configures MIB counters reset policy.
                [FALSE].
 
                When auto-zero counters is enabled, the addressed statistical
                counter is cleared after host read.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - TRUE for resetting counters after reading

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigAutoZeroCounters(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigWriteWithResponse

 @Description   This routine configures write with response.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/diable write with response

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigWriteWithResponse(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigWopTxBd

 @Description   This routine configures the buffer descriptor taking.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Wait (TRUE) or poll (FALSE) BDs

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigWopTxBd(t_Handle h_Etsec, bool newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigErrorDisabled

 @Description   This routine configures the eTSEC errors.
                [0].

                Errors disabled by this function won't trigger interrupt
                assertion and won't be reported in the BD.
                Errors not provided to this function as part of the mask
                parameter will stay enabled. This will allow setting
                the corresponding bit in IEVENT register and the
                corresponding field in the BD. However, in order to generate an
                actual interrupt for the enabled errors, the interrupt
                should be enabled in IMASK register via ETSEC_ConfigExceptions().
 
 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Bit mask for errors to disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().

 @See           @ref etsec_errors_mask "ETSEC Errors Bit Mask".
*//***************************************************************************/
t_Error ETSEC_ConfigErrorDisabled(t_Handle h_Etsec, uint32_t newVal);

#ifdef ETSEC_FIFO_INTERFACE_SUPPORT
/**************************************************************************//**
 @Function      ETSEC_ConfigFifoCrcApp

 @Description   This routine configures FIFO CRC appending.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable -  Enable/disable append CRC to fifo

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigFifoCrcApp(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigFifoCrcChk

 @Description   This routine configures FIFO CRC checking.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable -  Enable/disable CRC checking on FIFO

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigFifoCrcChk(t_Handle h_Etsec, bool enable);

#endif /* ETSEC_FIFO_INTERFACE_SUPPORT */

/**************************************************************************//**
 @Function      ETSEC_ConfigMaxFrameLength

 @Description   This routine configures the maximum frame length.
                [0x600].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Maximum frame length

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMaxFrameLength(t_Handle h_Etsec, uint16_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigBackoffEnable

 @Description   This routine enables alternate binary exponential backoff
                for half duplex operation.
                [FALSE].
 
                When alternate backoff setting is enabled, the value
                provided by ETSEC_ConfigBackoffVal() routine is used
                as number of retries on collision for backoff truncation.

                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable backoff

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBackoffEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigBackoffVal

 @Description   This routine configures number of retries in case of collision
                for backoff truncation.
                [0x0A].
 
                This value replaces standard value of 10 when alternate
                backoff is enabled by ETSEC_ConfigBackoffEnable().
 
                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Backoff value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBackoffVal(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigBackpressureNoBackoff

 @Description   This routine configures no backoff for back pressure.
                [FALSE].
 
                When backoff is disabled the Tx MAC immediately re-transmits,
                following a collision, during back pressure operation.
                When enabled, the Tx MAC follows the binary exponential back off
                algorithm.

                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Enable/disable back pressure no backoff

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBackpressureNoBackoff(t_Handle h_Etsec, bool newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigNoBackoff

 @Description   This routine configures no backoff.
                [FALSE].
 
                When backoff is disabled the Tx MAC immediately re-transmits
                following a collision. When enabled, the Tx MAC follows the
                binary exponential back off algorithm.

                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Enable/disable no backoff

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigNoBackoff(t_Handle h_Etsec, bool newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigExcessDefer

 @Description   This routine configures excess defer enable.
                [TRUE].
 
                When enabled, excessively deferred packet will be
                transmitted.

                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigExcessDefer(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigMaxRetransmission

 @Description   This routine configures number of retransmission attempts
                following a collision before aborting the packet due to
                excessive collisions.
                [0x0F].
 
                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMaxRetransmission(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigCollisionWindow

 @Description   This routine configures collision window.
                [0x37].
 
                Specifies number of bytes during which collisions
                occur in properly configured networks. Note that
                value provided here doesn't count for preamble
                and SFD, thus its default corresponds to standard
                64-byte value.

                Relevant only for half duplex operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal - collision window

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigCollisionWindow(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigHugeFramesEnable

 @Description   This routine enables huge frames.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable -  Enable/disable huge frames

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigHugeFramesEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigRxFlowEnable

 @Description   This routine enables Rx flow control.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable Rx flow control

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigRxFlowEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigTxFlowEnable

 @Description   This routine enables Tx flow control.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable Tx flow control

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigTxFlowEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigLosslessFlowControl

 @Description   This routine enables lossless flow control.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable lossless flow control

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigLosslessFlowControl(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigLosslessFlowControlThreshold

 @Description   This routine enables lossless flow control.
                [4].

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     threshold   - Free Rx BDs threshold; if number of free
                              Rx BDs drops below this threshold PAUSE
                              control frame is sent automatically by
                              the controller (only in full duplex mode)
 @Param[in]     queue       - Rx queue index for which the threshold is
                              provided

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigLosslessFlowControlThreshold(t_Handle    h_Etsec,
                                                 uint8_t     threshold,
                                                 uint8_t     queue);

/**************************************************************************//**
 @Function      ETSEC_ConfigLengthCheckEnable

 @Description   This routine enables Rx frame length checking.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable frames length check

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigLengthCheckEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigPadAndCrcEnable

 @Description   This routine enables padding and CRC on frames.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigPadAndCrcEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigCrcEnable

 @Description   This routine enables CRC on frames.
                [TRUE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigCrcEnable(t_Handle h_Etsec, bool enable);


#ifdef ETSEC_L2_CACHE_SUPPORT
/**************************************************************************//**
 @Function      ETSEC_ConfigDataWriteType

 @Description   This routine configures data write type.
                [e_ETSEC_ALLOC].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New data write type

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigDataWriteType(t_Handle h_Etsec, e_EtsecL2CacheWriteType newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigBdWriteType

 @Description   This routine configures BD write type.
                [e_ETSEC_ALLOC].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - New BD write type

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBdWriteType(t_Handle h_Etsec, e_EtsecL2CacheWriteType newVal);

#endif /* ETSEC_L2_CACHE_SUPPORT */

/**************************************************************************//**
 @Function      ETSEC_ConfigExtractIndex

 @Description   This routine configures frame index extraction.
                [14].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Frame index

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigExtractIndex(t_Handle h_Etsec, uint16_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigExtractLength

 @Description   This routine configures length to extract from frame.
                [4].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Number of bytes to extract

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigExtractLength(t_Handle h_Etsec, uint16_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigNonBackToBackIpg1

 @Description   This routine configures inter-packet gap part 1.
                [0x40].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Inter-packet gap part 1

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigNonBackToBackIpg1(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigNonBackToBackIpg2

 @Description   This routine configures inter-packet gap part 2.
                [0x60].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Inter-packet gap part 2

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigNonBackToBackIpg2(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigMinIfgEnforcement

 @Description   This routine configures min IFG.
                [0x50].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Min IFG

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMinIfgEnforcement(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigBackToBackIpg

 @Description   This routine configures back to back inter packet gap.
                [0x60].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Back-to-back inter-packet gap

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigBackToBackIpg(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigPreambleLength

 @Description   This routine configures frame preamble length.
                [0x7].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Preamble length in bytes

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigPreambleLength(t_Handle h_Etsec, uint8_t newVal);

/**************************************************************************//**
 @Function      ETSEC_ConfigLoopInTbi

 @Description   This routine configures loop in TBI.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Loop enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigLoopInTbi(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigMagicPacketEnable

 @Description   This routine configures magic packet detection.
                [FALSE].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enable  - Magic packet detection enable/disable

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigMagicPacketEnable(t_Handle h_Etsec, bool enable);

/**************************************************************************//**
 @Function      ETSEC_ConfigPacketAlignmentPadding

 @Description   This routine configures packet alignmnet padding length.
                [0].

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     newVal  - Packet padding length

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      This routine must be called before ETSEC_Init().
*//***************************************************************************/
t_Error ETSEC_ConfigPacketAlignmentPadding(t_Handle h_Etsec, uint8_t newVal);

/** @} */ /* end of etsec_advanced_init_grp group */
/** @} */ /* end of etsec_init_grp group */



/**************************************************************************//**
 @Group         etsec_runtime_grp eTSEC Runtime Unit

 @Description   eTSEC runtime unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   eTSEC Tx FCB Flags
 
                These flags can be passed to the transmit routines for
                performing special operations on the transmitted frame.
 
                The different types of flags can be combined to one value.
*//***************************************************************************/
typedef enum e_EtsecTxFcbFlags
{
    e_ETSEC_TX_FCB_FLAGS_VLN    = 0x80000000,   /**< VLAN control word is valid  */
    e_ETSEC_TX_FCB_FLAGS_IP     = 0x40000000,   /**< Assume L3 is IP */
    e_ETSEC_TX_FCB_FLAGS_IP6    = 0x20000000,   /**< IP header is IPv6 */
    e_ETSEC_TX_FCB_FLAGS_TUP    = 0x10000000,   /**< Assume L4 is TCP/UDP */
    e_ETSEC_TX_FCB_FLAGS_UDP    = 0x08000000,   /**< L4 is UDP */
    e_ETSEC_TX_FCB_FLAGS_CIP    = 0x04000000,   /**< Generate IPv4 checksum */
    e_ETSEC_TX_FCB_FLAGS_CTU    = 0x02000000,   /**< Generate TCP/UDP checksum */
    e_ETSEC_TX_FCB_FLAGS_NPH    = 0x01000000,   /**< Disable TCP or UDP pseudo header */
    e_ETSEC_TX_FCB_FLAGS_PTP    = 0x00010000    /**< PTP frame - generate a time stamp on transmission */
} e_EtsecTxFcbFlags;

/**************************************************************************//**
 @Description   eTSEC Tx BD Flags
 
                These flags can be passed to the transmit routines for
                performing special operations on the transmitted frame.
 
                The different types of flags can be combined to one value.
*//***************************************************************************/
typedef enum e_EtsecTxBdFlags
{
    e_ETSEC_TX_BD_FLAGS_PAD_CRC     = 0x4000,   /**< Frame is padded up to 64 bytes and
                                                     CRC is always appended */
    e_ETSEC_TX_BD_FLAGS_CRC         = 0x0400,   /**< CRC is appended after the last frame buffer */
    e_ETSEC_TX_BD_FLAGS_PREAMBLE    = 0x0200,   /**< Frame includes 8-byte preamble */
    e_ETSEC_TX_BD_FLAGS_HUGE_FRAME  = 0x0080,   /**< Allow large frames to be transmitted without truncation */
    e_ETSEC_TX_BD_FLAGS_CTRL_FRAME  = 0x0040,   /**< Pause frame is transmitted */
    e_ETSEC_TX_BD_FLAGS_FCB         = 0x0002    /**< Frame control block is expected before first frame buffer */
} e_EtsecTxBdFlags;

/**************************************************************************//**
 @Function      ETSEC_Rx

 @Description   Receive routine.
 
                This function handles data buffers received into the specified
                Rx queue and passes them to the user through f_RxStore callback.
                The function is invoked automatically by driver when operating
                in interrupt mode and must be issued by application when
                working in polling mode. In polling mode, this routine should
                be invoked often enough to keep BD ring from overflowing with
                received data.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     rxQ     - Receive queue index

 @Return        None
*//***************************************************************************/
t_Error ETSEC_Rx(t_Handle h_Etsec, uint8_t rxQ);

/**************************************************************************//**
 @Function      ETSEC_Tx

 @Description   This routine shall be called to initiate data transmission.
 
 @Param[in]     h_Etsec         - Handle to the eTSEC object.
 @Param[in]     txQ             - Transmit queue number
 @Param[in]     p_Data          - Pointer to data buffer
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length          - Pointer to data buffer length
 @Param[in]     lastBuffer      - Last buffer in frame indication
 @Param[in]     flags           - Transmit flags
 
 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error ETSEC_Tx(t_Handle   h_Etsec,
                 uint8_t    txQ,
                 uint8_t    *p_Data,
                 t_Handle   h_BufContext,
                 uint16_t   length,
                 bool       lastBuffer,
                 uint16_t   flags);

/**************************************************************************//**
 @Function      ETSEC_TxConf

 @Description   Transmit confirmation routine.
 
                Returns the transmitted frames to the application and clears the
                transmit BDs for new frames. The user's f_TxConf callback routine
                is called for each transmitted frame, returning the frame and the
                BD status to the application.
 
                If this routine is not called, each frame will be automatically
                returned and reported in the same manner, when the BD ring has
                wrapped around and this frame's BD's location is reached again.
 
 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     txQ     - Transmit queue number
 
 @Return        None.
 *//***************************************************************************/
t_Error ETSEC_TxConf(t_Handle h_Etsec, uint8_t txQ);


/**************************************************************************//**
 @Function      ETSEC_TxMacPause

 @Description   Initiates transmission of MAC control PAUSE frame for the
                specified duration.

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     pauseTime   - Pause time. The pause period is measured in
                              pause quanta, a speed independent constant of
                              512 bit-times
 @Param[in]     exPauseTime - Extended pause time.

 @Return        None
*//***************************************************************************/
void ETSEC_TxMacPause(t_Handle h_Etsec, uint16_t pauseTime, uint16_t exPauseTime);

/**************************************************************************//**
 @Function      ETSEC_TxHalfDupFlowCntrl

 @Description   Half duplex flow control - stops/resumes reception of packets

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     enabled - If TRUE, stops reception of packets.
                          If FALSE, resumes reception of packets.

 @Return        None
*//***************************************************************************/
void ETSEC_TxHalfDupFlowCntrl(t_Handle h_Etsec, bool enabled);

/** @} */ /* end of etsec_runtime_grp group */


/**************************************************************************//**
 @Group         etsec_exception_grp eTSEC Exception Unit

 @Description   eTSEC exception unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        etsec_exbm
 
 @Collection    eTSEC Exceptions Bit Mask
 
                Exceptions and events reported through the exceptions callback.
                They can be masked/unmasked using ETSEC_ConfigExceptions().
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define IMASK_BREN      0x80000000      /**< Babbling (too long) frame received */
#define IMASK_RXCEN     0x40000000      /**< Control frame received */
#define IMASK_BSYEN     0x20000000      /**< Rx busy */
#define IMASK_EBERREN   0x10000000      /**< Internal bus error during DMA transaction */
#define IMASK_MSROEN    0x04000000      /**< MIB counter overflow */
#define IMASK_BTEN      0x01000000      /**< Babbling (too long frame) Tx error */
#define IMASK_TXCEN     0x00800000      /**< Control frame transmitted */
#ifndef ETSEC_REV2
#define IMASK_TXEEN     0x00400000      /**< Tx error: one of IMASK_EBERREN, IMASK_LCEN,
                                             IMASK_CRLEN or IMASK_XFUNEN; use ETSEC_TxResumeAfterHalt()
                                             to restart the transmitter */
#endif /* ETSEC_REV2 */
#define IMASK_LCEN      0x00040000      /**< Late collision */
#define IMASK_CRLEN     0x00020000      /**< Collision retry limit exceeded */
#define IMASK_XFUNEN    0x00010000      /**< Transmitter underrun */
#ifdef ETSEC_REV2
#define IMASK_TWKEN     0x00001000      /**< Timer wakeup - frame received while in sleep mode */
#endif /* ETSEC_REV2 */
#define IMASK_MAGEN     0x00000800      /**< Magic packet detected */
#define IMASK_FIREN     0x00000008      /**< Filer result is invalid - no matching rule */
#define IMASK_FIQEN     0x00000004      /**< Filed frame to invalid/disabled Rx queue */
#define IMASK_DPEEN     0x00000002      /**< Data parity error */
#define IMASK_PERREN    0x00000001      /**< Rx frame parser error */
/* @} */

/**************************************************************************//**
 @Function      ETSEC_RxResumeAfterHalt

 @Description   Re-enables the Ethernet Controller receiver after it was halted.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_RxResumeAfterHalt(t_Handle h_Etsec);

/**************************************************************************//**
 @Function      ETSEC_TxResumeAfterHalt

 @Description   Resumes transmission after the Ethernet controller went into
                transmit halt state while operating in "wait" mode.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_TxResumeAfterHalt(t_Handle h_Etsec);

/**************************************************************************//*
 @Function      ETSEC_ErrorsAndEvents

 @Description   eTSEC errors interupt handler.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_ErrorsAndEvents(t_Handle h_Etsec);

/** @} */ /* end of etsec_exception_grp group */


/**************************************************************************//**
 @Group         etsec_control_grp eTSEC Control Unit

 @Description   eTSEC control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/*********************************************************************************//**
 @Description   eTSEC MIB registers
*//**********************************************************************************/
typedef struct t_EtsecStatistics
{
    uint32_t  tr64;   /**< transmit and receive 64 byte frame counter */
    uint32_t  tr127;  /**< transmit and receive 65 to 127 byte frame counter */
    uint32_t  tr255;  /**< transmit and receive 128 to 255 byte frame counter */
    uint32_t  tr511;  /**< transmit and receive 256 to 511 byte frame counter */
    uint32_t  tr1k;   /**< transmit and receive 512 to 1023 byte frame counter */
    uint32_t  trmax;  /**< transmit and receive 1024 to 1518 byte frame counter */
    uint32_t  trmgv;  /**< transmit and receive 1519 to 1522 byte good VLAN frame count */

    /* RECEIVE COUNTERS */

    uint32_t  rbyt;   /**< receive byte counter */
    uint32_t  rpkt;   /**< receive packet counter */
    uint32_t  rfcs;   /**< receive FCS error counter */
    uint32_t  rmca;   /**< RMCA receive multicast packet counter */
    uint32_t  rbca;   /**< receive broadcast packet counter */
    uint32_t  rxcf;   /**< receive control frame packet counter */
    uint32_t  rxpf;   /**< receive PAUSE frame packet counter */
    uint32_t  rxuo;   /**< receive unknown OP code counter */
    uint32_t  raln;   /**< receive alignment error counter */
    uint32_t  rflr;   /**< receive frame length error counter */
    uint32_t  rcde;   /**< receive code error counter */
    uint32_t  rcse;   /**< receive carrier sense error counter */
    uint32_t  rund;   /**< receive undersize packet counter */
    uint32_t  rovr;   /**< receive oversize packet counter */
    uint32_t  rfrg;   /**< receive fragments counter */
    uint32_t  rjbr;   /**< receive jabber counter */
    uint32_t  rdrp;   /**< receive drop */

    /* TRANSMIT COUNTERS */

    uint32_t  tbyt;   /**< transmit byte counter */
    uint32_t  tpkt;   /**< transmit packet counter */
    uint32_t  tmca;   /**< transmit multicast packet counter */
    uint32_t  tbca;   /**< transmit broadcast packet counter */
    uint32_t  txpf;   /**< transmit PAUSE control frame counter */
    uint32_t  tdfr;   /**< transmit deferral packet counter */
    uint32_t  tedf;   /**< transmit excessive deferral packet counter */
    uint32_t  tscl;   /**< transmit single collision packet counter */
    uint32_t  tmcl;   /**< transmit multiple collision packet counter */
    uint32_t  tlcl;   /**< transmit late collision packet counter */
    uint32_t  txcl;   /**< transmit excessive collision packet counter */
    uint32_t  tncl;   /**< transmit total collision counter */
    uint32_t  tpfh;   /**< transmit PAUSE frames honored counter */
    uint32_t  tdrp;   /**< transmit drop frame counter */
    uint32_t  tjbr;   /**< transmit jabber frame counter */
    uint32_t  tfcs;   /**< transmit FCS error counter */
    uint32_t  txcf;   /**< transmit control frame counter */
    uint32_t  tovr;   /**< transmit oversize frame counter */
    uint32_t  tund;   /**< transmit undersize frame counter */
    uint32_t  tfrg;   /**< transmit fragments frame counter */

    /* GENERAL REGISTERS */

    uint32_t  car1;   /**< carry register one register */
    uint32_t  car2;   /**< carry register two register */
    uint32_t  cam1;   /**< carry register one mask register */
    uint32_t  cam2;   /**< carry register two mask register */

} t_EtsecStatistics;

/**************************************************************************//**
 @Function      ETSEC_Enable

 @Description   This routine enables the Ethernet controller for reception
                (if rxEnable is TRUE), and for transmission (if txEnable
                 is TRUE).

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     mode    - Communication mode (Rx and Tx)

 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error ETSEC_Enable(t_Handle h_Etsec,  e_CommMode mode);

/**************************************************************************//**

 @Function      ETSEC_Disable

 @Description   This routine disables the Ethernet controller for reception
                and/or for transmission.

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     mode    - Communication mode (Rx and Tx).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_Disable(t_Handle h_Etsec,  e_CommMode mode);

/**************************************************************************//**
 @Function      ETSEC_GetStatistics

 @Description   Passes the network statistics gathered by the eTSEC.

 @Param[in]     h_Etsec             - Handle to the eTSEC object.
 @Param[out]    p_TsecStatistics    - Pointer to data structure containing
                                      all statistics counters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_GetStatistics(t_Handle h_Etsec, t_EtsecStatistics *p_TsecStatistics);

/**************************************************************************//**
 @Function      ETSEC_ResetStatistics

 @Description   Resets all statistics counters.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_ResetStatistics(t_Handle h_Etsec);

/**************************************************************************//**
 @Anchor        etsec_errors_mask
 
 @Collection    ETSEC Errors Bit Mask
 
                ETSEC errors that should be used with ETSEC_ErrEnable() and
                ETSEC_ErrDisable() functions
 @{
*//***************************************************************************/
#define ETSEC_ERRORS_BSY       0x20000000   /**< Busy condition interrupt - Frame received and discarded
                                                 because of lack of buffers  */
#define ETSEC_ERRORS_EBERR     0x10000000   /**< Internal bus error */
#define ETSEC_ERRORS_BABT      0x01000000   /**< Babbling transmit error - Frame was transmitted with
                                                 excessive frame length */
#ifndef ETSEC_REV2
#define ETSEC_ERRORS_TXE       0x00400000   /**< Transmit error - Error occured and caused TSTAT[THLT] */
#endif /* ETSEC_REV2 */
#define ETSEC_ERRORS_LC        0x00040000   /**< Late collision */
#define ETSEC_ERRORS_CRL       0x00020000   /**< Collision retry limit exceeded */
#define ETSEC_ERRORS_XFUN      0x00010000   /**< Trasmit underrun */
#define ETSEC_ERRORS_FIR       0x00000008   /**< Receive queue filer result is invalid */
#define ETSEC_ERRORS_FIQ       0x00000004   /**< Filed frame to invalid receive queue */
#define ETSEC_ERRORS_DPE       0x00000002   /**< Internal data parity error */
#define ETSEC_ERRORS_PERR      0x00000001   /**< Receive frame parsing error */
/* @} */ /* end of ETSEC Errors Bit Mask */


/**************************************************************************//**
 @Function      ETSEC_ActivateFiler

 @Description   Bind filer table to eTSEC and make it operational

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     h_Table - Handle to filer table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Etsec must have previously created with ETSEC_Config().
 @Cautions      h_Table must have previously created with ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_ActivateFiler(t_Handle h_Etsec, t_Handle h_Table);

/**************************************************************************//**
 @Function      ETSEC_FilerDisable

 @Description   Disable the filer operation.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_FilerDisable(t_Handle h_Etsec);

/**************************************************************************//**
 @Function      ETSEC_FilerEnable

 @Description   Enables the filer operation

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_FilerEnable(t_Handle h_Etsec);

/**************************************************************************//*
 @Function      ETSEC_RxInterrupt

 @Description   Rx interrupt treatment

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_RxInterrupt(t_Handle h_Etsec);

/**************************************************************************//**
 @Function      ETSEC_RemoveAddr

 @Description   Remove address (group or individual) from the Ethernet
                Controller addresses list (hash table). It calculates the
                address CRC and clears the corrsponding bit in the group/address
                individual register if this address is the only one for this
                bin.

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     p_EnetAddr  - Address to be removed.

 @Return        None
*//***************************************************************************/
void ETSEC_RemoveAddr(t_Handle h_Etsec, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      ETSEC_AddAddr

 @Description   Add address (group or individual) to the Ethernet Controller
                addresses list (hash table). It calculates the address CRC
                and sets the corrsponding bit in the group/individual
                address register.

 @Param[in]     h_Etsec       - Handle to the eTSEC object.
 @Param[in]     p_EnetAddr    - Address to be added.

 @Return        None
*//***************************************************************************/
void ETSEC_AddAddr(t_Handle h_Etsec, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      ETSEC_StationAddressAdd

 @Description   Add station address

 @Param[in]     h_Etsec - Handle to the eTSEC object.
 @Param[in]     p_EnetAddr  - New MAC address

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_StationAddressAdd(t_Handle h_Etsec, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      ETSEC_StationAddressClear

 @Description   Clear MAC address from acceptable addresses list

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     p_EnetAddr  - MAC address

 @Return        None
*//***************************************************************************/
t_Error ETSEC_StationAddressClear(t_Handle h_Etsec, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      ETSEC_StationAddressGetAll

 @Description   Retrieves all the controller's station addresses.

 @Param[in]     h_Etsec     - Handle to the eTSEC object.
 @Param[in]     p_EnetAddr  - Array of MAC addresses

 @Return        Number of found addresses
*//***************************************************************************/
uint8_t ETSEC_StationAddressGetAll(t_Handle h_Etsec, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      ETSEC_StationAddressClearAll

 @Description   Delete all additional station addresses

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        None
*//***************************************************************************/
void ETSEC_StationAddressClearAll(t_Handle h_Etsec);

/**************************************************************************//**
 @Function      ETSEC_AdjustLink

 @Description   Adjusts the Ethernet controller configuration with new
                phy speed/duplex setup.
 
                This function should be issued by application upon change
                in phy link synchronization speed and/or duplex mode.
                It adjust ETH MAC layer configuration according to the new
                phy link parameters.

 @Param[in]     h_Etsec     - Handle to the eTSEC object
 @Param[in]     speed       - Ethernet speed
 @Param[in]     fullDuplex  - Duplex mode: TRUE for full duplex,
                                           FALSE for half duplex

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_AdjustLink(t_Handle h_Etsec, e_EnetSpeed speed, bool fullDuplex);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      ETSEC_DumpRegs

 @Description   Dump eTSEC registers.

 @Param[in]     h_Etsec - Handle to the eTSEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_DumpRegs(t_Handle h_Etsec);
#endif  /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of etsec_control_grp group */
/** @} */ /* end of etsec_grp group */

#endif /* __ETSEC_EXT_H */
