/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          etsec_filer_ext.h

 @Description   eTSEC Filer Application Programming Interface
*//***************************************************************************/
#ifndef __ETSEC_FILER_EXT_H
#define __ETSEC_FILER_EXT_H

#include "std_ext.h"

/**************************************************************************//**
 @Group         etsec_grp eTSEC Application Programming Interface

 @Description   eTSEC API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         etsec_filer_grp eTSEC Filer Application Programming Interface

 @Description   eTSEC Filer API functions, definitions and enums

 @{
*//***************************************************************************/

#define ETSEC_FILER_MAX_ELEMENTS    255         /**< Maximal filer table size */


/**************************************************************************//**
 @Description   Filter specifier
 
                Defines protocol properties to be used by filer entry
*//***************************************************************************/
typedef enum e_EtsecFilerFltr
{
    e_ETSEC_FLTR_MASK,                          /**< Mask for the next entry's property,
                                                     allows to use only part of field bits */
    e_ETSEC_FLTR_BRCST_DST_ADDR,                /**< Filer entry will be matched if parser has
                                                     seen MAC broadcast destination address
                                                     in the frame */
    e_ETSEC_FLTR_VLN_TAG,                       /**< Filer entry will be matched if parser has
                                                     signed the frame as VLAN */
    e_ETSEC_FLTR_CANONICAL_FORMAT_INDICATOR,    /**< CFI part of VLAN tag in case of VLAN frame,
                                                     zero therwise */
    e_ETSEC_FLTR_JUMBO_FRAME,                   /**< Filer entry will be matched if parser has
                                                     signed this frame as jumbo frame */
    e_ETSEC_FLTR_IP_FRAGMENTATION,              /**< Filer entry will be matched if IPv4 or IPv6
                                                     fragment has been encountered */
    e_ETSEC_FLTR_FIFO_INTERFACE,                /**< Filer entry will be matched if packet
                                                     entered on eTSEC FIFO interface */
    e_ETSEC_FLTR_IPV4,                          /**< Filer entry will be matched if parser has
                                                     seen IPv4 header */
    e_ETSEC_FLTR_IPV6,                          /**< Filer entry will be matched if parser has
                                                     seen IPv6 header */
    e_ETSEC_FLTR_IPV4_CS,                       /**< Filer entry will be matched if IPv4 header
                                                     checksum has been checked */
    e_ETSEC_FLTR_IPV4_CS_CORRECT,               /**< Filer entry will be matched if IPv4 header
                                                     checksum has been found correct */
    e_ETSEC_FLTR_TCP,                           /**< Filer entry will be matched if parser has
                                                     seen TCP header */
    e_ETSEC_FLTR_UDP,                           /**< Filer entry will be matched if parser has
                                                     seen UDP header */
    e_ETSEC_FLTR_PARSE_ERR,                     /**< Filer entry will be matched on parser error,
                                                     such as header inconsistency */
    e_ETSEC_FLTR_ETH_FRAME_ERR,                 /**< Filer entry will be matched Ethernet framing
                                                     error preventing parsing */
    e_ETSEC_FLTR_ARB,                           /**< User-defined protocol fields configured by
                                                     ETSEC_FILER_ArbitraryParser() */
    e_ETSEC_FLTR_DEST_ADDR_H,                   /**< Destination MAC address, most significant
                                                     24 bits */
    e_ETSEC_FLTR_DEST_ADDR_L,                   /**< Destination MAC address, least significant
                                                     24 bits */
    e_ETSEC_FLTR_SRC_ADDR_H,                    /**< Source MAC address, most significant 24 bits */
    e_ETSEC_FLTR_SRC_ADDR_L,                    /**< Source MAC address, least significant 24 bits */
    e_ETSEC_FLTR_ETHERTYPE,                     /**< Ethertype of next layer protocol */
    e_ETSEC_FLTR_VLAN_ID,                       /**< VLAN ID field of VLAN tag */
    e_ETSEC_FLTR_VLAN_PRI,                      /**< VLAN priority field of VLAN tag */
    e_ETSEC_FLTR_IPV4_TOS,                      /**< IPv4 TOS or IPv6 traffic class field */
    e_ETSEC_FLTR_L4_PROTOCOL,                   /**< Layer 4 protocol field */
    e_ETSEC_FLTR_IP_DST_ADDR,                   /**< Destination IP address - full address for IPv4 or
                                                     32 most significant bits of IPv6 address */
    e_ETSEC_FLTR_IP_SRC_ADDR,                   /**< Source IP address - full address for IPv4 or
                                                     32 most significant bits of IPv6 address */
    e_ETSEC_FLTR_TCP_UDP_DST_PORT,              /**< Destination port number for TCP or UDP header */
    e_ETSEC_FLTR_TCP_UDP_SRC_PORT,              /**< Source port number for TCP or UDP header */
    e_ETSEC_FLTR_LAST_DUMMY                     /**< Last/dummy enum entry */
 
} e_EtsecFilerFltr;

/**************************************************************************//**
 @Description   Comparison operation specifier
 
                Specifies comparison operation to perform between extracted field
                and filer element property.
*//***************************************************************************/
typedef enum e_EtsecFilerComparisonType
{
    e_ETSEC_FILER_COMPR_EQ           = 0x0,     /**< =  */
    e_ETSEC_FILER_COMPR_BIGGER_OR_EQ = 0x1,     /**< >= */
    e_ETSEC_FILER_COMPR_NOT_EQ       = 0x2,     /**< != */
    e_ETSEC_FILER_COMPR_SMALLER      = 0x3      /**< <  */
} e_EtsecFilerComparisonType;

/**************************************************************************//**
 @Description   Action descriptor specifier
 
                Action to perform on termination rule when matched.
*//***************************************************************************/
typedef enum e_EtsecFilerActionType
{
    e_ETSEC_FILER_ACCEPT = 0x00,                /**< Accept frame */
    e_ETSEC_FILER_REJECT = 0x01                 /**< Reject frame */
} e_EtsecFilerActionType;

/**************************************************************************//**
 @Description   Type of filer rule
 
                Filer rule may be either terminating, when acceptance or rejection
                of frame will occur, or part of rules group. For the latter,
                clusters of rules or rules compound from several sub-rules
                should be used.
*//***************************************************************************/
typedef enum e_EtsecFilerEntryType
{
    e_ETSEC_FILER_ENTRY_CLUSTER_START,          /**< Start of cluster */
    e_ETSEC_FILER_ENTRY_CLUSTER_CLOSE,          /**< End of cluster */
    e_ETSEC_FILER_ENTRY_AND_NEXT,               /**< Conjunct to the next entry - continue to check
                                                     next entry */
    e_ETSEC_FILER_ENTRY_ACTION                  /**< Terminating rule - accept or reject */
} e_EtsecFilerEntryType;

/**************************************************************************//**
 @Description   Filer table entry element
*//***************************************************************************/
typedef struct t_EtsecFilerElement
{
    e_EtsecFilerFltr            type;               /**< Specific filter type */
    uint32_t                    key;                /**< Element property - concrete value to match */
    uint32_t                    keyMask;            /**< Mask to be applied to parser result prior to matching
                                                         this property */
    e_EtsecFilerComparisonType  cmp;                /**< Comparsion operation between parser result and this property */
    bool                        maskModificationEn; /**< Enable mask modification for entry */
    e_EtsecFilerEntryType       entryType;          /**< Relation to next element in rule */
#ifdef ETSEC_REV2
    bool                        hash;               /**< Include unmasked part of parser result in hash */
#endif /* ETSEC_REV2 */
} t_EtsecFilerElement;

#ifdef ETSEC_REV2
/**************************************************************************//**
 @Description   Filer queue selector
 
                Frame accepted by eTSEC filer may be forwarded to Rx queue
                specified as part of rule parameters (see t_EtsecFilerRuleParam
                structure) or filed based on one of the 4 queue mapping
                registers configured by ETSEC_FILER_QueueMapping() function.
 
                Note that frame distribution according to queue mapping
                register is valid only when round robin queue selection
                policy is configured by ETSEC_ConfigFilerQRoundRobin()
                routine or when hashing is enabled for at least one element
                of the current rule.
*//***************************************************************************/
typedef enum e_EtsecFilerQSelector
{
    e_ETSEC_FILER_Q_VALUE           = 0x0,      /**< Use queue value specified in filer table entry */
    e_ETSEC_FILER_Q_MAP_REGISTER1   = 0x1,      /**< Distribute based on the first queue mapping register */
    e_ETSEC_FILER_Q_MAP_REGISTER2   = 0x2,      /**< Distribute based on the second queue mapping register */
    e_ETSEC_FILER_Q_MAP_REGISTER3   = 0x3,      /**< Distribute based on the third queue mapping register */
    e_ETSEC_FILER_Q_MAP_REGISTER4   = 0x4       /**< Distribute based on the fourth queue mapping register */
} e_EtsecFilerQSelector;
#endif /* ETSEC_REV2 */

/**************************************************************************//**
 @Description   Parameters structure for filer rule
 
                Filer rule may be compound from single element (one termination
                rule), or consist from array of elements representing compound
                rule or cluster. In either case, last entry in elements array
                must be of action type.
*//***************************************************************************/
typedef struct t_EtsecFilerRuleParam
{
    uint16_t                numOfElements;  /**< Number of rule elements */
    t_EtsecFilerElement     *p_Elements;    /**< Array of elements with a single action */
    e_EtsecFilerActionType  action;         /**< Action for rule */
    uint8_t                 rxQ;            /**< Rx queue index for accepted frames */
#ifdef ETSEC_REV2
    e_EtsecFilerQSelector   queueSelector;  /**< Policy for Rx queue selection for accepted frames */
    bool                    interrupt;      /**< Specifies if interrupt should be raised
                                                 on frame acceptance; interrupt is asserted to
                                                 the same core filed frame's Rx queue belongs to */
#endif /* ETSEC_REV2 */
} t_EtsecFilerRuleParam;

/**************************************************************************//**
 @Description   Arbitrary byte offset control
*//***************************************************************************/
typedef enum e_EtsecFilerByteOffsetCtl
{
    e_ETSEC_FILER_BYTE_OFFSET_CTL_NO,           /**< Not extracted  */
    e_ETSEC_FILER_BYTE_OFFSET_CTL_FROM_DA,      /**< Offset starts from (first byte of ethernet DA - 8):
                                                     zero offset points at start of Ethernet preamble and
                                                     offset = 8 points at the first byte of MAC DA */
    e_ETSEC_FILER_BYTE_OFFSET_CTL_AFTER_L2,     /**< Offset starts after last byte of L2 */
    e_ETSEC_FILER_BYTE_OFFSET_CTL_AFTER_L3      /**< Offset starts after last byte of L3 */
} e_EtsecFilerByteOffsetCtl;

/**************************************************************************//**
 @Description   Arbitrary parser parameters
 
                User-defined protocol offsets for parser.
                Up to 4 offsets can be provided. Unused offsets fileds
                should be zeroed out.
*//***************************************************************************/
typedef struct t_EtsecFilerRbifxParam
{
    e_EtsecFilerByteOffsetCtl  byteCtl[4];      /**< Location of offset */
    uint8_t                    byteOffset[4];   /**< Offset value */
} t_EtsecFilerRbifxParam;


/**************************************************************************//**
 @Group         etsec_filer_init_grp eTSEC Filer Initialization Unit

 @Description   eTSEC Filer initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ETSEC_FILER_Config

 @Description   Create handle to a filer table

 @Param[in]     size            - The maximum number of rules in table.
 @Param[in]     defaultAction   - The default action of the table.
 @Param[in]     defaultQ        - The default queue of the table.
 
 @Return        Handle to filer table on success. NULL pointer otherwise.
*//***************************************************************************/
t_Handle ETSEC_FILER_Config(uint16_t               size,
                            e_EtsecFilerActionType defaultAction,
                            uint8_t                defaultQ);

/**************************************************************************//*
 @Function      ETSEC_FILER_Bind

 @Description   Bind a table to an eTSEC.

 @Param[in]     h_Table - Handle to filer table.
 @Param[in]     p_Addr  - Pointer to etsec memory map.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_Bind(t_Handle h_Table, uint32_t *p_Addr);

/**************************************************************************//**
 @Function      ETSEC_FILER_Free

 @Description   Releases filer table resources.

 @Param[in]     h_Table - Handle to the table to be freed.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
 @Cautions      h_Table must be disabled.
 *//***************************************************************************/
t_Error ETSEC_FILER_Free(t_Handle h_Table);

/**************************************************************************//**
 @Function      ETSEC_FILER_Duplicate

 @Description   Duplicate filer table.

 @Param[in]     h_Table - Handle to the table to be duplicated.
 
 @Return        Pointer to the new duplicated table on success. NULL otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
 *//***************************************************************************/
t_Handle ETSEC_FILER_Duplicate (t_Handle h_Table);

/** @} */ /* end of etsec_filer_init_grp group */

/**************************************************************************//**
 @Group         etsec_filer_control_grp eTSEC Filer Control Unit

 @Description   eTSEC Filer control unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ETSEC_FILER_InsertRule

 @Description   Insert a new rule to filer table

 @Param[in]     h_Table         - Handle to the table to perform insertion.
 @Param[in]     *p_RuleParam    - Parameters of the rule to be inserted.
 @Param[out]    p_RuleIndex     - Index of inserted entry; the value is undefined
                                  in case error code is returned by this function.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_InsertRule(t_Handle               h_Table,
                               t_EtsecFilerRuleParam  *p_RuleParam,
                               uint16_t               *p_RuleIndex);

/**************************************************************************//**
 @Function      ETSEC_FILER_ModifyRuleElement

 @Description   Modify filter of a single entry in filer table.

 @Param[in]     h_Table     - Handle to the table.
 @Param[in]     ruleIndex   - Index of rule modified element belongs to, as
                              returned by ETSEC_FILER_InsertRule() function.
 @Param[in]     p_NewFilter - Pointer to a user-supplied new filter for the element.
 @Param[in]     elementNum  - Index of the element in the rule.
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
 @Cautions      Rule must have been previously inserted using ETSEC_FILER_InsertRule().
 @Cautions      If no entry matching the one supplied to this call exists in the
                table, this call will fail.
*//***************************************************************************/
t_Error ETSEC_FILER_ModifyRuleElement(t_Handle             h_Table,
                                      uint16_t             ruleIndex,
                                      t_EtsecFilerElement  *p_NewFilter,
                                      uint16_t             elementNum);

/**************************************************************************//**
 @Function      ETSEC_FILER_ModifyRule

 @Description   Modify rule's attributes.

 @Param[in]     h_Table     - Handle to the table.
 @Param[in]     ruleIndex   - Index of rule in the table as returned by
                              ETSEC_FILER_InsertRule() function.
 @Param[in]     action      - Rule's action.
 @Param[in]     rxQ         - RxQ of accepted frames.
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
 @Cautions      Rule must have been previously inserted using ETSEC_FILER_InsertRule().
 @Cautions      If no entry matching the one supplied to this call exists in the
                table, this call will fail.
*//***************************************************************************/
t_Error ETSEC_FILER_ModifyRule(t_Handle                h_Table,
                               uint16_t                ruleIndex,
                               e_EtsecFilerActionType  action,
                               uint8_t                 rxQ);

/**************************************************************************//**
 @Function      ETSEC_FILER_DeleteRule

 @Description   Delete a rule from filer table.

 @Param[in]     h_Table     - Handle to the table.
 @Param[in]     ruleIndex   - Index of the rule to be deleted, as
                              returned by ETSEC_FILER_InsertRule() function.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
 @Cautions      Rule must have been previously inserted using ETSEC_FILER_InsertRule().
*//***************************************************************************/
t_Error ETSEC_FILER_DeleteRule(t_Handle h_Table, uint16_t ruleIndex);

/**************************************************************************//**
 @Function      ETSEC_FILER_UpdateDefaultRule

 @Description   Replace the default rule in the filer table.

 @Param[in]     h_Table - Handle to the table.
 @Param[in]     action  - Action for rule.
 @Param[in]     rxQ     - Rx Queue index for accepted frames.
 
 @Return        Handle to new default rule on success. NULL otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_UpdateDefaultRule(t_Handle                 h_Table,
                                      e_EtsecFilerActionType   action,
                                      uint8_t                  rxQ);

/**************************************************************************//**
 @Function      ETSEC_FILER_Defrag

 @Description   Defragment filer table.
 
                Table defragmentation is useful after series of add/delete rules
                that may leave "holes" (unused empty entries) in the filer table.
                Defragmentation routine takes care of this situation by re-arranging
                the table and eliminating the holes. This is to increase table
                utilization and to allow new rules future addition.

 @Param[in]     h_Table - Handle to the Table.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_Defrag(t_Handle h_Table);

/**************************************************************************//**
 @Function      ETSEC_FILER_ArbitraryParser

 @Description   Configure bit-fields for frame parsing.
 
                It is possible to define used-defined protocol fields in
                addition to standard fields supported by eTSEC parser.
                Up to 4 octets can be extracted from the frame and are
                concatenated to one 32-bit word. This word can be compared against
                filer table entry of ::e_ETSEC_FLTR_ARB type.

 @Param[in]     h_Table             - Handle to the Table.
 @Param[in]     p_ArbitraryParam    - Parameters to configure 4 bytes offsets
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_ArbitraryParser(t_Handle h_Table, t_EtsecFilerRbifxParam* p_ArbitraryParam);

#ifdef ETSEC_REV2
/**************************************************************************//**
 @Function      ETSEC_FILER_QueueMapping

 @Description   Configure one of 4 queue mapping registers.
 
                This function provides a mapping from queue round
                robin or hash value to actual Rx queue index
                a frame will be filed to. Up to 4 different mapping
                options can be used.
 
                The desired mapping option is then set in filer entry
                parameters through #e_EtsecFilerQSelector enumeration.
 
                Selection between round robin or hash logic is
                configured by ETSEC_ConfigFilerQRoundRobin()
                rountine.
 
                The mapping word parameter must be represented in the
                following format:
 
                @copydoc DOXYGEN_TABLE_QueueMapping
                @if DOXYGEN_IGNORE

                 0  2 3  5  .......               21 23 24      31
                --------------------------------------------------
                | Q0 | Q1 | .......              | Q7  | not used |

                @endif
 
                - The first 3 bits (0...2) specify Rx queue index for
                filing when round robin or lower 3 bits of hash result
                is 000.
                - The next 3 bits (3...5) specify Rx queue index for
                filing when round robin or lower 3 bits of hash result
                is 001, etc.
                - Finally, bits 21...23 specify Rx queue index for filing
                when round robin or lower 3 bits of hash result is 111.
                - The last 8 bits (24...31) are not used.
 
 @Param[in]     h_Table     - Handle to the filer table.
 @Param[in]     mapping     - Queue mapping in the format described above.
 @Param[in]     index       - Ring mapping register index, may vary
                              from 0 to 3.
 
 @Return        E_OK on success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using ETSEC_FILER_Config().
*//***************************************************************************/
t_Error ETSEC_FILER_QueueMapping(t_Handle h_Table, uint32_t mapping, uint8_t index);
#endif /* ETSEC_REV2 */

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      ETSEC_FILER_DumpTable

 @Description   Dump eTSEC filer table.

 @Param[in]     h_Table - Handle to the filer table.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error ETSEC_FILER_DumpTable(t_Handle h_Table);
#endif  /* (DEBUG_ERRORS > 0) */
 

/** @} */ /* end of etsec_filer_control_grp group */
/** @} */ /* end of etsec_filer_grp group */
/** @} */ /* end of etsec_grp group */




#endif /* __ETSEC_FILER_EXT_H */
