/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          etsec_rtc_ext.h

 @Description   External definitions and API for eTSEC RTC.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __ETSEC_RTC_EXT_H__
#define __ETSEC_RTC_EXT_H__



/**************************************************************************//**
 @Group         etsec_rtc_grp eTSEC RTC Application Programming Interface

 @Description   ETSEC RTC functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         etsec_rtc_init_grp eTSEC RTC Initialization Unit

 @Description   eTSEC RTC initialization API.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   eTSEC RTC Alarm ID.
*//***************************************************************************/
typedef enum e_EtsecRtcAlarmId
{
    e_ETSEC_RTC_ALARM_1 = 0,       /**< Alarm signal 1 */
    e_ETSEC_RTC_ALARM_2,           /**< Alarm signal 2 */
    e_ETSEC_RTC_ALARM_DUMMY_LAST
} e_EtsecRtcAlarmId;

#define ETSEC_RTC_NUM_OF_ALARMS    e_ETSEC_RTC_ALARM_DUMMY_LAST
                                /**< Total number of eTSEC RTC alarm signals */

/**************************************************************************//**
 @Description   eTSEC RTC Periodic Pulse ID.
*//***************************************************************************/
typedef enum e_EtsecRtcPulseId
{
    e_ETSEC_RTC_PULSE_1 = 0,       /**< Periodic pulse 1 */
    e_ETSEC_RTC_PULSE_2,           /**< Periodic pulse 2 */
    e_ETSEC_RTC_PULSE_3,           /**< Periodic pulse 3 */
    e_ETSEC_RTC_PULSE_DUMMY_LAST
} e_EtsecRtcPulseId;

#define ETSEC_RTC_NUM_OF_PULSES    e_ETSEC_RTC_PULSE_DUMMY_LAST
                                /**< Total number of ETSEC RTC periodic pulses */

/**************************************************************************//**
 @Description   eTSEC RTC External Trigger ID.
*//***************************************************************************/
typedef enum e_EtsecRtcTriggerId
{
    e_ETSEC_RTC_TRIGGER_1 = 0,     /**< External trigger 1 */
    e_ETSEC_RTC_TRIGGER_2,         /**< External trigger 2 */
    e_ETSEC_RTC_TRIGGER_DUMMY_LAST
} e_EtsecRtcTriggerId;

#define ETSEC_RTC_NUM_OF_TRIGGERS  e_ETSEC_RTC_TRIGGER_DUMMY_LAST
                                /**< Total number of eTSEC RTC external triggers */


/**************************************************************************//**
 @Description   eTSEC RTC Alarm Polarity Options.
*//***************************************************************************/
typedef enum e_EtsecRtcAlarmPolarity
{
    e_ETSEC_RTC_ALARM_POLARITY_ACTIVE_HIGH,    /**< Active-high output polarity */
    e_ETSEC_RTC_ALARM_POLARITY_ACTIVE_LOW      /**< Active-low output polarity */
} e_EtsecRtcAlarmPolarity;

/**************************************************************************//**
 @Description   eTSEC RTC Trigger Polarity Options.
*//***************************************************************************/
typedef enum e_EtsecRtcTriggerPolarity
{
    e_ETSEC_RTC_TRIGGER_ON_RISING_EDGE,    /**< Trigger on rising edge */
    e_ETSEC_RTC_TRIGGER_ON_FALLING_EDGE    /**< Trigger on falling edge */
} e_EtsecRtcTriggerPolarity;

/**************************************************************************//**
 @Description   eTsec RTC Periodic Pulse Start Mode.
*//***************************************************************************/
typedef enum e_EtsecRtcPulseStartMode
{
    e_ETSEC_RTC_PULSE_START_AUTO,      /**< Start pulse when RTC is enabled */
    e_ETSEC_RTC_PULSE_START_ON_ALARM   /**< Start pulse on alarm 1 event */
} e_EtsecRtcPulseStartMode;

/**************************************************************************//**
 @Description   eTsec RTC Optional Clock Sources.
*//***************************************************************************/
typedef enum e_EtsecSrcClock
{
    e_ETSEC_RTC_SOURCE_CLOCK_EXTERNAL,  /**< external high precision timer reference clock */
    e_ETSEC_RTC_SOURCE_CLOCK_SYSTEM,    /**< system clock */
    e_ETSEC_RTC_SOURCE_CLOCK_TRANSMIT,  /**< transmit clock */
    e_ETSEC_RTC_SOURCE_CLOCK_OSCILATOR, /**< RTC clock oscilator */
    e_ETSEC_RTC_SOURCE_CLOCK_DUMMY_LAST /**< RTC clock dummy */
}e_EtsecSrcClock;


/**************************************************************************//**
 @Description   eTSEC RTC events callback routine type.
*//***************************************************************************/
typedef void (t_EtsecRtcEventsCallBack)(t_Handle h_App, uint32_t events, uint32_t flags);


/**************************************************************************//**
 @Description   eTSEC RTC configuration parameters structure.
 
                This structure should be passed to ETSEC_RTC_Config().
*//***************************************************************************/
typedef struct t_EtsecRtcParam
{
    /* eTSEC RTC basic parameters */
    uint32_t                    rtcFreqHz;      /**< RTC target frequency in Hz */
 
    /* System integration parameters */
    uint32_t                    baseAddress;    /**< Base address of eTSEC registers */
    uint32_t                    srcClockFreqHz; /**< RTC source clock in Hz */
    int                         irq;            /**< eTSEC RTC interrupt ID (system-specific number) */
 
    /* Application interface */
    t_Handle                    h_App;          /**< Handle to application object */
    t_EtsecRtcEventsCallBack    *f_Events;      /**< RTC events callback routine */
} t_EtsecRtcParam;


/**************************************************************************//**
 @Function      ETSEC_RTC_Config

 @Description   Configures the eTSEC RTC module according to user's parameters.
 
                The driver assigns default values to some eTSEC RTC parameters.
                These parameters can be overwritten using the advanced
                configuration routines.

 @Param[in]     p_EtsecRtcParam - eTSEC RTC configuration parameters.
 
 @Return        Handle to the new eTSEC RTC object; NULL pointer on failure.
*//***************************************************************************/
t_Handle ETSEC_RTC_Config(t_EtsecRtcParam *p_EtsecRtcParam);

/**************************************************************************//**
 @Function      ETSEC_RTC_Init

 @Description   Initializes the eTSEC RTC driver and hardware.

 @Param[in]     h_EtsecRtc - Handle to eTSEC RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_Init(t_Handle h_EtsecRtc);

/**************************************************************************//**
 @Function      ETSEC_RTC_Free

 @Description   Frees the eTSEC RTC object and all allocated resources.

 @Param[in]     h_EtsecRtc - Handle to eTSEC RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_Free(t_Handle h_EtsecRtc);


/**************************************************************************//**
 @Group         etsec_rtc_adv_config_grp  eTSEC RTC Advanced Configuration Unit

 @Description   eTSEC RTC advanced configuration functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigSourceClock

 @Description   Configures the source clock of the RTC.
 
 @Param[in]     h_EtsecRtc      - Handle to eTSEC RTC object.
 @Param[in]     srcClock        - Source clock selection.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigSourceClock(t_Handle         h_EtsecRtc,
                                    e_EtsecSrcClock  srcClock);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigFrequencyCompensationBypass

 @Description   Configures the RTC to bypass the frequency compensation
                mechanism. [FALSE]
 
                In this mode, the RTC clock is identical to the source clock.
                This feature can be useful when the system contains an external
                RTC with inherent frequency compensation.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     enabled     - TRUE to bypass frequency compensation;
                              FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigFrequencyCompensationBypass(t_Handle h_EtsecRtc, bool enabled);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigInvertedInputClockPhase

 @Description   Configures the RTC to invert the source clock phase on input.
                [FALSE]
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     inverted    - TRUE to invert the source clock phase on input.
                              FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigInvertedInputClockPhase(t_Handle h_EtsecRtc, bool inverted);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigInvertedOutputClockPhase

 @Description   Configures the RTC to invert the output clock phase.
                [FALSE]

 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     inverted    - TRUE to invert the output clock phase.
                              FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigInvertedOutputClockPhase(t_Handle h_EtsecRtc, bool inverted);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigOutputClockDivisor

 @Description   Configures the divisor for generating the output clock from
                the RTC clock. [2]

 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     divisor     - Divisor for generation of the output clock.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigOutputClockDivisor(t_Handle h_EtsecRtc, uint16_t divisor);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigAlarmPolarity

 @Description   Configures the polarity (active-high/active-low) of a specific
                alarm signal. [e_ETSEC_RTC_ALARM_POLARITY_ACTIVE_HIGH]

 @Param[in]     h_EtsecRtc      - Handle to eTSEC RTC object.
 @Param[in]     alarmId         - Alarm ID.
 @Param[in]     alarmPolarity   - Alarm polarity.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigAlarmPolarity(t_Handle             h_EtsecRtc,
                                   e_EtsecRtcAlarmId       alarmId,
                                   e_EtsecRtcAlarmPolarity alarmPolarity);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigExternalTriggerPolarity

 @Description   Configures the polarity (rising/falling edge) of a specific
                external trigger signal. [e_ETSEC_RTC_TRIGGER_ON_RISING_EDGE]

 @Param[in]     h_EtsecRtc      - Handle to eTSEC RTC object.
 @Param[in]     triggerId       - Trigger ID.
 @Param[in]     triggerPolarity - Trigger polarity.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigExternalTriggerPolarity(t_Handle            h_EtsecRtc,
                                             e_EtsecRtcTriggerId       triggerId,
                                             e_EtsecRtcTriggerPolarity triggerPolarity);

/**************************************************************************//**
 @Function      ETSEC_RTC_ConfigPeriodicPulseStartMode

 @Description   Configures the method for triggering all periodic pulses.
                [e_ETSEC_RTC_PULSE_START_AUTO]
 
                The user can select to start the pulses when alarm #1 event
                is set, or immediately when the RTC is enabled.
                The first method is useful for having the periodic pulses
                phase-aligned to the RTC output clock. In this mode, there are
                some hardware constraints on the values of alarm #1 and the
                pulse periods. The driver may slightly modify the users values
                to match the hardware constraints.

 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     startMode   - Pulses start mode.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously created using ETSEC_RTC_Config().
*//***************************************************************************/
t_Error ETSEC_RTC_ConfigPeriodicPulseStartMode(t_Handle                h_EtsecRtc,
                                            e_EtsecRtcPulseStartMode   startMode);

/** @} */ /* end of etsec_rtc_adv_config_grp */
/** @} */ /* end of etsec_rtc_init_grp */


/**************************************************************************//**
 @Group         etsec_rtc_control_grp eTSEC RTC Control Unit

 @Description   eTSEC RTC runtime control API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      eTSEC_RTC_Enable

 @Description   Enables the RTC (time count is started).
 
                The user can select to resume the time count from previous
                point, or to restart the time count.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     resetClock  - Restart the time count from zero.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_Enable(t_Handle h_EtsecRtc, bool resetClock);

/**************************************************************************//**
 @Function      ETSEC_RTC_Disable

 @Description   Disables the RTC (time count is stopped).
 
 @Param[in]     h_EtsecRtc - Handle to eTSEC RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_Disable(t_Handle h_EtsecRtc);

/**************************************************************************//**
 @Function      ETSEC_RTC_ModifyFrequencyCompensation

 @Description   Modifies the frequency compensation value in runtime.
 
 @Param[in]     h_EtsecRtc          - Handle to eTSEC RTC object.
 @Param[in]     freqCompensation    - New frequency compensation value.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_ModifyFrequencyCompensation(t_Handle h_EtsecRtc,
                                           uint32_t freqCompensation);

/**************************************************************************//**
 @Function      ETSEC_RTC_ModifyEventsMask

 @Description   Modifies the RTC events mask in runtime.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     eventsMask  - New events mask. This value should contain
                              any combination of the ETSEC_RTC_EVENT_xxx flags.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().

 @See           @ref etsec_rtc_events "ETSEC RTC Events".
*//***************************************************************************/
t_Error ETSEC_RTC_ModifyEventsMask(t_Handle h_EtsecRtc, uint32_t eventsMask);

/**************************************************************************//**
 @Function      ETSEC_RTC_SetClockOffset

 @Description   Sets the clock offset (usually relative to another clock).
 
                The user can pass a negative offset value.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     offset      - New clock offset (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_SetClockOffset(t_Handle h_EtsecRtc, int64_t offset);

/**************************************************************************//**
 @Function      ETSEC_RTC_SetAlarm

 @Description   Schedules an alarm event to a given RTC time.
 
 @Param[in]     h_EtsecRtc  - Handle to ETSEC RTC object.
 @Param[in]     alarmId     - Alarm ID.
 @Param[in]     alarmTime   - Requested alarm time (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_SetAlarm(t_Handle        h_EtsecRtc,
                           e_EtsecRtcAlarmId  alarmId,
                           uint64_t        alarmTime);

/**************************************************************************//**
 @Function      ETSEC_RTC_SetPeriodicPulse

 @Description   Sets a periodic pulse.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     pulseId     - Periodic pulse ID.
 @Param[in]     pulsePeriod - Requested pulse period (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_SetPeriodicPulse(t_Handle        h_EtsecRtc,
                                   e_EtsecRtcPulseId  pulseId,
                                   uint32_t        pulsePeriod);

/**************************************************************************//**
 @Function      ETSEC_RTC_GetExternalTriggerTimeStamp

 @Description   Gets the last timestamp captured due to an external trigger.
 
 @Param[in]     h_EtsecRtc  - Handle to eTSEC RTC object.
 @Param[in]     triggerId   - Trigger ID.
 @Param[out]    p_TimeStamp - The requested timestamp (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
t_Error ETSEC_RTC_GetExternalTriggerTimeStamp(t_Handle         h_EtsecRtc,
                                              e_EtsecRtcTriggerId triggerId,
                                              uint64_t         *p_TimeStamp);

/**************************************************************************//**
 @Function      ETSEC_RTC_GetCurrentTime

 @Description   Returns the current RTC time.
 
 @Param[in]     h_EtsecRtc - Handle to eTSEC RTC object.

 @Return        The current RTC time (in nanoseconds).
 
 @Cautions      h_EtsecRtc must have been previously initialized using ETSEC_RTC_Init().
*//***************************************************************************/
uint64_t ETSEC_RTC_GetCurrentTime(t_Handle h_EtsecRtc);



/** @} */ /* end of etsec_rtc_control_grp */


/**************************************************************************//**
 @Group         etsec_rtc_exceptions_grp eTSEC RTC Exceptions Unit

 @Description   eTSEC RTC exceptions API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        etsec_rtc_events
 
 @Collection    eTSEC RTC Events
 
                Exceptions and events reported through the eTSEC RTC events
                callback routine. These events can be masked/unmasked using
                the ETSEC_RTC_ModifyEventsMask() routine.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define ETSEC_RTC_EVENT_EXT_TRIGGER_2_TS  0x02000000  /**< External trigger 2 time stamp ready;
                                                         Call ETSEC_RTC_GetExternalTriggerTimeStamp()
                                                         to get the time stamp */
#define ETSEC_RTC_EVENT_EXT_TRIGGER_1_TS  0x01000000  /**< External trigger 1 time stamp ready;
                                                         Call ETSEC_RTC_GetExternalTriggerTimeStamp()
                                                         to get the time stamp */
#define ETSEC_RTC_EVENT_ALARM_2           0x00020000  /**< Alarm 2 time has been reached */
#define ETSEC_RTC_EVENT_ALARM_1           0x00010000  /**< Alarm 1 time has been reached */
#define ETSEC_RTC_EVENT_PERIODIC_PULSE_1  0x00000080  /**< Periodic pulse 1 generated */
#define ETSEC_RTC_EVENT_PERIODIC_PULSE_2  0x00000040  /**< Periodic pulse 2 generated */
#define ETSEC_RTC_EVENT_PERIODIC_PULSE_3  0x00000020  /**< Periodic pulse 3 generated */

#define ETSEC_RTC_EVENT_ALL \
        (ETSEC_RTC_EVENT_EXT_TRIGGER_2_TS | ETSEC_RTC_EVENT_EXT_TRIGGER_1_TS | \
         ETSEC_RTC_EVENT_ALARM_2 | ETSEC_RTC_EVENT_ALARM_1 | ETSEC_RTC_EVENT_PERIODIC_PULSE_1 | \
         ETSEC_RTC_EVENT_PERIODIC_PULSE_2 | ETSEC_RTC_EVENT_PERIODIC_PULSE_3)
                                                    /**< All events */

/* @} */

/** @} */ /* end of etsec_rtc_exceptions_grp */
/** @} */ /* end of etsec_rtc_grp */


#endif /* __ETSEC_RTC_EXT_H__ */
