/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          gtimers_ext.h

 @Description   External definitions and API for the General Timers module.

 @Cautions      None.
*//***************************************************************************/
#ifndef __GTIMERS_EXT_H
#define __GTIMERS_EXT_H


/**************************************************************************//**
 @Group         gtimers_grp General Timers Application Programming Interface

 @Description   General Timers functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         gtimers_init_grp General Timers Initialization Unit

 @Description   General Timers init functions, definitions and enums.

 @{
*//***************************************************************************/

#define GTIMERS_NUM_OF_TIMERS   4   /**< Total number of timers in General Timers module */

/**************************************************************************//**
 @Description   Input clock
*//***************************************************************************/
typedef enum e_GtimerInClock
{
    e_GTIMER_SYS_CLK       = 1,  /**< system bus clock                  */
    e_GTIMER_SYS_DIV16_CLK = 2,  /**< slow go clock - system clock / 16 */
    e_GTIMER_TIN_CLK       = 3   /**< TIN                               */
} e_GtimerInClock;

/**************************************************************************//**
 @Description   timer number
*//***************************************************************************/
typedef enum e_GtimerNum
{
    e_GTIMERS_TMR1          = 0, /**< Timer 1                */
    e_GTIMERS_TMR2          = 1, /**< Timer 2                */
    e_GTIMERS_TMR3          = 2, /**< Timer 3                */
    e_GTIMERS_TMR4          = 3, /**< Timer 4                */
    e_GTIMERS_ANY_TMR       = 4, /**< Any timer -used when asking for a free timer from the driver */
    e_GTIMERS_NOT_AVAILABLE = 5  /**< Not valid timer        */
} e_GtimerNum;

/**************************************************************************//**
 @Description   gate mode
*//***************************************************************************/
typedef enum e_GtimerGateMode
{
    e_GTIMERS_GATE_RESTART = 0,  /**< Restart gate mode       */
    e_GTIMERS_GATE_NORMAL  = 1,  /**< Normal gate mode        */
    e_GTIMERS_GATE_DISABLE = 2   /**< TGAEn signal is ignored */
} e_GtimerGateMode;

/**************************************************************************//**
 @Description   output mode
*//***************************************************************************/
typedef enum e_GtimerOutputMode
{
    e_GTIMERS_OUTPUT_TOGGLE = 0,  /**< Output - toggle the signal */
    e_GTIMERS_OUTPUT_LOW    = 1   /**< Output - active low pulse  */
} e_GtimerOutputMode;

/**************************************************************************//**
 @Description   cascade  mode, 16/32/64 bit timer.
*//***************************************************************************/
typedef enum e_GtimerCascadeMode
{
    e_GTIMER_16_BIT,    /**< 16 bit timer */
    e_GTIMER_32_BIT,    /**< 32 bit timer */
    e_GTIMER_64_BIT     /**< 64 bit timer */
} e_GtimerCascadeMode;

/**************************************************************************//**
 @Description   capture edge
*//***************************************************************************/
typedef enum e_GtimerCaptureEdge
{
    e_GTIMER_CAPTURE_DISABLE      = 0, /**< Disable interrupt on capture event  */
    e_GTIMER_CAPTURE_RISING_EDGE  = 1, /**< Capture on rising TIN signal edge   */
    e_GTIMER_CAPTURE_FALLING_EDGE = 2, /**< Capture on falling TIN signal edge  */
    e_GTIMER_CAPTURE_ANY_EDGE     = 3  /**< Capture on any change of TIN signal */
} e_GtimerCaptureEdge;


/**************************************************************************//**
 @Description   Reference callback routine, called each time reference value was reached.
*//***************************************************************************/
typedef void (t_GtimersRefCallback)(t_Handle h_App, e_GtimerNum timerNum);

/**************************************************************************//**
 @Description   Capture callback routine, called each time capture event occurred.
*//***************************************************************************/
typedef void (t_GtimersCapCallback)(t_Handle h_App, e_GtimerNum timerNum);

/**************************************************************************//**
 @Description   Configuration parameters for the General Timers module.
*//***************************************************************************/
typedef struct t_GtimersParam
{
    /* General Timers basic parameters */
    e_GtimerCascadeMode cascadeMode;        /**< Cascade mode - configures how many hardware timers
                                                 shall be cascaded to create a software timer.
                                                 Valid options are:
                                                 - 4 timers of 16 bits (timers are not cascaded)
                                                 - 2 timers of 32 bits (two pairs of cascaded timers)
                                                 - 1 timer of 64 bits (all timers are cascaded)

    /* System integration parameters */
    uint32_t            baseAddress;        /**< Base address of General Timers registers. */
    uint32_t            sourceClockFreq;    /**< Specifies the source clock frequency to
                                                 the GTM module - should be greater than zero.
                                                 If the frequency is given in MHz, the reference
                                                 value will be given in microseconds.
                                                 If the frequency is given in KHz, the reference
                                                 value will be given in milliseconds. */
    int                 timersIrq[GTIMERS_NUM_OF_TIMERS];
                                            /**< Array of timer interrupt IDs (system-specifc number) */
} t_GtimersParam;


/**************************************************************************//**
 @Function      GTIMERS_Config

 @Description   Initialize the General Timers configuration, using the default values
                and the user parameters structure.

 @Param[in]     p_GtimersParam - Pointer to configuration parameters structure.

 @Return        Handle to the General Timers or NULL if configuration failed.
*//***************************************************************************/
t_Handle GTIMERS_Config(t_GtimersParam *p_GtimersParam);

/**************************************************************************//**
 @Function      GTIMERS_ConfigGateMode

 @Description   Set the gate mode for s specific timer.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Param[in]     timerNum - The number of the timer we refer to.
                           Allowed values are :
                                                - e_GTIMERS_TMR1
                                                - e_GTIMERS_TMR2
                                                - e_GTIMERS_TMR3
                                                - e_GTIMERS_TMR4

 @Param[in]     gateMode - The available modes are:
                                                - e_GTIMERS_GATE_DISABLE - TGATEn signal is ignored
                                                - e_GTIMERS_GATE_RESTART - restart gate mode
                                                - e_GTIMERS_GATE_NORMAL  - normal gate mode

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ConfigGateMode(t_Handle h_Gtimers, e_GtimerNum timerNum, e_GtimerGateMode gateMode);

/**************************************************************************//**
 @Function      GTIMERS_ConfigCaptureEdge

 @Description   Set the capture edge for a specific timer.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Param[in]     timerNum - The number of the timer we refer to.
                           Allowed values are:
                                             - e_GTIMERS_TMR1
                                             - e_GTIMERS_TMR2
                                             - e_GTIMERS_TMR3
                                             - e_GTIMERS_TMR4

 @Param[in]     captureEdge - The available options to configure the capture edge are:
                                             - e_GTIMER_CAPTURE_DISABLE      - disable interrupt on capture event
                                             - e_GTIMER_CAPTURE_RISING_EDGE  - capture on rising TINn signal
                                             - e_GTIMER_CAPTURE_FALLING_EDGE - capture on falling TINn signal
                                             - e_GTIMER_CAPTURE_ANY_EDGE     - capture on every change of TINn signal

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ConfigCaptureEdge(t_Handle h_Gtimers,e_GtimerNum timerNum,e_GtimerCaptureEdge captureEdge);

/**************************************************************************//**
 @Function      GTIMERS_ConfigInClock

 @Description   Set the clock source to the timer.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Param[in]     timerNum - The number of the timer we refer to.
                           Allowed values are:
                                             - e_GTIMERS_TMR1
                                             - e_GTIMERS_TMR2
                                             - e_GTIMERS_TMR3
                                             - e_GTIMERS_TMR4

 @Param[in]     inClock - The available options are:
                                            - e_GTIMER_SYS_CLK         - system bus clock
                                            - e_GTIMER_SYS_DIV16_CLK   - slow go clock- the system bus clock divided by 16
                                            - e_GTIMER_TIN_CLK         - from outside source on the TIN pin

 @Param[in]     clkFreq - Clock frequency in MHZ, or the value of the prescaler.

 @Return        E_OK if successful or error type if failed.

 @Cautions      The frequency that is configured should match the value of the option chosen in the inClock parameter
*//***************************************************************************/
t_Error GTIMERS_ConfigInClock(t_Handle h_Gtimers,e_GtimerNum timerNum,e_GtimerInClock inClock,uint32_t clkFreq);

/**************************************************************************//**
 @Function      GTIMERS_ConfigOutputMode

 @Description   Set the output mode of the timer.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Param[in]     timerNum - The number of the timer we refer to
                           Allowed values are:
                                             - e_GTIMERS_TMR1
                                             - e_GTIMERS_TMR2
                                             - e_GTIMERS_TMR3
                                             - e_GTIMERS_TMR4

 @Param[in]      outputMode - Two available values:
                                            - e_GTIMERS_OUTPUT_TOGGLE - toggle the signal
                                            - e_GTIMERS_OUTPUT_LOW    - active low pulse

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ConfigOutputMode(t_Handle h_Gtimers,e_GtimerNum timerNum,e_GtimerOutputMode outputMode);

/**************************************************************************//**
 @Function      GTIMERS_ConfigMaxFrequencyThreshold

 @Description   Set the maximum threshold for frequency prescalers calculations.
 
                When calculating prescalers for a given frequency, a certain threshold
                for calculation accuracy is allowed.
                [3]
 
 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     newVal    - New threshold value.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ConfigMaxFrequencyThreshold (t_Handle h_Gtimers, uint8_t newVal);

/**************************************************************************//**
 @Function      GTIMERS_Init

 @Description   Initialize the General Timers.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_Init(t_Handle h_Gtimers);

/**************************************************************************//**
 @Function      GTIMERS_Free

 @Description   Free driver resources.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_Free(t_Handle h_Gtimers);

/** @} */ /* end of gtimers_init_grp */

/*************************************************************************//**
 @Group         gtimers_run_grp General Timers Control Unit

 @Description   General Timers control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      GTIMERS_AllocateTimer

 @Description   Get a free timer from the driver.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum  - The number of a specific timer wanted,or any free timer available.
                            Allowed values are:
                                             - e_GTIMERS_TMR1
                                             - e_GTIMERS_TMR2
                                             - e_GTIMERS_TMR3
                                             - e_GTIMERS_TMR4
                                             - e_GTIMERS_ANY_TMR

 @Return        The number of the timer allocated for you or if there was no free timer e_GTIMERS_NOT_AVAILABLE.

 @Cautions      When all timers are cascaded to one 64 bit timer only e_GTIMERS_TMR1 can be requested.
                When all timers are cascaded to two 32 bit timers only e_GTIMERS_TMR1 or
                e_GTIMERS_TMR2 can be requested.
                When using 16 bit timers all 4 timers can be requested.
                e_GTIMERS_ANY_TMR can be requested in any bit number.
*//***************************************************************************/
e_GtimerNum GTIMERS_AllocateTimer(t_Handle h_Gtimers,e_GtimerNum timerNum);

/**************************************************************************//**
 @Function      GTIMERS_ReleaseTimer

 @Description   Release a specific timer.

 @Param[in]     h_Gtimers  - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum   - The number of the timer we want to free.
                             Allowed values are:
                                             - e_GTIMERS_TMR1
                                             - e_GTIMERS_TMR2
                                             - e_GTIMERS_TMR3
                                             - e_GTIMERS_TMR4
                                             - e_GTIMERS_ANY_TMR

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ReleaseTimer(t_Handle h_Gtimers,e_GtimerNum timerNum);

/**************************************************************************//**
 @Function      GTIMERS_SetCaptureCallback

 @Description   Set the capture callback.
 
                The timer we want to use for capture functionality must be configured to generate
                an interrupt on a capture event (GTIMERS_ConfigCaptureEdge())
                during the configuration phase to be able to use the callback function.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum  - The number of the timer
 @Param[in]     f_CapCallback - Callback function in case capture event occurred.

 @Return        E_OK if successful or error type if failed.

 @Cautions      This routine should be called after allocating the timer (GTIMERS_AllocateTimer())
                and before starting it (GTIMERS_StartTimer()).
                Because the capture functionality works with a specific timer, GTIMERS_AllocateTimer()
                should be called with the specific timer.
*//***************************************************************************/
t_Error GTIMERS_SetCaptureCallback(t_Handle h_Gtimers, e_GtimerNum timerNum,t_GtimersCapCallback *f_CapCallback);

/**************************************************************************//**
 @Function      GTIMERS_StartTimer

 @Description   Starting the timer.
                Starts the run of the timer after its allocation for the first time
                or re-startes a timer that was stoped.

 @Param[in]     h_Gtimers     - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum      - The number of the timer
 @Param[in]     f_RefCallback - Callback function in case the counter of the timer reached the reference value.
 @Param[in]     h_App         - Application id, configured by the user, giving the ability to identify the timer source.
 @Param[in]     referenceVal  - 64-bit reference value. If using a value smaller then 64-bit timer (HW register size),
                                the value of the reference should be in the LSB bits , followed by zeros at
                                the MSB bits. If the user chose to use more bits, then the HW timer MSB bits
                                will be implemented in software (referred here as SW timer).

 @Param[in]     runningMode - 3 modes available for running:
                                    - FREE_RUN - counter continues to increase after reaching the ref val;
                                    - PERIODIC - counter restarts counting from 0 after reaching the ref val;
                                    - SINGLE   - counter stops counting after reaching the ref val.

 @Return        E_OK if successful or error type if failed.

 @Cautions      A timer must be first asked using GTIMERS_AllocateTimer() before starting it.
                To continue a stopped timer operation GTIMERS_ResumeTimer() should be used.
*//***************************************************************************/
t_Error GTIMERS_StartTimer(t_Handle             h_Gtimers,
                           e_GtimerNum          timerNum,
                           t_GtimersRefCallback *f_RefCallback,
                           t_Handle             h_App,
                           uint64_t             referenceVal,
                           e_TimerMode          runningMode);

/**************************************************************************//**
 @Function      GTIMERS_StopTimer

 @Description   Stop the timer.

 @Param[in]     h_Gtimers  - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum   - The number of the timer

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_StopTimer(t_Handle h_Gtimers,e_GtimerNum timerNum);

/**************************************************************************//**
 @Function      GTIMERS_SetTimerReference

 @Description   Set new reference value to the timer.
 
                If the new reference value number of bits is bigger than the HW timer,
                the driver will modify the timer to be a 64 bit SW timer. If a 64 bit
                SW timer reference was modified to a smaller value, the timer still
                acts as a 64 bit SW timer.

 @Param[in]     h_Gtimers   - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum    - The number of the timer.
 @Param[in]     refVal64    - 64-bit new reference value.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_SetTimerReference(t_Handle h_Gtimers,e_GtimerNum timerNum,uint64_t refVal64);

/**************************************************************************//**
 @Function      GTIMERS_SetTimerCounter

 @Description   Set new counter value to the timer.
                If the timer was running when the function was called, the timer is stopped
                so that any interrupts are received in the middle of the the counter change operation.
                The timer counter is modified to the configured value and then it resumes running.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum  - The number of the timer
 @Param[in]     counter64 - 64-bit new value of the counter.

 @Return        E_OK if successful or error type if failed.

 @Cautions      If we are working on less bit timer than the reference we are asked for, then this
                function sets also the extra bits. This can cause a problem to the SW timer.
*//***************************************************************************/
t_Error GTIMERS_SetTimerCounter(t_Handle h_Gtimers,e_GtimerNum timerNum,uint64_t counter64);


/**************************************************************************//**
 @Function      GTIMERS_GetTimerCounter

 @Description   Get the counter value of the timer.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum  - The number of the timer
 @Param[out]    counter64 - Pointer to 64-bit value of the counter.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_GetTimerCounter(t_Handle h_Gtimers,e_GtimerNum timerNum,uint64_t *counter64);

/**************************************************************************//**
 @Function      GTIMERS_GetTimerCapturedVal

 @Description   Get the captured value of the timer.

 @Param[in]     h_Gtimers  - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum   - The number of the timer
 @Param[out]    p_Captured64 - Pointer to 64-bit value of the counter.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_GetTimerCapturedVal(t_Handle h_Gtimers,e_GtimerNum timerNum,uint64_t *p_Captured64);

/**************************************************************************//**
 @Function      GTIMERS_ResumeTimer

 @Description   Restarts a stopped timer.

 @Param[in]     h_Gtimers  - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum   - The number of the timer

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ResumeTimer(t_Handle h_Gtimers,e_GtimerNum timerNum);

/**************************************************************************//**
 @Function      GTIMERS_ResetTimerCounter

 @Description   Resets the timer counter.

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().
 @Param[in]     timerNum  - The number of the timer

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error GTIMERS_ResetTimerCounter(t_Handle h_Gtimers,e_GtimerNum timerNum);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      GTIMERS_DumpRegs

 @Description   Dumps the memory of GTimers module

 @Param[in]     h_Gtimers - Handle to the General Timers as returned from the GTIMERS_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error GTIMERS_DumpRegs(t_Handle h_Gtimers);

#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of gtimers_run_grp */
/** @} */ /* end of gtimers_grp */


#endif /* __GTIMERS_EXT_H */
