/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          i2c_ctrl_ext.h

 @Description   external definition and API for the I2C.

 @Cautions      None.

*//***************************************************************************/

#ifndef __I2C_CTRL_EXT_H
#define __I2C_CTRL_EXT_H

#include "std_ext.h"
#include "i2c_std_ext.h"

/**************************************************************************//**

 @Group         i2c_ctrl_grp I2C Controller Application Programming Interface

 @Description   I2C functions,definitions and enums.

 @{

*//***************************************************************************/



/**************************************************************************//**

 @Group         i2c_ctrl_init_grp I2C Controller Initialization Unit

 @Description   I2C INIT functions,definitions and enums.

 @{

*//***************************************************************************/

/**************************************************************************//**
 @Description   I2C parameter structure.

                Hold all the parameter that are needed to initialize the I2C.
                Include parameters that the driver must get from the user,
                If the user want to change any of the parameters that have default
                value from the driver he can do it by using the configParam functions.

*//***************************************************************************/
typedef struct t_I2cCtrlParam
{
    /* I2C basic parameters */
    uint32_t    i2cBusFreqHz;   /**< Required bus frequency (in Hz) */

    /* System integration parameters */
    uint32_t    baseAddress;    /**< Base address of I2C controller registers */
    uint32_t    i2cClkInHz;     /**< Input clock to the I2c (in Hz) */
    int         irq;            /**< I2C interrupt ID (system-specific number) */

} t_I2cCtrlParam;


/**************************************************************************//**
 @Description   user function to be called in case of data reception in slave mode.
  return: TRUE - DONE, FALSE - continue.
*//***************************************************************************/
typedef bool (t_I2cCtrlSlaveRxCallBack)(uint8_t data);


/**************************************************************************//**
 @Description   user function to be called in case of data transmission in slave mode.
*//***************************************************************************/
typedef uint8_t (t_I2cCtrlSlaveTxCallBack)(void);

/**************************************************************************//**
 @Description   I2C slave mode parameter structure.
*//***************************************************************************/
typedef struct t_I2cCtrlSlaveParam
{
    uint8_t                     slaveAddress;/**< I2C slave address */
    t_I2cCtrlSlaveRxCallBack    *f_RxSlave;  /**< User function to be called upon data arrival   */
    t_I2cCtrlSlaveTxCallBack    *f_TxSlave;  /**< User function to be called upon data request   */
}t_I2cCtrlSlaveParam;


/**************************************************************************//**
 @Function      I2C_CTRL_Config

 @Description   Initialize the I2C controller configuration.

 @Param         p_I2cCtrlParam - (in) I2C controller param.

 @Return        Handle to the I2C controller or NULL if initialization failed.

 @Cautions      None

*//***************************************************************************/
t_Handle I2C_CTRL_Config(t_I2cCtrlParam *p_I2cCtrlParam);


/**************************************************************************//**
 @Function      I2C_CTRL_Init

 @Description   Initialize the I2C controlller.

 @Param         h_I2cCtrl  - (in) Handle to the I2C controller as returned from the I2C_CTRL_Config().

 @Return        t_Handle to the I2C or NULL if initialization failed.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_Init(t_Handle h_I2cCtrl);


/**************************************************************************//**
 @Function      I2C_CTRL_Free

 @Description   Free the I2C controller.

 @Param         h_I2cCtrl  - (in) Handle to the I2C controlller as returned from the I2C_CTRL_Config().

 @Return        t_Handle to the I2C or NULL if initialization failed.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_Free(t_Handle h_I2cCtrl);


/**************************************************************************//**

 @Group         i2c_ctrl_advanced_init_grp I2C Controller Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{

*//***************************************************************************/


/**************************************************************************//**
 @Function      I2C_CTRL_ConfigDigitalFilterSamplingRate

 @Description   Set the I2C requested digital filter sampling rate.
                Default value:[10000000].

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         samplingRateHz - (in) The requesteddigital filter sampling rate.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_ConfigDigitalFilterSamplingRate(t_Handle h_I2cCtrl, uint32_t samplingRateHz);


/**************************************************************************//**
 @Function      I2C_CTRL_ConfigAcceptBroadcast

 @Description   Enable/Disable the broadcast accept capability.
                Default value:[TRUE].

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         acceptBroadcast - (in) Accept broadcast or not.
                                    0 - Disable the broadcast accept capability.
                                    1 - Enable the broadcastmessages at address zero.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_ConfigAcceptBroadcast(t_Handle h_I2cCtrl, bool acceptBroadcast);

/**************************************************************************//**
 @Function      I2C_CTRL_ConfigSlaveMode

 @Description   Set the I2C slave mode.

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         p_I2cCtrlSlaveParam - (in) I2C slave mode param structure.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_ConfigSlaveMode(t_Handle h_I2cCtrl, t_I2cCtrlSlaveParam *p_I2cCtrlSlaveParam);

/**************************************************************************//**
 @Function      I2C_CTRL_ConfigTimeout

 @Description   Set the time in miliseconds we should retry in case of Tx/Rx halt and busy bus.
                Default value:[100].

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         byteWaitUsec - (in) The timeout value (in micro seconds).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_ConfigByteWaitUsec(t_Handle h_I2cCtrl, uint32_t byteWaitUsec);


/**************************************************************************//**
 @Function      I2C_CTRL_ConfigInterruptMode

 @Description   Set the interrupt operation mode.
                Default value:[FALSE].

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         interruptMode - (in) The interrupt operation mode. TRUE - interrupts ON, FALSE - interrupts OFF.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_ConfigInterruptMode(t_Handle h_I2cCtrl, bool interruptMode);


/** @} */ /* end of i2c_ctrl_advanced_init_grp */
/** @} */ /* end of i2c_ctrl_init_grp */


/**************************************************************************//**

 @Group         i2c_ctrl_runtime_grp I2C Controller Runtime Unit

 @Description   I2C RUN Time functions,definitions and enums.

 @{

*//***************************************************************************/

/**************************************************************************//**
 @Function      I2C_CTRL_TxRx

 @Description   Transmit or receive data block to or from I2C device.

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Param         direction - (in) Transmit or receive transaction.

 @Param         devAddr - (in) The I2C destination device number.

 @Param         data - (in) At Tx mode - pointer to the data to be passed.
                            At Rx mode - pointer where to write the received data.

 @Param         size - (in) Data length.

 @Param         repeatedStart - (in) indicates if the user asked to work with repeated Start at the next transaction start.

 @Return        Number of characters that had been sent or received,
                -1 in case there is a failure in sending the address.

 @Cautions      None

*//***************************************************************************/
int32_t I2C_CTRL_TxRx(t_Handle h_I2cCtrl,
                e_I2cTransMode direction,
                uint16_t devAddr,
                uint8_t *data,
                int32_t size,
                bool repeatedStart);


/**************************************************************************//**
 @Function      I2C_CTRL_StopTxRx

 @Description   Generate STOP signal in order to free the bus.

 @Param         h_I2cCtrl  - (in) Handle to the I2C as returned from the I2C_CTRL_Config().

 @Return        None.

 @Cautions      None

*//***************************************************************************/
void I2C_CTRL_StopTxRx(t_Handle h_I2cCtrl);

/** @} */ /* end of i2c_ctrl_runtime_grp */



/**************************************************************************//**

 @Group         i2c_ctrl_control_grp I2C Controller Control Unit

 @Description   I2C CONTROL functions,definitions and enums.

 @{

*//***************************************************************************/



/**************************************************************************//**
 @Function      I2C_CTRL_Enable

 @Description   Enable the I2C controller operation.

 @Param         h_I2cCtrl  - (in) Handle to the I2C controller as returned from the I2C_CTRL_Config().

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_Enable(t_Handle h_I2cCtrl);


/**************************************************************************//**
 @Function      I2C_CTRL_Disable

 @Description   Disable the I2C controller operation.

 @Param         h_I2cCtrl  - (in) Handle to the I2C controller as returned from the I2C_CTRL_Config().

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None

*//***************************************************************************/
t_Error I2C_CTRL_Disable(t_Handle h_I2cCtrl);



/** @} */ /* end of i2c_ctrl_control_grp */



/** @} */ /* end of i2c_ctrl_grp */

#endif /* __I2C_CTRL_EXT_H */
