/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          iphc_ext.h

 @Description   IP Header Compression Application Programming Interface.
*//***************************************************************************/
#ifndef __IPHC_EXT_H
#define __IPHC_EXT_H

#include "std_ext.h"
#include "error_ext.h"


/**************************************************************************//**
 @Group         iphc_grp IP Header Compression Application Programming Interface

 @Description   Header Compression API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Enumeration to select the Header Compression/Decompression
                 Context Table size.
*//***************************************************************************/
typedef enum
{
     e_IPHC_64_BYTES_CONTEXT_TABLE   /**< 64 bytes context table size */
    ,e_IPHC_92_BYTES_CONTEXT_TABLE   /**< 92 bytes context table size */
    ,e_IPHC_128_BYTES_CONTEXT_TABLE  /**< 128 bytes context table size */
} e_IphcContextTblSize;

/**************************************************************************//**
 @Group         comp_init_grp Compressor Initialization Unit

 @Description   Compressor API functions, definitions and enumerations

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Compressor parameters structure,
                used to initialize the compressor module.
*//***************************************************************************/
typedef struct t_IphcCompressorParam
{
    t_Handle    h_Qe;       /**< An initialized QE module handle. */
    uint8_t     pramMemId;  /**< Parameter RAM partition ID */
} t_IphcCompressorParam;

/**************************************************************************//**
 @Function      IPHC_Compressor_Config

 @Description   Configures the compressor.
                This routine allocates memory for the compressor configuration
                and initialization structure and sets user defined and default
                values to the compressor parameters.

 @Param[in]     p_IphcCompressorParam -  Pointer to the compressor
                                          configuration parameter structure.

 @Return        A handle (descriptor) of the newly created compressor structure.
*//***************************************************************************/
t_Handle IPHC_Compressor_Config(t_IphcCompressorParam *p_IphcCompressorParam);

/**************************************************************************//**
 @Function      IPHC_Compressor_Init

 @Description   Initializes the compressor.
                This routine initializes the t_Compressor structure and the
                header compression parameter table present in PRAM.

 @Param[in]     h_Compressor -  Handle to the compressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  IPHC_Compressor_Init(t_Handle h_Compressor);

/**************************************************************************//**
 @Function      IPHC_Compressor_Free

 @Description   Frees the memory occupied by the compressor data structures.
                This routine releases the memory occupied by the
                compressor tables in PRAM and in external memory.

 @Param[in]     h_Compressor -  Handle to the compressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void  IPHC_Compressor_Free(t_Handle h_Compressor);


/**************************************************************************//**
 @Group         comp_advanced_init_grp Compressor Advanced Configuration Unit

 @Description   IPHC - Compressor Configuration APIs used to
                 change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Enumeration to select the context (CID) size.
*//***************************************************************************/
typedef enum
{
     e_IPHC_COMPRESOR_8_BIT_CID_SIZE        /**< 8 bit CID */
    ,e_IPHC_COMPRESSOR_16_BIT_CID_SIZE        /**< 16 bit CID */
} e_IphcCompressorCidSize;

/**************************************************************************//**
 @Description   Enumeration to select the Real Time Stamp Register.
*//***************************************************************************/
typedef enum
{
     e_IPHC_COMPRESSOR_TIME_STAMP_REG1    /**< RTSR 1 */
    ,e_IPHC_COMPRESSOR_TIME_STAMP_REG2    /**< RTSR 2 */
} e_IphcCompressorTimeStamp;

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigCidSize

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'cidSize' according to the following description:
                Context Identifier (i.e. CID) size.
                [e_IPHC_COMPRESSOR_16_BIT_CID_SIZE]
 
 @Param[in]     h_Compressor -  Handle to the compressor.
 @Param[in]     cidSize         -  Context size in bits.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigCidSize(t_Handle h_Compressor, e_IphcCompressorCidSize cidSize);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigTimeStamp

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'rtsrSelect' according to the following description:
                Real Time Stamp Register used for timing feature.
                [e_IPHC_COMPRESSOR_TIME_STAMP_REG1]
 
 @Param[in]     h_Compressor -  Handle to the compressor.
 @Param[in]     rtsrSelect   -  RTSR number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigTimeStamp(t_Handle h_Compressor, e_IphcCompressorTimeStamp rtsrSelect);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigHcctSize

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'hcctSize' according to the following description:
                Header Compression Context Table (i.e. HCCT) size.
                [e_IPHC_64_BYTES_CONTEXT_TABLE]
 
 @Param[in]     h_Compressor -  Handle to the compressor.
 @Param[in]     hcctSize     -  HCCT size.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigHcctSize(t_Handle h_Compressor, e_IphcContextTblSize hcctSize);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_Compressor    - Handle to the compressor.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigFirmwareExternalStructsMemory(t_Handle   h_Compressor,
                                                            uint8_t    memId,
                                                            uint32_t   memAttributes);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigTcpSpace

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxTcpCidVal' according to the following
                description:
                Maximum CID value for TCP.
                [15]
 
 @Param[in]     h_Compressor -  Handle to the compressor.
 @Param[in]     maxTcpCidVal -  Maximum CID value for TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigTcpSpace(t_Handle h_Compressor, uint8_t maxTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigNonTcpSpace

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxNonTcpCidVal' according to the following
                description:
                Maximum CID value for Non-TCP.
                [50]
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[in]     maxNonTcpCidVal -  Maximum CID value for Non-TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigNonTcpSpace(t_Handle h_Compressor, uint16_t maxNonTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigMaxTime

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxTimeVal' according to the following description:
                Maximum time beyond which Compressed Headers will not be
                sent since the last Full Header.
                [0x3fffffff]
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[in]     maxTimeVal        -  Maximum time to send compressed headers.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigMaxTime(t_Handle h_Compressor, uint32_t maxTimeVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigMaxPeriod

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxPeriodVal' according to the following
                description:
                Maximum number of Compressed Non-TCP Headers that can be sent
                without sending a Full Header.
                [16]
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[in]     maxPeriodVal    -  Maximum number of compressed
                                    non-TCP headers that can be sent before
                                    sending a Full Header.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigMaxPeriod(t_Handle h_Compressor, uint16_t maxPeriodVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigMaxHeaderSizeToCompress

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxHeaderSizeToCompress' according to the
                following description:
                Maximum header size in octets that may be compressed.
                [28]
 
 @Param[in]     h_Compressor             -  Handle to the compressor.
 @Param[in]     maxHeaderSizeToCompress    -  Maximum header size to compress.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigMaxHeaderSizeToCompress(t_Handle h_Compressor, uint16_t maxHeaderSizeToCompress);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigMinWrap

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'minTimeGenValWrap' according to the
                following description:
                Minimum time (in seconds) of generation value wrap around.
                [3000]
 
 @Param[in]     h_Compressor         -  Handle to the compressor.
 @Param[in]     minTimeGenValWrap    -  Minimum time of generation value
                                        wrap around.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigMinWrap(t_Handle h_Compressor, uint32_t minTimeGenValWrap);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigNumOfGenerationChanges

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'numOfGenChanges' according to the
                following description:
                Number of generation changes that a CID cannot exceed
                within MIN_WRAP seconds.
                [32]
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[in]     numOfGenChanges    -  Maximum number of generation changes
                                    allowed within min. wrap seconds.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigNumOfGenerationChanges(t_Handle h_Compressor, uint8_t numOfGenChanges);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigGenerationThreshold

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'generationThresholdVal' according to the
                following description:
                Threshold value for generation number change.
                [3]
 
 @Param[in]     h_Compressor             -  Handle to the compressor.
 @Param[in]     generationThresholdVal    -  Threshold value for generation
                                            change. This variable can be
                                            configured to zero to disable
                                            the interrupt for compressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigGenerationThreshold(t_Handle h_Compressor, uint8_t generationThresholdVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigMaxInternalTcpCid

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxInternalTcpCidVal' according to the
                following description:
                Maximum TCP CID value whose context table lies in
                internal memory.
                [3]
 
 @Param[in]     h_Compressor             -  Handle to the compressor.
 @Param[in]     maxInternalTcpCidVal    -  Maximum TCP CID value whose
                                            context is internal.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigMaxInternalTcpCid(t_Handle h_Compressor, uint16_t maxInternalTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigExternalNonTcpCid

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'externalNonTcpCidVal' according to the
                following description:
                First Non-TCP CID value whose context table lies
                in external memory.
                [16]
 
 @Param[in]     h_Compressor             -  Handle to the compressor.
 @Param[in]     externalNonTcpCidVal    -  First non-TCP CID value whose
                                            context is external; all CIDs
                                            equal to or greater than that
                                            CID are external.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigExternalNonTcpCid(t_Handle h_Compressor, uint16_t externalNonTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Compressor_ConfigCidCamSize

 @Description   Changes the internal driver data base from its default
                 configuration.
                Please set 'maxNoOfThreads' according to the
                following description:
                Size of CID CAM in bytes.
                [32]
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[in]     maxNoOfThreads    -  Size of CID CAM in bytes. This field
                                    depends on the maximum number of threads.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ConfigCidCamSize(t_Handle h_Compressor, uint8_t maxNoOfThreads);

/** @} */ /* end of comp_advanced_init_grp */
/** @} */ /* end of comp_init_grp group */


/**************************************************************************//**
 @Group         comp_runtime_ctrl_grp Compressor Runtime Control Unit

 @Description   IPHC - Compressor Runtime Control APIs used to
                 control the compressor.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Enumeration to select the Header Compression type.
*//***************************************************************************/
typedef enum
{
      e_IPHC_IP_UDP_COMPRESSION = 1    /**< IPv4/UDP header compression */
     ,e_IPHC_IP_ONLY_COMPRESSION       /**< IPv4 Only header compression */
} e_IphcCompressionType;

/**************************************************************************//**
 @Description   Compressor context parameters structure,
                used to initialize a context for the compressor.
*//***************************************************************************/
typedef struct
{
    uint16_t                cidNumber;                  /**< Context identifier number. */
    e_IphcCompressionType   compressionType;            /**< Type of compression for a context. */
    uint8_t                 compressionSize;            /**< Header size to compress. */
    uint8_t                 initialGenerationValue;     /**< Initial generation value. */
} t_IphcCompressorContextParam;

/**************************************************************************//**
 @Description   Compressor statistics parameters structure,
                used to get the compressor statistics.
*//***************************************************************************/
typedef struct
{
    uint32_t    compressorFailCounter;            /**< Number of frames that failed compression. */
    uint32_t    compressedNonTcpCounter;          /**< Number of Non-TCP frames compressed by the compressor. */
    uint32_t    fullHeaderCounter;                /**< Number of Full Header frames compressed by the compressor. */
    uint32_t    generationNumChangeCounter;       /**< Number of changes in generation number. */
    uint32_t    generationNumViolationCounter;    /**< Number of violations in generation number. */
    uint32_t    compressedFramesByteCounter;      /**< Number of bytes in compressed frames compressed by the compressor (Compressed header+payload) */
    uint32_t    fullHeaderFramesByteCounter;      /**< Number of bytes in FULL Headers frames compressed by the compressor(Full header +payload)  */
} t_IphcCompressorStatistics;

/**************************************************************************//**
 @Function      IPHC_Compressor_Enable

 @Description   Enables the compressor.
                This routine enables the compressor by setting the
                "compressor enable" bit in header compression
                parameter table present in PRAM.

 @Param[in]     h_Compressor -  Handle to the compressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  IPHC_Compressor_Enable(t_Handle h_Compressor);

/**************************************************************************//**
 @Function      IPHC_Compressor_Disable

 @Description   Disables the compressor.
                This routine resets the compressor enable bit present in
                header compression parameter table in PRAM.

 @Param[in]     h_Compressor - Handle to the compressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_Disable(t_Handle h_Compressor);

/**************************************************************************//**
 @Function      IPHC_Compressor_SetContext

 @Description   Initializes the compressor context at runtime.
                This routine intializes the parameters present in
                Header Compression Context Table in PRAM.

 @Param[in]     h_Compressor                     -  Compressor handle.
 @Param[in]     p_IphcCompressorContextParam    -  Pointer to the header
                                                    compression context
                                                    configuration structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_SetContext(t_Handle h_Compressor, t_IphcCompressorContextParam *p_IphcCompressorContextParam);

/**************************************************************************//**
 @Function      IPHC_Compressor_ResetContext

 @Description   Resets the compressor context at runtime.
                This routine resets the parameters present in
                Header Compression Context Table in PRAM.

 @Param[in]     h_Compressor     -  Compressor handle.
 @Param[in]     cidNumber        -  Context number.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_ResetContext(t_Handle h_Compressor, uint16_t cidNumber);


/**************************************************************************//**
 @Function      IPHC_Compressor_GetStatistics

 @Description   Gets the compressor statistics.
                This routine reads the compressor statistics counters value
                and returns them to the application.

 @Param[in]     h_Compressor                -  Handle to the compressor.
 @Param[in,out] p_IphcCompressorStatistics -  Pointer to compressor
                                               statistics structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_GetStatistics(t_Handle h_Compressor, t_IphcCompressorStatistics *p_IphcCompressorStatistics);

/**************************************************************************//**
 @Function      IPHC_Compressor_GetHcctSize

 @Description   Gets the compressor context table size.
                This routine accesses the parameter table to get the size of
                Header Compression Context Table (HCCT).
 
 @Param[in]     h_Compressor -  Handle to the compressor.
 @Param[out]    p_HcctSize     -  Pointer to HCCT size.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_GetHcctSize (t_Handle h_Compressor, e_IphcContextTblSize *p_HcctSize);

/**************************************************************************//**
 @Function      IPHC_Compressor_GetParameterTblBase

 @Description   Gets the compressor parameter table base address.
                This routine accesses the parameter table to get the size of
                Header Compression Context Table (HCCT).
 
 @Param[in]     h_Compressor     -  Handle to the compressor.
 @Param[out]    p_compParamTbl    -  Pointer to HCPT.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Compressor_GetParameterTblBase (t_Handle h_Compressor, uint32_t *p_compParamTbl);

/** @} */ /* end of comp_runtime_ctrl_grp */


/**************************************************************************//**
 @Collection   Callbacks typedefs
@{
*//***************************************************************************/

/* @} */

/**************************************************************************//**
 @Group         decomp_init_grp Decompressor Initialization Unit

 @Description   Decompressor API functions, definitions and enumerations

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Decompressor parameters structure,
                used to initialize the decompressor.
*//***************************************************************************/
typedef struct t_IphcDecompressorParam
{
    t_Handle    h_Qe;       /**< An initialized QE module handle. */
    uint8_t     pramMemId;  /**< Parameter RAM partition ID */
} t_IphcDecompressorParam;

/**************************************************************************//**
 @Function      IPHC_Decompressor_Config

 @Description   Configures the decompressor.
                This routine allocates memory for the decompressor configuration
                and initialization structure and sets user defined and default
                values to the decompressor parameters.

 @Param[in]     p_IphcDecompressorParam - Pointer to the decompressor
                                           configuration parameter structure.

 @Return        A handle (descriptor) of the newly created decompressor structure.
*//***************************************************************************/
t_Handle IPHC_Decompressor_Config(t_IphcDecompressorParam *p_IphcDecompressorParam);

/**************************************************************************//**
 @Function      IPHC_Decompressor_Init

 @Description   Initializes the decompressor.
                This routine initializes the t_Decompressor structure and the
                header decompression parameter table present in PRAM.

 @Param[in]     h_Decompressor - Handle to the decompressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  IPHC_Decompressor_Init(t_Handle h_Decompressor);

/**************************************************************************//**
 @Function      IPHC_Decompressor_Free

 @Description   Frees the memory allocated for the decompressor.
                This routine releases the memory occupied by the
                decompressor tables in PRAM and in external memory.

 @Param[in]     h_Decompressor - Handle to the decompressor.

 @Return        None.
*//***************************************************************************/
void IPHC_Decompressor_Free(t_Handle h_Decompressor);


/**************************************************************************//**
 @Group         decomp_advanced_init_grp Decompressor Advanced Configuration Unit

 @Description   IPHC - Decompressor Configuration APIs used to
                 change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      t_GetMinWrapFramesCounterCB
 
 @Description   TBD
 
 @Param[in]     h_Arg   - User's application descriptor.
 @Param[out]    p_Cnt   - The exception.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
typedef t_Error t_IphcDecompGetFramesCounterForMinWrapCB (t_Handle h_Arg, uint32_t *p_Cnt);

/**************************************************************************//**
 @Description   Select the decompression type.
*//***************************************************************************/
#define IPHC_IP_UDP_DECOMPRESSION        0x80        /**< IP/UDP header decompression. */
#define IPHC_IP_ONLY_DECOMPRESSION       0x40        /**< IP Only header decompression. */
#define IPHC_IP_TCP_RTP_DECOMPRESSION    0x20        /**< IP/TCP/RTP header decompression. */

/**************************************************************************//**
 @Description   Select the decompression interrupt mask.
*//***************************************************************************/
#define IPHC_DECOMPRESSION_INTR_MASK_NEW_ADL       0x80000000    /**< Interrupt mask for New ADL interrupt. */
#define IPHC_DECOMPRESSION_INTR_MASK_GEN_CH        0x40000000    /**< Interrupt mask for Generation change interrupt. */
#define IPHC_DECOMPRESSION_INTR_MASK_DISCONNECT    0x20000000    /**< Interrupt mask for decompression disconnect interrupt. */


/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigDropCidOutOfRange

 @Description   Configures the decompressor whether to drop or accept the frames
                whose CIDs are not in TCP/Non-TCP space.
                This routine configures a value for the EXR bit present in HDPT.
                [TRUE]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     dropCidOutofRange  - To drop or accept the frames whose CIDs are not in
                                     TCP/Non-TCP space.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigDropCidOutOfRange(t_Handle h_Decompressor, bool  dropCidOutofRange);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigAutomaticLearning

 @Description   Configures the decompressor whether to learn decompression
                automatically or not.
                This routine configures the value for the ADL bit
                present in HDPT.
                [TRUE]

 @Param[in]     h_Decompressor - Handle to the decompressor.
 @Param[in]     automaticLearn - To learn decompression automatically or not.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigAutomaticLearning(t_Handle h_Decompressor, bool automaticLearn);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigDropNotSupportedFrames

 @Description   Configures the decompressor to drop or not the not supported frames.
                This routine configures the value for the NS Drop bit
                present in HDPT.
                [TRUE]

 @Param[in]     h_Decompressor           - Handle to the decompressor.
 @Param[in]     dropNotSupportedFrames     - To drop or not the not supported
                                           frames.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigDropNotSupportedFrames(t_Handle h_Decompressor, bool dropNotSupportedFrames);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigHdctSize

 @Description   Configures the decompressor context table size.
                This routine configures the value for the HDCT size bit
                present in HDPT.
                [e_IPHC_64_BYTES_CONTEXT_TABLE]

 @Param[in]     h_Decompressor - Handle to the decompressor.
 @Param[in]     hdctSize       - HDCT size.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigHdctSize(t_Handle h_Decompressor, e_IphcContextTblSize hdctSize);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigIgnoreDBitFrames

 @Description   Configures the decompressor to ignore or do not ignore frames
                with D-bit set.
                This routine configures the value for the D-bit mode bit
                present in HDPT.

 @Param[in]     h_Decompressor   - Handle to the decompressor.
 @Param[in]     ignoreDBitFrames - To ignore or to not ignore D-bit frames.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigIgnoreDBitFrames(t_Handle h_Decompressor, bool ignoreDBitFrames);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigDecompressionTypes

 @Description   Configures the decompressor decompression types enabled.
                 [IPHC_IP_UDP_DECOMPRESSION]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     decompressionTypes - Decompression types enabled.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigDecompressionTypes(t_Handle h_Decompressor, uint8_t decompressionTypes);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_Decompressor  - Handle to the decompressor.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigFirmwareExternalStructsMemory(t_Handle  h_Decompressor,
                                                              uint8_t   memId,
                                                              uint32_t  memAttributes);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigMinTcpSpace

 @Description   Configures the decompressor minimum CID value for TCP.
                This routine configures value for the min TCP_SPACE field
                present in HDPT.
                [0]

 @Param[in]     h_Decompressor - Handle to the decompressor.
 @Param[in]     minTcpCidVal   - Minimum CID value for TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigMinTcpSpace(t_Handle h_Decompressor, uint8_t minTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigTcpSpace

 @Description   Configures the decompressor maximum CID value for TCP.
                This routine configures value for the TCP_SPACE field
                present in HDPT.
                [15]

 @Param[in]     h_Decompressor - Handle to the decompressor.
 @Param[in]     maxTcpCidVal   - Maximum CID value for TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigTcpSpace(t_Handle h_Decompressor, uint8_t maxTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigMinNonTcpSpace

 @Description   Configures the decompressor minimum CID value for Non-TCP.
                This routine configures value for the minimum NON_TCP_SPACE field
                present in HDPT.
                [0]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     minNonTcpCidVal - Minimum CID value for Non-TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigMinNonTcpSpace(t_Handle h_Decompressor, uint16_t minNonTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigNonTcpSpace

 @Description   Configures the decompressor maximum CID value for Non-TCP.
                This routine configures value for the NON_TCP_SPACE field
                present in HDPT.
                [50]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     maxNonTcpCidVal - Maximum CID value for Non-TCP.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigNonTcpSpace(t_Handle h_Decompressor, uint16_t maxNonTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigNonAdlMaxHeaderSizeToDecompress

 @Description   Configures the decompressor maximum header size in octets
                 that may be decompressed.
                This routine configures value for the MAX_HEADER field
                present in HDPT.
                [0]

 @Param[in]     h_Decompressor                 - Handle to the decompressor.
 @Param[in]     maxHeaderSizeToDecompress    - Maximum number of bytes in the
                                              header to decompress.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigNonAdlMaxHeaderSizeToDecompress(t_Handle h_Decompressor, uint16_t maxHeaderSizeToDecompress);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigMinWrapEn

 @Description   Configures the driver to enable the MIN_WRAP validation.
                [FALSE, 3000]

 @Param[in]     h_Decompressor                  - Handle to the decompressor.
 @Param[in]     minWrap                         - Minimum time of generation value wrap around.
 @Param[in]     p_GetFramesCounterForMinWrapCB  - A callback that the driver should call to
                                                  poll to check the specific portocol
                                                  counter; this parameter is optional.
 @Param[in]     h_Arg                           - Argument to pass when the callback
                                                  will be called by the driver.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigMinWrapEn (t_Handle                                   h_Decompressor,
                                           uint32_t                                   minWrap,
                                           t_IphcDecompGetFramesCounterForMinWrapCB   *p_GetFramesCounterForMinWrapCB,
                                           t_Handle                                   h_Arg);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigNonAdlDiffGen

 @Description   Configures the change in generation number allowed to be
                treated by the QE.
                This routine configures value for the DiffGen field
                present in HDPT.
                [0x3F]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     diffGenVal       - value of change in generation number
                                  allowed to be treated by the QE.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigNonAdlDiffGen(t_Handle h_Decompressor, uint8_t diffGenVal );

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigMaxInternalTcpCid

 @Description   Configures the maximum TCP CID that its context is internal;
                 all other CIDs are external.
                This routine configures value for the Max Internal TCP CID
                field present in HDPT.
                [3]

 @Param[in]     h_Decompressor             - Handle to the decompressor.
 @Param[in]     maxInternalTcpCidVal    - Maximum TCP CID value whose
                                           context is internal.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigMaxInternalTcpCid(t_Handle h_Decompressor, uint16_t maxInternalTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigExternalNonTcpCid

 @Description   Configures the first non-TCP CID that its context is external.
                This routine configures value for the External non-TCP CID
                field present in HDPT.
                [16]

 @Param[in]     h_Decompressor             - Handle to the decompressor.
 @Param[in]     externalNonTcpCidVal    - First non-TCP CID value whose
                                           context is external; all CIDs
                                           equal to or greater than that
                                           CID are external.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigExternalNonTcpCid(t_Handle h_Decompressor, uint16_t externalNonTcpCidVal);

/**************************************************************************//**
 @Function      IPHC_Decompressor_ConfigInterruptMask

 @Description   Configures interrupt mask for header decompression.
                This routine configures value for the Header Decompression
                Interrupt Mask field present in HDPT.
                [IPHC_DECOMPRESSION_INTR_MASK_NEW_ADL]

 @Param[in]     h_Decompressor     - Handle to the decompressor.
 @Param[in]     interruptMask     - interrupt mask for header decompression.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_ConfigInterruptMask(t_Handle h_Decompressor, uint32_t interruptMask );

/** @} */ /* end of decomp_advanced_init_grp */
/** @} */ /* end of decomp_init_grp group */


/**************************************************************************//**
 @Group         decomp_runtime_ctrl_grp Decompressor Runtime Control Unit

 @Description   IPHC - Decompressor Runtime Control APIs used to
                 control the decompressor.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Enumeration to select the decompression type for a CID.
*//***************************************************************************/
typedef enum
{
    e_IPHC_CID_NO_DECOMPRESSION         /**< No decompression for the CID. */
    ,e_IPHC_CID_IP_UDP_DECOMPRESSION    /**< IP/UDP header decompression for the CID. */
    ,e_IPHC_CID_IP_ONLY_DECOMPRESSION   /**< IP Only header decompression for the CID. */
} e_IphcCidDecompressionType;

/**************************************************************************//**
 @Description   Decompressor context parameters structure,
                used to initialize a context for the decompressor.
*//***************************************************************************/
typedef struct
{
    uint16_t                    cidNumber;                    /**< Context identifier number. */
    e_IphcCidDecompressionType  decompressionType;            /**< Type of decompression for a context. */
    uint8_t                     decompressionSize;            /**< Header size to decompress. */
    uint8_t                     generationNumber;             /**< Initial generation value. */
} t_IphcDecompressorContextParam;

/**************************************************************************//**
 @Description   Decompressor context update parameters structure,
                used to update a context for the decompressor at runtime.
*//***************************************************************************/
typedef struct
{
    uint16_t    cidNumber;            /**< Context identifier number. */
    bool        udpChecksum;        /**< UDP checksum field. If set, indicates to the QE
                                         that the compressed header received on this CID
                                         includes UDP Checksum */
     uint8_t    generationNumber;    /**< Generation number. */
    uint8_t     *p_FullHeader;        /**< Pointer to the new full header. */
} t_IphcDecompressorUpdateContext;

/**************************************************************************//**
 @Description   Decompressor statistics structure.
*//***************************************************************************/
typedef  struct
{
    uint32_t    cidNotInNonTcpSpaceCounter;                 /**< Number of Non-TCP frames received with CID not within defined range. */
    uint32_t    compressedIpv4UdpAndIpv4OnlyCounter;         /**< Number of compressed IPv4/UDP frames decompressed by the decompressor. */
    uint32_t    fullHeaderCounter;                          /**< Number of Full Headers frames decompressed by the decompressor. */
    uint32_t    fullHeaderAdlCounter;                       /**< Number of Full Headers frames received with invalid context and
                                                                 Automatic Decompression Learning. */
    uint32_t    fullHeaderContextTblInvalidCounter;         /**< Number of Full Headers frames received with invalid context. */
    uint32_t    compressedHeaderContextTblInvalidCounter;   /**< Number of compressed frames received with invalid context. */
    uint32_t    fullHeaderGenNumChAutoRecoveryCounter;      /**< Number of Full Headers frames received with a change in
                                                                 generation number and automatic recovery. */
    uint32_t    fullHeaderGenNumChSendCpuCounter;           /**< Number of FH frames received with a change in generation
                                                                 number and sent to CPU. */
    uint32_t    genNumChCompressedHeaderDropCounter;        /**< Number of Compressed non-TCP frames dropped due a change
                                                                 in generation number. */
    uint32_t    errorFramesByteCounter;        			    /**< Total bytes in frames that were not decompressed due to errors. */
} t_IphcDecompressorStatistics;

/**************************************************************************//**
 @Function      IPHC_Decompressor_Enable

 @Description   Enables the decompressor.
                This routine sets the decompressor enable bit present in
                header decompression parameter table in PRAM.

 @Param[in]     h_Decompressor - Handle to the decompressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_Enable(t_Handle h_Decompressor);

/**************************************************************************//**
 @Function      IPHC_Decompressor_Disable

 @Description   Disables the decompressor.
                This routine resets the decompressor enable bit present in
                header decompression parameter table in PRAM.

 @Param[in]     h_Decompressor - Handle to the decompressor.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_Disable(t_Handle h_Decompressor);

/**************************************************************************//**
 @Function      IPHC_Decompressor_SetContext

 @Description   Initializes the decompressor context in PRAM.
                This routine intializes the parameters present in
                Header Decompression Context Table in PRAM.

 @Param[in]     h_Decompressor                     - Decompressor handle.
 @Param[in]     p_IphcDecompressorContextParam    - Pointer to the header
                                                   decompression context
                                                   configuration structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_SetContext(t_Handle h_Decompressor, t_IphcDecompressorContextParam *p_IphcDecompressorContextParam);

/**************************************************************************//**
 @Function      IPHC_Decompressor_UpdateContext

 @Description   Updates the decompressor context at runtime when it encounters
                changes in generation number.
                This routine updates the parameters present in
                Header Decompression Context Table in PRAM at runtime.

 @Param[in]     h_Decompressor                     - Decompressor handle.
 @Param[in]     p_IphcDecompressorUpdateContext - Pointer to the header
                                                   decompression context
                                                   updating structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_UpdateContext(t_Handle h_Decompressor, t_IphcDecompressorUpdateContext *p_IphcDecompressorUpdateContext);

/**************************************************************************//**
 @Function      IPHC_Decompressor_InvalidateContext

 @Description   Invalidates the decompressor context at runtime.
                This routine resets the valid bit present in
                Header Decompression Context Table in PRAM.

 @Param[in]     h_Decompressor     - Decompressor handle.
 @Param[in]     cidNumber        - Context number.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_InvalidateContext(t_Handle h_Decompressor, uint16_t cidNumber);

/**************************************************************************//**
 @Function      IPHC_Decompressor_GetStatistics

 @Description   Gets the decompressor statistics.
                This routine reads the decompressor statistics counter values
                and returns them to the application.

 @Param[in]     h_Decompressor                  -  Handle to the decompressor.
 @Param[out]    p_IphcDecompressorStatistics -  Pointer to decompressor
                                                 statistics structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_GetStatistics(t_Handle h_Decompressor, t_IphcDecompressorStatistics *p_IphcDecompressorStatistics);

/** @} */ /* end of decomp_runtime_ctrl_grp */
/** @} */ /* end of iphc_grp group */


/* IPHC functions needed to configure IW structures */

/**************************************************************************//*
 @Function      IPHC_Decompressor_GetParameterTblBase

 @Description   Gets the decompressor parameter table base address.
                This routine is used by the IW driver to get the base address of
                Header Decompression Parameter Table (HDPT) to configure the
                decompression PCD.
 
 @Param[in]     h_Decompressor             -  Handle to the decompressor.
 @Param[out]    p_DecompParamTblAddr    -  Pointer to HDPT.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_GetParameterTblBase(t_Handle h_Decompressor, uint32_t *p_DecompParamTblAddr);

/**************************************************************************//*
 @Function      IPHC_Decompressor_IsMiss1PtrRequired

 @Description   Gets according to decompressor parameter if Miss1 Ptr has to be configured.
                This routine is used by the IW driver
 
 
 @Param[in]     h_Decompressor             -  Handle to the decompressor.
 @Param[out]    isRequired                -  the answer will be returned here.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_IsMiss1PtrRequired(t_Handle h_Decompressor, bool *isRequired);

/**************************************************************************//*
 @Function      IPHC_Decompressor_IsMiss2PtrRequired

 @Description   Gets according to decompressor parameter if Miss2 Ptr has to be configured.
                This routine is used by the IW driver
 
 
 @Param[in]     h_Decompressor             -  Handle to the decompressor.
 @Param[out]    isRequired                -  the answer will be returned here.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IPHC_Decompressor_IsMiss2PtrRequired(t_Handle h_Decompressor, bool *isRequired);


#endif /* __IPHC_EXT_H */
