/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ipsec_ext.h

 @Description   structures and definitions for operating ipsec with the relevant algorithms.

*//***************************************************************************/

#ifndef __IPSEC_EXT_H
#define __IPSEC_EXT_H

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_ipsec SEC IPSEC

 @Description   SEC IPSEC functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sec_ipsec_init SEC IPSEC Initialization Unit

 @Description   SEC IPSEC initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   enum of the different Cipher algorithm types
 
                enum that describes the different cipher algorithm types
                and modes
*//***************************************************************************/
typedef enum e_CipherAlgType{
	e_SDES_ECB ,
    e_SDES_CBC ,
	e_TDES_ECB ,
    e_TDES_CBC ,
	e_AES_ECB  ,
    e_AES_CBC
}e_CipherAlgType;

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      ipsec_alloc_ctx

 @Description   Allocate memory for the ipsec ctx structure.
 
 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle ipsec_alloc_ctx();

/**************************************************************************//**
 @Function      ipsec_cipher_setalg

 @Description   The routine initializes the cipher algorithm name.

				The routine initializes the cipher algorithm name in
                the CipherAlgType field in the ipsec ctx structure.
                Moreover, the function initializes the padRestack field
                to default and the flag field to zero (indicates that
                the default operation mode is ECB).
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     type    - cipher algorithm type.

 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_setalg(void *ctx, e_CipherAlgType type);

/**************************************************************************//**
 @Function      ipsec_cipher_setkey

 @Description   The routine sets the specific key and its length in the context structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     key     - key handler.
 @Param[in]     keylen  - key length.
 @Param[in]     flags   - Not relevant.

 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_setkey(void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/**************************************************************************//**
 @Function      ipsec_cipher_setiv

 @Description   The routine sets the iv and its length in the context structure

                The routine sets the iv and its length in the
                context structure.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - Iv (Initialization vector) handler.
 @Param[in]     len - Iv length.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_setiv(void *ctx, void *src, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_cipher_set_srclen

 @Description   The routine sets the data source length in the ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     len - the source length.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_set_srclen(void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_cipher_encrypt

 @Description   The routine initializes the input and output data pointers.

				The routine initializes the input and output data
                pointers in the ipsec ctx structure that are required for
                the encryption operation. In order to operate the
                request there is a need to call to ipsec_encrypt_operate
                function.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst - the cipher output handler..
 @Param[in]     src - the cipher input handler.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      ipsec_cipher_decrypt

 @Description   The routine initializes the input and output data pointers.

				The routine initializes the input and output data
                pointers in the ipsec ctx structure that are required for
                the decryption operation. In order to operate the
                request there is a need to call to ipsec_decrypt_operate
                function.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst     - the cipher output handler.
 @Param[in]     src     - the cipher input handler.
 @Param[in]     restack - indicates if to restore decrypt key (relevant in AES algorithm)

 @Return        None.
*//***************************************************************************/
void     ipsec_cipher_decrypt(void *ctx, unsigned char *dst, const unsigned char *src, bool restack);

/**************************************************************************//**
 @Function      ipsec_digest_init

 @Description   The routine updates the HashAlgTye field in the ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     type    - the digest algorithm type.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_digest_init(void *ctx, e_HashAlgType type);

/**************************************************************************//**
 @Function      ipsec_digest_update

 @Description   The routine sets the data source and its length in the specific ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     data    - the digest input handler.
 @Param[in]     len     - the digest input length.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_digest_update(void *ctx, const unsigned char *data, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_digest_update_ctx

 @Description   The routine sets the key source and its length in the specific ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - the digest key handler.
 @Param[in]     len - the digest key length.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_digest_update_ctx(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_digest_setpad

 @Description   The routine sets the pad option.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_digest_setpad(void *ctx);

/**************************************************************************//**
 @Function      ipsec_digest_final

 @Description   The routine sets the output data pointer in the ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     out - output handler.
 
 @Return        None.
*//***************************************************************************/
void     ipsec_digest_final(void *ctx, unsigned char *out);

#ifdef SEC2

/* ipsec esp */
#define ESP_DESC_TYPE 0x00000008

/**************************************************************************//**
 @Function      ipsec_esp_alloc_ctx

 @Description   Allocate memory for the ipsec ctx structure.
 
 @Return        ctx - ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle ipsec_esp_alloc_ctx();

/**************************************************************************//**
 @Function      ipsec_esp_cipher_setalg

 @Description   The routine initializes the cipher algorithm name.

				The routine initializes the cipher algorithm name in
                the CipherAlgType field in the ipsec esp ctx structure.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     type    - cipher algorithm type.

 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_setalg(void *ctx, e_CipherAlgType type);

/**************************************************************************//**
 @Function      ipsec_esp_cipher_setkey

 @Description   The routine sets the specific key and its length in the context structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     key     - key handler.
 @Param[in]     keylen  - key length.
 @Param[in]     flags   - Not relevant.

 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_setkey(void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/**************************************************************************//**
 @Function      ipsec_esp_cipher_setiv

 @Description   The routine sets the iv and its length in the context structure

                The routine sets the iv and its length in the
                context structure.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - Iv (Initialization vector) handler.
 @Param[in]     len - Iv length.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_setiv(void *ctx, void *src, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_esp_cipher_set_outiv

 @Description   The routine sets the out iv and its length in the context structure

                The routine sets the out iv and its length in the
                context structure.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - out Iv( Initialization vector) handler.
 @Param[in]     len - out Iv length.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_set_outiv(void *ctx, void *src, unsigned int len)	;

/**************************************************************************//**
 @Function      ipsec_esp_cipher_set_srclen

 @Description   The routine sets the data source length in the ipsec ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     len - the source length.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_set_srclen(void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_esp_cipher_encrypt

 @Description   The routine initializes the input and output data pointers.

				The routine initializes the input and output data
                pointers in the ipsec esp ctx structure that are required for
                the encryption operation. In order to operate the
                request there is a need to call to ipsec_esp_encrypt_operate
                function.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst - the cipher output handler..
 @Param[in]     src - the cipher input handler.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      ipsec_esp_cipher_decrypt

 @Description   The routine initializes the input and output data pointers.

				The routine initializes the input and output data
                pointers in the ipsec esp ctx structure that are required for
                the decryption operation. In order to operate the
                request there is a need to call to ipsec_esp_decrypt_operate
                function.
				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     dst - the cipher output handler.
 @Param[in]     src - the cipher input handler.

 @Return        None.
*//***************************************************************************/
void ipsec_esp_cipher_decrypt(void *ctx, unsigned char *dst, const unsigned char *src);

/**************************************************************************//**
 @Function      ipsec_esp_digest_init

 @Description   The routine updates the HashAlgTye field in the ipsec esp ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     type    - the digest algorithm type.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_digest_init(void *ctx, e_HashAlgType type);

/**************************************************************************//**
 @Function      ipsec_esp_digest_update

 @Description   The routine sets the data source and its length in the specific ipsec esp ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx     - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     data    - the digest input handler.
 @Param[in]     len     - the digest input length.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_digest_update(void *ctx, const unsigned char *data, unsigned int len);

/**************************************************************************//**
 @Function      ipsec_esp_digest_update_ctx

 @Description   The routine sets the key source and its length in the specific ipsec esp ctx structure.

				The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 @Param[in]     src - the digest key handler.
 @Param[in]     len - the digest key length.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_digest_update_ctx(void *ctx, unsigned char *src, unsigned int len);

#endif /* SEC2 */

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_ipsec_runtime SEC IPSEC Runtime Unit

 @Description   SEC IPSEC Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/
/**************************************************************************//**
 @Function      ipsec_encrypt_operate

 @Description   The routine operate a request according to the parameters in the ipsec ctx structure.

				The routine initializes a request structure
                according to the Cipher algorithm name and the Flag
                field. Then, the function fills in the relevant
                fields for the specific request and calls to ioctl
                function in order to process the request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 
 @Return        None.
*//***************************************************************************/
void      ipsec_encrypt_operate(void *ctx);

/**************************************************************************//**
 @Function      ipsec_decrypt_operate

 @Description   The routine operate a request according to the parameters in the ipsec ctx structure.

				The routine initializes a request structure
                according to the Cipher algorithm name and the Flag
                field. Then, the function fills in the relevant
                fields for the specific request and calls to ioctl
                function in order to process the request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 
 @Return        None.
*//***************************************************************************/
void      ipsec_decrypt_operate(void *ctx);

#ifdef SEC2
/**************************************************************************//**
 @Function      ipsec_esp_encrypt_operate

 @Description   The routine operate a request according to the parameters in the ipsec esp ctx structure.

				The routine initializes a request structure
                according to the Cipher algorithm name and the Flag
                field. Then, the function fills in the relevant
                fields for the specific request and calls to ioctl
                function in order to process the request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_encrypt_operate(void *ctx);

/**************************************************************************//**
 @Function      ipsec_esp_decrypt_operate

 @Description   The routine operate a request according to the parameters in the ipsec esp ctx structure.

				The routine initializes a request structure
                according to the Cipher algorithm name and the Flag
                field. Then, the function fills in the relevant
                fields for the specific request and calls to ioctl
                function in order to process the request.
                The routine is not a part of the Crypto Api standard routines.

 @Param[in]     ctx - ctx handler - holds the relevant parameters for the operation.
 
 @Return        None.
*//***************************************************************************/
void ipsec_esp_decrypt_operate(void *ctx);

#endif /* SEC2 */

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec ipsec */
/** @} */ /* end of group sec  */

#endif /* IPSEC_EXT */

