/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          IW_PPPOHT_ext.h

 @Description   Interworking-PPPoHT Application Programming Interface.
*//***************************************************************************/
#ifndef __IW_PPPOHT_EXT_H
#define __IW_PPPOHT_EXT_H

#include "error_ext.h"
#include "std_ext.h"
#include "PPPoHT_ext.h"


/**************************************************************************//**
 @Description   IW-PPPOHT initialization structure
 
                Initialization parameters structure, used when calling
                the configuration routine IW_PPPoHT_ConfigAndInit().
*//***************************************************************************/
typedef struct
{
    uint16_t    totalNumOfIntTxQds;             /**< Total number of internal Tx QD's used for the IW queues */
                                                /**< Note that this number includes all txQd's of IW links/classes */
                                                /**< that are defined IW in one or both directions !!! */
    uint16_t    totalNumOfIntTxQdsExtension;    /**< Total number of internal Tx QD's with extension. */
                                                /**< Note that this number can't be bigger than totalNumOfIntTxQds */
    uint16_t    totalNumOfExtTxQds;             /**< Total number of external Tx QD's used for the IW queues */
                                                /**< Note that this number includes all txQd's of IW links/classes */
                                                /**< that are defined IW in one or both directions !!! */
    uint16_t    totalNumOfExtTxQdsExtension;    /**< Total number of external Tx QD's with extension. */
                                                /**< Note that this number can't be bigger than totalNumOfExtTxQds */
    uint8_t     numOfFilterTables;              /**< Total number of PCD tables used for the IW PPPOHT */
    uint8_t     *p_NumOfPidsPerFilterTable;     /**< For each PCD table, how many PIDs it serves */
    t_Handle    h_TxFbps;                       /**< Tx FBPs - The same handle must be passed also to all
                                                     interworked bundles and links */
    t_Handle    h_Mcc;                          /**< A handle to the MCC module */
    uint8_t     pramMemId;                      /**< Parameter RAM partition ID */
    uint8_t     fwExtStructsMemId;              /**< Memory partition ID for the firmware's external structures */
    uint32_t    fwExtStructsMemAttributes;      /**< Memory attributes mask for the firmware's external structures
                                                     (a combination of MEMORY_ATTR_x flags) */
    uint32_t    dataMemAttributes;              /**< Memory attributes mask for all data in IW (a combination of MEMORY_ATTR_x flags) */
    e_CommMode  globalIwDir;                    /**< e_COMM_MODE_RX should be set if at least one link/bundle will work */
                                                /**< with IW in Rx direction, e_COMM_MODE_TX should be set if at least */
                                                /**< one link/bundle will work with IW in Tx direction*/
} t_IwPppohtParam;

/**************************************************************************//**
 @Description   IW-PPPOHT export parameters
 
                Used by the application in order to get IW-PPPOHT parameters
*//***************************************************************************/
typedef struct
{
    uint32_t    iwPppohtGlobalPramAddr;     /**< Address of the IW glogal parameter RAM page */
    uint32_t    intTxQdBaseAddr;            /**< Base address of the internal TxQd block */
    uint32_t    intTxQdExtensionBaseAddr;   /**< Base address of the internal TxQdE block */
    uint32_t    extTxQdBaseAddr;            /**< Base address of the external TxQd block */
    uint32_t    extTxQdExtensionBaseAddr;   /**< Base address of the external TxQdE block */
    uint8_t     numOfFilterTables;          /**< Total number of PCD tables used for the IW PPPOHT */
    uint8_t     *p_NumOfPidsPerFilterTable; /**< For each PCD table, how many PIDs it serves */
    uint8_t     fwExtStructsMemId;          /**< Memory partition ID for firmware's external structures (BDs, QDs, interrupt queues) */
    uint32_t    fwExtStructsMemAttr;        /**< Memory partition attributes of firmware's external structures */
    uint32_t    dataMemAttributes;          /**< Memory attributes for IW data buffers */
    uint8_t     mccNum;                     /**< MCC Id */
} t_IwPppohtDetails;

/**************************************************************************//**
 @Description   IW-PPPOHT port export parameters
 
                Used by the application in order to get IW-PPPOHT port parameters
*//***************************************************************************/
typedef struct
{
    bool    isBundle;   /**< boolean indicates whther the port is bundle or link */
    uint8_t id;         /**< the bundle or link id */
    uint8_t queueId;    /**< the link-queue or bundle-class id */
} t_IwPppohtPortDetails;


/**************************************************************************//**
 @Function      IW_PPPoHT_ConfigAndInit

 @Description   Configures an IW-PPPoHT module.
                This routine allocates the IW-PPPoHT driver structure. It also
                allocates the structures and memory areas required for the
                management of IW-PPPOHT resources.

 @Param[in]     p_IwPppohtParam - A pointer to the initializing parameter
                                  structure for the IW-PPPoHT.

 @Return        A handle (descriptor) of the newly created IW-PPPoHT structure.
*//***************************************************************************/
t_Handle    IW_PPPoHT_ConfigAndInit(t_IwPppohtParam *p_IwPppohtParam);

/**************************************************************************//**
 @Function      IW_PPPoHT_Free

 @Description   Free IW-PPPoHT resources.

 @Param[in]     h_IwPppoht - A handle to a IW-PPPoHT device.

 @Return        None
*//***************************************************************************/
void        IW_PPPoHT_Free(t_Handle h_IwPppoht);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetDetails

 @Description   A routine for exporting IW-PPPOHT parameters

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[out]    p_IwPppohtDetails - A structure of return parameters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetDetails(t_Handle h_IwPppoht, t_IwPppohtDetails *p_IwPppohtDetails);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetLinkAppId

 @Description   A routine for returning the Link application handle

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[in]     linkId -            Link global index
 @Param[out]    h_AppHandle -       A pointer to the Link application handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetLinkAppId (t_Handle h_IwPPPoHT, uint8_t linkId, t_Handle *h_AppHandle);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetLinkHandle

 @Description   A routine for returning the Link handle

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[in]     linkId -            Link global index
 @Param[out]    h_Link -            A pointer to the Link handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetLinkHandle (t_Handle h_IwPppoht, uint8_t linkId, t_Handle *h_Link);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetBundleAppId

 @Description   A routine for returning the bundle application handle

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[in]     bundleId -          Bundle index
 @Param[out]    p_AppHandle -       A pointer to the bundle application handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetBundleAppId (t_Handle h_IwPPPoHT, uint8_t bundleId, t_Handle *p_AppHandle);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetBundleHandle

 @Description   A routine for returning the bundle handle

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[in]     bundleId -          Bundle index
 @Param[out]    p_Bundle -          A pointer to the bundle handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetBundleHandle (t_Handle h_IwPppoht, uint8_t bundleId, t_Handle *p_Bundle);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetFilterBaseAddr

 @Description   A routine for exporting A PID-PCD table address

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[in]     index -             PCD index
 @Param[out]    p_BaseAddr -        Address of the specified PID-to-PCD table
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetFilterBaseAddr(t_Handle h_IwPppoht, uint8_t index, uint32_t *p_BaseAddr);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetTxFbpsBaseAddr

 @Description   A routine for returning the Tx buffer pools parameter
                tables base address.

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[out]    p_BaseAddr -        Tx FBPs base address.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetTxFbpsBaseAddr(t_Handle h_IwPppoht, uint32_t  *p_BaseAddr);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetMccHandle

 @Description   A routine for returning the MCC handle

 @Param[in]     h_IwPppoht -        The IW-PPPoHT handle.
 @Param[out]    h_Mcc -             A pointer to the MCC handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetMccHandle(t_Handle h_IwPppoht, t_Handle  *h_Mcc);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetPortDetails

 @Description   This routine returns a specific port details according to a given
                TX queue index.

 @Param[in]     h_IwPppoht              - The IW-PPPoHT handle.
 @Param[in]     txQuueId                - the IW Tx queue index.
 @Param[in]     isExt                   - boolean indicates whether the queue is external or internal.
 @Param[out]    p_IwPppohtPortDetails   - The relevant port details.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_GetPortDetails(t_Handle               h_IwPppoht,
                                     uint8_t                txQuueId,
                                     bool                   isExt,
                                     t_IwPppohtPortDetails  *p_IwPppohtPortDetails);

/**************************************************************************//**
 @Function      IW_PPPoHT_GetRegistrationCallback

 @Description   Return a callback for registering a PPPOHT module in
                the IW-PPPOHT.

 @Param[in]     h_IwPppoht - The IW-PPPoHT handle.

 @Return        The callback routine.
*//***************************************************************************/
t_IwRegistrationCallback  *IW_PPPoHT_GetRegistrationCallback(t_Handle h_IwPppoht);

/**************************************************************************//**
 @Function      IW_PPPoHT_LinkRx

 @Description   Rx routine for the specified link.

 @Param[in]     h_IwPppoht -    The IW-PPPoHT handle.
 @Param[in]     linkId -        link id.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IW_PPPoHT_LinkRx(t_Handle h_IwPppoht, uint8_t linkId);

/**************************************************************************//**
 @Function      IW_PPPoHT_BundleRx

 @Description   Rx routine for the specified bundle.

 @Param[in]     h_IwPppoht -    The IW-PPPoHT handle.
 @Param[in]     bundleId -      Bundle Id.
 @Param[in]     clsIndex -      Class index.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error IW_PPPoHT_BundleRx(t_Handle h_IwPppoht, uint8_t bundleId, uint8_t clsIndex);


/**************************************************************************//**
 @Description   IW-PPPOHT Link initialization structure
 
                Initialization parameters structure, used when calling
                the configuration routine IW_PPPoHT_LINK ConfigAndInit().
*//***************************************************************************/
typedef struct
{
    t_Handle                h_IwPppoht;         /**< A handle to the main IW-PPPOHT object */
    uint8_t                 filterTableIndex;   /**< The PIDs-to-PCDs table used for this link */
                                                /**< Used only when Rx IW is used */
    e_CommMode              iwDir;              /**< Iw direction  */
    bool                    iwTxQueues[PPPOHT_MAX_NUM_OF_TX_QS_PER_LINK];
    bool                    iwP2pSupport;       /**< TRUE for links iw-ed with other PPP links, FALSE otherwise. */
} t_IwPppohtLinkParam;


/**************************************************************************//**
 @Function      IW_PPPoHT_LINK_ConfigAndInit

 @Description   Configures an IW-PPPoHT Link module.
                This routine allocates and initializes the IW-PPPoHT link
                driver structure.
 
 @Param[in]     p_IwPppohtLinkParam - A pointer to the initializing parameter
                                      structure for the IW-PPPoHT link.

 @Return        A handle (descriptor) of the newly created IW-PPPoHT link structure.
*//***************************************************************************/
t_Handle                IW_PPPoHT_LINK_ConfigAndInit(t_IwPppohtLinkParam *p_IwPppohtLinkParam);

/**************************************************************************//**
 @Function      IW_PPPoHT_LINK_Free

 @Description   Free IW-PPPoHT Link resources.

 @Param[in]     h_IwPppohtLink - A handle to a IW-PPPoHT Link device.

 @Return        None
*//***************************************************************************/
void                    IW_PPPoHT_LINK_Free(t_Handle h_IwPppohtLink);

/**************************************************************************//**
 @Function      IW_PPPoHT_LINK_GetRegistrationCallback

 @Description   Return a callback for registering a PPPOHT Link module in
                the IW-PPPOHT Link.

 @Param[in]     h_IwPppoht - The IW-PPPoHT Link handle.

 @Return        The callback routine.
*//***************************************************************************/
t_IwRegistrationCallback  *IW_PPPoHT_LINK_GetRegistrationCallback(t_Handle h_IwPppohtLink);

/**************************************************************************//**
 @Function      IW_PPPoHT_LINK_GetTxQdIndex

 @Description   A routine for returning the TxQD index and memory location
                information for a given link Tx queue.

 @Param[in]     h_IwPppohtLink -    The IW-PPPoHT Link handle.
 @Param[in]     queueIndex -        Tx queue index
 @Param[out]    p_TxQdIndex -       QD index.
 @Param[out]    p_IsExtTxQd -       Internal/External memory indication.
 @Param[out]    p_MccId - MCC Id of the wrapped PPPoHT object
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_LINK_GetTxQdIndex(t_Handle   h_IwPppohtLink,
                                        uint8_t    queueIndex,
                                        uint16_t   *p_TxQdIndex,
                                        bool       *p_IsExtTxQd,
                                        uint8_t    *p_MccId);


/**************************************************************************//**
 @Description   IW-PPPOHT Bundle initialization structure
 
                Initialization parameters structure, used when calling
                the configuration routine IW_PPPoHT_BUNDLE_ConfigAndInit().
*//***************************************************************************/
typedef struct
{
    t_Handle                h_IwPppoht;     /**< A handle to the main IW-PPPOHT object */
    uint8_t                 numOfClasses;   /**< Number of active bundle classes */
    uint8_t                 filterTablesIndexes[PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE];
                                            /**< The PIDs-to-PCDs table indexes used for this link */
                                            /**< Used only when Rx IW is used */
    bool                    iwTxClasses[PPPOHT_MAX_NUM_OF_CLASSES_PER_BUNDLE];
                                            /**< For each class, whether its Tx queue is used by
                                                 IW (TRUE) or by CPU (FALSE) */
    e_CommMode              iwDir;          /**< Iw direction  */
    bool                    iwP2pSupport;   /**< TRUE for bundles iw-ed with other PPP bundles, FALSE otherwise. */
} t_IwPppohtBundleParam;


/**************************************************************************//**
 @Function      IW_PPPoHT_BUNDLE_ConfigAndInit

 @Description   Configures an IW-PPPoHT Bundle module.
                This routine allocates and initializes the IW-PPPoHT Bundle
                driver structure.
 
 @Param[in]     p_IwPppohtBundleParam - A pointer to the initializing parameter
                                        structure for the IW-PPPoHT Bundle.

 @Return        A handle (descriptor) of the newly created IW-PPPoHT link structure.
*//***************************************************************************/
t_Handle                IW_PPPoHT_BUNDLE_ConfigAndInit(t_IwPppohtBundleParam *p_IwPppohtBundleParam);

/**************************************************************************//**
 @Function      IW_PPPoHT_BUNDLE_Free

 @Description   Free IW-PPPoHT Bundle resources.

 @Param[in]     h_IwPppohtBundle - A handle to a IW-PPPoHT Bundle device.

 @Return        None
*//***************************************************************************/
void                    IW_PPPoHT_BUNDLE_Free(t_Handle h_IwPppohtBundle);

/**************************************************************************//**
 @Function      IW_PPPoHT_BUNDLE_GetRegistrationCallback

 @Description   Return a callback for registering a PPPOHT Bundle module in
                the IW-PPPOHT Bundle
 
 @Param[in]     h_IwPppohtBundle - The IW-PPPoHT Bundle handle.

 @Return        The callback routine.
*//***************************************************************************/
t_IwRegistrationCallback *IW_PPPoHT_BUNDLE_GetRegistrationCallback(t_Handle h_IwPppohtBundle);

/**************************************************************************//**
 @Function      IW_PPPoHT_BUNDLE_GetTxQdIndex

 @Description   A routine for returning the TxQD index and memory location
                information for a given bundle class.

 @Param[in]     h_IwPppohtBundle -  The IW-PPPoHT Bundle handle.
 @Param[in]     queueIndex -        Class index
 @Param[out]    p_TxQdIndex -       QD index.
 @Param[out]    p_IsExtTxQd -       Internal/External memory indication.
 @Param[out]    p_IwPppohtDetails - Internal/External memory indication.
 @Param[out]    p_MccId           - MCC Id of the wrapped PPPoHT object

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     IW_PPPoHT_BUNDLE_GetTxQdIndex(t_Handle  h_IwPppohtBundle,
                                          uint8_t   classIndex,
                                          uint16_t  *p_TxQdIndex,
                                          bool      *p_IsExtTxQd,
                                          uint8_t   *p_MccId);

#endif /* __IW_PPPOHT_EXT_H */
