/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          mcc_ch_ext.h

 @Description   MCC Channel Application Programming Interface.

*//***************************************************************************/
#ifndef __MCC_CH_EXT_H
#define __MCC_CH_EXT_H


#include "error_ext.h"
#include "std_ext.h"



/**************************************************************************//**
 @Group         mcc_ch_grp MCC Channel Application Programming Interface

 @Description   MCC Channel API functions, definitions and enums

 @{
*//***************************************************************************/

#define MAX_NUM_OF_MCC_CHANNELS     256
#define MCC_RX_ALIGNMENT            8

/**************************************************************************//**
 @Group         mcc_ch_init_grp MCC Channel Initialization Unit

 @Description   MCC Channel initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

typedef void    (t_ExceptionsRoutine)(t_Handle, uint32_t, uint16_t);


#define TX_IRQ 6         /**< IRQ number of transmit             */

/**************************************************************************//**
 @Description   MCC event types
*//***************************************************************************/
typedef enum e_MccEvents
{
    e_MCC_EV_INT = 1,    /**< Global interrupt                   */
    e_MCC_EV_QOV,        /**< Queue overflow                     */
    e_MCC_EV_QOV_N_INT   /**< Global interrupt & queue overflow  */
} e_MccEvents;

/**************************************************************************//**
 @Description   MCC channel initialization structure, used when calling
                MCC_CH_Config().
*//***************************************************************************/
typedef struct t_MccChParam
{
    uint8_t     channelNum;     /**< The number of the MCC channel [0-255] */
    t_Handle    h_App;          /**< A Handle to the upper layer handler.  */
    t_Handle    h_Mcc;          /**< A handle to the MCC                   */
    uint16_t    rxBdRingSize;   /**< Size of Rx Bd Ring (0 uses for common)*/
    uint16_t    txBdRingSize;   /**< Size of Tx Bd Ring (0 uses for common)*/

    /**< Upper layer's routine - called to report MCC channel interrupts.  */
    void        (*f_Events) (t_Handle h_Arg, uint16_t status);
} t_MccChParam;


/**************************************************************************//**
 @Function      MCC_CH_ConfigAndInit

 @Description   Configures and initialize an MCC channel.
                This routine allocates the MCC channel structure, initializes
                the channel parameters at the parameter ram and the memory mapped
                registers.

 @Param[in]     p_MccChParam  - A pointer to the initializing parameter
                                     structure for the MCC channel.

 @Return        A handle of the newly created MCC channel structure.
*//***************************************************************************/
t_Handle MCC_CH_ConfigAndInit            (t_MccChParam *p_MccChParam);

/**************************************************************************//**
 @Function      MCC_CH_Free

 @Description   Frees an MCC channel and all its resources.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_Free                     (t_Handle h_Channel);

/** @} */ /* end of mcc_ch_init_grp group */


/**************************************************************************//**
 @Group         mcc_ch_control_grp MCC Channel Runtime Control Unit

 @Description   MCC Channel runtime control unit API functions, definitions
                and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_CH_GetBDsBase

 @Description   Get the base address of an MCC channels BDs.

 @Param[in]     h_Channel - A handle to MCC Channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetBDsBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetMrblr

 @Description   Get the max receive buffer length parameter.

 @Param[in]     h_Channel - A handle to MCC channel.

 @Return        mrblr.
*//***************************************************************************/
uint16_t MCC_CH_GetMrblr (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetFwExtStructsMemAttributes

 @Description   Get the memory attributes for MCC Channels firmware's external structures.

 @Param[in]     h_Channel - A handle to MCC Channel.

 @Return        Memory attributes for the firmware's external structures.
*//***************************************************************************/
uint32_t MCC_CH_GetFwExtStructsMemAttributes(t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetMaxRxIntrQId

 @Description   Get the maximal supported MCC Rx interrupt queue id.

 @Param[in]     h_Channel - A handle to MCC Channel.

 @Return        Maximal interrupt queue id (zero if none or error).
*//***************************************************************************/
uint8_t  MCC_CH_GetMaxRxIntrQId (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetRxBdRingBase

 @Description   Get the Rx BD ring base address of the specific MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetRxBdRingBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetTxBdRingBase

 @Description   Get the Tx BD ring base address of the specific MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetTxBdRingBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetRxBdRingLength

 @Description   Get the Rx BD ring length address of the specific MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetRxBdRingLength  (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetTxBdRingLength

 @Description   Get the Tx BD ring length address of the specific MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetTxBdRingLength (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetRxContextBase

 @Description   Get the RX ....

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetRxContextBase(t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetTxContextBase

 @Description   Get the TX ....

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetTxContextBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetPramBase

 @Description   Get the base address of an MCC channel parameter RAM.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetPramBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_GetExtraPramBase

 @Description   Get the base address of an MCC channel extra parameter RAM.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_CH_GetExtraPramBase (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_EnableIntr

 @Description   The mask of the given irq will be removed unless it was disabled
                more then once, in such case it should be enabled the amount of
                times it was disabled in order to receive interrupts once more.

 @Param[in]     h_Channel - A handle to an MCC channel.
 @Param[in]     irq       - Interrput number to be allocated.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_EnableIntr (t_Handle h_Channel, uint8_t irq);

/**************************************************************************//**
 @Function      MCC_CH_DisableIntr

 @Description   The given irq interrups will be masked until enabled.

 @Param[in]     h_Channel - A handle to an MCC channel.
 @Param[in]     irq       - Interrput number to be allocated.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_DisableIntr (t_Handle h_Channel, uint8_t irq);

/**************************************************************************//**
 @Function      MCC_CH_InitTx

 @Description   Initialize transmition on an MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_InitTx (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_InitRx

 @Description   Initialize reception on an MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_InitRx (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_StopTx

 @Description   Stop transmition on an MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_StopTx (t_Handle h_Channel);

/**************************************************************************//**
 @Function      MCC_CH_StopRx

 @Description   Stop reception on an MCC channel.

 @Param[in]     h_Channel - A handle to an MCC channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_CH_StopRx (t_Handle h_Channel);

/** @} */ /* end of mcc_ch_control_grp group */
/** @} */ /* end of mcc_ch_grp group */


#endif /* __MCC_CORE_EXT_H */
