/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          mcc_ext.h

 @Description   MCC Application Programming Interface.

*//***************************************************************************/
#ifndef __MCC_EXT_H
#define __MCC_EXT_H


#include "error_ext.h"
#include "std_ext.h"
#include "qe_ext.h"
#include "mcc_ch_ext.h"

/**************************************************************************//**
 @Group         mcc_grp MCC Application Programming Interface

 @Description   MCC API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mcc_init_grp MCC Initialization Unit

 @Description   MCC initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/

typedef t_Error (t_CheckConstraint)(t_Handle, uint8_t *);

/**************************************************************************//**
 @Description   MCC initialization structure, used when calling MCC_Config().
*//***************************************************************************/
typedef struct t_MccParam
{
    /* System integration parameters */
    uint32_t            baseAddress;        /**< Base address of MCC registers in memory. */
    uint8_t             mccNum;             /**< The number of the MCC block */
    t_Handle            h_Qe;               /**< Handle to the QE structure */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID */
    int                 irq;                /**< MCC interrupt ID (system-specific number) */
    bool                useDataManagment;   /**< Will the MCC manage data structures like BDs etc. */
 
    /* Application interface */
    t_Handle            h_App;              /**< Handle to the application */
    t_ExceptionsRoutine *f_Exceptions;      /**< Upper layer's routine - called to report
                                                 MCC interrupt events and errors */
} t_MccParam;


/**************************************************************************//**
 @Function      MCC_Config

 @Description   Configures an MCC device.
 
                This routine allocates the MCC structure and sets user
                and default parameters in the MCC driver parameters structure.

 @Param[in]     p_MccParam  - A pointer to the initializing parameter
                              structure for the MCC.

 @Return        A handle of the newly created MCC structure.
*//***************************************************************************/
t_Handle MCC_Config(t_MccParam *p_MccParam);

/**************************************************************************//**
 @Function      MCC_Init

 @Description   Initializes an MCC device.

                This routine initializes the MCC structure, the global
                parameters at the parameter RAM and the memory mapped
                MCC registers. It clears the super channel transmit table
                and the Interrupt tables. When done the routine unmasks
                MCC interrupts on this device.

 @Param[in]     h_Mcc - A handle to MCC.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_Init(t_Handle h_Mcc);

/**************************************************************************//**
 @Function      MCC_Free

 @Description   Frees all global MCC port resources.

 @Param[in]     h_Mcc - handle to MCC.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_Free(t_Handle h_Mcc);


/**************************************************************************//**
 @Group         mcc_advanced_init_grp MCC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_ConfigHighChannelNum

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                The highest channel number used by the MCC block, for all the MCC
                groups.
                NOTE - for each TDM you are using, the MCC driver must
                initialize at least 32 channels although you do not need them
                all.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigHighChannelNum (t_Handle h_Mcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_ConfigExtendedChannelsSupport

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                This function enable the Extended channels table with a high channel
                as requested. The Extended channel table is a channel table in which the
                size of each entry is extended. For example, when using the extended table
                user can use 256 SS7 channels (although each channel needs 128 bytes).

 @Param[in]     h_Mcc                   - A handle to MCC.
 @Param[in]     highExtendedChannelNum  - The highest channel which will be used in the extended
                                          table,.
 @Param[in]     supportNonExtendedCh    - Should the driver support non extended channels as well.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigExtendedChannelsSupport (t_Handle h_Mcc, uint8_t highExtendedChannelNum, bool supportNonExtendedCh);

/**************************************************************************//**
 @Function      MCC_ConfigHighChannelNumMem

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                The highest channel number used by the MCC block for which memory will
                be allocated, for all the MCC groups.
                NOTE - for each TDM you are using, the MCC driver must
                initialize at least 32 channels although you do not need them
                all.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigHighChannelNumMem (t_Handle h_Mcc, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_Mcc           - Handle to MCC object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_ConfigFirmwareExternalStructsMemory(t_Handle    h_Mcc,
                                                uint8_t     memId,
                                                uint32_t    memAttributes);


/**************************************************************************//**
 @Function      MCC_ConfigCommonBdRingLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Common BD ring length for all channels. [64]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigCommonBdRingLen (t_Handle h_Mcc, uint16_t newVal);

/**************************************************************************//**
 @Description   MCC bd ring pools structure, used when calling MCC_ConfigBdRingPools().
*//***************************************************************************/
typedef struct t_MccBdRingPoolsParam
{
    uint16_t     bdRingLength;     /**< The length of each bdRing          */
    uint16_t     sizeOfPool;       /**< The number of bdRings in each pool */
} t_MccBdRingPoolsParam;

/**************************************************************************//**
 @Function      MCC_ConfigRxBdRingPools

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Configure the length of each Rx bdRing and the number of Rx bdRings
                in each pool

 @Param[in]     h_Mcc               - A handle to MCC.
 @Param[in]     numOfPools          - How many pools to create.
 @Param[in]     bdRingPoolsParams   - An array of bdRing pools parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      - Change is applied only upon calling MCC_Init.
                - If Total num of bdRings will not sum to num of channels, a pool of
                  bdRings of the common length will be added to sum to this number.
*//***************************************************************************/
t_Error MCC_ConfigRxBdRingPools (t_Handle h_Mcc, uint8_t numOfPools, t_MccBdRingPoolsParam* bdRingPoolsParams);

/**************************************************************************//**
 @Function      MCC_ConfigTxBdRingPools

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Configure the length of each Tx bdRing and the number of Tx bdRings
                in each pool

 @Param[in]     h_Mcc               - A handle to MCC.
 @Param[in]     numOfPools          - How many pools to create.
 @Param[in]     bdRingPoolsParams   - An array of bdRing pools parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      - Change is applied only upon calling MCC_Init.
                - If Total num of bdRings will not sum to num of channels, a pool of
                  bdRings of the common length will be added to sum to this number.
*//***************************************************************************/
t_Error MCC_ConfigTxBdRingPools (t_Handle h_Mcc, uint8_t numOfPools, t_MccBdRingPoolsParam* bdRingPoolsParams);

/**************************************************************************//**
 @Function      MCC_ConfigIntrQueueSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Size of the Interrupt tables. [512]

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     queueNum    - the interupt queue which size is being changed.
 @Param[in]     newVal      - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigIntrQueueSize (t_Handle h_Mcc, uint8_t queueNum, uint32_t newVal);

/**************************************************************************//**
 @Function      MCC_ConfigMaxRxBufLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                maximum receive buffer length. [64]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigMaxRxBufLen (t_Handle h_Mcc, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_ConfigRxFramesThreshold

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                received frames threshold value. [1]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigRxFramesThreshold (t_Handle h_Mcc, uint32_t newVal);

/**************************************************************************//**
 @Function      MCC_ConfigChannelUnderrun

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                received channel underrun indication. [TRUE]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigChannelUnderrun(t_Handle h_Mcc, bool newVal);

/**************************************************************************//**
 @Function      MCC_ConfigUseMiddleHostCommand

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Use the middle host-command for initializing Rx and Tx channel. [TRUE]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_Init.
*//***************************************************************************/
t_Error MCC_ConfigUseMiddleHostCommand(t_Handle h_Mcc, bool newVal);

/**************************************************************************//**
 @Function      MCC_ConfigNumOfRxIntrQueues

 @Description   Number of receive interrupts queues (1-4).
 
                No real priority is implemented by this division. First
                queues will be handled before others, but all Rx related
                interrupts will be handled before first queue will be
                handled again. [1]

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling the channel Init routine.
*//***************************************************************************/
t_Error MCC_ConfigNumOfRxIntrQueues(t_Handle h_Mcc, uint8_t newVal);

/** @} */ /* end of mcc_advanced_init_grp group */
/** @} */ /* end of mcc_init_grp group */


/**************************************************************************//**
 @Group         mcc_control_grp MCC Runtime Control Unit

 @Description   MCC runtime control unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_GetPramBase

 @Description   Get the base address of an MCC global parameter RAM.

 @Param[in]     h_Mcc - A handle to MCC.

 @Return        base address.
*//***************************************************************************/
uint32_t MCC_GetPramBase (t_Handle h_Mcc);

/**************************************************************************//**
 @Function      MCC_GetPramPartitionId

 @Description   Get the memory partition ID used for allocating the MCC
                parameter RAM.

 @Param[in]     h_Mcc - A handle to MCC.

 @Return        Memory partition ID of the parameter RAM.
*//***************************************************************************/
uint8_t MCC_GetPramPartitionId(t_Handle h_Mcc);

/**************************************************************************//**
 @Function      MCC_AllocIntr

 @Description   Allocate interrupt resource of the MCC.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     f_Isr   - Interrupt service callback routine to be called
                          when the interrupt occurs.
 @Param[in]     h_Arg   - An argument to pass to the f_Isr.
 @Param[in]     events  - Enum that represents the events to be
                          listened (interrupt, queue-overflow).

 @Return        Interrpt number on success; (-1) otherwise.
*//***************************************************************************/
int      MCC_AllocIntr (t_Handle     h_Mcc,
                        void         (*f_Isr) (t_Handle h_Arg, int irq, e_MccEvents events),
                        t_Handle     h_Arg,
                        e_MccEvents  events);

/**************************************************************************//**
 @Function      MCC_AllocIntrForce

 @Description   Try to allocate/occupy specific interrupt resource of the MCC.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     f_Isr   - Interrupt service callback routine to be called
                          when the interrupt occurs.
 @Param[in]     h_Arg   - An argument to pass to the f_Isr.
 @Param[in]     events  - Enum that represent the events to be
                          listened (interrupt, queue-overflow).
 @Param[in]     irq     - Interrput number to be allocated.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocIntrForce (t_Handle     h_Mcc,
                             void         (*f_Isr) (t_Handle h_Arg, int irq, e_MccEvents events),
                             t_Handle     h_Arg,
                             e_MccEvents  events,
                             int          irq);

/**************************************************************************//**
 @Function      MCC_FreeIntr

 @Description   De-allocate interrupt resource of the MCC.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     irq     - Interrput number to be free.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_FreeIntr (t_Handle     h_Mcc, int irq);

/**************************************************************************//**
 @Function      MCC_EnableIntr

 @Description   The mask of the givven irq will be removed unless it was diabled
                more then once, in such case it should be enabled the amount of
                times it was diabled in order to receive interrupts once more.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     irq     - Interrput number to be allocated.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_EnableIntr (t_Handle h_Mcc, uint8_t irq);

/**************************************************************************//**
 @Function      MCC_DisableIntr

 @Description   The givven irq interrups will be masked until enabled.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     irq     - Interrput number to be allocated.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_DisableIntr (t_Handle h_Mcc, uint8_t irq);

/**************************************************************************//**
 @Function      MCC_RegisterToSuperChannel

 @Description   Register a virtual MCC channel to a specific MCC super-channel.

 @Param[in]     h_Mcc             - A handle to MCC.
 @Param[in]     virtualChannelNum - A handle to an MCC virtual channel.
 @Param[in]     superChannelNum   - A handle to an MCC super-channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_RegisterToSuperChannel (t_Handle h_Mcc, uint8_t virtualChannelNum, uint8_t superChannelNum);

/**************************************************************************//**
 @Function      MCC_UnRegisterFromSuperChannel

 @Description   De-Register a virtual MCC channel from its MCC super-channel.
                The virtual channel will become a regular channel.

 @Param[in]     h_Mcc             - A handle to MCC.
 @Param[in]     virtualChannelNum - A handle to an MCC virtual channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_UnRegisterFromSuperChannel (t_Handle h_Mcc, uint8_t virtualChannelNum);

/**************************************************************************//**
 @Function      MCC_AllocChannel

 @Description   Allocate one channel of the MCC. The routine is searching
                for a free, un-occupied channel from the TDM requires by the user.
                The channel number can be in the range of 0 up to 'highChannelNum'.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     tdmId   - From which TDM to allocate the channels.

 @Return        Channel number on success; (-1) otherwise.
*//***************************************************************************/
int      MCC_AllocChannel               (t_Handle h_Mcc, e_Tdm tdmId);

/**************************************************************************//**
 @Function      MCC_AllocChannelsByConstraint

 @Description   Allocate a group of channels of the MCC. The routine is searching
                for a free, un-occupied channels from the TDM requires by the user.
                The channel numbers can be in the range of 0 up to 'highChannelNum'.
                Two channel numbers are successors only if they are from the same
                MCC group. Therefore, the 'f_CheckConstraint' should check each
                channel number.

 @Param[in]     h_Mcc             - A handle to MCC.
 @Param[in]     f_CheckConstraint - Interrput number to be free.
 @Param[in]     h_Arg             - A handle to pass to the f_CheckConstraint.
 @Param[in]     tdmId             - From which TDM to allocate the channels.
 @Param[in]     numOfChannels     - Number of channels to allocate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocChannelsByConstraint   (t_Handle          h_Mcc,
                                          t_CheckConstraint *f_CheckConstraint,
                                          t_Handle          h_Arg,
                                          e_Tdm             tdmId,
                                          uint16_t          numOfChannels);

/**************************************************************************//**
 @Function      MCC_AllocChannelsMemory

 @Description   This function works just like MCC_AllocChannelsByContrain but it
                allocates only the channeles memory. The driver will try to match
                first channels whom physical part is already allocated.
                It is up to the users constraint function to save the channels numbers.

 @Param[in]     h_Mcc             - A handle to MCC.
 @Param[in]     f_CheckConstraint - Interrput number to be free.
 @Param[in]     h_Arg             - A handle to pass to the f_CheckConstraint.
 @Param[in]     tdmId             - From which TDM to allocate the channels.
 @Param[in]     numOfChannels     - Number of channels to allocate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocChannelsMemory         (t_Handle          h_Mcc,
                                          t_CheckConstraint *f_CheckConstraint,
                                          t_Handle          h_Arg,
                                          e_Tdm             tdmId,
                                          uint16_t          numOfChannels);

/**************************************************************************//**
 @Function      MCC_AllocPhysicalChannels

 @Description   This function works just like MCC_AllocChannelsByContrain but it
                allocates only the channeles physical part, this means that the
                channels will only be able to be used as virtual channels attached
                to a super channel.
                The driver will try to match first channels whom memory is already allocated.
                It is up to the users constraint function to save the channels numbers.

 @Param[in]     h_Mcc             - A handle to MCC.
 @Param[in]     f_CheckConstraint - Interrput number to be free.
 @Param[in]     h_Arg             - A handle to pass to the f_CheckConstraint.
 @Param[in]     tdmId             - From which TDM to allocate the channels.
 @Param[in]     numOfChannels     - Number of channels to allocate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocPhysicalChannels       (t_Handle          h_Mcc,
                                          t_CheckConstraint *f_CheckConstraint,
                                          t_Handle          h_Arg,
                                          e_Tdm             tdmId,
                                          uint16_t          numOfChannels);

/**************************************************************************//**
 @Function      MCC_AllocChannelForce

 @Description   Try to allocate/occupy a specific channel of the MCC. The routine is
                checking if this channel is free/un-occupied.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be occupied; The channel
                              numbers can be in the range of 0 up to 'highChannelNum'.
 @Param[in]     tdmId       - From which TDM to allocate the channels.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocChannelForce           (t_Handle h_Mcc, uint8_t channelNum, e_Tdm tdmId);

/**************************************************************************//**
 @Function      MCC_AllocPhysicalChannelForce

 @Description   TBD.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be occupied; The channel
                              numbers can be in the range of 0 up to 'highChannelNum'.
 @Param[in]     tdmId       - From which TDM to allocate the channels.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  MCC_AllocPhysicalChannelForce   (t_Handle h_Mcc, uint8_t channelNum, e_Tdm tdmId);

/**************************************************************************//**
 @Function      MCC_AllocExtendedChannel

 @Description   Allocate one extended channel of the MCC. The routine is searching
                for a free, un-occupied channel from the TDM requires by the user.
                The channel number can be in the range of 0 up to 'highExtendedChannelNum'.

 @Param[in]     h_Mcc   - A handle to MCC.
 @Param[in]     tdmId   - From which TDM to allocate the channels.

 @Return        Channel number on success; (-1) otherwise.
*//***************************************************************************/
int MCC_AllocExtendedChannel (t_Handle h_Mcc, e_Tdm tdmId);

/**************************************************************************//**
 @Function      MCC_AllocExtendedChannelForce

 @Description   Try to allocate/occupy a specific extended channel of the MCC. The routine is
                checking if this channel is free/un-occupied.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be occupied; The channel
                              numbers can be in the range of 0 up to 'highExtendedChannelNum'.
 @Param[in]     tdmId       - From which TDM to allocate the channels.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_AllocExtendedChannelForce (t_Handle h_Mcc, uint8_t channelNum, e_Tdm tdmId);

/**************************************************************************//**
 @Function      MCC_FreeChannel

 @Description   Free a specific channel of the MCC that was allocated/occupied.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be freed; The channel
                              numbers can be in the range of 0 up to 'highChannelNum'.
*//***************************************************************************/
void     MCC_FreeChannel                (t_Handle h_Mcc, uint8_t channelNum);

/**************************************************************************//**
 @Function      MCC_FreeChannelMem

 @Description   Free the memory of a specific channel of the MCC that was allocated/occupied.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be freed; The channel
                              numbers can be in the range of 0 up to 'highChannelNum'.
*//***************************************************************************/
void     MCC_FreeChannelMem                (t_Handle h_Mcc, uint8_t channelNum);

/**************************************************************************//**
 @Function      MCC_FreeChannelPhy

 @Description   Free onlt the phtsical port of a specific channel of the MCC.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be freed; The channel
                              numbers can be in the range of 0 up to 'highChannelNum'.
*//***************************************************************************/
void     MCC_FreeChannelPhy                (t_Handle h_Mcc, uint8_t channelNum);

/**************************************************************************//**
 @Function      MCC_InitChannelTx

 @Description   TBD.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be Tx initialized;
                              The channel numbers can be in the range of 0 up
                              to 'highChannelNum'.
*//***************************************************************************/
t_Error  MCC_InitChannelTx (t_Handle h_Mcc, uint8_t channelNum);

/**************************************************************************//**
 @Function      MCC_StopChannelTx

 @Description   TBD.

 @Param[in]     h_Mcc       - A handle to MCC.
 @Param[in]     channelNum  - Channel number to be Tx stopped;
                              The channel numbers can be in the range of 0 up
                              to 'highChannelNum'.
*//***************************************************************************/
t_Error  MCC_StopChannelTx (t_Handle h_Mcc, uint8_t channelNum);

/**************************************************************************//**
 @Function      MCC_GetId

 @Description   Returns the Id of the MCC handle.

 @Param[in]     h_Mcc       - A handle to MCC.
 
 @Return        MCC_Id when successful 0xFF for failure.
*//***************************************************************************/
uint8_t  MCC_GetId (t_Handle h_Mcc);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      MCC_DumpRegs

 @Description   Dump (print) the MCC registers.

 @Param[in]     h_Mcc - A handle to MCC.
*//***************************************************************************/
void     MCC_DumpRegs(t_Handle h_Mcc);

#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of mcc_control_grp group */

/** @} */ /* end of mcc_grp group */

#endif /* __MCC_EXT_H */
