/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mcc_hdlc_ext.h

 @Description   MCC HDLC Channel Application Programming Interface
*//***************************************************************************/
#ifndef __MCC_HDLC_EXT_H
#define __MCC_HDLC_EXT_H

#include "std_ext.h"
#include "mcc_ch_ext.h"


/**************************************************************************//**
 @Group         mcc_hdlc_ch_grp    MCC HDLC Channel Application Programming Interface

 @Description   MCC HDLC channel API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mcc_hdlc_ch_init_grp    MCC HDLC Channel Initialization Unit

 @Description   MCC HDLC channel initialization unit API functions, definitions
                and enums.

 @{
*//***************************************************************************/

typedef enum e_MccHdlcChInterrupts
{
    e_MCC_HDLC_CH_NO_INTERRUPTS = 0,
    e_MCC_HDLC_CH_RX_INTR,
    e_MCC_HDLC_CH_TX_INTR,
    e_MCC_HDLC_CH_RX_N_TX_INTR
} e_MccHdlcChInterrupts;

#define MCC_HDLC_CH_EX_UN       0x0200  /**< Underrun event. This event is unmaskable. */
#define MCC_HDLC_CH_EX_NID      0x0020  /**< Non-idle event                            */
#define MCC_HDLC_CH_EX_IDLE     0x0010  /**< Idle event                                */
#define MCC_HDLC_CH_EX_MRF      0x0008  /**< Maximum frame length violation event      */
#define MCC_HDLC_CH_EX_BSY      0x0002  /**< Busy event. This event is unmaskable.     */

/**************************************************************************//**
 @Description   MCC HDLC channel initialization structure,
                used when calling MCC_HDLC_CH_Config().
*//***************************************************************************/
typedef struct t_MccHdlcChParam
{
    /* Channel parameters */
    uint8_t             channelNum;         /**< The channel number.                        */
 
    /* System integration parameters */
    t_Handle            h_Qe;               /**< A descriptor of the relevant QE module.    */
    t_Handle            h_Mcc;              /**< A descriptor of the relevant MCC module.   */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information               */
    uint32_t            dataMemAttributes;  /**< Memory attributes for data buffers         */

    /* Application interface */
    t_Handle            h_App;              /**< A handle to the application.               */
    t_RxStoreFunction   *f_RxStore;         /**< Upper layer's routine - called to pass a
                                                 received frame to the application.         */
    t_TxConfFunction    *f_TxConf;          /**< Upper layer's routine - called to handle
                                                 transmitted frames may be set to NULL to
                                                 avoid unwanted routine calls.              */
    void                (*f_Exceptions)(t_Handle h_App, uint32_t globalEvent, uint16_t channelEvent);
                                            /**< Upper layer's routine - called to report
                                                 interrupt events and errors.               */

} t_MccHdlcChParam;


/**************************************************************************//**
 @Function      MCC_HDLC_CH_Config

 @Description   Configures an MCC HDLC channel.
 
                This routine allocates the MCC channel structure (for HDLC mode)
                and sets user and default parameters in the MCC HDLC channel
                driver parameters structure.

 @Param[in]     p_MccHdlcChParam - A handle to the an MCC HDLC channel
                                   initialization structure as given by the user.

 @Return        A handle of the newly created MCC HDLC channel structure.
*//***************************************************************************/
t_Handle MCC_HDLC_CH_Config(t_MccHdlcChParam *p_MccHdlcChParam);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_Init

 @Description   Initializes the MCC HDLC channel.
 
                This function initializes all HDLC specific parameters - dual
                port RAM and the driver's structure. It then calls a routine
                for the initialization of the general MCC channel parameters.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_Init(t_Handle h_MccHdlcCh);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_Free

 @Description   Free all resources.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        None.
*//***************************************************************************/
void MCC_HDLC_CH_Free(t_Handle h_MccHdlcCh);


/**************************************************************************//**
 @Group         mcc_hdlc_ch_advanced_init_grp MCC HDLC Channel Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigMaxFrameLength

 @Description   Configures the maximum frame length. [0x400]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigMaxFrameLength (t_Handle h_MccHdlcCh, uint16_t newVal);


/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigCrcSize

 @Description   Configures the CRC length in bytes (2,4). [2]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigCrcSize (t_Handle h_MccHdlcCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigInterrupts

 @Description   Configures interrupts mode. [e_MCC_HDLC_CH_RX_INTR]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigInterrupts(t_Handle h_MccHdlcCh, e_MccHdlcChInterrupts newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigTxBdRingSize

 @Description   Configures the size of the Tx BD ring pool. [0]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigTxBdRingSize(t_Handle h_MccHdlcCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigRxBdRingSize

 @Description   Configures the size of the Rx BD ring pool. [0]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigRxBdRingSize(t_Handle h_MccHdlcCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigExceptions

 @Description   Configures the enabled exceptions. [MCC_HDLC_CH_EX_MRF]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigExceptions(t_Handle h_MccHdlcCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigTimeStampInsert

 @Description   Configures time-stamp insertion. [FALSE]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigTimeStampInsert (t_Handle h_MccHdlcCh, bool newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigRxIntrQueue

 @Description   Configures the Rx interrupt queue index. [0]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigRxIntrQueue (t_Handle h_MccHdlcCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigNumOfFlags

 @Description   Configures the number of flags between frames. [0]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigNumOfFlags (t_Handle h_MccHdlcCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigNumOfPadChars

 @Description   Configures the number of pad charachters. [0]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigNumOfPadChars (t_Handle h_MccHdlcCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigIdleMode

 @Description   Configures the idle pattern between frames. [TRUE]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - FALSE - no idle pattern between frames;
                              TRUE  - at least 1 idle sent between frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigIdleMode (t_Handle h_MccHdlcCh, bool newVal);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_ConfigReverseBitOrder

 @Description   Configures reverse bit order. [FALSE]

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     newVal      - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_HDLC_CH_Init.
*//***************************************************************************/
t_Error MCC_HDLC_CH_ConfigReverseBitOrder (t_Handle h_MccHdlcCh, bool newVal);

/** @} */ /* end of mcc_hdlc_ch_advanced_init_grp group */
/** @} */ /* end of mcc_hdlc_ch_init_grp group */


/**************************************************************************//**
 @Group         mcc_hdlc_ch_runtime_grp MCC HDLC Channel Runtime Unit

 @Description   MCC HDLC channel runtime unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define MCC_HDLC_CH_EX_RX_LG    0x0020          /**< Frame length violation */
#define MCC_HDLC_CH_EX_RX_NO    0x0010          /**< Rx Non Octet Aligned Packet */
#define MCC_HDLC_CH_EX_RX_AB    0x0008          /**< Frame Aborted */
#define MCC_HDLC_CH_EX_RX_CR    0x0004          /**< CRC Error */

#ifndef VERIFICATION_PASS_ERR_FRAME

#define MCC_HDLC_CH_EX_RX_FATAL_ERROR      ( MCC_HDLC_CH_EX_RX_AB | MCC_HDLC_CH_EX_RX_CR  | MCC_HDLC_CH_EX_RX_NO )
#define MCC_HDLC_CH_EX_RX_REPORTED_ERROR   MCC_HDLC_CH_EX_RX_LG

#else

#define MCC_HDLC_CH_EX_RX_FATAL_ERROR      0
#define MCC_HDLC_CH_EX_RX_REPORTED_ERROR   ( MCC_HDLC_CH_EX_RX_AB | MCC_HDLC_CH_EX_RX_CR  | MCC_HDLC_CH_EX_RX_NO | MCC_HDLC_CH_EX_RX_LG)

#endif /* VERIFICATION_PASS_ERR_FRAME */


/**************************************************************************//**
 @Function      MCC_HDLC_CH_Tx

 @Description   MCC HDLC channel transmit routine.
 
                The application calls this function for transmitting a
                HDLC data buffer on a specified MCC channel.
                The buffer is loaded to the next available BD.

 @Param[in]     h_MccHdlcCh     - A handle to an MCC HDLC Channel.
 @Param[in]     p_Data          - The address of the data for sending.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length          - The number of octets for transmitting.
 @Param[in]     lastUserBuffer  - Indicates the last buffer of a frame.


 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_Tx(t_Handle h_MccHdlcCh,
                       uint8_t  *p_Data,
                       t_Handle h_BufContext,
                       uint32_t length,
                       bool     lastUserBuffer);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_Rx

 @Description   MCC HDLC channel receive routine.
 
                This function is called either by the driver  or by
                the user depending on the operation mode. In
                interrupts mode the routine may be called directly from
                the Rx interrupt handler, or by the user (depending on
                'intrManagerId' parameter. In polling mode, the routine must
                be explicitly called by the user.
                It is up to the user to reassemble
                the HDLC frame the routine will invoke the users callback on each received buffer.
                It will do so for all incomming buffers unless the callback indicates that it wishes
                to pause reception.


 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_Rx(t_Handle h_MccHdlcCh);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_TxConf

 @Description   MCC HDLC channel function for forced transmit confirmation.
 
                This routine collects the transmitted frames/buffers and
                returns them to the application. It also prepares the transmit
                BDs for new frames. For each frame/buffer (depending on the
                data exchange mode) the application callback routine will be
                called (f_TxConf in t_MccHdlcChParam).
 
                Normally, transmit confirmation is done automatically by the
                transmit routine. This routine can be called after transmission
                was stopped, to collect all remaining frames/buffers from the
                BD ring.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        None.
*//***************************************************************************/
void MCC_HDLC_CH_TxConf(t_Handle h_MccHdlcCh);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_TxClean

 @Description   MCC HDLC channel function for cleannig all the BDs.
 
                This routine clean all the buffers in the BD ring and send it to
                the user using the callback given.
                It is used uppon channel underrun.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        None.
*//***************************************************************************/
void MCC_HDLC_CH_TxClean(t_Handle h_MccHdlcCh);

/** @} */ /* end of mcc_hdlc_ch_runtime_grp group */


/**************************************************************************//**
 @Group         mcc_hdlc_ch_control_grp MCC HDLC Channel Control Unit

 @Description   MCC HDLC channel CONTROL functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_HDLC_CH_Enable

 @Description   Enable an MCC HDLC channel.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     dir         - A direction (RX/TX) to enable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_Enable(t_Handle h_MccHdlcCh, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_Disable

 @Description   Disable an MCC HDLC channel.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
 @Param[in]     dir         - A direction (RX/TX) to disable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_Disable(t_Handle h_MccHdlcCh, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_EnableIntr

 @Description   Enable the intrupts of an MCC HDLC channel.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_EnableIntr(t_Handle h_MccHdlcCh);

/**************************************************************************//**
 @Function      MCC_HDLC_CH_DisableIntr

 @Description   Disable the intrupts of an MCC HDLC channel.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_HDLC_CH_DisableIntr(t_Handle h_MccHdlcCh);


/**************************************************************************//**
 @Function      MCC_HDLC_CH_DumpRegs

 @Description   Dump (print) an MCC HDLC channel registers.

 @Param[in]     h_MccHdlcCh - A handle to an MCC HDLC Channel.
*//***************************************************************************/
void MCC_HDLC_CH_DumpRegs(t_Handle h_MccHdlcCh);

/** @} */ /* end of mcc_hdlc_ch_control_grp group */
/** @} */ /* end of mcc_hdlc_ch_grp group */


#endif /* __MCC_HDLC_EXT_H */
