/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mcc_ss7_ext.h

 @Description   MCC SS7 Channel Application Programming Interface
*//***************************************************************************/
#ifndef __MCC_SS7_EXT_H
#define __MCC_SS7_EXT_H

#include "std_ext.h"
#include "mcc_ext.h"


/**************************************************************************//**
 @Group         mcc_ss7_ch_grp    MCC SS7 channel Application Programming Interface

 @Description   MCC SS7 channel API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mcc_ss7_ch_init_grp    MCC SS7 channel Initialization Unit

 @Description   MCC SS7 channel initialization unit API functions, definitions
                and enums.

 @{
*//***************************************************************************/

typedef enum e_MccSS7ChInterrupts
{
    e_MCC_SS7_CH_NO_INTERRUPTS = 0,
    e_MCC_SS7_CH_RX_INTR,
    e_MCC_SS7_CH_TX_INTR,
    e_MCC_SS7_CH_RX_N_TX_INTR
} e_MccSS7ChInterrupts;


typedef enum e_MccSS7ChStandard
{
    e_MCC_SS7_CH_STANDARD_ITU_T,
    e_MCC_SS7_CH_STANDARD_JAPANESE,
    e_MCC_SS7_CH_STANDARD_CHINESE
}   e_MccSS7ChStandard;
 
typedef enum
{
    e_MCC_SS7_CH_MODE_SS7,
    e_MCC_SS7_CH_MODE_ESS7
}e_MccSS7ChMode;

typedef enum e_MccSS7ChBusyTxOp
{
    e_MCC_SS7_CH_BUSY_TX_SEND_FLAGS,
    e_MCC_SS7_CH_BUSY_TX_SEND_FISU
} e_MccSS7ChBusyTxOp;


typedef enum e_MccSS7ChErm
{
    e_MCC_SS7_CH_NO_ERM,
    e_MCC_SS7_CH_AERM,
    e_MCC_SS7_CH_AERM_EMERGENCY,
    e_MCC_SS7_CH_SUERM,
    e_MCC_SS7_CH_EIM
}   e_MccSS7ChErm;


#define MCC_SS7_CH_EX_OCT   0x2000  /**< N octets recieved in octet counting mode. */
#define MCC_SS7_CH_EX_SUERM 0x1000  /**< SU error monitor threshold reached.    */
#define MCC_SS7_CH_EX_FISU  0x0800  /**< FISU transmition started.              */
#define MCC_SS7_CH_EX_UN    0x0200  /**< Underrun event. This event is unmaskable.      */
#define MCC_SS7_CH_EX_AERM  0x0040  /**< Alignment errorrate monitor threshold reached. */
#define MCC_SS7_CH_EX_NID   0x0020  /**< Non-idle event                          */
#define MCC_SS7_CH_EX_IDLE  0x0010  /**< Idle event                              */
#define MCC_SS7_CH_EX_MRF   0x0008  /**< Maximum frame length violation event    */
#define MCC_SS7_CH_EX_BSY   0x0002  /**< Busy event. This event is unmaskable.   */

/**************************************************************************//**
 @Description   MCC SS7 channel initialization structure,
                used when calling MCC_SS7_CH_Config().
*//***************************************************************************/
typedef struct t_MccSS7ChParam
{
    /* Channel parameters */
    uint8_t             channelNum;         /**< The channel number.                        */

    /* System integration parameters */
    t_Handle            h_Qe;               /**< A descriptor of the relevant QE module.    */
    t_Handle            h_Mcc;              /**< A descriptor of the relevant MCC module.   */
    uint32_t            dataMemAttributes;  /**< Memory attributes for data buffers         */

    /* Application interface */
    t_Handle            h_App;              /**< A handle to the application.               */
    t_RxStoreFunction   *f_RxStore;         /**< Upper layer's routine - called to pass a
                                                 received frame to the application.         */
    t_TxConfFunction    *f_TxConf;          /**< Upper layer's routine - called to handle
                                                 transmitted frames may be set to NULL to
                                                 avoid unwanted routine calls.              */
    void                (*f_Exceptions)(t_Handle h_App, uint32_t globalEvent, uint16_t channelEvent);
                                            /**< Upper layer's routine - called to report
                                                 interrupt events and errors.               */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information               */

} t_MccSS7ChParam;


/**************************************************************************//**
 @Function      MCC_SS7_CH_Config

 @Description   Configures an MCC SS7 channel.
 
                This routine allocates the MCC channel structure (for SS7 mode)
                and sets user and default parameters in the MCC SS7 channel
                driver parameters structure.

 @Param[in]     p_MccSS7ChParam - A handle to the an MCC SS7 channel
                                       initialization structure as given by the user.

 @Return        A handle of the newly created MCC SS7 channel structure.
*//***************************************************************************/
t_Handle MCC_SS7_CH_Config(t_MccSS7ChParam *p_MccSS7ChParam);

/**************************************************************************//**
 @Function      MCC_SS7_CH_Init

 @Description   Initializes the MCC SS7 channel.
 
                The function initializes all SS7 specific parameters - dual port
                RAM and the driver's structure. It then calls a routine
                for the initialization of the general MCC channel parameters.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_Init(t_Handle h_MccSS7Ch);

/**************************************************************************//**
 @Function      MCC_SS7_CH_Free

 @Description   Free all resources.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        None.
*//***************************************************************************/
void MCC_SS7_CH_Free(t_Handle h_MccSS7Ch);


/**************************************************************************//**
 @Group         mcc_ss7_ch_advanced_init_grp MCC SS7 channel Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigMaxFrameLength

 @Description   Configures the maximum frame length. [288]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigMaxFrameLength (t_Handle h_MccSS7Ch, uint16_t newVal);


/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigInterrupts

 @Description   Configures interrupt mode. [e_MCC_SS7_CH_RX_INTR]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigInterrupts(t_Handle h_MccSS7Ch, e_MccSS7ChInterrupts newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigExceptions

 @Description   Configures the enabled exceptions. [MCC_SS7_CH_EX_OCT]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigExceptions(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigTimeStampInsert

 @Description   Configures time-stamp insertion. [FALSE]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigTimeStampInsert (t_Handle h_MccSS7Ch, bool newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigRxIntrQueue

 @Description   Configures the Rx interrupt queue index. [0]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigRxIntrQueue (t_Handle h_MccSS7Ch, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigNumOfFlags

 @Description   Configures the number of flags between frames. [0]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigNumOfFlags (t_Handle h_MccSS7Ch, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigNumOfPadChars

 @Description   Configures the number of pad charachters. [0]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigNumOfPadChars (t_Handle h_MccSS7Ch, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigIdleMode

 @Description   Configures the idle pattern between frames. [TRUE]

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     newVal     - FALSE - no idle pattern between frames;
                             TRUE  - At least 1 idle sent between frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigIdleMode (t_Handle h_MccSS7Ch, bool newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigEffectiveStandard

 @Description   Configures the effective SS7 standard. [e_MCC_SS7_CH_STANDARD_ITU_T]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - Requested SS7 standard (ITU-T or Japanese).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigEffectiveStandard(t_Handle h_MccSS7Ch,
                                           e_MccSS7ChStandard newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigShortFramesDiscard

 @Description   Configures short frames discard mode. [TRUE]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - do not discard short frames;
                             TRUE  - discard short frames.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigShortFramesDiscard(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigSignalUnitFiltering

 @Description   Configures the signal unit filtering mode. [FALSE]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - disable signal unit filtering;
                             TRUE - enable signal unit filtering.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigSignalUnitFiltering(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigSignalUnitFilteringMask

 @Description   Configures the signal unit filtering mask. [0]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     p_Mask     - Pointer to 5 bytes of filtering mask.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigSignalUnitFilteringMask(t_Handle h_MccSS7Ch,
                                                 uint8_t  *p_Mask);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigBusyTxOperation

 @Description   Configures the operation mode for busy transmitter. [e_MCC_SS7_CH_BUSY_TX_SEND_FLAGS]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - Operation to perform while transmit buffers are
                             not ready.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigBusyTxOperation(t_Handle h_MccSS7Ch,
                                         e_MccSS7ChBusyTxOp newVal);


/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigOctetCountingOnOverrun

 @Description   Configures the octet counting mode on overrun condition. [TRUE]
 
                This setting is ignored in Japanese mode.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - disable OCM on overrun condition;
                             TRUE  - enable OCM on overrun condition.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigOctetCountingOnOverrun(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigOctetCountingForITU

 @Description   Configures the octet counting mode on ITU-T conditions. [TRUE]
 
                This setting is ignored in Japanese mode.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - disable OCM on ITU-T conditions;
                             TRUE  - enable OCM on ITU-T conditions.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigOctetCountingForITU(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigInitialErrorRateMonitoring

 @Description   Configures the error rate monitoring mode. [e_MCC_SS7_CH_NO_ERM]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - Requested Error Rate Monitoring Mode.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigInitialErrorRateMonitoring(t_Handle      h_MccSS7Ch,
                                                    e_MccSS7ChErm newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigN

 @Description   Configures the N value (increment threshold for SUERM and AERM (ITU only)
                in Octet Counting Mode). [16]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for N.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigN(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigD

 @Description   Configures the D value (the signal units to signal units error ratio).
                [ITU: 256, Japanese: 16]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for D.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigD(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigM

 @Description   Configures the M value (AERM interrupt threshold, ITU-T mode only).
                [5]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for M.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigM(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigT_Suerm

 @Description   Configures the T value (interrupt threshold value for SUERM).
                [ITU: 64, Japanese: 285]

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for T.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigT_Suerm(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigT_AermITU

 @Description   Changes the T value for AERM in ITU-T mode from its default
                configuration [4].
 
                For AERM in ITU-T mode, T is the increment threshold value.

 @Param[in]     h_MccSS7Ch  - Handle to the MCC SS7 channel.
 @Param[in]     newVal      - New value for T.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigT_AermITU(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigU

 @Description   Configures the value for the down counter for EIM in GB only.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for U.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigU(t_Handle h_MccSS7Ch, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigExtendedSequenceNumber

 @Description   Enables/disables the Extended Sequence Number feature of
                ESS7.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled - FALSE - disable Extended Sequence Number;
                          TRUE - enable Extended Sequence Number.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigExtendedSequenceNumber(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ConfigChannelMode

 @Description   Selects the channel mode (SS7 or ESS7).

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     newVal     - New value for mode.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Config()
                and before calling MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ConfigChannelMode(t_Handle h_MccSS7Ch, e_MccSS7ChMode newVal);

/** @} */ /* end of mcc_ss7_ch_advanced_init_grp group */
/** @} */ /* end of mcc_ss7_ch_init_grp group */


/**************************************************************************//**
 @Group         mcc_ss7_ch_runtime_grp MCC SS7 channel Runtime Unit

 @Description   MCC SS7 channel runtime unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define MCC_SS7_CH_EX_RX_LG    0x0020          /* Frame length violation      */
#define MCC_SS7_CH_EX_RX_NO    0x0010          /* Rx Non Octet Aligned Packet */
#define MCC_SS7_CH_EX_RX_AB    0x0008          /* Frame Aborted               */
#define MCC_SS7_CH_EX_RX_CR    0x0004          /* CRC Error                   */
#define MCC_SS7_CH_EX_RX_SF    0x0004          /* Short Frame Error           */

#ifndef VERIFICATION_PASS_ERR_FRAME
#define MCC_SS7_CH_EX_RX_FATAL_ERROR      (MCC_SS7_CH_EX_RX_AB | MCC_SS7_CH_EX_RX_CR  | MCC_SS7_CH_EX_RX_NO )
#define MCC_SS7_CH_EX_RX_REPORTED_ERROR   (MCC_SS7_CH_EX_RX_LG | MCC_SS7_CH_EX_RX_SF)
#else
#define MCC_SS7_CH_EX_RX_FATAL_ERROR      0
#define MCC_SS7_CH_EX_RX_REPORTED_ERROR   (MCC_SS7_CH_EX_RX_AB | MCC_SS7_CH_EX_RX_CR  | MCC_SS7_CH_EX_RX_NO | MCC_SS7_CH_EX_RX_LG | MCC_SS7_CH_EX_RX_SF)
#endif

/**************************************************************************//**
 @Description   MCC SS7 extended transmit features.
 
 @Cautions      The SS7 extended transmit features are valid only for
                single-buffer frames. They are ignored when multi-buffer
                frames are transmitted.
*//***************************************************************************/
typedef enum e_MccSS7ChExtTxFlags
{
    e_MCC_SS7_CH_TX_NO_FLAGS               = 0x00000000,
    e_MCC_SS7_CH_TX_RETRANSMIT             = 0x40000000,
    e_MCC_SS7_CH_TX_DELAY                  = 0x00100000,
    e_MCC_SS7_CH_TX_RETRANSMIT_AND_DELAY   = 0x40100000
} e_MccSS7ChExtTxFlags;


/**************************************************************************//**
 @Function      MCC_SS7_CH_Tx

 @Description   MCC SS7 channel transmit routine.
 
                The application calls this function for transmitting a
                SS7 data buffer on a specified MCC channel.
                The buffer is loaded to the next available BD.

 @Param[in]     h_MccSS7Ch  - A handle to an MCC SS7 Channel.
 @Param[in]     p_Data      - The address of the data for sending.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length      - The number of octets for transmitting.
 @Param[in]     lastUserBuffer  - Indicates the last buffer of a frame.
 @Param[in]     extFlags    - Extended SS7 transmit flags (valid for a
                              single-buffer frame only).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_Tx(t_Handle              h_MccSS7Ch,
                      uint8_t               *p_Data,
                      t_Handle              h_BufContext,
                      uint32_t              length,
                      bool                  lastUserBuffer,
                      e_MccSS7ChExtTxFlags  extFlags);

/**************************************************************************//**
 @Function      MCC_SS7_CH_Rx

 @Description   MCC SS7 channel receive routine.
 
                This function is called either by the driver or by
                the user depending on the operation mode. In
                interrupts mode the routine may be called directly from
                the Rx interrupt handler, or by the user (depending on
                'intrManagerId' parameter. In polling mode, the routine must
                be explicitly called by the user.
                It is up to the user to reassemble
                the SS7 frame the routine will invoke the users callback on each received buffer.
                It will do so for all incomming buffers unless the callback indicates that it wishes
                to pause reception.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_Rx(t_Handle h_MccSS7Ch);

/**************************************************************************//**
 @Function      MCC_SS7_CH_TxConf

 @Description   MCC SS7 channel function for forced transmit confirmation.
 
                This routine collects the transmitted frames/buffers and
                returns them to the application. It also prepares the transmit
                BDs for new frames. For each frame/buffer (depending on the
                data exchange mode) the application callback routine will be
                called (f_TxConf in t_MccSS7ChParam).
 
                Normally, transmit confirmation is done automatically by the
                transmit routine. This routine can be called after transmission
                was stopped, to collect all remaining frames/buffers from the
                BD ring.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        None.
*//***************************************************************************/
void MCC_SS7_CH_TxConf(t_Handle h_MccSS7Ch);

/**************************************************************************//**
 @Function      MCC_SS7_CH_TxClean

 @Description   MCC SS7 channel function for cleannig all the BDs.
 
                This routine clean all the buffers in the BD ring and send it to
                the user using the callback given.
                It is used uppon channel underrun.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        None.
*//***************************************************************************/
void MCC_SS7_CH_TxClean(t_Handle h_MccSS7Ch);

/** @} */ /* end of mcc_ss7_ch_runtime_grp group */


/**************************************************************************//**
 @Group         mcc_ss7_ch_control_grp MCC SS7 channel Control Unit

 @Description   MCC SS7 channel CONTROL functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_SS7_CH_Enable

 @Description   Enable an MCC SS7 channel.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     dir        - A direction (Rx/Tx) to enable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_Enable(t_Handle h_MccSS7Ch, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_SS7_CH_Disable

 @Description   Disable an MCC SS7 channel.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 @Param[in]     dir        - A direction (Rx/Tx) to disable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_Disable(t_Handle h_MccSS7Ch, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_SS7_CH_EnableIntr

 @Description   Enable the intrupts of an MCC SS7 channel.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_EnableIntr(t_Handle h_MccSS7Ch);

/**************************************************************************//**
 @Function      MCC_SS7_CH_DisableIntr

 @Description   Disable the intrupts of an MCC SS7 channel.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_SS7_CH_DisableIntr(t_Handle h_MccSS7Ch);

/**************************************************************************//**
 @Function      MCC_SS7_ModifyErrorRateMonitoring

 @Description   Changes the error rate monitoring mode while the MCC SS7
                channel is active.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     ermMode    - Requested Error Rate Monitoring Mode.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ModifyErrorRateMonitoring(t_Handle      h_MccSS7Ch,
                                             e_MccSS7ChErm ermMode);
 
/**************************************************************************//**
 @Function      MCC_SS7_CH_ModifyShortFramesDiscard

 @Description   Changes the short frames discard mode while the MCC SS7
                channel is active.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - do not discard short frames;
                             TRUE  - discard short frames.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ModifyShortFramesDiscard(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_ModifySignalUnitFiltering

 @Description   Changes the signal unit filtering mode while the MCC SS7
                channel is active.

 @Param[in]     h_MccSS7Ch - Handle to the MCC SS7 channel.
 @Param[in]     enabled    - FALSE - disable signal unit filtering;
                             TRUE  - enable signal unit filtering.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Can be called only after a successful call to MCC_SS7_CH_Init().
*//***************************************************************************/
t_Error MCC_SS7_CH_ModifySignalUnitFiltering(t_Handle h_MccSS7Ch, bool enabled);

/**************************************************************************//**
 @Function      MCC_SS7_CH_DumpRegs

 @Description   Dump (print) an MCC SS7 channel registers.

 @Param[in]     h_MccSS7Ch - A handle to an MCC SS7 Channel.
 
 @Return        None.
*//***************************************************************************/
void MCC_SS7_CH_DumpRegs(t_Handle h_MccSS7Ch);

/** @} */ /* end of mcc_ss7_ch_control_grp group */
/** @} */ /* end of mcc_ss7_ch_grp group */


#endif /* __MCC_SS7_EXT_H */
