/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mcc_trans_ext.h

 @Description   MCC TRANS channel Application Programming Interface
*//***************************************************************************/

#ifndef __MCC_TRANS_EXT_H
#define __MCC_TRANS_EXT_H

#include "std_ext.h"
#include "mcc_ext.h"

#define MCC_TRANS_CH_EX_UN       0x0200     /**< Underrun event. This event is unmaskable. */
#define MCC_TRANS_CH_EX_NID      0x0020     /**< Non-idle event                            */
#define MCC_TRANS_CH_EX_IDLE     0x0010     /**< Idle event                                */
#define MCC_TRANS_CH_EX_MRF      0x0008     /**< Maximum frame length violation event      */
#define MCC_TRANS_CH_EX_BSY      0x0002     /**< Busy event. This event is unmaskable.     */


/**************************************************************************//**
 @Group         mcc_trans_ch_grp    MCC TRANS Channel Application Programming Interface

 @Description   MCC TRANS channel API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mcc_trans_ch_init_grp    MCC TRANS Channel Initialization Unit

 @Description   MCC TRANS channel initialization unit API functions, definitions
                and enums.

 @{
*//***************************************************************************/

typedef enum
{
    e_MCC_TRANS_CH_NO_INTERRUPTS = 0,
    e_MCC_TRANS_CH_RX_INTR,
    e_MCC_TRANS_CH_TX_INTR,
    e_MCC_TRANS_CH_RX_N_TX_INTR
} e_MccTransChInterrupts;

typedef enum
{
    e_MCH_TRANS_NO_SYNC = 0,
    e_MCH_TRANS_SLOT_SYNC,
    e_MCH_TRANS_8BIT_SYNC,
    e_MCH_TRANS_16BIT_SYNC
} e_MchTransSync;


/**************************************************************************//**
 @Description   MCC TRANS channel initialization structure,
                used when calling MCC_TRANS_CH_Config.
*//***************************************************************************/
typedef struct t_MccTransChParam
{
    /* Channel parameters */
    uint8_t     		channelNum;         /**< The channel number.                        */

    /* System integration parameters */
    t_Handle    		h_Qe;               /**< A descriptor of the relevant QE module.    */
    t_Handle    		h_Mcc;              /**< A descriptor of the relevant MCC module.   */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information               */
    uint32_t            dataMemAttributes;  /**< Memory attributes for data buffers         */

    /* Application interface */
    t_Handle    		h_App;              /**< A handle to the upper layer handler.       */
    t_RxStoreFunction   *f_RxStore; 	    /**< Upper layer's routine - called to pass
                                    	         a received frame to the application.       */
    t_TxConfFunction    *f_TxConf;  	    /**< Upper layer's routine - called to handle
                                        	     transmitted frames may be set to NULL to
                                        	     avoid unwanted routine calls.              */
    void        		(*f_Exceptions)(t_Handle h_App, uint32_t globalEvent, uint16_t channelEvent);
		                                    /**< Upper layer's routine - called to report
		                                         interrupt events and errors.               */

} t_MccTransChParam;


/**************************************************************************//**
 @Function      MCC_TRANS_CH_Config

 @Description   Configures an MCC TRANS channel.
 
                This routine allocates the MCC channel structure (for
                TRANS mode) and sets user and default parameters
                in the MCC TRANS channel driver parameters structure.

 @Param[in]     p_MccTransChParam - A handle to the an MCC TRANS channel
                                         initialization structure as given by the user.

 @Return        A handle (descriptor) of the newly created MCC TRANS channel structure.
*//***************************************************************************/
t_Handle MCC_TRANS_CH_Config(t_MccTransChParam *p_MccTransChParam);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Init

 @Description   Initializes the MCC TRANS channel.
                It initializes all TRANS specific parameters - dual port
                ram and the driver's structure. It then calls a routine
                for the initialization of the general MCC channel parameters.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Init(t_Handle h_MccTransCh);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Free

 @Description   Free all resources.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        None.
*//***************************************************************************/
void MCC_TRANS_CH_Free(t_Handle h_MccTransCh);


/**************************************************************************//**
 @Group         mcc_trans_ch_advanced_init_grp MCC TRANS Channel Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/*****************************************************************************/
/*              Parameters config routines                                   */
/*****************************************************************************/

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigMaxFrameLength

 @Description   Configures the maximum frame length. [64]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_CH_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigMaxFrameLength (t_Handle h_MccTransCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigInverted

 @Description   Configures if the channel is inverted. [FALSE]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_CH_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigInverted (t_Handle h_MccTransCh, bool newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigInterrupts

 @Description   Configures the channel to interrupt mode. [e_MCC_TRANS_CH_RX_INTR]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigInterrupts(t_Handle               h_MccTransCh,
                                      e_MccTransChInterrupts newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigTxBdRingSize

 @Description   Configures the size of the Tx Bd ring pool. [0]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigTxBdRingSize(t_Handle h_MccTransCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigRxBdRingSize

 @Description   Configures the size of the Rx Bd ring pool. [0]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigRxBdRingSize(t_Handle h_MccTransCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigExceptions

 @Description   Configures the enabled exceptions. [MCC_TRANS_CH_EVENT_MRF]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigExceptions(t_Handle h_MccTransCh, uint16_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigTimeStampInsert

 @Description   Configures time-stamp insertion. [FALSE]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigTimeStampInsert (t_Handle h_MccTransCh, bool newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigRxIntrQueue

 @Description   Configures the Rx interrupt queue index. [0]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigRxIntrQueue (t_Handle h_MccTransCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigNumOfPadChars

 @Description   Configures the number of pad charachters. [0]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_CH_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigNumOfPadChars (t_Handle h_MccTransCh, uint8_t newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigReverseBitOrder

 @Description   Configures reverse bit order. [FALSE]

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     newVal       - A new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_CH_Init.
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigReverseBitOrder (t_Handle h_MccTransCh, bool newVal);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_ConfigRxSyncPattern

 @Description   Configures the synchronization pattern. [e_MCH_TRANS_NO_SYNC]

 @Param[in]     h_MccTransCh    - Handle to an MCC TRANS channel
 @Param[in]     syncMode        - The syncronazation mode desired.
 @Param[in]     syncPattern     - The syncronazation pattern desired.
                                  NOTE - This field valid only if syncMode is
                                  either e_MCH_TRANS_16BIT_SYNC or e_MCH_TRANS_8BIT_SYNC.

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling MCC_TRANS_CH_Init().
*//***************************************************************************/
t_Error MCC_TRANS_CH_ConfigRxSyncPattern (t_Handle h_MccTransCh, e_MchTransSync syncMode, uint16_t syncPattern);


/** @} */ /* end of mcc_trans_ch_advanced_init_grp group */
/** @} */ /* end of mcc_trans_ch_init_grp group */


/**************************************************************************//**
 @Group         mcc_trans_ch_runtime_grp MCC TRANS Channel Runtime Unit

 @Description   MCC TRANS channel runtime unit API functions, definitions and enums.

 @{
*//***************************************************************************/

#define MCC_TRANS_CH_EX_RX_LG    0x0020     /* Frame length violation      */
#define MCC_TRANS_CH_EX_RX_NO    0x0010     /* Rx Non Octet Aligned Packet */
#define MCC_TRANS_CH_EX_RX_AB    0x0008     /* Frame Aborted               */

#ifndef VERIFICATION_PASS_ERR_FRAME
#define MCC_TRANS_CH_EX_RX_FATAL_ERROR      (MCC_TRANS_CH_EX_RX_AB | MCC_TRANS_CH_EX_RX_NO)
#define MCC_TRANS_CH_EX_RX_REPORTED_ERROR   MCC_TRANS_CH_EX_RX_LG
#else
#define MCC_TRANS_CH_EX_RX_FATAL_ERROR      0
#define MCC_TRANS_CH_EX_RX_REPORTED_ERROR   (MCC_TRANS_CH_EX_RX_AB | MCC_TRANS_CH_EX_RX_NO | MCC_TRANS_CH_RX_LG)
#endif /* VERIFICATION_PASS_ERR_FRAME */


/**************************************************************************//**
 @Function      MCC_TRANS_CH_Tx

 @Description   The application calls this function for transmitting a
                TRANS data buffer on a specified MCC channel.
                The buffer is loaded to the next available BD.

 @Param[in]     h_MccTransCh    - A handle to an MCC TRANS channel.
 @Param[in]     p_Data          - The address of the data for sending.
 @Param[in]     h_BufContext    - User's context for the given data buffer
 @Param[in]     length          - The number of octets for transmitting.
 @Param[in]     lastUserBuffer  - Indicates the last buffer of a frame.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Tx(t_Handle    h_MccTransCh,
                        uint8_t     *p_Data,
                        t_Handle    h_BufContext,
                        uint32_t    length,
                        bool        lastUserBuffer);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Rx

 @Description   MCC TRANS channel receive routine.
 
                This function is called either by the driver or by the user
                depending on the operation mode.
                In interrupts mode the routine may be called directly from
                the Rx interrupt handler, or by the user (depending on
                'intrManagerId' parameter.
                In polling mode, the routine must be explicitly called by the user.
                It is up to the user to reassemble
                the TRANS frame the routine will invoke the users callback on
                each received buffer. It will do so for all incomming buffers
                unless the callback indicates that it wishes to pause reception.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Rx(t_Handle h_MccTransCh);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_TxConf

 @Description   MCC TRANS channel function for forced transmit confirmation.
 
                This routine collects the transmitted buffers and returns them
                to the application. It also prepares the transmit
                BDs for new frames. For each buffer the application callback
                routine will be called (f_TxConf in t_MccTransChParam).
 
                Normally, transmit confirmation is done automatically by the
                transmit routine. This routine can be called after transmission
                was stopped, to collect all remaining buffers from the BD ring.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        None.
*//***************************************************************************/
void MCC_TRANS_CH_TxConf(t_Handle h_MccTransCh);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_TxClean

 @Description   MCC TRANS channel function for cleannig all the BDs.
 
                This routine clean all the buffers in the BD ring and send it to
                the user using the callback given.
                It is used uppon channel underrun.

 @Param[in]     h_MccTransCh - A handle to an MCC Trans channel.

 @Return        None.
*//***************************************************************************/
void MCC_TRANS_CH_TxClean(t_Handle h_MccTransCh);

/** @} */ /* end of mcc_trans_ch_runtime_grp group */


/**************************************************************************//**
 @Group         mcc_trans_ch_control_grp MCC TRANS Channel Control Unit

 @Description   MCC TRANS channel CONTROL functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Enable

 @Description   Enable an MCC TRANS channel.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     dir          - A direction (RX/TX) to enable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Enable(t_Handle h_MccTransCh, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Disable

 @Description   Disable an MCC TRANS channel.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     dir          - A direction (RX/TX) to disable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Disable(t_Handle h_MccTransCh, e_CommMode dir);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_Restart

 @Description   Restart an MCC TRANS channel operation.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
 @Param[in]     dir          - A direction (RX/TX) to disable.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_Restart(t_Handle h_MccTransCh, e_CommMode dir);


/**************************************************************************//**
 @Function      MCC_TRANS_CH_DisableIntr

 @Description   Disable the intrupts of an MCC TRANS channel.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_DisableIntr(t_Handle h_MccTransCh);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_EnableIntr

 @Description   Enable the intrupts of an MCC TRANS channel.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error MCC_TRANS_CH_EnableIntr(t_Handle h_MccTransCh);

/**************************************************************************//**
 @Function      MCC_TRANS_CH_DumpRegs

 @Description   Dump (print) an MCC TRANS channel registers.

 @Param[in]     h_MccTransCh - A handle to an MCC TRANS channel.
*//***************************************************************************/
void MCC_TRANS_CH_DumpRegs(t_Handle h_MccTransCh);

/** @} */ /* end of mcc_trans_ch_control_grp group */
/** @} */ /* end of mcc_trans_ch_grp group */


#endif /* __MCC_TRANS_EXT_H */
