/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mtc_ext.h

 @Description   external definition and API for the MTC.
 
 @Cautions      None.
*//***************************************************************************/
#ifndef __MTC_EXT_H
#define __MTC_EXT_H

#include "part_ext.h"


#define MTC_ATM_MAX_PERIPHERALS            8

/**************************************************************************//**
 @Group         mtc_grp MTC Application Programming Interface

 @Description   MTC functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//*
 @Description            defines
*//***************************************************************************/

#define MTC_EVENT_OR     0x8000
#define MTC_EVENT_UR     0x4000
#define MTC_EVENT_CDT    0x2000
#define MTC_EVENT_ROF    0x0020
#define MTC_EVENT_TOF    0x0010
#define MTC_EVENT_EOF    0x0008
#define MTC_EVENT_COF    0x0004
#define MTC_EVENT_IOF    0x0002
#define MTC_EVENT_FOF    0x0001

/**************************************************************************//*
 @Description            enums
*//***************************************************************************/
typedef enum
{
    e_MTC_CF_FALSE = 0x0000
    ,e_MTC_CF_IDLE     = 0x0080
     ,e_MTC_CF_UNASSIGNED = 0x0100
    ,e_MTC_CF_IDLE_UNASSIGNED = 0x0180
} e_CF;


typedef enum
{
    e_MTC_SYNC_BIT = 0x0000
    ,e_MTC_SYNC_BYTE = 0x0040
} e_SYNC_MODE;

typedef enum
{
    e_BIG_ENDIAN = 0x10
    ,e_LITTLE_ENDIAN = 0x08
 
} e_BO;

/**************************************************************************//*
 @Description   MTC structure. Passed to MTC_Config().
 *//***************************************************************************/
typedef struct
{
    t_Handle    h_AtmController;    /**< Handle to the ATM controller */
    uint8_t     txIntQ;             /**< User callback function called by driver to get data buffer */
    uint8_t     rxIntQ;             /**< Tx queue number. */
} t_MtcParam;

/**************************************************************************//**
 @Group         mtc_common_grp MTC_COMMON Application Programming Interface

 @Description   MTC_COMMON functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         mtc_common_int_grp MTC_COMMON Application Programming Interface

 @Description   MTC_COMMON functions,definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Description   ATM controller parameters for MTC
 *//***************************************************************************/
typedef struct
{
    uint32_t    txPrfLen;           /**< number of entries in Tx Pending request FIFO */
    uint32_t    rxPrfLen;           /**< number of entries in Rx Pending request FIFO */
    t_Handle    h_AtmController;    /**< Handle to ATM controller object */
    bool        txMphyEn;           /**< if TxMPHY enable */
    bool        rxMphyEn;           /**< if RxMPHY enable */
} t_MtcCommonAtmControllerParam;

/**************************************************************************//**
 @Description   Interrupt Queue data structure
 *//***************************************************************************/
typedef struct
{
    uint32_t    intQLen;            /**< The number of entries in the interrupt queue */
    uint8_t     intQMemId;          /**< Memory partition ID for interrupt queues */
    uint32_t    intQMemAttributes;  /**< Memory attributes mask for interrupt queues */
} t_MtcCommonIntQParam;

/**************************************************************************//**
 @Description   MTC COMMON structure. Passed to MTC_COMMON_Config().
 *//***************************************************************************/
typedef struct
{
    t_Handle    h_Qe;               /**< Handle to the controlling QE */
    uint8_t     pramMemId;          /**< Parameter RAM partition ID */
    uint32_t    numOfIntQueues;     /**< Number of MTC Interrupt tables*/
    t_Handle    h_MtcCommonAtmController[MTC_ATM_MAX_PERIPHERALS];
                                    /**< Handles to ATM controller objects */
} t_MtcCommonParam;

/**************************************************************************//**
 @Function      MTC_COMMON_Config

 @Description   Configures a global MTC handler(data structure for all MTC's)
                This routine allocates the MTC_COMMON structure and sets user
                and default parameters in the MTC_COMMON driver parameters structure.

 @Param         p_MtcCommonParam  - A pointer to the initializing parameter
                                    structure for the MTC_COMMON.

 @Return        A handle of the newly created MTC_COMMON structure.
*//***************************************************************************/
t_Handle MTC_COMMON_Config(t_MtcCommonParam* p_MtcCommonParam);

/**************************************************************************//**
 @Function      MTC_COMMON_Init

 @Description   Initializes a global MCC port.
                This routine initializes the t_MtcCommon structure, the global
                parameters at the parameter ram and the memory mapped
                MTC_COMMON registers.

 @Param         h_MtcCommon - A handle of the MTC_COMMON object.

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
t_Error MTC_COMMON_Init(t_Handle h_MtcCommon);

/**************************************************************************//**
 @Function      MTC_COMMON_Free

 @Description   Frees all MTC_COMMON resources

 @Param         h_MtcCommon - A handle of the MTC_COMMON object.

 @Return        E_OK on success, other value otherwise.

*//***************************************************************************/
void MTC_COMMON_Free(t_Handle h_MtcCommon);

/**************************************************************************//**
 @Group         mtc_common_advanced_init_grp MTC COMMON Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      MTC_COMMON_ConfigIntQ

 @Description   Configures some interrupt queue parameters for a specific interrupt queue.

 @Param         h_MtcCommon     - Handle to MTC_COMMON
 @Param         intQ            - Number of specific interrupt queue
 @Param         p_MtcIntQParam  - Data structure with modified parameters
 
 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_COMMON_ConfigIntQ (t_Handle h_MtcCommon, uint8_t intQ, t_MtcCommonIntQParam* p_MtcIntQParam);

/**************************************************************************//**
 @Function      MTC_COMMON_ConfigAtmControllerParam

 @Description   Configures ATM controller parameters.
 
 @Param         h_MtcCommon                     - Handle to MTC_COMMON
 @Param         p_MtcCommonAtmControllerParam   - Data structure with modified parameters
 
 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error MTC_COMMON_ConfigAtmControllerParam (t_Handle h_MtcCommon, t_MtcCommonAtmControllerParam* p_MtcCommonAtmControllerParam);

/** @} */ /* end of mtc_common_advanced_init_grp group */
/** @} */ /* end of mtc_common_init_grp group */
/** @} */ /* end of mtc_common_grp group */
/** @} */ /* end of mtc_grp group */

t_Error MTC_Config (t_Handle h_Mtc);
t_Error MTC_Init(t_Handle h_Mtc, uint32_t mtcPramBaseAddr, uint16_t mtcPramSize,
                    e_ModuleId peripheral);
t_Error MTC_Free(t_Handle h_Mtc);
t_Error MTC_Enable(t_Handle h_Mtc, e_CommMode ActiveMode);
void MTC_Disable(t_Handle h_Mtc, e_CommMode ActiveMode);
t_Error MTC_RxAtmEnable(t_Handle h_Mtc);
t_Error MTC_RxAtmDisable(t_Handle h_Mtc);
t_Handle MTC_COMMON_GetQeHandler(t_Handle h_MtcCommon);
t_Error MTC_COMMON_CheckIntQ(t_Handle h_MtcCommon, uint16_t intQNum, e_ModuleId peripheral);

t_Error MTC_ConfigTxEn (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigRxEn (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigRxPScr (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigTxPScr (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigRxCoset (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigTxCoset (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigSbc (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigTxIma (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigRxIma (t_Handle h_Mtc, bool newVal);
t_Error MTC_ConfigAlpha (t_Handle h_Mtc, uint8_t newVal);
t_Error MTC_ConfigDelta (t_Handle h_Mtc, uint8_t newVal);
t_Error MTC_ConfigBbs (t_Handle h_Mtc, e_SYNC_MODE newVal);
t_Error MTC_ConfigIntMask(t_Handle h_Mtc, uint16_t newVal);
t_Error MTC_ConfigCellFiltering(t_Handle h_Mtc, e_CF newVal);
t_Error MTC_ConfigTxCfSize(t_Handle h_Mtc, uint32_t newVal);
t_Error MTC_ConfigRxCfSize(t_Handle h_Mtc, uint32_t newVal);
t_Error MTC_ConfigTxCosetVal(t_Handle h_Mtc, uint8_t newVal);
t_Error MTC_ConfigRxCosetVal(t_Handle h_Mtc, uint8_t newVal);
t_Error MTC_ConfigThd(t_Handle h_Mtc, bool newVal);
t_Error MTC_GetRxCellFifoSize(t_Handle h_Mtc, uint32_t *val);

#endif /* __MTC_EXT_H */

