/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCI_CTRL_83XX_EXT_H
#define __PCI_CTRL_83XX_EXT_H

#include "pci_ctrl_ext.h"
#include "pci_std_ext.h"

/**************************************************************************//**
 @Group         pci_ctrl_grp PCI Controller Application Programming Interface

 @Description   PCI Controller API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pci_ctrl_control_grp PCI Controller Control Unit

 @Description   PCI Controller control functions and definitions

 @{
*//***************************************************************************/

 
t_Error PCI_CTRL_SetErrorsPolicy     (t_Handle           h_PciCtrl,
                             uint32_t           pciErrType,
                             e_PciErrorPolicy   pciErrorPolicy);
 
/**************************************************************************//**
 @Description  Inbound window attribute
*//***************************************************************************/
typedef struct t_PciInboundAttributes
{
    e_PciInboundReadTransType       rtt;                /**< Read transaction type                         */
    e_PciInboundWriteTransType      wtt;                /**< Write transaction type                        */
    bool                            prefetchable;       /**< Indicate if the window is prefetchable or not */
    void                            (*f_WindowErrorHandle)(uint32_t winNum, void *errorAttributes);
                                                        /**< Callback function to handle errors that occured
                                                             in this window; This functionality supported only
                                                             for PCI inbound windows. for PCI Express,
                                                             f_WindowErrorHandle should be NULL.                                        */
} t_PciInboundAttributes;

/**************************************************************************//**
 @Description  Outbound window attribute
*//***************************************************************************/
typedef struct t_PciOutboundAttributes
{
    uint8_t ioSpace;                /**< I/O space; Determines whether the window is mapped to the PCI memory space
                                         or PCI I/O space.                                                           */
    uint8_t streamEnable;           /**< Streaming enable; Defines whether the transactions that are translated
                                         through this window may be streamed on the PCI bus.                         */
    void    (*f_WindowErrorHandle)(uint32_t winNum, void *errorAttributes);
                                    /**< Callback function to handle errors that occured in this window; This functionality
                                         supported only for PCI's outbound windows; for PCI Express, f_WindowErrorHandle should be NULL.*/
} t_PciOutboundAttributes;

/** @} */ /* end of pci_ctrl_control_grp */

/**************************************************************************//**
 @Group         pci_ctrl_exception_grp PCI Controller Exceptions Unit

 @Description   PCI Controller exceptions functions and definitions.

 @{
*//***************************************************************************/

/***************************************************************************//**
@Description Error Type - This field is encoded to indicate the type of the first
             PCI error captured.
*//***************************************************************************/
typedef enum e_PciErrors
{
    e_PCI_ADDRESS_PARITY_ERROR      = 0x0,  /**< Address parity error                             */
    e_PCI_WRITE_DATA_PARITY_ERROR   = 0x1,  /**< Write data parity error                         */
    e_PCI_READ_DATA_PARITY_ERROR    = 0x2,  /**< Read data parity error                         */
    e_PCI_MASTER_ABORT              = 0x3,  /**< Master abort                                     */
    e_PCI_TARGET_ABORT              = 0x4,  /**< Target abort                                     */
    e_PCI_SYSTEM_ERROR              = 0x5,  /**< System error indication recived                 */
    e_PCI_PARITY_ERROR_ON_READ      = 0x6,  /**< Parity error indication error recived on read */
    e_PCI_PARITY_ERROR_ON_WRITE     = 0x7   /**< Parity error indication error recived on wriet*/
} e_PciErrors;

/***************************************************************************//**
@Description Beat number - This field provides the data beat number on which
            the error occurred for data parity errors.The value of this
            field is undefined for other error types.
*//***************************************************************************/
typedef enum e_PciErrorBeatNumber
{
    e_PCI_1ST                   = 0x0,  /**< 1st bit           */
    e_PCI_2ND                   = 0x1,  /**< 2nd bit           */
    e_PCI_3RD                   = 0x2,  /**< 3rd bit           */
    e_PCI_4TH                   = 0x3,  /**< 4th bit           */
    e_PCI_5TH                   = 0x4,  /**< 5th bit           */
    e_PCI_6TH                   = 0x5,  /**< 6th bit           */
    e_PCI_7TH                   = 0x6,  /**< 7th bit           */
    e_PCI_8TH                   = 0x7,  /**< 8th bit           */
    e_PCI_9TH_OR_BEYOND         = 0x8   /**< 9th bit or beyond */
} e_PciErrorBeatNumber;

/***************************************************************************//**
@Description   Transaction size - Indicates the size of the transaction in
               units of doublewords (8 bytes).
*//***************************************************************************/
typedef enum e_PciTransactionSize
{
    e_PCI_4_DWORDS_TRANS        = 0x0,  /**< 4 double words  */
    e_PCI_1_DWORDS_TRANS        = 0x1,  /**< 1 double word   */
    e_PCI_2_DWORDS_TRANS        = 0x2,  /**< 2 double words  */
    e_PCI_3_DWORDS_TRANS        = 0x3   /**< 3 double word   */
} e_PciTransactionSize;

/***************************************************************************//**
 @Description   PCI error source
 
                Indicates the source of the captured error.
*//***************************************************************************/
typedef enum e_PciErrorSrc
{
    e_PCI_EXTERNAL_MASTER_SRC   = 0x0,  /**< External master source */
    e_PCI_DMA_SRC               = 0x5   /**< DMA source             */
} e_PciErrorSrc;

/***************************************************************************//**
 @Description   PCI Controller Error attributes

                This structure contains information about errors that occured,
                including more detailed information about the last error.
*//***************************************************************************/
typedef struct t_PciErrorAttributes
{
    e_PciErrors             errorType;          /**< First error type - indicate the type of the
                                                     first PCI error captured                       */
    e_PciErrorBeatNumber    beatNumber;         /**< Bit number - the data bit number on which
                                                     the error occurred for data parity errors only */
    e_PciTransactionSize    transactionSize;    /**< Transaction size - Indicates the size of the
                                                     transaction in units of doublewords (8 bytes)  */
    e_PciErrorSrc           errorSource;        /**< Error source - the source of PCI transaction   */
    uint64_t                errorAddress;       /**< PCI error address                              */
    uint32_t                errorData;          /**< PCI error data - the data associated with
                                                     the first detected error                       */
    bool                    multipleError;      /**< Multiple errors - the same error type occurs more then once*/
    bool                    addressParityError; /**< Address parity error on a PCI access initiated by a device
                                                     other than this PCI controller                             */
    bool                    systemError;        /**< PCI system error - PCI_SERR input signal is asserted       */
    bool                    masterParityError;  /**< Master parity error - write access initiated by this
                                                     PCI controller or when a data parity error is detected by
                                                     this PCI controller on a read access that it initiated     */
    bool                    targetParityError;  /**< Target parity error - read access or a data parity error
                                                     is detected by this PCI controller on a write access       */
    bool                    noResponse;         /**< No response - there is no response to a transaction
                                                     initiated by this PCI controller on the PCI bus            */
    bool                    targetAbort;        /**< Target abort - target abort occurs on a transaction
                                                     initiated by this PCI controller                           */
 
} t_PciErrorAttributes;


/** @} */ /* end of pci_ctrl_exception_grp */
/** @} */ /* end of pci_ctrl_grp */

#endif /* __PCI_CTRL_83XX_EXT_H */

