/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCI_CTRL_85XX_EXT_H
#define __PCI_CTRL_85XX_EXT_H

#include "pci_ctrl_ext.h"
#include "pci_std_ext.h"

/**************************************************************************//**
 @Group         pci_ctrl_grp PCI Controller Application Programming Interface

 @Description   PCI API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pci_ctrl_control_grp PCI Controller Control Unit

 @Description   PCI Controller control functions and definitions

 @{
*//***************************************************************************/

#define PCI_INBOUND_WINDOWS_NUM       4  /**< Number of inbound windows    */
#define PCI_OUTBOUND_WINDOWS_NUM      5  /**< Number of outbound windows   */

/**************************************************************************//**
 @Description   Inbound window attributes.
*//***************************************************************************/
typedef struct t_PciInboundAttributes
{
    e_PciTargetInterface         target;        /**< Target interface       */
    e_PciInboundReadTransType    rtt;           /**< Read transaction type  */
    e_PciInboundWriteTransType   wtt;           /**< Write transaction type */
    bool                         prefetchable;  /**< Indicate if the window
                                                     is prefetchable or not */

} t_PciInboundAttributes;

/**************************************************************************//**
 @Description   Read outbound transaction type to run on PCI
*//***************************************************************************/
typedef enum e_PciOutboundReadTransType
{
    e_PCI_OUTBOUND_RTT_MEMORY_READ = 0x4,       /**< Memory Read */
    e_PCI_OUTBOUND_RTT_IO_READ     = 0x8        /**< I/O Read    */
} e_PciOutboundReadTransType;

/**************************************************************************//**
 @Description   Write outbound transaction type to run on PCI
*//***************************************************************************/
typedef enum e_PciOutboundWriteTransType
{
    e_PCI_OUTBOUND_WTT_MEMORY_WRITE = 0x4,      /**< Memory Write  */
    e_PCI_OUTBOUND_WTT_IO_WRITE     = 0x8       /**< I/O Write     */
} e_PciOutboundWriteTransType;

/**************************************************************************//**
 @Description   Outbound window attributes.
*//***************************************************************************/
typedef struct t_PciOutboundAttributes
{
    e_PciOutboundReadTransType      rtt;        /**< Read transaction type   */
    e_PciOutboundWriteTransType     wtt;        /**< Write transaction type  */
} t_PciOutboundAttributes;


/**************************************************************************//**
 @Function      PCI_SetDiscardTimerControl.

 @Description   Sets the number of system clocks to purge a non-prefetchable
                inbound read buffer

 @Param[in]     h_PciCtrl           - Handle to the PCI controller object.
 @Param[in]     preloadTimeValue    - Number of system clocks.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_SetDiscardTimerControl(t_Handle h_PciCtrl, uint32_t preloadTimeValue);

/**************************************************************************//**
 @Function      PCI_CTRL_SetErrorsPolicy.

 @Description   Sets the error policy of the PCI controller.

 @Param[in]     h_PciCtrl       - Handle to the PCI controller object.
 @Param[in]     pciErrType      - The error type for which to set the policy.
 @Param[in]     pciErrorPolicy  - The selected error policy.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_SetErrorsPolicy(t_Handle           h_PciCtrl,
                                 uint32_t           pciErrType,
                                 e_PciErrorPolicy   pciErrorPolicy);

/** @} */ /* end of pci_ctrl_control_grp */

/**************************************************************************//**
 @Group         pci_ctrl_exception_grp PCI Controller Exceptions Unit

 @Description   PCI Controller exceptions functions and definitions

 @{
*//***************************************************************************/

/***************************************************************************//**
 @Description   PCI error source
 
                Indicates the source of the first error captured.
*//***************************************************************************/
typedef enum e_PciErrorSrc
{
    e_PCI_PCI_ERR_SRC                   = 0x0,      /**< PCI source                     */
    e_PCI_PCI_EXPRESS_ERR_SRC           = 0x10,     /**< PCI Express source             */
    e_PCI_LOCAL_BUS_CTRL_ERR_SRC        = 0x100,    /**< Local bus controller source    */
    e_PCI_SRIO_ERR_SRC                  = 0x1100,   /**< Serial RapidIO source          */
    e_PCI_TLU_ERR_SRC                   = 0x1110,   /**< TLU source                     */
    e_PCI_E500_CORE_INSTRUCTION_ERR_SRC = 0x10000,  /**< e500 core (instruction) source */
    e_PCI_E500_CORE_DATA_ERR_SRC        = 0x10001,  /**< e500 core (data) source        */
    e_PCI_QE_ERR_SRC                    = 0x10100,  /**< QUICC Engine Block source      */
    e_PCI_DMA_ERR_SRC                   = 0x10101,  /**< DMA source                     */
    e_PCI_TSEC1_ERR_SRC                 = 0x11000,  /**< TSEC1 source                   */
    e_PCI_TSEC2_ERR_SRC                 = 0x11001,  /**< TSEC2 source                   */
    e_PCI_RAPIDIO_MSG_ERR_SRC           = 0x11100,  /**< RapidIO message units source   */
    e_PCI_RAPIDIO_DOORBELL_ERR_SRC      = 0x11101,  /**< RapidIO doorbell units source  */
    e_PCI_RAPIDIO_PORT_WRITE_ERR_SRC    = 0x11110   /**< RapidIO port-write units source*/

} e_PciErrorSrc;

/***************************************************************************//**
 @Description   PCI Controller Error attribures

                This structure contains information about errors that occured,
                including more detailed information about the last error.
*//***************************************************************************/
typedef struct t_PciErrorAttributes
{
    uint8_t         pciCommand;                 /**< PCI command                                    */
    e_PciErrorSrc   errorSource;                /**< The source of the PCI transaction              */
    uint64_t        errorAddress;               /**< PCI error address capture register - Memory transaction address        */
    uint64_t        errorData;                  /**< PCI error data low capture register            */
    bool            multipleError;              /**< Indicate if more then one PCI error from the same type were detected   */
    bool            addressParityError;         /**< Address parity error                           */
    bool            systemError;                /**< Received system error (SERR) error             */
    bool            masterParityError;          /**< Master parity error (PERR)                     */
    bool            targetParityError;          /**< Target parity error (PERR)                     */
    bool            masterAbort;                /**< Master abort error                             */
    bool            targetAbort;                /**< Target abort error                             */
    bool            outboundWriteError;         /**< Outbound write memory space violation error    */
    bool            outboundReadError;          /**< Outbound read memory space violation error     */
    bool            inboundReadError;           /**< Inbound read memory space violation error      */
    bool            splitCompletionMsgError;    /**< Split completion message error                 */
    bool            timeoutError;               /**< Time-out error                                 */
 
} t_PciErrorAttributes;


/**************************************************************************//**
 @Collection    PCI Controller Errors Mask

 @{
*//***************************************************************************/
#define PCI_ERROR_MULTIPLE          0x80000000  /**< Multiple errpr          */
#define PCI_ERROR_PARITY            0x00000400  /**< Address parity error    */
#define PCI_ERROR_SYSTEM            0x00000200  /**< Pci system error        */
#define PCI_ERROR_MASTER_PARITY     0x00000100  /**< Master parity error     */
#define PCI_ERROR_TARGET_PARITY     0x00000080  /**< Target parity error     */
#define PCI_ERROR_MASTER_ABORT      0x00000040  /**< Master abort error      */
#define PCI_ERROR_TARGET_ABORT      0x00000020  /**< Target abort error      */
#define PCI_ERROR_OWMSV             0x00000010  /**< Outbound write memory space violation error */
#define PCI_ERROR_ORMSV             0x00000008  /**< Outbound read memory space violation error  */
#define PCI_ERROR_IRMSV             0x00000004  /**< Inbound read memory space violation error   */
#define PCI_ERROR_SCM               0x00000002  /**< Split completion message error              */
#define PCI_ERROR_TIME_OUT          0x00000001  /**< Time-out error                              */
/* @} */


/** @} */ /* end of pci_ctrl_exception_grp */
/** @} */ /* end of pci_ctrl_grp */



#endif /* __PCI_CTRL_85XX_EXT_H */
