/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCI_CTRL_EXT_H
#define __PCI_CTRL_EXT_H

#include "list_ext.h"
#include "part_ext.h"
#include "pci_std_ext.h"

/*************************************************************************//**
 @Group         pci_ctrl_grp PCI Controller Application Programming Interface

 @Description   PCI Controller API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pci_ctrl_init_grp PCI Controller Initialization Unit

 @Description   PCI Controller initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PCI controller configuration parameters structure
*//***************************************************************************/
typedef struct t_PciCtrlParam
{
    uint32_t    baseAddress;            /**< Base address of PCI controller registers */
    t_Handle    h_App;                  /**< Handle to the upper layer */
    t_Handle    h_Part;                 /**< Handle to an integration object */
    bool        hostMode;               /**< Host mode indication */
    int         intA;                   /**< PCI INTA interrupt ID */
    int         intB;                   /**< PCI INTB interrupt ID */
    int         intC;                   /**< PCI INTC interrupt ID */
    int         intD;                   /**< PCI INTD interrupt ID */
    int         irq;                    /**< PCI controller interrupt ID */
    int         pmeIrq;                 /**< Wake up interrupt ID */
    int         msiIrq0;                /**< MSI 0 interrupt ID */
    int         msiIrq1;                /**< MSI 1 interrupt ID */
    int         msiIrq2;                /**< MSI 2 interrupt ID */
    int         msiIrq3;                /**< MSI 3 interrupt ID */
    int         msiIrq4;                /**< MSI 4 interrupt ID */
    int         msiIrq5;                /**< MSI 5 interrupt ID */
    int         msiIrq6;                /**< MSI 6 interrupt ID */
    int         msiIrq7;                /**< MSI 7 interrupt ID */
    uint32_t    localAccessWindowBase;  /**< PCI local access window address */
    uint32_t    localAccessWindowSize;  /**< PCI local access window size */
    void        (*f_ErrorHandle)(void *p_ErrorAttributes);
                                        /**< Callback to inform about errors occured */
    void        (*f_DeviceInterruptHandle)(t_Handle h_App, uint32_t intrSrc);
                                        /**< Callback to handle external interrupts     */
    void        (*f_PmeHandle)(t_Handle h_App);
                                        /**< Callback to handle power manegement wake up request    */
    void        (*f_MsiHandle)(t_Handle h_App, uint32_t msgNumber, uint32_t deviceNumber);
                                        /**< Callback to handle MSI message             */
} t_PciCtrlParam;

/**************************************************************************//**
 @Function      PCI_CTRL_Config

 @Description   Creates and configures a PCI Object

                Creates and configures a PCI Controller cbject and
                register the specific function according to the
                parameters that are passed in the data structure.


 @Param[in]     p_PciParams - Pointer to initialization parameters
                              structure as defined at pci_ext.h.

 @Retval        Handle to PCI object : on success
 @Retval        NULL: on error

 @Cautions      This routine must be called prior to any other PCI routine.
*//***************************************************************************/
t_Handle PCI_CTRL_Config(t_PciCtrlParam *p_PciParams);


/**************************************************************************//**
 @Function       PCI_CTRL_Init

 @Description    Initialization of PCI unit.
                 This routine initializes the PCI Controller according
                 to the parameters previously provided by the configurations routines.
                 This routine starts the bus scan process in order identify the devices
                 on the bus and connect between those devices to the registered device drivers.

 @Param[in]      h_PciCtrl - Handle to the PCI object.

 @Return         E_OK on success; Error code otherwise.

 @Cautions       h_PciCtrl must have previously created with PCI_Config().
*//***************************************************************************/
t_Error PCI_CTRL_Init(t_Handle h_PciCtrl);


/**************************************************************************//**
 @Function      PCI_CTRL_Free

 @Description   Frees all PCI controller's resources

 @Param[in]     h_PciCtrl - Handle to the PCI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_Free(t_Handle h_PciCtrl);


/** @} */ /* end of pci_ctrl_init_grp */

/**************************************************************************//**
 @Group         pci_ctrl_control_grp PCI Controller Control Unit

 @Description   PCI Controller control functions.

 @{
*//***************************************************************************/

#define PCI_ANY_WINDOW_NUMBER   0xFFFFFFFF   /**< Use this value to open inbound/outbound windows
                                                  if the cpecific window's number doesn't matter    */

/**************************************************************************//**
 @Description   Inbound window parameters.
*//***************************************************************************/
typedef struct t_PciInboundParams
{
    uint64_t      translationAddr;  /**< The address of the inbound translation.               */
    uint64_t      baseAddress;      /**< The address of the base address (PCI memory address); */
    uint64_t      size;             /**< Window size in bytes.                                 */
    void          *p_Attributes;    /**< Pointer to controller-specific window attributes structure.*/
} t_PciInboundParams;

/**************************************************************************//**
 @Description   Outbound window parameters.
*//***************************************************************************/
typedef struct t_PciOutboundParams
{
    uint64_t translation;   /**< Translation address of the outbound window.                 */
    uint64_t size;          /**< The size of the outbound window in bytes.                   */
    void     *p_Attributes; /**< Pointer to controller-specific window attributes structure. */
} t_PciOutboundParams;


/**************************************************************************//**
 @Function      PCI_CreateInboundWindow

 @Description   Initializes an inbound window.

                The attributes for the window and window's base and translation
                addresses must be passed to the routine.
                The driver will assign a free window Id for this window.

 @Param[in]     h_PciCtrl - Handle to the PCI object.
 @Param[in,out] p_InboundParams - Pointer to window data structue.
 @Param[in,out] p_WinId - Pointer to window's number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_CreateInboundWindow( t_Handle              h_PciCtrl,
                                      t_PciInboundParams    *p_InboundParams,
                                      uint8_t               *p_WinId);

/**************************************************************************//**
 @Function      PCI_CreateOutboundWindow

 @Description   Initializes an outbound window.

                The attributes for the window are assigned by the user. The
                driver allocates window base address.
                All addresses will be returned to the user to use on setting
                a proper inbound window for the Agent/EP.
                The driver will assign a free window Id for this window.

 @Param[in]     h_PciCtrl      - Handle to the PCI object.
 @Param[in,out] p_OutboundParams - Pointer to window data structue.
 @Param[in,out] p_WinId - Pointer to window's number.
 @Param[in,out] p_BaseAddr - Pointer to window's base address.

 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error PCI_CTRL_CreateOutboundWindow(t_Handle             h_PciCtrl,
                                      t_PciOutboundParams  *p_OutboundParams,
                                      uint8_t              *p_WinId,
                                      uint64_t             *p_BaseAddr);


/**************************************************************************//**
 @Function      PCI_FreeInboundWindow

 @Description   Disables the specified inbound window and releases all
                allocated resources.

 @Param[in]     h_PciCtrl   - Handle to the PCI object.
 @Param[in]     winNum        - The number of the window to disable

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
uint64_t PCI_CTRL_FreeInboundWindow(t_Handle  h_PciCtrl, uint8_t winNum);


/**************************************************************************//**
 @Function      PCI_FreeOutboundWindow

 @Description   Disables the specified outbound window and releases all
                allocated resources.

 @Param[in]     h_PciCtrl   - Handle to the PCI object.
 @Param[in]     winNum        - The number of the window to disable

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
uint64_t PCI_CTRL_FreeOutboundWindow(t_Handle h_PciCtrl, uint8_t winNum);


/**************************************************************************//**
 @Function      PCI_CTRL_ConfigWrite

 @Description   Issues a maintenance write request to the destination device.

                Use this routine in order to write dword (32 bit) to the device configuration space.

 @Param[in]     h_PciCtrl    - Handle to the PCI object.
 @Param[in]     pciBus       - The bus Id of the device to write to
 @Param[in]     pciDevice    - The Id of the device to write to
 @Param[in]     pciFunc      - The function number of the device to write to
 @Param[in]     cfgRegOffset - The offset of the register to write to
 @Param[in]     value        - The value to write

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_ConfigWrite(t_Handle     h_PciCtrl,
                             uint32_t     pciBus,
                             uint32_t     pciDevice,
                             uint32_t     pciFunc,
                             uint32_t     cfgRegOffset,
                             uint32_t     value);


/**************************************************************************//**
 @Function      PCI_CTRL_ConfigRead

 @Description   Issues a maintenance read request to the destination device.

                Use this routine in order to read dword (32 bit) from the device configuration space.

 @Param[in]     h_PciCtrl    - Handle to the PCI object.
 @Param[in]     pciBus       - The bus Id of the device to read from
 @Param[in]     pciDevice    - The Id of the device to read from
 @Param[in]     pciFunc      - The function number of the device to read from
 @Param[in]     cfgRegOffset - The offset of the register to read from
 @Param[out]    p_RetVal     - The read value

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_ConfigRead(t_Handle     h_PciCtrl,
                            uint32_t     pciBus,
                            uint32_t     pciDevice,
                            uint32_t     pciFunc,
                            uint32_t     cfgRegOffset,
                            uint32_t     *p_RetVal);


/**************************************************************************//**
 @Function      PCI_CTRL_SendWakeUpRequest

 @Description   Sends wake up request.

                This routine is for EP usage only.
                The EP can use this routine in order to ask the RC to be
                transffered to D0 state (full pwer mode).

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Param[in]     funcNumber  - Agent's function number to wake up.

 @Return        E_OK - If action was successful, the occurred error otherwise.

*//***************************************************************************/
t_Error  PCI_CTRL_SendWakeUpRequest(t_Handle h_PciCtrl, uint32_t funcNumber);


/**************************************************************************//**
 @Function      PCI_CTRL_SendMsi

 @Description   Sends MSI.

                This routine is for EP usage only.
                The EP can use this routine in order to send
                message signaled interrupt to the host.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Param[in]     msgNum  - Agent's message number.

 @Return        E_OK - If action was successful, the occurred error otherwise.

*//***************************************************************************/
t_Error PCI_CTRL_SendMsi(t_Handle h_PciCtrl, uint32_t msgNum);


/**************************************************************************//**
 @Function      PCI_CTRL_IsMsiSupported

 @Description   MSI support indication.

                This routine returns if the controller supports the MSI capability.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Return        True if MSI supported, False if dosen't.

*//***************************************************************************/
bool PCI_CTRL_IsMsiSupported(t_Handle h_PciCtrl);


/**************************************************************************//**
 @Function      PCI_CTRL_SetErrorsPolicy

 @Description   Sets a new error policy.

                This routine disable/enable interrupts for all PCI's error types.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Param[in]     pciErrType - Error type. Each bit represent different PCI's error

 @Param[in]     pciErrorPolicy - Error Policy. interrupts disabled or enabled.

 @Return        True if MSI supported, False if dosen't.

*//***************************************************************************/
t_Error PCI_CTRL_SetErrorsPolicy (t_Handle h_PciCtrl, uint32_t pciErrType, e_PciErrorPolicy pciErrorPolicy);


/**************************************************************************//**
 @Function      PCI_CTRL_GetMsiirAddr

 @Description   Returns the MSIIR address.

                This routine returns the address of the MSIIR register.
                Used in order to open a window to MSIIR register for MSI messaged.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Return        MSIIR address, 0 if MSI is not supported.

*//***************************************************************************/
t_Error PCI_CTRL_GetMsiirAddr(t_Handle h_PciCtrl);

/**************************************************************************//**
 @Function      PCI_CTRL_EnableAgentConfigCycles

 @Description   Enables EP inbound configuration cycles.

                This routine is for EP usage only.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Return        E_OK - If action was successful, the occurred error otherwise.

*//***************************************************************************/
t_Error PCI_CTRL_EnableAgentConfigCycles(t_Handle h_PciCtrl);

/**************************************************************************//**
 @Function      PCI_CTRL_IsLinkUp

 @Description   Checks if link is up for PCI Express.

 @Param[in]     h_PciCtrl - The handle to the pci object.

 @Return        E_OK - If action was successful, the occurred error otherwise.

*//***************************************************************************/
bool PCI_CTRL_IsLinkUp(t_Handle h_PciCtrl);

/**************************************************************************//**
 @Function      PCI_CTRL_GetSelfPowerState

 @Description   Checks if link is up for PCI Express.

 @Param[in]     h_PciCtrl - The handle to the pci object.
 @Param[out]    p_State - Requested current power state.

 @Return        E_OK - If action was successful, the occurred error otherwise.

*//***************************************************************************/
t_Error PCI_CTRL_GetSelfPowerState(t_Handle h_PciCtrl, e_PciPowerState *p_State);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      PCI_CTRL_DumpRegs

 @Description   Dumps the memory of PCI module

 @Param[in]     h_PciCtrl - PCI handle to the pci object

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PCI_CTRL_DumpRegs(t_Handle h_PciCtrl);
#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of pci_ctrl_control_grp */
/** @} */ /* end of pci_ctrl_grp */

#endif /* __PCI_CTRL_EXT_H */
