/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCIE_CTRL_83XX_EXT_H
#define __PCIE_CTRL_83XX_EXT_H

#include "pci_std_ext.h"
#include "pci_ctrl_ext.h"

/**************************************************************************//**
 @Group         pci_ctrl_grp PCI Controller Application Programming Interface

 @Description   PCI Controller API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pci_ctrl_control_grp PCI Controller Control Unit

 @Description   PCI Controller control functions and definitions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Mapping type attribute for inbound window
*//***************************************************************************/
typedef enum e_PcieMappingType
{
    e_PCIE_INTERRUPT_MAPPING_TYPE               = 0x0,  /**<  Interrupt for mailbox     */
    e_PCIE_IO_MAPPING_TYPE                      = 0x1,  /**<  I/O                       */
    e_PCIE_PREFETCHABLE_MEMORY_MAPPING_TYPE     = 0x2,  /**<  Prefetchable memory       */
    e_PCIE_NON_PREFETCHABLE_MEMORY_MAPPING_TYPE = 0x3   /**<  Non prefetchable memory   */
 
} e_PcieMappingType;

/**************************************************************************//**
 @Description   Window type for outbouns window
*//***************************************************************************/
typedef enum e_PcieWindowType
{
    e_PCIE_CFG_WINDOW_TYPE         = 0x0,  /**<  Configuration  */
    e_PCIE_IO_WINDOW_TYPE          = 0x1,  /**<  I/O            */
    e_PCIE_MEMORY_WINDOW_TYPE      = 0x2   /**<  Memory         */
 
} e_PcieWindowType;

/**************************************************************************//**
 @Description   Inbound window parameters.
*//***************************************************************************/
typedef struct t_PcieInboundAttributes
{
    e_PcieMappingType  type;            /**<  Indicate the type that pex transactions to the window address is mapped to */
    bool               noSnoop;         /**<  Snooping enable for inbound packets                                        */
    bool               noSnoopOverride; /**<  Enable no snoop override; if this bit is set,
                                              the no snoop attribute in the packet is overridden
                                              by the no snoop bit in the PAB_PEX_AMAP CTRL register                      */
} t_PcieInboundAttributes;

/**************************************************************************//**
 @Description   Outbound window parameters
*//***************************************************************************/
typedef struct t_PcieOutboundAttributes
{
    uint8_t             trafficClass;       /**<  Indicate the traffic class of the packet.
                                                  Applicable only if user wants to send traffic using multiple TC
                                                  but single VC.                                                             */
    bool                noSnoop;            /**<  Enable no snoop bit for the packet.                                        */
    bool                relaxedOrderingEn;  /**<  Enable relaxed ordering bit for the packet.                                */
    e_PcieWindowType    windowType;         /**<  Indicate the type that AXI transactions to the window address is maaped to.*/
    void                (*f_WindowErrorHandle)(uint32_t winNum, void *errorAttributes);
                                            /**<  Callback function to handle errors that occured in this window; This functionality
                                                  supported only for PCI's outbound windows; for PCI Express, f_WindowErrorHandle should be NULL.*/

} t_PcieOutboundAttributes;



/**************************************************************************//**
 @Function      PCIE_CTRL_SetReadTimeout

 @Description   Set read timeout.
 
                Set the maximum timeout value for which the descriptor read engine
                expect the read response to be available.
                if a descriptor read engine initiate a descriptor fetch operation
                on AXI and there is no respone withine the timeout value, an error is flagged.
 
 @Param[in]     h_PciCtrl - Handle to the PCI controller object.

 @Param[in]     timeout     - The maximum number of clocks that a descriptor
                              read engine shuold wait for read response.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error PCIE_CTRL_SetReadTimeout(t_Handle h_PciCtrl, uint32_t timeout);

t_Error PCIE_CTRL_OpenCfgWindow(t_Handle h_PciCtrl, uint32_t translation);


/**************************************************************************//**
 @Function      PCIE_CTRL_SetReadyTimer

 @Description   Set ready timer.
 
                Set the timer value the DMA engine should wait for
                before reading the next descriptor once it encounters
                a descriptor that is not ready.
 
 @Param[in]     h_PciCtrl   - Handle to the PCI controller object.
 @Param[in]     readyTimer  - The number of clocks that a DMA engine should wait to
                              before checking if the next descriptor is ready when it
                              encounters invalid descriptor.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error PCIE_CTRL_SetReadyTimer(t_Handle h_PciCtrl, uint32_t readyTimer);

/**************************************************************************//**
 @Function      PCIE_CTRL_GetNumOfInboundWindows

 @Description   Retrive the number of controller's inbound windows.

 @Return        Controller's inbound windows number.

*//***************************************************************************/
uint32_t PCIE_CTRL_GetNumOfInboundWindows(void);

/**************************************************************************//**
 @Function      PCIE_CTRL_GetNumOfOutboundWindows

 @Description   Retrive the number of controller's outbound windows.

 @Return        Controller's outbound windows number.

*//***************************************************************************/
uint32_t PCIE_CTRL_GetNumOfOutboundWindows(void);


/** @} */ /* pci_ctrl_control_grp */

/**************************************************************************//**
 @Group         pci_ctrl_exception_grp PCI Controller Exceptions Unit

 @Description   PCI Controller exceptions functions.

 @{
*//***************************************************************************/

/***************************************************************************//**
 @Description   PCIE Controller Error attributes

                This structure contains information about errors that occured,
                including more detailed information about the last error.
*//***************************************************************************/
typedef struct t_PcieErrorAttributes
{
    bool parityError;               /**< Parity error - A poisoned received                                         */
    bool systemError;               /**< System error occured                                                       */
    bool masterAbort;               /**< The requestor receives a completion with unsupported request completion    */
    bool receivedTargetAbort;       /**< Completion with completer abort completion received                        */
    bool signaledTargetAbort;       /**< Set whenever a device completes a request using completer abort completion status  */
    bool masterDataParityError;     /**< Set by the requestor (primary side for Type1 headers) when either the
                                         requestor receives a completion
                                         marked poisoned or the requestor poisons
                                         a write request; Note that the parity error enable bit (bit 6)
                                         in the command register must be set for this bit to be set.                */
 
    bool pexIntRDMATransAbort;      /**< PEX Read DMA transaction aborted                     */
    bool pexIntWDMATransAbort;      /**< PEX Write DMA transaction aborted                    */
    bool pexIntPexPioTransAbort;    /**< PEX PIO transaction aborted on PEX side              */
    bool pexIntAxiPioTransAbort;    /**< AXI PIO transaction aborted on PEX side              */

    bool axiIntRDMATransAbort;      /**< AXI Read DMA transaction aborted                     */
    bool axiIntWDMATransAbort;      /**< AXI Write DMA transaction aborted                    */
    bool axiIntPexPioTransAbort;    /**< PEX PIO transaction aborted on AXI side              */
    bool axiIntAxiPioTransAbort;    /**< AXI PIO transaction aborted on AXI side              */

    bool overflowError;             /**< PCIE received overflow error.                      */
    bool ECRCError;                 /**< TLP is received by PCIE and it fails ECRC check    */
    bool poisonedTLP;               /**< Poisoned TLP is received by PCI Express            */
    bool malformedTLP;              /**< Malformed TLP is received by PCI Express            */
    bool completionTimeout;         /**< PCIE completion timeout occurs                        */
    bool completerAbort;            /**< A completer abort error is detected by PCIE        */
    bool unsupportedRequest;        /**< An unsupported request error is detected by PCIE    */
    bool INTD;                      /**< INTD interrupt is received on the PEX link         */
    bool INTC;                      /**< INTC interrupt is received on the PEX link         */
    bool INTB;                      /**< INTB interrupt is received on the PEX link         */
    bool INTA;                      /**< INTA interrupt is received on the PEX link         */

} t_PcieErrorAttributes;

/** @} */ /* end of pci_ctrl_exception_grp */
/** @} */ /* end of pci_ctrl_grp */

#endif __PCIE_CTRL_83XX_EXT_H
