/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __PCIE_CTRL_85XX_EXT_H
#define __PCIE_CTRL_85XX_EXT_H

#include "pci_ctrl_ext.h"

/**************************************************************************//**
 @Group         pci_ctrl_grp PCI Controller Application Programming Interface

 @Description   PCI Controller API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pci_ctrl_control_grp PCI Controller Control Unit

 @Description   PCI Controller control functions and definitions

 @{
*//***************************************************************************/

#define PCIE_INBOUND_WINDOWS_NUM       4  /**< Number of inbound windows  */
#define PCIE_OUTBOUND_WINDOWS_NUM      5  /**< Number of outbound windows */


/**************************************************************************//**
 @Description  The traffic class of the outbound packet.
*//***************************************************************************/
typedef enum e_PcieTrafficClass
{
     e_PCIE_TC0,    /**< Traffic class 0 */
     e_PCIE_TC1,    /**< Traffic class 1 */
     e_PCIE_TC2,    /**< Traffic class 2 */
     e_PCIE_TC3,    /**< Traffic class 3 */
     e_PCIE_TC4,    /**< Traffic class 4 */
     e_PCIE_TC5,    /**< Traffic class 5 */
     e_PCIE_TC6,    /**< Traffic class 6 */
     e_PCIE_TC7     /**< Traffic class 7 */
} e_PcieTrafficClass;


/**************************************************************************//**
 @Description   Read transaction type for PCI Express windows.
*//***************************************************************************/
typedef enum e_PcieOutboundReadTransType
{
    e_PCIE_OUTBOUND_RTT_READ_CONFIGURATION  = 0x2,  /**< Configuration read; Supported only when in RC mode and size of less than or equal to 4 bytes and not
                                                         crossing 4-byte address boundary.  */
    e_PCIE_OUTBOUND_RTT_READ_MEMORY         = 0x4,  /**< Memory read                        */
    e_PCIE_OUTBOUND_RTT_READ_IO             = 0x8   /**< IO read; Supported only when in RC mode and size of less than or equal to 4 bytes and not crossing 4-byte
                                                         address boundary.                  */
} e_PcieOutboundReadTransType;

/**************************************************************************//**
 @Description   Write transaction type for PCI Express windows.
*//***************************************************************************/
typedef enum e_PcieOutboundWriteTransType
{
    e_PCIE_OUTBOUND_WTT_WRITE_CONFIGURATION = 0x2,  /**< Configuration write; Supported only when in RC mode and size of less than or equal to 4 bytes and not
                                                         crossing 4-byte address boundary  */
    e_PCIE_OUTBOUND_WTT_WRITE_MEMORY        = 0x4,  /**< Memory write                      */
    e_PCIE_OUTBOUND_WTT_WRITE_MESSAGE       = 0x5,  /**< Message write; Only support 4-byte size access on a 4-byte address boundary.*/
    e_PCIE_OUTBOUND_WTT_WRITE_IO            = 0x8   /**< IO Write; Supported only when in RC mode and size of less than or equal to 4 bytes and not crossing
                                                         4-byte address boundary           */
} e_PcieOutboundWriteTransType;

/**************************************************************************//**
 @Description  Outbound window attribute
*//***************************************************************************/
typedef struct t_PcieOutboundAttributes
{
    e_PcieTrafficClass           trafficClass;      /**< Target interface;
                                                         This field only applies to memory transaction                              */
    e_PcieOutboundReadTransType  rtt;               /**< Read transaction type                         */
    e_PcieOutboundWriteTransType wtt;               /**< Write transaction type                        */
    bool                         relaxedOrdering;   /**< Relaxed ordering enable;
                                                         This bit only applies to memory transactions  */
    bool                         noSnoop;           /**< No snoop enable;
                                                         This bit only applies to memory transactions  */
} t_PcieOutboundAttributes;


/**************************************************************************//**
 @Description   Read transaction type for PCI Express windows.
*//***************************************************************************/
typedef enum e_PcieInboundReadTransType
{
    e_PCIE_INBOUND_RTT_READ_NO_SNOOP             = 0x4,  /**< Do not snoop at read transaction                     */
    e_PCIE_INBOUND_RTT_READ_SNOOP                = 0x5,  /**< Snoop at read transaction                            */
    e_PCIE_INBOUND_RTT_READ_SNOOP_UNLOCK_CACHE   = 0x7   /**< Snoop at read transaction, and unlock L2 cache line  */
} e_PcieInboundReadTransType;

/**************************************************************************//**
 @Description   Write transaction type for PCI Express windows.
*//***************************************************************************/
typedef enum e_PcieInboundWriteTransType
{
    e_PCIE_INBOUND_WTT_WRITE_NO_SNOOP            = 0x4,  /**< Do not snoop at write transaction                            */
    e_PCIE_INBOUND_WTT_WRITE_SNOOP               = 0x5,  /**< Snoop at write transaction                                   */
    e_PCIE_INBOUND_WTT_WRITE_SNOOP_ALLOC_CACHE   = 0x6,  /**< Snoop at write transaction, and allocate L2 cache line       */
    e_PCIE_INBOUND_WTT_WRITE_SNOOP_LOCK_CACHE    = 0x7   /**< Snoop at write transaction, allocate and lock L2 cache line  */
} e_PcieInboundWriteTransType;

/**************************************************************************//**
 @Description  Inbound window attribute
*//***************************************************************************/
typedef struct t_PcieInboundAttributes
{
    e_PciTargetInterface        target;         /**< Target interface                              */
    e_PcieInboundReadTransType  rtt;            /**< Read transaction type                         */
    e_PcieInboundWriteTransType wtt;            /**< Write transaction type                        */
    bool                        prefetchable;   /**< Indicate if the window is prefetchable or not */

} t_PcieInboundAttributes;


/**************************************************************************//**
 @Function      PCIE_SetOutboundCompletionTimeout.

 @Description   Sets the timeout value for outbound completion.

 @Param[in]     h_PciCtrl   - Handle to the PCI controller object.
 @Param[in]     timeoutCnt  - Preload timer value

 @Return        E_OK - if action was successful, the occurred error otherwise.
*//***************************************************************************/
t_Error PCIE_CTRL_SetOutboundCompletionTimeout(t_Handle h_PciCtrl, uint32_t timeoutCnt);

/** @} */ /* end of pci_ctrl_control_grp */

/**************************************************************************//**
 @Group         pci_ctrl_exception_grp PCI Controller Exceptions Unit

 @Description   PCI Controller exceptions functions and definitions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    PCIe Error masks

 @{
*//***************************************************************************/
#define PCIE_ERROR_MUPLTIPLE    0x80000000  /**< Multiple errors                               */
#define PCIE_ERROR_PCT          0x00800000  /**< PCI Express completion time-out               */
#define PCIE_ERROR_PCAC         0x00200000  /**< PCI Express completer abort (CA) completion.  */
#define PCIE_ERROR_PNM          0x00100000  /**< PCI Express no map.                           */
#define PCIE_ERROR_CDNSC        0x00080000  /**< Completion with data not successful.          */
#define PCIE_ERROR_CRSNC        0x00040000  /**< CRS non configuration.                        */
#define PCIE_ERROR_ICCA         0x00020000  /**< Invalid PCIE_CONFIG_ADDR/PCIE_CONFIG_DATA configuration access. */
#define PCIE_ERROR_IACA         0x00010000  /**< Invalid ATMU configuration access.            */
#define PCIE_ERROR_CRST         0x00008000  /**< CRS thresholded.                              */
#define PCIE_ERROR_MIS          0x00004000  /**< Message invalid size.                         */
#define PCIE_ERROR_IOIS         0x00002000  /**< I/O invalid size.                             */
#define PCIE_ERROR_CIS          0x00001000  /**< Configuration invalid size.                   */
#define PCIE_ERROR_CIEP         0x00000800  /**< Configuration invalid EP.                     */
#define PCIE_ERROR_IOIEP        0x00000400  /**< I/O invalid EP                                */
#define PCIE_ERROR_OAC          0x00000200  /**< Outbound ATMU crossing.                       */
#define PCIE_ERROR_IOIA         0x00000100  /**< I/O invalid address.                          */
/* @} */

/***************************************************************************//**
 @Description   PCI error source

                Indicates the source of the first error captured.
*//***************************************************************************/
typedef enum e_PcieErrorSrc
{
    e_PCIE_PCI_ERR_SRC                      = 0x0,      /**< PCI source                            */
    e_PCIE_PCI_EXPRESS_ERR_SRC              = 0x10,     /**< PCI Express (for inbound transaction) */
    e_PCIE_SECURITY_ERR_SRC                 = 0x111,    /**< Security source source                */
    e_PCIE_BOOT_SEQUENCER_ERR_SRC           = 0x1010,   /**< Boot sequencer source                 */
    e_PCIE_SRIO_ERR_SRC                     = 0x1100,   /**< Serial RapidIO source                 */
    e_PCIE_E500_CORE_INSTRUCTION_ERR_SRC    = 0x10000,  /**< Processor instruction source          */
    e_PCIE_E500_CORE_DATA_ERR_SRC           = 0x10001,  /**< Processor data source                 */
    e_PCIE_DMA_ERR_SRC                      = 0x10101,  /**< DMA source                            */
    e_PCIE_TSEC1_ERR_SRC                    = 0x11000,  /**< eTSEC1 source                         */
    e_PCIE_TSEC2_ERR_SRC                    = 0x11001,  /**< eTSEC2 source                         */
    e_PCIE_RAPIDIO_MSG_ERR_SRC              = 0x11100   /**< RapidIO message/doorbell/port source  */

} e_PcieErrorSrc;

/***************************************************************************//**
 @Description   PCI Express Transaction originator

                Indicates whether the originator of the transaction is from
                PEX_CONFIG_ADDR/PEX_CONFIG_DATA.
*//***************************************************************************/
typedef enum e_PcieTransOriginator
{
    e_PEX_CONFIG_REGS_ORIGINATED        = 0x0,  /**< Transaction originated from PEX_CONFIG_ADDR/PEX_CONFIG_DATA.    */
    e_PEX_NON_CONFIG_REGS_ORIGINATED    = 0x1   /**< Transaction not originated from PEX_CONFIG_ADDR/PEX_CONFIG_DATA.*/

} e_PcieTransOriginator;

/***************************************************************************//**
 @Description   PCI Controller Error attribures

                This structure contains information about errors that occured,
                including more detailed information about the last error.
*//***************************************************************************/
typedef struct t_PcieErrorAttributes
{
    e_PcieTransOriginator   TransactionOriginator;  /**< This field Indicates whether the
                                                         originator of the transaction is
                                                         from PEX_CONFIG_ADDR/PEX_CONFIG_DATA.         */
    e_PcieErrorSrc          errorSource;            /**< This field indicates the internal platform
                                                         global source ID that the error transaction
                                                         originates; This field only applies to non
                                                         PEX_CONFIG_ADDR/PEX_CONFIG_DATA transactions. */
    bool                    multipleError;          /**< Multiple errors of the same type.             */
    bool                    pexCompletionTimeout;   /**< PCI Express completion time-out -
                                                         A completion time-out condition was
                                                         detected for a non-posted outbound PCI
                                                         Express transaction.                          */
    bool                    pexCompleterAbortcompletion;
                                                    /**< PCI Express completer abort (CA) completion -
                                                         A completion with CA status was received.     */
    bool                    pexNoMap;               /**< PCI Express no map; Detect an inbound
                                                         transaction that was not mapped to any
                                                         inbound window.                               */
    bool                    completionDataNotSuccessful;/**< Completion with data not successful.      */

    bool                    crsNonConfiguration;    /**< CRS non configuration - A completion was
                                                         detected for a non configuration cycle and
                                                         with CRS status.                              */
    bool                    invalidConfigAccess;    /**< Invalid PEX_CONFIG_ADDR/PEX_CONFIG_DATA
                                                         configuration access.                         */
    bool                    ivalidAtmuConfigAccess; /**< Invalid ATMU configuration access.            */
    bool                    crsThresholded;         /**< CRS thresholded; An outbound configuration
                                                         transaction was retried and thresholded due
                                                         to a CRS completion status.                   */
    bool                    msgInvalidSize;         /**< Message invalid size - An outbound message
                                                         transaction that is greater than 4 bytes or
                                                         crosses a 4-byte boundary was detected.       */
    bool                    ioInvalidSize;          /**< I/O invalid size; An outbound I/O transaction
                                                         that is greater than 4 bytes or crosses a
                                                         4-byte boundary was detected.                 */
    bool                    configurationInvalidSize;/**< Configuration invalid size.                  */
    bool                    configurationInvalidEP; /**< Configuration invalid EP.                     */
    bool                    ioInvalidEP;            /**< I/O invalid EP; An outbound I/O transaction
                                                         request was seen when in EP mode.             */
    bool                    outboundAtmuCrossing;   /**< Outbound ATMU crossing; An outbound crossing
                                                         ATMU transaction was detected.                */
    bool                    ioInvalidAddress;       /**< I/O invalid address; An outbound I/O
                                                         transaction with a translated address of
                                                         greater than 4G was detected.                 */
} t_PcieErrorAttributes;


/** @} */ /* end of pci_ctrl_exception_grp */
/** @} */ /* end of pci_ctrl_grp */

#endif /* __PCIE_CTRL_85XX_EXT_H */
