/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          pm_ext.h

 @Description   performance monitoring external structures and definitions.
*//***************************************************************************/
#ifndef __PM_EXT_H
#define __PM_EXT_H

#include "pm_integration_ext.h"


/**************************************************************************//**
 @Group         pm_grp Performance Monitor Application Programming Interface

 @Description   Performance Monitor API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         pm_init Performance Monitor Initialization Unit


 @Description   Performance Monitor initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Threshold and Burstiness multiplier valid values.
*//***************************************************************************/
typedef enum e_PmTBMult
{
      e_PM_TB_MULT_1   = 0x0
    , e_PM_TB_MULT_2   = 0x1
    , e_PM_TB_MULT_4   = 0x2
    , e_PM_TB_MULT_8   = 0x3
    , e_PM_TB_MULT_16  = 0x4
    , e_PM_TB_MULT_32  = 0x5
    , e_PM_TB_MULT_64  = 0x6
    , e_PM_TB_MULT_128 = 0x7
} e_PmTBMult;


/**************************************************************************//**
 @Description   Trigger Control options.
*//***************************************************************************/
typedef enum e_PmTrigCntl
{
      e_PM_TRIGGER_OFF         = 0x0
    , e_PM_TRIGGER_ON_CHANGE   = 0x1
    , e_PM_TRIGGER_ON_OVERFLOW = 0x2
 
} e_PmTrigCntl;

/**************************************************************************//**
 @Description    PM event setup parameters, used with PM_SetupCounters().
 *//***************************************************************************/
typedef struct t_PmEventParam
{
    e_PmEvent      event;
    e_PmTBMult     pmTBMult;         /**< Threshold and burstiness multiplier */
    uint8_t        threshold;        /**< Threshold - events whose number of occurences exceed this value are counted */
    uint8_t        burstSize;        /**< Fewest event occurences that constitute a burst */
    uint8_t        burstGranularity; /**< Maximum number of clock cycles between events that are considered part of a single burst */
    uint8_t        burstDistance;    /**< Number of clock cycles between bursts (used with threshold and burstiness multiplier) */
    e_PmTrigCntl   pmTrigOnCntl;     /**< Trigger-on control - indicates the condition under which triggering to start counting occurs */
    e_PmEvent      pmTrigOnSel;      /**< Trigger-on select - choose the event that should trigger event counting to start */
    e_PmTrigCntl   pmTrigOffCntl;    /**< Trigger-off control - indicates the condition under which triggering to stop counting occurs */
    e_PmEvent      pmTrigOffSel;     /**< Trigger-off select - choose the event that should trigger event counting to start */

} t_PmEventParam;

/**************************************************************************//**
 @Description    PM configuration parameters structure.
 *//***************************************************************************/
typedef struct t_PmParam
{
    uint32_t    baseAddress;    /**< Base address of PM registers in memory */
    int         irq;            /**< PM interrupt ID (system-specific number) */

} t_PmParam;


/**************************************************************************//**
 @Function      PM_ConfigAndInit

 @Description   Creates a PM descriptor.
 
 @Param[in]     p_PmParam - Pointer to PM configuration parameters structure.
 
 @Return        Handle to a PM descriptor on success, NULL on error
*//***************************************************************************/
t_Handle PM_ConfigAndInit(t_PmParam *p_PmParam);

/**************************************************************************//**
 @Function      PM_SetupCounters

 @Description   Fully initializes PM and enables it.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_ConfigAndInit()
 @Param[in]     numOfEvents - number of PM events (should be greater than 0)
 @Param[in]     p_PmEventParam - PM event parameters array in the size of numOfEvents

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PM_SetupCounters(t_Handle h_Pm, uint8_t numOfEvents, t_PmEventParam *p_PmEventParam);

/**************************************************************************//**
 @Function      PM_Free

 @Description   Frees all resources used by PM.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_Config()

 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error PM_Free(t_Handle h_Pm);

/** @} */ /* end of pm_init */

/**************************************************************************//**
 @Group         pm_control Performance Monitor Control Unit

 @Description   Performance Monitor Runtime Control functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PM_Start

 @Description   Start counting all the events.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_ConfigAndInit()

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void PM_Start(t_Handle h_Pm);

/**************************************************************************//**
 @Function      PM_Stop

 @Description   Stop counting all initialized events.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_ConfigAndInit()

 @Return        None.
*//***************************************************************************/
void PM_Stop(t_Handle h_Pm);

/**************************************************************************//**
 @Function      PM_GetCounter

 @Description   Get counter of a specific event.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_ConfigAndInit()
 @Param[in]     event - the required event.

 @Return        64 bit counter.
*//***************************************************************************/
uint64_t PM_GetCounter(t_Handle h_Pm, e_PmEvent event);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      PM_DumpRegs

 @Description   Dumps PM registers.

 @Param[in]     h_Pm - Handle to the PM descriptor returned by PM_ConfigAndInit()

 @Return        None.

 @Cautions      h_Pm must have been previously created using PM_ConfigAndInit().
*//***************************************************************************/
void PM_DumpRegs(t_Handle h_Pm);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of pm_control */
/** @} */ /* end of pm_grp */


#endif /* __PM_EXT_H */

