/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          pppos_link_ext.h

 @Description   PPPOS Link external structures and definitions.
*//***************************************************************************/
#ifndef __PPPOS_LINK_EXT_H
#define __PPPOS_LINK_EXT_H


#include "ucc_pos_port_ext.h"


/**************************************************************************//**
 @Group         ucc_pos_grp UCC POS Application Programming Interface

 @Description   UCC POS functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pppos_link_grp PPPOS Link API

 @Description   PPPOS Link functions, definitions and enumerators

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         pppos_link_initialization_grp PPPOS Link Initialization Unit

 @Description   PPPOS Link initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   General definitions
*//***************************************************************************/
#define PPPOS_LINK_RX_PPP_QUEUE_ID 0
#define PPPOS_LINK_RX_LCP_QUEUE_ID 2
#define PPPOS_LINK_TX_LCP_MIN_PID  0xC000


/**************************************************************************//**
 @Description   Rx status flags
*//***************************************************************************/
#define PPPOS_LINK_RX_STATUS_SEGMENT_SIZE_ERROR (1<< 8)
#define PPPOS_LINK_RX_STATUS_IP_CHECKSUM_ERROR  (1<< 7)
#define PPPOS_LINK_RX_STATUS_FRAME_TOO_LONG     (1<< 6)
#define PPPOS_LINK_RX_STATUS_CRC_ERROR          (1<< 5)
#define PPPOS_LINK_RX_STATUS_FRAME_TOO_SHORT    (1<< 4)
#define PPPOS_LINK_RX_STATUS_GENERAL_ERROR      (1<< 3)
#define PPPOS_LINK_RX_STATUS_FRAME_PARITY_ERROR (1<< 2)
#define PPPOS_LINK_RX_STATUS_FIFO_OVERRUN       (1<< 1)
#define PPPOS_LINK_RX_STATUS_PROTOCOL_ERROR     (1<< 0)


/**************************************************************************//**
 @Description   Tx confirmation user callback routine.
                Invoked by driver upon the transmission of each frame.

 @Param[in]     h_App          - User's application descriptor.
 @Param[in]     txDataQueueNum - Tx data queue number.
 @Param[in]     p_Data         - A pointer to the confirmed data buffer.
 @Param[in]     h_BufContext   - The user's private context associated with the given data buffer
 @Param[in]     lastBuffer     - Indicates whether this is the last buffer of the confirmed frame.
*//***************************************************************************/
typedef void (t_PpposTxConfRoutine)(t_Handle h_App,
                                    uint8_t  txDataQueueNum,
                                    uint8_t* p_Data,
                                    t_Handle h_BufContext);

/**************************************************************************//**
 @Description   Rx storing user callback routine.
                Invoked by driver upon the reception of each frame.

 @Param[in]     h_App          - User's application descriptor.
 @Param[in]     rxDataQueueNum - Rx data queue number.
 @Param[in]     p_Data         - A pointer to the data buffer.
 @Param[in]     h_BufContext   - The user's private context associated with the given data buffer.
 @Param[in]     length         - If not the last buffer:
                                    The number of valid bytes in the buffer.
                                 If the last buffer:
                                    If if the data is not NULL:
                                        The number of bytes in the entire frame, excluding FCS.
                                    If the data is NULL:
                                        The number of bytes that the former store invokation of this queue has exceeded.
 @Param[in]     status         - A bitwise-or between any of the following flags (Relevant only for the last buffer of the frame):
                                 PPPOS_LINK_RX_STATUS_SEGMENT_SIZE_ERROR
                                 PPPOS_LINK_RX_STATUS_IP_CHECKSUM_ERROR
                                 PPPOS_LINK_RX_STATUS_FRAME_TOO_LONG
                                 PPPOS_LINK_RX_STATUS_CRC_ERROR
                                 PPPOS_LINK_RX_STATUS_FRAME_TOO_SHORT
                                 PPPOS_LINK_RX_STATUS_GENERAL_ERROR
                                 PPPOS_LINK_RX_STATUS_FRAME_PARITY_ERROR
                                 PPPOS_LINK_RX_STATUS_FIFO_OVERRUN
                                 PPPOS_LINK_RX_STATUS_PROTOCOL_ERROR
 @Param[in]     position       - Buffer poistion (LAST_BUF, FIRST_BUF, SINGLE_BUF, MIDDLE_BUF).
 @Param[in]     pid            - The PID (protocol identifier) of the incomming frame.
                                 When the parser is not in use, queue 0 receives plain PPP frames and queue 2 receives LCP frames.

 @Return        Continue Rx operation for all the data in the queue or not.
*//***************************************************************************/
typedef e_RxStoreResponse (t_PpposRxStoreRoutine)(t_Handle h_App,
                                                  uint8_t  rxDataQueueNum,
                                                  uint8_t* p_Data,
                                                  t_Handle h_BufContext,
                                                  uint32_t length,
                                                  uint16_t status,
                                                  uint8_t  position,
                                                  uint16_t pid);


/**************************************************************************//**
 @Description   Rx recovery user callback routine.
                Invoked by driver upon any error in the reception of a frame.

 @Param[in]     h_App          - User's application descriptor.
 @Param[in]     rxDataQueueNum - Rx data queue number.
 @Param[in]     rxErrorFlags   - Rx error flags.
*//***************************************************************************/
typedef void (t_PpposRxRecoverRoutine) (t_Handle h_App,
                                        uint8_t  rxDataQueueNum,
                                        uint16_t rxErrorFlags);


/**************************************************************************//**
 @Description   Configuration parameters
*//***************************************************************************/
typedef struct t_PpposLinkParam
{
    e_UpcDevice              devId;              /**< ID of the device over which the link is running                                                  */
    uint8_t                  portId;             /**< ID of the port over which the link is running                                                    */
    t_Handle                 h_UccPos;           /**< Handle to the UCC POS of the link                                                                */
    t_BufferPoolInfo         bufferPoolInfo;     /**< Data buffer pool information                                                                     */
    uint32_t                 dataMemAttributes;  /**< Memory attributes for data buffers                                                               */
 
    t_PpposTxConfRoutine*    f_TxConfRoutine;    /**< User's callback routine invoked by driver upon the transmission of each frame                    */
    t_PpposRxStoreRoutine*   f_RxStoreRoutine;   /**< User's callback routine invoked by driver upon the reception of each frame                       */
    t_PpposRxRecoverRoutine* f_RxRecoverRoutine; /**< User's callback routine invoked by driver upon any error in the reception of a frame             */
    t_Handle                 h_App;              /**< Handle to an application context; Passed to the application through the user's callback routines */
}
t_PpposLinkParam;


/**************************************************************************//**
 @Function      PPPOS_LINK_Config

 @Description   Configures a PPP over SONET link.

 @Param[in]     p_PpposLinkParam - PPP over SONET link configuration parameters.

 @Return        Handle to the PPP over SONET link on success; NULL otherwise.
*//***************************************************************************/
t_Handle PPPOS_LINK_Config(t_PpposLinkParam* p_PpposLinkParam);


/**************************************************************************//**
 @Function      PPPOS_LINK_Init

 @Description   Initializes the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_Init(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_Free

 @Description   Deallocates the link object + memory allocated during its configuration and initialization.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_Free(t_Handle h_PpposLink);


/**************************************************************************//**
 @Group         pppos_link_advance_config_grp PPPOS Link Advanced Configuration Unit

 @Description   PPPOS Link advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigInternalLoopbackMode

 @Description   Configures the Internal Loopback mode of the link.

 @Param[in]     h_PpposLink          - Handle to the link object.
 @Param[in]     internalLoopbackMode - When TRUE, internal loopback is enabled.
                The default Internal Loopback mode of the link is [FALSE].

 @Cautions      Internal loopback can be enabled only when the link is running over port 0 on device 0.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigInternalLoopbackMode(t_Handle h_PpposLink,bool internalLoopbackMode);


/**************************************************************************//**
 @Description   CRC types
*//***************************************************************************/
typedef enum e_PpposLinkCrcType
{
    e_PPPOS_LINK_NO_CRC     = e_UCC_POS_PORT_NO_CRC,     /**< No CRC     generation/verification */
    e_PPPOS_LINK_16_BIT_CRC = e_UCC_POS_PORT_16_BIT_CRC, /**< 16 bit CRC generation/verification */
    e_PPPOS_LINK_32_BIT_CRC = e_UCC_POS_PORT_32_BIT_CRC  /**< 32 bit CRC generation/verification */
}
e_PpposLinkCrcType;


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigCrcType

 @Description   Configures the CRC type of the link.

 @Param[in]     h_PpposLink - Handle to the link object.
 @Param[in]     crcType     - Type of CRC generated upon Tx and verified upon Rx.
                The default CRC type of the link is [e_PPPOS_LINK_16_BIT_CRC].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigCrcType(t_Handle h_PpposLink,e_PpposLinkCrcType crcType);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigIpChecksumMode

 @Description   Configures the IP Checksum mode of the link.

 @Param[in]     h_PpposLink    - Handle to the link object.
 @Param[in]     ipChecksumMode - When TRUE, IP checksum is generated upon Tx and verified upon Rx.
                The default IP Checksum mode of the link is [FALSE].

 @Cautions      IP Checksum can be generated upon Tx and verified upon Rx only when the link's Rx parser is enabled.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigIpChecksumMode(t_Handle h_PpposLink,bool ipChecksumMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigAcfcMode

 @Description   Configures the ACFC (Address-And-Control Character Field Compression) mode of the link.

 @Param[in]     h_PpposLink - Handle to the link object.
 @Param[in]     acfcMode    - When TRUE, ACFC is enabled upon Tx and enabled upon Rx.
                The default ACFC (Address-And-Control Character Field Compression) mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigAcfcMode(t_Handle h_PpposLink,bool acfcMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigPfcMode

 @Description   Configures the PFC (Protocol-ID Field Compression) mode of the link.

 @Param[in]     h_PpposLink - Handle to the link object.
 @Param[in]     pfcMode     - When TRUE, PFC is enabled upon Tx and enabled upon Rx.
                The default PFC (Protocol-ID Field Compression) mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigPfcMode(t_Handle h_PpposLink,bool pfcMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxErrorSignalMode

 @Description   Configures the Error Signal mode of the link.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     txErrorSignalMode - When TRUE, an error signal is generated upon Tx (on the POS L2 bus, at the end of every frame).
                The default Error Signal mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxErrorSignalMode(t_Handle h_PpposLink,bool txErrorSignalMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxExtendedSchedMode

 @Description   Configures the Extended Scheduling mode of the link.
                Extended Scheduling mode supports two features:
                Tx traffic scheduling (using weights for each data queue).
                Tx traffic shaping (using rate limits for each data queue).

 @Param[in]     h_PpposLink         - Handle to the link object.
 @Param[in]     txExtendedSchedMode - When TRUE, extended scheduling is enabled upon Tx.
                The default Extended Scheduling mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxExtendedSchedMode(t_Handle h_PpposLink,bool txExtendedSchedMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxRegulationMode

 @Description   Configures the Tx Regulation mode of the link.
                Regulation can be applied when the link's UCC reaches the
                maximum number of links that can be in Tx process simultaneously.

 @Param[in]     h_PpposLink      - Handle to the link object.
 @Param[in]     txRegulationMode - When TRUE, regulation is enabled upon Tx.
                The default Tx Regulation mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxRegulationMode(t_Handle h_PpposLink,bool txRegulationMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxFrameEventsMode

 @Description   Configures the Frame Events mode of the link.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     txFrameEventsMode - When TRUE, frame events are enabled upon Tx.
                The default Frame Events mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxFrameEventsMode(t_Handle h_PpposLink,bool txFrameEventsMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxFrameEventsMode

 @Description   Configures the Frame Events mode of the link.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     rxFrameEventsMode - When TRUE, frame events are enabled upon Rx.
                The default Frame Events mode of the link is [FALSE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxFrameEventsMode(t_Handle h_PpposLink,bool rxFrameEventsMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxErrorEventsMode

 @Description   Configures the Error Events mode of the link.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     rxErrorEventsMode - When TRUE, error events are enabled upon Rx.
                The default Error Events mode of the link is [TRUE].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxErrorEventsMode(t_Handle h_PpposLink,bool rxErrorEventsMode);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxStatCounters

 @Description   Configures the Tx statistical counters.

 @Param[in]     h_PpposLink        - Handle to the link object.
 @Param[in]     txStatCountersMode - When TRUE, Tx statistical counters are enabled.
 @Param[in]     memId              - Memory partition ID for link's Tx statistical counters.
 @Param[in]     memAttributes      - Memory attributes for the link's Tx statistical counters.

 @Cautions      When the link's UCC is in Single-Port mode, Tx statistical counters must reside in internal memory.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxStatCounters(t_Handle h_PpposLink,bool txStatCountersMode,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxStatCounters

 @Description   Configures the Rx statistical counters.

 @Param[in]     h_PpposLink        - Handle to the link object.
 @Param[in]     rxStatCountersMode - When TRUE, Rx statistical counters are enabled.
 @Param[in]     memId              - Memory partition ID for link's Rx statistical counters.
 @Param[in]     memAttributes      - Memory attributes for the link's Rx statistical counters.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxStatCounters(t_Handle h_PpposLink,bool rxStatCountersMode,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxDataQueuesMemory

 @Description   Configures memory information for the link's Tx data queues.

                The default location of the Tx data queues is according
                to the configuration of the structure partition in the UCC POS common object.

 @Param[in]     h_PpposLink   - Handle to the link object.
 @Param[in]     memId         - Memory partition ID for link's Tx data queues.
 @Param[in]     memAttributes - Memory attributes for the link's Tx data queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxDataQueuesMemory(t_Handle h_PpposLink,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxDataQueuesMemory

 @Description   Configures memory information for the link's Rx data queues.

                The default location of the Rx data queues is according
                to the configuration of the structure partition in the UCC POS common object.

 @Param[in]     h_PpposLink   - Handle to the link object.
 @Param[in]     memId         - Memory partition ID for link's Rx data queues.
 @Param[in]     memAttributes - Memory attributes for the link's Rx data queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxDataQueuesMemory(t_Handle h_PpposLink,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxDataQueueDescTableMemory

 @Description   Configures memory information for the link's Rx data queue descriptors.

                The default location of the Rx data queue descriptors is according
                to the configuration of the structure partition in the UCC POS common object.

 @Param[in]     h_PpposLink   - Handle to the link object.
 @Param[in]     memId         - Memory partition ID for link's Rx data queue descriptors.
 @Param[in]     memAttributes - Memory attributes for the link's Rx data queue descriptors.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxDataQueueDescTableMemory(t_Handle h_PpposLink,uint8_t memId,uint32_t memAttributes);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxNumOfDataQueues

 @Description   Configures the Tx number of data queues.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     txNumOfDataQueues - Tx number of data queues.
                The default Tx number of data queues is [1].

 @Cautions      txNumOfDataQueues cannot be less than [MIN_VAL_txNumOfDataQueues].
                txNumOfDataQueues cannot be more than [MAX_VAL_txNumOfDataQueues].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxNumOfDataQueues(t_Handle h_PpposLink,uint8_t txNumOfDataQueues);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxNumOfDataQueues

 @Description   Configures the Rx number of data queues.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     rxNumOfDataQueues - Rx number of data queues.
                The default Rx number of data queues is [3].

 @Cautions      rxNumOfDataQueues cannot be less than [MIN_VAL_rxNumOfDataQueues].
                rxNumOfDataQueues cannot be more than [MAX_VAL_rxNumOfDataQueues].
                rxNumOfDataQueues can only be [3] when the link's Rx parser is disabled.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxNumOfDataQueues(t_Handle h_PpposLink,uint8_t rxNumOfDataQueues);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxDataQueueLength

 @Description   Configures the length of a Tx data queue.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     txDataQueueId     - ID of the Tx data queue.
 @Param[in]     txDataQueueLength - Length of the Tx data queue.
                The default length of each one of the Tx data queues is [128].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxDataQueueLength(t_Handle h_PpposLink,uint8_t txDataQueueId,uint8_t txDataQueueLength);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxDataQueueLength

 @Description   Configures the length of an Rx data queue.

 @Param[in]     h_PpposLink       - Handle to the link object.
 @Param[in]     rxDataQueueId     - ID of the Rx data queue.
 @Param[in]     rxDataQueueLength - Length of the Rx data queue.
                The default length of each one of the Rx data queues is [128].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxDataQueueLength(t_Handle h_PpposLink,uint8_t rxDataQueueId,uint8_t rxDataQueueLength);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigTxDataQueueFrameEqn

 @Description   Associates an Tx data queue to one of the Tx frame event queues.

 @Param[in]     h_PpposLink         - Handle to the link object.
 @Param[in]     txDataQueueId       - ID of the Tx data queue.
 @Param[in]     txDataQueueFrameEqn - ID of the Tx frame event queue associated to the Tx data queue.
                The default Tx frame event queue associated to each one of the Tx data queues is Tx frame event queue [0].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigTxDataQueueFrameEqn(t_Handle h_PpposLink,uint8_t txDataQueueId,uint8_t txDataQueueFrameEqn);


/**************************************************************************//**
 @Function      PPPOS_LINK_ConfigRxDataQueueFrameEqn

 @Description   Associates an Rx data queue to one of the Rx frame event queues.

 @Param[in]     h_PpposLink         - Handle to the link object.
 @Param[in]     rxDataQueueId       - ID of the Rx data queue.
 @Param[in]     rxDataQueueFrameEqn - ID of the Rx frame event queue associated to the Rx data queue.
                The default Rx frame event queue associated to each one of the Rx data queues is Rx frame event queue [0].

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_ConfigRxDataQueueFrameEqn(t_Handle h_PpposLink,uint8_t rxDataQueueId,uint8_t rxDataQueueFrameEqn);


/**************************************************************************//*
 @Function      PPPOS_LINK_GetTxDataQueueLength

 @Description   Retrieves the length of a Tx data queue.

 @Param[in]     h_PpposLink   - Handle to the link object.
 @Param[in]     txDataQueueId - ID of the Tx data queue.

 @Cautions      This routine can be invoked only after configuration and before initialization.

 @Return        The length of the Tx data queue.
*//***************************************************************************/
uint8_t PPPOS_LINK_GetTxDataQueueLength(t_Handle h_PpposLink,uint8_t txDataQueueId);


/**************************************************************************//*
 @Function      PPPOS_LINK_GetRxDataQueueLength

 @Description   Retrieves the length of an Rx data queue.

 @Param[in]     h_PpposLink   - Handle to the link object.
 @Param[in]     rxDataQueueId - ID of the Rx data queue.

 @Cautions      This routine can be invoked only after configuration and before initialization.

 @Return        The length of the Rx data queue.
*//***************************************************************************/
uint8_t PPPOS_LINK_GetRxDataQueueLength(t_Handle h_PpposLink,uint8_t rxDataQueueId);


/** @} */ /* end of pppos_link_advance_config_grp */
/** @} */ /* end of pppos_link_initialization_grp */
/**************************************************************************//**
 @Group         pppos_link_runtime_control_grp PPPOS Link Runtime Control Unit

 @Description   PPPOS Link runtime control routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      PPPOS_LINK_Enable

 @Description   Enables the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_Enable(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_Disable

 @Description   Disables the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_Disable(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_SetPriority

 @Description   Changes the priority of the link.

 @Param[in]     h_PpposLink - Handle to the link object.
 @Param[in]     rxPriority  - Rx priority.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_SetPriority(t_Handle h_PpposLink,e_UpcPortPriority rxPriority);


/**************************************************************************//**
 @Function      PPPOS_LINK_SetIntRate

 @Description   Changes the internal rate of the link.

 @Param[in]     h_PpposLink - Handle to the link object.
 @Param[in]     txIntRate   - Tx internal rate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_SetIntRate(t_Handle h_PpposLink,uint32_t txIntRate);


/**************************************************************************//**
 @Description   Tx data queue service parameters
*//***************************************************************************/
typedef struct t_PpposLinkTxServiceParam
{
    uint8_t bwPercentage;
    /**< The relative bandwidth (weight) of the Tx data queue */
 
    bool strictPriority;
    /**< FALSE - The scheduling of the Tx data queue depends on its weight      */
    /**< TRUE  - The scheduling of the Tx data queue is done with high priority */
 
    bool transmitAsap;
    /**< FALSE - The Tx data queue might endure shaper stall for bit rate regulation          */
    /**< TRUE  - The Tx data queue should be serviced for transmission right after scheduling */
 
    bool extraBandwidth;
    /**< FALSE - The bandwidth of the Tx data queue is monitored by the shaper bandwidth mechanism     */
    /**< TRUE  - The bandwidth of the Tx data queue is not monitored by the shaper bandwidth mechanism */
}
t_PpposLinkTxServiceParam;


/**************************************************************************//**
 @Function      PPPOS_LINK_SetTxQueueService

 @Description   Changes the service for the link's Tx data queues.

 @Param[in]     h_PpposLink      - Handle to the link object.
 @Param[in]     p_TxServiceParam - Service parameters for each Tx data queue.

 @Cautions      The relative bandwidth of each Tx data queue must be 1-100.
                The total bandwidth of all Tx data queues must be exactly 100.
                The relative bandwidth of a Tx data queue is irrelevant in strict priority.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_SetTxQueueService(t_Handle h_PpposLink,t_PpposLinkTxServiceParam p_TxServiceParam[]);


/**************************************************************************//**
 @Function      PPPOS_LINK_SetTxShaping

 @Description   Changes the link's Tx traffic shaping.

 @Param[in]     h_PpposLink    - Handle to the link object.
 @Param[in]     averageBitRate - The average bit rate of the link (bits per second).
 @Param[in]     maxBurstLength - The longest burst of the link (bytes per burst).
                                 A burst is a group of packets with minimal IPG.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_SetTxTrafficShaping(t_Handle h_PpposLink,uint32_t averageBitRate,uint32_t maxBurstLength);


/**************************************************************************//**
 @Function      PPPOS_LINK_EnableTxStatCounters

 @Description   Enables the Tx statistical counters of the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_EnableTxStatCounters(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_EnableRxStatCounters

 @Description   Enables the Rx statistical counters of the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_EnableRxStatCounters(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_DisableTxStatCounters

 @Description   Disables the Tx statistical counters of the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_DisableTxStatCounters(t_Handle h_PpposLink);


/**************************************************************************//**
 @Function      PPPOS_LINK_DisableRxStatCounters

 @Description   Disables the Rx statistical counters of the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_DisableRxStatCounters(t_Handle h_PpposLink);


/**************************************************************************//**
 @Description   Tx statistical counters
*//***************************************************************************/
typedef struct t_PpposLinkTxStatCounters
{
    uint32_t numOfBytesOk ; /**< Total number of bytes transmitted OK  */
    uint32_t numOfFramesOk; /**< Total number of frames transmitted OK */
}
t_PpposLinkTxStatCounters;


/**************************************************************************//**
 @Function      PPPOS_LINK_RetrieveTxStatCounters

 @Description   Retrieves the Tx statistical counters of the link.

 @Param[in]     h_PpposLink      - Handle to the link object.
 @Param[out]    p_TxStatCounters - Pointer to a Tx statistical counters object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_RetrieveTxStatCounters(t_Handle h_PpposLink,t_PpposLinkTxStatCounters* p_TxStatCounters);


/**************************************************************************//**
 @Description   Rx statistical counters
*//***************************************************************************/
typedef struct t_PpposLinkRxStatCounters
{
    uint32_t numOfBytesOk         ; /**< Total number of bytes received OK                             */
    uint32_t numOfFramesOk        ; /**< Total number of frames received OK                            */
    uint16_t numOfFramesCrcError  ; /**< Total number of frames received with CRC error                */
    uint16_t numOfFramesTooLong   ; /**< Total number of frames received longer than the maximum limit */
    uint16_t numOfFramesDropped   ; /**< Total number of frames dropped due to lack of resources       */
    uint16_t numOfFramesLost      ; /**< Total number of frames lost due to internal MAC error         */
    uint16_t numOfInvalidAccFrames; /**< Total number of invalid ACC frames received                   */
    uint32_t numOfBytes           ; /**< Total number of bytes received                                */
}
t_PpposLinkRxStatCounters;


/**************************************************************************//**
 @Function      PPPOS_LINK_RetrieveRxStatCounters

 @Description   Retrieves the Rx statistical counters of the link.

 @Param[in]     h_PpposLink      - Handle to the link object.
 @Param[out]    p_RxStatCounters - Pointer to an Rx statistical counters object.

 @Return        E_OK upon success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_RetrieveRxStatCounters(t_Handle h_PpposLink,t_PpposLinkRxStatCounters* p_RxStatCounters);


/**************************************************************************//**
 @Function      PPPOS_LINK_GetUnderrunEventIndication

 @Description   Reports whether an underrun event has occurred for the link.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        TRUE if an underrun event has occurred; FALSE otherwise.
*//***************************************************************************/
bool PPPOS_LINK_GetUnderrunEventIndication(t_Handle h_PpposLink);


#if (DEBUG_ERRORS > 0)


/**************************************************************************//**
 @Function      PPPOS_LINK_DumpRegisters

 @Description   Dumps the current values of the link registers.

 @Param[in]     h_PpposLink - Handle to the link object.

 @Return        None.
*//***************************************************************************/
void PPPOS_LINK_DumpRegisters(t_Handle h_PpposLink);


#endif /* (DEBUG_ERRORS > 0) */


/**************************************************************************//**
 @Function      PPPOS_LINK_Tx

 @Description   Transmits a data buffer on one of the PPP over SONET transmit queues.

                Accepts data as a pointer to a single data buffer (array of
                bytes).

                This routine eventually causes the user's transmit confirmation
                callback routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                PPPOS_LINK_TxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.

 @Param[in]     h_PpposLink      - Handle to the link object.
 @Param[in]     txDataQueueNum   - Tx data queue number.
 @Param[in]     p_Data           - Pointer to data to transmit.
 @Param[in]     h_BufContext     - User's context for the given data buffer.
 @Param[in]     length           - Length of data to transmit.
 @Param[in]     lastBuffer       - Indicates whether the referred buffer is the last one in the frame or not.
 @Param[in]     pidInsert        - Indicates whether the PID that is specified in the next field should be inserted into the frame or not.
 @Param[in]     pid              - PID (protocol identifier).
 @Param[in]     ipcOffset        - The offset for the IP header (where the checksum should be calculated).
                                   when this value is set to 0xFF no IP checksum is generated.
                                   The checksum generation is subject to the PPPOS_LINK_ConfigIpChecksumEnable routine.
                                   Note that the IP header must reside in the first TxBD of the frame, and in the first 64/128
                                   bytes of the frame (respectively depending on the segment size).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                PPPOS_LINK_TxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.
*//***************************************************************************/
t_Error PPPOS_LINK_Tx(t_Handle h_PpposLink,
                      uint8_t  txDataQueueNum,
                      uint8_t* p_Data,
                      t_Handle h_BufContext,
                      uint16_t length,
                      bool     lastBuffer,
                      bool     pidInsert,
                      uint16_t pid,
                      uint8_t  ipcOffset);


/**************************************************************************//**
 @Function      PPPOS_LINK_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
 
                Invokes user's transmit confirmation callback routine for
                previously transmitted data on one of the transmit queues.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.

 @Param[in]     h_PpposLink    - Handle to the link object.
 @Param[in]     txDataQueueNum - Tx data queue number.

 @Return        None.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                the transmit routines) simultaneously at task level and at
                interrupt level may lead to unpredictable results.
*//***************************************************************************/
void PPPOS_LINK_TxConf(t_Handle h_PpposLink, uint8_t txDataQueueNum);


/**************************************************************************//**
 @Function      PPPOS_LINK_Rx

 @Description   Handles data buffers received in the specified receive queue.

                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific queue.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.

 @Param[in]     h_PpposLink    - Handle to the link object.
 @Param[in]     rxDataQueueNum - Rx data queue number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_Rx(t_Handle h_PpposLink, uint8_t rxDataQueueNum);


/**************************************************************************//*
 @Function      PPPOS_LINK_RxError

 @Description   Upon Rx error, invokes a user callback recovery routine.

 @Param[in]     h_PpposLink    - Handle to the link object.
 @Param[in]     rxDataQueueNum - Rx data queue number.
 @Param[in]     rxErrorFlags   - Rx error flags.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PPPOS_LINK_RxError(t_Handle h_PpposLink, uint8_t rxDataQueueNum, uint16_t rxErrorFlags);


/** @} */ /* end of pppos_link_runtime_control_grp */
/** @} */ /* end of pppos_link_grp */
/** @} */ /* end of ucc_pos_grp */


#endif /* __PPPOS_LINK_EXT_H */
