/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ptp_ext.h

 @Description   External definitions and API for PTP (IEEE 1588).
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __PTP_EXT_H
#define __PTP_EXT_H

#include "part_integration_ext.h"

/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_init_grp PTP Initialization Unit

 @Description   PTP initialization API.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   PTP Standard Time Representation Structure.
*//***************************************************************************/
typedef struct t_PtpTime
{
    uint32_t    sec;    /**< Seconds, unsigned */
    int32_t     nsec;   /**< Nano-seconds, signed (sign is for the whole value) */
} t_PtpTime;


/**************************************************************************//**
 @Description   PTP Message Types.
*//***************************************************************************/
typedef enum e_PtpMessage
{
    e_PTP_MSG_SYNC = 0,     /**< Sync message */
    e_PTP_MSG_DELAY_REQ,    /**< Delay_req message */
    e_PTP_MSG_FOLLOW_UP,    /**< Follow_up message */
    e_PTP_MSG_DELAY_RESP,   /**< Delay_resp message */
    e_PTP_MSG_MANAGEMENT,   /**< Management message */
#ifdef PTP_V2
    e_PTP_MSG_PDELAY_REQ,   /**< Pdelay_req message */
    e_PTP_MSG_PDELAY_RESP,  /**< Pdelay_resp message */
    e_PTP_MSG_PDELAY_FOLLOW_UP, /**< Pdelay resp Follow_up message */
    e_PTP_MSG_ANNOUNCE,         /**< Announce message */
    e_PTP_MSG_SIGNALING,        /**< signaling message */
#endif /* PTP_V2 */
    e_PTP_MSG_DUMMY_LAST
} e_PtpMessage;

#ifdef PTP_V2
/**************************************************************************//**
 @Description   PTP Time Message Timestamp Point.
*//***************************************************************************/
typedef enum e_PtpMessageTimestampPoint
{
    e_PTP_TIMESTAMP_IN_SFD            ,/**< Message timestamp point shall correspond to internal SFD */
    e_PTP_TIMESTAMP_EXT_SOF            /**< Message timestamp point shall correspond to external SOF */
} e_PtpMessageTimestampPoint;

/**************************************************************************//**
 @Description   PTP Time Message Timestamp Point.
*//***************************************************************************/
typedef enum e_PtpVersion
{
    e_PTP_VER1, /**< ptp version 1 */
    e_PTP_VER2  /**< ptp version 2 */
} e_PtpVersion;
#endif /* PTP_V2 */

/**************************************************************************//**
 @Description   PTP Time Stamp Delivery Mode.
*//***************************************************************************/
typedef enum e_PtpTsuDeliveryMode
{
    e_PTP_TSU_DELIVERY_IN_BAND,     /**< In-band time-stamp delivery mode */
    e_PTP_TSU_DELIVERY_OUT_OF_BAND  /**< Out-of-band time-stamp delivery mode */
} e_PtpTsuDeliveryMode;


/** @} */ /* end of ptp_init_grp */


/**************************************************************************//**
 @Group         ptp_exceptions_grp PTP Exceptions Unit

 @Description   PTP exceptions API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PTP timestamp events callback function.
*//***************************************************************************/
typedef void (t_PtpEventsCallBack)(t_Handle h_App, uint32_t events, uint32_t flags);

/** @} */ /* end of ptp_exceptions_grp */
/** @} */ /* end of ptp_grp */


/* Including API for specific integration */
#include "ptp_integration_ext.h"


/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_init_grp PTP Initialization Unit

 @Description   PTP initialization API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   PTP configuration parameters structure.
 
                This structure should be passed to PTP_Config().
*//***************************************************************************/
typedef struct t_PtpParam
{
#ifdef PTP_V2
    /* PTP basic parameters */
    e_PtpVersion        ptpVer;          /**<ptp version - v2 or V1 */
#endif /* PTP_V2 */

    /* System integration parameters */
    t_Handle            h_Part;         /**< Handle to controlling device */

    /* Application interface */
    t_PtpEventsCallBack *f_TsuEvents;   /**< Time-Stamp Unit events callback routine */

} t_PtpParam;


/**************************************************************************//**
 @Description   PTP TSU configuration parameters structure.
 
                This structure should be passed to PTP_TSU_Config().
*//***************************************************************************/
typedef struct t_PtpTsuParam
{
    t_Handle                h_App;                  /**< Application handle */
    e_PtpTsuDeliveryMode    deliveryMode;           /**< In-band / Out-of-band */
    uint16_t                txTimeStampsQueueSize;  /**< Size of Tx time-stamps queue */
    uint16_t                rxTimeStampsQueueSize;  /**< Size of Rx time-stamps queue;
                                                         This value should be zero when
                                                         deliveryMode is set to In-band */
} t_PtpTsuParam;


/**************************************************************************//**
 @Function      PTP_Config

 @Description   Configures the PTP module according to user's parameters.
 
                The driver assigns default values to some PTP parameters.
                These parameters can be overwritten using the advanced
                configuration routines.

 @Param[in]     p_PtpParam - PTP configuration parameters.
 
 @Return        Handle to the new PTP object; NULL on failure.
*//***************************************************************************/
t_Handle PTP_Config(t_PtpParam *p_PtpParam);


/**************************************************************************//**
 @Function      PTP_TSU_Config

 @Description   Configures the TSU of a specific PTP port, based on user's
                parameters.
 
                This routine must be invoked for each PTP port in the system.
                It should not be called for ports that are not used for PTP.

                The driver assigns default values to some TSU parameters.
                These parameters can be overritten using the advanced
                configuration routines.

 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     port        - The selected PTP port.
 @Param[in]     p_TsuParam  - TSU configuration parameters.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_TSU_Config(t_Handle         h_Ptp,
                       e_PtpPort        port,
                       t_PtpTsuParam    *p_TsuParam);

/**************************************************************************//**
 @Function      PTP_Init

 @Description   Initializes the PTP driver and hardware.

 @Param[in]     h_Ptp - Handle to PTP object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_Init(t_Handle h_Ptp);

/**************************************************************************//**
 @Function      PTP_Free

 @Description   Frees the PTP object and all allocated resources.

 @Param[in]     h_Ptp - Handle to PTP object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_Free(t_Handle h_Ptp);


/**************************************************************************//**
 @Group         ptp_adv_config_grp  PTP Advanced Configuration Unit

 @Description   PTP advanced configuration functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PTP_TSU_ConfigEventsMask

 @Description   Configures the events mask for a specific TSU.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     port        - PTP port ID.
 @Param[in]     eventsMask  - Selected events mask.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously created using PTP_Config().
*//***************************************************************************/
t_Error PTP_TSU_ConfigEventsMask(t_Handle   h_Ptp,
                                 e_PtpPort  port,
                                 uint32_t   eventsMask);


/** @} */ /* end of ptp_adv_config_grp */
/** @} */ /* end of ptp_init_grp */


/**************************************************************************//**
 @Group         ptp_control_grp PTP Control Unit

 @Description   PTP runtime control API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      PTP_TSU_Enable

 @Description   Enables the TSU operation.
 
 @Param[in]     h_Ptp   - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port    - The Port ID matching the TSU.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
t_Error PTP_TSU_Enable(t_Handle h_Ptp, e_PtpPort port);

/**************************************************************************//**
 @Function      PTP_TSU_Disable

 @Description   Disables the TSU operation.
 
 @Param[in]     h_Ptp   - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port    - The Port ID matching the TSU.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
t_Error PTP_TSU_Disable(t_Handle h_Ptp, e_PtpPort port);

/**************************************************************************//**
 @Function      PTP_TSU_GetRxTimeStamp

 @Description   Reads the timestamp of a received PTP frame.
 
                The timestamps are stored in a queue. This routine returns
                the first timestamp in the queue.
 
 @Param[in]     h_Ptp       - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port        - The Port ID matching the TSU.
 @Param[out]    p_TimeStamp - Timestamp structure to fill.

 @Return        TRUE if there is a valid timestamp available; FALSE otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
bool PTP_TSU_GetRxTimeStamp(t_Handle h_Ptp, e_PtpPort port, t_PtpTime *p_TimeStamp);

/**************************************************************************//**
 @Function      PTP_TSU_GetTxTimeStamp

 @Description   Reads the timestamp of a transmitted PTP frame.
 
                The timestamps are stored in a queue. This routine returns
                the first timestamp in the queue.
 
 @Param[in]     h_Ptp       - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port        - The Port ID matching the TSU.
 @Param[out]    p_TimeStamp - Timestamp structure to fill.

 @Return        TRUE if there is a valid timestamp available; FALSE otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
bool PTP_TSU_GetTxTimeStamp(t_Handle h_Ptp, e_PtpPort port, t_PtpTime *p_TimeStamp);

/**************************************************************************//**
 @Function      PTP_TSU_FlushRxTimeStamps

 @Description   Flushes all timestamps from the Rx timestamps queue.
 
 @Param[in]     h_Ptp   - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port    - The Port ID matching the TSU.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
t_Error PTP_TSU_FlushRxTimeStamps(t_Handle h_Ptp, e_PtpPort port);

/**************************************************************************//**
 @Function      PTP_TSU_FlushTxTimeStamps

 @Description   Flushes all timestamps from the Tx timestamps queue.
 
 @Param[in]     h_Ptp   - Handle to the PTP as returned from the PTP_Config.
 @Param[in]     port    - The Port ID matching the TSU.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The PTP port must have been previously configured using
                PTP_TSU_Config(), and h_Ptp must have been previously
                initialized using PTP_Init().
*//***************************************************************************/
t_Error PTP_TSU_FlushTxTimeStamps(t_Handle h_Ptp, e_PtpPort port);

/**************************************************************************//**
 @Function      PTP_RTC_GetExternalTriggerTimeStamp

 @Description   Converts a PTP RTC raw timestamp to PTP time format.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     p_TimeStamp - Pointer to the raw timestamp to convert.
 @Param[out]    p_PtpTime   - The converted timestamp (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void PTP_RTC_ConvertTimeStampFormat(t_Handle    h_Ptp,
                                    uint64_t    *p_TimeStamp,
                                    t_PtpTime   *p_PtpTime);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      PTP_DumpRegs

 @Description   Dumps the registers of the PTP object.

 @Param[in]     h_Ptp - Handle to PTP object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_Ptp must have been previously initialized using PTP_Init().
*//***************************************************************************/
t_Error PTP_DumpRegs(t_Handle h_Ptp);

#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of ptp_control_grp */
/** @} */ /* end of ptp_grp */




#endif /* __PTP_EXT_H */
