/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          ptp_rtc_etsec_ext.h

 @Description   External definitions and API for PTP RTC.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __PTP_ETSEC_RTC_EXT_H__
#define __PTP_ETSEC_RTC_EXT_H__


#include "etsec_ext.h"
#include "ptp_ext.h"
#include "etsec_rtc_ext.h"


/**************************************************************************//**
 @Group         ptp_grp PTP Application Programming Interface

 @Description   PTP functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_rtc_grp PTP RTC Application Programming Interface

 @Description   PTP RTC functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ptp_rtc_control_grp PTP RTC Control Unit

 @Description   PTP RTC runtime control API.
 
 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   PTP RTC Alarm ID.
*//***************************************************************************/
typedef enum e_PtpRtcAlarmId
{
    e_PTP_RTC_ALARM_1 = e_ETSEC_RTC_ALARM_1,       /**< Alarm signal 1 */
    e_PTP_RTC_ALARM_2 = e_ETSEC_RTC_ALARM_2        /**< Alarm signal 2 */
} e_PtpRtcAlarmId;

#define PTP_RTC_NUM_OF_ALARMS   ETSEC_RTC_NUM_OF_ALARMS
                                /**< Total number of PTP RTC alarm signals */

/**************************************************************************//**
 @Description   PTP RTC Periodic Pulse ID.
*//***************************************************************************/
typedef enum e_PtpRtcPulseId
{
    e_PTP_RTC_PULSE_1 = e_ETSEC_RTC_PULSE_1,       /**< Periodic pulse 1 */
    e_PTP_RTC_PULSE_2 = e_ETSEC_RTC_PULSE_2,       /**< Periodic pulse 2 */
    e_PTP_RTC_PULSE_3 = e_ETSEC_RTC_PULSE_3        /**< Periodic pulse 3 */
} e_PtpRtcPulseId;

#define PTP_RTC_NUM_OF_PULSES   ETSEC_RTC_NUM_OF_PULSES
                                /**< Total number of PTP RTC periodic pulses */

/**************************************************************************//**
 @Description   PTP RTC External Trigger ID.
*//***************************************************************************/
typedef enum e_PtpRtcTriggerId
{
    e_PTP_RTC_TRIGGER_1 = e_ETSEC_RTC_TRIGGER_1,   /**< External trigger 1 */
    e_PTP_RTC_TRIGGER_2 = e_ETSEC_RTC_TRIGGER_2    /**< External trigger 2 */
} e_PtpRtcTriggerId;

#define PTP_RTC_NUM_OF_TRIGGERS ETSEC_RTC_NUM_OF_TRIGGERS
                                /**< Total number of PTP RTC external triggers */



/**************************************************************************//**
 @Function      PTP_RTC_SetClockOffset

 @Description   Sets the clock offset (usually relative to another clock).
 
                The user can pass a negative offset value.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     p_PtpTime   - New clock offset (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PTP_RTC_SetClockOffset(t_Handle h_Ptp, t_PtpTime *p_PtpTime);

/**************************************************************************//**
 @Function      PTP_RTC_SetAlarm

 @Description   Schedules an alarm event to a given PTP RTC time.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     alarmId     - Alarm ID.
 @Param[in]     p_PtpTime   - Requested alarm time (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PTP_RTC_SetAlarm(t_Handle           h_Ptp,
                         e_PtpRtcAlarmId    alarmId,
                         t_PtpTime          *p_PtpTime);

/**************************************************************************//**
 @Function      PTP_RTC_SetPeriodicPulse

 @Description   Sets a periodic pulse.
 
 @Param[in]     h_Ptp           - Handle to PTP object.
 @Param[in]     pulseId         - Periodic pulse ID.
 @Param[in]     p_PtpTimePeriod - Requested pulse period (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PTP_RTC_SetPeriodicPulse(t_Handle           h_Ptp,
                                 e_PtpRtcPulseId    pulseId,
                                 t_PtpTime          *p_PtpTimePeriod);

/**************************************************************************//**
 @Function      PTP_RTC_GetExternalTriggerTimeStamp

 @Description   Gets the last timestamp captured due to an external trigger.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     triggerId   - Trigger ID.
 @Param[out]    p_TimeStamp - The requested timestamp (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PTP_RTC_GetExternalTriggerTimeStamp(t_Handle            h_Ptp,
                                            e_PtpRtcTriggerId   triggerId,
                                            t_PtpTime           *p_TimeStamp);

/**************************************************************************//**
 @Function      PTP_RTC_GetCurrentTime

 @Description   Returns the current PTP RTC time.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[out]    p_PtpTime   - Current time (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PTP_RTC_GetCurrentTime(t_Handle h_Ptp, t_PtpTime *p_PtpTime);

/**************************************************************************//**
 @Function      PTP_RTC_GetExternalTriggerTimeStamp

 @Description   Converts a PTP RTC raw timestamp to PTP time format.
 
 @Param[in]     h_Ptp       - Handle to PTP object.
 @Param[in]     p_TimeStamp - Pointer to the raw timestamp to convert.
 @Param[out]    p_PtpTime   - The converted timestamp (in PTP time format).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void PTP_RTC_ConvertTimeStampFormat(t_Handle    h_Ptp,
                                    uint64_t    *p_TimeStamp,
                                    t_PtpTime   *p_PtpTime);


/** @} */ /* end of ptp_rtc_control_grp */
/** @} */ /* end of ptp_rtc_grp */
/** @} */ /* end of ptp__grp */

#endif /* __PTP_ETSEC_RTC_EXT_H__ */
