/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          qe_ext.h

 @Description   External header file for QE unit routines

*//***************************************************************************/
#ifndef __QE_EXT_H
#define __QE_EXT_H

#include "qe_integration_ext.h" /* Keep first to solve MPC830x QE access bug */
#include "part_ext.h"

#ifdef NCSW_LINUX
    #include <asm/page.h>
#endif
/**************************************************************************//**
 @Group         qe__grp QUICC Engine Application Programming Interface

 @Description   QE API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         qe_init_grp QE Initialization Unit

 @Description   QE Initialization Unit Functions and definitions

 @{
*//***************************************************************************/

#define QE_NUM_OF_UCCS          e_UCC_DUMMY_LAST
#define QE_NUM_OF_UPCS          e_UPC_DUMMY_LAST
#define QE_NUM_OF_SIS           e_SI_DUMMY_LAST
#define QE_NUM_OF_TDMS          e_TDM_DUMMY_LAST
#define QE_NUM_OF_CLK_SIGNALS   e_QE_CLK_SIGNAL_DUMMY_LAST
#define QE_NUM_OF_EXT_REQUESTS  4

/**************************************************************************//**
 @Description   QE external request enumeration
*//***************************************************************************/
typedef enum e_QeExtRequestId
{
    e_QE_EXT_REQUEST_1 = 0, /**< External request 1 */
    e_QE_EXT_REQUEST_2 = 1, /**< External request 2 */
    e_QE_EXT_REQUEST_3 = 2, /**< External request 3 */
    e_QE_EXT_REQUEST_4 = 3, /**< External request 4 */
    e_QE_EXT_REQUEST_ANY    /**< Any external request */
} e_QeExtRequestId;

/**************************************************************************//**
 @Description   QE external request modes

                Sensitivity/polarity modes for external request pins.
*//***************************************************************************/
typedef enum e_QeExtRequestMode
{
    e_QE_EXT_REQ_MODE_ACTIVE_HIGH = 0,      /**< External request pin is level-sensitive,
                                                 active-high */
    e_QE_EXT_REQ_MODE_ACTIVE_LOW,           /**< External request pin is level-sensitive,
                                                 active-low */
    e_QE_EXT_REQ_MODE_HIGH_TO_LOW_CHANGE,   /**< External request pin is edge-sensitive,
                                                 triggered on high-to-low change */
    e_QE_EXT_REQ_MODE_LOW_TO_HIGH_CHANGE    /**< External request pin is edge-sensitive,
                                                 triggered on low-to-high change */
} e_QeExtRequestMode;


/**************************************************************************//**
 @Description   QE configuration parameters structure
*//***************************************************************************/
typedef struct t_QeParam
{
    /* System integration parameters */
    uint32_t    baseAddress;        /**< Base address of QE registers in memory. */
    uint32_t    muramBaseAddress;   /**< Base address of Multi-User RAM. */
    uint8_t     pramMemId;          /**< MURAM partition ID for allocating SDMA buffer and other structures */
    uint32_t    qeClockFreqHz;      /**< QE clock frequency in Hz */
    t_Handle    h_Part;             /**< A handle to the chip descriptor */
    int         sdmaIrq;            /**< QE SDMA interrupt ID (system-specific number) */
    int         virtualTaskIrq;     /**< QE virtual task interrupt ID (system-specific number) */
    int         extReqIrqs[QE_NUM_OF_EXT_REQUESTS];
                                    /**< QE external requests interrupt IDs (system-specific number) */

    /* Application interface */
    t_Handle    h_App;              /**< A handle to the application object */
    void        (*f_SdmaExceptions)(t_Handle h_App, bool secondaryBus, uint32_t addr, uint8_t snum);
                                    /**< Application's callback routine - called by
                                        the QE driver to report SDMA errors. */
} t_QeParam;


/**************************************************************************//**
 @Function      QE_Config

 @Description   Creates descriptor for the QE module.

                The routine returns a handle (descriptor) to the QE object.
                This descriptor must be passed as first parameter to all other
                QE function calls.

                No actual initialization or configuration of QE hardware is
                done by this routine.

 @Param[in]     p_QeParam   - Pointer to data structure of parameters

 @Return        Handle to QE object, or NULL on failure.
*//***************************************************************************/
t_Handle QE_Config(t_QeParam *p_QeParam);

/**************************************************************************//**
 @Function      QE_Init

 @Description   Initializes the QE module's PRAM.

 @Param[in]     h_Qe - QE module descriptor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_Init(t_Handle h_Qe);

/**************************************************************************//**
 @Function      QE_Free

 @Description   Frees all resources that were assigned to QE module by QE_Config().

                Calling this routine invalidates the descriptor.

 @Param[in]     h_Qe - QE module descriptor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_Free(t_Handle h_Qe);

/**************************************************************************//**
 @Group         qe_advanced_init_grp    QE Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_ConfigSdmaBufferBlocks

 @Description   Configures the number of 512-bytes blocks for internal SDMA buffer.
                [QE_SDMA_MAX_INTERNAL_BUFFER_BLOCKS]

                The valid values: 1 to QE_SDMA_MAX_INTERNAL_BUFFER_BLOCKS);
                High values may be required for higher performance requirements.

 @Param[in]     h_Qe                - QE module descriptor
 @Param[in]     sdmaBufferBlocks    - Number of 512-bytes blocks for SDMA buffer.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigSdmaBufferBlocks(t_Handle h_Qe, uint8_t sdmaBufferBlocks);

/**************************************************************************//**
 @Function      QE_ConfigSdmaDynamicBusSelection

 @Description   Configures dynamic bus selection for the SDMA.
                [Disabled]

                For each transaction initiated by the QE RISC, the SDMA must
                select the destination bus: either the system bus or the
                secondary bus. The SDMA implements two mechanisms for bus
                selection:
                - User programmable window:
                  The user maps a memory window to a selected bus. The user
                  selects the window base address and the size of the window.
                  Any access matching this window is automatically routed to
                  the selected bus. Other accesses are routed to the other bus.
                  The minimum window size is 64K bytes.
                - User programmable bus selection by protocol:
                  The SDMA selects the bus according to the configuration of
                  each protocol. Refer to the programming model of specific
                  peripherals for details.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     enabled         - TRUE to enable dynamic bus selection;
                                  FALSE to use bus selection according to protocol.
 @Param[in]     windowBase      - Bus access window base.
 @Param[in]     windowSize      - Bus access window size (in bytes); Set to zero
                                  if the given bus should not be accessed (i.e
                                  the other bus must always be selected).
 @Param[in]     secondaryBus    - FALSE to assign the window to system bus;
                                  TRUE to assign the window to QE secondary bus.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigSdmaDynamicBusSelection(t_Handle   h_Qe,
                                         bool       enabled,
                                         uint32_t   windowBase,
                                         uint32_t   windowSize,
                                         bool       secondaryBus);

/**************************************************************************//**
 @Function      QE_ConfigSystemBusSnoopingEnabled

 @Description   Enable or disable system bus snooping (global mode).
                [TRUE]

                The SDMA in the QE is able to dynamically signal to the CPU
                whether or not the current cycle on the system bus is to be
                snooped for cache coherency.

                This routine allows the user to globally disable all snooping,
                or enable global snooping. When global snooping is enabled, it
                is possible to configure each QE peripheral specifically to
                enable or disable snooping for its own bus cycles.

                Note that the secondary bus of the QE cannot be snooped.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     snoopEnable - TRUE to enable bus snooping;
                              FALSE to disable bus snooping.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigSystemBusSnoopingEnabled(t_Handle h_Qe, bool snoopEnable);

/**************************************************************************//**
 @Function      QE_ConfigSystemBusEventsEnabled

 @Description   Enable or disable system bus error events (interrupt).
                [TRUE]

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     enabled - TRUE to enable system bus error events;
                          FALSE to disable system bus error events.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigSystemBusEventsEnabled(t_Handle h_Qe, bool enabled);

/**************************************************************************//**
 @Function      QE_ConfigSecondaryBusEventsEnabled

 @Description   Enable or disable secondary bus error events (interrupt).
                [TRUE]

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     enabled - TRUE to enable secondary bus error events;
                          FALSE to disable secondary bus error events.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigSecondaryBusEventsEnabled(t_Handle h_Qe, bool enabled);

/**************************************************************************//**
 @Function      QE_ConfigTimeStamp1Period

 @Description   Enable QE time-stamp 1 with a given period (in nano-seconds).
                [500]

                Pass zero to disable the time-stamper.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     period  - Time stamp 1 period (in nano-seconds)

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigTimeStamp1Period(t_Handle h_Qe, uint32_t period);

/**************************************************************************//**
 @Function      QE_ConfigTimeStamp2Period

 @Description   Enable QE time-stamp 2 with a given period (in nano-seconds).
                [1000]

                Pass zero to disable the time-stamper.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     period  - Time stamp 2 period (in nano-seconds)

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following QE_Config() and before QE_Init().
*//***************************************************************************/
t_Error QE_ConfigTimeStamp2Period(t_Handle h_Qe, uint32_t period);

/**************************************************************************//**
 @Function      QE_ConfigExternalRequestMode

 @Description   Configures the sensitivity/polarity modes of external request pins.
                [e_QE_EXT_REQ_MODE_LOW_TO_HIGH_CHANGE]

 @Param[in]     h_Qe            - Handle to QE object.
 @Param[in]     extRequestId    - Specific external request index.
 @Param[in]     extRequestMode  - Selected operation mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_ConfigExternalRequestMode(t_Handle           h_Qe,
                                     e_QeExtRequestId   extRequestId,
                                     e_QeExtRequestMode extRequestMode);


/** @} */ /* end of qe_advanced_init_grp group */

/** @} */ /* end of qe_init_grp group */


/**************************************************************************//**
 @Group         qe_ctrl_grp QE Control Unit

 @Description   QE Control Unit Functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_GetQeClockFreq

 @Description   Gets QE clock frequency in Hz.

 @Param[in]     h_Qe - pointer to QE descriptor

 @Return        QE clock frequency in Hz.
*//***************************************************************************/
uint32_t QE_GetQeClockFreq(t_Handle h_Qe);


/**************************************************************************//**
 @Description   QE Time Stampers enumeration
*//***************************************************************************/
typedef enum e_QeTimeStamper
{
    e_QE_TIME_STAMPER_1 = 0,    /**< Time Stamper 1 */
    e_QE_TIME_STAMPER_2,        /**< Time Stamper 2 */
    e_QE_TIME_STAMPER_DUMMY_LAST
} e_QeTimeStamper;

/**************************************************************************//**
 @Function      QE_GetTimeStamp

 @Description   Gets a QE timestamp

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     timeStamper - Time stamper (register) ID.

 @Retval        Timestamp value
*//***************************************************************************/
uint32_t QE_GetTimeStamp(t_Handle h_Qe, e_QeTimeStamper timeStamper);

/**************************************************************************//**
 @Function      QE_GetTimeStampPeriodNanoSec

 @Description   Get the period of a given timestamp in Nano seconds.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     timeStamper - Time stamper (register) ID.

 @Return        The period (in nano-seconds) of each time stamp tick.
*//***************************************************************************/
uint32_t QE_GetTimeStampPeriodNanoSec(t_Handle h_Qe, e_QeTimeStamper timeStamper);


/**************************************************************************//**
 @Description   Virtual traps ID for QE microcode patch.
*//***************************************************************************/
typedef enum e_QeVirtTrapId
{
    e_QE_VIRT_TRAP_ETH_IW_PRAM = 0,
    e_QE_VIRT_TRAP_ATM_IW_PRAM,
    e_QE_VIRT_TRAP_PPP_IW_PRAM,
    e_QE_VIRT_TRAP_ETH_RX_GLOBAL_PRAM,
    e_QE_VIRT_TRAP_ATM_GLOBAL_PRAM,
    e_QE_VIRT_TRAP_VP_GLOBAL_PRAM
} e_QeVirtTrapId;

/**************************************************************************//**
 @Description   QE microcode patch type.

                When common IRAM is disabled, each of the RISCs has its own
                portion of the instruction RAM. Performance may be better since
                RISCs are not competing on a common resource when fetching
                instructions. In case the two RISCs should run the same code,
                the code should be duplicated in the dedicated RISC regions.

                When common IRAM is enabled, all RISCs are sharing a common
                instruction RAM. This mode must be used when the code is too
                large to fit into the IRAM portion of a single RISC.
*//***************************************************************************/
typedef enum e_QePatchType
{
    e_QE_COMMON_IRAM = 0,
    e_QE_COMMON_PATCH,
    e_QE_MULTIPLE_PATCHS
} e_QePatchType;

/**************************************************************************//**
 @Description   QE microcode patch loading parameters.
*//***************************************************************************/
typedef struct t_QePatch
{
     uint32_t   *p_Patch;
     uint32_t   size;
     uint32_t   *p_Traps;
     uint8_t    numOfTraps;
} t_QePatch;


/**************************************************************************//**
 @Collection    QE microcode patch extended mode bits.
 @{
*//***************************************************************************/
#define QE_UCODE_EXT_MODE_GENERAL_PUSH_CMD          0x8000000000000000LL
#define QE_UCODE_EXT_MODE_UCC_ATM_RX_INIT_PUSH_CMD  0x4000000000000000LL
#define QE_UCODE_EXT_MODE_ADD_REMOVE_CMD_VALIDATE   0x2000000000000000LL
#define QE_UCODE_EXT_MODE_HC_FOR_ETH                0x1000000000000000LL
#define QE_UCODE_EXT_MODE_ETH_TX_THREAD_PUSH_CMD    0x0800000000000000LL
#define QE_UCODE_EXT_MODE_QE_ENET22                 0x0400000000000000LL
/* @} */

/**************************************************************************//**
 @Function      QE_LoadUCodePatch

 @Description   Loads a microcode patch (or patches) to the IRAM and sets
                the traps, virtual traps and extended modes.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     type            - Patch type
 @Param[in]     p_Patches       - Array of patches description
 @Param[in]     iRamOffsets     - Array of IRAM offsets for patches
 @Param[in]     p_VirtTraps     - Array of virtual traps
 @Param[in]     numOfVirtTraps  - Number of virtual traps
 @Param[in]     extModes        - Extended mode bits

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_LoadUCodePatch(t_Handle      h_Qe,
                          e_QePatchType type,
                          t_QePatch     *p_Patches,
                          uint32_t      *iRamOffsets,
                          uint32_t      *p_VirtTraps,
                          uint32_t      numOfVirtTraps,
                          uint64_t      extModes);

/**************************************************************************//**
 @Function      QE_LoadUCodeToRam

 @Description   Loads firmware(microcode) to the IRAM and sets extended modes,this function is used when
                the chip is not supporting ROM microcode,only RAM.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     p_Patch         - pointer to microcode beginning
 @Param[in]     size            - size of microcode
 @Param[in]     extModes        - Extended mode bits

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_LoadUCodeToRam(t_Handle      h_Qe,
                          uint32_t      *p_Patch,
                          uint32_t      size,
                          uint64_t      extModes);


/**************************************************************************//**
 @Function      QE_SetBrg

 @Description   Sets frequency and enables brg.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     brg     - which brg should be set
 @Param[in]     freq    - desired frequency in Hz for brg to run at

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_SetBrg(t_Handle h_Qe, e_QeClock brg, uint32_t freq);

/**************************************************************************//**
 @Function      QE_SetBrgFromExtSrc

 @Description   Sets frequency and enables BRG from an external clock source.

 @Param[in]     h_Qe         - QE module descriptor
 @Param[in]     brg          - which brg should be set
 @Param[in]     clock        - which clock should be set for this brg as an input
 @Param[in]     extClockFreq - The frequency of the external clock in Hz
 @Param[in]     freq         - desired frequency in Hz for brg to run at

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_SetBrgFromExtSrc(t_Handle h_Qe, e_QeClock brg, e_QeClock clock, uint32_t extClockFreq, uint32_t freq);

/**************************************************************************//**
 @Function      QE_FreeBrg

 @Description   Disables brg, allowing QE_SetBrg() to be used.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     brg     - which brg should be disabled

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_FreeBrg(t_Handle h_Qe, e_QeClock brg);


/**************************************************************************//**
 @Function      QE_EnableTimeoutTask

 @Description   Enables the QE timeout task.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     timeoutTaskSnum - Timeout task SNUM
 @Param[in]     timePeriod      - Required timeout period (in nano-seconds).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_EnableTimeoutTask(t_Handle h_Qe, uint8_t timeoutTaskSnum, uint32_t timePeriod);

/**************************************************************************//**
 @Function      QE_DisableTimeoutTask

 @Description   Disables the QE timeout task.

 @Param[in]     h_Qe - QE module descriptor

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_DisableTimeoutTask(t_Handle h_Qe);

/**************************************************************************//**
 @Function      QE_EnablePollingTask

 @Description   Enables the QE polling task for a given Virtual Port SNUM.

                Up to four VP SNUMs can be enabled with the polling task.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     pollingTaskSnum - Polling task SNUM
 @Param[in]     timePeriod      - Required timeout period (in nano-seconds).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_EnablePollingTask(t_Handle h_Qe, uint8_t pollingTaskSnum, uint32_t timePeriod);

/**************************************************************************//**
 @Function      QE_DisablePollingTask

 @Description   Disables the QE polling task for a given Virtual Port SNUM.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     pollingTaskSnum - Polling task SNUM.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_DisablePollingTask(t_Handle h_Qe, uint8_t pollingTaskSnum);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**************************************************************************//**
 @Function      QE_EnableIprTimeoutTask

 @Description   Enables the QE timeout task for IP Reassembly.

 @Param[in]     h_Qe                    - QE module descriptor
 @Param[in]     numOfChecksPerSecond    - Required frequency of the IPR timeout task
 @Param[in]     timeoutPagePtr          - IP Reassembly timeout task page pointer

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_EnableIprTimeoutTask(t_Handle        h_Qe,
                                uint32_t        numOfChecksPerSecond,
                                uint32_t        timeoutPagePtr);

/**************************************************************************//**
 @Function      QE_DisableIprTimeoutTask

 @Description   Disables the QE timeout task for IP Reassembly.

 @Param[in]     h_Qe                - QE module descriptor

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt-safe (do not call from interrupt).
*//***************************************************************************/
t_Error QE_DisableIprTimeoutTask(t_Handle h_Qe);

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**************************************************************************//**
 @Function      QE_RegisterToVTIntr

 @Description   Register to Virtual-Task interrupt.

 @Param[in]     h_Qe    - QE module descriptor.
 @Param[in]     snum    - SNUM.
 @Param[in]     f_Isr   - User callback routine which will be called when
                          the interrupt occurs.
 @Param[in]     handle  - The parameter for the user callback routine.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_RegisterToVTIntr(t_Handle    h_Qe,
                            uint8_t     snum,
                            void        (*f_Isr)(t_Handle handle, uint8_t snum, uint32_t events),
                            t_Handle    handle);

/**************************************************************************//**
 @Function      QE_UnRegisterFromVTIntr

 @Description   Un-Register from Virtual-Task interrupt.

 @Param[in]     h_Qe    - QE module descriptor.
 @Param[in]     snum    - SNUM.
*//***************************************************************************/
void QE_UnRegisterFromVTIntr( t_Handle h_Qe, uint8_t snum );


/**************************************************************************//**
 @Function      QE_AllocExtRequestIntrSrc

 @Description   Allocate an available external request interrupt source.

                The allocated interrupt source must be freed when not needed
                anymore using the QE_FreeExtRequestIntrSrc() routine.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     extRequestId    - External request ID; Use e_QE_EXT_REQUEST_ANY
                                  to get any available source.

 @Return        External request interrupt source if available; -1 otherwise.
*//***************************************************************************/
int QE_AllocExtRequestIntrSrc(t_Handle h_Qe, e_QeExtRequestId extRequestId);

/**************************************************************************//**
 @Function      QE_FreeExtRequestIntrSrc

 @Description   Free an external request interrupt source.

                The interrupt source should have been previously allocated using
                the QE_AllocExtRequestIntrSrc() routine.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - the returned interrupt source. This is the
                          returned value from QE_AllocExtRequestIntrSrc().

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_FreeExtRequestIntrSrc(t_Handle h_Qe, int intrSrc);

/**************************************************************************//**
 @Function      QE_GetAndClearExtReqEvents

 @Description   Gets the unmasked events from an external request events
                register and clears these events.

                Masked events are not returned by this routine. Instead, the
                user may call the QE_GetExtReqEvents() routine to get all
                external request events.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - The external request interrupt source.

 @Return        Unmasked external request events (32-bits), as taken from the
                events register; Returns zero on error.

 @Cautions      For performance reasons, this routine does not verify that the
                interrupt source is indeed an external request interrupt source.
                It is the responsibility of the user to pass the correct
                interrupt source.
*//***************************************************************************/
uint32_t QE_GetAndClearExtReqEvents(t_Handle h_Qe, int intrSrc);

/**************************************************************************//**
 @Function      QE_GetExtReqEvents

 @Description   Gets all events from an external request events register.

                Both masked and unmasked events are returned by this routine.
                This routine may be used for polling the event register when
                interrupts are disabled.

                To get only unmasked events and clear them in one step, use the
                QE_GetAndClearExtReqEvents() routine.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - The external request interrupt source.

 @Return        External request events (32-bits), as taken from the events register;
                Returns zero on error.

 @Cautions      For performance reasons, this routine does not verify that the
                interrupt source is indeed an external request interrupt source.
                It is the responsibility of the user to pass the correct
                interrupt source.
*//***************************************************************************/
uint32_t QE_GetExtReqEvents(t_Handle h_Qe, int intrSrc);

/**************************************************************************//**
 @Function      QE_ClearExtReqEvents

 @Description   Clears the given events in the external request events register.

                The user provides a mask of the events that need to be cleared.
                Event bits that are not set in the mask will not be cleared.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - The external request interrupt source.
 @Param[in]     events  - The events to clear.

 @Return        None.

 @Cautions      For performance reasons, this routine does not verify that the
                interrupt source is indeed an external request interrupt source.
                It is the responsibility of the user to pass the correct
                interrupt source.
*//***************************************************************************/
void QE_ClearExtReqEvents(t_Handle h_Qe, int intrSrc, uint32_t events);

/**************************************************************************//**
 @Function      QE_GetExtReqEventsMask

 @Description   Gets the external request events mask register.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - The external request interrupt source.

 @Return        External request events mask (32-bits), as taken from the events
                mask register; Returns zero on error.

 @Cautions      For performance reasons, this routine does not verify that the
                interrupt source is indeed an external request interrupt source.
                It is the responsibility of the user to pass the correct
                interrupt source.
*//***************************************************************************/
uint32_t QE_GetExtReqEventsMask(t_Handle h_Qe, int intrSrc);

/**************************************************************************//**
 @Function      QE_SetExtReqEventsMask

 @Description   Writes the external request events mask register.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - The external request interrupt source.
 @Param[in]     mask    - The events mask to write.

 @Return        None.

 @Cautions      For performance reasons, this routine does not verify that the
                interrupt source is indeed an external request interrupt source.
                It is the responsibility of the user to pass the correct
                interrupt source.
*//***************************************************************************/
void QE_SetExtReqEventsMask(t_Handle h_Qe, int intrSrc, uint32_t mask);


#define QE_MONITOR_NUM_OF_COUNTERS  2   /**< Number of supported QE monitors counters */

/**************************************************************************//**
 @Description   QE monitor information.

                This structure is returned by the QE_MonitorGetCounters() routine.
*//***************************************************************************/
typedef struct t_QeRiscsMonitor
{
    struct
    {
        uint8_t     percentValue[QE_NUM_OF_RISCS];  /**< Event percentage value */
        uint64_t    absValue[QE_NUM_OF_RISCS];      /**< Event absolute count value */
    } counters[QE_MONITOR_NUM_OF_COUNTERS];

    uint64_t        clkCount[QE_NUM_OF_RISCS];      /**< Clock counter value */

} t_QeRiscsMonitor;


/**************************************************************************//**
 @Description   QE monitor event detection information.

                This structure is returned by the QE_MonitorGetCounters() routine.
*//***************************************************************************/
typedef struct t_QeMonDetectEvent
{
#ifdef QE_REVISION_1X
    uint8_t     snum;       /**< SNUMs value to be tracked by QE Monitor */
#else
    uint8_t     snum[128];  /**< List of SNUMs values to be tracked by QE Monitor */
#endif /* QE_REVISION_1X */
    uint32_t    pc;         /**< PC value to be tracked by QE Monitor */

} t_QeMonDetectEvent;


/**************************************************************************//**
 @Description   QE monitor counter type.
*//***************************************************************************/
typedef enum e_QeMonitorType
{
     e_QE_MONITOR_DUMMY = 0                     /**< Dummy flag value */
    ,e_QE_MONITOR_RISC_UTIL                     /**< Count QE ldsched stall signals, returns non ldsched stall signals */
    ,e_QE_MONITOR_LDSCHED_STALL                 /**< Count QE ldsched stall signals */
    ,e_QE_MONITOR_MURAM_ECC_ERRORS              /**< Count QE MURAM ECC errors */
    ,e_QE_MONITOR_IRAM_ECC_ERRORS               /**< Count QE IRAM ECC errors */
    ,e_QE_MONITOR_ALL_ECC_ERRORS                /**< Count QE both MURAM and IRAM ECC errors*/
    ,e_QE_MONITOR_SYS_BUS_DMA_DONE              /**< Count QE DMA done on system bus */
    ,e_QE_MONITOR_SECONDARY_BUS_DMA_DONE        /**< Count QE DMA done on secondary bus */
    ,e_QE_MONITOR_ALL_DMA_DONE                  /**< Count QE DMA done on both system and local bus */
    ,e_QE_MONITOR_DETECT_EVENT_EDU0             /**< Count QE detect Events for SNUMs*/
    ,e_QE_MONITOR_DETECT_EVENT_EDU1             /**< Count QE detect Events for PCs*/
    ,e_QE_MONITOR_SYS_BUS_MURAM_DMA_EMRG        /**< Count QE MURAM DMA emergency signals on system bus */
    ,e_QE_MONITOR_SYS_BUS_EMRG                  /**< Count QE bus emergency signals on system bus */
    ,e_QE_MONITOR_SYS_BUS_CMD_Q_EMRG            /**< Count QE CMD queue emergency signals on system bus */
    ,e_QE_MONITOR_SYS_BUS_ALL_EMRG              /**< Count QE all emergency signals on system bus */
#ifndef QE_REVISION_1X
    ,e_QE_MONITOR_SECONDARY_BUS_MURAM_DMA_EMRG  /**< Count QE MURAM DMA emergency signals on local bus */
    ,e_QE_MONITOR_SECONDARY_BUS_EMRG            /**< Count QE bus emergency signals on local bus */
    ,e_QE_MONITOR_SECONDARY_BUS_CMD_Q_EMRG      /**< Count QE CMD queue emergency signals on local bus */
    ,e_QE_MONITOR_SECONDARY_BUS_ALL_EMRG        /**< Count QE all emergency signals on local bus */
    ,e_QE_MONITOR_ALL_BUSES_MURAM_DMA_EMRG      /**< Count QE MURAM DMA emergency signals on both system and local bus */
    ,e_QE_MONITOR_ALL_BUSES_EMRG                /**< Count QE bus emergency signals on both system and local bus */
    ,e_QE_MONITOR_ALL_BUSES_CMD_Q_EMRG          /**< Count QE CMD queue emergency signals on both system and local bus */
    ,e_QE_MONITOR_ALL_BUSES_ALL_EMRG            /**< Count QE all emergency signals on both system and local bus */
 #endif /* QE_REVISION_1X */
} e_QeMonitorType;

/**************************************************************************//**
 @Function      QE_MonitorStart

 @Description   Start the QE RISCs monitoring.

 @Param[in]     h_Qe          - QE module descriptor

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MonitorStart(t_Handle h_Qe);

/**************************************************************************//**
 @Function      QE_MonitorStop

 @Description   Stop the QE RISCs monitoring.

 @Param[in]     h_Qe - QE module descriptor

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MonitorStop(t_Handle h_Qe);

/**************************************************************************//**
 @Function      QE_MonitorSetCounters

 @Description   Set the QE RISCs monitoring registers.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     eventType       - The event mask to register to the QE monitor module
 @Param[in]     p_DetectEvent   - The event value (SNUM and/or PC) to be tracked
                                  by the QE monitor module (valid only when a detect
                                  event is registered to the QE monitor);
@Param[out]     p_CounterId     - The counter ID value that will count the registered
                                  event;

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MonitorSetCounters(t_Handle              h_Qe,
                              e_QeMonitorType       eventType,
                              t_QeMonDetectEvent    *p_DetectEvent,
                              uint32_t              *p_CounterId);

/**************************************************************************//**
 @Function      QE_MonitorGetCounters

 @Description   Gets the QE RISCs counters.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[out]    p_RiscsMon  - RISCs monitoring structure.

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MonitorGetCounters(t_Handle h_Qe, t_QeRiscsMonitor *p_RiscsMon);


/**************************************************************************//**
 @Description   Port owner possibilities.
*//***************************************************************************/
typedef enum e_PortOwner
{
    e_PORT_OWNER_UPC = 1,   /**< Port owner is UPC */
    e_PORT_OWNER_MTC,       /**< Port owner is MTC */
    e_PORT_OWNER_DUMMY_LAST
} e_PortOwner;

/**************************************************************************//**
 @Function      QE_MapSpecificPort

 @Description   register ONE port number.

 @Param[in]     h_Qe        - QE module descriptor.
 @Param[in]     utopiaCtl   - Utopia controller number
 @Param[in]     device      - Device number
 @Param[in]     port        - Port number
 @Param[in]     rxCommCtl   - Rx communication controller.
 @Param[in]     txCommCtl   - Tx communication controller.
 @Param[in]     portOwner   - The port owner (UPC, MTC )

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_MapSpecificPort(t_Handle     h_Qe,
                           uint8_t      utopiaCtl,
                           uint8_t      device,
                           uint8_t      port,
                           uint8_t      rxCommCtl,
                           uint8_t      txCommCtl,
                           e_PortOwner  portOwner);

/**************************************************************************//**
 @Function      QE_MapSpecificPortsRange

 @Description   Map specific ports.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     firstUtopiaCtl  - First uid
 @Param[in]     lastUtopiaCtl   - Last uid
 @Param[in]     firstDevice     - First device number
 @Param[in]     lastDevice      - Last device number
 @Param[in]     firstPort       - First port number
 @Param[in]     lastPort        - Last port number
 @Param[in]     rxCommCtl       - Rx communication controller.
 @Param[in]     txCommCtl       - Tx communication controller.
 @Param[in]     portOwner       - The port owner (UPC, MTC)

 @Return        E_OK on success; Error code otherwise.

 @Cautions      When mapping POS ports:
                Rx and Tx communication controllers must be the same controller.
*//***************************************************************************/
t_Error QE_MapSpecificPortsRange(t_Handle       h_Qe,
                                 uint8_t        firstUtopiaCtl,
                                 uint8_t        lastUtopiaCtl,
                                 uint8_t        firstDevice,
                                 uint8_t        lastDevice,
                                 uint8_t        firstPort,
                                 uint8_t        lastPort,
                                 uint8_t        rxCommCtl,
                                 uint8_t        txCommCtl,
                                 e_PortOwner    portOwner);

/**************************************************************************//**
 @Function      QE_MapPorts

 @Description   Map unspecific ports.

 @Param[in]     h_Qe        - QE module descriptor.
 @Param[in]     numOfPorts  - Requested number of ports to map.
 @Param[in]     rxCommCtl   - Rx communication controller.
 @Param[in]     txCommCtl   - Tx communication controller.
 @Param[in]     portOwner   - The port owner (UPC, MTC )

 @Return        Number of allocated ports.
*//***************************************************************************/
uint16_t QE_MapPorts(t_Handle       h_Qe,
                     uint16_t       numOfPorts,
                     uint8_t        rxCommCtl,
                     uint8_t        txCommCtl,
                     e_PortOwner    portOwner);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      QE_DumpRegs

 @Description   Dumps the registers of the QE object.

 @Param[in]     h_Qe - QE module descriptor

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Qe must have been previously initialized using QE_Init().
*//***************************************************************************/
t_Error QE_DumpRegs(t_Handle h_Qe);

#endif /* DEBUG_ERRORS > 0 */

/**************************************************************************//*
 @Description   QE Mux groups
                according to registers separation
*//***************************************************************************/
typedef enum e_QeMuxGroup
{
    e_QE_MUX_UCC_CLK,
    e_QE_MUX_UPC_CLK,
    e_QE_MUX_SI1_CLK,
    e_QE_MUX_SI1_SYNC,
    e_QE_MUX_SI2_CLK,
    e_QE_MUX_SI2_SYNC,
    e_QE_MUX_USB_CLK,
    e_QE_MUX_RTC_CLK,
    e_QE_MUX_DUMMY_LAST
} e_QeMuxGroup;

/**************************************************************************//*
 @Function      QE_SetMux

 @Description   Sets the QE MUX (including legality checking) for a specified device

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     grp         - Signals are grouped according to types. use e_QeMuxGroup.
 @Param[in]     idInGroup   - Id of TDM, UCC or UPC.
 @Param[in]     clock       - QE clock enum
 @Param[in]     dir         - Rx or Tx

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error    QE_SetMux(t_Handle       h_Qe,
                     e_QeMuxGroup   grp,
                     uint8_t        idInGroup,
                     e_QeClock      clock,
                     e_CommMode     dir);

/**************************************************************************//*
 @Function      QE_MuxSetUccForMiiMng

 @Description   Sets the QE MUX for MII management

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     uccNum  - Id of UCC.

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MuxSetUccForMiiMng(t_Handle h_Qe, e_Ucc uccNum);


/**************************************************************************//*
 @Function      QE_MuxGetUccForMiiMng

 @Description   Gets the UCC that control the MII management

 @Param[in]     h_Qe    - QE module descriptor
 @Param[out]    uccNum  - Id of UCC.

 @Return        E_OK on success, other value otherwise
*//***************************************************************************/
t_Error QE_MuxGetUccForMiiMng(t_Handle h_Qe, e_Ucc *uccNum);


/**************************************************************************//*
 @Function      QE_MuxSetUpcTxRxInt

 @Description   The routine set the cmxupcr register - for UPC driver use only.

 @Param[in]     h_Qe         - QE module descriptor
 @Param[in]     upcId        - upc number (upc1 or upc2).
 @Param[in]     txClock      - transmitter clock source.
 @Param[in]     rxClock      - receiver clock source.
 @Param[in]     intRateClock - internal rate clock source.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_MuxSetUpcTxRxInt(t_Handle    h_Qe,
                            e_Upc       upcId,
                            e_QeClock   txClock,
                            e_QeClock   rxClock,
                            e_QeClock   intRateClock);


/**************************************************************************//*
 @Description   Interrupt table entry.
*//***************************************************************************/
typedef _Packed  struct t_QeIntrTableEntry
{
    volatile uint16_t  status;  /*< high byte - V bit, W bit, and  interrupt flags. */
    volatile uint16_t  chnum;   /*< channel number */
} _PackedType t_QeIntrTableEntry;

/**************************************************************************//*
 @Description   Basic buffer descriptor (BD) structure
*//***************************************************************************/
typedef _Packed struct t_QeBd
{
     volatile uint16_t  status;
     volatile uint16_t  length;
     volatile uint32_t  buffer;
} _PackedType t_QeBd;


#define QE_SIZEOF_BD                    sizeof(t_QeBd)

#define BD_STATUS_MASK                  0xffff0000
#define BD_LENGTH_MASK                  0x0000ffff

#define BD_BUFFER_ARG(_p_Bd)            ((t_QeBd *)(_p_Bd))->buffer
#define BD_STATUS_LENGTH_ARG(_p_Bd)     (*(volatile uint32_t *)(_p_Bd))

#define BD_STATUS_AND_LENGTH(_p_Bd) \
    GET_UINT32(BD_STATUS_LENGTH_ARG(_p_Bd))

#define BD_BUFFER_CLEAR(_p_Bd) \
    WRITE_UINT32(BD_BUFFER_ARG(_p_Bd), 0);
#define BD_BUFFER(_p_Bd) \
    (uint8_t *)XX_PhysToVirt((uint32_t)GET_UINT32(BD_BUFFER_ARG(_p_Bd)))
#define BD_BUFFER_NEW(_p_Bd, _f_PhysToVirt) \
    (uint8_t *)_f_PhysToVirt((uint32_t)GET_UINT32(BD_BUFFER_ARG(_p_Bd)))

#define BD_STATUS_AND_LENGTH_SET(_p_Bd, val) \
    WRITE_UINT32(BD_STATUS_LENGTH_ARG(_p_Bd), (val))


/* Macro for retrieving the next BD ID in the ring */
#define BD_NEXT_ID(_bdId, _bdStatus, _wrapFlag) \
    (uint16_t)(((_bdStatus) & (_wrapFlag)) ? 0 : ((_bdId) + 1))

/* Macro for retrieving the next BD in the ring, according to pointer type */
#define BD_NEXT_PTR(_p_Bd, _p_BdRing, _bdStatus, _wrapFlag) \
    (((_bdStatus) & (_wrapFlag)) ? _p_BdRing : ((_p_Bd) + 1))

/* Macro for retrieving the next BD in the ring, according to specified size */
#define BD_NEXT_BY_SIZE(_p_Bd, _p_BdRing, _bdSize, _bdStatus, _wrapFlag) \
    (((_bdStatus) & (_wrapFlag)) ? _p_BdRing : ((uint8_t *)(_p_Bd) + (_bdSize)))


#ifndef BD_BUFFER_SET
#define BD_BUFFER_SET(_p_Bd, _p_Buffer) \
        WRITE_UINT32(BD_BUFFER_ARG(_p_Bd), (uint32_t)XX_VirtToPhys((void*)(_p_Buffer)))
#define BD_BUFFER_SET_NEW(_p_Bd, _p_Buffer, _f_VirtToPhys) \
        WRITE_UINT32(BD_BUFFER_ARG(_p_Bd), (uint32_t)_f_VirtToPhys((void*)(_p_Buffer)))
#endif


/* BRG configuration register */
#define QE_BRGC_DIVISOR_MAX     0xFFF

#define NANOSEC_PER_MICROSEC    1000


/**************************************************************************//*
 @Function      QE_GetPart

 @Description   Gets the part associated with the QE.

 @Param[in]     h_Qe - QE module descriptor

 @Return        the part associated with the QE
*//***************************************************************************/
t_Handle QE_GetPart(t_Handle h_Qe);

/**************************************************************************//*
 @Function      QE_GetMuramBase

 @Description   Gets the base address of the Multi-User RAM.

 @Param[in]     h_Qe - QE module descriptor

 @Return        The base address of the Multi-User RAM.
*//***************************************************************************/
uint32_t QE_GetMuramBase(t_Handle h_Qe);

/**************************************************************************//*
 @Function      QE_IsAddrInMuram

 @Description   Checks if a given address is contained in the Multi-User RAM.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     address - the address to check

 @Retval        TRUE    - The address is contained in the Multi-User RAM;
 @Retval        FALSE   - The address is not contained in the Multi-User RAM.
*//***************************************************************************/
bool QE_IsAddrInMuram(t_Handle h_Qe, uint32_t address);

/**************************************************************************//*
 @Function      QE_IsMemPartitionInMuram

 @Description   Checks if a given partition is contained in the Multi-User RAM.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     memPartitionId  - The memory partition ID to check

 @Retval        TRUE    - The partition is contained in the Multi-User RAM;
 @Retval        FALSE   - The partition is not contained in the Multi-User RAM.
*//***************************************************************************/
bool QE_IsMemPartitionInMuram(t_Handle h_Qe, uint8_t memPartitionId);


/**************************************************************************//*
 @Description   QE CECR Commands.
*//***************************************************************************/
typedef enum e_QeCmdOpcode
{
      e_QE_CMD_RESET                         = 0x80000000
    , e_QE_CMD_ASSIGN_RISC                   = 0x00000010
    , e_QE_CMD_ASSIGN_PAGE                   = 0x00000012
    , e_QE_CMD_ASSIGN_PAGE_TO_DEVICE         = 0x00000016
    , e_QE_CMD_PUSH_SCHED                    = 0x0000000f
    , e_QE_CMD_SET_TIMER                     = 0x00000008
    , e_QE_CMD_INIT_TX_RX                    = 0x00000000
    , e_QE_CMD_INIT_RX                       = 0x00000001
    , e_QE_CMD_INIT_TX                       = 0x00000002
    , e_QE_CMD_ENTER_HUNT_MODE               = 0x00000003
    , e_QE_CMD_STOP_TX                       = 0x00000004
    , e_QE_CMD_GRACEFUL_STOP_TX              = 0x00000005
    , e_QE_CMD_RESTART_TX                    = 0x00000006
    , e_QE_CMD_SET_GROUP_ADDRESS             = 0x00000008
    , e_QE_CMD_RESET_BCS                     = 0x0000000a
    , e_QE_CMD_SET_LAST_RX_REQUEST_THR       = 0x00000013
    , e_QE_CMD_ADD_REMOVE_HASH_ENTRY         = 0x00000013
    , e_QE_CMD_ADD_REMOVE_LPM_ENTRY          = 0x00000015
    , e_QE_CMD_START_FLOW_CONTROL            = 0x00000014
    , e_QE_CMD_STOP_FLOW_CONTROL             = 0x00000015
    , e_QE_CMD_GRACEFUL_STOP_RX              = 0x0000001a
    , e_QE_CMD_RESTART_RX                    = 0x0000001b
    , e_QE_CMD_L2_SWITCH                     = 0x00000007
    , e_QE_CMD_ATM_TX                        = 0x0000000a
    , e_QE_CMD_ATM_INIT_MULTI_THREAD         = 0x00000011
    , e_QE_CMD_MSP_CELL_POOL_GET             = 0x0000000b
    , e_QE_CMD_MSP_CELL_POOL_PUT             = 0x0000000c
    , e_QE_CMD_IMA                           = 0x0000000d
    , e_QE_CMD_USB_STOP_TX                   = 0x0000000a
    , e_QE_CMD_USB_RESTART_TX                = 0x0000000c
    , e_QE_CMD_QMC_STOP_TX                   = 0x0000000c
    , e_QE_CMD_QMC_STOP_RX                   = 0x0000000d
    , e_QE_CMD_MCC_INIT_TX_RX_16             = 0x00000003
    , e_QE_CMD_MCC_STOP_TX                   = 0x00000004
    , e_QE_CMD_MCC_INIT_TX_1                 = 0x00000005
    , e_QE_CMD_MCC_INIT_RX_1                 = 0x00000006
    , e_QE_CMD_MCC_RESET                     = 0x00000007
    , e_QE_CMD_MCC_INSERT_CELL               = 0x00000009
    , e_QE_CMD_MCC_STOP_RX                   = 0x00000009
    , e_QE_CMD_SS7_RESET_SU_FILL             = 0x00000011
    , e_QE_CMD_PPPoHT_INIT_TX_LINK           = 0x0000000b
    , e_QE_CMD_PPPoHT_INIT_TX_BACKEND        = 0x0000000d
    , e_QE_CMD_PPPoHT_INIT_MUX               = 0x0000000e
    , e_QE_CMD_PPPoHT_INIT_RX_LINK           = 0x0000001b
    , e_QE_CMD_PPPoHT_INIT_RX_BACKEND        = 0x0000001d
    , e_QE_CMD_PPPoHT_INIT_DEMUX             = 0x0000001e
    , e_QE_CMD_PPPoHT_INIT_TX_RX_LINK        = 0x0000002b
    , e_QE_CMD_PPPoHT_INIT_IW                = 0x0000002c
    , e_QE_CMD_PPPoHT_INIT_TX_RX_BACKEND     = 0x0000002d
    , e_QE_CMD_PPPoHT_INIT_MUX_DEMUX         = 0x0000002e
    , e_QE_CMD_PPPoHT_LINK_BUFFER_RELEASE    = 0x0000001C
    , e_QE_CMD_UCC_POS_COMMON_TX_INIT        = 0x00000025
    , e_QE_CMD_UCC_POS_TX_INIT               = 0x00000026
    , e_QE_CMD_UCC_POS_PORT_TX_INIT          = 0x00000027
    , e_QE_CMD_UCC_POS_PORT_TX_GRACEFUL_STOP = 0x00000005
    , e_QE_CMD_UCC_POS_PORT_TX_RESTART       = 0x00000006
    , e_QE_CMD_UCC_POS_COMMON_RX_INIT        = 0x00000022
    , e_QE_CMD_UCC_POS_RX_INIT               = 0x00000023
    , e_QE_CMD_UCC_POS_PORT_RX_INIT          = 0x00000024
    , e_QE_CMD_UCC_POS_PORT_RX_GRACEFUL_STOP = 0x0000001a
    , e_QE_CMD_UCC_POS_PORT_RX_RESTART       = 0x0000001b
    , e_QE_CMD_IW_THREADS_INIT               = 0x0000001a
    , e_QE_CMD_IP_REASSEMBLY_INIT            = 0x00000038
    , e_QE_CMD_IP_REASSEMBLY_ABORT           = 0x0000001c
    , e_QE_CMD_IP_FRAGMENTATION_INIT         = 0x0000002e
    , e_QE_CMD_MCC_MID_INIT_TX_1             = 0x0000002f
    , e_QE_CMD_MCC_MID_INIT_TX_RX_16         = 0x00000030
    , e_QE_CMD_VP_INIT                       = 0x0000001a
    , e_QE_CMD_VP_START                      = 0x0000001b
    , e_QE_CMD_IPSEC                         = 0x0000001d
    , e_QE_CMD_UMCC_INIT                     = 0x0000000e
    , e_QE_CMD_PTP_TX_ACC_INIT               = 0x00000013

} e_QeCmdOpcode;


/**************************************************************************//*
 @Description   QE CECR Sub Block - sub block of QE command.
*//***************************************************************************/
typedef enum e_QeCmdSubBlock
{
      e_QE_CMD_SUBBLOCK_INVALID     = 0x00000000
    , e_QE_CMD_SUBBLOCK_UCCFAST1    = 0x02000000
    , e_QE_CMD_SUBBLOCK_UCCFAST2    = 0x02200000
    , e_QE_CMD_SUBBLOCK_UCCFAST3    = 0x02400000
    , e_QE_CMD_SUBBLOCK_UCCFAST4    = 0x02600000
    , e_QE_CMD_SUBBLOCK_UCCFAST5    = 0x02800000
    , e_QE_CMD_SUBBLOCK_UCCFAST6    = 0x02a00000
    , e_QE_CMD_SUBBLOCK_UCCFAST7    = 0x02c00000
    , e_QE_CMD_SUBBLOCK_UCCFAST8    = 0x02e00000
    , e_QE_CMD_SUBBLOCK_USB         = 0x03200000
    , e_QE_CMD_SUBBLOCK_MCC1        = 0x03800000
    , e_QE_CMD_SUBBLOCK_MCC2        = 0x03a00000
    , e_QE_CMD_SUBBLOCK_GENERAL     = 0x03c00000
    , e_QE_CMD_SUBBLOCK_UCCSLOW1    = 0x00000000
    , e_QE_CMD_SUBBLOCK_UCCSLOW2    = 0x00200000
    , e_QE_CMD_SUBBLOCK_UCCSLOW3    = 0x00400000
    , e_QE_CMD_SUBBLOCK_UCCSLOW4    = 0x00600000
    , e_QE_CMD_SUBBLOCK_UCCSLOW5    = 0x00800000
    , e_QE_CMD_SUBBLOCK_UCCSLOW6    = 0x00a00000
    , e_QE_CMD_SUBBLOCK_UCCSLOW7    = 0x00c00000
    , e_QE_CMD_SUBBLOCK_UCCSLOW8    = 0x00e00000
    , e_QE_CMD_SUBBLOCK_SPI1        = 0x01400000
    , e_QE_CMD_SUBBLOCK_SPI2        = 0x01600000
    , e_QE_CMD_SUBBLOCK_SERIAL_TASK = 0x01c20000
    , e_QE_CMD_SUBBLOCK_TIMER       = 0x01e00000

} e_QeCmdSubBlock;


/**************************************************************************//*
 @Description   QE CECR Protocol - For non-MCC, specifies mode for QE CECR command.
*//***************************************************************************/
typedef enum e_QeCmdProtocol
{
      e_QE_CMD_PROTOCOL_UNSPECIFIED = 0x00
    , e_QE_CMD_PROTOCOL_HDLC        = 0x00
    , e_QE_CMD_PROTOCOL_TRANS       = 0x00
    , e_QE_CMD_PROTOCOL_QMC         = 0x02
    , e_QE_CMD_PROTOCOL_UART        = 0x04
    , e_QE_CMD_PROTOCOL_ATM_POS     = 0x0A
    , e_QE_CMD_PROTOCOL_ETHERNET    = 0x0C
    , e_QE_CMD_PROTOCOL_L2_SWITCH   = 0x0D

} e_QeCmdProtocol;


/**************************************************************************//*
 @Description   BMR byte order
*//***************************************************************************/
typedef enum e_QeBmrByteOrderBO
{
      e_QE_BMR_BYTE_ORDER_BO_PPC = 0x08     /* PowerPC little endian */
    , e_QE_BMR_BYTE_ORDER_BO_MOT = 0x10     /* Motorola big endian   */
} e_QeBmrByteOrderBO;


/**************************************************************************//*
 @Function      QE_IssueCmd

 @Description   Issues a QE command synchronously via the CECR register.

                This function waits for command completion.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     cmd             - Opcode as in CECR documentation
 @Param[in]     subBlockOrSnum  - For most commands it is the sub-block code
                                  (e_QeCmdSubBlock);
                                  For e_QE_CMD_ASSIGN_PAGE, e_QE_CMD_ASSIGN_RISC
                                  and e_QE_CMD_PUSH_SCHED opcodes, it is the
                                  required SNUM.
 @Param[in]     protocolInfo    - For MCC use the MCC channel number;
                                  For non-MCC use e_QeCmdProtocol.
 @Param[in]     cmdData         - Value to be placed into CECDR register
                                  before execution of command.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_IssueCmd(t_Handle        h_Qe,
                    e_QeCmdOpcode   cmd,
                    uint32_t        subBlockOrSnum,
                    uint8_t         protocolInfo,
                    uint32_t        cmdData);

/**************************************************************************//*
 @Function      QE_IssueCmdAsync

 @Description   Issues a QE command asynchronously, via the CECR register.

                This function does not wait for current command completion,
                but it does wait for previous command completion (so it is
                safe to call it consecutively with two commands).

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     cmd             - Opcode as in CECR documentation
 @Param[in]     subBlockOrSnum  - For most commands it is the sub-block code
                                  (e_QeCmdSubBlock);
                                  For e_QE_CMD_ASSIGN_PAGE, e_QE_CMD_ASSIGN_RISC
                                  and e_QE_CMD_PUSH_SCHED opcodes, it is the
                                  required SNUM.
 @Param[in]     protocolInfo    - For MCC use the MCC channel number;
                                  For non-MCC use e_QeCmdProtocol.
 @Param[in]     cmdData         - Value to be placed into CECDR register
                                  before execution of command.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_IssueCmdAsync(t_Handle        h_Qe,
                         e_QeCmdOpcode   cmd,
                         uint32_t        subBlockOrSnum,
                         uint8_t         protocolInfo,
                         uint32_t        cmdData);

/**************************************************************************//*
 @Function      QE_WaitForCmdCompletion

 @Description   Waits for CECR command flag bit to clear.

 @Param[in]     h_Qe - QE module descriptor

 @Retval        None.
*//***************************************************************************/
void QE_WaitForCmdCompletion(t_Handle h_Qe);

/**************************************************************************//*
 @Function      QE_GetUCodeVirtualTrap

 @Description   Get the value of a specified microcode virtual trap entry.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     virtTrapId  - Virtual trap ID

 @Return        Virtual trap value; Zero if virtual trap is inactive.
*//***************************************************************************/
uint32_t QE_GetUCodeVirtualTrap(t_Handle h_Qe, e_QeVirtTrapId virtTrapId);

/**************************************************************************//*
 @Function      QE_GetUCodeExtendedModes

 @Description   Get the value of the microcode extended modes.

 @Param[in]     h_Qe - QE module descriptor.

 @Return        Extended mode value; Zero if extended mode is inactive.
*//***************************************************************************/
uint64_t QE_GetUCodeExtendedModes(t_Handle h_Qe);


/**************************************************************************//**
 @Function      QE_ModifyExternalRequestMode

 @Description   Modify the sensitivity/polarity modes of external request pins.
                [e_QE_EXT_REQ_MODE_LOW_TO_HIGH_CHANGE]

 @Param[in]     h_Qe            - Handle to QE object.
 @Param[in]     extRequestId    - Specific external request index.
 @Param[in]     extRequestMode  - Selected operation mode.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_ModifyExternalRequestMode(t_Handle           h_Qe,
                                     e_QeExtRequestId   extRequestId,
                                     e_QeExtRequestMode extRequestMode);

/**************************************************************************//*
 @Function      QE_GetStaticSnum

 @Description   Gets the value of an SNUM that is statically assigned to an
                I/O peripheral.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     moduleId    - Module ID
 @Param[in]     commMode    - e_COMM_MODE_TX or e_COMM_MODE_RX
 @Param[out]    p_Snum      - Returns requested SNUM, or 0xFF if not applicable.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetStaticSnum(t_Handle h_Qe, e_ModuleId moduleId, e_CommMode commMode, uint8_t *p_Snum);

/**************************************************************************//*
 @Function      QE_GetExtReqSnum

 @Description   Gets the value of an SNUM that is statically assigned to an
                external request.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     intrSrc - External request interrupt source.
 @Param[out]    p_Snum  - Returns requested SNUM, or 0xFF if not applicable.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetExtReqSnum(t_Handle h_Qe, int intrSrc, uint8_t *p_Snum);

/**************************************************************************//*
 @Function      QE_GetSerialTaskSnum

 @Description   Gets the value of an SNUM that is statically assigned to
                the serial task.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[out]    p_Snum  - Requested SNUM, or 0xFF if not applicable.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetSerialTaskSnum(t_Handle h_Qe, uint8_t *p_Snum);

/**************************************************************************//*
 @Function      QE_GetTimerTaskSnum

 @Description   Gets the value of an SNUM that is statically assigned to
                the timer task.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[out]    p_Snum  - Returns requested SNUM, or 0xFF if not applicable.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetTimerTaskSnum(t_Handle h_Qe, uint8_t *p_Snum);

/**************************************************************************//*
 @Function      QE_GetHostCommandSnum

 @Description   Gets the value of an SNUM that is statically assigned to
                the host command task.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[out]    p_Snum  - Returns requested SNUM, or 0xFF if not applicable.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetHostCommandSnum(t_Handle h_Qe, uint8_t *p_Snum);

/**************************************************************************//*
 @Function      QE_GetFreeSnum

 @Description   Gets and reserves an available SNUM for use

 @Param[in]     h_Qe    - QE module descriptor
 @Param[out]    p_Snum  - Available SNUM
 @Param[in]     internalThread - indication if an internal thread is required

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_GetFreeSnum(t_Handle h_Qe, uint8_t *p_Snum, bool internalThread);

/**************************************************************************//*
 @Function      QE_ReleaseSnum

 @Description   Returns a used SNUM after use.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     snum    - SNUM to return

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_ReleaseSnum(t_Handle h_Qe, uint8_t snum);


/**************************************************************************//*
 @Function      QE_MuxSetUccGrant

 @Description   Sets grant bit in appropriate CMXCUR register based on UCC number.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     uccNum  - UCC number
 @Param[in]     setClr  - boolean whether to set grant bit or not

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_MuxSetUccGrant(t_Handle h_Qe, e_Ucc uccNum, bool setClr);

/**************************************************************************//*
 @Function      QE_MuxSetUccTsa

 @Description   Sets TSA bit in appropriate CMXCUR register based on UCC number.

 @Param[in]     h_Qe    - QE module descriptor
 @Param[in]     uccNum  - UCC number
 @Param[in]     setClr  - boolean whether to set tsa bit or not

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error QE_MuxSetUccTsa(t_Handle h_Qe, e_Ucc uccNum, bool setClr);

/**************************************************************************//*
 @Function      QE_GetUccFastCmdSubBlock

 @Description   Translates fast UCC number into appropriate QE Command reg. sub block.

 @Param[in]     uccNum - number of fast UCC  (0-7).

 @Return        QE Command reg. sub block. CR_SUBBLOCK_INVALID if invalid input.
*//***************************************************************************/
e_QeCmdSubBlock QE_GetUccFastCmdSubBlock( e_Ucc uccNum );

/**************************************************************************//*
 @Function      QE_GetUccSlowCmdSubBlock

 @Description   Translates slow UCC number into appropriate QE Command reg. sub block.

 @Param[in]     uccNum - Number of slow UCC  (0-7).

 @Return        QE Command reg. sub block. CR_SUBBLOCK_INVALID if invalid input.
*//***************************************************************************/
e_QeCmdSubBlock QE_GetUccSlowCmdSubBlock( e_Ucc uccNum );


/**************************************************************************//*
 @Function      QE_RegisterPortsRange

 @Description   register port number.

 @Param[in]     h_Qe            - QE module descriptor
 @Param[in]     firstUtopiaCtl  - First uid
 @Param[in]     lastUtopiaCtl   - Last uid
 @Param[in]     firstDevice     - First device number
 @Param[in]     lastDevice      - Last device number
 @Param[in]     firstPort       - First port number
 @Param[in]     lastPort        - Last port number
 @Param[in]     portOwner       - Owner of the port.
 @Param[in]     unRegister      - FALSE - register port;
                                  TRUE - unregister ports.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_RegisterPortsRange(t_Handle      h_Qe,
                              uint8_t       firstUtopiaCtl,
                              uint8_t       lastUtopiaCtl,
                              uint8_t       firstDevice,
                              uint8_t       lastDevice,
                              uint8_t       firstPort,
                              uint8_t       lastPort,
                              e_PortOwner   portOwner,
                              bool          unRegister);

/**************************************************************************//*
 @Function      QE_GetCommCtlOfDevice

 @Description   Gets the communication controller number of a specific
                Utopia device.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     utopiaCtl   - Utopia controller number.
 @Param[in]     device      - Device number.
 @Param[in]     commMode    - Tx/Rx mode.

 @Return        Communication controller number, or 0xFF if function failed.
*//***************************************************************************/
uint8_t QE_GetCommCtlOfDevice(t_Handle      h_Qe,
                              uint8_t       utopiaCtl,
                              uint8_t       device,
                              e_CommMode    commMode);

/**************************************************************************//*
 @Function      QE_GetPortOwner

 @Description   Gets the port owner of a specific port.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     utopiaCtl   - Upc index.
 @Param[in]     device      - Device index.
 @Param[in]     portId      - Port index.

 @Return        port owner one of the e_PortOwner.
*//***************************************************************************/
e_PortOwner QE_GetPortOwner( t_Handle      h_Qe,
                             uint8_t       utopiaCtl,
                             uint8_t       device,
                             uint8_t       portId );


/**************************************************************************//**
 @Description   QE serial task's interrupt source enumeration
 
                These values are passed to QE_EnableSerialTaskInterrupt()
                or QE_DisableSerialTaskInterrupt() routines.
*//***************************************************************************/
typedef enum e_QeSerialIntrSrc
{
    e_QE_SERIAL_INTR_SRC_UCC_1,         /**< Serial task is triggered by interrupt from UCC-1 */
    e_QE_SERIAL_INTR_SRC_UCC_2,         /**< Serial task is triggered by interrupt from UCC-2 */
    e_QE_SERIAL_INTR_SRC_UCC_3,         /**< Serial task is triggered by interrupt from UCC-3 */
    e_QE_SERIAL_INTR_SRC_UCC_4,         /**< Serial task is triggered by interrupt from UCC-4 */
    e_QE_SERIAL_INTR_SRC_UCC_5,         /**< Serial task is triggered by interrupt from UCC-5 */
    e_QE_SERIAL_INTR_SRC_UCC_6,         /**< Serial task is triggered by interrupt from UCC-6 */
    e_QE_SERIAL_INTR_SRC_UCC_7,         /**< Serial task is triggered by interrupt from UCC-7 */
    e_QE_SERIAL_INTR_SRC_UCC_8,         /**< Serial task is triggered by interrupt from UCC-8 */
    e_QE_SERIAL_INTR_SRC_PTP_2,         /**< Serial task is triggered by interrupt from PTP-2 */
    e_QE_SERIAL_INTR_SRC_PTP_1,         /**< Serial task is triggered by interrupt from PTP-1 */
    e_QE_SERIAL_INTR_SRC_RTC,           /**< Serial task is triggered by interrupt from RTC */
    e_QE_SERIAL_INTR_SRC_USB,           /**< Serial task is triggered by interrupt from USB */
    e_QE_SERIAL_INTR_SRC_SPI_1,         /**< Serial task is triggered by interrupt from SPI-1 */
    e_QE_SERIAL_INTR_SRC_SPI_2,         /**< Serial task is triggered by interrupt from SPI-2 */
    e_QE_SERIAL_INTR_SRC_MCC_1,         /**< Serial task is triggered by interrupt from MCC-1 */
    e_QE_SERIAL_INTR_SRC_MCC_2,         /**< Serial task is triggered by interrupt from MCC-2 */
    e_QE_SERIAL_INTR_SRC_DUMMY_LAST
}e_QeSerialIntrSrc;

/**************************************************************************//*
 @Function      QE_EnableSerialTaskInterrupt

 @Description   Enable interrupting QE serial task instead of CPU.
 
                This routine enables QE to respond to an interrupt
                generated by QE peripheral, thus off-loading CPU from
                interrupt processing.

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     intrSrc     - Peripheral to enable interrupt to QE from

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_EnableSerialTaskInterrupt(t_Handle h_Qe, e_QeSerialIntrSrc intrSrc);

/**************************************************************************//*
 @Function      QE_DisableSerialTaskInterrupt

 @Description   Disable interrupting QE serial task instead of CPU.
 
                This routine disables QE response to an interrupt
                generated by QE peripheral, which has been previously
                enables through QE_EnableSerialTaskInterrupt().

 @Param[in]     h_Qe        - QE module descriptor
 @Param[in]     intrSrc     - Peripheral to disable interrupt to QE from

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error QE_DisableSerialTaskInterrupt(t_Handle h_Qe, e_QeSerialIntrSrc intrSrc);


/** @} */ /* end of qe_ctrl_grp group */

/** @} */ /* end of qe__grp group */


#endif /* __QE_EXT_H */

