/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          qe_fltr_filter_ext.h

 @Description   Declarations for public data types and functions
                 for the Extended Filtering Module
*/

#ifndef __QE_FLTR_FILTER_EXT_H
#define __QE_FLTR_FILTER_EXT_H


#ifdef __cplusplus
extern "C"{
#endif

/* INCLUDE FILES */
#include "std_ext.h"
#include "xx_ext.h"

#include "qe_fltr_table_ext.h"




/* CONSTANTS */
#define QE_FLTR_KEY_GENERAL_MAX_NUM_FIELDS          6   /* Maximum numbaer of fields supported by the e_QE_FLTR_FILTER_TYPE_GENERAL */
#define QE_FLTR_AD_SIZE                             8   /* Size in Bytes of an ACTIOND_DESCRIPTOR PCD */


/* MACROS */
/* Macros useful for key compression in Indexed Table */
#define QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BITS(num_bits)      ~(~(uint32_t)0 >> (num_bits))
#define QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BYTES(num_bytes)    QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BITS(num_bytes<<3)


/* ENUMS */
/**************************************************************************//**
 @Group          fltr_grp Extended Filtering Application Programming Interface

 @Description    Extended Filtering API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          fltr_filter_grp Filter Object Application Programming Interface

 @Description    Filter Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         fltr_filter_init_grp Filter Object Initialization Unit

 @Description   Filter Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description   Enumeration of Filter Types
*/
typedef enum e_QeFltrFilterType{
 
      e_QE_FLTR_FILTER_TYPE_ETH_FAST        = 0x00          /**< Used on the Ethernet interface. Performs no header parsing. Can generate a key from the Ethernet L2 frame header fields and performs lookup using it. */
    , e_QE_FLTR_FILTER_TYPE_ETH_FAST_TERM   = 0x01          /**< Used on the Ethernet interface (in Termination mode, not Interworking ). Performs no header parsing. Can generate a key from the Ethernet L2 frame header fields and performs lookup using it. */
    , e_QE_FLTR_FILTER_TYPE_ETH             = 0x02          /**< Used on the Ethernet interface. Performs automatic parsing of headers (recognizes valid protocol stacks form L2, L3, L4 frame header fields) and generate a key from parsed headers, if requested. Performs lookup with the extracted key. */
    , e_QE_FLTR_FILTER_TYPE_PPP_FAST        = 0x03          /**< Used on the PPP interface. This Filter is used to handle a received PPP frame.The frame can be demultiplexed by the QUICCEngine and each subframe be forwarded separately or the whole frame can be forwarded. In case of demux-ing each subframe's PID will be checked against the PPP PID-Filter Table configured in the PPP DEV Object. In case of forwarding the whole frame the result of this Filter is a LookupKey from LinkID/ClassID/FilterID */
    , e_QE_FLTR_FILTER_TYPE_PPP             = 0x04          /**< Used on the PPP interface. Parse the IP/UDP/L2TP/PPP encapsulations and generate lookup key from IP/UDP/L2TP frame header fields */
    , e_QE_FLTR_FILTER_TYPE_PPP_IPHC        = 0x05          /**< Used on the PPP interface. Performs decompressiond of IPHC Compressed frames */
    , e_QE_FLTR_FILTER_TYPE_AAL5_FAST       = 0x07          /**< Used on the ATM interface. Generate a lookup key from AAL5 VPI/VCI code */
    , e_QE_FLTR_FILTER_TYPE_AAL2_FAST       = 0x08          /**< Used on the ATM interface. Generate a lookup key from AAL2 CodeKey (VPI/VCI/CID) */
    , e_QE_FLTR_FILTER_TYPE_AAL5_LLC_FAST   = 0x09          /**< Used on the ATM interface. Generate lookup key from ATM AAL5 RFC2684 bridged LLC encapsulated frames */
    , e_QE_FLTR_FILTER_TYPE_AAL5_VCM_FAST   = 0x10          /**< Used on the ATM interface. Generate lookup key from ATM AAL5 RFC2684 bridged VC multiplexed frames */
    , e_QE_FLTR_FILTER_TYPE_AAL5_LLC        = 0x11          /**< Used on the ATM interface. Generate a lookup key from ATM-RFC2684/2364 LLC encapsulated frames */
    , e_QE_FLTR_FILTER_TYPE_AAL5_VCM        = 0x12          /**< Used on the ATM interface. Generate a lookup key from ATM-RFC2684/2364 VC multiplexed frames */
    , e_QE_FLTR_FILTER_TYPE_AAL2            = 0x13          /**< Used on the ATM interface. Generate a lookup key by concatenating AAL2CodeKey taken from the AAL2 RQD and a programmable number of bytes from a programmable offset from beginning of AAL2 frame */
    , e_QE_FLTR_FILTER_TYPE_GENERAL         = 0x14          /**< Used on any interface. Generates a lookup Key by concatenating up to as many 6 fields extracted from the frame */
    , e_QE_FLTR_FILTER_TYPE_PASSTHROUGH     = 0x15          /**< Used on any interface. Used to forward the frames to outgoing port without performing any filtering */

} e_QeFltrFilterType;

/**
 @Description   Expected Protocols Flags. Valid for any FLTR_FILTER/interface that supports detection of the selected protocols. A frame reaching a FLTR_FILTER and not having the selected protocols set on that FLTR_FILTER will go to the PARSE_MISS exit of that FLTR_FILTER
*/
typedef enum e_QeFltrExpectProtocol{

      e_QE_FLTR_FLG_EXPECT_NONE                 = 0x00000000    /**< No specific protocol expected */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI1              = 0x80000000    /**< Expect 1 VLAN tag in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI2              = 0x40000000    /**< Expect 2 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI3              = 0x20000000    /**< Expect 3 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI4              = 0x10000000    /**< Expect 4 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI5              = 0x08000000    /**< Expect 5 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_TCI6              = 0x04000000    /**< Expect 6 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI1                 = 0x00800000    /**< Expect 1 VLAN tag in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI2                 = 0x00400000    /**< Expect 2 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI3                 = 0x00200000    /**< Expect 3 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI4                 = 0x00100000    /**< Expect 4 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI5                 = 0x00080000    /**< Expect 5 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_TCI6                 = 0x00040000    /**< Expect 6 stacked VLAN tags in Ethernet frame */
    , e_QE_FLTR_FLG_EXPECT_NO_IP_OPTIONS        = 0x00020000    /**< Expect no IP fragmentation in the IP frame. If it is a IP fragment parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_NO_IP_FRAGMENTATION  = 0x00010000    /**< Expect no IP fragmentation in the IP frame. If it is a IP fragment parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_PPPoE                = 0x00008000    /**< Expect PPP over Ethernet Session encapsulation */
    , e_QE_FLTR_FLG_EXPECT_IP                   = 0x00004000    /**< Expect IP to be present in the frame <hr> */
    , e_QE_FLTR_FLG_EXPECT_TCP                  = 0x00002000    /**< Expect TCP  to be present in the frame */
    , e_QE_FLTR_FLG_EXPECT_UDP                  = 0x00001000    /**< Expect UDP  to be present in the frame */
    , e_QE_FLTR_FLG_EXPECT_SCTP                 = 0x00000800    /**< Expect SCTP to be present in the frame */
    , e_QE_FLTR_FLG_EXPECT_ESP_HDR              = 0x00000001    /**< Expect ESP header to be present in the frame */
    , e_QE_FLTR_FLG_EXPECT_IKE_HDR              = 0x02000000    /**< Expect IKE header to be present in the frame */
    , e_QE_FLTR_FLG_EXPECT_NO_ICMP              = 0x00000020    /**< Expect no Options in the IP frame. If found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_NO_IGMP              = 0x00000010    /**< Expect no Options in the IP frame. If found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_NO_ARP               = 0x00000008    /**< Expect no Options in the IP frame. If found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_NO_DHCP              = 0x00000004    /**< Expect no Options in the IP frame. If found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_NO_TCP_CTRL          = 0x00000002    /**< Expect no Options in the IP frame. If found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_L2TP                 = 0x00000400    /**< Expect L2TP header in the frame. If not found, parse miss occurs on this filter */
    , e_QE_FLTR_FLG_EXPECT_PPP_MUX              = 0x00000200    /**< Expect PPP MUX after L2TP header in the frame. If not found, parse miss occurs on this filter */

} e_QeFltrExpectProtocol;

/**
 @Description   Extracted Header Fields Flags. Valid for any FLTR_FILTER/interface that supports detection of the e_QeFltrExpectProtocol protocols. Generally, selected header fields can be extracted (used) with FLTR_FILTERs that perform automatic header parsing. However there are FLTR_FILTERs that assume a known protocol stack is present (xxx_FAST FLTR_FILTER types ).
*/
typedef enum e_QeFltrExtractProtocol{

      e_QE_FLTR_FLG_EXTRACT_NONE                = 0x00000000    /**< No Lookup key generation, specify an Ethernet Parse Only Filter */
    , e_QE_FLTR_FLG_EXTRACT_MAC_DST             = 0x80000000    /**< Extract MAC Source Address      field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_MAC_SRC             = 0x40000000    /**< Extract MAC Destination Address field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_TCI1                = 0x20000000    /**< Extract VLAN #1 TCI             field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_TCI2                = 0x10000000    /**< Extract VLAN #2 TCI             field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_ETYPE               = 0x08000000    /**< Extract EtherType               field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_PPP_SID             = 0x04000000    /**< Extract PPP Session ID          field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_PPP_PID             = 0x02000000    /**< Extract PPP Protocol ID         field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_IP_SRC              = 0x01000000    /**< Extract IP Source Address       field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_IP_DST              = 0x00800000    /**< Extract IP Destination Address  field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_PTYPE               = 0x00400000    /**< Extract IP Protocol Type        field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_IP_TOS              = 0x00200000    /**< Extract IP Type of Service      field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_TUSP_SRC            = 0x00100000    /**< Extract TCP/UDP/SCTP Src. Port  field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_TUSP_DST            = 0x00080000    /**< Extract TCP/UDP/SCTP Dest. Port field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_TCP_FLAGS           = 0x00040000    /**< Extract TCP Flags               field and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_L2TP_CONTROL        = 0x00020000    /**< Extract first two Bytes from the L2TP (v2 or v3) over UDP Control bits and include it in the Lookup Key. In case of L2TPv3 over IP, the value 0x0003 will be extracted */
    , e_QE_FLTR_FLG_EXTRACT_L2TP_ID             = 0x00010000    /**< Extract the L2TP Tunnel ID/Session ID field (4 Bytes) field and include it in the Lookup Key. If L2TPv3 over IP header is found, the extract field is 4 Bytes Session ID. If L2TPv3 over UDP header is found, the extract field is 4 Bytes session ID or control connection ID depending on type of frame. If L2TP (L2TPv2) header was found , the extract field is 2 bytes session ID and 2 bytes tunnel ID */
    , e_QE_FLTR_FLG_EXTRACT_PPP_PID_TUN         = 0x00008000    /**< Extract 2 Bytes from PPP Protocol ID Field of the PPP frame found after the L2TP Tunelling header and include it in the Lookup Key. Note that the least significant bit of the most significant byte (which is 0 in the dimming packets which comply to RFC1661: The Point-To-Point Protocol) will be set in the extracted lookup key */
    , e_QE_FLTR_FLG_EXTRACT_IPSEC_SPI           = 0x00008000    /**< Extract 4 byte SPI field from ESP header and include it in the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_CID                 = 0x00004000    /**< Valid for PPP_IPHC filter. Used for CID extraction. Available only when ACTION_DECOMPRESS is not requested. */
    , e_QE_FLTR_FLG_EXTRACT_SRC_PORT            = 0x00002000    /**< It adds different information for various FILTERs/protocols: For Ethernet (ETH, ETH_FAST): adds the SNUM of the Rx distributor (1byte). For PPP (PPP, PPP_FAST): adds the LINK ID or BUNDLE ID + CLASS ID (2bytes). For AAL5 (AAL5, AAL5_FAST): adds Rx Channel Code (2bytes). For AAL2 (AAL2, AAL2_FAST): adds the RQD Code Key (2bytes). */
    , e_QE_FLTR_FLG_EXTRACT_FLTR_ID             = 0x00001000    /**< Add the user-specified one-byte Filter ID to the Lookup Key */
    , e_QE_FLTR_FLG_EXTRACT_PAYLOAD             = 0x00000800    /**< Extract a part of the payload carried by AAL2 cells */

} e_QeFltrExtractProtocol;

/**
 @Description   Filter Command Flags for Expected Multiprotocol Encapsulations over AAL5
*/
typedef enum e_QeFltrExpectRfc{

      e_QE_FLTR_FLG_EXPECT_RFC_NONE             = 0x00          /**< No specific RFC encapsulation expected */
    , e_QE_FLTR_FLG_EXPECT_RFC2684_B            = 0x80          /**< Expect RFC2684 bridged encapsulation in the frame */
    , e_QE_FLTR_FLG_EXPECT_RFC2684_R            = 0x40          /**< Expect RFC2684 routed  encapsulation in the frame */
    , e_QE_FLTR_FLG_EXPECT_RFC2364              = 0x20          /**< Expect RFC2364 encapsulation in the frame */
    , e_QE_FLTR_FLG_EXPECT_RFC2364_NCPID        = 0x10          /**< Expect RFC2684 with non-compressed PID */
    , e_QE_FLTR_FLG_EXPECT_RFC2364_CPID         = 0x08          /**< Expect RFC2684 with compressed PID */

} e_QeFltrExpectRfc;

/**
 @Description    FLTR_FILTER Filter Action Flags. These flags extend the capability of the FLTR_FILTER object, providing additional functionalities, besides PARSING, KEY GENERATION and LOOKUP.
*/
typedef enum e_QeFltrFilterAction{

      e_QE_FLTR_FLG_ACTION_NONE                             = 0x0000        /**< No specific action required */
    , e_QE_FLTR_GENERAL_FLG_ACTION_CONCATENATE_LOOKUP_KEY   = 0x8000        /**< Valid for e_QE_FLTR_FILTER_TYPE_GENERAL FLTR_FILTER type. Concatenate the lookup key generated by this filter to the previous generated lookup key. Valid only when this is not the first filter in the chain */
    , e_QE_FLTR_PPP_FLG_ACTION_DEMUX                        = 0x4000        /**< Valid for e_QE_FLTR_FILTER_TYPE_PPP_FAST type. Enable this filter to perform DEMULTIPLEXING of PPP Superframes. Used only for e_QE_FLTR_FILTER_TYPE_PPP_FAST filter type. Can't be used in the same time while using e_QE_FLTR_FLG_EXTRACT_SRC_PORT and e_QE_FLTR_FLG_EXTRACT_FLTR_ID flags */
    , e_QE_FLTR_PPP_FLG_ACTION_DECOMPRESS                   = 0x2000        /**< Valid for e_QE_FLTR_FILTER_TYPE_PPP_IPHC type. Enable this filter to perform DECOMPRESSION. If used, the FLTR_FILTER will not generate a key using IPHC CID */
    , e_QE_FLTR_ETH_FLG_ACTION_CLASSIFY_MAC_DST_ADDRESSES   = 0x1000        /**< Valid for e_QE_FLTR_FILTER_TYPE_ETH_FAST and e_QE_FLTR_FILTER_TYPE_ETH type. Enable this Filter to perform MAC_DST Classification - unicast / multicast / broadcast */

#if (!defined(QE_IW_IPREASS_CUSTOM_UNAVAILABLE) || !defined(QE_IW_IPREASS_FULL_UNAVAILABLE))

    , e_QE_FLTR_ETH_FLG_ACTION_REASSEMBLE_IP_FRAGMENTS      = 0x0800        /**< Valid for e_QE_FLTR_FILTER_TYPE_ETH type. Enable this Filter to perform Reassembly of IP fragments */

#endif  /* not QE_IW_IPREASS_CUSTOM_UNAVAILABLE || not QE_IW_IPREASS_FULL_UNAVAILABLE */

    , e_QE_FLTR_ACTION_MASK_KEY                             = 0x0400        /**< Valid for any FLTR_FILTER that performs LOOKUP operations. It will the value specified in the 'keyMask' array, to mask the key befor performing the LOOKUP in the provided FLTR_TABLE */
    , e_QE_FLTR_ACTION_STORE_KEY                            = 0x0200        /**< Valid for any FLTR_FILTER that generates a key. */
    , e_QE_FLTR_ACTION_RESTORE_KEY                          = 0x0100        /**< Valid for any FLTR_FILTER that is able to generate LOOKUP operations. It will the value specified in the 'keyMask' array, to mask the key befor performing the LOOKUP in the provided FLTR_TABLE */
    , e_QE_FLTR_ACTION_CONTINUE_PARSING                     = 0x0080        /**< Valid for e_QE_FLTR_FILTER_TYPE_PPP type. Enables this filter to continue parsing from where the previous linked filter finished parsing (e.g. it allows parsing IPoL2TP ) */

}e_QeFltrFilterAction;

/**
 @Description   The base from whitch the offset is calculated for extraction of the key.
                If the mode chosen is not consistent with the headers found in the frame (for e.g. if
                mode e_QE_FLTR_KEY_ORIGIN_IP_START is chosen, and no IP header is found in the frame), a field with value zero is
                inserted in the LookupKey.
*/
typedef enum e_QeFltrKeyBaseMode{
 
      e_QE_FLTR_KEY_BASE_MODE_LAST_ENTRY    = 0x00  /**< The last entry for the key generation */
    , e_QE_FLTR_KEY_BASE_MODE_FRAME_START   = 0x01  /**< The first byte of the frame */
    , e_QE_FLTR_KEY_ORIGIN_ETH_MAC_START    = 0x02  /**< The first byte of the Ethernet MAC header, used only for ATM RFC2684B. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC_FAST,  e_QE_FLTR_FILTER_TYPE_AAL5_VCM_FAST, e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM */
    , e_QE_FLTR_KEY_ORIGIN_ETH_MAC_END      = 0x03  /**< The first byte following the Ethernet MAC header. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC_FAST, e_QE_FLTR_FILTER_TYPE_AAL5_VCM_FAST, e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM, e_QE_FLTR_FILTER_TYPE_ETH_FAST, e_QE_FLTR_FILTER_TYPE_ETH */
    , e_QE_FLTR_KEY_ORIGIN_PPPOE_START      = 0x04  /**< The first byte of the PPPoE header; If PPPoE header doesn't exist in the frame the base is IP header. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM, e_QE_FLTR_FILTER_TYPE_ETH */
    , e_QE_FLTR_KEY_ORIGIN_IP_START         = 0x05  /**< The first byte of the IP header. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM, e_QE_FLTR_FILTER_TYPE_ETH, e_QE_FLTR_FILTER_TYPE_PPP */
    , e_QE_FLTR_KEY_ORIGIN_IP_END           = 0x06  /**< The first byte after the IP header. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM, e_QE_FLTR_FILTER_TYPE_ETH, e_QE_FLTR_FILTER_TYPE_PPP */
    , e_QE_FLTR_KEY_ORIGIN_RFC2XX4_END      = 0x07  /**< The first byte following the  RFC2684 or RFC2364 headers. May be used only if this filter is chained after the following filter types: e_QE_FLTR_FILTER_TYPE_AAL5_LLC_FAST, e_QE_FLTR_FILTER_TYPE_AAL5_VCM_FAST, e_QE_FLTR_FILTER_TYPE_AAL5_LLC, e_QE_FLTR_FILTER_TYPE_AAL5_VCM */

} e_QeFltrKeyBaseMode;

/**
 @Description   User Parameters for the Ethernet Filter Types
                (opcodes:    e_QE_FLTR_OPCODE_ETH, e_QE_FLTR_OPCODE_ETH_FAST)
*/
typedef struct t_QeFltrFilterEth{

    uint32_t            expectFlags;        /**< Combination of flags for the expected protocol stack in the incoming frame (use the values defined in e_QeFltrExpectProtocol) */
    uint32_t            extractFlags;       /**< Combination of flags for selecting the fields to include in the generated lookup key (use the values defined in e_QeFltrEthExtractField ) */
    t_Handle            h_IpReass;          /**< Handle to the IP REASS object. Mandatory when using IPREASS FULL. Not needed for IPREASS CUSTOM */

} t_QeFltrFilterEth;


/**
 @Description   User Parameters for the AAL5 Filter Types
*/
typedef struct t_QeFltrFilterAal5{

    e_QeFltrExpectRfc   rfcFlags;           /**< Flag for selection of multiprotocol encapsulations over AAL5 (RFCs 2684b/r, RFC 2364 - values defined in e_QeFltrExpectRfc) */
    uint32_t            expectFlags;        /**< Combination of flags for the expected protocol stack in the incoming frame (use the values defined in e_QeFltrExpectProtocol) */
    uint32_t            extractFlags;       /**< Combination of flags for selecting the fields to include in the generated lookup key (use the values defined in e_QeFltrAal5ExtractField) */
} t_QeFltrFilterAal5;

/**
 @Description   User Parameters for the AAL2 Filter Types
*/
typedef struct t_QeFltrFilterAal2{

    uint16_t            extractFlags;       /**< Combination of flags for selecting the fields to include in the generated lookup key (use the values defined in e_QeFltrAal2ExtractField) */
    uint8_t             keyOffset;          /**< Offset in the frame from where to begin extracting the key */
    uint8_t             keySize;            /**< The size of the extracted key */
} t_QeFltrFilterAal2;

/**
 @Description   User Parameters for Filters that parse PPP frames
*/
typedef struct t_QeFltrFilterPpp{

    uint8_t             parseOffset;        /**< Offset in the frame from where to begin parsing */
    uint32_t            expectFlags;        /**< Combination of flags for the expected protocol stack in the incoming frame (use the values defined in e_QeFltrExpectProtocol ) */
    uint32_t            extractFlags;       /**< Combination of flags for selecting the fields to include in the generated lookup key (use the values defined in e_QeFltrPppExtractField) */
    t_Handle            h_Decompressor;     /**< Handle to the IPHC_Decompressor object. Mandatory for  e_QE_FLTR_FILTER_TYPE_PPP_IPHC filter type */
} t_QeFltrFilterPpp;

/**
 @Description   User Parameters for the General Filter Types
*/
typedef struct t_QeFltrFilterGeneralField{

    e_QeFltrKeyBaseMode    baseMode;        /**< Flag specifying one of several possible bases in the frame to use as reference when extracting one filed of the key */
    uint8_t                baseOffset;      /**< Offset in the frame relative to the the specified base from where to begin extracting the key */
    uint8_t                keySize;         /**< The size of the extracted field of the key */

} t_QeFltrFilterGeneralField;

/**
 @Description   User Parameters for the General Filter Types
*/
typedef struct t_QeFltrFilterGeneral{

    t_QeFltrFilterGeneralField field[QE_FLTR_KEY_GENERAL_MAX_NUM_FIELDS]; /**< The field's information that will be extracted for generating the key */
 
} t_QeFltrFilterGeneral;

/**
 @Description   User Parameters for the Filter Object
*/
typedef struct t_QeFltrFilterParam
{
    t_Handle                h_AppId;                /**< Handle to the upper layer */
    t_Handle                h_QeId;                 /**< Handle to the QUICC Engine CB */
    uint8_t                 pramMemId;              /**< Parameter RAM partition ID */
    e_QeFltrFilterType      type;                   /**< Type of the filter */

    union _filter{
        t_QeFltrFilterEth           eth;            /**< Parameters specific to Ethernet filters */
        t_QeFltrFilterAal5          aal5;           /**< Parameters specific to AAL5 filters */
        t_QeFltrFilterAal2          aal2;           /**< Parameters specific to AAL2 filters */
        t_QeFltrFilterPpp           ppp;            /**< Parameters specific to filters that parse PPP frames */
        t_QeFltrFilterGeneral       gen;            /**< Parameters specific to General filters */

    } filter;

    t_Handle                        h_TableId;      /**< Handle to CAM emulation, Indexed, Four/Eight Way Hash or LPM table in which to perform lookup. Must use handle returned by QE_FLTR_TABLE_Config(), after QE_FLTR_TABLE_Init(). */

    uint32_t                        compressBits;   /**< The Compress Bits if key compression is used (in cases when it's irrelevant, it may contain an invalid value) */

    uint8_t                         keyMask[QE_FLTR_MAX_LOOKUP_KEY_SIZE];   /**< Contains the mask value to be applied to the lookup key if the useMask flag is set. */
 
    uint8_t                         userId;         /**< Id to be added to lookup key if addId flag is set. It is also used to intentify the filter in the Interrupt Queue Entry for Lookup and Last events. */
    uint16_t                        actionFlags;    /**< Filter action flags*/

} t_QeFltrFilterParam;

/**
 @Function      QE_FLTR_FILTER_Config

 @Description   Configures a filter to be part of a filtering graph.
                The configured filter is the root filter in the graph.

 @Param         p_Params    - (In)  User parameters for filter.

 @Return        t_Handle    Handle to the created filter CMD chain if successful.
 @Return        NULL        In case of failure to create object or invalid parameters.
*/
t_Handle QE_FLTR_FILTER_Config (t_QeFltrFilterParam    *p_Params);

/**
 @Function      QE_FLTR_FILTER_Init
 
 @Description   Initializes a filtering graph consisting of multi-chains composed
                from many filters connected through ParseMiss and LookupMiss pointers in PRAM memory.

 @Param         h_Filter    - (In) Handle to the filter who is the root of the filter graph.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      h_Filter must have been previously created using QE_FLTR_FILTER_Config().
                h_Filter must be the root filter of the graph.
*/
t_Error    QE_FLTR_FILTER_Init (t_Handle h_Filter);

/**
 @Function      QE_FLTR_FILTER_Free

 @Description   Frees memory and resources associated to a Filter object.

 @Param         h_Filter    - (In)    Handle to filter to be freed, obtained via QE_FLTR_Config function.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      h_Filter must have been previously created using QE_FLTR_FILTER_Config().
 @Cautions      Any resources used by the filter and initialized separately
                (e.g. tables, other filters) are NOT freed by this function recursively.
*/
t_Error    QE_FLTR_FILTER_Free (t_Handle h_Filter);

/**************************************************************************//**
 @Group         fltr_filter_init_advanced_grp Filter Object Advanced Initialization Unit

 @Description   Filter Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Filter Debug Mode Flags
*/
typedef enum e_QeFltrDebugModeFlag{

      e_QE_FLTR_DEBUG_MODE_ALL_DISCARD                          = 0x00000000    /**< No debug mode is selected */
    , e_QE_FLTR_DEBUG_MODE_ALL_FORWARD_TO_CPU                   = 0x00000001    /**< Frame Discard if Parse Miss */
 
    , e_QE_FLTR_DEBUG_MODE_PARSE_MISS_FRAME_DISCARD             = 0x00000002    /**< Frame Discard if Parse Miss */
    , e_QE_FLTR_DEBUG_MODE_PARSE_MISS_FRAME_FORWARD_TO_CPU      = 0x00000004    /**< Frame Forward to CPU if Parse Miss */

    , e_QE_FLTR_DEBUG_MODE_LOOKUP_MISS_FRAME_DISCARD            = 0x00000008    /**< Frame Discard if Lookup Miss */
    , e_QE_FLTR_DEBUG_MODE_LOOKUP_MISS_FRAME_FORWARD_TO_CPU     = 0x00000010    /**< Frame Forward to CPU if Lookup Miss */

    , e_QE_FLTR_DEBUG_MODE_PARSE_HIT_FRAME_DISCARD              = 0x00000020    /**< Frame Discard if Lookup Miss */
    , e_QE_FLTR_DEBUG_MODE_PARSE_HIT_FRAME_FORWARD_TO_CPU       = 0x00000040    /**< Frame Forward to CPU if Lookup Miss */

    , e_QE_FLTR_DEBUG_MODE_HDEC_MISS1_FRAME_DISCARD             = 0x00000080    /**< Frame Discard if Lookup Miss */
    , e_QE_FLTR_DEBUG_MODE_HDEC_MISS1_FRAME_FORWARD_TO_CPU      = 0x00000100    /**< Frame Forward to CPU if Lookup Miss */

    , e_QE_FLTR_DEBUG_MODE_HDEC_MISS2_FRAME_DISCARD             = 0x00000200    /**< Frame Discard if Lookup Miss */
    , e_QE_FLTR_DEBUG_MODE_HDEC_MISS2_FRAME_FORWARD_TO_CPU      = 0x00000400    /**< Frame Forward to CPU if Lookup Miss */

    , e_QE_FLTR_DEBUG_MODE_MULTICAST_FRAME_DISCARD              = 0x00000800    /**< Frame Discard if Last is configured on ETH Classifier Multicast relation */
    , e_QE_FLTR_DEBUG_MODE_MULTICAST_FRAME_FORWARD_TO_CPU       = 0x00001000    /**< Frame Forward to CPU if Last is configured on ETH Classifier Multicast relation */

    , e_QE_FLTR_DEBUG_MODE_BROADCAST_FRAME_DISCARD              = 0x00002000    /**< Frame Discard if Last is configured on ETH Classifier Broadcast relation  */
    , e_QE_FLTR_DEBUG_MODE_BROADCAST_FRAME_FORWARD_TO_CPU       = 0x00004000    /**< Frame Forward to CPU if Last is configured on ETH Classifier Broadcast relation */

    , e_QE_FLTR_DEBUG_MODE_UNICAST_FRAME_DISCARD                = 0x00008000    /**< Frame Discard if Last is configured on ETH Classifier Broadcast relation  */
    , e_QE_FLTR_DEBUG_MODE_UNICAST_FRAME_FORWARD_TO_CPU         = 0x00010000    /**< Frame Forward to CPU if Last is configured on ETH Classifier Broadcast relation */

} e_QeFltrDebugModeFlag;

/**
 @Description    Filter Interrupt Flags
*/
typedef enum e_QeFltrInterruptFlag{

      e_QE_FLTR_INTR_NONE                   = 0x0000    /**< No interrupts are enabled on the FLTR_FILTER. */
    , e_QE_FLTR_INTR_ALL                    = 0x0001    /**< All the available interrupts for the respective FLTR_FILTER are enabled. Distinction between different e_QE_FLTR_INTR_LAST_~exit_point~ interrupts cannot be made, as the same event type is generated. */
    , e_QE_FLTR_INTR_LAST_PARSE_MISS        = 0x0002    /**< Enable interrupt for frames reaching the ParseMiss exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_PARSE_HIT         = 0x0004    /**< Enable interrupt for frames reaching the ParseHit exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_LOOKUP_MISS       = 0x0008    /**< Enable interrupt for frames reaching the LookupMiss exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_HDEC_MISS1        = 0x0010    /**< Enable interrupt for frames reaching the HdecMiss1 exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_HDEC_MISS2        = 0x0020    /**< Enable interrupt for frames reaching the HdecMiss2 exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_MULTICAST         = 0x0040    /**< Enable interrupt for frames reaching the Multicast exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_BROADCAST         = 0x0080    /**< Enable interrupt for frames reaching the Broadcast exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_LAST_UNICAST           = 0x0100    /**< Enable interrupt for frames reaching the Unicast exit of the filter, when no FILTER is connected to this exit */
    , e_QE_FLTR_INTR_UNRECOGNIZED_FRAME     = 0x0200    /**< Enable interrupt for frames not recognized by the parser (different from ParseMiss) */
    , e_QE_FLTR_INTR_LOOKUP_MISS            = 0x0400    /**< Enable interrupt for lookup keys not found in the TABLE associated to the filter */
 
} e_QeFltrInterruptFlag;


/**
 @Description    Filter Counter Flags
*/
typedef enum e_QeFltrCounterFlag{

      e_QE_FLTR_COUNT_NONE                  = 0x0000    /**< No counters are enabled. */
    , e_QE_FLTR_COUNT_ALL                   = 0x0001    /**< All the available counters for the respective FLTR_FILTER are enabled. Distinction between different e_QE_FLTR_COUNT_LAST_~exit_point~ interrupts cannot be made, as the same event type is generated and counted. */
    , e_QE_FLTR_COUNT_LAST_PARSE_MISS       = 0x0002    /**< Enable counting the Parse Miss Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_PARSE_HIT        = 0x0004    /**< Enable counting the Parse Hit Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_LOOKUP_MISS      = 0x0008    /**< Enable counting the Lookup Miss Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_HDEC_MISS1       = 0x0010    /**< Enable counting the HDec Miss1 Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_HDEC_MISS2       = 0x0020    /**< Enable counting the HDec Miss2 Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_MULTICAST        = 0x0040    /**< Enable counting the ETH Classifier Multicast Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_BROADCAST        = 0x0080    /**< Enable counting the ETH Classifier Broadcast Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */
    , e_QE_FLTR_COUNT_LAST_UNICAST          = 0x0100    /**< Enable counting the ETH Classifier Unicast Last Actions. The General Miss special statistics counter will be incremented for the reveiving Device */

} e_QeFltrCounterFlag;


/**
 @Description    ETH Classifier Parsing Modes
*/
typedef enum e_QeFltrEthClassifierParsingMode{

      e_QE_FLTR_ETH_CLASSIFIER_PARSING_MODE_FIRMWARE    = 0x01    /**< Configure the Filter to perform the clasification of the MAC_DST address based on the Firmware indications */
    , e_QE_FLTR_ETH_CLASSIFIER_PARSING_MODE_HARDWARE    = 0x02    /**< Configure the Filter to perform the clasification of the MAC_DST address based on the Hardware indications received from the receiver virtual FIFO. */

} e_QeFltrEthClassifierParsingMode;

/**
 @Function      QE_FLTR_FILTER_ConfigParseHit

 @Description   Creates a new filter that will be chained on the Parse Hit relation of an existing Parse Only filter.

 @Param         h_ParentFilter      - (In)    Handle to the Parse only filter whose Parse Hit relation will be the new filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the ParseHit relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a Parse Only filter ( a FLTR_FILTER with no extractFlags set, that is able to perform PARSING ).
*/
t_Error QE_FLTR_FILTER_ConfigParseHit (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigBroadcast

 @Description   Links a child filter on the Broadcast relation of an existing filter, that contains the Classify Action.

 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose Broadcast relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the Broadcast relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a FLTR_FILTER that has e_QE_FLTR_ETH_FLG_ACTION_CLASSIFY_MAC_DST_ADDRESSES configured.
*/
t_Error QE_FLTR_FILTER_ConfigBroadcast (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigMulticast

 @Description   Links a child filter on the Multicast relation of an existing filter, that contains the Classify Action.

 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose Multicast relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the Multicast relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a FLTR_FILTER that has e_QE_FLTR_ETH_FLG_ACTION_CLASSIFY_MAC_DST_ADDRESSES configured.
*/
t_Error QE_FLTR_FILTER_ConfigMulticast (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigUnicast

 @Description   Links a child filter on the Unicast relation of an existing Classify-Only filter ( no parse or lookup flags were provided at Config() ).

 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose Unicast relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the Unicast relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a FLTR_FILTER that has only e_QE_FLTR_ETH_FLG_ACTION_CLASSIFY_MAC_DST_ADDRESSES configured ( no expectFlags nor extractFlags selected ).
*/
t_Error QE_FLTR_FILTER_ConfigUnicast (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigParseMiss

 @Description   Creates a new filter that will be chained on the Parse Miss relation of an existing filter.

 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose ParseMiss relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the ParseMiss relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a FLTR_FILTER that has expectFlags set.
*/
t_Error QE_FLTR_FILTER_ConfigParseMiss (t_Handle    h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigLookupMiss

 @Description   Creates a new filter that will be chained on the Lookup Miss relation of an existing filter.

 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose LookupMiss relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the LookupMiss relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      The root filter must be a FLTR_FILTER that perform LOOKUP (either it's a FILTR_FILTER that generates a key - through extractFlags - or it RESTORES a previously generated key).
*/
t_Error    QE_FLTR_FILTER_ConfigLookupMiss (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigHdecMiss1
 
 @Description   Configures the Filter Graph path for IPHDC Event: 'CID out of range'
                when Exceed Range Support mode is enabled.
 
 @Param         h_ParentFilter      - (In)    Handle to the parent filter whose HdecMiss1 relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter       - (In)    Handle to the filter that is linked on the HdecMiss1 relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Valid only for e_QE_FLTR_FILTER_TYPE_PPP_IPHC filter type
                Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error    QE_FLTR_FILTER_ConfigHdecMiss1 (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ConfigHdecMiss2
 
 @Description   Configures the Filter Graph path for various IPHDC Events:
                    - invalid context and ADL mode disabled
                    - invalid context, ADL mode, non FH frame and NS drop mode enabled
                    - valid context, no generation match, FH frame, IP/UDP frame not RTP
                    ( see IPHC parser specs for more details )
 
 @Param         h_Filter          - (In)    Handle to the parent filter whose HdecMiss2 relation will be the h_ChildFilter filter.
 @Param         h_ChildFilter     - (In)    Handle to the filter that is linked on the HdecMiss2 relation to the h_ParentFilter
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Valid only for e_QE_FLTR_FILTER_TYPE_PPP_IPHC filter type
                Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error    QE_FLTR_FILTER_ConfigHdecMiss2 (t_Handle    h_Filter, t_Handle h_ChildFilter);

 
/**
 @Function      QE_FLTR_FILTER_ConfigDebugMode
 
 @Description   Configures the debug mode for parse miss and lookup miss for the filter.
                [e_QE_FLTR_DEBUG_MODE_DISCARD_ALL]
 
 @Param         h_Filter        - (In)  Handle to the filter.
 @Param         newDebugMode    - (In)  The new debug mode to configure for the filter.
                                        specified as a combination of the values defined in e_QeFltrDebugModeFlag.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error    QE_FLTR_FILTER_ConfigDebugMode (t_Handle    h_Filter, uint32_t newDebugMode);

/**
 @Function      QE_FLTR_FILTER_ConfigIpFirstFragmentCheck
 
 @Description   Configures the filter on how to interpret the First Fragment of the frame
                [FALSE]
 
 @Param         h_Filter        - (In)    Handle of the filter.
 @Param         enabled         - (In)    If TRUE, the first fragment is considered as a frame fragment
                                          If FALSE the first fragment is not considered as a frame fragment
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
                This flag is available only for the following Filter Types:
                e_QE_FLTR_FILTER_TYPE_ETH
                e_QE_FLTR_FILTER_TYPE_PPP
                e_QE_FLTR_FILTER_TYPE_AAL5_LLC
                e_QE_FLTR_FILTER_TYPE_AAL5_VCM
*/
t_Error    QE_FLTR_FILTER_ConfigIpFirstFragmentCheck (t_Handle    h_Filter, bool enabled);

/**
 @Function      QE_FLTR_FILTER_ConfigIpTTLCheck
 
 @Description   Configures the filter either to check or not the TTL Field in the IP frame.
                [FALSE]
 
 @Param         h_Filter    - (In)  Handle to the filter.
 @Param         enabled     - (In)  If TRUE, the TTL Field is checked as follows: if TTL is less than MinTTL (set in DEV object) or greater than MaxTTL (set in DEV object),
                                    the frame is considered as unrecognized frame
                                    If FALSE, the TTL Field in the frame is not checked.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
                This flag is available only for the following Filter Types:
                e_QE_FLTR_FILTER_TYPE_ETH
                e_QE_FLTR_FILTER_TYPE_PPP
                e_QE_FLTR_FILTER_TYPE_AAL5_LLC
                e_QE_FLTR_FILTER_TYPE_AAL5_VCM
*/
t_Error    QE_FLTR_FILTER_ConfigIpTTLCheck (t_Handle h_Filter, bool enabled);

/**
 @Function      QE_FLTR_FILTER_ConfigIpChecksumCheck
 
 @Description   Configures the filter either to check or not the IP checksum Field in the IP frame.
                [FALSE]
 
 @Param         h_Filter    - (In)  Handle to the filter.
 @Param         enabled     - (In)  If TRUE, the IP checksum in the frame is checked and if it does not correspond the IP checksum calculated on the IP header in the received frame,
                                    the frame is considered as unrecognized frame.
                                    If FALSE, the the IP checksum in the frame is not checked.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
                This flag is available only for the following Filter Types:
                e_QE_FLTR_FILTER_TYPE_PPP
                e_QE_FLTR_FILTER_TYPE_AAL5_LLC
                e_QE_FLTR_FILTER_TYPE_AAL5_VCM
*/
t_Error    QE_FLTR_FILTER_ConfigIpChecksumCheck (t_Handle h_Filter, bool enabled);

/**
 @Function      QE_FLTR_FILTER_ConfigAgingSupport
 
 @Description   Enable/disable this Filter object to provide aging support for associated lookup table entries.
                [FALSE]
 
 @Param         h_Filter    - (In)    Handle to the filter.
 @Param         enabled     - (In)    TRUE to enable the aging support
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ConfigAgingSupport(t_Handle h_Filter, bool enabled);

/**
 @Function      QE_FLTR_FILTER_ConfigInterrupts
 
 @Description   Enables interrupts on specific filtering events for the filter.
                [none]
 
 @Param         h_Filter        - (In)    Handle to the filter.
 @Param         interruptFlags  - (In)    Indication of which interrupts to enable for this filter,
                                         specified as a combination of the values defined in e_QeFltrInterruptFlag.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ConfigInterrupts (t_Handle h_Filter, uint16_t interruptFlags);

/**
 @Function      QE_FLTR_FILTER_ConfigCounters
 
 @Description   Enables counting Filter events.
                [none]
 
 @Param         h_Filter        - (In)    Handle to the filter.
 @Param         counterFlags    - (In)  Flags for enabling counters for this filter (combination of e_QeFltrCounterFlag).
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ConfigCounters (t_Handle h_Filter, uint8_t counterFlags);

/**
 @Function      QE_FLTR_FILTER_ConfigExtendLookupKey
 
 @Description   Configure the Filter to extend the lookup key with the field described.
 
 @Param         h_Filter        - (In)    Handle to the filter.
 @Param         p_Field         - (In)  Pointer to the field which describes the lookup key extension.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      Available only for the following filter types:
                e_QE_FLTR_FILTER_TYPE_ETH_FAST
                e_QE_FLTR_FILTER_TYPE_ETH
                e_QE_FLTR_FILTER_TYPE_PPP
                e_QE_FLTR_FILTER_TYPE_AAL5_LLC_FAST
                e_QE_FLTR_FILTER_TYPE_AAL5_VCM_FAST
                e_QE_FLTR_FILTER_TYPE_AAL5_LLC
                e_QE_FLTR_FILTER_TYPE_AAL5_VCM
                e_QE_FLTR_FILTER_TYPE_AAL2
                e_QE_FLTR_FILTER_TYPE_GENERAL
*/
t_Error    QE_FLTR_FILTER_ConfigExtendLookupKey (t_Handle h_Filter, t_QeFltrFilterGeneralField *p_Field);

/**
 @Function      QE_FLTR_FILTER_ConfigEthClassifierParsingMode
 
 @Description   Configure the ETH Filter parsing mode used to perform the clasification of the MAC_DST address.
                [e_QE_FLTR_ETH_CLASSIFIER_PARSING_MODE_FIRMWARE]
 
 @Param         h_Filter    - (In)    Handle to the filter.
 @Param         mode        - (In)  Indicate Parsing mode (use one of e_QeFltrEthClassifierParsingMode values).
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init().
 @Cautions      Relevant only for the following filter types:
                e_QE_FLTR_FILTER_TYPE_ETH_FAST
                e_QE_FLTR_FILTER_TYPE_ETH
*/
t_Error QE_FLTR_FILTER_ConfigEthClassifierParsingMode ( t_Handle h_Filter,
                                                      e_QeFltrEthClassifierParsingMode mode );

/** @} */ /* end of fltr_filter_init_advanced_grp group */
/** @} */ /* end of fltr_filter_init_grp group */


/**************************************************************************//**
 @Group         fltr_filter_runtime_grp Filter Object Runtime Unit

 @Description   Filter Object API Runtime Unit

 @{
*//***************************************************************************/

/**
 @Function      QE_FLTR_FILTER_ModifyPassThroughActions

 @Description   Modify at runtime the specific forwarding actions.

 @Param         h_Filter        - (In) Handle to the Filter for which the LookupMiss relation is checked.
 @Param         p_ActionDesc    - (In) Pointer to a user-supplied t_QeFltrActionDescriptor.

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ModifyPassThroughActions (t_Handle h_Filter, t_QeFltrActionDescriptor *p_ActionDesc);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE
/**
 @Function      QE_FLTR_FILTER_ModifyIpReassemblyAction

 @Description   Modify at runtime the IP Reassembly action.

 @Param         h_Filter        - (In) Handle to the Filter for which the reassembly action is modified.
 @Param         enabled         - (In) If TRUE, the reassembly action is enabled
                                       If FALSE, the reassembly action is disabled

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error    QE_FLTR_FILTER_ModifyIpReassemblyAction (t_Handle h_Filter, bool enabled);
#endif /* QE_IW_IPREASS_FULL_UNAVAILABLE */

/**
 @Function      QE_FLTR_FILTER_ModifyCounters

 @Description   Modify the selection of enabled/disabled counters

 @Param         h_Filter        - (In) Handle to the Filter for which the active counters are set
 @Param         counters        - (In) Selection of counters from e_QeFltrCounterFlag

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ModifyCounters (t_Handle h_Filter, uint8_t counters);

/**
 @Function      QE_FLTR_FILTER_ModifyCounters

 @Description   Modify the selection of enabled/disabled counters

 @Param         h_Filter        - (In) Handle to the Filter for which the active counters are set
 @Param         interrupts        - (In) Selection of interrupts from e_QeFltrInterruptFlag

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_ModifyInterrupts (t_Handle h_Filter, uint16_t interrupts);

/**
 @Function      QE_FLTR_FILTER_ModifyParseMiss

 @Description   Modifies a filter's exit point, at run-time

 @Param         h_ParentFilter      - (In)    Handle to the filter whose PARSE_MISS relation will be h_ChildFilter;
 @Param         h_ChildFilter       - (In)    Handle to the filter that will be connected to the PARSE_MISS relation of h_ParentFilter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init() for each of the two filters.
*/
t_Error QE_FLTR_FILTER_ModifyParseMiss (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ModifyBroadcast

 @Description   Modifies a filter's exit point, at run-time

 @Param         h_ParentFilter      - (In)    Handle to the filter whose BROADCAST relation will be h_ChildFilter;
 @Param         h_ChildFilter       - (In)    Handle to the filter that will be connected to the BROADCAST relation of h_ParentFilter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init() for each of the two filters.
*/
t_Error QE_FLTR_FILTER_ModifyBroadcast (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ModifyMulticast

 @Description   Modifies a filter's exit point, at run-time

 @Param         h_ParentFilter      - (In)    Handle to the filter whose MULTICAST relation will be h_ChildFilter;
 @Param         h_ChildFilter       - (In)    Handle to the filter that will be connected to the MULTICAST relation of h_ParentFilter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init() for each of the two filters.
*/
t_Error QE_FLTR_FILTER_ModifyMulticast (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ModifyHdecMiss1

 @Description   Modifies a filter's exit point, at run-time

 @Param         h_ParentFilter      - (In)    Handle to the filter whose HDEC_MISS1 relation will be h_ChildFilter;
 @Param         h_ChildFilter       - (In)    Handle to the filter that will be connected to the HDEC_MISS1 relation of h_ParentFilter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init() for each of the two filters.
*/
t_Error QE_FLTR_FILTER_ModifyHdecMiss1 (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/**
 @Function      QE_FLTR_FILTER_ModifyHdecMiss2

 @Description   Modifies a filter's exit point, at run-time

 @Param         h_ParentFilter      - (In)    Handle to the filter whose HDEC_MISS2 relation will be h_ChildFilter;
 @Param         h_ChildFilter       - (In)    Handle to the filter that will be connected to the HDEC_MISS2 relation of h_ParentFilter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Allowed only following QE_FLTR_FILTER_Config() and before QE_FLTR_FILTER_Init() for each of the two filters.
*/
t_Error QE_FLTR_FILTER_ModifyHdecMiss2 (t_Handle h_ParentFilter, t_Handle h_ChildFilter);

/** @} */ /* end of fltr_filter_runtime_grp group */


/**************************************************************************//**
 @Group         fltr_filter_control_grp Filter Object Control Unit

 @Description   Filter Object API Control Unit

 @{
*//***************************************************************************/

/**
 @Description   Largest External Table Lookup Key Sizes
*/
typedef enum e_QeFltrLargestExternalTableLookupKeySize{

      e_QE_FLTR_LARGEST_EXTERNAL_TABLE_LOOKUP_KEY_SIZE_NONE     = 0x00    /**< Not used */
    , e_QE_FLTR_LARGEST_EXTERNAL_TABLE_LOOKUP_KEY_SIZE_8_BYTES  = 0x3F    /**< The largest external table lookup key size is 8  bytes */
    , e_QE_FLTR_LARGEST_EXTERNAL_TABLE_LOOKUP_KEY_SIZE_16_BYTES = 0x5F    /**< The largest external table lookup key size is 16 bytes */
    , e_QE_FLTR_LARGEST_EXTERNAL_TABLE_LOOKUP_KEY_SIZE_24_BYTES = 0x7F    /**< The largest external table lookup key size is 24 bytes */

} e_QeFltrLargestExternalTableLookupKeySize;

/**
 @Function      QE_FLTR_FILTER_GetLargestExternalMemLookupKeySize

 @Description   Returns the largest size of the lookup keys in the chain.

 @Param         h_Filter            - (In)  - Handle to the Filter in which the largest lookup key will be searched.
 @Param         p_LookupKeySize        - (Out) - The largest size of the lookup keys in the chain.

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_GetLargestExternalMemLookupKeySize (t_Handle h_Filter, e_QeFltrLargestExternalTableLookupKeySize *p_LookupKeySize);

/**
 @Function      QE_FLTR_FILTER_GetPointer

 @Description   Retrieves an opaque pointer to the filter's internal data structures
                (needed by Termination Drivers).

 @Param         h_Filter    - (In)    Handle to the Filter object.
 @Param         p_RootAddr    - (Out)    Base address of the filter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_GetPointer (t_Handle h_Filter, uint32_t *p_RootAddr);

/**
 @Function      QE_FLTR_FILTER_GetType

 @Description   Retrieves the type of the filter
                (needed by Termination Drivers).

 @Param         h_Filter    - (In)    Handle to the Filter object.
 @Param         p_Type      - (Out)    Type of the filter - select one from the e_QeFltrFilterType enumeration.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_GetType (t_Handle h_Filter, e_QeFltrFilterType *p_Type);

/**
 @Function      QE_FLTR_FILTER_GetTable
 
 @Description   Retrieves the handle to the Lookup Table associated with this filter.
 
 @Param         h_Filter    - (In)    Handle to the filter whose table we want to retrieve.
 @Param         h_Table     - (Out)    Handle to the filter's table.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_GetTable (t_Handle h_Filter, t_Handle *h_Table);

/**
 @Function      QE_FLTR_FILTER_GetActionFlags
 
 @Description   Retrieves the selected action flags for this filter.
 
 @Param         h_Filter        - (In)    Handle to the filter whose action flags we want to retrieve.
 @Param         p_ActionFlags    - (Out)    Action flags
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
                Works only for filters that support action-flags
*/
t_Error QE_FLTR_FILTER_GetActionFlags(t_Handle h_Filter, uint16_t *p_ActionFlags);

/**
 @Function      QE_FLTR_FILTER_GetRoot
 
 @Description   Retrieves the 'ROOT' state of a FLTR_FILTER
 
 @Param         h_Filter        - (In)    Handle to the filter whose action flags we want to retrieve.
 @Param         p_IsRoot        - (Out)   Value that indicates if the filter is a root ( and should call FLTR_FILTER_Free() in order to free the memory)
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_GetRoot(t_Handle h_Filter, bool *p_IsRoot);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**
 @Function      QE_FLTR_FILTER_DumpPcdChain
 
 @Description   Dumps the PCD chains wrapped by this filter.
 
 @Param         h_Filter    - (In)    Handle to the root filter.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_FLTR_FILTER_Init().
*/
t_Error QE_FLTR_FILTER_DumpPcdChain(t_Handle h_Filter);
#endif /* (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0)) */

/** @} */ /* end of fltr_filter_control_grp group */
/** @} */ /* end of fltr_filter_grp group */

/** @} */ /* end of fltr_grp group */


#ifdef __cplusplus
}
#endif

#endif /* __QE_FILTERING_EXT_H */

