/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          qe_fltr_table_ext.h

 @Description   Declarations for public data types and functions
                 for the Extended Filtering Module
*/

#ifndef __QE_FLTR_TABLE_EXT_H
#define __QE_FLTR_TABLE_EXT_H


#ifdef __cplusplus
extern "C"{
#endif

        /* INCLUDE FILES */
#include "std_ext.h"
#include "xx_ext.h"




        /* MACROS */
#define QE_FLTR_MAX_LOOKUP_KEY_SIZE            24
#define QE_FLTR_AD_SIZE                     8

/*    Macros useful for key compression in Indexed Table */
#define QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BITS(num_bits)        ~(~(uint32_t)0 >> (num_bits))
#define QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BYTES(num_bytes)    QE_FLTR_TABLE_INDEXED_NO_COMPRESS_FIRST_BITS(num_bytes<<3)


        /* ENUMS */

/**************************************************************************//**
 @Group          fltr_grp Extended Filtering Application Programming Interface

 @Description    Extended Filtering API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          fltr_table_grp Table Object Application Programming Interface

 @Description    Table Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         fltr_table_init_grp Table Object Initialization Unit

 @Description   Table Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Lookup Key Sizes for CAM, INDEXED, HASH Table
*/
typedef enum {

      e_QE_FLTR_TABLE_LOOKUP_KEY_SIZE_2_BYTES  = 2  /**< Used ONLY for CAM table. The Table Lookup Key size is 2 bytes */
    , e_QE_FLTR_TABLE_LOOKUP_KEY_SIZE_8_BYTES  = 8    /**< The Table Lookup Key size is 8 bytes */
    , e_QE_FLTR_TABLE_LOOKUP_KEY_SIZE_16_BYTES = 16    /**< The Table Lookup Key size is 16 bytes */
    , e_QE_FLTR_TABLE_LOOKUP_KEY_SIZE_24_BYTES = 24    /**< The Table Lookup Key size is 24 bytes */

} e_QeFltrTableLookupKeySize;


/**
 @Description   Lookup Table Types
*/
typedef enum {

      e_QE_FLTR_TABLE_TYPE_CAM            = 1    /**< The lookup table is CAM Emulation Table */
    , e_QE_FLTR_TABLE_TYPE_INDEXED        = 2    /**< The lookup table is Indexed Table */
    , e_QE_FLTR_TABLE_TYPE_HASH            = 3    /**< The lookup table is Hash Table */
    , e_QE_FLTR_TABLE_TYPE_LPM            = 4    /**< The lookup table is Longest Prefix Match */

} e_QeFltrTableType;


/**
 @Description   Possible Numbers of Ways in the Hash Table
*/
typedef enum {

      e_QE_FLTR_TABLE_HASH_NUM_WAYS_4    = 4 /**< There are 4 ways per set in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_HASH_NUM_WAYS_8    = 8    /**< There are 8 ways per set in the Hash Lookup Table */

} e_QeFltrTableHashNumWays;


/**
 @Description   Possible Numbers of Sets in the Hash Table
*/
typedef enum {

      e_QE_FLTR_TABLE_NUM_SETS_2    = 0        /**< There are   2 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_4    = 1        /**< There are   4 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_8    = 2        /**< There are   8 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_16    = 3        /**< There are  16 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_32    = 4        /**< There are  32 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_64    = 5        /**< There are  64 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_128    = 6        /**< There are 128 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_256    = 7        /**< There are 256 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_512    = 8        /**< There are 512 sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_1K    = 9        /**< There are  1k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_2K    = 10    /**< There are  2k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_4K    = 11    /**< There are  4k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_8K    = 12    /**< There are  8k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_16K    = 13    /**< There are 16k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_32K    = 14    /**< There are 32k sets in the Hash Lookup Table */
    , e_QE_FLTR_TABLE_NUM_SETS_64K    = 15    /**< There are 64k sets in the Hash Lookup Table */

} e_QeFltrTableHashNumSets;


        /* STRUCTURES AND OTHER TYPEDEFS */
/**
@Description   CAM Emulation Lookup Table Initialization Structure
*/
typedef struct {

    e_QeFltrTableLookupKeySize  lookupKeySize;      /**< Number of bytes in lookup key */
    uint8_t                     lookupTableSize;    /**< Number of entries in CAM lookup table (maximum is 32) */
    uint8_t                     pramMemId;          /**< Parameter RAM memory partition ID */

} t_QeFltrTableCamParam;


/**
@Description   Indexed Lookup Table Initialization Structure
*/
typedef struct {

    uint32_t    lookupTableSize;    /**< Number of entries in the table  */
    uint8_t     pramMemId;          /**< Parameter RAM memory partition ID */
    uint8_t     tableMemId;         /**< Memory partition ID for the table allocation */
    uint32_t    tableMemAttributes; /**< Memory attributes for the table's memory */

} t_QeFltrTableIndexedParam;


/**
@Description   Hash Lookup Table Initialization Structure
*/
typedef struct {

    e_QeFltrTableLookupKeySize  lookupKeySize;      /**< Number of bytes in lookup key */
    e_QeFltrTableHashNumWays    numWays;            /**< Number of ways in table */
    e_QeFltrTableHashNumSets    numSets;            /**< Number of sets in the table (use enum fields in e_QeFltrTableHashNumSets) */
    uint8_t                     pramMemId;          /**< Parameter RAM memory partition ID */
    uint8_t                     tableMemId;         /**< Memory partition ID for the table allocation */
    uint32_t                    tableMemAttributes; /**< Memory attributes for the table's memory */

} t_QeFltrTableHashParam;


/**
@Description   Longest Prefix Match Table Lookup Initialization Structure
*/
typedef struct {

    uint16_t    numOfBinTrieBuffers;    /**< Maximum number of Binary Trie Buffers (should be positive) */
    uint8_t     pramMemId;              /**< Parameter RAM memory partition ID */
    uint8_t     tableMemId;             /**< Memory partition ID for the table allocation */
    uint32_t    tableMemAttributes;     /**< Memory attributes for the table's memory */

} t_QeFltrTableLpmParam;


/**
@Description   Lookup Table Initialization Structure
*/
typedef struct {

    e_QeFltrTableType   type;                   /**< The type of the table to create - used as selector for the union fields */
    t_Handle            h_QeId;                 /**< Handle to the QUICC Engine CB */
 
    union _param
    {
        t_QeFltrTableCamParam       cam;        /**< Specific user params for the CAM EMULATION table (type = e_QE_FLTR_TABLE_TYPE_CAM)     */
        t_QeFltrTableIndexedParam   indexed;    /**< Specific user params for the INDEXED       table (type = e_QE_FLTR_TABLE_TYPE_INDEXED) */
        t_QeFltrTableHashParam      hash;       /**< Specific user params for the HASH          table (type = e_QE_FLTR_TABLE_HASH)         */
        t_QeFltrTableLpmParam       lpm;        /**< Specific user params for the LPM           table (type = e_QE_FLTR_TABLE_LPM)          */

    } param;

} t_QeFltrTableParam;


/**
 @Function      QE_FLTR_TABLE_Config

 @Description   Configures a Lookup Table.

 @Param            p_Params    - (In)    User parameters for table.
 
 @Return        t_Handle    Handle to the created Lookup Table if successful.
 @Return        NULL        In case of failure to create object or invalid parameters.
*/
t_Handle QE_FLTR_TABLE_Config (t_QeFltrTableParam *p_Params);


/**
 @Function      QE_FLTR_TABLE_Init

 @Description   Initializes a Lookup Table.

 @Param            h_Table        - (In)    Handle to the newly-created table.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Config().
*/
t_Error    QE_FLTR_TABLE_Init (t_Handle h_Table);


/**
 @Function      QE_FLTR_TABLE_Free

 @Description   Releases the resources allocated to a Lookup Table.

 @Param            h_Table        - (In)    Handle to the Table to be freed.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Config().
*/
t_Error    QE_FLTR_TABLE_Free (t_Handle h_Table);


/**************************************************************************//**
 @Group         fltr_table_init_advanced_grp Table Object Advanced Initialization Unit

 @Description   Table Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description   Type definition for user callback function called by QE_FLTR_TABLE_AgingProcessEntries
                for deciding if an entry must be deleted or not from the aging management list.

 @Param         h_Table  (In)    Handle to the Table being scanned by aging.
 @Param         h_Entry  (In)    Application Id provided by user into InsertEntry API.

 @Return        TRUE if the entry must be removed by the driver from the aging management list and
                returned to the user. FALSE if entry must remain in aging management list.
*/
typedef bool (t_QeFltrTableAgingCallback) (t_Handle h_Table, t_Handle h_Entry);

/**
@Description   Lookup Table Aging Initialization Structure
*/
typedef struct {

    uint32_t                   totalNumOfEntries;          /**< Total number of aging enabled table entries */
    uint32_t                   numOfEntriesPerTablePass;   /**< Number of aging enabled table entries to process per pass; a zero value will enable the aging to process all the entries per pass */
    uint16_t                   numOfPassesPerTable;        /**< Number of passes per table, in an Aging Period. The user must call the QE_FLTR_TABLE_AgingProcessEntries function for numOfPassesPerTable times per aging period. Tuhs, each entry in the table will be checked for numOfPassesPerTable times in an aging period. */
    bool                       deleteAgedEntries;          /**< Allow the user to specify if the aged entries should be removed from the table by the aging mechanism */
    t_QeFltrTableAgingCallback *f_EntryAgingCallback;      /**< If not NULL this function will be called by QE_FLTR_TABLE_AgingProcessEntries to decide if the entry must be deleted or not from the aging management list */
} t_QeFltrTableAgingParam;


/**
 @Function      QE_FLTR_TABLE_ConfigAging
 
 @Description   Configures the Aging mechanism parameters for this lookup table.
                The user will poll the aging-enabled entries using the QE_FLTR_TABLE_AgingProcessEntries,
                by a number of numOfPassesPerTable times, in an aging-period. At each polling a number
                of numOfEntriesPerTablePass will be processed.
 
                 [FALSE]
 
 @Param            h_Table            - (In)    Handle to the table.
 @Param            p_Params        - (In)    User parameter for setting the Aging mechanism
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      Allowed only following QE_FLTR_TABLE_Config() and before QE_FLTR_TABLE_Init().
*/
t_Error QE_FLTR_TABLE_ConfigAging (t_Handle h_Table, t_QeFltrTableAgingParam *p_Params);

/** @} */ /* end of fltr_table_init_advanced_grp group */
/** @} */ /* end of fltr_table_init_grp group */



/**************************************************************************//**
 @Group         fltr_table_runtime_grp Table Object Runtime Unit

 @Description   Table Object API Runtime Unit

 @{
*//***************************************************************************/

/**
 @Description   Action Descriptor Specifier (TAD/IWAD)
*/
typedef enum {

      e_QE_FLTR_ETH_TAD    = 1    /**< Ethernet Termination Action Descriptor */
    , e_QE_FLTR_ATM_TAD    = 2    /**< ATM Termination Action Descriptor */
    , e_QE_FLTR_PPP_TAD    = 3    /**< PPP Termination Action Descriptor */
    , e_QE_FLTR_IWAD    = 4    /**< Interworking Action Descriptor */
    , e_QE_FLTR_PLAD    = 5    /**< Parse and Lookup Action Descriptor */
    , e_QE_FLTR_REJAD    = 6 /**< Reject Action Descriptor */
    , e_QE_FLTR_RAWAD   = 7 /**< Raw action descriptor */
    , e_QE_FLTR_VP_TAD    = 8    /**< VP Termination Action Descriptor */

} e_QeFltrActionDescriptorType;


/**
 @Description   Termination Vlan operation tagged
*/
typedef enum
{
      e_QE_FLTR_VLAN_OPERATION_TAGGED_NOP                                    = 0x0    /**< Tagged - no operation                           */
    , e_QE_FLTR_VLAN_OPERATION_TAGGED_REPLACE_VID_PORTION_OF_Q_TAG           = 0x1    /**< Tagged - replace vid portion of q tag           */
    , e_QE_FLTR_VLAN_OPERATION_TAGGED_IF_VID0_REPLACE_VID_WITH_DEFAULT_VALUE = 0x2    /**< Tagged - if vid0 replace vid with default value */
    , e_QE_FLTR_VLAN_OPERATION_TAGGED_EXTRACT_Q_TAG_FROM_FRAME               = 0x3    /**< Tagged - extract q tag from frame               */

} e_QeFltrVlanOperationTagged;


/**
 @Description   Termination Vlan operation non-tagged
*/
typedef enum
{
      e_QE_FLTR_VLAN_OPERATION_NON_TAGGED_NOP          = 0x0    /**< Non tagged - no operation */
    , e_QE_FLTR_VLAN_OPERATION_NON_TAGGED_Q_TAG_INSERT = 0x1    /**< Non tagged - q tag insert */

} e_QeFltrVlanOperationNonTagged;


/**
 @Description   Termination Rx Quality of Service Mode
*/
typedef enum
{
      e_QE_FLTR_QOS_MODE_DEFAULT                    = 0x0    /**< Default queue                   */
    , e_QE_FLTR_QOS_MODE_QUEUE_NUM_FROM_L2_CRITERIA = 0x1    /**< The queue is determined by L2 criteria, configured with UCC_GETH_ConfigVlanPriorityTable API */
    , e_QE_FLTR_QOS_MODE_QUEUE_NUM_FROM_L3_CRITERIA = 0x2    /**< The queue is determined by L3 criteria, configured with UCC_GETH_ConfigIpPriorityTable API */

} e_QeFltrQoSMode;


/**
 @Description   Destination Protocol IDs
*/
typedef enum {

      e_QE_FLTR_DEST_PROTO_NONE             = 0x0    /**< No destination protocol selected */
    , e_QE_FLTR_DEST_PROTO_ETH_UNICAST      = 0x2    /**< The destination protocol is Ethernet */
    , e_QE_FLTR_DEST_PROTO_ETH_MULTICAST    = 0x1    /**< The destination protocol is Ethernet multicast */
    , e_QE_FLTR_DEST_PROTO_AAL5_UNICAST     = 0x4    /**< The destination protocol is AAL5 unicast */
    , e_QE_FLTR_DEST_PROTO_AAL5_MULTICAST   = 0x5    /**< The destination protocol is AAL5 multicast */
    , e_QE_FLTR_DEST_PROTO_AAL2_SSSAR       = 0x9    /**< The destination protocol is AAL2 SSAR */
    , e_QE_FLTR_DEST_PROTO_ATM_PW           = 0x3   /**< The destination protocol is ATM PseudoWire */
    , e_QE_FLTR_DEST_PROTO_PPPoHT           = 0x6   /**< The destination protocol is PPP */
    , e_QE_FLTR_DEST_PROTO_VP               = 0xC   /**< The destination protocol is VP */
    , e_QE_FLTR_DEST_PROTO_CPU_SWAP_QUEUE   = 0xB   /**< The destination protocol is CPU Swap queue */

} e_QeFltrDestProtocol;


/**
 @Description   Ethernet Termination Action Descriptor
*/
typedef struct {

    bool                            rxNonDynamicExtendedFeaturesMode;
                                                /**< TRUE if any non-dynamic Rx ExtendedFeatures are selected. Non-dynamic means those features not specified in fields below */
    e_QeFltrVlanOperationTagged     vTagOP;     /**< VLAN operation for tagged incoming frames */
    e_QeFltrVlanOperationNonTagged  vNonTagOP;  /**< VLAN operation for non-tagged incoming frames */
    e_QeFltrQoSMode                 rQoS;       /**< RQoS mode */
    uint8_t                         vPri;       /**< [0-7] VLAN tag priority field. Used as Rx queue if rQoS is 00 */
    uint16_t                        vid;        /**< VLAN Tag */
    uint16_t                        iwctIndex;  /**< Index to Inter-working Connection Table, it should be taken from CDESC object */

} t_QeFltrEthTermActionDescriptor;


/**
 @Description   VP Termination Action Descriptor
*/
typedef struct {

    uint8_t  			cpuRxQueue;             /**< CPU's queue number for receive frame */
    bool                enableHitInterrupt;     /**< Enable lookup hit interrupt */
    uint16_t            iwctIndex;              /**< Index to Inter-working Connection Table, it should be taken from CDESC object */

} t_QeFltrVpTermActionDescriptor;


/**
 @Description   ATM Termination Action Descriptor
*/
typedef struct {

    uint8_t  			cpuRxQueue;             /**< CPU's queue number for receive frame */
    bool                enableHitInterrupt;     /**< Enable lookup hit interrupt */
    uint16_t            iwctIndex;              /**< Index to Inter-working Connection Table, it should be taken from CDESC object */

} t_QeFltrAtmTermActionDescriptor;


/**
 @Description   PPP Termination Action Descriptor
*/
typedef struct {

    bool                enableHitInterrupt;     /**< Enable lookup hit interrupt */

} t_QeFltrPppTermActionDescriptor;

 
/**
 @Description   Parse and Lookup Action Descriptor
*/
typedef struct {

    t_Handle h_FilterNext;          /**< Handle to the next Filter object */
    bool     forwardToCpu;          /**< If set, this bit will effect the forwardToCpu flag from the final
                                         IWAD by toggling it to TRUE. CPU's queue selection will be made
                                         based on the value for cpuRxQueue in the final IWAD. */
} t_QeFltrParseLookupActionDescriptor;


/**
 @Description   Interworking Action Descriptor
*/
typedef struct {

    bool                    forwardToCpu;           /**< Forward matching frame to CPU */
    bool                    isExtTqd;               /**< External TQD, it should be taken from Termination */
    bool                    enableHitInterrupt;     /**< Enable lookup hit interrupt */
    uint8_t      			cpuRxQueue;             /**< CPU's queue number for receive frame */
    e_QeFltrDestProtocol    destProtocol;           /**< This field identifies the destination protocol of the outgoing port */
    uint16_t                iwctIndex;              /**< Index to Inter-working Connection Table, it should be taken from CDESC object */
    uint16_t                iwIndex;                /**< Index used to access the transmit (destination) queue for the current frame, it should be taken from Termination */
#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE
    bool                    ipReassemblyEnabled;    /**< Indicates if this IWAD will forward fragments to an output port through the IP REASSEMBLY path, rather than directly to that port  */
#endif /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */
} t_QeFltrInterworkingActionDescriptor;


/**
 @Description   ATM Termination Action Descriptor
*/
typedef struct {

    uint16_t statisticsIndex;       /**< Index to statistics counters in the statistics table. Only Simple Statistics mode is supported. Use QE_IW_DEV_GetIntStatsCounterIndex or QE_IW_DEV_GetIntStatsCounterIndex to get a valid counter index. */
    bool     enableHitInterrupt;    /**< Issue lookup hit interrupt */

} t_QeFltrRejectActionDescriptor;


/**
 @Description   Raw Action Descriptor
*/
typedef struct {

    uint8_t  p_Entry[QE_FLTR_AD_SIZE];          /**< Pointer to custom user data */

} t_QeFltrRawActionDescriptor;


/**
 @Description   Filter Action Descriptor
*/
typedef struct {

    e_QeFltrActionDescriptorType    type;   /**< One of the e_QE_FLTR_TAD/e_QE_FLTR_IWAD values defined in e_QeFltrActionDescriptorType */
    bool                            aging;  /**< Aging enabled per table entry */
    uint16_t                        numPassesPerEntry;/**< Number of Table Passes, after which this entry shall be considered Aged */
 
    union _ad{
        t_QeFltrEthTermActionDescriptor         eth;    /**< Ethernet Termination Action Descriptor */
        t_QeFltrVpTermActionDescriptor          vp;     /**< VP Termination Action Descriptor */
        t_QeFltrAtmTermActionDescriptor         atm;    /**< ATM Termination Action Descriptor */
        t_QeFltrPppTermActionDescriptor         ppp;    /**< PPP Termination Action Descriptor */
        t_QeFltrInterworkingActionDescriptor    iw;     /**< Interworking Action Descriptor */
        t_QeFltrParseLookupActionDescriptor     pl;     /**< Parse and Lookup Action Descriptor */
        t_QeFltrRejectActionDescriptor          rej;    /**< Reject Action Descriptor */
        t_QeFltrRawActionDescriptor             raw;    /**< Raw action descriptor */

    } ad;
 
 

} t_QeFltrActionDescriptor;


/**
 @Description   Lookup Key.

 The lookup key is expressed as an array of bytes (key size is implied from the context).
 In case of Indexed Table Operations, only the first 4 bytes are relevant and should represent the compressed key.
 The compression should be done as follows: All bits which are set will result in bits in the final LookupKey.
 Bits which are zero result in deletion of the corresponding bit in the LookupKey, thus resulting in compression
 of the LookupKey to a programmable number of bits (maximum 32 bits), which determine the size of the Indexed LookupTable
*/
typedef struct {

    uint8_t     byte[QE_FLTR_MAX_LOOKUP_KEY_SIZE];  /**< The lookup key expressed as an array of bytes (key size is implied from the context) */
    uint8_t     keyPrefixLength;                    /**< Valid only for LPM Table Operations; In case of operations in the LPM Table - Identify the prefix relevant bits of the input key (should be in 0-32 range) */
 
} t_QeFltrLookupKey;


/**
 @Description   QE Extended Filtering Aging Management Deleted Entry
*/
typedef struct {

    t_Handle    h_Entry;    /**< Return the Application Id provided by user into InsertEntry API.
                                 In this cases the aged entry is not deleted */
} t_QeFltrDeletedEntry;


/**
 @Function      QE_FLTR_TABLE_Lookup

 @Description   Performs a lookup operation on a Lookup Table.

 @Param         h_Table            - (In)    Handle to the Table in which to perform the Lookup.
 @Param         p_LookupKey        - (In)    Pointer to the Lookup Key to find an entry for.
 @Param         h_Entry            - (Out)    Handle to the table entry corresponding to the supplied key,
                                         handle that can be used in subsequent specific API calls.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
 @Cautions      The results returned in h_Entry are valid only if the returned value is E_OK.
*/
t_Error    QE_FLTR_TABLE_Lookup (t_Handle                    h_Table,
                             t_QeFltrLookupKey            *p_LookupKey,
                             t_Handle                    *h_Entry);


/**
 @Function      QE_FLTR_TABLE_InsertEntry

 @Description   Inserts a new key into a Lookup Table.

 @Param         h_Table         - (In)    Handle to the Table in which to perform the Insertion.
 @Param         p_LookupKey     - (In)    Pointer to the Lookup Key to insert.
 @Param         p_ActionDesc    - (In)    Pointer to a user-supplied t_QeFltrActionDescriptor to associate with the key.
 @Param         h_Entry         - (In)    Application-provided Id that can be used for aging mechanism.
                                          This Id will be returned by QE_FLTR_TABLE_AgingProcessEntries API
                                - (Out)   Handle to the table entry corresponding to the inserted entry for the supplied key,
                                          handle that can be used in subsequent specific API calls.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
 @Cautions      If a key matching the one supplied to this call already exists in the table,
                then the information associated with that key is preserved and this call fails with E_ALREADY_EXISTS.
                However, the handle passed in h_Entry is valid even if E_ALREADY_EXISTS was returned and
                it points to the entry corresponding to the matching key already existing in the table.
*/
t_Error    QE_FLTR_TABLE_InsertEntry (t_Handle              h_Table,
                                  t_QeFltrLookupKey         *p_LookupKey,
                                  t_QeFltrActionDescriptor  *p_ActionDesc,
                                  t_Handle                  *h_Entry);


/**
 @Function      QE_FLTR_TABLE_ModifyEntry

 @Description   Modifies an existing entry in a Lookup Table.

 @Param         h_Table         - (In)    Handle to the Table in which to perform the Insertion/Modification.
 @Param         p_LookupKey     - (In)    Pointer to the Lookup Key to insert/modify.
 @Param         p_ActionDesc    - (In)    Pointer to a user-supplied t_QeFltrActionDescriptor to associate with the key.
 @Param         h_Entry         - (Out)   Handle to the table entry corresponding to the supplied key,
                                          handle that can be used in subsequent specific API calls.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
 @Cautions        If no key matching the one supplied to this call exists in the table,
                 then this call fails.
*/
t_Error    QE_FLTR_TABLE_ModifyEntry (t_Handle                    h_Table,
                                  t_QeFltrLookupKey            *p_LookupKey,
                                  t_QeFltrActionDescriptor    *p_ActionDesc,
                                  t_Handle                    *h_Entry);


/**
 @Function      QE_FLTR_TABLE_InsertOrModifyEntry

 @Description   Inserts a key into a Lookup Table or modifies its entry if it exists.

 @Param         h_Table         - (In)   Handle to the Table in which to perform the Insertion.
 @Param         p_LookupKey     - (In)   Pointer to the Lookup Key to insert.
 @Param         p_ActionDesc    - (In)   Pointer to a user-supplied t_QeFltrActionDescriptor to associate with the key.
 @Param         h_Entry         - (Out)  Handle to the table entry corresponding to the supplied key,
                                         handle that can be used in subsequent specific API calls.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
 @Cautions        If a key matching the one supplied to this call already exists in the table,
                 then the information associated with that key is overwritten by the new information.
*/
t_Error    QE_FLTR_TABLE_InsertOrModifyEntry (t_Handle                    h_Table,
                                          t_QeFltrLookupKey            *p_LookupKey,
                                          t_QeFltrActionDescriptor    *p_ActionDesc,
                                          t_Handle                    *h_Entry);


/**
 @Function      QE_FLTR_TABLE_DeleteEntry

 @Description   Deletes an existing key from a Lookup Table.

 @Param         h_Table     - (In)  Handle to the Table in which to perform the Lookup.
 @Param         p_LookupKey - (In)  Pointer to the Lookup Key to delete from the table.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
 
                In case of aging enabled, use DeleteEntry only if the entry was marked as aged by QE_FLTR_TABLE_AgingProcessEntries
                and not deleted (h_Entry provided to InsertEntry API is not NULL. otherwise the entry is deleted automatically by AgingProcess API)
*/
t_Error    QE_FLTR_TABLE_DeleteEntry (t_Handle                    h_Table,
                                  t_QeFltrLookupKey            *p_LookupKey);


/**
 @Function      QE_FLTR_TABLE_DeleteAgingEntry

 @Description   Deletes an existing key from both the Lookup Table and the Aging Management List.

 @Param         h_Table     - (In)  Handle to the Table in which to perform the Lookup.
 @Param         h_Entry     - (In)  Handle to the entry returned during Insert operation with aging enabled.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
*/
t_Error    QE_FLTR_TABLE_DeleteAgingEntry (t_Handle h_Table, t_Handle h_Entry);

/**
 @Function      QE_FLTR_TABLE_AgingProcessEntries

 @Description   Process the next specified number of aging enabled table entries in the lookup table.
                The entries that were previously marked as aged because they did not experience a lookup hit will be deleted from the lookup table.
                The deleted entries will be provided in the application array pointed by the p_DeletedEntries and their number in p_NumOfEntries.

 @Param         h_Table                 - (In)  Handle to the Lookup Table associated to Filter object.
 @Param         p_NumOfDeletedEntries   - (Out) The number of deleted aged entries.
 @Param         p_DeletedEntries        - (Out) The pointer to the memory location where the information of the
                                                deleted entries will be inserted.
                                                If the application provides a NULL pointer, the API function does not insert any information about the deleted entries.
                                                The memory location must be large enough in order to support insertion of all deleted entries per pass.
 
 @Return        E_OK if    success.
                E_NOT_FOUND when the lookup table has no more aging enabled entries.
                Error code otherwise.
*/
t_Error QE_FLTR_TABLE_AgingProcessEntries (t_Handle h_Table,
                                          uint16_t *p_NumOfDeletedEntries,
                                          t_QeFltrDeletedEntry *p_DeletedEntries);

/**
 @Function      QE_FLTR_TABLE_CheckAgingEnabledEntry

 @Description   Check the aging status of entry pointed by h_Entry parameter.
                The entry that was previously marked as aged because he did not experience a lookup hit will be deleted from the lookup table.
                The Lookup Key related to that entry is returned to the user.

 @Param         h_Table                 - (In)  Handle to the Lookup Table associated to Filter object.
 @Param         h_Entry                 - (In)  Handle to the entry returned during Insert operation with aging enabled.
 @Param         p_LookupKey             - (Out) The pointer to the memory location where the lookup key will be inserted.
                                                If the application provides a NULL pointer, the API function does not insert any information.
 
 @Return        E_OK            - The entry was not marked as agged
                E_TIMEOUT       - The entry was identified as aged and removed from agin management list. The user have to remove this entry from Lookup Table.
                E_NOT_SUPPORTED - The entry doesn't have aging enable.
                Error code otherwise.
*/
t_Error QE_FLTR_TABLE_CheckAgingEnabledEntry (t_Handle h_Table,
                                              t_Handle h_Entry,
                                              t_QeFltrLookupKey *p_LookupKey);


/**
 @Function      QE_FLTR_TABLE_CopyActionDescriptor
 
 @Description   Copy the Action descriptor associated with source key to an action descriptor associated
                with destination key

 @Param         h_Table             - (In)  Handle to the Lookup Table associated to Filter object.
 @Param         p_LookupKeySrc      - (In)  The source lookup key.
 @Param         p_LookupKeyDst      - (In)  The destination lookup key.
 @Param         aging               - (In)  Aging enabled per table entry.
 @Param         numPassesPerEntry   - (In)  Number of passes per entry.

 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
*/
t_Error QE_FLTR_TABLE_CopyActionDescriptor (t_Handle h_Table,
                                           t_QeFltrLookupKey *p_LookupKeySrc,
                                           t_QeFltrLookupKey *p_LookupKeyDst,
                                           bool aging,
                                           uint16_t numPassesPerEntry);

/** @} */ /* end of fltr_table_runtime_grp group */


/**************************************************************************//**
 @Group         fltr_table_control_grp Table Object Control Unit

 @Description   Table Object API Control Unit

 @{
*//***************************************************************************/

/**
 @Function      QE_FLTR_TABLE_GetRawEntry

 @Description   Retrives the data from a specified table entry pointed by h_Handle.

 @Param         h_Table         - (In)    Handle to the Table in which to perform the Lookup.
 @Param         h_Entry         - (In)    Pointer to the Lookup Key to find an entry for.
 @Param         p_ActionDesc    - (Out)   Pointer to the Action Descriptor table entry corresponding to the supplied entry handle,
                                          Only Raw Action Descriptor is returned.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
*/
t_Error    QE_FLTR_TABLE_GetRawEntry (t_Handle                    h_Table,
                                  t_Handle                    h_Entry,
                                  t_QeFltrActionDescriptor    *p_ActionDesc);
 
/**
 @Function      QE_FLTR_TABLE_ConvertToActionDescriptorEntry

 @Description   Converts the user formatted Action Descriptor in Raw Action Descriptor format.

 @Param         h_Table             - (In)    Handle to the Table currently used.
 @Param         p_InputActionDesc   - (In)    The Action Decriptor provided by the user
 @Param         p_ActionDesc        - (Out)   The corresponding Raw Action Descriptor entry returned.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Table must have been previously created using QE_FLTR_TABLE_Init().
*/
t_Error QE_FLTR_TABLE_ConvertToActionDescriptorEntry (t_Handle h_Table,
                                                      t_QeFltrActionDescriptor *p_InputActionDesc,
                                                      t_QeFltrActionDescriptor *p_ActionDesc);



/** @} */ /* end of fltr_table_control_grp group */
/** @} */ /* end of fltr_table_grp group */

/** @} */ /* end of fltr_grp group */


#ifdef __cplusplus
}
#endif

#endif /* __QE_FLTR_TABLE_EXT_H */

