/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_CONN_EXT_H
#define __QE_IW_CONN_EXT_H




/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_conn_grp IW CONNECTION Object Application Programming Interface

 @Description    IW CONNECTION Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_conn_init_grp IW CONNECTION Object Initialization Unit

 @Description   IW CONNECTION Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Connection destination. Identify the destination protocol of the outgoing port.
*/
typedef enum {

      e_QE_IW_CONN_NONE = 0             /**< Reject all matching packets */
	, e_QE_IW_CONN_TO_ETH_UNICAST       /**< Connection to Ethernet Unicast */
	, e_QE_IW_CONN_TO_AAL5_UNICAST      /**< Connection to ATM AAL5 Unicast */
	, e_QE_IW_CONN_TO_AAL5_MULTICAST    /**< Connection to ATM AAL5 Multicast */
	, e_QE_IW_CONN_TO_AAL2_SSSAR        /**< Connection to ATM AAL2 SSSAR */
    , e_QE_IW_CONN_TO_ATM_PW            /**< Connection to ATM Pseudo Wire */
	, e_QE_IW_CONN_TO_PPPoHT_PLAIN      /**< Connection to PPPoHT Plain*/
	, e_QE_IW_CONN_TO_PPPoHT_MLMC       /**< Connection to PPPoHT ML/MC */
	, e_QE_IW_CONN_TO_VP                /**< Connection to VP */

	, e_QE_IW_CONN_ETH_TO_CPU           /**< Ethernet forwarding to CPU */
	, e_QE_IW_CONN_ATM_TO_CPU           /**< ATM forwarding to CPU */
	, e_QE_IW_CONN_PPPoHT_TO_CPU        /**< PPP forwarding to CPU */
    , e_QE_IW_CONN_VP_TO_CPU            /**< VP forwarding to CPU */
 

} e_QeIwConnectionType;


/**
 @Description    User Parameters for Connection Object
*/
typedef struct {

    t_Handle                h_AppId;        /**< Application-provided Id for the connection */
    t_Handle                h_Common;       /**< Handle to the Interworking Common object */
    e_QeIwConnectionType    connType;       /**< Connection destination. It is used to identify the destination protocol of the outgoing port */
    t_Handle                h_TxChannel;    /**< Tx Channel handle or Multicast Group in IW mode. If isRedundant is TRUE (available only for Ethernet transmitter), this field can be NULL */
    t_Handle                h_ConnDescr;    /**< Handle to the Connection Descriptor for this Connection; for connection with CPU it should be NULL */
    uint16_t                destIndex;      /**< For Ethernet : Unicast Transmit Queue Index.
                                                 For ATM : unused.
                                                 For VP: the VP queue index of VP instance selected for TxChannel.
                                                 For PPPoHT: Link/Class Queue Index
                                                 In case the destination is an ETH Logical Port, then, this parameter should be set according the following table:
                                                   Logical Port #       Tx Queue #        destIndex value
                                                        0                 0  .. 7           0  .. 7
                                                        1                 8  .. 15          8  .. 15
                                                        2                 16 .. 23          16 .. 23
                                                        3                 24 .. 31          24 .. 31
                                                        4                 32 .. 39          32 .. 39
                                                        5                 40 .. 47          40 .. 47
                                                        6                 48 .. 55          48 .. 55
                                                        7                 56 .. 63          56 .. 63 */
} t_QeIwConnectionParam;


/**
 @Function      QE_IW_CONN_Config

 @Description   Configures a Connection object based on user-supplied parameters for IW mode.
 
                Connection can then be initialized using QE_IW_CONN_Init()
                and enabled using QE_IW_CONN_Enable().

 @Param         p_Params    - (In) Parameters passed by user/application to create a Connection object.

 @Return        Handle to a created Connection object if success.
 @Return        NULL in case of failure to create object or invalid parameters.
 
 @Cautions      Does not affect hardware, just internal driver structures.

*/
t_Handle QE_IW_CONN_Config (t_QeIwConnectionParam *p_Params);


/**
 @Function      QE_IW_CONN_Init

 @Description   This function initializes driver and hardware for IW Connection.
 
                Connection can then be enabled using QE_IW_CONN_Enable().

 @Param         h_Connection    - (In)    Connection handle that will be initialized.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      h_Connection must have been previously created using QE_IW_CONN_Config().

*/
t_Error    QE_IW_CONN_Init (t_Handle h_Connection);


/**
 @Function      QE_IW_CONN_Free

 @Description   This function frees memory used by the Connection object.

 @Param         h_Connection    - (In)    Connection handle that will be freed.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      h_Connection must have been previously created using QE_IW_CONN_Config().
 
*/
t_Error    QE_IW_CONN_Free (t_Handle h_Connection);


/**************************************************************************//**
 @Group         iw_conn_init_advanced_grp IW CONNECTION Object Advanced Initialization Unit

 @Description   IW CONNECTION Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description   Statistics Counter for Connection - Internal or External
*/
typedef enum
{
      e_QE_IW_CONN_STATISTICS_MODE_NONE         = 0     /**< No Statistics Counters */
    , e_QE_IW_CONN_STATISTICS_MODE_SIMPLE_INT   = 1     /**< Use simple statistics counter into internal memory */
    , e_QE_IW_CONN_STATISTICS_MODE_SIMPLE_EXT   = 2     /**< Use simple statistics counter into external memory */
 
} e_QeIwConnectionStatsMode;


/**
 @Description   Ethernet Termination VLAN Operation
*/
typedef struct {

    e_QeFltrVlanOperationTagged     vTagOP;     /**< VLAN operation for tagged incoming frames */
    e_QeFltrVlanOperationNonTagged  vNonTagOP;  /**< VLAN operation for non-tagged incoming frames */
    e_QeFltrQoSMode                 rQoS;       /**< RQoS mode */
    uint8_t                         vPri;       /**< [0-7] VLAN tag priority field. Used as Rx queue if rQoS is 00 */
    uint16_t                        vid;        /**< VLAN Tag */

} t_QeEthTermVlanOperation;


/**
 @Description   Rule for configuring the kook-up hit relations: the look-up hit relation between two filters or the last look-up hit relation for TAD/IWAD
*/
typedef struct {

    t_Handle            h_Filter;       /**< Filter handle */
    t_Handle            h_NextFilter;   /**< Next Filter handle (NULL for the last rule) */
    t_QeFltrLookupKey   key;            /**< Key to be inserted in filter's table */
    bool                forwardToCpu;   /**< For Last Rule: Forward matching frame to CPU For Intermediate Rules: if set, this bit will affect the forwardToCpu flag from the final IWAD by toggling it to TRUE.  CPU's queue selection will be made based on the value for cpuRxQueue in the final IWAD. */

} t_QeIwRule;

/**
 @Description   Logical Queue and Logical Port Selection Mode for Connection
*/
typedef enum
{
      e_QE_IW_CONN_QUEUE_SELECTION_MODE_NORMAL         = 0x00    /**< The Logical Queue and Logical Port are selected based on the LQnum and PQnum in the TQD */
    , e_QE_IW_CONN_QUEUE_SELECTION_MODE_VPRI_1ST_VLAN  = 0x80    /**< The Logical Queue and Logical Port are selected based on the mapping from the VPri1 to LQnum table */
    , e_QE_IW_CONN_QUEUE_SELECTION_MODE_VPRI_2ND_VLAN  = 0x81    /**< The Logical Queue and Logical Port are selected based on the mapping from the VPri2 to LQnum table */
    , e_QE_IW_CONN_QUEUE_SELECTION_MODE_IPTOS          = 0x82    /**< The Logical Queue and Logical Port are selected based on the mapping from the IP TOS to LQnum table */
 
} e_QeIwQueueSelectionMode;

/**
 @Function      QE_IW_CONN_ConfigAddRule

 @Description   Add a Rule in the Connection's RuleSet.

 @Param         h_Connection    - (In)    Connection handle that will be initialized.
 @Param         p_Rule          - (In)    Rule that will be added.

 @Return        E_OK if success. Error code otherwise.
 
*/
t_Error QE_IW_CONN_ConfigAddRule (t_Handle h_Connection, t_QeIwRule *p_Rule);


/**
 @Function      QE_IW_CONN_ConfigDebugCpuQueue

 @Description   Configures the Rx queue for frames sent to CPU.
 
 @Param         h_Connection     - (In)    Handle to the newly created Connection object.
 @Param         cpuRxQueue       - (In)    New value.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().

*/
t_Error QE_IW_CONN_ConfigDebugCpuQueue (t_Handle h_Connection, uint8_t cpuRxQueue);


/**
 @Function      QE_IW_CONN_ConfigForwardInterrupt
 
 @Description   Enables/disables reception of interrupt if the frame received on this connection is forwarded to outgoing port.
 
                This routine is relevant only for Interworking Connections
                [FALSE]
 
 @Param         h_Connection    - (In)    Handle to the newly created Connection object.
 @Param         enabled         - (In)    New value.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().

*/
t_Error    QE_IW_CONN_ConfigForwardInterrupt (t_Handle h_Connection, bool enabled);


/**
 @Function      QE_IW_CONN_ConfigEthToCpuRxNonDynamicExFeaturesMode

 @Description   Configures the rx non-dynamic extended features.
                [FALSE]
 
                This mode should be taken from GETH Termination using
                "UCC_GETH_GetRxNonDynamicExtendedFeaturesMode" function
 
                Non-dynamic means those features are not specified in connection with CPU.
                [FALSE]

 @Param         h_Connection                        - (In)    Handle to the newly created Connection object.
 @Param         rxNonDynamicExtendedFeaturesMode    - (Out) New value
 
 @Return        E_OK in case of success. Error code otherwise.
 
 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().

*/
t_Error QE_IW_CONN_ConfigEthToCpuRxNonDynamicExFeaturesMode(t_Handle h_Connection,
                                                            bool rxNonDynamicExtendedFeaturesMode);


/**
 @Function      QE_IW_CONN_ConfigEthToCpuVlanOperation
 
 @Description   Configures the VLAN operation on the frames sent to CPU
                [0x00]
                [0x0000]
                [e_QE_FLTR_VLAN_OPERATION_TAGGED_NOP]
                [e_QE_FLTR_VLAN_OPERATION_NON_TAGGED_NOP]
                [e_QE_FLTR_QOS_MODE_DEFAULT]
 
 @Param         h_Connection    - (In)    Handle to the newly created Connection object.
 @Param         p_EthVlanOp     - (In)    New VLAN operation.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().

*/
t_Error    QE_IW_CONN_ConfigEthToCpuVlanOperation (t_Handle h_Connection,
                                                t_QeEthTermVlanOperation *p_EthVlanOp);

/**
 @Function      QE_IW_CONN_ConfigStatisticsMode
 
 @Description   Instructs the connection to request statistics counter/s
                in internal/external memory.
                [e_QE_IW_CONN_STATISTICS_MODE_NONE]
 
 @Param         h_Connection    - (In)    Handle to the newly created Connection object.
 @Param         newStatsMode    - (In)    New Mode for allocating the Statistics Counter/s.
 
 @Return        E_OK in case of success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().

*/
t_Error    QE_IW_CONN_ConfigStatisticsMode (t_Handle h_Connection,
                                         e_QeIwConnectionStatsMode newStatsMode);

/**
 @Function      QE_IW_CONN_ConfigEthTxRedundancy
 
 @Description   Configures the connection as redundant. The connection destination port
                is the one configured in the DEV object.
                [FALSE]
 
 @Param         h_Connection    - (In)    Handle to the newly created Connection object.
 @Param         isRedundant     - (In)    Enable/disable redundancy for this connection.
 
 @Return        E_OK in case of success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().
                The queue number is the one set in Connection Param structure.

*/
t_Error    QE_IW_CONN_ConfigEthTxRedundancy (t_Handle h_Connection,
                                             bool     isRedundant);

/**
 @Function      QE_IW_CONN_ConfigAgingEnabledEntry
 
 @Description   Marks for aging the corresponding lookup table entry.
                If FALSE, there will be no marking for aging management for the corresponding lookup table entry.
                If TRUE, the corresponding lookup table entry will be mark for aging management and it will became an aging enabled table entry.
                [FALSE]
 
 @Param         h_Connection        - (In)  Handle to the newly created Connection object.
 @Param         enabled             - (In)  The new value for the enabled flag.
 @Param         numPassesPerEntry   - (In)  The number of table-passes before this entry shall be considered as aged.
                                            Number-of-table-passes is set on the FLTR_TABLE object.
 
 @Return        E_OK in case of success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().
 
*/
t_Error    QE_IW_CONN_ConfigAgingEnabledEntry (t_Handle   h_Connection,
                                                 bool     enabled,
                                                 uint16_t numPassesPerEntry);

#ifndef QE_IW_IPREASS_CUSTOM_UNAVAILABLE

/**
 @Function      QE_IW_CONN_ConfigIpReassemblyFrameOrdering
 
 @Description   Configures the frame order between regular and reassembled frames.
                In case one reassembled and one regular frame from the same flow are
                received in the same time, the reassembled frame is dropped.
                [FALSE]
 
 @Param         h_Connection    - (In)    Handle to the newly created Connection object.
 @Param         enabled         - (In)    Enable/disable frame order.
 
 @Return        E_OK in case of success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_CONN_Config() and before QE_IW_CONN_Init().
                Is available only for IP Reassembly Custom.
*/
t_Error	QE_IW_CONN_ConfigIpReassemblyFrameOrdering (t_Handle h_Connection, bool enabled);

#endif  /* QE_IW_IPREASS_CUSTOM_UNAVAILABLE */

/** @} */ /* end of iw_conn_init_advanced_grp group */
/** @} */ /* end of iw_conn_init_grp group */


/**************************************************************************//**
 @Group         iw_conn_control_grp IW CONNECTION Object Control Unit

 @Description   IW CONNECTION Object API Control Unit

 @{
*//***************************************************************************/

/**
 @Description   Ethernet Unicast Transmit Queue Number
*/
typedef enum
{
      e_QE_IW_TX_ETH_QUEUE_NUM_0 = 0
    , e_QE_IW_TX_ETH_QUEUE_NUM_1 = 1
    , e_QE_IW_TX_ETH_QUEUE_NUM_2 = 2
    , e_QE_IW_TX_ETH_QUEUE_NUM_3 = 3
    , e_QE_IW_TX_ETH_QUEUE_NUM_4 = 4
    , e_QE_IW_TX_ETH_QUEUE_NUM_5 = 5
    , e_QE_IW_TX_ETH_QUEUE_NUM_6 = 6
    , e_QE_IW_TX_ETH_QUEUE_NUM_7 = 7
 
} e_QeIwTxEthQueueNum;

/**
 @Function      QE_IW_CONN_Enable

 @Description   This function enables the Connection for Interworking.
 
                Connection can then be disabled using QE_IW_CONN_Disable().

 @Param         h_Connection    - (In)    Handle to the IW Connection.
 
 @Return        E_OK in case of success. Error code otherwise.
 
 @Cautions      h_Connection must have been previously created using QE_IW_CONN_Config()
                and initialized using QE_IW_CONN_Init().
 
*/
t_Error    QE_IW_CONN_Enable (t_Handle h_Connection);


/**
 @Function      QE_IW_CONN_Disable

 @Description   This function disables the IW Connection.
 
                Connection can be re-enabled using QE_IW_CONN_Enable().

 @Param         h_Connection    - (In)    Handle to the IW Connection.
 
 @Return        E_OK in case of success. Error code otherwise.
 
 @Cautions      h_Connection must have been previously created using QE_IW_CONN_Config()
                and initialized using QE_IW_CONN_Init().
 
*/
t_Error    QE_IW_CONN_Disable (t_Handle h_Connection);


/**
 @Function      QE_IW_CONN_ModifyConnectionDescriptor

 @Description   Assigns a connection descriptor object to a connection.
 
 @Param         h_Connection    - (In)    Handle to the Connection object.
 @Param         h_ConnDesctor   - (In)    Handle to the Connection Descriptor
                                     object to attach to this Connection.
 
 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Allowed only after QE_IW_CONN_Init() was called on the Connection object
                and after QE_CDESC_Init() was called on the Connection Descriptor.

*/
t_Error QE_IW_CONN_ModifyConnectionDescriptor (t_Handle h_Connection, t_Handle h_ConnDesctor);

/**
 @Function      QE_IW_CONN_EnableEthTxRedundancy

 @Description   Enables redundancy for this connection. The connection destination port
                is the one configured in the DEV object.
 
 @Param         h_Connection    - (In)    Handle to the Connection object.
 @Param            txEthQueueNum   - (In)    Ethernet Unicast Transmit Queue Number. The destination port
                                            is the one configured in the DEV object.
 
 @Return        E_OK if    success; Error code otherwise.
 
 @Cautions      Allowed only after QE_IW_CONN_Init() was called on the Connection object
                Available only for Ethernet receiver.
 
*/
t_Error QE_IW_CONN_EnableEthTxRedundancy (t_Handle   h_Connection,
                                          e_QeIwTxEthQueueNum    txEthQueueNum);


/**
 @Function      QE_IW_CONN_DisableEthTxRedundancy

 @Description   Disables redundancy for this connection. The connection destination port
                is the one configured with this function. Disabling the redundancy means that
                the connection will have a new destiation port than the one configured in
                the DEV object. To disable the connection the QE_IW_CONN_Disable function should be called.
 
 @Param         h_Connection        - (In)    Handle to the Connection object.
 @Param         h_TxChannel         - (In)    Tx Channel handle or Multicast Group in IW mode.
 @Param         txEthQueueNum        - (In)    Ethernet Unicast Transmit Queue Number.
 
 @Return        E_OK if success; Error code otherwise.
 
 @Cautions      Allowed only after QE_IW_CONN_Init() was called on the Connection object.
                Available only for Ethernet receiver.
 
*/
t_Error QE_IW_CONN_DisableEthTxRedundancy (t_Handle   h_Connection,
                                           t_Handle   h_TxChannel,
                                           e_QeIwTxEthQueueNum    txEthQueueNum);

/**
 @Function      QE_IW_CONN_GetStatistics

 @Description   Retrieves at runtime the statistics (reject counter) gathered for this Connection.

 @Param         h_Connection    - (In)    Handle to the Connection,
 @Param         p_Stats         - (Out)    Pointer to user-supplied data where to place the statistics gathered

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().

*/
t_Error    QE_IW_CONN_GetStatistics (t_Handle h_Connection, uint32_t *p_Stats);


/**
 @Function      QE_IW_CONN_StopStatistics

 @Description   Stop the statistics gathering for this Connection.
 
 @Param         h_Connection    - (In)    Handle to the Connection,

 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CONN_Init().
 
*/
t_Error QE_IW_CONN_StopStatistics ( t_Handle h_Connection );


/**
 @Function      QE_IW_CONN_RestartStatistics

 @Description   Restart the statistics gathering for this Connection.
 
                Restarting the statistics gathering for this Connection,
                is valid operation only if were configured as enabled prior to QE_IW_CONN_Init() call.
 

 @Param         h_Connection        - (In)    Handle to the Connection,

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CONN_Init().
 
*/
t_Error QE_IW_CONN_RestartStatistics ( t_Handle h_Connection );


/**
 @Function      QE_IW_CONN_ResetStatistics

 @Description   Clear the statistics counters for this Connection.
 
                In order to safely clear the statistics counters for this Connection,
                this routine disable the statistics gathering, clear counters, and enable the statistics if
                were enabled prior to QE_IW_CONN_ResetStatistics call.
                Reset the statistics counters for for this Connection,
                is valid operation only if statistics gathering was configured as enabled prior to QE_IW_CONN_Init() call.
 

 @Param         h_Connection        - (In)    Handle to the Connection,

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CONN_Init.
 
*/
t_Error QE_IW_CONN_ResetStatistics ( t_Handle h_Connection );


/**
 @Function      QE_IW_CONN_GetAppId

 @Description   Retrieves the App handle.

 @Param         h_Connection        - (In)  Handle to the Connection,
 @Param         h_AppId             - (Out)    Pointer to user-supplied App handle
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CONN_Init().

*/
t_Error QE_IW_CONN_GetAppId (t_Handle h_Connection, t_Handle *h_AppId);


/**
 @Function      QE_IW_CONN_CheckAgingEnabledEntry

 @Description   Check the aging status of the connection.
                The lookup table entry (IWAD) related to this Connection that was previously marked as aged because he did not experience a lookup hit will be deleted from the lookup table.

 @Param         h_Connection        - (In)  Handle to the Connection,
 
 @Return        E_OK            - The related Connection entry was not marked as aged
                E_TIMEOUT       - The related Connection entry was identified as aged and removed from agin management list. The user have to free this Connection object.
                E_NOT_SUPPORTED - The Connection doesn't have aging enable.
                Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_CONN_Init().

*/
t_Error QE_IW_CONN_CheckAgingEnabledEntry (t_Handle h_Connection);


/**
 @Function      QE_IW_CONN_SetExponentialWeightFactor

 @Description   Set the Exponential Weight Factor for Interworking for WRED functionality
                Note: If this function is not called, the exponential weight factor is initialized to 0.
 
 @Param         h_Connection    - (In)    Handle to the Connection object.
 @Param         expWeightFactor - (In)    Exponential Weight Factor; Should be calculated according to IW WRED specification.
                                          This is a parameter specific to the destination queue (unique combination of (txChannel,destIndex) )
 
 @Return        E_OK if    success; Error code otherwise.
 
 @Cautions      Allowed only after QE_IW_CONN_Init() was called on the Connection object
                Currently this feature is supported only when an Ethernet device is the receiving side, and ATM, PPP or another
                ETH are the destination devices.
                BE AWARE that this function actually affects the destination queue. All the connections towards the same destination use the same expWeightFactor,
                so changing this value for one connection will affect all of them.
 
*/
t_Error	QE_IW_CONN_SetExponentialWeightFactor (t_Handle h_Connection, uint8_t expWeightFactor);


/**
 @Function      QE_IW_CONN_ConfigEthQueueSelectionMode

 @Description   Set the way in which the Logical Queue and the Logical Port in which the frame will be placed are selected.

 @Param         h_Connection                      - (In)    Handle to the Connection object.
 @Param         txEthQueueSelMode                 - (In)    New Mode for Queue selection.

 @Return        E_OK if success. Error code otherwise.
 
*/
t_Error QE_IW_CONN_ConfigEthQueueSelectionMode (t_Handle h_Connection, e_QeIwQueueSelectionMode txEthQueueSelMode);
 
 
/** @} */ /* end of iw_conn_control_grp group */
/** @} */ /* end of iw_conn_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_CONN_EXT_H */
