/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          qe_iw_ipsec_ext.h

 @Description   External definitions and API for the IW IPSec object.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __QE_IW_IPSEC_EXT_H
#define __QE_IW_IPSEC_EXT_H


#include "part_ext.h"


/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_ipsec_grp    IW IPSec Object Application Programming Interface

 @Description   IW IPSec functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_ipsec_init_grp   IW IPSec Object Initialization Unit

 @Description   IW IPSec Initialization unit functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   User callback function prototype.
*//***************************************************************************/
typedef void (t_QeIwSecException)(t_Handle h_App,uint32_t eventReg);                        /**< Sec interrupt callback should be sent to SEC driver */
typedef void (t_QeIwIpsecUserException)(t_Handle h_App,uint8_t code,t_Handle h_AppSaEntry); /**< IPSec exception callback to user */

/**************************************************************************//**
 @Description   IV size
*//***************************************************************************/
typedef enum e_QeIwIpsecIvSize
{
    e_QE_IW_IPSEC_IV_SIZE_8    = 0, /**< 8  bytes IV */
    e_QE_IW_IPSEC_IV_SIZE_16   = 1, /**< 16 bytes IV */
    e_QE_IW_IPSEC_IV_SIZE_NONE = 2  /**< No IV */
} e_QeIwIpsecIvSize;

/**************************************************************************//**
 @Description   Encryption/Decryption algorithms
*//***************************************************************************/
typedef enum e_QeIwIpsecEncryptAlgorithms
{
    e_QE_IW_IPSEC_ENCRYPT_ALGORITHM_AES_CBC      = 1, /**< AES-CBC encryption algorithm */
    e_QE_IW_IPSEC_ENCRYPT_ALGORITHM_AES_CTR      = 2, /**< AES-CTR encryption algorithm */
    e_QE_IW_IPSEC_ENCRYPT_ALGORITHM_3DES         = 3, /**< 3DES encryption algorithm */
    e_QE_IW_IPSEC_ENCRYPT_ALGORITHM_NONE         = 0  /**< No encryption algorithm */

} e_QeIwIpsecEncryptAlgorithms;

/**************************************************************************//**
 @Description   Authentication algorithms
*//***************************************************************************/
typedef enum e_QeIwIpsecAuthnAlgorithms
{
    e_QE_IW_IPSEC_AUTHENTICATE_ALGORITHM_HMAC_SHA1_96 = 1,      /**< HMAC-SHA-1 (96b HMAC) authentication algorithm */
    e_QE_IW_IPSEC_AUTHENTICATE_ALGORITHM_HMAC_MD5_96  = 2,      /**< HMAC-MD5-96 authentication algorithm */
    e_QE_IW_IPSEC_AUTHENTICATE_ALGORITHM_AES_XCBC_MAC_96 = 3,   /**< AES-XCBC-MAC-96 authentication algorithm */
    e_QE_IW_IPSEC_AUTHENTICATE_ALGORITHM_NONE         = 0       /**< No authentication algorithm */

} e_QeIwIpsecAuthnAlgorithms;

/**************************************************************************//**
 @Description   IPSec algorithm combination to be used by SAs-
                user should chose a combination of one encryption and one authentication
                algorithm to be used by the SAs
*//***************************************************************************/
typedef struct t_QeIwIpsecAlgorithms
{
    e_QeIwIpsecEncryptAlgorithms        encryptionAlgorithm;      /**< Encryption algorithm */
    e_QeIwIpsecAuthnAlgorithms          authenticationAlgorithm;  /**< Authentication algorithm */

} t_QeIwIpsecAlgorithms;

/**************************************************************************//**
 @Description   SEC channels selection
*//***************************************************************************/
typedef enum e_QeIwIpsecSecChannelsPair
{
    e_QE_IW_IPSEC_CHN_1_2   = 0,    /**< First two channels of the SEC will be used for IPSec */
    e_QE_IW_IPSEC_CHN_3_4   = 1     /**< Second two channels of the SEC will be used for IPSec */
} e_QeIwIpsecSecChannelsPair;

/**************************************************************************//**
 @Description   SA's lifetime mode - QUICC Engine block supports SA lifetime management
                by keeping track on number of encrypted/decrypted bytes/frames per SA
                and taking actions at certain thresholds
*//***************************************************************************/
typedef enum e_QeIwIpsecSaLifeTimeMode
{
    e_QE_IW_IPSEC_SA_LIFETIME_DISABLED    = 0,  /**< SA lifetime mode disabled */
    e_QE_IW_IPSEC_SA_LIFETIME_BYTE_MODE   = 1,  /**< SA lifetime byte mode     */
    e_QE_IW_IPSEC_SA_LIFETIME_PACKET_MODE = 2   /**< SA lifetime packet mode   */
} e_QeIwIpsecSaLifeTimeMode;

/**************************************************************************//**
 @Description   IPSec padding check algorithm
*//***************************************************************************/
typedef enum e_QeIwIpsecPadCheckMode
{
    e_QE_IW_IPSEC_PAD_CHECK_DISABLED    = 0,    /**< Padding check is disabled */
    e_QE_IW_IPSEC_PAD_CHECK_USER        = 1,    /**< Compare the padding to the user programmed value in QE_IW_IPSEC_Config15BytePadVal or the default value */
    e_QE_IW_IPSEC_PAD_CHECK_LENGTH      = 2     /**< Compare the padding to the value in the pad length field */
} e_QeIwIpsecPadCheckMode;

/**************************************************************************//**
 @Description   Anti replay mode - Packets within the window will be dropped if they are a
                replay. Packets which are out of the window will be dropped.
*//***************************************************************************/
typedef enum e_QeIwIpsecSaAntiReplayMode
{
    e_QE_IW_IPSEC_SA_ANTI_REPLAY_DISABLED = 0,  /**< Anti replay is disabled */
    e_QE_IW_IPSEC_SA_ANTI_REPLAY_32_WIN   = 1,  /**< Anti replay window size is 32 */
    e_QE_IW_IPSEC_SA_ANTI_REPLAY_64_WIN   = 2   /**< Anti replay window size is 64 */
} e_QeIwIpsecSaAntiReplayMode;

/**************************************************************************//**
 @Description   The IPSec driver user's parameter structure.
                Passed to IPSEC_Config().
*//***************************************************************************/
typedef struct t_QeIwIpsecParam
{
    t_Handle                 h_App;              /**< Application handle */
    t_Handle                 h_Qe;               /**< An initialized QE module handle. */
    uint32_t                 secBaseAddress;     /**< SEC base address */
    int                      extIntrSrc;         /**< External request used to connect the SEC to the QE */
    uint8_t                  pramMemId;          /**< Parameter RAM partition ID */
    uint32_t                 dataMemAttributes;  /**< Memory attributes mask for data buffers */
    t_BufferPoolInfo         bufferPoolInfo;     /**< Data buffer pool information; used for intermediate queue and interrupt error queue used for packets */
    uint32_t                 maxThreadsNum;      /**< Maximum threads that are going to be used - sum of all threads in all VPs using IPSec */
    uint32_t                 secInterruptMask;   /**< Iinterrupts that a SEC driver using the free channels want to accept,error interrupts are sent by default */
    t_QeIwSecException       *f_SecException;    /**< Sec interrupt callback should be sent to SEC driver */
    t_QeIwIpsecUserException *f_IpsecException;  /**< IPSec exception callback to user */
   	t_RxStoreFunction        *f_RxStore;         /**< User callback function called by driver to pass received error frames to the application */

} t_QeIwIpsecParam;

/**************************************************************************//**
 @Description   SA parameter structure.
                Passed to QE_IW_IPSEC_AddSaEntry().
*//***************************************************************************/

typedef struct t_QeIwIpsecSaParam
{
    t_Handle                    h_App;              /**< SA application handle */
    uint32_t                    spi;                /**< SPI index of SA */
    bool                        outBound;           /**< Direction of SA - outbound or inbound */
    e_QeIwIpsecSaLifeTimeMode   lifeTimeMode;       /**< Lifetime mode */
    e_QeIwIpsecSaAntiReplayMode antiReplayMode;     /**< Anti replay mode */
    uint8_t                     interruptMask;      /**< IPSec SA exceptions types - IW errors mask */
    e_QeIwIpsecIvSize           ivSize;             /**< IV size */
    uint8_t                     cipherKeySize;      /**< Cipher key size in bytes */
    uint32_t                    *P_CipherKey;       /**< Cipher key pointer */
    uint8_t                     authenticKeySize;   /**< Authentication key size in bytes */
    uint32_t                    *P_AutenticKey;     /**< Authentication key pointer */
    uint64_t                    softLimit;          /**< Lifetime soft limit */
    uint32_t                    aditionalSoftLimit; /**< Aditional lifetime soft limit,this value added with the soft limit creates hard limit */
    t_QeIwIpsecAlgorithms       algorithms;         /**< Algorithms that will be used by the SA */
    e_QeIwIpsecPadCheckMode     padCheckMode;       /**< Pad checking mode */
    bool                        errToCpu;           /**< Error to CPU - if enabled and swap to CPU queue configured error frames will be sent to user */
    bool                        tunnelMode;         /**< Tunnel/Transport mode- TRUE, tunnel mode,FALSE, transport mode */
    uint16_t                    retSaEntryId;       /**< Returned SA entry index for user to use in his IWAD */
} t_QeIwIpsecSaParam;

/**************************************************************************//**
 @Description   Global counters structure
*//***************************************************************************/
typedef struct t_QeIwIpsecGlobalCounters
{
    uint32_t    hardLifeTimeIn;     /**< SA Hard Lifetime Counter for Inbound frames    */
    uint32_t    hardLifeTimeOut;    /**< SA Hard Lifetime Counter for Outbound frames   */
    uint32_t    snOverflow;         /**< SN overflow Counter (for outbound packets)     */
    uint32_t    replay;             /**< Replayed packets counter                       */
    uint32_t    latePackets;        /**< Late packets counter                           */
    uint32_t    authenticationErr;  /**< Authentication Error counter                   */
    uint32_t    protocolErr;        /**< Protocol Error counter                         */
} t_QeIwIpsecGlobalCounters;

/**************************************************************************//**
 @Description   SA counters structure
*//***************************************************************************/
typedef struct t_QeIwIpsecSaCounters
{
    bool        expired;                /**< Security Association Expired  */
    uint16_t	snOverflow;			    /**< SN overflow Counter           */
    uint64_t	lifeTimeBytes;  	    /**< SA Lifetime Bytes Counter     */
    uint16_t	replayCounter;  		/**< Replayed packets counter      */
    uint16_t	latePackets;			/**< Late packets counter          */
    uint16_t	authenticationErr;		/**< Authentication error counter  */
    uint16_t	protocolErr;			/**< Protocol error counter        */
    uint64_t	lifeTimePackets;		/**< SA Lifetime Packets Counter   */
    uint16_t	hardLifetTimeCounter;	/**< SA Hard Lifetime crossed Counter */
} t_QeIwIpsecSaCounters;
 
/**************************************************************************//**
 @Function      QE_IW_IPSEC_Config

 @Description   Initialize the IPSec configuration, using the default values
                and the user parameters structure.

 @Param[in]     p_IpsecParam - Pointer to IPSec configuration parameters.
 
 @Return        Handle to the IPSec control structure or NULL if configuration failed.
*//***************************************************************************/
t_Handle QE_IW_IPSEC_Config(t_QeIwIpsecParam *p_IpsecParam);



/**************************************************************************//**
 @Function      QE_IW_IPSEC_Init

 @Description   Initialize the IPSec.

 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().

 @Return        E_OK if successful or error type if failed.

 @Cautions      None
*//***************************************************************************/
t_Error QE_IW_IPSEC_Init(t_Handle h_Ipsec);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_Free

 @Description   Free the IPSec.

 @Param[in]     h_Ipsec - Handle to the IPSec as returned from the IPSEC_Config().

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_Free(t_Handle h_Ipsec);


/**************************************************************************//**
 @Group         iw_ipsec_advanced_init_grp  IW IPSec Object Advanced Configuration Unit

 @Description   IW IPSec advanced configuration functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigMaxIvSize

 @Description   Change the maximum IV size allowed to be used from its default
                configuration [e_QE_IW_IPSEC_IV_SIZE_16]

 @Param[in]     h_Ipsec - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     maxIvSize - maximum IV size in bytes - 0 , 8 or 16 .

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigMaxIvSize(t_Handle h_Ipsec, e_QeIwIpsecIvSize maxIvSize);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigAlgorithms

 @Description   Change the algorithm used by the IPSec driver from its default
                configuration.For now the driver supports these default combinations:
                1.AES  + SHA1
                2.AES  + MD5
                3.3DES + SHA1
                4.3DES + MD5
                5.SHA1(authentication only)
                6.MD5 (authentication only)

 @Param[in]     h_Ipsec - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     p_IpsecAlgorithmsArray - pointer to array of t_QeIwIpsecAlgorithms structure.
                                         Each structure should include one encryption algorithm and one authentication algorithm.
                                         There can be up to eight combinations configured.
                                         Last algorithm should have e_QE_IW_IPSEC_AUTHENTICATE_ALGORITHM_NONE configured as authentication algorithm if less than eight algorithms used.
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigAlgorithms(t_Handle h_Ipsec, t_QeIwIpsecAlgorithms *p_IpsecAlgorithmsArray);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigRiscAllocation

 @Description   Configures the QE RISC on which the IPSec is running.
                [QE_RISC_ALLOC_ALL_RISCS]
 
 @Param[in]     h_Ipsec        - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Param[in]     riscAllocation - RISC allocation

 @Return        E_OK on success; Error code otherwise

 @Cautions For the 'riscAllocation' argument, use the 'RISC Allocation' macros.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigRiscAllocation(t_Handle h_Ipsec, uint8_t riscAllocation);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_Config15BytePadVal

 @Description   Configures the template which is used by the IPSec controller in
                padding up to 15 bytes in the ESP trailer. To be compliant with RFC4303, CPU should
                initialize this field with the following value:0x01020304
                                                               0x05060708
                                                               0x090A0B0C
                                                               0x0D0E0F
 
 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Param[in]     p_PadVal - Pointer to fifteen byte padding value
 
 @Return        E_OK on success; Error code otherwise
*//***************************************************************************/
t_Error QE_IW_IPSEC_Config15BytePadVal(t_Handle h_Ipsec, uint8_t  *p_PadVal);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigCryptoChannelsPair

 @Description   Change the pair of SEC channels to be used by the IPSec controller from its default
                configuration [e_QE_IW_IPSEC_CHN_1_2]

 @Param[in]     h_Ipsec      - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     channelsPair - SEC controller pair of channels to be used either 1st and 2nd or 3rd and 4th

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigCryptoChannelsPair(t_Handle     h_Ipsec, e_QeIwIpsecSecChannelsPair     channelsPair);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigMaxSaEntriesNum

 @Description   Change the number of SA entries allowed to be used simultaneously by the IPSec controller from its default
                configuration [32]

 @Param[in]     h_Ipsec         - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     maxSaEntriesNum - Maximum SA entries

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigMaxSaEntriesNum(t_Handle    h_Ipsec, uint32_t     maxSaEntriesNum);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_Ipsec       - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     memId         - Memory partition ID.
 
 @Param[in]     memAttributes - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigFirmwareExternalStructsMemory(t_Handle    h_Ipsec,
                                                        uint8_t     memId,
                                                        uint32_t    memAttributes);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigSaMemory

 @Description   Change the location of SAPT table from its default
                configuration.[FALSE(internal memory)]

 @Param[in]     h_Ipsec       - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     External      - TRUE if SAPT located in external memory,
                                FALSE if SAPT located in internal memory .
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigSaMemory(t_Handle    h_Ipsec,
                                   bool        External);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigPerformanceImprovement

 @Description   Configuring a special mode in which the performance of the package can
                be Improved. This mode has the most effect on short frames.
 
 @Param[in]     h_Ipsec       - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     performanceImprovement      - TRUE performance improvement mode is enabled.
                                              FALSE performance improvement mode is disabled.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      In order to work in this special mode the firmware external structure should be
                located on the SRAM - this should be configured through QE_IW_IPSEC_ConfigFirmwareExternalStructsMemory
                memId = e_MEM_L2_SRAM, memAttributes = MEMORY_ATTR_CACHEABLE
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigPerformanceImprovement(t_Handle    h_Ipsec,
                                                 bool        performanceImprovement);


/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigMultiHostInterrupts

 @Description   Configuring of the interrupt source that will be used by the Ucode.
                If the platform allows it the Ucode can work with one of two SEC intterupts. Usually
                if the OS wants to work with the SEC  directly it will be using the regular first interrupt and the Ucode will use
                the second one.
                If the Ucode is conected to the SEC by the secondary interrupt the SEC should be configured in a way that the pair of channels
                used by the Ucode  will reside in the alternate adsress(SEC MCR register).
 
 @Param[in]     h_Ipsec       - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     secSecondInterruptUsed      - TRUE second interrupt will be used.
                                              FALSE first interrupt is used.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions
*//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigMultiHostInterrupts(t_Handle    h_Ipsec,
                                              bool        secSecondInterruptUsed);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_ConfigSwapToCpuQueueParams

 @Description   Change the default parameters of the swap to cpu error queue

 @Param[in]     h_Ipsec         - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     isExternal      - is the swap queue is internal or external queue.
 
 @Param[in]     interruptQueue  - Index of interrupt queue to be used.

 @Param[in]     bdRingLen       - BD ring size in number of BDs.
 
 @Param[in]     expWeightFactor - Exponential weight factor for WRED.

 @Return        E_OK if successful or error type if failed.
 *//***************************************************************************/
t_Error QE_IW_IPSEC_ConfigSwapToCpuQueueParams(t_Handle    h_Ipsec,
                                               bool        isExternal,
                                               uint8_t     interruptQueue,
                                               uint16_t    bdRingLen,
                                               uint8_t     expWeightFactor);
/** @} */ /* end of iw_ipsec_advanced_init_grp */
/** @} */ /* end of iw_ipsec_init_grp */

/**************************************************************************//**
 @Group         iw_ipsec_runtime_grp    IW IPSec Object Runtime Unit

 @Description   IW IPSec runtime functions

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_IW_IPSEC_AddSaEntry

 @Description   Add SA entry to SAD.User gives the SA entry parameters, and a new SA entry is created,
                If an old SA entry had the same spi and direction(outbound,inbound) and application handle the old one will
                be replaced by the new one with a new internal driver index ,this index need to be updated in the IWAD of the connection using this entry .

 @Param[in]     h_Ipsec - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     P_IpsecSAParam - Pointer to SA entry parameters structure.
                                 The variable retSaEntryId is returning to the user the internal driver SA index,
                                 this index will be used later for deleting the SA entry or reading statistics.
 
 @Return        E_OK if successful or error type if failed.
 *//***************************************************************************/
t_Error QE_IW_IPSEC_AddSaEntry(t_Handle     h_Ipsec, t_QeIwIpsecSaParam     *P_IpsecSAParam);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_RemoveSaEntry

 @Description   Remove SA entry from SAPT.This SAPT can't be used any more.

 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().

 @Param[in]     saEntryId  - index of SA entry used by the driver,the index was given to the user  when he called
                             QE_IW_IPSEC_AddSaEntry, in the variable retSaEntryId part of P_IpsecSAParam.
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_RemoveSaEntry(t_Handle     h_Ipsec, uint16_t     saEntryId);


/**************************************************************************//**
 @Function      QE_IW_IPSEC_FlushSAD

 @Description   Remove all SA entries from SAD.

 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_FlushSAD(t_Handle     h_Ipsec);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_GetGlobalCounters

 @Description   Read IPSec global counters.

 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Param[out]    p_IpsecGlobalCounters - pointer to global counters structure.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_GetGlobalCounters(t_Handle     h_Ipsec, t_QeIwIpsecGlobalCounters  *p_IpsecGlobalCounters);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_GetSaCounters

 @Description   Read SA statistics.

 @Param[in]     h_Ipsec   - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Param[in]     saEntryId - SA entry id  that was returned in the field retSaEntryId
                            part of P_IpsecSAParam, when configuring a new SA through QE_IW_IPSEC_AddSaEntry().
 
 @Param[out]    p_IpsecSaCounters - pointer to global counters structure.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_GetSaCounters(t_Handle                  h_Ipsec,
                                  uint16_t                  saEntryId,
                                  t_QeIwIpsecSaCounters     *p_IpsecSaCounters);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_DisableSecThreads

 @Description   IPsec threads disabled command.
                In order to recover from a halted channel due to channel error, the User should
                perform the following steps:
 
                1. Disable new threads from activating the relevant SEC channel(s). This is done
                by calling this function QE_IW_IPSEC_DisableSecThreads.
 
                2. Disable the relevant SEC channel(s) and clear the errors bit in the Interrupt Clear Register (ICR).
                For further information see "Crypto-Channel Configuration Register (CCCR)" or "Channel
                Configuration Register (CCR)" and "Interrupt Clear Register (ICR)" sections in the "Security
                Engine (SEC)" chapter of the device reference manual.

                3. Release old threads that are waiting for a done indication from the SEC channel and enable new
                threads to activate the SEC. This is done  by calling the function QE_IW_IPSEC_EnableSecThreads
 
                4. Restart the disabled SEC channel(s). For further information see Channel Error Interrupt section
                in the Security Engine (SEC) chapter of the device reference manual.
 
                There is no need to call this function for starting or stopping to work, it should be called only
                as part of the sequence mentioned above as a result of a SEC error.
 
 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_DisableSecThreads(t_Handle     h_Ipsec);

/**************************************************************************//**
 @Function      QE_IW_IPSEC_EnableSecThreads

 @Description   IPsec threads enable command.
                In order to recover from a halted channel due to channel error, the User should
                perform the following steps:
 
                1. Disable new threads from activating the relevant SEC channel(s). This is done
                by calling the function QE_IW_IPSEC_DisableSecThreads.
 
                2. Disable the relevant SEC channel(s) and clear the errors bit in the Interrupt Clear Register (ICR).
                For further information see "Crypto-Channel Configuration Register (CCCR)" or "Channel
                Configuration Register (CCR)" and "Interrupt Clear Register (ICR)" sections in the "Security
                Engine (SEC)" chapter of the device reference manual.

                3. Release old threads that are waiting for a done indication from the SEC channel and enable new
                threads to activate the SEC. This is done  by calling this function QE_IW_IPSEC_EnableSecThreads
 
                4. Restart the disabled SEC channel(s). For further information see Channel Error Interrupt section
                in the Security Engine (SEC) chapter of the device reference manual.
 
                There is no need to call this function for starting or stopping to work, it should be called only
                as part of the sequence mentioned above as a result of a SEC error.

 @Param[in]     h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error QE_IW_IPSEC_EnableSecThreads(t_Handle     h_Ipsec);

/** @} */ /* end of iw_ipsec_runtime_grp */

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**
 @Function      QE_IW_IPSEC_DumpRegs

 @Description   Dump IPSEC GSPT and SAPT Parameter Tables regs.

 @Param         h_Ipsec  - Handle to the IPSec as returned from the IPSEC_Config().
*/
t_Error QE_IW_IPSEC_DumpRegs ( t_Handle h_Ipsec );
#endif

/**************************************************************************//**
 @Group         iw_ipsec_exception_grp  IW IPSec Object Exceptions Unit

 @Description   IW IPSec exceptions unit functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
IPSec SEC exceptions types , configured through secInterruptMask parameter through IPSec_Config
*//***************************************************************************/

#define ITO        0x00010000 /**< Internal Time Out */

#define CH4_DN_OV  0x00000800 /**< Channel 4 done overflow. */

#define CH3_DN_OV  0x00000400 /**< Channel 4 done overflow. */

#define CH2_DN_OV  0x00000200 /**< Channel 4 done overflow. */

#define CH1_DN_OV  0x00000100 /**< Channel 4 done overflow. */

#define CH4_ERR    0x00000080 /**< Channel 4 error */

#define CH4_DN     0x00000040 /**< Channel 4 done  */

#define CH3_ERR    0x00000020 /**< Channel 3 error */

#define CH3_DN     0x00000010 /**< Channel 3 done  */

#define CH2_ERR    0x00000008 /**< Channel 2 error */

#define CH2_DN     0x00000004 /**< Channel 2 done  */

#define CH1_ERR    0x00000002 /**< Channel 1 error */

#define CH1_DN     0x00000001 /**< Channel 1 done  */

/**************************************************************************//**
IPSec SA exceptions types , configured through interruptMask parameter for each SA
*//***************************************************************************/

#define SLIE     0x80 /**< Soft Limit Interrupt Enable */

#define HLIE     0x40 /**< Hard Limit Interrupt Enable. */

#define FPIE     0x20 /**< First Packet Interrupt Enable */

#define AFIE     0x10 /**< Authentication Failure Interrupt Enable */

#define TEIE     0x08 /**< ESP Trailer Error Interrupt Enable */

#define LPIE     0x04 /**< Late Packet Interrupt Enable */

#define PRIE     0x02 /**< Packet Replay Interrupt Enable */

#define SXIE     0x01 /**< SN Expired Interrupt Enable */

/**************************************************************************//**
IPSec exceptions codes
*//***************************************************************************/

#define IPSEC_SA_OUT_HARD_LIFETIME_INTERRUPT  0x00 /**< SA lifetime hard limit crossed (for outbound packet). */
#define IPSEC_SA_IN_HARD_LIFETIME_INTERRUPT   0x02 /**< SA lifetime hard limit crossed (for inbound packet). */
#define IPSEC_SN_COUNTER_OVERFLOW_INTERRUPT   0x04 /**< SN counter overflow (for outbound packet). */
#define IPSEC_REPLAY_PACKET_INTERRUPT         0x05 /**< Packet is a replay (for inbound packet). */
#define IPSEC_LATE_PACKET_INTERRUPT           0x06 /**< Late packet (for inbound packet). */
#define IPSEC_ESP_TRAILER_ERROR_INTERRUPT     0x07 /**< Wrong padding in ESP trailer (for inbound packet). */
#define IPSEC_AUTH_FAILURE_INTERRUPT          0x08 /**< Authentication Failure (for inbound packet). */
#define IPSEC_SA_OUT_FIRST_PACKET_INTERRUPT   0x09 /**< First packet processed by SA. (for outbound packet). */
#define IPSEC_SA_IN_FIRST_PACKET_INTERRUPT    0x0a /**< First packet processed by SA. (for inbound packet). */
#define IPSEC_SA_OUT_SOFT_LIFETIME_INTERRUPT  0x0c /**< SA lifetime soft limit crossed. (for outbound packet). */
#define IPSEC_SA_IN_SOFT_LIFETIME_INTERRUPT   0x0d /**< SA lifetime soft limit crossed. (for inbound packet). */

/** @} */ /* end of iw_ipsec_exception_grp */
/** @} */ /* end of iw_ipsec_grp */
/** @} */ /* end of iw_grp */


typedef void (t_QeIwIpsecException)(t_Handle h_App,uint32_t saEntryId,uint8_t code);        /**< IW error interrupt callback */

typedef struct t_QeIwIpsecIwDetails
{
    uint32_t                *p_GpstBase;        /**< GSPT base address */
    uint32_t                *p_SaptBase;        /**< SAPT base address */
    uint32_t                fwExtStructsMemAttributes;/**< Memory attributes mask for the firmware's external structures */
    uint32_t                dataMemAttributes;  /**< Memory attributes mask for data buffers */
    t_BufferPoolInfo        bufferPoolInfo;     /**< Data buffer pool information; used for intermediate queue and interrupt error queue used for packets */
    bool                    saptOnExtMem;       /**< SAPT on external memory */
    t_QeIwIpsecException    *f_IwIpsecException;/**< IW interrupt will be sent to IPsec driver */
    bool                    swapQueueIsExternal;/* is the swap queue is internal or external queue */
} t_QeIwIpsecIwDetails;

t_Error QE_IW_IPSEC_GetIwDetails(t_Handle     h_Ipsec,t_QeIwIpsecIwDetails    *p_Iwdetails);
t_Error QE_IW_IPSEC_SetSwapToCpuQeue(t_Handle h_Ipsec,t_Handle h_VpPort,uint32_t queueId);
t_Error QE_IW_IPSEC_FreeSwapToCpuQeue (t_Handle h_Ipsec,t_Handle h_VpPort,bool *isExternal);


#endif /* __QE_IW_IPSEC_EXT_H */
