/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_MCAST_EXT_H
#define __QE_IW_MCAST_EXT_H




/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_mcast_grp IW MULTICAST Object Application Programming Interface

 @Description    IW MULTICAST Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_mcast_init_grp IW MULTICAST Object Initialization Unit

 @Description   IW MULTICAST Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Types of Multicast Groups
*/
typedef enum {

    e_QE_IW_MCAST_TYPE_AAL5    = 1    /**< AAL5 Multicast Group */
 
} e_QeIwMcastGroupType;


/**
 @Description   Multicast Group User Parameters
*/
typedef struct {

    t_Handle                h_AppId;        /**< Application-provided ID for the Multicast Group */
    t_Handle                h_Common;       /**< Handle to the IW Common object */
 
    e_QeIwMcastGroupType    type;           /**< The type of the Multicast Group (ETH/AAL5) */

    uint8_t                 maxMembersNum;  /**< Maximum number of members for this group */
 
    t_Handle                h_FirstChannel; /**< Handle to the first channel in the group - if removed the group object is destroyed */
 
} t_QeIwMcastGroupParam;


/**
 @Function      QE_IW_MCAST_Config

 @Description   Configures a multicast object based on user parameters.
 
 @Param         p_Params    - (In)    Multicast Group's parameters.
 
 @Return        Handle to the new Multicast Group.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle QE_IW_MCAST_Config (t_QeIwMcastGroupParam    *p_Params);


/**
 @Function      QE_IW_MCAST_Init

 @Description   Initializes a Multicast Group object.
 
 @Param         h_McastGroup    - (In)    Handle to the Multicast Group.
 
 @Return        E_OK if    success. Error code otherwise.
 
 @Cautions      h_McastGroup must have been previously created using QE_IW_MCAST_Config().
*/
t_Error QE_IW_MCAST_Init (t_Handle h_McastGroup);


/**
 @Function      QE_IW_MCAST_Free

 @Description   This function removes all channels from Multicast Group and
                frees memory of the Multicast handler.

 @Param         h_McastGroup    - (In)    Handle to multicast group.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_McastGroup must have been previously created using QE_IW_MCAST_Config().
*/
t_Error QE_IW_MCAST_Free (t_Handle h_McastGroup);

/** @} */ /* end of iw_mcast_init_grp group */


/**************************************************************************//**
 @Group         iw_mcast_control_grp IW MULTICAST Object Control Unit

 @Description   IW MULTICAST Object API Control Unit

 @{
*//***************************************************************************/

/**
 @Function      QE_IW_MCAST_AddAAL5Member

 @Description   Adds an AAL5 or Ethernet channel to the Multicast Group.

 @Param         h_McastGroup    - (In)    Handle to Multicast Group,
 @Param         h_Channel       - (In)    Handle to AAL5 Channel.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Addition of an Ethernet or an AAL2 channel to a AAL5 Multicast Group
                is hazardous and an error for that cannot be reported!
 @Cautions      Predictable results only when used after QE_IW_MCAST_Init().
*/
t_Error QE_IW_MCAST_AddAAL5Member (t_Handle h_McastGroup, t_Handle h_Channel);


/**
 @Function      QE_IW_MCAST_RemoveAAL5Member

 @Description   Removes an AAL5 or Ethernet channel from the Multicast Group.

 @Param         h_McastGroup    - (In)    Handle to Multicast Group,
 @Param         h_Channel       - (In)    Handle to AAL5 Channel.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Removal of a member from a Multicast Group of an incorrect type
                is hazardous and an error for that cannot be reported!
 @Cautions      Attempted removal of a non-member, albeit of the correct type,
                has no effect on the given member, nor on the Multicast Group.
 @Cautions      Predictable results only when used after QE_IW_MCAST_Init().
*/
t_Error QE_IW_MCAST_RemoveAAL5Member (t_Handle h_McastGroup, t_Handle h_Channel);

/**
 @Function      QE_IW_MCAST_GetNumMembers

 @Description   Retrieves the number of members in this Multicast Group.

 @Param         h_McastGroup    - (In)    Handle to the Multicast Group.
 @Param         p_NumMembers    - (Out) Handle to the Multicast Group number of members.
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_MCAST_Init().
*/
t_Error QE_IW_MCAST_GetNumMembers (t_Handle h_McastGroup, uint8_t *p_NumMembers);


/**
 @Function      QE_IW_MCAST_GetIwIndex

 @Description   Returns the IW index used to access the multicast transmit (destination) queue for the current frame.

 @Param         h_McastGroup    - (In)    Handle to the Multicast Group.
 
 @Return        IW index. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_MCAST_Init().
*/
uint16_t QE_IW_MCAST_GetIwIndex (t_Handle h_McastGroup);

/** @} */ /* end of iw_mcast_control_grp group */
/** @} */ /* end of iw_mcast_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_MCAST_EXT_H */
