/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_PLCR_EXT_H
#define __QE_IW_PLCR_EXT_H



#include "qe_ext.h"

/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_plcr_grp IW POLICER Object Application Programming Interface

 @Description    IW POLICER Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_plcr_init_grp IW POLICER Object Initialization Unit

 @Description   IW POLICER Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Identifiers for  the types of Policer modes (color blind or aware)
*/
typedef enum
{
      e_QE_IW_PLCR_COLOR_BLIND = 0    /**< Color Blind Policer */
    , e_QE_IW_PLCR_COLOR_AWARE        /**< Color Aware Policer */

} e_QeIwPlcrColorMode;


/**
 @Description    User Parameters for configuring a Policer Profile
*/
typedef struct {
 
    t_Handle                h_AppId;    /**< Handle to user defined data */
    t_Handle                h_Dev;      /**< Handle to the Dev object - the upper layer for this object */

    e_QeIwPlcrColorMode     colorMode;  /**< Selects the Color Mode of the Policer */

    uint32_t                cir;        /**< Committed information rate (bytes or packets depending on traffic rate mode) */
    uint32_t                cbs;        /**< Committed burst size (bytes or packets depending on traffic rate mode) */
 
    uint32_t                eir;        /**< Excess information rate (bytes or packets depending on traffic rate mode */
    uint32_t                ebs;        /**< Excess burst rate (bytes or packets depending on traffic rate mode */
 
    bool                    isInternal; /**< Whether to allocate the Connection Descriptor in internal or in external memory */

} t_QeIwPlcrParam;


/**
 @Function      QE_IW_PLCR_Config

 @Description   Sets user supplied values for this profile and increments the reference count for this profile

 @Param         p_Params - (In) Policer parameters
 
 @Return        Handle to the new Policer object if success.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle QE_IW_PLCR_Config(t_QeIwPlcrParam* p_Params);


/**
 @Function      QE_IW_PLCR_Init

 @Description   Initializes microcode structures associated with this Policer
                based on already configured information.

 @Param         h_Policer - (In) Policer object to be initialized
 
 @Return        E_OK if    success. Error code otherwise.

 @Cautions      h_Policer must have been previously created using QE_IW_PLCR_Config().
*/
t_Error QE_IW_PLCR_Init(t_Handle h_Policer);


/**
 @Function      QE_IW_PLCR_Free

 @Description   Decrements the reference count for this policer

 @Param         h_Policer - (In) Policer object to be freed
 
 @Return        E_OK if success.
 @Return        E_BUSY if there are referring connections.
*/
t_Error QE_IW_PLCR_Free(t_Handle  h_Policer);


/**************************************************************************//**
 @Group         iw_plcr_init_advanced_grp IW POLICER Object Advanced Initialization Unit

 @Description   IW POLICER Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
@Description   Specifies the units for measuring the traffic rate in the policer
*/
typedef enum
{
      e_QE_IW_PLCR_BYTES_PER_SECOND = 0     /**< The traffic rate of the policer is measured bytes/second */
    , e_QE_IW_PLCR_PACKETS_PER_SECOND       /**< The traffic rate of the policer is measured packets/second */
 
} e_QeIwPlcrTrafficRateMode;


/**
 @Function      QE_IW_PLCR_ConfigTimeStampRegister

 @Description   Selects the TimeStampRegister used
                [e_QE_TIME_STAMPER_1]
 
 @Param         h_Policer           - (In)  Handle to the policer object.
 @Param         timeStampRegister   - (In)  New value for the time stamp register.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_PLCR_Config() and before QE_IW_PLCR_Init().
*/
t_Error QE_IW_PLCR_ConfigTimeStampRegister(t_Handle h_Policer, e_QeTimeStamper timeStampRegister);


/**
 @Function      QE_IW_PLCR_ConfigTrafficRateMode

 @Description   Selects the units that measure the traffic rate in policer
                [e_QE_IW_PLCR_BYTES_PER_SECOND]
 
 @Param         h_Policer       - (In)  Handle to the policer object.
 @Param         trafficRateMode - (In)  New value for the traffic rate mode.

 @Return        E_OK if    success. Error code otherwise.

 @Cautions      Allowed only following QE_IW_PLCR_Config() and before QE_IW_PLCR_Init().
*/
t_Error QE_IW_PLCR_ConfigTrafficRateMode(t_Handle  h_Policer, e_QeIwPlcrTrafficRateMode trafficRateMode);

/** @} */ /* end of iw_plcr_init_advanced_grp group */
/** @} */ /* end of iw_plcr_init_grp group */


/**************************************************************************//**
 @Group         iw_plcr_control_grp IW POLICER Object Control Unit

 @Description   IW POLICER Object API Control Unit

 @{
*//***************************************************************************/

/**
@Description   Policer Statistics
*/
typedef struct {

    uint32_t    totalPackets_Cnt;       /**< Total packets counter. Incremented for each packet */
    uint32_t    preGreenPackets_Cnt;    /**< Precolored GREEN packets counter */
    uint32_t    finalGreenPackets_Cnt;  /**< Final GREEN packets counter. */
    uint32_t    preYelloPackets_Cnt;    /**< Precolored YELLOW packets counter + GREEN packets that did not conform with the committed bucket. */
    uint32_t    finalYelloPackets_Cnt;  /**< Final YELLOW packet count */
    uint32_t    finalRedPackets_Cnt;    /**< Final RED packet count */
 
} t_QeIwPlcrStatistics;


/**
 @Function      QE_IW_PLCR_GetStatistics

 @Description   Retrieves at runtime the statistics gathered for this Policer.

 @Param         h_Policer   - (In)  Handle to the Policer object.
 @Param         p_Stats     - (Out) Pointer to user-supplied structure where to place the statistics gathered

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error QE_IW_PLCR_GetStatistics(t_Handle h_Policer, t_QeIwPlcrStatistics *p_Stats);


/**
 @Function      QE_IW_PLCR_ResetStatistics

 @Description   Clear the special statistics counters.
 
                The Reset of the statistics counters is valid operation only if
                statistics gathering was configured as enabled prior to QE_IW_PLCR_Init call.
 
 @Param         h_Policer - (In)  Handle to the Policer object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_PLCR_Init().
*/
t_Error QE_IW_PLCR_ResetStatistics(t_Handle h_Policer);


/**
 @Function      QE_IW_PLCR_GetAppId

 @Description   Retrieves the App handle.

 @Param         h_Policer - (In)  Handle to the Policer object.
 @Param         h_AppId   - (Out) Pointer to user-supplied App handle.
 
 @Return        E_OK if success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_PLCR_Init().
*/
t_Error QE_IW_PLCR_GetAppId(t_Handle  h_Policer, t_Handle *h_AppId);


#if (DEBUG_ERRORS > 0)
/**
 @Function      QE_IW_PLCR_DumpRegs

 @Description   Dumps QE IW PLCR registers and parameter RAM.

 @Param         h_Policer - (In)  Handle to the Policer object.
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Policer must have been previously created using QE_IW_PLCR_Config()
                and initialized using QE_IW_PLCR_Init().
*/
t_Error QE_IW_PLCR_DumpRegs(t_Handle h_Policer);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of iw_plcr_control_grp group */
/** @} */ /* end of iw_plcr_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_PLCR_EXT_H */
