/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __QE_IW_QM_EXT_H
#define __QE_IW_QM_EXT_H




/**************************************************************************//**
 @Group          iw_grp Interworking Application Programming Interface

 @Description    Interworking API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group          iw_qm_grp IW QUEUE MANAGER Object Application Programming Interface

 @Description    IW QUEUE MANAGER Object API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         iw_qm_init_grp IW QUEUE MANAGER Object Initialization Unit

 @Description   IW QUEUE MANAGER Object API Initialization Unit

 @{
*//***************************************************************************/

/**
 @Description    Identifies the Queue Management algorithm used by the Queue Manager
*/
typedef enum {

    e_QE_IW_QM_WRED = 0    /**< The WRED algorithm is used by the Queue Manager */

} e_QeIwQmType;


/**
 @Description    User Parameters for configuring the WRED algorithm
*/
typedef struct {

    uint16_t    greenMinThreshold;      /**< Minimum Threshold in the WRED algorithm for Green packets */
    uint16_t    greenMaxThreshold;      /**< Maximum Threshold in the WRED algorithm for Green packets */

    uint16_t    yellowMinThreshold;     /**< Minimum Threshold in the WRED algorithm for Yellow packets */
    uint16_t    yellowMaxThreshold;     /**< Maximum Threshold in the WRED algorithm for Yellow packets */

    uint16_t    redMinThreshold;        /**< Minimum Threshold in the WRED algorithm for Red packets */
    uint16_t    redMaxThreshold;        /**< Maximum Threshold in the WRED algorithm for Red packets */

} t_QeIwQmWredParam;


/**
 @Description    User Parameters for configuring an Queue Manager
*/
typedef struct {

    t_Handle        h_AppId;        /**< Handle to the upper layer */
    t_Handle        h_Dev;          /**< Handle to the Dev object */
    e_QeIwQmType    qmType;         /**< Queue Management algorithm used by the Queue Manager */

    union _qm
    {
        t_QeIwQmWredParam    wredParam;  /**< Parameters for configuring the WRED algorithm */
    } qm;

} t_QeIwQmParam;


/**
 @Function      QE_IW_QM_Config

 @Description   Creates a new Queue Manager (QM) object and initializes its internal data
                with default values and with the values specified by the user in p_QmParam.

 @Param         p_QmParam     - (In) User parameters for QM.
 
 @Return        Handle to the new QM object if success.
 @Return        NULL in case of failure to create object or invalid parameters.
*/
t_Handle QE_IW_QM_Config(t_QeIwQmParam *p_QmParam);


/**
 @Function      QE_IW_QM_Init

 @Description   Initializes microcode structures associated with this Queue Manager (QM)
                based on already configured information.

 @Param         h_QueueManager  - (In) Handle to the QM object.

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      h_QueueManager must have been previously created using QE_IW_QM_Config().
*/
t_Error QE_IW_QM_Init(t_Handle  h_QueueManager);


/**
 @Function      QE_IW_QM_Free

 @Description   Frees the Queue Manager (QM) object.

 @Param         h_QueueManager        - (In) QM to be freed.
 
 @Return        E_OK if success.
 @Return        E_BUSY if there are referring connections.
*/
t_Error QE_IW_QM_Free(t_Handle  h_QueueManager);


/**************************************************************************//**
 @Group         iw_qm_init_advanced_grp IW QUEUE MANAGER Object Advanced Initialization Unit

 @Description   IW QUEUE MANAGER Object API Advanced Initialization Unit

 @{
*//***************************************************************************/

/**
 @Function      QE_IW_QM_ConfigWredGreenMaxDropProbability

 @Description   The packet drop probability is based on the minimum threshold,
                maximum threshold, and max drop probability.When the average
                queue depth is above the minimum threshold, WRED starts dropping packets.
                The rate of packet drop increases linearly as the average queue size
                increases until the average queue size reaches the maximum threshold.
                The max probability is the fraction of packets dropped when
                the average queue depth is at the maximum threshold. When the average queue size
                is above the maximum threshold, all packets are dropped.
 
                [100]

 @Param         h_QueueManager          - (In) Handle to the Queue Manager (QM) object.
 @Param         greenMaxDropProbability - (In) New value for the Green Exponential Weight Factor.

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_QM_Config() and before QE_IW_QM_Init().
*/
t_Error QE_IW_QM_ConfigWredGreenMaxDropProbability(t_Handle h_QueueManager, uint8_t greenMaxDropProbability);


/**
 @Function      QE_IW_QM_ConfigWredYellowMaxDropProbability

 @Description   The packet drop probability is based on the minimum threshold,
                maximum threshold, and max drop probability.When the average
                queue depth is above the minimum threshold, WRED starts dropping packets.
                The rate of packet drop increases linearly as the average queue size
                increases until the average queue size reaches the maximum threshold.
                The max probability is the fraction of packets dropped when
                the average queue depth is at the maximum threshold. When the average queue size
                is above the maximum threshold, all packets are dropped.
 
                [100]

 @Param         h_QueueManager           - (In) Handle to the Queue Manager (QM) object.
 @Param         yellowMaxDropProbability - (In) New value for the Yellow Exponential Weight Factor.

 @Return        E_OK if success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_QM_Config() and before QE_IW_QM_Init().
*/
t_Error QE_IW_QM_ConfigWredYellowMaxDropProbability(t_Handle h_QueueManager, uint8_t yellowMaxDropProbability);


/**
 @Function      QE_IW_QM_ConfigWredRedMaxDropProbability

 @Description   The packet drop probability is based on the minimum threshold,
                maximum threshold, and max drop probability.When the average
                queue depth is above the minimum threshold, WRED starts dropping packets.
                The rate of packet drop increases linearly as the average queue size
                increases until the average queue size reaches the maximum threshold.
                The max probability is the fraction of packets dropped when
                the average queue depth is at the maximum threshold. When the average queue size
                is above the maximum threshold, all packets are dropped.
 
                [100]

 @Param         h_QueueManager        - (In) Handle to the Queue Manager (QM) object.
 @Param         redMaxDropProbability - (In) New value for the Red Exponential Weight Factor.

 @Return        E_OK if    success; Error code otherwise.

 @Cautions      Allowed only following QE_IW_QM_Config() and before QE_IW_QM_Init().
*/
t_Error QE_IW_QM_ConfigWredRedMaxDropProbability(t_Handle h_QueueManager, uint8_t redMaxDropProbability);

/** @} */ /* end of iw_qm_init_advanced_grp group */
/** @} */ /* end of iw_qm_init_grp group */

/**************************************************************************//**
 @Group         iw_qm_control_grp IW QUEUE MANAGER Object Control Unit

 @Description   IW QUEUE MANAGER Object API Control Unit

 @{
*//***************************************************************************/

/**
@Description   QUEUE MANAGER Statistics
*/
typedef struct {

    uint32_t    greenDropPackets_Cnt;   /**< Incremented for each GREEN packet that was dropped. */
    uint32_t    yellowDropPackets_Cnt;  /**< Incremented for each YELLOW packet that was dropped. */
    uint32_t    redDropPackets_Cnt;     /**< Incremented for each RED packet that was dropped */
 
} t_QeIwQmStatistics;


/**
 @Function      QE_IW_QM_GetStatistics

 @Description   Retrieves at runtime the statistics gathered for this Policer.

 @Param         h_QueueManager  - (In)  Handle to the Policer object.
 @Param         p_Stats         - (Out) Pointer to user-supplied structure where to place the statistics gathered

 @Return        E_OK if success. Error code otherwise.
 
 @Cautions      Predictable results only when used after QE_IW_CDESC_Init().
*/
t_Error    QE_IW_QM_GetStatistics(t_Handle  h_QueueManager, t_QeIwQmStatistics *p_Stats);


/**
 @Function      QE_IW_QM_ResetStatistics

 @Description   Clear the special statistics counters.
 
                The Reset of the statistics counters is valid operation only if
                statistics gathering was configured as enabled prior to QE_IW_QM_Init call.

 @Param         h_QueueManager - (In)  Handle to the QM object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      Predictable results only when used after QE_IW_QM_Init().
*/
t_Error QE_IW_QM_ResetStatistics(t_Handle h_QueueManager);


/**
 @Function      QE_IW_QM_ModifyWredGreenMaxDropProbability

 @Description   Modifies the value of the WRED Queue Manager (QM) Green Maximum Drop Probability.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         greenMaxDropProbability - (In)  New value for the Green Exponential Weight Factor.

 @Return        E_OK if success. Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredGreenMaxDropProbability(t_Handle  h_QueueManager, uint8_t greenMaxDropProbability);


/**
 @Function      QE_IW_QM_ModifyWredYellowMaxDropProbability

 @Description   Modifies the value of the WRED Queue Manager (QM) Yellow Maximum Drop Probability.

 @Param         h_QueueManager           - (In)  Handle to the QM object.
 @Param         yellowMaxDropProbability - (In)  New value for the Yellow Exponential Weight Factor.

 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredYellowMaxDropProbability(t_Handle h_QueueManager, uint8_t yellowMaxDropProbability);


/**
 @Function      QE_IW_QM_ModifyWredRedMaxDropProbability

 @Description   Modifies the value of the WRED Queue Manager (QM) Red Maximum Drop Probability.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         redMaxDropProbability   - (In)  New value for the Red Exponential Weight Factor.
 
 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredRedMaxDropProbability(t_Handle h_QueueManager, uint8_t redMaxDropProbability);


/**
 @Function      QE_IW_QM_ModifyWredGreenMinThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Green Minimum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         greenMinThreshold       - (In)  New value for the Green Minimum Threshold.
 
 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredGreenMinThreshold(t_Handle h_QueueManager, uint16_t greenMinThreshold);


/**
 @Function      QE_IW_QM_ModifyWredGreenMaxThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Green Maximum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         greenMaxThreshold       - (In)  New value for the Green Maximum Threshold.
 
 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredGreenMaxThreshold(t_Handle h_QueueManager, uint16_t greenMaxThreshold);


/**
 @Function      QE_IW_QM_ModifyWredYellowMinThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Yellow Minimum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         yellowMinThreshold      - (In)  New value for the Yellow Minimum Threshold.

 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredYellowMinThreshold(t_Handle h_QueueManager, uint16_t yellowMinThreshold);


/**
 @Function      QE_IW_QM_ModifyWredYellowMaxThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Yellow Maximum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         yellowMaxThreshold      - (In)  New value for the Yellow Maximum Threshold.

 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredYellowMaxThreshold(t_Handle h_QueueManager, uint16_t yellowMaxThreshold);


/**
 @Function      QE_IW_QM_ModifyWredRedMinThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Red Minimum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         redMinThreshold         - (In)  New value for the Red Minimum Threshold.

 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredRedMinThreshold(t_Handle h_QueueManager, uint16_t redMinThreshold);


/**
 @Function      QE_IW_QM_ModifyWredRedMaxThreshold

 @Description   Modifies the value of the WRED Queue Manager (QM) Red Maximum Threshold.

 @Param         h_QueueManager          - (In)  Handle to the QM object.
 @Param         redMaxThreshold         - (In)  New value for the Red Maximum Threshold.

 @Return        E_OK if success; Error code otherwise.
*/
t_Error QE_IW_QM_ModifyWredRedMaxThreshold(t_Handle  h_QueueManager, uint16_t redMaxThreshold);


/**
 @Function      QE_IW_QM_GetAppId

 @Description   Retrieves the App handle.

 @Param         h_QueueManager          - (In)    Handle to the QM object ,
 @Param         h_AppId                 - (Out)    Pointer to user-supplied App handle
 
 @Return        E_OK if success. Error code otherwise.
*/
t_Error    QE_IW_QM_GetAppId (t_Handle h_QueueManager, t_Handle *h_AppId);


#if (DEBUG_ERRORS > 0)
/**
 @Function      QE_IW_QM_DumpRegs

 @Description   Dumps QE IW QM registers and parameter RAM.

 @Param         h_QueueManager - (In) Handle to the QM object
 
 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_QueueManager must have been previously created using QE_IW_QM_Config()
                and initialized using QE_IW_QM_Init().
*/
t_Error QE_IW_QM_DumpRegs(t_Handle h_QueueManager);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of iw_qm_control_grp group */
/** @} */ /* end of iw_qm_grp group */
/** @} */ /* end of iw_grp group */

#endif /* __QE_IW_QM_EXT_H */
