/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          qe_rtc_ext.h

 @Description   External definitions and API for QE RTC.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __QE_RTC_EXT_H__
#define __QE_RTC_EXT_H__


#include "qe_ext.h"


/**************************************************************************//**
 @Group         qe_rtc_grp QE RTC Application Programming Interface

 @Description   QE RTC functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         qe_rtc_init_grp QE RTC Initialization Unit

 @Description   QE RTC initialization API.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   QE RTC Alarm ID.
*//***************************************************************************/
typedef enum e_QeRtcAlarmId
{
    e_QE_RTC_ALARM_1 = 0,       /**< Alarm signal 1 */
    e_QE_RTC_ALARM_2,           /**< Alarm signal 2 */
    e_QE_RTC_ALARM_DUMMY_LAST
} e_QeRtcAlarmId;

#define QE_RTC_NUM_OF_ALARMS    e_QE_RTC_ALARM_DUMMY_LAST
                                /**< Total number of QE RTC alarm signals */

/**************************************************************************//**
 @Description   QE RTC Periodic Pulse ID.
*//***************************************************************************/
typedef enum e_QeRtcPulseId
{
    e_QE_RTC_PULSE_1 = 0,       /**< Periodic pulse 1 */
    e_QE_RTC_PULSE_2,           /**< Periodic pulse 2 */
    e_QE_RTC_PULSE_3,           /**< Periodic pulse 3 */
    e_QE_RTC_PULSE_DUMMY_LAST
} e_QeRtcPulseId;

#define QE_RTC_NUM_OF_PULSES    e_QE_RTC_PULSE_DUMMY_LAST
                                /**< Total number of QE RTC periodic pulses */

/**************************************************************************//**
 @Description   QE RTC External Trigger ID.
*//***************************************************************************/
typedef enum e_QeRtcTriggerId
{
    e_QE_RTC_TRIGGER_1 = 0,     /**< External trigger 1 */
    e_QE_RTC_TRIGGER_2,         /**< External trigger 2 */
    e_QE_RTC_TRIGGER_DUMMY_LAST
} e_QeRtcTriggerId;

#define QE_RTC_NUM_OF_TRIGGERS  e_QE_RTC_TRIGGER_DUMMY_LAST
                                /**< Total number of QE RTC external triggers */


/**************************************************************************//**
 @Description   QE RTC Alarm Polarity Options.
*//***************************************************************************/
typedef enum e_QeRtcAlarmPolarity
{
    e_QE_RTC_ALARM_POLARITY_ACTIVE_HIGH,    /**< Active-high output polarity */
    e_QE_RTC_ALARM_POLARITY_ACTIVE_LOW      /**< Active-low output polarity */
} e_QeRtcAlarmPolarity;

/**************************************************************************//**
 @Description   QE RTC Trigger Polarity Options.
*//***************************************************************************/
typedef enum e_QeRtcTriggerPolarity
{
    e_QE_RTC_TRIGGER_ON_RISING_EDGE,    /**< Trigger on rising edge */
    e_QE_RTC_TRIGGER_ON_FALLING_EDGE    /**< Trigger on falling edge */
} e_QeRtcTriggerPolarity;

#ifndef PTP_V2
/**************************************************************************//**
 @Description   QE RTC Periodic Pulse Start Mode.
*//***************************************************************************/
typedef enum e_QeRtcPulseStartMode
{
    e_QE_RTC_PULSE_START_AUTO,      /**< Start pulse when RTC is enabled */
    e_QE_RTC_PULSE_START_ON_ALARM   /**< Start pulse on alarm 1 event */
} e_QeRtcPulseStartMode;

#else /* PTP_V2 */

/**************************************************************************//**
 @Description   QE RTC Periodic Pulse width.
*//***************************************************************************/
typedef enum e_QeRtcPulseWidth
{
    e_QE_RTC_PULSE_WIDTH_INTERNAL_TIMER,          /**< signal width is identical to the width selected timer clock */
    e_QE_RTC_PULSE_WIDTH_OUTPUT_CLK               /**< signal width is identical to the width of the IEEE1588 timer
                                                        reference output clock */
} e_QeRtcPulseWidth;
#endif  /* PTP_V2 */


/**************************************************************************//**
 @Description   QE RTC events callback routine type.
*//***************************************************************************/
typedef void (t_QeRtcEventsCallBack)(t_Handle h_App, uint32_t events, uint32_t flags);


/**************************************************************************//**
 @Description   QE RTC configuration parameters structure.
 
                This structure should be passed to QE_RTC_Config().
*//***************************************************************************/
typedef struct t_QeRtcParam
{
    /* QE RTC basic parameters */
    uint32_t                rtcFreqHz;      /**< RTC target frequency in Hz */

    /* System integration parameters */
    uint32_t                baseAddress;    /**< Base address of QE RTC registers in memory */
    t_Handle                h_Qe;           /**< Handle to QE object */
    int                     irq;            /**< QE RTC interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle                h_App;          /**< Application ID */
    t_QeRtcEventsCallBack   *f_Events;      /**< RTC events callback routine */

} t_QeRtcParam;


/**************************************************************************//**
 @Function      QE_RTC_Config

 @Description   Configures the QE RTC module according to user's parameters.
 
                The driver assigns default values to some QE RTC parameters.
                These parameters can be overwritten using the advanced
                configuration routines.

 @Param[in]     p_QeRtcParam - QE RTC configuration parameters.
 
 @Return        Handle to the new QE RTC object; NULL on failure.
*//***************************************************************************/
t_Handle QE_RTC_Config(t_QeRtcParam *p_QeRtcParam);

/**************************************************************************//**
 @Function      QE_RTC_Init

 @Description   Initializes the QE RTC driver and hardware.

 @Param[in]     h_QeRtc - Handle to QE RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_Init(t_Handle h_QeRtc);

/**************************************************************************//**
 @Function      QE_RTC_Free

 @Description   Frees the QE RTC object and all allocated resources.

 @Param[in]     h_QeRtc - Handle to QE RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_Free(t_Handle h_QeRtc);


/**************************************************************************//**
 @Group         qe_rtc_adv_config_grp  QE RTC Advanced Configuration Unit

 @Description   QE RTC advanced configuration functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_RTC_ConfigSourceClock

 @Description   Configures the source clock of the RTC. [Half QE clock]
 
                The source clock can be selected from the QUICC Engine's bank
                of clocks (note that not all options are valid).
 
                If this routine is not used, the source clock of the RTC shall
                be half of the QUICC Engine's clock.
 
 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     srcClock        - Source clock selection.
 @Param[in]     srcClockFreqHz  - Source clock frequency in Hz.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigSourceClock(t_Handle   h_QeRtc,
                                 e_QeClock  srcClock,
                                 uint32_t   srcClockFreqHz);

/**************************************************************************//**
 @Function      QE_RTC_ConfigFrequencyCompensationBypass

 @Description   Configures the RTC to bypass the frequency compensation
                mechanism. [FALSE]
 
                In this mode, the RTC clock is identical to the source clock.
                This feature can be useful when the system contains an external
                RTC with inherent frequency compensation.
 
 @Param[in]     h_QeRtc - Handle to QE RTC object.
 @Param[in]     enabled - TRUE to bypass frequency compensation;
                          FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigFrequencyCompensationBypass(t_Handle h_QeRtc, bool enabled);

/**************************************************************************//**
 @Function      QE_RTC_ConfigInvertedInputClockPhase

 @Description   Configures the RTC to invert the source clock phase on input.
                [FALSE]
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     inverted    - TRUE to invert the source clock phase on input.
                              FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigInvertedInputClockPhase(t_Handle h_QeRtc, bool inverted);

/**************************************************************************//**
 @Function      QE_RTC_ConfigInvertedOutputClockPhase

 @Description   Configures the RTC to invert the output clock phase.
                [FALSE]

 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     inverted    - TRUE to invert the output clock phase.
                              FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigInvertedOutputClockPhase(t_Handle h_QeRtc, bool inverted);

/**************************************************************************//**
 @Function      QE_RTC_ConfigOutputClockDivisor

 @Description   Configures the divisor for generating the output clock from
                the RTC clock. [0x0001]

 @Param[in]     h_QeRtc - Handle to QE RTC object.
 @Param[in]     divisor - Divisor for generation of the output clock.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigOutputClockDivisor(t_Handle h_QeRtc, uint16_t divisor);

/**************************************************************************//**
 @Function      QE_RTC_ConfigAlarmPolarity

 @Description   Configures the polarity (active-high/active-low) of a specific
                alarm signal. [e_QE_RTC_ALARM_POLARITY_ACTIVE_HIGH]

 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     alarmId         - Alarm ID.
 @Param[in]     alarmPolarity   - Alarm polarity.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigAlarmPolarity(t_Handle             h_QeRtc,
                                   e_QeRtcAlarmId       alarmId,
                                   e_QeRtcAlarmPolarity alarmPolarity);

/**************************************************************************//**
 @Function      QE_RTC_ConfigExternalTriggerPolarity

 @Description   Configures the polarity (rising/falling edge) of a specific
                external trigger signal. [e_QE_RTC_TRIGGER_ON_RISING_EDGE]

 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     triggerId       - Trigger ID.
 @Param[in]     triggerPolarity - Trigger polarity.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigExternalTriggerPolarity(t_Handle               h_QeRtc,
                                             e_QeRtcTriggerId       triggerId,
                                             e_QeRtcTriggerPolarity triggerPolarity);

#ifdef PTP_V2
/**************************************************************************//**
 @Function      E_RTC_ConfigPeriodicPulseWidth

 @Description   Configures the Pulse Per Second signal width. [e_QE_RTC_TRIGGER_ON_RISING_EDGE]

 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     pulseWidth      - two options :1.signal width is identical to the width selected timer clock
                                               2.signal width is identical to the width of the IEEE1588 timer
                                                 reference output clock

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigPeriodicPulseWidth(t_Handle                h_QeRtc,
                                        e_QeRtcPulseWidth       pulseWidth);

/**************************************************************************//**
 @Function      QE_RTC_ConfigExternalTriggerPolarity

 @Description   Configures for each external triger if we want to use debug loop
 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     triggerId       - Trigger ID

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_ConfigExternalTriggerDebug(t_Handle                h_QeRtc,
                                          e_QeRtcTriggerId        triggerId);
 
/**************************************************************************//**
 @Function      QE_RTC_ConfigPeriodicPulseStartTime

 @Description   configures to the three pulse per second when to start the down counting
                time to be configured is the RTC counter value,not relative time to the time of configuration
 @Param[in]     h_QeRtc         - Handle to QE RTC object.
 @Param[in]     startTime       - value in nano sec

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously created using QE_RTC_Config().
*//***************************************************************************/
t_Error QE_RTC_SetPeriodicPulseStartTime(t_Handle         h_QeRtc,
                                         uint64_t         startTime);
#endif /*PTP_V2*/
/** @} */ /* end of qe_rtc_adv_config_grp */
/** @} */ /* end of qe_rtc_init_grp */


/**************************************************************************//**
 @Group         qe_rtc_control_grp QE RTC Control Unit

 @Description   QE RTC runtime control API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QE_RTC_Enable

 @Description   Enables the RTC (time count is started).
 
                The user can select to resume the time count from previous
                point, or to restart the time count.
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     resetClock  - Restart the time count from zero.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_Enable(t_Handle h_QeRtc, bool resetClock);

/**************************************************************************//**
 @Function      QE_RTC_Disable

 @Description   Disables the RTC (time count is stopped).
 
 @Param[in]     h_QeRtc - Handle to QE RTC object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_Disable(t_Handle h_QeRtc);

/**************************************************************************//**
 @Function      QE_RTC_ModifyFrequencyCompensation

 @Description   Modifies the frequency compensation value in runtime.
 
 @Param[in]     h_QeRtc             - Handle to QE RTC object.
 @Param[in]     freqCompensation    - New frequency compensation value.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_ModifyFrequencyCompensation(t_Handle h_QeRtc,
                                           uint32_t freqCompensation);

/**************************************************************************//**
 @Function      QE_RTC_ModifyEventsMask

 @Description   Modifies the RTC events mask in runtime. [QE_RTC_EVENT_ALL]
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     eventsMask  - New events mask. This value should contain
                              any combination of the QE_RTC_EVENT_xxx flags.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().

 @See           @ref qe_rtc_events "QE RTC Events".
*//***************************************************************************/
t_Error QE_RTC_ModifyEventsMask(t_Handle h_QeRtc, uint32_t eventsMask);

/**************************************************************************//**
 @Function      QE_RTC_SetClockOffset

 @Description   Sets the clock offset (usually relative to another clock).
 
                The user can pass a negative offset value.
 
 @Param[in]     h_QeRtc - Handle to QE RTC object.
 @Param[in]     offset  - New clock offset (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_SetClockOffset(t_Handle h_QeRtc, int64_t offset);

/**************************************************************************//**
 @Function      QE_RTC_SetAlarm

 @Description   Schedules an alarm event to a given RTC time.
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     alarmId     - Alarm ID.
 @Param[in]     alarmTime   - Requested alarm time (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_SetAlarm(t_Handle        h_QeRtc,
                        e_QeRtcAlarmId  alarmId,
                        uint64_t        alarmTime);

/**************************************************************************//**
 @Function      QE_RTC_SetPeriodicPulse

 @Description   Sets a periodic pulse.
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     pulseId     - Periodic pulse ID.
 @Param[in]     pulsePeriod - Requested pulse period (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
                if used after RTC_Enable in order for the pulses to be alignd with
                the output clock, QE_RTC_SetPeriodicPulseStartTime() or QE_RTC_SetAlarm() (ptp v2/v1)
                should be called with a bigger value than the current RTC clock after this function.
*//***************************************************************************/
t_Error QE_RTC_SetPeriodicPulse(t_Handle        h_QeRtc,
                                e_QeRtcPulseId  pulseId,
                                uint32_t        pulsePeriod);

/**************************************************************************//**
 @Function      QE_RTC_GetExternalTriggerTimeStamp

 @Description   Gets the last timestamp captured due to an external trigger.
 
 @Param[in]     h_QeRtc     - Handle to QE RTC object.
 @Param[in]     triggerId   - Trigger ID.
 @Param[out]    p_TimeStamp - The requested timestamp (in nanoseconds).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
t_Error QE_RTC_GetExternalTriggerTimeStamp(t_Handle         h_QeRtc,
                                           e_QeRtcTriggerId triggerId,
                                           uint64_t         *p_TimeStamp);

/**************************************************************************//**
 @Function      QE_RTC_GetCurrentTime

 @Description   Returns the current RTC time.
 
 @Param[in]     h_QeRtc - Handle to QE RTC object.

 @Return        The current RTC time (in nanoseconds).
 
 @Cautions      h_QeRtc must have been previously initialized using QE_RTC_Init().
*//***************************************************************************/
uint64_t QE_RTC_GetCurrentTime(t_Handle h_QeRtc);



/** @} */ /* end of qe_rtc_control_grp */


/**************************************************************************//**
 @Group         qe_rtc_exceptions_grp QE RTC Exceptions Unit

 @Description   QE RTC exceptions API.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        qe_rtc_events
 
 @Collection    QE RTC Events
 
                Exceptions and events reported through the QE RTC events
                callback routine. These events can be masked/unmasked using
                the QE_RTC_ConfigEventsMask() routine.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define QE_RTC_EVENT_EXT_TRIGGER_2_TS  0x02000000  /**< External trigger 2 time stamp ready;
                                                         Call QE_RTC_GetExternalTriggerTimeStamp()
                                                         to get the time stamp */
#define QE_RTC_EVENT_EXT_TRIGGER_1_TS  0x01000000  /**< External trigger 1 time stamp ready;
                                                         Call QE_RTC_GetExternalTriggerTimeStamp()
                                                         to get the time stamp */
#define QE_RTC_EVENT_PERIODIC_PULSE_1  0x00000080  /**< Periodic pulse 1 generated */
#define QE_RTC_EVENT_PERIODIC_PULSE_2  0x00000040  /**< Periodic pulse 2 generated */
#define QE_RTC_EVENT_PERIODIC_PULSE_3  0x00000020  /**< Periodic pulse 3 generated */
#ifdef PTP_V2
#define QE_RTC_EVENT_ALARM_2           0x00010000  /**< Alarm 2 time has been reached */
#define QE_RTC_EVENT_ALARM_1           0x00020000  /**< Alarm 1 time has been reached */
#define QE_RTC_EVENT_FIPER_START       0x00008000  /**< Current time equaled the FIPER Start Value (FSV) */
#define QE_RTC_EVENT_ALL \
        (QE_RTC_EVENT_EXT_TRIGGER_2_TS | QE_RTC_EVENT_EXT_TRIGGER_1_TS | \
         QE_RTC_EVENT_ALARM_2 | QE_RTC_EVENT_ALARM_1 | QE_RTC_EVENT_PERIODIC_PULSE_1 | \
         QE_RTC_EVENT_PERIODIC_PULSE_2 | QE_RTC_EVENT_PERIODIC_PULSE_3 | QE_RTC_EVENT_FIPER_START )
                                                     /**< All events */
#else  /* not PTP_V2*/
#define QE_RTC_EVENT_ALARM_2           0x00020000  /**< Alarm 2 time has been reached */
#define QE_RTC_EVENT_ALARM_1           0x00010000  /**< Alarm 1 time has been reached */

#define QE_RTC_EVENT_ALL \
        (QE_RTC_EVENT_EXT_TRIGGER_2_TS | QE_RTC_EVENT_EXT_TRIGGER_1_TS | \
         QE_RTC_EVENT_ALARM_2 | QE_RTC_EVENT_ALARM_1 | QE_RTC_EVENT_PERIODIC_PULSE_1 | \
         QE_RTC_EVENT_PERIODIC_PULSE_2 | QE_RTC_EVENT_PERIODIC_PULSE_3)
                                                    /**< All events */
#endif /*PTP_V2*/
/* @} */

/** @} */ /* end of qe_rtc_exceptions_grp */
/** @} */ /* end of qe_rtc_grp */


#endif /* __QE_RTC_EXT_H__ */
