/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          qe_tdm_ext.h

 @Description   TDM Application Programming Interface
*//***************************************************************************/

#ifndef __QE_TDM_EXT
#define __QE_TDM_EXT


#include "std_ext.h"
#include "qe_ext.h"


/**************************************************************************//**
 @Group         tdm_grp TDM Application Programming Interface

 @Description   TDM API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         si_grp SI Application Programming Interface

 @Description   SI API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         si_init_grp SI Initialization Unit

 @Description   SI initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SI initialization structure, used when calling SI_Init.
*//***************************************************************************/
typedef struct
{
    t_Handle    h_Qe;   /**< A handle of the QE device */
} t_SiParam;

/**************************************************************************//**
 @Function      SI_Config

 @Description   General initialization of the SI.

 @Param         siBaseAddress    - (In) The base address of the SI memory map.
 @Param         siRamBaseAddress - (In) The base address of the SI RAM memory map.
 @Param         p_SiParam        - (In) a handle of the SI initializing
                                        parameter structure as given by the user.

 @Return        p_Handle  - (In) a handle to the SI structure, NULL on failure.
*//***************************************************************************/
t_Handle    SI_Config(uint32_t siBaseAddress, uint32_t siRamBaseAddress, t_SiParam *p_SiParam);

/**************************************************************************//**
 @Function      SI_Init

 @Description   General initialization of the SI.

 @Param         h_Si  - (In) a handle to the SI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     SI_Init(t_Handle h_Si);

/**************************************************************************//**
 @Function      SI_Free

 @Description   Delete an SI.

 @Param         h_Si  - (In) a handle to the SI object.
*//***************************************************************************/
void        SI_Free(t_Handle h_Si);

/** @} */ /* end of si_init_grp group */


/**************************************************************************//**
 @Group         si_control_grp SI Control Unit

 @Description   SI control unit API functions, definitions and enums

 @{
*//***************************************************************************/

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      SI_DumpRegs

 @Description   Dump (print) all SI registers.

 @Param         h_Si  - (In) a handle to the SI object.
*//***************************************************************************/
void        SI_DumpRegs(t_Handle h_Si);
#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of si_control_grp group */
/** @} */ /* end of si_grp group */


/**************************************************************************//**
 @Group         tdm_init_grp TDM Initialization Unit

 @Description   TDM initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description  TDM diagnostic modes
*//***************************************************************************/
typedef enum{
    e_TDM_SIMR_DIAG_NORMAL   = 0x0000     /**< Normal mode */
    ,e_TDM_SIMR_DIAG_ECHO    = 0x0400     /**< Echo mode */
    ,e_TDM_SIMR_DIAG_INT_LPB = 0x0800     /**< Internal loop mode */
    ,e_TDM_SIMR_DIAG_LPB     = 0x0c00     /**< Loopback mode */
} e_TdmDiag;


#define TDM_MAX_NUM_OF_SLOTS    512

#define TDM_NUM_OF_STROBES      8


/**************************************************************************//**
 @Description   TDM initialization structure, used when calling TDM_Init.
*//***************************************************************************/
typedef struct
{
    t_Handle    h_Si;           /**< a Handle to the SI structure. */
    e_Tdm       tdmId;          /**< TDM num, of type e_Tdm */
    e_QeClock   rxClock;        /**< one of the clocks available for this TDM. */
    e_QeClock   txClock;        /**< one of the clocks available for this TDM. */
    uint16_t    maxNumOfSlots;  /**< Max number of slots for this TDM */
 
} t_TdmParam;


/**************************************************************************//**
 @Description  TDM Slot configuration structure for MCC
*//***************************************************************************/
typedef struct
{
    bool     loopOrEcho;       /**< TRUE: at the transmitter-echo, at the receiver-loopback */
    bool     isSuper;          /**< TRUE if this is a super channel                         */
    uint8_t  channelNum;       /**< MCC channel number                                      */
    bool     isFirst;          /**< relevant only for super channel, properly configure
                                     for transparent, must be cleared for HDLC              */
} t_TdmSlotMccParam;

/**************************************************************************//**
 @Description   TDM Slot configuration structure for UCC
*//***************************************************************************/
typedef struct
{
    bool    switchTxRx;  /**< TRUE to switch Tx and Rx pins */
    e_Ucc   uccId;       /**< Ucc ID */
    bool    strobeSelect[TDM_NUM_OF_STROBES];
                         /**< strobe select: TRUE for a chosen strobe to be asserted.
                              up to 4 strobes and must be from the same group (1-4 or 5-8) */
} t_TdmSlotUccParam;


/**************************************************************************//**
 @Description   Slot route structure, used when calling TDM_SLOT_Route.
*//***************************************************************************/
typedef struct
{
    bool    isMcc;                      /**< TRUE if the serial is MCC.                       */
    union
    {
        t_TdmSlotMccParam    mccParam;  /**< this slot is an MCC slot                         */
        t_TdmSlotUccParam    uccParam;  /**< this slot is not an MCC slot                     */
    }       controllerParams;           /**< A union for choosing between MCC or UCC TDM Slot */
} t_TdmSlotRouteParam;


/**************************************************************************//**
 @Description   Slot creating structure, used when calling TDM_FRAME_Config.
*//***************************************************************************/
typedef struct
{
    t_Handle        h_Tdm;          /**< a handle to the TDM of this slot                     */
    e_CommMode      dir;            /**< the TDM direction - Rx, Tx or both                   */
    uint16_t        numOfSlots;     /**< number of slots in h_Slots[]                         */
} t_TdmFrameParam;


/**************************************************************************//**
 @Function      TDM_Config

 @Description   General initialization of the TDM.

 @Param         p_TdmParam - (In) a handle of the TDM initializing
                parameter structure as given by the user.

 @Return        A handle to the TDM structure, NULL on failure.
*//***************************************************************************/
t_Handle      TDM_Config(t_TdmParam *p_TdmParam);

/**************************************************************************//**
 @Function      TDM_Init

 @Description   General initialization of the TDM.

 @Param         h_Tdm  - (In) a handle to the TDM object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error       TDM_Init(t_Handle h_Tdm);

/**************************************************************************//**
 @Function      TDM_Free

 @Description   Delete a TDM.

 @Param         h_Tdm  - (In) a handle to the TDM object.

 @Return        None.
*//***************************************************************************/
void          TDM_Free(t_Handle h_Tdm);


/**************************************************************************//**
 @Group         tdm_advanced_init_grp TDM Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      TDM_ConfigUseShadow

 @Description   Configure the TDM with 'new val' - TRUE for using shadow for dynamic routing.[TRUE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigUseShadow (t_Handle h_Tdm, bool newVal);

/**************************************************************************//**
 @Function      TDM_ConfigResetOnDisable

 @Description   Configure the TDM with 'new val' - TRUE for going back to the first RAM (even after running from shadow RAM)
                when TDM is disabled.[FALSE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigResetOnDisable (t_Handle h_Tdm, bool newVal);

/**************************************************************************//**
 @Function      TDM_ConfigRxFrameSyncDelay

 @Description   Configure the TDM with 'new val' - Receive frame sync delay, 0- no bit delay,x- x
                bit delay (x=1,2,3) [0]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigRxFrameSyncDelay (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigSyncLevel

 @Description   Configure the TDM with 'new val' - sync level - when '0' L1RSYNC and L1TSYNC signals are
                active on logic '1', otherwise on logic '0'[0].

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigSyncLevel (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigClockEdge

 @Description   Configure the TDM with 'new val' - Clock edge - when delayedSyncSample=0,0- data transmitted
                on rising edge, received on falling, 1- opposite, When
                delayedSyncSample=1, opposite.
                [0].

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigClockEdge (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigFrameSyncEdge

 @Description   Configure the TDM with 'new val' - Frame sync edge -  1 for rising.[0]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigFrameSyncEdge (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigGrantMode

 @Description   Configure the TDM with 'new val' - Grant mode - 0 for GCI/SCIT,1 for IDL.[0]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigGrantMode (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigTxFrameSyncDelay

 @Description   Configure the TDM with 'new val' - Transmit frame sync delay, 0- no bit delay, x- x bit delay
                (x=1,2,3).[0]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigTxFrameSyncDelay (t_Handle h_Tdm, uint8_t newVal);

/**************************************************************************//**
 @Function      TDM_ConfigLowL1TXDSignal

 @Description   Configure the TDM with 'new val' - TRUE for setting L1TXDx to 0.[FALSE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigLowL1TXDSignal (t_Handle h_Tdm, bool newVal);

/**************************************************************************//**
 @Function      TDM_ConfigDelayedSyncSample

 @Description   Configure the TDM with 'new val' - 1 for delayed sync sample.
                This routine is part of QE_SI2 errata work around.
                [FALSE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigDelayedSyncSample (t_Handle h_Tdm, bool newVal);

/**************************************************************************//**
 @Function      TDM_ConfigRxIntSync

 @Description   Configure the TDM with 'new val' - if not 0, use this clock for TDM Rx Sync.[e_QE_CLK_NONE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigRxIntSync (t_Handle h_Tdm, e_QeClock newVal);

/**************************************************************************//**
 @Function      TDM_ConfigTxIntSync

 @Description   Configure the TDM with 'new val' - if not 0, use this clock for TDM Rx Sync.[e_QE_CLK_NONE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigTxIntSync (t_Handle h_Tdm, e_QeClock newVal);

/**************************************************************************//**
 @Function      TDM_ConfigEhancedDiag

 @Description   Configure the TDM with 'new val' - if TRUE, loop/echo feature is available for all UCC entries
                (instead of switchRx/Tx), and  switchRx/Tx is available for all
                MCC channels entries (instead of loop/echo).[FALSE]

 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigEhancedDiag (t_Handle h_Tdm, bool newVal);

/**************************************************************************//**
 @Function      TDM_ConfigDiagMode

 @Description   Configure the TDM with 'new val' - e_TdmDiag: Nornal, echo, internal loop back or loop back[e_TDM_SIMR_DIAG_NORMAL]
 
 @Param         h_Tdm - (In) handle to TDM
 @Param         newVal    - (In) new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigDiagMode (t_Handle h_Tdm, e_TdmDiag newVal);

/**************************************************************************//**
 @Function      TDM_ConfigNonCommonTxRx

 @Description   Force the TDM not to use common receive and transmit pins.
                When not called, given the same clock for Rx and Tx the TDM will do as followed:
                RCLK will be used as the Rx/Tx clocks and RSYNC will be used as Rx/Tx SYNC.
 
 @Param         h_Tdm   - (In) handle to TDM

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigNonCommonTxRx (t_Handle h_Tdm);

#ifdef TDM_HIGH_SPEED_MODE_SUPPORT
/**************************************************************************//**
 @Function      TDM_ConfigHighSpeedMode

 @Description   Configures high-speed mode.
 
                Configures the frequency ratio between the serial clock and the
                Quick Engine clock.
                If enabled, the min frequency ratio between the serial clock and
                the Quick Engine clock is 1:8 and the max frequency ratio is 1:16.
                Only UCC entries are allowed.
                If disabled, the min frequency ratio between the serial clock and
                the Quick Engine clock is 1:16.
                [FALSE]
 
 @Param         h_Tdm   - (In) handle to TDM
 @Param         newVal  - (In) new value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error TDM_ConfigHighSpeedMode (t_Handle h_Tdm, bool newVal);
#endif /* TDM_HIGH_SPEED_MODE_SUPPORT */

/** @} */ /* end of TDM_advanced_init_grp group */

/** @} */ /* end of TDM_init_grp group */


/**************************************************************************//**
 @Group         tdm_control_grp TDM Control Unit

 @Description   TDM control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      TDM_DisableLowL1TXDSignal

 @Description   This routine is relevant only if LowL1TXDSignal was enabled
                (by calling TDM_ConfigLowL1TXDSignal with parameter - TRUE).
                It must be called before first transmission attempt.

 @Param         h_Tdm  -          (In) a handle to the TDM object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Must be called before first transmission attempt.
*//***************************************************************************/
t_Error    TDM_DisableLowL1TXDSignal(t_Handle h_Tdm);

/**************************************************************************//**
 @Function      TDM_Enable

 @Description   Enables a TDM channel

 @Param         h_Tdm  - (In) a handle to the TDM object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  TDM_Enable(t_Handle h_Tdm);

/**************************************************************************//**
 @Function      TDM_Disable

 @Description   Disables a TDM channel.

 @Param         h_Tdm  - (In) a handle to the TDM object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The user must set a frame prior to enabling the TDM.
*//***************************************************************************/
t_Error  TDM_Disable(t_Handle h_Tdm);

/**************************************************************************//**
 @Function      TDM_SwitchFrame

 @Description   Switch between the frames of the TDM (simulate SW MultiFrame).

 @Param         h_Tdm  - (In) a handle to the TDM object.
 @Param         dir    - (In) the TDM direction - Rx, Tx or both.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      In order to switch frames, the TDM must have two different frames
                set on it.
*//***************************************************************************/
t_Error  TDM_SwitchFrame(t_Handle h_Tdm, e_CommMode dir);

bool     TDM_HasShadow  (t_Handle h_Tdm);

bool     TDM_IsEnabled  (t_Handle h_Tdm);

e_Tdm    TDM_GetTdmId   (t_Handle h_Tdm);


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**************************************************************************//**
 @Function      TDM_DumpRegs

 @Description   Dump (print) all TDM registers.

 @Param         h_Tdm  - (In) a handle to the TDM object.
*//***************************************************************************/
void TDM_DumpRegs(t_Handle h_Tdm);
#endif /* (defined(DEBUG_ERRORS) && ... */

/**************************************************************************//**
 @Function      TDM_FRAME_Config

 @Description   Create a frame handle from a given list of slots parameters.
                The slots are set continuesly as disabled. The last slot has
                its entry set as last.

 @Param         p_TdmFrameParam  - (In) a handle to a structure of parameters needed to define the frame.
 
 @Return        A handle to the TDM_FRAME structure, NULL on failure.
 *//***************************************************************************/
t_Handle  TDM_FRAME_Config(t_TdmFrameParam *p_TdmFrameParam);

/**************************************************************************//**
 @Function      TDM_FRAME_ConfigSlotsSizes

 @Description   Change the sizes of the Freame's slots to the given values.

 @Param         h_TdmFrame  -   (In) a handle to the Frame object.
 @Param         p_SlotsSize -   (In) an array of the slots sizes in bits.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      This change of configuration is allowed only before routing.
 *//***************************************************************************/
t_Error  TDM_FRAME_ConfigSlotsSizes(t_Handle h_TdmFrame, uint8_t *p_SlotsSize);

/**************************************************************************//**
 @Function      TDM_FRAME_SlotRoute

 @Description   Change the route of one slot

 @Param         h_TdmFrame  - (In) a handle to the Frame object.
 @Param         index  - (In) the slot location in the frame.
 @Param         p_TdmSlotRouteParam  - (In) a handle to a structure of parameters
                                             describing the new route of the slot
 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error    TDM_FRAME_SlotRoute(t_Handle h_TdmFrame,uint32_t index,t_TdmSlotRouteParam *p_TdmSlotRouteParam);

/**************************************************************************//**
 @Function      TDM_FRAME_ChangeSlotStatus

 @Description   Change the status of one slot, i.e. if enable is TRUE then the slot will be written as is
                when set. If enable is FALSE then the next time the slot is set it will be written as unrouted.
                Note that the slot still remember its route and can be enabled any time.

 @Param         h_TdmFrame  - (In) a handle to the Frame object.
 @Param         index       - (In) the slot location in the frame.
 @Param         enable      - (In) boolean value indicating whether to enable or disable this time slot
 @Return        E_OK on success; Error code otherwise.
 *//***************************************************************************/
t_Error    TDM_FRAME_ChangeSlotStatus(t_Handle h_TdmFrame, uint32_t index, bool enable);

/**************************************************************************//**
 @Function      TDM_FRAME_SlotSet

 @Description   Write the slot into the SI RAM acording to the given enable value.

 @Param         h_TdmFrame  - (In) a handle to a Frame structure.
 @Param         index  - (In) the slot location in the frame.
 @Param         enable - (In) The way the slot will be written to the SI RAM.
                              Note that this will run over the slot's status.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Slot is not necesarily updated in the next frame cycle.
                Slot cannot be set befor its frame was set.
*//***************************************************************************/
t_Error       TDM_FRAME_SlotSet(t_Handle h_TdmFrame, uint32_t index, bool enable);

/**************************************************************************//**
 @Function      TDM_FRAME_Free

 @Description   Delete a Frame.

 @Param         h_TdmFrame  - (In) a handle to the Frame object.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      In order to free a frame the frame must not be active.
                If the frame is currently running in its TDM, TDM_Disable must
                first be called before TDM_FRAME_Free.
*//***************************************************************************/
t_Error   TDM_FRAME_Free(t_Handle h_TdmFrame);

/**************************************************************************//**
 @Function      TDM_FRAME_Set

 @Description   Writes the slots of a frame to the SI RAM.

 @Param         h_TdmFrame -   (In) a handle to the Frame object.
 @Param         activate   -   (In) should the frame be set to run or written
                                    in the backround. When set to run the frame
                                    will become active when TDM is enabled or set
                                    to run once it will become enabled.
                                    When written in the backround TDM_SwitchFrame()
                                    will be needed in order to make the frame active.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error   TDM_FRAME_Set(t_Handle h_TdmFrame, bool activate);

/**************************************************************************//**
 @Function      TDM_FRAME_SetMultiframe

 @Description   Writes the slots of two frames to the SI RAM and set the TDM to multiframe mode.

 @Param         h_TdmFrame1 -  (In) a handle to a structure of type t_TdmFrame
                                    containing the parameters of the first frame.
 @Param         repetitions1 - (In) number of repetitions for the first frame.
 @Param         h_TdmFrame2 -  (In) a handle to a structure of type t_TdmFrame
                                    containing the parameters of the second frame.
 @Param         repetitions2 - (In) number of repetitions for the second frame.

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The frame must be continuous and the slots should apear in order.
*//***************************************************************************/
t_Error   TDM_FRAME_SetMultiframe(t_Handle h_TdmFrame1,uint8_t repetitions1,
                                  t_Handle h_TdmFrame2,uint8_t repetitions2);

t_Handle TDM_FRAME_GetTdmHandle (t_Handle h_TdmFrame);

/** @} */ /* end of tdm_control_grp group */
/** @} */ /* end of tdm_grp group */

 
#endif /*__QE_TDM_EXT */
