/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          qmc_ext.h QMC device external API.

 @Description   QMC external functions structures and definitions.

 @Cautions      None.

*//***************************************************************************/

#ifndef __QMC_EXT
#define __QMC_EXT

#include "error_ext.h"
#include "std_ext.h"
#include "ucc_ext.h"
#include "ucc_slow_ext.h"
#include "qe_ext.h"
#include "part_ext.h"
#include "cq_ext.h"
#include "xx_ext.h"

/**************************************************************************//**
 @Group         qmc_grp QMC Application Programming Interface

 @Description   QMC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qmc_init_grp QMC Initialization Unit

 @Description   Configuration functions used to allocate resources, memory and data structures,
                 for the driver and to set user and default parameters in to them.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   QMC maximum number of channels allowed on single QMC device.
*//***************************************************************************/
#define QMC_NUM_OF_CH        64

/**************************************************************************//**
 @Description   QMC user parameters structure, used as input to the
                QMC_Config() routine.
 
                Resource allocation for channels and time slots are done at init
                time so it is advisable to set the "totalNumOfChannels" and the "*xMaxTimeSlot"
                to the largest value that will be needed for this QMC device. Changes
                of this data structures at run time are not possible.
*//***************************************************************************/
typedef struct t_QmcParam
{
    /* QMC basic parameters */
    uint16_t            totalNumOfChannels; /**< Total number of channels (Rx & Tx) to be used on this QMC. */
    uint16_t            rxMaxTimeSlot;      /**< Maximum time slots to support Rx. */
    uint16_t            txMaxTimeSlot;      /**< Maximum time slots to support Tx. */

    /* System integration parameters */
    uint32_t            baseAddress;        /**< Base address of QMC registers in memory. */
    t_Handle            h_Qe;               /**< Handle to the controlling QE device. */
    e_Ucc               uccId;              /**< UCC ID to set the QMC device on. */
    int                 irq;                /**< UCC interrupt ID (system-specific number) */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID */
    uint32_t            dataMemAttributes;  /**< Memory attributes for data buffers */

    /* Application interface */
    t_Handle            h_App;              /**< Upper layer application handle. */
    void                (*f_Exceptions)(t_Handle h_App, uint16_t globalEvent);
                                            /**< Upper layer's routine - called to report
                                                 interrupt events and errors. */
} t_QmcParam;


/**************************************************************************//**
 @Description   QMC channel's types - HDLC/TRNS.
*//***************************************************************************/
typedef enum { e_QMC_HDLC_CH, e_QMC_TRNS_CH, e_QMC_NONE } e_QmcChType;

/**************************************************************************//**
 @Description   QMC channel's crc types(HDLC only) - 16/32 bit long.
*//***************************************************************************/
typedef enum { e_QMC_NO_CRC=0, e_QMC_CRC16=2, e_QMC_CRC32=4 } e_QmcCrcSize;

/**************************************************************************//**
 @Description   QMC reverse data flag (TRANSPARENT only).
*//***************************************************************************/
typedef enum { e_QMC_NORMAL=0, e_QMC_REVERSE=1 } e_QmcTrnsReverse;


/**************************************************************************//**
 @Description   QMC channel parameter input structure common for HDLC/TRNS.
*//***************************************************************************/
typedef struct t_QmcChParam
{
    /* System integration parameters */
    t_Handle            h_Qmc;          /**< Handle to the channel's QMC device. */
    t_BufferPoolInfo    bufferPoolInfo; /**< Data buffer pool information */

    /* Application interface */
    t_Handle            h_App;          /**< Upper layer application handle. */
    t_RxStoreFunction   *f_RxStore;     /**< Pointer to an upper layer routine to be
                                             called by the API in order to pass received
                                             frames/data buffers. */
    t_TxConfFunction    *f_TxConf;      /**< Upper layer's routine - called to handle
                                             transmitted frames. */
    void                (*f_Exceptions)(t_Handle h_App, uint32_t channelNum, uint16_t channelEvent);
                                        /**< Upper layer's routine - called to
                                             report interrupt events and errors. */

} t_QmcChParam;


/**************************************************************************//**
 @Description   enum of the different types of Time Slot table.
*//***************************************************************************/
typedef enum e_TimeSlotTbl
{
    e_TIME_SLOT_TBL_NONE = 0,
    e_TIME_SLOT_TBL_TX,
    e_TIME_SLOT_TBL_RX,
    e_TIME_SLOT_TBL_RX_TX
} e_TimeSlotTbl;

/**************************************************************************//**
 @Description QMC global BD ring pool maximum size limitation.
*//***************************************************************************/
#define QMC_MAX_BD_POOL_SIZE        (1024*64)

/**************************************************************************//**
 @Description QMC TSA slot initialization parameters.
*//***************************************************************************/
typedef struct t_QmcTsaSlotParam
{
    e_TimeSlotTbl   tableType;      /* Table type: Rx or Tx                                             */
    uint16_t        slotNum;        /* Slot number                                                      */
    uint8_t         mask;           /* Subchanneling support                                            */
    t_Handle        h_QmcCh;        /* Channel handle to be assigned for the slot                       */
    bool            firstByteSlot;  /* TRUE if this the slot in which the first data byte will appear   */
    bool            lastSlot;       /* TRUE if this the last slot on the table                          */
}t_QmcTsaSlotParam;

/**************************************************************************//**
 @Function      QMC_Config

 @Description   Configures a global QMC device.
                This routine allocates the QMC data structures and sets user
                and default parameters in the QMC device driver parameters structure.

 @Param[in]     p_QmcParam   - Pointer to user's parameters.

 @Return        Handle to a new QMC instance, NULL if error.
*//***************************************************************************/
t_Handle QMC_Config(t_QmcParam *p_QmcParam);

/**************************************************************************//**
 @Function      QMC_Init

 @Description   Initializes a global QMC device.
 
                This routine performs the following:
                1. UCC SLOW device on which it works.
                2. t_Qmc structure, for data which required at runtime.
                3. Global parameters at the QMC parameter ram.
                4. Interrupt table and BD ring memroy allocation.
                5. QMC TSA Rx & Tx tables.

 @Param[in]     h_Qmc - Handle to QMC device allocated by the config routine.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_Init (t_Handle h_Qmc);

/**************************************************************************//**
 @Function      QMC_InitTsaSlot

 @Description   Initialize QMC single time slot in the global time slot assignment table.

 @Param[in]     h_Qmc             - Handle to QMC device.
 @Param[in]     p_QmcTsaSlotParam - QMC TSA slot initialization parameters

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config
                and initialized using QMC_Init.
*//***************************************************************************/
t_Error QMC_InitTsaSlot(t_Handle            h_Qmc,
                        t_QmcTsaSlotParam   *p_QmcTsaSlotParam);

/**************************************************************************//**
 @Function      QMC_Enable

 @Description   Enables a QMC device.

                This routine enables Tx and/or Rx through the General UCC Mode
                Register.

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     mode    - Tx, Rx, or both.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_Enable (t_Handle h_Qmc, e_CommMode mode);

/**************************************************************************//**
 @Function      QMC_Disable

 @Description   Disables a QMC device.

                This routine disables Tx and/or Rx through the General UCC Mode
                Register.
                QMC device can be reenabled using the QMC_Enable() routine.

 @Param[in]     h_Qmc   - Handle to the QMC device.
 @Param[in]     mode    - Communication mode: TX, RX or both.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_Disable(t_Handle h_Qmc, e_CommMode mode);

/**************************************************************************//**
 @Function      QMC_Free

 @Description   Free memory allocated for QMC device.

 @Param[in]     h_Qmc - Handle to the QMC device.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error  QMC_Free ( t_Handle h_Qmc );


/**************************************************************************//**
 @Group         qmc_advanced_init_grp QMC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QMC_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_Qmc           - Handle to QMC object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigFirmwareExternalStructsMemory (t_Handle   h_Qmc,
                                                 uint8_t    memId,
                                                 uint32_t   memAttributes);

/**************************************************************************//**
 @Function      QMC_ConfigRxMaxBufLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Maximum receive buffer length, common for all HDLC channels on this
                QMC device. Defines the number of bytes written by the QE to each BD's
                data buffer before moving to the next BD.[0x60]

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigRxMaxBufLen ( t_Handle h_Qmc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_ConfigRxFramesThreshold

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Global receive frame thrashold. Defines the number of frames required
                in order to generate a Rx interrupt (if QMC_RF is unmasked).
                [1]

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigRxFramesThreshold ( t_Handle h_Qmc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_ConfigIntrTblSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                QMC interrupt Table size, commomn for all channels.
                [128].

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigIntrTableSize ( t_Handle h_Qmc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_ConfigRxBdRingLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Number of BDs in the the Rx BD ring, common for all channels.
                [32].

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigRxBdRingLen ( t_Handle h_Qmc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_ConfigTxBdRingLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Number of BDs in the the Tx BD ring, common for all channels.
                [32].

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigTxBdRingLen ( t_Handle h_Qmc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_ConfigCommonRxTx

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Flag indicating if the Tx TSA & Rx TSA are common.
                [FALSE].

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     newVal  - New value.

 @Return        E_OK if success, error code otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config().
                May be called only before QMC_Init().
                Change is applied only upon calling QMC_Init().
*//***************************************************************************/
t_Error QMC_ConfigCommonRxTx ( t_Handle h_Qmc, bool newVal );

/** @} */ /* end of qmc_advanced_init_grp group */
/** @} */ /* end of qmc_init_grp group */

/**************************************************************************//**
 @Group         qmc_control_grp QMC Control Unit

 @Description   Control unit functions.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      QMC_GetUccId

 @Description   Return the QMC's ucc id number.

 @Param[in]     h_Qmc - Handle to QMC device.
 
 @Return        Ucc id number on valid Qmc handle, -1 otherwise.

 @Cautions      h_Qmc must have been previously created using QMC_Config()
                and initialized using QMC_Init().
*//***************************************************************************/
int QMC_GetUccId(t_Handle h_Qmc);

/**************************************************************************//**
 @Function      QMC_GetChannelType

 @Description   Return the given channel comm type - HDLC/ TRANSPARENT.

 @Param[in]     h_Qmc   - Handle to QMC device.
 @Param[in]     h_QmcCh - Handle to QMC channel.

 @Return        HDLC/ TRANSPARENT value.

 @Cautions      h_Qmc must have been previously created using QMC_Config()
                and initialized using QMC_Init().
*//***************************************************************************/
e_QmcChType QMC_GetChannelType(t_Handle h_Qmc, t_Handle h_QmcCh);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      QMC_DumpRegs

 @Description   Prints the QMC registers map.

 @Param[in]     h_Qmc - Handle to the QMC device.

 @Return        None.
*//***************************************************************************/
void QMC_DumpRegs(t_Handle h_Qmc);
#endif /* (DEBUG_ERRORS > 0) */

 /** @} */ /* end of qmc_control_grp group */
 
/**************************************************************************//*
 @Description   QMC UCC global event register masks
*//***************************************************************************/
#define QMC_UCCE_GOV           0x0001
#define QMC_UCCE_GUN           0x0002
#define QMC_UCCE_GINT          0x0004
#define QMC_UCCE_IQOV          0x0008
#define QMC_UCCE_GLOBAL        (QMC_UCCE_IQOV |  QMC_UCCE_GUN | QMC_UCCE_GOV)
#define QMC_UCCM               (QMC_UCCE_IQOV |  QMC_UCCE_GUN | QMC_UCCE_GOV | QMC_UCCE_GINT)
#define QMC_INTR_RX            (QMC_INTR_RXF | QMC_INTR_RXB)

/**************************************************************************//*
 @Description   QMC global events
*//***************************************************************************/
#define QMC_EV_UNDERRUN         0x0010
#define QMC_EV_BUSY             0x0004

/** @} */ /* end of qmc_grp group */

#endif
