/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          qmc_hdlc_ext.h QMC HDLC channel external API.

 @Description   QMC HDLC channel external functions structures and definitions.

 @Cautions      None.
 
*//***************************************************************************/
#ifndef __QMC_HDLC_EXT_H
#define __QMC_HDLC_EXT_H

#include "std_ext.h"
#include "error_ext.h"
#include "qmc_ext.h"

/**************************************************************************//**
 @Group         qmc_grp QMC Application Programming Interface

 @Description   QMC API functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qmc_hdlc_grp QMC HDLC Application Programming Interface

 @Description   QMC HDLC API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         qmc_hdlc_init_grp QMC HDLC Initialization Unit.

 @Description   Configuration functions used to allocate resources and to set user and default
                 parameters.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   QMC HDLC channel user parameters structure.
*//***************************************************************************/
typedef t_QmcChParam t_QmcHdlcParam;

/**************************************************************************//**
 @Function      QMC_HDLC_Config

 @Description   Configures a QMC HDLC channel.
 
                This routine allocates the QMC channel data structures and sets user
                and default parameters in the QMC channel driver parameters structure.

 @Param[in]     p_QmcHdlcParam - Pointer to QMC HDLC channel parametrs structure.

 @Return        Handle to new QMC HDLC channel.
*//***************************************************************************/
t_Handle QMC_HDLC_Config(t_QmcHdlcParam *p_QmcHdlcParam);

/**************************************************************************//**
 @Function      QMC_HDLC_Init

 @Description   HDLC channel initilization.
 
                This routine initializes the HDLC channel's specific parameters
                at the channel's parameter RAM.
 
                Global channel initilization:
                 1. Assignment t_Qmc parameters out of the driver parameters.
                 2. Allocation and Initialization of the channel's Tx & Rx BD ring.
                 3. Initialization of SI RAM & QMC TSA slots.
                 4. Initialization of global parameters at the channel's specific
                    parameter ram.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_HDLC_Init(t_Handle h_QmcHdlc);

/**************************************************************************//**
 @Function      QMC_HDLC_Enable

 @Description   Enables the given QMC channel.

                This routine enables Tx and/or Rx for this channel through the rstate and
                tstate registers in the channel's specific paraneter ram.

 @Param[in]     h_QmcHdlc   - Handle to QMC HDLC channel.
 @Param[in]     mode        - Tx/Rx communication mode.

 @Return        E_OK on success, error code otherwise.

 @Cautions      None.
*//***************************************************************************/
t_Error QMC_HDLC_Enable(t_Handle h_QmcHdlc, e_CommMode mode);

/**************************************************************************//**
 @Function      QMC_HDLC_Free

 @Description   Free QMC HDLC channel resources.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, Error code otherwise.
*//***************************************************************************/
t_Error QMC_HDLC_Free(t_Handle h_QmcHdlc);


/**************************************************************************
                    ADVANCED CONFIG FUNCTIONS
***************************************************************************/
/**************************************************************************//**
 @Group         qmc_hdlc_advanced_init_grp QMC HDLC Advanced Configuration Unit.

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      QMC_HDLC_ConfigPad

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Number of pad chars, 0-15, to transmit after the frame's data transmission
                is done [0].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigPad(t_Handle h_QmcHdlc, uint8_t newVal);

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigRxInterrupts

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Flag indicating Rx method - interupt / polling. [TRUE].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigRxInterrupts ( t_Handle h_QmcHdlc, bool newVal );

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigIntrManagerId

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                If zero, driver invokes driver receive routine
                directly inside interrupt.
                If non-zero, driver instead defers invocation of
                driver receive routine to user. Driver informs user by invoking
                XX_Call(). It is then responsibility of user to ensure
                that drive receive routine is eventually called. XX_Call()
                will not be called again until user calls driver receive routine.
                Driver calls XX_Call() as follows:
                XX_Call(this value, NULL, appId, 0, 0).
                [0].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigIntrManagerId ( t_Handle h_QmcHdlc, uint32_t newVal );

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigIntMask

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Channel's interupts mask. [QMC_INTR_MRF].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigIntMask ( t_Handle h_QmcHdlc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigRxMaxFrameLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Maximum receive frame length (without CRC). An interrupt is
                generated upon this violation (in not masked) [1024].
 
 @Param[in]     h_QmcHdlc   - Handle to QMC HDLC Channel.
 @Param[in]     newVal      - N ew value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigRxMaxFrameLen ( t_Handle h_QmcHdlc, uint16_t newVal );

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigIdleMode

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Flag indicating if this channel's idle mode is enabled. If true this channel
                is going to transmit idle pattern if no data is read for transmission.
                [FALSE].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigIdleMode ( t_Handle h_QmcHdlc, bool newVal );


/**************************************************************************//**
 @Function      QMC_HDLC_ConfigCrcSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Channel's CRC size, 16 bits or 32 bits. [e_QMC_CRC16].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigCrcSize ( t_Handle h_QmcHdlc, e_QmcCrcSize newVal );

/**************************************************************************//**
 @Function      QMC_HDLC_ConfigNOF

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Number of flags (0x7E) transmitted before each frame. [1].

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC Channel.
 @Param[in]     newVal    - New value.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config().
                May be called only before QMC_HDLC_Init().
                Change is applied only upon calling QMC_HDLC_Init().
*//***************************************************************************/
t_Error QMC_HDLC_ConfigNof ( t_Handle h_QmcHdlc, uint8_t newVal );


/** @} */ /* end of qmc_hdlc_advanced_init_grp group */
/** @} */ /* end of qmc_hdlc_init_grp group */

/**************************************************************************//**
 @Group         qmc_hdlc_control_grp QMC HDLC Control Unit.

 @Description   Control unit functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QMC_HDLC_StopRx

 @Description   Stop transmission on a given HDLC channel.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config
                and initialized using QMC_HDLC_Init.
*//***************************************************************************/
t_Error QMC_HDLC_StopTx(t_Handle h_QmcHdlc);

/**************************************************************************//**
 @Function      QMC_HDLC_StopRx

 @Description   Stop transmission on a given HDLC channel.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config
                and initialized using QMC_HDLC_Init.
*//***************************************************************************/
t_Error QMC_HDLC_StopRx(t_Handle h_QmcHdlc);

/**************************************************************************//**
 @Function      QMC_HDLC_RestartTx

 @Description   Restart transmission on a given channel.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config
                and initialized using QMC_HDLC_Init.
*//***************************************************************************/
t_Error QMC_HDLC_RestartTx(t_Handle h_QmcHdlc);

/**************************************************************************//**
 @Function      QMC_HDLC_RestartRx

 @Description   Restart transmission on a given channel.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.

 @Cautions      h_QmcHdlc must have been previously created using QMC_HDLC_Config
                and initialized using QMC_HDLC_Init.
*//***************************************************************************/
t_Error QMC_HDLC_RestartRx (t_Handle h_QmcHdlc);

/**************************************************************************//**
 @Function      QMC_HDLC_TxConf

 @Description   QMC HDLC channel function for forced transmit confirmation.
 
                This routine collects the transmitted frames/buffers and
                returns them to the application. It also prepares the transmit
                BDs for new frames. For each frame/buffer (depending on the
                data exchange mode) the application callback routine will be
                called (f_TxConf in t_QmcChParam).
 
                Normally, transmit confirmation is done automatically by the
                transmit routine. This routine can be called after transmission
                was stopped, to collect all remaining frames/buffers from the
                BD ring.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_HDLC_TxConf(t_Handle h_QmcHdlc);

/** @} */ /* end of qmc_hdlc_control_grp group */

/**************************************************************************//**
 @Group         qmc_hdlc_runtime_grp QMC HDLC Runtime Unit

 @Description   Rx / Tx functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      QMC_HDLC_Tx

 @Description   Transmit data over QMC HDLC channel.

 @Param[in]     h_QmcHdlc       - Handle to QMC HDLC channel.
 @Param[in]     p_Data          - Pointer to a single data buffer.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length          - Data length.
 @Param[in]     lastUserBuffer  - Indicating if this is the last buffer of the frame.

 @Return        E_OK on success, error code otherwise.
*//***************************************************************************/
t_Error QMC_HDLC_Tx(t_Handle    h_QmcHdlc,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint32_t    length,
                    bool        lastUserBuffer);

/**************************************************************************//**
 @Function      QMC_HDLC_Rx

 @Description   Receive data over QMC HDLC channel.

                This function is called by the upper application to instigate
                receive handling. Upon interrupt on receive, depending on queueId
                parameter, either this routine is called directly by the driver or
                by the user.
                QMC_HDLC_Rx routine collects the buffer from the port BD's and
                passes them to the application by calling the application's
                f_RxStore routine.

 @Param[in]     h_QmcHdlc - Handle to QMC HDLC channel.

 @Return        E_OK on success, Error code otherwise.
*//***************************************************************************/
t_Error QMC_HDLC_Rx(t_Handle h_QmcHdlc);

/** @} */ /* end of qmc_hdlc_runtime_grp group  */
/** @} */ /* end of qmc_hdlc_grp group  */
/** @} */ /* end of qmc_grp group  */

#endif  /* __QMC_HDLC_EXT_H */
