/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**

 @File          rtc83_ext.h

 @Description   83xx Real Time Clock external structures and definitions.
*//***************************************************************************/

#ifndef __RTC83_EXT_H
#define __RTC83_EXT_H


/**************************************************************************//**
 @Group         rtc83_grp RTC83 Application Programming Interface

 @Description   RTC API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         rtc83_init_grp  RTC83 Initialization Unit

 @Description   RTC initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Application's callback function, to be called by the driver.

 @Param[in]     h_App   - Handle to the application's object, as provided in
                          the RTC83 configuration parameters.
 @Param[in]     curTime - Provides the current RTC count.

*//***************************************************************************/
typedef void (t_Rtc83CallBack)(t_Handle h_App, uint32_t curTime);

/**************************************************************************//**
 @Description   Input clock options
 *//***************************************************************************/
typedef enum e_Rtc83InputClk
{
    e_RTC83_INPUT_CLK_SYSTEM    = 0x00, /**< Input clock is internal system clock */
    e_RTC83_INPUT_CLK_EXTERNAL  = 0x01  /**< Input clock is external */
} e_Rtc83InputClk;


/**************************************************************************//**
 @Description   RTC83 initialization structure, used when calling RTC83_Config().
 *//***************************************************************************/
typedef struct t_Rtc83Param
{
    /* System integration parameters */
    uint32_t        baseAddress;        /**< Base address of RTC registers in memory. */
    uint32_t        inputClockFreqHz;   /**< Input clock frequency (whether internal or external) in Hz */
    int             secIrq;             /**< Per-second interrupt ID (system-specific number) */
    int             alarmIrq;           /**< Alarm interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle        h_App;              /**< Handle to the application's object */
    t_Rtc83CallBack *f_SecCallBack;     /**< Second interrupt user callback function */
    t_Rtc83CallBack *f_AlarmCallBack;   /**< Alarm interrupt user callback function */
} t_Rtc83Param;


/**************************************************************************//**
 @Function      RTC83_Config

 @Description   Configure the RTC module.

 @Param[in]     p_RtcParam  - Pointer to the RTC configuration parameters.

 @Return        Handle to the RTC unit on success, NULL otherwise.
*//***************************************************************************/
t_Handle RTC83_Config(t_Rtc83Param *p_RtcParam);

/**************************************************************************//**
 @Function      RTC83_Init

 @Description   Fully initializes the RTC module.

 @Param[in]     h_Rtc - Handle to the RTC83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_Init(t_Handle h_Rtc);

/**************************************************************************//**
 @Function      RTC83_Free

 @Description   Free RTC descriptor from memory.

 @Param[in]     h_Rtc - Handle to the RTC83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_Free(t_Handle h_Rtc);


/**************************************************************************//**
 @Group         rtc83_adv_init_grp  RTC83 Advanced Configuration Unit

 @Description   RTC configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      RTC83_ConfigInputSource

 @Description   Configures whether the RTC input source clock is internal
                (system clock) or external. [e_RTC83_INPUT_CLK_SYSTEM].

                The RTC input clock can be the system bus clock or an external
                (on-board) oscillator.

 @Param[in]     h_Rtc       - Handle to the RTC83 object.
 @Param[in]     inputSource - New value for the input source clock.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_ConfigInputSource(t_Handle h_Rtc, e_Rtc83InputClk inputSource);

/** @} */ /* end of rtc83_adv_init_grp */
/** @} */ /* end of rtc83_init_grp */


/**************************************************************************//**
 @Group         rtc83_control_grp   RTC83 Control UNIT

 @Description   RTC control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      RTC83_Start

 @Description   Start the RTC count.

 @Param[in]     h_Rtc - Handle to the RTC83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_Start(t_Handle h_Rtc);

/**************************************************************************//**
 @Function      RTC83_Stop

 @Description   Stop the RTC count.

 @Param[in]     h_Rtc - Handle to the RTC83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_Stop(t_Handle h_Rtc);

/**************************************************************************//**
 @Function      RTC83_GetTime

 @Description   Get current RTC counter value.

 @Param[in]     h_Rtc       - Handle to the RTC83 object.
 @Param[out]    p_CurTime   - Returns the current RTC count.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_GetTime(t_Handle h_Rtc, uint32_t *p_CurTime);

/**************************************************************************//**
 @Function      RTC83_SetTime

 @Description   Modify the value of the RTC83 counter.

 @Param[in]     h_Rtc   - Handle to the RTC83 object.
 @Param[in]     newTime - New value for the RTC83 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This function can be called only after RTC83_Init().
*//***************************************************************************/
t_Error RTC83_SetTime(t_Handle h_Rtc, uint32_t newTime);

/**************************************************************************//**
 @Function      RTC83_SetAlarm

 @Description   Modify the value of the alarm register.

 @Param[in]     h_Rtc       - Handle to the RTC83 object.
 @Param[in]     alarmValue  - New value for the alarm register.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This function can be called only after RTC83_Init().
*//***************************************************************************/
t_Error RTC83_SetAlarm(t_Handle h_Rtc, uint32_t alarmValue);

/**************************************************************************//**
 @Function      RTC83_SetAlarmInterruptMode

 @Description   Enable/disable the alarm interrupt.

 @Param[in]     h_Rtc   - Handle to the RTC83 object.
 @Param[in]     enabled - TRUE to enable the interrupt; FALSE to disable it.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This function can be called only after RTC83_Init().
*//***************************************************************************/
t_Error RTC83_SetAlarmInterruptMode(t_Handle h_Rtc, bool enabled);

/**************************************************************************//**
 @Function      RTC83_SetSecondsInterruptMode

 @Description   Enable/disable the seconds interrupt.

 @Param[in]     h_Rtc   - Handle to the RTC83 object.
 @Param[in]     enabled - TRUE to enable the interrupt; FALSE to disable it.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This function can be called only after RTC83_Init().
*//***************************************************************************/
t_Error RTC83_SetSecondsInterruptMode(t_Handle h_Rtc, bool enabled);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      RTC83_DumpRegs

 @Description   Prints the registers of RTC83 module

 @Param[in]     h_Rtc - Handle to the RTC83 object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error RTC83_DumpRegs(t_Handle h_Rtc);

#endif /* DEBUG_ERRORS > 0 */

/** @} */ /* end of rtc83_control_grp */
/** @} */ /* end of rtc83_grp */


#endif /* __RTC83_EXT_H */

