/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          sata_ext.h

 @Description   external definition and API for the SATA controller.
 
 @Cautions      None.
*//***************************************************************************/
#ifndef __SATA_EXT_H
#define __SATA_EXT_H

/**************************************************************************//**
 @Group         sata_grp SATA Application Programming Interface

 @Description   SATA timers functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sata_init_grp SATA Initialization Unit

 @Description   SATA init functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Power state restriction - the user can restrict the power state of the SATA bus
*//***************************************************************************/
typedef enum e_SataPowerRestrict
{
    e_SATA_POWER_RESTRICT_NONE                = 0, /**< No restrictions */
    e_SATA_POWER_RESTRICT_PARTIAL             = 1, /**< Partial power management not allowed  */
    e_SATA_POWER_RESTRICT_SLUMBER             = 2, /**< Slumber power management not allowed  */
    e_SATA_POWER_RESTRICT_PARTIAL_AND_SLUMER  = 3  /**< Both partial and slumber states not allowed */
}e_SataPowerRestrict;


/**************************************************************************//**
 @Description   SATA speed
*//***************************************************************************/
typedef enum e_SataSpeed
{
    e_SATA_SPEED_INVALID    = 0, /**< Invalid speed (device not present or communication not established) */
    e_SATA_SPEED_I          = 1, /**< First-generation rate  */
    e_SATA_SPEED_II         = 2  /**< Second-generation rate  */
} e_SataSpeed;

/**************************************************************************//**
 @Description   SATA status and contol registers
*//***************************************************************************/
typedef enum e_SataSupersetReg
{
    e_SATA_REG_SSTATUS,         /**< Sstatus register    */
    e_SATA_REG_SERROR,          /**< Serror register     */
    e_SATA_REG_SCONTROL,        /**< Scontrol register   */
    e_SATA_REG_SACTIVE          /**< Sactive register    */
} e_SataSupersetReg;

/**************************************************************************//**
 @Description   SATA device type
*//***************************************************************************/
typedef enum e_SataDeviceType
{
    e_SATA_DEVICE_ATA,           /**< Regular serial ATA device(HD) */
    e_SATA_DEVICE_ATAPI,         /**< Serial ATA device that supports packet comand interface(DVD) */
    e_SATA_DEVICE_PMP,           /**< SATA port multiplier          */
    e_SATA_DEVICE_UNRECOGNIZED   /**< Unrecognized device           */
}e_SataDeviceType;


/**************************************************************************//**
 @Description   port number - when the device on the other side is PMP
                a user need to choose which port of the PMP is the destination
                of his command.
*//***************************************************************************/
typedef enum e_SataPortNum
{
    e_SATA_PORT1,        /**< 1st  port */
    e_SATA_PORT2,        /**< 2nd  port */
    e_SATA_PORT3,        /**< 3rd  port */
    e_SATA_PORT4,        /**< 4th  port */
    e_SATA_PORT5,        /**< 5th  port */
    e_SATA_PORT6,        /**< 6th  port */
    e_SATA_PORT7,        /**< 7th  port */
    e_SATA_PORT8,        /**< 9th  port */
    e_SATA_PORT9,        /**< 9th  port */
    e_SATA_PORT10,       /**< 10th port */
    e_SATA_PORT11,       /**< 11th port */
    e_SATA_PORT12,       /**< 12th port */
    e_SATA_PORT13,       /**< 13th port */
    e_SATA_PORT14,       /**< 14th port */
    e_SATA_PORT15,       /**< 15th port */
    e_SATA_PORT_PMP      /**< PMP control port */
} e_SataPortNum;

/**************************************************************************//**
 @Description   Detection status - the status of the connection with the device on the bus
*//***************************************************************************/
typedef enum e_SataDetectionStatus
{
    e_SATA_NO_DEVICE        = 0, /**< No device detected and PHY communication not established        */
    e_SATA_PHY_ERROR        = 1, /**< Device presence detected but PHY communication not established  */
    e_SATA_DEVICE_ONLINE    = 3, /**< Device presence detected and PHY communication established      */
    e_SATA_PHY_OFFLINE      = 4  /**< PHY in offline mode as a result of the interface being
                                      disabled or running in a BIST                                   */
} e_SataDetectionStatus;


/**************************************************************************//**
@Description SATA status FIS - ATA registers sent from the device to us
             reading the registers give more information about the device status
*//***************************************************************************/
typedef struct t_SataStatusReg
{
    uint8_t fisType;        /**< FIS type - always 0x34*/
    uint8_t pmPort;         /**< Port num in the port multiplier */
    uint8_t status;         /**< Status register */
    uint8_t error;          /**< error register */
    uint8_t lbaLow;         /**< lbaLow register */
    uint8_t lbaMid;         /**< lbaMid register */
    uint8_t lbaHigh;        /**< lbaHigh register */
    uint8_t device;         /**< device register */
    uint8_t lbaLowExp;      /**< lbaLowExp register */
    uint8_t lbaMidExp;      /**< lbaMidExp register */
    uint8_t lbaHighExp;     /**< lbaHighExp register */
    uint8_t res0;
    uint8_t sectorNum;      /**< sectorNum register */
    uint8_t sectorNumExp;   /**< sectorNumExp register */
    uint8_t res1[6];
 
} t_SataStatusReg;


/**************************************************************************//**
 @Description   User's callback routine for exceptions handling (invoked by the driver).

                When this routine is called it means that a fatal error or hot
                swap of a device happened. The user should know that all commands
                are cancelled, and that there is no device attached to the bus
                until the driver calls t_SataDiscoveredDeviceCallBack.
 
 @Param[in]     h_App       - Application handle, provided by the user in the
                              configuration structure passed to SATA_Config().
 @Param[in]     Execeptions - Events that caused this routine to be invoked by
                              the driver.
                              values may be any combination of the SATA exceptions
                              bit masks.
 @Return        None.
 
 @See           @ref sata_exbm "SATA Exceptions Bit Mask".
 *//***************************************************************************/
typedef void (t_SataExceptionsCallBack)(t_Handle h_App,uint32_t Execeptions);

/**************************************************************************//**
 @Description   User's callback routine for phy ready state (invoked by the driver).

                When this routine is called it means that a phy ready was accepted from the
                physical layer.
                This is mostly used when using PMP that doesn't sends his signature automaticaly.
                After reaching this callback,the user knows to send SRTS command to the PMP ,
                in order to reset it , and get the PMP signature.
 
 @Param[in]     h_App       - Application handle, provided by the user in the
                              configuration structure passed to SATA_Config().
 
 @Return        None.
 
 @See           @ref sata_exbm "SATA Exceptions Bit Mask".
 *//***************************************************************************/
typedef void (t_SataPhyReadyCallBack)(t_Handle h_App);

/**************************************************************************//**
 @Description   User's callback routine for a new device discovery (invoked by the driver).

                When this routine is called it means that a new signature was accepted by the controller.
                The driver let the user know about this new signatue.
 
 @Param[in]     h_App       - Application handle, provided by the user in the
                              configuration structure passed to SATA_Config().
 @Param[in]     deviceType  - The type of the discovered device.
 
 @Return        None.
 
 @Cautions      User must not assume it's the same device that was connected before
                because SATA supports hot plug.
                If we are working with PMP user may accept more than one signature,the users should
                know which signature was accepted from what port.
 *//***************************************************************************/
typedef void (t_SataDiscoveredDeviceCallBack)(t_Handle h_App,e_SataDeviceType deviceType);

/**************************************************************************//**
 @Description   User's callback routine for notification (invoked by the driver)
                The notification is sent by the PMP for a distant device(behind the PMP)

 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to SATA_Config().
 @Param[in]     port    - Port number of the device behind the PMP.
 
 @Return        None.
 *//***************************************************************************/
typedef void (t_SataNotifyCallback)(t_Handle h_App,e_SataPortNum port);

/**************************************************************************//**
 @Description   User's callback routine for succesfuly completed commands (invoked by the driver)
 
 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to SATA_Config().
 @Param[in]     command - Number of the command in the command queue.
 
 @Param[in]     p_Sfis  - Pointer to the status FIS if exist,if it doesn't NULL is returned.
 
 @Return        None.
 *//***************************************************************************/
typedef void (t_SataCmdCompleteCallback)(t_Handle h_App,uint8_t command,t_SataStatusReg *p_Sfis);

/**************************************************************************//**
 @Description   User's callback routine for error command (invoked by the driver)
 
 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to SATA_Config().
 @Param[in]     command - Number of the command in the command queue.
 @Param[in]     p_Sfis  - Pointer to the status FIS if exist, if it doesn't
                          NULL is returned.
 
 @Return        None.
 
 @Cautions      When command is errored , all commands sent to the same device are
                aborted.User should send this commands again.
 *//***************************************************************************/
typedef void (t_SataCmdErrorCallback)(t_Handle h_App,uint8_t command,t_SataStatusReg *p_Sfis);


/*********************************************************************************//**
 @Description   SATA initialization structure, used when calling SATA_Config().
*//**********************************************************************************/
typedef struct t_SataParam
{
    t_Handle                        h_App;                           /**< Handle to the upper layer handler */
    t_Handle                        h_Part;                          /**< Handle to the part's structure.  */
    int                             irq;                             /**< SATA interrupt ID (system-specific number) */
    t_SataExceptionsCallBack        *f_SataExceptionsCallBack;       /**< Exceptions callback */
    t_SataPhyReadyCallBack          *f_SataPhyReadyCallBack;         /**< Phy ready callback */
    t_SataDiscoveredDeviceCallBack  *f_SataDiscoveredDeviceCallBack; /**< New device discovery callback */
    t_SataNotifyCallback            *f_SataNotifyCallback;           /**< Notification callback */
    t_SataCmdCompleteCallback       *f_SataCmdCompleteCallback;      /**< Command copleted succesfuly callback */
    t_SataCmdErrorCallback          *f_SataCmdErrorCallback;         /**< Command error callback */
    uint8_t                         commandDescMemId;                /**< Memory partition ID of the SATA command descriptors structures */
    uint32_t                        commandDescMemAttributes;        /**< Memory attributes of the SATA command descriptors structures */
    uint32_t                        sataClockIn;                     /**< clock frequncy supplied to the Sata block */
    uint32_t                        baseAddress;                     /**< Base Address for the SATA module */
} t_SataParam;

/*********************************************************************************//**
 @Description   Host to device registers - used by the driver to build the FIS towards the host
*//**********************************************************************************/
typedef struct t_SataH2dReg
{
    e_SataPortNum pmPort;      /**< device port number - when using PMP */
    uint8_t       command;     /**< Command register value */
    uint8_t       features;    /**< features register value */
    uint8_t       lbaLow;      /**< lbaLow register value */
    uint8_t       lbaMid;      /**< lbaMid register value */
    uint8_t       lbaHigh;     /**< lbaHigh register value */
    uint8_t       device;      /**< device register value */
    uint8_t       lbaLowExp;   /**< lbaLowExp register value */
    uint8_t       lbaMidExp;   /**< lbaMidExp register value */
    uint8_t       lbaHighExp;  /**< lbaHighExp register value */
    uint8_t       featuresExp; /**< featuresExp register value */
    uint8_t       sectorNum;   /**< sectorNum register value */
    uint8_t       sectorNumExp;/**< sectorNumExp register value */
    uint8_t       control;     /**< control register value */
 
} t_SataH2dReg;


/**************************************************************************//**
 @Function      SATA_Config

 @Description   Create the SATA object and returns its handle.
 
                Some parameters are set implicitly to driver's default values.
                This routine must be called prior to any other SATA routine.

 @Param[in]     p_SataParam - Pointer to the configuration parameters structure.
 
 @Return        Handle to the initialized SATA object.
                This handle uniquely identifies the SATA object and shall be
                used when calling other SATA API routines.
*//***************************************************************************/
t_Handle SATA_Config(t_SataParam *p_SataParam);

/**************************************************************************//**
 @Function      SATA_Init

 @Description   Initialization of SATA controller.
 
                This routine initializes the SATA controller according to the
                selected configuration.

 @Param[in]     h_Sata - Handle to the SATA object.

 @Return        E_OK on success. Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_Init(t_Handle h_Sata);

/**************************************************************************//**
 @Function      SATA_Free

 @Description   This routine is called to shut down the SATA Controller.
 
                It disables the transmission of commands and
                releases all previously allocated resources.

 @Param[in]     h_Sata - Handle to the SATA object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SATA_Free(t_Handle h_Sata);

/**************************************************************************//**
 @Group         sata_advanced_init_grp SATA Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SATA_ConfigPowerMngRestrict

 @Description   This routine is called to restrictsome of the power states

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[in]     powerRestrict   - The requested restriction.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called before SATA_Init().
*//***************************************************************************/
t_Error SATA_ConfigPowerMngRestrict(t_Handle h_Sata,e_SataPowerRestrict powerRestrict);

/**************************************************************************//**
 @Function      SATA_ConfigDataTransferMaxSize

 @Description   This routine is called to configure the FIS maximum size

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     fisMaxSize  - Maximum byte size of a FIS, should be word aligned.
                              value must be smaller than 0x2000.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called before SATA_Init().
*//***************************************************************************/
t_Error SATA_ConfigDataTransferMaxSize(t_Handle h_Sata,uint32_t fisMaxSize);

/**************************************************************************//**
 @Function      SATA_ConfigRxFifoWaterMarkSize

 @Description   This routine is called to configure the size of the water mark in the rx fifo

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[in]     waterMarkSize   - This sets the number of locations in the 58-deep
                                  Rx FIFO that can be used before the transport
                                  layer instructs the link layer to transmit
                                  HOLDS to the transmitting end.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called before SATA_Init().
*//***************************************************************************/
t_Error SATA_ConfigRxFifoWaterMarkSize(t_Handle h_Sata,uint8_t waterMarkSize);

/**************************************************************************//**
 @Function      SATA_ConfigPhyNotReadyTimeout

 @Description   This routine is called to configure the PHY_READY timer

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[in]     timeoutMicroSec - The timeout value of the PHY_READY timer if
                                  PHY_READY is de-asserted as long as the timer
                                  was programed a PHY_RESET is issued. Value is
                                  in microseconds. If the value is zero, the
                                  timer is disabled.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called before SATA_Init().
*//***************************************************************************/
t_Error SATA_ConfigPhyNotReadyTimeout(t_Handle h_Sata, uint32_t timeoutMicroSec);

/**************************************************************************//**
 @Function      SATA_ConfigIntrThreshold

 @Description   This routine is called to configure the interrupt coalescing

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     cmdNum      - The number of command completions that will raise
                              the interrupt.
 @Param[in]     microSec    - The number of microseconds a command complete
                              wasn't handled before the interrupt will be signaled.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called before SATA_Init().
*//***************************************************************************/
t_Error SATA_ConfigIntrThreshold(t_Handle h_Sata, uint32_t cmdNum, uint32_t microSec);

/** @} */ /* end of sata_advanced_init_grp group */
/** @} */ /* end of sata_init_grp group */

/**************************************************************************//**
 @Group         SATA_ctrl_grp SATA Control Unit

 @Description   SATA control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Power state
*//***************************************************************************/
typedef enum e_SataPowerState
{
    e_SATA_POWER_STATE_INVALID   = 0, /**< power state is invalid - this value will be returned when there is no device present */
    e_SATA_POWER_STATE_PARTIAL   = 1, /**< partial power state */
    e_SATA_POWER_STATE_SLUMBER   = 2, /**< slumber power state */
    e_SATA_POWER_STATE_ACTIVE    = 4  /**< active power state  */
} e_SataPowerState;

/**************************************************************************//**
 @Description   Link state error counters
*//***************************************************************************/
typedef struct t_SataLinkLayerErrors
{
    uint8_t     controlChar;    /**< Kchar error gray count */
    uint8_t     phyInternal;    /**< PHY internal error gray count */
    uint8_t     code;           /**< Code error gray count */
    uint8_t     disparity;      /**< Disparity error gray count */
}t_SataLinkLayerErrors;


/**************************************************************************//**
 @Function      SATA_SetPmpAttached

 @Description   This routine is called to inform the driver on the existence of PMP.

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     pmpAttached - TRUE if pmp exist,FALSE if not .

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_SetPmpAttached(t_Handle h_Sata, bool  pmpAttached);

/**************************************************************************//**
 @Function      SATA_SetMaxSpeed

 @Description   This routine is called to configure maximum bus speed.

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     maxSpeed    - Maximum bus speed .

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_SetMaxSpeed(t_Handle h_Sata, e_SataSpeed maxSpeed);

/**************************************************************************//**
 @Function      SATA_SetPowerState

 @Description   This routine is called to configure the power state.

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     powerState  - Requested power state.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_SetPowerState(t_Handle h_Sata, e_SataPowerState powerState);

/**************************************************************************//**
 @Function      SATA_SetSystemBusPriority

 @Description   This routine is called to configure the priority level for
                CSB arbitration for SATA controllers dma requests

 @Param[in]     h_Sata        - Handle to the SATA object.
 @Param[in]     dataPriority  - Priority when the request belongs to data transfer.
                                Priority value should be 0-3.
 @Param[in]     descPriority  - Priority when the request belongs to descriptor fetch.
                                Priority value should be 0-3.
 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      This routine must be called after SATA_Init().
*//***************************************************************************/
t_Error SATA_SetSystemBusPriority(t_Handle               h_Sata,
                                  uint8_t                dataPriority,
                                  uint8_t                descPriority);
 
/**************************************************************************//**
 @Function      SATA_GetLinkLayerErrorCount

 @Description   Get the status counters of the link layer.

 @Param[in]     h_Sata                   - Handle to the SATA object.
 @Param[out]    p_SataLinkLayerErrors    - Pointer to the counters structure.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      Counters are reset after they have been raed.
*//***************************************************************************/
t_Error SATA_GetLinkLayerErrorCount(t_Handle h_Sata, t_SataLinkLayerErrors *p_SataLinkLayerErrors);

/**************************************************************************//**
 @Function      SATA_ReadSupersetRegister

 @Description   Read one of the Control and Status registers.

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[in]     supersetReg - Register name.
 @Param[out]    p_RegVal    - Returns the value of requested register.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      Serror register will be deleted after it has been read.
*//***************************************************************************/
t_Error SATA_ReadSupersetRegister(t_Handle          h_Sata,
                                  e_SataSupersetReg supersetReg,
                                  uint32_t          *p_RegVal);

/**************************************************************************//**
 @Function      SATA_SoftReset

 @Description   Resets the controller.
 
                The controller is brought offline and back online.
                This operation clears All command queues and all devices are
                disconected.

 @Param[in]     h_Sata      - Handle to the SATA object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      User should know that after calling this function he should
                identify the device again and execute all his commands again.
*//***************************************************************************/
t_Error SATA_SoftReset(t_Handle h_Sata);

/**************************************************************************//**
 @Function      SATA_GetPowerState

 @Description   Gets the current power State.

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[out]    p_PowerState    - Returns current power state.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_GetPowerState(t_Handle h_Sata, e_SataPowerState *p_PowerState);

/**************************************************************************//**
 @Function      SATA_GetSpeedStatus

 @Description   Gets the current speed.

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[out]    p_SpeedStatus   - Returns current bus speed.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_GetSpeedStatus(t_Handle h_Sata,e_SataSpeed *p_SpeedStatus);

/**************************************************************************//**
 @Function      SATA_GetDetectionStatus

 @Description   Gets the status of the device detection on the other side of the SATA bus.

 @Param[in]     h_Sata              - Handle to the SATA object.
 @Param[in]     p_DetectionStatus   - Returns the current detection status.
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_GetDetectionStatus(t_Handle                h_Sata,
                                e_SataDetectionStatus   *p_DetectionStatus);

/**************************************************************************//**
 @Function      SATA_GetTransportLayerStateMachine

 @Description   Gets the status of the transport layer.

 @Param[in]     h_Sata      - Handle to the SATA object.
 @Param[out]    p_TxState   - Returns the Tx status.
 @Param[out]    p_RxState   - Returns the Rx status.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_GetTransportLayerStateMachine(t_Handle h_Sata,
                                           uint8_t  *p_TxState,
                                           uint8_t  *p_RxState);

/**************************************************************************//**
 @Function      SATA_GetLinkLayerStateMachine

 @Description   Gets the status of the link layer.

 @Param[in]     h_Sata  - Handle to the SATA object.
 @Param[out]    p_State - Returns the link layer status.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
*//***************************************************************************/
t_Error SATA_GetLinkLayerStateMachine(t_Handle h_Sata, uint8_t *p_State);

/** @} */ /* end of SATA_ctrl_grp */
 
/**************************************************************************//**
 @Group         sata_runtime_grp SATA Runtime Unit

 @Description   SATA runtime unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Command type
*//***************************************************************************/
typedef enum e_SataCommandType
{
    e_SATA_COMMAND_ATA,         /**< ATA command   */
    e_SATA_COMMAND_NCQ,         /**< NCQ command   */
    e_SATA_COMMAND_RESET        /**< Reset command */
}e_SataCommandType;

/**************************************************************************//**
@Description    Number of the segments in the segmented data list that can be passed for each command
*//***************************************************************************/
#define SATA_MAX_DMA_BLOCKS 32

/**************************************************************************//**
@Description    Segmented data list transfered as part of the command to the driver
*//***************************************************************************/
typedef struct t_SataCommandData
{
    uint32_t    numOfEntries;     /**< num of entries in the list */
    struct
    {
        uint32_t    address;      /**< Data address ,aligned to word */
        uint32_t    length;       /**< length of the segment, only 20 bits used, must be word aligned */
    } block[SATA_MAX_DMA_BLOCKS]; /**< one data segment structure */
} t_SataCommandData;

/**************************************************************************//**
 @Function      SATA_CmndExecute

 @Description   This routine is called send a command on the bus to the device
                on the other side.
 
                This routine shouldn't be called for ATAPI devices, except
                for reset command.

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[in]     p_SataH2dReg    - Pointer to the ATA registers structure that creates the command  .
 @Param[in]     commandType     - The command type.
 @Param[in]     p_Dmalist       - Pointer to segmented memory list - the data (read/write)
                                  can be a list of memory areas that the controller
                                  will read to/from.

 @Param[in]     commandTag- number of the command in the command queue(value range(0-15).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      The user shouldn't send a command to a place in the command queue that is already in use
                and he didn't get command complete callback or command error callback on that specific place.
*//***************************************************************************/
t_Error SATA_CmndExecute(t_Handle h_Sata,t_SataH2dReg *p_SataH2dReg,e_SataCommandType commandType,t_SataCommandData *p_Dmalist,uint8_t commandTag);

/**************************************************************************//**
 @Function      SATA_AtapiCmndExecute

 @Description   This routine is called to send a command on the bus to ATAPI
                device on the other side.
 
                Reset command should be still called through SATA_CmndExecute().

 @Param[in]     h_Sata          - Handle to the SATA object.
 @Param[in]     p_SataH2dReg    - Pointer to the ATA registers structure that
                                  creates the command.
 @Param[in]     p_Dmalist       - Pointer to segmented memory list - the data
                                  (read/write) can be a list of memory areas that
                                  the controller will read to/from.
 @Param[in]     commandTag      - Number of the command in the command queue(value range(0-15).
 @Param[in]     p_AtapiCmd      - Pointer to the ATAPI command .
 @Param[in]     atapiCmdLength  - Byte length of the ATAPI command, should be only 12 or 16 byte.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Sata must have been previously created with SATA_Config().
 @Cautions      The user shouldn't send a command to a place in the command queue
                that is already in use and he didn't get command complete callback
                or command error callback on that specific place.
*//***************************************************************************/
t_Error SATA_AtapiCmndExecute(t_Handle h_Sata,t_SataH2dReg *p_SataH2dReg,t_SataCommandData *p_Dmalist,uint8_t commandTag,uint8_t *p_AtapiCmd,uint8_t atapiCmdLength);

/** @} */ /* end of sata_runtime_grp  */


/**************************************************************************//**
 @Group         sata_exceptions_grp SATA Exceptions Unit
 
 @Description   SATA Exceptions Unit
 
 @{
*//***************************************************************************/
 
/**************************************************************************//**
 @Anchor        sata_exbm
 
 @Collection    SATA Exceptions Bit Mask
 
                Exceptions and events reported through the exceptions callback.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define SATA_PHY_OFFLINE      0x00000010 /**< PHY changed his state from PHY READY */
#define SATA_MASTER_ERR       0x00004000 /**< SATA controller master error. Indicates if the host received an error on the CSB
                                              Master interface during the access to memory. */
#define SATA_DATA_UNDERRUN    0x00002000 /**< Data underrun */
#define SATA_DATA_OVERRRUN    0x00001000 /**< Data overrun. */
#define SATA_CRC_ERR_TX       0x00000800 /**< CRC error tx */
#define SATA_CRC_ERR_RX       0x00000400 /**< CRC error rx */
#define SATA_FIFO_OVERFLOW_TX 0x00000200 /**< Tx FIFO is in overflow condition
                                              while sending FIS */
#define SATA_FIFO_OVERFLOW_RX 0x00000100 /**< Rx FIFO is in overflow condition
                                              while receiving FIS */
 
/* @} */

/** @} */ /* end of sata_exceptions_grp */
/** @} */ /* end of sata_grp */

#endif /* __SATA_EXT_H */

