/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#ifndef __SEC_DCL_EXT_H
#define __SEC_DCL_EXT_H


#include "std_ext.h"


/**************************************************************************//**
 @Group         sec_dcl_mdl_grp SEC DCL Application Programming Interface

 @Description   Functions, definitions and enumerators

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         sec_dcl_mdl_initialization_grp Initialization Unit

 @Description   Initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Configuration Parameters
*//***************************************************************************/
typedef struct t_SecDclParam
{
    t_Handle h_Sec      ; /**< SEC object                                     */
    int      memId      ; /**< Memory partition ID                            */
    uint32_t alignment  ; /**< Alignment of each descriptor buffer            */
    uint32_t prefixSize ; /**< Size of data preceding each descriptor buffer  */
    uint32_t postfixSize; /**< Size of data succeeding each descriptor buffer */
}
t_SecDclParam;


/**************************************************************************//**
 @Function      SEC_DCL_Config

 @Description   Allocates the SEC DCL object.

 @Param[in]     p_Param - Configuration parameters:
                          h_Sec       - SEC object.
                          memId       - Memory partition ID.
                          alignment   - Alignment of each descriptor buffer.
                          prefixSize  - Size of data preceding each descriptor buffer.
                          postfixSize - Size of data succeeding each descriptor buffer.

 @Return        Handle to the SEC DCL object on success; NULL otherwise.
*//***************************************************************************/
t_Handle SEC_DCL_Config(t_SecDclParam* p_Param);


/**************************************************************************//**
 @Function      SEC_DCL_Init

 @Description   Initializes the SEC DCL.

 @Param[in]     h_SecDcl - SEC DCL object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_DCL_Init(t_Handle h_SecDcl);


/**************************************************************************//**
 @Function      SEC_DCL_Free

 @Description   Deallocates the SEC DCL object.

 @Param[in]     h_SecDcl - SEC DCL object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_DCL_Free(t_Handle h_SecDcl);


/** @} */ /* end of sec_dcl_mdl_initialization_grp */


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_grp Runtime Control Unit

 @Description   Runtime control routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_job_desc_grp Job Descriptor Construction Interface

 @Description   Job descriptor construction routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Sequence Parameters
*//***************************************************************************/
typedef struct t_SeqParam
{
    bool     sgt   ;
    uint32_t size  ;
    uint8_t* p_Data;
}
t_SeqParam;


/**************************************************************************//**
 @Description   Sequence Job-Descriptor Parameters
*//***************************************************************************/
typedef struct t_SeqJobDescParam
{
     t_Handle   h_ShrDesc;
     t_SeqParam inputSeq ;
     t_SeqParam outputSeq;
}
t_SeqJobDescParam;


/**************************************************************************//**
 @Function      InitSeqJobDesc

 @Description   Creates a Sequence Job-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_JobDescParam - Job-Descriptor parameters.

 @Return        Job-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitSeqJobDesc(t_Handle           h_SecDcl,
                                t_SeqJobDescParam* p_JobDescParam);


/**************************************************************************//**
 @Description   Split-Key Hashing-Algorithm Options
*//***************************************************************************/
typedef enum e_SplitKeyHashAlg
{
    e_SPLIT_KEY_HASH_ALG_MD5     ,
    e_SPLIT_KEY_HASH_ALG_SHA1    ,
    e_SPLIT_KEY_HASH_ALG_SHA2_224,
    e_SPLIT_KEY_HASH_ALG_SHA2_256,
    e_SPLIT_KEY_HASH_ALG_SHA2_384,
    e_SPLIT_KEY_HASH_ALG_SHA2_512
}
e_SplitKeyHashAlg;


/**************************************************************************//**
 @Description   Split-Key Job-Descriptor Parameters
*//***************************************************************************/
typedef struct t_SplitKeyJobDescParam
{
    uint8_t*          p_InputKey ;
    uint8_t*          p_OutputKey;
    e_SplitKeyHashAlg hashAlg    ;
}
t_SplitKeyJobDescParam;


/**************************************************************************//**
 @Function      SEC_DCL_InitSplitKeyJobDesc

 @Description   Creates a Split-Key Job-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_JobDescParam - Job-Descriptor parameters.

 @Return        Job-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitSplitKeyJobDesc(t_Handle                h_SecDcl,
                                     t_SplitKeyJobDescParam* p_JobDescParam);


/** @} */ /* end of sec_dcl_mdl_runtime_control_job_desc_grp */


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_shr_desc_grp Shared Descriptor Construction Interface

 @Description   Shared descriptor construction routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Key Parameters
*//***************************************************************************/
typedef struct t_KeyParam
{
    bool     imm   ;
    uint32_t size  ;
    uint8_t* p_Data;
}
t_KeyParam;


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_shr_desc_ipsec_grp Shared Descriptor Construction Interface for IPSEC

 @Description   Shared descriptor construction functions, definitions and enumerators for IPSEC

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   IPSEC Cryptographic-Algorithm Options
*//***************************************************************************/
typedef enum e_IpsecCryptoAlg
{
    e_IPSEC_CRYPTO_ALG_DES      ,
    e_IPSEC_CRYPTO_ALG_3DES     ,
    e_IPSEC_CRYPTO_ALG_NULL_ENC ,
    e_IPSEC_CRYPTO_ALG_AES_CBC  ,
    e_IPSEC_CRYPTO_ALG_AES_CTR  ,
    e_IPSEC_CRYPTO_ALG_AES_CCM8 ,
    e_IPSEC_CRYPTO_ALG_AES_CCM12,
    e_IPSEC_CRYPTO_ALG_AES_CCM16,
    e_IPSEC_CRYPTO_ALG_AES_GCM8 ,
    e_IPSEC_CRYPTO_ALG_AES_GCM12,
    e_IPSEC_CRYPTO_ALG_AES_GCM16,
    e_IPSEC_CRYPTO_ALG_AES_GMAC ,
    e_IPSEC_CRYPTO_ALG_AES_XTS
}
e_IpsecCryptoAlg;


/**************************************************************************//**
 @Description   IPSEC Authentication-Algorithm Options
*//***************************************************************************/
typedef enum e_IpsecAuthenAlg
{
    e_IPSEC_AUTHEN_ALG_AES_CCM_GCM      ,
    e_IPSEC_AUTHEN_ALG_HMAC_MD5_96      ,
    e_IPSEC_AUTHEN_ALG_HMAC_SHA1_96     ,
    e_IPSEC_AUTHEN_ALG_AES_XCBC_MAC_96  ,
    e_IPSEC_AUTHEN_ALG_HMAC_MD5_128     ,
    e_IPSEC_AUTHEN_ALG_HMAC_SHA1_160    ,
    e_IPSEC_AUTHEN_ALG_AES_CMAC_96      ,
    e_IPSEC_AUTHEN_ALG_HMAC_SHA2_256_128,
    e_IPSEC_AUTHEN_ALG_HMAC_SHA2_384_192,
    e_IPSEC_AUTHEN_ALG_HMAC_SHA2_512_256
}
e_IpsecAuthenAlg;


/**************************************************************************//**
 @Description   IPSEC Shared-Descriptor Parameters
*//***************************************************************************/
typedef struct t_IpsecShrDescParam
{
    uint32_t         dataMoveSize;
    t_KeyParam       cryptoKey   ;
    t_KeyParam       authenKey   ;
    e_IpsecCryptoAlg cryptoAlg   ;
    e_IpsecAuthenAlg authenAlg   ;
}
t_IpsecShrDescParam;


/**************************************************************************//**
 @Description   IPSEC Anti-Replay-Size Options
*//***************************************************************************/
typedef enum e_IpsecArs
{
    e_IPSEC_ARS_0 ,
    e_IPSEC_ARS_32,
    e_IPSEC_ARS_64
}
e_IpsecArs;


/**************************************************************************//**
 @Description   IPSEC Version Options
*//***************************************************************************/
typedef enum e_IpsecVer
{
    e_IPSEC_VER_4,
    e_IPSEC_VER_6
}
e_IpsecVer;


/**************************************************************************//**
 @Description   IPSEC Header Options
*//***************************************************************************/
typedef enum e_IpsecHdr
{
    e_IPSEC_HDR_NONE    ,
    e_IPSEC_HDR_IN_FRAME,
    e_IPSEC_HDR_IN_PARAM
}
e_IpsecHdr;


/**************************************************************************//**
 @Description   IPSEC Mode Options
*//***************************************************************************/
typedef enum e_IpsecMode
{
    e_IPSEC_MODE_TRANSPORT,
    e_IPSEC_MODE_TUNNEL
}
e_IpsecMode;


/**************************************************************************//**
 @Description   IPSEC Type Options
*//***************************************************************************/
typedef enum e_IpsecType
{
    e_IPSEC_TYPE_CBC,
    e_IPSEC_TYPE_CTR,
    e_IPSEC_TYPE_CCM,
    e_IPSEC_TYPE_GCM
}
e_IpsecType;


/**************************************************************************//**
 @Description   IPSEC-Encapsulation Parameters
*//***************************************************************************/
typedef struct t_IpsecEncapParam
{
    bool        copyDf   ;
    bool        decTtl   ;
    bool        enCksm   ;
    bool        copyDs   ;
    bool        randIv   ;
    bool        extSeqNum;
    uint8_t     nextHdr  ;
    uint8_t     nhOffset ;
    uint64_t    seqNum   ;
    uint32_t    spi      ;
    uint16_t    ipHdrLen ;
    uint8_t*    p_IpHdr  ;
    e_IpsecVer  ipsecVer ;
    e_IpsecHdr  ipsecHdr ;
    e_IpsecMode ipsecMode;
    e_IpsecType ipsecType;

    union
    {
        struct
        {
            uint8_t iv[16];
        }
        cbc;

        struct
        {
            uint32_t nonce    ;
            uint32_t initCount;
            uint8_t  iv[8]    ;
        }
        ctr;

        struct
        {
            uint32_t salt     ;
            uint8_t  b0Flags  ;
            uint8_t  ctrFlags ;
            uint16_t initCount;
            uint8_t  iv[8]    ;
        }
        ccm;

        struct
        {
            uint32_t salt ;
            uint8_t  iv[8];
        }
        gcm;
    }
    alg;
}
t_IpsecEncapParam;


/**************************************************************************//**
 @Description   IPSEC-Decapsulation Parameters
*//***************************************************************************/
typedef struct t_IpsecDecapParam
{
    bool        decTtl   ;
    bool        copyDs   ;
    bool        enCksm   ;
    bool        extSeqNum;
    bool        outFmt   ;
    uint16_t    ipHdrLen ;
    uint8_t     nhOffset ;
    uint64_t    seqNum   ;
    e_IpsecArs  ipsecArs ;
    e_IpsecVer  ipsecVer ;
    e_IpsecMode ipsecMode;
    e_IpsecType ipsecType;

    union
    {
        struct
        {
            uint32_t salt     ;
            uint32_t initCount;
        }
        ctr;

        struct
        {
            uint32_t salt     ;
            uint8_t  ivFlags  ;
            uint8_t  ctrFlags ;
            uint16_t initCount;
        }
        ccm;

        struct
        {
            uint32_t salt;
        }
        gcm;
    }
    alg;
}
t_IpsecDecapParam;


/**************************************************************************//**
 @Function      SEC_DCL_InitIpsecEncapShrDesc

 @Description   Creates an IPSEC-Encapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_EncapParam   - Encapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitIpsecEncapShrDesc(t_Handle             h_SecDcl,
                                       t_IpsecShrDescParam* p_ShrDescParam,
                                       t_IpsecEncapParam*   p_EncapParam);


/**************************************************************************//**
 @Function      SEC_DCL_InitIpsecDecapShrDesc

 @Description   Creates an IPSEC-Decapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_DecapParam   - Decapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitIpsecDecapShrDesc(t_Handle             h_SecDcl,
                                       t_IpsecShrDescParam* p_ShrDescParam,
                                       t_IpsecDecapParam*   p_DecapParam);


/** @} */ /* end of sec_dcl_mdl_runtime_control_shr_desc_ipsec_grp */


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_shr_desc_dtls_grp Shared Descriptor Construction Interface for DTLS

 @Description   Shared descriptor construction functions, definitions and enumerators for DTLS

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   DTLS Cipher-Algorithm Options
*//***************************************************************************/
typedef enum e_DtlsCipherAlg
{
    e_DTLS_CIPHER_ALG_RC4_40_MD5          ,
    e_DTLS_CIPHER_ALG_RC4_128_MD5         ,
    e_DTLS_CIPHER_ALG_RC4_128_SHA         ,
    e_DTLS_CIPHER_ALG_DES40_CBC_SHA       ,
    e_DTLS_CIPHER_ALG_DES_CBC_SHA         ,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA    ,
    e_DTLS_CIPHER_ALG_DES_CBC_MD5         ,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_MD5    ,
    e_DTLS_CIPHER_ALG_RC4_40_SHA          ,
    e_DTLS_CIPHER_ALG_DES40_CBC_MD5       ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA     ,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA     ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA_256 ,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA_256 ,
    e_DTLS_CIPHER_ALG_AES_192_CBC_SHA_160 ,
    e_DTLS_CIPHER_ALG_AES_192_CBC_SHA_384 ,
    e_DTLS_CIPHER_ALG_AES_192_CBC_SHA_224 ,
    e_DTLS_CIPHER_ALG_AES_192_CBC_SHA_512 ,
    e_DTLS_CIPHER_ALG_AES_192_CBC_SHA_256 ,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA_160,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA_384,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA_224,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA_512,
    e_DTLS_CIPHER_ALG_3DES_EDE_CBC_SHA_256,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA_160 ,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA_384 ,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA_224 ,
    e_DTLS_CIPHER_ALG_AES_256_CBC_SHA_512 ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA_160 ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA_384 ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA_224 ,
    e_DTLS_CIPHER_ALG_AES_128_CBC_SHA_512
}
e_DtlsCipherAlg;


/**************************************************************************//**
 @Description   DTLS Shared-Descriptor Parameters
*//***************************************************************************/
typedef struct t_DtlsShrDescParam
{
    uint32_t        dataMoveSize;
    t_KeyParam      cryptoKey   ;
    t_KeyParam      authenKey   ;
    e_DtlsCipherAlg cipherAlg   ;
}
t_DtlsShrDescParam;


/**************************************************************************//**
 @Description   DTLS Anti-Replay-Size Options
*//***************************************************************************/
typedef enum e_DtlsArs
{
    e_DTLS_ARS_0 ,
    e_DTLS_ARS_32,
    e_DTLS_ARS_64
}
e_DtlsArs;


/**************************************************************************//**
 @Description   DTLS-Encapsulation Parameters
*//***************************************************************************/
typedef struct t_DtlsEncapParam
{
    bool     wbIv   ;
    bool     expIv  ;
    uint8_t  type   ;
    uint16_t version;
    uint16_t epoch  ;
    uint64_t seqNum ;
    uint8_t  iv[16] ;
}
t_DtlsEncapParam;


/**************************************************************************//**
 @Description   DTLS-Decapsulation Parameters
*//***************************************************************************/
typedef struct t_DtlsDecapParam
{
    bool      wbIv   ;
    bool      expIv  ;
    uint16_t  epoch  ;
    uint64_t  seqNum ;
    uint8_t   iv[16] ;
    e_DtlsArs dtlsArs;
}
t_DtlsDecapParam;


/**************************************************************************//**
 @Function      SEC_DCL_InitDtlsEncapShrDesc

 @Description   Creates a DTLS-Encapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_EncapParam   - Encapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitDtlsEncapShrDesc(t_Handle            h_SecDcl,
                                      t_DtlsShrDescParam* p_ShrDescParam,
                                      t_DtlsEncapParam*   p_EncapParam);


/**************************************************************************//**
 @Function      SEC_DCL_InitDtlsDecapShrDesc

 @Description   Creates a DTLS-Decapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_DecapParam   - Decapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitDtlsDecapShrDesc(t_Handle            h_SecDcl,
                                      t_DtlsShrDescParam* p_ShrDescParam,
                                      t_DtlsDecapParam*   p_DecapParam);


/** @} */ /* end of sec_dcl_mdl_runtime_control_shr_desc_dtls_grp */


/**************************************************************************//**
 @Group         sec_dcl_mdl_runtime_control_shr_desc_macsec_grp Shared Descriptor Construction Interface for MACSEC

 @Description   Shared descriptor construction functions, definitions and enumerators for MACSEC

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   MACSEC Cipher-Algorithm Options
*//***************************************************************************/
typedef enum e_MacsecCipherAlg
{
    e_MACSEC_CIPHER_ALG_AES_GCM_16
}
e_MacsecCipherAlg;


/**************************************************************************//**
 @Description   MACSEC Shared-Descriptor Parameters
*//***************************************************************************/
typedef struct t_MacsecShrDescParam
{
    uint32_t          dataMoveSize;
    t_KeyParam        cipherKey   ;
    e_MacsecCipherAlg cipherAlg   ;
}
t_MacsecShrDescParam;


/**************************************************************************//**
 @Description   MACSEC-Encapsulation Parameters
*//***************************************************************************/
typedef struct t_MacsecEncapParam
{
    uint16_t aadLength;
    uint64_t sci      ;
    uint16_t etherType;
    uint8_t  tciAn    ;
    uint32_t pn       ;
}
t_MacsecEncapParam;


/**************************************************************************//**
 @Description   MACSEC-Decapsulation Parameters
*//***************************************************************************/
typedef struct t_MacsecDecapParam
{
    bool     enAr     ;
    uint16_t aadLength;
    uint64_t sci      ;
    uint8_t  arLen    ;
    uint32_t pn       ;
}
t_MacsecDecapParam;


/**************************************************************************//**
 @Function      SEC_DCL_InitMacsecEncapShrDesc

 @Description   Creates a MACSEC-Encapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_EncapParam   - Encapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitMacsecEncapShrDesc(t_Handle              h_SecDcl,
                                        t_MacsecShrDescParam* p_ShrDescParam,
                                        t_MacsecEncapParam*   p_EncapParam);


/**************************************************************************//**
 @Function      SEC_DCL_InitMacsecDecapShrDesc

 @Description   Creates a MACSEC-Decapsulation Shared-Descriptor.

 @Param[in]     h_SecDcl       - SEC DCL object.
 @Param[in]     p_ShrDescParam - Shared-Descriptor parameters.
 @Param[in]     p_DecapParam   - Decapsulation parameters.

 @Return        Shared-Descriptor object.
*//***************************************************************************/
t_Handle SEC_DCL_InitMacsecDecapShrDesc(t_Handle              h_SecDcl,
                                        t_MacsecShrDescParam* p_ShrDescParam,
                                        t_MacsecDecapParam*   p_DecapParam);


/** @} */ /* end of sec_dcl_mdl_runtime_control_shr_desc_macsec_grp */


/** @} */ /* end of sec_dcl_mdl_runtime_control_shr_desc_grp */


/**************************************************************************//**
 @Function      SEC_DCL_GetDescPrefix

 @Description   Returns a pointer to the data preceding a descriptor buffer.

 @Param[in]     h_Desc - Descriptor object.

 @Return        Pointer to the data preceding the descriptor buffer.
*//***************************************************************************/
uint8_t* SEC_DCL_GetDescPrefix(t_Handle h_Desc);


/**************************************************************************//**
 @Function      SEC_DCL_GetDescBuffer

 @Description   Returns a pointer to a descriptor buffer.

 @Param[in]     h_Desc - Descriptor object.

 @Return        Pointer to the descriptor buffer.
*//***************************************************************************/
uint8_t* SEC_DCL_GetDescBuffer(t_Handle h_Desc);


/**************************************************************************//**
 @Function      SEC_DCL_GetDescPostfix

 @Description   Returns a pointer to the data succeeding a descriptor buffer.

 @Param[in]     h_Desc - Descriptor object.

 @Return        Pointer to the data succeeding the descriptor buffer.
*//***************************************************************************/
uint8_t* SEC_DCL_GetDescPostfix(t_Handle h_Desc);


/**************************************************************************//**
 @Function      SEC_DCL_GetBufferSize

 @Description   Returns the size of a descriptor buffer.

 @Param[in]     h_Desc - Descriptor object.

 @Return        Size of the descriptor buffer.
*//***************************************************************************/
uint32_t SEC_DCL_GetBufferSize(t_Handle h_Desc);


/**************************************************************************//**
 @Function      SEC_DCL_FreeDesc

 @Description   Deallocates a Descriptor object.

 @Param[in]     h_Desc - Descriptor object.

 @Return        None.
*//***************************************************************************/
void SEC_DCL_FreeDesc(t_Handle h_Desc);


/** @} */ /* end of sec_dcl_mdl_runtime_control_grp */


/** @} */ /* end of sec_dcl_mdl_grp */


#endif /* __SEC_DCL_EXT_H */
