/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          sec_ext.h

 @Description   SEC external structures and definitions.
*//***************************************************************************/
#ifndef __SEC_EXT_H
#define __SEC_EXT_H


#include "std_ext.h"


/**************************************************************************//**
 @Group         sec_mdl_grp SEC Application Programming Interface

 @Description   Functions, definitions and enumerators

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Collection   General Definitions
*//***************************************************************************/
#define SEC_MAX_NUM_OF_QI_JOBS 15 /**< Maximum number of Qman Interface jobs */
#define SEC_TOTAL_NUM_OF_JQS    4 /**< Total number of job queues            */
/* @} */


/**************************************************************************//**
 @Group         sec_mdl_initialization_grp SEC Initialization Unit

 @Description   Initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Configuration Parameters
*//***************************************************************************/
typedef struct t_SecParam
{
    uint32_t guestId ; /**< Guest ID     */
    uint64_t baseAddr; /**< Base address */
}
t_SecParam;


/**************************************************************************//**
 @Function      SEC_Config

 @Description   Allocates the SEC object.

 @Param[in]     p_Param - Configuration parameters:
                          guestId  - Guest ID.
                          baseAddr - Base address.

 @Return        Handle to the SEC object on success; NULL otherwise.
*//***************************************************************************/
t_Handle SEC_Config(t_SecParam* p_Param);


/**************************************************************************//**
 @Function      SEC_Init

 @Description   Initializes the SEC General registers (Master user only).

 @Param[in]     h_Sec - SEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_Init(t_Handle h_Sec);


/**************************************************************************//**
 @Function      SEC_Free

 @Description   Deallocates the SEC object.

 @Param[in]     h_Sec - SEC object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_Free(t_Handle h_Sec);


/**************************************************************************//**
 @Group         sec_mdl_advance_config_grp SEC Advanced Configuration Unit

 @Description   Advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Soft Reset Options
*//***************************************************************************/
typedef enum e_SecSoftReset
{
    e_SEC_SOFT_RESET_NEVER  , /**< Soft reset never initiated     */
    e_SEC_SOFT_RESET_ON_INIT  /**< Soft reset upon initialization */
}
e_SecSoftReset;


/**************************************************************************//**
 @Function      SEC_ConfigSoftReset

 @Description   Configures the SEC Soft-Reset mode.

 @Param[in]     h_Sec     - SEC object.
 @Param[in]     softReset - Soft reset (default value = [e_SEC_SOFT_RESET_ON_INIT]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Only the Master user can configure the Soft-Reset mode.
*//***************************************************************************/
t_Error SEC_ConfigSoftReset(t_Handle       h_Sec,
                            e_SecSoftReset softReset);


/**************************************************************************//**
 @Description   Address Space Options
*//***************************************************************************/
typedef enum e_SecAddrSpace
{
    e_SEC_ADDR_SPACE_32_BIT, /**< 32-bit address space */
    e_SEC_ADDR_SPACE_64_BIT  /**< 64-bit address space */
}
e_SecAddrSpace;


/**************************************************************************//**
 @Function      SEC_ConfigAddrSpace

 @Description   Configures the SEC Address-Space mode.

                If any core uses 64-bit address space, then 64-bit address space configuration must be applied.
                If all cores use 32-bit address space, then 32-bit address space configuration may  be applied.

 @Param[in]     h_Sec     - SEC object.
 @Param[in]     addrSpace - Address space (default value = [e_SEC_ADDR_SPACE_64_BIT]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Only the Master user can configure the Address-Space mode.
*//***************************************************************************/
t_Error SEC_ConfigAddrSpace(t_Handle       h_Sec,
                            e_SecAddrSpace addrSpace);


/**************************************************************************//**
 @Description   Byte Swap Options
*//***************************************************************************/
typedef enum e_SecByteSwap
{
    e_SEC_BYTE_SWAP_NONE   , /**< No byte swap                                                                                        */
    e_SEC_BYTE_SWAP_1_IN_2 , /**< 00|01 02|03 04|05 06|07 08|09 0A|0B 0C|0D 0E|0F ==> 01|00 03|02 05|04 07|06 09|08 0B|0A 0D|0C 0F|0E */
    e_SEC_BYTE_SWAP_1_IN_4 , /**< 00|01|02|03 04|05|06|07 08|09|0A|0B 0C|0D|0E|0F ==> 03|02|01|00 07|06|05|04 0B|0A|09|08 0F|0E|0D|0C */
    e_SEC_BYTE_SWAP_1_IN_8 , /**< 00|01|02|03|04|05|06|07 08|09|0A|0B|0C|0D|0E|0F ==> 07|06|05|04|03|02|01|00 0F|0E|0D|0C|0B|0A|09|08 */
    e_SEC_BYTE_SWAP_1_IN_16, /**< 00|01|02|03|04|05|06|07|08|09|0A|0B|0C|0D|0E|0F ==> 0F|0E|0D|0C|0B|0A|09|08|07|06|05|04|03|02|01|00 */
    e_SEC_BYTE_SWAP_2_IN_4 , /**< 00_01|02_03 04_05|06_07 08_09|0A_0B 0C_0D|0E_0F ==> 02_03|00_01 06_07|04_05 0A_0B|08_09 0E_0F|0C_0D */
    e_SEC_BYTE_SWAP_2_IN_8 , /**< 00_01|02_03|04_05|06_07 08_09|0A_0B|0C_0D|0E_0F ==> 06_07|04_05|02_03|00_01 0E_0F|0C_0D|0A_0B|08_09 */
    e_SEC_BYTE_SWAP_2_IN_16, /**< 00_01|02_03|04_05|06_07|08_09|0A_0B|0C_0D|0E_0F ==> 0E_0F|0C_0D|0A_0B|08_09|06_07|04_05|02_03|00_01 */
    e_SEC_BYTE_SWAP_4_IN_8 , /**< 00_01_02_03|04_05_06_07 08_09_0A_0B|0C_0D_0E_0F ==> 04_05_06_07|00_01_02_03 0C_0D_0E_0F|08_09_0A_0B */
    e_SEC_BYTE_SWAP_4_IN_16, /**< 00_01_02_03|04_05_06_07|08_09_0A_0B|0C_0D_0E_0F ==> 0C_0D_0E_0F|08_09_0A_0B|04_05_06_07|00_01_02_03 */
    e_SEC_BYTE_SWAP_8_IN_16  /**< 00_01_02_03_04_05_06_07|08_09_0A_0B_0C_0D_0E_0F ==> 08_09_0A_0B_0C_0D_0E_0F|00_01_02_03_04_05_06_07 */
}
e_SecByteSwap;


/** @} */ /* end of sec_mdl_advance_config_grp */
/** @} */ /* end of sec_mdl_initialization_grp */


/**************************************************************************//**
 @Group         sec_mdl_runtime_control_grp SEC Control Unit

 @Description   Runtime control routines

 @{
*//***************************************************************************/


/**************************************************************************//*
 @Function      SEC_GetPtrSize

 @Description   Returns the SEC pointer size.

 @Param[in]     h_Sec - SEC object.

 @Return        SEC pointer size.
*//***************************************************************************/
uint32_t SEC_GetPtrSize(t_Handle h_Sec);


/**************************************************************************//*
 @Function      SEC_GetBurstSize

 @Description   Returns the SEC burst size.

 @Param[in]     h_Sec - SEC object.

 @Return        SEC burst size.
*//***************************************************************************/
uint32_t SEC_GetBurstSize(t_Handle h_Sec);


/**************************************************************************//*
 @Function      SEC_GetMajRevNum

 @Description   Returns the SEC major-revision number.

 @Param[in]     h_Sec - SEC object.

 @Return        SEC major-revision number.
*//***************************************************************************/
uint8_t SEC_GetMajRevNum(t_Handle h_Sec);


/**************************************************************************//*
 @Function      SEC_GetMinRevNum

 @Description   Returns the SEC minor-revision number.

 @Param[in]     h_Sec - SEC object.

 @Return        SEC minor-revision number.
*//***************************************************************************/
uint8_t SEC_GetMinRevNum(t_Handle h_Sec);


/**************************************************************************//*
 @Function      SEC_InitJqLiodnReg

 @Description   Initializes the LIODN register of a given job queue.

 @Param[in]     h_Sec          - SEC object.
 @Param[in]     jobQueueId     - Job queue ID.
 @Param[in]     seqLiodnVal    - Sequential LIODN value.
 @Param[in]     nonSeqLiodnVal - Non-sequential LIODN value.
 @Param[in]     allowMtdJobs   - Allow MTD jobs.

 @Return        None.
*//***************************************************************************/
t_Error SEC_InitJqLiodnReg(t_Handle h_Sec,
                           uint8_t  jobQueueId,
                           uint16_t seqLiodnVal,
                           uint16_t nonSeqLiodnVal,
                           bool     allowMtdJobs);


/**************************************************************************//*
 @Function      SEC_CalcByteSwapFlags

 @Description   Calculates a flag combination that produces a given byte-swap.

 @Param[in]     byteSwap  - Required byte-swap.
 @Param[in]     swap1In4  - A flag for reversing 4 units, where each unit is 1-byte long.
 @Param[in]     swap2In4  - A flag for reversing 2 units, where each unit is 2-byte long.
 @Param[in]     swap4In8  - A flag for reversing 2 units, where each unit is 4-byte long.
 @Param[in]     swap8In16 - A flag for reversing 2 units, where each unit is 8-byte long.

 @Return        The flag combination that produces the given byte-swap.
*//***************************************************************************/
uint64_t SEC_CalcByteSwapFlags(e_SecByteSwap byteSwap,
                               uint64_t      swap1In4,
                               uint64_t      swap2In4,
                               uint64_t      swap4In8,
                               uint64_t      swap8In16);


/**************************************************************************//**
 @Collection   Job Status Sources
*//***************************************************************************/
#define SEC_JOB_STATUS_SOURCE_NONE 0x0 /**< No status source                          */
#define SEC_JOB_STATUS_SOURCE_CCB  0x2 /**< Cryptographic Control Block status source */
#define SEC_JOB_STATUS_SOURCE_JHU  0x3 /**< Jump Halt User status source              */
#define SEC_JOB_STATUS_SOURCE_DECO 0x4 /**< Descriptor Controller status source       */
#define SEC_JOB_STATUS_SOURCE_JQ   0x6 /**< Job Queue status source                   */
#define SEC_JOB_STATUS_SOURCE_JHT  0x7 /**< Jump Halt Test status source              */
/* @} */


/**************************************************************************//**
 @Collection   Cryptographic Control Block Descriptor Errors
*//***************************************************************************/
#define SEC_CCB_DESC_ERROR_NONE              0x0 /**< No error                           */
#define SEC_CCB_DESC_ERROR_MODE              0x1 /**< Mode error                         */
#define SEC_CCB_DESC_ERROR_DATA_SIZE         0x2 /**< Data size error                    */
#define SEC_CCB_DESC_ERROR_KEY_SIZE          0x3 /**< Key size error                     */
#define SEC_CCB_DESC_ERROR_PKHA_A_MEM_SIZE   0x4 /**< PKHA A memory size error           */
#define SEC_CCB_DESC_ERROR_PKHA_B_MEM_SIZE   0x5 /**< PKHA B memory size error           */
#define SEC_CCB_DESC_ERROR_DATA_OUT_OF_SEQ   0x6 /**< Data arrived out of sequence error */
#define SEC_CCB_DESC_ERROR_PKHA_DIV_BY_ZERO  0x7 /**< PKHA divide-by-zero error          */
#define SEC_CCB_DESC_ERROR_PKHA_MODULES_EVEN 0x8 /**< PKHA modulus even error            */
#define SEC_CCB_DESC_ERROR_DES_KEY_PARITY    0x9 /**< DES key parity error               */
#define SEC_CCB_DESC_ERROR_ICV_CHECK         0xA /**< ICV check failed                   */
#define SEC_CCB_DESC_ERROR_HARDWARE          0xB /**< Hardware error                     */
#define SEC_CCB_DESC_ERROR_CCM_AAD_SIZE      0xC /**< Unsupported CCM AAD size           */
#define SEC_CCB_DESC_ERROR_CHA_NOT_RESET     0xD /**< Class 1 CHA not reset              */
#define SEC_CCB_DESC_ERROR_CHA_COMB_SELECTED 0xE /**< Invalid CHA combination selected   */
#define SEC_CCB_DESC_ERROR_CHA_SELECTED      0xF /**< Invalid CHA selected               */
/* @} */


/**************************************************************************//**
 @Collection   Cryptographic Control Block Algorithms
*//***************************************************************************/
#define SEC_CCB_ALGORITHM_AES          0x1 /**< AES            */
#define SEC_CCB_ALGORITHM_DES_3DES     0x2 /**< DES or 3DES    */
#define SEC_CCB_ALGORITHM_ARC4         0x3 /**< ARC4           */
#define SEC_CCB_ALGORITHM_MD5_SHA      0x4 /**< MD5 or SHA     */
#define SEC_CCB_ALGORITHM_RNG          0x5 /**< RNG            */
#define SEC_CCB_ALGORITHM_SNOW_F8      0x6 /**< SNOW f8        */
#define SEC_CCB_ALGORITHM_KASUMI_F8_F9 0x7 /**< Kasumi f8 / f9 */
#define SEC_CCB_ALGORITHM_PUBLIC_KEY   0x8 /**< Public Key     */
#define SEC_CCB_ALGORITHM_CRC          0x9 /**< CRC            */
#define SEC_CCB_ALGORITHM_SNOW_F9      0xA /**< SNOW f9        */
/* @} */


/**************************************************************************//**
 @Collection   Descriptor Controller Descriptor Errors
*//***************************************************************************/
#define SEC_DECO_DESC_ERROR_NONE               0x00 /**< No error                              */
#define SEC_DECO_DESC_ERROR_SGT_LENGTH         0x01 /**< SGT length error                      */
#define SEC_DECO_DESC_ERROR_JQ_CONTROL         0x03 /**< JQ control error                      */
#define SEC_DECO_DESC_ERROR_CMD                0x04 /**< Invalid command                       */
#define SEC_DECO_DESC_ERROR_KEY_CMD            0x06 /**< Invalid key command                   */
#define SEC_DECO_DESC_ERROR_LOAD_CMD           0x07 /**< Invalid load command                  */
#define SEC_DECO_DESC_ERROR_STORE_CMD          0x08 /**< Invalid store command                 */
#define SEC_DECO_DESC_ERROR_OPERATION_CMD      0x09 /**< Invalid operation command             */
#define SEC_DECO_DESC_ERROR_FIFO_LOAD_CMD      0x0A /**< Invalid fifo-load command             */
#define SEC_DECO_DESC_ERROR_FIFO_STORE_CMD     0x0B /**< Invalid fifo-store command            */
#define SEC_DECO_DESC_ERROR_MOVE_CMD           0x0C /**< Invalid move command                  */
#define SEC_DECO_DESC_ERROR_JUMP_CMD           0x0D /**< Invalid jump command                  */
#define SEC_DECO_DESC_ERROR_MATH_CMD           0x0E /**< Invalid math command                  */
#define SEC_DECO_DESC_ERROR_SIGNED_HASH_CMD    0x0F /**< Invalid signed-hash command           */
#define SEC_DECO_DESC_ERROR_SEQUENCE_CMD       0x10 /**< Invalid sequence command              */
#define SEC_DECO_DESC_ERROR_SKIP_DATA_TYPE     0x11 /**< Skip data type invalid                */
#define SEC_DECO_DESC_ERROR_SHARED_HEADER      0x12 /**< Shared header error                   */
#define SEC_DECO_DESC_ERROR_HEADER             0x13 /**< Header error                          */
#define SEC_DECO_DESC_ERROR_BURSTER            0x14 /**< Burster error                         */
#define SEC_DECO_DESC_ERROR_CONTEXT_REG_LEN    0x15 /**< Context register length error         */
#define SEC_DECO_DESC_ERROR_DMA                0x16 /**< DMA error                             */
#define SEC_DECO_DESC_ERROR_JQ_RESET           0x1A /**< Job failed due to job queue reset     */
#define SEC_DECO_DESC_ERROR_FAIL_MODE          0x1B /**< Job failed due to fail mode           */
#define SEC_DECO_DESC_ERROR_WATCHDOG_TIMEOUT   0x1C /**< Watchdog timer timeout error          */
#define SEC_DECO_DESC_ERROR_KEY_COPY           0x1D /**< Key copy failure                      */
#define SEC_DECO_DESC_ERROR_DATA_COPY          0x1E /**< Data copy failure                     */
#define SEC_DECO_DESC_ERROR_LIODN              0x1F /**< LIODN error                           */
#define SEC_DECO_DESC_ERROR_RESET              0x20 /**< Reset initiated via the DRR           */
#define SEC_DECO_DESC_ERROR_NONCE              0x21 /**< Nonce error                           */
#define SEC_DECO_DESC_ERROR_META_DATA_SIZE     0x22 /**< Meta data too large                   */
#define SEC_DECO_DESC_ERROR_DNR                0x80 /**< DNR (do not run) error                */
#define SEC_DECO_DESC_ERROR_PROTOCOL_CMD       0x81 /**< Undefined protocol command            */
#define SEC_DECO_DESC_ERROR_PDB_SETTIGS        0x82 /**< Invalid setting in PDB                */
#define SEC_DECO_DESC_ERROR_LATE               0x83 /**< Anti-replay late error                */
#define SEC_DECO_DESC_ERROR_REPLAY             0x84 /**< Anti-replay replay error              */
#define SEC_DECO_DESC_ERROR_SEQ_NUM_OVERFLOW   0x85 /**< Sequence number overflow              */
#define SEC_DECO_DESC_ERROR_SIGVER_SIGNATURE   0x86 /**< Sigver invalid signature              */
#define SEC_DECO_DESC_ERROR_DSA_SIGN           0x87 /**< DSA sign illegal test descriptor      */
#define SEC_DECO_DESC_ERROR_PROTOCOL_FORMAT    0x88 /**< Protocol format error                 */
#define SEC_DECO_DESC_ERROR_PROTOCOL_SIZE      0x89 /**< Protocol size error                   */
#define SEC_DECO_DESC_ERROR_BLOB_MODE          0xC1 /**< Undefined blob mode                   */
#define SEC_DECO_DESC_ERROR_BLOB_SEC_MEM_MODE  0xC2 /**< Secure memory blob mode error         */
#define SEC_DECO_DESC_ERROR_BLOB_KEY_SIZE      0xC4 /**< Black blob key or input size error    */
#define SEC_DECO_DESC_ERROR_BLOB_KEY_DEST      0xC5 /**< Invalid key destination               */
#define SEC_DECO_DESC_ERROR_BLOB_TRST_SEC_MODE 0xC8 /**< Trusted/Secure mode error             */
#define SEC_DECO_DESC_ERROR_TTL_OR_HOP_LIMIT   0xF0 /**< IPsec TTL or hop limit field 0        */
#define SEC_DECO_DESC_ERROR_3GPP_HFN           0xF1 /**< 3GPP HFN matches or exceeds threshold */
/* @} */


/**************************************************************************//**
 @Collection   Job Queue Descriptor Errors
*//***************************************************************************/
#define SEC_JQ_DESC_ERROR_NONE           0x00 /**< No error                             */
#define SEC_JQ_DESC_ERROR_READ_DESC_ADDR 0x1E /**< Error reading the descriptor address */
#define SEC_JQ_DESC_ERROR_READ_DESC      0x1F /**< Error reading the descriptor         */
/* @} */


/**************************************************************************//**
 @Collection   Jump Halt Test Errors
*//***************************************************************************/
#define SEC_JHT_ERROR_PKHA_IS_ZERO  0x80 /**< The result of a PKHA operation is zero            */
#define SEC_JHT_ERROR_PKHA_GCD_ONE  0x40 /**< The greatest common divisor of two numbers is one */
#define SEC_JHT_ERROR_PKHA_IS_PRIME 0x20 /**< The given number is probably prime                */
#define SEC_JHT_ERROR_MATH_N        0x08 /**< The result is negative                            */
#define SEC_JHT_ERROR_MATH_Z        0x04 /**< The result of a math operation is zero            */
#define SEC_JHT_ERROR_MATH_C        0x02 /**< The math operation resulted in a carry or borrow  */
#define SEC_JHT_ERROR_MATH_NV       0x01 /**< A combination of the sign bit and overflow        */
/* @} */


/**************************************************************************//**
 @Description   Job Status
*//***************************************************************************/
typedef struct t_SecJobStatus
{
    uint8_t sourceId; /**< Source ID */

    union
    {
        struct /**< Cryptographic Control Block Details */
        {
            bool    jump     ; /**< The descriptor made a jump to another descriptor */
            uint8_t descIndex; /**< Index into the failing descriptor                */
            uint8_t algorithm; /**< Algorithm that generated the error               */
            uint8_t descError; /**< Descriptor error                                 */
        }
        ccbDetails;

        struct /**< Jump Halt User Details */
        {
            bool    jump     ; /**< The descriptor made a jump to another descriptor */
            uint8_t descIndex; /**< Index into the failing descriptor                */
            uint8_t status   ; /**< User status field from Jump command              */
        }
        jhuDetails;

        struct /**< Descriptor Controller Details */
        {
            bool    jump     ; /**< The descriptor made a jump to another descriptor */
            uint8_t descIndex; /**< Index into the failing descriptor                */
            uint8_t descError; /**< Descriptor error                                 */
        }
        decoDetails;

        struct /**< Job Queue Details */
        {
            uint8_t numOfDars; /**< Number of descriptor addresses requested */
            uint8_t descError; /**< Descriptor error                         */
        }
        jqDetails;

        struct /**< Jump Halt Test Details */
        {
            bool    jump     ; /**< The descriptor made a jump to another descriptor */
            uint8_t descIndex; /**< Index into the failing descriptor                */
            uint8_t condCodes; /**< Condition codes field from Jump command          */
        }
        jhtDetails;
    };
}
t_SecJobStatus;


/**************************************************************************//**
 @Function      SEC_DecodeJobStatus

 @Description   Decodes an encoded job-status.

 @Param[in]     jobStatus - Encoded job-status.

 @Return        Decoded job-status.

 @Cautions      Encoded job-status format is not asserted.
*//***************************************************************************/
t_SecJobStatus SEC_DecodeJobStatus(uint32_t jobStatus);


#if (DEBUG_ERRORS > 0)


/**************************************************************************//**
 @Function      SEC_PrintJobStatus

 @Description   Decodes an encoded job-status and prints it.

 @Param[in]     jobStatus - Encoded job-status.

 @Return        None.

 @Cautions      Encoded job-status format is not asserted.
*//***************************************************************************/
void SEC_PrintJobStatus(uint32_t jobStatus);


/**************************************************************************//**
 @Function      SEC_DumpRegisters

 @Description   Dumps the contents of the SEC General registers.

 @Param[in]     h_Sec - SEC object.

 @Return        None.
*//***************************************************************************/
void SEC_DumpRegisters(t_Handle h_Sec);


#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of sec_mdl_runtime_control_grp */
/** @} */ /* end of sec_mdl_grp */


#endif /* __SEC_EXT_H */
