/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          sec_qi_ext.h

 @Description   SEC QI external structures and definitions.
*//***************************************************************************/
#ifndef __SEC_QI_EXT_H
#define __SEC_QI_EXT_H


#include "sec_ext.h"


/**************************************************************************//**
 @Group         sec_qi_mdl_grp SEC QI Application Programming Interface

 @Description   Functions, definitions and enumerators

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         sec_qi_mdl_initialization_grp SEC QI Initialization Unit

 @Description   Initialization routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Configuration Parameters
*//***************************************************************************/
typedef struct t_SecQiParam
{
    t_Handle h_Sec          ; /**< SEC object                       */
    uint64_t baseAddr       ; /**< Base address                     */
    uint16_t seqLiodnBase   ; /**< Sequential LIODN base            */
    uint16_t nonSeqLiodnBase; /**< Non-sequential LIODN base        */
    uint16_t nonSeqLiodnMask; /**< Non-sequential LIODN offset mask */
}
t_SecQiParam;


/**************************************************************************//**
 @Function      SEC_QI_Config

 @Description   Allocates the SEC QI object.

 @Param[in]     p_Param - Pointer to SEC QI Configuration parameters.

 @Return        Handle to the SEC QI object on success; NULL otherwise.
*//***************************************************************************/
t_Handle SEC_QI_Config(t_SecQiParam* p_Param);


/**************************************************************************//**
 @Function      SEC_QI_Init

 @Description   Initializes the SEC QI registers.

 @Param[in]     h_SecQi - SEC QI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_Init(t_Handle h_SecQi);


/**************************************************************************//**
 @Function      SEC_QI_Free

 @Description   Deallocates the SEC QI object.

 @Param[in]     h_SecQi - SEC QI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_Free(t_Handle h_SecQi);


/**************************************************************************//**
 @Group         sec_qi_mdl_advance_config_grp SEC QI Advanced Configuration Unit

 @Description   Advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      SEC_QI_ConfigStopOnError

 @Description   Configures the SEC QI whether or not to stop on error.

 @Param[in]     h_SecQi     - SEC QI object.
 @Param[in]     stopOnError - Stop on error (default value = [FALSE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_ConfigStopOnError(t_Handle h_SecQi,
                                 bool     stopOnError);


/**************************************************************************//**
 @Description   Frame Color Options
*//***************************************************************************/
typedef enum e_SecQiFrameColor
{
    e_SEC_QI_FRAME_COLOR_GREEN   , /**< Green    */
    e_SEC_QI_FRAME_COLOR_YELLOW  , /**< Yellow   */
    e_SEC_QI_FRAME_COLOR_RED     , /**< Red      */
    e_SEC_QI_FRAME_COLOR_OVERRIDE  /**< Override */
}
e_SecQiFrameColor;


/**************************************************************************//**
 @Description   Frame Count Options
*//***************************************************************************/
typedef enum e_SecQiFrameCount
{
    e_SEC_QI_FRAME_COUNT_MAXIMUM_1, /**< At most one frame  */
    e_SEC_QI_FRAME_COUNT_MAXIMUM_3  /**< Up to three frames */
}
e_SecQiFrameCount;


/**************************************************************************//**
 @Description   Scheduling Options
*//***************************************************************************/
typedef enum e_SecQiScheduling
{
    e_SEC_QI_SCHEDULING_NONE          , /**< No dequeue operations executed                      */
    e_SEC_QI_SCHEDULING_PRI_WITH_IC   , /**< Priority precedence with intra-class scheduling     */
    e_SEC_QI_SCHEDULING_AFQ_WITH_IC   , /**< Active FQ precedence with intra-class scheduling    */
    e_SEC_QI_SCHEDULING_AFQ_WITHOUT_IC  /**< Active FQ precedence without intra-class scheduling */
}
e_SecQiScheduling;


/**************************************************************************//**
 @Description   Command Configuration Parameters
*//***************************************************************************/
typedef struct t_SecQiCmdParam
{
    e_SecQiFrameColor enqFrameColor  ;
    e_SecQiFrameCount deqFrameCount  ;
    e_SecQiScheduling deqScheduling  ;
    uint32_t          deqByteCountThr;
    uint32_t          deqSpfCountThr ;
}
t_SecQiCmdParam;


/**************************************************************************//**
 @Function      SEC_QI_ConfigCommands

 @Description   Configures the SEC QI commands.

 @Param[in]     h_SecQi    - SEC QI object.
 @Param[in]     p_CmdParam - Command configuration parameters:
                             - enqFrameColor   - Enqueue frame color                     (default value = [e_SEC_QI_FRAME_COLOR_OVERRIDE  ]).
                             - deqFrameCount   - Dequeue frame count                     (default value = [e_SEC_QI_FRAME_COUNT_MAXIMUM_3  ]).
                             - deqScheduling   - Dequeue scheduling                      (default value = [e_SEC_QI_SCHEDULING_PRI_WITH_IC  ]).
                             - deqByteCountThr - Dequeue byte-count threshold            (default value = [65535]).
                             - deqSpfCountThr  - Dequeue subportal-frame-count threshold (default value = [4 ]).

                A dequeue byte-count threshold of 65535 is equivalent to an infinite threshold.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      deqByteCountThr must be between [MIN_VAL_deqByteCountThr] and [MAX_VAL_deqByteCountThr].
                deqSpfCountThr  must be between [MIN_VAL_deqSpfCountThr ] and [MAX_VAL_deqSpfCountThr ].
*//***************************************************************************/
t_Error SEC_QI_ConfigCommands(t_Handle         h_SecQi,
                              t_SecQiCmdParam* p_CmdParam);


/**************************************************************************//**
 @Description   Byte Swap Configuration Parameters
*//***************************************************************************/
typedef struct t_SecQiByteSwapParam
{
    e_SecByteSwap icd; /* Input control data  */
    e_SecByteSwap ocd; /* Output control data */
    e_SecByteSwap imd; /* Input message data  */
    e_SecByteSwap omd; /* Output message data */
}
t_SecQiByteSwapParam;


/**************************************************************************//**
 @Function      SEC_QI_ConfigByteSwap

 @Description   Configures the SEC QI Byte-Swap mode.

 @Param[in]     h_SecQi         - SEC QI object.
 @Param[in]     p_ByteSwapParam - Byte swap configuration parameters:
                                  - icd - Input control data  (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - ocd - Output control data (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - imd - Input message data  (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - omd - Output message data (default value = [e_SEC_BYTE_SWAP_NONE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_ConfigByteSwap(t_Handle              h_SecQi,
                              t_SecQiByteSwapParam* p_ByteSwapParam);


/** @} */ /* end of sec_qi_mdl_advance_config_grp */
/** @} */ /* end of sec_qi_mdl_initialization_grp */


/**************************************************************************//**
 @Group         sec_qi_mdl_runtime_control_grp SEC QI Control Unit

 @Description   Runtime control routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SEC_QI_Enable

 @Description   Enables the SEC QI.

 @Param[in]     h_SecQi - SEC QI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_Enable(t_Handle h_SecQi);

/**************************************************************************//**
 @Function      SEC_QI_Disable

 @Description   Disables the SEC QI.

 @Param[in]     h_SecQi - SEC QI object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_Disable(t_Handle h_SecQi);


/**************************************************************************//**
 @Collection   Error Flags
*//***************************************************************************/
#define SEC_QI_OFTL_ERROR 0x40 /**< Output frame too large error */
#define SEC_QI_CFWR_ERROR 0x20 /**< Compound frame write error   */
#define SEC_QI_BTS_ERROR  0x10 /**< Buffer too small error       */
#define SEC_QI_BPD_ERROR  0x08 /**< Buffer pool depletion error  */
#define SEC_QI_OFWR_ERROR 0x04 /**< Output frame write error     */
#define SEC_QI_CFRD_ERROR 0x02 /**< Compound frame read error    */
#define SEC_QI_PHRD_ERROR 0x01 /**< PreHeader read error         */
/* @} */


/**************************************************************************//**
 @Function      SEC_QI_RetrieveErrors

 @Description   Retrieves the SEC QI errors.

 @Param[in]     h_SecQi      - SEC QI object.
 @Param[out]    p_ErrorFlags - Error flags.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_RetrieveErrors(t_Handle h_SecQi,
                              uint8_t* p_ErrorFlags);


/**************************************************************************//**
 @Description   Job Status Registers
*//***************************************************************************/
typedef struct t_SecQiJobStatus
{
    uint16_t done ; /**< Flags indicating which jobs are done               */
    uint16_t exec ; /**< Flags indicating which jobs are executing          */
    uint16_t inUse; /**< Flags indicating which job-data-buffers are in use */
}
t_SecQiJobStatus;


/**************************************************************************//**
 @Function      SEC_QI_RetrieveJobStatus

 @Description   Retrieves the SEC QI job-status.

 @Param[in]     h_SecQi     - SEC QI object.
 @Param[out]    p_JobStatus - Job status registers:
                              - done  - Flags indicating which jobs are done.
                              - exec  - Flags indicating which jobs are executing.
                              - inUse - Flags indicating which job-data-buffers are in use.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_QI_RetrieveJobStatus(t_Handle          h_SecQi,
                                 t_SecQiJobStatus* p_JobStatus);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      SEC_QI_DumpRegisters

 @Description   Dumps the contents of the SEC QI registers.

 @Param[in]     h_SecQi - SEC QI object.

 @Return        None.
*//***************************************************************************/
void SEC_QI_DumpRegisters(t_Handle h_SecQi);

#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of sec_qi_mdl_runtime_control_grp */


#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(push,1)
#endif /* defined(__MWERKS__) && !defined(__GNUC__) */

/**************************************************************************//**
 @Description   Pre-Header Type
*//***************************************************************************/
typedef _Packed struct t_SecPreHeader
{
    volatile uint32_t first ;
    volatile uint32_t second;
}
_PackedType t_SecPreHeader;


/**************************************************************************//**
 @Description   Context A
*//***************************************************************************/
typedef _Packed struct t_SecContextA
{
    volatile uint32_t addr_hi;
    volatile uint32_t addr_lo;
}
_PackedType t_SecContextA;


/**************************************************************************//**
 @Description   Context-B Type
*//***************************************************************************/
typedef uint32_t t_SecContextB;


#if defined(__MWERKS__) && !defined(__GNUC__)
#pragma pack(pop)
#endif /* defined(__MWERKS__) && !defined(__GNUC__) */


/**************************************************************************//**
 @Collection   Pre-Header Macros
*//***************************************************************************/
#define SEC_PH_RSLS_MASK                  0x80000000
#define SEC_PH_SDLEN_MASK                 0x0000007F
#define SEC_PH_FSGT_MASK                  0x20000000
#define SEC_PH_LONG_MASK                  0x10000000
#define SEC_PH_OFFSET_MASK                0x0C000000
#define SEC_PH_ABS_MASK                   0x02000000
#define SEC_PH_ADDBUF_MASK                0x01000000
#define SEC_PH_POOLID_MASK                0x00FF0000
#define SEC_PH_BUFSIZE_MASK               0x0000FFFF
#define SEC_PH_GET_RSLS(preHeader)        ((((t_SecPreHeader*)preHeader)->first  & SEC_PH_RSLS_MASK   ) >> (31- 0))
#define SEC_PH_GET_SDLEN(preHeader)       ((((t_SecPreHeader*)preHeader)->first  & SEC_PH_SDLEN_MASK  ) >> (31-31))
#define SEC_PH_GET_FSGT(preHeader)        ((((t_SecPreHeader*)preHeader)->second & SEC_PH_FSGT_MASK   ) >> (31- 2))
#define SEC_PH_GET_LONG(preHeader)        ((((t_SecPreHeader*)preHeader)->second & SEC_PH_LONG_MASK   ) >> (31- 3))
#define SEC_PH_GET_OFFSET(preHeader)      ((((t_SecPreHeader*)preHeader)->second & SEC_PH_OFFSET_MASK ) >> (31- 5))
#define SEC_PH_GET_ABS(preHeader)         ((((t_SecPreHeader*)preHeader)->second & SEC_PH_ABS_MASK    ) >> (31- 6))
#define SEC_PH_GET_ADDBUF(preHeader)      ((((t_SecPreHeader*)preHeader)->second & SEC_PH_ADDBUF_MASK ) >> (31- 7))
#define SEC_PH_GET_POOLID(preHeader)      ((((t_SecPreHeader*)preHeader)->second & SEC_PH_POOLID_MASK ) >> (31-15))
#define SEC_PH_GET_BUFSIZE(preHeader)     ((((t_SecPreHeader*)preHeader)->second & SEC_PH_BUFSIZE_MASK) >> (31-31))
#define SEC_PH_SET_RSLS(preHeader,val)    (((t_SecPreHeader* )preHeader)->first  = ((((t_SecPreHeader*)preHeader)->first  & ~SEC_PH_RSLS_MASK   ) | ((val << (31- 0)) & SEC_PH_RSLS_MASK   )))
#define SEC_PH_SET_SDLEN(preHeader,val)   (((t_SecPreHeader* )preHeader)->first  = ((((t_SecPreHeader*)preHeader)->first  & ~SEC_PH_SDLEN_MASK  ) | ((val << (31-31)) & SEC_PH_SDLEN_MASK  )))
#define SEC_PH_SET_FSGT(preHeader,val)    (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_FSGT_MASK   ) | ((val << (31- 2)) & SEC_PH_FSGT_MASK   )))
#define SEC_PH_SET_LONG(preHeader,val)    (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_LONG_MASK   ) | ((val << (31- 3)) & SEC_PH_LONG_MASK   )))
#define SEC_PH_SET_OFFSET(preHeader,val)  (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_OFFSET_MASK ) | ((val << (31- 5)) & SEC_PH_OFFSET_MASK )))
#define SEC_PH_SET_ABS(preHeader,val)     (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_ABS_MASK    ) | ((val << (31- 6)) & SEC_PH_ABS_MASK    )))
#define SEC_PH_SET_ADDBUF(preHeader,val)  (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_ADDBUF_MASK ) | ((val << (31- 7)) & SEC_PH_ADDBUF_MASK )))
#define SEC_PH_SET_POOLID(preHeader,val)  (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_POOLID_MASK ) | ((val << (31-15)) & SEC_PH_POOLID_MASK )))
#define SEC_PH_SET_BUFSIZE(preHeader,val) (((t_SecPreHeader* )preHeader)->second = ((((t_SecPreHeader*)preHeader)->second & ~SEC_PH_BUFSIZE_MASK) | ((val << (31-31)) & SEC_PH_BUFSIZE_MASK)))
/* @} */


/**************************************************************************//**
 @Collection   Context-B Macros
*//***************************************************************************/
#define SEC_CONTEXTB_FQID_MASK              0x00FFFFFF
#define SEC_CONTEXTB_GET_FQID(contextB)     (*((t_SecContextB*)contextB) & SEC_CONTEXTB_FQID_MASK)
#define SEC_CONTEXTB_SET_FQID(contextB,val) (*((t_SecContextB*)contextB) = ((*((t_SecContextB *)contextB) & ~SEC_CONTEXTB_FQID_MASK) | (val & SEC_CONTEXTB_FQID_MASK)))
/* @} */


/** @} */ /* end of sec_qi_mdl_grp */


#endif /* __SEC_QI_EXT_H */
