/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          serdes_ext.h

 @Description   SERDES Application Programming Interface.
*//***************************************************************************/

#ifndef __SERDES_EXT_H
#define __SERDES_EXT_H


/**************************************************************************//**
 @Group         serdes_grp SerDes PHY Application Programming Interface

 @Description   SerDes PHY public functions, definitions and enums
 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         serdes_init_grp SerDes PHY Initialization Unit

 @Description   SerDes PHY initialization unit public API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SERDES protocol selection
*//***************************************************************************/
typedef enum e_SerDesProtocol
{
    e_SERDES_PROTOCOL_SGMII_x1 = 0, /**<  */
    e_SERDES_PROTOCOL_SATA_x1,      /**<  */
    e_SERDES_PROTOCOL_PCIE_x1,      /**<  */
    e_SERDES_PROTOCOL_PCIE_x2       /**<  */
} e_SerDesProtocol;

/**************************************************************************//**
 @Description   SERDES lanes activation
*//***************************************************************************/
typedef enum e_SerDesLaneActivation
{
    e_SERDES_LANE_A,        /**<  */
    e_SERDES_LANE_E,        /**<  */
    e_SERDES_BOTH_LANES     /**<  */
} e_SerDesLaneActivation;

/**************************************************************************//**
 @Description   SERDES reference clock selection
*//***************************************************************************/
typedef enum e_SerDesRefClock
{
    e_SERDES_REF_CLK_100_MHZ    = 0x00000000,   /**< 100 MHz reference clock */
    e_SERDES_REF_CLK_125_MHZ    = 0x10000000,   /**< 125 MHz reference clock */
    e_SERDES_REF_CLK_150_MHZ    = 0x30000000    /**< 150 MHz reference clock */
} e_SerDesRefClock;


/**************************************************************************//**
 @Description   SERDES configuration parameters
*//***************************************************************************/
typedef struct t_SerDesParam
{
    t_Handle                h_Part;         /**< Handle to the integration object */
    uint32_t                baseAddress;    /**< Address of SERDES registers in memory */
    e_SerDesProtocol        protocol;       /**< Protocol running over the SERDES */
    e_SerDesLaneActivation  laneActivation; /**< Which lanes to activate */
    e_SerDesRefClock        refClock;       /**< Reference clock */

} t_SerDesParam;


/**************************************************************************//**
 @Function      SERDES_Config

 @Description   Creates the SERDES object.

 @Param[in]     p_SerDesParam - Pointer to configuration parameters structure.

 @Return        SERDES handle (descriptor); NULL if failed.
*//***************************************************************************/
t_Handle    SERDES_Config(t_SerDesParam *p_SerDesParam);

/**************************************************************************//**
 @Function      SERDES_Init

 @Description   Initializes the SERDES hardware.

                The initialization is done according to previously selected
                configuration.

 @Param[in]     h_SerDes - SERDES handle, returned by SERDES_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     SERDES_Init(t_Handle h_SerDes);

/**************************************************************************//**
 @Function      SERDES_Free

 @Description   Frees all resources used by the SERDES object.

 @Param[in]     h_SerDes - SERDES handle, returned by SERDES_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     SERDES_Free(t_Handle h_SerDes);

/** @} */ /* end of serdes_init_grp */

/**************************************************************************//**
 @Group         serdes_ctrl_grp SerDes PHY Control Unit

 @Description   SerDes PHY control unit public API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SERDES_GetProtocol

 @Description   Queries the SERDES object to find the current active protocol.

 @Param[in]     h_SerDes - SERDES handle, returned by SERDES_Config().

 @Param[out]    p_Protocol - returns the protocol type currently active

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     SERDES_GetProtocol(t_Handle h_SerDes, e_SerDesProtocol *p_Protocol);

/**************************************************************************//**
 @Function      SERDES_PowerDown

 @Description   Moves the SERDES hardware to power-down mode.

                This routine is useful for power-saving modes, can be called
                without initialization of the SERDES object.

 @Param[in]     baseAddress - base address of the SERDES block.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error     SERDES_PowerDown(uint32_t baseAddress);


/** @} */ /* end of serdes_ctrl_grp */
/** @} */ /* end of serdes_grp */


#endif /* __SERDES_EXT_H */
