/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          spi_ctrl_ext.h

 @Description   External definitions and API of the SPI controller driver.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __SPI_CTRL_EXT_H
#define __SPI_CTRL_EXT_H


#include "part_ext.h"
#include "spi_std_ext.h"


/**************************************************************************//**
 @Group         spi_ctrl_grp    SPI Controller Application Programming Interface

 @Description   SPI controller functions, definitions and enums.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         spi_ctrl_init_grp   SPI Controller Initialization Unit

 @Description   SPI controller initialization functions and definitions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   SPI controller transfer completion callback function - called
                by the driver at end of any successful or unsuccessful transfer.

 @Param[in]     h_App       - Handle to the user's object, as provided in the
                              t_SpiCtrlParam configuration parameters structure.
 @Param[in]     h_Context   - Handle to the user's context for the specific transfer,
                              as provided in the SPI_CTRL_TxRx() routine.
 @Param[in]     status      - Status and error flags related to the specific
                              transfer; a zero value represents a successful
                              transfer; possible error flags are listed in
                              @ref spi_ctrl_evbm "SPI controller transfer errors".
 
 @Return        None.
*//***************************************************************************/
typedef void (t_SpiCtrlTransferDoneCb)(t_Handle h_App, t_Handle h_Context, uint32_t status);

/**************************************************************************//**
 @Description   SPI controller driver configuration parameters.
*//***************************************************************************/
typedef struct t_SpiCtrlParam
{
    /* SPI basic parameters */
    bool                    master;             /** Must be set to TRUE; Only master mode is supported. */
 
    /* System integration parameters */
    uint32_t                baseAddress;        /**< Base address of SPI controller registers in memory. */
    uint32_t                sourceClockFreqHz;  /**< The source clock frequency to the SPI baud rate generator [Hz] */
    int                     irq;                /**< SPI controller interrupt ID (system-specific number) */

    /* Application interface */
    t_Handle                h_App;              /**< User's handle - will be passed in the
                                                     transfer completion callback without change */
    t_SpiCtrlTransferDoneCb *f_TransferDone;    /**< Transfer complete callback routine */

} t_SpiCtrlParam;


/**************************************************************************//**
 @Function      SPI_CTRL_Config

 @Description   Configure the SPI controller driver according to the user's
                configuration parameters structure.
 
                The driver decides on a few default values for the advanced
                configuration options. These default values may be changed using
                the set of advanced configuration routines.
 
                The driver does not initialize any of the controller registers
                at this phase.
 
                The returned handle must be passed to any consequent SPI routine.

 @Param[in]     p_SpiCtrlParam - Pointer to SPI controller configuration parameters.
 
 @Return        Handle to the SPI controller; NULL if configuration failed.
*//***************************************************************************/
t_Handle SPI_CTRL_Config(t_SpiCtrlParam *p_SpiCtrlParam);

/**************************************************************************//**
 @Function      SPI_CTRL_Init

 @Description   Initialize the SPI controller registers according to previous
                configuration.

 @Param[in]     h_SpiCtrl  - Handle to the SPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_Init(t_Handle h_SpiCtrl);

/**************************************************************************//**
 @Function      SPI_CTRL_Free

 @Description   Free the SPI controller driver object.
 
                This routine may be called at any time after SPI_CTRL_Config(),
                to release the SPI driver object and make it as unusable.
 
                The routine disables the SPI controller before freeing the
                driver.

 @Param[in]     h_SpiCtrl  - Handle to the SPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_Free(t_Handle h_SpiCtrl);


/**************************************************************************//**
 @Group         spi_ctrl_advanced_init_grp  SPI Controller Advanced Configuration Unit

 @Description   SPI controller advanced configuration functions.

 @{
*//***************************************************************************/

#ifdef SPI_QE_INTEGRATION
/**************************************************************************//**
 @Description   SPI QE parameters structure.
 
                This structure should be passed to SPI_CTRL_ConfigQeMode(),
                when the user wishes to operate the SPI controller in QE mode.
*//***************************************************************************/
typedef struct t_SpiQeParam
{
    t_Handle    h_Qe;       /**< Handle to the controlling QE */
    uint8_t     pramMemId;  /**< Parameter RAM partition ID */
} t_SpiQeParam;

/**************************************************************************//**
 @Function      SPI_CTRL_ConfigQeMode

 @Description   Configures whether the SPI controller operates in QUICC Engine
                mode or in normal (CPU) mode. [FALSE]
 
                The QUICC Engine may offload CPU operation by writing and reading
                the SPI controller Tx and Rx registers.
 
                See SPI_CTRL_TxRx() for information on differences in data buffer
                format when operating in QE mode, and SPI_CTRL_ConfigQeDataMemory()
                for information on data memory attributes.
 
 @Param[in]     h_SpiCtrl       - Handle to the SPI controller object.
 @Param[in]     p_SpiQeParam    - Pointer to SPI parameters structure for QE mode.

 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error SPI_CTRL_ConfigQeMode(t_Handle h_SpiCtrl, t_SpiQeParam *p_SpiQeParam);

/**************************************************************************//**
 @Function      SPI_CTRL_ConfigQeDataMemory

 @Description   Configures memory partition and attributes for data buffers in
                QE mode. [0 (default heap), MEMORY_ATTR_CACHEABLE]
 
                Note that the memory partition ID is only used for allocation of
                two internal dummy Tx/Rx data buffers in QE mode. However, the
                memory attributes must represent the real attributes of all
                transferred data buffers (user's and driver's).

 @Param[in]     h_SpiCtrl       - Handle to the SPI controller object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_ConfigQeDataMemory(t_Handle    h_SpiCtrl,
                                    uint8_t     memId,
                                    uint32_t    memAttributes);

/**************************************************************************//**
 @Function      SPI_CTRL_ConfigQeRequestsPriority

 @Description   Configures the QE requests priority. [FALSE]
 
 @Param[in]     h_SpiCtrl       - Handle to the SPI controller object.
 @Param[in]     highPriority    - FALSE for normal requests priority;
                                  TRUE for higher requests priority (emergency mode).
 
 @Return        E_OK if successful or error type if failed.
*//***************************************************************************/
t_Error SPI_CTRL_ConfigQeRequestsPriority(t_Handle h_SpiCtrl, bool highPriority);


#else /* not SPI_QE_INTEGRATION */

/**************************************************************************//**
 @Function      SPI_CTRL_ConfigOpenDrain

 @Description   Configures the open-drain mode of the SPI controller pins.
                [FALSE]
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     openDrain   - FALSE - All pins are actively driven;
                              TRUE  - All pins are set to open-drain mode.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_ConfigOpenDrain(t_Handle h_SpiCtrl, bool openDrain);

#endif /* not SPI_QE_INTEGRATION */

/**************************************************************************//**
 @Function      SPI_CTRL_ConfigPollingMode

 @Description   Configures the SPI controller to operate in interrupts or
                polling mode. [FALSE]
 
                In polling mode, the driver disables the controller interrupts
                and calls to SPI_CTRL_TxRx() are blocking until the transfer
                is completed.
 
                In interrupts mode, the driver operates in non-blocking mode.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     polling     - FALSE for interrupt mode; TRUE for polling mode.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_ConfigPollingMode(t_Handle h_SpiCtrl, bool polling);

/** @} */ /* end of spi_ctrl_advanced_init_grp */
/** @} */ /* end of spi_ctrl_init_grp */


/**************************************************************************//**
 @Group         spi_ctrl_runtime_grp    SPI Controller Runtime Unit

 @Description   SPI controller runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SPI_CTRL_TxRx

 @Description   Initiates a transfer on the SPI bus.
 
                A typical transfer consists of both Tx and Rx data buffers, and
                an identical number of bytes to send and to read. However, the
                user may also request a Tx-only transfer, for which the p_RxData
                pointer should be set to NULL. The user may also request to skip
                a few Rx bytes at the start of the transfer. This mode is useful
                for reads of SPI flash memories, where the first valid data is
                received several characters after the transmission begins (after
                the instruction opcode and memory address have been transmitted).
 
                In polling mode, this routine is blocking until the transfer is
                completed. In interrupt mode, this routine is non-blocking, and
                the transfer progress is managed by the driver's interrupt handler.
                In both modes, transfer completion is reported to the user through
                the transfer completion callback function (if it is not NULL).
 
                The Tx data should contain byte-aligned consecutive words in memory.
                If the word size is 4-8 bits, each word should be right-aligned in
                a single byte. If the word size is 9-16 bits, each word should
                be right-aligned in two bytes. If the word size is 32 bits, then each
                word is fully contained in four bytes. The same format is applied to
                received data. If the SPI controller is used in QUICC Engine mode
                (in devices that support this option), then 2-byte or 4-byte words
                in the Tx buffer need also to be byte-swapped (as in little-endian
                format) after being right-aligned.
 
                The user may provide a private context for each transfer. This
                context will be returned in the transfer completion callback
                without change.
 
 @Param[in]     h_SpiCtrl       - Handle to the SPI controller driver object.
 @Param[in]     p_TxData        - Pointer to data for transmission.
 @Param[in]     p_RxData        - Pointer to a data buffer to fill with read data;
                                  The buffer size must be at least rxBytesToRead bytes.
 @Param[in]     txBytesToSend   - Number of bytes to transmit.
 @Param[in]     rxBytesToSkip   - Number of received bytes to skip (discard).
 @Param[in]     rxBytesToRead   - Number of received bytes to put in the Rx data buffer.
 @Param[in]     h_Context       - User's context for this transfer - will be passed
                                  in the transfer completion callback without change.

 @Return        E_OK if successful; Error code otherwise.

 @Cautions      The user must not call this routine to initiate a new transfer
                before the previous transfer was completed.
*//***************************************************************************/
t_Error SPI_CTRL_TxRx(t_Handle  h_SpiCtrl,
                      uint8_t   *p_TxData,
                      uint8_t   *p_RxData,
                      uint16_t  txBytesToSend,
                      uint8_t   rxBytesToSkip,
                      uint16_t  rxBytesToRead,
                      t_Handle  h_Context);

/** @} */ /* end of spi_ctrl_runtime_grp */


/**************************************************************************//**
 @Group         spi_ctrl_control_grp    SPI Controller Control Unit

 @Description   SPI controller control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SPI_CTRL_Enable

 @Description   Enable the SPI controller to allow transfers on the bus.
 
                Normally, the user should not call this routine, because the
                controller is automatically enabled when SPI_CTRL_TxRx() is
                called.
 
 @Param[in]     h_SpiCtrl  - Handle to the SPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_Enable(t_Handle h_SpiCtrl);

/**************************************************************************//**
 @Function      SPI_CTRL_Disable

 @Description   Disable the SPI controller.
 
                The user may use this routine to move the controller into
                idle, power-saving mode. If a transfer is active when this
                routine is called, it will be terminated. Thus, it is not
                recommended to call this routine before the current transfer is
                completed.
 
 @Param[in]     h_SpiCtrl  - Handle to the SPI controller driver object.

 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_Disable(t_Handle h_SpiCtrl);

/**************************************************************************//**
 @Function      SPI_CTRL_SetBusFrequency

 @Description   Set the SPI bus frequency for the next transfer.
 
                The exact frequency may not be reached due to divisor limitations.
                The routine will set the closest possible frequency and return it
                in the p_ActualFreq parameter.
 
 @Param[in]     h_SpiCtrl       - Handle to the SPI controller driver object.
 @Param[in]     busFrequencyHz  - The required SPI bus frequency [Hz].
 @Param[out]    p_ActualFreq    - Returns the actual bus clock frequency that
                                  was set by the driver.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetBusFrequency(t_Handle   h_SpiCtrl,
                                 uint32_t   busFrequencyHz,
                                 uint32_t   *p_ActualFreq);
 
/**************************************************************************//**
 @Function      SPI_CTRL_SetBitOrder

 @Description   Set the SPI bit ordering mode for the next transfer.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     bitOrder    - The selected bit order mode.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetBitOrder(t_Handle h_SpiCtrl, e_SpiBitOrder bitOrder);

/**************************************************************************//**
 @Function      SPI_CTRL_SetWordSize

 @Description   Set the SPI word size for the next transfer.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     bitsInWord  - The SPI word size in bits (4-16 or 32 bits).
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetWordSize(t_Handle h_SpiCtrl, uint8_t bitsInWord);

/**************************************************************************//**
 @Function      SPI_CTRL_SetClockPolarity

 @Description   Set the SPI clock polarity for the next transfer.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     polarity    - The selected clock polarity.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetClockPolarity(t_Handle h_SpiCtrl, e_SpiClockPolarity polarity);

/**************************************************************************//**
 @Function      SPI_CTRL_SetClockPhase

 @Description   Set the SPI clock phase for the next transfer.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     phase       - The selected clock phase.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetClockPhase(t_Handle h_SpiCtrl, e_SpiClockPhase phase);

#ifdef SPI_QE_INTEGRATION
/**************************************************************************//**
 @Function      SPI_CTRL_SetInterWordGap

 @Description   Set the gap (in bit time) between transmitted words.
 
 @Param[in]     h_SpiCtrl       - Handle to the SPI controller driver object.
 @Param[in]     interWordGap    - The required inter-word gap, in bits (0-31).
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetInterWordGap(t_Handle h_SpiCtrl, uint8_t interWordGap);
#endif /* SPI_QE_INTEGRATION */

/**************************************************************************//**
 @Function      SPI_CTRL_SetLoopbackMode

 @Description   Set the SPI controller loopback mode, which may be useful in
                debug phase.
 
                The loopback mode affects all chip selects, but will be effective
                only in the next transfer. If the user wants to use loopback mode
                for a subset of the chip selects, this mode must be disabled
                when other chip selects are set active.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 @Param[in]     loopEnabled - FALSE for normal SPI operation;
                              TRUE to enable controller loopback mode.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_SetLoopbackMode(t_Handle h_SpiCtrl, bool loopEnabled);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      SPI_CTRL_DumpRegs

 @Description   Prints the current values of all SPI controller registers.
 
 @Param[in]     h_SpiCtrl   - Handle to the SPI controller driver object.
 
 @Return        E_OK if successful; Error code otherwise.
*//***************************************************************************/
t_Error SPI_CTRL_DumpRegs(t_Handle h_SpiCtrl);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of spi_ctrl_control_grp */


/**************************************************************************//**
 @Group         spi_ctrl_exception_grp  SPI Controller Exceptions Unit

 @Description   SPI controller definitions for the exceptions unit.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        spi_ctrl_evbm
 
 @Collection    SPI controller transfer errors
 
                These are errors and events that may end an SPI bus transfer.
                A transfer which encounters any one of these errors is stopped.
 
                These values can be or'ed in any combination into the status
                parameter of the transfer completion callback function.
 @{
*//***************************************************************************/
#ifndef SPI_QE_INTEGRATION
#define SPI_ERROR_DNR       0x00002000  /**< Data not ready - SPISEL is asserted before
                                             data is ready in the SPI, IDLE is sent on
                                             the line and SPI_ERROR_UN is also asserted;
                                             the SPI should be disabled and enabled before
                                             resuming operation (slave mode only). */
#else  /* SPI_QE_INTEGRATION */
#define SPI_ERROR_MME_QE    0x00002000  /**< Multi-Master error; set when SPISEL is
                                             asserted externally while the SPI is in
                                             master mode (QE mode). */
#define SPI_ERROR_TXE       0x00001000  /**< Tx error - error during transmission (QE mode). */
#define SPI_ERROR_BSY       0x00000400  /**< Busy - no Rx buffer is available (QE mode). */
#endif /* SPI_QE_INTEGRATION */
#define SPI_ERROR_OV        0x00001000  /**< Overrun occurred during reception (CPU mode). */
#define SPI_ERROR_UN        0x00000800  /**< Underrun - SPI transmitter does not have
                                             data to transmit on time, and the Tx FIFO
                                             is flushed (slave mode only, CPU mode). */
#define SPI_ERROR_MME       0x00000400  /**< Multi-Master error; set when SPISEL is
                                             asserted externally while the SPI is in
                                             master mode (CPU mode). */
/* @} */

/** @} */ /* end of spi_ctrl_exception_grp */
/** @} */ /* end of spi_ctrl_grp */


#endif /* __SPI_CTRL_EXT_H */

