/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          srtp_ext.h

 @Description   structures and definitions for operating the relevant algorithms for srtp.

*//***************************************************************************/

#ifndef __SRTP_EXT_H
#define __SRTP_EXT_H

/**************************************************************************//**
 @Group         sec_id SEC Application Programming Interface

 @Description   SEC functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sec_srtp SEC SRTP

 @Description   SEC SRTP functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sec_srtp_init SEC SRTP Initialization Unit

 @Description   SEC SRTP initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   indicates the required direction of the operation - Snoop In or Out.
 
 				Snoop In - indicates that the encryption operation and the hash
 				operation will work on the same input data.
 				Snoop Out - indicates that the hash	operation will use the output
 				from the encryption operation.
*//***************************************************************************/
typedef enum
{
	e_SNOOP_OUT,
    e_SNOOP_IN
}e_Snoop;

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      srtp_alloc_ctx

 @Description   Allocate memory for the srtp ctx structure.
 
                The srtp ctx is a structure that holds the relevant
                parameters for the operation.
                The routine is not a part of the Crypto Api standard routines.

 @Return        ctx handler - holds the relevant parameters for the operation.
*//***************************************************************************/
t_Handle srtp_alloc_ctx();

/**************************************************************************//**
 @Function      srtp_cipher_setkey

 @Description   The routine sets the specific key and its length in the context structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         key -    (In) handler to the key.
 @Param         keylen - (In) the length of the key.
 @Param         flags -  (In) Not relevant.

 @Return        return status.
*//***************************************************************************/
int      srtp_cipher_setkey(void *ctx, const unsigned char *key, unsigned int keylen, unsigned int *flags);

/**************************************************************************//**
 @Function      srtp_cipher_setiv

 @Description   The routine sets the iv and it's length in the context structure.

                The routine sets the iv and its length in the
                context structure.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -    (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -    (In) Iv(Initialization vector) handler.
 @Param         len -    (In) Iv length.

 @Return        None.
*//***************************************************************************/
void     srtp_cipher_setiv(void *ctx, void *src, unsigned int len);

/**************************************************************************//**
 @Function      srtp_cipher_set_srclen

 @Description   The routine sets the data source length in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx - (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len - (In) the source length.

 @Return        None.
*//***************************************************************************/
void     srtp_cipher_set_srclen(void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      srtp_digest_update_ctx

 @Description   The routine sets the key source and its length in the specific ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) the digest key handler.
 @Param         len -  (In) the digest key length.

 @Return        None.
*//***************************************************************************/
void     srtp_digest_update_ctx(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      srtp_set_snoop

 @Description   The routine sets the snoop in or snoop out indication.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx   -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         snoop -  (In) the snoop type(In or Out).
 
 @Return        None.
*//***************************************************************************/
void     srtp_set_snoop(void *ctx,e_Snoop snoop);

/**************************************************************************//**
 @Function      srtp_digest_final

 @Description   The routine sets the output data pointer in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         out -  (In) output handler.

 @Return        None.
*//***************************************************************************/
void     srtp_digest_final(void *ctx, unsigned char *out);

#ifdef SEC2
/**************************************************************************//**
 @Function      srtp_cipher_digest_set_src

 @Description   The routine sets the data source length and its pointer in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx - (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src - (In) data source input handler.
 @Param         len - (In) the source length.

 @Return        None.
*//***************************************************************************/
void srtp_cipher_digest_set_src(void *ctx, const unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      srtp_cipher_setOutIv

 @Description   The routine sets the output context length and its pointer in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx - (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst - (In) context output handler.
 @Param         len - (In) the output context length.

 @Return        None.
*//***************************************************************************/
void srtp_cipher_setOutIv(void *ctx, unsigned char *dst, unsigned int len);

/**************************************************************************//**
 @Function      srtp_digest_update

 @Description   The routine sets the initial hmac bytes in the specific ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         len -  (In) the initial digest only length.

 @Return        None.
*//***************************************************************************/
void     srtp_digest_update(void *ctx, unsigned int len);

/**************************************************************************//**
 @Function      srtp_cipher_encrypt

 @Description   The routine initializes the input and output data pointers in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the cipher output handler.

 @Return        None.
*//***************************************************************************/
void srtp_cipher_encrypt(void *ctx, unsigned char *dst);

#else
/**************************************************************************//**
 @Function      srtp_digest_update

 @Description   The routine sets the input hmac data and its length in the specific ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         src -  (In) the digest input handler.
 @Param         len -  (In) the digest input length.

 @Return        None.
*//***************************************************************************/
void     srtp_digest_update(void *ctx, unsigned char *src, unsigned int len);

/**************************************************************************//**
 @Function      srtp_cipher_encrypt

 @Description   The routine initializes the input and output data pointers in the ctx structure.

                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.
 @Param         dst -  (In) the cipher output handler.
 @Param         src -  (In) the cipher input handler.

 @Return        None.
*//***************************************************************************/
void     srtp_cipher_encrypt(void *ctx, unsigned char *dst, const unsigned char *src);

#endif/* SEC2 */

/** @} */ /* end of group sec init */

/**************************************************************************//**
 @Group         sec_srtp_runtime SEC SRTP Runtime Unit

 @Description   SEC SRTP Run Time functions.

 @{
*//***************************************************************************/

/*****************************************************************************/
/* Accessories routines                                                      */
/*****************************************************************************/

/**************************************************************************//**
 @Function      srtp_operate

 @Description   The routine operates the request according to the parameters in the ctx structure.

                The routine initializes a request structure
                according to the different parameter in the ctx strcuture.
                Then, the function calls to ioctl
                function in order to process the request.
                The routine is not a part of the Crypto Api standard routines.

 @Param         ctx -  (In) ctx handler - holds the relevant parameters for the operation.

 @Return        None.
*//***************************************************************************/
void      srtp_operate(void *ctx);

/** @} */ /* end of group sec runtime */
/** @} */ /* end of group sec srtp  */
/** @} */ /* end of group sec  */

#endif
