/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**
 @File          ssi_ext.h

 @Description   External definition and API for the SSI.
 
 @Cautions      None.
*//***************************************************************************/

#ifndef __SSI_EXT_H
#define __SSI_EXT_H

/**************************************************************************//**
 @Group         ssi_grp    SSI Application Programming Interface

 @Description   SSI functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ssi_init_grp   SSI Initialization Unit

 @Description   SSI Initialization unit functions, definitions and enums.

 @{
*//***************************************************************************/

/**
@Description   SSI Functional Mode.

               The mode the SSI works in.
*/
typedef enum e_SsiMode
{
    e_SSI_MODE_I2S_MASTER   = 0x01  /**< The SSI is set to work in I2S Master Mode      */
  , e_SSI_MODE_I2S_SLAVE            /**< The SSI is set to work in I2S Slave Mode       */
  , e_SSI_MODE_NORMAL               /**< The SSI is set to work in Normal Mode          */
  , e_SSI_MODE_NETWORK              /**< The SSI is set to work in Network Mode         */
  , e_SSI_MODE_AC97                 /**< The SSI is set to work in AC97 Mode            */
 
} e_SsiMode;

/**
@Description   SSI Functional Mode.

               The mode the SSI's Tx and Rx sections work in.
*/
typedef enum e_SsiTxRxMode
{
    e_SSI_TXRX_MODE_SYNC    = 0x01  /**< The SSI Tx and Rx sections are synchronized.           */
  , e_SSI_TXRX_MODE_ASYNC   = 0x02  /**< The SSI Tx and Rx sections are not synchronized.       */
 
} e_SsiTxRxMode;

/**
 @Description   Maximum number of supported channels per SSI module.
 */
#define SSI_MAX_NUM_OF_SLOTS    32

/**
 @Description   User callback function, called if driver Rx, Tx or functional events arise.
                The user provides this function and the driver invokes it.

 @Param[in]     h_App   - Application handle given by the user at driver configuration.
 
 @Param[in]     events  - Selection of user provided events from e_SsiInterrupt.
 */
typedef void (t_SsiEventsCallback)(t_Handle h_App, uint32_t events);

/**
 @Description   The SSI driver user's parameter structure.
                Passed by the user to SSI_Config() routine.
*/
typedef struct t_SsiParam
{
    /* SSI basic parameters */
    e_SsiMode               ssiMode;            /**< The working mode of the SSI. */
    e_SsiTxRxMode           ssiTxRxMode;        /**< The mode the SSI's Tx and Rx sections work in. */
   	uint32_t                ssiIntClock;        /**< Frequency (in kHz) of the SSI clock, when clock is generated internally */
   	uint32_t                ssiIntTxFrameRate;  /**< Frequency (in kHz) of the Tx Frame rate, when clock is generated internally */
   	uint32_t                ssiIntRxFrameRate;  /**< Frequency (in kHz) of the Rx Frame rate, when clock is generated internally */
   	uint8_t    				numOfRxChannels; 	/**< Maximum number of enabled Rx slots */
    uint8_t    				numOfTxChannels; 	/**< Maximum number of enabled Tx slots */
    t_BufferPoolInfo    	bufferPoolInfo;     /**< Rx and Tx buffers */

    /* System integration parameters */
    uint32_t                baseAddress;        /**< Base address of SSI controller registers in memory.    */
    int                     irq;                /**< SSI controller interrupt ID (system-specific number)   */

    /* Application interface */
    t_Handle                h_App;              /**< Application handle    */
    t_SsiEventsCallback     *f_EventsCallback;  /**< Callback function to be called in case selected events are generated. */
} t_SsiParam;

/**
 @Function      SSI_Config

 @Description   Initialize the SSI configuration, using the default values
                and the user parameters structure.

 @Param[in]     p_SsiParam - Pointer to SSI controller configuration parameters.
 
 @Return        Handle to the SSI controller or NULL if configuration failed.
*/
t_Handle SSI_Config(t_SsiParam *p_SsiParam);

/**
 @Function      SSI_Init

 @Description   Initialize the SSI.

 @Param[in]     h_Ssi  - Handle to the SSI as returned from the SSI_Config().

 @Return        E_OK if successful or error type if failed.

 @Cautions      None
*/
t_Error SSI_Init(t_Handle h_Ssi);

/** @Function   SSI_Free

 @Description   Frees the SSI.

 @Param[in]     h_Ssi  - Handle to the SSI as returned from the SSI_Config().

 @Return        E_OK if successful or error type if failed.
*/
t_Error SSI_Free(t_Handle h_Ssi);

/**************************************************************************//**
 @Group         ssi_advanced_init_grp  SSI Advanced Configuration Unit

 @Description   SSI advanced configuration functions.

 @{
*//***************************************************************************/
/**
 @Description    Select transmit/receive data direction
*/
typedef enum e_SsiDataDirection{

      e_SSI_DATA_DIRECTION_MSB_FIRST   = 0x0    /**< Data transmitted/received msb first */
    , e_SSI_DATA_DIRECTION_LSB_FIRST   = 0x1    /**< Data transmitted/received lsb first */

} e_SsiDataDirection;

/**
 @Description    Select transmit/receive data alignment
*/
typedef enum e_SsiDataAlignment{

      e_SSI_DATA_ALIGNMENT_MSB       = 0x0    /**< Data transmitted msb aligned */
    , e_SSI_DATA_ALIGNMENT_LSB       = 0x1    /**< Data transmitted lsb aligned */

} e_SsiDataAlignment;

/**
 @Description    Select transmit frame sync length.
*/
typedef enum e_SsiFrameSyncLength{

      e_SSI_FRAME_SYNC_ONE_WORD_LONG         = 0x0    /**< Transmit frame sync is one-word long         */
    , e_SSI_FRAME_SYNC_ONE_CLOCK_BIT_LONG    = 0x1    /**< Transmit frame sync is one-clock-bit long    */

} e_SsiFrameSyncLength;

/**
 @Description    Select word length.
*/
typedef enum e_SsiWordLength{

      e_SSI_WORD_LENGTH_8_BITS         = 0x0003    /**<  8 bits per word */
    , e_SSI_WORD_LENGTH_10_BITS        = 0x0004    /**< 10 bits per word */
    , e_SSI_WORD_LENGTH_12_BITS        = 0x0005    /**< 12 bits per word */
    , e_SSI_WORD_LENGTH_16_BITS        = 0x0007    /**< 16 bits per word */
    , e_SSI_WORD_LENGTH_18_BITS        = 0x0008    /**< 18 bits per word */
    , e_SSI_WORD_LENGTH_20_BITS        = 0x0009    /**< 20 bits per word */
    , e_SSI_WORD_LENGTH_22_BITS        = 0x000a    /**< 22 bits per word */
    , e_SSI_WORD_LENGTH_24_BITS        = 0x000b    /**< 24 bits per word */

} e_SsiWordLength;

/**
 @Description    Select FIFO type
*/
typedef enum e_SsiFifo{

      e_SSI_FIFO_FIRST = 0   /**< First FIFO is selected */
    , e_SSI_FIFO_SECOND      /**< Second FIFO is selected */

} e_SsiFifo;

/**
 @Description    Select watermark for the FIFOs.
*/
typedef enum e_SsiFifoWatermark{

      e_SSI_FIFO_WATERMARK_1         = 0b0001    /**<  Interrupt issued when at least one data word has been
                                                  written to the receive FIFO or when there are more than
                                                  or equal to 1 empty slots in transmit FIFO.                   */
    , e_SSI_FIFO_WATERMARK_2         = 0b0010    /**<  Interrupt issued when more than or equal to 2 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 2 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_3         = 0b0011    /**<  Interrupt issued when more than or equal to 3 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 3 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_4         = 0b0100    /**<  Interrupt issued when more than or equal to 4 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 4 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_5         = 0b0101    /**<  Interrupt issued when more than or equal to 5 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 5 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_6         = 0b0110    /**<  Interrupt issued when more than or equal to 6 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 6 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_7         = 0b0111    /**<  Interrupt issued when more than or equal to 7 data word
                                                  have been written to the receive FIFO or when there are
                                                  more than or equal to 7 empty slots in transmit FIFO.         */
    , e_SSI_FIFO_WATERMARK_8         = 0b1000    /**<  Interrupt issued when more than or equal to 8 data word
                                                  have been written to the receive FIFO or when there are
                                                  8 empty slots in transmit FIFO.                               */

} e_SsiFifoWatermark;

/**
@Description   SSI exception events
*/
typedef enum e_SsiExceptionEvents{

    e_SSI_EXCEPTION_EVENT_NONE          = 0,          /**< No exception */
    e_SSI_EXCEPTION_EVENT_RX_OVERRUN_1  = 0x00000800, /**< Receiver Overrun Error Data Register 1 */
    e_SSI_EXCEPTION_EVENT_RX_OVERRUN_0  = 0x00000400, /**< Receiver Overrun Error Data Register 0 */
    e_SSI_EXCEPTION_EVENT_TX_UNDERRUN_1 = 0x00000200, /**< Receiver Overrun Error Data Register 0 */
    e_SSI_EXCEPTION_EVENT_TX_UNDERRUN_0 = 0x00000100  /**< Receiver Overrun Error Data Register 0 */
 
} e_SsiExceptionEvents;

/**
 @Description   Configuration parameters for DMA
*/
typedef struct t_SsiDmaParam
{
    t_Handle    h_Dma;          /**< Handle to the DMA descriptor */
    uint8_t     channelId;      /**< DMA channel number dedicated to SSI receive section;
                                     Allocation will be forced for this channel   */
    uint8_t     memId;          /**< Memory partition id for allocating internal DMA structures */
    uint32_t    memAttributes;  /**< Memory attributes */
}t_SsiDmaParam;

/**
 @Function      SSI_ConfigRxDmaMode

 @Description   Configure the DMA channel for the receive section of the SSI
                Default value is [FALSE]

 @Param[in]     h_Ssi        - Handle to the SSI object.
 @Param[in]     numOfDmaChs  - Number of DMA channels dedicated to SSI receive section
 @Param[in]     p_DmaParam   - Pointer to array of size numOfDmaChs containing DMA configuration
                               parameters for for SSI receive section
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Ssi must have previously created with SSI_Config().
 
 @Cautions      This routine must be called before SSI_Init().
 @Cautions      Dedicated DMA channel should not have been previously created.
*/
t_Error SSI_ConfigRxDmaMode(t_Handle h_Ssi, uint8_t numOfDmaChs, t_SsiDmaParam *p_DmaParam);

/**
 @Function      SSI_ConfigTxDmaMode

 @Description   Configure the DMA channel for the receive section of the SSI
                Default value is [FALSE]

 @Param[in]     h_Ssi       - Handle to the SSI object.
 @Param[in]     numOfDmaChs - Number of DMA channels dedicated to SSI transmit section
 @ @Param[in]   p_DmaParam  - Pointer to array of size numOfDmaChs containing DMA configuration
                              parameters for SSI transmit section

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_Ssi must have previously created with SSI_Config().
 
 @Cautions      This routine must be called before SSI_Init().
 @Cautions      Dedicated DMA channel should not have been previously created
*/
t_Error SSI_ConfigTxDmaMode(t_Handle h_Ssi, uint8_t numOfDmaChs, t_SsiDmaParam *p_DmaParam);

/**
 @Function      SSI_ConfigExceptions

 @Description   Configure the SSI exception events.

                Default value is [0].

 @Param[in]     h_Ssi   - Handle to the SSI object.
 @Param[in]     events  - Selection of user provided events from e_SsiExceptionEvents

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None.
*/
t_Error SSI_ConfigExceptions(t_Handle h_Ssi, uint32_t events);

/**
 @Function      SSI_ConfigDualOperationMode

 @Description   Allows the SSI to operate in dual operation mode. In this mode, two
                data registers will be used. The data is alternately handled by the two
                data registers.

                Default value is [FALSE].

 @Param[in]     h_Ssi           - Handle to the SSI object.
 @Param[in]     dualOperation   - FALSE for normal SSI operation
                                  TRUE to enable dual operation mode

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None.
*/
t_Error SSI_ConfigDualOperationMode(t_Handle h_Ssi, bool dualOperation);

/**
 @Function      SSI_ConfigNumOfTxBuffers

 @Description   Configure the number of buffers that the user can schedule for transmission
                through consecutive calls of SSI_CH_Tx().
                Default value is [1].

 @Param[in]     h_Ssi           - Handle to the SSI object.
 @Param[in]     numOfTxBuffers  - Number of buffers

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None.
*/
t_Error SSI_ConfigNumOfTxBuffersPerChannel(t_Handle h_Ssi, uint8_t numOfTxBuffers);

/**
 @Function      SSI_ConfigFifoWatermark

 @Description   This configuration controls the thresholds for each SSI FIFO. The interrupt is issued
                whenever the data level in the specified FIFO reaches the selected threshold.
 
                Default value is [e_SSI_FIFO_WATERMARK_8] for the full watermark threshold and [e_SSI_FIFO_WATERMARK_1]
                for the empty watermark threshold.
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     fifoId              -   The id of the FIFO to be configured.
 @Param[in]     fifoFullWatermark   -   New value for the full watermark.
 @Param[in]     fifoEmptyWatermark  -   New value for the empty watermark.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigFifoWatermark(t_Handle           h_Ssi,
                                e_SsiFifo          fifoId,
                                e_SsiFifoWatermark fifoFullWatermark,
                                e_SsiFifoWatermark fifoEmptyWatermark);

/**
 @Function      SSI_ConfigIdleStateClockLow

 @Description   This configuration is used to set the the idle state of the transmit clock
                port during SSI internal gated mode to value "zero".

                Default value is [TRUE].
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     idleStateClockLow   -   If true, set the bit to zero.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigIdleStateClockLow(t_Handle h_Ssi, bool idleStateClockLow);

/**
 @Function      SSI_ConfigLoopbackMode

 @Description   This configuration sets the SSI in Loopback Mode.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     loopbackEnable      -   TRUE to activate the mode.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigLoopbackMode(t_Handle h_Ssi, bool loopbackEnable);

/**
 @Function      SSI_ConfigRxFifoEnable

 @Description   This flag enables receive FIFO.

                Default value is [TRUE].

 @Param[in]     h_Ssi       - Handle to the SSI object.
 @Param[in]     rxFifo      - FALSE - Receive FIFO disabled.
                              TRUE -  Receive FIFO enabled.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxFifoEnable(t_Handle h_Ssi, bool rxFifo);

/**
 @Function      SSI_ConfigTxFifoEnable

 @Description   This flag enables transmit FIFO.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi       -  Handle to the SSI object.
 @Param[in]     txFifo      -  FALSE - Transmit FIFO disabled.
                               TRUE -  Transmit FIFO enabled.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxFifoEnable(t_Handle h_Ssi, bool txFifo);

/**
 @Function      SSI_ConfigRxIntSync

 @Description   This flag controls the direction and source of the transmit frame
                synchronization signal.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi                 -  Handle to the SSI object.
 @Param[in]     rxIntFrameSync        -  TRUE - internal Frame synchronization
                                         FALSE - external Frame synchronization
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxIntSync(t_Handle h_Ssi, bool rxIntFrameSync);

/**
 @Function      SSI_ConfigTxIntSync

 @Description   This flag controls the direction and source of the transmit frame
                synchronization signal.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi               -  Handle to the SSI object.
 @Param[in]     txIntFrameSync      -  TRUE - internal Frame synchronization
                                       FALSE - external Frame synchronization
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxIntSync(t_Handle h_Ssi, bool txIntFrameSync);

/**
 @Function      SSI_ConfigRxIntClk

 @Description   This flag controls the direction and source of the clock signal

                Default value is [TRUE].
 
 @Param[in]     h_Ssi               -    Handle to the SSI object.
 @Param[in]     rxIntClockDirection -    TRUE - internal clock
                                         FALSE - external clock
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxIntClk(t_Handle h_Ssi, bool rxIntClockDirection);

/**
 @Function      SSI_ConfigTxIntClk

 @Description   This flag controls the direction and source of the clock signal

                Default value is [TRUE].
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     txIntClockDirection -   TRUE = internal clock
                                        FALSE = external clock
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxIntClk(t_Handle h_Ssi, bool txIntClockDirection);

/**
 @Function      SSI_ConfigRxShiftType

 @Description   This flag controls whether the msb or lsb will be received first in a sample.

                Default value is [e_SSI_DATA_DIRECTION_LSB_FIRST].

 @Param[in]     h_Ssi            -   Handle to the SSI object.
 @Param[in]     dataDirection    -   Data transmitted msb or lsb first.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      The CODEC device labels the msb as bit 0, whereas the Core labels the lsb
                as bit 0. Therefore, when using a standard CODEC, Core msb (CODEC lsb) is shifted
                in first.
*/
t_Error SSI_ConfigRxShiftType(t_Handle h_Ssi, e_SsiDataDirection dataDirection);

/**
 @Function      SSI_ConfigTxShiftType

 @Description   This flag controls whether the msb or lsb will be transmitted first in a sample.
 
                Default value is [e_SSI_DATA_DIRECTION_LSB_FIRST].
 
 @Param[in]     h_Ssi            -  Handle to the SSI object.
 @Param[in]     dataDirection    -  Data transmitted msb or lsb first.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      The CODEC device labels the msb as bit 0, whereas the Core labels the lsb
                as bit 0. Therefore, when using a standard CODEC, Core msb (CODEC lsb) is shifted
                in first.
*/
t_Error SSI_ConfigTxShiftType(t_Handle h_Ssi, e_SsiDataDirection dataDirection);

/**
 @Function      SSI_ConfigRxAlignmentType

 @Description   The received data is MSB or LSB aligned.
 
                Default value is [e_SSI_DATA_ALIGNMENT_LSB].
 
 @Param[in]     h_Ssi            -   Handle to the SSI object.
 @Param[in]     dataAlign        -   Data received msb or lsb first.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxAlignmentType(t_Handle h_Ssi, e_SsiDataAlignment dataAlign);

/**
 @Function      SSI_ConfigTxAlignmentType

 @Description   The transmited data is MSB or LSB aligned.

                Default value is [e_SSI_DATA_ALIGNMENT_LSB].

 @Param[in]     h_Ssi                  -   Handle to the SSI object.
 @Param[in]     dataAlign              -   Data aligned msb or lsb first.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxAlignmentType(t_Handle h_Ssi, e_SsiDataAlignment dataAlign);

/**
 @Function      SSI_ConfigRxFrameSyncLength

 @Description   This flag controls the length of the frame sync signal to be generated or
                recognized for the receive section. The length of a word-long frame sync
                is the same as the length of the selected data word.
 
                Default value is [e_SSI_FRAME_SYNC_ONE_WORD_LONG].
 
 @Param[in]     h_Ssi            -   Handle to the SSI object.
 @Param[in]     frameSyncLength  -   Transmit frame sync is one-word long or one-clock-bit long.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxFrameSyncLength(t_Handle h_Ssi, e_SsiFrameSyncLength frameSyncLength);

/**
 @Function      SSI_ConfigTxFrameSyncLength

 @Description   This flag controls the length of the frame sync signal to be generated or
                recognized for the transmit section. The length of a word-long frame sync
                is the same as the length of the selected data word.

                Default value is [e_SSI_FRAME_SYNC_ONE_WORD_LONG].
 
 @Param[in]     h_Ssi            -   Handle to the SSI object.
 @Param[in]     frameSyncLength  -   Transmit frame sync is one-word long or one-clock-bit long.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxFrameSyncLength(t_Handle h_Ssi, e_SsiFrameSyncLength frameSyncLength);

/**
 @Function      SSI_ConfigRxEarlyFrameSync

 @Description   This flag controls the length of the frame sync signal to be generated or
                recognized for the receive section. The length of a word-long frame sync
                is the same as the length of the selected data word.
 
                Default value is [FALSE].
 
 @Param[in]     h_Ssi            -  Handle to the SSI object.
 
 @Param[in]     rxEarlyFrameSync -  FALSE for no early frame sync,
                                    TRUE for early frame sync .
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxEarlyFrameSync (t_Handle h_Ssi, bool rxEarlyFrameSync);

/**
 @Function      SSI_ConfigTxEarlyFrameSync

 @Description   This flag controls the length of the frame sync signal to be generated or
                recognized for the receive section. The length of a word-long frame sync
                is the same as the length of the selected data word.
 
                Default value is [FALSE].
 
 @Param[in]     h_Ssi            -  Handle to the SSI object.
 
 @Param[in]     txEarlyFrameSync -  FALSE for no early frame sync,
                                    TRUE for early frame sync .
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxEarlyFrameSync (t_Handle h_Ssi, bool txEarlyFrameSync);

/**
 @Function      SSI_ConfigRxWordLength

 @Description   This configuration is used to control the length of the data words being
                transferred by the SSI.
                In I2S Master mode, the SSI works with a fixed word length of 32, and the
                WL configuration is used to control the amount of valid data in those 32 bits.
                In AC97 mode of operation, if word length is set to any value other than 16 bits,
                it will result in a word length of 20 bits.

                Default value is [e_SSI_WORD_LENGTH_8_BITS].

 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     wordLenght          -   Number of bits per word.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxWordLength(t_Handle h_Ssi, e_SsiWordLength wordLenght);

/**
 @Function      SSI_ConfigTxWordLength

 @Description   This configuration is used to control the length of the data words being
                transferred by the SSI.
                In I2S Master mode, the SSI works with a fixed word length of 32, and the
                WL configuration is used to control the amount of valid data in those 32 bits.
                In AC97 mode of operation, if word length is set to any value other than 16 bits,
                it will result in a word length of 20 bits.

                Default value is [e_SSI_WORD_LENGTH_8_BITS].

 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     wordLenght          -   Number of bits per word.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxWordLength(t_Handle h_Ssi, e_SsiWordLength wordLenght);

/**
 @Function      SSI_ConfigRxSignExtensionEnabled

 @Description   This configuration allows SSI to store the received data
                word in sign extended form. This configuration affects data
                storage only in case received data is lsb aligned.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi                     -   Handle to the SSI object.
 @Param[in]     signExtensionEnabled      -   If true, sign extension is turned on.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxSignExtensionEnabled(t_Handle h_Ssi, bool signExtensionEnabled);

/**
 @Function      SSI_ConfigTxSignExtensionEnabled

 @Description   This configuration allows SSI to store the transmit data
                word in sign extended form. This configuration affects data
                storage only in case received data is lsb aligned.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi                     -   Handle to the SSI object.
 @Param[in]     signExtensionEnabled      -   If true, sign extension is turned on.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxSignExtensionEnabled(t_Handle h_Ssi, bool signExtensionEnabled);

/**
 @Function      SSI_ConfigRxClockFallingEdge

 @Description   This configuration controls which bit clock edge is used to latch in data
                for the receive section.

                Default value is [TRUE].

 @Param[in]     h_Ssi                 -   Handle to the SSI object.
 @Param[in]     rxClockFallingEdge    -   If true, data will be latched on the
                                          falling edge of the bit clock.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxClockFallingEdge(t_Handle h_Ssi, bool rxClockFallingEdge);

/**
 @Function      SSI_ConfigTxClockFallingEdge

 @Description   This configuration controls which bit clock edge is used to clock out data
                for the transmit section.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi                 -  Handle to the SSI object.
 @Param[in]     txClockFallingEdge    -  If true, data will be clocked out on the
                                             falling edge of the bit clock.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxClockFallingEdge(t_Handle h_Ssi, bool txClockFallingEdge);

/**
 @Function      SSI_ConfigRxSyncActiveLow

 @Description   This configuration controls the active state of the frame sync I/O
                signal for the receive section of SSI.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi             -   Handle to the SSI object.
 @Param[in]     rxSyncActiveLow   -   If true, frame sync is active low.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRxSyncActiveLow(t_Handle h_Ssi, bool rxSyncActiveLow);

/**
 @Function      SSI_ConfigTxSyncActiveLow

 @Description   This configuration controls the active state of the frame sync I/O
                signal for the receive section of SSI.

                Default value is [TRUE].
 
 @Param[in]     h_Ssi            -   Handle to the SSI object.
 @Param[in]     txSyncActiveLow  -   If true, frame sync is active low.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTxSyncActiveLow(t_Handle h_Ssi, bool txSyncActiveLow);

/**
 @Function      SSI_ConfigRx2TxClockLoopbackMode

 @Description   This configuration connects the receive clock to the transmit clock.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     clockLoopbackEnable -   TRUE to activate the mode.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRx2TxClockLoopbackMode(t_Handle h_Ssi, bool clockLoopbackEnable);

/**
 @Function      SSI_ConfigTx2RxClockLoopbackMode

 @Description   This configuration connects the transmit clock to the receive clock.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi               -   Handle to the SSI object.
 @Param[in]     clockLoopbackEnable -   TRUE to activate the mode.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTx2RxClockLoopbackMode(t_Handle h_Ssi, bool clockLoopbackEnable);

/**
 @Function      SSI_ConfigRx2TxFrameSyncLoopbackMode

 @Description   This configuration connects the receive frame sync to the transmit
                frame sync.

                Default value is [FALSE].

 @Param[in]     h_Ssi                   -   Handle to the SSI object.
 @Param[in]     frameSyncLoopbackEnable -   TRUE to activate the mode.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigRx2TxFrameSyncLoopbackMode(t_Handle h_Ssi, bool frameSyncLoopbackEnable);

/**
 @Function      SSI_ConfigTx2RxFrameSyncLoopbackMode

 @Description   This configuration connects the transmit frame sync to the receive
                frame sync.

                Default value is [FALSE].
 
 @Param[in]     h_Ssi                   -  Handle to the SSI object.
 @Param[in]     frameSyncLoopbackEnable -  TRUE to activate the mode.
 
 @Return        E_OK if successful or error type if failed.
 
 @Cautions      None.
*/
t_Error SSI_ConfigTx2RxFrameSyncLoopbackMode(t_Handle h_Ssi, bool frameSyncLoopbackEnable);

/** @} */ /* end of ssi_advanced_init_grp */
/** @} */ /* end of ssi_init_grp */

/**************************************************************************//**
 @Group         ssi_control_grp  SSI Control Unit

 @Description   SSI Control functions

 @{
*//***************************************************************************/

/**
 @Description   SSI Channel initialization structure
*/
typedef struct t_SsiChParam
{
    t_Handle                h_Ssi;      		/**< Handle to the SSI module */
    t_Handle                h_App;      		/**< Handle  */
    uint8_t                 channelId;  		/**< ID of the channel to be initialized. */
    e_CommMode  			commMode;			/**< Communication direction for the channel */
    t_RxStoreFunction       *f_RxStore;         /**< Function to be called when a new buffer was received.
                                                     Note that status, position and flags parameters are not used. */
    t_TxConfFunction        *f_TxConf;          /**< User's callback function called by driver upon
                                                     completion of a buffer transmission; Not invoked if NULL */
}t_SsiChParam;

/**
 @Function      SSI_Enable

 @Description   Enables the SSI.

 @Param[in]     h_Ssi    -  Handle to the SSI object.
 @Param[in]     commMode -  Selection of SSI section(s) to enable.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None.
*/
t_Error SSI_Enable(t_Handle h_Ssi, e_CommMode commMode);

/**
 @Function      SSI_Disable

 @Description   Disable the SSI.

 @Param[in]     h_Ssi    -  Handle to the SSI object.
 @Param[in]     commMode -  Selection of SSI section(s) to disable.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      None.
*/
t_Error SSI_Disable(t_Handle h_Ssi, e_CommMode commMode);

/**
 @Function      SSI_CH_ConfigAndInit

 @Description   Configuration and initialization of the SSI Channel.

 @Param[in]     p_SsiChParam  -  Pointer to a t_SsiChParam struct.

 @Return        Handle to a SSI Channel instance. NULL otherwise.
*/
t_Handle        SSI_CH_ConfigAndInit(t_SsiChParam *p_SsiChParam);

/**
 @Function      SSI_CH_Free

 @Description   Delete a SSI Channel instance

 @Param[in]     h_SsiCh  -  Handle to a SSI Channel instance.

 @Return        Handle to a SSI Channel instance. NULL otherwise.
*/
t_Error         SSI_CH_Free(t_Handle h_SsiCh);

/**
 @Function      SSI_CH_Enable

 @Description   Enables receive communication on a channel.

 @Param[in]     h_SsiCh  -  Handle to a SSI Channel instance..

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using SSI_CH_ConfigAndInit().
*/
t_Error         SSI_CH_Enable(t_Handle h_SsiCh);

/**
 @Function      SSI_CH_Disable

 @Description   Disables receive communication on a channel.

 @Param[in]     h_SsiCh  -  Handle to a SSI Channel instance..

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using SSI_CH_ConfigAndInit().
*/
t_Error         SSI_CH_Disable(t_Handle h_SsiCh);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      SSI_DumpRegs

 @Description   Dumps the memory of SSI module

 @Param[in]     h_Ssi - SSI Port descriptor returned by SSI_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SSI_DumpRegs(t_Handle h_Ssi);

#endif

/** @} */ /* end of ssi_control_grp group */

/**************************************************************************//**
 @Group         ssi_runtime_grp  SSI Runtime Unit

 @Description   SSI Runtime functions

 @{
*//***************************************************************************/
/**
 @Function      SSI_CH_Tx

 @Description   Transmit function. It schedules a channel(buffer) for transmission.
                Only single buffer is supported.

 @Param[in]     h_SsiCh     -  Handle to a SSI Channel instance.
 
 @Param[in]     p_Data      -  Pointer to the buffer to be transmitted.

 @Param[in]     h_Context   -  Context of the buffer, to be returned in the TxConf() callback.
 
 @Param[in]     length      -  Length of the buffer to be transmitted.

 @Param[in]     flags       -  Not used.

 @Return        E_OK in case no errors were encountered

 @Cautions      channel must have been previously initialized using SSI_CH_ConfigAndInit().
*/
t_Error SSI_CH_Tx(t_Handle h_SsiCh,
                  uint8_t  *p_Data,
                  t_Handle h_Context,
                  uint32_t length,
                  uint32_t flags);

/**
 @Function      SSI_CTRL_CH_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
                Will free all unsent buffer-related data, or provide the buffers to te user,
                if f_TxConf was provided.

 @Param[in]     h_SsiCh     -  Handle to a SSI Channel instance.
 
 @Return        E_OK in case no errors were encountered
*/
void SSI_CH_TxConf(t_Handle h_SsiCh);

/** @} */ /* end of ssi_runtime_grp group */
/** @} */ /* end of ssi_grp */

#endif /* __SSI_EXT_H */

