/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          tdm_ctrl_ext.h

 @Description   TDM Controller Application Programming Interface
*//***************************************************************************/

#ifndef __TDM_CTRL_EXT_H
#define __TDM_CTRL_EXT_H


#include "std_ext.h"
#include "ncsw_ext.h"


/**************************************************************************//**
 @Group         tdm_grp TDM Controller Application Programming Interface

 @Description   TDM controller functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         tdm_init_grp TDM Controller Initialization Unit

 @Description   TDM controller initialization unit functions, definitions and enums

 @{
*//***************************************************************************/

/**
 @Description  TDM channel's size
*/
typedef enum e_TdmCtrlChannelsType
{
      e_TDM_CTRL_CHANNEL_SIZE_8BIT       = 0  /**< 8 bits per word */
    , e_TDM_CTRL_CHANNEL_SIZE_8BIT_u_LAW = 1  /**< 12 bits companded to 8 bits, according to u-law */
    , e_TDM_CTRL_CHANNEL_SIZE_8BIT_A_LAW = 2  /**< 13 bits companded to 8 bits, according to A-law */
    , e_TDM_CTRL_CHANNEL_SIZE_16BIT      = 3  /**< 16 bits per word */

} e_TdmCtrlChannelsType;

/**
 @Description  TDM diagnostic modes
*/
typedef enum e_TdmCtrlDiag
{
     e_TDM_CTRL_DIAG_NORMAL  = 0x0000     /**< Normal mode: route signal through phy */
    ,e_TDM_CTRL_DIAG_INT_LPB = 0x0004     /**< Internal loop mode: route TD output to RD input */
} e_TdmCtrlDiag;

/**
 @Description  TDM interrupts
*/
typedef enum e_TdmCtrlInterrupt
{
      e_TDM_CTRL_INTR_NONE              = 0x00000000    /**< No interrupts selected */
    , e_TDM_CTRL_INTR_RX_SYNC_ERR       = 0x00000008    /**< Sync Error */
    , e_TDM_CTRL_INTR_RX_SYNC_OVERRUN   = 0x00000004    /**< Receive Overrun Error */
    , e_TDM_CTRL_INTR_TX_SYNC_ERR       = 0x00080000    /**< Sync Error */
    , e_TDM_CTRL_INTR_TX_SYNC_UNDERRUN  = 0x00040000    /**< Transmit Underrun Error */

} e_TdmCtrlInterrupt;

/**
 @Description   User callback function, called if driver Rx, Tx, DMA or functional events arise.
                The user provides this function and the driver invokes it.

 @Param[in]     h_App   - Application handle given by the user at driver configuration.
 @Param[in]     events  - Selection of user provided events from e_TdmInterrupt.
 */
typedef void (t_TdmCtrlEventsCallBack)(t_Handle h_App, uint32_t events);

/**
 @Description   Maximum number of supported channels per TDM module.

 @Cautions      Do not change this value. It is HW dependent.
 */
#define TDM_CTRL_MAX_NUM_OF_SLOTS    128

/**
 @Description  FIFO Threshold. An interrups will be generate if the Rx or Tx FIFOs reach the numer of words specified through this values.
*/
typedef enum e_TdmCtrlFifoCounter
{
      e_TDM_CTRL_FIFO_1_WORDS    = 0
    , e_TDM_CTRL_FIFO_2_WORDS    = 1
    , e_TDM_CTRL_FIFO_3_WORDS    = 2
    , e_TDM_CTRL_FIFO_4_WORDS    = 3

} e_TdmCtrlFifoCounter;

/**
 @Description   DMA Support related configuration options.
*/
typedef struct t_DmaSupportParams
{
    e_CommMode  commMode;       /**< Direction(s) to activate DMA support on */
    int         memId;          /**< Memory type for DMA related internal structures (interleaved buffers and TCDs) */
    uint32_t    memAttributes;  /**< Memory attributes for above structures. */
}t_DmaSupportParams;

/**
 @Description   TDM initialization structure, used when calling TDM_Init.
*/
typedef struct
{
    uint32_t                baseAddress;        /**< Base address of the TDM module */
    uint32_t                baseAddressDma;     /**< Base address of the TDM_DMA module */
    uint32_t                tdmCtrlClockIn;     /**< Frequency (in Hz) of the TDM Input clock */
    int                     irq;                /**< Rx, Tx and DMA interrupt */
    int                     irqErr;             /**< TDM Error Interrupt */

    uint16_t                numRxChannels;      /**< Max number of oppened Rx slots for this TDM. Reception is started on all channels, once the Rx section is enabled */
    e_TdmCtrlChannelsType   rxChannelsType;     /**< Word size for Rx */
    uint16_t                numTxChannels;      /**< Max number of oppened Tx slots for this TDM */
    e_TdmCtrlChannelsType   txChannelsType;     /**< Word size for Tx */
    t_BufferPoolInfo        bufferPoolInfo;     /**< Rx and Tx buffer's details */
 
    t_Handle                h_App;              /**< Handle to be provided as a parameter for the provided callback functions */
    t_TdmCtrlEventsCallBack *f_EventsCallback;  /**< Callback function to be called in case selected events are generated. */

} t_TdmCtrlParam;

/**
 @Function      TDM_CTRL_Config

 @Description   General configuration of the TDM controller.

 @Param[in]     p_TdmCtrlParam - Pointer to the TDM configuration parameters.

 @Return        A handle to the TDM structure, NULL on failure.
*/
t_Handle        TDM_CTRL_Config(t_TdmCtrlParam *p_TdmCtrlParam);

/**
 @Function      TDM_CTRL_Init

 @Description   General initialization of the TDM controller.

 @Param[in]     h_TdmCtrl - handle to the TDM object.

 @Return        E_OK on success; Error code otherwise.
*/
t_Error         TDM_CTRL_Init(t_Handle h_TdmCtrl);

/**
 @Function      TDM_CTRL_Free

 @Description   Release the TDM controller and its resources.

 @Param[in]     h_TdmCtrl - a handle to the TDM object.

 @Return        None.
*/
t_Error         TDM_CTRL_Free(t_Handle h_TdmCtrl);

/**************************************************************************//**
 @Group         tdm_advanced_init_grp TDM Controller Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**
 @Function      TDM_CTRL_ConfigRxBlockSize

 @Description   User will receive data in blocks of size 'rxBlockSize'. By default, maximum size of a buffer from 'bufferPoolInfo' will be used.

 @Param[in]     h_TdmCtrl   -  Handle to TDM
 @Param[in]     rxBlockSize -  Contiguous data blocks' size to be provided through f_RxBufferCallback.


 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
                Only 24 channel can be used.
*/
t_Error         TDM_CTRL_ConfigRxBlockSize (t_Handle h_TdmCtrl, uint32_t rxBlockSize);

/**
 @Function      TDM_CTRL_ConfigT1Support

 @Description   If called, will enable T1 support for the TDM module.[FALSE]

 @Param[in]     h_TdmCtrl       -  Handle to TDM
 @Param[in]     t1Support       -  TRUE for activating T1 support.


 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
                Only 24 channel can be used.
*/
t_Error         TDM_CTRL_ConfigT1Support (t_Handle h_TdmCtrl, bool t1Support);

/**
 @Function      TDM_CTRL_ConfigDiagMode

 @Description   Configure the TDM with 'diagMode'. [e_TDM_CTRL_DIAG_NORMAL]

 @Param[in]     h_TdmCtrl       -  Handle to TDM
 @Param[in]     diagMode        -  Diagnostic mode
 

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigDiagMode (t_Handle h_TdmCtrl, e_TdmCtrlDiag diagMode);

/**
 @Function      TDM_CTRL_ConfigNonCommonTxRx

 @Description   Force the TDM not to use common receive and transmit pins. When not called, given different clocks for Rx and Tx the TDM will use Tx Frame Sync as both Rx/Tx Frame Sync( RFS = TFS ). [FALSE]

 @Param[in]     h_TdmCtrl           -  Handle to TDM
 @Param[in]     nonCommonRxTx       -  TRUE for separate Sync/Clock signals.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigNonCommonTxRx (t_Handle h_TdmCtrl, bool nonCommonRxTx);

/**
 @Function      TDM_CTRL_ConfigFifoSupport

 @Description   When called, Rx/Tx FIFO will be enabled. [e_COMM_MODE_NONE]

 @Param[in]     h_TdmCtrl       -  Handle to TDM
 @Param[in]     commMode        -  Enable FIFO support fo RX and/or TX

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigFifoSupport (t_Handle h_TdmCtrl, e_CommMode commMode);

/**
 @Function      TDM_CTRL_ConfigRxFifoThreshold

 @Description   Configure the number of elements in receive FIFO, that will generate the FIFO Full event. [e_TDM_CTRL_FIFO_1_WORDS]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     fifoCounter     -  Threshold value
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxFifoThreshold (t_Handle h_TdmCtrl, e_TdmCtrlFifoCounter fifoCounter);

/**
 @Function      TDM_CTRL_ConfigTxFifoThreshold

 @Description   Configure the number of elements in transmit FIFO, that will generate the FIFO Full event. [e_TDM_CTRL_FIFO_1_WORDS]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     fifoCounter     -  Threshold value
 
 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxFifoThreshold (t_Handle h_TdmCtrl, e_TdmCtrlFifoCounter fifoCounter);

/**
 @Function      TDM_CTRL_ConfigRxFifoWide

 @Description   FIFO entries will not be 8/16bit samples, but one 64bit sample (of 8/16bit packed samples).[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     rxFifoWide      -  TRUE for activating WIDE FIFO mode on Rx.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxFifoWide (t_Handle h_TdmCtrl, bool rxFifoWide);

/**
 @Function      TDM_CTRL_ConfigTxFifoWide

 @Description   FIFO entries will not be 8/16bit samples, but one 64bit sample (of 8/16bit packed samples).[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     txFifoWide      -  TRUE for activating WIDE FIFO mode on Tx.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*//***************************************************************************/
t_Error         TDM_CTRL_ConfigTxFifoWide (t_Handle h_TdmCtrl, bool txFifoWide);

/**************************************************************************//**
 @Function      TDM_CTRL_ConfigRxSyncChannelLength

 @Description   Rx Frame Sync (RFS) can be asserted for the period of the entire first channel of a frame. By default, it is asserted for a single cycle.[FALSE]

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     rxSyncChannelLength -  TRUE for activating Rx Sync signal for the entire period of the first channel.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxSyncChannelLength (t_Handle h_TdmCtrl, bool rxSyncChannelLength);

/**
 @Function      TDM_CTRL_ConfigTxSyncChannelLength

 @Description   If called, Tx Frame Sync (TFS) will be asserted for the period of the entire first channel of a frame. By default, it is asserted for a single cycle.[FALSE]

 @Param[in]     h_TdmCtrl               -  handle to TDM
 @Param[in]     txSyncChannelLength     -  TRUE for activating Tx Sync signal for the entire period of the first channel.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxSyncChannelLength (t_Handle h_TdmCtrl, bool txSyncChannelLength);

/**
 @Function      TDM_CTRL_ConfigRxIntSync

 @Description   If called, Rx Frame Sync (RFS) signal will be internally generated (driven out). By default, it expects externally generated (driven in).[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     rxIntSync       -  TRUE for internally generating the Rx Sync signal (and drive it out).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxIntSync (t_Handle h_TdmCtrl, bool rxIntSync);

/**
 @Function      TDM_CTRL_ConfigTxIntSync

 @Description   If called, Tx Frame Sync (TFS) signal will be internally generated (driven out). By default, it expects externally generated (driven in).[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     txIntSync       -  TRUE for internally generating the Rx Sync signal (and drive it out).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxIntSync (t_Handle h_TdmCtrl, bool txIntSync);

/**
 @Function      TDM_CTRL_ConfigRxIntClk

 @Description   If called will configure Rx Clock (RCK) signal to be internally generated (driven out). Default is externally generated (driven in).[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     rxIntDataClk    -  TRUE for internally generating the Rx Clock signal (and drive it out).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxIntClk (t_Handle h_TdmCtrl, bool rxIntDataClk);

/**
 @Function      TDM_CTRL_ConfigTxIntClk

 @Description   If called will configure Tx Clock (TCK) signal to be internally generated (driven out). Default is externally generated (driven in).[FALSE]

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     txIntDataClk        -  TRUE for internally generating the Tx Clock signal (and drive it out).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxIntClk (t_Handle h_TdmCtrl, bool txIntDataClk);

/**
 @Function      TDM_CTRL_ConfigRxSyncActiveLow

 @Description   Rx Sync (RFS) signal can be active on logical 0.Default is active on logical 1.[FALSE]

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     rxSyncActiveLow     -  TRUE for generating Rx Sync signal as logic '0'.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxSyncActiveLow (t_Handle h_TdmCtrl,bool rxSyncActiveLow);

/**
 @Function      TDM_CTRL_ConfigTxSyncActiveLow

 @Description   Tx Sync (RFS) signal can be active on logical 0.Default is active on logical 1.[FALSE]

 @Param[in]     h_TdmCtrl       -  handle to TDM
 @Param[in]     txSyncActiveLow -  TRUE for generating Tx Sync signal as logic '0'.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxSyncActiveLow (t_Handle h_TdmCtrl, bool txSyncActiveLow);

/**
 @Function      TDM_CTRL_ConfigDmaSupport

 @Description   Activate DMA servicing for Rx and/or Tx.[e_COMM_MODE_NONE]

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     p_DmaSupportParams  -  Parameters necessary for DMA support configuration.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigDmaSupport (t_Handle h_TdmCtrl, t_DmaSupportParams *p_DmaSupportParams);

/**
 @Function      TDM_CTRL_ConfigTxDmaBlockSize

 @Description   All DMA transfers used to put data on line will handle the same amount of data (txBlockSize), thus, when DMA TX support is enabled, all the channels should transmit txBlockSize.

 @Param[in]     h_TdmCtrl   -  Handle to TDM
 @Param[in]     txBlockSize -  Size of internal DMA transfers


 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxDmaBlockSize (t_Handle h_TdmCtrl, uint32_t txBlockSize);

/**
 @Function      TDM_CTRL_ConfigNumTxBuffersPerChannel

 @Description   More Tx Buffers can be scheduled for transmission through consecutive calls of TDM_CTRL_CH_Tx(). Set the maximum number of buffers here.[2]

 @Param[in]     h_Tdm               -  Handle to TDM
 @Param[in]     numBuffersPerChannel-  Maximum number of buffers per channel.


 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error TDM_CTRL_ConfigNumTxBuffersPerChannel (t_Handle h_Tdm, uint8_t numBuffersPerChannel);

/**
 @Function      TDM_CTRL_ConfigRxFrameSyncDelay

 @Description   Configure the TDM with 'delay' - Transmit frame sync delay, 0- no bit delay,x- x bit delay (x=1,2,3) [0]

 @Param[in]     h_TdmCtrl   -  handle to TDM
 @Param[in]     delay       -  new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxFrameSyncDelay (t_Handle h_TdmCtrl, uint8_t delay);

/**
 @Function      TDM_CTRL_ConfigTxFrameSyncDelay

 @Description   Configure the TDM with 'delay' - Transmit frame sync delay, 0- no bit delay,x- x bit delay (x=1,2,3) [0]

 @Param[in]     h_TdmCtrl   -  handle to TDM
 @Param[in]     delay       -  new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxFrameSyncDelay (t_Handle h_TdmCtrl, uint8_t delay);

/**
 @Function      TDM_CTRL_ConfigRxClockFallingEdge

 @Description   Received data can be driven out on the falling edge of the Rx Clock (RCK). By default, data is driven on the rising edge. [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     rxClockFallingEdge  -  TRUE for sampling data on the Rx Clock's falling edge

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxClockFallingEdge (t_Handle h_TdmCtrl, bool rxClockFallingEdge);

/**
 @Function      TDM_CTRL_ConfigTxClockFallingEdge

 @Description   If called, received data will be driven out on the falling edge of the Tx Clock (TCK). By default, data is driven on the rising edge. [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     txClockFallingEdge  -  TRUE for sampling data on the Tx Clock's falling edge

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxClockFallingEdge (t_Handle h_TdmCtrl, bool txClockFallingEdge);

/**
 @Function      TDM_CTRL_ConfigRxFrameSyncFallingEdge

 @Description   RxFrameSync signal (RFS) can be driven on the Falling edge of the Rx Clock (RCK). By default, RFS is driven on the rising edge. [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     rxSyncFallingEdge   -  TRUE for generating Rx Sync signal on the falling edge of the Rx Clock

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxFrameSyncFallingEdge (t_Handle h_TdmCtrl, bool rxSyncFallingEdge);

/**
 @Function      TDM_CTRL_ConfigTxFrameSyncFallingEdge

 @Description   TxFrameSync signal (RFS) can be driven on the Falling edge of the Tx Clock (TCK). By default, RFS is driven on the rising edge. [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     txSyncFallingEdge   -  TRUE for generating Tx Sync signal on the falling edge of the Tx Clock

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxFrameSyncFallingEdge (t_Handle h_TdmCtrl, bool txSyncFallingEdge);

/**
 @Function      TDM_CTRL_ConfigRxReversedDataOrder

 @Description   The first bit of a received channel can be stored as the most significant bit in the on-chip memory. By default it is stored as least significant [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     rxReversedDataOrder -  TRUE for storing the first received bit as the MSB bit

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigRxReversedDataOrder (t_Handle h_TdmCtrl, bool rxReversedDataOrder);

/**
 @Function      TDM_CTRL_ConfigTxReversedDataOrder

 @Description   The most significant bit in the memory can be sent out as the first transmit data bit. By default, the least significant bit is sent out first. [FALSE].

 @Param[in]     h_TdmCtrl           -  handle to TDM
 @Param[in]     txReversedDataOrder -  TRUE for transmitting the MSB bit in the memory as the first bit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigTxReversedDataOrder (t_Handle h_TdmCtrl, bool txReversedDataOrder);

/**
 @Function      TDM_CTRL_ConfigInterrupts

 @Description   Configure the active interrupts.[0]

 @Param[in]     h_TdmCtrl   -  handle to TDM
 @Param[in]     interrupts  -  new value

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling TDM_Init.
*/
t_Error         TDM_CTRL_ConfigInterrupts (t_Handle h_TdmCtrl, uint32_t interrupts);

/** @} */ /* end of tdm_advanced_init_grp group */
/** @} */ /* end of tdm_init_grp group */


/**************************************************************************//**
 @Group         tdm_control_grp TDM Controller Control Unit

 @Description   TDM control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**
 @Description   TDM initialization structure, used when calling TDM_Init.
*/
typedef struct t_TdmCtrlChParam
{
    t_Handle            h_TdmCtrl;          /**< Handle to the TDM module */
    t_Handle            h_App;              /**< Handle that will be provided back in f_RxBufferCallback */
    uint8_t             channelId;          /**< ID of the channel to be initialized. */
    e_CommMode          commMode;           /**< Communication dir(s) for the channel */
    t_TxConfFunction    *f_TxConf;          /**< User's callback function called by driver upon
                                                 completion of a buffer transmission; Not invoked if NULL */
    t_RxStoreFunction   *f_RxBufferCallback;/**< Function to be called when a new buffer was received.
                                                 Note that status, position and flags parameters are not used. */
} t_TdmCtrlChParam;

/**
 @Function      TDM_CTRL_Enable

 @Description   Allows enablement of specific sections of the TDM (Receiver and/or Transmitter)

 @Param[in]     h_TdmCtrl   -  a handle to the TDM object.
 @Param[in]     commMode    -  Selection of TDM section(s) to enable.

 @Return        E_OK on success; Error code otherwise.
*/
t_Error         TDM_CTRL_Enable(t_Handle h_TdmCtrl, e_CommMode commMode);

/**
 @Function      TDM_CTRL_Disable

 @Description   Disables a TDM channel.

 @Param[in]     h_TdmCtrl   -  a handle to the TDM object.
 @Param[in]     commMode    -  Selection of TDM section(s) to disable.
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The user must set a frame prior to enabling the TDM.
*/
t_Error         TDM_CTRL_Disable(t_Handle h_TdmCtrl, e_CommMode commMode);

/**
 @Function      TDM_CTRL_CH_ConfigAndInit

 @Description   General configuration and initialization of the TDM Channel.

 @Param[in]     p_TdmCtrlChParam    -  Pointer to a t_TdmChParam struct.

 @Return        Handle to a TDM Channel instance. NULL otherwise.
*/
t_Handle        TDM_CTRL_CH_ConfigAndInit(t_TdmCtrlChParam *p_TdmCtrlChParam);

/**
 @Function      TDM_CTRL_CH_ConfigAndInit

 @Description   Delete a TDM Channel instance.

 @Param[in]     h_TdmCtrlCh  -  Handle to a TDM Channel instance..

 @Return        N/A
*/
t_Error         TDM_CTRL_CH_Free(t_Handle h_TdmCtrlCh);

/**
 @Function      TDM_CTRL_CH_Enable

 @Description   Enables a channel (continuous reception). As soon as a buffer from bufferPoolInfo is filled, it is provided through f_RxBufferCallback, and reception in a new buffer taken from the pool is started. Tx enablement is done when data to be transmitted was added.

 @Param[in]     h_TdmCtrlCh -  Handle to a TDM Channel instance.

 @Cautions      Buffer pool sizing and processing-time per buffer must be taken into consideration: if received buffers are not returned in the pool, it will eventually run-out of buffers.
 @Return        E_OK in case no errors were encountered
 
*/
t_Error         TDM_CTRL_CH_Enable(t_Handle h_TdmCtrlCh);

/**
 @Function      TDM_CTRL_CH_Disable

 @Description   Disables reception on the channel.

 @Param[in]     h_TdmCtrlCh -  Handle to a TDM Channel instance.

 @Cautions      Buffer pool sizing and processing-time per buffer must be taken into consideration: if received buffers are not returned in the pool, it will eventually run-out of buffers.
 @Return        E_OK in case no errors were encountered
 
*/
t_Error         TDM_CTRL_CH_Disable(t_Handle h_TdmCtrlCh);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
/**
 @Function      TDM_CTRL_DumpRegs

 @Description   Dump (print) all TDM registers.

 @Param[in]     h_TdmCtrl  -  a handle to the TDM object.
*/
t_Error         TDM_CTRL_DumpRegs(t_Handle h_TdmCtrl);

#endif /* (defined(DEBUG_ERRORS) && ... */

/** @} */ /* end of tdm_control_grp group */

/**************************************************************************//**
 @Group         tdm_runtime_grp TDM Controller Runtime Unit

 @Description   TDM Runtime unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**
 @Function      TDM_CTRL_CH_Tx

 @Description   Transmit function. It schedules a channel(buffer) for transmission. Multiple buffers cannot be scheduled for the same channel.

 @Param[in]     h_TdmCtrlCh -  Handle to a TDM Channel instance.
 @Param[in]     p_Data      -  Pointer to the buffer to be transmitted.
 @Param[in]     h_Context   -  Context of the buffer, to be returned in the TxConf() callback.
 @Param[in]     length      -  Length of the buffer to be transmitted.
 @Param[in]     flags       -  Not used for now.

 @Return        E_OK in case no errors were encountered
*/
t_Error         TDM_CTRL_CH_Tx(t_Handle h_TdmCtrlCh,
                               uint8_t  *p_Data,
                               t_Handle h_Context,
                               uint32_t length,
                               uint32_t flags);

/**
 @Function      TDM_CTRL_CH_TxConf

 @Description   Will free all unsent buffer-related data, or provide the buffers to te user, if f_TxConf was set.
 @Param[in]     h_TdmCtrlCh -  Handle to a TDM Channel instance.

 @Return        E_OK in case no errors were encountered
*/
void            TDM_CTRL_CH_TxConf(t_Handle h_TdmCtrlCh);

/** @} */ /* end of tdm_runtime_grp group */
/** @} */ /* end of tdm_grp group */

 
#endif /* __TDM_CTRL_EXT_H */

