/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_aal0_ext.h

 @Description   UCC Atm Aal0 external structures and definitions.
 
                Notes:
                1. If using the framed mode option, the cell will be wrapped up
                   in a single bufferred frame type SBF.
                   The following info's will hold the data describing the cell.
                   info3 -- Time stamp for incomming cells.
                   info4 -- Cell Type.
*//***************************************************************************/

#ifndef __UCC_ATM_AAL0_EXT_H
#define __UCC_ATM_AAL0_EXT_H

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal0_channel_grp UCC ATM AAL0 Channel

 @Description   Ucc Atm Aal0 Channel

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal0_channel_init_grp UCC ATM AAL0 Channel Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/
#include "ucc_atm_ch_ext.h"

/**************************************************************************//**
 @Description   AAL0 channel initialization parameters.
 *//***************************************************************************/
typedef struct t_UccAtmAal0Param
{
    t_UccAtmChannelParam    uccAtmChParam;      /**< ATM channel general driver parameters */
    t_BufferPoolInfo        bufferPoolInfo;     /**< Data buffer pool information */
    uint32_t                dataMemAttributes;  /**< Data memory partition attributes  */
    bool                    crc10;              /**< 10 bit CRC support */
 
    /*****************/
    /* Rx parameters */
    /*****************/
    t_RxStoreFunction       *f_RxStore;	        /**< pointer to application layer's routine - called to pass a received frame or data pointer to the application. */
    bool                    rxInterrupts;       /**< working with Receive interrupts indication */
    uint32_t                intrManagerId;      /**< the required queue id for messages from the driver to the upper layer
                                                     (relevant only if the work is with recevie interrupts(rxInterrupts parameter is set) */
    bool                    timeStampEnable;    /**< enable/disable the option to add time stamp information to the end of the buffer - useful for OAM support */
    bool                    atmPayloadOnly;     /**< if enabled only the cell payload is written to the buffer, if disabled all the cell including the header and udc are written to the buffer */
    uint16_t                rxBdRingLength;     /**< Rx BD ring pool length; If it equals 0 -
                                                     the default (or configured by UCC_ATM_ConfigBdRingLen) UCC BD ring length will be used */
 
    /*****************/
    /* Tx Parameters */
    /*****************/
    t_TxConfFunction        *f_TxConf;          /**< pointer to upper layer's routine - called to handle transmitted frames.*/
    bool                    headerInBuffer;     /**< Cell header is transmitted as part of the buffer;
                                                     If FALSE, the cell header is set according to VPI, VCI passed by the user. */
    uint16_t                txBdRingLength;     /**< Tx BD ring pool length; If it equals 0 -
                                                     the default (or configured by UCC_ATM_ConfigBdRingLen) UCC BD ring length will be used */

} t_UccAtmAal0Param;

/**************************************************************************//**
 @Description   AAL0 channel cell type.
 *//***************************************************************************/
typedef enum e_Aal0CellType
{
      e_AAL0_CELL_TYPE_USER = 0,             /**< cell type - User data Cell */
      e_AAL0_CELL_TYPE_OAM  = 0x00100000     /**< cell type - OAM (Operation And Maintenance) Cell */
} e_Aal0CellType;

/**************************************************************************//**
 @Anchor        aal0_oam_idx
 
 @Collection    AAL0 OAM Application Handles Array Indices
 
                These are the indices of the array of port and original channel
                application handles, which is passed in the f_RxStore callback.
 @{
*//***************************************************************************/
#define RX_STORE_OAM_PORT_HANDLE_INDEX      0   /**< Port Application ID handle index */
#define RX_STORE_OAM_CHANNEL_HANDLE_INDEX   1   /**< Original Channel Application ID handle index */
/* @} */

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_ConfigAndInit

 @Description   Config and Initialize driver and hardware for AAL0 channel.
 
 @Param[in]     p_UccAtmAal0Param - user parameters for Atm Aal0 channel.

 @Return        Atm Aal0 channel handler.
*//***************************************************************************/
t_Handle UCC_ATM_AAL0_ConfigAndInit( t_UccAtmAal0Param *p_UccAtmAal0Param );

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_Free

 @Description   Free memory of the AAL0 channel handler.

 @Param[in]     h_UccAtmAal0 - Atm Aal0 channel handler.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      aal0Channel must have been previously initialized using UCC_ATM_AAL0_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL0_Free(t_Handle h_UccAtmAal0);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_SetAutomaticDataForwarding

 @Description   Set the possibility to switch ATM AAL0 cells from one port to another without core intervantion.

                The ATM receiver and transmitter will be programed to process the same BD table,
                so when the ATM receiver fills an AAL0 buffer, the ATM transmitter sends it.
 
 @Param[in]     h_UccAtmAal0Rx - Atm Aal0 Rx channel handler to be used.
 @Param[in]     h_UccAtmAal0Tx - Atm Aal0 Tx channel handler to be used.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      AAL0 Rx and Tx channels must have been previously initialized.
*//***************************************************************************/
t_Error UCC_ATM_AAL0_SetAutomaticDataForwarding(t_Handle h_UccAtmAal0Rx, t_Handle h_UccAtmAal0Tx);

/** @} */ /* end of ucc_atm_aal0_channel_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal0_channel_ctrl_grp UCC ATM AAL0 Channel Control Unit

 @Description   Control and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_TxConf

 @Description   Forces confirmation on transmitted cells or data buffers.
 
                Invokes user's tx confirmation call back routine for previously
                transmitted data on the specific AAL0 channel.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.
 
 @Param[in]     h_UccAtmAal0 - driver's handler of the ATM channel.
 
 @Return        None.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                the transmit routines) simultaneously at task level and at
                interrupt level may lead to unpredictable results.
*//***************************************************************************/
void    UCC_ATM_AAL0_TxConf ( t_Handle h_UccAtmAal0 );

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellType

 @Description   Add type to OAM Cell Type Field.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        None.
*//***************************************************************************/
uint8_t UCC_ATM_AAL0_GetOamCellType(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_SetOamCellType

 @Description   Add type to OAM Cell Type Field.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 @Param[in]     oamType - OAM Type to be added to OAM cell.
 
 @Return        None.
*//***************************************************************************/
void UCC_ATM_AAL0_SetOamCellType(t_Handle h_OamCell, uint8_t oamType);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmMscn

 @Description   Get Monitoring Cell Sequence Number value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Monitoring Cell Sequence Number.
*//***************************************************************************/
uint8_t UCC_ATM_AAL0_GetOamCellPmMcsn(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmRmscn

 @Description   Get Reported Monitoring Cell Sequence Number value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Reported Monitoring Cell Sequence Number.
*//***************************************************************************/
uint8_t UCC_ATM_AAL0_GetOamCellPmRmcsn(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmTuc

 @Description   Get Total User Cells (CLP 0+1) value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Total User Cells (CLP 0+1).
*//***************************************************************************/
uint16_t UCC_ATM_AAL0_GetOamCellPmTuc(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmTuc0

 @Description   Get Total User Cells (CLP 0) value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Total User Cells (CLP 0).
*//***************************************************************************/
uint16_t UCC_ATM_AAL0_GetOamCellPmTuc0(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmTrcc

 @Description   Get Total Received Cell Count (CLP 0+1) value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Total Received Cell Count (CLP 0+1).
*//***************************************************************************/
uint16_t UCC_ATM_AAL0_GetOamCellPmTrcc(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmTrcc0

 @Description   Get Total Received Cell Count (CLP 0) value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Total Received Cell Count (CLP 0).
*//***************************************************************************/
uint16_t UCC_ATM_AAL0_GetOamCellPmTrcc0(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmTimeStamp

 @Description   Get Time Stamp of Transmission value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Time Stamp of Transmission.
*//***************************************************************************/
uint32_t UCC_ATM_AAL0_GetOamCellPmTimeStamp(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmBedc

 @Description   Get Block Error Detection Code (CLP 0+1) value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Block Error Detection Code
*//***************************************************************************/
uint16_t UCC_ATM_AAL0_GetOamCellPmBedc(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmBler

 @Description   Get Block Error Result value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Block Error Result.
*//***************************************************************************/
uint8_t UCC_ATM_AAL0_GetOamCellPmBler(t_Handle h_OamCell);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_SetOamCellPmBler

 @Description   Set Block Error Result value on OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 @Param[in]     blerVal - Block Error Result value to set on OAM cell.
 
 @Return        None.
*//***************************************************************************/
void UCC_ATM_AAL0_SetOamCellPmBler(t_Handle h_OamCell, uint8_t blerVal);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_GetOamCellPmSecbc

 @Description   Get Severly Errored Cell Block Count value from OAM Cell.
 
 @Param[in]     h_OamCell - Handle to the OAM cell data.
 
 @Return        Block Error Result.
*//***************************************************************************/
uint8_t UCC_ATM_AAL0_GetOamCellPmSecbc(t_Handle h_OamCell);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_AAL0_DumpRegs

 @Description   Dumps ATM AAL0 Channel registers and parameters RAM.

 @Param[in]     h_UccAtmAal0 - Handle to the AAL0 channel object.

 @Return        None.

 @Cautions      AAL0 Channel must have been previously initialized using UCC_ATM_AAL0_ConfigAndInit().

*//***************************************************************************/
void UCC_ATM_AAL0_DumpRegs(t_Handle h_UccAtmAal0);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_aal0_channel_ctrl_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal0_channel_runtime_grp UCC ATM AAL0 Channel Runtime Unit

 @Description   Runtime Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        aal0_rx_bm
 
 @Collection    UCC ATM AAL0 Rx Errors and Status Flags
 
                These values can be or'ed in any combination into the status
                parameter of the f_RxStore callback.
 @{
*//***************************************************************************/
#define UCC_ATM_AAL0_RX_ERRORS_PNC  0x0080  /**< UPC Non conformance; The buffer
                                                 contains a cell which was not
                                                 conformed by one of the buckets. */
#define UCC_ATM_AAL0_RX_ERRORS_CRE  0x0020  /**< CRC error. */
#define UCC_ATM_AAL0_RX_OAM         0x0010  /**< OAM indication */
/* @} */

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_Tx

 @Description   Transmits a data buffer(which is actually a data of a cell) on an AAL0 channel.

                Accepts data as a pointer to a single data buffer (array of
                bytes).

                This routine eventually causes the user's transmit confirmation
                call-back routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_ATM_AAL0_TxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.
 
 @Param[in]     h_UccAtmAal0    - ATM AAL0 channel handler to be initialized.
 @Param[in]     p_Data          - Pointer to the cells data.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     cellType        - The type of the cell, one of: CELL_USER, CELL_OAM.
 
 @Return        E_OK if succeeded to put the cell into Tx BDs; Error code otherwise.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                UCC_ATM_AAL0_TxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.
*//***************************************************************************/
t_Error UCC_ATM_AAL0_Tx(t_Handle        h_UccAtmAal0,
                        uint8_t         *p_Data,
                        t_Handle        h_BufContext,
                        e_Aal0CellType  cellType);

/**************************************************************************//**
 @Function      UCC_ATM_AAL0_Rx

 @Description   Handles data received in the specific AAL0 channel.

                Receive data are passed to the user via the user receive call
                back routine corresponding to the channel.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.
 
 @Param[in]     h_UccAtmAal0 - driver's handler of the ATM channel.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL0_Rx ( t_Handle h_UccAtmAal0 );

/** @} */ /* end of aal0_runtime_grp group */

/** @} */ /* end of ucc_atm_aal0_channel_grp group */

/** @} */ /* end of group ucc_atm_grp */

#endif /* __UCC_ATM_AAL0_EXT_H */
