/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_aal1_ext.h

 @Description   UCC Atm Aal1 external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_AAL1_EXT_H
#define __UCC_ATM_AAL1_EXT_H

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal1_channel_grp UCC ATM AAL1 Channel

 @Description   Ucc Atm Aal1 Channel

 @{
*//***************************************************************************/
#include "ucc_atm_ch_ext.h"
/* AAL1 Errors Mask */
#define UCC_ATM_AAL1_RX_ERRORS_PNC      0x0080  /**< UCC ATM Channel Rx errors returned via RxStore callback:
                                                 UPC Non conformance. The buffer contains a cell which
                                                 was not conformed by one of the buckets. */

#define UCC_ATM_AAL1_RX_ERRORS_SNE      0x0800  /**< UCC ATM Channel Rx errors returned via RxStore callback:
                                                 Sequence Number Error. */

/**************************************************************************//**
 @Group         ucc_atm_aal1_channel_init_grp UCC ATM AAL1 Channel Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   AAL1 Channel parameters.
 *//***************************************************************************/

/**************************************************************************//**
 @Description   AAL1 channel user-defined parameters.

                The structure contains some parameters that are general parameters
                and not specific AAL1 parameters. However, they are written here
                since they are not suitable for AAL1 (like f_GetDataBuf,f_TxConf...)
 *//***************************************************************************/
typedef struct t_UccAtmAal1Param
{
    t_UccAtmChannelParam    uccAtmChParam;      /**< ATM channel general driver parameters */
    t_BufferPoolInfo        bufferPoolInfo;     /**< Data buffer pool information */
    uint32_t                dataMemAttributes;  /**< Data memory partition attributes  */

    bool                    partFillMode;   /**< True - Partially filled cells mode is used */
    bool                    structFormat;   /**< True - Rx structured format is used */
    uint16_t                structBlockSize;/**< Structured Block Size */
    bool                    srtsMode;       /**< True - SRTS mode is used; For unstructured format only. */


    /*****************/
    /* Rx parameters */
    /*****************/
    t_RxStoreFunction      *f_RxStore;	     /**< Pointer to upper layer's routine - called to pass a received frame to the application. */
    bool                    rxInterrupts;    /**< Working with Rx interrupts */
    uint32_t                intrManagerId;   /**< The required queue id for messages from the driver to the upper layer  */

    uint8_t                 rxValidOctetSize;/**< Valid octet size for receive ( 1-47 for unstructured format, 1-46 for strctured format); Used only if partially filled cells mode is selected. */
    uint8_t                 rxSrtsDevice;    /**< Rx SRTS Address 4 lsbs */
    uint16_t                rxBdRingLength;  /**< Rx BD ring pool length; If it equals 0 -
                                                the default (or configured by UCC_ATM_ConfigBdRingLen()) UCC BD ring length will be used */

    /*****************/
    /* Tx Parameters */
    /*****************/
    t_TxConfFunction       *f_TxConf;        /**< Pointer to upper layer's routine - called to handle transmitted frames.*/

    uint8_t                 txValidOctetSize;/**< Valid octet size for transmit( 1-47 for unstructured format, 1-46 for strctured format); Used only if partially filled cells mode is selected. */
    uint8_t                 txSrtsDevice;    /**< Tx SRTS Address 4 lsbs */
    uint16_t                txBdRingLength;  /**< Tx BD ring pool length; If it equals 0 -
                                                the default (or configured by UCC_ATM_ConfigBdRingLen()) UCC BD ring length will be used */
 
} t_UccAtmAal1Param;

/**************************************************************************//**
 @Function      UCC_ATM_AAL1_ConfigAndInit

 @Description   Configs and Initializes driver and hardware for atm channel.
 
 @Param[in]     p_UccAtmAal1Param - user parameters for Atm Aal1 channel.

 @Return        Atm Aal1 channel handler.
*//***************************************************************************/
t_Handle UCC_ATM_AAL1_ConfigAndInit( t_UccAtmAal1Param *p_UccAtmAal1Param );

/**************************************************************************//**
 @Function      UCC_ATM_AAL1_Free

 @Description   Frees memory of the Aal1 channel handler.

 @Param[in]     h_UccAtmAal1 - Atm Aal1 channel handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      aal1Channel must have been previously initialized using UCC_ATM_AAL1_Init().
*//***************************************************************************/
t_Error UCC_ATM_AAL1_Free(t_Handle h_UccAtmAal1);

/** @} */ /* end of ucc_atm_aal1_channel_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal1_channel_ctrl_grp UCC ATM AAL1 Channel Control Unit

 @Description   Control and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL1_TxConf

 @Description   Collects the transmitted data buffers and confirms the application about them.
 
                It is also prepearing the Tx BDs for new data.
                If the Tx interrupts are disabled, the application should
                call that routine to get confirmation about the transmitted
                data buffers. Otherwise, the routine is called from the interrupt
                service routine during the Tx routine. In that case the
                application is informed by calling the application specific
                f_TxConf routine.

 @Param[in]     h_UccAtmAal1 - driver's handler of the ATM channel
 
 @Return        None.
*//***************************************************************************/
void    UCC_ATM_AAL1_TxConf ( t_Handle h_UccAtmAal1 );

/** @} */ /* end of ucc_atm_aal1_channel_ctrl_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal1_channel_runtime_grp UCC ATM AAL1 Channel Runtime Unit

 @Description   Runtime Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL1_Tx

 @Description   Transmits data in an AAL1 channel.
 
                Sets the given data buffer into the Tx BD. If there is no free
                Tx BD, returns an error. The application is responsible
                to transmit the same buffer again (if required).

 @Param[in]     h_UccAtmAal1    - ATM AAL1 channel handler to be initialized.
 @Param[in]     p_Data          - Pointer to the data to transmit.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length          - Data length.
@if UNDER_CONSTRUCTION_UDC
 @Param[in]     p_TxExtraHeader - The frame extra header ( if there is ).
@endif
 
 @Return        E_OK if succeeded to put the cell into Tx BDs; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL1_Tx(t_Handle    h_UccAtmAal1,
                        uint8_t     *p_Data,
                        t_Handle    h_BufContext,
                        uint16_t    length
                        );

/**************************************************************************//**
 @Function      UCC_ATM_AAL1_Rx

 @Description   Reads the received data and passes them to the application.
 
                For each Rx BD that holds a received buffer, it calls the user's
                routine to pass the data and additional information to the application.
 
                When intrManagerId parameter is 0, which means that no message
                mechanism is used, and receive interrupts are enabled,
                the routine is called from the driver's interrupt routine
                upon Rx buffer interrupt.
 
                When intrManagerId parameter grater than 0 (which means that there
                is a message queue) and rx interrupts are enabled, the XX_Call()
                routine is called from the driver's interrupt routine upon
                Rx buffer interrupt. In that case, the channel's receive
                interrupts are cleared until the application will call the
                UCC_ATM_AAL1_Rx() routine to collect the received cells.
                The XX_Call() routine should just inform the application about
                received data. The application is responsible to call the
                UCC_ATM_AAL1_Rx() to collect the received data.
 
                The application should call the UCC_ATM_AAL1_Rx() routine
                when polling mechanism is used ( when receive interrupts
                are disabled ).
 
                At the end of the routine it enables the Rx interrupts ( if
                intrManagerId not 0) and saves the last BD in the channel's data
                structure in order to use it in the next call.
 
                Note that before notifying the upper layer on the Rx interrupt,
                all interrupts upon receiving buffers on this channel are
                disabled until leaving UCC_ATM_AAL1_Rx() routine.
 
 @Param[in]     h_UccAtmAal1 - driver's handler of the ATM channel
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL1_Rx(t_Handle h_UccAtmAal1);

/** @} */ /* end of aal1_runtime_grp group */

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_AAL1_DumpRegs

 @Description   Dumps ATM AAL1 Channel registers and parameters RAM.

 @Param[in]     h_UccAtmAal1 - Handle to the AAL1 channel object.

 @Return        None.

 @Cautions      AAL1 Channel must have been previously initialized using UCC_ATM_AAL1_ConfigAndInit().

*//***************************************************************************/
void UCC_ATM_AAL1_DumpRegs(t_Handle h_UccAtmAal1);
#endif /* DEBUG_ERRORS > 0 */

/** @} */ /* end of ucc_atm_aal1_channel_grp group */
/** @} */ /* end of group ucc_atm_grp */

#endif /* __UCC_ATM_AAL1_EXT_H */
