/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_aal5_ext.h

 @Description   UCC Atm Aal5 external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_AAL5_EXT_H
#define __UCC_ATM_AAL5_EXT_H

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_aal5_channel_grp UCC ATM AAL5 Channel

 @Description   Ucc Atm Aal5 Channel

 @{
*//***************************************************************************/
#include "ucc_atm_ch_ext.h"

/**************************************************************************//**
 @Group         ucc_atm_aal5_channel_init_grp UCC ATM AAL5 Channel Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   AAL5 Hierarchical Queue (child) Parameters.
 
                This structure contains information about a single Queue
                out of the (up to 8) hierarchical channel queues.
 
*//***************************************************************************/
typedef struct t_UccAtmAal5HierarchicalQueueParam
{
    t_Handle    h_App;        /**< Handle to the upper layer handler of the ATM channel */
 
    bool        cellBased;    /**< TRUE => Hierarchical Cell Based Queue;
                                   FALSE => Hierarchical Frame Based Queue. */
    uint16_t    increment;    /**< Child priority increment */
 
    uint16_t    vpi;          /**< Virtual path identifier value */
    uint16_t    vci;          /**< Virtual channel identifier value */
    uint8_t     ptiAndClp;    /**< Payload type and cell loss priority value */
    bool        autoVcOff;    /**< Auto VC Off mode enable; Relevant only if hierarchical
                                   parent channel Auto VC On is disabled */
    uint16_t    dummyFrameLength;
                              /**< Relevant only if this is a frame based queue and autoVcOff==FALSE;
                                   Dummy frame length to use in the WFQ algorithm in case there is
                                   no data to transmit in this queue; In that case the WFQ algorithm
                                   will act as if this queue transmitted a frame of this value size;
                                   This value must be greater than 0. */

    bool        cpcsUU;       /**< CPCS UU enable indication */
 
} t_UccAtmAal5HierarchicalQueueParam;
 
/**************************************************************************//**
 @Description   AAL5 Hierarchical Channel Parameters.
 
                This structure contains information about the ( Up to 8 )
                Weighted Fair Queue Channels.
 
*//***************************************************************************/
typedef struct t_UccAtmAal5HierarchicalWfqParam
{
    uint8_t                             queuesNum;      /**< Up to 8 Queues */
    t_UccAtmAal5HierarchicalQueueParam *queueParam;     /**< Array of Queues parameters */
 
} t_UccAtmAal5HierarchicalWfqParam;


/**************************************************************************//*
 @Description   AAL5 Channel Interworking Parameters.
 
                This structure is relevant only for Aal5 IW channels.
*//***************************************************************************/
typedef struct t_UccAtmAal5IwParams
{
    e_CommMode  iwCommMode;
                /**< Indicates if this channel is activated for IW in Tx, Rx or both */
    bool        rxFastPass;
                /**< Use Fast Pass mode while filtering */
    uint8_t     rxFilterIndex;
                /**< Rx Filter (Parse Command Descriptor) index */

} t_UccAtmAal5IwParams;

/**************************************************************************//**
 @Description   Enumeration of Hierarchical parent associated queues types.
*//***************************************************************************/
typedef enum e_UccAtmAal5HierarchicalQueuesType
{
      e_FRAME_BASED_QUEUES_ONLY = 0     /**< Only Frame Based Queues */
    , e_CELL_BASED_QUEUES_ONLY  = 1     /**< Only Cell Based Queues  */
    , e_FRAME_AND_CELL_BASED_QUEUES = 2 /**< Frame Based and Cell Based Queues ( mixed ) */
} e_UccAtmAal5HierarchicalQueuesType;

/**************************************************************************//**
 @Description   AAL5 channel user-defined parameters.
 
                The structure contains some parameters that are general parameters
                and not specific AAL5 parameters. However, they are written here
                since they are not suitable for AAL5 (like f_GetDataBuf,f_TxConf...)
*//***************************************************************************/
typedef struct t_UccAtmAal5Param
{
    t_UccAtmChannelParam    uccAtmChParam;  /**< ATM channel general driver parameters */
    t_BufferPoolInfo        bufferPoolInfo; /**< Data buffer pool information - relevant
                                                 only if FBP is not used */
    uint32_t                dataMemAttributes;
                                            /**< Data memory partition information;
                                                 valid for both Tx and Rx, although if
                                                 FBP is used, this information is taken
                                                 from the FBP parameters. */
    /*****************/
    /* Rx parameters */
    /*****************/
    t_RxStoreFunction       *f_RxStore;     /**< Pointer to upper layer's routine - called to
                                                 pass a received frame to the application. */
    bool                    rxInterrupts;   /**< working with Rx interrupts */
    uint32_t                intrManagerId;  /**< the required queue id for messages from the driver to the upper layer  */

    uint32_t                timeOutMiliSec; /**< The required maximum timeout (in milliseconds)
                                                 between two received cells; That value is used
                                                 when an application provides polling on reassembly
                                                 timeout for AAL5 channels */
 
    uint8_t                 fbpNum;         /**< The required index of the available global
                                                 buffer pool (1-4), or 0 if FBP is not used */
    uint16_t                rxBdRingLength; /**< Rx BD ring pool length; If it equals 0 -
                                                 the default (or configured by UCC_ATM_ConfigBdRingLen) UCC BD ring length will be used */
 
    /*****************/
    /* Tx Parameters */
    /*****************/
    t_TxConfFunction        *f_TxConf;      /**< Pointer to upper layer's routine - called to
                                                 handle transmitted frames. */
    uint16_t                txBdRingLength; /**< Tx BD ring pool length; If it equals 0 -
                                                 the default (or configured by UCC_ATM_ConfigBdRingLen) UCC BD ring length will be used */

    e_UccAtmAal5HierarchicalQueuesType
                            hierarchicalQueuesType;
                                            /**< Relevant only for Hierarchical parent channel initialization -
                                                 Are there going to be only Frame Based queues, only Cell Based queue or both. */
 
    bool                    cpcsUU;         /**< If TRUE - CPCS-UU enabled (placed at the end of the frame) */
 
    t_UccAtmAal5IwParams    *p_IwParams;    /**< Pointer IW parameters structure;
                                                 NULL for non-IW mode */
} t_UccAtmAal5Param;


/**************************************************************************//**
 @Function      UCC_ATM_AAL5_ConfigAndInit

 @Description   Config and Initialize driver and hardware for AAL5 channel.

 @Param[in]     p_UccAtmAal5Param - User parameters for Atm channel.

 @Return        Handle to created channel if success; NULL otherwise.
*//***************************************************************************/
t_Handle UCC_ATM_AAL5_ConfigAndInit(t_UccAtmAal5Param *p_UccAtmAal5Param);

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_HierarchicalConfigAndInit

 @Description   Config and Initialize driver and hardware for AAL5 Hierarchical channel.
 
                This function takes care of hierarchical parent and all hierarchical
                children configuration and initialization.

 @Param[in]     p_Aal5Param     - User parameters for AAL5 hierarchical parent channel.
 @Param[in]     p_WfqParam      - Hierarchical parameters for hierarchical children (queues).
 @Param[out]    p_WfqChHandles  - Handle to created WFQ channels.

 @Return        Handle to created channel if success; NULL otherwise.
*//***************************************************************************/
t_Handle UCC_ATM_AAL5_HierarchicalConfigAndInit(t_UccAtmAal5Param *p_Aal5Param,
                                                t_UccAtmAal5HierarchicalWfqParam *p_WfqParam,
                                                t_Handle *p_WfqChHandles);

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_Free

 @Description   Free memory of the Atm channel handler.

 @Param[in]     h_UccAtmAal5 - aal5 channel Atm handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      aal5AtmChannel must have been previously initialized using UCC_ATM_AAL5_ConfigAndInit().
*//***************************************************************************/
t_Error UCC_ATM_AAL5_Free(t_Handle h_UccAtmAal5);

/** @} */ /* end of ucc_atm_aal5_channel_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal5_channel_ctrl_grp UCC ATM AAL5 Channel Control Unit

 @Description   Control and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_AddHierarchicalQueue

 @Description    Adds a hierarchical single queue to an already initialized
                hierarchical parent.
                The queue can be either Cell based or Frame based.

 @Param[in]     h_UccAtmAal5 - driver's handler of the hierarchical parent
 @Param[in]     p_Aal5Param - User parameters for AAL5 hierarchical channel.
 @Param[in]     p_QueueParam - Queue parameters.

 @Return        Handle to the Added Weighted Fair Queue channel.
 
 @Cautions      The hierarchical parent must have been previously initialized
                using UCC_ATM_AAL5_HierarchicalConfigAndInit().
*//***************************************************************************/
t_Handle UCC_ATM_AAL5_AddHierarchicalQueue( t_Handle h_UccAtmAal5,
                                            t_UccAtmAal5Param *p_Aal5Param,
                                            t_UccAtmAal5HierarchicalQueueParam *p_QueueParam );

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
 
                Invokes user's tx confirmation call back routine for previously
                transmitted data on the specific AAL5 channel.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.
 
 @Param[in]     h_UccAtmAal5 - driver's handler of the ATM channel
 
 @Return        None.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                the transmit routines) simultaneously at task level and at
                interrupt level may lead to unpredictable results.
*//***************************************************************************/
void UCC_ATM_AAL5_TxConf(t_Handle h_UccAtmAal5);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_AAL5_DumpRegs

 @Description   Dumps ATM Port registers and parameters RAM.

 @Param[in]     h_UccAtmAal5 - Handle to the AAL5 channel object.

 @Return        None.

 @Cautions      AAL5 Channel must have been previously initialized using UCC_ATM_AAL5_ConfigAndInit().

*//***************************************************************************/
void UCC_ATM_AAL5_DumpRegs(t_Handle h_UccAtmAal5);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_aal5_channel_ctrl_grp group */

/**************************************************************************//**
 @Group         ucc_atm_aal5_channel_runtime_grp UCC ATM AAL5 Channel Runtime Unit

 @Description   Runtime Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    UCC ATM AAL5 Rx status bits.
 
                The AAL5 status and errors returned via f_RxStore callback.
 @{
*//***************************************************************************/
#define UCC_ATM_AAL5_RX_STATUS_MAXBD3   0x0100  /**< IW CPU frame was truncated according to allowed BDs limit */
#define UCC_ATM_AAL5_RX_STATUS_PNC      0x0080  /**< UPC Non conformance; The buffer contains a cell which
                                                     was not conformed by one of the buckets. */
#define UCC_ATM_AAL5_RX_STATUS_CLP      0x0020  /**< Cell Loss priority; At least one cell was received with CLP = 1. */
#define UCC_ATM_AAL5_RX_STATUS_CNG      0x0010  /**< Congestion Indication; At least one cell was received with PTI middle bit is set. */
#define UCC_ATM_AAL5_RX_STATUS_ABRT     0x0008  /**< Abort message indication; The message was received with length field zero. */
#define UCC_ATM_AAL5_RX_STATUS_CPCS_UU  0x0004  /**< CPCS-UU field is provided in the low 16 bits of the flags parameter. */
#define UCC_ATM_AAL5_RX_STATUS_LNE      0x0002  /**< Length error; If the legnth is greater than 47 or less than zero. */
#define UCC_ATM_AAL5_RX_STATUS_CRE      0x0001  /**< CRC Error. */
/* @} */

/**************************************************************************//**
 @Collection    UCC ATM AAL5 Rx flags.
 
                The AAL5 flags returned via f_RxStore callback.
 
                Note that the low 16 bits of the flags parameter are reserved
                for the CPCS-UU value, in case UCC_ATM_AAL5_RX_STATUS_CPCS_UU
                is set in the status parameter.
 @{
*//***************************************************************************/
#define UCC_ATM_AAL5_RX_FLAG_IW_QUEUE   0x80000000  /**< Frame was received through one of the IW CPU queues;
                                                         The reported queue ID represents the IW CPU queues index. */
/* @} */

/**************************************************************************//**
 @Collection    AAL5 Transmit indication.

 @{
*//*****************************************************************/

#define UCC_ATM_AAL5_TX_CLP		0x00200000      /**< UCC ATM AAL5 Tx CLP (Cell Loss Priority) indication */
#define UCC_ATM_AAL5_TX_CNG		0x00100000      /**< UCC ATM AAL5 Tx CNG (Congestion) indication */

/* @} */

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_Tx

 @Description   Transmits a data buffer(which is actually a data buffer of a frame) on an AAL5 channel.

                Accepts data as a pointer to a single data buffer (array of
                bytes). If CPCS-UU mode is enabled for this channel, CPCS-UU Data (2 bytes)
                should be set in the end of the frame ('length' bytes from the start).

                This routine eventually causes the user's transmit confirmation
                call-back routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_ATM_AAL5_TxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.
 
 @Param[in]     h_UccAtmAal5   - ATM channel handler.
 @Param[in]     p_Data         - the address of the data for sending.
 @Param[in]     h_BufContext   - User's context for the given data buffer.
 @Param[in]     length         - the number of octets for transmitting.
 @Param[in]     lastBuffer 	   - is last buffer in frame.
@if UNDER_CONSTRUCTION_UDC
 @Param[in]     txExtraHeader  - the frame extra header ( if there is ).
@endif
 @Param[in]     flags          - flags to be added (ored) to firstTxBdStatus;
                                      i.e. clp and cng indications using the macros :
                                      UCC_ATM_AAL5_TX_CLP and UCC_ATM_AAL5_TX_CNG.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                UCC_ATM_AAL5_TxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_Tx(t_Handle    h_UccAtmAal5,
                        uint8_t     *p_Data,
                        t_Handle    h_BufContext,
                        uint16_t    length,
                        bool        lastBuffer
                      , uint32_t    flags);

/**************************************************************************//**
 @Function      UCC_ATM_AAL5_Rx

 @Description   Handles data buffers received in the specific AAL5 channel.

                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific channel.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.

                When intrManagerId parameter is 0, which means that no message
                mechanism is used, and receive interrupts are enabled,
                the routine is called from the driver's interrupt routine
                upon Rx frame interrupt.
 
                When intrManagerId parameter grater then 0 (which means that there
                is a message queue) and rx interrupts are enabled, the XX_Call()
                routine is called from the driver's interrupt routine upon
                Rx frame interrupt. In that case, the channel's receive
                interrupts are cleared until the application will call the
                UCC_ATM_AAL5_Rx() routine to collect the received frames.
                The XX_Call() routine should just inform the application about
                received data. The application is responsible to call the
                UCC_ATM_AAL5_Rx() to collect the received data.
 
                The application should call the UCC_ATM_AAL5_Rx() routine
                when polling mechanism is used or when receive interrupts
                are disabled.
 
                At the end of the routine it enables the Rx interrupts ( if
                intrManagerId not 0) and saves the BD and a frame's data buffer in
                the channel's data structure in order to use them in the next
                call.
 
 @Param[in]     h_UccAtmAal5 - driver's handler of the ATM channel
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_Rx(t_Handle h_UccAtmAal5);


/** @} */ /* end of ucc_atm_aal5_channel_runtime_grp group */
/** @} */ /* end of ucc_atm_aal5_channel_grp group */
/** @} */ /* end of group ucc_atm_grp */


/**************************************************************************//*
 @Description   AAL5 IW details structure (interface with IW driver).
*//***************************************************************************/
typedef struct
{
    uint16_t    channelCode;    /** Tx channel code */
    e_CommMode  iwCommMode;     /** Indicates whether the AAL5 channel is used
                                    for IW in Tx mode, Rx mode, or both */
} t_UccAtmAal5IwDetails;

/**************************************************************************//*
 @Function      UCC_ATM_AAL5_GetIwDetails

 @Description   Gets the IW details of this channel (interface with IW driver).

 @Param[in]     h_UccAtmAal5    - AAL5 channel handle.
 @Param[in]     p_IwDetails     - Pointer to the AAL5 IW details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_GetIwDetails(t_Handle              h_UccAtmAal5,
                                  t_UccAtmAal5IwDetails *p_IwDetails);

/**************************************************************************//*
 @Function      UCC_ATM_AAL5_AddToMulticastGroup

 @Description   Adds a channel to a multicast group.

 @Param[in]     h_UccAtmAal5    - AAL5 channel handle.
 @Param[in]     multicastGroup  - Multicast group ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_AddToMulticastGroup(t_Handle h_UccAtmAal5, uint16_t multicastGroup);


/**************************************************************************//*
 @Function      UCC_ATM_AAL5_RemoveFromMulticastGroup

 @Description   Removes a channel from a multicast group.

 @Param[in]     h_UccAtmAal5 - AAL5 channel handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_RemoveFromMulticastGroup(t_Handle h_UccAtmAal5);

/**************************************************************************//*
 @Function      UCC_ATM_AAL5_ModifyRxFilterIndex

 @Description   Modifies the index of the filter which is used for filteing
                incoming packets.

 @Param[in]     h_UccAtmAal5    - AAL5 channel handle.
 @Param[in]     rxFilterIndex   - Rx Filter (Parse Command Descriptor) index.
 @Param[in]     fastPass        - Use Fast Pass mode while filtering.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_ModifyRxFilterIndex(t_Handle   h_UccAtmAal5,
                                         uint8_t    rxFilterIndex,
                                         bool       fastPass);

/**************************************************************************//*
 @Function      UCC_ATM_AAL5_GetIwIndex

 @Description   Get the IW index for Interworking Action Descriptor (IWAD).

 @Param[in]     h_UccAtmAal5    - AAL5 channel handle.
 @Param[out]    p_IwIndex       - Will get the IW TQD index (Tx channel code).
 @Param[out]    p_IsExtTqd      - Will be TRUE if channel is external.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_GetIwIndex(t_Handle h_UccAtmAal5, uint16_t *p_IwIndex, bool *p_IsExtTqd);

/**************************************************************************//*
 @Function      UCC_ATM_AAL5_SetExponentialWeightFactor

 @Description   Set the Exponential Weight Factor for Interworking for WRED functionality.
 
                WRED feature is supported only for Unicast channels.
                In case of Multi Cast channels, expWeightFactor value will be saved and used in case
                the channel is removed from the Multicast group.
                Note: If this function is not called, the exponential weight factor is initialized to 0.
 
 @Param[in]     h_UccAtmAal5    - AAL5 channel handle.
 @Param[out]    expWeightFactor - Exponential Weight Factor; Should be calculated according to IW WRED specification.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_ATM_AAL5_SetExponentialWeightFactor(t_Handle h_UccAtmAal5, uint8_t expWeightFactor);


#endif /* __UCC_ATM_AAL5_EXT_H */
