/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_ch_ext.h

 @Description   UCC Atm Channel external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_ATM_CH_EXT_H
#define __UCC_ATM_CH_EXT_H

#include "std_ext.h"
#include "ucc_atm_port_ext.h"
#include "ucc_atm_ext.h"

/* Cell header definitions */
#define CELL_HEADER_VPI_SHIFT 20
#define CELL_HEADER_VCI_SHIFT 4
#define CELL_HEADER(vpi,vci,ptiAndClp) \
    ( (uint32_t)(((vpi << CELL_HEADER_VPI_SHIFT) | (vci << CELL_HEADER_VCI_SHIFT) | (ptiAndClp))) )

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_channel_grp UCC ATM Channel

 @Description   Ucc Atm Channel

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_channel_init_grp UCC ATM Channel Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/
#include "ucc_atm_ext.h"

/**************************************************************************//**
 @Description   Traffic Shaping types.
 *//***************************************************************************/
typedef enum e_UccAtmChTrafficShaping
{
      e_UCC_ATM_CH_UBR        /**< unspecified bit rate */
    , e_UCC_ATM_CH_UBR_PLUS   /**< expanded unspecified bit rate */
    , e_UCC_ATM_CH_CBR        /**< constant bit rate */
    , e_UCC_ATM_CH_VBR        /**< variable bit rate */
    , e_UCC_ATM_CH_GBR        /**< guaranteed bit rate */
    , e_UCC_ATM_CH_TRAFFIC_TYPES_NUM
 
} e_UccAtmChTrafficShaping;

/**************************************************************************//**
 @Description   Vbr parameters to initialize Vbr traffic type.
 *//***************************************************************************/
typedef struct t_UccAtmVbrParam
{
    uint32_t    scrBitPerSec;         /**< Sustain cell rate (in bit per second) */
    uint16_t    mbsCells;             /**< Maximum burst size (in cells) */
    uint32_t    outOfBufRateBitPerSec;/**< Out of buffer rate (bit per second)*/
    bool        vbrType2;             /**< True if works with VBR type 2 */
} t_UccAtmVbrParam;

/**************************************************************************//**
 @Description   Ubr+ parameters to initialize Ubr+ traffic type.
 *//***************************************************************************/
typedef struct t_UccAtmUbrPlusParam
{
    uint32_t  mcrBitPerSec;     	/**< Minimum cell rate (in bits per second) */
    uint16_t  mdaSlots_mdaMicSec;	/**< Maximum delay allowed; Note : In APC it should be in slots,
    									and In GCRA - in Micro Seconds */
} t_UccAtmUbrPlusParam;

/**************************************************************************//**
 @Description   Gbr parameters to initialize Ubr part of Gbr traffic type.
 *//***************************************************************************/
typedef struct t_UccAtmGbrUbrParam
{
    uint32_t  ubrPcrBitPerSec;     /**< Pcr cell rate of Gbr Ubr (in bits per second) */
 
} t_UccAtmGbrUbrParam;

/**************************************************************************//**
 @Description   Cell Filtering.
 *//***************************************************************************/
typedef enum e_UccAtmChOamCellFilterType
{
      e_NO_OAM_CELL_FILTER                      = 0x00000000
    , e_SEGMENT_OAM_CELL_FILTER                 = 0x00200000 /**< send cells with PTI = 100 to the raw cell queue */
    , e_END_TO_END_OAM_CELL_FILTER              = 0x00100000 /**< send cells with PTI = 101 to the raw cell queue */
    , e_SEGMENT_AND_END_TO_END_OAM_CELL_FILTER  = 0x00300000 /**< send cells with PTI = 100 or PTI = 101 to the raw cell queue */
} e_UccAtmChOamCellFilterType;

/**************************************************************************//**
 @Description   Atm Channel parameters initialization structure.
 *//***************************************************************************/
typedef struct t_UccAtmChannelParam
{
    t_Handle                    h_App;          /**< Handle to the upper layer handler of the ATM channel */
    t_Handle                    h_Port;         /**< Handle to the driver's handler of the ATM port */
    t_Handle        	        h_RxUcc;        /**< Handle to the receive UCC */
    e_CommMode                  commMode;	    /**< Indicates if the channel is initialized to half duplex operation
                                                     like Receive or Transmit or full duplex operation (Rx and Tx) */
    bool                        isInternal;     /**< True - the channel tables resides in the internal memory;
                                                     False - the channel tables resides in the external memory */
    /*****************/
    /* Rx parameters */
    /*****************/
    t_Handle                    h_Policer;      /**< The policer handle which is required for this channel;
                                                     If NULL - the channel doesn't work with policer tables */
    e_UccAtmIntQ                rxIntQueue;     /**< Receiver Interrupt queue number (should be between 1-4)
                                                     Note: The required interrupt queue should match the
                                                     interrupt queue type initialization at the UCC level */
    e_UccAtmChOamCellFilterType oamCellFilterType;/**< Which type of cell to send to the raw cell queue */
 
    /*****************/
    /* Tx Parameters */
    /*****************/
    bool                        txVconMode;     /**< Enable auto vc ON for switched channels */
    e_UccAtmChTrafficShaping    trafficType;    /**< ATM traffic type */
    union uccAtmTrafficParam
    {
        t_UccAtmVbrParam        vbr;            /**< VBR Traffic Type parameters */
        t_UccAtmUbrPlusParam    ubrPlus;        /**< UBR Plus Traffic Type parameters */
        t_UccAtmGbrUbrParam     gbrUbr;         /**< GBR Traffic Type (UBR Part) parameters */
    } uccAtmTrafficParam;                       /**< VBR, UBR+, GBR traffic specific parameters */
    uint32_t                    pcrBitPerSec;   /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    int                         schedLevel;     /**< the required Scheduler priority level table that this channel should be inserted */
    bool                        autoVcOff;      /**< enable/disable the autoVcOff option; 0 - Do not remove VC from the scheduling table and continue to schedule it to transmit; 1 - otherwise  */
    uint16_t                    vpi;            /**< virtual path identifier value (include also the GFC value in case of UNI mode )*/
    uint16_t                    vci;            /**< virtual channel identifier value */
    uint8_t                     ptiAndClp;      /**< payload type and cell loss priority value */
    e_UccAtmIntQ                txIntQueue;     /**< Transmitter Interrupt queue number (should be between 1-4) */
 
} t_UccAtmChannelParam;

/** @} */ /* end of channel_atm_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_channel_control_grp UCC ATM Channel Control Unit

 @Description   Control Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_CH_Enable

 @Description   Enables receive or/and transmit communication on a channel.

                - In case of Transmit - issue the ATM Transmit command.
                - In case of Receive  - update Address resolution tables and actually
                finishes the last step in inserting the channel code to the relevant
                address mapping tables.
                Note: In case policer mode needed for this channel, the UCC_ATM_CH_PolicerEnable()
                function should be called.

 @Param[in]     h_UccAtmCh - AtmChannel handler to be enabled.
 @Param[in]     mode - channel communication mode - receive or transmit or receive and transmit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().
 
*//***************************************************************************/
t_Error UCC_ATM_CH_Enable(t_Handle h_UccAtmCh, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_ATM_CH_Disable

 @Description   Disables receive or/and communication on a channel.

                - In case of Transmit - Stops channel's transmission by
                removing the channel from the scheduling table.
                - In case of Receive  - Stops channel's data reception.
                It clears the channel code from the address resolution tables.

 @Param[in]     h_UccAtmCh - AtmChannel handler to be initialized.
 @Param[in]     mode - channel communication mode - receive or transmit or receive and transmit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_Disable(t_Handle h_UccAtmCh,e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_ATM_CH_EnableAsync

 @Description   Enables receive or/and transmit communication on a channel, Asynchronically.

                - In case of Transmit - issue the ATM Transmit command.
                - In case of Receive  - update Address resolution tables and actually
                finishes the last step in inserting the channel code to the relevant
                address mapping tables.
                Note: In case policer mode needed for this channel, the UCC_ATM_CH_PolicerEnable()
                function should be called.
                The routine is asynchronic since it doesn't wait until the
                enable operation will be finished.

 @Param[in]     h_UccAtmCh - AtmChannel handler to be enabled.
 @Param[in]     mode - channel communication mode - receive or transmit or receive and transmit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().
 
*//***************************************************************************/
t_Error UCC_ATM_CH_EnableAsync(t_Handle h_UccAtmCh, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_ATM_CH_DisableAsync

 @Description   Disables receive or/and communication on a channel, Asynchronically.

                - In case of Transmit - Stops channel's transmission by
                removing the channel from the scheduling table.
                - In case of Receive  - Stops channel's data reception.
                It clears the channel code from the address resolution tables.
                The routine is asynchronic since it doesn't wait until the
                disable operation will be finished.
 

 @Param[in]     h_UccAtmCh - AtmChannel handler to be initialized.
 @Param[in]     mode - channel communication mode - receive or transmit or receive and transmit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_DisableAsync(t_Handle h_UccAtmCh,e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_ATM_CH_PolicerEnable

 @Description   Enables Policer use for this channel.

 @Param[in]     h_UccAtmCh - AtmChannel handler.
 @Param[in]     h_PolicerTable - Policer table handler.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_PolicerEnable( t_Handle h_UccAtmCh, t_Handle h_PolicerTable );

/**************************************************************************//**
 @Function      UCC_ATM_CH_PolicerDisable

 @Description   Disables Policer use for this channel.

 @Param[in]     h_UccAtmCh - AtmChannel handler.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_PolicerDisable( t_Handle h_UccAtmCh );

/**************************************************************************//**
 @Function      UCC_ATM_CH_EnablePerformanceMonitoring

 @Description   Enables Performance Monitoring for this channel.

                This function can be called before and after enabling the channel.
 
 @Param[in]     h_UccAtmCh - AtmChannel handler to be updated.
 @Param[in]     pmCommMode - PM Communication Mode (Rx, Tx or Both) to enable.
 @Param[in]     pmTableIndex - PM Table Index : 0 - <UCC total PMTs-1>.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().
                The Performance Monitoring table must have been previously initialized
                using UCC_ATM_SetPerformanceMonitoringTableParam().

*//***************************************************************************/
t_Error UCC_ATM_CH_EnablePerformanceMonitoring( t_Handle h_UccAtmCh, e_CommMode pmCommMode, uint8_t pmTableIndex );

/**************************************************************************//**
 @Function      UCC_ATM_CH_ModifyPeakCellRate

 @Description   Changes the Channel's Peak cell rate.

                This function can be called before and after enabling the channel.
 
 @Param[in]     h_UccAtmCh - AtmChannel handler to be updated.
 @Param[in]     pcrBitPerSec - Peak cell rate (given in bit per second)

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_ModifyPeakCellRate( t_Handle h_UccAtmCh, uint32_t pcrBitPerSec );

/**************************************************************************//**
 @Function      UCC_ATM_CH_DisablePerformanceMonitoring

 @Description   Disables Performance Monitoring for this channel.

                This function can be called before and after enabling the channel.
                Note : There is no need to call this function before disabling a
                channel. If a channel is disabled, it's PM functionality is disabled too.
 
 @Param[in]     h_UccAtmCh - AtmChannel handler to be updated.
 @Param[in]     pmCommMode - PM Communication Mode (Rx, Tx or Both) to enable.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
t_Error UCC_ATM_CH_DisablePerformanceMonitoring( t_Handle h_UccAtmCh, e_CommMode pmCommMode);

/**************************************************************************//**
 @Function      UCC_ATM_CH_IsTxStopped

 @Description   Check if the channel is already stopped to transmit.

 @Param[in]     h_UccAtmCh - h_UccAtmCh handler to be initialized.

 @Return        Indication is the channel is already stopped.
*//***************************************************************************/
bool UCC_ATM_CH_IsTxStopped(t_Handle h_UccAtmCh);

/**************************************************************************//**
 @Function      UCC_ATM_CH_IsRxStopped

 @Description   Check if the channel is already stopped to receive.
 
 @Param[in]     h_UccAtmCh - h_UccAtmCh handler to be initialized.

 @Return        Indication is the channel is already stopped.
*//***************************************************************************/
bool UCC_ATM_CH_IsRxStopped(t_Handle h_UccAtmCh);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_CH_DumpRegs

 @Description   Dumps ATM Channel registers and parameters RAM.

 @Param[in]     h_UccAtmCh - Handle to the channel object.
 @Param[in]     hierarchicalCh - Indicates if it's an AAL5 Hierarchical Channel.
 @Param[in]     hierarchicalQueue - Indicates if it's an AAL5 Hierarchical Queue.
 
 @Return        None.

 @Cautions      Channel must have been previously initialized using UCC_ATM_CH_ConfigAndInit().

*//***************************************************************************/
void UCC_ATM_CH_DumpRegs(t_Handle h_UccAtmCh, bool hierarchicalCh, bool hierarchicalQueue);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_channel_control_grp group */
/** @} */ /* end of channel_atm_grp group */
/** @} */ /* end of group ucc_atm_grp */

/**************************************************************************//*
 @Function      UCC_ATM_CH_GetRxChannelCode

 @Description   Get the Rx channel code of a channel (interface with IMA driver).

 @Param[in]     h_UccAtmCh - Handle to the channel.

 @Return        The Rx channel code of the given channel.
*//***************************************************************************/
uint16_t UCC_ATM_CH_GetRxChannelCode(t_Handle h_UccAtmCh);

#endif /* __UCC_ATM_CH_EXT_H */
