/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_atm_plcr_ext.h

 @Description   UCC Atm Policer external structures and definitions.
*//***************************************************************************/
#ifndef __UCC_ATM_PLCR_EXT_H
#define __UCC_ATM_PLCR_EXT_H

#include "std_ext.h"

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_policer_grp UCC ATM Policer

 @Description   Ucc Atm Policer

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_atm_policer_init_grp UCC ATM Policer Initialization Unit

 @Description   Initialization Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Leaky Bucket number.
 *//***************************************************************************/
typedef enum e_UccAtmPlcrLbNum
{
      e_UCC_ATM_PLCR_LB_1  = 0    /**< Leaky bucket number 1 */
    , e_UCC_ATM_PLCR_LB_2  = 1    /**< Leaky bucket number 2 */
    , MAX_UCC_ATM_PLCR_LB  = 2    /**< max leaky buckets */
}e_UccAtmPlcrLbNum;

/**************************************************************************//**
 @Description   Discard Modes.
 *//***************************************************************************/
typedef enum e_UccAtmPlcrDiscardMode
{
      e_UCC_ATM_PLCR_TAG  =  0x0000 /**< policer discard mode - tag */
    , e_UCC_ATM_PLCR_DROP =  0x0008 /**< policer discard mode - drop */
} e_UccAtmPlcrDiscardMode;

/**************************************************************************//**
 @Description   Leaky Bucket Filter.
 *//***************************************************************************/
typedef enum e_UccAtmPlcrLbFilter
{
      e_UCC_ATM_PLCR_DISABLE    = 0x0000    /**< policer leaky bucket filter - disable */
    , e_UCC_ATM_PLCR_GCRA_CLP0  = 0x0001    /**< policer leaky bucket filter - only CLP0 */
    , e_UCC_ATM_PLCR_GCRA_CLP1  = 0x0002    /**< policer leaky bucket filter - only CLP1 */
    , e_UCC_ATM_PLCR_GCRA_CLP01 = 0x0003    /**< policer leaky bucket filter - CLP0+CLP1 */
    , e_UCC_ATM_PLCR_F_GCRA     = 0x0004    /**< policer leaky bucket filter - frame GCRA */
} e_UccAtmPlcrLbFilter;

/**************************************************************************//**
 @Description   Policer Operation Modes.
 *//***************************************************************************/
typedef enum e_UccAtmPlcrOperationMode
{
      e_UCC_ATM_PLCR_MODE_DISABLE         = 0
    , e_UCC_ATM_PLCR_MODE_CELL_BASE       = 0x01    /**< policer operation mode - cell base */
    , e_UCC_ATM_PLCR_MODE_FRAME_AWARENESS = 0x02    /**< policer operation mode - frame awareness */
    , e_UCC_ATM_PLCR_MODE_GFR             = 0x03    /**< policer operation mode - GFR */
} e_UccAtmPlcrOperationMode;

/**************************************************************************//**
 @Description   Leaky Bucket Parameters.
 *//***************************************************************************/
typedef struct
{
    uint32_t                 cdvtNanoSec;       /**< Cell Delay Variation Time (in nano seconds) - important for calculation the Bucket Limit variable */
    uint32_t                 bitRatePerSec;     /**< Bit Rate (in bit per second)  - important for calculation the Bucket increment variable */
    e_UccAtmPlcrDiscardMode  discardMode;       /**< tag or drop discard operations */
    e_UccAtmPlcrLbFilter     lbFilter;          /**< Leaky Bucket Filter */
} t_UccAtmPlcrLbParam;

/**************************************************************************//**
 @Description   Policer Table parameters to initialize Policer Table.
 *//***************************************************************************/
typedef struct
{
    t_Handle                   h_Ucc;                     /**< ucc handle that the specific policer table belongs to
                                                          NOTE: The parameter is relevant only in case the work
                                                                is with regular policer tables numbers management
                                                                and not with Mini-Cam policer tables management */
    bool                       isInternalPolicerTable;    /**< indicates if the user asked for policer table which resides on the MURAM
                                                          NOTE: The parameter is relevant only in case the work
                                                                is with regular policer tables numbers management
                                                                and not with Mini-Cam policer tables management */
    e_UccAtmPlcrOperationMode  operationMode;             /**< policer operation mode - cell base,frame awareness or GFR */
    t_UccAtmPlcrLbParam        leakyBucketParam[MAX_UCC_ATM_PLCR_LB];  /**< two leaky bucket are exist for every policer table */
    uint32_t                   mfsCells;                  /**< Maximum Frame Size in cell unit
                                                               Note:The field is relevant only in case of GFR mode */
} t_UccAtmPlcrParam;

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_Config

 @Description   Configures a policer table based on user parameters.

 @Param         p_UccAtmPlcrParam - (In) user parameters for a policer table initialization.

 @Return        Handle to the created policing table if success; NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle UCC_ATM_PLCR_Config(t_UccAtmPlcrParam *p_UccAtmPlcrParam);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigMiniCam

 @Description   Configures a policer table based on user parameters.

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         h_MiniCamPort - (In) handle to the Mini-Cam port

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().

*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigMiniCam(t_Handle h_UccAtmPlcr, t_Handle h_MiniCamPort);

/**************************************************************************//**
 @Group         ucc_atm_policer_config_grp UCC ATM Policer Advanced Config Routines

 @Description   Advanced Config Routines

 @{
*//***************************************************************************/

/*
 * These declarations generated automatically by FunGen 1.1 $
 * Time:             Nov  8 2004 06:42:11
 * Source file:      GENUccAtmPlcrFuncs.c
 */

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigUpThArrivalTime

 @Description   Configures the update threoretical arrival time from its default configuration.
 
                Default value: [FALSE].

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         upThArrivalTime    - (In) the required update theoretical arrival time.

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().

 @Cautions      When working with VP policing, it's not allowed
                to ask for not updating the theoretical arrival time in both buckets.
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigUpThArrivalTime (t_Handle h_UccAtmPlcr, bool upThArrivalTime);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigClpDrop

 @Description   Configures the CLP drop from its default configuration.
 
                Default value: [FALSE].

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         clpDrop    - (In) value to change clpDrop to. enable/disable the option to drop the remain cells of frame if there is a CLP change within the frame [FALSE]

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigClpDrop (t_Handle h_UccAtmPlcr, bool clpDrop);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigEnableNcclpIntr

 @Description   Configures the enable/disable the NCCLP - Non Conformed CLP0/1 counters overflow interrupt from its default configuration.
 
                Default value: [TRUE].

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         enableNcclpIntr    - (In) TRUE to enable the NCCLP - Non Conformed CLP0/1 counters overflow interrupt
                                          FALSE to disable the NCCLP - Non Conformed CLP0/1 counters overflow interrupt.

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigEnableNcclpIntr (t_Handle h_UccAtmPlcr, bool enableNcclpIntr);


/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigEnableClpIntr

 @Description   Configures the enable/disable the CLP - CLP0/1 counters overflow interrupt from its default configuration.
 
                Default value: [TRUE].

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         enableClpIntr    - (In) TRUE to enable the CLP - CLP0/1 counters overflow interrupt
                                        FALSE to disable the CLP - CLP0/1 counters overflow interrupt.

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigEnableClpIntr (t_Handle h_UccAtmPlcr, bool enableClpIntr);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ConfigEnableCdcFcIntr

 @Description   Configures enable the CDC - cell drop counter overflow interrupt parameter from its default configuration.
 
                Default value: [TRUE].

 @Param         h_UccAtmPlcr - (In) handle to Ucc Atm Plcr
 @Param         enableCdcFcIntr    - (In) TRUE to enable the CDC - cell drop counter overflow interrupt and the FC - frame counters (FDC & FC) overflow interrupt
                                          FALSE to disable the CDC - cell drop counter overflow interrupt.

 @Cautions      Does not affect hardware, just internal driver structures.
                Change is applied only upon calling UCC_ATM_PLCR_Init(); May not be used after UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ConfigEnableCdcFcIntr (t_Handle h_UccAtmPlcr, bool enableCdcFcIntr);


/*
 * End of automatically generated declarations
 */
/** @} */ /* end of ucc_atm_policer_config_grp group */

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_Init

 @Description   Initializes driver and hardware for UCC ATM Policer handler.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously created using UCC_ATM_PLCR_Config().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_Init(t_Handle h_UccAtmPlcr);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_Free

 @Description   Frees driver and hardware for UCC ATM Policer handler.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously created using UCC_ATM_PLCR_Config().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_Free(t_Handle h_UccAtmPlcr);

/** @} */ /* end of ucc_atm_policer_init_grp group */

/**************************************************************************//**
 @Group         ucc_atm_policer_control_grp UCC ATM Policer Control Unit

 @Description   Control Structures and Routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ModifyBitRate

 @Description   Modify the increment fraction value in the policer table for a specific leaky bucket.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         uccAtmPlcrLbNum - (In) whether it's the first leaky bucket or the second.
 @Param         bitRate - (In) the bit rate value has an influence of the Bucket Increment variable in the policer table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ModifyBitRate(t_Handle h_UccAtmPlcr, e_UccAtmPlcrLbNum uccAtmPlcrLbNum, uint32_t bitRate);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ModifyDiscardMode

 @Description   Modify the discard mode in the policer table for a specific leaky bucket.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         uccAtmPlcrLbNum - (In) whether it's the first leaky bucket or the second.
 @Param         uccAtmPlcrDiscardMode - (In) the new required discard mode to be modifiedd in the policer table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ModifyDiscardMode(t_Handle h_UccAtmPlcr, e_UccAtmPlcrLbNum uccAtmPlcrLbNum, e_UccAtmPlcrDiscardMode uccAtmPlcrDiscardMode);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ModifyLbFilter

 @Description   Modify the leaky bucket filter in the policer table for a specific leaky bucket.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         uccAtmPlcrLbNum - (In) whether it's the first leaky bucket or the second.
 @Param         uccAtmPlcrLbFilter - (In) the new required leaky bucket filter to be modified in the policer table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ModifyLbFilter(t_Handle h_UccAtmPlcr, e_UccAtmPlcrLbNum uccAtmPlcrLbNum, e_UccAtmPlcrLbFilter uccAtmPlcrLbFilter);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_ModifyCdvt

 @Description   Modify the cell delay variation time in the policer table for a specific leaky bucket.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         uccAtmPlcrLbNum - (In) whether it's the first leaky bucket or the second.
 @Param         cdvt - (In) the Cdvt value has an influence of the Bucket Limit variable in the policer table.


 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_ModifyCdvt(t_Handle h_UccAtmPlcr, e_UccAtmPlcrLbNum uccAtmPlcrLbNum, uint32_t cdvt);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetClp0Counter

 @Description   Get the CLP0(Cell loss priority) counter from the policer table.

                The Clp0 counter counts the incoming cells.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_Clp0Count - (Out) CLP0 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetClp0Counter(t_Handle h_UccAtmPlcr, uint16_t *p_Clp0Count);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetClp1Counter

 @Description   Get the CLP1(Cell loss priority) counter from the policer table.

                The Clp1 counter counts the incoming cells.
 
 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_Clp1Count - (Out) CLP1 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetClp1Counter(t_Handle h_UccAtmPlcr, uint16_t *p_Clp1Count);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetFrameCounter

 @Description   Get the frame counter from the policer table.
 
                The frame counter counts the incoming frames which were not dropped.
                This counter is applied only in frame mode.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_FrameCount - (Out) CLP 0 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetFrameCounter(t_Handle h_UccAtmPlcr, uint16_t *p_FrameCount);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetNonConformedClp0Counter

 @Description   Get the non conformed CLP0 counter from the policer table.
 
                The non conformed counter counts the cells which were not
                conformed by the policer leaky buckets(Fail in one of the bucket
                cause counting the cell as non conforming.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_Ncclp0Count - (Out) CLP0 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetNonConformedClp0Counter(t_Handle h_UccAtmPlcr, uint16_t *p_Ncclp0Count);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetNonConformedClp1Counter

 @Description   Get the non conformed CLP1 counter from the policer table.
 
                The non conformed counter counts the cells which were not
                conformed by the policer leaky buckets(Fail in one of the bucket
                cause counting the cell as non conforming.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_Ncclp1Count - (Out) CLP1 counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetNonConformedClp1Counter(t_Handle h_UccAtmPlcr, uint16_t *p_Ncclp1Count);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetFrameDropCounter

 @Description   Get the frame drop counter from the policer table.
 
                The frame drop counter is updated when a frame or part of it
                is dropped by the policer. This counter is applied only
                in frame mode.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_FrameDropCount - (Out) frame drop counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetFrameDropCounter(t_Handle h_UccAtmPlcr, uint16_t *p_FrameDropCount);

/**************************************************************************//**
 @Function      UCC_ATM_PLCR_GetCellDropCounter

 @Description   Get the cell drop counter from the policer table.
 
                The cell drop counter is updated when a cell is dropped
                by the policer. This counter is applied only in cell base mode.

 @Param         h_UccAtmPlcr - (In) Ucc Atm Policer handler to be initialized.
 @Param         p_CellDropCount - (Out) cell drop counter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
t_Error UCC_ATM_PLCR_GetCellDropCounter(t_Handle h_UccAtmPlcr, uint16_t *p_CellDropCount);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UCC_ATM_PLCR_DumpRegs

 @Description   Dumps ATM Port registers and parameters RAM.

 @Param         h_UccAtmPlcr - (In) Handle to the Policer Table object.

 @Return        None.

 @Cautions      uccAtmPlcr must have been previously inited using UCC_ATM_PLCR_Init().
*//***************************************************************************/
void UCC_ATM_PLCR_DumpRegs(t_Handle h_UccAtmPlcr);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of ucc_atm_policer_control_grp group */
/** @} */ /* end of ucc_atm_policer_grp group */
/** @} */ /* end of group ucc_atm_grp */

#endif /* __UCC_ATM_PLCR_EXT_H */
