/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_geth_ext.h

 @Description   UCC Gigabit Ethernet external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_GETH_EXT
#define __UCC_GETH_EXT


#include "std_ext.h"
#include "ucc_fast_ext.h"
#include "qe_ext.h"
#include "part_ext.h"
#include "enet_ext.h"
#include "qe_filtering_ext.h"
#ifdef UCC_GETH_VP_MODE_SUPPORT
#include "vp_ext.h"
#endif  /* UCC_GETH_VP_MODE_SUPPORT */


/**************************************************************************//**
 @Group         geth_grp UCC Gigabit Ethernet Application Programming Interface

 @Description   UCC GETH functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         geth_init_grp UCC GETH Initialization Unit

 @Description   UCC GETH Initialization Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Possible VLAN operations on incoming tagged frames.
 
                These VLAN operations are programmed in one of two methods:
                - In address recognition mode, using UCC_GETH_ConfigVlanOperationTagged().
                - In extended filtering mode, through the Termination Action
                  Descriptor parameters.
*//***************************************************************************/
typedef enum e_UccGethVlanOperationTagged
{
    e_UCC_GETH_VLAN_OPERATION_TAGGED_NOP                                    = 0x0 /**< Tagged - no operation */
  , e_UCC_GETH_VLAN_OPERATION_TAGGED_REPLACE_VID_PORTION_OF_Q_TAG           = 0x1 /**< Tagged - replace VID portion of Q-tag */
  , e_UCC_GETH_VLAN_OPERATION_TAGGED_IF_VID0_REPLACE_VID_WITH_DEFAULT_VALUE = 0x2 /**< Tagged - if VID=0, replace VID with default value */
  , e_UCC_GETH_VLAN_OPERATION_TAGGED_EXTRACT_Q_TAG_FROM_FRAME               = 0x3 /**< Tagged - extract Q-tag from frame */
  , e_UCC_GETH_VLAN_OPERATION_TAGGED_REPLACE_TCI                            = 0x5 /**< Tagged - Replace TCI (IW mode only) */
  , e_UCC_GETH_VLAN_OPERATION_TAGGED_IF_VID0_REPLACE_TCI                    = 0x6 /**< Tagged - If VID=0, replace TCI (IW mode only) */
} e_UccGethVlanOperationTagged;

/**************************************************************************//**
 @Description   Possible VLAN operations on incoming non-tagged frames.

                These VLAN operations are programmed in one of two methods:
                - In address recognition mode, using UCC_GETH_ConfigVlanOperationNonTagged().
                - In extended filtering mode, through the Termination Action
                  Descriptor parameters.
*//***************************************************************************/
typedef enum e_UccGethVlanOperationNonTagged
{
    e_UCC_GETH_VLAN_OPERATION_NON_TAGGED_NOP          = 0x0 /**< Non-tagged - no operation */
  , e_UCC_GETH_VLAN_OPERATION_NON_TAGGED_Q_TAG_INSERT = 0x1 /**< Non-tagged - Q-tag insertion */
} e_UccGethVlanOperationNonTagged;

/**************************************************************************//**
 @Description   Receiver Quality of Service Mode
 
                These options select the QoS mode of the receiver. They are
                programmed in one of two methods:
                - In address recognition mode, using UCC_GETH_ConfigRxQoSMode().
                - In extended filtering mode, through the Termination Action
                  Descriptor parameters.
 
                See UCC_GETH_ConfigRxQoSMode() for more details.
*//***************************************************************************/
typedef enum e_UccGethQoSMode
{
    e_UCC_GETH_QOS_MODE_DEFAULT                    = 0x0 /**< Default queue */
  , e_UCC_GETH_QOS_MODE_QUEUE_NUM_FROM_L2_CRITERIA = 0x1 /**< Queue determined by L2 criteria */
  , e_UCC_GETH_QOS_MODE_QUEUE_NUM_FROM_L3_CRITERIA = 0x2 /**< Queue determined by L3 criteria */
} e_UccGethQoSMode;

/**************************************************************************//**
 @Description   Statistics Gathering Mode
 
                These are bit flags, 'or' them together for combined functionality.
*//***************************************************************************/
typedef enum e_UccGethStatisticsGatheringMode
{
    e_UCC_GETH_STATISTICS_GATHERING_MODE_NONE                   = 0x00000000 /**< No statistics gathering */
  , e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON                   = 0x00000001 /**< Enable RMON statistics gathering */
  , e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_TX       = 0x00000004 /**< Enable extended RMON Tx statistics gathering */
  , e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_RX       = 0x00000008 /**< Enable extended RMON Rx statistics gathering */
  , e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_TX_PER_Q = 0x00000014 /**< Enable extended RMON Tx statistics gathering
                                                                                  including per queue counters; availability of
                                                                                  this feature depends on ucode package */
} e_UccGethStatisticsGatheringMode;

/**************************************************************************//**
 @Description   Transmitter Padding and CRC Mode
 
                These options affect the transmitter, and relate to padding
                of short packets and adding CRC to transmitted packets.
                They can be configured using UCC_GETH_ConfigPadAndCrcMode().
 
                Note that padding without CRC is not possible.
*//***************************************************************************/
typedef enum e_UccGethMaccfg2PadAndCrcMode
{
    e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_NONE        = 0x00000000 /**< Neither padding of short frames nor CRC */
  , e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_CRC_ONLY    = 0x00000002 /**< Append CRC only */
  , e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_PAD_AND_CRC = 0x00000004 /**< Pad short frames and append CRC */
} e_UccGethMaccfg2PadAndCrcMode;

/**************************************************************************//**
 @Description   Automatic Flow Control Mode
*//***************************************************************************/
typedef enum e_UccGethUpsmrFlowControlMode
{
    e_UCC_GETH_UPSMR_AUTOMATIC_FLOW_CONTROL_MODE_NONE                 = 0x00000000 /**< No automatic flow control                                    */
  , e_UCC_GETH_UPSMR_AUTOMATIC_FLOW_CONTROL_MODE_PAUSE_WHEN_EMERGENCY = 0x00004000 /**< Send pause frame when RxFIFO reaches its emergency threshold */
} e_UccGethUpsmrFlowControlMode;

/**************************************************************************//**
 @Description   MII Configuration Management Clock Divisor
*//***************************************************************************/
typedef enum e_UccGethMiimcfgClockDivide
{
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_4     = 0x00000001,   /**< Divide by 4   */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_6     = 0x00000002,   /**< Divide by 6   */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_8     = 0x00000003,   /**< Divide by 8   */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_10    = 0x00000004,   /**< Divide by 10  */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_14    = 0x00000005,   /**< Divide by 14  */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_20    = 0x00000006,   /**< Divide by 20  */
    e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_28    = 0x00000007    /**< Divide by 28  */
} e_UccGethMiimcfgClockDivide;

/**************************************************************************//**
 @Description   Number of Ethernet Threads
 
                These are the valid options for the number of UCC Ethernet
                threads (either Tx or Rx).
 
                The configuration of threads number is done using
                UCC_GETH_ConfigNumThreadsTx() or UCC_GETH_ConfigNumThreadsRx()
*//***************************************************************************/
typedef enum e_UccGethNumOfThreads
{
    e_UCC_GETH_NUM_OF_THREADS_1  = 0x1  /**< 1 thread */
  , e_UCC_GETH_NUM_OF_THREADS_2  = 0x2  /**< 2 threads */
  , e_UCC_GETH_NUM_OF_THREADS_4  = 0x0  /**< 4 threads */
  , e_UCC_GETH_NUM_OF_THREADS_6  = 0x3  /**< 6 threads */
  , e_UCC_GETH_NUM_OF_THREADS_8  = 0x4  /**< 8 threads */
} e_UccGethNumOfThreads;


/**************************************************************************//**
 @Description   UCC GETH diagnostic mode
*//***************************************************************************/
typedef enum e_UccGethGUMRDiagnosticModeDIAG
{
    e_UCC_GETH_DIAGNOSTIC_NORMAL             = e_UCC_FAST_DIAGNOSTIC_NORMAL            , /**< normal             */
    e_UCC_GETH_DIAGNOSTIC_LOCAL_LOOP_BACK    = e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   , /**< loop back          */
    e_UCC_GETH_DIAGNOSTIC_AUTO_ECHO          = e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         , /**< echo               */
    e_UCC_GETH_DIAGNOSTIC_LOOP_BACK_AND_ECHO = e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO  /**< loop back and echo */
} e_UccGethGUMRDiagnosticModeDIAG;


#ifdef UCC_GETH_PTP_SUPPORT
/**************************************************************************//**
 @Description   UCC GETH PTP (IEEE 1588) mode
*//***************************************************************************/
typedef enum e_UccGethPtpMode
{
    e_UCC_GETH_PTP_DISABLED,    /**< PTP is disabled */
    e_UCC_GETH_PTP_IN_BAND,     /**< PTP is enabled in in-band mode */
    e_UCC_GETH_PTP_OUT_OF_BAND  /**< PTP is enabled in out-of-band mode */
} e_UccGethPtpMode;
#endif /* UCC_GETH_PTP_SUPPORT */


/**************************************************************************//**
 @Description   User's callback routine events handling (invoked by the driver).

 @Param[in]     h_App   - Application handle, provided by the user in the
                          configuration structure passed to UCC_GETH_Config().
 @Param[in]     events  - Events that caused this routine to be invoked by
                          the driver.
 @Param[in]     flags   - Protocol dependent indicator.
                          In UCC GETH they have a meaning for Rx busy event
                          only: The LSB is the busy queues vector (bit per queue:
                          0x1 is queue 0, 0x2 is queue 1, 0x4 is queue 2, etc).

 @Return        None.
 
 @See           @ref geth_exbm "UCC GETH Exceptions Bit Mask".
 *//***************************************************************************/
typedef void (t_UccGethException)(t_Handle h_App, uint32_t events, uint32_t flags);


/**************************************************************************//**
 @Description   UCC GETH initialization structure, used when calling UCC_GETH_Config().
 *//***************************************************************************/
typedef struct t_UccGethParam
{
    /* Ethernet protocol parameters */
    t_EnetAddr          macAddress;             /**< MAC address of device; First octet is sent first */
    e_EnetMode          enetMode;               /**< Ethernet operation mode (MAC-PHY interface and speed) */
    bool                halfDuplex;             /**< TRUE for half duplex; FALSE for full duplex */

    /* System integration parameters */
    uint32_t            baseAddress;            /**< Base address of UCC GETH registers in memory. */
    t_Handle            h_Qe;                   /**< Handle to the controlling QE object */
    e_Ucc               uccId;                  /**< UCC ID */
    e_QeClock           rxClock;                /**< Receive clock to use */
    e_QeClock           txClock;                /**< Transmit clock to use */
    int                 irq;                    /**< UCC interrupt ID (system-specific number) */
    uint8_t             pramMemId;              /**< Parameter RAM partition ID */
    uint32_t            dataMemAttributes;      /**< Memory attributes mask for data buffers */
    t_BufferPoolInfo    bufferPoolInfo;         /**< Data buffer pool information; Basically used
                                                     for Rx buffers, but in IW mode also for Tx buffers */

    /* Application interface */
    t_Handle            h_App;                  /**< Handle to an application context; This handle shall be passed
                                                     to the application in the UCC Ethernet callback routines */
    t_RxStoreFunction   *f_RxStore;             /**< User's callback function called by driver with receive data;
                                    	        	 If the last buffer contains CRC only the buffer will be NULL; The length
                                    	        	 will indicate the number of bytes at the end of the  last stored buffer
                                    	        	 that were CRC bytes. */
    t_TxConfFunction    *f_TxConf;              /**< User's callback function called by driver upon
                                                     completion of a buffer transmission; Not invoked if NULL */
    t_UccGethException  *f_Exceptions;          /**< Callback routine invoked by the driver on exception events;
                                                     If NULL, no exceptions are reported */
} t_UccGethParam;


/**************************************************************************//**
 @Function      UCC_GETH_Config

 @Description   Configures the UCC Ethernet based on user's parameters.

                Communication can then be initialized using UCC_GETH_Init()
                and enabled using UCC_GETH_Enable().

 @Param[in]     p_UccGethParam - User parameters for GETH object.

 @Return        Handle to created channel if successful; NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle UCC_GETH_Config(t_UccGethParam *p_UccGethParam);

/**************************************************************************//**
 @Function      UCC_GETH_Init

 @Description   Initializes driver and hardware for communication on the UCC Ethernet.

                Communication can then be enabled using UCC_GETH_Enable().

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config().
*//***************************************************************************/
t_Error UCC_GETH_Init(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_Free

 @Description   Frees all resources used by the UCC Ethernet.

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config().
 
 @Cautions      Does not disable channel, for which UCC_GETH_Disable() can be used.
*//***************************************************************************/
t_Error UCC_GETH_Free(t_Handle h_UccGeth);


/* Config Routines */

/**************************************************************************//**
 @Group         geth_advanced_init_grp UCC GETH Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UCC_GETH_ConfigQueueBdRingLenTx

 @Description   Configures the length of transmit BD ring. [32]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     bdRingLenTx - Transmit BD ring length.
 @Param[in]     queue       - Queue index.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigQueueBdRingLenTx(t_Handle h_UccGeth, uint16_t bdRingLenTx, uint8_t queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigQueueBdRingLenRx

 @Description   Configures the length of receive BD ring. [32]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     bdRingLenRx - Receive BD ring length.
 @Param[in]     queue       - Queue index.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigQueueBdRingLenRx(t_Handle h_UccGeth, uint16_t bdRingLenRx, uint8_t queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigVlanPriorityTable

 @Description   Configures an entry in the VLAN priority table.[0]
 
                This table is used only under specific combinations of Rx QoS
                mode and whether the received frame is VLAN tagged and/or
                an IP packet. See UCC_GETH_ConfigRxQoSMode() for details.

                When this table is used, the VLAN priority (VPri) of the received
                frame is used as index into this table. The value in the indexed
                entry in the table is used as the receive queue number into which
                the received frame is put. The table can hold up to 8 entries.

                Note that if this table is used, entry values must range from 0 to
                one less than number configured via UCC_GETH_ConfigNumQueuesRx().

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     entryValue  - Receive queue for receiving the matched frames.
 @Param[in]     index       - index into table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigVlanPriorityTable (t_Handle h_UccGeth, uint8_t entryValue, uint8_t index);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIpPriorityTable

 @Description   Configures an entry in the IP priority table. [0]
 
                This table is used only under specific combinations of Rx QoS
                mode and whether the received frame is an IP packet.
                See UCC_GETH_ConfigRxQoSMode() for details.

                When this table is used, the IP TOS or Traffic class of the
                received frame is used as index into this table. The value in
                the indexed entry in the table is used as the receive queue
                number into which to place the received frame. The table
                can hold up to 64 entries.

                When used with IPv4 TOS field, the higher 6 bits of TOS field
                are extracted as index into the table.

                Note that if this table is used, entry values must range from 0 to
                one less than number configured via UCC_GETH_ConfigNumQueuesRx().
 
                In Interworking mode, this table is used to map IP TOS values to
                VPri values for VLAN marking feature. Thus, if this feature is
                used, the table cannot be used for its original IP priority mapping,
                (the user must not select it in UCC_GETH_ConfigRxQoSMode()).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     entryValue  - For termination: Receive queue for receiving
                              the matched frames.
                              For interworking: VPri value to be used in
                              VLAN marking.
 @Param[in]     index       - Index into table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigIpPriorityTable (t_Handle h_UccGeth, uint8_t entryValue, uint8_t index);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxVlanTable

 @Description   Configures an entry in the Tx VLAN tag insertion table.
                [0]
 
                Tx VLAN tag insertion is invoked using the e_UccGethTxBdFlags
                parameter of the UCC_GETH_Tx() routine.
 
                The value in the indexed entry is the value of the VLAN tag
                to insert.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     entryValue  - The value of the VLAN tag to insert.
 @Param[in]     index       - index into table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxVlanTable (t_Handle h_UccGeth, uint32_t entryValue, uint8_t index);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigInterruptCoalescingMaxValue

 @Description   Configures the interrupt coalescing threshold for a receive queue.
                [1]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - The interrupt coalescing threshold.
 @Param[in]     queue       - Rx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigInterruptCoalescingMaxValue(t_Handle h_UccGeth,
                                                   uint8_t  threshold,
                                                   uint8_t  queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIntrManagerId

 @Description   Configures a receive queue to operate in interrupt manager mode.
                [0]
 
                Interrupt manager mode can be used only if the receive queue
                is configured for interrupt mode (that is the default mode, see
                UCC_GETH_ConfigRxQueueInterruptMode()).
 
                If the interrupt manager ID is non-zero, the driver defers the
                invocation of the receive routine to the user. The driver
                notifies the user through XX_Call(), passing a pointer to a
                function, and the user is responsible for calling that function
                eventually (usually not from within XX_Call() itself). The given
                function shall invoke the receive routines of all queues that
                operate in this mode and have received frames waiting in the queue.
 
                XX_Call() will not be called again (for any receive queue), until
                the user calls the function mentioned above.
 
                If the interrupt manager ID is zero (default mode), this mode is
                disabled for the given receive queue. In this case, the driver
                invokes the receive routine of the queue directly from the
                interrupt routine.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     intrManagerId   - Interrupt manager ID (pass 0 to disable).
 @Param[in]     queue           - Rx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
 
 @Cautions      All queues that operate in interrupt manager mode must use the
                same interrupt manager ID.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIntrManagerId(t_Handle   h_UccGeth,
                                     uint32_t   intrManagerId,
                                     uint8_t    queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxQueueInterruptMode

 @Description   Configures the Rx interrupt mode for a receive queue.
                [TRUE]
 
                When working via interrupts, the receive routine is called
                by the driver in interrupt context. When working via polling
                (Rx interrupt is disabled), the user needs to call the UCC_GETH_Rx()
                routine to receive data.
 
                In either method, the receive routine invokes the user callback
                routine (set in the parameters structure passed to UCC_GETH_Config())
                for handling the received data.
 
                Note, that in interrupt mode there is also an option to work
                in interrupt manager mode. See UCC_GETH_ConfigIntrManagerId().

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     rxInterrupts    - TRUE to enable Rx interrupt mode;
                                  FALSE to work in polling mode.
 @Param[in]     queue           - Rx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxQueueInterruptMode(t_Handle    h_UccGeth,
                                            bool        rxInterrupts,
                                            uint8_t     queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumQueuesTx

 @Description   Configures the number of Tx queues. [1]
 
                The number of queues must range from 1 to 8. The queues shall
                have queue numbers 0 through one less than number given here.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     numQueuesTx - Requested number of transmit queues.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumQueuesTx(t_Handle h_UccGeth, uint8_t numQueuesTx);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumQueuesRx

 @Description   Configures the number of Rx queues. [1]
 
                The number of queues must range from 1 to 8. The queues shall
                have queue numbers 0 through one less than number given here.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     numQueuesRx - Requested number of receive queues.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumQueuesRx(t_Handle h_UccGeth, uint8_t numQueuesRx);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumThreadsTx

 @Description   Configures the number of threads for the transmitter.
                [for 100BT: e_UCC_GETH_NUM_OF_THREADS_1,
                 for 1000BT: e_UCC_GETH_NUM_OF_THREADS_4]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     numThreadsTx    - Requested number of transmitter threads.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumThreadsTx(t_Handle h_UccGeth, e_UccGethNumOfThreads numThreadsTx);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumThreadsRx

 @Description   Configures the number of threads for the receiver.
                [for 100BT: e_UCC_GETH_NUM_OF_THREADS_1,
                 for 1000BT: e_UCC_GETH_NUM_OF_THREADS_4]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     numThreadsRx    - Requested number of receiver threads.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumThreadsRx(t_Handle h_UccGeth, e_UccGethNumOfThreads numThreadsRx);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRiscTx

 @Description   Configures the QE RISC on which to run Tx threads.
                [QE_RISC_ALLOC_ALL_RISCS]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     riscAllocation  - Requested RISC allocation.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
                For the 'riscAllocation' argument, use the 'RISC Allocation' macros.
*//***************************************************************************/
t_Error UCC_GETH_ConfigRiscTx(t_Handle h_UccGeth, uint8_t riscAllocation);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRiscRx

 @Description   Configures the QE RISC on which to run Rx threads.
                [QE_RISC_ALLOC_ALL_RISCS]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     riscAllocation  - Requested RISC allocation.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
                For the 'riscAllocation' argument, use the 'RISC Allocation' macros.
*//***************************************************************************/
t_Error UCC_GETH_ConfigRiscRx(t_Handle h_UccGeth, uint8_t riscAllocation);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigIpCheckSumCheck

 @Description   Enables/disables checking of IPv4 checksums.
                [FALSE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable checking of IPv4 checksums.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigIpCheckSumCheck(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigIpCheckSumGenerate

 @Description   Enables/disables generation of IPv4 checksums.
                [FALSE]
 
                IP checksum is generated on IP header whose location in the frame
                is represented by an offset from the beginning of the frame.
                This offset is determined by an entry in a table whose contents
                are set using UCC_GETH_ConfigTxChecksumOffsetTable().
 
                The entire IPv4 header (including any IPv4 options) must be in
                the first data buffer of the frame. The IP checksum field in the
                IPv4 header of the packet is expected be cleared to zero before
                transmission.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable generation of IPv4 checksums.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigIpCheckSumGenerate(t_Handle h_UccGeth, bool enabled);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxChecksumOffsetTable

 @Description   Configures an entry in the IP checksum offset table.
                [0]
 
                The entries in the IP checksum offset table are used as offset
                from the beginning of IP packets, for which IP checksum is
                calculated. If the entry contains 0, checksum is not calculated.

                The user can select an entry in the table for each transmitted
                IP packet, via the e_UccGethTxBdFlags flags of the transmit
                routines. The flags and this table are relevant only if
                UCC_GETH_ConfigIpCheckSumGenerate() has been called with TRUE.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     entryValue  - Offset from beginning of IP packet.
 @Param[in]     index       - index into table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxChecksumOffsetTable (t_Handle h_UccGeth, uint8_t entryValue, uint8_t index);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigMultipleStationAddresses

 @Description   Enables/disables matching of multiple station addresses in the
                receiver. [FALSE]
 
                Relevant only for non extended parsing mode, i.e. legacy
                MPC82xx-like filtering.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable multiple station addresses.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMultipleStationAddresses(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxExtendedFiltering

 @Description   Enables/disables extended filtering mode of the receiver.
                [FALSE]
 
                The UCC Ethernet can operate in one of two modes of frames
                filtering:
                - Address Recognition , MPC82xx-compatible (default mode)
                - Extended Parsing and Filtering
 
                The extended filtering mode allows for enhanced frame filtering
                based on fields extracted from the Ethernet header of the frame.
                The choice of the fields is user programmable.
 
                The following routines must also be called for Extended Filtering
                mode:
                - UCC_GETH_ConfigLargestExternalLookupKeySize() - required by the
                  Ethernet receiver for correct operation.
                - UCC_GETH_ConfigExtendedFilteringChainPointer() - to set the
                  pointer to the filtering chain that shall be used by the receiver.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable Rx extended filtering mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxExtendedFiltering(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//*
 @Function      UCC_GETH_ConfigL2Filter

 @Description   Configures the L2 filter for extended filtering mode (or IW).
                [0]
 
                This routine is relevant (and mandatory) only if
                UCC_GETH_ConfigRxExtendedFiltering() has been called with TRUE.
 
                The filter is constructed using the QE Filter API. This pointer
                is used by the UCC for all receive queues, therefore the
                filter must apply for all active receive queues.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     filterRootAddr  - Pointer to the L2 filter.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigL2Filter(t_Handle h_UccGeth, uint32_t filterRootAddr);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigLargestExternalLookupKeySize

 @Description   Configures the size of the largest lookup key of any lookup
                table in external memory. [e_QE_FLTR_LARGEST_EXTERNAL_TABLE_LOOKUP_KEY_SIZE_NONE]
 
                This value is required for correct operation of the UCC Ethernet
                receiver in extended filtering mode.
 
                This routine is relevant (and mandatory) only if
                UCC_GETH_ConfigRxExtendedFiltering() has been called with TRUE.
 
 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     size        - Largest external lookup key size.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigLargestExternalLookupKeySize(t_Handle h_UccGeth,
                                                    e_QeFltrLargestExternalTableLookupKeySize size);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTypeOrLenThreshold

 @Description   Configures the cutoff point less than which, Ethernet type/len
                field is considered length, and otherwise considered type.
                [1536]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Threshold between type and length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTypeOrLenThreshold(t_Handle h_UccGeth, uint16_t threshold);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigDynamicMaxFrameLength

 @Description   Enables/disables the dynamic maximum frame length mode.
                [FALSE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable dynamic maximum frame length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigDynamicMaxFrameLength(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigDynamicMinFrameLength

 @Description   Enables/disables the dynamic minimum frame length mode.
                [FALSE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable dynamic minimum frame length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigDynamicMinFrameLength(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNonBackToBackInterFrameGapPart1

 @Description   Configures the optional carrier sense window referenced in
                IEEE 802.3/4.2.3.2.1 `carrier deference' in bits.
                [0x40]
 
                If carrier is detected during the timing of IPGR1, the MAC
                defers to carrier. If, however, carrier becomes active after
                IPGR1,the MAC continues timing IPGR2 and transmits, knowingly
                causing a collision; thus, ensuring fair access to medium.
 
                The valid range of values is 0x00 to IPGR2. 2/3 of IPGR2 is
                recommended.
 
                Note: IPGR1 is controlled by ConfigNonBackToBackInterFrameGapPart1()
                and IPGR2 is controlled by ConfigNonBackToBackInterFrameGapPart2().

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     value       - Non back-to-back inter-frame gap part 1.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNonBackToBackInterFrameGapPart1(t_Handle h_UccGeth, uint8_t value);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNonBackToBackInterFrameGapPart2

 @Description   Configures the non back-to-back inter-frame gap in bits.
                [0x60]
 
                The maximum value allowed is 127.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     value       - Non back-to-back inter-frame gap part 2.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNonBackToBackInterFrameGapPart2(t_Handle h_UccGeth, uint8_t value);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMinimumInterFrameGapEnforcement

 @Description   Configures the minimum number of bits of IFG to enforce between
                frames. [0x50]
 
                A frame whose IFG is less than the programmed value is dropped.
                The maximum value allowed is 255.

 @Param[in]     h_UccGeth           - Handle to UCC GETH object.
 @Param[in]     minInterFrameGap    - Minimum number of IFG bits to enforce
                                      between frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMinimumInterFrameGapEnforcement(t_Handle h_UccGeth,
                                                       uint8_t  minInterFrameGap);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigBackToBackInterFrameGap

 @Description   Configures the inter-frame gap between back-to-back frames in bits.
                [0x60]
 
                The maximum value allowed is 127.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     value       - Back-to-back inter-frame gap (in bits).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigBackToBackInterFrameGap(t_Handle h_UccGeth, uint8_t value);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigStatisticsMode

 @Description   Configures the statistics gathering method. [e_UCC_GETH_STATISTICS_GATHERING_MODE_NONE]
 
                The UCC Ethernet implements a set of statistical counters. Some of
                the counters are implemented in hardware, and the rest are
                implemented in firmware.
 
                The user can select which counters group to activate, and also
                get the values of these counters on demand, by calling
                UCC_GETH_GetStatistics().

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     statisticsMode  - Requested statistics mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigStatisticsMode(t_Handle                          h_UccGeth,
                                      e_UccGethStatisticsGatheringMode  statisticsMode);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigVlanOperationTagged

 @Description   Configures the VLAN operation for tagged incoming frames.
                [e_UCC_GETH_VLAN_OPERATION_TAGGED_NOP]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     vlanOperation   - VLAN operation to perform.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigVlanOperationTagged(t_Handle                     h_UccGeth,
                                           e_UccGethVlanOperationTagged vlanOperation);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigVlanOperationNonTagged

 @Description   Configures the VLAN operation for non-tagged incoming frames.
                [e_UCC_GETH_VLAN_OPERATION_NON_TAGGED_NOP]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     vlanOperation   - VLAN operation to perform.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigVlanOperationNonTagged(t_Handle                          h_UccGeth,
                                              e_UccGethVlanOperationNonTagged   vlanOperation);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxQoSMode

 @Description   Configures the Quality of Service mode of the receiver.
                [e_UCC_GETH_QOS_MODE_DEFAULT]

                The received frame is directed to one of the receive queues
                according to following table:

                @copydoc DOXYGEN_TABLE_ConfigRxQoSMode
                @if DOXYGEN_IGNORE

                ------------------------------------------------|--------------------------------------------------------------
                                                                |                 Incoming Frame Type
                       Value handed to this routine             |--------------------------------------------------------------
                                                                | VLAN tagged | VLAN tagged | Not VLAN tagged | Not VLAN tagged
                                                                |   not IP    |     IP      |     not IP      |      IP
                ================================================|==============================================================
                e_UCC_GETH_QOS_MODE_DEFAULT                     | Default queue as programmed in VPri field of TCI.
                                                                | TCI is programmed via UCC_GETH_ConfigTci().
                ------------------------------------------------|--------------------------------------------------------------
                e_UCC_GETH_QOS_MODE_QUEUE_NUM_FROM_L2_CRITERIA  | VLAN        | VLAN        |                 |
                                                                | priority    | priority    |                 |
                                                                | translated  | translated  | Default queue   | Default queue
                                                                | by VLAN     | by VLAN     |                 |
                                                                | priority    | priority    |                 |
                                                                | table       | table       |                 |
                ------------------------------------------------|--------------------------------------------------------------
                e_UCC_GETH_QOS_MODE_QUEUE_NUM_FROM_L3_CRITERIA  | VLAN        | IP TOS or   |                 | IP TOS or
                                                                | priority    | traffic     |                 | traffic
                                                                | translated  | class,      | Default queue   | class,
                                                                | by VLAN     | translated  |                 | translated
                                                                | priority    | by IP       |                 | by IP
                                                                | table       | priority    |                 | priority
                                                                |             | table       |                 | table
                ---------------------------------------------------------------------------------------------------------------
                @endif

                The VLAN priority table is programmed via UCC_GETH_ConfigVlanPriorityTable().
                The IP priority table is programmed via UCC_GETH_ConfigIpPriorityTable().

                Note: This routine is relevant only in non-extended filtering mode.
                In extended filtering mode, this functionality is controlled via the
                entry in the Termination Action Descriptor.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     rxQosMode   - Requested QoS mode of the receiver.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxQoSMode(t_Handle h_UccGeth, e_UccGethQoSMode rxQosMode);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigIpAddressAlignment

 @Description   Enables/disables the IP address alignment mode.
                [FALSE]
 
                The IP address alignment is achieved by shifting the received
                frame by two bytes, to align to 4-byte boundary. Valid data in
                the first data buffer begins two bytes into the buffer.
 
                If this mode is enabled, data buffers must be large enough
                to accommodate the extra two bytes.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable IP address alignment mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigIpAddressAlignment(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigLengthCheckRx

 @Description   Enables/disables length field checking in received frames.
                [FALSE]
 
                if this mode is enabled, the firmware checks the length field
                of each received frame, to ensure that it matches the actual
                data field length. If the actual received frame length does not
                match the value in the length field, then the value of
                inRangeLenRxER statistics counter is incremented.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable length field checking.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigLengthCheckRx(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMiiPreambleSuppress

 @Description   Enables/disables the MII management preamble suppression.
                [FALSE]
 
                Enabling this mode reduces the management cycle from 64 clocks
                to 32 clocks. This is in accordance with IEEE 802.3/22.2.4.4.2.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable MII management preamble
                              suppression.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMiiPreambleSuppress(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMiiClockDivide

 @Description   Configures the amount by which to divide core complex bus clock
                to generate the MII source clock (EC_MDC). [e_UCC_GETH_MIIMCFG_MANAGEMENT_CLOCK_DIVIDE_BY_28]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     divisor     - Amount by which to divide the source clock.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMiiClockDivide(t_Handle h_UccGeth, e_UccGethMiimcfgClockDivide divisor);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigAltBinaryExponentialBackoffTruncation

 @Description   Configures an alternative value for the standard value of the
                alternate binary exponential backoff truncation setting.
                [10]
 
                The IEEE-802.3 standard specifies that any collision after the
                tenth uses one less than 2^10 as the maximum backoff time.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     value       - Requested truncation limit.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigAltBinaryExponentialBackoffTruncation(t_Handle h_UccGeth, uint8_t value);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigBackPressureNoBackoff

 @Description   Enables/disables immediate retransmission by the Tx MAC
                following a collision during back-pressure operation.
                [FALSE]
 
                Calling this routine with TRUE alters the protocol standard
                rules as defined in IEEE-802.3 standard.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable immediate retransmission.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigBackPressureNoBackoff(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNoBackoff

 @Description   Enables/disables immediate retransmission by the Tx MAC
                following a collision.
                [FALSE]
 
                Calling this routine with TRUE alters the protocol standard
                rules as defined in IEEE-802.3 standard.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable immediate retransmission.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigNoBackoff(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigExcessDefer

 @Description   Enables/disables transmission of excessively deferred frames.
                [TRUE]
 
                An excess defer condition occurs when the Ethernet controller
                waits for more than 3036-byte time while attempting to send a
                frame.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable transmission of excessively
                              deferred frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigExcessDefer(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMaxRetransmission

 @Description   Configures the maximum number of retransmission attempts allowed
                following a collision, before aborting the frame.
                [15]
 
                According to the IEEE-802.3 standard, this value should be 15.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     maxAttempts - Number of retransmission attempts allowed.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMaxRetransmission(t_Handle h_UccGeth, uint8_t maxAttempts);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigCollisionWindow

 @Description   Configures the slot time or collision window during which
                collisions occur in properly configured networks.
                [0x37]
 
                Because the collision window starts at the beginning of
                transmission, the preamble and SFD are included in this value.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     windowSize  - Collision window size.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigCollisionWindow(t_Handle h_UccGeth, uint8_t windowSize);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigPromiscuous

 @Description   Enables/disables the promiscuous receiver mode. [FALSE]
 
                In promiscuous mode, all incoming frames are accepted regardless
                of their destination address.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable promiscuous receiver mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigPromiscuous(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigBroadcastAddressReject

 @Description   Enables/disables rejection of all frames containing the
                broadcast address (unless promiscuous mode is enabled).
                [FALSE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable rejection of broadcast frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigBroadcastAddressReject(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigReceiveLookupMissFrames

 @Description   Enables/disables reception of frames that were not matched by
                extended filtering lookup. [FALSE]
 
                This routine is relevant only for extended filtering mode -
                see UCC_GETH_ConfigRxExtendedFiltering().
 
                Such "missed" frames shall be received in queue 0.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to receive frames that were not matched
                              by extended filtering lookup.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigReceiveLookupMissFrames(t_Handle h_UccGeth, bool enabled);




/**************************************************************************//**
 @Function      UCC_GETH_ConfigReceiveShortFrames

 @Description   Enables/disables reception of short frames. [FALSE]
 
                Short frames are frames that are shorter than 64 bytes. The user
                can call UCC_GETH_ConfigMinFrameLength() to override the standard
                value. Such frames shall always be received in receive queue 0.
 
 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable reception of short frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigReceiveShortFrames(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigFlowControlMode

 @Description   Enables/disables hardware-initiated flow control. [e_UCC_GETH_UPSMR_AUTOMATIC_FLOW_CONTROL_MODE_NONE]
 
                The hardware starts transmitting PAUSE frames when the UCC
                receive FIFO reaches a programmable emergency threshold. When
                the receive FIFO goes below the threshold, the hardware sends
                one last Pause frame with the pause period set to zero, to
                resume transmission at the far end.
 
                Use UCC_GETH_ConfigHardwareFlowControlPausePeriod() and
                UCC_GETH_ConfigHardwareFlowControlExtensionField() to configure
                the pause period and extension field of the Pause frame.
 
                Use UCC_GETH_ConfigRxVirtFifoEmergThresh() to configure the UCC
                receive FIFO threshold.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     flowControlmode - Requested flow control mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigFlowControlMode(t_Handle                      h_UccGeth,
                                       e_UccGethUpsmrFlowControlMode flowControlmode);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigHardwareFlowControlPausePeriod

 @Description   Configures the pause time for hardware-initiated flow control.
                [0xf000]
 
                The units of the selected time period depends on the line speed,
                as follows:
                - For 10BT, 51.2 microseconds.
                - For 100BT, 5.12 microseconds.
                - For 1000BT, 4.096 microseconds.
 
                This routine is relevant only if hardware-initiated flow control
                was enabled using UCC_GETH_ConfigFlowControlMode().

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     pausePeriod - Selected pause period.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigHardwareFlowControlPausePeriod(t_Handle  h_UccGeth,
                                                      uint16_t  pausePeriod);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigHardwareFlowControlExtensionField

 @Description   Configures the extension field for hardware-initiated flow control.
                [0]
 
                This routine is relevant only if hardware-initiated flow control
                was enabled using UCC_GETH_ConfigFlowControlMode().

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     extensionField  - Extension field value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigHardwareFlowControlExtensionField(t_Handle   h_UccGeth,
                                                         uint16_t   extensionField);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigReceiveFlowControl

 @Description   Enables/disables detection and acting on received PAUSE flow
                control frames. [TRUE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - FALSE to disable acting on received PAUSE frames;
                              TRUE to enable acting on received PAUSE frames.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigReceiveFlowControl(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigLossLessFlowControl

 @Description   Enables/disables loss less flow control functionality.
                [FALSE]
 
                If this mode is enabled, the UCC Ethernet transmits a PAUSE
                frame if the corresponding Rx BD ring has reached a programmable
                threshold of empty BDs. This mode affects all receive queues.
 
                Use UCC_GETH_ConfigLossLessFlowControlThreshold() to override
                the default threshold value (per receive queue).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable loss less flow control.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigLossLessFlowControl(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigLossLessFlowControlThreshold

 @Description   Configures the threshold for loss less flow control functionality
                (per receive queue). [0]
 
                Determines the minimum number of Rx BDs that should be empty in
                the Rx BD ring. If this threshold is reached, a flow control
                frame is automatically transmitted. This routine is relevant
                only if loss less flow control was enabled using
                UCC_GETH_ConfigLossLessFlowControl().
 
                The user must configure the loss less flow control threshold to
                a value that ensures that there are always some empty BDs in the
                Rx BD ring.
 
                If this parameter is set to zero (default value), the driver
                calculates a default threshold, suitable for the worst case
                condition, as described below:
 
                Default Threshold = (MTU / Max Rx buffer length) + N + 5

                MTU is the Maximum Transfer Unit (maximum frame size). N is
                determined as follows: all frames which are currently located in
                the Virtual FIFO must have empty Rx BDs available. In the worst
                case condition the Virtual FIFO is filled with frames of minimum
                size. Therefore:
 
                N = Virtual FIFO size / (Min frame size + 8)
 
                This feature may require more BDs than normally assigned for the
                Rx BD ring, because the worst case scenario is considered in the
                threshold calculations.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Requested loss less flow control threshold.
 @Param[in]     queue       - Rx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigLossLessFlowControlThreshold(t_Handle    h_UccGeth,
                                                    uint16_t    threshold,
                                                    uint8_t     queue);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxQueueMgmtWeightFactor

 @Description   Configures the exponential weight factor for advanced queue
                management functionality (per receive queue).
                [0]
 
                The average queue size is calculated by applying and exponential
                weight low pass filter to the queue size samples. Consequently,
                the average queue size is based on the previous average and the
                current size of the queue. The formula is:
 
                average = (old_average * (1-1/2^n)) + (current_queue_size * 1/2^n)
 
                where n is the exponential weight factor (expWeightFactor), as
                configured by this routine.
 
                For high values of expWeightFactor, the previous average becomes
                more important. A large factor smooths out the peaks and lows in
                queue length. The average queue size is unlikely to change very
                quickly, avoiding drastic swings in size. The WRED process will
                be slow to start dropping packets, but it may continue dropping
                packets for a time after the actual queue size has fallen below
                the minimum threshold. The slow-moving average will accommodate
                temporary bursts in traffic.
 
                If the value of expWeightFactor gets too high, WRED will not react
                to congestion. Packets will be passed on or dropped as if WRED
                were not in effect.
 
                For low values of expWeightFactor, the average queue size closely
                tracks the current queue size. The resulting average may fluctuate
                with changes in the traffic levels. In this case, the WRED process
                responds quickly to long queues. Once the queue falls below the
                minimum threshold, the process will stop dropping packets.
 
                If the value of expWeightFactor gets too low, WRED will overreact
                to temporary traffic bursts and drop traffic unnecessarily.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     expWeightFactor - Exponential weight factor.
 @Param[in]     queue           - Rx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxQueueMgmtWeightFactor(t_Handle h_UccGeth,
                                               uint8_t  expWeightFactor,
                                               uint8_t  queue);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxQueueMgmtWeightFactor

 @Description   Configures the exponential weight factor for advanced queue
                management functionality (per transmit queue).
                [0]

                See UCC_GETH_ConfigTxQueueMgmtWeightFactor() for more details
                on selecting an exponential weight factor for queue management.
 
                This routine is relevant only for interworking Tx queues (not
                for CPU transmit queues).

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     expWeightFactor - Exponential weight factor.
 @Param[in]     queue           - Tx queue number.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxQueueMgmtWeightFactor(t_Handle h_UccGeth,
                                               uint8_t  expWeightFactor,
                                               uint8_t  queue);

/**************************************************************************//**
 @Description   Parameters for Rx virtual FIFO overrun smoother.
 
                These parameters are required for enabling the Rx virtual FIFO
                smoother using the UCC_GETH_ConfigRxOverrunSmoother() routine.
*//***************************************************************************/
typedef struct t_UccGethRxOvrSmootherParam
{
    uint32_t    rxBurstRate;            /**< Rx burst rate [bps]; this value
                                             usually equals maxmimum line rate. */
    uint32_t    rxSustainedRate;        /**< Rx sustained rate [bps]; This value
                                             should not exceed Rx burst rate. */
    uint16_t    averageRxPacketSize;    /**< Average packet size on Rx */
} t_UccGethRxOvrSmootherParam;

/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxOverrunSmoother

 @Description   Configures the Rx virtual FIFO overrun smoother for the
                Ethernet port.
                [FALSE]

 @Param[in]     h_UccGeth               - Handle to UCC GETH object.
 @Param[in]     enabled                 - TRUE to enable the Rx virtual
                                          FIFO overrun smoother.
 @Param[in]     p_RxOvrSmootherParam    - Overrun smoother parameters; Can
                                          be set to NULL if the enabled
                                          parameter is set to FALSE.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxOverrunSmoother(t_Handle                       h_UccGeth,
                                         bool                           enabled,
                                         t_UccGethRxOvrSmootherParam    *p_RxOvrSmootherParam);

#ifdef UCC_GETH_PTP_SUPPORT
/**************************************************************************//**
 @Function      UCC_GETH_ConfigPtpMode

 @Description   Configures the PTP (IEEE 1588) mode for the Ethernet port.
                [e_UCC_GETH_PTP_DISABLED]
 
                Note that when PTP in-band mode is enabled, the Rx soft
                preamble option must not be used.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     ptpMode     - Selected PTP mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigPtpMode(t_Handle h_UccGeth, e_UccGethPtpMode ptpMode);
#endif /* UCC_GETH_PTP_SUPPORT */


/**************************************************************************//**
 @Function      UCC_GETH_ConfigPadAndCrcMode

 @Description   Configures short frames padding and/or CRC appending to all frames.
                [e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_PAD_AND_CRC]
 
                Note that using the short frames padding option requires adding
                CRC as well.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     padAndCrcMode   - Padding and CRC mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigPadAndCrcMode(t_Handle                       h_UccGeth,
                                     e_UccGethMaccfg2PadAndCrcMode  padAndCrcMode);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigPreambleLength

 @Description   Configures the length in bytes of the preamble field.
                [7]
 
 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     preambleLength  - Requested preamble length; Valid values
                                  are 3-7 bytes.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigPreambleLength(t_Handle h_UccGeth, uint8_t preambleLength);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigSoftPreambleTx

 @Description   Enables/disables soft transmit preamble mode.
                [FALSE]
 
                If soft transmit preamble is enabled, the user can call the
                transmit routines with e_UCC_GETH_TX_BD_FLAGS_PROGRAMMABLE_PREAMBLE
                in the flags parameter, then the first 8 bytes of the data buffer
                are used as 7 bytes preamble and one byte of place holder. The
                Ethernet controller transmits the SFD automatically. All eight
                bytes must reside in one data buffer.
 
                If the transmit routine is called without including the
                e_UCC_GETH_TX_BD_FLAGS_PROGRAMMABLE_PREAMBLE in the flags parameter,
                then the default preamble (7 bytes) is transmitted.
 
                The user must not use UCC_GETH_ConfigPreambleLength() in this mode.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable soft transmit preamble.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().

 
 @Cautions      Not valid for RMII, TBI and RTBI interfaces.
*//***************************************************************************/
t_Error UCC_GETH_ConfigSoftPreambleTx(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigSoftPreambleRx

 @Description   Enables/disables soft receive preamble mode.
                [FALSE]
 
                If soft transmit preamble is enabled, the received preamble and
                SFD are placed in the first eight bytes of the first data buffer
                of the frame.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable soft receive preamble.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().

 
 @Cautions      Not valid for RMII, TBI and RTBI interfaces.
*//***************************************************************************/
t_Error UCC_GETH_ConfigSoftPreambleRx(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMaxFrameLength

 @Description   Configures the maximum Ethernet frame length in bytes.
                [1518]
 
 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     maxFrameLength  - Requested maximum frame length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMaxFrameLength(t_Handle h_UccGeth, uint16_t maxFrameLength);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMinFrameLength

 @Description   Configures the minimum Ethernet frame length in bytes.
                [64]
 
                If the receiver detects an incoming frame shorter than the
                minimum length, it discards that frame unless the
                UCC_GETH_ConfigReceiveShortFrames() routine was called with TRUE.
 
                Note, that short frames are always received in queue 0.
                The minimum frame length must be less than the configured
                maximum receive buffer length.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     minFrameLength  - Requested minimum frame length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMinFrameLength(t_Handle h_UccGeth, uint16_t minFrameLength);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigMaxD2Length

 @Description   Configures the maximum DMA2 length.
                [0xFFFF (no truncation)]
 
                This parameter lets the user prevent system bus writes after a
                frame exceeds a certain size. The value of MAXD2 is valid in
                promiscuous or extended filtering mode when no address match is
                detected. If the Ethernet controller detects an incoming frame
                larger than this value, the rest of the frame is discarded, and
                the length reported to the application shall be MaxD2.
 
                In a monitor station, MaxD2 can be much less than the maximum
                frame length, to receive entire frames addressed to this station,
                but receive only the headers of all other frames.

                If this functionality is required, this value must be smaller
                than the maximum frame length.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     maxD2Length - Maximum DMA2 length (in bytes).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigMaxD2Length(t_Handle h_UccGeth, uint16_t maxD2Length);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigDefaultVlanType

 @Description   Configures the inserted VLAN type value (as part of the Q-tag)
                for some operating modes. [0x8100]
 
                See the description of UCC_GETH_ConfigRxQoSMode(),
                UCC_GETH_ConfigVlanOperationTagged(),  and
                UCC_GETH_ConfigVlanOperationNonTagged() routines for
                information on modes which involve VLAN insertion.
 
                This routine is relevant only in non-extended filtering mode.
                For extended filtering mode, this functionality is controlled via
                the entry in the Termination Action Descriptor.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     vlanType    - Default VLAN type.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigDefaultVlanType(t_Handle h_UccGeth, uint16_t vlanType);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigDefaultVlanTci

 @Description   Configures the inserted VLAN TCI field (as part of the Q-tag)
                for some operating modes. [0x0000]
 
                See the description of UCC_GETH_ConfigRxQoSMode(),
                UCC_GETH_ConfigVlanOperationTagged(),  and
                UCC_GETH_ConfigVlanOperationNonTagged() routines for
                information on modes which involve VLAN insertion.
 
                This routine is relevant only in non-extended filtering mode.
                For extended filtering mode, this functionality is controlled via
                the entry in the Termination Action Descriptor.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     vlanTci     - Default VLAN TCI.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigDefaultVlanTci(t_Handle h_UccGeth, uint16_t vlanTci);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigExceptions

 @Description   Configures the exceptions and events for which to invoke
                the user's exceptions callback. [All]
 
                The supplied value should be an or'ed combination of any, none,
                or all of the following:
                UCC_GETH_UCCE_CBPR, UCC_GETH_UCCE_SCAR, UCC_GETH_UCCE_BSY,
                UCC_GETH_UCCE_RXC, UCC_GETH_UCCE_TXC, UCC_GETH_UCCE_TXE.
 
                The user's exceptions callback is part of the configuration
                structure passed to UCC_GETH_Config().

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     exceptionsMask  - Mask of enabled exceptions.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
 

 @See           @ref geth_exbm "UCC GETH Exceptions Bit Mask".
*//***************************************************************************/
t_Error UCC_GETH_ConfigExceptions(t_Handle h_UccGeth, uint32_t exceptionsMask);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTbiPhysicalAddress

 @Description   Configures the TBI physical address base. [0x10]
 
                This TBI address is set to the number given here plus the UCC
                number's offset from UCC1. For example, if the address given
                here is 5, and the UCC ID is e_UCC_3, then this UCC will be
                assigned a TBI address equal to 5 + (e_UCC_3 - e_UCC_1) = 7.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     tbiAddress  - TBI physical address base.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTbiPhysicalAddress(t_Handle h_UccGeth, uint8_t tbiAddress);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigFirmwareExternalStructsMemory(t_Handle   h_UccGeth,
                                                     uint8_t    memId,
                                                     uint32_t   memAttributes);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigInitTx

 @Description   Enables/disables the initialization of the transmitter.
                [TRUE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - FALSE to skip transmitter initialization;
                              TRUE to initialize the transmitter;

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigInitTx(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigInitRx

 @Description   Enables/disables the initialization of the receiver.
                [TRUE]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - FALSE to skip receiver initialization;
                              TRUE to initialize the receiver;

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigInitRx(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxVirtFifoSize

 @Description   Configures the Rx virtual FIFO size in bytes, allocated in the
                multi-user RAM.
                [For under 1000BT: 512,
                 For 1000BT: 2048]

 @Param[in]     h_UccGeth - Handle to UCC GETH object.
 @Param[in]     size      - Rx virtual FIFO size.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxVirtFifoSize(t_Handle h_UccGeth, uint16_t size);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxVirtFifoEmergThresh

 @Description   Configures the lower emergency threshold level of the Rx virtual
                FIFO.
                [For under 1000BT: 256,
                 For 1000BT: 1024]
 
                This emergency threshld is the minimum amount of bytes in virtual
                FIFO for which RISC requests for data handling get emergency
                priority. The recommended value for most applications is 1/2 of
                the Rx virtual FIFO size. To disable the emergency state, program
                this threshold to a value which exceeds the Rx virtual FIFO size.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Rx virtual FIFO emergency threshold.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxVirtFifoEmergThresh(t_Handle h_UccGeth, uint16_t threshold);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxVirtFifoSpecEmergThresh

 @Description   Configures the Rx virtual FIFO special emergency threshold.
                [For under 1000BT: 384,
                 For 1000BT: 1536]
 
                This threshod is the upper threshold level of the Rx virtual FIFO.
                The UCC enters a special emergency state when the virtual FIFOs
                level exceeds this threshold, and exits this state when the
                virtual FIFOs level drops below the normal emergency threshold.
 
                During the special emergency state, the UCC transmits Pause flow
                control frames to the link partner.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Rx virtual FIFO special emergency threshold.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxVirtFifoSpecEmergThresh(t_Handle h_UccGeth, uint16_t threshold);


/**************************************************************************//**
 @Description   UCC GETH Rx Virtual FIFO Block Size
 
                The QUICC Engine splits incoming frames to several blocks,
                processes and delivers them one by one to the driver. Choosing
                a bigger Virtual FIFO block size can improve the packet
                processing rate.
*//***************************************************************************/
typedef enum e_UccGethRxVirtFifoBlockSize
{
    e_UCC_GETH_RX_VIRT_FIFO_BLOCK_SIZE_128_BYTES = 0x00000000, /**< Block size is 128 bytes */
    e_UCC_GETH_RX_VIRT_FIFO_BLOCK_SIZE_192_BYTES = 0x00BF0000, /**< Block size is 192 bytes */
    e_UCC_GETH_RX_VIRT_FIFO_BLOCK_SIZE_248_BYTES = 0x00F70000  /**< Block size is 248 bytes */
} e_UccGethRxVirtFifoBlockSize;


/**************************************************************************//**
 @Function      UCC_GETH_ConfigRxVirtFifoBlockSize

 @Description   Configures the DMA block size of Rx virtual FIFO.
                [e_UCC_GETH_RX_VIRT_FIFO_BLOCK_SIZE_128_BYTES]
 
 @Param[in]     h_UccGeth           - Handle to UCC GETH object.
 @Param[in]     virtFifoBlockSize   - Rx virtual FIFO block size.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigRxVirtFifoBlockSize(t_Handle                     h_UccGeth,
                                           e_UccGethRxVirtFifoBlockSize virtFifoBlockSize);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxVirtFifoSize

 @Description   Configures the Tx virtual FIFO size in bytes, allocated in the
                multi-user RAM.
                [For under 1000BT: 512,
                 For 1000BT: 2048]

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     size        - Tx virtual FIFO size.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxVirtFifoSize(t_Handle h_UccGeth, uint16_t size);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxVirtFifoEmergThresh

 @Description   Configures the Tx virtual FIFO emergency threshold.
                [For under 1000BT: 256,
                 For 1000BT: 1024]
 
                When the number of bytes in the virtual FIFO drops below this
                threshold, the UCC transmitter enters emergency state.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Tx virtual FIFO emergency threshold.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxVirtFifoEmergThresh(t_Handle h_UccGeth, uint16_t threshold);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxVirtFifoTxThresh

 @Description   Configures the Tx virtual FIFO transmit threshold.
                [For under 1000BT: 512,
                 For 1000BT: 2048]
 
                This threshold represents the amount of bytes that should be in
                the virtual FIFO before transmission starts (transmission will
                start before reaching that watermark only if the entire frame is
                in the virtual FIFO).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     threshold   - Tx virtual FIFO transmit threshold.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxVirtFifoTxThresh(t_Handle h_UccGeth, uint16_t threshold);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTxPollingTimer

 @Description   Configures the transmitter polling timer. [256]
 
                Use this routine to program the amount of time in serial clocks
                between consecutive polls of an empty Tx BD. The minimum value
                allowed is 64 clocks.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     clocks      - Number of serial clocks between consecutive
                              polls of the Tx BD.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().

 
 @Cautions      The polling timer has great influence on the entire QUICC Engine,
                therefore it is recommended to keep the default value.
*//***************************************************************************/
t_Error UCC_GETH_ConfigTxPollingTimer(t_Handle h_UccGeth, uint16_t clocks);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigDiagnosticMode

 @Description   Configures the diagnostic mode. [e_UCC_FAST_DIAGNOSTIC_NORMAL]
 
                Use this routine to set the UCC Ethernet in loopback mode, echo
                mode or both.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     diagMode    - Diagnostic mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigDiagnosticMode(t_Handle                          h_UccGeth,
                                      e_UccGethGUMRDiagnosticModeDIAG   diagMode);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigTSA

 @Description   Configures the UCC Ethernet to connect to the Time-Slot Assigner
                (TSA) instead of the normal pins. [FALSE]
 
                The TSA allows the UCC Ethernet to multiplex data on the TDM
                serial interface.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to connect the UCC Ethernet to the TSA.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigTSA(t_Handle h_UccGeth, bool enabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigSchedulerMode

 @Description   Configures the scheduler operation mode.
                [TRUE]
 
                If the application requires any of the following, then the scheduler must be enabled:
                -More than one Tx queue for any of the logical ports.
                -Rate and burst shaping for any of the logical ports.
                Otherwise, the scheduler can be disabled in order to reduce memory
                consumption and to optimize the transmitter operation.
 
 @Param[in]     h_UccGeth           - Handle to UCC GETH object.
 @Param[in]     schedulerEnabled    - TRUE to enable the scheduler.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigSchedulerMode(t_Handle h_UccGeth, bool schedulerEnabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigShapingPerQueue

 @Description   Configures the Shaping-Per-Queue functionality.
                [FALSE]
 
                If the application requires various rate and burst shaping
                for different Tx queues on the same logical port,
                then the Shaping-Per-Queue functionality must be enabled for that port.
                Otherwise, the Shaping-Per-Queue functionality can be disabled for that port in order
                to reduce memory consumption and to optimize the transmitter operation.
 
 @Param[in]     h_UccGeth              - Handle to UCC GETH object.
 @Param[in]     logicalPortId          - Logical port ID.
 @Param[in]     shapingPerQueueEnabled - TRUE to enable the Shaping-Per-Queue functionality.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigShapingPerQueue(t_Handle h_UccGeth, uint8_t logicalPortId, bool shapingPerQueueEnabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigIpFragmentation

 @Description   Configures the IP fragmentation mode.
                [FALSE]
 
                When this mode is enabled, transmitted IP packets that exceed
                the configured IP MTU size are fragmented by the firmware.
 
                IP MTU values can be modified using the UCC_GETH_ModifyIpMtu()
                routine. The default IP MTU value is 576 bytes.
 
 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     ipFragEnabled   - TRUE to enable IP fragmentation in Tx.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIpFragmentation(t_Handle h_UccGeth, bool ipFragEnabled);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumOfLogicalTxPorts

 @Description   Configures logical Tx ports.
                [Default number of logical ports: 1]
 
                The transmitter can schedule up to eight logical ports, each
                having its dedicated bandwidth. The logical ports are scheduled
                using a WRR algorithm.
 
                Each logical port can have up to eight transmit queues, which
                are scheduled by a dedicated scheduler/shaper unit.
 
                The user should configure the actual number of Tx queues
                for each of the logical Tx ports.
 
                Note that when multiple logical Tx ports are used, the indexing
                method of Tx queues is as follows:
 
                - The 8 Tx queues of logical Tx port 0 are indexed 0 .. 7.
                - The 8 Tx queues of logical Tx port 1 are indexed 8 ..15.
                - The 8 Tx queues of logical Tx port 2 are indexed 16..23.
                - ...
                - The 8 Tx queues of logical Tx port 7 are indexed 56..63.
 
                This indexing method is relevant for every UCC GETH function
                that expects a Tx queue index as one of its parameters.
 
 @Param[in]     h_UccGeth                   - Handle to UCC GETH object.
 @Param[in]     numOfLogicalTxPorts         - Number of logical Tx ports.
 @Param[in]     p_QueuesInLogicalTxPorts    - Number of Tx queues for each
                                              logical port (array of size
                                              numOfLogicalTxPorts).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      When this routine configures multiple logical Tx ports, the
                driver ignores the number of Tx queues that may have been set
                using the UCC_GETH_ConfigNumQueuesTx() routine.
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumOfLogicalTxPorts(t_Handle h_UccGeth,
                                           uint8_t  numOfLogicalTxPorts,
                                           uint8_t  *p_QueuesInLogicalTxPorts);


/**************************************************************************//**
 @Function      UCC_GETH_ConfigNumOfPrefetchedTxBDs

 @Description   Configures the number of prefetched BDs (per Tx queue).
                [4]
 
                This routine is only relevant when working with multiple
                logical Tx ports (refer to UCC_GETH_ConfigNumOfLogicalTxPorts()).
                It configures how many BDs are prefetched by the transmitter.
                A higher number increases performance but occupies more memory
                space in the MURAM.
 
 @Param[in]     h_UccGeth               - Handle to UCC GETH object.
 @Param[in]     numOfPrefetchedTxBDs    - Number of prefetched Tx BDs
                                          (valid values: 1,2 or 4).
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigNumOfPrefetchedTxBDs(t_Handle    h_UccGeth,
                                            uint8_t     numOfPrefetchedTxBDs);

#ifdef UCC_GETH_CACHE_STASHING_SUPPORT
/**************************************************************************//**
 @Function      UCC_GETH_ConfigCacheStashing

 @Description   Configures L2 cache stashing mode.
                [FALSE]
 
                For performance improvement, the Ethernet receiver can transfer
                the first data block (Virtual FIFO block size) of each incoming
                frame to the L2 cache, thereby enabling the CPU to read the
                header of each frame without refering to the external memory.
 
 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     enabled     - TRUE to enable L2 cache stashing; FALSE to disable.
 
 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigCacheStashing(t_Handle h_UccGeth, bool enabled);
#endif /* UCC_GETH_CACHE_STASHING_SUPPORT */

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIwEnabled

 @Description   Configures the IW communication mode for this UCC GETH.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     iwCommMode  - IW communication mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIwMode(t_Handle h_UccGeth, e_CommMode iwCommMode);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIwTxQueuesNum

 @Description   Configures the number of IW transmit queues (IW mode only).

 @Param[in]     h_UccGeth           - Handle to UCC GETH object.
 @Param[in]     iwTxQueuesNum       - Number of IW transmit queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIwTxQueuesNum(t_Handle h_UccGeth, uint8_t iwTxQueuesNum);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIwCpuRxQueuesNum

 @Description   Configures the number of IW CPU receive queues (IW mode only).

 @Param[in]     h_UccGeth           - Handle to UCC GETH object.
 @Param[in]     iwCpuRxQueuesNum    - Number of IW CPU receive queues.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIwCpuRxQueuesNum(t_Handle h_UccGeth, uint8_t iwCpuRxQueuesNum);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigL3Filter

 @Description   Configures the L3 filter (IW mode only).

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     filterRootAddr  - Pointer to the L3 filter chain.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigL3Filter(t_Handle h_UccGeth, uint32_t filterRootAddr);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigIwExtendedThreadPage

 @Description   Extends the IW thread page for special functionalities
                (IW mode only).
                [FALSE]
 
 @Param[in]     h_UccGeth               - Handle to UCC GETH object.
 @Param[in]     iwExtendedThreadPage    - TRUE for extended IW thread page;
                                          FALSE otherwise.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConfigIwExtendedThreadPage(t_Handle    h_UccGeth,
                                            bool        iwExtendedThreadPage);

/**************************************************************************//**
 @Function      UCC_GETH_ConfigLogicalTxPortsIwQueuesNum

 @Description   Configures the number of IW queues in each logical Tx port.
 
                This routine is only relevant when using multiple logical Tx
                ports (refer to the UCC_GETH_ConfigNumOfLogicalTxPorts() routine).
 
                Each logical port can have up to eight transmit queues, and the
                user may configure some of them as IW queues. For example, if
                the user configures four IW queues, then the first four queues
                of that logical Tx port will be used for IW, and the other four
                queues will be used by the CPU.
 
                The user can configure zero as the number of IW queues for any
                of the logical Tx ports.
 
 @Param[in]     h_UccGeth                   - Handle to UCC GETH object.
 @Param[in]     numOfLogicalTxPorts         - Number of logical Tx ports to
                                              configure for IW; must not exceed
                                              the total number of logical Tx ports.
 @Param[in]     p_IwQueuesInLogicalTxPorts  - Number of IW queues for each
                                              logical port (array of size
                                              numOfLogicalTxPorts).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      When multiple logical Tx ports are configured, the driver
                ignores the number of IW Tx queues that may have been set
                using the UCC_GETH_ConfigIwTxQueuesNum() routine.
*//***************************************************************************/
t_Error UCC_GETH_ConfigLogicalTxPortsIwQueuesNum(t_Handle   h_UccGeth,
                                                 uint8_t    numOfLogicalTxPorts,
                                                 uint8_t    *p_IwQueuesInLogicalTxPorts);


#ifdef UCC_GETH_VP_MODE_SUPPORT
/**************************************************************************//**
 @Function      UCC_GETH_ConfigVpMode

 @Description   Configures VP Mode for UCC GETH Rx controller.
                [FALSE]
 
                UCC GETH Rx controller operating in VP mode transfers frames
                to Virtual Port (VP) after performing coarse classification
                based on RxQoS. All or part of Rx ETH queues are connected to
                VP queues in that mode, such that futher processing is performed
                by VP package based on VP queue PCD chain. In VP mode, UCC GETH
                object is aware of VP Port object, queues of which are used.
                VP queues utilized by GETH UCC object are configured and initialized
                internally by UCC GETH driver.
 
                VP mode is useful, e.g. for Burst Tolerance application where GETH
                port suffers from high rate burtst of short frames traffic. In GETH
                legacy mode, such scenario leads to Rx Virtual FIFO overflow and
                results in frames being discarded. Using VP mode for Burst Tolerance
                setup permits freeing Rx Virtual FIFOs at early stage that improves
                performance and allows to withstand burst traffic.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object
 @Param[in]     vpMode      - VP mode: TRUE - enable, FALSE - disable
 @Param[in]     queuesNum   - Number of UCC GETH Rx queues connected to VP queues
 @Param[in]     h_VpPort    - Handle to VP Port object that is used by this UCC GETH object

 @Return        E_OK on success; Error code otherwise

 @Cautions      Allowed only following UCC_GETH_Config() and before UCC_GETH_Init()
*//***************************************************************************/
t_Error UCC_GETH_ConfigVpMode(t_Handle  h_UccGeth,
                              bool      vpMode,
                              uint8_t   queuesNum,
					          t_Handle  h_VpPort);
	
	
/**************************************************************************//**
 @Function      UCC_GETH_ConfigVpModeQueueFilter

 @Description   Configures filter/PCD chain for UCC GETH Rx queue connected to VP queue.
 
                This function mast be issued for every Rx queue connected
                to VP in VP mode.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object
 @Param[in]     h_Filter        - Filter handle of PCD chain for this VP queue
 @Param[in]     queue           - VP queue number

 @Return        E_OK on success; Error code otherwise

 @Cautions      Valid only in VP mode.
                Allowed only following UCC_GETH_Config() and before UCC_GETH_Init()
*//***************************************************************************/
t_Error UCC_GETH_ConfigVpModeQueueFilter(t_Handle h_UccGeth, t_Handle h_Filter, uint8_t queue);



/**************************************************************************//**
 @Function      UCC_GETH_ConfigVpModeQueuePriority

 @Description   Configures priority mode and weight for UCC GETH Rx queue
                connected to VP queue.
                [e_VP_FIXED_PRIORITY_MODE]
 
                VP queues may be scheduled based on strict/fixed priority or
                weighted fair queueing (WFQ). In case of strict priority, queue
                with higher priority (lower index) is always scheduled unless
                it doesn't have data to send. In case of WFQ, queues are scheduled
                according to their respective weight.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object
 @Param[in]     mode            - Priority mode
 @Param[in]     weight          - Priority weight, not relevant when working
                                  in FIXED (i.e. strict priority) mode
 @Param[in]     queue           - VP queue number

 @Return        E_OK on success; Error code otherwise

 @Cautions      Valid only in VP mode.
                Allowed only following UCC_GETH_Config() and before UCC_GETH_Init()
*//***************************************************************************/
t_Error UCC_GETH_ConfigVpModeQueuePriority(t_Handle              h_UccGeth,
                                           e_VpQueuePriorityMode mode,
                                           uint16_t              weight,
                                           uint8_t               queue);


/**************************************************************************//**
 @Description   UCC GETH VP Mode CPU Swap Queues Configuration Parameters
 
                This structure is used when calling UCC_GETH_ConfigVpModeCpuSwapQueues()
                in VP Mode.
 *//***************************************************************************/
typedef struct t_UccGethVpModeCpuSwapQParam
{
    uint16_t intDescNum;                    /**< Number of VP CPU Swap queues with descriptor on internal memory */
	uint16_t extDescNum;                    /**< Number of VP CPU Swap queues with descriptor on external memory */
	uint16_t bdRingLen;                     /**< VP CPU Swap queue BD ring size in number of BDs */
	uint8_t  expWeightFactor;               /**< Exponential weight factor used by WRED algorithm for VP CPU Swap queue */
    uint8_t  eventQueue;                    /**< VP CPU Swap queue event queue index, up to 16 event queues
                                                 can be allocated in VP Common module */
} t_UccGethVpModeCpuSwapQParam;


/**************************************************************************//**
 @Function      UCC_GETH_ConfigVpModeCpuSwapQueues

 @Description   Configures VP CPU Swap queues to be used by UCC GETH object.
 
                VP CPU Swap queues implement efficient frame termination
                to CPU in VP package performed by swapping BD buffers without
                need in coping data. UCC GETH object operating in VP mode
                and wishing to receive traffic through VP CPU Swap queues
                needs to allocate them from VP package by invoking this
                function. The allocated queues will be reserved for specific
                objects (UCC GETH in that case) and cannot be used by other
                modules.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object
 @Param[in]     p_Params    - Configuration parameters for CPU Swap queues

 @Return        E_OK on success; Error code otherwise

 @Cautions      Valid only in VP mode.
                Allowed only following UCC_GETH_Config() and before UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ConfigVpModeCpuSwapQueues(t_Handle h_UccGeth,
								           t_UccGethVpModeCpuSwapQParam *p_Params);

#endif /* UCC_GETH_VP_MODE_SUPPORT */


/** @} */ /* end of geth_advanced_init_grp */

/** @} */ /* end of geth_init_grp */


/**************************************************************************//**
 @Group         geth_runtime_grp UCC GETH Runtime Unit

 @Description   UCC GETH Runtime Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        geth_txerbm
 
 @Collection    UCC GETH Transmit Errors Bit Mask
 
                Transmitter errors returned via the f_TxConf callback routine.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define UCC_GETH_TX_ERRORS_DEF      0x0200  /**< Deferred frame - if UCC_GETH_ConfigExcessDefer()
                                                 was called with TRUE, this frame was sent late because
                                                 of deferring; otherwise, this frame was aborted. */
#define UCC_GETH_TX_ERRORS_EXDEF    0x0100  /**< Excessive deferring has occurred in this frame. */
#define UCC_GETH_TX_ERRORS_LC       0x0080  /**< Late collision - a collision occurred after 64 bytes
                                                 were sent; the transmission was terminated. */
#define UCC_GETH_TX_ERRORS_RL       0x0040  /**< Retransmission limit - the transmitter failed
                                                 (max. retry limit + 1) attempts to successfully send
                                                 the frame due to repeated collisions. */
#define UCC_GETH_TX_ERRORS_RC_MASK  0x003C  /**< Retransmission count mask - number of attempts that
                                                 were needed to send the frame; this is a mask for
                                                 extracting a 4-bit field - must rightshift it by
                                                 UCC_GETH_TX_ERRORS_RC_SHIFT */
#define UCC_GETH_TX_ERRORS_RC_SHIFT 2       /**< Shift for retransmission count mask - see
                                                 UCC_GETH_TX_ERRORS_RC_MASK */
#define UCC_GETH_TX_ERRORS_UN       0x0002  /**< Underrun; the transmission was terminated. */
#define UCC_GETH_TX_ERRORS_CSL      0x0001  /**< Carrier sense lost. */
 
/* @} */

/**************************************************************************//**
 @Anchor        geth_rxerbm
 
 @Collection    UCC GETH Receive Errors Bit Mask
 
                Receiver errors returned via the f_RxStore callback routine.
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 
                All values, excluding UCC_GETH_RX_STATUS_IPCH, are valid only
                on the last buffer of a frame.
 @{
*//***************************************************************************/
#define UCC_GETH_RX_STATUS_TR   0x2000  /**< Truncation - set if the frame was truncated
                                             according to MAXD2/MAXD3 value (only possible when
                                             UCC_GETH_RX_STATUS_M is set). */
#ifdef UCC_GETH_PTP_SUPPORT
#define UCC_GETH_RX_STATUS_PTP  0x0200  /**< PTP (IEEE 1588) frame - valid only if PTP
                                             mode was enabled. */
#endif /* UCC_GETH_PTP_SUPPORT */
#define UCC_GETH_RX_STATUS_BDM0 0x0200  /**< Set when BDM0 bit in LAST PCD is set (valid only
                                             when PTP is not enabled) */
#define UCC_GETH_RX_STATUS_M    0x0100  /**< Miss (valid in extended filtering mode or when
                                             address recognition mode and promiscuous mode are
                                             used together) - frame was accepted in promiscuous
                                             mode, but was flagged as a 'miss' by the internal
                                             address recognition, or frame was sent to CPU after
                                             a lookup miss in extended filtering mode. */
#define UCC_GETH_RX_STATUS_BC   0x0080  /**< Broadcast - set if destination address matches the
                                             Ethernet broadcast address. */
#define UCC_GETH_RX_STATUS_MC   0x0040  /**< Multicast - set if destination address is multicast
                                             (including broadcast). */
#define UCC_GETH_RX_STATUS_LG   0x0020  /**< Frame length violation - set if the frame length
                                             exceeds the configured maximum frame length. */
#define UCC_GETH_RX_STATUS_NO   0x0010  /**< Non-octet aligned frame - set if the frame length
                                             (in bits) is not divisible by eight. */
#define UCC_GETH_RX_STATUS_SH   0x0008  /**< Short frame - set if the frame length is less than
                                             the configured minimum frame length. */
#define UCC_GETH_RX_STATUS_CR   0x0004  /**< CRC error */
#define UCC_GETH_RX_STATUS_OV   0x0002  /**< Overrun - set if a receive FIFO overrun occurred
                                             during frame reception; if this bit is set, the other
                                             status bits (M, LG, NO, SH, CR) lose their normal
                                             meaning and should be ignored. */
#define UCC_GETH_RX_STATUS_IPCH 0x0001  /**< IP checksum check failed - set if the IP checksum
                                             check on IPv4 header failed. */
/* @} */

/**************************************************************************//**
 @Collection    UCC GETH Receive Flags
 
                Receive flags passed via the f_RxStore callback routine.
 
 @{
*//***************************************************************************/

#define UCC_GETH_RX_FLAG_CPU_QUEUE          0x00000000  /**< Buffer is received in a regular ETH
                                                             Rx queue, queue ID passed with
                                                             f_RxStore indicates ETH Rx queue number */

#ifdef UCC_GETH_VP_MODE_SUPPORT
#define UCC_GETH_RX_FLAG_CPU_SWAP_QUEUE     0x00000001  /**< Buffer is received in CPU Swap queue -
                                                             queue ID passed with f_RxStore indicates
                                                             CPU Swap queue number.
 
                                                             Internal queue numbers are reported as they are
                                                             while external queue numbers are reported with
                                                             offset of total number of ETH internal swap queues,
                                                             e.g.: if 2 Swap queues with internal queue descriptors
                                                             and 4 Swap queues with external queue descriptors were
                                                             configured by UCC_GETH_ConfigVpModeCpuSwapQueues() function,
                                                             then queue IDs 0...1 indicate internal swap queues and
                                                             queue IDs 2...5 indicate external swap queues */
#endif  /* UCC_GETH_VP_MODE_SUPPORT */

/* @} */

#ifdef UCC_GETH_VP_MODE_SUPPORT
#define UCC_GETH_VP_MODE_SWAP_QUEUE_POLLING_ID  0xFF    /**< In VP mode when using CPU swap queues in polling
                                                             mode this queue ID should be passed to UCC_GETH_Rx()
                                                             to indicate that Rx should be performed from swap
                                                             queue (as opposed to fast termination queue).
                                                             Note that in swap queue case, polling is performed
                                                             on swap event queue and not on swap queue BD ring,
                                                             thus all swap queues sharing the same event
                                                             queue are polled simultaneously. */
#endif  /* UCC_GETH_VP_MODE_SUPPORT */


/**************************************************************************//**
 @Description   UCC GETH Tx BD Flags
 
                These flags can be passed to the transmit routines for
                performing special operations on the transmitted frame.
 
                The different types of flags can be combined to one value.
*//***************************************************************************/
typedef enum
{
    e_UCC_GETH_TX_BD_FLAGS_PAD_AND_CRC            = 0x4000 /**< Pad short frames and add CRC - identical to
                                                                calling UCC_GETH_ConfigPadAndCrcMode() with
                                                                e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_PAD_AND_CRC,
                                                                only on a per transmitted frame basis; ignored
                                                                on all but last buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_CRC_ONLY               = 0x0400 /**< Add CRC - identical to calling
                                                                UCC_GETH_ConfigPadAndCrcMode() with
                                                                e_UCC_GETH_MACCFG2_PAD_AND_CRC_MODE_CRC_ONLY,
                                                                only on a per transmitted frame basis; ignored
                                                                on all but last buffer of the frame, or if the
                                                                e_UCC_GETH_TX_BD_FLAGS_PAD_CRC is used */
  , e_UCC_GETH_TX_BD_FLAGS_PROGRAMMABLE_PREAMBLE  = 0x0100 /**< Programmable preamble - see details in the
                                                                UCC_GETH_ConfigSoftPreambleTx() routine;
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_PTP                    = 0x0080 /**< PTP frame - generate a time stamp on transmission;
                                                                illegal on all but first buffer of the frame. */
 
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_0      = 0x0020 /**< Insert VLAN tag from entry 0 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_1      = 0x0024 /**< Insert VLAN tag from entry 1 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_2      = 0x0028 /**< Insert VLAN tag from entry 2 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_3      = 0x002c /**< Insert VLAN tag from entry 3 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_4      = 0x0030 /**< Insert VLAN tag from entry 4 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_5      = 0x0034 /**< Insert VLAN tag from entry 5 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_6      = 0x0038 /**< Insert VLAN tag from entry 6 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_INSERT_VLAN_TAG_7      = 0x003c /**< Insert VLAN tag from entry 7 in Tx VLAN tag table;
                                                                table is defined using UCC_GETH_ConfigTxVlanTable();
                                                                illegal on all but first buffer of the frame. */
 
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_0     = 0x0000 /**< Generate IP checksum using entry 0 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_1     = 0x0001 /**< Generate IP checksum using entry 1 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_2     = 0x0002 /**< Generate IP checksum using entry 2 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_3     = 0x0003 /**< Generate IP checksum using entry 3 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_4     = 0x0040 /**< Generate IP checksum using entry 4 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_5     = 0x0041 /**< Generate IP checksum using entry 5 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_6     = 0x0042 /**< Generate IP checksum using entry 6 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
  , e_UCC_GETH_TX_BD_FLAGS_IP_HEADER_OFFSET_7     = 0x0043 /**< Generate IP checksum using entry 7 in IP header offset table;
                                                                table is defined using UCC_GETH_ConfigTxChecksumOffsetTable();
                                                                ignored on all but first buffer of the frame. */
} e_UccGethTxBdFlags;


/**************************************************************************//**
 @Function      UCC_GETH_Tx

 @Description   Transmits a data buffer on one of the UCC Ethernet transmit queues.

                Accepts data as a pointer to a single data buffer (array of
                bytes).

                This routine eventually causes the user's transmit confirmation
                callback routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_GETH_TxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     txQ             - Tx queue on which to transmit.
 @Param[in]     p_Data          - Pointer to data to transmit.
 @Param[in]     h_BufContext    - User's context for the given data buffer.
 @Param[in]     length          - Length of data to transmit.
 @Param[in]     lastBuffer      - Indicates whether or not the referred buffer
                                  is the last one in the frame.
 @Param[in]     flags           - Additional transmit mode flags.
                                  Any or'ed combination of the elements of the
                                  e_UccGethTxBdFlags enumeration.
                                  Use zero for standard operation.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Read the e_UccGethTxBdFlags enumeration documentation
                for cautions concerning the flags parameter. Failure to adhere
                to the cautions may lead to unpredictable transmitter behavior.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                UCC_GETH_TxConf()) simultaneously at task level and at interrupt
                level (for example, inside the Rx user callback routine) may lead
                to unpredictable results.
*//***************************************************************************/
t_Error UCC_GETH_Tx(t_Handle    h_UccGeth,
                    uint8_t     txQ,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint16_t    length,
                    bool        lastBuffer,
                    uint16_t    flags);

#ifdef UCC_GETH_VP_MODE_SUPPORT
/**************************************************************************//**
 @Function      UCC_GETH_Rx

 @Description   Handles data buffers received in the specified receive queue.

                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific queue.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.
 
                When operating UCC GETH in VP mode and using CPU swap queues
                in polling mode, #UCC_GETH_VP_MODE_SWAP_QUEUE_POLLING_ID const
                should be used for rxQ to indicate reception from swap queue.
                Since polling of swap queues are performed on swap event
                queue and not on swap queue BD ring, no real indication for
                swap queue number is required and all swap queues sharing the
                same event queue are polled simultaneously.
 
                It is recommended to use different swap event queuues for
                different UCC GETH ports when operating in polling mode
                (see t_UccGethVpModeCpuSwapQParam struct passed to
                UCC_GETH_ConfigVpModeCpuSwapQueues() function), so that
                UCC_GETH_Rx() function invocation with h_UccGeth handle specified
                would poll only swap queues of the relevant UCC GETH port
                represented by this handle.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     rxQ         - Rx queue on which to receive.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
#else   /* UCC_GETH_VP_MODE_SUPPORT */
/**************************************************************************//**
 @Function      UCC_GETH_Rx

 @Description   Handles data buffers received in the specified receive queue.

                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific queue.

                Must be invoked often enough to keep BD ring from overflowing
                with incoming receive data. How it is invoked and by whom
                depends on the receive mode. In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     rxQ         - Rx queue on which to receive.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
#endif  /* UCC_GETH_VP_MODE_SUPPORT */
t_Error UCC_GETH_Rx(t_Handle h_UccGeth, uint8_t rxQ);

/** @} */ /* end of geth_runtime_grp */


/**************************************************************************//**
 @Group         geth_control_grp UCC GETH Control Unit

 @Description   UCC GETH Control Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_GETH_Enable

 @Description   Enables transmit and/or receive communication on the UCC Ethernet.

                Communication can be disabled using UCC_GETH_Disable().

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     mode        - Communication mode (Tx, Rx, or both).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_Enable(t_Handle h_UccGeth, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_GETH_Disable

 @Description   Disables transmit and/or receive communication on the UCC Ethernet.

                Channel can be re-enabled using UCC_GETH_Enable().

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     mode        - Communication mode (Tx, Rx, or both).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config(),
                initialized using UCC_GETH_Init(), and enabled using
                UCC_GETH_Enable().
*//***************************************************************************/
t_Error UCC_GETH_Disable(t_Handle h_UccGeth, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_GETH_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
 
                Invokes user's transmit confirmation callback routine for
                previously transmitted data on one of the transmit queues.

                Normally, this functionality is performed in the transmit
                routines for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     txQ         - Tx queue.

 @Return        None.

 @Cautions      This routine is not interrupt safe, meaning using it (and/or
                the transmit routines) simultaneously at task level and at
                interrupt level may lead to unpredictable results.
*//***************************************************************************/
void UCC_GETH_TxConf(t_Handle h_UccGeth, uint8_t txQ);


/**************************************************************************//**
 @Function      UCC_GETH_AdjustLink

 @Description   Adjusts the Ethernet link with new speed/duplex setup.
 
                This function should be issued by application upon
                change in phy link synchronization speed and/or duplex mode.
                It adjust ETH MAC layer configuration according to the new phy
                link parameters.
 
 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     speed       - Ethernet speed.
 @Param[in]     fullDuplex  - TRUE for Full-Duplex mode;
                              FALSE for Half-Duplex mode.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_AdjustLink(t_Handle h_UccGeth, e_EnetSpeed speed, bool fullDuplex);

/**************************************************************************//**
 @Function      UCC_GETH_82xxFilteringHashEntryAdd

 @Description   Adds an Ethernet address to the address recognition hash.
 
                If the incoming frames address does not match any of the
                station addresses (and it is not a broadcast address), the
                controller can perform address recognition using a hash table
                of Ethernet addresses.
 
                The hash table is maintained by both the controller and the
                driver. The controller performs a a partial match of the
                incoming address according to some CRC calculations on the address.
                The driver completes the filtering by performing an exact match
                against all addresses that were added by the user and match that
                hash bin. This process ensures that only frames with desired
                destination address are accepted.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     p_EnetAddr  - Pointer to the address to add.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Has effect only in MPC82xx address recognition mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_82xxFilteringHashEntryAdd(t_Handle h_UccGeth, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      UCC_GETH_82xxFilteringHashEntryClear

 @Description   Clears an Ethernet address from the address recognition hash.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     p_EnetAddr  - Pointer to the address to clear.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Has effect only in MPC82xx address recognition mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_82xxFilteringHashEntryClear(t_Handle h_UccGeth, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      UCC_GETH_82xxFilteringHashEntryClearAll

 @Description   Completely clears all individual or group Ethernet addresses
                from the address recognition hash.

                The user selects which address type to clear from the hash
                (group addresses or individual addresses).

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     enetAddrType    - Which hash to clear.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Has effect only in MPC82xx address recognition mode.

 @Cautions      This routine disables the UCC Ethernet, clears the hash, and
                then re-enables the UCC Ethernet, therefore it is likely to
                be slow.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_82xxFilteringHashEntryClearAll(t_Handle h_UccGeth, e_EnetAddrType enetAddrType);

/**************************************************************************//**
 @Function      UCC_GETH_StationAddressAdd

 @Description   Adds an Ethernet address to the address recognition registers.
 
                The user can add up to four additional addresses that shall be
                accepted by the Ethernet receiver. Those addresses can be added
                only after UCC GETH module was initialized, and can be modified
                at runtime (although the frames filtering results are
                unpredictable during the change).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     p_EnetAddr  - Pointer to the address to add.

 @Retval        E_OK                - Address was added.
 @Retval        E_INVALID_VALUE     - invalid Ethernet address such as multicast
                                      address, which has no effect in these registers.
 @Retval        E_ALREADY_EXISTS    - Address is already in one of the station
                                      addresses, therefore this request is ignored.
 @Retval        E_FULL              - All station address registers are in use.
                                      Must first clear at least one of the station address registers,
                                      using either UCC_GETH_StationAddressClear() or
                                      UCC_GETH_StationAddressClearAll().

 @Cautions      Has no effect unless UCC_GETH_ConfigMultipleStationAddresses()
                has been called with TRUE.

 @Cautions      Has effect only in MPC82xx address recognition mode or in IW mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_StationAddressAdd(t_Handle h_UccGeth, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      UCC_GETH_StationAddressClear

 @Description   Clears an address from the address recognition registers.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     p_EnetAddr  - Pointer to the address to be cleared.

 @Retval        E_OK        - Address was cleared.
 @Retval        E_NOT_FOUND - Address not found among station address registers.
                              No action taken.

 @Cautions      Has no effect unless UCC_GETH_ConfigMultipleStationAddresses()
                has been called with TRUE.

 @Cautions      Has effect only in MPC82xx address recognition mode or in IW mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_StationAddressClear(t_Handle h_UccGeth, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      UCC_GETH_StationAddressClearAll

 @Description   Clears all addresses from the address recognition registers.

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        None.

 @Cautions      Has no effect unless UCC_GETH_ConfigMultipleStationAddresses()
                has been called with TRUE.

 @Cautions      Has effect only in MPC82xx address recognition mode or in IW mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_StationAddressClearAll(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_StationAddressGetAll

 @Description   Gets currently used addresses from the address recognition registers.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[out]    p_EnetAddr  - Pointer to array of Ethernet addresses (t_EnetAddr)
                              into which to place all addresses found in address
                              recognition registers.
                              Must be of sufficient size to hold as many as 4 entries.

 @Return        Number of addresses found and returned.

 @Cautions      Returns meaningless values unless UCC_GETH_ConfigMultipleStationAddresses()
                has been called with TRUE.

 @Cautions      Has effect only in MPC82xx address recognition mode or in IW mode.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
uint8_t UCC_GETH_StationAddressGetAll(t_Handle h_UccGeth, t_EnetAddr *p_EnetAddr);

/**************************************************************************//**
 @Function      UCC_GETH_ModifyVPriBasedEarlyDrop

 @Description   Enable, disable or tune the Rx virtual FIFO VPri-based
                early drop mechanism for the Ethernet port.
 
                Each bit in the VPri drop mask (vlanPriorityDropMask parameter)
                controls whether a frame which contains the corresponding VPri
                value should be dropped in case of Rx Virtual FIFO emergency
                state. Bit 0 of this field corresponds to Vpri=000, bit 1
                corresponds to VPri=001 and so on.

                Encoding of each bit:
                - 0 - Do not drop frames containing the corresponding VPri value.
                - 1 - Drop frames containing the corresponding VPri value.
 
                A zero mask disables the VPri-based early drop mechanism.
                This is also the initial state of the early drop mechanism.
 
                In case that the incoming frame does not contain a VLAN tag, its
                VPri value is determined by

 @Param[in]     h_UccGeth               - Handle to UCC GETH object.
 @Param[in]     vlanPriorityDropMask    - VPri mask for emergency early drop;

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ModifyVPriBasedEarlyDrop(t_Handle  h_UccGeth,
                                          uint8_t   vlanPriorityDropMask);

/**************************************************************************//**
 @Function      UCC_GETH_TxFlowControlStart

 @Description   Sends a PAUSE flow control frame on the line, telling other
                controllers to stop transmitting for the specified time period.
 
                The hardware automatically transmits additional PAUSE frames
                when the pause period has expired, until the user stops the
                Flow Control transmission using UCC_GETH_TxFlowControlStop().

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     pausePeriod     - Requested pause time, in units dependent on
                                  line speed as follows:
                                  - For 10BT: 51.2 microseconds.
                                  - For 100BT: 5.12 microseconds.
                                  - For 1000BT: 4.096 microseconds.
 @Param[in]     extensionField  - Reserved for future extensions of flow
                                  control frames; must be zero.

 @Return        None.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_TxFlowControlStart(t_Handle h_UccGeth, uint16_t pausePeriod, uint16_t extensionField);

/**************************************************************************//**
 @Function      UCC_GETH_TxFlowControlStop

 @Description   Cancels a previous PAUSE flow control request.

                If UCC_GETH_TxFlowControlStart() was used previously and its
                associated pause time has not yet expired, this function can be
                used to announce to other controllers that it is permitted to
                resume transmission.

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        None.


 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_TxFlowControlStop(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_TxFlowControlParametersSet

 @Description   Modifies the parameters of PAUSE flow control frames.
 
                This routine affects both hardware- and software-initiated
                flow control.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     pausePeriod     - Requested pause time, in units dependent on
                                  line speed as follows.
                                  - For 10BT: 51.2 microseconds.
                                  - For 100BT: 5.12 microseconds.
                                  - For 1000BT: 4.096 microseconds.
 @Param[in]     extensionField  - Reserved for future extensions of flow
                                  control frames; must be zero.

 @Return        None.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_TxFlowControlParametersSet(t_Handle   h_UccGeth,
                                         uint16_t   pausePeriod,
                                         uint16_t   extensionField);

/**************************************************************************//**
 @Function      UCC_GETH_MagicPacketDetectionEnable

 @Description   Enables magic packet detection.

                This mode is useful for low-power systems. After this routine
                is called, no additional frames are received until a magic packet
                frame is detected. When such a frame arrives, the CPU is awakened
                from its low power state.
 
                This mode is disabled automatically when a magic packet arrives,
                or by the user, through UCC_GETH_MagicPacketDetectionDisable().
                The UCC Ethernet controller shall resume its previous state
                (before the magic packet detection mode was entered).

 @Param[in]     h_UccGeth  - Handle to UCC GETH object.

 @Return        None.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_MagicPacketDetectionEnable(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_MagicPacketDetectionDisable

 @Description   Disables magic packet detection.
 
                The UCC Ethernet controller resumes its previous state (before
                the magic packet detection mode was entered).

 @Param[in]     h_UccGeth  - Handle to UCC GETH object.

 @Return        None.


 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_MagicPacketDetectionDisable(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_TransmitOnDemand

 @Description   Immediately forces a poll of the transmitter for data to be sent.

                Typically, the hardware performs a periodic poll for data that the
                transmit routine has set up to be transmitted. In cases where
                this polling cycle is not fast enough, this optional routine can
                be invoked to force a poll right away, instead. Proper use for
                each transmission for which this functionality is desired is to
                call the transmit routine and then this routine right after.

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        None.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
 *//***************************************************************************/
void UCC_GETH_TransmitOnDemand(t_Handle h_UccGeth);


/**************************************************************************//**
 @Function      UCC_GETH_GetRxNonDynamicExtendedFeaturesMode

 @Description   Determine whether any rx non-dynamic extended features are active.
 
                Result should be placed in rxNonDynamicExtendedFeaturesMode
                field of user-provided TAD structure.
 
                Non-dynamic means those features are not specified in TAD.

 @Param[in]     h_UccGeth                           - Handle to UCC GETH object.
 @Param[out]    rxNonDynamicExtendedFeaturesMode    - Whether any non-dynamic
                                                      Rx Extended Features are selected.
 @Return        void

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_GetRxNonDynamicExtendedFeaturesMode(t_Handle h_UccGeth, bool *rxNonDynamicExtendedFeaturesMode);


/**************************************************************************//**
 @Description   Tx firmware statistics structure, used when calling UCC_GETH_GetStatistics().
 *//***************************************************************************/
typedef struct
{
    uint32_t  siColTx;            /**< Single collision */
    uint32_t  mulColTx;           /**< Multiple collision */
    uint32_t  lateColTxFr;        /**< Late collision */
    uint32_t  frAbortDueCol;      /**< Frames aborted due to transmit collision */
    uint32_t  frLostInMACTxEr;    /**< Frames lost due to internal MAC error transmission that are not counted on any other counter */
    uint32_t  carrierSenseERTx;   /**< Carrier sense error */
    uint32_t  frTxOK;             /**< Frames transmitted OK */
    uint32_t  txFrExcessiveDefer; /**< Frames with deferral time greater than specified threshold */
    uint32_t  txPkts256;          /**< Total packets (including bad) between 256  and 511       octets */
    uint32_t  txPkts512;          /**< Total packets (including bad) between 512  and 1023      octets */
    uint32_t  txPkts1024;         /**< Total packets (including bad) between 1024 and 1518      octets */
    uint32_t  txPktsJumbo;        /**< Total packets (including bad) between 1024 and MAXLength octets */
} t_UccGethTxFirmwareStatistics;

/**************************************************************************//**
 @Description   GETH Rx firmware statistics structure, used when calling UCC_GETH_GetStatistics().
 *//***************************************************************************/
typedef struct
{
    uint32_t   frRxFCSEr;          /**< Frames with crc error */
    uint32_t   frAlignEr;          /**< Frames with alignment error */
    uint32_t   inRangeLenRxER;     /**< In range length error */
    uint32_t   outRangeLenRxER;    /**< Out of range length error */
    uint32_t   frTooLong;          /**< Frame too long */
    uint32_t   runt;               /**< Runt */
    uint32_t   veryLongEvent;      /**< Very long event */
    uint32_t   symbolError;        /**< Symbol error */
    uint32_t   dropBsy;            /**< Drop because of BD not ready */
    uint32_t   RESERVED00;
    uint32_t   RESERVED01;
    uint32_t   mismatchDrop;       /**< Drop because of MAC filtering (e.g. address or type mismatch) */
    uint32_t   underPkts;          /**< Total frames less than 64 octets */
    uint32_t   pkts256;            /**< Total frames (including bad) between 256 and 511 octets */
    uint32_t   pkts512;            /**< Total frames (including bad) between 512 and 1023 octets */
    uint32_t   pkts1024;           /**< Total frames (including bad) between 1024 and 1518 octets */
    uint32_t   pktsJumbo;          /**< Total frames (including bad) between 1024 and MAXLength octets */
    uint32_t   frLossInMACEr;      /**< Frames lost because of internal MAC error that is not counted in any other counter */
    uint32_t   pauseFr;            /**< Pause frames */
    uint32_t   vPriEarlyDrop;      /**< Total frames dropped due to VPri based Early Drop mechanism. */
    uint32_t   removeVlan;         /**< Total frames that had their VLAN tag removed  */
    uint32_t   replaceVlan;        /**< Total frames that had their VLAN tag replaced */
    uint32_t   insertVlan;         /**< Total frames that had their VLAN tag inserted */
    uint32_t   ipCheckSumEr;       /**< Frames with erroneous IP checksum */
} t_UccGethRxFirmwareStatistics;

/**************************************************************************//**
 @Description   GETH hardware statistics structure, used when calling UCC_GETH_GetStatistics().
 *//***************************************************************************/
typedef struct
{
    uint32_t tx64;           /**< Total number of frames (including bad frames) transmitted that were exactly of the minimal length (64 for un tagged, 68 for tagged, or with length exactly equal to the parameter MINLength */
    uint32_t tx127;          /**< Total number of frames (including bad frames) transmitted that were between MINLength (Including FCS length==4) and 127 octets */
    uint32_t tx255;          /**< Total number of frames (including bad frames) transmitted that were between 128 (Including FCS length==4) and 255 octets */
    uint32_t rx64;           /**< Total number of frames received including bad frames that were exactly of the minimal length (64 bytes) */
    uint32_t rx127;          /**< Total number of frames (including bad frames) received that were between MINLength (Including FCS length==4) and 127 octets */
    uint32_t rx255;          /**< Total number of frames (including bad frames) received that were between 128 (Including FCS length==4) and 255 octets */
    uint32_t txok;           /**< Total number of octets residing in frames that where involved in successful transmission */
    uint16_t txcf;           /**< Total number of PAUSE control frames transmitted by this MAC */
    uint32_t tmca;           /**< Total number of frames that were transmitted successfully with the group address bit set that are not broadcast frames */
    uint32_t tbca;           /**< Total number of frames transmitted successfully that had destination address field equal to the broadcast address */
    uint32_t rxfok;          /**< Total number of frames received OK */
    uint32_t rxbok;          /**< Total number of octets received OK */
    uint32_t rbyt;           /**< Total number of octets received including octets in bad frames. Must be implemented in HW because it includes octets in frames that never even reach the UCC */
    uint32_t rmca;           /**< Total number of frames that were received successfully with the group address bit set that are not broadcast frames */
    uint32_t rbca;           /**< Total number of frames received successfully that had destination address equal to the broadcast address */
    uint32_t rovc;           /**< Total number of frames that were discarded due to Rx FIFO overrun */
} t_UccGethHardwareStatistics;

/**************************************************************************//**
 @Function      UCC_GETH_GetStatistics

 @Description   Gets values of hardware and/or firmware statistics counters.

 @Param[in]     h_UccGeth                       - Handle to UCC GETH object.
 @Param[in]     p_UccGethTxFirmwareStatistics   - Structure into which to put statistics.
 @Param[in]     p_UccGethRxFirmwareStatistics   - Structure into which to put statistics.
 @Param[in]     p_UccGethHardwareStatistics     - Structure into which to put statistics.

 @Return        None.


 @Cautions      Does not zero counters. For each of the three statistics types,
                does not return statistics unless driver has been previously
                configured to gather that type of statistics. For each of the
                three statistics types, does not return statistics if the
                provided structure pointer is NULL.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_GetStatistics(t_Handle                        h_UccGeth,
                            t_UccGethTxFirmwareStatistics   *p_UccGethTxFirmwareStatistics,
                            t_UccGethRxFirmwareStatistics   *p_UccGethRxFirmwareStatistics,
                            t_UccGethHardwareStatistics     *p_UccGethHardwareStatistics);

/**************************************************************************//**
 @Description   Tx firmware per queue statistics structure,
                used when calling UCC_GETH_GetPerQueueStatistics().
 *//***************************************************************************/
typedef struct
{
    uint32_t    frTxOK;           /**< Frames transmitted OK per Tx queue */
    uint32_t    octetsTxOK;       /**< Octets transmitted OK per Tx queue; this counter
                                       reflects bytes transmitted by the firmware and
                                       doesn't count for any additional bytes (e.g., CRC or
                                       padding) appended by the transmitter */
    uint32_t    frTxErr;          /**< Frames transmitted with error per Tx queue
                                       (internal MAC error, collisions, carrier
                                        sense error, excessive Tx defer) */
    uint32_t    txEnqueueBusy;    /**< Number of IW enqueue busy per Tx queue */
} t_UccGethTxFirmwarePerQueueStatistics;

/**************************************************************************//**
 @Function      UCC_GETH_GetPerQueueStatistics

 @Description   Gets values of Tx firmware per queue statistics counters.

 @Param[in]     h_UccGeth                       - Handle to UCC GETH object.
 @Param[in]     txQ                             - Tx queue index for that statistics are requested.
 @Param[in]     p_TxFirmwarePerQueueStatistics  - Structure into which to put statistics.

 @Return        None.


 @Cautions      Does not zero counters. Does not return statistics unless
                driver has been previously configured to gather that type
                of statistics. Does not return statistics if the provided
                structure pointer is NULL.

 @Cautions      Tx firmware per queue statistics availability depends on
                context of ucode package.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_GetPerQueueStatistics(t_Handle                                h_UccGeth,
                                    uint8_t                                 txQ,
                                    t_UccGethTxFirmwarePerQueueStatistics   *p_TxFirmwarePerQueueStatistics);

/**************************************************************************//**
 @Function      UCC_GETH_StopStatistics

 @Description   Stops the statistics gathering.
 
 				The user can select for which counters group to stop the statistics
 				gathering. The supplied statistics counter group should be an or'ed
 				combination of any, none, or all of the following:
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_TX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_RX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     statisticsMode  - Requested statistics counter group.
                                  Any or'ed combination of the elements of the
                                  e_UccGethStatisticsGatheringMode enumeration.

 @Return        None

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_StopStatistics(t_Handle                           h_UccGeth,
                             e_UccGethStatisticsGatheringMode   statisticsMode);

/**************************************************************************//**
 @Function      UCC_GETH_RestartStatistics

 @Description   Restarts the statistics gathering.
 
 				The user can select for which counters group to restart the
 				statistics gathering. The supplied statistics counter group should
 				be an or'ed combination of any, none, or all of the following:
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_TX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_RX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON
 
                Restarting the statistics gathering for RMON extended Tx or RMON
                extended Rx, is valid operation only if were configured as enabled
                prior to UCC_GETH_Init() call.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     statisticsMode  - Requested statistics counter group.
                                  Any or'ed combination of the elements of the
                                  e_UccGethStatisticsGatheringMode enumeration.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_RestartStatistics(t_Handle                         h_UccGeth,
                                   e_UccGethStatisticsGatheringMode statisticsMode);

/**************************************************************************//**
 @Function      UCC_GETH_ResetStatistics

 @Description   Clears the statistics counters.
 
 				The user can select which counters group to clear. The supplied
 				statistics counter group should be an or'ed combination of any,
 				none, or all of the following:
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_TX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON_EXTENDED_RX,
  				e_UCC_GETH_STATISTICS_GATHERING_MODE_RMON
 
                In order to safely clear the statistics counters for RMON extended
                Tx or RMON extended Rx, this routine disables the statistics
                gathering, clears the counters, and enables the statistics if were
                enabled prior to UCC_GETH_ResetStatistics() call.
 
                Reset the statistics counters for RMON extended Tx or RMON extended Rx,
                is valid operation only if statistics gathering was configured as
                enabled prior to UCC_GETH_Init() call.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     statisticsMode  - Requested statistics counter group.
                                  Any or'ed combination of the elements of the
                                  e_UccGethStatisticsGatheringMode enumeration.

 @Return        None

 @Cautions      Predictable results only when used after UCC_GETH_Init().
*//***************************************************************************/
void UCC_GETH_ResetStatistics(t_Handle                          h_UccGeth,
                              e_UccGethStatisticsGatheringMode  statisticsMode);


/**************************************************************************//**
 @Description   Tx queue service parameters.
 
                This structure is used with the UCC_GETH_SetQueuesService()
                routine. It contains the parameters for scheduling a Tx queue.
                The scheduler will service the queue according to these
                parameters and the matching parameters of other Tx queues in
                the same port.
*//***************************************************************************/
typedef struct t_UccGethQueueService
{
    bool    isWeighted;         /**< Indicates whether the scheduling for this
                                     queue is weighted in which case scheduling
                                     depends on the associated queues weight (WFQ);
                                     If set to FALSE, the scheduling will be done
                                     with a higher priority, and the queue will be
                                     granted transmission regardless of its weight (SP). */
    bool    scheduleAndTxAsap;  /**< Indicates whether this queue should be serviced
                                     for transmission immediately following scheduling,
                                     without stalling for average bit rate regulation
                                     (TRUE), or this queue might endure shaper stall
                                     for bit rate regulation (FALSE). */
    bool    isBwMonitored;      /**< Indicates whether this queue's bandwidth is
                                     monitored by the shaper; When set to TRUE, the
                                     shapers bandwidth mechanism will include the
                                     queue's bandwidth with the regulated bandwidth;
                                     When set to FALSE the queue's bandwidth will not
                                     be monitored (extra bandwidth enabled). */
    uint8_t bwPercentage;       /**< Indicates the relative bandwidth of this queue
                                     (in percentage); The user should make sure that
                                     the sum of all bandwidths is 100. */
} t_UccGethQueueService;


/**************************************************************************//**
 @Function      UCC_GETH_SetQueuesService

 @Description   Sets the scheduling parameters for the queues of a given
                (logical) Tx port.
 
                This routine determines the service method among the Tx queues
                of a given Tx port.
 
                Note that the index of the logical port must be in the range of
                logical ports that was previously configured using the
                UCC_GETH_ConfigNumOfLogicalTxPorts() routine.
                When working with one Tx port (normal mode), the port index
                should always be set to zero.
 
                The user must pass an array of scheduling parameters (queue
                service parameters structure) sized according to the number of
                Tx queues of that port.
 
                If this routine is not called, all queues will be set for strict
                priority, which is equivalent to the following settings:
                - isWeighted        = FALSE
                - scheduleAndTxAsap = TRUE
                - isBwMonitored     = FALSE
                This routine can be called only if the scheduler is enabled
                but the Shaping-Per-Queue functionality is not enabled for the given port.
 
 @Param[in]     h_UccGeth       - Handle to UCC GETH object
 @Param[in]     logicalTxPort   - The logical Tx port.
 @Param[in]     p_QueueService  - Array of scheduling parameters structures
                                  for each of the Tx queues of the port.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_SetQueuesService(t_Handle              h_UccGeth,
                                  uint8_t               logicalTxPort,
                                  t_UccGethQueueService *p_QueueService);


#define UCC_GETH_SHAPE_ALL_PORTS 0xFF


/**************************************************************************//**
 @Function      UCC_GETH_SetShaper

 @Description   Sets the shaping parameters for a given (logical) Tx port.
 
                This routine determines the traffic shaping method of a given
                Tx port. The shaping parameters contain both the average bit
                rate and the maximum burst length allowed for the port.
 
                Note that the index of the logical port must be in the range of
                logical ports that was previously configured using the
                UCC_GETH_ConfigNumOfLogicalTxPorts() routine.
                When working with one Tx port (normal mode), the port index
                should always be set to zero.
 
                If this routine is not called, the port bandwidth is not limited.
                This routine can be called only if the scheduler is enabled
                but the Shaping-Per-Queue functionality is not enabled for the given port.
 
                In order to determine the average bit rate and the maximum burst length
                allowed for all ports together, set the logicalTxPort argument to UCC_GETH_SHAPE_ALL_PORTS
                (in which case, the Shaping-Per-Queue functionality can be enabled for any port).
 
 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     logicalTxPort   - The logical Tx port.
 @Param[in]     maxBurstLength  - Maximum burst length (bytes) allowed for this port.
 @Param[in]     averageBitRate  - Average bit rate (bps) allowed for this port.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_SetShaper(t_Handle h_UccGeth,
                           uint8_t  logicalTxPort,
                           uint32_t maxBurstLength,
                           uint32_t averageBitRate);
 
/**************************************************************************//**
 @Description   Tx queue service and shaping parameters.
 
                This structure is used with the UCC_GETH_SetQueuesServiceAndShaping()
                routine. It contains the parameters for scheduling a Tx queue.
                The scheduler will service the queue and shape its traffic according to these
                parameters and the matching parameters of other Tx queues in
                the same port.
*//***************************************************************************/
typedef struct t_UccGethQueueServiceAndShaping
{
    bool    isWeighted;         /**< Indicates whether the scheduling for this
                                     queue is weighted in which case scheduling
                                     depends on the associated queues weight (WFQ);
                                     If set to FALSE, the scheduling will be done
                                     with a higher priority, and the queue will be
                                     granted transmission regardless of its weight (SP). */
    uint8_t  bwPercentage;      /**< Indicates the relative bandwidth of this queue
                                     (in percentage); The user should make sure that
                                     the sum of all bandwidths is 100. */
    bool     applyShaping;      /**< Indicates whether to apply rate and burst shaping for this queue. */
    uint32_t maxBurstLength;    /**< Maximum burst length (bytes) allowed for this queue. */
    uint32_t averageBitRate;    /**< Average bit rate (bps) allowed for this queue. */
 
} t_UccGethQueueServiceAndShaping;


/**************************************************************************//**
 @Function      UCC_GETH_SetQueuesServiceAndShaping

 @Description   Sets the scheduling parameters for the queues of a given
                (logical) Tx port.
 
                This routine determines the service method among the Tx queues
                and the traffic shaping method of each Tx queue on a given Tx port.
 
                Note that the index of the logical port must be in the range of
                logical ports that was previously configured using the
                UCC_GETH_ConfigNumOfLogicalTxPorts() routine.
                When working with one Tx port (normal mode), the port index
                should always be set to zero.
 
                The user must pass an array of scheduling parameters (queue
                service and shaper parameters structure) sized according to the number of
                Tx queues of that port.
 
                If this routine is not called, all queues will be set for strict
                priority and for bypassing the shaper.
                This routine can be called only if the Shaping-Per-Queue functionality is enabled for the given port.
 
 @Param[in]     h_UccGeth                - Handle to UCC GETH object
 @Param[in]     logicalTxPort            - The logical Tx port.
 @Param[in]     p_QueueServiceAndShaping - Array of scheduling parameters structures
                                           for each of the Tx queues of the port.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_SetQueuesServiceAndShaping(t_Handle                       h_UccGeth,
                                            uint8_t                        logicalTxPort,
                                            t_UccGethQueueServiceAndShaping *p_QueueServiceAndShaping);

/**************************************************************************//**
 @Description   Tx port service parameters.
 
                This structure is used with the UCC_GETH_SetLogicalTxPortsService()
                routine. It contains the parameters for scheduling a Tx port.
                The scheduler will service the port according to these
                parameters and the matching parameters of other Tx ports.
*//***************************************************************************/
typedef struct t_UccGethPortService
{
    bool    scheduleAndTxAsap;  /**< Indicates whether this port should be serviced
                                     for transmission immediately following scheduling,
                                     without stalling for average bit rate regulation
                                     (TRUE), or this port might endure shaper stall
                                     for bit rate regulation (FALSE). */
    bool    isBwMonitored;      /**< Indicates whether this port's bandwidth is
                                     monitored by the shaper; When set to TRUE, the
                                     shapers bandwidth mechanism will include the
                                     port's bandwidth with the regulated bandwidth;
                                     When set to FALSE the port's bandwidth will not
                                     be monitored (extra bandwidth enabled). */
    uint8_t bwPercentage;       /**< Indicates the relative bandwidth of this port
                                     (in percentage); The user should make sure that
                                     the sum of all bandwidths is 100. */
} t_UccGethPortService;


/**************************************************************************//**
 @Function      UCC_GETH_SetLogicalTxPortsService

 @Description   Sets the scheduling parameters for the logical Tx ports.
 
                This routine determines the service method among the logical
                Tx ports.
 
                This routine is relevant only when working with multiple
                logical Tx ports (this can be configured using the
                UCC_GETH_ConfigNumOfLogicalTxPorts() routine).
 
                The user must pass an array of service parameters for all
                of the logical Tx ports. The user should make sure that
                the sum of all bandwidth percentages is exactly 100. The
                firmware applies a WRR algorithm on the logical ports according
                to the given bandwidths.
 
                The default scheduling mode is to have a fair sharing of the
                physical bandwidth among all logical ports.
 
 @Param[in]     h_UccGeth             - Handle to UCC GETH object
 @Param[in]     p_UccGethPortService  - Array of service parameters for all
                                        logical Tx ports.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_SetLogicalTxPortsService(t_Handle             h_UccGeth,
                                          t_UccGethPortService *p_UccGethPortService);

/**************************************************************************//**
 @Function      UCC_GETH_EnableLogicalTxPort

 @Description   Enables a logical Tx port in the scheduler.
 
                The logical Tx port index must be in the range that was
                configured using the UCC_GETH_ConfigNumOfLogicalTxPorts() routine.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object
 @Param[in]     logicalTxPort   - The logical Tx port to enable.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_EnableLogicalTxPort(t_Handle h_UccGeth, uint8_t logicalTxPort);

/**************************************************************************//**
 @Function      UCC_GETH_DisableLogicalTxPort

 @Description   Disables a logical Tx port in the scheduler.
 
                The logical Tx port index must be in the range that was
                configured using the UCC_GETH_ConfigNumOfLogicalTxPorts() routine.

 @Param[in]     h_UccGeth       - Handle to UCC GETH object
 @Param[in]     logicalTxPort   - The logical Tx port to disable.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_DisableLogicalTxPort(t_Handle h_UccGeth, uint8_t logicalTxPort);

/**************************************************************************//**
 @Function      UCC_GETH_ModifyIpMtu

 @Description   Modifies one of the IP MTU values.
 
                This routine is relevant only if IP fragmentation mode was
                previously enabled using the UCC_GETH_ConfigIpFragmentation()
                routine.
 
                When working with multiple logical Tx ports, the IP MTU value is
                associated with a logical Tx port. The index must be in the range
                of logical Tx ports that was previously configured using the
                UCC_GETH_ConfigNumOfLogicalTxPorts() routine.
 
                When working with a single Tx port (normal mode), the IP MTU
                value is associated with a Tx queue. The index value must be in
                the range of queues that was previously configured using the
                UCC_GETH_ConfigNumQueuesTx() routine.
 
                The default IP MTU value is 576 bytes.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object
 @Param[in]     index       - The index of the IP MTU value to update.
 @Param[in]     ipMtu       - The new IP MTU value; use zero to completely
                              disable IP fragmentation on this queue/port.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_ModifyIpMtu(t_Handle h_UccGeth, uint8_t index, uint16_t ipMtu);


#ifdef UCC_GETH_VP_MODE_SUPPORT
/**************************************************************************//**
 @Function      UCC_GETH_TranslateVpCpuSwapQueueIndex

 @Description   Translate ETH VP CPU Swap queue number.
 
                This function translates VP CPU Swap queue number from within GETH
                "pool" of Swap queues to general VP CPU Swap queue number from
                within VP module Swap queue indices. General VP CPU Swap queue index
                is required by QE_IW_CONN_Config() function during IW connection creation.

 @Param[in]     h_UccGeth               - Handle to UCC GETH object
 @Param[in]     swapQueue               - ETH VP CPU Swap queue number: for Swap queues with
                                          internal queue descriptors provide real queue number
                                          while for queues with external queue descriptor provide
                                          (queue number + total number of internal queues), where
                                          number of internal queues should equal to this set by
                                          UCC_GETH_ConfigVpModeCpuSwapQueues() function
 @Param[out]    p_VpCpuSwapQueueIndex   - VP CPU Swap queue index to be passed to QE_IW_CONN_Config()

 @Return        E_OK on success; Error code otherwise

 @Cautions      Valid only in VP mode.
                Allowed only after UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_TranslateVpCpuSwapQueueIndex(t_Handle h_UccGeth,
                                              uint16_t swapQueue,
                                              uint16_t *p_VpCpuSwapQueueIndex);
 
#endif /* UCC_GETH_VP_MODE_SUPPORT */


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      UCC_GETH_DumpRegs

 @Description   Dumps UCC GETH registers and parameters RAM.

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_DumpRegs(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_DumpThreadsData

 @Description   Dumps the data memory of UCC GETH threads (Rx and Tx).

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_DumpThreadsData(t_Handle h_UccGeth);

/**************************************************************************//**
 @Function      UCC_GETH_DumpBdRings

 @Description   Dumps the memory of UCC GETH BD rings (Rx and Tx).

 @Param[in]     h_UccGeth - Handle to UCC GETH object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccGeth must have been previously created using UCC_GETH_Config()
                and initialized using UCC_GETH_Init().
*//***************************************************************************/
t_Error UCC_GETH_DumpBdRings(t_Handle h_UccGeth);

#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of geth_control_grp */


/**************************************************************************//**
 @Group         geth_exceptions_grp UCC GETH Exceptions Unit

 @Description   UCC GETH Exceptions Unit

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Anchor        geth_exbm
 
 @Collection    UCC GETH Exceptions Bit Mask
 
                Exceptions and events reported through the exceptions callback.
                They can be masked/unmasked using UCC_GETH_ConfigExceptions().
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define UCC_GETH_UCCE_CBPR  0x10000000  /**< In half duplex mode, the transmitter is in backpressure state;
                                             In full duplex mode, the transmitter is in PAUSE state. */
#define UCC_GETH_UCCE_SCAR  0x40000000  /**< Hardware Statistics Carry overflow event */
#define UCC_GETH_UCCE_BSY   0x08000000  /**< Busy; Set when a frame is received and discarded due to
                                             lack of free Rx Buffer descriptors; The accompanied flags
                                             parameter provides the busy queues vector
                                             (bit per queue: 0x1 is queue 0, 0x2 is queue 1, 0x4 is queue 2, etc). */
#define UCC_GETH_UCCE_RXC   0x04000000  /**< Rx flow control; A control frame has been received. */
#define UCC_GETH_UCCE_TXC   0x02000000  /**< Tx flow control; A control frame has been sent. */
#define UCC_GETH_UCCE_TXE   0x01000000  /**< Tx error on the channel;
                                             In full duplex mode, this is an underrun condition. */
/* @} */


/** @} */ /* end of geth_exceptions_grp */

/** @} */ /* end of geth_grp */




/**************************************************************************//*
 @Description   IW exceptions callback function prototype (interface with IW driver).
*//***************************************************************************/
typedef void (t_UccGethIwExceptions)(t_Handle h_IwDev, uint32_t events);

/**************************************************************************//*
 @Function      UCC_GETH_SetIwExceptionsCallback

 @Description   Sets the IW exception callbacks (interface with IW driver).
 
                If the callback routine parameter is NULL, or both iwEvents
                and parserEvents are FALSE, the relevant exceptions will be
                masked (disabled).

 @Param[in]     h_UccGeth       - Handle to UCC GETH object.
 @Param[in]     h_IwDev         - Handle to IW device driver.
 @Param[in]     f_IwExceptions  - Callback routine for IW interrupts.
 @Param[in]     iwEvents        - TRUE if IW events are required.
 @Param[in]     parserEvents    - TRUE if parse and lookup events are required.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_SetIwExceptionsCallback(t_Handle               h_UccGeth,
                                         t_Handle               h_IwDev,
                                         t_UccGethIwExceptions  *f_IwExceptions,
                                         bool                   iwEvents,
                                         bool                   parserEvents);

/**************************************************************************//*
 @Description   IW details structure (interface with IW driver).
*//***************************************************************************/
typedef struct t_UccGethIwDetails
{
    e_Ucc               uccId;                      /**< The UCC ID */
    t_Handle            h_App;                      /**< Application ID (for exceptions/events) */
    e_CommMode          iwCommMode;                 /**< IW communication mode */
    uint8_t             fwExtStructsMemId;          /**< Memory partition ID of firmware external structures */
    uint32_t            fwExtStructsMemAttributes;  /**< Memory attributes of firmware external structures */
    uint32_t            dataMemAttributes;          /**< Memory attributes of data buffers */
    t_BufferPoolInfo    bufferPoolInfo;             /**< Data buffer pool information */
    uint8_t             iwCpuRxQueuesNum;           /**< Number of Rx queues for CPU frames from IW
                                                         (1-4, not including queue no. 1) */
    uint8_t             numOfLogicalTxPorts;        /**< Number of logical Tx ports */
    uint8_t         	iwTxQueuesNum;              /**< Number of Tx queues for IW traffic (1-8) */
    uint8_t         	iwQueuesInLogicalPort[8];
                                                    /**< Number of Tx queues for IW traffic (1-8) in each logical Tx port;
                                                         Only indices 0 to (numOfLogicalTxPorts - 1) are valid. */
    uint32_t            iwGlobalPramBase;           /**< Base address of IW global parameters RAM */
    uint32_t            txGlobalPramBaseAddr;       /**< Base address of Tx Global Parameter RAM */
    uint32_t            ipPriorityTableBase;        /**< Base address of the IP priority table */
    uint16_t            maxFrameLength;             /**< Maximum frame length */
    uint8_t             rxVirtFifoBlockSize;        /**< Rx Virtual Fifo Block Size */
    e_UccGethQoSMode    rxQoSMode;                  /**< Quality of Service mode for Rx. */

} t_UccGethIwDetails;


/**************************************************************************//*
 @Function      UCC_GETH_GetIwDetails

 @Description   Gets the IW details of this UCC GETH (interface with IW driver).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[out]    p_IwDetails - Pointer to IW details structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_GetIwDetails(t_Handle h_UccGeth, t_UccGethIwDetails *p_IwDetails);

/**************************************************************************//*
 @Function      UCC_GETH_GetIwIndex

 @Description   Gets the IW index for Interworking Action Descriptor (IWAD).

 @Param[in]     h_UccGeth   - Handle to UCC GETH object.
 @Param[in]     txQueue     - Transmit queue number.
 @Param[out]    p_IwIndex   - Will get the IW TQD index.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_GetIwIndex(t_Handle h_UccGeth, uint8_t txQueue, uint16_t *p_IwIndex);

/**************************************************************************//*
 @Function      UCC_GETH_ConvertIwIndex

 @Description   Converts a given IW index to UCC ID and Tx queue number.
 
                The IW index may have been reported in an IW event queue entry.

 @Param[in]     iwIndex     - The IW index (TQD index).
 @Param[out]    p_UccId     - Returns the UCC ID.
 @Param[out]    p_TxQueue   - Returns the Tx queue number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_ConvertIwIndex(uint16_t iwIndex, e_Ucc *p_UccId, uint8_t *p_TxQueue);

/**************************************************************************//*
 @Description   UCC GETH RX CPU Queue Type
*//***************************************************************************/
typedef enum e_UccGethCpuRxQueue
{
    e_UCC_GETH_CPU_RX_QUEUE         /* Regular UCC GETH CPU Rx queue */
#ifdef UCC_GETH_VP_MODE_SUPPORT
    ,e_UCC_GETH_CPU_SWAP_QUEUE      /* VP CPU Swap queue */
#endif /* UCC_GETH_VP_MODE_SUPPORT */
} e_UccGethCpuRxQueue;


/**************************************************************************//*
 @Function      UCC_GETH_GetCpuIwIndex

 @Description   Provides required details for creating IW connection to one
                of ETH Rx queues: regular ETH Rx queue or Rx CPU Swap queue.

 @Param[in]     h_UccGeth   - Handle to UCC GETH object
 @Param[in]     rxQueue     - Rx queue number (ETH Rx CPU queue or CPU Swap queue);
                              this parameter is actually destIndex as provided in
                              t_QeIwConnectionParam
 @Param[out]    p_QueueType - Pointer to Rx CPU queue type (regular or swap)
 @Param[out]    p_IwIndex   - IW index in case of swap queue; this parameter
                              has no meaning for regular Rx CPU queues
 @Param[out]    p_IsExt     - Indication whether QD resides in external or internal
                              memory; TRUE for external; this parameter
                              has no meaning for regular Rx CPU queues

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_GETH_GetCpuIwIndex(t_Handle             h_UccGeth,
                               uint16_t             rxQueue,
                               e_UccGethCpuRxQueue  *p_QueueType,
                               uint16_t             *p_IwIndex,
                               bool                 *p_IsExt);




#endif /* __UCC_GETH_EXT */
