/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_hdlc_ext.h

 @Description   UCC HDLC external defenitions.
*//***************************************************************************/
#ifndef __UCC_HDLC_EXT_H
#define __UCC_HDLC_EXT_H

#include "ucc_fast_ext.h"
#include "qe_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         ucc_hdlc_id UCC HDLC Application Programming Interface

 @Description   HDLC functions, definitions and enums.

 @{
*//***************************************************************************/

/* UCC HDLC API Routines. */

/**************************************************************************//**
 @Group         ucc_hdlc_init_id UCC HDLC Initialization Unit

 @Description   HDLC Initialization functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   HDLC adress lengths
*//***************************************************************************/
typedef enum e_UccHdlcAddrLen
{
      e_UCC_HDLC_ADDR_LEN_0  = 0    /**< */
    , e_UCC_HDLC_ADDR_LEN_8  = 8    /**< */
    , e_UCC_HDLC_ADDR_LEN_16 = 16   /**< */
} e_UccHdlcAddrLen;

/**************************************************************************//**
 @Description   HDLC CRC/FCS lengths
*//***************************************************************************/
typedef enum e_UccHdlcCrcLen
{
    e_UCC_HDLC_CRC_LEN_16 =  0x00000000,/**< */
    e_UCC_HDLC_CRC_LEN_32 =  0x00000080 /**< */
} e_UccHdlcCrcLen;

/**************************************************************************//**
 @Description   HDLC NBO Bit, Nibble, Octet.
*//***************************************************************************/
typedef enum e_UccHdlcNbo
{
    e_UCC_HDLC_NBO_BIT      = 0x00000000, /**< One data bit per clock.      */
    e_UCC_HDLC_NBO_NIBBLE   = 0x00008000, /**< Four data bits per clock.    */
    e_UCC_HDLC_NBO_OCTET    = 0x00010000  /**< Eight data bits per clock.   */
} e_UccHdlcNbo;

/**************************************************************************//**
 @Description   User callback function called by driver for event handling.

                User provides this function. Driver invokes it.

 @Param         h_App     - (In) the initial h_App given by the user at driver
                            initialization.

 @Param         errors  - (In) the events that caused the exception to be invoked by
                            the driver. These events may be any combination
                            of e_UccHdlcEventBits.

 @Param         flags   - (In) flags for driver application indication may be used for
                            general future massages handed from driver to user.

 @Return        None.
 *//***************************************************************************/
typedef void (t_UccHdlcException) ( t_Handle h_App, uint32_t errors, uint32_t flags );

/**************************************************************************//**
 @Description   HDLC interface basic parameters
*//***************************************************************************/
typedef struct t_UccHdlcParam
{
    /* HDLC protocol parameters */
    bool                busModeEnable;          /**< TRUE for flow bus mode -
                                                     support of collision detection and retransmit*/
    /* The addresses that the hdlc communicaton controler will 'listen' on*/
    /* note however that one can accept a group of addresses by masking out part of the bits */
    /* used for the address bit mask.  */
    e_UccHdlcAddrLen    addressBitLen;          /**< number of bits in address*/
    uint16_t            addr1;                  /**< the address the hdlc channel will accept (one of four addresses)*/
    uint16_t            addr2;                  /**< the address the hdlc channel will accept (one of four addresses)*/
    uint16_t            addr3;                  /**< the address the hdlc channel will accept (one of four addresses)*/
    uint16_t            addr4;                  /**< the address the hdlc channel will accept (one of four addresses)*/

    /* System integration parameters */
    uint32_t            baseAddress;            /**< Base address of UCC HDLC registers in memory. */
    t_Handle            h_Qe;                   /**< Handle to the controlling QE */
    e_Ucc               uccId;                  /**< UCC ID */
    e_QeClock           rxClock;                /**< Rx clock to use */
    e_QeClock           txClock;                /**< Tx clock to use */
    int                 irq;                    /**< UCC interrupt ID (system-specific number) */
    uint8_t             pramMemId;              /**< Parameter RAM partition ID */
    uint32_t            dataMemAttributes;      /**< Memory attributes mask for data buffers */
    t_BufferPoolInfo    bufferPoolInfo;         /**< Data buffer pool information */

    /* Application interface */
    t_Handle            h_App;                  /**< Handle to the application */
    t_RxStoreFunction   *f_RxStore;             /**< Application's routine - called to pass a received buffer to the application;
                                    	        	 If the last buffer contains CRC only the buffer will be NULL; The length
                                    	        	 will indicate the number of bytes at the end of the  last stored buffer
                                    	        	 that were CRC bytes. */
    t_TxConfFunction    *f_TxConf;              /**< Application's routine - called to handle transmitted frames
                                                     may be set to NULL to avoid unwanted routine calls.
                                                     Depending on the data exchange mdoe, confirmation is either
                                                     done upon each buffer or upon a whole transmitted frame.*/
    t_UccHdlcException  *f_Exceptions;          /**< The exception routine that should be invoked,
                                                     upon internal event.*/

} t_UccHdlcParam;


/**************************************************************************//**
 @Function      UCC_HDLC_Config

 @Description   Creates UCC HDLC channel descriptor according to user parameters.

                User parameters are the minimum necessary to specify channel
                behavior. Additional parameters can be specified later using
                UCC_HDLC_ConfigParamX functions.

 @Param         p_UccHdlcParam -  (In) pointer to structure of user-specified parameters.

 @Return        UCC HDLC channel descriptor. NULL if failed.
*//***************************************************************************/
t_Handle UCC_HDLC_Config(t_UccHdlcParam *p_UccHdlcParam);

/**************************************************************************//**
 @Function      UCC_HDLC_Init

 @Description   Fully initializes UCC HDLC just short of being enabled.

 @Param         h_UccHdlc - (In) UCC HDLC descriptor returned by UCC_HDLC_Config().

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_HDLC_Init ( t_Handle h_UccHdlc );

/**************************************************************************//**
 @Group         ucc_hdlc_advanced_init_grp UCC HDLC Advanced Configuration Unit
 
 @Description   Configuration functions used to change default values.
 
 @{
*//***************************************************************************/

 
/* << Parameter config routines Start >> */
/*
 * These declarations generated automatically by FunGen 1.1
 * Time:             May 17 2005 15:23:30
 * Source file:      GenFuncs.c
 */

/**************************************************************************//**
 @Function      UCC_HDLC_ConfigAddrMask

 @Description   Set the address mask for addrlen 0 , 8 , 16 use 0x0000 , 0x00FF , 0xFFFF accordingly (0xFFFF depending on addressBitLen)

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigAddrMask ( t_Handle h_UccHdlc, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigMinimalInterframeGap

 @Description   Set the minimal number of flags between the opening and closing flags.
                Default value: [1]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigMinimalInterframeGap ( t_Handle h_UccHdlc, uint8_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigFlagShareEnable

 @Description   Indicate whether flag sharing is enabled,
                (decrement the number of inerframe flags).
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigFlagShareEnable ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigEnableMultiFrameInFifo

 @Description   Inidicate whether or not more than one frame is alowed in FIFO.
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigEnableMultiFrameInFifo ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigEnableRetransmit

 @Description   Inidicate whether or not retransmit is enabled.
                Default value: [FALSE]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigEnableRetransmit ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigInsertTimeStamp

 @Description   Inidicate whether or not the controller should insert a timestamp prior to data in each BD.
                Pay attention to the fact that the allocated buffer should contain a four byte
                prefix. !!!
                The time stamp can be found in info 3 when in driver framed mode.
                In all modes ( framed or not framed ), the first 4 bytes of each buffer.
                contain the time stamp. (of the following buffer). Therefore,
                the data follows
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigInsertTimeStamp ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigEnableBRM

 @Description   Indicate whther or not ~RTS (signal) in bus mode.
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigEnableBRM ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigDisableRxDuringTx

 @Description   Inidicate whether or not to disable the receiver during transmition.
                Relevant in bus mode.
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigDisableRxDuringTx ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigNibbleMode

 @Description   Indicate the number of data bits in a clock cycle.
                e_UCC_HDLC_NBO_BIT, e_UCC_HDLC_NBO_NIBBLE, e_UCC_HDLC_NBO_OCTET .
                [e_UCC_HDLC_NBO_BIT]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigNibbleMode ( t_Handle h_UccHdlc, e_UccHdlcNbo newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxFramesThreshHold

 @Description   Set the number of frames arriving that should cause an RXF interrupt.
                Default value: [1]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxFramesThreshHold ( t_Handle h_UccHdlc, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigCrcLen

 @Description   The length of the CRC [e_UCC_HDLC_CRC_LEN_32]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigCrcLen ( t_Handle h_UccHdlc, e_UccHdlcCrcLen newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigMaxFrameLen

 @Description   Maximal frame length (including: address , control , data ,CRC excluding flags).

                Default value: [0x100]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigMaxFrameLen ( t_Handle h_UccHdlc, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxBdRingLen

 @Description   Number of entries in transmit BD ring.

                Default value: [16]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxBdRingLen ( t_Handle h_UccHdlc, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxBdRingLen

 @Description   Number of entries in receive  BD ring.

                Default value: [16]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxBdRingLen ( t_Handle h_UccHdlc, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigIntrManagerId

 @Description   If zero, invoke driver receive routine directly inside interrupt.
                If non-zero, use XX_Call to enqueue call to driver receive routine.
                In such case, intrManagerId is provided as argument to XX_Call().
                [0]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigIntrManagerId ( t_Handle h_UccHdlc, uint32_t newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigInterruptUponReceive

 @Description   Interrupt host upon frame receive indicator (interrupt driven receive).
                TRUE for interrupt driven FALSE for polling reception.
                [TRUE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigInterruptUponReceive ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigExceptions

 @Description   The events user expects to receive an interrupt upon occurrence.
                A combination of the following available events:
                UCC_HDLC_UCCE_TXE, UCC_HDLC_UCCE_BSY, UCC_HDLC_UCCE_FLG,
                UCC_HDLC_UCCE_IDL. (TXE = transmit error, BSY = Busy condition,
                FLG = HDLC Flag status changed, IDL = Idle sequence status changed)
                [UCC_HDLC_UCCE_TXE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigExceptions ( t_Handle h_UccHdlc, uint32_t newVal );


/*
 * End of automatically generated declarations
 */

/* << Parameter config routines End >> */

/** @} */ /* end of ucc_hdlc_advanced_init_grp group */

/* These enums converted by hand by matanyah at Mon May 16 10:21 2005 */

/**************************************************************************//**
 @Description   UCC HDLC receiver decoding mode
*//***************************************************************************/
typedef enum e_UccHdlcGUMRReceiverDecodingMethodRENC
{
    e_UCC_HDLC_RX_ENCODING_NRZ          = e_UCC_FAST_RX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_HDLC_RX_ENCODING_NRZI         = e_UCC_FAST_RX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_HDLC_RX_ENCODING_RESERVED0    = e_UCC_FAST_RX_ENCODING_RESERVED0, /**<      */
    e_UCC_HDLC_RX_ENCODING_RESERVED1    = e_UCC_FAST_RX_ENCODING_RESERVED1  /**<      */
}e_UccHdlcGUMRReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   UCC HDLC transmitter encoding mode

*//***************************************************************************/
typedef enum e_UccHdlcGUMRTransmitterEncodingMethodTENC
{
    e_UCC_HDLC_TX_ENCODING_NRZ          = e_UCC_FAST_TX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_HDLC_TX_ENCODING_NRZI         = e_UCC_FAST_TX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_HDLC_TX_ENCODING_RESERVED0    = e_UCC_FAST_TX_ENCODING_RESERVED0, /**<      */
    e_UCC_HDLC_TX_ENCODING_RESERVED1    = e_UCC_FAST_TX_ENCODING_RESERVED1  /**<      */
}e_UccHdlcGUMRTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   UCC HDLC diagnostic mode
*//***************************************************************************/
typedef enum e_UccHdlcGUMRDiagnosticModeDIAG
{
    e_UCC_HDLC_DIAGNOSTIC_NORMAL            = e_UCC_FAST_DIAGNOSTIC_NORMAL            , /**< normal             */
    e_UCC_HDLC_DIAGNOSTIC_LOCAL_LOOP_BACK   = e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   , /**< loop back          */
    e_UCC_HDLC_DIAGNOSTIC_AUTO_ECHO         = e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         , /**< echo               */
    e_UCC_HDLC_DIAGNOSTIC_LOOP_BACK_AND_ECHO= e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO  /**< loop back and echo */
}e_UccHdlcGUMRDiagnosticModeDIAG;


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigBusModeEnable

 @Description   Indicates whether or not bus mode is enabled. (userBusModeEnable)

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigBusModeEnable ( t_Handle h_UccHdlc, bool newVal );


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_UccHdlc       - Handle to UCC HDLC object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigFirmwareExternalStructsMemory(t_Handle   h_UccHdlc,
                                                     uint8_t    memId,
                                                     uint32_t   memAttributes);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigInitTx

 @Description   TRUE to initialize for transmission.

                Default value: [TRUE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigInitTx(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigInitRx

 @Description   TRUE to initialize for reception.  [TRUE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigInitRx(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigGrantSupport

 @Description   TRUE to provide grant support.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigGrantSupport(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTSA

@Description    Configures the UCC HDLC to connect to the Time-Slot Assigner
                (TSA) instead of the normal pins.
                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTSA(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigCDP

 @Description   FALSE for envelope mode (normal operation),
                TRUE for pulse mode.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigCDP(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigCDSynchronous

 @Description   FALSE for CD input that is asynchronous with data,
                TRUE for synchronous.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigCDSynchronous(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigCTSP

 @Description   FALSE for envelope mode (normal operation),
                TRUE for pulse mode.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigCTSP(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigCTSSynchronous

 @Description   FALSE for CTS input that is asynchronous with data,
                TRUE for synchronous.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigCTSSynchronous(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigInvertTxClock

 @Description   TRUE to invert internal transmit clock.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigInvertTxClock(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRTSM

 @Description   FALSE to send idles between frames,
                TRUE to send flags/syncs between frames.

                Default value: [FALSE]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRTSM(t_Handle h_UccHdlc, bool newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigDataSyncPattern1

 @Description   Data synchronization pattern 1.

                Default value: [0x7E]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigDataSyncPattern1(t_Handle h_UccHdlc, uint8_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigDataSyncPattern2

 @Description   Data synchronization pattern 2.

                Default value: [0x7E]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigDataSyncPattern2(t_Handle h_UccHdlc, uint8_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxVirtFifoSize

 @Description   Rx virtual FIFO Size register.

                Default value: [0x100]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxVirtFifoSize(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxVirtFifoEmergThresh

 @Description   Set Rx virtual FIFO Emergency Threshold.
                Default value: [0x88]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxVirtFifoEmergThresh(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxVirtFifoSpecEmergThresh

 @Description   Rx virtual FIFO Special Emergency Threshold.

                Default value: [0xD0]

 @Param         h_UccHdlc   - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxVirtFifoSpecEmergThresh(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxVirtFifoSize

 @Description   Tx virtual FIFO Size register.

                Default value: [0x100]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxVirtFifoSize(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxVirtFifoEmergThresh

 @Description   Tx virtual FIFO Emergency Threshold.

                Default value: [0x88]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxVirtFifoEmergThresh(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxVirtFifoTxThresh

 @Description   Tx virtual FIFO Tx Threshold.

                Default value: [0x40]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxVirtFifoTxThresh(t_Handle h_UccHdlc, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxPollingTimer

 @Description   Tx Polling Timer [256]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxPollingTimer(t_Handle h_UccHdlc, uint16_t newVal);

/**************************************************************************//**
 @Function      UCC_HDLC_ConfigTxEncoding

 @Description   Transmitter encoding [e_UCC_FAST_TX_ENCODING_NRZ]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal    - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigTxEncoding(t_Handle h_UccHdlc, e_UccHdlcGUMRTransmitterEncodingMethodTENC newVal);

/**************************************************************************//**
 @Function      UCC_HDLC_ConfigRxDecoding

 @Description   Receiver decoding [e_UCC_FAST_RX_ENCODING_NRZ]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigRxDecoding(t_Handle h_UccHdlc, e_UccHdlcGUMRReceiverDecodingMethodRENC newVal);


/**************************************************************************//**
 @Function      UCC_HDLC_ConfigDiagnosticMode

 @Description   Diagnostic mode.

                Default value: [e_UCC_FAST_DIAGNOSTIC_NORMAL]

 @Param         h_UccHdlc - (In) handle to UCC HDLC
 @Param         newVal    - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_HDLC_Init().
*//***************************************************************************/
t_Error UCC_HDLC_ConfigDiagnosticMode(t_Handle h_UccHdlc, e_UccHdlcGUMRDiagnosticModeDIAG newVal);


/* End of prototypes generated by ../../gen/fungen/fastslowext.pl at Mon May 16 13:38:43 2005 */


/**hw hardware specific wrappers. end*/

/**************************************************************************//**
 @Function      UCC_HDLC_Free

 @Description   Frees all resources used by UCC specified UCC HDLC.

 @Param         h_UccHdlc - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().

 @Return        None.

 @Cautions      Does not disable UCC.
*//***************************************************************************/
void UCC_HDLC_Free (t_Handle  h_UccHdlc);

/** @} */ /* end of group UCC HDLC Initialization Unit*/

/**************************************************************************//**
 @Group         ucc_hdlc_control_id UCC HDLC Control Unit

 @Description   HDLC Control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
  HDLC UCCE - HDLC Event register bits
 *//***************************************************************************/
#define UCC_HDLC_UCCE_TXE   0x00100000  /**< Transmit error. */
#define UCC_HDLC_UCCE_BSY   0x00040000  /**< Busy condition, received frame was discarded due to lack of buffers. */
#define UCC_HDLC_UCCE_FLG   0x00000200  /**< HDLC Flag status changed. */
#define UCC_HDLC_UCCE_IDL   0x00000100  /**< Idle sequence status changed. */


/**************************************************************************//**
 @Description   a structure for returning channels counters
*//***************************************************************************/
typedef struct t_UccHdlcCnt
{
    uint16_t    disfc;        /**< discarded frame counter                  */
    uint16_t    crcec;        /**< CRC error counter                        */
    uint16_t    abtsc;        /**< abort sequence counter                   */
    uint16_t    nmarc;        /**< nonmatching address rx cnt               */
} t_UccHdlcCnt;


/** @} */ /* end of group UCC HDLC Control Unit*/

/**************************************************************************//**
 @Group         ucc_hdlc_runtime_id UCC HDLC Runtime Unit

 @Description   HDLC Runtime functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
  @Description   transmit BD's status (erroneous conditions).
 *//***************************************************************************/
typedef enum e_UccHdlcTxBdFlags
{
    e_UCC_HDLC_TX_UN = 0x0002,         /**< underrun */
    e_UCC_HDLC_TX_CT = 0x0001          /**< ~CTS lost  */
}e_UccHdlcTxBdFlags;

/**************************************************************************//**
  @Description   Receive BD's status.
 *//***************************************************************************/
typedef enum e_UccHdlcRxBdFlags
{
    e_UCC_HDLC_RX_LG = 0x0020, /**< Rx Frame length violation (>mflr)                      */
    e_UCC_HDLC_RX_NO = 0x0010, /**< Rx Non Octet - received frame contains a number of bits not divisible by eight. */
    e_UCC_HDLC_RX_OV = 0x0002, /**< receiver overrun occurred during frame reception.      */
    e_UCC_HDLC_RX_CD = 0x0001  /**< ~CD lost- the signal negated during frame reception.   */
}e_UccHdlcRxBdFlags;



/**************************************************************************//**
 @Function      UCC_HDLC_Tx

 @Description   Transmits a data buffer on the HDLC link.

 @Param         h_UccHdlc       - (In) The UCC HDLC descriptor returned by UCC_HDLC_Config().
 @Param         p_Data          - (In) The address of the data for sending.
 @Param         h_BufContext    - (In) User's context for the given data buffer.
 @Param         length          - (In) The number of octets for transmitting.
 @Param         lastUserBuffer  - (In) Indicate whether or not the referred buffer
                                       is the last one in the frame.

 @Return        E_OK (0) in success, other value otherwise.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_HDLC_Tx(t_Handle    h_UccHdlc,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint32_t    length,
                    bool        lastUserBuffer);

/**************************************************************************//**
 @Function      UCC_HDLC_Rx

 @Description   Receive routine - its up to the user to reassemble
                the HDLC frame the routine will invoke the users callback on each received buffer.
                It will do so for all incomming buffers unless the callback indicates that it wishes
                to pause reception, I.E. This Pause will take effect till the next Frame arrives, assuming
                The Receive is interrupt driven or untill the user invokes this routine otherwise..

 @Param         h_UccHdlc - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().

 @Return        E_OK (0) in success, other value otherwise.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_HDLC_Rx(t_Handle h_UccHdlc);

/** @} */ /* end of group UCC HDLC Runtime Unit*/


/**************************************************************************//**
 @Group         ucc_hdlc_control_id UCC HDLC Control Unit

 @Description   HDLC Control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_HDLC_Enable

 @Description   Enables the fully initialized UCC HDLC controller.

                May be called only after UCC_HDLC_Init().

 @Param         h_UccHdlc - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().
 @Param         mode - (In) Tx Rx or both.

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_HDLC_Enable(t_Handle h_UccHdlc, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_HDLC_Disable

 @Description   Disables the fully initialized UCC HDLC controller.

                May be called only after UCC_HDLC_Init().

 @Param         h_UccHdlc - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().
 @Param         mode    - (In) TX, RX, or both.

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_HDLC_Disable(t_Handle h_UccHdlc, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_HDLC_TxConf

 @Description   Force confirmation on transmitted frames.
                Force driver to pole the TX frame queue (TX BD ring), and
                confirm transmission (invoke associated call back), on transmitted frames.
                The use of this routine is optional. (Confirmation will eventually occur anyway once
                a complete sweep of transmission is done.) This routine is good when there's an
                importance to the period between transmission and confirmation, or for a 'dead' channel that
                transmission on it is done once in a blue moon.


 @Param         h_UccHdlc - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().

 @Return        t_Error.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_HDLC_TxConf(t_Handle h_UccHdlc);

/**************************************************************************//**
 @Function      UCC_HDLC_GetCounters

 @Description   Retrieves counters.

 @Param         h_UccHdlc    - (In) the UCC HDLC descriptor returned by UCC_HDLC_Config().

 @Param         p_UccHdlcCnt - (In) the address of the cnt structure for updating.

 @Return        t_Error.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_HDLC_GetCounters(t_Handle h_UccHdlc, t_UccHdlcCnt *p_UccHdlcCnt);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      UCC_HDLC_DumpRegs

 @Description   Dumps UCC HDLC registers and parameters RAM.

 @Param         h_UccHdlc   - (In) handle to UCC HDLC

 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error UCC_HDLC_DumpRegs( t_Handle h_UccHdlc );

/**************************************************************************//**
 @Function      UCC_HDLC_DumpBdRings

 @Description   Dumps the memory of UCC HDLC BD rings (Rx and Tx).

 @Param         h_UccHdlc   - (In) handle to UCC HDLC

 @Return        E_OK on success; Error code otherwise.

*//***************************************************************************/
t_Error UCC_HDLC_DumpBdRings( t_Handle h_UccHdlc );

#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of group UCC HDLC Control Unit*/
/** @} */ /* end of group UCC HDLC */


#endif /* __UCC_HDLC_EXT_H */
