/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_trns_ext.h

 @Description   UCC TRANSPARENT external definitions.
*//***************************************************************************/
#ifndef __UCC_TRNS_EXT_H
#define __UCC_TRNS_EXT_H

#include "ucc_fast_ext.h"
#include "qe_ext.h"
#include "part_ext.h"

/**************************************************************************//**
 @Group         ucc_trns_id UCC TRANSPARENT Application Programming Interface

 @Description   Transparent functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         ucc_trns_init_id UCC TRANSPARENT Initialization Unit

 @Description   Transparent Initialization functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   TRANSPARENT CRC/FCS lengths
*//***************************************************************************/
typedef enum e_UccTrnsCrcLen
{
    e_UCC_TRNS_CRC_LEN_16 =  0x00000000,/**< */
    e_UCC_TRNS_CRC_LEN_32 =  0x00000080 /**< */
}e_UccTrnsCrcLen;

/**************************************************************************//**
 @Description   TRANSPARENT NBO Bit, Nibble, Octet.
*//***************************************************************************/
typedef enum e_UccTrnsNbo
{
    e_UCC_TRNS_NBO_BIT      = 0x00000000, /**< One data bit per clock.      */
    e_UCC_TRNS_NBO_NIBBLE   = 0x00008000, /**< Four data bits per clock.    */
    e_UCC_TRNS_NBO_OCTET    = 0x00010000  /**< Eight data bits per clock.   */
}e_UccTrnsNbo;

/**************************************************************************//**
 @Description   User callback function called by driver for event handling.

                User provides this function. Driver invokes it.

 @Param         appId     - (In) the initial appId given by the user at driver
                            initialization.

 @Param         errors  - (In) the events that caused the exception to be invoked by
                            the driver.

 @Param         flags   - protocol dependant indicator, may be used to notify application
                            of various things.

 @Return        None.
 *//***************************************************************************/
typedef void (t_UccTrnsException) ( t_Handle appId, uint32_t errors, uint32_t flags );

/**************************************************************************//**
 @Description   TRANSPARENT interface basic parameters
*//***************************************************************************/
typedef struct t_UccTrnsParam
{
    /* System integration parameters */
    uint32_t            baseAddress;        /**< Pointer to UCC TRNS registers in memory. */
    t_Handle            h_Qe;               /**< Handle to the controlling QE */
    e_Ucc               uccId;              /**< The UCC ID  */
    e_QeClock           rxClock;            /**< Rx clock to use */
    e_QeClock           txClock;            /**< Tx clock to use */
    int                 irq;                /**< UCC interrupt ID (system-specific number) */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID */
    uint32_t            dataMemAttributes;  /**< Memory attributes mask for data buffers */
    t_BufferPoolInfo    bufferPoolInfo;     /**< Data buffer pool information */

    /* Application interface */
    t_Handle            h_App;              /**< Handle to the application */
    t_RxStoreFunction   *f_RxStore;         /**< Application's routine - called to pass a received buffer to the application;
                                    	    	 If the last buffer contains CRC only the buffer will be NULL; The length
                                    	    	 will indicate the number of bytes at the end of the  last stored buffer
                                    	    	 that were CRC bytes. */
    t_TxConfFunction    *f_TxConf;          /**< Application's routine - called to handle transmitted frames
                                                 may be set to NULL to avoid unwanted routine calls.
                                                 Depending on the data exchange mdoe, confirmation is either
                                                 done upon each buffer or upon a whole transmitted frame.*/
    t_UccTrnsException  *f_Exceptions;      /**< The exception invoked by driver upon events;
                                                 User may choose to set it to NULL by that avoiding
                                                 recption of events (no harm done). */
} t_UccTrnsParam;


/**************************************************************************//**
 @Function      UCC_TRNS_Config

 @Description   Creates UCC TRANSPARENT channel descriptor according to user parameters.

                User parameters are the minimum necessary to specify channel
                behavior. Additional parameters can be specified later using
                UCC_TRNS_ConfigParamX functions.

 @Param         p_UccTrnsParam -  (In) pointer to structure of user-specified parameters.

 @Return        UCC TRNS channel descriptor. NULL if failed.
*//***************************************************************************/
t_Handle UCC_TRNS_Config(t_UccTrnsParam *p_UccTrnsParam);

/**************************************************************************//**
 @Function      UCC_TRNS_Init

 @Description   Fully initializes UCC TRANSPARENT just short of being enabled.

 @Param         h_UccTrns - (In) UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_TRNS_Init ( t_Handle h_UccTrns );

/**************************************************************************//**
 @Group         ucc_trns_advanced_init_grp UCC TRANSPARENT Advanced Configuration Unit
 
 @Description   Configuration functions used to change default values.
 
 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_TRNS_ConfigNibbleMode

 @Description   Indicate the number of data bits in a clock cycle.
                e_UCC_TRNS_NBO_BIT, e_UCC_TRNS_NBO_NIBBLE, e_UCC_TRNS_NBO_OCTET .
                [e_UCC_TRNS_NBO_BIT]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigNibbleMode ( t_Handle h_UccTrns, e_UccTrnsNbo newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigCrcLen

 @Description   Transparent CRC len e_UCC_FAST_16_BIT_CRC or
                e_UCC_FAST_32_BIT_CRC.
                Default value: [e_UCC_FAST_16_BIT_CRC]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigCrcLen ( t_Handle h_UccTrns, e_UccFastGUMRTransparentCRCTypeTCRC newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxBdRingLen

 @Description   Set the number of entries in transmit BD ring.
                Default value: [16]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxBdRingLen ( t_Handle h_UccTrns, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxBdRingLen

 @Description   Set the number of entries in receive  BD ring.
                Default value: [16]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxBdRingLen ( t_Handle h_UccTrns, uint16_t newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigIntrManagerId

 @Description   Set the interrupt manager ID.
                If zero, invoke driver receive routine directly inside interrupt;
                If non-zero, use XX_Call to enqueue call to driver receive routine.
                In such case, intrManagerId is provided as argument to XX_Call().
                [0]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigIntrManagerId ( t_Handle h_UccTrns, uint32_t newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigInterruptUponReceive

 @Description   Request interruption upon incomming frames.
                Interrupt host upon frame receive indicator (interrupt driven receive).
                TRUE for interrupt driven FALSE for polling reception.
                [TRUE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigInterruptUponReceive ( t_Handle h_UccTrns, bool newVal );


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigExceptions

 @Description   Set the events user expects to receive an interrupt upon occurrence.
                A combination of the following available events.
                UCC_TRNS_UCCE_TXE, UCC_TRNS_UCCE_BSY.
                (TXE = transmit error, BSY = Busy condition.)
                [UCC_TRNS_UCCE_OTHER]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigExceptions ( t_Handle h_UccTrns, uint32_t newVal );


/**************************************************************************//**
 @Description   UCC TRNS receiver decoding mode
*//***************************************************************************/
typedef enum e_UccTrnsGUMRReceiverDecodingMethodRENC
{
    e_UCC_TRNS_RX_ENCODING_NRZ          = e_UCC_FAST_RX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_TRNS_RX_ENCODING_NRZI         = e_UCC_FAST_RX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_TRNS_RX_ENCODING_RESERVED0    = e_UCC_FAST_RX_ENCODING_RESERVED0, /**<      */
    e_UCC_TRNS_RX_ENCODING_RESERVED1    = e_UCC_FAST_RX_ENCODING_RESERVED1  /**<      */
}e_UccTrnsGUMRReceiverDecodingMethodRENC;

/**************************************************************************//**
 @Description   UCC TRNS transmitter encoding mode

*//***************************************************************************/
typedef enum e_UccTrnsGUMRTransmitterEncodingMethodTENC
{
    e_UCC_TRNS_TX_ENCODING_NRZ          = e_UCC_FAST_TX_ENCODING_NRZ      , /**< nrz  */
    e_UCC_TRNS_TX_ENCODING_NRZI         = e_UCC_FAST_TX_ENCODING_NRZI     , /**< nrzi */
    e_UCC_TRNS_TX_ENCODING_RESERVED0    = e_UCC_FAST_TX_ENCODING_RESERVED0, /**<      */
    e_UCC_TRNS_TX_ENCODING_RESERVED1    = e_UCC_FAST_TX_ENCODING_RESERVED1  /**<      */
}e_UccTrnsGUMRTransmitterEncodingMethodTENC;

/**************************************************************************//**
 @Description   UCC TRNS diagnostic mode
*//***************************************************************************/
typedef enum e_UccTrnsGUMRDiagnosticModeDIAG
{
    e_UCC_TRNS_DIAGNOSTIC_NORMAL            = e_UCC_FAST_DIAGNOSTIC_NORMAL            , /**< normal             */
    e_UCC_TRNS_DIAGNOSTIC_LOCAL_LOOP_BACK   = e_UCC_FAST_DIAGNOSTIC_LOCAL_LOOP_BACK   , /**< loop back          */
    e_UCC_TRNS_DIAGNOSTIC_AUTO_ECHO         = e_UCC_FAST_DIAGNOSTIC_AUTO_ECHO         , /**< echo               */
    e_UCC_TRNS_DIAGNOSTIC_LOOP_BACK_AND_ECHO= e_UCC_FAST_DIAGNOSTIC_LOOP_BACK_AND_ECHO  /**< loop back and echo */
}e_UccTrnsGUMRDiagnosticModeDIAG;

/**************************************************************************//**
 @Description   UCC TRNS CRC length
*//***************************************************************************/
typedef enum e_UccTrnsGUMRTransparentCRCTypeTCRC
{
    e_UCC_TRNS_16_BIT_CRC     = e_UCC_FAST_16_BIT_CRC   , /**< 16 bit */
    e_UCC_TRNS_CRC_RESERVED0  = e_UCC_FAST_CRC_RESERVED0, /**<        */
    e_UCC_TRNS_32_BIT_CRC     = e_UCC_FAST_32_BIT_CRC   , /**< 32 bit */
    e_UCC_TRNS_CRC_RESERVED1  = e_UCC_FAST_CRC_RESERVED1  /**<        */
}e_UccTrnsGUMRTransparentCRCTypeTCRC;

/**************************************************************************//**
 @Description   UCC TRNS Sync length (transparent mode only)
*//***************************************************************************/
typedef enum e_UccTrnsGUMRSyncLenSYNL
{
    e_UCC_TRNS_SYNC_LEN_NOT_USED  = e_UCC_FAST_SYNC_LEN_NOT_USED , /**< not used  */
    e_UCC_TRNS_SYNC_LEN_AUTOMATIC = e_UCC_FAST_SYNC_LEN_AUTOMATIC, /**< automatic */
    e_UCC_TRNS_SYNC_LEN_8_BIT     = e_UCC_FAST_SYNC_LEN_8_BIT    , /**< 8 bit     */
    e_UCC_TRNS_SYNC_LEN_16_BIT    = e_UCC_FAST_SYNC_LEN_16_BIT     /**< 16 bit    */
}e_UccTrnsGUMRSyncLenSYNL;


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_UccTrns       - Handle to UCC TRNS object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigFirmwareExternalStructsMemory(t_Handle   h_UccTrns,
                                                     uint8_t    memId,
                                                     uint32_t   memAttributes);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigInitTx

 @Description   Set the Tx initialion requirement.
                TRUE to initialize for transmission.
                Default value: [TRUE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigInitTx(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigInitRx

 @Description   Set the Tx initialion requirement.
                TRUE to initialize for reception.
                Default value: [TRUE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigInitRx(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigGrantSupport

 @Description   TRUE to provide grant support.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigGrantSupport(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTSA

@Description    Configures the UCC Transparent to connect to the Time-Slot Assigner
                (TSA) instead of the normal pins.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTSA(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigCDP

 @Description   FALSE for envelope mode (normal operation),
                TRUE for pulse mode.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigCDP(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigCDSynchronous

 @Description   FALSE for CD input that is asynchronous with data,
                TRUE for synchronous.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigCDSynchronous(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigCTSP

 @Description   FALSE for envelope mode (normal operation),
                TRUE for pulse mode.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigCTSP(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigCTSSynchronous

 @Description   FALSE for CTS input that is asynchronous with data,
                TRUE for synchronous.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigCTSSynchronous(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigInvertTxClock

 @Description   TRUE to invert internal transmit clock.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigInvertTxClock(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxTxSync

 @Description   TRUE for receiver-transmitter synchronization.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxTxSync(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRTSM

 @Description   FALSE to send idles between frames,
                TRUE to send flags/syncs between frames.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRTSM(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigReverseDataTransparent

 @Description   TRUE to reverse data on transparent channels.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigReverseDataTransparent(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigReceiveSyncTiming

 @Description   TRUE for receive synchronization timing.
                Default value: [FALSE]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigReceiveSyncTiming(t_Handle h_UccTrns, bool newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigDataSyncPattern1

 @Description   Data synchronization pattern 1.
                Default value: [0x7E]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigDataSyncPattern1(t_Handle h_UccTrns, uint8_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigDataSyncPattern2

 @Description   Data synchronization pattern 2.
                Default value: [0x7E]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigDataSyncPattern2(t_Handle h_UccTrns, uint8_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxVirtFifoSize

 @Description   Set the Rx virtual FIFO size.
                Default value: [0x100]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxVirtFifoSize(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxVirtFifoEmergThresh

 @Description   Set the Rx virtual FIFO Emergency Threshold.
                Default value: [0x88]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxVirtFifoEmergThresh(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxVirtFifoSpecEmergThresh

 @Description   Set the Rx virtual FIFO Special Emergency Threshold.
                Default value: [0xD0]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxVirtFifoSpecEmergThresh(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxVirtFifoSize

 @Description   Set the Tx virtual FIFO Size register.
                Default value: [0x100]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxVirtFifoSize(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxVirtFifoEmergThresh

 @Description   Set the Tx virtual FIFO Emergency Threshold.
                Default value: [0x88]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxVirtFifoEmergThresh(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxVirtFifoTxThresh

 @Description   Set the Tx virtual FIFO Tx Threshold.
                Default value: [0x40]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxVirtFifoTxThresh(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxPollingTimer

 @Description   Configures the amount of time in serial clocks between consecutive polls of an empty
                transmit BD (UFPT).
                Default value: [256]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxPollingTimer(t_Handle h_UccTrns, uint16_t newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTxEncoding

 @Description   Set the Transmitter encoding (NRZ/NRZI).
                Default value: [e_UCC_FAST_TX_ENCODING_NRZ]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTxEncoding(t_Handle h_UccTrns, e_UccTrnsGUMRTransmitterEncodingMethodTENC newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigRxDecoding

 @Description   Receiver decoding.
                Default value: [e_UCC_FAST_RX_ENCODING_NRZ]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigRxDecoding(t_Handle h_UccTrns, e_UccTrnsGUMRReceiverDecodingMethodRENC newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigDiagnosticMode

 @Description   Diagnostic mode.
                Default value: [e_UCC_FAST_DIAGNOSTIC_NORMAL]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigDiagnosticMode(t_Handle h_UccTrns, e_UccTrnsGUMRDiagnosticModeDIAG newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTransparentCRCType

 @Description   Set the transparent CRC type.
                Default value: [e_UCC_FAST_16_BIT_CRC]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTransparentCRCType(t_Handle h_UccTrns, e_UccTrnsGUMRTransparentCRCTypeTCRC newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_ConfigTransparentSyncLenRx

 @Description   Transparent sync length for receiver.
                Default value: [e_UCC_FAST_SYNC_LEN_NOT_USED]

 @Param         h_UccTrns - (In) handle to UCC TRNS
 @Param         newVal      - (In) new value

 @Return        E_OK if okay, other value otherwise.

 @Cautions      Change is applied only upon calling UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_ConfigTransparentSyncLenRx(t_Handle h_UccTrns, e_UccTrnsGUMRSyncLenSYNL newVal);


/**************************************************************************//**
 @Function      UCC_TRNS_Free

 @Description   Frees all resources used by UCC specified UCC TRANSPARENT.

 @Param         h_UccTrns - (In) the UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().

 @Return        None.

 @Cautions      Does not disable UCC.
*//***************************************************************************/
void UCC_TRNS_Free (t_Handle  h_UccTrns);

/** @} */ /* end of group UCC TRANSPARENT Advanced Configuration Unit */

/** @} */ /* end of group UCC TRANSPARENT Initialization Unit*/

/**************************************************************************//**
 @Group         ucc_trns_control_id UCC TRANSPARENT Control Unit

 @Description   Transparent Control functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 TRNS UCCE - TRNS Event register
 *//***************************************************************************/
 #define    UCC_TRNS_UCCE_TXE  0x00100000  /**< Transmit error.*/
 #define    UCC_TRNS_UCCE_BSY  0x00040000  /**< Busy condition, received frame was discarded due to lack of buffers.*/


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      UCC_TRNS_DumpRegs

 @Description   Dumps UCC Transparent registers and parameters RAM.

 @Param         h_UccTrns   - (In) handle to the UCC Transparent object

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccTrns must have been previously created using UCC_TRNS_Config()
                and initialized using UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_DumpRegs( t_Handle h_UccTrns );

/**************************************************************************//**
 @Function      UCC_TRNS_DumpBdRings

 @Description   Dumps the memory of UCC Transparent BD rings (Rx and Tx).

 @Param         h_UccTrns   - (In) handle to the UCC Transparent object

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccTrns must have been previously created using UCC_TRNS_Config()
                and initialized using UCC_TRNS_Init().
*//***************************************************************************/
t_Error UCC_TRNS_DumpBdRings( t_Handle h_UccTrns );

#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of group UCC TRANSPARENT Control Unit*/

/**************************************************************************//**
 @Group         ucc_trns_runtime_id UCC TRANSPARENT Runtime Unit

 @Description   TRANSPARENT Runtime functions.

 @{
*//***************************************************************************/


/**************************************************************************//**
  @Description   transmit BD's status (erroneous conditions).
 *//***************************************************************************/
typedef enum e_UccTrnsTxBdFlags
{
    e_UCC_TRNS_TX_UN = 0x0002, /**< underrun */
    e_UCC_TRNS_TX_CT = 0x0001  /**< ~CTS lost  */
}e_UccTrnsTxBdFlags;

/**************************************************************************//**
  @Description   Receive BD's status.
 *//***************************************************************************/
typedef enum e_UccTrnsRxBdFlags
{
    e_UCC_TRNS_RX_LG = 0x0020,  /**< Rx Frame length violation (>mflr)                      */
    e_UCC_TRNS_RX_NO = 0x0010,  /**< Rx Non Octet - received frame contains a number of bits not divisible by eight. */
    e_UCC_TRNS_RX_OV = 0x0002,  /**< receiver overrun occurred during frame reception.      */
    e_UCC_TRNS_RX_CD = 0x0001   /**< ~CD lost- the signal negated during frame reception.   */
}e_UccTrnsRxBdFlags;

/**************************************************************************//**
 @Description   a structure for returning channels counters
*//***************************************************************************/
typedef struct t_UccTrnsCnt
{
    uint16_t    disfc;  /**< discarded frame counter                  */
    uint16_t    crcec;  /**< CRC error counter                        */
} t_UccTrnsCnt;


/**************************************************************************//**
 @Function      UCC_TRNS_Tx

 @Description   Transmits raw data on the Transparent link.

 @Param         h_UccTrns       - (In) The UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().
 @Param         p_Data          - (In) The address of the data for sending.
 @Param         h_BufContext    - (In) User's context for the given data buffer
 @Param         length          - (In) The number of octets for transmitting.
 @Param         lastUserBuffer  - (In) Is this buffer last in current transmitted frame.

 @Return        E_OK (0) in success, other value otherwise.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_TRNS_Tx(t_Handle    h_UccTrns,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint16_t    length,
                    bool        lastUserBuffer);

/**************************************************************************//**
 @Function      UCC_TRNS_Rx

 @Description   Receive routine.
 
                It's up to the user to reassemble the Transparent frame.
                The routine will invoke the users callback on each received buffer.
                It will do so for all incomming buffers unless the callback indicates that it wishes
                to pause reception, I.E. This Pause will take effect till the next Frame arrives, assuming
                The Receive is interrupt driven or untill the user invokes this routine otherwise..

 @Param         h_UccTrns - (In) the UCC TRNS descriptor returned by UCC_TRNS_Config().

 @Return        E_OK (0) in success, other value otherwise.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_TRNS_Rx(t_Handle h_UccTrns);


/** @} */ /* end of group UCC TRANSPARENT Runtime Unit*/

/**************************************************************************//**
 @Group         ucc_trns_control_id UCC TRANSPARENT Control Unit

 @Description   Transparent Control functions.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UCC_TRNS_Enable

 @Description   Enables the fully initialized UCC TRANSPARENT controller.

                May be called only after UCC_TRNS_Init().

 @Param         h_UccTrns - (In) the UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().
 @Param         mode - (In) Tx Rx or both.

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_TRNS_Enable(t_Handle h_UccTrns, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_TRNS_Disable

 @Description   Disables the fully initialized UCC TRANSPARENT controller.

                May be called only after UCC_TRNS_Init().

 @Param         h_UccTrns - (In) the UCC TRNS descriptor returned by UCC_TRNS_Config().
 @Param         mode    - (In) TX, RX, or both.

 @Return        E_OK (0) in success, other value otherwise.
*//***************************************************************************/
t_Error UCC_TRNS_Disable(t_Handle h_UccTrns, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_TRNS_TxConf

 @Description   Force confirmation on transmitted frames.
                Force driver to pole the TX frame queue (TX BD ring), and
                confirm transmission (invoke associated call back), on transmitted frames.
                The use of this routine is optional. (Confirmation will eventually occur anyway once
                a complete sweep of transmission is done.) This routine is good when there's an
                importance to the period between transmission and confirmation, or for a 'dead' channel that
                transmission on it is done once in a blue moon.

 @Param         h_UccTrns - (In) the UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().

 @Return        t_Error.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_TRNS_TxConf(t_Handle h_UccTrns);

/**************************************************************************//**
 @Function      UCC_TRNS_GetCounters

 @Description   Retrieve counters.

 @Param         h_UccTrns - (In) the UCC TRANSPARENT descriptor returned by UCC_TRNS_Config().

 @Param         p_UccTrnsCnt - (In) the address of the cnt structure for updating.

 @Return        t_Error.

 @Cautions      None.
*//***************************************************************************/
t_Error UCC_TRNS_GetCounters(t_Handle h_UccTrns, t_UccTrnsCnt *p_UccTrnsCnt);


/** @} */ /* end of group UCC TRANSPARENT Control Unit*/

/** @} */ /* end of group UCC TRANSPARENT */


#endif /* __UCC_TRNS_EXT_H */
