/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          ucc_uart_ext.h

 @Description   UCC UART external structures and definitions.
*//***************************************************************************/

#ifndef __UCC_UART_EXT_H
#define __UCC_UART_EXT_H

#include "std_ext.h"
#include "ucc_ext.h"
#include "ucc_slow_ext.h"
#include "qe_ext.h"
#include "part_ext.h"


/**************************************************************************//**
 @Group         uart_grp UCC UART Application Programming Interface

 @Description   UCC UART functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         uart_init_grp UCC UART Initialization Unit

 @Description   UCC UART Initialization Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UCC UART Diagnostic Mode
 *//***************************************************************************/
typedef enum e_UccUartDiagnosticMode
{
    e_UCC_UART_DIAGNOSTIC_MODE_NORMAL        = e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_NORMAL       , /**< Normal */
    e_UCC_UART_DIAGNOSTIC_MODE_LOOPBACK      = e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_LOOPBACK     , /**< Loop back */
    e_UCC_UART_DIAGNOSTIC_MODE_ECHO          = e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_ECHO         , /**< Echo */
    e_UCC_UART_DIAGNOSTIC_MODE_LOOPBACK_ECHO = e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_LOOPBACK_ECHO  /**< Loop back and echo */
} e_UccUartDiagnosticMode;

/**************************************************************************//**
 @Description   UART number of data bits
 *//***************************************************************************/
typedef enum e_UccUartDataBits
{
    e_UCC_UART_DATA_BITS_5  = 0x0000,   /**< 5 data bits */
    e_UCC_UART_DATA_BITS_6  = 0x1000,   /**< 6 data bits */
    e_UCC_UART_DATA_BITS_7  = 0x2000,   /**< 7 data bits */
    e_UCC_UART_DATA_BITS_8  = 0x3000    /**< 8 data bits */
} e_UccUartDataBits;

/**************************************************************************//**
 @Description   UART number of stop bits
 *//***************************************************************************/
typedef enum e_UccUartStopBits
{
    e_UCC_UART_STOP_BITS_1,             /**< 1 stop bit */
    e_UCC_UART_STOP_BITS_1_AND_A_HALF,  /**< 1.5 stop bits */
    e_UCC_UART_STOP_BITS_2              /**< 2 stop bit */
} e_UccUartStopBits;

/**************************************************************************//**
 @Description   UART mode
 *//***************************************************************************/
typedef enum e_UccUartMode
{
    e_UCC_UART_MODE_NORMAL              = 0x0000,  /**< Normal mode */
    e_UCC_UART_MODE_AUTO_MULTIDROP      = 0x0c00,  /**< Auto multidrop mode */
    e_UCC_UART_MODE_NONAUTO_MULTIDROP   = 0x0400   /**< Non-auto multidrop mode */
} e_UccUartMode;

/**************************************************************************//**
 @Description  UART parity - same for Tx and Rx
 *//***************************************************************************/
typedef enum e_UccUartParity
{
    e_UCC_UART_PARITY_ODD,          /**< Odd parity */
    e_UCC_UART_PARITY_EVEN,         /**< Even parity */
    e_UCC_UART_PARITY_LOW_SPACE,    /**< Low (space) parity */
    e_UCC_UART_PARITY_HIGH_MARK,    /**< High (mark) parity */
    e_UCC_UART_PARITY_NONE          /**< No parity */
} e_UccUartParity;


/**************************************************************************//**
 @Description   User's callback routine events handling (invoked by the driver).

 @Param         appId   - (In) Application handle, provided by the user in the
                               configuration structure passed to UCC_UART_Config().
 @Param         events  - (In) Events that caused this routine to be invoked
                               by the driver.
 @Param         flags   - (In) Protocol dependent indicator: for UCC_UART_UCCE_CCR
                               event contains the rejected control character.

 @Return        None.

 @See           @ref uart_exbm "UCC UART Exceptions Bit Mask".
 *//***************************************************************************/
typedef void (t_UccUartException)(t_Handle appId, uint32_t events, uint32_t flags);


/**************************************************************************//**
 @Description   UCC UART initialization structure, used when calling UCC_UART_Config().
 *//***************************************************************************/
typedef struct
{
    /* UART basic parameters */
    e_UccUartParity     parity;             /**< Type of parity */
    e_UccUartDataBits   dataBits;           /**< Number of data bits per char */
    e_UccUartStopBits   stopBits;           /**< Number of stop bits per char */
    bool                flowControl;        /**< TRUE for flow control */
    uint16_t            rxBdBufferSize;     /**< Maximum buffers size for Rx BD ring */

    /* System integration parameters */
    uint32_t            baseAddress;        /**< Pointer to UCC HDLC registers in memory. */
    t_Handle            h_Qe;               /**< Handle to the controlling QE module */
    e_Ucc               uccId;              /**< UCC ID */
    e_QeClock           rxClock;            /**< Rx clock to use */
    e_QeClock           txClock;            /**< Tx clock to use */
    int                 irq;                /**< UCC interrupt ID (system-specific number) */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID */
    uint8_t             rxDataBufMemId;     /**< Memory partition ID for allocating Rx data buffers */
    uint32_t            dataMemAttributes;  /**< Memory attributes mask for data buffers */

    /* Application interface */
    t_Handle            h_App;              /**< Handle to the application */
    t_UccUartException  *f_Exceptions;      /**< Callback invoked by driver on exception events;
                                                 if NULL, no exceptions are reported */
    t_TxConfFunction    *f_TxConf;          /**< Application's routine - called to handle transmitted frames
                                                 may be set to NULL to avoid unwanted routine calls.
                                                 Depending on the data exchange mdoe, confirmation is either
                                                 done upon each buffer or upon a whole transmitted frame.*/

} t_UccUartParam;



/**************************************************************************//**
 @Function      UCC_UART_Config

 @Description   Creates UCC UART channel descriptor according to user parameters.

                User parameters are the minimum necessary to specify channel
                behavior. Additional parameters can be specified later using
                UCC_UART_ConfigParamX functions.

 @Param         p_UccUartParam -  (In) pointer to structure of user-specified parameters.

 @Return        UCC UART channel descriptor; NULL if failed.
*//***************************************************************************/
t_Handle UCC_UART_Config(t_UccUartParam *p_UccUartParam);

/**************************************************************************//**
 @Function      UCC_UART_Init

 @Description   Fully initializes UCC UART just short of being enabled.

 @Param         h_UccUart - (In) UCC UART descriptor returned by UCC_UART_Config().

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_UART_Init(t_Handle h_UccUart);

/**************************************************************************//**
 @Function      UCC_UART_Free

 @Description   Frees all resources used by the UCC UART.

 @Param         h_UccUart - (in) Handle to UCC UART object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Does not disable channel, for which UCC_UART_Disable() can be used.
*//***************************************************************************/
t_Error UCC_UART_Free(t_Handle  h_UccUart);


/**************************************************************************//**
 @Group         uart_advanced_init_grp UCC UART Advanced Configuration Unit

 @Description   Configuration functions for changing default values.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UCC_UART_ConfigUartMode

 @Description   Configures the UART protocol mode
 
                The UCC UART supports normal mode, manual multidrop mode and
                automatic multidrop mode.

                Default value: [e_UCC_UART_MODE_NORMAL]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         uccUartMode - (In) UART operation mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigUartMode(t_Handle h_UccUart, e_UccUartMode uccUartMode);

/**************************************************************************//**
 @Function      UCC_UART_ConfigRxControlCharMask

 @Description   Configures the receiver control characters mask
 
                Set the bits of the corresponding control characters that need
                to be compared. Zero bits mask the comparison.

                Default value: [0xFF]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         ctlCharMask - (In) Control characters mask.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigRxControlCharMask(t_Handle h_UccUart, uint8_t ctlCharMask);


/**************************************************************************//**
 @Function      UCC_UART_ConfigRxDataNoStopBits

 @Description   Enables/disables accepting received data without stop bits.
 
                This mode is valid only if synchronous mode is enabled using
                UCC_UART_ConfigSynchronousMode().
 
                If this mode is disabled, the receiver operates normally, but
                at least one stop bit is needed between characters. A framing
                error is issued if a stop bit is missing. Break status is set
                if an all-zero character is received with a zero stop bit.
 
                If this mode is enabled, the controller receives data without
                stop bits. Useful in V.14 applications where data is supplied
                synchronously and all stop bits of a particular character can
                be omitted for cross-network rate adaptation. The receiver
                continues if a stop bit is missing. If the stop bit is a zero,
                the next bit is considered the first data bit of the next
                character. A framing error is issued if a stop bit is missing,
                but a break status is reported only after two consecutive break
                characters have no stop bits.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) TRUE to enable Rx data without stop bits.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigRxDataNoStopBits(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigSynchronousMode

 @Description   Enables/disables synchronous mode.
 
                The clock in this mode is 1x and the reception is synchronous
                with the clock.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) TRUE to enable synchronous mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigSynchronousMode(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigRxBufferSize

 @Description   Configures the size of internal accumulating Rx data buffer.

                Default value:  [1024]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         bufferSize  - (In) Buffer size in bytes.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigRxBufferSize(t_Handle h_UccUart, uint32_t bufferSize);


/**************************************************************************//**
 @Function      UCC_UART_ConfigExceptions

 @Description   Configures the exceptions and events for which to invoke
                the user's exceptions callback.
 
                The supplied value should be an or'ed combination of any, none,
                or all of the following:
                UCC_UART_UCCE_AB, UCC_UART_UCCE_IDL, UCC_UART_UCCE_BRKE,
                UCC_UART_UCCE_BRKS, UCC_UART_UCCE_CCR, UCC_UART_UCCE_BSY.
 
                The user's exceptions callback is part of the configuration
                structure passed to UCC_UART_Config().

                Default value:  [UCC_UART_UCCE_AB]

 @Param         h_UccUart       - (in) Handle to UCC UART object.
 @Param         exceptionsMask  - (In) Mask of enabled exceptions.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().

 @See           @ref uart_exbm "UCC UART Exceptions Bit Mask".
*//***************************************************************************/
t_Error UCC_UART_ConfigExceptions(t_Handle h_UccUart, uint16_t exceptionsMask);


/**************************************************************************//**
 @Function      UCC_UART_ConfigNumControlCharEntries

 @Description   Configures the number of entries in the control characters table.
 
                The entries in the table are set using the
                UCC_UART_ConfigControlCharEntry() routine.
 
                Default value: [0]

 @Param         h_UccUart       - (in) Handle to UCC UART object.
 @Param         numOfEntries    - (In) Number of entries.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigNumControlCharEntries(t_Handle h_UccUart, uint8_t numOfEntries);


/**************************************************************************//**
 @Function      UCC_UART_ConfigControlChar

 @Description   Configures an entry in the control characters table.
 
                The table contains up to 8 entries, and it can be configured
                using UCC_UART_ConfigNumControlCharEntries().
 
                Recognized characters can be accepted normally in the receive
                buffer or rejected to a temporary storage. If UCC_UART_UCCE_CCR
                event is enabled (see UCC_UART_ConfigExceptions()), each
                rejected character shall be reported to the user in the
                exceptions callback routine. Otherwise, the user can read it
                via UCC_UART_GetMatchedControlChar() - this should be done
                quickly enough before another control character overwrites it.
 
                The MSBs of character values smaller than 8 bits are padded
                with zeros.                .
 
                Default value: [0x00]
 
 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         character   - (In) The control character's value
 @Param         reject      - (In) TRUE to reject the character;
                                   FALSE to accept it in the receive buffer.
 @Param         index       - (in) index into table.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigControlChar(t_Handle h_UccUart,
                                   uint8_t  character,
                                   bool     reject,
                                   uint8_t  index);


/**************************************************************************//**
 @Description   UCC UART Oversampling Rate
 *//***************************************************************************/
typedef enum e_UccUartOversamplingRate
{
    e_UCC_UART_OVERSAMPLING_RATE_1  = e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_1 ,  /**< 1x  clock mode */
    e_UCC_UART_OVERSAMPLING_RATE_8  = e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_8 ,  /**< 8x  clock mode */
    e_UCC_UART_OVERSAMPLING_RATE_16 = e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_16,  /**< 16x clock mode */
    e_UCC_UART_OVERSAMPLING_RATE_32 = e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_32   /**< 32x clock mode */
} e_UccUartOversamplingRate;





/**************************************************************************//**
 @Function      UCC_UART_ConfigInitTx

 @Description   Enables/disables the initialization of the transmitter.
 
                Default value: [TRUE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) FALSE to skip transmitter initialization;
                                   TRUE to initialize the transmitter;

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigInitTx(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigInitRx

 @Description   Enables/disables the initialization of the receiver.

                Default value: [TRUE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) TRUE to initialize the receiver;
                                   FALSE to skip receiver initialization.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigInitRx(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigBdRingLenTx

 @Description   Number of entries in transmit BD ring.

                Default value: [32]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         bdRingLenTx - (In) Transmit BD ring length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigBdRingLenTx(t_Handle h_UccUart, uint32_t bdRingLenTx);


/**************************************************************************//**
 @Function      UCC_UART_ConfigBdRingLenRx

 @Description   Number of entries in receive  BD ring.
 
                Default value: [32]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         bdRingLenRx - (In) Receive BD ring length.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigBdRingLenRx(t_Handle h_UccUart, uint32_t bdRingLenRx);


/**************************************************************************//**
 @Function      UCC_UART_ConfigIntrManagerId

 @Description   Configures a receive queue to operate in interrupt manager mode.
 
                Interrupt manager mode can be used only if the UART is
                configured for interrupt mode (that is the default mode, see
                UCC_UART_ConfigRxInterrupts()).
 
                If the interrupt manager ID is non-zero, the driver defers the
                invocation of the receive routine to the user. The driver
                notifies the user through XX_Call(), passing a pointer to a
                receive function, and the user is responsible for calling that
                function eventually (usually not from within XX_Call() itself).
                XX_Call() will not be called again, until the user calls that
                receive function.
 
                If the interrupt manager ID is zero (default mode), this mode is
                disabled. In this case, the driver invokes the receive routine
                directly from the interrupt routine.

                Default value: [0]

 @Param         h_UccUart       - (in) Handle to UCC UART object.
 @Param         intrManagerId   - (In) Interrupt manager ID (pass 0 to disable).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigIntrManagerId(t_Handle h_UccUart, uint32_t intrManagerId);


/**************************************************************************//**
 @Function      UCC_UART_ConfigRxInterrupts

 @Description   Configures the Rx interrupt mode for the receiver.
 
                When working via interrupts, the receive routine is called
                by the driver in interrupt context. When working via polling
                the receive routine is called from within the UCC_UART_Rx()
                routine.
 
                The user must call the UCC_UART_Rx() routine often enough
                to prevent an overflow in the internal buffer or the Rx BD ring.
 
                Note, that in interrupt mode there is also an option to work
                in interrupt manager mode. See UCC_UART_ConfigIntrManagerId().
 
                Default value: [TRUE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) FALSE to disable Rx interrupts;
                                   TRUE to enable Rx interrupts.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigRxInterrupts(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigGrantSupport

 @Description   Enables/disables grant support for the UCC
 
                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) TRUE to enable grant support.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigGrantSupport(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigTSA

 @Description   Configures the UCC UART to connect to the Time-Slot Assigner
                (TSA) instead of the normal pins.
 
                The TSA allows the UCC UART to multiplex data on the TDM
                serial interface.
 
                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         enabled     - (In) TRUE to connect the UCC UART to the TSA.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigTSA(t_Handle h_UccUart, bool enabled);


/**************************************************************************//**
 @Function      UCC_UART_ConfigCDP

 @Description   Configures the CD signal in pulse or envelope mode.
 
                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         pulse       - (In) TRUE for pulse mode;
                                   FALSE for envelope mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigCDP(t_Handle h_UccUart, bool pulse);


/**************************************************************************//**
 @Function      UCC_UART_ConfigCDSynchronous

 @Description   Configures the CD signal sampling.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         synchronous - (In) TRUE for signal that is synchronous with data;
                                   FALSE for signal that is asynchronous with data.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigCDSynchronous(t_Handle h_UccUart, bool synchronous);


/**************************************************************************//**
 @Function      UCC_UART_ConfigCTSP

 @Description   Configures the CTS signal in pulse or envelope mode.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         pulse       - (In) TRUE for pulse mode;
                                   FALSE for envelope mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigCTSP(t_Handle h_UccUart, bool pulse);


/**************************************************************************//**
 @Function      UCC_UART_ConfigCTSSynchronous

 @Description   Configures the CTS signal sampling.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         synchronous - (In) TRUE for signal that is synchronous with data;
                                   FALSE for signal that is asynchronous with data.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigCTSSynchronous(t_Handle h_UccUart, bool synchronous);

/**************************************************************************//**
 @Function      UCC_UART_ConfigMaxIdle

 @Description   Configure the maximum number of idles
                after receiving the number of idles a Buffer will be closed
                (I.E. the data will be received).

                Default value: [5]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         maxIdle     - (In) max idle.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigMaxIdle( t_Handle h_UccUart, uint8_t maxIdle );


/**************************************************************************//**
 @Function      UCC_UART_ConfigInvertRxData

 @Description   Configure with TRUE to invert received data.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         invert      - (In) TRUE to invert received data.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigInvertRxData(t_Handle h_UccUart, bool invert);

/**************************************************************************//**
 @Function      UCC_UART_ConfigInvertTxData

 @Description   Configure with TRUE to invert transmitted data.

                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         invert      - (In) TRUE to invert transmitted data.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigInvertTxData(t_Handle h_UccUart, bool invert);

/**************************************************************************//**
 @Function      UCC_UART_ConfigRTSM

 @Description   Configures the RTS signal mode.

                Default value: [FALSE]

 @Param         h_UccUart       - (in) Handle to UCC UART object.
 @Param         alwaysAsserted  - (In) TRUE to send flags/syncs between frames;
                                       FALSE to send idles between frames (RTS
                                       is negated between frames).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigRTSM(t_Handle h_UccUart, bool alwaysAsserted);

/**************************************************************************//**
 @Function      UCC_UART_ConfigDiagnosticMode

 @Description   Configures the diagnostic mode.

                Use this routine to set the UCC UART in loopback mode, echo
                mode or both.
 
                Default value: [e_UCC_SLOW_GUMR_L_DIAGNOSTIC_MODE_DIAG_NORMAL]
 
 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         diagMode    - (In) Diagnostic mode.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigDiagnosticMode(t_Handle                  h_UccUart,
                                      e_UccUartDiagnosticMode   diagMode);

/**************************************************************************//**
 @Function      UCC_UART_ConfigOversamplingRateTx

 @Description   Configures the oversampling rate for the transmitter.
 
                Default value: [e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_TX_TDCR_16]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         rate        - (In) Transmitter oversampling rate

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigOversamplingRateTx(t_Handle                  h_UccUart,
                                          e_UccUartOversamplingRate rate);


/**************************************************************************//**
 @Function      UCC_UART_ConfigOversamplingRateRx

 @Description   Configures the oversampling rate for the receiver.

                Default value: [e_UCC_SLOW_GUMR_L_OVERSAMPLING_RATE_RX_RDCR_16]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         rate        - (In) Receiver oversampling rate

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigOversamplingRateRx(t_Handle                  h_UccUart,
                                          e_UccUartOversamplingRate rate);


/**************************************************************************//**
 @Function      UCC_UART_ConfigTxBlockingMode

 @Description   Configures the blocking mode for Tx operations.
 
                Default value: [FALSE]

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         newVal      - (In) blocking mode

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UCC_UART_Config() and before UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_ConfigTxBlockingMode(t_Handle h_UccUart, bool newVal);


/** @} */ /* end of uart_advanced_init_grp group */
/** @} */ /* end of uart_init_grp group */


/**************************************************************************//**
 @Group         uart_runtime_grp UCC UART Runtime Unit

 @Description   UCC UART Runtime Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        uart_txerbm
 
 @Collection    UCC UART Transmit Errors Bit Mask
 
                Transmitter errors returned via the f_TxConf callback routine.
 @{
*//***************************************************************************/
#define UCC_UART_TX_ERRORS_CT   0x0001  /**< CTS lost - CTS negated during transmission */

/* @} */


/**************************************************************************//**
 @Description   UCC UART Transmit Flags

                These flags can be passed to the transmit routines for
                performing special operations on the transmitted frame.
 
                The different types of flags can be combined to one value.
*//***************************************************************************/
typedef enum
{
    e_UCC_UART_TX_BD_FLAGS_CTS_REPORT       = 0x0800 /**< Clear-to-send report - normal CTS lost error
                                                          reporting and three bits of idle are sent
                                                          between consecutive buffers. */
  , e_UCC_UART_TX_BD_FLAGS_ADDRESS          = 0x0400 /**< Address buffer (valid only in multidrop mode,
                                                          automatic or manual) - the buffer contains
                                                          address characters. */
  , e_UCC_UART_TX_BD_FLAGS_SEND_PREAMBLE    = 0x0100 /**< Preamble - before sending data, the
                                                          controller sends an idle character consisting
                                                          of all ones; if the data length of this BD is
                                                          zero, only a preamble is sent. */
  , e_UCC_UART_TX_BD_FLAGS_STOP_BIT         = 0x0080 /**< No stop bit or shaved stop bit - in synchronous
                                                          mode, data in this buffer is sent without stop
                                                          bits; otherwise, the stop bits are shaved
                                                          depending on fractional stop-bits setting. */
} e_UccUartTxFlags;


/**************************************************************************//**
 @Function      UCC_UART_Tx

 @Description   Transmits a data buffer on one of the UCC Ethernet transmit queues.

                Accepts data as a pointer to a single data buffer (array of
                bytes).

                This routine eventually causes the user's transmit confirmation
                callback routine to be invoked for all data sent. Latency
                before the callback is invoked is not guaranteed. The
                UCC_UART_TxConf() function (see Control section) can be used to
                force invocation of callback for all transmissions already
                completed by hardware.

 @Param         h_UccUart       - (in) Handle to UCC UART object.
 @Param         p_Data          - (in) Pointer to data to transmit.
 @Param         h_BufContext    - (in) User's context for the given data buffer.
 @Param         length          - (in) Length of data to transmit.
 @Param         flags           - (in) Additional transmit mode flags.
                                       Any or'ed combination of the elements of the
                                       e_UccUartTxFlags enumeration.
                                       se zero for standard operation.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_UART_Tx(t_Handle    h_UccUart,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint16_t    length,
                    uint16_t    flags);

/**************************************************************************//**
 @Function      UCC_UART_TxOutOfSeq

 @Description   Transmits a character out of sequence, bypassing other
                characters already passed to UCC_UART_Tx().

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         charSend    - (in) Character to send out of sequence.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void UCC_UART_TxOutOfSeq(t_Handle h_UccUart, uint8_t charSend);

/**************************************************************************//**
 @Function      UCC_UART_Rx

 @Description   Receives a data buffer.

                Puts as much data as are available, up to the given length,
                in the provided buffer.

                This routine should be called often enough by the user to
                prevent overflow of the internal buffer (or the Rx BD ring).

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         p_Data      - (in) Pointer to data buffer for received data.
 @Param         length      - (in) Maximum length of data to return (in bytes).

 @Return        Number of bytes returned.
*//***************************************************************************/
uint16_t UCC_UART_Rx(t_Handle h_UccUart, uint8_t *p_Data, uint16_t length);


/** @} */ /* end of uart_runtime_grp group */


/**************************************************************************//**
 @Group         uart_control_grp UCC UART Control Unit

 @Description   UCC UART Control Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UCC_UART_Enable

 @Description   Enables transmit and/or receive communication on the UCC UART.

                Communication can be disabled using UCC_UART_Disable().

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         mode        - (In) Communication mode (Tx, Rx, or both).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_Enable (t_Handle h_UccUart, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_UART_Disable

 @Description   Disables transmit and/or receive communication on the UCC UART.

                Channel can be re-enabled using UCC_UART_Enable().

 @Param         h_UccUart   - (in) Handle to UCC UART object.
 @Param         mode        - (In) Communication mode (Tx, Rx, or both).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UCC_UART_Disable (t_Handle h_UccUart, e_CommMode mode);

/**************************************************************************//**
 @Function      UCC_UART_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.
 
                Invokes user's transmit confirmation callback routine for
                previously transmitted data.

                Normally, this functionality is performed in the transmit
                routine for previous data, but with some latency proportional
                to the length of the Tx BD ring.
 
                This routine is optional, and can be called by the user when
                either transmission confirmation is required sooner, or no more
                data are expected to be transmitted.

 @Param         h_UccUart - (in) Handle to UCC UART object.

 @Return        None.
*//***************************************************************************/
void UCC_UART_TxConf(t_Handle h_UccUart);

/**************************************************************************//**
 @Function      UCC_UART_FreezeTx

 @Description   Freezes transmission until UCC_UART_ContinueTx is called.

 @Param         h_UccUart - (in) Handle to UCC UART object.

 @Return        None.
*//***************************************************************************/
t_Error UCC_UART_FreezeTx(t_Handle h_UccUart);


/**************************************************************************//**
 @Function      UCC_UART_DumpRegs

 @Description   Dumps UCC UART registers and parameters RAM.

 @Param[in]     h_UccUart - Handle to UCC UART object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      h_UccUart must have been previously created using UCC_UART_Config()
                and initialized using UCC_UART_Init().
*//***************************************************************************/
t_Error UCC_UART_DumpRegs(t_Handle h_UccUart);

/**************************************************************************//**
 @Function      UCC_UART_ContinueTx

 @Description   Resumes transmission stopped via UCC_UART_FreezeTx.

 @Param         h_UccUart - (in) Handle to UCC UART object.

 @Return        None.
*//***************************************************************************/
t_Error UCC_UART_ContinueTx(t_Handle h_UccUart);


/**************************************************************************//**
 @Description   a structure for returning channels counters
 *//***************************************************************************/
typedef struct t_UccUartStatistics
{
    uint16_t  parec;            /**< Rx parity error counter */
    uint16_t  frmec;            /**< Rx framing error counter */
    uint16_t  nosec;            /**< Rx noise counter */
    uint16_t  brkec;            /**< Rx break character counter */
} t_UccUartStatistics;

/**************************************************************************//**
 @Function      UCC_UART_GetMatchedControlChar

 @Description   Gets most recent control character that was recognized and
                rejected (i.e. not added to the receive buffer).
 
                This routine is needed only when UCC_UART_UCCE_CCR is masked,
                because otherwise the character is suplied by the driver in the
                exceptions callback routine flags.

 @Param         h_UccUart - (in) Handle to UCC UART object.

 @Return        None.
*//***************************************************************************/
uint16_t UCC_UART_GetMatchedControlChar(t_Handle h_UccUart);

/**************************************************************************//**
 @Function      UCC_UART_GetStatistics

 @Description   Gets UCC UART's statistics.

 @Param         h_UccUart           - (in) Handle to UCC UART object.
 @Param         p_UccUartStatistics - (Out) pointer to an array for returning the counters.

 @Return        None.
*//***************************************************************************/
void UCC_UART_GetStatistics(t_Handle h_UccUart, t_UccUartStatistics *p_UccUartStatistics);

/** @} */ /* end of uart_control_grp group */


/**************************************************************************//**
 @Group         uart_exceptions_grp UCC UART Exceptions Unit

 @Description   UCC UART Exceptions Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Anchor        uart_exbm
 
 @Collection    UCC UART Exceptions Bit Mask
 
                Exceptions and events reported through the exceptions callback.
                They can be masked/unmasked using UCC_UART_ConfigExceptions().
 
                These values can be or'ed in any combination into the events
                parameter of the callback function.
 @{
*//***************************************************************************/
#define UCC_UART_UCCE_AB    0x0200  /**< Autobaud - set when an autobaud lock is detected */
#define UCC_UART_UCCE_IDL   0x0100  /**< Idle sequence status changed - set when the channel
                                         detects a change in the serial line; idle state is
                                         entered when a character of all ones is received;
                                         it is exited when a zero is received. */
#define UCC_UART_UCCE_BRKE  0x0040  /**< Break end - set when an idle bit is received after
                                         a break sequence. */
#define UCC_UART_UCCE_BRKS  0x0020  /**< Break start - set when the first character of a
                                         break sequence is received; multiple BRKS events are
                                         not received if a long break sequence is received. */
#define UCC_UART_UCCE_CCR   0x0008  /**< Control character received and rejected - set when a
                                         control character is recognized and stored in the
                                         receive control character register (RCCR);
                                         The character is supplied in the exceptions callback
                                         flags. */
#define UCC_UART_UCCE_BSY   0x0004  /**< Busy - set when a character is received and discarded
                                         due to a lack of empty receive buffers. */

/* @} */

/** @} */ /* end of uart_exceptions_grp group */
/** @} */ /* end of uart_grp group */

#endif /* __UCC_UART_EXT_H */

