/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          umcc_ext.h

 @Description   UMCC Application Programming Interface.

*//***************************************************************************/
#ifndef __UMCC_EXT_H
#define __UMCC_EXT_H


#include "error_ext.h"
#include "std_ext.h"
#include "qe_tdm_ext.h"
#include "mcc_ch_ext.h"


/**************************************************************************//**
 @Group         umcc_grp UMCC Application Programming Interface

 @Description   UMCC API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         umcc_init_grp UMCC Initialization Unit

 @Description   UMCC initialization unit API functions, definitions and enums.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   UMCC initialization structure, used when calling UMCC_Config().
*//***************************************************************************/
typedef struct t_UMccParam
{
    /* System integration parameters */
    e_Ucc               uccId;              /**< Id of UCC to run the UMCC                */
    uint32_t            baseAddress;        /**< Base address of UCC registers in memory  */
    t_Handle            h_Qe;               /**< Handle to the controlling QE device      */
    int                 irq;                /**< UCC interrupt ID                         */
    uint8_t             pramMemId;          /**< Parameter RAM partition ID               */
    uint32_t            maxNumOfTxSlots;    /**< Maximal number of Tx slots for this UMCC */
    uint32_t            maxNumOfRxSlots;    /**< Maximal number of Rx slots for this UMCC */
    uint16_t            maxNumOfChannels;   /**< Maximal number of channels for this UMCC */

    /* Application interface */
    t_Handle            h_App;              /**< Handle to the application                */
    t_ExceptionsRoutine *f_Exceptions;      /**< Upper layer's routine - called to report
                                                 UMCC interrupt events and errors         */
} t_UMccParam;



/**************************************************************************//**
 @Function      UMCC_Config

 @Description   Configures an UMCC device.
 
                This routine allocates the UMCC structure and sets user
                and default parameters in the UMCC driver parameters structure.

 @Param[in]     p_UMccParam - A pointer to the initializing parameter
                              structure for the UMCC.

 @Return        A handle of the newly created UMCC structure.
*//***************************************************************************/
t_Handle UMCC_Config (t_UMccParam *p_UMccParam);

/**************************************************************************//**
 @Function      UMCC_Init

 @Description   Initializes an UMCC device.

                This routine initializes the MCC structure, the global
                parameters at the parameter RAM and the memory mapped
                UMCC (UCC) registers. It clears the interrupt tables.
                When done the routine unmasks UMCC interrupts on this device.

 @Param[in]     h_UMcc - A handle to UMCC.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error  UMCC_Init(t_Handle h_UMcc);

/**************************************************************************//**
 @Group         umcc_advanced_init_grp UMCC Advanced Configuration Unit

 @Description   Configuration functions used to change default values.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UMCC bd ring pools structure, used when calling UMCC_ConfigBdRingPools().
*//***************************************************************************/
typedef struct t_UMccBdRingPoolsParam
{
    uint16_t     bdRingLength;     /**< The length of each bdRing          */
    uint16_t     sizeOfPool;       /**< The number of bdRings in each pool */
} t_UMccBdRingPoolsParam;

/**************************************************************************//**
 @Function      UMCC_ConfigFirmwareExternalStructsMemory

 @Description   Configures memory partition and attributes for the firmware's
                external structures, such as buffer descriptors.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_UMcc          - Handle to UMCC object.
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigFirmwareExternalStructsMemory(t_Handle    h_UMcc,
                                                  uint8_t     memId,
                                                  uint32_t    memAttributes);

/**************************************************************************//**
 @Function      UMCC_ConfigCommonBdRingLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Common BD ring length for all channels. [64]

 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigCommonBdRingLen (t_Handle h_UMcc, uint16_t newVal);


/**************************************************************************//**
 @Function      UMCC_ConfigRxBdRingPools

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Configure the length of each Rx bdRing and the number of Rx bdRings
                in each pool

 @Param[in]     h_UMcc              - Handle to UMCC object
 @Param[in]     numOfPools          - How many pools to create.
 @Param[in]     p_BdRingPoolsParams - An array of bdRing pools parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      - Change is applied only upon calling UMCC_Init()
                - If Total num of bdRings will not sum to num of channels, a pool of
                  bdRings of the common length will be added to sum to this number
*//***************************************************************************/
t_Error  UMCC_ConfigRxBdRingPools (t_Handle                 h_UMcc,
                                   uint8_t                  numOfPools,
                                   t_UMccBdRingPoolsParam   *p_BdRingPoolsParams);

/**************************************************************************//**
 @Function      UMCC_ConfigTxBdRingPools

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Configure the length of each Tx bdRing and the number of Tx bdRings
                in each pool

 @Param[in]     h_UMcc              - Handle to UMCC object
 @Param[in]     numOfPools          - How many pools to create.
 @Param[in]     p_BdRingPoolsParams - An array of bdRing pools parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      - Change is applied only upon calling UMCC_Init()
                - If Total num of bdRings will not sum to num of channels, a pool of
                  bdRings of the common length will be added to sum to this number
*//***************************************************************************/
t_Error  UMCC_ConfigTxBdRingPools (t_Handle                 h_UMcc,
                                   uint8_t                  numOfPools,
                                   t_UMccBdRingPoolsParam   *p_BdRingPoolsParams);

/**************************************************************************//**
 @Function      UMCC_ConfigIntrQueueSize

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                Size of the Interrupt tables. [512]

 @Param[in]     h_UMcc      - Handle to UMCC object
 @Param[in]     queueNum    - The interupt queue which size is being changed.
 @Param[in]     newVal      - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigIntrQueueSize (t_Handle     h_UMcc,
                                   uint8_t      queueNum,
                                   uint32_t     newVal);

/**************************************************************************//**
 @Function      UMCC_ConfigMaxRxBufLen

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                maximum receive buffer length. [64]

 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigMaxRxBufLen (t_Handle h_UMcc, uint16_t newVal);

/**************************************************************************//**
 @Function      UMCC_ConfigRxFramesThreshold

 @Description   Changes the internal driver data base from its default configuration.
                Please set 'newVal' according to the following description:
                received frames threshold value. [1]

 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigRxFramesThreshold (t_Handle h_UMcc, uint32_t newVal);

/**************************************************************************//**
 @Function      UMCC_ConfigNumOfRxIntrQueues

 @Description   Number of receive interrupts queues (1-4).
 
                No real priority is implemented by this division. First
                queues will be handled before others, but all Rx related
                interrupts will be handled before first queue will be
                handled again. [1]

 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     newVal  - New value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigNumOfRxIntrQueues (t_Handle h_UMcc, uint8_t newVal);

/**************************************************************************//**
 @Function      UMCC_ConfigSlotsSizes

 @Description   Set new slot sizes for Tx routing table and/or Rx routing table.
                [8]

 @Param[in]     h_UMcc      - Handle to UMCC object
 @Param[in]     p_SlotsSize - New slots sizes.
 @Param[in]     dir         - Routing type to update (transmit, receive or both)

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Change is applied only upon calling UMCC_Init().
*//***************************************************************************/
t_Error  UMCC_ConfigSlotsSizes (t_Handle    h_UMcc,
                                uint8_t     *p_SlotsSize,
                                e_CommMode  dir);

/** @} */ /* end of umcc_advanced_init_grp group */

/** @} */ /* end of umcc_init_grp group */


/**************************************************************************//**
 @Group         umcc_control_grp UMCC Runtime Control Unit

 @Description   UMCC runtime control unit API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UMCC slot parameters structure
*//***************************************************************************/
typedef struct
{
    uint8_t     chId;
    bool        enable;
 
} t_UMccSlotParam;

/**************************************************************************//**
 @Function      UMCC_Enable

 @Description   Enables the UMCC object for transmit and/or receive.

 @Param[in]     h_UMcc   - Handle to UMCC object
 @Param[in]     commMode - Communication mode (transmit, receive or both)

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UMCC_Enable (t_Handle h_UMcc, e_CommMode commMode);

/**************************************************************************//**
 @Function      UMCC_Disable

 @Description   Disables the UMCC object for transmit and/or receive.

 @Param[in]     h_UMcc   - Handle to UMCC object
 @Param[in]     commMode - Communication mode (transmit, receive or both)

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UMCC_Disable (t_Handle h_UMcc, e_CommMode commMode);

/**************************************************************************//**
 @Function      UMCC_RouteSlotSet

 @Description   Routes the requested Tx and/or Rx slot to the specified UMCC
                channel and enables the slot.

 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     slotNum - Number of the requested slot
 @Param[in]     dir     - Communication mode of the requested slot (transmit, receive or both)
 @Param[in]     chId    - Channel ID to be routed to this slot
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      The requested slot should be unrouted.
*//***************************************************************************/
t_Error UMCC_RouteSlotSet (t_Handle     h_UMcc,
                           uint8_t      slotNum,
                           e_CommMode   dir,
                           uint8_t      chId);

/**************************************************************************//**
 @Function      UMCC_RouteSlotEnable

 @Description   Enables the specified Tx and/or Rx slot.
 
 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     slotNum - Number of the requested slot
 @Param[in]     dir     - Communication mode of the requested slot (transmit, receive or both)

 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Upon slot re-enable special sequence should be followed.
                Please review the appropriate documentation for more details.
*//***************************************************************************/
t_Error UMCC_RouteSlotEnable (t_Handle         h_UMcc,
                              uint8_t          slotNum,
                              e_CommMode       dir);

/**************************************************************************//**
 @Function      UMCC_RouteSlotDisable

 @Description   Disables the specified Tx and/or Rx slot.
 
 @Param[in]     h_UMcc  - Handle to UMCC object
 @Param[in]     slotNum - Number of the requested slot
 @Param[in]     dir     - Communication mode of the requested slot (transmit, receive or both)

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UMCC_RouteSlotDisable (t_Handle         h_UMcc,
                               uint8_t          slotNum,
                               e_CommMode       dir);

/**************************************************************************//**
 @Function      UMCC_RouteTableSet

 @Description   Replaces the current Tx and/or Rx routing tables with new tables,
                built according the the specified parameters.
                The new table may have a different number of slots, different slots
                sizes and different routing.
 
 @Param[in]     h_UMcc       - Handle to UMCC object
 @Param[in]     dir          - Communication mode of the requested slot (transmit, receive or both)
 @Param[in]     numOfSlots   - Number of slots in the new routing table
 @Param[in]     p_SlotParams - New slot routing parameters
 @Param[in]     p_SlotSizes  - New slot sizes
 
 @Return        E_OK on success; Error code otherwise.
 
 @Cautions      Upon routing table replacement a special sequence should be followed.
                Please review the appropriate documentation for more details.
*//***************************************************************************/
t_Error UMCC_RouteTableSet (t_Handle         h_UMcc,
                            e_CommMode       dir,
                            uint32_t         numOfSlots,
                            t_UMccSlotParam  *p_SlotParams,
                            uint8_t          *p_SlotSizes);

/**************************************************************************//**
 @Function      UMCC_Free

 @Description   Free the UMCC object and all its resources

 @Param[in]     h_UMcc - Handle to UMCC object

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UMCC_Free (t_Handle h_UMcc);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UMCC_DumpRegs

 @Description   Print UMCC registers and PRAM page

 @Param[in]     h_UMcc - Handle to UMCC object

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
void UMCC_DumpRegs (t_Handle h_UMcc);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of umcc_control_grp group */

/** @} */ /* end of umcc_grp group */

#endif /* __UMCC_EXT_H */
