/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          upc_ext.h

 @Description   UTOPIA/POS Application Programming Interface
*//***************************************************************************/

#ifndef __UPC_EXT_H
#define __UPC_EXT_H

#include "std_ext.h"
#include "utopia_pos_ext.h"
#include "qe_ext.h"


/**************************************************************************//**
 @Group         upc_grp UTOPIA/POS Bus Controller API

 @Description   UTOPIA/POS Bus Controller API

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         upc_init_grp UPC Initialization Unit

 @Description   UPC Initialization API

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   UPC Operation Mode
 *//***************************************************************************/
typedef enum e_UpcOpMode
{
    e_UPC_OP_MODE_SLAVE,    /**< Slave operation mode    */
    e_UPC_OP_MODE_MASTER,   /**< Master operation mode   */
    e_UPC_OP_MODE_LOOPBACK  /**< Loopback operation mode */
}
e_UpcOpMode;


/**************************************************************************//**
 @Description   UPC Device
 *//***************************************************************************/
typedef enum e_UpcDevice
{
    e_UPC_DEVICE_1, /**< Device #1   */
    e_UPC_DEVICE_2, /**< Device #2   */
    e_UPC_DEVICE_3, /**< Device #3   */
    e_UPC_DEVICE_4, /**< Device #4   */
    e_UPC_ALL_DEVS  /**< All devices */
}
e_UpcDevice;


#define UPC_NUM_OF_PORTS_PER_DEV    32                          /**< Number of ports per device          */
#define UPC_NUM_OF_RATES_PER_DEV    4                           /**< Number of internal rates per device */


/**************************************************************************//**
 @Description   UPC Configuration Parameters
 *//***************************************************************************/
typedef struct
{
    /* System integration parameters */
    uint32_t            baseAddress;    /**< Memory mapped registers base address */
    e_Upc               id;             /**< ID                                   */
    t_Handle            h_Qe;           /**< QE handle                            */
    e_QeClock           txClock;        /**< Tx clock                             */
    e_QeClock           rxClock;        /**< Rx clock                             */
    uint32_t            clockFreq;      /**< Clock frequency                      */
 
    /* Protocol parameters */
    e_UtopiaPosProtocol protocol;       /**< Protocol       */
    e_UpcOpMode         opMode;         /**< Operation mode */
 
    struct
    {
        e_UtopiaPosDevMode  opMode;           /**< Operation mode         */
        e_UtopiaPosBusWidth busWidth;         /**< Data bus width         */
        e_Ucc               servingUccId;     /**< Serving UCC ID         */
        uint8_t             numOfActivePorts; /**< Number of active ports */
    }
    device[UPC_NUM_OF_DEVS];  /**< Attributes of each device */
}
t_UpcParam;


/**************************************************************************//**
 @Function      UPC_Config

 @Description   Configures a UPC.

 @Param[in]     p_UpcParam - UPC configuration parameters.

 @Return        UPC handle.
*//***************************************************************************/
t_Handle UPC_Config(t_UpcParam *p_UpcParam);


/**************************************************************************//**
 @Function      UPC_Init

 @Description   Initializes the UPC.

 @Param[in]     h_Upc - UPC handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_Init(t_Handle h_Upc);


/**************************************************************************//**
 @Function      UPC_Free

 @Description   Deallocates the UPC handle's related memory.

 @Param[in]     h_Upc - UPC handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_Free(t_Handle h_Upc);


/** @} */ /* end of upc_init_grp group */


/**************************************************************************//**
 @Group         upc_adv_config_grp UPC Advanced Configuration Unit

 @Description   UPC Advanced Configuration API

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   UPC Address Mode
 *//***************************************************************************/
typedef enum e_UpcAddressMode
{
    e_UPC_5_BIT_ADDRESS_MODE, /**< 5-bit address mode */
    e_UPC_6_BIT_ADDRESS_MODE  /**< 6-bit address mode */
}
e_UpcAddressMode;


/**************************************************************************//**
 @Function      UPC_ConfigIntRateClock

 @Description   Configures the internal rate clock of the UPC to any of the following options:
                1. An external clock.
                2. An internal baud-rate generator.
                3. The Tx clock itself (which is also used by default).

 @Param[in]     h_Upc          - UPC handle.
 @Param[in]     txIntRateClock - Tx internal rate clock.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_ConfigIntRateClock(t_Handle h_Upc, e_QeClock txIntRateClock);


/**************************************************************************//**
 @Function      UPC_ConfigUtopiaHecPlaceHolder

 @Description   Configures the HEC placeholder of the UPC.
                The UPC does not generate the HEC.
                Instead, it only outputs this constant value as a placeholder for the HEC.
                This value is then replaced by the ATM PHY with an actual value for the HEC.

 @Param[in]     h_Upc            - UPC handle.
 @Param[in]     txHecPlaceHolder - Tx HEC placeholder (default value: [0]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Applicable only when the UPC is configured for UTOPIA protocol.
*//***************************************************************************/
t_Error UPC_ConfigUtopiaHecPlaceHolder(t_Handle h_Upc, uint16_t txHecPlaceHolder);


/**************************************************************************//**
 @Function      UPC_ConfigPosMinSegmentSize

 @Description   Configures the minimum segment size of the UPC.
                The minimum segment size must be set to an even value.

 @Param[in]     h_Upc            - UPC handle.
 @Param[in]     txMinSegmentSize - The smallest Tx segment size of any of the PHYs that are served by the UPC (default value: [128]).
 @Param[in]     rxMinSegmentSize - The smallest Rx segment size of any of the PHYs that are served by the UPC (default value: [128]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Applicable only when the UPC is configured for POS protocol.
*//***************************************************************************/
t_Error UPC_ConfigPosMinSegmentSize(t_Handle h_Upc, uint8_t txMinSegmentSize, uint8_t rxMinSegmentSize);


/** @} */ /* end of upc_adv_config_grp group */


/**************************************************************************//**
 @Group         upc_dev_adv_config_grp UPC Device Advanced Configuration Unit

 @Description   UPC Device Advanced Configuration API

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   UPC Device Priority Mode
 *//***************************************************************************/
typedef enum e_UpcDevPriorityMode
{
    e_UPC_DEV_ROUND_ROBIN_PRIORITY_MODE, /**< Round-Robin priority mode */
    e_UPC_DEV_FIXED_PRIORITY_MODE        /**< Fixed priority mode       */
}
e_UpcDevPriorityMode;


/**************************************************************************//**
 @Function      UPC_ConfigDevPriorityMode

 @Description   Configures the priority mode of a device on the UPC.

 @Param[in]     h_Upc          - UPC handle.
 @Param[in]     deviceId       - Device ID.
 @Param[in]     txPriorityMode - Tx priority mode (default value: [e_UPC_DEV_ROUND_ROBIN_PRIORITY_MODE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_ConfigDevPriorityMode(t_Handle h_Upc, e_UpcDevice deviceId, e_UpcDevPriorityMode txPriorityMode);


/**************************************************************************//**
 @Description   UPC Device Rate Mode
 *//***************************************************************************/
typedef enum e_UpcDevRateMode
{
    e_UPC_DEV_EXTERNAL_RATE_MODE, /**< External rate mode */
    e_UPC_DEV_INTERNAL_RATE_MODE  /**< Internal rate mode */
}
e_UpcDevRateMode;


/**************************************************************************//**
 @Function      UPC_ConfigDevRateMode

 @Description   Configures the rate mode of a device on the UPC.

 @Param[in]     h_Upc      - UPC handle.
 @Param[in]     deviceId   - Device ID.
 @Param[in]     txRateMode - Tx rate mode (default value: [e_UPC_DEV_EXTERNAL_RATE_MODE]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Internal-Rate mode is not recommended when transmitting variable length POS frames.
*//***************************************************************************/
t_Error UPC_ConfigDevRateMode(t_Handle h_Upc, e_UpcDevice deviceId, e_UpcDevRateMode txRateMode);


/**************************************************************************//**
 @Description   UPC Device Internal Rate Parameters
 *//***************************************************************************/
typedef struct
{
    uint32_t txIntRates[UPC_NUM_OF_RATES_PER_DEV]; /**< Tx internal rates       */
    uint8_t  txExpirationCounter;                  /**< Tx expiration counter   */
    uint16_t txPosFixedFrameSize;                  /**< Tx POS fixed-frame size */
}
t_UpcDevIntRateParams;


/**************************************************************************//**
 @Function      UPC_ConfigDevIntRateParams

 @Description   Configures the internal rate parameters of a device on the UPC.

 @Param[in]     h_Upc             - UPC handle.
 @Param[in]     deviceId          - Device ID.
 @Param[in]     p_TxIntRateParams - Tx internal rate parameters:
                                    txIntRates          - Tx internal rates       (default value: [0], 0, 0, 0).
                                    txExpirationCounter - Tx expiration counter   (default value: [0]).
                                    txPosFixedFrameSize - Tx POS fixed-frame size (default value: [0]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      For UTOPIA, the Tx expiration counter can be set to any value between 1 and 15,
                and its recommended value is 7. For POS, the Tx expiration counter must be set to 1.
*//***************************************************************************/
t_Error UPC_ConfigDevIntRateParams(t_Handle h_Upc, e_UpcDevice deviceId, t_UpcDevIntRateParams* p_TxIntRateParams);


/**************************************************************************//**
 @Function      UPC_ConfigDevBackToBack

 @Description   Configures the Back-To-Back functionality of a device on the UPC.

 @Param[in]     h_Upc       - UPC handle.
 @Param[in]     deviceId    - Device ID.
 @Param[in]     txBtbEnable - Tx back to back enabled (default value: [FALSE]).
 @Param[in]     rxBtbEnable - Rx back to back enabled (default value: [FALSE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_ConfigDevBackToBack(t_Handle h_Upc, e_UpcDevice deviceId, bool txBtbEnable, bool rxBtbEnable);


/**************************************************************************//**
 @Function      UPC_ConfigDevParityCheck

 @Description   Configures the Parity-Check functionality of a device on the UPC.

 @Param[in]     h_Upc       - UPC handle.
 @Param[in]     deviceId    - Device ID.
 @Param[in]     rxPchEnable - Rx parity check enabled (default value: [TRUE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_ConfigDevParityCheck(t_Handle h_Upc, e_UpcDevice deviceId, bool rxPchEnable);


/**************************************************************************//**
 @Function      UPC_ConfigDevUtopiaIdleCellsDiscard

 @Description   Configures the Idle-Cells-Discard functionality of a device on the UPC.

 @Param[in]     h_Upc       - UPC handle.
 @Param[in]     deviceId    - Device ID.
 @Param[in]     rxIcdEnable - Rx idle cells discard enabled (default value: [TRUE]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Applicable only when the UPC is configured for UTOPIA protocol.
*//***************************************************************************/
t_Error UPC_ConfigDevUtopiaIdleCellsDiscard(t_Handle h_Upc, e_UpcDevice deviceId, bool rxIcdEnable);


/**************************************************************************//**
 @Description   UPC Device User-Defined-Cells Configuration Parameters
 *//***************************************************************************/
typedef struct
{
    bool    txEnable;   /**< Tx user defined cells enabled               */
    bool    rxEnable;   /**< Rx user defined cells enabled               */
    uint8_t txExhSize;  /**< Tx user defined cells extra header size     */
    uint8_t rxExhSize;  /**< Rx user defined cells extra header size     */
    bool    hecInclude; /**< User defined cells HEC included (Tx and Rx) */
    bool    rxHecCheck; /**< Rx user defined cells HEC checked           */
    bool    rxHecCoset; /**< Rx user defined cells HEC xor'ed with 0x55  */
}
t_UpcDevUdcParam;


/**************************************************************************//**
 @Function      UPC_ConfigDevUtopiaUserDefinedCells

 @Description   Configures the User-Defined-Cells functionality of a device on the UPC.
                The user defined cells extra header size can be set to any value between 1 and 12.

 @Param[in]     h_Upc      - UPC handle.
 @Param[in]     deviceId   - Device ID.
 @Param[in]     p_UdcParam - User defined cells configuration parameters:
                             txEnable   - Tx user defined cells enabled              (default value: [FALSE]).
                             rxEnable   - Rx user defined cells enabled              (default value: [FALSE]).
                             txExhSize  - Tx user defined cells extra header size    (default value: [0]).
                             rxExhSize  - Rx user defined cells extra header size    (default value: [0]).
                             hecInclude - User defined cells HEC included            (default value: [FALSE]).
                             rxHecCheck - Rx user defined cells HEC checked          (default value: [FALSE]).
                             rxHecCoset - Rx user defined cells HEC xor'ed with 0x55 (default value: [FALSE]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Applicable only when the UPC is configured for UTOPIA protocol.
*//***************************************************************************/
t_Error UPC_ConfigDevUtopiaUserDefinedCells(t_Handle h_Upc, e_UpcDevice deviceId, t_UpcDevUdcParam* p_UdcParam);


/** @} */ /* end of upc_dev_adv_config_grp group */


/**************************************************************************//**
 @Group         upc_runtime_ctrl_grp UPC Runtime Control Unit

 @Description   UPC Runtime Control API

 @{
*//***************************************************************************/


/**************************************************************************//*
 @Function      UPC_GetId

 @Description   Returns the UPC ID.

 @Param[in]     h_Upc - UPC handle.

 @Return        UPC ID.
*//***************************************************************************/
uint8_t UPC_GetId(t_Handle h_Upc);


/**************************************************************************//*
 @Function      UPC_GetDevTxDatagramSize

 @Description   Returns the Tx datagram size of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.

 @Return        Tx datagram size.
*//***************************************************************************/
uint16_t UPC_GetDevTxDatagramSize(t_Handle h_Upc, e_UpcDevice deviceId);


/**************************************************************************//*
 @Function      UPC_GetDevRxDatagramSize

 @Description   Returns the Rx datagram size of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.

 @Return        Rx datagram size.
*//***************************************************************************/
uint16_t UPC_GetDevRxDatagramSize(t_Handle h_Upc, e_UpcDevice deviceId);


/**************************************************************************//*
 @Function      UPC_GetDevRateMode

 @Description   Returns the rate mode of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.

 @Return        Tx rate mode.
*//***************************************************************************/
e_UpcDevRateMode UPC_GetDevRateMode(t_Handle h_Upc, e_UpcDevice deviceId);


/**************************************************************************//*
 @Function      UPC_GetUccNumOfPortsPerDev

 @Description   Returns the number of ports running over a device on the UPC, which is served by a given UCC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.
 @Param[in]     uccId    - UCC ID.

 @Return        Rx datagram size.
*//***************************************************************************/
uint8_t UPC_GetUccNumOfPortsPerDev(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t uccId);


/**************************************************************************//*
 @Function      UPC_EnablePort

 @Description   Enables a port of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.
 @Param[in]     portId   - Port ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_EnablePort(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t portId);


/**************************************************************************//*
 @Function      UPC_DisablePort

 @Description   Disables a port of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.
 @Param[in]     portId   - Port ID.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_DisablePort(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t portId);


/**************************************************************************//**
 @Description   UPC Port Priority
 *//***************************************************************************/
typedef enum e_UpcPortPriority
{
    e_UPC_PORT_HIGH_PRIORITY, /**< High priority */
    e_UPC_PORT_LOW_PRIORITY   /**< Low priority  */
}
e_UpcPortPriority;


/**************************************************************************//*
 @Function      UPC_SetPortPriority

 @Description   Changes the priority of a port of a device on the UPC.

 @Param[in]     h_Upc      - UPC handle.
 @Param[in]     deviceId   - Device ID.
 @Param[in]     portId     - Port ID.
 @Param[in]     rxPriority - Rx priority.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_SetPortPriority(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t portId, e_UpcPortPriority rxPriority);


/**************************************************************************//*
 @Function      UPC_SetPortIntRate

 @Description   Changes the internal rate of a port of a device on the UPC.

 @Param[in]     h_Upc     - UPC handle.
 @Param[in]     deviceId  - Device ID.
 @Param[in]     portId    - Port ID.
 @Param[in]     txIntRate - Tx internal rate.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UPC_SetPortIntRate(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t portId, uint32_t txIntRate);


/**************************************************************************//*
 @Function      UPC_GetPortUnderrunEventIndication

 @Description   Reports whether an underrun event has occurred for a port of a device on the UPC.

 @Param[in]     h_Upc    - UPC handle.
 @Param[in]     deviceId - Device ID.
 @Param[in]     portId   - Port ID.

 @Return        TRUE if an underrun event has occurred; FALSE otherwise.
*//***************************************************************************/
bool UPC_GetPortUnderrunEventIndication(t_Handle h_Upc, e_UpcDevice deviceId, uint8_t portId);


#if (DEBUG_ERRORS > 0)


/**************************************************************************//**
 @Function      UPC_DumpRegs

 @Description   Prints the contents of the memory mapped registers of the UPC.

 @Param[in]     h_Upc - UPC handle.
 
 @Return        None.
*//***************************************************************************/
void UPC_DumpRegs(t_Handle h_Upc);


#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of upc_runtime_ctrl_grp group */


/** @} */ /* end of upc_grp group */


#endif /* __UPC_EXT_H */
