/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          utopia_port_ext.h

 @Description   External header file for utopia port qe unit
 
 @Cautions      None.

*//***************************************************************************/

/**************************************************************************//**
@Group         ucc_atm_grp UCC ATM Application Programming Interface

@Description   Ucc Atm API functions, definitions and enums

@{
*//***************************************************************************/

#ifndef __UTOPIA_PORT_EXT_H
#define __UTOPIA_PORT_EXT_H

#include "std_ext.h"
#include "ucc_atm_port_ext.h"
#include "upc_ext.h"

/**************************************************************************//**
 @Group         utopia_port_grp UTOPIA Port

 @Description   Utopia Port Qe

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         utopia_port_init_grp UTOPIA Port Initialization Unit

 @Description   Utopia Port Qe initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   APC levels in the scheduling table.
 *//***************************************************************************/
typedef enum e_UtopiaPortSchedLevels
{
      e_UTOPIA_PORT_SCHED_LEVEL_1 = e_UCC_ATM_PORT_SCHED_LEVEL_1
    , e_UTOPIA_PORT_SCHED_LEVEL_2 = e_UCC_ATM_PORT_SCHED_LEVEL_2
    , e_UTOPIA_PORT_SCHED_LEVEL_3 = e_UCC_ATM_PORT_SCHED_LEVEL_3
    , e_UTOPIA_PORT_SCHED_LEVEL_4 = e_UCC_ATM_PORT_SCHED_LEVEL_4
    , e_UTOPIA_PORT_SCHED_LEVEL_5 = e_UCC_ATM_PORT_SCHED_LEVEL_5
    , e_UTOPIA_PORT_SCHED_LEVEL_6 = e_UCC_ATM_PORT_SCHED_LEVEL_6
    , e_UTOPIA_PORT_SCHED_LEVEL_7 = e_UCC_ATM_PORT_SCHED_LEVEL_7
    , e_UTOPIA_PORT_SCHED_LEVEL_8 = e_UCC_ATM_PORT_SCHED_LEVEL_8
    , e_UTOPIA_PORT_TOTAL_SCHED_LEVELS = e_UCC_ATM_PORT_TOTAL_SCHED_LEVELS/* Must be last */
} e_UtopiaPortSchedLevels;

#define UTOPIA_MAX_GCRA_LEVELS 4

/**************************************************************************//**
 @Description   UPC Port Priority
 *//***************************************************************************/
typedef enum e_UtopiaPortUpcPriority
{
    e_UTOPIA_PORT_UPC_HIGH_PRIORITY = e_UCC_ATM_PORT_UPC_HIGH_PRIORITY, /**< High priority */
    e_UTOPIA_PORT_UPC_LOW_PRIORITY  = e_UCC_ATM_PORT_UPC_LOW_PRIORITY   /**< Low priority  */
 
} e_UtopiaPortUpcPriority;

/**************************************************************************//**
 @Description   Utopia Port Qe (PHY) initialization structure.
 *//***************************************************************************/
typedef struct
{
    t_Handle    h_App;          /**< Port application handle */
    uint8_t     port;           /**< Ucc Atm port number (0-31) */
    t_Handle    h_RxCommCtl;    /**< Receive communication controller */
    t_Handle    h_TxCommCtl;    /**< Transmit communication controller */
    t_Handle    h_AtmCtl;       /**< Bus controller handler */
    e_UpcDevice atmCtlPhyDevId; /**< Bus controller phy device ID */

    /* Scheduler ( Apc / Gcra ) variables */
    uint32_t    txRate; /**< For internal-rate mode, one of the 4 internal rates of the port's UPC-Device (in bps)
                             For external-rate mode, the line rate of the optical carrier (in bps) */

    uint8_t     schedLevels;    /**< Number of the required APC(Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;
                                /**< The fast channel rate (in bits per second) */
    uint32_t    slowChannelRateBitPerSec[e_UTOPIA_PORT_TOTAL_SCHED_LEVELS];
                                /**< The slow channel rate per priority level (in bits per second) */
} t_UtopiaPortParam;

/**************************************************************************//**
 @Function      UTOPIA_PORT_Config

 @Description   Configures an Utopia Port based on user parameters and set
                to default some UTOPIA port parameters.

                The UTOPIA port handle can then be initialized using UTOPIA_PORT_Init()

 @Param[in]     p_UtopiaPortParam - user parameters for Utopia Atm Port.

 @Return        Handle to created UTOPIA Port if success, NULL otherwise.

 @Cautions      Does not affect hardware, just internal driver structures.
*//***************************************************************************/
t_Handle UTOPIA_PORT_Config(t_UtopiaPortParam *p_UtopiaPortParam);

/**************************************************************************//**
 @Function      UTOPIA_PORT_Init

 @Description   Initializes driver and hardware for UTOPIA port.

                The initialization includes:
                   - Initialize shaper (APC or GCRA)
                   - registers initialization
 
 @Param[in]     h_UtopiaPort - UTOPIA Port handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      utopiaPort must have been previously created using UTOPIA_PORT_Config().
*//***************************************************************************/
t_Error UTOPIA_PORT_Init (t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_Free

 @Description   Free memory of the UTOPIA port handler.

 @Param[in]     h_UtopiaPort - Utopia Port handler to be initialized.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      utopiaPort must have been previously inited using UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_Free (t_Handle h_UtopiaPort);


/**************************************************************************//**
 @Group         utopia_port_config_grp UTOPIA Port Advanced Configuration Unit

 @Description   Configuration functions used to change default values

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   APC Scheduler initialization structure
 
                For UTOPIA_PORT_ConfigVpShapingScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt[e_UTOPIA_PORT_TOTAL_SCHED_LEVELS];
                                            /**< If true - the control slot indicates that TCTE should be read for each channel;
                                                 Must be false in case of working with apc scalable mode or with traffic type which is different then CBR */

    uint8_t     maxIteration;               /**< Max iteration allowed */
    bool        burstInhibit;               /**< Indicates if enable the burst inhibit option in the scheduler_MODE field */
    uint8_t     schedScaleFactor;           /**< Scheduler scaling factor */

} t_UtopiaApcSchedParam;

/**************************************************************************//**
 @Description   Utopia Port Shaper Type.
 *//***************************************************************************/
typedef enum {
       e_UTOPIA_PORT_APC            = e_UCC_ATM_PORT_APC_SHAPER_TYPE        /**< shaper type - APC (Atm Pace Controller) */
     , e_UTOPIA_PORT_GCRA_SCHEDULER = e_UCC_ATM_PORT_GCRA_SHAPER_TYPE       /**< shaper type - GCRA( Generic Cell Rate Algorithm) */
     , e_UTOPIA_PORT_VP_SHAPING     = e_UCC_ATM_PORT_VP_SHAPING_TYPE        /**< shaper type - Vp Shaping (Virtual Path Shaping) */
} e_UtopiaPortShaperType;

/**************************************************************************//**
 @Description   Information regarding GBR traffic of channels on a GCRA port last priority
 *//***************************************************************************/
typedef enum
{
    e_UTOPIA_GCRA_NO_GBR_MODE      = e_UCC_ATM_GCRA_NO_GBR_MODE,     /**< There are no GBR channels in this GCRA port */
    e_UTOPIA_GCRA_MIXED_GBR_MODE   = e_UCC_ATM_GCRA_MIXED_GBR_MODE,  /**< Some of the channels on last priority work with GBR traffic */
    e_UTOPIA_GCRA_FULL_GBR_MODE    = e_UCC_ATM_GCRA_FULL_GBR_MODE    /**< All channels on last priority work with GBR traffic */
 
} e_UtopiaGcraGbrMode;

/**************************************************************************//**
 @Description   GCRA Scheduler initialization structure.
 
                For UTOPIA_PORT_ConfigVpShapingScheduler and UTOPIA_PORT_ConfigGcraScheduler.
 *//***************************************************************************/
typedef struct
{
    bool        nonExtendedCt;      /**< No extended connection table needed for this port's channels */
    bool        isInternal;         /**< Indication if the user asked to work with internal/external channels in this port */
    uint8_t     gcraChannels;       /**< The actual number of gcra channels per the specific port */
    bool        workConservingMode; /**< Work Conserving Mode - Vps on the last priority are scheduled according to their
                                         weighted PCR/PCR Fraction regardless of the system clock */
    uint8_t     *p_WrrPriPercentages;/**< Array of up to 4 WRR (Weighted Round Robin) priorities percentages;
                                         Relevant only for WRR Mode, in which selection between the up to 4 priorities
                                         is done according to their relative weights; Note: For regular fixed priority mode
                                         (non-WRR) this pointer should be NULL. */
    e_UtopiaGcraGbrMode gbrMode;    /**< Full GBR mode / Mixed GBR mode / No GBR mode */

} t_UtopiaGcraSchedParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level APC initialization structure.

                This structurte includes the parameters needed for a single
                APC parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    vpi;                        /**< Virtual Path Identifier of this VP group channels */
 
	/* Scheduling level */
	e_UtopiaPortSchedLevels	schedLevel;	    /**< Scheduling level in high level GCRA scheduler */

    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[e_UTOPIA_PORT_TOTAL_SCHED_LEVELS];
                                            /**< The rate of the slow channel - bits per second per priority level */

    /* APC specific parameters */
    t_UtopiaApcSchedParam apcSchedParam;

} t_UtopiaLowLevelApcParam;

/**************************************************************************//**
 @Description   VP Shaping Low Level GCRA initialization structure.

                This structurte includes the parameters needed for a single
                GCRA parameter table under a VP Shaping port.
 *//***************************************************************************/
typedef struct
{
    /* VP identifier */
    uint16_t    				vpi;		/**< Virtual Path Identifier of this VP group channels */

	/* Scheduling level */
	e_UtopiaPortSchedLevels	schedLevel; 	/**< Scheduling level in high level GCRA scheduler */

    /* Basic scheduler parameters, the same passed to port configuration
       both for regular APC and for regular GCRA initialization : */
    uint32_t    pcrBitPerSec;               /**< Peak cell rate (in bit per second); ( Pcr*CellsPerSlot <= LineRate ) */
    uint8_t     schedLevels;                /**< number of the required APC (Atm Pace Controller) levels */
    uint32_t    fastChannelRateBitPerSec;   /**< The rate of the fast channel - bits per second */
    uint32_t    slowChannelRateBitPerSec[UTOPIA_MAX_GCRA_LEVELS];
                                            /**< The rate of the slow channel per priority level (in bits per second) */

    /* GCRA specific parameters */
    t_UtopiaGcraSchedParam gcraSchedParam;

} t_UtopiaLowLevelGcraParam;

/**************************************************************************//**
 @Description   VP Shaping Scheduling initialization structure.
 *//***************************************************************************/
typedef struct
{
    uint8_t                         vpsNum;             /**< Number of Vps under this Vp Shaping port */
    bool                            autoVpOnOff;        /**< Auto Vp On Off enable */
    t_UtopiaLowLevelApcParam        *p_LowLevelApcSchedParam;
                                                        /**< If not NULL => GCRA over APC Vp Shaping (Low level
                                                        scheduling type is APC) and this is the array of all
                                                        low level APC schedulers parameters */
    t_UtopiaLowLevelGcraParam       *p_LowLevelGcraSchedParam;
                                                        /**< If not NULL => GCRA over GCRA Vp Shaping (Low level
                                                        scheduling type is GCRA) and this is the array of all
                                                        low level GCRA schedulers parameters */
} t_UtopiaPortVpShapingParam;

/**************************************************************************//**
 @Description   Mini-Cam parameters structure.
 *//***************************************************************************/
typedef struct
{
    bool                        uniMode;       /**< Indicates if to take the GFC value into the CAM search or mask it
                                                    FALSE - take the GFC field,
                                                    TRUE - mask the GFC field in the incoming cell. */
    bool                        isOverlapping; /**< Indication if the specific port belongs to an overlap group. */
    uint8_t                     overlapGroup;  /**< The number of the overlap group
                                                    (relevant only if isOverlapping parameter is set). */
    e_MaxChannelsPerMiniCamPort numOfChannels; /**< The actual number of how many channels are going to work under
                                                    this Mini-Cam port;
                                                    NOTE: the parameter is important to improve performance
                                                    and also to search for an available channel code. */
} t_UtopiaPortMiniCamParam;


/**************************************************************************//**
 @Group         utopia_port_config_apc_grp UTOPIA Port APC Advanced Configuration Unit

 @Description   Configuration functions used to change APC default values

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigMaxIteration

 @Description   Changes the max iteration value from its default configuration.

                Max iteration allowed;
                defualt value:[8].

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     maxIteration - new max iteration value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().
*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigMaxIteration (t_Handle h_UtopiaPort, uint8_t maxIteration);

/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigBurstInhibit

 @Description   Configures the burst inhibit option.

                Default value: [FALSE].
 
 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     burstInhibit  - TRUE to set the burst inhibit
                                     FALSE to unset burst inhibit.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigBurstInhibit(t_Handle h_UtopiaPort, bool burstInhibit);


/** @} */ /* end of utopia_port_config_apc_grp group */

/**************************************************************************//**
 @Group         utopia_port_config_gcra_grp UTOPIA Port GCRA Advanced Configuration Unit

 @Description   Configuration functions used to change GCRA default values

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigGcraScheduler

 @Description   Configures a Gcra APC scheduler.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     p_UtopiaPortGcraSchedParam  - GCRA parameters structure.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigGcraScheduler(t_Handle h_UtopiaPort, t_UtopiaGcraSchedParam *p_UtopiaPortGcraSchedParam);

/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigVpShapingScheduler

 @Description   Configures Vp Shaping mode.

 @Param[in]     h_UtopiaPort                - UCC ATM Port handle.
 @Param[in]     p_UtopiaPortVpShapingParam  - Vp Shaping parameters.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before
                UTOPIA_PORT_Init().
*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigVpShapingScheduler(t_Handle h_UtopiaPort, t_UtopiaPortVpShapingParam *p_UtopiaPortVpShapingParam);

/** @} */ /* end of utopia_port_config_gcra_grp group */

/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigNonExtendedCt

 @Description   Configures a nonExtendedCt parameter for a specific scheduling table.

                Default value:[FALSE].
 
 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     schedLevel - which apc level to update.
 @Param[in]     nonExtendedCt  - new value to set to nonExtendedCt parameter.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigNonExtendedCt(t_Handle h_UtopiaPort, e_UtopiaPortSchedLevels schedLevel, bool nonExtendedCt);

/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigMiniCam

 @Description   Configures Mini Cam parameter for the specific port.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     p_MiniCamParam - mini cam parameters structure.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigMiniCam(t_Handle h_UtopiaPort, t_UtopiaPortMiniCamParam *p_MiniCamParam);

/**************************************************************************//**
 @Function      UTOPIA_PORT_ConfigTimeOutMicroSec

 @Description   Configures the time to wait until a channel stop request is finished.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     timeOut - the time to wait until a stop request is finished in micro seconds.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only following UTOPIA_PORT_Config() and before UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_ConfigTimeOutMicroSec (t_Handle h_UtopiaPort, uint16_t timeOut);

/** @} */ /* end of utopia_port_config_grp group */
/** @} */ /* end of utopia_port_init_grp group */

/**************************************************************************//**
 @Group         utopia_port_control_grp UTOPIA Port Control Unit

 @Description   Utopia Port Qe control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      UTOPIA_PORT_EnablePolicer

 @Description   Enable the work with policer for a specific port;
                Should be called only in case of Mini-Cam ports.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UTOPIA_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UTOPIA_PORT_EnablePolicer (t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_DisablePolicer

 @Description   Disable the work with policer for a specific port;
                Should be called only in case of Mini-Cam ports.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UTOPIA_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UTOPIA_PORT_DisablePolicer (t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_Enable

 @Description   Enable the port.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_Enable(t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_Disable

 @Description   Disable the port.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_Disable(t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_EnableMiniCamVpSwitch

 @Description   Enables Mini-Cam VP Switch mode for all of the channels of h_UtopiaPort
                that are configured with the passed VPI value.

 @Param[in]     h_UtopiaPort    - handle to Port object.
 @Param[in]     vpi             - the VPI value.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      Allowed only after UCC_ATM_PORT_Init() and only for a Mini-CAM port.

*//***************************************************************************/
t_Error UTOPIA_PORT_EnableMiniCamVpSwitch(t_Handle h_UtopiaPort, uint16_t vpi);

/**************************************************************************//**
 @Function      UTOPIA_PORT_DisableMiniCamVpSwitch

 @Description   Disables Mini-Cam VP Switch mode for all of the channels of h_UtopiaPort
                that are configured with the passed VPI value.

 @Param[in]     h_UtopiaPort    - handle to Port object.
 @Param[in]     vpi             - the VPI value.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error UTOPIA_PORT_DisableMiniCamVpSwitch(t_Handle h_UtopiaPort, uint16_t vpi);

/**************************************************************************//**
 @Function      UTOPIA_PORT_SetPortPriority

 @Description   Set the port Rx priority in the UPC;
                Default value for all ports at init point is High Priority.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.
 @Param[in]     rxPriority   - The Rx priority in UPC.

 @Return        E_OK in case of success otherwise error Code.

 @Cautions      Allowed only after UTOPIA_PORT_Init().

*//***************************************************************************/
t_Error UTOPIA_PORT_SetPortPriority(t_Handle h_UtopiaPort, e_UtopiaPortUpcPriority rxPriority);

/**************************************************************************//**
 @Function      UTOPIA_PORT_GetPortUnderrunEventIndication

 @Description   Returns the TRUE if underrun occurred and clears the register.

 @Param[in]     h_UtopiaPort - handle to Utopia Port object.

 @Return        TRUE if port Tx underrun occurred; FALSE otherwise.

 @Cautions      Allowed only after UTOPIA_PORT_Init().

*//***************************************************************************/
bool UTOPIA_PORT_GetPortUnderrunEventIndication(t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_IsAlive

 @Description   Checks if the phy is working and there is a traffic or not.
 
                To be used by the user application for verifying the phy is still
                alive in case of Channel transmit operation or Tx Channel stop failure.

 @Param[in]     h_UtopiaPort - handle to Port object.

 @Return        TRUE if the phy is working, FALSE - otherwise.

 @Cautions      The routine is relevant only in case that there are active channels
                in the system. Otherwize, there is no indication if the phy is alive.

*//***************************************************************************/
bool UTOPIA_PORT_IsAlive(t_Handle h_UtopiaPort);

/**************************************************************************//**
 @Function      UTOPIA_PORT_ForceTxChannelsDisable

 @Description   Force disable of all the channels of the specific port.
 
                To be used by the user application in case there is a phy error
                (for example - if UTOPIA_PORT_IsAlive() returned FALSE) for termination of the
                channels of this port.
                Note :A regular channel disable routine ( UCC_ATM_CH_Disable() or
                UCC_ATM_CH_DisableAsync() ) will not work in case of a phy error,
                because the microcode stop command depends on the APC/GCRA scheduling.
                Therefore, this routine is required.

 @Param[in]     h_UtopiaPort - handle to Port object.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      The routine has nothing to do in case that the port has some
                channels that work with auto VcOn mode since that there is a chance
                that the channels will be removed from the scheduler by the driver
                and the ucode will insert those channels again to the scheduler
                due to this special Auto VcOn mode.

*//***************************************************************************/
t_Error UTOPIA_PORT_ForceTxChannelsDisable(t_Handle h_UtopiaPort);

#if (DEBUG_ERRORS > 0)
/**************************************************************************//**
 @Function      UTOPIA_PORT_DumpRegs

 @Description   Dumps port registers and parameters RAM.

 @Param[in]     h_UtopiaPort - handle to Port object.

 @Return        None.

 @Cautions      h_UtopiaPort must have been previously created using UTOPIA_PORT_Config()
                and initialized using UTOPIA_PORT_Init().
*//***************************************************************************/
void UTOPIA_PORT_DumpRegs(t_Handle h_UtopiaPort);
#endif /* (DEBUG_ERRORS > 0) */

/** @} */ /* end of utopia_port_control_grp group */
/** @} */ /* end of utopia_port_grp group */
/** @} */ /* end of group ucc_atm_grp */

#endif /* __UTOPIA_PORT_EXT_H */
