/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          vp_ext.h

 @Description   VP driver API - structures and functions.

 @Cautions      None.
*/

#ifndef __VP_EXT_H
#define __VP_EXT_H

#include "error_ext.h"
#include "qe_ext.h"


/**
@Group         vp_api Virtual Port Application Programming Interface

@Description   Virtual Port API functions, structures, definitions and enums

@{
*/


/**
@Group         vp_dev_grp VP_DEV Object Application Programming Interface

@Description   VP_DEV object API functions, structures, definitions and enums

@{
*/




/**
@Group         vp_dev_init_grp VP_DEV Object Initialization Unit

@Description   VP_DEV API Initialization Unit

@{
*/

/**
 @Description   Type definition for user callback function called by the VP driver for
                reporting overflow exceptions.

 @Param[in]     h_VpDev         - Handle to VP_DEV object
 @Param[in]     interruptQueue  - Interrupt queue for which overflow has occured
 
 @Return        N/A
*/
typedef void (t_VpCpuSwapOvExceptions)(t_Handle h_VpDev, uint8_t interruptQueue);

/**
 @Description   Configuration parameters from the upper layer
*/
typedef struct t_VpDevParam
{
	t_Handle                h_Qe;                   /**< Handle to the QE object */
	uint8_t                 pramMemId;              /**< Parameter RAM partition ID */
    t_BufferPoolInfo        bufferPoolInfo;         /**< Data buffer pool information */
    uint32_t                dataMemAttributes;      /**< Memory attributes mask for data buffers */
    t_VpCpuSwapOvExceptions *f_VpCpuSwapExceptions; /**< User-provided callback for CPU Swap overflow exception reporting */

} t_VpDevParam;


/**
 @Function      VP_DEV_Config

 @Description   Configures the VP_DEV object. Returns handle to the configured
                VP_DEV object or NULL in case of an error.

 @Param[in]     p_VpDevParams -  User parameters to initialize VP_DEV object

 @Return        t_Handle
*/
t_Handle VP_DEV_Config(t_VpDevParam *p_VpDevParams);


/**
 @Function      VP_DEV_Init

 @Description   Initialize dev structures of VP. Returns status of operation

 @Param[in]     h_VpDev -  Handle to VP_DEV object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_DEV_Init(t_Handle h_VpDev);


/**
 @Function      VP_DEV_Free

 @Description   Release the resources of the VP_DEV object.

 @Param[in]     h_VpDev -  Handle to VP_DEV object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_DEV_Free(t_Handle h_VpDev);

/** @} */ /* end of vp_dev_init_grp */


/**
@Group         vp_dev_init_advanced_grp VP_DEV Object Advanced Initialization Unit

@Description   VP_DEV API Advanced Initialization Unit

@{
*/

/**
 @Description   Configuration parameters for CPU Swap Interrupt Queues
*/
typedef struct t_VpCpuSwapInterruptQueueParam
{
    uint16_t        numEntries;         /**< Number of interrupt queue entries */
    uint16_t        coalescingVal;      /**< Interrupt coalescing value, relevant in interrupt mode */
    bool            interruptEnabled;   /**< Interrupt mode, TRUE to enable */
 
} t_VpCpuSwapInterruptQueueParam;


/**
 @Function      VP_DEV_ConfigPollingTimerPeriod

 @Description   Configures Virtual Port polling timer period
				[1024]
	
 @Param[in]     h_VpDev            -  Handle to VP_DEV object
 @Param[in]     pollingTimerPeriod -  Timer period for polling mode

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigPollingTimerPeriod(t_Handle h_VpDev, uint32_t pollingTimerPeriod);


/**
 @Function      VP_DEV_ConfigFirmwareExternalStructsMemory

 @Description   Configures the memory where buffer descriptors are located.
                [0 (default heap), MEMORY_ATTR_CACHEABLE]

 @Param[in]     h_VpDev         - Handle to VP_DEV object
 @Param[in]     memId           - Memory partition ID.
 @Param[in]     memAttributes   - Memory attributes mask (a combination of MEMORY_ATTR_x flags).

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigFirmwareExternalStructsMemory(t_Handle  h_VpDev,
                                                    uint8_t  memId,
                                                    uint32_t memAttributes);


/**
 @Function      VP_DEV_ConfigCpuSwapInterruptQueues

 @Description   Configures the Interworking CPU Swap Interrupt Queues.
				[1]
	
 @Param[in]     h_VpDev              - Handle to VP_DEV objectt
 @Param[in]     numOfInterruptQueues - Number of Interrupt Queues to be configured;
                                       up to 16 Interrupt Queues are possible
 @Param[in]     p_InterruptQueueParams
                                    -  Pointer to array of size numOfInterruptQueues containing
                                       queue configuration parameters for every Interrupt Queue

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigCpuSwapInterruptQueues(t_Handle                        h_VpDev,
                                            uint8_t                         numOfInterruptQueues,
                                            t_VpCpuSwapInterruptQueueParam  *p_InterruptQueueParams);


/**
 @Function      VP_DEV_ConfigNumOfIntCpuSwapQueues
	
 @Description   Configures the total number of VP CPU Swap Queues allocated
                in internal memory
				[0]

 @Param[in]     h_VpDev               - Handle to VP_DEV object
 @Param[in]     numOfIntCpuSwapQueues - Number of CPU Swap Queues allocated
                                        in internal memory

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigNumOfIntCpuSwapQueues(t_Handle h_VpDev,
                                           uint16_t numOfIntCpuSwapQueues);


/**
 @Function      VP_DEV_ConfigNumOfExtCpuSwapQueues

 @Description   Configures the total number of VP CPU Swap Queues allocated
                in external memory
				[0]
	
 @Param[in]     h_VpDev               - Handle to VP_DEV object
 @Param[in]     numOfExtCpuSwapQueues - Number of CPU Swap Queues allocated
                                        in external memory

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigNumOfExtCpuSwapQueues(t_Handle h_VpDev,
                                           uint16_t numOfExtCpuSwapQueues);


/**
 @Function      VP_DEV_ConfigNumOfReservedIntCpuSwapQueues

 @Description   Configures the number of internal CPU Swap Queues  that are reserved;
                reserved CPU Swap queues are queues reserved on VP_DEV object that will not
                be managed by Virtual Port instances.
 
                The number of queues that can be reserved is dependent on IPSec and IP Reassembly
                modules. If IPSec module is configured to use one internal CPU Swap queue for debug
                purposes, than queue id 0 must be reserved to be used by IP Sec. If IP Reassembly
                module is configured to use one internal CPU Swap Queue for debug purposes, than queue
                id 1 (or queue id 0 if it hasn't been reserved for IP Sec) must be reserved to be used
                by IP Reassembly.
 
                In order to use an internal CPU Swap Queue, the IPSec and IP Reassembly constraints
                must be taken in account.

				[0]

 @Param[in]     h_VpDev                        - Handle to VP_DEV object
 @Param[in]     numOfReservedIntCpuSwapQueues  - Number of reserved
                                                internal CPU Swap Queues

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigNumOfReservedIntCpuSwapQueues(t_Handle  h_VpDev,
                                                   uint16_t  numOfReservedIntCpuSwapQueues);


/**
 @Function      VP_DEV_ConfigNumOfReservedExtCpuSwapQueues

 @Description   Configures the number of external CPU Swap Queues  that are reserved;
                reserved CPU Swap queues are queues reserved on VP_DEV object that will not
                be managed by Virtual Port instances.
 
                The number of queues that can be reserved is dependent on IPSec and IP Reassembly
                modules. If IPSec module is configured to use one external CPU Swap queue for debug
                purposes, than queue id 0 must be reserved to be used by IP Sec. If IP Reassembly
                module is configured to use one external CPU Swap Queue for debug purposes, than queue
                id 1 (or queue id 0 if it hasn't been reserved for IP Sec) must be reserved to be used
                by IP Reassembly.
 
                In order to use an internal CPU Swap Queue, the IPSec and IP Reassembly constraints
                must be taken in account.

				[0]
	
 @Param[in]     h_VpDev                        - Handle to VP_DEV object
 @Param[in]     numOfReservedExtCpuSwapQueues  - Number of reserved
                                                 external CPU Swap Queues

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_DEV_Config() and before VP_DEV_Init()
*/
t_Error VP_DEV_ConfigNumOfReservedExtCpuSwapQueues(t_Handle  h_VpDev,
                                                   uint16_t  numOfReservedExtCpuSwapQueues);


/** @} */ /* end of vp_dev_init_advanced_grp */
/** @} */ /* end of vp_dev_grp */


/**
@Group         vp_port_grp VP_PORT Object Application Programming Interface

@Description   VP_PORT object API functions, structures, definitions and enums

@{
*/



/**
@Group         vp_port_init_grp VP_PORT Object Initialization Unit

@Description   VP_PORT API Initialization Unit

@{
*/

/**
 @Description   User's callback routine events handling (invoked by the driver).

 @Param[in]     appId  - Application handle, provided by the user in the configuration structure .
 @Param[in]     events - Events that caused this routine to be invoked by the driver.
 @Param[in]     flags  - Protocol dependent indicator.
 @Return        None.
*/
typedef void (t_VpExceptions)(t_Handle appId, uint32_t events, uint32_t flags);

/**
 @Description   User's asynchronous callback function (invoked by the driver).

 @Param[in]     appId   - Application handle, provided by the user in the configuration structure .
 @Param[in]     queueId - Queue for which notification was generated
 @Return        None.
*/
typedef t_Error (t_VpRxAsyncCallback)(t_Handle appId, uint8_t queueId);

/**
 @Description   VP triggering mode (frames from the BD rings are sent when a host command is issued)
*/
typedef enum e_VpTriggerMode
{
	e_VP_INSERT_ON_DEMAND = 0,  /**< The user prepares frames in the BD ring and issues a host command. */
	e_VP_POLLING,               /**< Polling triggering mode (frames from the BD rings are sent periodically on a timer event) */
	e_VP_EXTERNAL_REQUEST       /**< External Request triggering mode (frames from the BD rings are sent upon an external interrupt request) */
} e_VpTriggerMode;


/**
 @Description   Data Copy Mode used by the VP Queues
*/
typedef enum e_VpDataCopyMode
{
	e_VP_DATA_COPY_NONE = 0,        /**< Data copy mode none */
	e_VP_DATA_COPY_SIMPLE_ONLY,     /**< Data copy with 8 bytes BDs */
	e_VP_DATA_COPY_EXTENDED_ONLY,   /**< Data copy with 16 bytes BDs */
	e_VP_DATA_COPY_MIXED            /**< Data copy with 8 bytes BDs and 16 bytes BDs */
} e_VpDataCopyMode;


/**
 @Description   VP exception events. They can be set using VP_PORT_ConfigExceptions()
*/
typedef enum e_VpExceptionEvents
{
    e_VP_EXCEPTION_EVENT_NONE           = 0,          /**< No exception */
	e_VP_EXCEPTION_EVENT_CPU_QUEUE_1_RX = 0x00000001, /**< VP CPU queue 1 receive event */
	e_VP_EXCEPTION_EVENT_CPU_QUEUE_2_RX = 0x00000002, /**< VP CPU queue 2 receive event */
	e_VP_EXCEPTION_EVENT_CPU_QUEUE_3_RX = 0x00000004, /**< VP CPU queue 3 receive event */
	e_VP_EXCEPTION_EVENT_CPU_QUEUE_4_RX = 0x00000008, /**< VP CPU queue 4 receive event */
	e_VP_EXCEPTION_EVENT_CPU_QUEUE_BUSY = 0x00000010  /**< VP CPU queue Busy event */
} e_VpExceptionEvents;


/**
 @Description   VP_PORT configuration parameters from the upper layer
*/
typedef struct t_VpPortParam
{
	t_Handle            h_AppId;            /**< the handle to the upper layer object */
	t_Handle            h_VpDev;            /**< the handle to the VP_DEV object */
	t_VpExceptions      *f_VpExceptions;    /**< Callback routine called by the driver on exception events;
	                                             If NULL, no exceptions are reported */
    t_TxConfFunction    *f_TxConf;          /**< user callback function called by driver to confirm a transmitted frame */
	t_RxStoreFunction   *f_RxStore;         /**< User callback function called by driver to pass
	                                             received buffers to the application */
} t_VpPortParam;


/**
 @Function      VP_PORT_Config

 @Description   Configures the VP_PORT object. Returns handle to the
                configured object or NULL in case of an error.

 @Param[in]     p_VpPortParams -  VP_PORT object parameters

 @Return        t_Handle
*/
t_Handle VP_PORT_Config(t_VpPortParam *p_VpPortParams);


/**
 @Function      VP_PORT_Init

 @Description   Initialize the VP_PORT object. Returns status of operation

 @Param[in]     h_VpPort -  Handle to VP_PORT object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_Init(t_Handle h_VpPort);


/**
 @Function      VP_PORT_Free

 @Description   Release the resources of the VP_PORT object.

 @Param[in]     h_VpPort -  Handle to VP_PORT object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_Free(t_Handle h_VpPort);

/** @} */ /* end of vp_port_init_grp */


/**
@Group         vp_port_init_advanced_grp VP_PORT Object Advanced Initialization Unit

@Description   VP_PORT API Advanced Initialization Unit

@{
*/

/**
 @Function      VP_PORT_ConfigTriggerMode

 @Description   Configures the triggering mode.
                [e_VP_INSERT_ON_DEMAND]

 @Param[in]     h_VpPort    -  Handle to VP_PORT object
 @Param[in]     triggerMode -  Trigger mode to be used by Virtual Port

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigTriggerMode(t_Handle h_VpPort, e_VpTriggerMode triggerMode);

#ifndef QE_VP_THREAD_POOL_UNAVAILABLE
/**
 @Function      VP_PORT_ConfigThreadPool

 @Description   Configures the thread pool support.
                [FALSE]

 @Param[in]     h_VpPort          -  Handle to VP_PORT object
 @Param[in]     threadPoolEnabled -  Thread Pool operation mode - enabled/disable

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigThreadPool(t_Handle h_VpPort, bool threadPoolEnabled);
#endif /*QE_VP_THREAD_POOL_UNAVAILABLE*/

/**
 @Function      VP_PORT_ConfigNumOfThreads
 
 @Description   Configures number of threads for this instance.
                [1]

 @Param[in]     h_VpPort     -  Handle to VP_PORT object
 @Param[in]     numOfThreads -  Number of threads used by this instance

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigNumOfThreads(t_Handle h_VpPort, uint8_t numOfThreads);


/**
 @Function      VP_PORT_ConfigNumOfVpQueues
 
 @Description   Configures the number of VP Queues for this instance.
                [1]

 @Param[in]     h_VpPort      -  Handle to VP_PORT object
 @Param[in]     numOfVpQueues -  Number of VP Queues to be enabled for this
                                 instance. It must include also the VP Queues
                                 created by other protocols in the QE.
                                 Ex: Ethernet in VP mode.
 
 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigNumOfVpQueues(t_Handle h_VpPort, uint8_t numOfVpQueues);


/**
 @Function      VP_PORT_ConfigNumOfCpuQueues
 
 @Description   Configures the number of CPU Queues.
                [1]

 @Param[in]     h_VpPort       -  Handle to VP_PORT object
 @Param[in]     numOfCpuQueues -  Number of CPU Queues to be configured

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigNumOfCpuQueues(t_Handle h_VpPort, uint8_t numOfCpuQueues);


/**
 @Function      VP_PORT_ConfigNumOfIntCpuSwapQueues

 @Description   Configures the number of internal VP CPU Swap Queues for which management
                (translation between queue descriptor index and queue id ) is provided.
                [0]

 @Param[in]     h_VpPort              - Handle to VP_PORT object
 @Param[in]     numOfIntCpuSwapQueues - Number of internal CPU Swap Queues. It must not include
                                        the CPU Swap Queues created by other protocols in the QE.
                                        Ex: Ethernet in VP mode.
 
 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigNumOfIntCpuSwapQueues(t_Handle h_VpPort,
                                           uint16_t  numOfIntCpuSwapQueues);


/**
 @Function      VP_PORT_ConfigNumOfExtCpuSwapQueues

 @Description   Configures the number of external VP CPU Swap Queues for which management
                (translation between queue descriptor index and queue id ) is provided.
                [0]

 @Param[in]     h_VpPort              - Handle to VP_PORT object
 @Param[in]     numOfExtCpuSwapQueues - Number of external CPU Swap Queues. It must not include
                                        the CPU Swap Queues created by other protocols in the QE.
                                        Ex: Ethernet in VP mode.
 
 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigNumOfExtCpuSwapQueues(t_Handle h_VpPort,
                                           uint16_t  numOfExtCpuSwapQueues);


/**
 @Function      VP_PORT_ConfigCpuSwapQueueAsyncMode
 
 @Description   Configures the CPU Swap Interrupt Queue to work in asynchronous receive mode. When
                an interrupt is issued due to a new frame being available, the driver will notify
                the user without processing the new frame. The user can disable any further Rx
                interrupts by calling VP_PORT_DisableCpuSwapQueueIntr and then can begin polling
                the VP Port for available frames. After polling of frames is finished, the user can
                enable Rx interrupt by calling VP_PORT_EnableCpuSwapQueueIntr in order to receive
                notifications for further received frames.

 @Param[in]     h_VpPort            -  Handle to VP_PORT object
 @Param[in]     queueId             -  Interrupt queue to be configured in asynchronous mode
 @Param[in]     f_RxAsyncCallback   -  Callback function called to notify the user when a
                                       new packet is available
 
 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigCpuSwapQueueAsyncMode(t_Handle            h_VpPort,
                                            uint16_t            queueId,
                                            t_VpRxAsyncCallback *f_RxAsyncCallback);


/**
 @Function      VP_PORT_ConfigCopyToCpuEnable
 
 @Description   Configures the Copy to CPU option.
                [FALSE]

 @Param[in]     h_VpPort        -  Handle to VP_PORT object
 @Param[in]     copyToCpuEnable -  Copy to CPU option

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigCopyToCpuEnable(t_Handle h_VpPort, bool copyToCpuEnable);


/**
 @Function      VP_PORT_ConfigCpuQueueLength
 
 @Description   Configures the CPU queue legth.
                [32]

 @Param[in]     h_VpPort    -  Handle to VP_PORT object
 @Param[in]     queueId     -  Queue to be configured
 @Param[in]     bdRingLen   -  length of the BD Ring

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigCpuQueueLength(t_Handle       h_VpPort,
                                     uint8_t        queueId,
                                     uint16_t       bdRingLen);


/**
 @Function      VP_PORT_ConfigCpuQueueCoalescingThreshold
 
 @Description   Configures the VP to issue a CPU frame interrupt
                every programmable number of frames.
                [1]

 @Param[in]     h_VpPort            -  Handle to VP_PORT object
 @Param[in]     queueId             -  Queue to be configured
 @Param[in]     coalescingThreshold -  Threshold to be used by the Queue

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigCpuQueueCoalescingThreshold(t_Handle       h_VpPort,
                                                  uint8_t        queueId,
                                                  uint32_t       coalescingThreshold);


/**
 @Function      VP_PORT_ConfigCpuQueueAsyncMode
 
 @Description   Configures the CPU Queue to work in asynchronous receive mode. When
                an interrupt is issued due to a new frame being available, the driver
                will notify the user without processing the new frame. The user can
                disable any further Rx interrupts by calling VP_PORT_DisableCpuQueueIntr
                and then can begin polling the VP Port for available frames.
                After polling of frames is finished, the user can enable Rx interrupt by
                calling VP_PORT_EnableCpuQueueIntr in order to receive notifications for
                further received frames.

 @Param[in]     h_VpPort            -  Handle to VP_PORT object
 @Param[in]     queueId             -  Queue to be configured in asynchronous mode
 @Param[in]     f_RxAsyncCallback   -  Callback function called to notify the user when a
                                       new frame is available
 
 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigCpuQueueAsyncMode(t_Handle              h_VpPort,
                                        uint16_t               queueId,
                                        t_VpRxAsyncCallback   *f_RxAsyncCallback);

/**
 @Function      VP_PORT_ConfigHeaderCompression

 @Description   Configures the Header Compression option.
                [FALSE]

 @Param[in]     h_VpPort             -  Handle to VP_PORT object
 @Param[in]     useHeaderCompression -  Use Header Compression

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigHeaderCompression(t_Handle h_VpPort,
                                        bool     useHeaderCompression);

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

/**
 @Function      VP_PORT_ConfigIpReassembly

 @Description   Configures the IP Reassembly option.
                [FALSE]

 @Param[in]     h_VpPort        -  Handle to VP_PORT object
 @Param[in]     useIpReassembly -  Use IP Reassembly

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigIpReassembly(t_Handle h_VpPort, bool useIpReassembly);

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
/**
 @Function      VP_PORT_ConfigDataCopyMode
 
 @Description   Configures the data copy mode.
                [e_VP_DATA_COPY_NONE]

 @Param[in]     h_VpPort                   -  Handle to VP_PORT object
 @Param[in]     dataCopyMode               -  Data Copy Mode
 @Param[in]     intermediateQueueBdRingLen -  The length of the intermediate
                                              queue required for the data copy mode

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigDataCopyMode(t_Handle         h_VpPort,
                                   e_VpDataCopyMode dataCopyMode,
                                   uint16_t         intermediateQueueBdRingLen);

#ifdef UCC_GETH_VP_MODE_SUPPORT
/**
 @Function      VP_PORT_ConfigVpQueuesExtendedFeaturesMode

 @Description   Configures VP Queues extended features mode
                Setting queue extended features mode is required
                for UCC in VP mode when Rx FW statistics and/or loss
                less flow control support are needed.
                [FALSE]

 @Param[in]     h_VpPort 	-  Handle of the VP_PORT object
 @Param[in]     isEnabled	-  VP queues extended features mode, TRUE to enable

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigVpQueuesExtendedFeatures(t_Handle h_VpPort, bool isEnabled);
#endif /* UCC_GETH_VP_MODE_SUPPORT */

/**
 @Function      VP_PORT_ConfigPerQueueSourcePort

 @Description   Configures VP per queues source port mode
                This mode allows setting different source port IDs used by parser
                for different VP queues; this feature is usefull when VP queues
                are utilized by other drivers, e.g. re-circulation or ETH in
                VP mode.
                [FALSE]

 @Param[in]     h_VpPort 	-  Handle of the VP_PORT object
 @Param[in]     isEnabled	-  Per queue source port mode, TRUE to enable

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init()
*/
t_Error VP_PORT_ConfigPerQueueSourcePort(t_Handle h_VpPort, bool isEnabled);


/**
 @Function      VP_PORT_ConfigRiscAllocation

 @Description   Configures the QE RISC on which the VP is running.
                [QE_RISC_ALLOC_ALL_RISCS]
 
 @Param[in]     h_VpPort        -  Handle to VP_PORT object
 @Param[in]     riscAllocation  -  RISC allocation

 @Return        E_OK on success; Error code otherwise
 @Cautions      For the 'riscAllocation' argument, use the 'RISC Allocation' macros.
*/
t_Error VP_PORT_ConfigRiscAllocation(t_Handle h_VpPort, uint8_t riscAllocation);


/**
 @Function      VP_PORT_ConfigExceptions

 @Description   Configures the VP exception events.

 @Param[in]     h_VpPort -  Handle to VP_PORT object
 @Param[in]     events   -  Combination of exception events defined
                            in e_VpExceptionEvents enumeration

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only following VP_PORT_Config() and before VP_PORT_Init().
*/
t_Error VP_PORT_ConfigExceptionEvents(t_Handle h_VpPort, uint32_t events);

 
/** @} */ /* end of vp_port_init_advanced_grp */

/**
@Group         vp_port_control_grp VP_PORT Object Control Unit

@Description   VP_PORT API Control Unit

@{
*/

/**
 @Description   Flags passed to the transmit routines for performing special operation modes.
*/
typedef enum e_VpQueueTxFlags
{
	e_VP_TX_FLAGS_NONE = 0,    /**<  */
	e_VP_TX_FLAGS_FORCE,    /**< send the frame immediately when VP_DEV is working in Polling mode (valid only for Polling
                            triggering mode configuration) */
	e_VP_TX_FLAGS_POSTPONE    /**< schedule this frame to be send on the next call of this function when VP_DEV is working in OnDemand
                            mode. (valid only for On Demand triggering mode configuration) */
} e_VpQueueTxFlags;


/**
 @Description   Flags passed to the receive routines.
*/
typedef enum e_VpQueueType
{
	e_VP_QUEUE = 0,     /**< VP Queue */
	e_VP_CPU_QUEUE,     /**< CPU Debug Queue  */
	e_VP_CPU_SWAP_QUEUE /**< CPU Swap Queue */

} e_VpQueueType;


/**
 @Description   VP Queue operation modes.
*/
typedef enum e_VpQueueOpMode
{
	e_VP_SWAP_BD            = 0x0001,    /**< The user allocates data buffers which are to be swapped with the data buffers of the transmitter */
	e_VP_FAST_SWAP_BD       = 0x0002,    /**< Similar to Swap BDs mode. In addition, number of BDs in frame and error indication are programmed
                                            in the first BD of frame and the total frame lengh is programmed in the last BD size. */
#ifdef UCC_GETH_VP_MODE_SUPPORT
    e_VP_INT_FAST_SWAP_BD   = 0x0003,    /**< Similar to Fast Swap BDs mode. Should be used when VP Queue is the destination queue for
                                             other protocols in the QE. Ex: Ethernet in VP mode */
#endif /* UCC_GETH_VP_MODE_SUPPORT */
    e_VP_RECIRCULATION_BD   = 0x0004,
	e_VP_DATA_COPY          = 0x0008,    /**< Intermediate BD ring is used to copy frame contents. 8 bytes BDs are used. */
	e_VP_DATA_COPY_EXTENDED = 0x0010,    /**< Intermediate BD ring is used to copy frame contents. 16 bytes BDs are used. */
	e_VP_DATA_COPY_WITH_PARSER_RESULTS = 0x0020,    /**< Intermediate BD ring is used to copy frame contents. 16 bytes BDs are used and parser results are stored. */
	e_VP_PARSE_ONLY         = 0x0040     /**< Parse Only mode */
} e_VpQueueOpMode;


/**
 @Description   VP Queue priority mode.
*/
typedef enum e_VpQueuePriorityMode
{
	e_VP_FIXED_PRIORITY_MODE = 0,   /**< Fixed priority mode */
	e_VP_WFQ_PRIORITY_MODE          /**< WFQ (Weighted Fair Queue) priority mode */
} e_VpQueuePriorityMode;


/**
 @Description   VP Queue configuration parameters
*/
typedef struct t_VpQueueParam
{
	e_VpQueueOpMode         operationMode;       /**< the operation mode of the queue: Swap, FastCopy, DataCopy, ParseOnly */
	t_Handle                h_Filter;            /**< pointer to the filter used by this Vp Queue */
	uint16_t                bdRingLen;           /**< BD ring length */
	e_VpQueuePriorityMode   priorityMode;        /**< priority mode for this Vp Queue */
	uint16_t                priorityWeight;      /**< the priority weight of the Vp Queue  for of WFQ scheduling algorithm */
	uint16_t                bdErrorMask;         /**< BD error mask */
    uint8_t                 sourcePort;          /**< source port; used to identify the source of a frame*/
} t_VpQueueParam;

#ifdef UCC_GETH_VP_MODE_SUPPORT
/*
 @Description   VP Queue service parameters
*/
typedef struct t_VpQueueService
{
    bool     softPreamble;           /**< Soft Preamble enable/disable */
	bool     lossLessFlowControl;    /**< LossLess Flow Control enable/disable */
	uint32_t nextRxBdOffset;         /**< Address of nextBdOffset field in loss less
                                          flow control ETH table corresponding to
                                          this queue */
	bool     statisticsMode;         /**< Statistics mode enable/disable */
	uint32_t statisticsBase;         /**< Statistics base */
} t_VpQueueService;
#endif /* UCC_GETH_VP_MODE_SUPPORT */

/**
 @Description  CPU Swap Queue configuration parameters
*/
typedef struct t_VpCpuSwapQueueParam
{
	t_Handle                h_AppId;                /**< The handle to the upper layer object */
    bool                    isExternal;             /**< Queue descriptor is on external or internal memory */
    uint8_t                 interruptQueue;         /**< Index of interrupt queue to be used*/
    uint16_t                bdRingLen;              /**< BD ring size in number of BDs */
    uint8_t                 expWeightFactor;        /**< Exponential weight factor for WRED */
	t_RxStoreFunction       *f_RxStore;             /**< User callback function called by driver to pass received buffers to the application */

} t_VpCpuSwapQueueParam;

#ifdef UCC_GETH_VP_MODE_SUPPORT
/*
 @Description   The details for the VP_PORT object
*/
typedef struct t_VpPortDetails
{
    t_Handle   h_VpDev;               /**< VP Dev Handle */
    uint32_t   vpGlobalPram;          /**< VP Global PRAM address */
    uint16_t   numOfIntCpuSwapQueues; /**< Number of total internal CPU Swap Queues*/
    bool       threadPoolEnabled;     /**< Thread Pool mode enabled/disabled */
} t_VpPortDetails;
#endif /* UCC_GETH_VP_MODE_SUPPORT*/

/**
 @Function      VP_PORT_Enable

 @Description   Enable the VP_PORT object.

 @Param[in]     h_VpPort - Handle to VP_PORT object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_Enable(t_Handle h_VpPort);


/**
 @Function      VP_PORT_Disable

 @Description   Disable the VP_PORT object.

 @Param[in]     h_VpPort -  Handle to VP_PORT object

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_Disable(t_Handle h_VpPort);

/**
 @Function      VP_PORT_EnableCpuQueueIntr

 @Description   Enable the receive interrupt on a CPU Queue

 @Param[in]     h_VpPort -  Handle to VP_PORT object
 @Param[in]     queueId  -  Queue for which to enable the interrupt

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_EnableCpuQueueIntr(t_Handle  h_VpPort, uint16_t  queueId);

/**
 @Function      VP_PORT_DisableCpuQueueIntr

 @Description   Disable the receive interrupt on a CPU Queue

 @Param[in]     h_VpPort -  Handle to VP_PORT object
 @Param[in]     queueId  -  Queue for which to disable the interrupt

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_DisableCpuQueueIntr(t_Handle  h_VpPort, uint16_t  queueId);

/**
 @Function      VP_PORT_EnableCpuSwapQueueIntr

 @Description   Enable the receive interrupt on a CPU Swap Interrupt Queue

 @Param[in]     h_VpPort -  Handle to VP_PORT object
 @Param[in]     queueId  -  Interrupt Queue for which to enable the interrupt

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_EnableCpuSwapQueueIntr(t_Handle  h_VpPort, uint16_t  queueId);

/**
 @Function      VP_PORT_DisableCpuSwapQueueIntr

 @Description   Disable the receive interrupt on a CPU Swap Interrupt Queue

 @Param[in]     h_VpPort -  Handle to VP_PORT object
 @Param[in]     queueId  -  Interrupt Queue for which to disable the interrupt

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_DisableCpuSwapQueueIntr(t_Handle  h_VpPort, uint16_t  queueId);

/**
 @Function      VP_PORT_AddVpQueue

 @Description   Add one Virtual Port Queue.
 
 @Param[in]     h_VpPort        -  Handle of the VP_PORT object.
 @Param[in]     p_VpQueueParams -  User parameters structure for the VP Queue.
 @Param[out]    p_QdIndex       -  Queue descriptor index for the added VP Queue.
                                   If set to NULL, queue descriptor index is not returned
                                   and translation between VP Queue Id and VP Queue descriptor
                                   index will be done by Virtual Port.
 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_AddVpQueue(t_Handle        h_VpPort,
                           t_VpQueueParam  *p_VpQueueParams,
                           uint8_t         *p_QdIndex);


/**
 @Function      VP_PORT_RemoveVpQueue

 @Description   Remove one Virtual Port Queue.
 
 @Param[in]     h_VpPort    -  Handle of the VP_PORT object.
 @Param[in]     qdIndex     -  Queue descriptor index to be released.
 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_RemoveVpQueue(t_Handle  h_VpPort, uint8_t qdIndex);


/**
 @Function      VP_PORT_AddVpCpuSwapQueues

 @Description   Add one or multiple CPU Swap Queues.The add routine supports three types
                of CPU Swap queues: "reserved"," managed" and "non-reserved and non-managed"
                queues. Special caution must be taken when adding a CPU Swap queue in order
                to properly configure the queue.

                Reserved CPU Swap queues are queues that have already been reserved on the VP_DEV
                module in order to assure the uniqueness of the queue on VP_DEV (for example:
                IP Sec and IP Reassembly).
 
                Managed CPU Swap queues are queues that are managed by Virtual Port, meaning that
                the Virtual Port will provide a translation between the logical queue id  and the
                queue descriptor index. Managed CPU Swap queues are queues used to perform interworking.

                Non-managed and non-reserved CPU Swap queues are queues that are usually created by
                other protocols in the QE (for example : Ethernet in VP mode), meaning that management
                for this type of queues will be provided  be the protocol that configured the queues.

 @Param[in]     h_VpPort             -  Handle to VP_PORT object
 @Param[in]     numOfCpuSwapQueues   -  Requested number of VP CPU Swap Queues
                to be added.The number of queues must range from 1 to 65536.
 
 @Param[in]     p_VpCpuSwapQueueParams -  VP CPU Swap Queue parameters structure.
                The numOfCpuSwapQueues will be configured with the same parameters.
 
 @Param[in]     reserved - If set to TRUE, add cpu swap queues from
                already reserved queue descriptor indexes.

 @Param[in]     p_QdIndexes

                Reserved queues: p_QdIndexes is an array of size numOfCpuSwapQueues;  the reserved
                qdIndexes will be taken from p_QdIndexes and must be provided with indexes from 0
                to numOfCpuSwapQueues relative to memory type: internal/external.  If the add routine
                was successful, the driver will return in p_QdIndexes  the absolute queue descriptors
                indexes that can be used by the user in order to remove the queues.
 
                Managed queues: p_QdIndexes is not used and must be set to NULL; When removing the queues,
                in order to gain access to the absolute queue descriptor indexes, the user must first call
                VP_PORT_GetQdIndexes and then call the remove routine with the newly obtained indexes.
 
                Non-reserved and non-managed queues: p_QdIndexes is an array of size numOfCpuSwapQueues;
                If the add routine was successful, the driver will return in p_QdIndexes the queue descriptors
                indexes of the added queues. The returned array can be used when removing the queues.
 
 @Return        E_OK on success; Error code otherwise
 */
t_Error VP_PORT_AddVpCpuSwapQueues(t_Handle              h_VpPort,
                                  uint16_t               numOfCpuSwapQueues,
                                  t_VpCpuSwapQueueParam  *p_VpCpuSwapQueueParams,
                                  bool                   reserved,
                                  uint16_t               *p_QdIndexes);


/**
 @Function      VP_PORT_RemoveVpCpuSwapQueues

 @Description   Delete one or multiple CPU Swap Queues.
 
 @Param[in]     h_VpPort            - Handle to VP_PORT object
 @Param[in]     numOfCpuSwapQueues	- Number of CPU Swap Queues to be removed
 @Param[in]     p_QdIndexes         - Pointer to array of size numOfCpuSwapQueues containing
                                      the qdIndexes to be released
 
 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_RemoveVpCpuSwapQueues(t_Handle  h_VpPort,
                                      uint16_t  numOfCpuSwapQueues,
                                      uint16_t  *p_QdIndexes);

#ifdef UCC_GETH_VP_MODE_SUPPORT
/**
 @Function      VP_PORT_SetVpQueueService

 @Description   Set the VP_PORT VP Queue service parameters.

 @Param[in]     h_VpPort         -  Handle of the VP_PORT object.
 @Param[in]     queueId          -  VP Queue to be configured with this parameters.
 @Param[in]     p_VpQueueService -  VP Queue service parameters

 @Return        E_OK on success; Error code otherwise

 @Cautions      Allowed only for Ethernet configured in VP mode.
*/
t_Error VP_PORT_SetVpQueueService(t_Handle           h_VpPort,
                                  uint8_t            queueId,
                                  t_VpQueueService   *p_VpQueueService);


/**
 @Function      VP_PORT_GetDetails

 @Description   Gets the VP_PORT object details from the Virtual Port

 @Param[in]     h_VpPort        -  Handle of the VP_PORT object.
 @Param[out]    p_VpPortDetails -  Pointer to VP_PORT details structure

 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_GetDetails(t_Handle h_VpPort, t_VpPortDetails *p_VpPortDetails);


/**
 @Function      VP_PORT_GetVpQueueBdRingBase

 @Description   Retrieves the value for the VP Queue BD Ring Base

 @Param[in]     h_VpPort   -  Handle of the VP_PORT object.
 @Param[in]     queueId    -  VP Queue for which to retrive the bdRingBase.
 @Param[out]    bdRingBase -  BD ring base.

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only after VP_PORT_AddVpQueue().
 */
t_Error VP_PORT_GetVpQueueBdRingBase(t_Handle h_VpPort, uint8_t queueId, uint32_t *bdRingBase);
#endif /* UCC_GETH_VP_MODE_SUPPORT */

/**
 @Function      VP_PORT_GetQdIndexes

 @Description   Gets the queue descriptors indexes for numOfCpuSwapQueues
 
 @Param[in]     h_VpPort            - Handle to VP_PORT object
 @Param[in]     numOfCpuSwapQueues	- Number of CPU Swap Queues to obtain the queue descriptors indexes
 @Param[in]     p_QueueIds          - Pointer to array of size numOfCpuSwapQueues containing
                                      the queue Ids for which queue descriptor indexes is desired
 
 @Return        E_OK on success; Error code otherwise
*/
t_Error VP_PORT_GetQdIndexes(t_Handle   h_VpPort,
                             uint16_t   numOfCpuSwapQueues,
                             uint16_t   *p_QueueIds);


/**
 @Function      VP_PORT_Tx

 @Description   Transmits a data buffer on this VP_PORT. The actual transmission moment occurs
                depending on the configured VP trigger mode (OnDemand, Polling, External
                request).

 @Param[in]     h_VpPort       -  handle of the VP_PORT object
 @Param[in]     queueId        -  VP Queue Id
 @Param[in]     p_Data         -  Pointer to data buffer to transmit
 @Param[in]     h_BufContext   -  User's context for the given data buffer.
 @Param[in]     length         -  Length of data to transmit
 @Param[in]     lastBuffer     -  Indicates whether or not the referred buffer is the last one in the frame
 @Param[in]     flags          -  additional transmit mode flags (any of the elements of the e_VpPortTxFlags
                                enumeration)

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only for VP Queues.
*/
t_Error VP_PORT_Tx( t_Handle    h_VpPort,
                    uint16_t    queueId,
                    uint8_t     *p_Data,
                    t_Handle    h_BufContext,
                    uint16_t    length,
                    bool        lastBuffer,
                    uint16_t    flags);

/**
 @Function      VP_PORT_TxConf

 @Description   Forces confirmation on transmitted frames or data buffers.

 @Param[in]     h_VpPort -  handle to the VP_PORT
 @Param[in]     queueId  -  VP Queue Id

 @Return        E_OK on success; Error code otherwise
 @Cautions      Allowed only for VP Queues.
*/
void VP_PORT_TxConf(t_Handle h_VpPort, uint16_t queueId);


/**
 @Function      VP_PORT_Rx

 @Description   Handles data buffers received in the specified VP_RX_CPU_QUEUE, e_VP_RX_VP_QUEUE
                or e_VP_RX_CPU_SWAP_QUEUE.
 
                Received data are passed to the user as pointers to data buffers,
                via the user receive callback routine corresponding to the
                specific queue.In Polling mode, the routine should
                be called by the user as often as necessary. In Interrupt mode
                the routine is invoked by the driver, without user intervention.

 @Param[in]     h_VpPort  -  handle of the VP_PORT object
 @Param[in]     queueId   -  Queue Id on which to receive data.
                             For VP Queues : queue Id from 0 .. VP_MAX_VP_QUEUES
                             For CPU Queues : queue Id from 0 .. VP_MAX_CPU_QUEUES
                             For CPU Swap Queues :
                             - for interrupt mode : queue Id from VP_MAX_CPU_QUEUES .. 65356 + VP_MAX_CPU_QUEUES
                             - for polling mode : the queue Id is actually the interrupt queue number
                             and is shifted with VP_MAX_CPU_QUEUES : interruptQ 0 - VP_MAX_CPU_QUEUES,
                             interruptQ15 - 15 + VP_MAX_CPU_QUEUES
 @Param[in]     queueType -  type of queue to receive from (VP, CPU or CPU swap queue)
 @Param[in]     flags     -  receive mode flags - type of Rx queue on which to receive
                             (any of the elements of the e_VpQueueRxFlags enumeration)

 @Return        E_OK on success; Error code otherwise.
*/
t_Error VP_PORT_Rx(t_Handle h_VpPort, uint16_t queueId, e_VpQueueType queueType, uint8_t flags);
 
/** @} */ /* end of vp_port_control_grp */
/** @} */ /* end of vp_port_grp */
/** @} */ /* end of vp_api */


/**************************************************************************/
/*                                                                        */
/*                IW_SUPPORT                                              */
/*                                                                        */
/**************************************************************************/

/**
 @Description    CPU Swap IW Interrupt Queue Entry
*/
typedef struct {

    uint8_t                 type;            /**< Exception type. Allows differentiating between CPU Swap interrupts (defined in ::e_QeIwCpuSwapIntrQueueEntryFlags) */
    uint8_t                 code;            /**< Code indicating the type of error */
    uint32_t                word1;           /**< For IP Sec: Security Parameter Index */
} t_VpCpuSwapIwException;

/*
 @Description   Callback routine for IW CPU Swap interrupts.

 @Param         h_IwDev     - (In) Handle to IW device driver
 @Param         p_Exception - (In) Pointer to a structure containing detailed exception information.

 @Return        None.
*/
typedef void (t_VpCpuSwapIwExceptions)(t_Handle h_IwDev, t_VpCpuSwapIwException  *exception);

/*
 @Description   Callback routine for IW interrupts.

 @Param         h_IwDev - (In) Handle to IW device driver
 @Param         events  - (In) Events that caused this routine to be invoked

 @Return        None.
*/
typedef void (t_VpIwExceptions)(t_Handle h_IwDev, uint32_t events);

/*
 @Function      VP_DEV_SetIwExceptionCallback

 @Description   Sets the IW exception callback (interface with IW driver).

 @Param[in]     h_VpDev                -  Handle to VP_DEV object
 @Param[in]     h_IwDev                -  Handle to IW DEV driver
 @Param[in]     f_IwExceptions         -  Callback routine for IW interrupts
 @Param[in]     f_IwCpuSwapExceptions  -  Callback routine for IW CPU Swap interrupts
 @Param[in]     iwEvents               -  TRUE if IW events are required
 @Param[in]     parserEvents           -  TRUE if parse and lookup events are required

 @Return   void
*/
t_Error VP_DEV_SetIwExceptionCallback(t_Handle                h_VpDev,
                                      t_Handle                h_IwDev,
                                      t_VpIwExceptions        f_IwExceptions,
                                      t_VpCpuSwapIwExceptions f_IwCpuSwapExceptions,
                                      bool                    iwEvents,
                                      bool                    parserEvents);

/*
 @Description   The IW details for the IW_DEV object.
*/
typedef struct t_VpIwDetails
{
	t_Handle            h_VpPort;                   /**< Handle of Virtual Port instance 0 */
	uint32_t            iwGlobalPramBase;           /**< Base address of IW global parameters RAM */
	uint32_t            dataMemAttributes;          /**< Memory attributes for data buffers */
    uint8_t             fwExtStructsMemId;          /**< Memory partition ID of external firmware structures */
	uint8_t             bdDataBmr;                  /**< BD and data Bus Mode Register */

#ifndef QE_IW_IPREASS_FULL_UNAVAILABLE

    t_BufferPoolInfo    bufferPoolInfo;             /**< Data buffer pool information */
	uint8_t             numOfVpThreads;             /**< Number of threads used by all VP instances */

#endif  /* QE_IW_IPREASS_FULL_UNAVAILABLE */
 
	uint32_t            globalPramBasePointersTbl;  /**< Address of Virtual Port global PRAM Base Pointers Table */
    uint32_t            intCpuSwapQdBase;           /**< Base address of internal VP CPU Swap queue descriptors */
	uint32_t            extCpuSwapQdBase;           /**< Base address of external VP CPU Swap queue descriptors */
	uint32_t            iwCpuSwapInterruptQdBase;   /**< Base address of CPU Swap interrupt queue descriptors */
    uint8_t             cpuSwapQSnum;               /**< SNUM for event register related to VP CPU Swap interrupt queues 1 - 15 */

} t_VpIwDetails;

/*
 @Function      VP_DEV_GetIwDetails

 @Description   Gets the IW details from the Virtual Port (interface with IW driver)

 @Param[in]     h_VpDev     -  Handle to VP_DEV object
 @Param[in]     p_IwDetails -  Pointer to IW details structure

 @Return        E_OK on success; Error code otherwise.
*/
t_Error VP_DEV_GetIwDetails(t_Handle h_VpDev, t_VpIwDetails *p_IwDetails);


/*
 @Function      VP_DEV_GetAppId

 @Description   Gets the h_AppId of the VP_DEV for the IW (interface with IW driver)

 @Param[in]     h_VpDev -  the handle of the VP_DEV object

 @Return        E_OK on success; Error code otherwise.
*/
t_Handle VP_DEV_GetAppId(t_Handle h_VpDev);


/*
 @Function      VP_PORT_GetIwIndex

 @Description   A routine for returning the Iw Index index for a
                given VP Queue or CPU Swap Queue.

 @Param[in]     h_VpPort    -    Handle to VP_PORT object.
 @Param[in]     queueId     -    IW destination index for VP Queue, as passed to QE_IW_CONN_Config()
 @Param[out]    p_TxQdIndex -    TX QD index.
 @Return        E_OK on success; Error code otherwise.
*/
t_Error VP_PORT_GetIwIndex(t_Handle h_VpPort, uint16_t queueId, uint16_t *p_TxQdIndex);


/*
 @Function      VP_PORT_GetCpuIwIndex

 @Description   A routine for returning the Iw Index index for a
                given CPU Swap Queue.

 @Param[in]     h_VpPort    -    Handle to VP_PORT object.
 @Param[in]     queueId     -    IW destination index, as passed to QE_IW_CONN_Config()
 @Param[out]    p_TxQdIndex -    TX QD index.
 @Param[out]    p_IsExt     -    Queue descriptor is on external/internal memory.
 
 @Return        E_OK on success; Error code otherwise.
*/
t_Error VP_PORT_GetCpuIwIndex(t_Handle      h_VpPort,
                              uint16_t      queueId,
                              uint16_t      *p_TxQdIndex,
                              bool          *p_IsExt);
 
/*****************************************************************************/
/*                                                                           */
/*                   DUMP FUNCTIONS                                          */
/*                                                                           */
/*****************************************************************************/
#if (DEBUG_ERRORS > 0)

t_Error VP_PORT_DumpVpGlobalPram(t_Handle h_VpPort);

t_Error VP_PORT_DumpVpQueueSchedTbl(t_Handle h_VpPort);

t_Error VP_PORT_DumpBdRings(t_Handle h_VpPort);

#endif /* DEBUG_ERRORS > 0 */


#endif  /* __VP_EXT_H */
