/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          dec_ext.h

 @Author        Shlomi Gridish (gridish@freescale.com)

 @Description   structures and definitions for initialize and operate Real
                Time clock that operates the decrementer interrupt.
*//***************************************************************************/
#ifndef __DEC_EXT
#define __DEC_EXT

#include "std_ext.h"


/**************************************************************************//**
 @Group         dec_id DEC - Decrementer Application Programming Interface

 @Description   Decrementer functions, definitions and enums.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Group         dec_init Decrementer Initialization Unit

 @Description   Decrementer initialization functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Decrementer configuration parameters
*//***************************************************************************/
typedef struct t_DecParam
{
    uint32_t    sysClk;
} t_DecParam;


/**************************************************************************//**
 @Function      DEC_ConfigAndInit

 @Description   Initialize and enable the decrementer.

                The routine calculate the required value in the decrementer
                according to the period variable and initialize the user
                callback routine with the input parameter of the callback handler.
                The calculation of the decrementer is:
                The decrementer is decremented once every four system_clock
                therefore - the time for decrement one from the DEC is
                (System-Clock/4)*10^6(mega) and this time is multiplied
                by the period between two decrementer interrupts.

 @Param         p_DecParam  -  (In) Decrementer parameters structure.

 @Return        A handle to the newly created decrementer module.
*//***************************************************************************/
t_Handle DEC_ConfigAndInit(t_DecParam *p_DecParam);

/**************************************************************************//**
 @Function      DEC_Free

 @Description   Disable the Real Time Clock.

                The routine set the decrementer with the maximum value
                and initialize the user callback routine to NULL.

 @Param         h_Dec  -  (In) An handle to the decrementer device.

 @Return        E_OK (0) on success, other value otherwise.
*//***************************************************************************/
t_Error DEC_Free(t_Handle h_Dec);

/** @} */ /* end of dec_init group */

/**************************************************************************//**
 @Group         dec_runtime Decrementer Runtime Unit

 @Description   Decrementer Run Time functions.

 @{
*//***************************************************************************/

t_Error DEC_Start(t_Handle      h_Dec,
                  uint8_t       id,
                  void          (*f_ExpiredCallback)(t_Handle h_App, uint8_t id),
                  t_Handle      h_App,
                  uint64_t      period,
                  e_TimerMode   mode);

t_Error DEC_Stop(t_Handle h_Dec, uint8_t id);

/**************************************************************************//**
 @Function      DEC_SetTime

 @Description   Set the value of the DEC.

                The routine calculate the required value in the decrementer
                according to the period variable.
                The calculation of the decrementer is:
                The decrementer is decremented once every four system_clock
                therefore - the time for decrement one from the DEC is
                (SYSTEM_CLOCK/4)*10^6(mega) and this time is multiplied
                by the period between two decrementer interrupts.

 @Param         h_Dec  -  (In) An handle to the decrementer device.
 @Param         id     -  (In) id.
 @Param         period -  (In) The required period between two decrementer
                               interrupts (in micro-seconds).

 @Return        1 - Fail - in case that the period exceeds the maximum. 0 - on success.
*//***************************************************************************/
t_Error DEC_SetTime(t_Handle    h_Dec,
                    uint8_t     id,
                    uint64_t    period);

/**************************************************************************//**
 @Function      DEC_GetTime

 @Description   Get the current value of the Real Time Clock.

                The routine read the decrementer.

 @Param         h_Dec    - (In)  An handle to the decrementer device.
 @Param         id       - (In) id.
 @Param         p_Period - (Out) The period from the last decrementer interrupt
                                 (in micro-seconds).

 @Return        Current value (in micro seconds).
*//***************************************************************************/
t_Error DEC_GetTime(t_Handle    h_Dec,
                    uint8_t     id,
                    uint64_t    *p_Period);

/** @} */ /* end of dec_runtime group */

/** @} */ /* end of dec_id group */


#endif /* __DEC_EXT */
