/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          rio_std_ext.h

 @Description   RapidIO common definitions

 @Cautions      None.

*//***************************************************************************/
#ifndef __RIO_STD_EXT_H
#define __RIO_STD_EXT_H

/**************************************************************************//**
 @Group         rio_std_id Common RIO Port Application Programming Interface

 @Description   Common RIO API functions, definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    RIO device id definitions

 @{
*//***************************************************************************/
#define RIO_LARGE_SYS_DEVID_MASK        0xFFFF      /**< RIO device id mask for large systems (up to 65,536 devices) */
#define RIO_SMALL_SYS_DEVID_MASK        0x00FF      /**< RIO device id mask for small systems (up to 256 devices)    */

#define RIO_DEVID_EXT_SHIFT             8           /**< RIO extended device id shift. Used for large systems */

/* @} */

/**************************************************************************//**
 @Collection    RIO memory mapping definitions

 @{
*//***************************************************************************/
#define RIO_VIRT_SPACE_SIZE             0x400000000     /**< RIO virtual space size         */
#define RIO_VIRT_SPACE_BASE             0x100000        /**< RIO virtual space base address */

/**************************************************************************//**
 @Description  Enumerates the different typed of RIO messaging unit object
*//***************************************************************************/
typedef enum e_RioMUObjType
{
    e_RIO_MU_MAILBOX,               /**< RIO mailbox (inbound messaging unit) */
    e_RIO_MU_OUT_QUEUE              /**< RIO outbound queue (outbound messaging unit) */
}e_RioMUObjType;

/**************************************************************************//**
 @Description  Supported I/O read operations that are used for I/O accesses
               to physical addresses in the target of the operation.
 
               An atomic operation is a combination of read and write
               operations.
*//***************************************************************************/
typedef enum e_RioIoOpReadType
{
    e_RIO_IO_OP_IO_READ_HOME,       /**< returns a current copy of cache-coherent data from the memory system */
    e_RIO_IO_OP_NREAD,              /**< basic read transaction                     */
    e_RIO_IO_OP_MAINT_READ,         /**< a read from the configuration registers    */
    e_RIO_IO_OP_ATOMIC_INCREMENT,   /**< post-increment the data                    */
    e_RIO_IO_OP_ATOMIC_DECREMENT,   /**< post-decrement the data                    */
    e_RIO_IO_OP_ATOMIC_SET,         /**< set the data (write 0b11111...)            */
    e_RIO_IO_OP_ATOMIC_CLEAR        /**< clear the data (write 0b00000...)          */
} e_RioIoOpReadType;

/**************************************************************************//**
 @Description  Supported I/O write operations that are used for I/O accesses
               to physical addresses in the target of the operation.
*//***************************************************************************/
typedef enum e_RioIoOpWriteType
{
    e_RIO_IO_OP_FLUSH,              /**< returns the ownership and current data to home memory  */
    e_RIO_IO_OP_SWRITE,             /**< a double-word-only version of the NWRITE               */
    e_RIO_IO_OP_NWRITE,             /**< basic write operation                                  */
    e_RIO_IO_OP_NWRITE_R,           /**< the sender notified when the transaction has completed at the destination */
    e_RIO_IO_OP_MAINT_WRITE         /**< a write to the configuration registers                 */
} e_RioIoOpWriteType;

/**************************************************************************//**
 @Description   Transaction flow level for outgoing requests
 
                This value is translated to packet priority in the physical layer.
*//***************************************************************************/
typedef enum e_RioTransFlowLevel
{
    e_RIO_TRANS_FLOW_LEVEL_LOW,     /**< Lowest packet priority */
    e_RIO_TRANS_FLOW_LEVEL_MEDIUM,  /**< Medium packet priority */
    e_RIO_TRANS_FLOW_LEVEL_HIGH     /**< Highest packet priority */
} e_RioTransFlowLevel;

/* @} */

/**************************************************************************//**
 @Collection    RapidIO Architectural Registers definitions

 @{
*//***************************************************************************/

/* Device Identity register */
#define RIO_DIDCAR_REG_OFFSET                0x0            /**<  */
#define RIO_DIDCAR_DEVICE_ID_MASK            0xFFFF0000     /**<  */
#define RIO_DIDCAR_DEVICE_ID_SHIFT           16             /**<  */
#define RIO_DIDCAR_DEVICE_VENDOR_ID_MASK     0x0000FFFF     /**<  */

/* Device Information register */
#define RIO_DICAR_REG_OFFSET                 0x4            /**<  */

/* Assembly Identity register */
#define RIO_AIDCAR_REG_OFFSET                0x8            /**<  */
#define RIO_AIDCAR_ASSEMBLY_ID_MASK          0xFFFF0000     /**<  */
#define RIO_AIDCAR_ASSEMBLY_ID_SHIFT         16             /**<  */
#define RIO_AIDCAR_ASSEMBLY_VENDOR_ID_MASK   0x0000FFFF     /**<  */

/* Assembly Information register */
#define RIO_AICAR_REG_OFFSET                 0xC            /**<  */
#define RIO_AICAR_ASSEMBLY_REV_MASK          0xFFFF0000     /**<  */
#define RIO_AICAR_ASSEMBLY_REV_SHIFT         16             /**<  */
#define RIO_AICAR_EXT_FEATURES_PTR_MASK      0x0000FFFF     /**<  */

/* Processing Element Features register */
#define RIO_PEFCAR_REG_OFFSET                0x10           /**<  */

#define RIO_PEFCAR_IS_BRIDGE                 0x80000000     /**<  */
#define RIO_PEFCAR_HAS_MEM                   0x40000000     /**<  */
#define RIO_PEFCAR_HAS_PROCCESSOR            0x20000000     /**<  */
#define RIO_PEFCAR_IS_SWITCH                 0x10000000     /**<  */

#define RIO_PEFCAR_SUPPORT_MAILBOX_0         0x00800000     /**<  */
#define RIO_PEFCAR_SUPPORT_MAILBOX_1         0x00400000     /**<  */
#define RIO_PEFCAR_SUPPORT_MAILBOX_2         0x00200000     /**<  */
#define RIO_PEFCAR_SUPPORT_MAILBOX_3         0x00100000     /**<  */

#define RIO_PEFCAR_SUPPORT_IN_DOORBELL       0x00080000     /**<  */
#define RIO_PEFCAR_LARGE_TRANS_SYSTEM        0x00000010     /**<  */

#define RIO_PEFCAR_EXT_FEATURES_PTR_VALID    0x00000008     /**<  */
#define RIO_PEFCAR_ADDRESS_SIZE_MASK         0x00000007     /**<  */
#define RIO_PEFCAR_66_50_34_BIT_ADDRESS      0x00000007     /**<  */
#define RIO_PEFCAR_66_34_BIT_ADDRESS         0x00000005     /**<  */
#define RIO_PEFCAR_50_34_BIT_ADDRESS         0x00000003     /**<  */
#define RIO_PEFCAR_34_BIT_ADDRESS_ONLY       0x00000001     /**<  */

/* Switch Port Information register */
#define RIO_SWITCH_PORT_INFO_CSR_OFFSET      0x14           /**<  */
#define RIO_SWITCH_PORT_TOTAL_MASK           0x0000FF00     /**<  */
#define RIO_SWITCH_PORT_TOTAL_SHIFT          8              /**<  */
#define RIO_SWITCH_PORT_NUM_MASK             0x000000FF     /**<  */

/* Source Operations register */
#define RIO_SOCAR_REG_OFFSET                 0x18           /**<  */
#define RIO_SOCAR_NREAD                      0x00008000     /**< Nread logical I/O operation    */
#define RIO_SOCAR_NWRITE                     0x00004000     /**< Nwrite logical I/O operation   */
#define RIO_SOCAR_STREAM_WRITE               0x00002000     /**< Swrite logical I/O operation   */
#define RIO_SOCAR_NWRITE_WITH_RESPONSE       0x00001000     /**< Nwrite_R logical I/O operation */
#define RIO_SOCAR_MESSAGE                    0x00000800     /**< Message operation */
#define RIO_SOCAR_DOORBELL                   0x00000400     /**< Doorbell operation */
#define RIO_SOCAR_ATOMIC_TEST_AND_SWAP       0x00000100     /**< Atomic-Test-and-Swap logical I/O operation */
#define RIO_SOCAR_ATOMIC_INC                 0x00000080     /**< Atomic-Inc logical I/O operation   */
#define RIO_SOCAR_ATOMIC_DEC                 0x00000040     /**< Atomic-Dec logical I/O operation   */
#define RIO_SOCAR_ATOMIC_SET                 0x00000020     /**< Atomic-Set logical I/O operation   */
#define RIO_SOCAR_ATOMIC_CLEAR               0x00000010     /**< Atomic-Clear logical I/O operation */
#define RIO_SOCAR_PORT_WRITE                 0x00000004     /**< Port-Write operation               */

/* Destination Operations register */
#define RIO_DOCAR_REG_OFFSET                 0x1C           /**<  */
#define RIO_DOCAR_NREAD                      0x00008000     /**< Nread logical I/O operation */
#define RIO_DOCAR_NWRITE                     0x00004000     /**< Nwrite logical I/O operation */
#define RIO_DOCAR_STREAM_WRITE               0x00002000     /**< Swrite logical I/O operation */
#define RIO_DOCAR_NWRITE_WITH_RESPONSE       0x00001000     /**< Nwrite_R logical I/O operation */
#define RIO_DOCAR_MESSAGE                    0x00000800     /**< Message operation */
#define RIO_DOCAR_DOORBELL                   0x00000400     /**< Doorbell operation */
#define RIO_DOCAR_ATOMIC_TEST_AND_SWAP       0x00000100     /**< Atomic-Test-and-Swap logical I/O operation */
#define RIO_DOCAR_ATOMIC_INC                 0x00000080     /**< Atomic-Inc logical I/O operation */
#define RIO_DOCAR_ATOMIC_DEC                 0x00000040     /**< Atomic-Dec logical I/O operation */
#define RIO_DOCAR_ATOMIC_SET                 0x00000020     /**< Atomic-Set logical I/O operation */
#define RIO_DOCAR_ATOMIC_CLEAR               0x00000010     /**< Atomic-Clear logical I/O operation */
#define RIO_DOCAR_PORT_WRITE                 0x00000004     /**< Port-Write operation */

/* Mailbox Command and Status Register */
#define RIO_MCSR_REG_OFFSET                  0x40           /**<  */
#define RIO_MCSR_MSG_UNIT_0_READY            0x80000000     /**<  */
#define RIO_MCSR_MSG_UNIT_0_FULL             0x40000000     /**<  */
#define RIO_MCSR_MSG_UNIT_0_EMPTY            0x20000000     /**<  */
#define RIO_MCSR_MSG_UNIT_0_BUSY             0x10000000     /**<  */
#define RIO_MCSR_MSG_UNIT_0_ERR              0x08000000     /**<  */
#define RIO_MCSR_MSG_UNIT_1_READY            0x00800000     /**<  */
#define RIO_MCSR_MSG_UNIT_1_FULL             0x00400000     /**<  */
#define RIO_MCSR_MSG_UNIT_1_EMPTY            0x00200000     /**<  */
#define RIO_MCSR_MSG_UNIT_1_BUSY             0x00100000     /**<  */
#define RIO_MCSR_MSG_UNIT_1_ERR              0x00080000     /**<  */
 
/* Doorbell and Port-Write control and status register */
#define RIO_PWDCSR_REG_OFFSET                0x4C           /**<  */
#define RIO_PWDCSR_DOORBELL_READY            0x80000000     /**<  */
#define RIO_PWDCSR_DOORBELL_FULL             0x40000000     /**<  */
#define RIO_PWDCSR_DOORBELL_EMPTY            0x20000000     /**<  */
#define RIO_PWDCSR_DOORBELL_BUSY             0x10000000     /**<  */
#define RIO_PWDCSR_DOORBELL_ERR              0x08000000     /**<  */
#define RIO_PWDCSR_PW_READY                  0x00000080     /**<  */
#define RIO_PWDCSR_PW_FULL                   0x00000040     /**<  */
#define RIO_PWDCSR_PW_EMPTY                  0x00000020     /**<  */
#define RIO_PWDCSR_PW_BUSY                   0x00000010     /**<  */
#define RIO_PWDCSR_PW_FAIL                   0x00000008     /**<  */
#define RIO_PWDCSR_PW_ERR                    0x00000004     /**<  */

/* Processing Element Logical Layer Control control and statuc register */

/* Local Configuration Space Base Address CSR */
#define RIO_LCSBACSR_OFFSET                  0x5C           /**<  */
#define RIO_LCSBACSR_ADDR_MASK               0x7FFE         /**<  */
#define RIO_LCSBACSR_ADDR_SHIFT              3              /**<  */

/* Base Device Id register */
#define RIO_BDID_REG_OFFSET                  0x60           /**<  */
#define RIO_BDID_SMALL_SYS_BID_MASK          0x00FF0000     /**<  */
#define RIO_BDID_SMALL_SYS_BID_SHIFT         16             /**<  */
#define RIO_BDID_LARGE_SYS_BID_MASK          0x0000FFFF     /**<  */

/* Host Base Device Id lock register */
#define RIO_HBDIDL_REG_OFFSET                0x68           /**<  */
#define RIO_HBDIDL_LOCK_MASK                 0x0000FFFF     /**<  */
#define RIO_HBDIDL_UNLOCKED                  0x0000FFFF     /**<  */

/* Component Tag register */
#define CTCSR_REG_OFFSET                     0x6C           /**<  */

/* Switch Route Config DestId CSR*/
#define RIO_SWITCH_ROUTE_DEST_ID_CSR_OFFSET  0x70           /**<  */

/* Switch Route Output Port CSR */
#define RIO_SWITCH_ROUTE_OUT_PORT_CSR_OFFSET 0x74           /**<  */
#define RIO_SWITCH_ROUTE_OUT_PORT_MASK       0x0000000F     /**<  */

/* @} */

/**************************************************************************//**
 @Collection    RapidIO Extended Features Registers definitions

 @{
*//***************************************************************************/

/* General Control register */
#define RIO_GCCSR_OFFSET_FROM_EXT_FEATURES   0x3C           /**<  */
#define RIO_GCCSR_HOST_DEVICE_ENABLE         0x80000000     /**<  */
#define RIO_GCCSR_MASTER_DEVICE_ENABLE       0x40000000     /**<  */
#define RIO_GCCSR_DEVICE_DISCOVERED          0x20000000     /**<  */

/* Error and Status register */
#define RIO_ESCSR_OFFSET_FROM_EXT_FEATURES   0x58           /**<  */
#define RIO_ESCSR_STEP                       0x20           /**<  */
#define RIO_ESCSR_INPUT_OUTPUT_INITIALIZED   0x00000002     /**<  */

/* @} */

/** @} */ /* end of rio_std_id group */

#endif /* __RIO_STD_EXT_H */
