/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_PLATFORM_EXT_H
#define __SYS_PLATFORM_EXT_H

#include "std_ext.h"
#include "platform_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_platform_grp    System Platform Interface

 @Description   Bare-board system interface for platform-related operations.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      SYS_GetMemoryMappedModuleBase

 @Description   Get the base address of a memory-mapped object.

 @Param[in]     module  - Object type (usually a sub-module type)
 @Param[in]     id      - Object identifier

 @Return        Base address of a memory-mapped object.
*//***************************************************************************/
static __inline__ uint32_t SYS_GetMemoryMappedModuleBase(e_SysModule module, uint32_t id)
{
    return PLATFORM_GetMemoryMappedModuleBase(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                              module,
                                              id);
}

/**************************************************************************//**
 @Function      SYS_GetInterruptId

 @Description   Gets the platform's interrupt ID of a given controller object.

 @Param[in]     module          - Controller object type (usually a sub-module type)
 @Param[in]     id              - Controller object identifier
 @Param[in]     intrType        - Interrupt type selection
 @Param[in]     intrRelatedId   - Specific ID that may be related to some
                                  interrupt types

 @Return        The interrupt ID of a given controller object;
                Returns -1 if not found or on any error.
*//***************************************************************************/
static __inline__ int SYS_GetInterruptId(e_SysModule        module,
                                         uint32_t           id,
                                         e_InterruptType    intrType,
                                         uint32_t           intrRelatedId)
{
    return PLATFORM_GetInterruptId(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                   module,
                                   id,
                                   intrType,
                                   intrRelatedId);
}

/**************************************************************************//**
 @Function      SYS_GetControllerClk

 @Description   Gets the input clock frequency of a given controller object.

 @Param[in]     module  - Controller object type (usually a sub-module type)
 @Param[in]     id      - Controller object identifier

 @Return        The input clock frequency of a given controller object (Hz).
*//***************************************************************************/
static __inline__ uint32_t SYS_GetControllerClk(e_SysModule module, uint32_t id)
{
    return PLATFORM_GetControllerClk(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                     module,
                                     id);
}

/**************************************************************************//**
 @Function      SYS_GetCoreClk

 @Description   Gets the core clock frequency of the device.

 @Return        The core clock frequency (Hz).
*//***************************************************************************/
static __inline__ uint32_t SYS_GetCoreClk(void)
{
    return PLATFORM_GetCoreClk(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM));
}

/**************************************************************************//**
 @Function      SYS_GetSystemBusClk

 @Description   Gets the system bus frequency of the device.

 @Return        The system bus frequency (Hz).
*//***************************************************************************/
static __inline__ uint32_t SYS_GetSystemBusClk(void)
{
    return PLATFORM_GetSystemBusClk(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM));
}

/**************************************************************************//**
 @Function      SYS_GetLocalBusClk

 @Description   Gets the local bus frequency of the device.

 @Return        The local bus frequency (Hz).
*//***************************************************************************/
static __inline__ uint32_t SYS_GetLocalBusClk(void)
{
    return PLATFORM_GetLocalBusClk(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM));
}

/**************************************************************************//**
 @Function      SYS_GetChipRevInfo

 @Description   Retrieves the revision information of the device.

 @Param[out]    p_ChipRevInfo - Returns the revision information structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_GetChipRevInfo(t_ChipRevInfo *p_ChipRevInfo)
{
    return PLATFORM_GetChipRevInfo(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                   p_ChipRevInfo);
}

/**************************************************************************//**
 @Function      SYS_GetTransactionSourceName

 @Description   Returns the name of a given bus transaction source.

 @Param[in]     transSrc - The requested transaction source.

 @Return        The name (string) of the given transaction source.
*//***************************************************************************/
static __inline__ char * SYS_GetTransactionSourceName(e_TransSrc transSrc)
{
    return PLATFORM_GetTransactionSourceName(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                             transSrc);

}

/**************************************************************************//**
 @Function      SYS_GetLocalAccessWindowInfo

 @Description   Retrieves the base address and size of a local access window that
                is associated with a given controller object.

 @Param[in]     module      - Controller object type (usually a sub-module type)
 @Param[in]     id          - Controller object identifier
 @Param[out]    p_BaseAddr  - Returns the base address of the local access window
 @Param[out]    p_Size      - Returns the size of the local access window

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_GetLocalAccessWindowInfo(e_SysModule  module,
                                                       uint32_t     id,
                                                       uint32_t     *p_BaseAddr,
                                                       uint32_t     *p_Size)
{
        return PLATFORM_GetLocalAccessWindowInfo(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),
                                                 module,
                                                 id,
                                                 p_BaseAddr,
                                                 p_Size);
}

/**************************************************************************//**
 @Function      SYS_SetLed

 @Description   Sets a selected LED on the board to on/off state.

 @Param[in]     led     - LED color selection
 @Param[in]     ledOn   - TRUE to turn the LED on; FALSE to turn it off.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_SetLed(e_LedColor led, bool ledOn)
{
    return PLATFORM_SetLed(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM), led, ledOn);
}

/**************************************************************************//**
 @Function      SYS_ConnectModuleToExtReq

 @Description   Connects a specific module of the chip to external request signal.
                Usually used when a Ucode is  using the exteranl request as an interrupt signal
                for controlling the block, without a CPU help.

 @Param[in]     module      - Module to be connected.
 @Param[in]     extReqNum   - External request number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
static __inline__ t_Error SYS_ConnectModuleToExtReq(e_SysModule module,uint8_t extReqNum)
{
    return PLATFORM_ConnectExternalRequest(SYS_GetUniqueHandle(e_SYS_MODULE_PLATFORM),  module, extReqNum);

}

/** @} */ /* end of sys_platform_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_PLATFORM_EXT_H */

