/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**************************************************************************//**
 @File          tasklet_ext.h

 @Description   structures and definitions for initialize and operate the
                Tasklet module.
                This module ...
*//***************************************************************************/
#ifndef TASKLET_EXT_H
#define TASKLET_EXT_H

#include "std_ext.h"
#include "list_ext.h"


/**************************************************************************//**
 @Group         Tasklet - Tasklets module

 @Description   Tasklet module functions,definitions and enums.
                Advantages:
                 - Bottom halfs are implemented as a linked list.  You can have as many
                   of them, as you want.
                 - No more scanning of a bit field is required upon call of a bottom half.
                 - Support for chained bottom half lists.  The RunTasksQueue() function can be
                   used as a bottom half handler.  This is for example useful for bottom
                   halfs, which want to be delayed until the next clock tick.

                 Notes:
                 - Bottom halfs are called in the reverse order that they were linked into
                   the list.

 @{
*//***************************************************************************/

typedef enum
{
    e_IMMEDIATE_TASKLET,
    e_DELAYED_TASKLET
} e_TaskPriority;


typedef struct {
    t_List   node;                  /**< Linked list of active bh's  */
    bool     readyToRun;            /**< Must be initialized to zero */
    void     (*routine)(void *);    /**< Function to call            */
    void     *data;                 /**< Argument to the function    */
} t_Tasklet;
#define TASKLET_OBJ(ptr) LIST_OBJECT(ptr, t_Tasklet, node)


typedef t_List t_TasksQ;


/*
 * Emit code to initialise a tq_struct's routine and data pointers
 */
#define PREPARE_TQUEUE(_tq, _routine, _data) \
    do {                                     \
        (_tq)->routine = _routine;           \
        (_tq)->data = _data;                 \
    } while (0)

/*
 * Emit code to initialise all of a tq_struct
 */
#define INIT_TQUEUE(_tq, _routine, _data)           \
    do {                                            \
        INIT_LIST(&(_tq)->node);                    \
        (_tq)->readyToRun = FALSE;                  \
        PREPARE_TQUEUE((_tq), (_routine), (_data)); \
    } while (0)


#define DECLARE_TASK_QUEUE(q)     LIST(q)
#define INIT_TASK_QUEUE(q)        INIT_LIST(q)


/**************************************************************************//**
 @Function      TASKLET_InitModule

 @Description   Initialize the tasklet module.

                This routine ....

*//***************************************************************************/
void TASKLET_InitModule(void);

/**************************************************************************//**
 @Function      TASKLET_ExitModule

 @Description   Exit the tasklet module.

                This routine ....

*//***************************************************************************/
void TASKLET_ExitModule(void);


/**************************************************************************//**
 @Function      TASKLET_Init

 @Description   If scheduled - marks for deletion, otherwise - deletes.

 @Return        E_OK - If this timer is queued successfully, error code - otherwise.
*//***************************************************************************/
t_Tasklet * TASKLET_Init(void (*routine)(void *), void *data);

/**************************************************************************//**
 @Function      TASKLET_Free

 @Description   If scheduled - marks for deletion, otherwise - deletes.

 @Param[in]     p_Tasklet - Pointer to tasklet structure to be queued.
*//***************************************************************************/
void TASKLET_Free(t_Tasklet *p_Tasklet);

/**************************************************************************//**
 @Function      TASKLET_QueueTask

 @Description   Initialize timer structure.

                The routine initializing the given timer structure with the
                given parameters.

 @Param[in]     p_Tasklet - Pointer to tasklet structure to be queued.
 @Param[in]     priority  - Priority to schedule this tasklet.

 @Return        E_OK - If this timer is queued successfully, error code - otherwise.
*//***************************************************************************/
t_Error TASKLET_QueueTask(t_Tasklet *p_Tasklet, e_TaskPriority priority);

/**************************************************************************//**
 @Function      TASKLET_IsQueued

 @Description   Check if a tasklet is queued.

                The routine checks whether the given tasklet is queued to the
                tasks list or not.

 @Param[in]     p_Tasklet  -  Pointer to tasklet structure.

 @Return        1 - If this tasklet is queued, 0 - otherwise.
*//***************************************************************************/
bool TASKLET_IsQueued(t_Tasklet *p_Tasklet);

/**************************************************************************//**
 @Function      TASKLET_SetData

 @Description   Update the data field of a timer.

                The routine updates the data field of the given timer by the
                given data.

 @Param[in]     p_Tasklet      -  Pointer to tasklet structure.
 @Param[in]     data           -  The argument for the callback routine.
*//***************************************************************************/
void TASKLET_SetData(t_Tasklet *p_Tasklet, t_Handle data);

/**************************************************************************//**
 @Function      TASKLET_GetData

 @Description   Get the data field of a tasklet.

                The routine returns the data field of the given tasklet.

 @Param[in]     p_Tasklet    -  Pointer to tasklet structure.

 @Return        t_Handle:p_Tasklet->data.
*//***************************************************************************/
t_Handle TASKLET_GetData(t_Tasklet *p_Tasklet);

/**************************************************************************//**
 @Function      TASKLET_RunTasksQueue

 @Description   Run all scheduled tasklets in the given queue.

                This routine ....

 @Param[in]     p_TaskQ    -  Pointer to tasklets queue.
*//***************************************************************************/
void TASKLET_RunTasksQueue(t_TasksQ *p_TaskQ);

/**************************************************************************//**
 @Function      TASKLET_FlushScheduledTasks

 @Description   Flush all scheduled tasklets.

                This routine ....

*//***************************************************************************/
void TASKLET_FlushScheduledTasks(void);

/**************************************************************************//**
 @Function      TASKLET_FlushEmptyTasks

 @Description   Flushes all empty tasklets (releases their memory).

*//***************************************************************************/
void TASKLET_FlushEmptyTasks(void);


/** @} */ /* end of group */


#endif /* TASKLET_EXT_H */
