/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/

#include "std_ext.h"
#include "error_ext.h"
#include "string_ext.h"
#include "sys_ext.h"
#include "mem_ext.h"
#include "buf_ext.h"
#include "mb_frame_ext.h"
#include "data_pool_mng.h"


t_Handle DataPoolMngGetPoolHandle(t_Handle       h_DataPoolMng,
                                         e_SysModule    module,
                                         uint32_t       id);

t_Error DataPoolMngPoolInit(t_Handle                 h_DataPoolMng,
                                   t_DataPoolSettings *p_DataPoolSettings,
								   uint32_t id);

t_Error  DataPoolMngPoolFree(t_Handle    h_DataPoolMng,
                                    e_SysModule module,
                                    uint32_t    id);


/*****************************************************************************/
t_Handle DATA_POOL_MNG_Init(t_DataPoolMngParam *p_DataPoolMngParam)
{
    t_DataPoolMng               *p_DataPoolMng;
	#if !defined (__QNXNTO__)
    t_SysSubModuleRegisterParam objRegParam;
    e_SysModule                 subModules[] = { e_SYS_SUBMODULE_DATA_POOL };
	#endif
    t_Error                     errCode;

    SANITY_CHECK_RETURN_VALUE(p_DataPoolMngParam, E_NULL_POINTER, NULL);

    /* Allocate the data pools manager structure */
    p_DataPoolMng = XX_Malloc(sizeof(t_DataPoolMng));
    if (!p_DataPoolMng)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Data pools manager"));
        return NULL;
    }
    memset(p_DataPoolMng, 0, sizeof(t_DataPoolMng));

    p_DataPoolMng->numOfDataPools = MAX_NUM_OF_DATA_POOLS;

    /* Allocate data pool handles */
    p_DataPoolMng->p_DataPools = (t_DataPoolCtrl *)
        XX_Malloc(sizeof(t_DataPoolCtrl) * p_DataPoolMng->numOfDataPools);
    if (!p_DataPoolMng->p_DataPools)
    {
        DATA_POOL_MNG_Free(p_DataPoolMng);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Data pools handles"));
        return NULL;
    }
    memset(p_DataPoolMng->p_DataPools, 0, sizeof(t_DataPoolCtrl) * p_DataPoolMng->numOfDataPools);

    /* Initialize pools for buffer and frame objects */

    if (p_DataPoolMngParam->numOfBufferObjects)
    {
        errCode = B_InitPool(p_DataPoolMngParam->numOfBufferObjects);
        if (errCode != E_OK)
        {
            DATA_POOL_MNG_Free(p_DataPoolMng);
            REPORT_ERROR(MAJOR, errCode, NO_MSG);
            return NULL;
        }
    }

    if (p_DataPoolMngParam->numOfFrameObjects)
    {
        errCode = F_InitPool(p_DataPoolMngParam->numOfFrameObjects);
        if (errCode != E_OK)
        {
            DATA_POOL_MNG_Free(p_DataPoolMng);
            REPORT_ERROR(MAJOR, errCode, NO_MSG);
            return NULL;
        }
    }

#if !defined __QNXNTO__
    /* Register sub-modules */
    objRegParam.owner = e_SYS_MODULE_DATA_POOL_MNG;
    objRegParam.numOfSubModules = ARRAY_SIZE(subModules);
    objRegParam.p_SubModules = subModules;
    objRegParam.f_InitSubModule = DataPoolMngPoolInit;
    objRegParam.f_FreeSubModule = DataPoolMngPoolFree;
    objRegParam.f_GetSubModule = DataPoolMngGetPoolHandle;

    errCode = SYS_RegisterSubModules(&objRegParam);
    ASSERT_COND(errCode == E_OK);
#endif
    return p_DataPoolMng;
}


/*****************************************************************************/
t_Error DATA_POOL_MNG_Free(t_Handle h_DataPoolMng)
{
    t_DataPoolMng   *p_DataPoolMng = (t_DataPoolMng *)h_DataPoolMng;

    SANITY_CHECK_RETURN_ERROR(p_DataPoolMng, E_INVALID_HANDLE);

    /* free buffer and frame object pools */
    F_FreePool();
    B_FreePool();

    if (p_DataPoolMng->p_DataPools)
    {
        XX_Free(p_DataPoolMng->p_DataPools);
    }
    XX_Free(p_DataPoolMng);

    return E_OK;
}


/*****************************************************************************/
t_Error DATA_POOL_MNG_GetPoolInfo(t_Handle          h_DataPoolMng,
                                  uint32_t          poolId,
                                  t_DataPoolInfo    *p_DataPoolInfo)
{
    t_DataPoolMng   *p_DataPoolMng = (t_DataPoolMng *)h_DataPoolMng;

    SANITY_CHECK_RETURN_ERROR(p_DataPoolMng, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_DataPoolInfo, E_NULL_POINTER);

    if (poolId >= p_DataPoolMng->numOfDataPools)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Data pool ID (should be 0-%d)", p_DataPoolMng->numOfDataPools - 1));
    }

    if (p_DataPoolMng->p_DataPools[poolId].bufMemAccess.h_MemMng)
    {
        p_DataPoolInfo->p_BufMemAccess = &(p_DataPoolMng->p_DataPools[poolId].bufMemAccess);
        p_DataPoolInfo->memPartitionId = p_DataPoolMng->p_DataPools[poolId].partitionId;
        p_DataPoolInfo->bufferSize =
            MEM_GetSize(p_DataPoolMng->p_DataPools[poolId].bufMemAccess.h_MemMng);
    }
    else
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Data pool %d", poolId));
    }

    return E_OK;
}


/*****************************************************************************/
uint8_t * DATA_POOL_DefaultGetBuf(t_Handle h_BufMemAccess, t_Handle *p_BufContextHandle)
{
    t_BufMemAccess  *p_BufMemAccess = (t_BufMemAccess *)h_BufMemAccess;
    t_Handle        h_Buf;

    ASSERT_COND(p_BufContextHandle);

    h_Buf = B_New(p_BufMemAccess,
                  MEM_GetSize(p_BufMemAccess->h_MemMng),
                  MEM_GetPrefixSize(p_BufMemAccess->h_MemMng),
                  NULL,
                  0);
    if (!h_Buf)
    {
        REPORT_ERROR(MINOR, E_NOT_AVAILABLE, ("Data buffer"));
        return NULL;
    }
    *p_BufContextHandle = h_Buf;

    return B_GetData(h_Buf);
}


/*****************************************************************************/
t_Error DATA_POOL_DefaultPutBuf(t_Handle h_BufMemAccess, uint8_t *p_Buffer, t_Handle h_BufContext)
{
    UNUSED(h_BufMemAccess);
    UNUSED(p_Buffer);

    ASSERT_COND(p_Buffer == B_GetData(h_BufContext));

    B_Delete(h_BufContext, TRUE);

    return E_OK;
}


/*****************************************************************************/
t_Handle DataPoolMngGetPoolHandle(t_Handle  h_DataPoolMng,
                                         e_SysModule    module,
                                         uint32_t       id)
{
    t_DataPoolMng   *p_DataPoolMng = (t_DataPoolMng *)h_DataPoolMng;

    ASSERT_COND(module == e_SYS_SUBMODULE_DATA_POOL);
    UNUSED(module);

    if (id >= p_DataPoolMng->numOfDataPools)
    {
        REPORT_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Data pool ID (should be 0-%d)", p_DataPoolMng->numOfDataPools - 1));
        return NULL;
    }

    return p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng;
}


/*****************************************************************************/
t_Error DataPoolMngPoolInit(t_Handle                 h_DataPoolMng,
                                   t_DataPoolSettings *p_DataPoolSettings,
								   uint32_t id)
{
    t_DataPoolMng       *p_DataPoolMng = (t_DataPoolMng *)h_DataPoolMng;
  //  t_DataPoolSettings  *p_DataPoolSettings;
    t_Error             errCode;
    uint8_t             *p_MemAddress;
    uint32_t            totalMemSize;
    char                dataPoolName[] = "DP00";

    ASSERT_COND(p_DataPoolMng);
    ASSERT_COND(p_DataPoolSettings);


    if (id >= p_DataPoolMng->numOfDataPools)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Data pool ID %d (should be 0-%d)", id, p_DataPoolMng->numOfDataPools - 1));
    }

    if (p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng)
    {
        RETURN_ERROR(MAJOR, E_ALREADY_EXISTS, ("Data pool %d", id));
    }

    /* Cast the settings pointer to the appropriate data structure */
    //p_DataPoolSettings = (t_DataPoolSettings *)p_SubModuleSettings->p_Settings;

    /* Initialize the data pool and store the handle */

    totalMemSize = MEM_ComputePartitionSize(p_DataPoolSettings->numOfBuffers,
                                            p_DataPoolSettings->bufferSize,
                                            p_DataPoolSettings->prefix,
                                            p_DataPoolSettings->postfix,
                                            p_DataPoolSettings->alignment);

    p_MemAddress = (uint8_t *)
        XX_MallocSmart(totalMemSize, p_DataPoolSettings->partitionId, 1);
    if (!p_MemAddress)
    {
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Data pool %d", id));
    }

    dataPoolName[2] = (char)('0' + id/10);
    dataPoolName[3] = (char)('0' + id%10);

    errCode = MEM_InitByAddress(dataPoolName,
                                &(p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng),
                                p_DataPoolSettings->numOfBuffers,
                                p_DataPoolSettings->bufferSize,
                                p_DataPoolSettings->prefix,
                                p_DataPoolSettings->postfix,
                                p_DataPoolSettings->alignment,
                                p_MemAddress);
    if (errCode != E_OK)
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Data pool %d", id));
    }
	XX_Print("%s: %s: nbuf %d size %d pref %d post %d align %d vaddr 0x%p totalSize %d partId 0x%x", __FUNCTION__,
			dataPoolName,
			p_DataPoolSettings->numOfBuffers,
			p_DataPoolSettings->bufferSize,
			p_DataPoolSettings->prefix,
			p_DataPoolSettings->postfix,
			p_DataPoolSettings->alignment,
			p_MemAddress, totalMemSize, p_DataPoolSettings->partitionId);

    p_DataPoolMng->p_DataPools[id].bufMemAccess.f_GetMem = MEM_Get;
    p_DataPoolMng->p_DataPools[id].bufMemAccess.f_PutMem = MEM_Put;

    /* Store data pool parameters */
    p_DataPoolMng->p_DataPools[id].partitionId = p_DataPoolSettings->partitionId;

    return E_OK;
}


/*****************************************************************************/
t_Error DataPoolMngPoolFree(t_Handle     h_DataPoolMng,
                                   e_SysModule  module,
                                   uint32_t     id)
{
    t_DataPoolMng   *p_DataPoolMng = (t_DataPoolMng *)h_DataPoolMng;

    ASSERT_COND(module == e_SYS_SUBMODULE_DATA_POOL);
    UNUSED(module);

    if (id >= p_DataPoolMng->numOfDataPools)
    {
        RETURN_ERROR(MAJOR, E_NOT_IN_RANGE,
                     ("Data pool ID (should be 0-%d)", p_DataPoolMng->numOfDataPools - 1));
    }

    if (p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng)
    {
        /* Free the data pool and update the handle */
        XX_FreeSmart(MEM_GetBase(p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng));
        MEM_Free(p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng);

        p_DataPoolMng->p_DataPools[id].bufMemAccess.h_MemMng = NULL;
    }

    return E_OK;
}


