/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

/******************************************************************************

 Copyright © 1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/

#include "std_ext.h"
#include "stdlib_ext.h"
#include "stdarg_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "part_ext.h"
#include "list_ext.h"

#include "xx.h"
#include "xx_ext.h"
#include <assert.h>
#include <semaphore.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <errno.h>
#include <sys/neutrino.h>
#include <time.h>


#define XX_CACHELINE_SIZE	64

typedef struct {
    void * vaddr;	/* The virtual address */
    uint64_t size;      /* Size passed to  mmap */
    uint64_t paddr;	/* Physical address */
    uint32_t partition_id;
	int		 cached; /* Whether cached or not */
} t_MmapInfo;

typedef struct {
    t_MmapInfo	meminfo;
    t_List      node;
} t_AllocEntry;


static LIST(g_MmapList); /* List to keep track of mmaps.*/
#define ALLOCENTRY_OBJ(ptr)  LIST_OBJECT(ptr, t_AllocEntry, node)

#ifndef DISABLE_UNUSED_CODE
static void DebugPrintMemList()
{
	t_AllocEntry    *p_AllocEntry;
   	t_List          *p_Pos;
	int i = 0;
   
 	LIST_FOR_EACH(p_Pos, &g_MmapList)
	{
		p_AllocEntry = ALLOCENTRY_OBJ(p_Pos);

		printf("Alloc Entry %d  Vaddr:%08x  Paddr:%08x Size:%08x\n Cached:%d",i, (uint32_t)p_AllocEntry->meminfo.vaddr,
										(uint32_t)CAST_UINT64_TO_POINTER(p_AllocEntry->meminfo.paddr),
										(uint32_t)CAST_UINT64_TO_POINTER(p_AllocEntry->meminfo.size),
										p_AllocEntry->meminfo.cached);		
		i++;								
	}
}
#endif


static void AddEntryToMmapList(void * vaddr, uint64_t size, uint64_t paddr, uint32_t partition_id, int cached)
{
    t_AllocEntry *entry = XX_Malloc(sizeof(t_AllocEntry));
    if(entry == NULL) {
         REPORT_ERROR(MAJOR,E_NO_MEMORY, ("Cannot allocate AllocEntry"));
	 assert(0);
    }

    entry->meminfo.vaddr = vaddr;
    entry->meminfo.size = size;
    entry->meminfo.paddr = paddr;
    entry->meminfo.partition_id = partition_id;
	entry->meminfo.cached = cached;
	    
    INIT_LIST(&entry->node);
    LIST_AddToTail(&entry->node, &g_MmapList);
    
    //DebugPrintMemList();
    
}


#define ISALIGN(address, alignment) (!(address & (alignment-1)))
static void * alloc_phy_aligned(uint32_t size, uint32_t alignment, int cached)
{
	void * addr = NULL;
	off64_t	offset;
	int mmap_prot = PROT_READ | PROT_WRITE;

	if(!cached) {
		mmap_prot |= PROT_NOCACHE;
	}

	if( (!POWER_OF_2(alignment)) && alignment != 1) {
        REPORT_ERROR(MAJOR,E_INVALID_VALUE, ("Alignment must be power of 2"));
		return NULL;
	}

	/* 	QNX mmap is atleast 4K aligned. Need to check if this
		assumption is true for the corresponding physical address
		as well. The check at the end will validate this.
	*/
	if(alignment <= (4*1024))
	{
		/* Nothing special to do for this */
		addr = mmap(0, size,  		/* addr, size */
			mmap_prot,  			/* prot */
			MAP_PHYS|MAP_ANON,     	/* flags */
			NOFD,					/* fd */
			0);						/* offset */

		if(addr == MAP_FAILED) {
        	REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mmap failed with %d",errno));
			addr = NULL;
		}
	}
	else
	{
		/* Need to allocate extra to meet alignment requirements */
		off64_t alignedPhyAddr = 0;

		uint32_t firstTry;
		firstTry = (uint32_t)mmap(0, size + (alignment - 1),
			mmap_prot,  					/* prot */
			MAP_PRIVATE| MAP_PHYS|MAP_ANON, /* flags - MAP private as we don't want other processes 
													to get this memory*/
			NOFD,							/* fd */
			0);								/* offset */

		if((void*)firstTry == MAP_FAILED) {
        		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mmap failed with %d",errno));
				return NULL;
			}

		/* Get the physical address and check alignment */
		if(mem_offset64((void*)firstTry, NOFD, 1, &offset, 0) == -1) {
        	REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mem_offset64 failed"));
			if(munmap((void*)firstTry,size + (alignment - 1)) == -1){
        		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mummap failed with %d",errno));
			}
			addr =  NULL;
		}
		else
		{
			/* Adjust for alignment */
			alignedPhyAddr = offset + (alignment - (offset % alignment));
			/* map the physical memory which was  */
			addr = mmap64(0, size,
					mmap_prot,
					MAP_PHYS|MAP_SHARED,
					NOFD,
					alignedPhyAddr );

			if(addr == MAP_FAILED) {
				/* Mapping physical address failed */
        		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mmap failed for physicall address with %d",errno));
				addr = NULL;
			}
			/* Unmap the first mapping */
			/* 	Problem when unmapping this memory. The next mmap returns a region with the
			 	same physical address. disabling the mmap for now
			*/
		//	if(munmap((void*)firstTry,size + (alignment - 1)) == -1){
       // 		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mummap failed with %d",errno));
	//		}
		}
	}

	/* Check if our assumptions are valid */
	/* Check virtual address alignment */
//	if(!ISALIGN((uint32_t)addr,alignment)) {
//		printf(" Virtual address is not aligned (that is ok)\n");
//	}

	/* Keep track of this allocation for later PhysToVirt lookups */
	if(addr) {
		AddEntryToMmapList(addr, size, XX_VirtToPhys(addr), 0, cached);
		
	}
	

	if(mem_offset64(addr, NOFD, 1, &offset, 0) == -1) {
       	REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("mem_offset64 failed"));
		} else {
		//printf("Physical address is %08x\n",(uint32_t) offset);
		if(!ISALIGN((uint32_t)offset, alignment)) {
       		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Physical address %08x is not aligned",(uint32_t)offset));
			}
		}
	//printf("alloc_phy_aligned: %08x (%08x) (%d) align:%08x (%d)\n",(uint32_t)addr,size,size,alignment,alignment); 
	return (void*)addr;
}

/* Need to unmap memory here */
static void FreeSmartAllocedMemory(void * vaddr)
{
    t_AllocEntry    *p_AllocEntry;
    t_List          *p_Pos;


        LIST_FOR_EACH(p_Pos, &g_MmapList)
    {
            p_AllocEntry = ALLOCENTRY_OBJ(p_Pos);

        if(p_AllocEntry->meminfo.vaddr == vaddr) {
        /* Found it */
#if defined(BUILDENV_iox)
            if (XX_g_MemSize && p_AllocEntry->meminfo.cached) {
                /* appliction supplied memory free not supported */
                printf("Freeing application supplied memory @ 0x%p\n", vaddr);
                printf("memory size: Initial 0x%x, Available 0x%x\n",
                        XX_g_MemSize, xx_l_MemAvlbl);
                printf("virtual address: Initial %p, Next %p\n",
                        XX_g_VaddrBase, xx_l_VaddrNext);
                printf("physical address: Initial %llx, Next %llx\n",
                        XX_g_PaddrBase, xx_l_PaddrNext);
                assert(0);
            }
#endif
            if(munmap(vaddr, p_AllocEntry->meminfo.size) != 0) {
                REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("munmap failed"));


            }
            LIST_Del(&p_AllocEntry->node);
            XX_Free(p_AllocEntry);
            break;

        }

        }
}

/************** CISCO Specific Addition marked with BUILDENV_iox **************************/

#if defined(BUILDENV_iox)
static uint64_t XX_g_PaddrBase;    /* Application supplied memory paddr */
static unsigned char *XX_g_VaddrBase;    /* Application supplied memory vaddr */
static uint32_t XX_g_MemSize;      /* Application supplied memory size */

static uint64_t xx_l_PaddrNext;    /* Next available memory paddr */
static unsigned char *xx_l_VaddrNext;    /* Next available memory vaddr */
static uint32_t xx_l_MemAvlbl;     /* Available memory size */

static void *
alloc_phy_iox(uint32_t size, uint32_t alignment, int cached)
{
    void *addr;

    if (!XX_g_MemSize) {
        /* application has not supplied memory */
        return alloc_phy_aligned(size, alignment, cached);
    }

    /* adjust size to page boundary */
    size += 0xfff;
    size &= ~0xfff;

    if (size > xx_l_MemAvlbl) {
        printf("No more memory! Initial 0x%x, Available 0x%x Need 0x%x\n",
                XX_g_MemSize, xx_l_MemAvlbl, size);
        printf("virtual address: Initial %p, Next %p\n",
                XX_g_VaddrBase, xx_l_VaddrNext);
        printf("physical address: Initial %llx, Next %llx\n",
                XX_g_PaddrBase, xx_l_PaddrNext);
        assert(0);
    }

    /* stash for lookup */
    AddEntryToMmapList(xx_l_VaddrNext, size, xx_l_PaddrNext, 0, cached);

    addr = xx_l_VaddrNext;
    xx_l_VaddrNext += size;
    xx_l_PaddrNext += size;
    xx_l_MemAvlbl -= size;

    return (addr);
}

void
XX_UseAppMemory(void *vaddr, uint64_t paddr, uint32_t size)
{
    XX_g_VaddrBase = vaddr;
    XX_g_PaddrBase = paddr;
    XX_g_MemSize = size;

    xx_l_PaddrNext = XX_g_PaddrBase;
    xx_l_VaddrNext = XX_g_VaddrBase;
    xx_l_MemAvlbl = XX_g_MemSize;

    return;
}
#endif


/* Look up the alloc list to do a quick P to V translation */
static physAddress_t VirtToPhys(const void * vaddr) 
{
	t_AllocEntry    *p_AllocEntry;
   	t_List          *p_Pos;
	physAddress_t 	paddr= 0;
   
	LIST_FOR_EACH(p_Pos, &g_MmapList)
   	{
        	p_AllocEntry = ALLOCENTRY_OBJ(p_Pos);
		if( (vaddr >= p_AllocEntry->meminfo.vaddr) && 
			(vaddr <  (p_AllocEntry->meminfo.vaddr + p_AllocEntry->meminfo.size))) {
			/* in range, compute vaddr */
			paddr = p_AllocEntry->meminfo.paddr + (vaddr - p_AllocEntry->meminfo.vaddr);
			break;
				
		}
		
	}

	return paddr;

}


static void * PhysToVirt(const uint64_t paddr)
{
	t_AllocEntry    *p_AllocEntry;
   	t_List          *p_Pos;
	void 		*vaddr = NULL;
   
    LIST_FOR_EACH(p_Pos, &g_MmapList)
   	{
        	p_AllocEntry = ALLOCENTRY_OBJ(p_Pos);
		
		if( (paddr >= p_AllocEntry->meminfo.paddr) && 
			(paddr <  (p_AllocEntry->meminfo.paddr + p_AllocEntry->meminfo.size))) {
			/* in range, compute vaddr */
			vaddr = p_AllocEntry->meminfo.vaddr + (paddr - p_AllocEntry->meminfo.paddr);
			break;
				
		}
		
    }
	if(vaddr == NULL) {
		REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Cannot find vaddr for paddr 0x%x", paddr));
	
	}
	return vaddr;
}


#ifdef DEBUG_XX_MALLOC


/*****************************************************************************/
void * XX_MallocDebug(uint32_t size, char *fname, int line)
{
	UNUSED(fname);
	UNUSED(line);
	return malloc(size);
}

/*****************************************************************************/
/* 	Partition ID 0 is non cached,
	Partition ID 10 is cached
*/
void * XX_MallocSmartDebug(uint32_t     size,
                           int          partitionId,
                           uint32_t     alignment,
                           char         *fname,
                           int          line)
{
	UNUSED(fname);
	UNUSED(line);
	return XX_MallocSmart(size,partitionId,alignment);
}

#else /* not DEBUG_XX_MALLOC */

/*****************************************************************************/
void * XX_Malloc(uint32_t size)
{
	return malloc(size);
}

/*****************************************************************************/
/* 	Partition ID 0 is non cached,
	Partition ID 10 is cached
*/
void * XX_MallocSmart(uint32_t size, int partitionId, uint32_t alignment)
{
    void * ret = NULL;
    if(partitionId == QNX_PARTITION_ID_NON_CACHED) {
        ret =  alloc_phy_aligned(size, alignment,0);
    } else
    if (partitionId == QNX_PARTITION_ID_CACHED) {
#if defined(BUILDENV_iox)
        ret = alloc_phy_iox(size, alignment, 1);
#else
        ret = alloc_phy_aligned(size, alignment,1);
#endif
    } else {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid Partition ID %d",partitionId));
    }
    return ret;
}
#endif /* not DEBUG_XX_MALLOC */

#if defined(BUILDENV_iox)
#include <pthread.h>
int (* _spin_lock_v)(sync_t *sync) = (int(*)(sync_t *sync))pthread_mutex_lock;
int (* _spin_unlock_v)(sync_t *sync) = (int(*)(sync_t *sync))pthread_mutex_unlock;
#endif

/*****************************************************************************/
void XX_Free(void *p_Memory)
{
	free(p_Memory);
}


/*****************************************************************************/
void XX_FreeSmart(void *p_Memory)
{
	FreeSmartAllocedMemory(p_Memory);
}


/*****************************************************************************/
void * XX_GetMemPartitionBase(int memPartitionId)
{
	return NULL;
}


/*****************************************************************************/
t_Error XX_SetIntr(int irq, t_Isr *f_Isr, t_Handle handle)
{
   // return SYS_SetIntr(irq, f_Isr, handle);
   	//assert(0);
	return E_OK;
}


/*****************************************************************************/
t_Error XX_FreeIntr(int irq)
{
//    return SYS_FreeIntr(irq);
   	assert(0);
	return E_OK;
}


/*****************************************************************************/
t_Error XX_EnableIntr(int irq)
{
 //   return SYS_EnableIntr(irq);
   	//assert(0);
	//InterruptUnmask(irq, -1);
	return E_OK;
}


/*****************************************************************************/
t_Error XX_DisableIntr(int irq)
{
  //  return SYS_DisableIntr(irq);
	//InterruptMask(irq, -1);
   	//assert(0);
	return E_OK;
}


/*****************************************************************************/
void XX_Exit(int status)
{
    exit(status);
}


#define NCSW_SLOG_MAJOR  18765
#define NCSW_SLOG_MINOR 0
/*****************************************************************************/
void XX_Print(char *format, ...)
{
    va_list args;
	int opcode = _SLOG_SETCODE(NCSW_SLOG_MAJOR,NCSW_SLOG_MINOR);

    va_start(args, format);
    vslogf (opcode, _SLOG_INFO, format, args);
    va_end(args);
}


/*****************************************************************************/
char XX_GetChar(void)
{
    return getchar();
}


/*****************************************************************************/
void * XX_PhysToVirt(physAddress_t addr)
{
    return PhysToVirt(addr);
}


/*****************************************************************************/
physAddress_t XX_VirtToPhys(void *addr)
{
	off64_t  offset;
	offset = VirtToPhys(addr);
	if(offset == 0) {
		if (mem_offset64 (addr, NOFD, 1, &offset, 0) == -1) {
			if(offset == 0) {
				REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Cannot find paddr for vaddr 0x%x",addr));
			}
			return (uint64_t)(0);
		}
	}
	return (offset);
}


/*****************************************************************************/
#if REPORT_EVENTS
void XX_EventById(uint32_t event, t_Handle appId, uint16_t flags, char *msg)
{
#if 0
    e_Event eventCode = (e_Event)event;

    UNUSED(eventCode);
    UNUSED(h_App);
    UNUSED(flags);
    UNUSED(msg);
#endif
}
#endif /* REPORT_EVENTS */

#if 0
/*****************************************************************************/
t_Error XX_Call(uint32_t qid, t_Error (*f)(t_Handle), t_Handle id, t_Handle h_App, uint16_t flags)
{

    return E_OK;
}

/*****************************************************************************/
/*                        Semaphore Service Routines                         */
/*****************************************************************************/
t_SemaphoreHandle XX_InitSemaphore(int val)
{
	sem_t *sem = XX_Malloc(sizeof(sem_t));
	assert(sem != NULL);
	if (sem) {
		if(sem_init(sem, 0, val) == 0) {
			//do nothing	
		}
		else {
			XX_Free(sem);
			sem = NULL;
			XX_Print("%s: %s : sem_init failed\n",__FILE__, __FUNCTION__);
		}
	}
	else {
		XX_Print("%s: %s : Cannot allocate semaphore\n",__FILE__, __FUNCTION__);
	}
	return (t_SemaphoreHandle) sem;
}


/* XX_InitBinSemaphore creates a binary semaphore, which is a semaphore */
/* with maximum count of 1. 'initialCount' should be ofcourse either 0 or 1.   */
t_SemaphoreHandle XX_InitBinSemaphore(int initialCount)
{
    return XX_InitSemaphore(initialCount);
	return NULL;
}


void XX_FreeSemaphore(t_SemaphoreHandle h_Semaphore)
{
	sem_t * sem = (sem_t*) h_Semaphore;

    if (sem) {
		if(sem_destroy(sem) == 0) {
        	XX_Free(sem);
		}
		else {
			XX_Print("%s: %s : sem_destroy failed\n",__FILE__, __FUNCTION__);
        	XX_Free(sem);
		}
	} 
	else {
		XX_Print("%s: %s : Invalid handle\n",__FILE__, __FUNCTION__);
	}
}


t_MutexHandle XX_InitMutex()
{
    return (t_MutexHandle)XX_InitSemaphore(1);
	return NULL;
}


t_MutexHandle XX_InitMutexLocked()
{
    return (t_MutexHandle)XX_InitSemaphore(0);
}


void XX_FreeMutex(t_MutexHandle h_Mutex)
{
    XX_FreeSemaphore(h_Mutex);
}


int XX_TryLock(t_MutexHandle h_Mutex)
{
    return sem_trywait((sem_t*) h_Mutex);
}


void XX_Lock(t_MutexHandle h_Mutex)
{
	sem_wait((sem_t*)h_Mutex);
}


void XX_Unlock(t_MutexHandle h_Mutex)
{
	sem_post((sem_t*)h_Mutex);
}


void XX_DownSemaphore(t_SemaphoreHandle h_Semaphore)
{
	sem_wait((sem_t*) h_Semaphore);
}


void XX_UpSemaphore(t_SemaphoreHandle h_Semaphore)
{
	sem_post((sem_t*)h_Semaphore);
}
#endif

static void * XX_MallocCacheAlign(uint32_t size)
{
	uint32_t cache_aligned_size = (size + XX_CACHELINE_SIZE-1) & ~(XX_CACHELINE_SIZE-1);
	void *pmem = XX_Malloc(2*cache_aligned_size + sizeof(uintptr_t));
	uintptr_t p = (uintptr_t)pmem;

	if(pmem == NULL)
		return pmem;

	if(((uintptr_t)pmem) & (sizeof(uintptr_t)-1)) {
        REPORT_ERROR(MINOR, E_NO_MEMORY, ("Initial mem not aligned to %d bytes (offset %d)",
			sizeof(uintptr_t), ((uintptr_t)pmem) & (sizeof(uintptr_t)-1)));
	}

	/*
	 * Align alloc mem to XX_CACHELINE_SIZE and save
	 * pointer to original allocation just before returned
	 * aligned ptr for XX_FreeCacheAlign().
	 * Note use of p+XX_CACHELINE_SIZE to ensure we use second
	 * cacheline if pmem is already aligned.
	 */
	p = (p + XX_CACHELINE_SIZE) & ~(XX_CACHELINE_SIZE-1);
	if((p-(uintptr_t)pmem) < sizeof(uintptr_t))
		p += XX_CACHELINE_SIZE; /* not enough room, move to second cacheline */
	((void**)p)[-1] = pmem; /* save orig addr for XX_FreeCacheAlign() */

	return (void *)p;
}

static void XX_FreeCacheAlign(void *p)
{
		XX_Free(((void **)p)[-1]);
}

/*****************************************************************************/
/*                        Spinlock Service Routines                          */
/*****************************************************************************/
#ifdef QNX_SPINLOCK_USES_MUTEX
#ifdef pthread_spin_lock
#undef pthread_spin_lock
#endif
#ifdef pthread_spin_trylock
#undef pthread_spin_trylock
#endif
#ifdef pthread_spin_unlock
#undef pthread_spin_unlock
#endif

#define pthread_spinlock_t		pthread_mutex_t
#define pthread_spin_init(s,a)	pthread_mutex_init(s,NULL)
#define pthread_spin_destroy	pthread_mutex_destroy
#define pthread_spin_lock		pthread_mutex_lock
#define pthread_spin_trylock	pthread_mutex_trylock
#define pthread_spin_unlock		pthread_mutex_unlock
#endif
t_Handle XX_InitSpinlock(void)
{
	pthread_spinlock_t* p_Spinlock = (pthread_spinlock_t*)XX_MallocCacheAlign(sizeof(pthread_spinlock_t));
    if (!p_Spinlock)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Spinlock structure"));
        return NULL;
    }

	if(pthread_spin_init(p_Spinlock,  PTHREAD_PROCESS_PRIVATE) != 0) {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Spinlock Init"));
		XX_FreeCacheAlign(p_Spinlock);	
		p_Spinlock = NULL;
	}
    return (t_Handle)p_Spinlock;
}

void XX_FreeSpinlock(t_Handle h_Spinlock)
{
	
	pthread_spinlock_t* p_Spinlock = (pthread_spinlock_t*)h_Spinlock;
	
    if (p_Spinlock) {
		if(pthread_spin_destroy(p_Spinlock) != 0) {
        	REPORT_ERROR(MAJOR, E_NO_MEMORY, ("pthread_spin_destroy failed"));
		}
   	    XX_FreeCacheAlign(h_Spinlock);
	}
}

void XX_LockSpinlock(t_Handle h_Spinlock)
{
	pthread_spinlock_t* p_Spinlock = (pthread_spinlock_t*)h_Spinlock;
	(void)pthread_spin_lock(p_Spinlock);
}

void XX_UnlockSpinlock(t_Handle h_Spinlock)
{
	pthread_spinlock_t* p_Spinlock = (pthread_spinlock_t*)h_Spinlock;
	(void)pthread_spin_unlock(p_Spinlock);
}

uint32_t XX_LockIntrSpinlock(t_Handle h_Spinlock)
{
#if !defined(QNX_NO_DISABLE_ALL_INTR)
	uint32_t flags = 0;
	pthread_spinlock_t* p_Spinlock = (pthread_spinlock_t*)h_Spinlock;
	do {
		flags = XX_DisableAllIntr();
		if(pthread_spin_trylock(p_Spinlock)==0)
			break; /* with interrupts disabled */
		XX_RestoreAllIntr(flags);
		//while((volatile)(p_Spinlock->__count) != 0) ;
		//cpu_relax();
	} while(1);
	return flags;
#else
	XX_LockSpinlock(h_Spinlock);
	return 0;
#endif
}

void XX_UnlockIntrSpinlock(t_Handle h_Spinlock, uint32_t intrFlags)
{
#if !defined(QNX_NO_DISABLE_ALL_INTR)
	XX_UnlockSpinlock(h_Spinlock);
	XX_RestoreAllIntr(intrFlags);
#else
	UNUSED(intrFlags);
	XX_UnlockSpinlock(h_Spinlock);
#endif
}


/*****************************************************************************/
/*                        Tasklet Service Routines                           */
/*****************************************************************************/
#include <pthread.h>


typedef struct {
	pthread_t threadHandle;
	pthread_attr_t pthreadAttrs;
	void (*start_routine)(void*);
	void * arg;

} t_ThreadAttrs;


static void * pthreadHandler(void * data) {
	t_ThreadAttrs * p_ThreadAttrs = (t_ThreadAttrs *) data;
	if(p_ThreadAttrs && p_ThreadAttrs->start_routine) {
		p_ThreadAttrs->start_routine(p_ThreadAttrs->arg);
	}
	
	return NULL;
	
} 

t_TaskletHandle XX_InitTasklet (void (*routine)(void *), void *data)
{
    //return (t_TaskletHandle)TASKLET_Init(routine, data);
	t_ThreadAttrs * p_ThreadAttrs = XX_Malloc(sizeof(t_ThreadAttrs));
	if(p_ThreadAttrs == NULL) {
			REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Thread Attributes"));
	} else {
		p_ThreadAttrs->threadHandle = 0;
		p_ThreadAttrs->start_routine = routine;
		p_ThreadAttrs->arg = data;
		pthread_attr_init(&p_ThreadAttrs->pthreadAttrs);

	}
		
	return (t_TaskletHandle)p_ThreadAttrs;
}


void XX_FreeTasklet (t_TaskletHandle h_Tasklet)
{
	t_ThreadAttrs * p_ThreadAttrs =  (t_ThreadAttrs *) h_Tasklet;
	if(	p_ThreadAttrs == NULL) {
		REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("h_Tasklet"));
	} else {
		/* Kill the thread */
		if(p_ThreadAttrs->threadHandle) {
			if(pthread_cancel(p_ThreadAttrs->threadHandle) != EOK) {
				REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("pthread_cancel"));
			}
		}
		XX_Free(p_ThreadAttrs);
	}
    
}


int XX_ScheduleTask(t_TaskletHandle h_Tasklet, int immediate)
{
	t_ThreadAttrs * p_ThreadAttrs =  (t_ThreadAttrs *) h_Tasklet;
	
	if(p_ThreadAttrs == NULL) {
		RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("h_Tasklet"));
	} else if (p_ThreadAttrs->threadHandle) {
		RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Thread already created"));
	
	} else {
		if( pthread_create(&p_ThreadAttrs->threadHandle, 
							&p_ThreadAttrs->pthreadAttrs,
							pthreadHandler,
							p_ThreadAttrs) != EOK) {
							
			RETURN_ERROR(MAJOR, E_INVALID_OPERATION, ("pthread_create"));				
			}
	
	}
	
	return E_OK;
}


void XX_FlushScheduledTasks(void)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


int XX_TaskletIsQueued(t_TaskletHandle h_Tasklet)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
	return 0;
}


void XX_SetTaskletData(t_TaskletHandle h_Tasklet, t_Handle data)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


t_Handle XX_GetTaskletData(t_TaskletHandle h_Tasklet)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
	return NULL;
}


/*****************************************************************************/
/*                        Timers Service Routines                            */
/*****************************************************************************/

uint32_t XX_CurrentTime(void)
{
	struct timespec now;	
	uint32_t ms=0;
	 if( clock_gettime( CLOCK_REALTIME, &now) == -1 ) {
    	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Error in time retrieval"));
  		return 0;
    }
	ms= now.tv_sec *1000 + now.tv_nsec/1000000;
	return ms;
}


t_Handle XX_CreateTimer(void)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
	return NULL;
}


void XX_FreeTimer(t_Handle h_Timer)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


void XX_StartTimer(t_Handle h_Timer,
                   uint32_t msecs,
                   bool     periodic,
                   void     (*f_TimerExpired)(t_Handle),
                   t_Handle h_Arg)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


uint32_t XX_GetExpirationTime(t_Handle h_Timer)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
	return 0;
}


void XX_StopTimer(t_Handle h_Timer)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


void XX_ModTimer(t_Handle h_Timer, uint32_t msecs)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
}


int XX_TimerIsActive(t_Handle h_Timer)
{
	REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Unimplemented"));
	return 0;
}


void XX_UDelay(uint32_t usecs)
{
	usleep(usecs);
}

/*****************************************************************************/
/*                        IPC Service Routines                               */
/*****************************************************************************/

t_Error XX_IpcRegisterMsgHandler(char                   addr[XX_IPC_MAX_ADDR_NAME_LENGTH],
                                 t_IpcMsgHandler        *f_MsgHandler,
                                 t_Handle               h_Module,
                                 uint32_t               replyLength)
{
#ifdef NCSW_MULTI_PARTITION
    return SYS_IpcRegisterMsgHandler(addr, f_MsgHandler, h_Module, replyLength);
#else
    UNUSED(addr);
    UNUSED(f_MsgHandler);
    UNUSED(h_Module);
    UNUSED(replyLength);
    return E_OK;
#endif /* NCSW_MULTI_PARTITION */
}

t_Error XX_IpcUnregisterMsgHandler(char addr[XX_IPC_MAX_ADDR_NAME_LENGTH])
{
#ifdef NCSW_MULTI_PARTITION
    return SYS_IpcUnregisterMsgHandler(addr);
#else
    UNUSED(addr);
    return E_OK;
#endif /* NCSW_MULTI_PARTITION */
}

t_Handle XX_IpcInitSession(char destAddr[XX_IPC_MAX_ADDR_NAME_LENGTH],
                           char srcAddr[XX_IPC_MAX_ADDR_NAME_LENGTH])
{
#ifdef NCSW_MULTI_PARTITION
    return SYS_IpcInitSession(destAddr, srcAddr);
#else
    UNUSED(destAddr);
    UNUSED(srcAddr);
    return NULL;
#endif /* NCSW_MULTI_PARTITION */
}

t_Error XX_IpcFreeSession(t_Handle h_Session)
{
#ifdef NCSW_MULTI_PARTITION
    return SYS_IpcFreeSession(h_Session);
#else
    UNUSED(h_Session);
    return E_OK;
#endif /* NCSW_MULTI_PARTITION */
}

t_Error XX_IpcSendMessage(t_Handle           h_Session,
                          uint8_t            *p_Msg,
                          uint32_t           msgLength,
                          uint8_t            *p_Reply,
                          uint32_t           *p_ReplyLength,
                          t_IpcMsgCompletion *f_Completion,
                          t_Handle           h_Arg)
{
#ifdef NCSW_MULTI_PARTITION
    return SYS_IpcSendMsg(h_Session,
                          p_Msg,
                          msgLength,
                          p_Reply,
                          p_ReplyLength,
                          f_Completion,
                          h_Arg);
#else
    UNUSED(h_Session);
    UNUSED(p_Msg);
    UNUSED(msgLength);
    UNUSED(p_Reply);
    UNUSED(p_ReplyLength);
    UNUSED(f_Completion);
    UNUSED(h_Arg);
    RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("XX_IpcSendMessage"));
#endif /* NCSW_MULTI_PARTITION */
}
