/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//*
 @File          p4080_ds.c

 @Description   P4080-DS board control routines
*//***************************************************************************/

#include "std_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "p4080_ds.h"
#include "vsc8244_ext.h"
#include "vsc8234_ext.h"
#include "tn2020_ext.h"


#define SET_PHY_ADDR(ethPortId, ethInterface, phyAddr)                                      \
do {                                                                                        \
    switch (ethPortId)                                                                      \
    {                                                                                       \
        case 0: phyAddr = (uint8_t)(ethInterface == e_ENET_IF_RGMII) ? 0x0 : 0x1c; break;   \
        case 1: phyAddr = (uint8_t)(ethInterface == e_ENET_IF_RGMII) ? 0x0 : 0x1d; break;                                                      \
        case 2: phyAddr = 0x1e; break;                                                      \
        case 3: phyAddr = 0x1f; break;                                                      \
        case 4: phyAddr = 0x04; break;                                                      \
        case 5: phyAddr = (uint8_t)(ethInterface == e_ENET_IF_RGMII) ? 0x0 : 0x1c; break;   \
        case 6: phyAddr = 0x1d; break;                                                      \
        case 7: phyAddr = 0x1e; break;                                                      \
        case 8: phyAddr = 0x1f; break;                                                      \
        case 9: phyAddr = 0x00; break;                                                      \
        default:                                                                            \
            RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("Ethernet port"));                    \
    }                                                                                       \
} while (0)

/*****************************************************************************/
static t_Error InitBcsrManager(t_P4080Ds *p_P4080Ds, uint64_t baseAddr)
{
    p_P4080Ds->p_PixisMemMap = CAST_UINT64_TO_POINTER(baseAddr);

    return E_OK;
}


/*****************************************************************************/
bool P4080DS_IsValidSerDesConfiguration (uint8_t val)
{
    return TRUE;
}

/*****************************************************************************/
t_Handle P4080DS_ConfigAndInit(t_P4080DsParams *p_P4080DsParams)
{
    t_P4080Ds   *p_P4080Ds;
    t_Error     errCode;

    p_P4080Ds = (t_P4080Ds *)XX_Malloc(sizeof(t_P4080Ds));
    if (!p_P4080Ds)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("P4080DS Control Structure"));
        return NULL;
    }
    memset(p_P4080Ds, 0, sizeof(t_P4080Ds));

    errCode = InitBcsrManager(p_P4080Ds, p_P4080DsParams->pixisBaseAddr);
    if (errCode != E_OK)
    {
        REPORT_ERROR(MAJOR, errCode, NO_MSG);
        return NULL;
    }


    return p_P4080Ds;
}

/*****************************************************************************/
t_Error P4080DS_Free(t_Handle h_P4080Ds)
{
    t_P4080Ds *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;

    SANITY_CHECK_RETURN_ERROR(p_P4080Ds, E_INVALID_HANDLE);

    XX_Free(p_P4080Ds);

    return E_OK;
}

t_Error P4080DS_GetCSForMemCtrl(t_Handle            h_P4080Ds,
                                uint8_t             id,
                                uint8_t             *p_Cs)
{
    t_P4080DsMemCtrlDesc    memCtrlsCs[] = P4080DS_MEMORY_CTRLS_CSs;
    int                     i, numOfMemCtrls;

    numOfMemCtrls = ARRAY_SIZE(memCtrlsCs);

    for (i=0; i<numOfMemCtrls; i++)
        if (memCtrlsCs[i].id == id)
        {
            *p_Cs = memCtrlsCs[i].chipSelect;
            return E_OK;
        }
    RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("mem-ctrl-id!"));
}

/*****************************************************************************/
t_Error P4080DS_InitEthernetPhy(t_Handle            h_P4080Ds,
                                uint32_t            ethPortId,
                                e_EnetMode          ethMode,
                                bool                autoNegotiation,
                                e_EnetDuplexMode    duplexMode,
                                bool                forceMaster,
                                e_DiagMode          loopbackMode,
                                t_Handle            h_MiiAccess,
                                t_MiiPhyWriteFunc   *f_SetPhyReg,
                                t_MiiPhyReadFunc    *f_GetPhyReg)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;
    e_EnetInterface ethInterface;
    t_Error         errCode = E_OK;
    uint8_t         phyAddr;
    bool            addTxOutputDelay, addRxOutputDelay;

    SANITY_CHECK_RETURN_ERROR(p_P4080Ds, E_INVALID_HANDLE);

    if (p_P4080Ds->ethModes[ethPortId] != e_ENET_MODE_INVALID)
        RETURN_ERROR(MAJOR, E_BUSY, ("Ethernet PHY"));

    ethInterface = ENET_INTERFACE_FROM_MODE(ethMode);

    SET_PHY_ADDR(ethPortId, ethInterface, phyAddr);

    /* Initialize the PHY */
    if (ethInterface == e_ENET_IF_RGMII)
    {
        addTxOutputDelay = addRxOutputDelay = ((ethInterface == e_ENET_IF_RGMII) ? TRUE : FALSE);

        errCode = VSC8244_Init(phyAddr,
                                ethMode,
                                autoNegotiation,
                                duplexMode,
                                forceMaster,
                                loopbackMode,
                                addTxOutputDelay,
                                addRxOutputDelay,
                                h_MiiAccess,
                                f_SetPhyReg,
                                f_GetPhyReg);
    }
    else if (ethInterface == e_ENET_IF_SGMII)
        errCode = VSC8234_Init(phyAddr,
                                ethMode,
                                autoNegotiation,
                                duplexMode,
                                forceMaster,
                                loopbackMode,
                                h_MiiAccess,
                                f_SetPhyReg,
                                f_GetPhyReg);
    else if (ethInterface == e_ENET_IF_XGMII)
        errCode = TN2020_Init(phyAddr,
                                ethMode,
                                autoNegotiation,
                                duplexMode,
                                forceMaster,
                                loopbackMode,
                                h_MiiAccess,
                                f_SetPhyReg,
                                f_GetPhyReg);
    else
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("there is no such interface on the p4080ds!"));
    if (errCode != E_OK)
        RETURN_ERROR(MAJOR, errCode, NO_MSG);

    p_P4080Ds->gethPortsCount++;

    p_P4080Ds->ethModes[ethPortId] = ethMode;

    return E_OK;

}

/*****************************************************************************/
e_EnetMode P4080DS_FreeEthernetPhy(t_Handle h_P4080Ds, uint32_t ethPortId)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;
    e_EnetMode      ethMode;
    e_EnetInterface ethInterface;

    SANITY_CHECK_RETURN_VALUE(p_P4080Ds, E_INVALID_HANDLE, e_ENET_MODE_INVALID);

    ethMode = (e_EnetMode)p_P4080Ds->ethModes[ethPortId];

    ethInterface = ENET_INTERFACE_FROM_MODE(ethMode);

    if (ethMode == e_ENET_MODE_INVALID)
    {
        REPORT_ERROR(MAJOR, E_ALREADY_FREE, ("Ethernet PHY"));
        return e_ENET_MODE_INVALID;
    }

    p_P4080Ds->gethPortsCount--;

    p_P4080Ds->ethModes[ethPortId] = e_ENET_MODE_INVALID;

    /* Return previous mode */
    return ethMode;
}

/*****************************************************************************/
bool P4080DS_GetEthernetLinkStatus(t_Handle             h_P4080Ds,
                                   uint32_t             ethPortId,
                                   t_Handle             h_MiiAccess,
                                   t_MiiPhyWriteFunc    *f_SetPhyReg,
                                   t_MiiPhyReadFunc     *f_GetPhyReg)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;
    t_MiiParam      miiParam;
    e_EnetInterface ethInterface;
    int             ret;

    SANITY_CHECK_RETURN_VALUE(p_P4080Ds, E_INVALID_HANDLE, e_ENET_MODE_INVALID);

    miiParam.numOfPhys = 1;
    miiParam.configFlags = 0;
    miiParam.maxWaitCycles = 1;
    miiParam.waitParam = 500000;    /* 0.5 second */
    miiParam.h_Driver = h_MiiAccess;
    miiParam.f_MiiPhyRead = f_GetPhyReg;
    miiParam.f_MiiPhyWrite = f_SetPhyReg;
    miiParam.f_MiiWait = (t_MiiWaitFunc*)XX_UDelay;

    ethInterface = ENET_INTERFACE_FROM_MODE(p_P4080Ds->ethModes[ethPortId]);

    SET_PHY_ADDR(ethPortId, ethInterface, miiParam.phyAddr);

    /* Check link status */
    if (ethInterface == e_ENET_IF_XGMII)
        ret = TN2020_LinkCheck(miiParam.phyAddr,h_MiiAccess,f_SetPhyReg,f_GetPhyReg);
    else
        ret = MII_PhyCtrl(&miiParam, MII_CTRL_CHECK_LINK);

    return (ret ? FALSE : TRUE);
}

/*****************************************************************************/
e_EnetMode P4080DS_GetEnetMode(t_Handle h_P4080Ds, uint32_t ethPortId)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;

    SANITY_CHECK_RETURN_VALUE(p_P4080Ds, E_INVALID_HANDLE, e_ENET_MODE_INVALID);

    return (e_EnetMode)p_P4080Ds->ethModes[ethPortId];
}

/*****************************************************************************/
t_Error P4080DS_InitSerialPort(t_Handle h_P4080Ds, e_P4080DsSerialPort serialPort)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;

    SANITY_CHECK_RETURN_ERROR(p_P4080Ds, E_INVALID_HANDLE);

    if (serialPort >= P4080DS_NUM_OF_SERIAL_PORTS)
        RETURN_ERROR(MAJOR, E_INVALID_ID,("Serial port ID"));

    if (p_P4080Ds->serialPortsEnabled[serialPort])
        RETURN_ERROR(MAJOR, E_BUSY, ("Serial port %d", serialPort));

    p_P4080Ds->serialPortsEnabled[serialPort] = TRUE;

    return E_OK;
}

/*****************************************************************************/
t_Error P4080DS_FreeSerialPort(t_Handle h_P4080Ds, e_P4080DsSerialPort serialPortId)
{
    t_P4080Ds   *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;

    SANITY_CHECK_RETURN_ERROR(p_P4080Ds, E_INVALID_HANDLE);

    if (serialPortId >= P4080DS_NUM_OF_SERIAL_PORTS)
        RETURN_ERROR(MAJOR, E_INVALID_ID,
                     ("Serial port ID (should be 0-%d)", P4080DS_NUM_OF_SERIAL_PORTS - 1));

    if (!p_P4080Ds->serialPortsEnabled[serialPortId])
        RETURN_ERROR(MAJOR, E_ALREADY_FREE, ("Serial port %d", serialPortId));


    p_P4080Ds->serialPortsEnabled[serialPortId] = FALSE;

    return E_OK;
}

/*****************************************************************************/
uint32_t P4080DS_GetSysClk(t_Handle h_P4080Ds)
{
    t_P4080Ds       *p_P4080Ds = (t_P4080Ds *)h_P4080Ds;
    uint8_t         pxSpd;

    SANITY_CHECK_RETURN_VALUE(p_P4080Ds, E_INVALID_HANDLE, e_ENET_MODE_INVALID);

    pxSpd = GET_UINT8(p_P4080Ds->p_PixisMemMap->px_spd);
    switch(pxSpd & PX_SPD_SYSCLK_MASK)
    {
        case(e_PIXIS_SYS_CLK_66_666):
            return PIXIS_SYS_CLK_66_666;
        case(e_PIXIS_SYS_CLK_75):
            return PIXIS_SYS_CLK_75;
        case(e_PIXIS_SYS_CLK_83_333):
            return PIXIS_SYS_CLK_83_333;
        case(e_PIXIS_SYS_CLK_90):
            return PIXIS_SYS_CLK_90;
        case(e_PIXIS_SYS_CLK_100):
            return PIXIS_SYS_CLK_100;
        case(e_PIXIS_SYS_CLK_111):
            return PIXIS_SYS_CLK_111;
        case(e_PIXIS_SYS_CLK_125):
            return PIXIS_SYS_CLK_125;
        case(e_PIXIS_SYS_CLK_133_333):
            return PIXIS_SYS_CLK_133_333;
        default:
            return PIXIS_SYS_CLK_100;
    }
    return PIXIS_SYS_CLK_100;
}
