/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//*
 @File          bcsr_mng.c

 @Description   board control routines
*//***************************************************************************/

#include "std_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "regsolver_ext.h"
#include "bcsr_mng.h"



/*****************************************************************************/
t_Handle BCSR_ConfigAndInit(t_BcsrMngParam *p_BcsrMngParam)
{
    t_BcsrMng   *p_BcsrMng;
    uint8_t     bcsrVal;
    int         i;

    p_BcsrMng = (t_BcsrMng *)XX_Malloc(sizeof(t_BcsrMng));
    if (!p_BcsrMng)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("BCSR Manager"));
        return NULL;
    }
    memset(p_BcsrMng, 0, sizeof(t_BcsrMng));

    p_BcsrMng->p_Bcsr = (uint8_t *)p_BcsrMngParam->bcsrBaseAddr;
    p_BcsrMng->numOfBcsrRegs = p_BcsrMngParam->numOfBcsrRegs;

    p_BcsrMng->p_BcsrShadow = (uint8_t *)
        XX_Malloc(sizeof(uint8_t) * p_BcsrMng->numOfBcsrRegs);
    if (!p_BcsrMng->p_BcsrShadow)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("BCSR Manager shadow registers"));
        BCSR_Free(p_BcsrMng);
        return NULL;
    }
    memset(p_BcsrMng->p_BcsrShadow, 0, sizeof(uint8_t) * p_BcsrMng->numOfBcsrRegs);

    p_BcsrMng->p_RegSolvers = (t_Handle *)
        XX_Malloc(sizeof(t_Handle) * p_BcsrMng->numOfBcsrRegs);
    if (!p_BcsrMng->p_RegSolvers)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("BCSR Manager resource solvers"));
        BCSR_Free(p_BcsrMng);
        return NULL;
    }
    memset(p_BcsrMng->p_RegSolvers, 0, sizeof(t_Handle) * p_BcsrMng->numOfBcsrRegs);

    for (i=0; i < p_BcsrMng->numOfBcsrRegs; i++)
    {
        if (p_BcsrMngParam->p_MaxUsersPerBcsrReg[i])
        {
            bcsrVal = GET_UINT8(p_BcsrMng->p_Bcsr[i]);

            p_BcsrMng->p_RegSolvers[i] =
                REGSOLVER_New(p_BcsrMngParam->p_MaxUsersPerBcsrReg[i], bcsrVal);

            if (!p_BcsrMng->p_RegSolvers[i])
            {
                REPORT_ERROR(MAJOR, E_NOT_AVAILABLE,
                             ("BCSR Manager resource solver %d", i));
                BCSR_Free(p_BcsrMng);
                return NULL;
            }

            /* Store reset value (will be restored in BCSR_Free) */
            p_BcsrMng->p_BcsrShadow[i] = bcsrVal;
        }
    }

    return p_BcsrMng;
}


/*****************************************************************************/
t_Error BCSR_Free(t_Handle h_BcsrMng)
{
    t_BcsrMng   *p_BcsrMng = (t_BcsrMng *)h_BcsrMng;
    int         i;

    SANITY_CHECK_RETURN_ERROR(p_BcsrMng, E_INVALID_HANDLE);

    if (p_BcsrMng)
    {
        if (p_BcsrMng->p_RegSolvers)
        {
            for (i=0; i < p_BcsrMng->numOfBcsrRegs; i++)
            {
                if (p_BcsrMng->p_RegSolvers[i])
                {
                    REGSOLVER_Delete(p_BcsrMng->p_RegSolvers[i]);

                    /* Restore BCSR reset value */
                    WRITE_UINT8(p_BcsrMng->p_Bcsr[i], p_BcsrMng->p_BcsrShadow[i]);
                }
            }

            XX_Free(p_BcsrMng->p_RegSolvers);
        }

        if (p_BcsrMng->p_BcsrShadow)
        {
            XX_Free(p_BcsrMng->p_BcsrShadow);
            p_BcsrMng->p_BcsrShadow = NULL;
        }

        XX_Free(p_BcsrMng);
    }

    return E_OK;
}


/*****************************************************************************/
t_Error BCSR_IssueRequests(t_Handle         h_BcsrMng,
                           e_BcsrOperation  operation,
                           uint8_t          numOfRequests,
                           t_BcsrRequest    *p_Requests)
{
    t_BcsrMng   *p_BcsrMng = (t_BcsrMng *)h_BcsrMng;
    uint32_t    intFlags;
    uint8_t     bcsrVal;
    bool        res;
    int         i, j;

    SANITY_CHECK_RETURN_ERROR(p_BcsrMng, E_INVALID_HANDLE);

    intFlags = XX_DisableAllIntr();

    if ((operation == e_BCSR_OP_SET_AND_LOCK) ||
        (operation == e_BCSR_OP_SET_AND_UNLOCK))
    {
        for (i=0; i < numOfRequests; i++)
        {
            res = REGSOLVER_LockBits(p_BcsrMng->p_RegSolvers[p_Requests[i].reg],
                                     p_Requests[i].value,
                                     p_Requests[i].mask);
            if (!res)
            {
                /* Constraints check failed due to some conflict - revert last changes */
                for (j=0; j < i; j++)
                {
                    REGSOLVER_UnlockBits(p_BcsrMng->p_RegSolvers[p_Requests[j].reg],
                                         p_Requests[j].mask);
                }

                XX_RestoreAllIntr(intFlags);
                RETURN_ERROR(MAJOR, E_CONFLICT, ("BCSR configuration"));
            }
        }

        /* Configuration is valid - now program the BCSR registers */
        for (i=0; i < numOfRequests; i++)
        {
            bcsrVal = (uint8_t)REGSOLVER_GetValue(p_BcsrMng->p_RegSolvers[p_Requests[i].reg]);

            WRITE_UINT8(p_BcsrMng->p_Bcsr[p_Requests[i].reg], bcsrVal);
        }
    }
    else if (operation == e_BCSR_OP_UNLOCK)
    {
        /* Need to unlock all bits - first check if they are locked */
        for (i=0; i < numOfRequests; i++)
        {
            res = REGSOLVER_CheckLock(p_BcsrMng->p_RegSolvers[p_Requests[i].reg],
                                      p_Requests[i].mask);
            if (!res)
            {
                XX_RestoreAllIntr(intFlags);
                RETURN_ERROR(MAJOR, E_ALREADY_FREE, ("BCSR configuration"));
            }
        }
    }
    else
    {
        XX_RestoreAllIntr(intFlags);
        RETURN_ERROR(MAJOR, E_INVALID_SELECTION, ("BCSR operation"));
    }

    if ((operation == e_BCSR_OP_UNLOCK) ||
        (operation == e_BCSR_OP_SET_AND_UNLOCK))
    {
        /* We can safely unlock all bits */
        for (i=0; i < numOfRequests; i++)
        {
            res = REGSOLVER_UnlockBits(p_BcsrMng->p_RegSolvers[p_Requests[i].reg],
                                       p_Requests[i].mask);
            /* Should never fail */
            ASSERT_COND(res);
        }
    }

    XX_RestoreAllIntr(intFlags);

    return E_OK;
}


