/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mii_mng.c

 @Description   MII routines
*//***************************************************************************/

#include "mii_mng_ext.h"


/******************************************************************************
*******************************************************************************
    THE FOLLOWING ARE INTERNAL DEFINITONS - THEY ARE NOT PART OF THE MII API
*******************************************************************************
******************************************************************************/

/* MII registers (section 22.2.4) */
#define MII_CR              0x00    /* Control Register */
#define MII_SR              0x01    /* Status Register */
#define MII_PHYIDR1         0x02    /* PHY Identifier 1 Register */
#define MII_PHYIDR2         0x03    /* PHY Identifier 2 Register */
#define MII_ANAR            0x04    /* Auto-Negotiation Advertisement Register */
#define MII_ANLPAR          0x05    /* Auto-Negotiation Link Partner Ability Register */
#define MII_ANER            0x06    /* Auto-Negotiation Expansion Register */
#define MII_ANNPTR          0x07    /* Auto-Negotiation Next-Page Transmit Register */
#define MII_ANLPNPR         0x08    /* Auto-Negotiation Link Partner Next-Page Register */
#define MII_MSCR            0x09    /* Master/Slave Control Register (100base-T2, 1000base-T)  */
#define MII_MSSR            0x0a    /* Master/Slave Status Register (100base-T2, 1000base-T) */
#define MII_EXTSR           0x0f    /* Extended Status Register */


/* MII Control Register (section 22.2.4.1) */
#define MII_CR_RESET        0x8000  /* Reset */
#define MII_CR_LOOPBACK     0x4000  /* Loopback */
#define MII_CR_SPEED_LSB    0x2000  /* Speed selection (LSB) */
#define MII_CR_AN_ENABLE    0x1000  /* Auto-negotiation enable */
#define MII_CR_PWR_DOWN     0x0800  /* Power down */
#define MII_CR_ISOLATE      0x0400  /* Isolate */
#define MII_CR_AN_RESTART   0x0200  /* Restart auto-negotiation */
#define MII_CR_FD           0x0100  /* Full-Duplex mode */
#define MII_CR_COL_TEST     0x0080  /* Collision test */
#define MII_CR_SPEED_MSB    0x0040  /* Speed selection (MSB) */

#define MII_CR_S10          0x0000              /* 10 Mb/s */
#define MII_CR_S100         MII_CR_SPEED_LSB    /* 100 Mb/s */
#define MII_CR_S1000        MII_CR_SPEED_MSB    /* 1000 Mb/s */

#define MII_CR_MIN_ENABLE   0x0000  /* Enable the PHY with minimum capability */


/* MII Status Register (section 22.2.4.2) */
#define MII_SR_100T4        0x8000  /* 100base-T4 (HD) capable */
#define MII_SR_100TX_FD     0x4000  /* 100base-TX FD capable */
#define MII_SR_100TX_HD     0x2000  /* 100base-TX HD capable */
#define MII_SR_10T_FD       0x1000  /* 10base-T FD capable */
#define MII_SR_10T_HD       0x0800  /* 10base-T HD capable */
#define MII_SR_100T2_FD     0x0400  /* 100base-T2 FD capable */
#define MII_SR_100T2_HD     0x0200  /* 100base-T2 HD capable */
#define MII_SR_EXTSTAT      0x0100  /* Extended status in MII_EXTSR
                                       (all 1000 Mb/s PHYs set it to 1) */
#define MII_SR_MFPS         0x0040  /* MII Frame Preamble Suppression */
#define MII_SR_AN_COMP      0x0020  /* Auto-negotiation complete */
#define MII_SR_RFAULT       0x0010  /* Link partner fault */
#define MII_SR_AN_ABLE      0x0008  /* Auto-negotiation capable */
#define MII_SR_LINK         0x0004  /* Link status */
#define MII_SR_JABBER       0x0002  /* Jabber detected */
#define MII_SR_EXTCAP       0x0001  /* Extended capability */

#define MII_SR_BASIC_MEDIA  (MII_SR_100T4 | MII_SR_100TX_FD | MII_SR_100TX_HD | \
                             MII_SR_10T_FD | MII_SR_10T_HD)

#define MII_SR_MEDIA        (MII_SR_BASIC_MEDIA | MII_SR_100T2_FD | MII_SR_100T2_HD)

/* Convert SR media capabilities to ANAR bits for auto-negotiation. */
#define MII_SR_MEDIA_TO_ANAR(sr)    ((((sr) & MII_SR_BASIC_MEDIA) >> 6) | \
                                     (((sr) & (MII_SR_100T2_FD | MII_SR_100T2_HD)) << 1))


/* Auto-Negotiation Advertisement Register (sections 28.2.4.1, 37.2.5.1, 28A, 28B) */
#define MII_ANAR_NP                 0x8000  /* Next page */
#define MII_ANAR_ACK                0x4000  /* link partner acknowledged */
#define MII_ANAR_RFAULT             0x2000  /* remote fault */
/*--------------------------------*/
#define MII_ANAR_PAUSE_ASYM         0x0800  /* supports asymmetric PAUSE (FD only) */
#define MII_ANAR_PAUSE_SYM          0x0400  /* supports symmetric PAUSE (FD only) */
/*---Overlapping bits-------------*/
#define MII_ANAR_100T2_FD           0x0800  /* 100base-T2 FD capable */
#define MII_ANAR_100T2_HD           0x0400  /* 100base-T2 HD capable */
/*--------------------------------*/
#define MII_ANAR_100T4              0x0200  /* 100base-T4 (HD) capable */
#define MII_ANAR_100TX_FD           0x0100  /* 100base-TX FD capable */
#define MII_ANAR_100TX_HD           0x0080  /* 100base-TX HD capable */
#define MII_ANAR_10T_FD             0x0040  /* 10base-T FD capable */
#define MII_ANAR_10T_HD             0x0020  /* 10base-T HD capable */
#define MII_ANAR_SEL_802_3          0x0001  /* protocol selector (802.3) */

/*-----1000base-X bits------------*/
#define MII_ANAR1000X_PAUSE_ASYM    0x0100  /* supports asymmetric PAUSE (FD only) */
#define MII_ANAR1000X_PAUSE_SYM     0x0080  /* supports symmetric PAUSE (FD only) */
#define MII_ANAR1000X_HD            0x0040  /* 1000base-X HD capable */
#define MII_ANAR1000X_FD            0x0020  /* 1000base-X FD capable */

#define MII_ANAR1000X_PAUSE_MASK    (MII_ANAR1000X_PAUSE_SYM | MII_ANAR1000X_PAUSE_ASYM)
/*--------------------------------*/

#define MII_ANAR_PAUSE_MASK         (MII_ANAR_PAUSE_SYM | MII_ANAR_PAUSE_ASYM)

#define MII_ANAR_MEDIA              (MII_ANAR_100TX_FD | MII_ANAR_100TX_HD | \
                                     MII_ANAR_100T2_FD | MII_ANAR_100T2_HD | \
                                     MII_ANAR_100T4 | MII_ANAR_10T_FD | MII_ANAR_10T_HD)

#define MII_ANAR_FD_MASK            (MII_ANAR_100TX_FD | MII_ANAR_100T2_FD | MII_ANAR_10T_FD)
#define MII_ANAR_HD_MASK            (MII_ANAR_100TX_HD | MII_ANAR_100T2_HD | MII_ANAR_10T_HD | \
                                     MII_ANAR_100T4)


/* NOTE: ANLPAR bits are identical to ANAR bits, so we use same definitions. */


/* Auto-Negotiation Expansion Register (sections 28.2.4.1, 37.2.5.1) */
#define MII_ANER_PDF            0x0010  /* parallel detection fault */
#define MII_ANER_LP_NP_ABLE     0x0008  /* link parter next-page ability */
#define MII_ANER_NP_ABLE        0x0004  /* local phy next-page ability */
#define MII_ANER_PAGE_RX        0x0002  /* page received */
#define MII_ANER_LP_AN_ABLE     0x0001  /* link parter auto-negotiation ability */


/* Master/Slave Control Register (100base-T2 / 1000base-T) (sections 32.5.3, 40.5.1.1) */
#define MII_MSCR_TEST_MASK      0xe000  /* see 802.3ab 40.6.1.1.2 */
#define MII_MSCR_MAN_CFG_EN     0x1000  /* enable manual master/slave control */
#define MII_MSCR_MASTER         0x0800  /* 1 = master, 0 = slave */
#define MII_MSCR_PORT_TYPE      0x0400  /* 100base-T2: 1 = repeater, 0 = DTE */
                                        /* 1000base-T: 1 = multi-port, 0 = single-port */
#define MII_MSCR_ADV_1000T_FD   0x0200  /* 1000base-T FD capable */
#define MII_MSCR_ADV_1000T_HD   0x0100  /* 1000base-T HD capable */


/* Master/Slave Status Register (100base-T2 / 1000base-T) (sections 32.5.3, 40.5.1.1) */
#define MII_MSSR_MAN_CFG_FLT    0x8000  /* master/slave config fault */
#define MII_MSSR_RES            0x4000  /* 100base-T2: Master/slave resolution complete */
                                        /* 1000base-T: Resolution: 1 = master, 0 = slave */
#define MII_MSSR_LOCAL_RXST     0x2000  /* local receiver status, 1 = ok */
#define MII_MSSR_REMOTE_RXST    0x1000  /* remote receiver status, 1 = ok */
#define MII_MSSR_LP_1000T_FD    0x0800  /* link partner is 1000base-T FD capable */
#define MII_MSSR_LP_1000T_HD    0x0400  /* link partner is 1000base-T HD capable */
#define MII_MSSR_IDLE_ERROR     0x00ff  /* IDLE error count */


/* Extended Status Register (section 22.2.4.4) */
#define MII_EXTSR_1000X_FD      0x8000  /* 1000base-X FD capable */
#define MII_EXTSR_1000X_HD      0x4000  /* 1000base-X HD capable */
#define MII_EXTSR_1000T_FD      0x2000  /* 1000base-T FD capable */
#define MII_EXTSR_1000T_HD      0x1000  /* 1000base-T HD capable */

#define MII_EXTSR_MEDIAMASK     (MII_EXTSR_1000X_FD | MII_EXTSR_1000X_HD | \
                                 MII_EXTSR_1000T_FD | MII_EXTSR_1000T_HD)


/* Macros for handling flags */
#define MII_CONFIG_SET(bits)            (p_MiiParam->configFlags |= (bits))
#define MII_CONFIG_CHECK(bits)          (p_MiiParam->configFlags & (bits))
#define MII_CONFIG_CLEAR(bits)          (p_MiiParam->configFlags &= ~(bits))
#define MII_RESULTS_SET(bits)           (p_MiiParam->configResults |= (bits))
#define MII_RESULTS_CHECK(bits)         (p_MiiParam->configResults & (bits))
#define MII_RESULTS_CLEAR(bits)         (p_MiiParam->configResults &= ~(bits))

/* Macro for returning on error condition */
#define MII_RETURN_ON_ERROR(retVal)    if ((retVal) != MII_OK) return (retVal);

/* Macro for Wait function */
#define MII_WAIT(waitParam) \
    if (p_MiiParam->f_MiiWait) p_MiiParam->f_MiiWait(waitParam)

/* Macro for PHY read function */
#define MII_PHY_READ(phyAddr, phyReg, p_Data) \
    p_MiiParam->f_MiiPhyRead(p_MiiParam->h_Driver, (phyAddr), (phyReg), (p_Data))

/* Macro for PHY write function */
#define MII_PHY_WRITE(phyAddr, phyReg, data) \
    p_MiiParam->f_MiiPhyWrite(p_MiiParam->h_Driver, (phyAddr), (phyReg), (data))


/* local function declarations */
static int  miiPhyFind(t_MiiParam *p_MiiParam);
static int  miiAbilityCheck(t_MiiParam *p_MiiParam);
static int  miiPhyConfigure(t_MiiParam *p_MiiParam);
static int  miiAutoNegotiation(t_MiiParam *p_MiiParam);
static int  miiManualModeSet(t_MiiParam *p_MiiParam);
static int  miiLinkCheck(t_MiiParam *p_MiiParam);
static int  miiPhyReset(t_MiiParam *p_MiiParam);
static int  miiPhyIsolate(t_MiiParam *p_MiiParam);
static int  miiPhyPowerDown(t_MiiParam *p_MiiParam);
static int  miiPhyRestartAutoNegotiation(t_MiiParam *p_MiiParam);


/**************************************************************************//*
 @Function      MII_PhyInitialize

 @Description   Configures the PHY for operation.

 @Param         p_MiiParam  - (in) pointer to an initialized MII information
                              structure.

 @Return        See MII Return Codes.

*//****************************************************************************/
int MII_PhyInitialize(t_MiiParam *p_MiiParam)
{
    int ret;

    /* check parameters */
    if (!(p_MiiParam) || (p_MiiParam->phyAddr > (MII_MAX_PHYS - 1)) ||
        (p_MiiParam->numOfPhys > (MII_MAX_PHYS - p_MiiParam->phyAddr)) ||
        !(p_MiiParam->f_MiiPhyRead) || !(p_MiiParam->f_MiiPhyWrite))
    {
        return MII_ILLEGAL_PARAMS;
    }

    /* check invalid mode combinations */
    if ((MII_CONFIG_CHECK(MII_CFG_HD) && MII_CONFIG_CHECK(MII_CFG_FD | MII_CFG_PAUSE)) ||
        (MII_CONFIG_CHECK(MII_CFG_AUTO) && MII_CONFIG_CHECK(MII_CFG_LOOPBACK)))
    {
        return MII_ILLEGAL_MODE;
    }

    /* reset configuration result flags */
    p_MiiParam->configResults = 0;

    /* find a valid PHY (will be stored in p_MiiParam) */
    ret = miiPhyFind(p_MiiParam);
    MII_RETURN_ON_ERROR(ret);

    /* PHY exists - now reset it, unless "no reset" was requested */
    if (!(MII_CONFIG_CHECK(MII_CFG_NO_RESET)))
    {
        ret = miiPhyReset(p_MiiParam);
        MII_RETURN_ON_ERROR(ret);
    }
    else
    {
        MII_RESULTS_SET(MII_CFG_NO_RESET);
    }

    /* configure the PHY */
    ret = miiPhyConfigure(p_MiiParam);

    return ret;
}


/**************************************************************************//*
 @Function      MII_PhyCtrl

 @Description   Controls the PHY after initialization.

 @Param         p_MiiParam  - (in) pointer to an initialized MII information
                              structure.
 @Param         command     - (in) requested command (MII_CTRL_xxx).

 @Return        See MII Return Codes.

*//****************************************************************************/
int MII_PhyCtrl(t_MiiParam * p_MiiParam, unsigned short command)
{
    /* check parameters */
    if (!(p_MiiParam) || (p_MiiParam->phyAddr > (MII_MAX_PHYS - 1)) ||
        !(p_MiiParam->f_MiiPhyRead) || !(p_MiiParam->f_MiiPhyWrite))
    {
        return MII_ILLEGAL_PARAMS;
    }

    switch (command)
    {
        case MII_CTRL_CHECK_LINK:
            return miiLinkCheck(p_MiiParam);

        case MII_CTRL_RESET:
            return miiPhyReset(p_MiiParam);

        case MII_CTRL_ISOLATE:
            return miiPhyIsolate(p_MiiParam);

        case MII_CTRL_POWER_DOWN:
            return miiPhyPowerDown(p_MiiParam);

        case MII_CTRL_RENEGOTIATE:
            return miiPhyRestartAutoNegotiation(p_MiiParam);

        default:
            return MII_ERROR;
    }
}


/**************************************************************************//**
 @Function      miiPhyFind

 @Description   Scans the requested PHY addresses range, until a valid PHY
                is found.

 @Param         p_MiiParam      - (in) pointer to MII information structure.

 @Retval        MII_OK          - A valid PHY was found (the valid PHY address
                                  was written to p_MiiParam->phyAddr).
 @Retval        MII_ERROR       - Register read error.
 @Retval        MII_INVALID_PHY - No valid PHY was found.

*//****************************************************************************/
static int miiPhyFind(t_MiiParam *p_MiiParam)
{
    unsigned char   phyAddr;
    unsigned short  phySR;
    int             ret;

    /* start scanning from the requested PHY address, and continue until a
       valid PHY is found, or until p_MiiParam->numOfPhys were scanned */
    for (phyAddr = p_MiiParam->phyAddr;
         phyAddr < (p_MiiParam->phyAddr + p_MiiParam->numOfPhys);
         phyAddr++)
    {
        /* check that the PHY is there */
        ret = MII_PHY_READ(phyAddr, MII_SR, &phySR);
        MII_RETURN_ON_ERROR(ret);

        if ((phySR != 0xffff) && ((phySR & (MII_SR_EXTSTAT | MII_SR_MEDIA)) != 0))
        {
            /* we assume there is a valid PHY in this address - store it */
            p_MiiParam->phyAddr = phyAddr;
            return MII_OK;
        }
    }

    return MII_INVALID_PHY;
}


/**************************************************************************//**
 @Function      miiPhyConfigure

 @Description   Sets the best operating mode for a PHY, according to the given
                MII information structure and the PHY abilities.

                If auto-negotiation was requested, this routine calls
                miiAutoNegotiation(). If manual configuration was selected,
                or if the auto-negotiation process failed and MII_CFG_FORCE
                is specified, it calls the miiManualModeSet() routine.

 @Param         p_MiiParam          - (in) pointer to MII information structure.

 @Retval        MII_OK              - Operation was successful.
 @Retval        MII_ERROR           - General error.
 @Retval        MII_LINK_DOWN       - Link is down.
 @Retval        MII_REMOTE_FAULT    - Link-partner's fault detected after
                                      manual configuration.

*//****************************************************************************/
static int miiPhyConfigure(t_MiiParam *p_MiiParam)
{
    int ret;

    /* find out the PHY's abilities and set flags accordingly */
    ret = miiAbilityCheck(p_MiiParam);
    MII_RETURN_ON_ERROR(ret);

    if (MII_CONFIG_CHECK(MII_CFG_AUTO))
    {
        /* perform auto-negotiation */
        ret = miiAutoNegotiation(p_MiiParam);
        if ((ret == MII_OK) || (!MII_CONFIG_CHECK(MII_CFG_FORCE)))
        {
            /* auto-negotiation was successful */
            MII_RESULTS_SET(MII_CFG_AUTO);
            return ret;
        }
    }

    /* Configure the PHY manually */
    ret = miiManualModeSet(p_MiiParam);

    return ret;
}


/**************************************************************************//**
 @Function      miiAbilityCheck

 @Description   Reads the status registers of the PHY and clears some of the
                configuration flags, if needed.

                This is the only function that may clear the unsupported
                configuration flags.

 @Param         p_MiiParam  - (in) pointer to MII information structure.

 @Retval        MII_OK      - Operation was successful.
 @Retval        MII_ERROR   - Register read error.

*//****************************************************************************/
static int miiAbilityCheck(t_MiiParam *p_MiiParam)
{
    unsigned short  phySR;
    unsigned short  phyEXTSR = 0;
    int             ret;

    /* read the status register of the PHY */
    ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
    MII_RETURN_ON_ERROR(ret);

    if (!(MII_CONFIG_CHECK(MII_CFG_NO_ABILITY_CLR)))
    {
        /* update configuration flags as needed */
        if (phySR & MII_SR_EXTSTAT)
        {
            ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_EXTSR, &phyEXTSR);
            MII_RETURN_ON_ERROR(ret);

            if (!(phyEXTSR & (MII_EXTSR_1000T_FD | MII_EXTSR_1000T_HD)))
            {
                MII_CONFIG_CLEAR(MII_CFG_1000T);
            }
            if (!(phyEXTSR & (MII_EXTSR_1000X_FD | MII_EXTSR_1000X_HD)))
            {
                MII_CONFIG_CLEAR(MII_CFG_1000X);
            }
        }
        if (!(phySR & (MII_SR_100T2_FD | MII_SR_100T2_HD)))
        {
            MII_CONFIG_CLEAR(MII_CFG_100T2);
        }
        if (!(phySR & (MII_SR_100TX_HD | MII_SR_100TX_FD)))
        {
            MII_CONFIG_CLEAR(MII_CFG_100TX);
        }
        if (!(phySR & MII_SR_100T4))
        {
            MII_CONFIG_CLEAR(MII_CFG_100T4);
        }
        if (!(phySR & (MII_SR_10T_HD | MII_SR_10T_FD)))
        {
            MII_CONFIG_CLEAR(MII_CFG_10T);
        }
        if (!(phySR & (MII_SR_10T_FD | MII_SR_100TX_FD | MII_SR_100T2_FD)) &&
            !(phyEXTSR & (MII_EXTSR_1000X_FD | MII_EXTSR_1000T_FD)))
        {
            MII_CONFIG_CLEAR(MII_CFG_FD);
        }
        if (!(phySR & (MII_SR_10T_HD | MII_SR_100TX_HD | MII_SR_100T2_HD)) &&
            !(phyEXTSR & (MII_EXTSR_1000X_HD | MII_EXTSR_1000T_HD)))
        {
            MII_CONFIG_CLEAR(MII_CFG_HD);
        }
    }
    if (!(phySR & MII_SR_AN_ABLE))
    {
        /* PHY does not support auto-negotiation */
        MII_CONFIG_CLEAR(MII_CFG_AUTO);
    }

    return MII_OK;
}


/**************************************************************************//**
 @Function      miiAutoNegotiation

 @Description   Performs auto-negotiation to set up the best possible link
                between the local PHY and the link-partner.

                The routine checks the configuration flags, and if any media
                was specifically requested, it masks the advertisement of all
                other media types. The next-page function is disabled.

                The function also calls miiLinkCheck() to verify that a link
                was established, and then updates the configuration result
                flags according to the highest-priority common ability. The
                priority resolution table is (partially) stated in Annex 28B
                of the IEEE 802.3 standard.

 @Param         p_MiiParam    - (in) pointer to MII information structure.

 @Retval        MII_OK              - Operation was successful.
 @Retval        MII_ERROR           - General error.
 @Retval        MII_WAIT_LIMIT      - Wait limit was reached before completion.
 @Retval        MII_LINK_DOWN       - Link is down after auto-negotiation.
 @Retval        MII_REMOTE_FAULT    - Link-partner's fault detected after
                                      auto-negotiation.

*//****************************************************************************/
static int miiAutoNegotiation(t_MiiParam *p_MiiParam)
{
    unsigned short  phySR;          /* status register */
    unsigned short  phyANER;        /* auto-negotiation expansion register */
    unsigned short  phyANAR;        /* auto-negotiation advertisement register */
    /*unsigned short  phyANLPAR;*/      /* auto-negotiation link partner ability register */
    unsigned short  phyEXTSR = 0;   /* extended status register */
    unsigned short  phyMSCR = 0;    /* 1000base-T/100base-T2 control register */
    /*unsigned short  phyMSSR;*/        /* 1000base-T/100base-T2 status register */
    /*unsigned short  autoNegResults;*/ /* basic abilities after negotiation */
    unsigned long   waitCycles = 0;
    unsigned long   ignoredMedia = 0;
    int             is1000X = 0, is100T2 = 0, is1000TFd = 0, is1000THd = 0;
    int             ret;

    /* check that the PHY has auto-negotiation ability */
    ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
    MII_RETURN_ON_ERROR(ret);

    if (!(phySR & MII_SR_AN_ABLE))
    {
        /* the PHY is not auto-negotiation capable */
        return MII_ERROR;
    }

    /* set selector field in ANAR for 802.3 */
    /* the next-page function is disabled */
    phyANAR = MII_ANAR_SEL_802_3;

    /* read ANER to clear previous status */
    ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_ANER, &phyANER);
    MII_RETURN_ON_ERROR(ret);

    /* check configuration flags and mask unneeded abilities */

    if (MII_CONFIG_CHECK(MII_CFG_ANY_MEDIA))
    {
        ignoredMedia = (MII_CFG_ANY_MEDIA & ~(MII_CONFIG_CHECK(MII_CFG_ANY_MEDIA)));
    }

    if (phySR & MII_SR_EXTSTAT)
    {
        /* read extended status register */
        ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_EXTSR, &phyEXTSR);
        MII_RETURN_ON_ERROR(ret);

        /* 1000base-X cannot be auto-negotiated with other capabilities */
        if (!(ignoredMedia & MII_CFG_1000X) &&
            (phyEXTSR & (MII_EXTSR_1000X_FD | MII_EXTSR_1000X_HD)))
        {
            is1000X = 1;

            /* check if 1000base-T should be advertised, and configure MSCR */
            if ((phyEXTSR & MII_EXTSR_1000X_FD) && !(MII_CONFIG_CHECK(MII_CFG_HD)))
            {
                phyANAR |= MII_ANAR1000X_FD;

                /* PAUSE functionality is relevant for Full Duplex only */
                if (MII_CONFIG_CHECK(MII_CFG_PAUSE))
                    phyANAR |= (MII_ANAR1000X_PAUSE_SYM | MII_ANAR1000X_PAUSE_ASYM);
            }

            if ((phyEXTSR & MII_EXTSR_1000X_HD) && !(MII_CONFIG_CHECK(MII_CFG_FD)))
            {
                phyANAR |= MII_ANAR1000X_HD;
            }
        }
        else /* not 1000base-X */
        {
            if (!(ignoredMedia & MII_CFG_1000T))
            {
                /* check if 1000base-T should be advertised, and configure MSCR */
                if ((phyEXTSR & MII_EXTSR_1000T_FD) && !(MII_CONFIG_CHECK(MII_CFG_HD)))
                {
                    phyMSCR |= MII_MSCR_ADV_1000T_FD;
                    is1000TFd = 1;
                }

                if ((phyEXTSR & MII_EXTSR_1000T_HD) && !(MII_CONFIG_CHECK(MII_CFG_FD)))
                {
                    phyMSCR |= MII_MSCR_ADV_1000T_HD;
                    is1000THd = 1;
                }
            }
        }
    }

    if (!is1000X)
    {
        if ((phyEXTSR & (MII_EXTSR_1000T_FD | MII_EXTSR_1000T_HD)) ||
            (phySR & (MII_SR_100T2_FD | MII_SR_100T2_HD)))
        {
            if (MII_CONFIG_CHECK(MII_CFG_MASTER))
            {
                phyMSCR |= (MII_MSCR_MAN_CFG_EN | MII_MSCR_MASTER);
            }

            /* write MSCR (for 100base-T2 it should always be all zero) */
            ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_MSCR, phyMSCR);
            MII_RETURN_ON_ERROR(ret);
        }

        /* copy abilities from status register */
        phyANAR |= MII_SR_MEDIA_TO_ANAR(phySR);

        if (ignoredMedia & MII_CFG_100T2)
        {
            phyANAR &= ~(MII_ANAR_100T2_HD | MII_ANAR_100T2_FD);
        }
        else if (phySR & (MII_SR_100T2_FD | MII_SR_100T2_HD))
        {
            is100T2 = 1;
        }

        if (ignoredMedia & MII_CFG_100TX)
            phyANAR &= ~(MII_ANAR_100TX_HD | MII_ANAR_100TX_FD);

        if (ignoredMedia & MII_CFG_100T4)
            phyANAR &= ~(MII_ANAR_100T4);

        if (ignoredMedia & MII_CFG_10T)
            phyANAR &= ~(MII_ANAR_10T_HD | MII_ANAR_10T_FD);

        if (MII_CONFIG_CHECK(MII_CFG_HD))
        {
            phyANAR &= ~(MII_ANAR_FD_MASK);
        }
        else
        {
            if (MII_CONFIG_CHECK(MII_CFG_FD))
                phyANAR &= ~(MII_ANAR_HD_MASK);

            /* PAUSE functionality is relevant for Full Duplex only */
            if (MII_CONFIG_CHECK(MII_CFG_PAUSE))
                phyANAR |= (MII_ANAR_PAUSE_SYM | MII_ANAR_PAUSE_ASYM);
        }
    }

    /* write the advertisement register */
    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_ANAR, phyANAR);
    MII_RETURN_ON_ERROR(ret);

    /* start the auto-negotiation process */
    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, (MII_CR_AN_ENABLE | MII_CR_AN_RESTART));
    MII_RETURN_ON_ERROR(ret);

    /* wait until auto-negotiation is complete */
    do
    {
        if (waitCycles++ >= p_MiiParam->maxWaitCycles)
            return MII_LINK_DOWN;

        MII_WAIT(p_MiiParam->waitParam);

        ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
        MII_RETURN_ON_ERROR(ret);

    } while (!(phySR & MII_SR_AN_COMP));

    /* auto-negotiation is complete - check the results */

    /* first, check if a link has been established */
    waitCycles = 0;
    do
    {
        if (waitCycles++ >= p_MiiParam->maxWaitCycles)
            return ret;

        ret = miiLinkCheck(p_MiiParam);

    } while (ret != MII_OK);




    return MII_OK;
}


/**************************************************************************//**
 @Function      miiManualModeSet

 @Description   Dictates an operating mode to the PHY.

                The routine checks the configuration flags and activates the
                highest-priority ability. If no media and no duplex mode are
                specified, operation mode defaults to 10base-T, Full Duplex.
                Note that unsupported modes were previously cleared from the
                configuration flags by miiAbilityCheck().

                The function also calls miiLinkCheck() to verify that a link
                was established, and updates the configuration result flags
                according to the selected mode.

                When the MII_CFG_FORCE flag is specified in the configuration
                flags and the link was not established, the selected operating
                mode shall be written again to the control register, without
                re-checking the link. In this case, MII_CFG_FORCE will be
                specified in the configuration result flags, but the PHY is
                not guaranteed to work properly.

 @Param         p_MiiParam    - (in) pointer to MII information structure.

 @Retval        MII_OK              - Operation was successful.
 @Retval        MII_ERROR           - General error.
 @Retval        MII_LINK_DOWN       - Link is down after manual configuration.
 @Retval        MII_REMOTE_FAULT    - Link-partner's fault detected after
                                      manual configuration.

*//****************************************************************************/
static int miiManualModeSet(t_MiiParam *p_MiiParam)
{
    unsigned short  phyCR = MII_CR_MIN_ENABLE;
    unsigned short  phyMSCR = 0;
    int             ret;

    /* set the highest requested speed */
    if (MII_CONFIG_CHECK(MII_CFG_ANY_1000))
    {
        phyCR |= MII_CR_S1000;
        MII_RESULTS_SET(MII_CONFIG_CHECK(MII_CFG_ANY_1000));
    }
    else if (MII_CONFIG_CHECK(MII_CFG_ANY_100))
    {
        phyCR |= MII_CR_S100;
        MII_RESULTS_SET(MII_CONFIG_CHECK(MII_CFG_ANY_100));
    }
    else
    {
        MII_RESULTS_SET(MII_CFG_ANY_10);
    }

    /* configure master/slave for 100base-T2 / 1000base-T */
    if (MII_CONFIG_CHECK(MII_CFG_1000T | MII_CFG_100T2))
    {
        phyMSCR = MII_MSCR_MAN_CFG_EN;

        if (MII_CONFIG_CHECK(MII_CFG_MASTER))
        {
            phyMSCR |= MII_MSCR_MASTER;
            MII_RESULTS_SET(MII_CFG_MASTER);
        }
    }

    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_MSCR, phyMSCR);
    MII_RETURN_ON_ERROR(ret);

    /* set Full/Half Duplex */
    if (MII_CONFIG_CHECK(MII_CFG_HD))
    {
        MII_RESULTS_SET(MII_CFG_HD);
    }
    else
    {
        phyCR |= MII_CR_FD;
        MII_RESULTS_SET(MII_CFG_FD);
        MII_RESULTS_CLEAR(MII_CFG_100T4);
    }

    /* set loopback mode if requested */
    if (MII_CONFIG_CHECK(MII_CFG_LOOPBACK))
    {
        phyCR |= MII_CR_LOOPBACK;
        MII_RESULTS_SET(MII_CFG_LOOPBACK);
    }

    /* write the PHY control register */
    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, phyCR);
    MII_RETURN_ON_ERROR(ret);

    if (!(MII_CONFIG_CHECK(MII_CFG_LOOPBACK)))
    {
        MII_WAIT(p_MiiParam->waitParam);

        /* check if the link is up */
        ret = miiLinkCheck(p_MiiParam);

        if (((ret == MII_LINK_DOWN) || (ret == MII_REMOTE_FAULT)) &&
            (MII_CONFIG_CHECK(MII_CFG_FORCE)))
        {
            /* force the operating mode, but do not check the link */
            ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, phyCR);
            MII_RESULTS_SET(MII_CFG_FORCE);
        }
    }

    return ret;
}


/**************************************************************************//**
 @Function      miiLinkCheck

 @Description   Checks if a link is established, and if a remote fault exists.

                Note: The Jabber Detection bit is not checked, because it is
                not relevant for PHYs operating at 100 Mb/s or above.

 @Param         p_MiiParam    - (in) pointer to MII information structure.

 @Retval        MII_OK              - Operation was successful.
 @Retval        MII_ERROR           - Register read error.
 @Retval        MII_LINK_DOWN       - Link is down.
 @Retval        MII_REMOTE_FAULT    - Link-partner's fault detected.

*//****************************************************************************/
static int miiLinkCheck(t_MiiParam *p_MiiParam)
{
    unsigned short  phySR;
    int             ret;

    /* read the PHY status */
    ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
    MII_RETURN_ON_ERROR(ret);

    if (!(phySR & MII_SR_LINK))
    {
        /* wait and read it again (latched bit indication) */
        MII_WAIT(p_MiiParam->waitParam);
        ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
        MII_RETURN_ON_ERROR(ret);

        if (!(phySR & MII_SR_LINK))
        {
            return MII_LINK_DOWN;
        }
    }

    /* link is up - check remote fault twice (latched bit indication) */
    ret  = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
    ret |= MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
    MII_RETURN_ON_ERROR(ret);

    if (phySR & MII_SR_RFAULT)
    {
        return MII_REMOTE_FAULT;
    }

    return MII_OK;
}


/**************************************************************************//**
 @Function      miiPhyReset

 @Description   Resets the PHY and waits until reset is done.

 @Param         p_MiiParam    - (in) pointer to MII information structure.

 @Retval        MII_OK          - Operation was successful.
 @Retval        MII_ERROR       - Register read error.
 @Retval        MII_WAIT_LIMIT  - Wait limit was reached.

*//****************************************************************************/
static int miiPhyReset(t_MiiParam *p_MiiParam)
{
    unsigned short  phyCR;
    unsigned long   waitCycles = 0;
    int             ret;

    /* reset the PHY */
    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, MII_CR_RESET);
    MII_RETURN_ON_ERROR(ret);

    /* wait until reset bit is deasserted */
    do
    {
        if (waitCycles++ >= p_MiiParam->maxWaitCycles)
            return MII_WAIT_LIMIT;

        MII_WAIT(p_MiiParam->waitParam);

        ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_CR, &phyCR);
        MII_RETURN_ON_ERROR(ret);

    } while (phyCR & MII_CR_RESET);

    return MII_OK;
}


/**************************************************************************//**
 @Function      miiPhyIsolate

 @Description   Electrically isolates the PHY device.

 @Param         p_MiiParam  - (in) pointer to MII information structure.

 @Retval        MII_OK      - Operation was successful.
 @Retval        MII_ERROR   - Register write error.

*//****************************************************************************/
static int miiPhyIsolate(t_MiiParam *p_MiiParam)
{
    int ret;

    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, MII_CR_ISOLATE);
    return ret;
}


/**************************************************************************//**
 @Function      miiPhyPowerDown

 @Description   Puts the PHY in power-down mode (low power consumption).

 @Param         p_MiiParam  - (in) pointer to MII information structure.

 @Retval        MII_OK      - Operation was successful.
 @Retval        MII_ERROR   - Register write error.

*//****************************************************************************/
static int miiPhyPowerDown(t_MiiParam *p_MiiParam)
{
    int ret;

    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, MII_CR_PWR_DOWN);
    return ret;
}


/**************************************************************************//**
 @Function      miiPhyRestartAutoNegotiation

 @Description   Restarts the auto-negotiation process in the PHY.

 @Param         p_MiiParam  - (in) pointer to MII information structure.

 @Retval        MII_OK      - Operation was successful.
 @Retval        MII_ERROR   - Register write error.

*//****************************************************************************/
static int miiPhyRestartAutoNegotiation(t_MiiParam *p_MiiParam)
{
    unsigned long   waitCycles = 0;
    unsigned short  phySR;
    int             ret;

    ret = MII_PHY_WRITE(p_MiiParam->phyAddr, MII_CR, (MII_CR_AN_ENABLE | MII_CR_AN_RESTART));

    /* wait until auto-negotiation is complete */
    do
    {
        if (waitCycles++ >= p_MiiParam->maxWaitCycles)
            return MII_WAIT_LIMIT;

        MII_WAIT(p_MiiParam->waitParam);

        ret = MII_PHY_READ(p_MiiParam->phyAddr, MII_SR, &phySR);
        MII_RETURN_ON_ERROR(ret);

    } while (!(phySR & MII_SR_AN_COMP));

    return MII_OK;
}


