/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "enet_ext.h"
#include "xx_ext.h"

#include "tn2020_ext.h"
#include "tn2020.h"


#define __ERR_MODULE__ MODULE_UNKNOWN

#define GET_PHY_AND_DEVICE_ADDR(phyAddr, deviceAddr) \
    ((phyAddr << 5) | deviceAddr)


/********************************************************************************/
t_Error TN2020_Init(uint8_t           phyAddr,
                    e_EnetMode        ethMode,
                    bool              autoNegotiation,
                    e_EnetDuplexMode  duplexMode,
                    bool              forceMaster,
                    e_DiagMode        loopbackMode,
                    t_Handle          h_MiiAccess,
                    t_MiiPhyWriteFunc *f_SetPhyReg,
                    t_MiiPhyReadFunc  *f_GetPhyReg)
{
    uint16_t    reg16;
    int         timeout=5000; /* in mili-sces */
    uint32_t    phyId, startTime;


    f_SetPhyReg(h_MiiAccess, GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_VENDOR_SPECIFIC_1), MDD_CR, 0x8000);
    XX_UDelay(1000000);

    /* Grab the bits from PHYIR1, and put them in the upper half */
    f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_PMA_PMD), MDD_IDR_1, &reg16);
    phyId = (reg16 & 0xffff) << 16;

    /* Grab the bits from PHYIR2, and put them in the lower half */
    f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_PMA_PMD), MDD_IDR_2, &reg16);
    phyId |= (reg16 & 0xffff);

    DBG(TRACE, ("TN2020: PHY-ID is %x\n", phyId));

    if (loopbackMode == e_DIAG_MODE_PHY_LOOPBACK)
    {
        f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_PMA_PMD), MDD_CR, &reg16);
        f_SetPhyReg(h_MiiAccess, GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_PMA_PMD),MDD_CR, reg16 | 0x0001);
        f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_PMA_PMD), MDD_CR, &reg16);
        if (!(reg16 & 0x0001))
            REPORT_ERROR(MAJOR,E_NOT_AVAILABLE,("Ethernet PHY 0x%x, Phy Loop", phyAddr));
        return E_OK;
    }

    /* wait until auto-negotiation is complete */
    startTime = XX_CurrentTime();
    do
    {
        f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_AN), MDD_SR, &reg16);
        if (reg16 & 0x0020)
            break;
    } while ((XX_CurrentTime() - startTime) < timeout);
    if ((XX_CurrentTime() - startTime) >= timeout)
    {
        REPORT_ERROR(MINOR, E_TIMEOUT, ("Ethernet PHY 0x%x, auto-negotiation Failed", phyAddr));
        return E_OK;
    }

    /* first, check if a link has been established */
    startTime = XX_CurrentTime();
    do
    {
        if (TN2020_LinkCheck(phyAddr,h_MiiAccess,f_SetPhyReg,f_GetPhyReg) == MII_OK)
            break;
    } while ((XX_CurrentTime() - startTime) < timeout);
    if ((XX_CurrentTime() - startTime) >= timeout)
    {
        REPORT_ERROR(MAJOR,E_NOT_AVAILABLE,("Ethernet PHY 0x%x, Link Down", phyAddr));
        return E_OK;
    }

    f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_VENDOR_SPECIFIC_1), MDD_SR, &reg16);
    if (!(reg16 & 0x0010))
        DBG(INFO, ("Auto-negotiation resolved to 10Gb/s speed"));
    else
        DBG(INFO, ("Auto-negotiation not resolved to 10Gb/s speed"));

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
    {
        uint8_t     reg;
        uint16_t    reg16;

        XX_Print("\tPHY %d regs:\n", phyAddr);
        for (reg=0; reg < 25; reg++)
        {
            f_GetPhyReg(h_MiiAccess, phyAddr, reg, &reg16);
            XX_Print("Reg %2d, Value = 0x%04x\n", reg, reg16);
        }
    }
#endif /* (defined(DEBUG_ERRORS) && ... */

    return E_OK;
}

int TN2020_LinkCheck(uint8_t           phyAddr,
                     t_Handle          h_MiiAccess,
                     t_MiiPhyWriteFunc *f_SetPhyReg,
                     t_MiiPhyReadFunc  *f_GetPhyReg)
{
    uint16_t  reg16;
    int       ret;

    /* read the PHY status */
    ret = f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_AN), MDD_SR, &reg16);
    if (ret != E_OK)
        return ret;

    if (!(reg16 & 0x0004))
    {
        /* wait and read it again (latched bit indication) */
        XX_UDelay(10000);
        ret = f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_AN), MDD_SR, &reg16);
        if (ret != E_OK)
            return ret;

        if (!(reg16 & 0x0004))
            return MII_LINK_DOWN;
    }

    /* link is up - check remote fault twice (latched bit indication) */
    ret = f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_AN), MDD_SR, &reg16);
    ret = f_GetPhyReg(h_MiiAccess,GET_PHY_AND_DEVICE_ADDR(phyAddr,MDD_AN), MDD_SR, &reg16);

    if (ret != E_OK)
        return ret;

    if (reg16 & 0x0010)
        return MII_REMOTE_FAULT;

    return MII_OK;
}
