/******************************************************************************

 © 1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "enet_ext.h"
#include "xx_ext.h"
#include "vsc8641_ext.h"


#define __ERR_MODULE__ MODULE_UNKNOWN

/********************************************************************************
*********************************************************************************
Notes:
1) Normally in the two low frequencies (10M,100M) the chip does not require AUTO
   NEGOTIATION to operate, and therefore in this case we shall manually set the
   PHY to operate in a non-negotiating mode.
2) When activated in 1000M the PHY requires that AUTO NEGOTIATION is available.
   To comply with the API of this system we shall provide the advertizing registers
   with information that supports 1000M only.
3) Physical addresses for the Cicada PHY chips are: {phyAddr=01H or 05H}.
*********************************************************************************
*********************************************************************************
*/


 /* enetInterface; Ethernet physical interface                                 */
 /* loopback - {TRUE-Loop back ON, FALSE- Normal operation.}                   */
 /*            When Loopback is activated then the transmit signal from the MAC*/
 /*            is redirected toward the MAC receive line. No signal is         */
 /*            transmitted to the network side.                                */
 /* halfDuplex - : {TRUE- half duplex operation mode, FALSE- full Duplex mode.}*/
 /* phyAddr - PHY address {0..31}.This address is HARD wired to every chip.    */
 /*           This address is latched when the HARDWARE reset signal is        */
 /*           deasserted. The saved value is used with communication on the    */
 /*           miiMng... bus. Only the PHY that it's latched address equals to  */
 /*           the phyAddr will respond to the message on the bus.              */
 /* h_MiiAccess - Base address of MII configuration cont. reg.   */
 /* f_SetPhyReg,f_GetPhyReg - Function pointers to write and read from PHY. */

t_Error VSC8641_Init(   uint8_t           phyAddr,
                        e_EnetMode        ethMode,
                        bool              autoNegotiation,
                        e_EnetDuplexMode  duplexMode,
                        e_DiagMode        loopbackMode,
                        t_Handle          h_MiiAccess,
                        t_MiiPhyWriteFunc *f_SetPhyReg,
                        t_MiiPhyReadFunc  *f_GetPhyReg)
{
    uint16_t    reg16;
    bool        PHY_Initialized=FALSE;

    // RESET PHY - Will provide us with a well defined hardware state.
    //             (This assumes that 'Reset-Sticky' bits remains in thier last state).
    reg16 = BMCR_RESET;                 /* Will cause the Software Reset. */
    f_SetPhyReg(h_MiiAccess, phyAddr, MII_8641_ModeControl, reg16);
    // Reset value for the 'MII_8641_ModeControl' register upon exit from the RESET
    // is 0x1040. This value implies that:
    // [Loopback is OFF, Forced Speed Selection is 1000M,ANEG is enabled,
    //  Power is UP, PHY is connected to MII, Half Duplex mode is selected and Collision
    //  test is disabled.

    XX_UDelay(200000);

    // Ensure that the 125MHz clock will permanntly stay in the interface to the MAC.
    f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_BypassControl, &reg16);
    reg16 |= 1;  /* 125MHz Clock Output Enable. */
    f_SetPhyReg(h_MiiAccess, phyAddr, MII_8641_BypassControl, reg16);

    if ( ethMode == e_ENET_MODE_RGMII_10)
    {
        // As we do not intend to implement Auto-Negotiation at this stage some of these
        // bits need to be modified.

        f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
        // Loopback may be active. !!! (if requested to do so.)
        reg16 &= ~BMCR_SPEED_SELECT_MASK;   /* NO software reset. No Auto-Negotiation,
                                               Speed Select adjusted for 10MHz.  */
        reg16 &= ~BMCR_AUTO_NEG_ENABLE;
        // HalfDuplex bit is CLEARED as it is a Reset default value.
        if (duplexMode)
           reg16 |= BMCR_FULL_DUPLEX;                        /* Set for Full Duplex. */

        f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl, reg16);

        // Auto-Negotiation Advertisment Register[04] need not be adjusted as ANEG is
        // disabled.

        reg16 = (EPHYCR_MAC_MODE_SEL_RGMII_VAL<<12); /* MAC interface mode selected as RGMII. */
        reg16 |= (EPHYCR_MAC_IF_DIO_PS_25_VAL<<9);   /* I/o pins will operate from 2.5V */
        reg16 |= EPHYCR_RGMII_SKEW_ENABLE_MASK;      /* Sounds good. */
        // All other functionality will use default values.
        f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ExtendedPHYCont1, reg16);
        PHY_Initialized=TRUE;
    }
//----[ End of "== e_ENET_10_RGMII". ]--------------------------

    else if ( ethMode == e_ENET_MODE_RGMII_100)
    {
        // As we do not intend to implement Auto-Negotiation at this stage some of these
        // bits need to be modified.

        f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
        reg16 &= ~BMCR_SPEED_SELECT_MASK;/* NO software reset. No Auto-Negotiation */
        reg16 |= BMCR_SPEED_SELECT_L;    /* Readjust speed select bits for 100MHz. */

        // HalfDuplex bit is CLEARED as it is a Reset default value.
        if (duplexMode)
           reg16 |= BMCR_FULL_DUPLEX;       /* Set for Full Duplex. */

        f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl, reg16);

        // Auto-Negotiation Advertisment Register[04] need not be adjusted as ANEG is
        // disabled.

        reg16 = (EPHYCR_MAC_MODE_SEL_RGMII_VAL<<12); /* MAC interface mode selected as RGMII. */
        reg16 |= (EPHYCR_MAC_IF_DIO_PS_25_VAL<<9);   /* I/o pins will operate from 2.5V */
        reg16 |= EPHYCR_RGMII_SKEW_ENABLE_MASK;      /* Sounds good. */
        // All other functionality will use default values.
        f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ExtendedPHYCont1, reg16);

        f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_AutoNegAdvert, &reg16);

        // Program Reg.04-MII_8641_AutoNegAdvertisemenRegister.
        reg16 = 0x0181;     // This value stands for:
                            // bit.8 - Advertise Capability - 100B-X FDX ['1'].
                            // bit.7 - Advertise Capability - 100B-X HDX ['1'].
                            // bit.4:0 - Advertise Selector Field .['0001']

        f_SetPhyReg(h_MiiAccess,  phyAddr, MII_8641_AutoNegAdvert, reg16);

        f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_1000BaseTControl, &reg16);
        // Program Reg.04-MII_8641_AutoNegAdvertisemenRegister.
        reg16 = 0x0000;     // Clears 1000B-T capability
        f_SetPhyReg(h_MiiAccess,  phyAddr, MII_8641_1000BaseTControl, reg16);


        XX_UDelay(400000);
        PHY_Initialized=TRUE;
    }
//----[ End of "== e_ENET_100_RGMII". ]-------------------------

    else
    { // Required speed is 1000MHz.

        // Program Reg.09-MII_8641_1000BaseTControl.
        reg16 = 0x0300;     // This value stands for:
                            // bit.15:13 - Transmitter Test Mode.{Normal}['000'].
                            // bit.12 - Disable M/S manual conf.       ['0']
                            // bit.11 - M/S manual configuration value.['0']
                            // bit.10 - Port type {Single port device}.['0']
                            // bit. 9 - 1000Base-T FDX Capability      ['1']
                            // bit. 8 - 1000Base-T HDX Capability      ['1']
                            // All other bits are reserved and stays '0'.
        f_SetPhyReg(h_MiiAccess, phyAddr, MII_8641_1000BaseTControl, reg16 );

        if ( ethMode == e_ENET_MODE_RGMII_1000)
        {
            f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
            reg16 &= ~BMCR_SPEED_SELECT_MASK;
            reg16 |= BMCR_SPEED_SELECT_H;    /* Readjust speed select bits for 1000MHz.*/

            // HalfDuplex bit is CLEARED as it is a Reset default value.
            if (duplexMode)
            reg16 |= BMCR_FULL_DUPLEX;      /* Set for Full Duplex. */


            f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl,reg16);

            reg16 = (EPHYCR_MAC_MODE_SEL_RGMII_VAL<<12); /* MAC interface mode selected as RGMII. */
            reg16 |= (EPHYCR_MAC_IF_DIO_PS_25_VAL<<9);   /* I/o pins will operate from 2.5V */
            reg16 |= EPHYCR_RGMII_SKEW_ENABLE_MASK;      /* Sounds good. */
            // All other functionality will use default values.

            reg16 &= 0x1122;
                            // This value stands for:
                            // bit.12    - Mac interface select             ['1'] - RGMII
                            // bit. 8    - RGMII skew timing compensation.  ['1'] - adds 1.7 ns delay to the RX_CLK
                            // bit. 5    - ActiPHY mode enable              ['1'] - Enable
                            // bit. 1    - GMII transmit pin reversal       ['1'] - GMII transmit pin order reversed
                            // All other bits are reserved and stays '0'.
            f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ExtendedPHYCont1,reg16);
        }
//------[ End of "== e_ENET_1000_RGMII". ]------------------------

        if(autoNegotiation)
        {
            f_GetPhyReg(h_MiiAccess,phyAddr, MII_8641_AutoNegAdvert, &reg16);

            // Program Reg.07-MII_8641_AutoNegNextPageTx.
            reg16 = 0xA001;     // This value stands for:
                                // bit.15 - Next page transmission request ['1'].
                                // bit.13 - Message Type {1-Message page}. ['1']
                                // bit.12 - Acknowledge2{Cann't comply}.   ['0']
                                // bit.11 - Toggle {Read Only}.            ['0']
                                // bit.10:0 - Message/Unformatted code.['00000000001']

            f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_AutoNegNextPageTx,reg16);

            f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
                reg16 |= BMCR_AUTO_NEG_ENABLE;

            f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl,reg16);

            // Restart Auto-Negotiation.
            f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
            reg16 |= 0x200;  /* Restart Auto-Negotiation. */
            f_SetPhyReg(h_MiiAccess, phyAddr, MII_8641_ModeControl, reg16);


        }
        else
        {
             f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
                reg16 &= ~BMCR_AUTO_NEG_ENABLE;

             f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl,reg16);

        }



        if (loopbackMode == e_DIAG_MODE_PHY_LOOPBACK)
        {
           f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ExtendedPHYCont2, &reg16);
           reg16 |= 1;                       /* Set Front End loop-back. */
           f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ExtendedPHYCont2,reg16);

           f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_BypassControl, &reg16);
           reg16 |= 20;                       /* Disable Automatic Pair Swap Correction. */
           f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_BypassControl,reg16);
        }

        PHY_Initialized = TRUE;
        XX_UDelay(1000000);

    } // Required speed is 1000MHz.

    // Loopback resides in the MAC side (e.g. signals which are transmitted by the MAC
    // on the TX Bus are returned toward the MAC on the RX Bus. All that time the
    // external interface remaines idle. No signals are transmitted over the network media.
    // for that reason ANEG functionality must end before Loopback activation.

    if (loopbackMode == e_DIAG_MODE_PHY_LOOPBACK)
    {
       f_GetPhyReg(h_MiiAccess,  phyAddr, MII_8641_ModeControl, &reg16);
       reg16 |= BMCR_LOOPBACK;                       /* Set Front End loop-back. */
       f_SetPhyReg(h_MiiAccess,phyAddr, MII_8641_ModeControl,reg16);
    }

    if (PHY_Initialized)
        return E_OK;
    else
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Unsupported interface"));
    }
}

