/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          buf_ext.h

 @Description   External definitions and prototypes for buffer object.

*//***************************************************************************/

#ifndef __BUF_EXT_H
#define __BUF_EXT_H


#include "std_ext.h"


/**************************************************************************//**
 @Group         etc_id   Utility Library Application Programming Interface

 @Description   External routines.

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Description   Buffer pool access parameters structure.
*//***************************************************************************/
typedef struct t_BufMemAccess
{
    t_Handle    h_MemMng;       /**< Handle to the buffers memory manager */
    void *      (*f_GetMem)(t_Handle h_MemMng);
                                /**< Callback - gets an available memory block */
    t_Error     (*f_PutMem)(t_Handle h_MemMng, void *p_MemBlock);
                                /**< Callback - returns the memory block */

} t_BufMemAccess;


/**************************************************************************//**
 @Group         buffer_id Buffer

 @Description   Buffer module functions, definitions and enums.

 @{
*//***************************************************************************/

/* Each buffer structure represents a data block with the following structure:
 *
 *
 * headLimit    p_Data     first/head                   last/tail       tailLimit
 * (fixed)      (fixed)    (variable)                   (variable)      (fixed)
 *    |            |        |                              |               |
 *    V            V        V                              V               V
 *    +------------+--------+------------------------------+---------------+
 *    | Head Room           /  Actual Data                 /  Tail Room    |
 *    | (variable)          /  (variable)                  /  (variable)   |
 *    |                     /                              /               |
 *    +------------+--------+------------------------------+---------------+
 *
 *  At any given moment, the actual data is between first and last markers.
 *  These markers can move back and forth using the API functions. The "first"
 *  marker can be negative (before p_Data), but the "last" marker cannot.
 *  The first marker is initialized to zero (i.e. pointed by p_Data).
 *
 */

/**************************************************************************//**
 @Description   Buffer structure
*//***************************************************************************/
typedef struct t_Buffer
{
/* MUST BE FIRST */
    struct t_Buffer     *p_Next;        /**< Pointer to next buffer */

    struct t_Buffer     *p_Prev;        /**< Pointer to previous buffer */
    t_BufMemAccess      *p_MemAccess;   /**< Pointer to memory manager access structure */
    uint8_t             *p_Data;        /**< Points to the zero-offset byte in data block */
    volatile uint16_t   owners;         /**< Number of owners of buffer */
    uint16_t            tailLimit;      /**< Maximum size of data after p_Data */
    uint16_t            headLimit;      /**< Maximum size of data before p_Data */
    uint16_t            last;           /**< Index of last data byte in buffer */
    int32_t             first;          /**< Index of first data byte in buffer */
    void*               p_UserPriv;       /**< user private data */

} t_Buffer;



/**************************************************************************//**
 @Function      B_Sizeof

 @Description   Returns the size of a buffer structure.

 @Return        Size of buffer structure.
*//***************************************************************************/
#define B_Sizeof()  sizeof(t_Buffer)


/**************************************************************************//**
 @Function      B_InitPool

 @Description   Initializes the global buffer objects pool.

 @Param[in]     numOfBuffers - Number of buffer objects in the global pool.

 @Return        None.
*//***************************************************************************/
t_Error B_InitPool(uint32_t numOfBuffers);

/**************************************************************************//**
 @Function      B_FreePool

 @Description   Frees the global buffers pool.

 @Return        None.
*//***************************************************************************/
void    B_FreePool(void);


/**************************************************************************//**
 @Function      B_New

 @Description   Allocates a new buffer structure.

                The user initializes the buffer structure with a pointer to a
                data block, a callback funtion for releasing the data block,
                the initial data length, and values which define the maximum
                space before and after that pointer.

 @Param[in]     p_MemAccess - Pointer to memory manager access structure.
                              This pointer must remain valid until the buffer
                              object it released.
 @Param[in]     tailLimit   - Maximum size of data after p_Data.
 @Param[in]     headLimit   - Maximum size of data before p_Data.
 @Param[in]     p_Data      - Pointer to the data. It is stored as the zero-offset
                              byte of the data, and shall be passed as the second
                              argument of p_MemAccess->f_PutMem.
                              If NULL, a new memory block is retrieved using the
                              memory access parameters structure.
 @Param[in]     length      - Initial length of data, starting from p_Data
                              (can be zero).

 @Return        A handle to the buffer structure on success; NULL otherwise.

 @Cautions
                - tailLimit must not be zero.
                - The sum (tailLimit + headLimit) must not exceed 0xFFFF (i.e.
                  less than 64KB).
                - length must not exceed tailLimit.

*//***************************************************************************/
t_Handle    B_New(t_BufMemAccess    *p_MemAccess,
                  uint16_t          tailLimit,
                  uint16_t          headLimit,
                  uint8_t           *p_Data,
                  uint16_t          length);

/**************************************************************************//**
 @Function      B_Delete

 @Description   Deletes a buffer structure, and optionally returns the buffer
                memory block to the memory manager.

                All memory that was used by the buffer structure is returned to
                its original pool. If the buffer structure had N owners, then
                no operation is taken until this function is called N times.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     releaseData - TRUE to return data memory block to the buffer
                              memory manager; FALSE otherwise.

 @Return        Pointer to the data memory block, or NULL if it was released.
*//***************************************************************************/
uint8_t *   B_Delete(t_Handle h_Buffer, bool releaseData);

/**************************************************************************//**
 @Function      B_ReleaseData

 @Description   Releases the data memory block associated with this buffer.

                The f_PutMem function of the buffer memory manager is called
                to release the data memory block.

 @Param[in]     h_Buffer - Buffer structure handle.

 @Cautions      This function must not be called if the buffer structure was
                initialized with a NULL f_PutMem callback.
*//***************************************************************************/
void        B_ReleaseData(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_SetDataBlock

 @Description   Sets a new data block for the buffer, and returns the previous
                one. The first byte of meaningful data is set to offset zero
                (pointed by p_Data), and the last is set according to the
                given length.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     p_Data      - A pointer to the data.
 @Param[in]     length      - Initial length of data, starting from p_Data
                              (can be zero).

 @Return        Pointer to the previous data block; This pointer can be used
                for releasing the data block.
*//***************************************************************************/
uint8_t *   B_SetDataBlock(t_Handle h_Buffer, uint8_t *p_Data, uint16_t length);

/**************************************************************************//**
 @Function      B_ReserveHeadRoom

 @Description   Increments the number of bytes that are reserved before the
                first byte of data.

                This function can be used to reserve area for protocol headers
                after initializing an empty buffer.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     headRoom    - Number of bytes to reserve before the first
                              byte of data.

 @Cautions      This function should be called only when data length is zero.
*//***************************************************************************/
void        B_ReserveHeadRoom(t_Handle h_Buffer, uint16_t headRoom);

/**************************************************************************//**
 @Function      B_AddToHead

 @Description   Adds data to the head of the buffer (before the first data byte).

                The number of added bytes is bounded by the buffer's head room.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     p_Data      - Pointer to the data.
 @Param[in]     length      - Data length (number of bytes to add).

 @Return        The number of bytes actually added.
*//***************************************************************************/
uint16_t    B_AddToHead(t_Handle h_Buffer, uint8_t *p_Data, uint16_t length);

/**************************************************************************//**
 @Function      B_AddToTail

 @Description   Adds data to the tail of the buffer (after the last data byte).

                The number of added bytes is bounded by the buffer's tail room.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     p_Data      - Pointer to the data.
 @Param[in]     length      - Data length (number of bytes to add).

 @Return        The number of bytes actually added.
*//***************************************************************************/
uint16_t    B_AddToTail(t_Handle h_Buffer, uint8_t *p_Data, uint16_t length);

/**************************************************************************//**
 @Function      B_CutFromHead

 @Description   Discards data from the head of the buffer.

                This function does not clear the contents of the removed bytes,
                it only moves the position of the first data byte.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to remove.

 @Return        The number of bytes actually discarded.
*//***************************************************************************/
uint16_t    B_CutFromHead(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_CutFromTail

 @Description   Discards data from the tail of the buffer.

                This function does not clear the contents of the removed bytes,
                it only moves the position of the last data byte.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to remove.

 @Return        The number of bytes actually discarded.
*//***************************************************************************/
uint16_t    B_CutFromTail(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_Clone

 @Description   Creates a new buffer structure with a copy of the data, and
                with the attributes of the original buffer.

                The new buffer is retrieved using the buffers pool access
                parameters, stored in the original buffer.

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        A handle to the new buffer structure on success; NULL otherwise.
*//***************************************************************************/
t_Handle    B_Clone(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_Print

 @Description   Prints the data in the buffer.

 @Param[in]     h_Buffer - Buffer structure handle.
*//***************************************************************************/
void        B_Print(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetMemAccessPtr

 @Description   Gets the buffer memory access pointer (as passed to B_New()).

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        Pointer to the buffer memory access pointer.
*//***************************************************************************/
#define     B_GetMemAccessPtr(h_Buffer) \
                (((t_Buffer *)(h_Buffer))->p_MemAccess)

/**************************************************************************//**
 @Function      B_GetTailLimit

 @Description   Gets the tail limit of the buffer (as passed to B_New()).

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        The tail limit of the buffer.
*//***************************************************************************/
#define     B_GetTailLimit(h_Buffer)    \
                (((t_Buffer *)(h_Buffer))->tailLimit)

/**************************************************************************//**
 @Function      B_GetHeadLimit

 @Description   Gets the head limit of the buffer (as passed to B_New()).

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        The head limit of the buffer.
*//***************************************************************************/
#define     B_GetHeadLimit(h_Buffer)    \
                (((t_Buffer *)(h_Buffer))->headLimit)


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      B_Link

 @Description   Adds an owner to an existing buffer (not interrupt safe).

 @Param[in]     h_Buffer - Buffer structure handle.
*//***************************************************************************/
void        B_Link(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_SetLength

 @Description   Sets the length of the data in a buffer.

                The new length is relative to the current position of the
                first data byte, and not to zero offset

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - length of data (in bytes).
*//***************************************************************************/
void        B_SetLength(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_GetLength

 @Description   Gets the length of the data in the buffer.

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        The data length.
*//***************************************************************************/
uint16_t    B_GetLength(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetData

 @Description   Gets the data of a buffer (address of first byte of data)

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        Pointer to the first byte of data.

 @Cautions      The returned pointer should not be used for releasing the data
                block, because it may be at offset from the data block pointer.
                To get the data block pointer, use B_GetDataBlock().
*//***************************************************************************/
uint8_t *   B_GetData(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetDataBlock

 @Description   Gets the pointer to the data block (as given in B_New() or
                B_SetDataBlock()).

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        Pointer to the data block.
*//***************************************************************************/
uint8_t *   B_GetDataBlock(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_PullHead

 @Description   Increments the position of the first data byte in the buffer.

                This function can be used to remove a header from the data.
                The function does not clear the contents of the removed bytes,
                it only moves the position of the first data byte.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to remove from the head.

 @Cautions      This function must be called only if length does not exceed
                the buffer's data length. For safer (and slower) operation,
                and for partial removal, use B_CutFromHead().
*//***************************************************************************/
void        B_PullHead(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_PushHead

 @Description   Decrements the position of the first data byte in the buffer.

                This function can be used to include a part which already
                exists in front of the data as part of the data (header).
                The function adds the header only if there is enough room for
                it at the head of the buffer (no partial addition).

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to add to the head.

 @Cautions      This function must be called only if length does not exceed
                the buffer's head room. For safer (and slower) operation,
                and for partial removal, use B_AddToHead().
*//***************************************************************************/
void        B_PushHead(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_PullTail

 @Description   Decrements the position of the last data byte in the buffer.

                This function can be used to remove a trailer from the data.
                The function does not clear the contents of the removed bytes,
                it only moves the position of the last data byte.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to remove from the tail.

 @Cautions      This function must be called only if length does not exceed
                the buffer's data length. For safer (and slower) operation,
                and for partial removal, use B_CutFromTail().
*//***************************************************************************/
void        B_PullTail(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_PushTail

 @Description   Increments the position of the last data byte in the buffer.

                This function can be used to include a part which already
                exists right after the data as part of the data (trailer).
                The function adds the trailer only if there is enough room for
                it at the tail of the buffer (no partial addition).

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     length      - Number of bytes to add to the tail.

 @Cautions      This function must be called only if length does not exceed
                the buffer's tail room. For safer (and slower) operation,
                and for partial removal, use B_AddToTail().
*//***************************************************************************/
void        B_PushTail(t_Handle h_Buffer, uint16_t length);

/**************************************************************************//**
 @Function      B_ResetData

 @Description   Sets the markers of first and last data bytes to zero.

                This function removes any reserved head room and sets the data
                length to zero. The data block pointer is not affected, and
                the data bytes are not cleared in any way.

 @Param[in]     h_Buffer - Buffer structure handle.
*//***************************************************************************/
void        B_ResetData(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetNext

 @Description   Gets the handle of the next buffer.

 @Param[in]     h_Buffer - Buffer structure handle.
*//***************************************************************************/
t_Handle    B_GetNext(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetPrev

 @Description   Gets the handle of the previous buffer.

 @Param[in]     h_Buffer - Buffer structure handle.
*//***************************************************************************/
t_Handle    B_GetPrev(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_GetUserPriv

 @Description   Gets the p_UserPriv field from the buffer.

 @Param[in]     h_Buffer - Buffer structure handle.

 @Return        The value of p_UserPriv field.
*//***************************************************************************/
void*    B_GetUserPriv(t_Handle h_Buffer);

/**************************************************************************//**
 @Function      B_SetNext

 @Description   Sets the next buffer.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     nextBuf     - Handle of buffer to set as next.
*//***************************************************************************/
void        B_SetNext(t_Handle h_Buffer, t_Handle nextBuf);

/**************************************************************************//**
 @Function      B_SetPrev

 @Description   Set the previous buffer.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     prevBuf     - Handle of buffer to set as previous.
*//***************************************************************************/
void        B_SetPrev(t_Handle h_Buffer, t_Handle prevBuf);

/**************************************************************************//**
 @Function      B_SetInfo3

 @Description   Sets the p_UserPriv field in the buffer.

 @Param[in]     h_Buffer    - Buffer structure handle.
 @Param[in]     p_UserPriv    - Value of p_UserPriv.
*//***************************************************************************/
void        B_SetUserPriv(t_Handle h_Buffer, void* p_UserPriv);

#else /* not DEBUG_ERRORS */


#define B_SetLength(p_B, length)    \
    ( ((t_Buffer *)(p_B))->last = (uint16_t)(((t_Buffer *)(p_B))->first + (length)) )

#define B_GetLength(p_B)            \
    ( (uint16_t)((int32_t)((t_Buffer *)(p_B))->last - ((t_Buffer *)(p_B))->first) )

#define B_GetData(p_B)              \
    ( (uint8_t *)(((t_Buffer *)(p_B))->p_Data + ((t_Buffer *)(p_B))->first) )

#define B_GetDataBlock(p_B)         \
    ( (uint8_t *)(((t_Buffer *)(p_B))->p_Data) )

#define B_PullHead(p_B, length)     \
    ( ((t_Buffer *)(p_B))->first += (length) )

#define B_PushHead(p_B, length)     \
    ( ((t_Buffer *)(p_B))->first -= (length) )

#define B_PullTail(p_B, length)     \
    ( ((t_Buffer *)(p_B))->last -= (length) )

#define B_PushTail(p_B, length)     \
    ( ((t_Buffer *)(p_B))->last += (length) )

#define B_ResetData(p_B)            \
    ( ((t_Buffer *)(p_B))->first = ((t_Buffer *)(p_B))->last = 0 )

#define B_Link(p_B)                 \
    ( ((t_Buffer *)(p_B))->owners++ )

#define B_GetNext(p_B)              \
    ( ((t_Buffer *)(p_B))->p_Next )

#define B_GetPrev(p_B)              \
    ( ((t_Buffer *)(p_B))->p_Prev )

#define B_GetUserPriv(p_B)              \
    ( (void*)(((t_Buffer *)(p_B))->p_UserPriv) )

#define B_SetNext(p_B, Next)        \
    ( ((t_Buffer *)(p_B))->p_Next = (Next) )

#define B_SetPrev(p_B, Prev)        \
    ( ((t_Buffer *)(p_B))->p_Prev = (Prev) )

#define B_SetUserPriv(p_B, p_UserPriv)        \
    ( ((t_Buffer *)(p_B))->p_UserPriv = (void*)(p_UserPriv) )


#endif  /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of buffer_id group */
/** @} */ /* end of etc_id group */


#endif /* __BUF_EXT_H */

