/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          shared_memory_ext.h

 @Description   Prototypes, externals and typedefs for shared_memory resource
*//***************************************************************************/

#ifndef __SHARED_MEMORY_EXT_H
#define __SHARED_MEMORY_EXT_H


/**************************************************************************//**
 @Function      t_TxConfRoutine

 @Description   Upper layer's Tx confirmation callback.

                This callback is used by shared memory data transmition
                to return the data buffer back to the application.

                User provides this function. Driver invokes it.

 @Param[in]     h_App           - Upper layer handle provided in t_SharedMemoryParam
                                  to SHARED_MEMORY_Init() routine, passed unchanged
                                  to the application.
 @Param[in]     queueId         - Queue number to which data has been sent.
 @Param[in]     p_Data          - Pointer to data buffer.
 @Param[in]     dataLength      - Data buffer length (in bytes).

*//***************************************************************************/
typedef void (t_TxConfRoutine)(t_Handle    h_App,
                               uint32_t    queueId,
                               uint8_t     *p_Data,
                               uint32_t    dataLength);

/**************************************************************************//**
 @Function      t_RxStoreRoutine

 @Description   Upper layer's Rx store callback.

                This callback is used by shared memory data transmition
                to forward received data to the application.

                User provides this function. Driver invokes it.

 @Param[in]     h_App           - Upper layer handle provided in t_SharedMemoryParam
                                  to SHARED_MEMORY_Init() routine, passed unchanged
                                  to the application.
 @Param[in]     queueId         - Queue number from which data has been received.
 @Param[in]     p_Data          - Pointer to data buffer.
 @Param[in]     dataLength      - Data buffer length (in bytes).

*//***************************************************************************/
typedef void (t_RxStoreRoutine)(t_Handle  h_App,
                                uint32_t  queueId,
                                uint8_t   *p_Data,
                                uint32_t  dataLength);

/**************************************************************************//**
 @Description   Required queue alignment - recommended to choose it according
                to cache line size.
*//***************************************************************************/
typedef enum e_SharedMemoryQueueAlignment
{
   e_4_BYTES_QUEUE_ALIGNMENT   = 4,     /**< 4-bytes alignment */
   e_32_BYTES_QUEUE_ALIGNMENT  = 32,    /**< 32-bytes alignment */
   e_64_BYTES_QUEUE_ALIGNMENT  = 64     /**< 64-bytes alignment */
} e_SharedMemoryQueueAlignment;

/**************************************************************************//**
 @Description   Shared memory parameters provided to SHARED_MEMORY_EarlyInit()
                routine.
*//***************************************************************************/
typedef struct t_SharedMemoryEarlyInitParam
{
    uint64_t                 baseAddress;           /**< Base address of the shared memory module. */
    uint32_t                 sharedMemorySize;      /**< The total size of the shared memory (in bytes). */
    uint32_t                 msgBodyLength;         /**< The length of the message body (in bytes.) */
    uint32_t                 numOfMessages;         /**< Number of messages to allocate in the shared memory. */
    uint32_t                 numOfMessageQueues;    /**< Number of message queues to allocate in the shared memory area;
                                                         NOTE: queue #0 will be reserved for internal use, so
                                                         number of queues actually available for application will be
                                                         one less than the value provided here. */
    e_SharedMemoryQueueAlignment queueAlignment;    /**< Queue alignment requirement. */
} t_SharedMemoryEarlyInitParam;

/**************************************************************************//**
 @Description   Shared memory parameters provided to SHARED_MEMORY_Init()
                routine.
*//***************************************************************************/
typedef struct t_SharedMemoryParam
{
    uint32_t                 id;            /**< Shared memory identifier. */
    t_Handle                 h_App;         /**< Upper layer handle, passed as is to the below callback routines. */
    t_TxConfRoutine          *f_TxConf;     /**< Upper layer's transmit confirmation callback, is issued to confirm
                                                 transmitted frames. */
    t_RxStoreRoutine         *f_RxStore;    /**< Upper layer's receive store callback, is issued to forward
                                                 received message to application. */
    uint64_t                 baseAddress;   /**< Base address of shared memory region. */
    bool                     interrupts;    /**< Interrupt or polling mode; TRUE for interrupt. */
    uint32_t                 irq;           /**< IRQ number; relevant only if interrupt mode is TRUE. */
} t_SharedMemoryParam;



/**************************************************************************//**
 @Function      SHARED_MEMORY_EarlyInit

 @Description   This routine is used to initialize the shared memory region.

                When shared memory is used as inter-partition communication
                (IPC) infrastructure, this function should be called only once
                by master partition prior to using IPC mechanism.

 @Param[in]     p_Param  - Shared memory early init parameter structure.

 @Return        E_OK on success; Error code is returned otherwise.

 @Cautions      This routine should be called only by the master partition
                in order to initialize once the shared memory map.
*//***************************************************************************/
t_Error SHARED_MEMORY_EarlyInit(t_SharedMemoryEarlyInitParam *p_Param);

/**************************************************************************//**
 @Function      SHARED_MEMORY_Init

 @Description   This routine is used to initialize shared memory driver object
                on every partition.

 @Param[in]     p_Param  - Shared memory parameter structure.

 @Return        Shared memory handle on success; NULL otherwise.
*//***************************************************************************/
t_Handle SHARED_MEMORY_Init(t_SharedMemoryParam *p_Param);

/**************************************************************************//**
 @Function      SHARED_MEMORY_Free

 @Description   This routine is used to free shared memory driver object.

 @Param[in]     h_SharedMemory  - Shared memory handle.

 @Return        E_OK on success; Error code is returned otherwise.
*//***************************************************************************/
t_Error SHARED_MEMORY_Free(t_Handle h_SharedMemory);

/**************************************************************************//**
 @Function      SHARED_MEMORY_Write

 @Description   This routine is used to write data to the shared memory.

 @Param[in]     h_SharedMemory  - Shared memory handle.
 @Param[in]     queueId         - Queue number to write the data.
 @Param[in]     p_Data          - Pointer to data.
 @Param[in]     length          - Data length (in bytes).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SHARED_MEMORY_Write(t_Handle   h_SharedMemory,
                            uint32_t   queueId,
                            uint8_t    *p_Data,
                            uint32_t   length);

/**************************************************************************//**
 @Function      SHARED_MEMORY_Read

 @Description   This routine is used to read data from the shared memory
                in polling mode.

                When operating in interrupt mode, this routine is issued
                internally by the driver. Thus, in case of interrupt
                mode using this routine is not allowed.

 @Param[in]     h_SharedMemory  - Shared memory handle.
 @Param[in]     queueId         - Queue number to read the data.

 @Return        E_OK is on success; Error code otherwise.
*//***************************************************************************/
t_Error SHARED_MEMORY_Read(t_Handle h_SharedMemory, uint32_t queueId);


/**************************************************************************//**
 @Function      SHARED_MEMORY_ReadConf

 @Description   This routine is used to return received data back to the shared
                memory.

 @Param[in]     h_SharedMemory  - Shared memory handle.
 @Param[in]     queueId         - Queue number from which the data was received
                                  as provided to f_RxStore callback.
 @Param[in]     p_Data          - Pointer to data.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error SHARED_MEMORY_ReadConf(t_Handle  h_SharedMemory,
                               uint32_t  queueId,
                               uint8_t  *p_Data);

/**************************************************************************//*
 @Function      SHARED_MEMORY_Interrupt

 @Description   Shared memory interrupt handle.

                This function is used to handle shared memory interrupts
                when operating in interrupt mode.

 @Param[in]     h_SharedMemory  - Shared memory handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SHARED_MEMORY_Interrupt(t_Handle h_SharedMemory);

#endif /* __SHARED_MEMORY_EXT_H */


