/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/***************************************************************************//**
@File          frame_ext.h

@Description   External Frame Data Structures And Definitions
*//***************************************************************************/

#ifndef __FRAME_EXT__
#define __FRAME_EXT__

#include "std_ext.h"
#include "list_ext.h"
#include "itg_common_ext.h"
#include "pg_ext.h"

/**************************************************************************//**
@Group          ExternalFrame

@Description    External frame defines, structures and functions

@{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Frame Defines
*//***************************************************************************/
#define POLYNOMIAL                          0x04c11db7L /* Standard CRC-32 polynomial */
#define NUM_OF_VTAGS                        6           /* Total number of VTAGS supported */
#define _BIG_ENDIAN_                        1           /* */
#define CONVERT_ENDIANNESS                  1           /* Convert endianness during serialize */
#define KEEP_ENDIANNESS                     0           /* Donot convert endianness during serialize */
#define RECURSIVE                           1           /* Recursive free */
#define NON_RECURSIVE                       0           /* Non-recursive free */

#define ENET_TCI_FIELD_SIZE                 2           /* Ethernet TCI field size */
#define ENET_QTAG_FIELD_SIZE                4           /* Ethernet QTAG field size */
#define ENET_ETYPE_FIELD_SIZE               2           /* Ethernet type field size */
#define ENET_BARE_HEADER_SIZE               2 * ENET_NUM_OCTETS_PER_ADDRESS + \
                                            ENET_ETYPE_FIELD_SIZE /* Ethernet bare header size */
#define SNAP_FIELD_SIZE                     5           /* SNAP field size */
#define NLPID_FIELD_SIZE                    1           /* NLPID field size */
#define LLC_FIELD_SIZE                      3           /* LLC field size */

#define PPP_PID_LCP                         0xc021      /* PID indicating an LCP frame*/
#define PPP_PID_IP                          0x0021      /* PID indicating an IPoPPP frame*/
#define PPP_PID_MLMC                        0x003d      /* PID indicating a MULTILINK frame*/
#define PPP_PID_MUX                         0x0059      /* PID indicating a MUX frame*/
#define PPP_PID_FIELD_SIZE                  2           /* PID field size */
#define PCDID_FIELD_SIZE                    1           /* PCD id field size */
#define PPPoE_VERTYPE_SIZE                  1           /* PPPoE version type size */
#define PPPoE_CODE_SIZE                     1           /* PPPoE code size */
#define PPPoE_SESSIONID_SIZE                2           /* PPPoE session id size */
#define PPPoE_LENGTH_SIZE                   2           /* PPPoE length size */
#define PPP_SRCPORT_FIELD_SIZE              2           /* PPP source port field size */
#define PPPoE_HEADER_SIZE                   PPPoE_VERTYPE_SIZE +   \
                                            PPPoE_CODE_SIZE +      \
                                            PPPoE_SESSIONID_SIZE + \
                                            PPPoE_LENGTH_SIZE /* PPPoE header size */

#define FIXED_HEADER_SIZE_PPPoE             6           /* PPPoE fixed header size */

#define IPv4_NUM_OCTETS_PER_ADDRESS         4           /* Number of octets per IPv4 address */
#define IPv4_PTYPE_FIELD_SIZE               1           /* IPv4 Protocol type field size */
#define IPv4_TOS_FIELD_SIZE                 1           /* IPv4 TOS field size */
#define IPv4_HEADER_SIZE                    20          /* IPv4 header size */
#define MASK_IPV4_HDR_LEN                   0x0F        /* Allow only last 4 bits in the byte*/

#define TCP_PORT_FIELD_SIZE                 2           /* TCP port field size */
#define TCP_FLAGS_FIELD_SIZE                1           /* TCP flags field size */
#define UDP_HEADER_SIZE                     8           /* UDP header size */

#define AUTO_FIELD_SIZE                     0           /* Auto field size */
#define HEADER_TYPE_ANY                     e_HEADER_TYPE_NONE /* */
#define ALL_HEADER_FIELDS                   0xFFFFFFFF  /* Flag to check all header fields */
#define NEXT_HEADER_TYPE                    0           /* Flag to check the next header */
#define VALID_FIELDS                        1           /* Flag to check the header's valid field */

/**************************************************************************//**
 @Description       Enumerations
*//***************************************************************************/
typedef enum {
    INSERT_BEFORE,
    INSERT_AFTER
}e_InsertionPrefference;

/**************************************************************************//**
 @Description   ProtocolInfo Structure
*//***************************************************************************/
typedef struct {
    t_Handle   h_TrafficIphc;    /**< Handle to Iphc */
} t_ProtocolInfo;

/**************************************************************************//**
 @Description   Functions
*//***************************************************************************/
/**************************************************************************//**
 @Description   CRC/Checksum Functions
*//***************************************************************************/
uint32_t FRAME_Compute32BitCRC(uint32_t crcAccum, uint8_t *pBuffer, uint32_t size);
uint16_t FRAME_Compute16BitCRC(uint16_t crcAccum, uint8_t *pBuffer, uint32_t size);
uint8_t  FRAME_ComputeHEC(uint8_t *p_Buffer, uint32_t size);

/**************************************************************************//**
 @Function      FRAME_Config

 @Description   Creates a new empty frame (no payload and no header)

 @Param[in]     params - NULL

 @Return        Frame handle on success, NULL on failure
*//***************************************************************************/
t_Handle FRAME_Config(t_Handle params);

/**************************************************************************//**
 @Function      FRAME_Free

 @Description   Destroys the frame

 @Param[in]     h_Frame - Handle to the frame

 @Return        E_OK on success
*//***************************************************************************/
t_Error FRAME_Free(t_Handle h_Frame);

/**************************************************************************//**
 @Function      FRAME_Init

 @Description   Updates the header dependencies throughout the frame
                (must be called before frame serialization)

 @Param[in]     h_Frame -  Handle to the frame

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_Init(t_Handle h_Frame);

/**************************************************************************//**
 @Function      FRAME_Serialize

 @Description   Serializes the actual frame (as it would be streamed over
                physical media) into the provided buffer (limited by size)

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     pBuffer - Handle to the buffer
 @Param[in]     pSize - Size of the buffer

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error  FRAME_Serialize(t_Handle h_Frame, uint8_t **pBuffer, uint32_t* pSize);

/**************************************************************************//**
 @Function      FRAME_ConfigAddHeader

 @Description   Adds on the header stack of the frame generator object a new
                header identified by a element in e_HeaderID enumeration type

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     headerID - Header type
 @Param[in]     h_NewHeader - Handle to the header to be added

 @Return        Header handle on success, NULL on failure
*//***************************************************************************/
t_Handle FRAME_ConfigAddHeader(t_Handle h_Frame, e_PgHeaderType headerID, t_Handle h_NewHeader);

/**************************************************************************//**
 @Function      FRAME_ConfigInsertHeader

 @Description   Inserts a new header in the h_Frame after or before the
                specified h_Header

 @Param[in]     h_Frame - Handle to the input frame
 @Param[in]     flags - Insertion position
 @Param[in]     h_Header - Handle to the header
 @Param[in]     headerID - Header type
 @Param[in]     h_NewHeader - Handle to the header to be inserted

 @Return        Header handle on success, NULL on failure
*//***************************************************************************/
t_Handle FRAME_ConfigInsertHeader(t_Handle h_Frame, e_InsertionPrefference flags, t_Handle h_Header,  e_PgHeaderType headerID, t_Handle h_NewHeader);

/**************************************************************************//**
 @Function      FRAME_ConfigRemoveHeader

 @Description   Removes a header from the frame

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_ConfigRemoveHeader(t_Handle h_Frame, t_Handle h_Header);

/**************************************************************************//**
 @Function      FRAME_InsertHeader

 @Description   Inserts a new header in the h_Frame after or before the
                specified h_Header

 @Param[in]     flags       - Insertion position
 @Param[in]     h_Header    - Handle to the existing header
                              in the frame as per flag information
 @Param[in]     h_NewHeader - Handle to the header to be inserted

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
 t_Error FRAME_InsertHeader(e_InsertionPrefference flags, t_Handle h_Header, t_Handle h_NewHeader);

/**************************************************************************//**
 @Function      FRAME_SetHeaderField

 @Description   Configures a specific field in the header described by the
                h_Header handle the type of the new value is dependent of the
                field that is set

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[in]     fieldID - Field identifier
 @Param[in]     h_Value - Handle to the field value
 @Param[in]     size - Field size

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_SetHeaderField(t_Handle h_Frame, t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size);

/**************************************************************************//**
 @Function      FRAME_GetHeaderField

 @Description   Retrieves the value of a specific field in the specified heade

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[in]     fieldID - Field identifier
 @Param[out]    h_Value - Handle to the field value
 @Param[in]     size - Field size

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_GetHeaderField(t_Handle h_Frame, t_Handle h_Header, uint32_t fieldID, t_Handle h_Value, uint32_t size);

/**************************************************************************//**
 @Function      FRAME_ConfigHeader

 @Description   Configures the header described by the h_Header handle.
                the value provided is a header "wrapper" : contains the actual
                header an an flags memeber specifying the fields to be
                initialized with values taken from this header

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header to be configured
 @Param[in]     hParams - Handle to the header
 @Param[in]     fieldIDs - Field identifiers

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_ConfigHeader(t_Handle h_Frame, t_Handle h_Header, t_Handle hParams, uint32_t fieldIDs);

/**************************************************************************//**
 @Function      FRAME_ConfigureHeader

 @Description   Configures a header

                This function creates the new header and configures the newly
                created header as described by the h_Header handle

 @Param[in]     headerID        - Header type
 @Param[in]     h_Header        - Handle to the header to be configured
 @Param[in]     p_ProtocolInfo  - Pointer to protocol info structure

 @Return        Header handle on success, NULL on failure
*//***************************************************************************/
t_Handle FRAME_ConfigureHeader(e_PgHeaderType headerID, t_Handle h_Header, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      FRAME_Free_Header

 @Description   Destroys the frame header

 @Param[in]     h_Frame - Handle to the frame header created by FRAME_ConfigureHeader

 @Return        E_OK on success
*//***************************************************************************/
t_Error FRAME_Free_Header(t_Handle h_Frame);

/**************************************************************************//**
 @Function      FRAME_CloneFrame

 @Description   Copies the source frame to the frame, all the settings on the
                frame are lost and replaced by those on the source frame

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_SourceFrame - Handle to the source frame

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_CloneFrame(t_Handle h_Frame, t_Handle h_SourceFrame );

/**************************************************************************//**
 @Function      FRAME_FindAndSetHeaderField

 @Description   Searches for the header of the type sepecified by
                headerID and the headerIndex (the n'th header of type x) and
                configures the specified field with the specified value of
                the provided type

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     headerID - Header type
 @Param[in]     headerIndex - Header index
 @Param[in]     fieldID - Field identifier
 @Param[in]     h_Value - Handle to the field value
 @Param[in]     size - Field size

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_FindAndSetHeaderField(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex, uint32_t fieldID, t_Handle h_Value, uint32_t size);

/**************************************************************************//**
 @Function      FRAME_FindAndGetHeaderField

 @Description   Searches for the header of the type sepecified by
                headerID and the headerIndex (the n'th header of type x) and
                returnes the specified field with the specified value of the
                provided type

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     headerID - Header type
 @Param[in]     headerIndex - Header index
 @Param[in]     fieldID - Field identifier
 @Param[out]    h_Value - Handle to the field value
 @Param[in]     size - Field size

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_FindAndGetHeaderField(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex, uint32_t fieldID, t_Handle h_Value, uint32_t size);

/**************************************************************************//**
 @Function      FRAME_ConfigHeaderAuto

 @Description   Sets a dynamic field to be computed automatic or be user specified

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[in]     fieldID - Field identifier
 @Param[in]     bSetAuto - Set or clear the auto flag

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_ConfigHeaderAuto(t_Handle h_Frame, t_Handle h_Header, uint32_t fieldID, bool bSetAuto);

/**************************************************************************//**
 @Function      FRAME_GetNextHeaderByType

 @Description   Returns the header that follows h_Header in the frame encapsulation

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[in]     headerID - Header type

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_GetNextHeaderByType(t_Handle h_Frame, t_Handle h_Header, e_PgHeaderType headerID);

/**************************************************************************//**
 @Function      FRAME_GetPrevHeaderByType

 @Description   Returns the header that precedes h_Header in the frame encapsulation

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[in]     headerID - Header type

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_GetPrevHeaderByType(t_Handle h_Frame, t_Handle h_Header, e_PgHeaderType headerID);

/**************************************************************************//**
 @Function      FRAME_GetPrevHeader

 @Description   Returns the header that precedes h_Header in the frame encapsulation

 @Param[in]     h_Header - Handle to the header

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_GetPrevHeader(t_Handle h_Header);

/**************************************************************************//**
 @Function      FRAME_GetInputHeader

 @Description   Returns the header of provided type in the input frame

 @Param[in]     headerID        - Header type
 @Param[in]     h_InputFrame    - Handle to the input frame

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_GetInputHeader(e_PgHeaderType headerID, t_Handle h_InputFrame);

/**************************************************************************//**
 @Function      FRAME_GetHeaderByTypeAndIndex

 @Description   Returns the n'th header of provided type that followes h_Header
                in the frame encapsulation

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     headerID - Header type
 @Param[in]     headerIndex - Header index

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_GetHeaderByTypeAndIndex(t_Handle h_Frame, e_PgHeaderType headerID, uint8_t headerIndex);

/**************************************************************************//**
 @Function      FRAME_GetHeaderSizeByType

 @Description   Returns the size of the specified headerId

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     headerID - Header type

 @Return        Header size
*//***************************************************************************/
uint32_t FRAME_GetHeaderSizeByType(t_Handle h_Frame, e_PgHeaderType headerID);

/**************************************************************************//**
 @Function      FRAME_GetHeaderType

 @Description   Returns the h_Header's type of header

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header

 @Return        Header type
*//***************************************************************************/
e_PgHeaderType FRAME_GetHeaderType(t_Handle h_Frame, t_Handle h_Header);

/**************************************************************************//**
 @Function      FRAME_GetFrameSize

 @Description   Gets the size of the frame starting from the specified header

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header

 @Return        Frame size
*//***************************************************************************/
uint32_t FRAME_GetFrameSize(t_Handle h_Frame, t_Handle h_Header);

/**************************************************************************//**
 @Function      FRAME_GetPayLoadOffest

 @Description   Gets the payload offset from the frame

 @Param[in]     h_Frame - Handle to the frame

 @Return        Payload offset
*//***************************************************************************/
uint32_t FRAME_GetPayLoadOffest(t_Handle h_Frame);

/**************************************************************************//**
 @Function      FRAME_GetHeaderError

 @Description   Gets the error from the header

 @Param[in]     h_Frame - Handle to the frame
 @Param[in]     h_Header - Handle to the header
 @Param[out]    h_Error - Handle to the error

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Error FRAME_GetHeaderError(t_Handle h_Frame, t_Handle h_Header, t_Handle h_Error);

/**************************************************************************//**
 @Function      FRAME_Configure

 @Description   Configures the frame

 @Param[in]     h_Frame        - Handle to the frame
 @Param[in]     h_InputFrame   - Handle to the input frame
 @Param[in]     p_ProtocolInfo - Pointer to protocol info structure

 @Return        E_OK on success, Appropriate error type on failure
*//***************************************************************************/
t_Handle FRAME_Configure(t_Handle h_Frame,t_Handle h_InputFrame, t_ProtocolInfo *p_ProtocolInfo);

/**************************************************************************//**
 @Function      FRAME_BufferInsert

 @Description   Supports for insert header manipulation

 @Param[in]     p_Buffer - Handle to the buffer
 @Param[in]     bufferSize - Handle to the header
 @Param[in]     size - Size of the data to be inserted
 @Param[in]     offset - Offset from where to insert

 @Return        E_OK
*//***************************************************************************/
t_Error FRAME_BufferInsert(uint8_t **p_Buffer, uint32_t *bufferSize, uint8_t* p_NewBuffer, uint32_t newBufferSize, uint32_t offset, bool updateLength);

/**************************************************************************//**
 @Function      FRAME_BufferReplace

 @Description   Supports for replace header manipulation

 @Param[in]     p_Buffer - Handle to the buffer
 @Param[in]     bufferSize - Handle to the header
 @Param[in]     size - Size of the data to be replaced
 @Param[in]     offset - Offset from where to replace

 @Return        E_OK
*//***************************************************************************/
t_Error FRAME_BufferReplace(uint8_t **p_Buffer, uint32_t *bufferSize, uint8_t* p_NewBuffer, uint32_t newBufferSize, uint32_t offset);

/**************************************************************************//**
 @Function      FRAME_BufferRemove

 @Description   Supports for remove header manipulation

 @Param[in]     p_Buffer - Handle to the buffer
 @Param[in]     bufferSize - Handle to the header
 @Param[in]     size - Size of the data to be removed
 @Param[in]     offset - Offset from where to remove

 @Return        E_OK
*//***************************************************************************/
t_Error FRAME_BufferRemove(uint8_t **p_Buffer, uint32_t *bufferSize, uint32_t size, uint32_t offset);

/** @} */ /* end of ExternalFrame group */

#endif /* __FRAME_EXT__ */

/******************************************************************************
**                                                                           **
**   END                                                                     **
**                                                                           **
** ***************************************************************************/

