/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          pg.c

  @Description   ITG Packet Generator general services
*//***************************************************************************/
#include "std_ext.h"
#include "stdlib_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "mem_ext.h"
#include "buf_ext.h"
#include "mb_frame_ext.h"
#include "frame_ext.h"
#include "fmanip_ext.h"
#include "iphc_manip.h"
#include "itg_common.h"
#include "traffic_generator.h"
#include "traffic_analyzer.h"
#include "pg.h"


/**************************************************************************//**
  @Group          PG

  @Description    PG Functions

  @{
*//***************************************************************************/
static void AddFrameToList(t_PgFrame *p_PgFrame, t_List *p_List)
{
     uint32_t             intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_PgFrame->frmsNode, p_List);
     XX_RestoreAllIntr(intFlags);
}

static t_PgFrame* RemoveFrameFromList(t_List *p_List)
{
     t_PgFrame            *p_PgFrame = NULL;
     uint32_t             intFlags;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_List))
     {
         p_PgFrame = PG_FRAME_OBJECT(p_List->p_Next);
         LIST_DelAndInit(&p_PgFrame->frmsNode);
     }

     XX_RestoreAllIntr(intFlags);
     return p_PgFrame;
}

static t_Handle CreateSerializedFrame(t_Handle          h_FrameGen,
                                      t_BufMemAccess    *p_BufMemAccess,
                                      uint32_t          dataByteAlignment)
{
     t_Handle   h_Frame     = NULL;         /* Frame handle */
     t_Handle   h_Buff      = NULL;         /* Buffer handle */
     uint8_t    *p_Data     = NULL;         /* Data pointer */
     uint16_t   count       = 0;            /* Number of bytes appended to the frame */
     uint32_t   size        = 0;            /* Frame size */
     uint8_t    *p_FrameBuffer = NULL;      /* Pointer to the serialized frame */

     uint32_t   dataAddr    = 0;            /* Data buffer address */
     uint32_t   resByte     = 0;            /* Bytes to reserve in the data buffer */
     t_Error    rc          = E_OK;

     if(!h_FrameGen)
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("h_FrameGen"));
        return NULL;
     }

     /* Create the data pointer */
     p_Data = p_BufMemAccess->f_GetMem(p_BufMemAccess->h_MemMng);
     if (!p_Data)
     {
         REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("No memory for Input Data"));
         return NULL;
     }

     /* Create the buffer */
     h_Buff = B_New(p_BufMemAccess,
                    MEM_GetSize(p_BufMemAccess->h_MemMng),
                    MEM_GetPrefixSize(p_BufMemAccess->h_MemMng),
                    p_Data,
                    0);

     if (!h_Buff)
     {
         REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("Can't create a new buffer"));
         return NULL;
     }

     /* Create the frame */
     h_Frame = F_New(h_Buff);

     if (!h_Frame)
     {
         REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("Can't create a new Frame"));
         return NULL;
     }

     /* Get the frame size and allocate the buffer to serialize the frame */
     size  = FRAME_GetFrameSize(h_FrameGen, h_FrameGen);
     if(size == 0)
     {
         REPORT_ERROR(MINOR, E_INVALID_VALUE, ("Frame Size = %d",size));
         return NULL;
     }
     p_FrameBuffer = XX_Malloc(size);
     if (!p_FrameBuffer)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("Failed to allocate data buffer!"));
         return NULL;
     }
     memset(p_FrameBuffer, 0, size);

     /* Serialize the frame */
     rc = FRAME_Serialize(h_FrameGen, &p_FrameBuffer, &size);
     if(rc != E_OK)
     {
         XX_Free(p_FrameBuffer);
         REPORT_ERROR(MAJOR, rc, ("Cannot serialize the Tx Frame!"));
         return NULL;
     }

     /* Get the first data byte alignment value */
     if (dataByteAlignment == 0)
     {
         /* User didn't specify alignment - generate random alignment */
         dataByteAlignment = (uint32_t)(rand()%4) + 1;
     }

     dataAddr = (uint32_t)p_Data;

     while(((dataAddr + resByte) % dataByteAlignment) != 0)
     {
         resByte++;
     }

     /* Reserve room for first data byte alignment */
     B_ReserveHeadRoom(h_Buff, (uint16_t)(resByte));

     /* Create the frame to be transmitted. If the frame size exceeds the buffer size,
     multiple buffers will be created */
     count = F_AppendData(h_Frame, p_FrameBuffer, (uint16_t)size);
     if(count != size)
     {
         XX_Free(p_FrameBuffer);
         REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Invalid length Returned by F_AppendData!"));
         return NULL;
     }

     XX_Free(p_FrameBuffer);

     return h_Frame;
}

static t_Error PopulateFrameEncapInfo(t_Handle h_FrameGen,t_TrafficFrameInfo *p_FrameInfo)
{
    t_Handle                     h_Header           = NULL;                 /* Header handle */
    e_PgHeaderType               headerType         = e_HEADER_TYPE_NONE;   /* Header type */
    e_PgHeaderType               *p_FrameHeaderType = NULL;                 /* Pointer to frame encapsulation */
    e_PgHeaderType               frameEncapsulation[MAX_NUM_OF_ENCAPSULATIONS]; /* Holds the frame encapsulation */

    uint8_t                      hdrIndex   = 0;        /* Header index */
    uint32_t                     errorType  = 0;        /* Holds the error types */
    uint32_t                     error      = 0;        /* Set if there is any error in frame header */
    t_Error                      rc         = E_OK;     /* Return value */

    memset(frameEncapsulation, e_HEADER_TYPE_NONE, sizeof(frameEncapsulation));

 /* Extract the header types and error types of TG/TA frame for statistics calculation */

     if(!h_FrameGen)
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("h_FrameGen"));
        return E_INVALID_HANDLE;
     }

     h_Header = h_FrameGen;
     do
     {
         /* Get the header type */
         headerType = FRAME_GetHeaderType(h_FrameGen,h_Header);

         /* Populate the frame encapsulation */
         frameEncapsulation[hdrIndex++] = headerType;

         /* Get the error flag for the header */
         rc =  FRAME_GetHeaderError(h_FrameGen,h_Header,&error);
         if(rc != E_OK)
         {
            REPORT_ERROR(MAJOR, rc, ("Could not get FrameHeader error"));
            return rc;
         }

         /* If the error flag is set, set the error type */
         if(error)
         {
             errorType |= error;
             error = 0;
         }

         /* Get the next header in the frame */
         h_Header = FRAME_GetNextHeaderByType(h_FrameGen,h_Header,HEADER_TYPE_ANY);

     } while(h_Header);

     /* Allocate memory to hold the header types */
     p_FrameHeaderType = (e_PgHeaderType *)XX_Malloc((hdrIndex+1) * sizeof(e_PgHeaderType));
     if (!p_FrameHeaderType)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_FrameHeaderType"));
         return E_NO_MEMORY;
     }
     memset(p_FrameHeaderType,0,(hdrIndex+1) * sizeof(e_PgHeaderType));

     memcpy(p_FrameHeaderType,frameEncapsulation,(hdrIndex+1) * sizeof(e_PgHeaderType));

     /* Configure the header information in the frame to use during statistics calculation */
     p_FrameInfo->h_FrameHeaderType = (t_Handle)p_FrameHeaderType;

     if(errorType)
     {
         /* Configure the error information in the frame to use during statistics calculation */
         p_FrameInfo->frameErrorType = errorType;
     }

    return E_OK;
}


static t_Error InitPayloadInfo(t_TrafficFrameInfo *p_FrameInfo)
{
    t_Handle                     h_Header       = NULL;                 /* Header handle */
    e_PgHeaderType               headerType     = e_HEADER_TYPE_NONE;   /* Header type */
    uint32_t                     headerLng      = 0;                    /* Header length*/
    uint32_t                     payloadLength  = 0;                    /* Payload length */
    t_Error                      rc             = E_OK;                 /* Return value */

     /* Set the payload offset in the frame */
    headerLng = FRAME_GetPayLoadOffest(p_FrameInfo->h_FrameGen);
    p_FrameInfo->payloadOffset = headerLng;

    if(!p_FrameInfo->h_FrameGen)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_FrameInfo->h_FrameGen"));
        return E_INVALID_HANDLE;
    }

    h_Header = p_FrameInfo->h_FrameGen;
    do
    {
        /* Get the header type */
        headerType = FRAME_GetHeaderType(p_FrameInfo->h_FrameGen,h_Header);

        if((headerType == e_HEADER_TYPE_PAYLOAD) || (headerType == e_HEADER_TYPE_RAW_DATA))
        {
            rc = FRAME_GetHeaderField(p_FrameInfo->h_FrameGen, h_Header, e_FIELD_PAYLOAD_SIZE, &payloadLength, sizeof(payloadLength));
            if(rc != E_OK)
            {
                REPORT_ERROR(MAJOR, rc, ("Could not get the payload length"));
                return rc;
            }
        }
        /* Get the next header in the frame */
        h_Header = FRAME_GetNextHeaderByType(p_FrameInfo->h_FrameGen, h_Header, HEADER_TYPE_ANY);

    } while(h_Header);

    /* Set the payload length in the frame */
    p_FrameInfo->payloadLength = payloadLength;

    return E_OK;
}


static t_Handle InitAndCreateFrameObject(t_TrafficFrameInfo *p_FrameInfo)
{
    t_Handle   h_Frame  = NULL;
    t_Error    rc       = E_OK;         /* Return value */

    if(!p_FrameInfo->h_FrameGen)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_FrameInfo->h_FrameGen"));
        return NULL;
    }

     /* Initialize the frame */
    rc = FRAME_Init(p_FrameInfo->h_FrameGen);
    if (rc != E_OK)
    {
        REPORT_ERROR(MAJOR, rc, ("Could not initialize the tx frame template in Golden pattern"));
        return NULL;
    }

    h_Frame = CreateSerializedFrame(p_FrameInfo->h_FrameGen, (t_BufMemAccess*)p_FrameInfo->h_BufMemAccess, p_FrameInfo->dataByteAlignment);
    if (!h_Frame)
    {
        REPORT_ERROR(MAJOR, E_INVALID_FRAME, ("Could not create the CreateFrame And GoldenP object"));
        return NULL;
    }

    rc = InitPayloadInfo(p_FrameInfo);
    if (rc != E_OK)
    {
        REPORT_ERROR(MAJOR, rc, ("Could not Initialize the Payload Infomation in Frame"));
        return NULL;
    }

    return h_Frame;
}

/**************************************************************************//*
 @Function      CreateFrame

 @Description   Creates frame list with the frame descriptor set passed as parameter.
                Invokes user call back function and stores h_FrameGen information
                if the GOLDENPATTERN/CLONE_TA_LIST_FROM_TG_LIST is enabled by user.

 @Param[in]     h_FrameSettings   - Handle to the frame descriptor set
 @Param[in]     frameId           - Frame index of the frame descriptor set
 @Param[in]     frameSeqNum       - Sequence number of the frame. They are set in the last four
                                    bytes of the frame after payload / RAW DATA field.

 @Return        h_Frame on SUCCESS or NULL and report appropriate error type on failure

 @Cautions      None
*//***************************************************************************/

static t_Handle CreateFrame(t_Handle            h_FrameDescriptor,
                             uint32_t           frameId,
                             uint32_t           frameSeqNum,
                             uint32_t           frameNum,
                             t_PgUserManipParam *p_UserManip,
                             t_BufMemAccess     *p_BufMemAccess,
                             uint32_t           dataByteAlignment,
                             t_Handle           h_TrafficIphc)
{

     t_Handle                    h_FrameGen     = NULL;     /* Frame list handle */
     t_Error                     rc             = E_OK;     /* Return value */
     t_Handle                    h_Frame        = NULL;     /* Frame handle */
     e_PgHeaderType              headerID       = e_HEADER_TYPE_NONE;
     t_TrafficFrameInfo          *p_FrameInfo   = NULL;
     uint32_t                    numOfManip     = 0;
     t_ProtocolInfo              *p_ProtocolInfo;

     t_PgFrameSetting *p_FrameDescriptor = &((t_PgFrameSetting *)h_FrameDescriptor)[frameId];
     t_PgFrameCallBackMapSettings *p_PgFrameCallBackMapSettings = NULL;

     p_ProtocolInfo = (t_ProtocolInfo *)XX_Malloc(sizeof(t_ProtocolInfo));
     if (!p_ProtocolInfo)
     {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_ProtocolInfo!"));
        return NULL;
     }

     memset(p_ProtocolInfo, 0, sizeof(t_ProtocolInfo));

     if(h_TrafficIphc)
        p_ProtocolInfo->h_TrafficIphc = h_TrafficIphc;

     /* Allocate memory to hold the frame informations */
     p_FrameInfo = (t_TrafficFrameInfo *)XX_Malloc(sizeof(t_TrafficFrameInfo));
     if (!p_FrameInfo)
     {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_FrameInfo!"));
        return NULL;
     }
     memset(p_FrameInfo, 0, sizeof(t_TrafficFrameInfo));

     /* Create the empty frame */
     h_FrameGen = FRAME_Config(NULL);
     if (!h_FrameGen)
     {
         REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Could not create the frame"));
         return NULL;
     }

     /* Configure the frame headers */
     if (!FRAME_Configure(h_FrameGen, p_FrameDescriptor, p_ProtocolInfo))
     {
         REPORT_ERROR(MAJOR, E_NOT_AVAILABLE, ("Could not configure the frame"));
         return NULL;
     }

     /* Configure the frame sequence number */
     if(frameSeqNum)
     {
         if(FRAME_GetNextHeaderByType(h_FrameGen, h_FrameGen, e_HEADER_TYPE_PAYLOAD))
            headerID = e_HEADER_TYPE_PAYLOAD;
         else
            headerID = e_HEADER_TYPE_RAW_DATA;

         rc = FRAME_FindAndSetHeaderField(h_FrameGen,headerID, 1, e_FIELD_SEQ_NUM, (t_Handle)&frameSeqNum, sizeof(frameSeqNum));
         if(rc != E_OK)
         {
             REPORT_ERROR(MAJOR, rc, ("Could not configure the frame sequence number"));
             return NULL;
         }
     }

     /* Configure the data memory mng parameters, data byte alignment data
        frame layout information, frame index data in the frame */
     p_FrameInfo->h_BufMemAccess    = (t_Handle)p_BufMemAccess;
     p_FrameInfo->dataByteAlignment = dataByteAlignment;
     p_FrameInfo->h_FrameGen        = h_FrameGen;
     p_FrameInfo->frameId           = frameId;
     p_FrameInfo->h_ProtocolInfo    = p_ProtocolInfo;

     /* Initialize the frame and create the frame for transmission*/
     h_Frame = InitAndCreateFrameObject(p_FrameInfo);

     /* Configure the pointer to t_TrafficFrameInfo structure in the frame */
     F_SetFrameInfo(h_Frame, p_FrameInfo);

     /* Invoke the user defined callback routine if user manipulation is enabled for the frame */

     if(p_UserManip && (p_UserManip->numOfUserManipFrame != 0))
     {
         p_PgFrameCallBackMapSettings = (t_PgFrameCallBackMapSettings *)(p_UserManip->p_PgFrameCallBackMapSettings);

         for(numOfManip = 0; numOfManip < p_UserManip->numOfUserManipFrame; numOfManip++)
         {
            if(frameId == p_PgFrameCallBackMapSettings[numOfManip].frameId && p_PgFrameCallBackMapSettings[numOfManip].f_UserFrameConfig)
            {
                p_PgFrameCallBackMapSettings[numOfManip].f_UserFrameConfig(h_Frame, p_UserManip->h_UserManipParam, frameId, frameNum);
                break;
            }
         }
     }

     rc = PopulateFrameEncapInfo(p_FrameInfo->h_FrameGen, p_FrameInfo);

     if (rc != E_OK)
     {
        REPORT_ERROR(MAJOR, rc, ("Could not populate the frame encapsulation infos in frame"));
        return NULL;
     }

     /* Configure the pointer to t_TrafficFrameInfo structure in the frame */
     F_SetFrameInfo(h_Frame, p_FrameInfo);

     //FRAME_Free(p_FrameInfo->h_FrameGen);

     return h_Frame;
}

static void UpdateFrame(t_Handle h_Frame, t_Handle h_OrigFrame)
{
    uint32_t        numOfBuffers    = 0;        /* Number of buffers */
    uint32_t        i               = 0;
    t_Handle        h_Buff          = NULL;     /* Handle to the buffer */

    /* Get the number of buffers in the original frame */
    numOfBuffers = F_NumOfBuffers(h_OrigFrame);

    /* Remove all the buffers in the original frame */
    for(i = 0; i < numOfBuffers; i++)
    {
        h_Buff = F_RemoveFirstBuffer(h_OrigFrame);
        B_Delete(h_Buff,TRUE);
    }

    /* Get the number of buffers in the manipulated frame */
    numOfBuffers = F_NumOfBuffers(h_Frame);

    /* Remove all the buffers from the manipulated frame and prepend to the original frame*/
    for(i = 0; i < numOfBuffers; i++)
    {
        h_Buff = F_RemoveFirstBuffer(h_Frame);
        F_AppendBuffer(h_OrigFrame,h_Buff);
    }

    /* Delete the manipulated frame */
    F_Delete(h_Frame,TRUE);
}

static t_Error UpdateFrameGen(uint32_t operationType, t_Handle h_Frame, e_PgHeaderType opHeaderType, t_Handle h_OpHeader, uint8_t headerIndex)
{
    t_TrafficFrameInfo      *p_FrameInfo        = NULL;         /* Pointer to frame information */
    t_Handle                h_Header            = NULL;         /* Handle to header in frame gen */
    t_Handle                h_PrevHeader        = NULL;         /* Handle to the previous header */
    t_Error                 errorCode           = E_OK;         /* Return value */
    e_InsertionPrefference  insertFlag;

    /* Get all frame information */
    p_FrameInfo = F_GetFrameInfo(h_Frame);

    /* Insert the new header as the first in frameGen */
    if(!opHeaderType)
    {
        insertFlag = INSERT_BEFORE;
        h_Header = p_FrameInfo->h_FrameGen;
    }
    else
    {
        insertFlag = INSERT_AFTER;
        /* Find the header in the frameGen - This piece will be hit by Header Insert/Remove/Replace functions */
        h_Header = FRAME_GetHeaderByTypeAndIndex(p_FrameInfo->h_FrameGen, opHeaderType, headerIndex);
        if (!h_Header)
        {
            RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("Header not found"));
        }
    }

    if(operationType == e_REPLACE_HEADER)
    {
        /* Helps to insert a replace header after prev header (acts like insertAfterHeader parameter) */
        h_PrevHeader = FRAME_GetPrevHeader(h_Header);

        if(h_PrevHeader == NULL)
        {
            /* Consider the h_Header as the reference header -
            helps to insert a replace header before reference header (acts like insertBeforeHeader parameter) */
            insertFlag = INSERT_BEFORE;
        }
    }

    if(operationType == e_REMOVE_HEADER || operationType == e_REPLACE_HEADER)
    {
        /* Remove the specified header from the frameGen*/
        errorCode = FRAME_ConfigRemoveHeader(p_FrameInfo->h_FrameGen, h_Header);
        if(errorCode != E_OK)
        {
            RETURN_ERROR(MAJOR, errorCode, ("Removal of the specified header from the internal frame failed!"));
        }
        /* Assign the prev header as the current header -
        helps to insert a replace header after prev header (acts like insertAfterHeader parameter) */
        if(h_PrevHeader)
        {
            h_Header = h_PrevHeader;
        }
    }

    if(operationType == e_INSERT_HEADER || operationType == e_REPLACE_HEADER)
    {
        /* Insert the new header after the specified header in the frameGen */
        errorCode = FRAME_InsertHeader(insertFlag, h_Header, h_OpHeader);
        if(errorCode != E_OK)
        {
            RETURN_ERROR(MAJOR, errorCode, ("Insertion of the specified header from the internal frame failed!"));
        }
        /* After inserting a new header at the start of frame gen update the frame gen handle */
        if(insertFlag == INSERT_BEFORE)
        {
            p_FrameInfo->h_FrameGen = h_OpHeader;
        }
    }

    return E_OK;
}

static t_Error HeaderFieldManipulation(t_Handle h_Frame, e_PgHeaderType headerType, t_Handle h_InputModifyHeader, uint8_t modifyOnHeaderSeqNum)
{
    t_Handle                h_OpHeader      = NULL;     /* Handle to header in frame gen */
    t_Handle                h_InputHeader   = NULL;     /* Handle to user configurated header info */
    t_Error                 rc              = E_OK;     /* Return value */
    uint32_t                fieldIDs        = 0;        /* User configured valid fileds */
    t_TrafficFrameInfo      *p_FrameInfo    = NULL;     /* Pointer to frame information */

    /* Get all frame information */
    p_FrameInfo = F_GetFrameInfo(h_Frame);

    /* Points to the header to be modified in the frameGen */
    h_OpHeader = FRAME_GetHeaderByTypeAndIndex(p_FrameInfo->h_FrameGen, headerType, modifyOnHeaderSeqNum);
    if (!h_OpHeader)
    {
        RETURN_ERROR(MAJOR, E_INVALID_HANDLE, ("Header not found"));
    }

    /* Create and Config new header node to get the validfields configured by the user */
    h_InputHeader = FRAME_ConfigureHeader(headerType, h_InputModifyHeader, p_FrameInfo->h_ProtocolInfo);

    /* Extract the valid field information from the input settings */
    rc = FRAME_GetHeaderField(NULL, h_InputHeader, VALID_FIELDS, &fieldIDs, sizeof(fieldIDs));
    if(rc != E_OK)
    {
        RETURN_ERROR(MAJOR, rc, ("Could not get the specified field"));
    }

    /* Configure the input fields in frameGen header node using the valid field infomation*/
    rc = FRAME_ConfigHeader(NULL, h_OpHeader, h_InputModifyHeader, fieldIDs);
    if(rc != E_OK)
    {
        RETURN_ERROR(MAJOR, rc, ("Failed to configure input fields in frameGen"));
    }

    FRAME_Free_Header(h_InputHeader);

    return E_OK;
}

t_PgFrame* PG_CreateFrame(void)
{
     t_PgFrame *p_PgFrame = NULL;

     p_PgFrame = (t_PgFrame *)XX_Malloc(sizeof(t_PgFrame));
     if (!p_PgFrame)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("Traffic frame!"));
         return NULL;
     }
     memset(p_PgFrame, 0 , sizeof(t_PgFrame));

     INIT_LIST(&p_PgFrame->frmsNode);

     return p_PgFrame;
}

t_Error PG_DestroyFrame(t_Handle h_Frame)
{
     t_TrafficFrameInfo *p_FrameInfo = NULL;                /* Holds the frame informations */

     if(!h_Frame)
     {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE,("h_Frame!"));
     }

     p_FrameInfo = F_GetFrameInfo(h_Frame);

     if(p_FrameInfo)
     {

        if(p_FrameInfo->h_FrameGen)
        {
            FRAME_Free(p_FrameInfo->h_FrameGen);
            p_FrameInfo->h_FrameGen = NULL;
        }

        if(p_FrameInfo->h_FrameHeaderType)
        {
            XX_Free(p_FrameInfo->h_FrameHeaderType);
            p_FrameInfo->h_FrameHeaderType = NULL;
        }

        if(p_FrameInfo->h_ProtocolInfo)
        {
            XX_Free(p_FrameInfo->h_ProtocolInfo);
            p_FrameInfo->h_ProtocolInfo = NULL;
        }
        XX_Free(p_FrameInfo);
        p_FrameInfo = NULL;
     }

     F_Delete(h_Frame, TRUE);
     return E_OK;
}

t_Handle PG_Init(t_PgFrameDescriptorSetParam *p_FrameDescriptorSettings, t_PgIphcParam *p_IphcParam)
{
    t_PacketGen                 *p_PacketGen  = NULL;
    t_PgFrameSetting            *p_FrameSet   = NULL;

    if(!p_FrameDescriptorSettings)
    {
        REPORT_ERROR(CRITICAL, E_INVALID_HANDLE , ("p_FrameDescriptorSettings"));
        return NULL;
    }

    if (!p_FrameDescriptorSettings->numOfFrameSettings)
    {
        REPORT_ERROR(CRITICAL, E_INVALID_VALUE , ("numOfFrameSettings"));
        return NULL;
    }

    p_PacketGen = (t_PacketGen *)XX_Malloc(sizeof(t_PacketGen));
    if (!p_PacketGen)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_PacketGen"));
        return NULL;
    }
    memset(p_PacketGen, 0, sizeof(t_PacketGen));

    p_FrameSet  = (t_PgFrameSetting*)XX_Malloc(sizeof(t_PgFrameSetting) * p_FrameDescriptorSettings->numOfFrameSettings);
    if (!p_FrameSet)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_FrameSet!"));
        return NULL;
    }

    memset(p_FrameSet, 0, sizeof(t_PgFrameSetting) * p_FrameDescriptorSettings->numOfFrameSettings);

    memcpy(p_FrameSet, p_FrameDescriptorSettings->p_FrameSettings,
        (sizeof(t_PgFrameSetting)* p_FrameDescriptorSettings->numOfFrameSettings));

    if(p_IphcParam)
    {
        if(p_IphcParam->numOfIphcCompressors)
        {
            p_PacketGen->h_TrafficIphc = PG_IPHC_Init(p_IphcParam);
        }
    }
    p_PacketGen->h_FrameSettings = (t_Handle)p_FrameSet;
    p_PacketGen->bufMemAccess = p_FrameDescriptorSettings->bufMemAccess;
    return (t_Handle)p_PacketGen;
}

t_Error PG_Free(t_Handle h_PacketGen)
{

    t_PacketGen *p_PacketGen = (t_PacketGen *)h_PacketGen;

    if(p_PacketGen)
    {
        if(p_PacketGen->h_FrameSettings)
        {
            XX_Free(p_PacketGen->h_FrameSettings);
        }
        if(p_PacketGen->h_TrafficIphc)
        {
            PG_IPHC_Free(p_PacketGen->h_TrafficIphc);
        }
        XX_Free(p_PacketGen);
    }

    return E_OK;
}

t_Error PG_IphcReset(t_Handle h_PacketGen)
{
    t_PacketGen *p_PacketGen = (t_PacketGen *)h_PacketGen;

    if(!p_PacketGen)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE , ("p_PacketGen"));
    }

    if(PG_IPHC_Reset(p_PacketGen->h_TrafficIphc) != E_OK)
    {
        RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("p_TrafficIphc\n"));
    }

    return E_OK;
}

t_Handle PG_GetList(t_Handle h_PgListSettings)
{
    t_PgListSettings *p_PgListSettings = (t_PgListSettings *)h_PgListSettings;

    if(!p_PgListSettings)
    {
        REPORT_ERROR(CRITICAL, E_INVALID_HANDLE , ("p_PgListSettings"));
        return NULL;
    }

    return (p_PgListSettings->p_List);
}

t_Error PG_GetNumOfListSubSet(t_Handle h_PgListSettings, uint32_t* p_NumOfFrameListSubSet)
{
    t_PgListSettings *p_PgListSettings = (t_PgListSettings *)h_PgListSettings;

    if(!p_PgListSettings)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE , ("p_PgListSettings"));
    }
    *p_NumOfFrameListSubSet = p_PgListSettings->numOfFrameListSubSet;
    return E_OK;
}

t_Handle PG_GetFrameWeights(t_Handle h_PgListSettings)
{
    t_PgListSettings *p_PgListSettings = (t_PgListSettings *)h_PgListSettings;

    if(!p_PgListSettings)
    {
        REPORT_ERROR(CRITICAL, E_INVALID_HANDLE , ("p_PgListSettings"));
        return NULL;
    }

    return (p_PgListSettings->p_PgFrameWeights);
}

t_Error PG_FragmentFrameList(t_List *p_FrameList, uint32_t ipMtu)
{
    t_PgFrame           *p_Frame, *p_FrameToRemove, *p_FragFrame;
    t_TrafficFrameInfo  *p_FrameInfo, *p_FragFrameInfo;
    uint16_t            ipPayloadSize, maxFragIpPayloadSize, fragIpFrameSize;
    uint8_t             ipV4HdrLen, ipFlags, *p_Data;
    t_Handle            h_IpHeader, h_Header, h_HeaderToRemove;
    t_Error             errorCode;
    uint32_t            fragIpPayloadSize, offset;
    t_PgPayloadType     framePayloadType;
    uint16_t            ipFragmentOffset;

    ASSERT_COND(p_FrameList);

    p_Frame     = LIST_OBJECT(LIST_FIRST(p_FrameList), t_PgFrame, frmsNode);

    p_FrameInfo = F_GetFrameInfo(p_Frame->h_Frame);

    do
    {
        h_IpHeader = FRAME_GetHeaderByTypeAndIndex(p_FrameInfo->h_FrameGen, e_HEADER_TYPE_IPv4, 1);
        if (!h_IpHeader)
        {
            /* The frame is not IP */
            REPORT_ERROR(WARNING, E_INVALID_FRAME, ("Cannot IP fragment a non-IP frame"));
            continue;
        }

        FRAME_GetHeaderField(   p_FrameInfo->h_FrameGen,
                                h_IpHeader,
                                e_FIELD_IPv4_HDR_LEN,
                                (t_Handle)&ipV4HdrLen,
                                1);
        FRAME_GetHeaderField(   p_FrameInfo->h_FrameGen,
                                h_IpHeader,
                                e_FIELD_IPv4_TOTAL_LEN,
                                (t_Handle)&ipPayloadSize,
                                2);
        ipPayloadSize -= ipV4HdrLen * 4;
        maxFragIpPayloadSize = (uint16_t)(ipMtu - (ipV4HdrLen * 4));
        /* In case that the IP fragment payload is not a multiple of 8 byte blocks, reduce it
         * to satisfy this limitation of the fragment offset field */
        if (maxFragIpPayloadSize % 8)
            maxFragIpPayloadSize &= 0xfff8;

        if (ipPayloadSize > maxFragIpPayloadSize)
        {
            /* We need to fragment. */
            ipFragmentOffset = 0;

            /* Calculate the offset of the IPv4 payload that needs to be fragmented */
            h_Header    = p_FrameInfo->h_FrameGen;
            offset      = FRAME_GetHeaderSizeByType(p_FrameInfo->h_FrameGen,
                                                    FRAME_GetHeaderType(p_FrameInfo->h_FrameGen, h_Header));
            if (FRAME_GetHeaderType(p_FrameInfo->h_FrameGen, h_Header) != e_HEADER_TYPE_IPv4)
                do
                {
                    h_Header    = FRAME_GetNextHeaderByType(p_FrameInfo->h_FrameGen,
                                                            h_Header, HEADER_TYPE_ANY);
                    offset      += FRAME_GetHeaderSizeByType(   p_FrameInfo->h_FrameGen,
                                                                FRAME_GetHeaderType(p_FrameInfo->h_FrameGen, h_Header));
                }
                while (FRAME_GetHeaderType(p_FrameInfo->h_FrameGen, h_Header) != e_HEADER_TYPE_IPv4);

            /* Do fragmentation */
            do
            {
                /* Create a new fragment frame */
                p_FragFrameInfo = (t_TrafficFrameInfo*) XX_Malloc(sizeof(t_TrafficFrameInfo));
                memset(p_FragFrameInfo, 0, sizeof(t_TrafficFrameInfo));

                p_FragFrameInfo->h_BufMemAccess     = p_FrameInfo->h_BufMemAccess;
                p_FragFrameInfo->dataByteAlignment  = p_FrameInfo->dataByteAlignment;

                p_FragFrameInfo->h_FrameGen = FRAME_Config(NULL);
                errorCode = FRAME_CloneFrame(p_FragFrameInfo->h_FrameGen, p_FrameInfo->h_FrameGen);
                if (errorCode != E_OK)
                {
                    XX_Free(p_FragFrameInfo);
                    return errorCode;
                }

                h_IpHeader = FRAME_GetHeaderByTypeAndIndex( p_FragFrameInfo->h_FrameGen,
                                                            e_HEADER_TYPE_IPv4,
                                                            sizeof(uint8_t));
                ASSERT_COND(h_IpHeader);
                h_Header = FRAME_GetNextHeaderByType(   p_FragFrameInfo->h_FrameGen,
                                                        h_IpHeader,
                                                        HEADER_TYPE_ANY);

                /* Remove all headers after IP until the payload */
                while (FRAME_GetHeaderType(p_FragFrameInfo->h_FrameGen, h_Header) != e_HEADER_TYPE_PAYLOAD)
                {
                    h_HeaderToRemove = h_Header;
                    h_Header = FRAME_GetNextHeaderByType(   p_FragFrameInfo->h_FrameGen,
                                                            h_Header,
                                                            HEADER_TYPE_ANY);
                    FRAME_ConfigRemoveHeader(p_FragFrameInfo->h_FrameGen, h_HeaderToRemove);
                }

                if (ipPayloadSize <= maxFragIpPayloadSize)
                {
                    /* Last fragment */
                    fragIpPayloadSize = (uint32_t)ipPayloadSize;
                    ipFlags = 0; /* No more fragments */
                }
                else
                {
                    fragIpPayloadSize = (uint32_t)maxFragIpPayloadSize;
                    ipFlags = 1;
                }
                FRAME_SetHeaderField(   p_FragFrameInfo->h_FrameGen,
                                        h_IpHeader,
                                        e_FIELD_IPv4_FLAG_M,
                                        (t_Handle)&ipFlags,
                                        sizeof(uint8_t));
                FRAME_SetHeaderField(   p_FragFrameInfo->h_FrameGen,
                                        h_IpHeader,
                                        e_FIELD_IPv4_OFFSET,
                                        (t_Handle)&ipFragmentOffset,
                                        sizeof(uint16_t));

                ipFragmentOffset    += (uint16_t)(maxFragIpPayloadSize >> 3);
                ipPayloadSize       -= fragIpPayloadSize;
                fragIpFrameSize     = (uint16_t)(fragIpPayloadSize + (ipV4HdrLen * 4));

                /* Update IP fragment length */
                FRAME_SetHeaderField(   p_FragFrameInfo->h_FrameGen,
                                        h_IpHeader,
                                        e_FIELD_IPv4_TOTAL_LEN,
                                        (t_Handle)&fragIpFrameSize,
                                        sizeof(uint16_t));

                /* Extract the fragment data from the original frame. */
                p_Data = (uint8_t*) XX_Malloc(fragIpPayloadSize);
                if (F_Read( p_Frame->h_Frame,
                            p_Data,
                            (uint16_t)offset,
                            (uint16_t)fragIpPayloadSize) != fragIpPayloadSize)
                {
                    XX_Free(p_Data);
                    REPORT_ERROR(MAJOR, E_INVALID_FRAME, NO_MSG);
                }
                else
                {
                    framePayloadType.type                   = e_PAYLOAD_USER;
                    framePayloadType.value.user.p_Buffer    = p_Data;
                    framePayloadType.value.user.size        = fragIpPayloadSize;

                    /* Set the buffer to the fragment frame as payload */
                    FRAME_SetHeaderField(   p_FragFrameInfo->h_FrameGen,
                                            h_Header,
                                            e_FIELD_PAYLOAD_TYPE,
                                            (t_Handle)&framePayloadType,
                                            sizeof(t_PgPayloadType));
                }
                offset += fragIpPayloadSize;

                PopulateFrameEncapInfo(p_FragFrameInfo->h_FrameGen, p_FragFrameInfo);
                InitPayloadInfo(p_FragFrameInfo);

                p_FragFrame             = PG_CreateFrame();
                p_FragFrame->h_Frame    = InitAndCreateFrameObject(p_FragFrameInfo);
                p_FragFrameInfo->h_App  = p_FragFrame;
                F_SetFrameInfo(p_FragFrame->h_Frame, p_FragFrameInfo);

                /* Remove the sequence number from the IP fragment. The sequence
                 * number is already in the original (unfragmented) frame. */
                F_CutFromTail(p_FragFrame->h_Frame, 4, NULL);

                /* Add the fragment to the end of the list */
                LIST_AddToTail(&p_FragFrame->frmsNode, p_FrameList);

                XX_Free(p_Data);
            }
            while (ipPayloadSize > 0);

            p_FrameToRemove = p_Frame;

            p_Frame = LIST_OBJECT(p_Frame->frmsNode.p_Next, t_PgFrame, frmsNode);

            /* Remove the original frame from the list. */
            LIST_Del(&p_FrameToRemove->frmsNode);

            /* Destroy the original (unfragmented) frame */
            PG_DestroyFrame(p_FrameToRemove->h_Frame);
            XX_Free(p_FrameToRemove);
        }
        else
            /* We don't need to fragment. Just move to the next frame in the list. */
            p_Frame = LIST_OBJECT(p_Frame->frmsNode.p_Next, t_PgFrame, frmsNode);

        p_FrameInfo = F_GetFrameInfo(p_Frame->h_Frame);
    }
    while (p_FrameList != &p_Frame->frmsNode);

    return E_OK;
}

t_Handle PG_CreateFrameList(t_Handle h_PacketGen, t_PgListParam *p_PgListParam)
{
    t_PacketGen                     *p_PacketGen      = (t_PacketGen *)h_PacketGen;

    t_PgListSettings                *p_PgListSettings = NULL;
    t_PgFrame                       *p_Frame;

    uint32_t                        frameId   = 0;          /* Frame identifier to be passed to the CreateFrame function */
    uint32_t                        frameDesc = 0, weightage = 0, frameNum = 0;
    t_List                          *p_List   = NULL;       /* Frames list */
    uint32_t                        frameDescrIndex = 0;

    if(!p_PacketGen || !p_PgListParam)
    {
        REPORT_ERROR(CRITICAL, E_INVALID_HANDLE, ("p_PacketGen"));
        return NULL;
    }

    p_PgListSettings  = (t_PgListSettings*)XX_Malloc(sizeof(t_PgListSettings));
    if (!p_PgListSettings)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_PgListSettings!"));
        return NULL;
    }
    memset(p_PgListSettings, 0, sizeof(t_PgListSettings));

    p_List  = (t_List*)XX_Malloc(sizeof(t_List));
    if (!p_List)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_List!"));
        return NULL;
    }
    memset(p_List, 0, sizeof(t_List));

    INIT_LIST(p_List);

    for (frameDesc = 0; frameDesc < p_PgListParam->numOfFrameListSubSet; frameDesc++)
    {
        frameId = p_PgListParam->p_FrameListSettings[frameDesc].id;

        for(weightage = 0; weightage < p_PgListParam->p_FrameListSettings[frameDesc].weight; weightage++)
        {
            p_Frame = PG_CreateFrame();
            if(!p_Frame)
            {
                REPORT_ERROR(MAJOR, E_NO_MEMORY, ("p_Frame"));
                return NULL;
            }

            *(p_PgListParam->p_FrameSeqNum) += p_PgListParam->seqNumStep;

            p_Frame->h_Frame = CreateFrame(p_PacketGen->h_FrameSettings,
                                                     frameId,
                                                     *(p_PgListParam->p_FrameSeqNum),
                                                     frameNum,
                                                     p_PgListParam->p_UserManipParam,
                                                     &p_PacketGen->bufMemAccess,
                                                     p_PgListParam->dataByteAlignment,
                                                     p_PacketGen->h_TrafficIphc);
            if(!p_Frame->h_Frame)
            {
                REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("Cannot Create Frame!"));
                return NULL;
            }
            /* Initialize the frame type for weight calculation */
            p_Frame->frameDescrSubSetIndex = frameDesc;

            AddFrameToList(p_Frame, p_List);

            frameNum++;
        }
    }

    p_PgListSettings->p_PacketGen = p_PacketGen;
    p_PgListSettings->p_List = p_List;

    /* Initialization for weight parameters */
    p_PgListSettings->numOfFrameListSubSet = p_PgListParam->numOfFrameListSubSet;
    p_PgListSettings->p_PgFrameWeights = (uint32_t *)XX_Malloc(p_PgListParam->numOfFrameListSubSet * sizeof(uint32_t));
    if (!p_PgListSettings->p_PgFrameWeights)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_PgListSettings->p_PgFrameWeights!"));
        return NULL;
    }
    memset(p_PgListSettings->p_PgFrameWeights, 0, p_PgListParam->numOfFrameListSubSet * sizeof(uint32_t));
    for(frameDescrIndex = 0; frameDescrIndex < p_PgListParam->numOfFrameListSubSet; frameDescrIndex++ )
    {
        /* Initialize the weights of all id's */
        p_PgListSettings->p_PgFrameWeights[frameDescrIndex] = p_PgListParam->p_FrameListSettings[frameDescrIndex].weight;
    }

    return (t_Handle)p_PgListSettings;
}

t_Handle PG_CreateFrameListFromExistingList(t_Handle h_List, t_PgUserManipParam *p_UserManipParam)
{
    t_List                          *p_List = NULL;     /* Frames list */
    t_List                          *p_FramesList, *p_Pos;
    t_PgFrame                       *p_SrcFrame = NULL, *p_DstFrame = NULL; /* Frame handle */
    t_TrafficFrameInfo              *p_DstFrameInfo, *p_SrcFrameInfo;
    t_PgListSettings                *p_PgFrameSetsList = (t_PgListSettings *)h_List;
    t_PgListSettings                *p_PgListSettings  = NULL;
    t_PgFrameCallBackMapSettings    *p_PgFrameCallBackMapSettings = NULL;
    uint32_t                        numOfManip, frameNum;
    bool                            suppressFrame = FALSE;

    p_PgListSettings  = (t_PgListSettings*)XX_Malloc(sizeof(t_PgListSettings));
    if (!p_PgListSettings)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_PgListSettings!"));
        return NULL;
    }
    memset(p_PgListSettings, 0, sizeof(t_PgListSettings));

    p_List  = (t_List*)XX_Malloc(sizeof(t_List));
    if (!p_List)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_List!"));
        return NULL;
    }
    memset(p_List, 0, sizeof(t_List));

    INIT_LIST(p_List);

    p_FramesList = p_PgFrameSetsList->p_List;

    frameNum = 0;

    /* Traverse the node in the existing list till the list is empty */
    LIST_FOR_EACH(p_Pos, p_FramesList)
    {
        /* Get the frame in each node */
        p_SrcFrame = PG_FRAME_OBJECT(p_Pos);
        if (!p_SrcFrame->h_Frame)
        {
            REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_SrcFrame"));
        }

        /* Get all frame information */
        p_SrcFrameInfo = F_GetFrameInfo(p_SrcFrame->h_Frame);

        if (p_UserManipParam && (p_UserManipParam->numOfUserManipFrame != 0))
        {
            p_PgFrameCallBackMapSettings = p_UserManipParam->p_PgFrameCallBackMapSettings;

            for (numOfManip = 0; numOfManip < p_UserManipParam->numOfUserManipFrame; numOfManip++)
            {
                if (p_SrcFrameInfo->frameId == p_PgFrameCallBackMapSettings[numOfManip].frameId && p_PgFrameCallBackMapSettings[numOfManip].f_UserFrameConfig)
                {
                    suppressFrame = p_PgFrameCallBackMapSettings[numOfManip].suppress;
                    break;
                }
            }
        }

        if (suppressFrame)
        {
            frameNum++;
            continue;
        }

       /* Allocate memory to hold the frame informations */

        p_DstFrameInfo = (t_TrafficFrameInfo *)XX_Malloc(sizeof(t_TrafficFrameInfo));
        if (!p_DstFrameInfo)
        {
            REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_DstFrameInfo!"));
            return NULL;
        }

        memset(p_DstFrameInfo, 0, sizeof(t_TrafficFrameInfo));

        p_DstFrame = PG_CreateFrame();
        if(!p_DstFrame)
        {
            REPORT_ERROR(MAJOR, E_NO_MEMORY, ("p_DstFrame"));
            return NULL;
        }

        /* Duplicate the frame 'h_Frame' */
        p_DstFrame->h_Frame = F_Clone(p_SrcFrame->h_Frame, TRUE);
        if(!p_DstFrame->h_Frame)
        {
            REPORT_ERROR(MINOR, E_INVALID_HANDLE, ("Cannot Create Frame!"));
            return NULL;
        }
        /* Initialize the frame type for weight calculation */
        p_DstFrame->frameDescrSubSetIndex = p_SrcFrame->frameDescrSubSetIndex;

        p_DstFrameInfo->payloadLength = p_SrcFrameInfo->payloadLength;
        p_DstFrameInfo->payloadOffset = p_SrcFrameInfo->payloadOffset;
        p_DstFrameInfo->frameId       = p_SrcFrameInfo->frameId;

        /* TO DO: Copy encapsulation details from srcFrameInfo into dstframeInfo */
        if(p_UserManipParam && (p_UserManipParam->numOfUserManipFrame != 0))
        {
            p_PgFrameCallBackMapSettings = p_UserManipParam->p_PgFrameCallBackMapSettings;

            for(numOfManip = 0; numOfManip < p_UserManipParam->numOfUserManipFrame; numOfManip++)
            {
                if(p_SrcFrameInfo->frameId == p_PgFrameCallBackMapSettings[numOfManip].frameId && p_PgFrameCallBackMapSettings[numOfManip].f_UserFrameConfig)
                {
                    p_PgFrameCallBackMapSettings[numOfManip].f_UserFrameConfig(p_DstFrame->h_Frame, p_UserManipParam->h_UserManipParam, p_SrcFrameInfo->frameId, frameNum);
                    break;
                }
            }
        }
        /* Configure the pointer to t_TrafficFrameInfo structure in the frame */
        F_SetFrameInfo(p_DstFrame->h_Frame, p_DstFrameInfo);

        /* AddToTail of the list 'p_DuplicateList' the created frame 'h_CloneFrame' */
        AddFrameToList(p_DstFrame, p_List);

        frameNum++;
    }

    p_PgListSettings->p_PacketGen = p_PgFrameSetsList->p_PacketGen;
    p_PgListSettings->p_List = p_List;

    /* Initialization for weight parameters */
    p_PgListSettings->numOfFrameListSubSet = p_PgFrameSetsList->numOfFrameListSubSet;
    p_PgListSettings->p_PgFrameWeights = (uint32_t *)XX_Malloc(p_PgFrameSetsList->numOfFrameListSubSet * sizeof(uint32_t));
    if (!p_PgListSettings->p_PgFrameWeights)
    {
        REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_PgListSettings->p_PgFrameWeights!"));
        return NULL;
    }
    memset(p_PgListSettings->p_PgFrameWeights, 0, p_PgFrameSetsList->numOfFrameListSubSet * sizeof(uint32_t));
    memcpy(p_PgListSettings->p_PgFrameWeights, p_PgFrameSetsList->p_PgFrameWeights, p_PgFrameSetsList->numOfFrameListSubSet * sizeof(uint32_t));

    return (t_Handle)p_PgListSettings;
}

t_Handle PG_MergeFrameLists(t_Handle *h_MergeLists, uint32_t numOfMergeLists)
{
    uint8_t             i = 0;
    t_PgListSettings    *p_PgListSettings = NULL, *p_NewList = NULL;
    uint32_t            numOfFrameListSubSet = 0;
    uint32_t            *p_PgFrameWeights = NULL;

    t_List              *p_FramesList, *p_Pos;
    t_PgFrame           *p_SrcFrame = NULL;

    for(i = 0; i < numOfMergeLists; i++)
    {
        p_NewList = (t_PgListSettings *)PG_CreateFrameListFromExistingList(h_MergeLists[i],NULL);
        if(!p_NewList)
        {
            REPORT_ERROR(CRITICAL, E_INVALID_HANDLE,("p_NewList!"));
            return NULL;
        }

        if(p_PgListSettings)
        {
            p_FramesList = p_NewList->p_List;
            /* Traverse the node in the existing list till the list is empty */
            LIST_FOR_EACH(p_Pos, p_FramesList)
            {
                /* Get the frame in each node */
                p_SrcFrame = PG_FRAME_OBJECT(p_Pos);
                if(!p_SrcFrame->h_Frame)
                {
                    REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_SrcFrame"));
                }
                /* Re-Initialize the frame type for weight calculation */
                p_SrcFrame->frameDescrSubSetIndex += p_PgListSettings->numOfFrameListSubSet;
            }

            LIST_Append(p_NewList->p_List,LIST_LAST(p_PgListSettings->p_List));
            /* Initialization of list for weight parameters */
            /* Reallocate memory for numOfFrameListSubSet for list 1 and list 2 */
            numOfFrameListSubSet = (uint32_t)(p_PgListSettings->numOfFrameListSubSet + p_NewList->numOfFrameListSubSet);
            p_PgFrameWeights = (uint32_t *)XX_Malloc(numOfFrameListSubSet * sizeof(uint32_t));
            if (!p_PgFrameWeights)
            {
                REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_PgFrameWeights!"));
                return NULL;
            }
            memset(p_PgFrameWeights, 0, numOfFrameListSubSet * sizeof(uint32_t));
            memcpy(p_PgFrameWeights, p_PgListSettings->p_PgFrameWeights, p_PgListSettings->numOfFrameListSubSet * sizeof(uint32_t));
            memcpy(p_PgFrameWeights + p_PgListSettings->numOfFrameListSubSet, p_NewList->p_PgFrameWeights, p_NewList->numOfFrameListSubSet * sizeof(uint32_t));
            p_PgListSettings->numOfFrameListSubSet += p_NewList->numOfFrameListSubSet;
            XX_Free(p_PgListSettings->p_PgFrameWeights);
            p_PgListSettings->p_PgFrameWeights = p_PgFrameWeights;

            XX_Free(p_NewList->p_List);
            XX_Free(p_NewList->p_PgFrameWeights);
            XX_Free(p_NewList);
        }
        else
        {
            p_PgListSettings = p_NewList;
        }
    }

    return (t_Handle)p_PgListSettings;
}

void PG_FreeFrameList(t_Handle h_List)
{
    t_PgFrame           *p_PgFrame   = NULL;
    t_PgListSettings    *p_PgListSettings = (t_PgListSettings *)h_List;

    p_PgFrame = RemoveFrameFromList(p_PgListSettings->p_List);

    while(p_PgFrame)
    {
        if(!PG_DestroyFrame(p_PgFrame->h_Frame))
        {
            XX_Free(p_PgFrame);
        }
        p_PgFrame = RemoveFrameFromList(p_PgListSettings->p_List);
    }
    XX_Free(p_PgListSettings->p_List);
    XX_Free(p_PgListSettings->p_PgFrameWeights);
    XX_Free(p_PgListSettings);
}

t_Handle PG_GetFrameFromFrameList(t_Handle h_List, uint32_t nthFrame)
{
    t_List              *p_Pos;
    t_PgListSettings    *p_PgFrameSetsList = (t_PgListSettings *)h_List;
    t_PgFrame           *p_PgFrame = NULL;
    uint32_t            i = 0;
    t_Handle            h_Frame = NULL;

    if(!h_List || !p_PgFrameSetsList->p_List)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid Handle"));
        return NULL;
    }

    if (!LIST_IsEmpty(p_PgFrameSetsList->p_List))
    {
        p_Pos = p_PgFrameSetsList->p_List->p_Next;
        /* Get the nth node in the List */
        for(i = 0 ; i < nthFrame ; p_Pos = LIST_NEXT(p_Pos), i++)
            ;
        p_PgFrame = PG_FRAME_OBJECT(p_Pos);
        h_Frame = p_PgFrame->h_Frame;
    }
    return h_Frame;
}

t_Error PG_HeaderInsert(t_Handle            h_Frame,
                        e_PgHeaderType      operateOnHeader,
                        e_PgHeaderType      insertAfterHeader,
                        uint8_t             operateOnHeaderIndex,
                        t_PgFrameSetting    *p_FrameSetting)
{

    uint8_t                 headerIndex         = (uint8_t)(operateOnHeaderIndex + 1);
                                                            /* Index of header in framegen */
    t_Handle                h_InputHeader       = NULL;     /* Handle to the user configured header info */
    t_Handle                h_OpHeader          = NULL;     /* Handle to the header that's to be inserted */
    t_TrafficFrameInfo      *p_FrameInfo        = NULL;     /* Pointer to frame information */
    t_Error                 rc                  = E_OK;     /* Return value */

    if(!h_Frame)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE, ("h_Frame"));
    }

    if(operateOnHeader == e_HEADER_TYPE_NONE)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_VALUE, ("Undefined header type!"));
    }

    /* Get the input settings to configure new insert header */
    h_InputHeader = FRAME_GetInputHeader(operateOnHeader, p_FrameSetting);

    /* Get all frame information */
    p_FrameInfo = F_GetFrameInfo(h_Frame);

    /* Create and Config header node to be inserted */
    h_OpHeader = FRAME_ConfigureHeader(operateOnHeader, h_InputHeader, p_FrameInfo->h_ProtocolInfo);

    /* Insert the new header in the frame's internal structure(frame gen)*/
    rc = UpdateFrameGen(e_INSERT_HEADER, h_Frame, insertAfterHeader, h_OpHeader, headerIndex);
    if(rc != E_OK)
    {
        RETURN_ERROR(MAJOR, rc, ("Failed to insert header!"));
    }

    return E_OK;
}

t_Error PG_HeaderRemove(t_Handle            h_Frame,
                        e_PgHeaderType      operateOnHeader,
                        uint8_t             operateOnHeaderIndex)
{

    uint8_t                 headerIndex         = (uint8_t)(operateOnHeaderIndex + 1);
                                                            /* Index of header in framegen */
    t_Error                 rc                  = E_OK;     /* Return value */

    if(!h_Frame)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE, ("h_Frame"));
    }

    if(operateOnHeader == e_HEADER_TYPE_NONE)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_VALUE, ("Undefined header type!"));
    }

    /* Remove the specified header from the frame's internal structure(frame gen) */
    rc = UpdateFrameGen(e_REMOVE_HEADER, h_Frame, operateOnHeader, NULL, headerIndex);
    if(rc != E_OK)
    {
        RETURN_ERROR(MAJOR, rc, ("Failed to remove header!"));
    }

    return E_OK;
}

t_Error PG_HeaderReplace(t_Handle           h_Frame,
                         e_PgHeaderType     operateOnHeader,
                         uint8_t            operateOnHeaderIndex,
                         t_PgFrameSetting   *p_FrameSetting)
{

    uint8_t                 headerIndex         = (uint8_t)(operateOnHeaderIndex + 1);
                                                            /* Index of header in framegen */
    t_Handle                h_InputHeader       = NULL;     /* Handle to the user configured header info */
    t_Handle                h_OpHeader          = NULL;     /* Handle to the header that's to be replaced with */
    t_Error                 rc                  = E_OK;     /* Return value */
    t_TrafficFrameInfo      *p_FrameInfo        = NULL;     /* Pointer to frame information */

    if(!h_Frame)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE, ("h_Frame"));
    }

    if(operateOnHeader == e_HEADER_TYPE_NONE)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_VALUE, ("Undefined header type!"));
    }

    /* Get the input settings to configure new replace header */
    h_InputHeader = FRAME_GetInputHeader(operateOnHeader, p_FrameSetting);

    /* Get all frame information */
    p_FrameInfo = F_GetFrameInfo(h_Frame);

    /* Create and Config header node to be replaced with */
    h_OpHeader = FRAME_ConfigureHeader(operateOnHeader, h_InputHeader, p_FrameInfo->h_ProtocolInfo);

    /* Replace the new header: Remove the already existing header and insert this new header
       in the frame's internal structure(frame gen) */
    rc = UpdateFrameGen(e_REPLACE_HEADER, h_Frame, operateOnHeader, h_OpHeader, headerIndex);
    if(rc != E_OK)
    {
        RETURN_ERROR(MAJOR, rc, ("Failed to replace header!"));
    }

    return E_OK;
}

t_Error PG_HeaderFieldManipulation(t_Handle             h_Frame,
                                   e_PgHeaderType       operateOnHeader,
                                   uint8_t              operateOnHeaderIndex,
                                   t_PgFrameSetting     *p_FrameSetting)
{

    uint8_t                 headerIndex         = (uint8_t)(operateOnHeaderIndex + 1);
                                                            /* Index of header in framegen */
    t_Handle                h_InputHeader       = NULL;     /* Handle to the user configured header info */

    if(!h_Frame)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_HANDLE, ("h_Frame"));
    }

    if(operateOnHeader == e_HEADER_TYPE_NONE)
    {
        RETURN_ERROR(CRITICAL, E_INVALID_VALUE, ("Undefined header type!"));
    }

    /* Get the input settings to configure new modify header */
    h_InputHeader = FRAME_GetInputHeader(operateOnHeader, p_FrameSetting);

    if(HeaderFieldManipulation(h_Frame, operateOnHeader, h_InputHeader, headerIndex) != E_OK)
    {
        RETURN_ERROR(CRITICAL, E_NOT_SUPPORTED, ("Failed to manipulate header fields!"));
    }

    return E_OK;
}

t_Error PG_CreateManipFrame(t_Handle h_OrigFrame)
{
    t_Handle                h_Frame         = NULL;     /* Handle to the manipulated frame */
    t_TrafficFrameInfo      *p_FrameInfo    = NULL;     /* Pointer to frame information */

    /* Get all frame information */
    p_FrameInfo = F_GetFrameInfo(h_OrigFrame);

    /* Initialize the frame and create the frame for transmission */
    h_Frame = InitAndCreateFrameObject(p_FrameInfo);

    /* Replace the buffers in the original frame with newly created
    buffers, that holds manipulated frame */
    UpdateFrame(h_Frame, h_OrigFrame);

    return E_OK;
}

/** @} */ /* end of PG group */

