/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          traffic_analyzer.c

  @Description   Traffic Analyzer functions
*//***************************************************************************/
#include "std_ext.h"
#include "stdlib_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "frame_ext.h"
#include "fmanip_ext.h"
#include "iphc_manip.h"
#include "itg_common.h"
#include "traffic_analyzer.h"

/**************************************************************************//**
 @Description    Port and Queue Defines
*//***************************************************************************/
#define PRINT_RX_PORT_STATUS(p_TrafficAnalyzerPort, queueIndex, r)                  \
    XX_Print("Port %02d Queue %02d" r ": Rx-%d RE-%d \r",                   \
             p_TrafficAnalyzerPort->portId,                                       \
             p_TrafficAnalyzerPort->p_Queues[queueIndex].queueId,                     \
             p_TrafficAnalyzerPort->p_Queues[queueIndex].rxFrames,                     \
             p_TrafficAnalyzerPort->p_Queues[queueIndex].rxErrors)

/**************************************************************************//**
 @Description    TA functions
*//***************************************************************************/

static t_PgFrame * FindRxFrameByDataInLst(t_List *p_List, t_Handle h_Frame)
{
     t_PgFrame          *p_TrafficAnalyzerFrame;
     t_List             *p_Pos;
     uint8_t            seqNum[MAX_SEQ_NUM_SIZE], taSeqNum[MAX_SEQ_NUM_SIZE];
     uint16_t           size;

     if(!h_Frame)
     {
         REPORT_ERROR(MAJOR, E_NO_MEMORY, ("h_Frame"));
         return NULL;
     }

     READ_SEQ_NUM(h_Frame, seqNum, size);
     if (size != MAX_SEQ_NUM_SIZE)
     {
        REPORT_ERROR(MAJOR, E_READ_FAILED, NO_MSG);
        return NULL;
     }

     LIST_FOR_EACH(p_Pos, p_List)
     {
        p_TrafficAnalyzerFrame = RECV_F_TRAFFIC_ANALYZER_FRAME_OBJECT(p_Pos);
        if(!p_TrafficAnalyzerFrame->h_Frame)
        {
            REPORT_ERROR(MAJOR, E_NO_MEMORY, ("p_TrafficAnalyzerFrame->h_Frame"));
            return NULL;
        }

        /* Get offset position in the expected frame for comparision */
        READ_SEQ_NUM(p_TrafficAnalyzerFrame->h_Frame, taSeqNum, size);
        if (size != MAX_SEQ_NUM_SIZE)
        {
            REPORT_ERROR(MAJOR, E_READ_FAILED, NO_MSG);
            return NULL;
        }

        if (!(memcmp(seqNum, taSeqNum, MAX_SEQ_NUM_SIZE)))
        {
             return p_TrafficAnalyzerFrame;
        }
     }

     return NULL;
}

static t_PgFrame * FindAndDequeueRxFrameInLst(t_List *p_List, t_Handle h_Frame)
{
     t_PgFrame      *p_TrafficAnalyzerFrame = NULL;
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_List))
     {
         p_TrafficAnalyzerFrame = FindRxFrameByDataInLst(p_List, h_Frame);
         if (p_TrafficAnalyzerFrame)
         {
            LIST_DelAndInit(&p_TrafficAnalyzerFrame->frmsNode);
         }
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficAnalyzerFrame;
}

static void EnqueueFrameToRxDataQ(t_TrafficAnalyzerQueue *p_TrafficAnalyzerQueue, t_PgFrame *p_TrafficAnalyzerFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_TrafficAnalyzerFrame->frmsNode, p_TrafficAnalyzerQueue->p_RxDataFramesLst);
     XX_RestoreAllIntr(intFlags);
}

static void EnqueueFrameToCompDepthQ(t_TrafficAnalyzerQueue *p_TrafficAnalyzerQueue, t_PgFrame *p_CompDepthFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_CompDepthFrame->frmsNode, &p_TrafficAnalyzerQueue->compDepthFramesLst);
     XX_RestoreAllIntr(intFlags);
}

static t_PgFrame * DequeueFrameFromRxDataQ(t_TrafficAnalyzerQueue *p_TrafficAnalyzerQueue)
{
     t_PgFrame      *p_TrafficAnalyzerFrame = NULL;
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_TrafficAnalyzerQueue->p_RxDataFramesLst))
     {
         p_TrafficAnalyzerFrame = RECV_F_TRAFFIC_ANALYZER_FRAME_OBJECT(p_TrafficAnalyzerQueue->p_RxDataFramesLst->p_Next);
         LIST_DelAndInit(&p_TrafficAnalyzerFrame->frmsNode);
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficAnalyzerFrame;
}

static t_PgFrame * DequeueFrameFromCompDepthQ(t_TrafficAnalyzerQueue *p_TrafficAnalyzerQueue)
{
     t_PgFrame      *p_CompDepthFrame = NULL;
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(&p_TrafficAnalyzerQueue->compDepthFramesLst))
     {
         p_CompDepthFrame = COMPDEPTH_F_TRAFFIC_ANALYZER_FRAME_OBJECT(p_TrafficAnalyzerQueue->compDepthFramesLst.p_Next);
         LIST_DelAndInit(&p_CompDepthFrame->frmsNode);
     }

     XX_RestoreAllIntr(intFlags);
     return p_CompDepthFrame;
}

static void FixedPatternPayloadParser(t_TrafficAnalyzer *p_TrafficAnalyzer, uint8_t *p_PayloadData, t_TrafficAnalyzerRxStatistics *p_RxPortStatistics, t_TrafficAnalyzerRxStatistics *p_RxGlobalStatistics, uint32_t payloadLength)
{
    bool        patternNotMatched = FALSE;
    uint8_t     patternIndx = 0;
    uint8_t     *rcvPattern = NULL;
    uint32_t    size = 0, remainingLength = 0;
    uint32_t    *portStatsPointer = NULL, *globalStatsPointer = NULL ;

    portStatsPointer = &p_RxPortStatistics->patternTypeOneCnt;
    globalStatsPointer = &p_RxGlobalStatistics->patternTypeOneCnt;

    while (patternIndx < MAX_NUM_FIXED_PATTERNS)
    {
        patternNotMatched = FALSE;
        size = p_TrafficAnalyzer->payloadFixedPattern[patternIndx]. patternSize;

        rcvPattern = p_PayloadData;
        remainingLength = payloadLength;

        while(remainingLength)
        {
            if (remainingLength < size)
            {
                size = remainingLength;
            }
            if(memcmp(rcvPattern, p_TrafficAnalyzer->payloadFixedPattern[patternIndx].patternValue ,size))
            {
                patternNotMatched = TRUE;
                break;
            }
            rcvPattern += size;
            remainingLength -= size;
        }
        if(!patternNotMatched)
        {
            portStatsPointer[patternIndx]++;
            globalStatsPointer[patternIndx]++;
            break;
        }
        patternIndx++;
    }

}

static void PayloadParser(t_TrafficAnalyzer *p_TrafficAnalyzer, t_TrafficAnalyzerRxStatistics *p_RxPortStatistics, t_TrafficAnalyzerRxStatistics *p_RxGlobalStatistics, t_Handle h_Frame)
{
     t_TrafficFrameInfo     *p_FrameInfo = NULL;            /* Holds the frame informations */
     t_Handle               h_Buff;
     uint16_t               buffIndx = 0;
     bool                   lastBuffer     = FALSE;
     uint8_t                *p_PayloadData = NULL;
     uint32_t               nxtByte   = 0;
     uint8_t                firstDiff = 0, nxtDiff = 0;
     bool                   diffNotEqual  = FALSE;
     uint32_t               payloadOffset = 0, payloadLength = 0;

     /* Get all frame information */
     p_FrameInfo = F_GetFrameInfo(h_Frame);

     h_Buff = F_GetBuffer (h_Frame, buffIndx++, &lastBuffer);
     payloadOffset = p_FrameInfo->payloadOffset;
     p_PayloadData = B_GetData(h_Buff) + payloadOffset;

     payloadLength = p_FrameInfo->payloadLength;

     /* Payload statistics calculation is valid only when payload length is greater than 1 */
     if(payloadLength > 1)
     {
         firstDiff = (uint8_t)(p_PayloadData[nxtByte+1] - p_PayloadData[nxtByte]);
         while(++nxtByte < payloadLength-1)
         {
             nxtDiff = (uint8_t)(p_PayloadData[nxtByte+1] - p_PayloadData[nxtByte]);
             if(firstDiff != nxtDiff)
             {
                 diffNotEqual = TRUE;
                 break;
             }
         }
         if(!diffNotEqual)
         {
             if(nxtDiff)
             {
                 p_RxPortStatistics->seqPayloadCnt++;
                 p_RxGlobalStatistics->seqPayloadCnt++;
             }
             else
             {
                 p_RxPortStatistics->fixedPayloadCnt++;
                 p_RxGlobalStatistics->fixedPayloadCnt++;
                 FixedPatternPayloadParser(p_TrafficAnalyzer,p_PayloadData, p_RxPortStatistics, p_RxGlobalStatistics, payloadLength);
             }
         }
         else
         {
             FixedPatternPayloadParser(p_TrafficAnalyzer,p_PayloadData, p_RxPortStatistics, p_RxGlobalStatistics, payloadLength);
         }
     }
}

static void DequeueAndCompareRxFrame(t_Handle h_Ta, uint16_t portId, uint32_t queueIndex, t_Handle h_Frame)
{
     t_TrafficAnalyzer         *p_TrafficAnalyzer       = (t_TrafficAnalyzer *)h_Ta;
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort   = NULL;
     t_PgFrame                 *p_TrafficAnalyzerFrame  = NULL;
     t_Handle                  h_Buff = NULL, h_AppBuff = NULL;
     t_TrafficFrameInfo        *p_FrameInfo = NULL, appFrameInfo;         /* Holds the frame informations */

     bool                      lastBuffer = FALSE, appLastBuffer = FALSE, equals = TRUE;

     uint8_t                   *p_Data = NULL, *p_TrafficAnalyzerData = NULL;
     uint8_t                   seqNum[MAX_SEQ_NUM_SIZE], taSeqNum[MAX_SEQ_NUM_SIZE];
     uint16_t                  buffIndx = 0, appBuffIndx = 0, size =0, portIndex = 0;
     uint32_t                  payloadOffset = 0, payloadLength = 0, frameLength = 0;
     uint32_t                  buffLength    = 0, appBuffLength = 0, temp1 = 0, temp2 = 0, cmpLen = 0, lengthLeft = 0;
     uint32_t                  queueId = 0;

     for(portIndex = 0; portIndex < p_TrafficAnalyzer->numOfPorts; portIndex++)
     {
        p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portIndex);
        if(p_TrafficAnalyzerPort->portId == portId)
            break;
     }

     if (p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueIndex] == TRUE)
     {
         p_TrafficAnalyzerFrame = FindAndDequeueRxFrameInLst(p_TrafficAnalyzerPort->p_Queues[queueIndex].p_RxDataFramesLst, h_Frame);
     }
     else
     {
         p_TrafficAnalyzerFrame = DequeueFrameFromRxDataQ(&p_TrafficAnalyzerPort->p_Queues[queueIndex]);
     }

     if (!p_TrafficAnalyzerFrame)
     {
         XX_Print("Data mismatch, frame not found\n");
         equals = FALSE;
         goto exitRx;
     }

     if (p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueIndex] == TRUE)
     {
         /* Increment the weights for the frameId received */
          p_TrafficAnalyzerPort->p_Queues[queueIndex].p_RxFrameTypeCounter[p_TrafficAnalyzerFrame->frameDescrSubSetIndex]++;
     }

     EnqueueFrameToRxDataQ(&p_TrafficAnalyzerPort->p_Queues[queueIndex], p_TrafficAnalyzerFrame);

     frameLength = F_GetLength(h_Frame);

     if (frameLength != F_GetLength(p_TrafficAnalyzerFrame->h_Frame))
     {
         XX_Print("Port %d: Frame - length mismatch: got %d instead of %d\n", p_TrafficAnalyzerPort->portId, F_GetLength(h_Frame), F_GetLength(p_TrafficAnalyzerFrame->h_Frame));
         equals = FALSE;
         goto exitRx;
     }

     if(frameLength < p_TrafficAnalyzerPort->underSizeFrameValue)
     {
         p_TrafficAnalyzerPort->rxStatistics.rxUndersizedFrames++;
         p_TrafficAnalyzer->globalRxStatistics.rxUndersizedFrames++;
     }
     else if(frameLength > p_TrafficAnalyzerPort->overSizeFrameValue)
     {
         p_TrafficAnalyzerPort->rxStatistics.rxOversizedFrames++;
         p_TrafficAnalyzer->globalRxStatistics.rxOversizedFrames++;
     }

     if(frameLength >= JUMBO_FRAME_SIZE)
     {
         p_TrafficAnalyzerPort->rxStatistics.rxJumboFrames++;
         p_TrafficAnalyzer->globalRxStatistics.rxJumboFrames++;
     }


     h_Buff = F_GetBuffer (h_Frame, buffIndx++, &lastBuffer);
     buffLength = (uint16_t)B_GetLength(h_Buff);
     p_Data = B_GetData(h_Buff);

     h_AppBuff = F_GetBuffer (p_TrafficAnalyzerFrame->h_Frame, appBuffIndx++, &appLastBuffer);
     appBuffLength = (uint16_t)B_GetLength(h_AppBuff);
     p_TrafficAnalyzerData = B_GetData(h_AppBuff);

     READ_SEQ_NUM(h_Frame, seqNum, size);
     if (size != MAX_SEQ_NUM_SIZE)
     {
        equals = FALSE;
        goto exitRx;
     }

     READ_SEQ_NUM(p_TrafficAnalyzerFrame->h_Frame, taSeqNum, size);
     if (size != MAX_SEQ_NUM_SIZE)
     {
        equals = FALSE;
        goto exitRx;
     }

     if(memcmp(seqNum, taSeqNum, MAX_SEQ_NUM_SIZE))
     {
         memcpy(&temp1, seqNum, MAX_SEQ_NUM_SIZE);
         memcpy(&temp2, taSeqNum, MAX_SEQ_NUM_SIZE);
         XX_Print("Port %d: Frame - sequence mismatch: got %d instead of %d\n",
             p_TrafficAnalyzerPort->portId, temp1, temp2);
         equals = FALSE;
     }

     if(p_TrafficAnalyzer->frameComparisonMode == e_FRAME_LENGTH_HEADER_PAYLOAD)
     {
        /* Get all frame information */
        p_FrameInfo = F_GetFrameInfo(p_TrafficAnalyzerFrame->h_Frame);

        memcpy(&appFrameInfo, p_FrameInfo, sizeof(t_TrafficFrameInfo));

        payloadLength = p_FrameInfo->payloadLength;
        /* Adjusting payloadLength for payload stats calculation by Sequence number bytes */
        payloadLength -= MAX_SEQ_NUM_SIZE;
        payloadOffset = p_FrameInfo->payloadOffset;

        /* Set the payload length in the frame */
        appFrameInfo.payloadLength = payloadLength;
        /* Set the payload offset in the frame */
        appFrameInfo.payloadOffset = payloadOffset;

        F_SetFrameInfo(h_Frame, &appFrameInfo);

        PayloadParser(p_TrafficAnalyzer, &p_TrafficAnalyzerPort->rxStatistics, &p_TrafficAnalyzer->globalRxStatistics, h_Frame);

        F_SetFrameInfo(h_Frame, NULL);
     }

     if(p_TrafficAnalyzer->frameComparisonMode != e_FRAME_LENGTH)
     {
        if(p_TrafficAnalyzer->frameComparisonMode == e_FRAME_LENGTH_HEADER)
        {
            /* Get all frame information */
            p_FrameInfo = F_GetFrameInfo(p_TrafficAnalyzerFrame->h_Frame);
            lengthLeft = p_FrameInfo->payloadOffset;
        }
        else
        {
            lengthLeft = F_GetLength(p_TrafficAnalyzerFrame->h_Frame);
        }

        while (lengthLeft)
        {
            cmpLen = (buffLength<=appBuffLength) ? buffLength : appBuffLength;
            if(p_TrafficAnalyzer->frameComparisonMode == e_FRAME_LENGTH_HEADER)
            {
                cmpLen = (cmpLen <= lengthLeft) ? cmpLen : lengthLeft;
            }
            if (memcmp(p_Data, p_TrafficAnalyzerData, cmpLen))
            {
                XX_Print("Port %d: Frame - data mismatch\n", p_TrafficAnalyzerPort->portId);
                equals = FALSE;
                break;
            }
            if (buffLength<appBuffLength && !lastBuffer)
            {
                h_Buff = F_GetBuffer (h_Frame, buffIndx++, &lastBuffer);
                buffLength = (uint16_t)B_GetLength(h_Buff);
                p_Data = B_GetData(h_Buff);

                p_TrafficAnalyzerData += cmpLen;
                appBuffLength -= cmpLen;
            }
            else if (appBuffLength<buffLength && !appLastBuffer)
            {
                h_AppBuff = F_GetBuffer (p_TrafficAnalyzerFrame->h_Frame, appBuffIndx++, &appLastBuffer);
                appBuffLength = (uint16_t)B_GetLength(h_AppBuff);
                p_TrafficAnalyzerData = B_GetData(h_AppBuff);

                p_Data += cmpLen;
                buffLength -= cmpLen;
            }
            else if (appBuffLength == buffLength)
            {
                if (!lastBuffer && !appLastBuffer)
                {
                    h_Buff = F_GetBuffer (h_Frame, buffIndx++, &lastBuffer);
                    buffLength = (uint16_t)B_GetLength(h_Buff);
                    p_Data = B_GetData(h_Buff);
                    h_AppBuff = F_GetBuffer (p_TrafficAnalyzerFrame->h_Frame, appBuffIndx++, &appLastBuffer);
                    appBuffLength = (uint16_t)B_GetLength(h_AppBuff);
                    p_TrafficAnalyzerData = B_GetData(h_AppBuff);
                }
            }

            lengthLeft -= cmpLen;
         }
      }

 exitRx:
     if (equals)
     {
         p_TrafficAnalyzerPort->p_Queues[queueIndex].rxFrames++;
         p_TrafficAnalyzerPort->rxStatistics.rxFrames++;
         p_TrafficAnalyzer->globalRxStatistics.rxFrames++;
         p_TrafficAnalyzerPort->rxStatistics.rxOctets += frameLength;
         p_TrafficAnalyzer->globalRxStatistics.rxOctets +=  frameLength;
     }
     else
     {
         p_TrafficAnalyzerPort->p_Queues[queueIndex].rxErrors++;
         p_TrafficAnalyzerPort->rxStatistics.rxErrors++;
         p_TrafficAnalyzer->globalRxStatistics.rxErrors++;
         p_TrafficAnalyzerPort->testPassed = FALSE;
     }

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
     PRINT_RX_PORT_STATUS(p_TrafficAnalyzerPort, queueIndex, " R");
     XX_Print("\n");
 #endif /* (defined(DEBUG_ERRORS) && ... */

     if (p_TrafficAnalyzerPort->f_UserInspection)
         p_TrafficAnalyzerPort->f_UserInspection(h_Frame);

     queueId = p_TrafficAnalyzerPort->p_Queues[queueIndex].queueId;

     if (p_TrafficAnalyzerPort->f_DevRxConf)
         p_TrafficAnalyzerPort->f_DevRxConf(p_TrafficAnalyzerPort->h_Dev, queueId, h_Frame);
     else
         PG_DestroyFrame(h_Frame);
}

static void CompareDepthQRxFrames(t_Handle h_Ta)
{
     t_TrafficAnalyzer          *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;
     t_PgFrame                  *p_CompDepthFrame;
     t_TrafficAnalyzerPort      *p_TrafficAnalyzerPort;
     uint16_t                    portId;
     uint32_t                    queueId;

     for(portId = 0; portId < p_TrafficAnalyzer->numOfPorts; portId++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);
         for (queueId=0; queueId<p_TrafficAnalyzerPort->numOfQueues; queueId++)
         {
             p_CompDepthFrame = DequeueFrameFromCompDepthQ(&p_TrafficAnalyzerPort->p_Queues[queueId]);
             while (p_CompDepthFrame)
             {
                 /* TBD : return value check */
                 DequeueAndCompareRxFrame(h_Ta, portId, queueId, p_CompDepthFrame->h_Frame);
                 XX_Free(p_CompDepthFrame);
                 p_CompDepthFrame = DequeueFrameFromCompDepthQ(&p_TrafficAnalyzerPort->p_Queues[queueId]);
             }
         }
     }

}

static void PollReceiveFrame(t_Handle h_Ta)
{
     t_TrafficAnalyzer         *p_TrafficAnalyzer = (t_TrafficAnalyzer *)(h_Ta);
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort;

     uint16_t portId = 0;
     uint32_t  queueIndex = 0;

     for(portId = 0; portId < p_TrafficAnalyzer->numOfPorts; portId++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);
         for (queueIndex = 0; queueIndex < p_TrafficAnalyzerPort->numOfQueues; queueIndex++)
         {
             p_TrafficAnalyzerPort->f_DevRx(p_TrafficAnalyzerPort->h_Dev, p_TrafficAnalyzerPort->p_Queues[queueIndex].queueId);
         }
     }

     /* Modify expiration time */
     XX_ModTimer(p_TrafficAnalyzer->h_Timer,p_TrafficAnalyzer->pollingPeriod);
}

t_Handle TA_Init(t_TrafficAnalyzerParam  *p_TrafficAnalyzerParam)
{
     t_TrafficAnalyzer         *p_TrafficAnalyzer;

     t_PgPayloadFixedPattern fixedPayloadPattern[] = PAYLOAD_FIXED_PATTERN_TYPES;

     p_TrafficAnalyzer = (t_TrafficAnalyzer *)XX_Malloc(sizeof(t_TrafficAnalyzer));
     if (!p_TrafficAnalyzer)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficAnalyzer"));
         return NULL;
     }
     memset(p_TrafficAnalyzer, 0, sizeof(t_TrafficAnalyzer));

     /* Init number of Ports and portLst */
     p_TrafficAnalyzer->numOfPorts = 0;

     INIT_LIST(&p_TrafficAnalyzer->portsLst);

     p_TrafficAnalyzer->comparisonDepth = p_TrafficAnalyzerParam->comparisonDepth;
     p_TrafficAnalyzer->frameComparisonMode = p_TrafficAnalyzerParam->frameComparisonMode;

     p_TrafficAnalyzer->pollingMode = p_TrafficAnalyzerParam->pollingMode;
     p_TrafficAnalyzer->pollingPeriod = p_TrafficAnalyzerParam->pollingPeriod;

     memcpy(&p_TrafficAnalyzer->payloadFixedPattern, &fixedPayloadPattern, sizeof(fixedPayloadPattern));

     return p_TrafficAnalyzer;
}

t_Error TA_Free(t_Handle h_Ta)
{
     t_TrafficAnalyzer         *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort;
     uint32_t                  portId = 0;
     int                       i = 0;

     if (!p_TrafficAnalyzer)
         RETURN_ERROR(MINOR, E_INVALID_HANDLE, NO_MSG);

     for(i = 0; i < p_TrafficAnalyzer->numOfPorts; i++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);
         TA_PORT_Free(p_TrafficAnalyzerPort);
     }

     XX_Free(p_TrafficAnalyzer);

     return E_OK;
}

t_Handle TA_PORT_Config(t_Handle h_Ta, t_TrafficAnalyzerPortParam *p_TrafficAnalyzerPortParam)
{
     t_TrafficAnalyzer               *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;
     t_TrafficAnalyzerPort           *p_TrafficAnalyzerPort;
     t_PgFrame                       *p_TrafficAnalyzerFrame;
     t_TrafficAnalyzerQueue          *p_TrafficAnalyzerQueue;
     t_TrafficAnalyzerQueueParam     *p_TrafficAnalyzerQueueParam;
     t_List                          *p_FramesList, *p_Pos;
     t_TrafficPort                   *p_TrafficPort;

     uint32_t   queueIndex;
     int       totalWeight = 0;
     uint32_t  tmpRxBytes = 0;

     uint32_t  frameDescrIndex = 0;
     t_Error rc = E_OK;             /* Return value */

     p_TrafficAnalyzerPort = (t_TrafficAnalyzerPort *)XX_Malloc(sizeof(t_TrafficAnalyzerPort));
     if (!p_TrafficAnalyzerPort)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficAnalyzerPort"));
         return NULL;
     }
     memset(p_TrafficAnalyzerPort, 0, sizeof(t_TrafficAnalyzerPort));

     p_TrafficPort = ITG_CreateTrafficPort();
     if (!p_TrafficPort)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficPort"));
         XX_Free(p_TrafficAnalyzerPort);
         return NULL;
     }
     p_TrafficPort->h_Port = p_TrafficAnalyzerPort;

     p_TrafficAnalyzerPort->p_TrafficAnalyzer = (struct t_TrafficAnalyzer *)h_Ta;

     ITG_AddPortToList(p_TrafficPort, &p_TrafficAnalyzerPort->p_TrafficAnalyzer->portsLst);
     p_TrafficAnalyzer->numOfPorts++;

     p_TrafficAnalyzerPort->inUse = TRUE;
     p_TrafficAnalyzerPort->portId = p_TrafficAnalyzerPortParam->portId;
     p_TrafficAnalyzerPort->h_Dev = p_TrafficAnalyzerPortParam->h_Dev;
     p_TrafficAnalyzerPort->f_DevRx = p_TrafficAnalyzerPortParam->f_DevRx;
     p_TrafficAnalyzerPort->f_DevRxConf = p_TrafficAnalyzerPortParam->f_DevRxConf;
     p_TrafficAnalyzerPort->numOfFrames = p_TrafficAnalyzerPortParam->numOfFrames;
     p_TrafficAnalyzerPort->numOfQueues = p_TrafficAnalyzerPortParam->numOfQueues;
     p_TrafficAnalyzerPort->underSizeFrameValue = p_TrafficAnalyzerPortParam->underSizeFrameValue;
     p_TrafficAnalyzerPort->overSizeFrameValue = p_TrafficAnalyzerPortParam->overSizeFrameValue;
     p_TrafficAnalyzerPort->maxQueueIndex       = p_TrafficAnalyzerPortParam->maxQueueIndex;
     p_TrafficAnalyzerPort->f_UserInspection = p_TrafficAnalyzerPortParam->f_UserInspection;

     p_TrafficAnalyzerPort->p_Queues = (t_TrafficAnalyzerQueue *)XX_Malloc(p_TrafficAnalyzerPort->numOfQueues * sizeof(t_TrafficAnalyzerQueue));
     if (!p_TrafficAnalyzerPort->p_Queues)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficAnalyzerPort->p_Queues"));
         XX_Free(p_TrafficAnalyzerPort);
         return NULL;
     }
     memset(p_TrafficAnalyzerPort->p_Queues, 0, p_TrafficAnalyzerPort->numOfQueues * sizeof(t_TrafficAnalyzerQueue) );

     p_TrafficAnalyzerPort->p_QueueIndexMap = (uint32_t *)XX_Malloc(p_TrafficAnalyzerPortParam->maxQueueIndex * sizeof(uint32_t));

     if (!p_TrafficAnalyzerPort->p_QueueIndexMap)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficAnalyzerPort->p_QueueIndexMap"));
         XX_Free(p_TrafficAnalyzerPort);
         XX_Free(p_TrafficAnalyzerPort->p_Queues);
         return NULL;
     }
     memset(p_TrafficAnalyzerPort->p_QueueIndexMap, 0, p_TrafficAnalyzerPortParam->maxQueueIndex * sizeof(uint32_t));

     p_TrafficAnalyzerPort->p_EnableFlexibleRxMode = (bool *)XX_Malloc(p_TrafficAnalyzerPort->numOfQueues * sizeof(bool));
     if (!p_TrafficAnalyzerPort->p_EnableFlexibleRxMode)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficAnalyzerPort->p_EnableFlexibleRxMode"));
         XX_Free(p_TrafficAnalyzerPort);
         XX_Free(p_TrafficAnalyzerPort->p_Queues);
         XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
         return NULL;
     }
     memset(p_TrafficAnalyzerPort->p_EnableFlexibleRxMode, 0, p_TrafficAnalyzerPort->numOfQueues * sizeof(bool));


     for (queueIndex = 0; queueIndex < p_TrafficAnalyzerPort->numOfQueues; queueIndex++)
     {
         p_TrafficAnalyzerQueueParam = &(p_TrafficAnalyzerPortParam->p_QueuesParam[queueIndex]);

         p_TrafficAnalyzerQueue = &p_TrafficAnalyzerPort->p_Queues[queueIndex];

         p_TrafficAnalyzerQueue->queueId = p_TrafficAnalyzerQueueParam->queueId;
         p_TrafficAnalyzerQueue->p_RxDataFramesLst = (t_List *)PG_GetList(p_TrafficAnalyzerQueueParam->h_FramesLst);
         if(p_TrafficAnalyzerQueue->p_RxDataFramesLst == NULL)
         {
             REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_TrafficAnalyzerQueue->p_RxDataFramesLst"));
             XX_Free(p_TrafficAnalyzerPort);
             XX_Free(p_TrafficAnalyzerPort->p_Queues);
             XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
             return NULL;
         }
         /* Store the first element of Rx list for re-chaining in the next run */
         p_TrafficAnalyzerQueue->p_FirstNode = LIST_FIRST(p_TrafficAnalyzerQueue->p_RxDataFramesLst);
         p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueIndex] = p_TrafficAnalyzerQueueParam->enableFlexibleRxMode;

         p_TrafficAnalyzerPort->p_QueueIndexMap[p_TrafficAnalyzerQueueParam->queueId] = queueIndex;

         if (p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueIndex] == TRUE)
         {
             rc = PG_GetNumOfListSubSet(p_TrafficAnalyzerQueueParam->h_FramesLst,&p_TrafficAnalyzerQueue->numOfFrameListSubSet);
             if(rc != E_OK)
             {
                 REPORT_ERROR(MAJOR, rc, ("Failed to get number of list sub set!"));
                 XX_Free(p_TrafficAnalyzerPort);
                 XX_Free(p_TrafficAnalyzerPort->p_Queues);
                 XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
                 return NULL;
             }

             p_TrafficAnalyzerQueue->p_PgFrameWeights = (uint32_t*)PG_GetFrameWeights(p_TrafficAnalyzerQueueParam->h_FramesLst);
             if(p_TrafficAnalyzerQueue->p_PgFrameWeights == NULL)
             {
                 REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("p_TrafficAnalyzerQueue->p_PgFrameWeights"));
                 XX_Free(p_TrafficAnalyzerPort);
                 XX_Free(p_TrafficAnalyzerPort->p_Queues);
                 XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
                 return NULL;
             }

             p_TrafficAnalyzerQueue->p_RxFrameTypeCounter = (uint32_t *)XX_Malloc(p_TrafficAnalyzerQueue->numOfFrameListSubSet * sizeof(uint32_t));
             if (!p_TrafficAnalyzerQueue->p_RxFrameTypeCounter)
             {
                 REPORT_ERROR(CRITICAL, E_NO_MEMORY,("p_TrafficAnalyzerQueue->p_RxFrameTypeCounter!"));
                 XX_Free(p_TrafficAnalyzerPort);
                 XX_Free(p_TrafficAnalyzerPort->p_Queues);
                 XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
                 return NULL;
             }
             memset(p_TrafficAnalyzerQueue->p_RxFrameTypeCounter, 0, p_TrafficAnalyzerQueue->numOfFrameListSubSet * sizeof(uint32_t));

             for(frameDescrIndex = 0; frameDescrIndex < p_TrafficAnalyzerQueue->numOfFrameListSubSet; frameDescrIndex++ )
             {
                 /* Calculate the sum of weights of all id's */
                 p_TrafficAnalyzerQueue->rxTotalWeightFrames +=  p_TrafficAnalyzerQueue->p_PgFrameWeights[frameDescrIndex];
             }
         }

         INIT_LIST(&p_TrafficAnalyzerQueue->compDepthFramesLst);

         if(p_TrafficAnalyzerQueue->p_RxDataFramesLst)
         {
             p_FramesList = (t_List *)p_TrafficAnalyzerQueue->p_RxDataFramesLst;

             LIST_FOR_EACH(p_Pos, p_FramesList)
             {
                 p_TrafficAnalyzerFrame = RECV_F_TRAFFIC_ANALYZER_FRAME_OBJECT(p_Pos);

                 /*  Update the Rx bytes  */
                 tmpRxBytes += F_GetLength(p_TrafficAnalyzerFrame->h_Frame);
                 ++totalWeight;
             }
         }
     }

     if (!IN_RANGE(TA_MIN_LINE_RATE,p_TrafficAnalyzerPortParam->lineRate,TA_MAX_LINE_RATE))
     {
        REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, ("Line rate!!!"));
        TA_PORT_Free(p_TrafficAnalyzerPort);
        return NULL;
     }

    /* The time needed for one byte to be sent - in nsecs */
     p_TrafficAnalyzerPort->byteTime = (uint32_t)(TA_BPS_TO_NANO_SEC_FACTOR / p_TrafficAnalyzerPortParam->lineRate);

     /* Calculate the number of bytes expected */
     if((p_TrafficAnalyzerPort->numOfFrames % totalWeight) == 0)
     {
        p_TrafficAnalyzerPort->rxBytes = (p_TrafficAnalyzerPort->numOfFrames / totalWeight) * tmpRxBytes;
     }
     else
     {
        p_TrafficAnalyzerPort->rxBytes = ((p_TrafficAnalyzerPort->numOfFrames / totalWeight)+1) * tmpRxBytes;
     }


    return p_TrafficAnalyzerPort;
}

void TA_PORT_Free(t_Handle h_TaPort)
{
     t_TrafficAnalyzerPort   *p_TrafficAnalyzerPort = (t_TrafficAnalyzerPort *)h_TaPort;
     t_TrafficAnalyzerQueue  *p_TrafficAnalyzerQueue;
     int                     j;
     t_TrafficPort           *p_TrafficPort = NULL;
     t_PgFrame               *p_CompDepthFrame;

     for (j=0; j < p_TrafficAnalyzerPort->numOfQueues; j++)
     {
         p_TrafficAnalyzerQueue = &p_TrafficAnalyzerPort->p_Queues[j];

         p_CompDepthFrame = DequeueFrameFromCompDepthQ(p_TrafficAnalyzerQueue);
         while (p_CompDepthFrame)
         {
             PG_DestroyFrame(p_CompDepthFrame->h_Frame);
             XX_Free(p_CompDepthFrame);
             p_CompDepthFrame = DequeueFrameFromCompDepthQ(p_TrafficAnalyzerQueue);
         }
         if(p_TrafficAnalyzerQueue->p_RxFrameTypeCounter)
         {
             XX_Free(p_TrafficAnalyzerQueue->p_RxFrameTypeCounter);
         }
     }

     if(p_TrafficAnalyzerPort->p_Queues)
     {
        XX_Free(p_TrafficAnalyzerPort->p_Queues);
     }
     if(p_TrafficAnalyzerPort->p_QueueIndexMap)
     {
        XX_Free(p_TrafficAnalyzerPort->p_QueueIndexMap);
     }
     if(p_TrafficAnalyzerPort->p_EnableFlexibleRxMode)
     {
        XX_Free(p_TrafficAnalyzerPort->p_EnableFlexibleRxMode);
     }

     p_TrafficPort = ITG_RemovePortFromList(h_TaPort, &p_TrafficAnalyzerPort->p_TrafficAnalyzer->portsLst);
     if (p_TrafficPort)
     {
         p_TrafficAnalyzerPort->p_TrafficAnalyzer->numOfPorts--;

         XX_Free(p_TrafficPort->h_Port);
         XX_Free(p_TrafficPort);
     }
}

t_Error TA_Run(t_Handle h_Ta)
{
     t_TrafficAnalyzer      *p_TrafficAnalyzer = (t_TrafficAnalyzer *)(h_Ta);
     t_TrafficAnalyzerPort  *p_TrafficAnalyzerPort;
     t_TrafficAnalyzerQueue *p_TrafficAnalyzerQueue = NULL;
     t_Handle               h_Timer = NULL;

     uint16_t           portId = 0;
     uint32_t           queueId = 0;

     p_TrafficAnalyzer->analyzerStopped = FALSE;

     for(portId = 0; portId < p_TrafficAnalyzer->numOfPorts; portId++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);

         p_TrafficAnalyzerPort->testPassed = TRUE;

         memset(&p_TrafficAnalyzerPort->rxStatistics,0,sizeof(t_TrafficAnalyzerRxStatistics));

         for(queueId = 0;queueId < p_TrafficAnalyzerPort->numOfQueues;queueId++)
         {
             p_TrafficAnalyzerQueue = &p_TrafficAnalyzerPort->p_Queues[queueId];

             p_TrafficAnalyzerQueue->rxFrames = 0;
             p_TrafficAnalyzerQueue->rxErrors = 0;
             p_TrafficAnalyzerQueue->compDepthCnt = 0;

             /* If flexibleRxMode is enabled, rechaining Rx list is not required, as the frames are received in random */
             if(!p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueId])
             {
                 if( LIST_FIRST(p_TrafficAnalyzerQueue->p_RxDataFramesLst) != p_TrafficAnalyzerQueue->p_FirstNode )
                 {
                     /* Re-chaining Rx list for the successive run */
                     ITG_RechainList(p_TrafficAnalyzerQueue->p_RxDataFramesLst,p_TrafficAnalyzerQueue->p_FirstNode);
                 }
             }
             else
             {
                /* Initialize p_RxFrameTypeCounter with values of 0 */
                memset(p_TrafficAnalyzerQueue->p_RxFrameTypeCounter, 0, p_TrafficAnalyzerQueue->numOfFrameListSubSet * sizeof(uint32_t));
             }
         }
     }

     memset(&p_TrafficAnalyzer->globalRxStatistics,0,sizeof(t_TrafficAnalyzerRxStatistics));

     if(p_TrafficAnalyzer->pollingMode)
     {

         h_Timer = XX_CreateTimer();
         if(!h_Timer)
         {
             RETURN_ERROR(MINOR, E_INVALID_HANDLE, ("Timer Init") );
         }

         p_TrafficAnalyzer->h_Timer = h_Timer;
         XX_StartTimer(h_Timer, p_TrafficAnalyzer->pollingPeriod, FALSE, (void (*)(t_Handle))PollReceiveFrame, h_Ta);
     }
     return E_OK;
}

void TA_Stop(t_Handle h_Ta)
{
     t_TrafficAnalyzer *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;

     p_TrafficAnalyzer->analyzerStopped = TRUE;
}

bool TA_WaitOnAllPorts(t_Handle h_Ta)
{
    t_TrafficAnalyzer       *p_TrafficAnalyzer = (t_TrafficAnalyzer *)(h_Ta);
    t_TrafficAnalyzerPort   *p_TrafficAnalyzerPort;
    t_TrafficAnalyzerQueue  *p_TrafficAnalyzerQueue = NULL;
    bool                    testPassed = TRUE, localKeepGoing = TRUE;
    uint16_t                portId = 0;
    uint32_t                prevRxCount = 0, timeToWait = 0, queueId = 0;
    t_Handle                h_Timer = NULL;

    uint32_t                frameDescrIndex = 0;
    uint32_t                totalRxFrames = 0;
    uint32_t                typeNoOfFrames = 0;
    uint32_t                maxTypeNoOfFrames = 0;
    uint32_t                lastFewFrames = 0;
    uint32_t                multFactor = 0;
    bool                    weightMismatch = FALSE;

     for(portId = 0; portId < p_TrafficAnalyzer->numOfPorts; portId++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);

         if(p_TrafficAnalyzerPort->inUse)
         {
             /* We need to compute the time we need to wait in micro-secs */
             p_TrafficAnalyzer->timeToWait = (uint32_t)((p_TrafficAnalyzerPort->byteTime * p_TrafficAnalyzerPort->rxBytes)/ 1000);
             if(p_TrafficAnalyzer->timeToWait < OPTIMAL_ANALYZER_TIME_TO_WAIT)
             {
                p_TrafficAnalyzer->timeToWait = OPTIMAL_ANALYZER_TIME_TO_WAIT;
             }

             while (!p_TrafficAnalyzer->analyzerStopped)
             {
                 if ((p_TrafficAnalyzerPort->rxStatistics.rxFrames + p_TrafficAnalyzerPort->rxStatistics.rxErrors)==
                         p_TrafficAnalyzerPort->numOfFrames)
                 {
                     //XX_Print("Port %d Passed\n", p_TrafficAnalyzerPort->portId);
                     p_TrafficAnalyzerPort->testPassed = TRUE;
                     if(p_TrafficAnalyzerPort->rxStatistics.rxErrors)
                     {
                         p_TrafficAnalyzerPort->testPassed = FALSE;
                     }
                     break;
                 }
                 else
                 {
                     timeToWait = p_TrafficAnalyzer->timeToWait/100;
                     localKeepGoing = FALSE;
                     while(--timeToWait && !localKeepGoing && !p_TrafficAnalyzer->analyzerStopped)
                     {
                         if (prevRxCount == (p_TrafficAnalyzerPort->numOfFrames-((p_TrafficAnalyzerPort->numOfFrames)% p_TrafficAnalyzer->comparisonDepth) ))
                         {
                             CompareDepthQRxFrames(h_Ta);
                         }
                         if (prevRxCount == p_TrafficAnalyzerPort->rxStatistics.rxFrames +
                                                 p_TrafficAnalyzerPort->rxStatistics.rxErrors)
                         {
                             //XX_Print("Port %d prevRxCount (1) = %d\n", p_TrafficAnalyzerPort->portId, prevRxCount);
                             XX_UDelay(100);
                             continue;
                         }
                         else
                         {
                             localKeepGoing = TRUE;
                             break;
                         }
                     }
                 }
                 if(!p_TrafficAnalyzer->analyzerStopped)
                 {
                     if (!localKeepGoing)
                     {
                         XX_Print("Traffic Analyzer Timed Out!\n");
                         p_TrafficAnalyzerPort->testPassed = FALSE;
                         break;
                     }
                     else
                     {
                         prevRxCount = p_TrafficAnalyzerPort->rxStatistics.rxFrames +
                                            p_TrafficAnalyzerPort->rxStatistics.rxErrors;

                         //XX_Print("Port %d prevRxCount (2) = %d\n", p_TrafficAnalyzerPort->portId, prevRxCount);
                     }
                 }
             }

             /* Compare the remaining frames after analyzer is stopped */
             if(p_TrafficAnalyzer->analyzerStopped)
             {
                 CompareDepthQRxFrames(h_Ta);
             }
             for(queueId = 0;queueId < p_TrafficAnalyzerPort->numOfQueues;queueId++)
             {
                 PRINT_RX_PORT_STATUS(p_TrafficAnalyzerPort, queueId, " S");
                 XX_Print("\n");
             }

             if (!p_TrafficAnalyzerPort->testPassed)
             {
                 testPassed = FALSE;
             }
         }
     }

     if(p_TrafficAnalyzer->pollingMode)
     {
         h_Timer = p_TrafficAnalyzer->h_Timer;

         XX_StopTimer(h_Timer);
         XX_FreeTimer(h_Timer);
     }

     /* Ensuring weight of frames received */
     for(portId = 0; portId < p_TrafficAnalyzer->numOfPorts; portId++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portId);
         for (queueId = 0; queueId < p_TrafficAnalyzerPort->numOfQueues; queueId++)
         {
             if (p_TrafficAnalyzerPort->p_EnableFlexibleRxMode[queueId] == TRUE)
             {
                 p_TrafficAnalyzerQueue = &p_TrafficAnalyzerPort->p_Queues[queueId];
                 totalRxFrames = p_TrafficAnalyzerQueue->rxFrames + p_TrafficAnalyzerQueue->rxErrors;
                 /* Calculate multiplication factor */
                 multFactor = totalRxFrames / p_TrafficAnalyzerQueue->rxTotalWeightFrames;
                 /* Calculate remainder number of frames after W frames */
                 lastFewFrames = totalRxFrames % p_TrafficAnalyzerQueue->rxTotalWeightFrames;

                 for(frameDescrIndex = 0; frameDescrIndex < p_TrafficAnalyzerQueue->numOfFrameListSubSet; frameDescrIndex++)
                 {
                     typeNoOfFrames = multFactor * p_TrafficAnalyzerQueue->p_PgFrameWeights[frameDescrIndex];

                     if(lastFewFrames)
                     {
                        /* If remainder frames available, the received frames should be between typeNoOfFrames and typeNoOfFrames + corresponding weight */
                        maxTypeNoOfFrames = typeNoOfFrames + p_TrafficAnalyzerQueue->p_PgFrameWeights[frameDescrIndex];
                        if( !((p_TrafficAnalyzerQueue->p_RxFrameTypeCounter[frameDescrIndex] >= typeNoOfFrames) &&
                            (p_TrafficAnalyzerQueue->p_RxFrameTypeCounter[frameDescrIndex] <= maxTypeNoOfFrames)) )
                        {
#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
                            XX_Print("Port %d Queue %d Type %d: Weight mismatch",portId,queueId,frameDescrIndex);
    #endif /* (defined(DEBUG_ERRORS) && ... */
                            weightMismatch = TRUE;
                        }
                     }
                     else
                     {
                        /* If remainder frames not available, the received frames should be equal to typeNoOfFrames */
                        if( !(p_TrafficAnalyzerQueue->p_RxFrameTypeCounter[frameDescrIndex] == typeNoOfFrames) )
                        {
#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
                            XX_Print("Port %d Queue %d Type %d: Weight mismatch",portId,queueId,frameDescrIndex);
    #endif /* (defined(DEBUG_ERRORS) && ... */
                            weightMismatch = TRUE;
                        }
                     }
                 }
             }
         }
     }
     if(weightMismatch)
     {
         XX_Print("Weight mismatch\n");
         p_TrafficAnalyzerPort->testPassed = FALSE;
         testPassed = FALSE;
     }
     return testPassed;
}

t_Error TA_GetStatistics( t_Handle h_Ta, t_TrafficAnalyzerRxStatistics *p_AnalyzerStats)
{
    t_TrafficAnalyzer   *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;

    if(!p_TrafficAnalyzer || !p_AnalyzerStats)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
        return E_INVALID_HANDLE;
    }

    memcpy(p_AnalyzerStats,&(p_TrafficAnalyzer->globalRxStatistics), sizeof(t_TrafficAnalyzerRxStatistics));

    return E_OK;
}

t_Error TA_PrintRxStatistics(t_Handle h_Ta)
{
     t_TrafficAnalyzer         *p_TrafficAnalyzer = (t_TrafficAnalyzer *)h_Ta;
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort;
     int i = 0;

     /* Check the handle */
     SANITY_CHECK_RETURN_ERROR(p_TrafficAnalyzer, E_INVALID_HANDLE);

     for(i = 0; i < p_TrafficAnalyzer->numOfPorts; i++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,i);
         XX_Print("\n===============================\n");
         XX_Print("Rx Stats : PORT %d\n", p_TrafficAnalyzerPort->portId);
         XX_Print("===============================\n");
         PRINT_STAT("Rx frames      : %d\n",p_TrafficAnalyzerPort->rxStatistics.rxFrames);
         PRINT_STAT("Rx error frames: %d\n",p_TrafficAnalyzerPort->rxStatistics.rxErrors);
         PRINT_STAT("Rx octets      : %d\n",p_TrafficAnalyzerPort->rxStatistics.rxOctets);
         PRINT_STAT("Rx Seq payload frames   : %d\n",p_TrafficAnalyzerPort->rxStatistics.seqPayloadCnt);
         PRINT_STAT("Rx fixed payload frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.fixedPayloadCnt);
         PRINT_STAT("Rx Undersized frames    : %d\n",p_TrafficAnalyzerPort->rxStatistics.rxUndersizedFrames);
         PRINT_STAT("Rx Oversized frames     : %d\n",p_TrafficAnalyzerPort->rxStatistics.rxOversizedFrames);
         PRINT_STAT("Rx Jumbo frames         : %d\n",p_TrafficAnalyzerPort->rxStatistics.rxJumboFrames);
         PRINT_STAT("Fixed pattern (0x5A5A) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeOneCnt);
         PRINT_STAT("Fixed pattern (0x55AA) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeTwoCnt);
         PRINT_STAT("Fixed pattern (0x5555) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeThreeCnt);
         PRINT_STAT("Fixed pattern (0x00FF) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeFourCnt);
         PRINT_STAT("Fixed pattern (0xF0F0) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeFiveCnt);
         PRINT_STAT("Fixed pattern (0x0055AAFF) frames : %d\n",p_TrafficAnalyzerPort->rxStatistics.patternTypeSixCnt);
     }
     XX_Print("\n===============================\n");
     XX_Print("Rx Stats across all ports\n");
     XX_Print("===============================\n");
     PRINT_STAT("Rx frames      : %d\n",p_TrafficAnalyzer->globalRxStatistics.rxFrames);
     PRINT_STAT("Rx error frames: %d\n",p_TrafficAnalyzer->globalRxStatistics.rxErrors);
     PRINT_STAT("Rx octets      : %d\n",p_TrafficAnalyzer->globalRxStatistics.rxOctets);
     PRINT_STAT("Rx Seq payload frames   : %d\n",p_TrafficAnalyzer->globalRxStatistics.seqPayloadCnt);
     PRINT_STAT("Rx fixed payload frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.fixedPayloadCnt);
     PRINT_STAT("Rx Undersized frames    : %d\n",p_TrafficAnalyzer->globalRxStatistics.rxUndersizedFrames);
     PRINT_STAT("Rx Oversized frames     : %d\n",p_TrafficAnalyzer->globalRxStatistics.rxOversizedFrames);
     PRINT_STAT("Rx Jumbo frames         : %d\n",p_TrafficAnalyzer->globalRxStatistics.rxJumboFrames);
     PRINT_STAT("Fixed pattern (0x5A5A) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeOneCnt);
     PRINT_STAT("Fixed pattern (0x55AA) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeTwoCnt);
     PRINT_STAT("Fixed pattern (0x5555) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeThreeCnt);
     PRINT_STAT("Fixed pattern (0x00FF) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeFourCnt);
     PRINT_STAT("Fixed pattern (0xF0F0) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeFiveCnt);
     PRINT_STAT("Fixed pattern (0x0055AAFF) frames : %d\n",p_TrafficAnalyzer->globalRxStatistics.patternTypeSixCnt);

     return E_OK;
}

t_Handle TA_GetPortHandle(t_Handle h_Ta, uint16_t portId)
{
     t_TrafficAnalyzer      *p_TrafficAnalyzer      = (t_TrafficAnalyzer *)h_Ta;
     t_TrafficAnalyzerPort  *p_TrafficAnalyzerPort  = NULL;

     uint16_t       portIndex = 0;
     bool           portFound = FALSE;

     for(portIndex = 0; portIndex < p_TrafficAnalyzer->numOfPorts; portIndex++)
     {
         p_TrafficAnalyzerPort = GET_NTH_ANALYZER_PORT_HANDLE(&p_TrafficAnalyzer->portsLst,portIndex);
         if(p_TrafficAnalyzerPort->portId == portId)
         {
             portFound = TRUE;
             break;
         }
     }

     if(!portFound)
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid Port Id"));
        return NULL;
     }
     return p_TrafficAnalyzerPort;
}

void TA_PORT_Enable(t_Handle h_TaPort)
{
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort = (t_TrafficAnalyzerPort *)h_TaPort;

     if(!p_TrafficAnalyzerPort)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid handle"));
     }
     else
     {
         p_TrafficAnalyzerPort->inUse = TRUE;
     }
}

void TA_PORT_Disable(t_Handle h_TaPort)
{
     t_TrafficAnalyzerPort     *p_TrafficAnalyzerPort = (t_TrafficAnalyzerPort *)h_TaPort;

     if(!p_TrafficAnalyzerPort)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid handle"));
     }
     else
     {
         p_TrafficAnalyzerPort->inUse = FALSE;
     }
}

t_Error TA_PORT_GetStatistics(t_Handle h_TrafficPort, t_TrafficAnalyzerRxStatistics *p_AnalyzerPortStats)
{
    t_TrafficAnalyzerPort    *p_TrafficAnalyzerPort =(t_TrafficAnalyzerPort *) h_TrafficPort;   /* Port handle */

    if(!h_TrafficPort || !p_AnalyzerPortStats)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
        return E_INVALID_HANDLE;
    }

    memcpy(p_AnalyzerPortStats,&(p_TrafficAnalyzerPort->rxStatistics), sizeof(t_TrafficAnalyzerRxStatistics));

    return E_OK;
}

e_RxStoreResponse TA_PORT_ReceiveFrame(t_Handle h_Port, uint32_t queueId, t_Handle h_Frame)
{
     t_TrafficAnalyzer       *p_TrafficAnalyzer;
     t_TrafficAnalyzerPort   *p_TrafficAnalyzerPort = (t_TrafficAnalyzerPort *)h_Port;
     t_PgFrame               *p_CompDepthFrame;
     uint32_t                status;
     uint16_t                packetsCompared = 0;
     uint32_t                queueIndex = 0;
     bool                    equals = TRUE;
     bool                    nonConfiguredQueue = FALSE;

     p_TrafficAnalyzer = (t_TrafficAnalyzer *)p_TrafficAnalyzerPort->p_TrafficAnalyzer;

     if( !(queueId >= p_TrafficAnalyzerPort->maxQueueIndex) )
     {
        queueIndex = p_TrafficAnalyzerPort->p_QueueIndexMap[queueId];
     }
     else
     {
         nonConfiguredQueue = TRUE;
     }

     /* Determine if frame is received from a non-configured queue */
     if(nonConfiguredQueue || p_TrafficAnalyzerPort->p_Queues[queueIndex].queueId != queueId)
     {
#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
         XX_Print("Frame received in a non-configured queue\n");
#endif /* (defined(DEBUG_ERRORS) && ... */
         if (p_TrafficAnalyzerPort->f_DevRxConf)
             p_TrafficAnalyzerPort->f_DevRxConf(p_TrafficAnalyzerPort->h_Dev, queueId, h_Frame);
         else
             PG_DestroyFrame(h_Frame);
         return e_RX_STORE_RESPONSE_CONTINUE;
     }

     //p_TrafficAnalyzerPort->testPassed = TRUE;

     status = F_GetStatus(h_Frame);
     if (status)
     {
         XX_Print("Port %d: Frame - error: 0x%04x\n", p_TrafficAnalyzerPort->portId, status);
         equals = FALSE;
         if (p_TrafficAnalyzerPort->f_DevRxConf)
             p_TrafficAnalyzerPort->f_DevRxConf(p_TrafficAnalyzerPort->h_Dev, queueId, h_Frame);
         else
             PG_DestroyFrame(h_Frame);
     }
     else
     {
         /* Store received frames in Comp Depth Queue if Comparison Depth is greater than 1.
            If not immediately compare frames */
         if (p_TrafficAnalyzer->comparisonDepth > 1)
         {
             p_TrafficAnalyzerPort->p_Queues[queueIndex].compDepthCnt++;

             p_CompDepthFrame = PG_CreateFrame();
             if (!p_CompDepthFrame)
             {
                 REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("Traffic-Comp Depth frame!"));
                 return e_RX_STORE_RESPONSE_PAUSE;
             }

             p_CompDepthFrame->h_Frame = h_Frame;

             if(p_TrafficAnalyzerPort->p_Queues[queueIndex].compDepthCnt < p_TrafficAnalyzer->comparisonDepth)
             {
                 EnqueueFrameToCompDepthQ(&p_TrafficAnalyzerPort->p_Queues[queueIndex], p_CompDepthFrame);
             }
             else
             {
                 /* The current frame is enqueued to the list before other frames are dequeued */
                 EnqueueFrameToCompDepthQ(&p_TrafficAnalyzerPort->p_Queues[queueIndex], p_CompDepthFrame);

                 do
                 {
                     p_CompDepthFrame = DequeueFrameFromCompDepthQ(&p_TrafficAnalyzerPort->p_Queues[queueIndex]);

                     DequeueAndCompareRxFrame(p_TrafficAnalyzer, p_TrafficAnalyzerPort->portId, queueIndex, p_CompDepthFrame->h_Frame);
                     XX_Free(p_CompDepthFrame);

                 } while(++packetsCompared < p_TrafficAnalyzer->comparisonDepth);

                 p_TrafficAnalyzerPort->p_Queues[queueIndex].compDepthCnt -= p_TrafficAnalyzer->comparisonDepth;
             }
         }
         else
         {
             /* TBD : return value check */
             DequeueAndCompareRxFrame(p_TrafficAnalyzer, p_TrafficAnalyzerPort->portId, queueIndex, h_Frame);
         }

     }
     if (!equals)
     {
         p_TrafficAnalyzerPort->p_Queues[queueIndex].rxErrors++;
         p_TrafficAnalyzerPort->rxStatistics.rxErrors++;
         p_TrafficAnalyzer->globalRxStatistics.rxErrors++;
         p_TrafficAnalyzerPort->testPassed = FALSE;
     }
     return e_RX_STORE_RESPONSE_CONTINUE;
}


