/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
  @File          traffic_generator.c

  @Description   Traffic Generator functions
*//***************************************************************************/
#include "std_ext.h"
#include "stdlib_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "frame_ext.h"
#include "fmanip_ext.h"
#include "iphc_manip.h"
#include "itg_common.h"
#include "traffic_generator.h"

/**************************************************************************//**
 @Description    Port and Queue Defines
*//***************************************************************************/
#define PRINT_TX_PORT_STATUS(p_TrafficGenPort, queueIndex, r)                   \
     XX_Print("Port %02d Queue %02d" r ": Tx-%d,TC-%d,TE-%d\r",                 \
            p_TrafficGenPort->portId,                                        \
            p_TrafficGenPort->p_Queues[queueIndex].queueId,                                                         \
            p_TrafficGenPort->p_Queues[queueIndex].txFrames,                      \
            p_TrafficGenPort->p_Queues[queueIndex].txConfedFrames,                \
            p_TrafficGenPort->p_Queues[queueIndex].txErrors)

/**************************************************************************//**
 @Description    TG functions
*//***************************************************************************/

static void AddFrameToConfLst(t_TrafficGenQueue *p_TrafficGenQueue, t_PgFrame *p_TrafficGenFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_TrafficGenFrame->frmsNode, &p_TrafficGenQueue->confFramesLst);
     XX_RestoreAllIntr(intFlags);
}

static void RemoveFrameFromConfLst(t_PgFrame *p_TrafficGenFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_DelAndInit(&p_TrafficGenFrame->frmsNode);
     XX_RestoreAllIntr(intFlags);
}

static t_PgFrame * RemoveNextFrameFromConfLst(t_TrafficGenQueue *p_TrafficGenQueue)
{
     t_PgFrame      *p_TrafficGenFrame = NULL;
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(&p_TrafficGenQueue->confFramesLst))
     {
         p_TrafficGenFrame = CONF_F_TRAFFIC_GEN_FRAME_OBJECT(p_TrafficGenQueue->confFramesLst.p_Next);
         LIST_DelAndInit(&p_TrafficGenFrame->frmsNode);
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficGenFrame;
}

static void EnqueueFrameToReadyQ(t_TrafficGenQueue *p_TrafficGenQueue, t_PgFrame *p_TrafficGenFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_AddToTail(&p_TrafficGenFrame->frmsNode, p_TrafficGenQueue->p_EmptyFramesLst);
     if(p_TrafficGenQueue->txShuffleFrames)
     {
         p_TrafficGenQueue->txConfFrameCounter++;
     }
     XX_RestoreAllIntr(intFlags);
}

static void AddFrameToHeadOfReadyQ(t_TrafficGenQueue *p_TrafficGenQueue, t_PgFrame *p_TrafficGenFrame)
{
     uint32_t       intFlags;

     intFlags = XX_DisableAllIntr();
     LIST_Add(&p_TrafficGenFrame->frmsNode, p_TrafficGenQueue->p_EmptyFramesLst);
     if(p_TrafficGenQueue->txShuffleFrames)
     {
         p_TrafficGenQueue->txFrameCounter++;
     }
     XX_RestoreAllIntr(intFlags);
}

static t_PgFrame * DequeueFrameFromReadyQ(t_TrafficGenQueue *p_TrafficGenQueue)
{
     t_PgFrame       *p_TrafficGenFrame = NULL;
     uint32_t        intFlags;
     t_List          *p_Pos;
     uint32_t        i,nthFrame = 0;

     intFlags = XX_DisableAllIntr();
     if (!LIST_IsEmpty(p_TrafficGenQueue->p_EmptyFramesLst))
     {
         p_Pos = p_TrafficGenQueue->p_EmptyFramesLst->p_Next;

         if(p_TrafficGenQueue->txShuffleFrames)
         {
             /* Transmit every txFramesInLst in shuffle mode. After transmitting the entire list
                reload txFrameCounter value by frames already confirmed. Remember remaining
                number of frames from the list to be added to dynamically without waiting to
                entire list transmission. */

             if (p_TrafficGenQueue->txLeftoverCounter && p_TrafficGenQueue->txConfFrameCounter)
             {
                /* Check how many frames can be added dynamically */
                if (p_TrafficGenQueue->txConfFrameCounter >= p_TrafficGenQueue->txLeftoverCounter)
                {
                    p_TrafficGenQueue->txFrameCounter += p_TrafficGenQueue->txLeftoverCounter;
                    p_TrafficGenQueue->txConfFrameCounter -= p_TrafficGenQueue->txLeftoverCounter;
                    p_TrafficGenQueue->txLeftoverCounter = 0;
                }
                else
                {
                    p_TrafficGenQueue->txFrameCounter += p_TrafficGenQueue->txConfFrameCounter;
                    p_TrafficGenQueue->txLeftoverCounter -= p_TrafficGenQueue->txConfFrameCounter;
                    p_TrafficGenQueue->txConfFrameCounter = 0;
                }
             }

             if(p_TrafficGenQueue->txFrameCounter == 0)
             {
                 /* On transmission of each pattern(txFramesInLst) refresh the counter values */
                 if (p_TrafficGenQueue->txConfFrameCounter)
                 {
                    p_TrafficGenQueue->txFrameCounter = p_TrafficGenQueue->txConfFrameCounter;
                    p_TrafficGenQueue->txLeftoverCounter = p_TrafficGenQueue->txFramesInLst -
                        p_TrafficGenQueue->txConfFrameCounter;
                    p_TrafficGenQueue->txConfFrameCounter = 0;
                 }
                 else
                 {
                     XX_RestoreAllIntr(intFlags);
                     return p_TrafficGenFrame;
                 }
             }

             nthFrame = (rand() % (p_TrafficGenQueue->txFrameCounter));
             /* Get the nth node in the List */
             for(i = 0 ; i < nthFrame ; p_Pos = LIST_NEXT(p_Pos), i++)
                ;
             p_TrafficGenQueue->txFrameCounter--;
         }

         p_TrafficGenFrame = RDY_F_TRAFFIC_GEN_FRAME_OBJECT(p_Pos);
         LIST_DelAndInit(&p_TrafficGenFrame->frmsNode);
     }

     XX_RestoreAllIntr(intFlags);
     return p_TrafficGenFrame;
}

static void UpdateTxStats(t_TrafficGen *p_TrafficGen, t_Handle h_TxFrame, uint16_t frameSize, uint16_t portId)
{
     e_PgHeaderType                 *p_FrameHeaderType = NULL;              /* Holds the frame encapsulation */
     e_PgHeaderType                 hdrType        = e_HEADER_TYPE_NONE;    /* Header type */
     t_TrafficGenTxStatistics       *p_PortStats   = NULL;                  /* Pointer to Tx port statistics */
     t_TrafficGenTxStatistics       *p_GlobalStats = NULL;                  /* Pointer to Tx global statistics */
     t_TrafficFrameInfo             *p_FrameInfo   = NULL;                  /* Holds the frame informations */
     t_TrafficGenPort               *p_TrafficGenPort;

     uint32_t                       hdrIndex  = 0;                          /* Loop index */
     bool                           ethHdrEncountered  = FALSE;             /* Flag to indicate Ethernet header has been encountered */
     bool                           vlanHdrEncountered = FALSE;             /* Flag to indicate VLAN header has been encountered */

     uint32_t                       *p_TxPortStats        = NULL;           /* Tx port statistics counter */
     uint32_t                       *p_TxGlobalStats      = NULL;           /* Tx global statistics counter */
     uint32_t                       *p_TxPortErrorStats   = NULL;           /* Tx port error statistics counter */
     uint32_t                       *p_TxGlobalErrorStats = NULL;           /* Tx global error statistics counter */

     uint32_t                       frameError = 0;                         /* Holds the frame error types */
     uint32_t                       errorType  = 1;                         /* To check if the error is set */
     uint32_t                       index      = 0;                         /* Array index */


     /* Check the handles */
     ASSERT_COND(h_TxFrame);
     ASSERT_COND(p_TrafficGen);

     p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

     /* Get the base of the transmit statistics structure */
     p_PortStats = &p_TrafficGenPort->txStatistics;
     p_GlobalStats = &p_TrafficGen->globalTxStatistics;

     p_TxPortErrorStats = &p_TrafficGenPort->txStatistics.txIPChecksumErrors;
     p_TxGlobalErrorStats = &p_TrafficGen->globalTxStatistics.txIPChecksumErrors;

     p_TxPortStats = &p_TrafficGenPort->txStatistics.txEthernetFrames;
     p_TxGlobalStats = &p_TrafficGen->globalTxStatistics.txEthernetFrames;

     /* Get all frame information */
     p_FrameInfo = F_GetFrameInfo(h_TxFrame);

     /* Get the frame encapsulation */
     p_FrameHeaderType = (e_PgHeaderType *)p_FrameInfo->h_FrameHeaderType;

     /* Get the frame error types */
     frameError = p_FrameInfo->frameErrorType;

     /* Extract the header type */
     while ((hdrType = p_FrameHeaderType[hdrIndex++]) != e_HEADER_TYPE_NONE)
     {
         index = hdrType;

         if((hdrIndex == 1) && (hdrType == e_HEADER_TYPE_ETH))
         {
             ethHdrEncountered = TRUE;
         }

         if(hdrType != e_HEADER_TYPE_PAYLOAD)
         {
             /* When multiple VLAN headers are encountered for a frame,
             increment the VLAN counter only once */
             if((hdrType == e_HEADER_TYPE_VLAN) && vlanHdrEncountered)
             {
                continue;
             }
             else if(hdrType == e_HEADER_TYPE_VLAN)
             {
                vlanHdrEncountered = TRUE;
             }

             /* Update the frame counter */
             p_TxPortStats[index-2]++;
             p_TxGlobalStats[index-2]++;
         }
     }

     /* Update the error statistics */
     if(frameError)
     {
         index = 0;
         while(errorType != e_ERROR_TYPE_NONE)
         {
            /* Check if the error bit is set for each error type */
            if(frameError & errorType)
            {
                 p_TxPortErrorStats[index]++;
                 p_TxGlobalErrorStats[index]++;
            }
            index++;
            errorType <<= 1;
         }
     }

     /* Update the undersize, oversize and jumbo frame counters based on the frame size */
     if(frameSize < p_TrafficGenPort->underSizeFrameValue)
     {
         p_PortStats->txUndersizedFrames++;
         p_GlobalStats->txUndersizedFrames++;
     }
     else if(frameSize > p_TrafficGenPort->overSizeFrameValue)
     {
         p_PortStats->txOversizedFrames++;
         p_GlobalStats->txOversizedFrames++;
     }

     /* Update the jumbo frame counter only for Ethernet frames */
     if(ethHdrEncountered && (frameSize >= JUMBO_FRAME_SIZE))
     {
         p_PortStats->txJumboFrames++;
         p_GlobalStats->txJumboFrames++;
     }

     /* Update the Tx Octets */
     p_PortStats->txOctets += frameSize;
     p_GlobalStats->txOctets += frameSize;
}

static void FrameStatistics(t_TrafficGenTxStatistics transmitStatistics)
{
    PRINT_STAT("Ethernet      : %d\n",transmitStatistics.txEthernetFrames);
    PRINT_STAT("Ethernet 802.3: %d\n",transmitStatistics.txEthernet802_3Frames);
    PRINT_STAT("PTP Ethernet  : %d\n",transmitStatistics.txPTPEthernetFrames);
    PRINT_STAT("VLAN          : %d\n",transmitStatistics.txVLANFrames);
    PRINT_STAT("VTagged802.3  : %d\n",transmitStatistics.txVTagged802_3Frames);
    PRINT_STAT("MPLS          : %d\n",transmitStatistics.txMPLSFrames);
    PRINT_STAT("PPPoE         : %d\n",transmitStatistics.txPPPoEFrames);
    PRINT_STAT("LLC           : %d\n",transmitStatistics.txLLCFrames);
    PRINT_STAT("NLP           : %d\n",transmitStatistics.txNLPFrames);
    PRINT_STAT("SNAP          : %d\n",transmitStatistics.txSNAPFrames);
    PRINT_STAT("HDLC          : %d\n",transmitStatistics.txHDLCFrames);
    PRINT_STAT("PPP           : %d\n",transmitStatistics.txPPPFrames);
    PRINT_STAT("PPP ML/MC     : %d\n",transmitStatistics.txPPPMLMCFrames);
    PRINT_STAT("IPv4          : %d\n",transmitStatistics.txIPv4Frames);
    PRINT_STAT("IPv6          : %d\n",transmitStatistics.txIPv6Frames);
    PRINT_STAT("TCP           : %d\n",transmitStatistics.txTCPFrames);
    PRINT_STAT("UDP           : %d\n",transmitStatistics.txUDPFrames);
    PRINT_STAT("SCTP          : %d\n",transmitStatistics.txSCTPFrames);
    PRINT_STAT("SCTP DATA     : %d\n",transmitStatistics.txSCTPDataFrames);
    PRINT_STAT("L2TPv2        : %d\n",transmitStatistics.txL2TPv2Frames);
    PRINT_STAT("L2TPv3        : %d\n",transmitStatistics.txL2TPv3Frames);
    PRINT_STAT("Pad           : %d\n",transmitStatistics.txPaddingFrames);
    PRINT_STAT("Tunneled IPv4    : %d\n",transmitStatistics.txTunneledIPv4Frames);
    PRINT_STAT("Tunneled IPv6    : %d\n",transmitStatistics.txTunneledIPv6Frames);
    PRINT_STAT("PTP V1 General UDP  : %d\n",transmitStatistics.txPTPV1GeneralUDPFrames);
    PRINT_STAT("PTP V2 General UDP  : %d\n",transmitStatistics.txPTPV2GeneralUDPFrames);
    PRINT_STAT("PTP V1 Event UDP    : %d\n",transmitStatistics.txPTPV1EventUDPFrames);
    PRINT_STAT("PTP V2 Event UDP    : %d\n",transmitStatistics.txPTPV2EventUDPFrames);
    PRINT_STAT("PTP V1 Message      : %d\n",transmitStatistics.txPTPV1MessageFrames);
    PRINT_STAT("PTP V2 Message      : %d\n",transmitStatistics.txPTPV2MessageFrames);
    PRINT_STAT("RTP         : %d\n",transmitStatistics.txRTPFrames);
    PRINT_STAT("ATM PW Frames       : %d\n",transmitStatistics.txATMPwDemuxFrames);
    PRINT_STAT("SAToP PW Frames : %d\n",transmitStatistics.txSAToPPwDemuxFrames);
    PRINT_STAT("AAL0        : %d\n",transmitStatistics.txAAL0Cells);
    PRINT_STAT("ATM Raw Data            : %d\n",transmitStatistics.txATMRawFrames);
    PRINT_STAT("Compressed frame        : %d\n",transmitStatistics.txCompressedFrames);
    PRINT_STAT("UDP checksum error      : %d\n",transmitStatistics.txUDPChecksumErrors);
    PRINT_STAT("TCP checksum error      : %d\n",transmitStatistics.txTCPChecksumErrors);
    PRINT_STAT("IP checksum error       : %d\n",transmitStatistics.txIPChecksumErrors);
    PRINT_STAT("FCS error               : %d\n",transmitStatistics.txFCSErrors);
    PRINT_STAT("IPv4 version error      : %d\n",transmitStatistics.txIPv4VersionErrors);
    PRINT_STAT("IPv6 version error      : %d\n",transmitStatistics.txIPv6VersionErrors);
    PRINT_STAT("RTP version error       : %d\n",transmitStatistics.txRTPVersionErrors);
    PRINT_STAT("AAL0 HEC error          : %d\n",transmitStatistics.txAAL0HECErrors);
    PRINT_STAT("Undersized frame        : %d\n",transmitStatistics.txUndersizedFrames);
    PRINT_STAT("Oversized frame         : %d\n",transmitStatistics.txOversizedFrames);
    PRINT_STAT("Jumbo frame     : %d\n",transmitStatistics.txJumboFrames);
    PRINT_STAT("Tx frames       : %d\n",transmitStatistics.txFrames);
    PRINT_STAT("Tx error frames : %d\n",transmitStatistics.txErrors);
    PRINT_STAT("Tx octets       : %d\n",transmitStatistics.txOctets);
    PRINT_STAT("Tx Bandwidth    : %d bps\n",transmitStatistics.txBandwidth);
}

static void BandwidthStatistics(t_Handle h_Tg)
{
     t_TrafficGen     *p_TrafficGen     = (t_TrafficGen *)h_Tg;/* Pointer to the traffic generator */
     t_TrafficGenPort *p_TrafficGenPort = NULL;     /* Pointer to the Tx port */
     uint32_t         timeInterval = 0;             /* Period between start and stop of Tx */
     uint32_t         nowTimeStamp = 0;             /* Current time stamp */
     uint16_t         portId = 0;                   /* Port identifier */
     uint32_t         totalNumOfTxBytes=0;          /* Cumulative Tx bytes */
     uint32_t         intFlags = 0;                 /* Interrupt flags */
     uint32_t         queue = 0, queueIndex = 0;                    /* Queue identifier */
     uint32_t         aggrBandwidth = 0;            /* Aggregated bandwidth across all ports */
     uint32_t         totalNumOfTxBytesPerPort = 0; /* Cumulative Tx bytes per port */
     uint32_t         firstPortLastTimeStamp = 0;   /* First port's last time stamp */

     /* Check the TG handle */
     ASSERT_COND(p_TrafficGen);

     /* Get the current time stamp */
     nowTimeStamp = XX_CurrentTime();

     intFlags = XX_DisableAllIntr();

     for (portId = 0;portId < p_TrafficGen->numOfPorts; portId++)
     {
         /* Get the port handle */
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

         if(!portId)
         {
             firstPortLastTimeStamp = p_TrafficGenPort->lastTimeStamp;
         }

         if (p_TrafficGenPort->inUse)
         {
             if (p_TrafficGenPort->testPassed)
             {
                 /* Display the queue statistics */
                 for(queue = 0;queue < p_TrafficGenPort->numOfQueues;queue++)
                 {
                     queueIndex = p_TrafficGenPort->p_Queues[queue].queueId;
                     if (p_TrafficGenPort->f_DevTxConf)
                         p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, (uint8_t)queueIndex);

                     //PRINT_TX_PORT_STATUS(p_TrafficGenPort, queue, " S");
                     //XX_Print("\n");
                 }

                 /* If Tx has not happened, test fails */
                 if (p_TrafficGenPort->txBytes == p_TrafficGenPort->counterHistory)
                     p_TrafficGenPort->testPassed = FALSE;

                 /* Update the cumulative Tx bytes across all ports */
                 totalNumOfTxBytes += (p_TrafficGenPort->txBytes-p_TrafficGenPort->counterHistory);

                 /* Update the cumulative Tx bytes per port */
                 totalNumOfTxBytesPerPort = (p_TrafficGenPort->txBytes-p_TrafficGenPort->counterHistory);

                 /* Update the Tx bytes in the history for future use */
                 p_TrafficGenPort->counterHistory = p_TrafficGenPort->txBytes;

                 /* Calculate the bandwidth for each port */
                 timeInterval = nowTimeStamp - p_TrafficGenPort->lastTimeStamp;

                 /* Update the last time stamp for each port */
                 p_TrafficGenPort->lastTimeStamp = nowTimeStamp;

                 /* Display the bandwidth statistics for each port */
                 p_TrafficGenPort->txStatistics.txBandwidth = (totalNumOfTxBytesPerPort<<3)/(timeInterval/1000);
             }
             else
                 XX_Print("Port %d - FAILED!\n", portId);
         }
     }

     /* Calculate the aggregated bandwidth across all ports during the period between the
     current time and the last time stamp of the last port */
     timeInterval = nowTimeStamp - firstPortLastTimeStamp;

     XX_RestoreAllIntr(intFlags);

     /* Display the aggregated bandwidth across all ports */
     aggrBandwidth = (totalNumOfTxBytes<<3)/(timeInterval/1000);
     p_TrafficGen->globalTxStatistics.txBandwidth = aggrBandwidth;
}

static t_Error ResetGenPortParam(bool allQueues, uint32_t queueId, t_TrafficGenPort *p_TrafficGenPort)
{
    t_TrafficGenQueue  *p_TrafficGenQueue;
    uint32_t           queueIndex = 0;
    uint32_t           queuesConfigured = 1;      /* Default queuesConfigured value when allQueues is FALSE */

    /* Reset statistics across ports/queues for RunBatch/Continuous test and
      for the particular port and queue for Run Simple Test */

    memset(&(p_TrafficGenPort->txStatistics), 0, sizeof(t_TrafficGenTxStatistics));

    p_TrafficGenPort->keepGoing = TRUE;
    p_TrafficGenPort->testPassed = TRUE;

    if(allQueues)
    {
        queuesConfigured = p_TrafficGenPort->numOfQueues;
    }
    else
    {
        for(queueIndex = 0; queueIndex < p_TrafficGenPort->numOfQueues; queueIndex++)
        {
            p_TrafficGenQueue = &(p_TrafficGenPort->p_Queues[queueIndex]);
            if(p_TrafficGenQueue->queueId == queueId)
            {
                break;
            }

        }
    }

    while(queuesConfigured)
    {
        p_TrafficGenQueue = &(p_TrafficGenPort->p_Queues[queueIndex++]);

        p_TrafficGenQueue->txConfedFrames = 0;
        p_TrafficGenQueue->txFrames = 0;
        p_TrafficGenQueue->txErrors = 0;
        p_TrafficGenQueue->txFrameCounter = p_TrafficGenQueue->txFramesInLst;
        p_TrafficGenQueue->txConfFrameCounter = 0;
        p_TrafficGenQueue->txLeftoverCounter = 0;
        queuesConfigured--;
    }

    return E_OK;
}

static t_Error ResetGenStatistics(t_Handle h_Tg)
{
    t_TrafficGen          *p_TrafficGen = (t_TrafficGen *)h_Tg;
    t_TrafficGenPort      *p_TrafficGenPort = NULL;
    uint16_t              portId  = 0;
    uint32_t              queueId = 0;

    /* Reset global statistics */
    memset(&(p_TrafficGen->globalTxStatistics), 0, sizeof(t_TrafficGenTxStatistics));

    for (portId = 0;portId < p_TrafficGen->numOfPorts;portId++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

         /* Reset the port statistics for every port and their queues */
         ResetGenPortParam(TRUE, queueId, p_TrafficGenPort);

        /* Initialize bandwidth counters */
        p_TrafficGenPort->lastTxTimeStamp = XX_CurrentTime();
        p_TrafficGenPort->lastTimeStamp = XX_CurrentTime();
        p_TrafficGenPort->txBytes = 0;
        p_TrafficGenPort->counterHistory = 0;
     }
    return E_OK;
}

static uint32_t GetQueueHandlePortSliceSched(t_TrafficGenPort  *p_TrafficGenPort)
{
     uint32_t queueId;

     p_TrafficGenPort->queueCounter++;
     queueId = (uint32_t)((p_TrafficGenPort->queueCounter)- 1);

     if(p_TrafficGenPort->queueCounter == p_TrafficGenPort->numOfQueues)
     {
        p_TrafficGenPort->queueCounter = 0;
        p_TrafficGenPort->getNextQueueHandle = FALSE;
     }
     return queueId;
}

static uint32_t GetQueueHandleQueueSliceSched(t_TrafficGenPort  *p_TrafficGenPort)
{
     p_TrafficGenPort->queueCounter++;
     if(p_TrafficGenPort->queueCounter > p_TrafficGenPort->numOfQueues)
     {
        p_TrafficGenPort->queueCounter = 1;
     }
     p_TrafficGenPort->getNextQueueHandle = FALSE;
     return (uint32_t)((p_TrafficGenPort->queueCounter)-1);
}

static void RechainTxList(t_Handle h_Tg)
{
     t_TrafficGen       *p_TrafficGen = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort   *p_TrafficGenPort = NULL;
     t_TrafficGenQueue  *p_TrafficGenQueue = NULL;
     t_PgFrame          *p_TrafficGenFrame = NULL;   /* Frame handle */

     uint16_t           portId  = 0;               /* Port identifier */
     uint32_t   queueId = 0;

     for (portId = 0;portId < p_TrafficGen->numOfPorts;portId++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);
         for(queueId = 0;queueId < p_TrafficGenPort->numOfQueues;queueId++)
         {
             p_TrafficGenQueue = &p_TrafficGenPort->p_Queues[queueId];

             /* Add confirm frame list to ready list */
             while (!LIST_IsEmpty(&p_TrafficGenQueue->confFramesLst))
             {
                 p_TrafficGenFrame = RemoveNextFrameFromConfLst(p_TrafficGenQueue);
                 if (p_TrafficGenFrame)
                 {
                     EnqueueFrameToReadyQ(p_TrafficGenQueue, p_TrafficGenFrame);
                 }
             }

             /* If txShuffleFrames is enabled, Tx list rechain is not required, as the transmission order is anyway in random */
             if(!p_TrafficGenQueue->txShuffleFrames)
             {
                 if( LIST_FIRST(p_TrafficGenQueue->p_EmptyFramesLst) != p_TrafficGenQueue->p_FirstNode )
                 {
                     ITG_RechainList(p_TrafficGenQueue->p_EmptyFramesLst,p_TrafficGenQueue->p_FirstNode);
                 }
             }
         }
     }
}

static void RunTest(t_TrafficGen  *p_TrafficGen, uint16_t portId)
{
     t_PgFrame           *p_TrafficGenFrame = NULL;   /* Frame handle */
     t_TrafficGenPort    *p_TrafficGenPort  = NULL;   /* Port handle */
     t_TrafficFrameInfo  *p_FrameInfo  = NULL;        /* Holds the frame informations */
     t_TrafficGenQueue   *p_TrafficGenQueue = NULL;   /* Queue handle */

     t_Error             err_TxConf    = E_OK;        /* Return confirmation status */
     t_Error             err_Tx        = E_OK;        /* Return transmission status */
     t_Error             err_Threshold = E_OK;        /* Return threshold status */

     uint32_t            queueId = 0;                 /* Queue identifider */
     uint32_t            intFlags;                    /* Interrupt flags */
     uint32_t            timeToWait    = 0;           /* Time to wait for next transmission */
     uint32_t            queueIterator = 0;           /* Number of frames to be transmitted per queue per loop */
     uint16_t            length;
     bool                noConfirmationMode;

     p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

     noConfirmationMode = p_TrafficGenPort->noConfirmationMode;

     while(p_TrafficGenPort->keepGoing && p_TrafficGenPort->getNextQueueHandle)
     {
        queueId = p_TrafficGen->f_SchedMech(p_TrafficGenPort);

        queueIterator = p_TrafficGen->queueIterator;

        while(queueIterator--)
        {
            if((p_TrafficGen->txMode == e_TG_MODE_BATCH) &&
            (p_TrafficGenPort->txStatistics.txFrames >= p_TrafficGenPort->numOfFrames))
            {
                p_TrafficGenPort->keepGoing = FALSE;
                break;
            }

            /* Update the time stamp */
            if (p_TrafficGenPort->tmpTxBytes >= p_TrafficGenPort->burstSize)
            {
                 /* We need to compute the time we need to wait in mili-secs */
                timeToWait = (uint32_t)
                    (DIV_CEIL(p_TrafficGenPort->byteTime * p_TrafficGenPort->tmpTxBytes, 1000000) + 1);

                while ((XX_CurrentTime() - p_TrafficGenPort->lastTxTimeStamp) < timeToWait) ;


                 p_TrafficGenPort->tmpTxBytes = 0;
                 p_TrafficGenPort->lastTxTimeStamp = XX_CurrentTime();
            }

            p_TrafficGenQueue = &p_TrafficGenPort->p_Queues[queueId];

            if(p_TrafficGenQueue->retransmitThreshold == TG_RETRANSMIT_THRESHOLD)
            {
                p_TrafficGenPort->keepGoing = FALSE;
                err_Threshold = E_TIMEOUT;
                break;
            }

            p_TrafficGenFrame = DequeueFrameFromReadyQ(p_TrafficGenQueue);
            if (!p_TrafficGenFrame)
            {
                /* Try to confirm all frames that was already sent */
                if (p_TrafficGenPort->f_DevTxConf)
                    p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, p_TrafficGenQueue->queueId);

                p_TrafficGenFrame = DequeueFrameFromReadyQ(p_TrafficGenQueue);
                if (!p_TrafficGenFrame)
                {
                    /* Waiting for timeout period (usec) to allow frames to be confirmed */
                    timeToWait = (uint32_t)
                        (20 * DIV_CEIL(p_TrafficGenPort->byteTime * p_TrafficGenPort->tmpTxBytes, 1000));

                    XX_UDelay(timeToWait);

                    /* Try to confirm again all frames that was already sent */
                    if (p_TrafficGenPort->f_DevTxConf)
                        p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, p_TrafficGenQueue->queueId);

                    p_TrafficGenFrame = DequeueFrameFromReadyQ(p_TrafficGenQueue);
                    if (!p_TrafficGenFrame)
                    {
                        p_TrafficGenPort->keepGoing = FALSE;
                        err_TxConf = E_TIMEOUT;
                        break;
                    }
                }
            }

            intFlags = XX_DisableAllIntr();

            /* Prepare for transmission. Note that confirmation may be done
               from within Tx routine. */

            /* Get all frame information */
            p_FrameInfo = F_GetFrameInfo(p_TrafficGenFrame->h_Frame);
            p_FrameInfo->h_App = (t_Handle)p_TrafficGenFrame;
            F_SetFrameInfo(p_TrafficGenFrame->h_Frame, p_FrameInfo);

            if (noConfirmationMode == FALSE)
            {
                AddFrameToConfLst(p_TrafficGenQueue, p_TrafficGenFrame);
            }

            length = F_GetLength(p_TrafficGenFrame->h_Frame);
            err_Tx = p_TrafficGenPort->f_DevTx(p_TrafficGenPort->h_Dev, p_TrafficGenQueue->queueId, p_TrafficGenFrame->h_Frame);

            if (err_Tx == E_OK)
            {
                /* Calculate the transmit statistics */
                UpdateTxStats(p_TrafficGen,p_TrafficGenFrame->h_Frame, length, portId);

                p_TrafficGenQueue->txFrames++;

                /* Increment the transmit frame counter for the port and across all ports */
                p_TrafficGenPort->txStatistics.txFrames++;
                p_TrafficGen->globalTxStatistics.txFrames++;

                /* Update the Tx bytes */
                p_TrafficGenPort->txBytes += length;
                p_TrafficGenPort->tmpTxBytes += length;

                if(p_TrafficGenQueue->retransmitThreshold)
                    p_TrafficGenQueue->retransmitThreshold = 0;

                if (noConfirmationMode == TRUE)
                {
                    PG_DestroyFrame(p_TrafficGenFrame->h_Frame);
                    XX_Free(p_TrafficGenFrame);
                }
             }
             else
             {
                if (noConfirmationMode == FALSE)
                {
                    RemoveFrameFromConfLst(p_TrafficGenFrame);
                }
                AddFrameToHeadOfReadyQ(p_TrafficGenQueue, p_TrafficGenFrame);
                p_TrafficGenQueue->retransmitThreshold++;
                XX_RestoreAllIntr(intFlags);
                break;
             }

             XX_RestoreAllIntr(intFlags);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
         PRINT_TX_PORT_STATUS(p_TrafficGenPort, queueId, " T");
#endif /* (defined(DEBUG_ERRORS) && ... */
        }
    }

    if (err_TxConf != E_OK || err_Threshold != E_OK)
        p_TrafficGenPort->testPassed = FALSE;
}

static bool RunBatchTest(t_Handle h_Tg)
{
     t_TrafficGen       *p_TrafficGen = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort   *p_TrafficGenPort = NULL;

     uint16_t           portId = 0;              /* Port identifier */
     uint32_t           totalNumFrames  = 0;     /* Total number of frames to be transmitted */
     uint32_t           portsFailed     = 0;        /* Holds the ports failed during transmission */
     uint32_t           portsConfigured = 0;    /* Holds the ports which are configured */

     portsConfigured = p_TrafficGen->numOfPorts;

     /* Calculate the total number of frames to be transmitted */
     for (portId = 0;portId < p_TrafficGen->numOfPorts;portId++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

         if (p_TrafficGenPort->inUse)
         {
             totalNumFrames += p_TrafficGenPort->numOfFrames;
         }
     }

     /* Transmit the frames among multiple ports in a round robin fashion */
     portId = 0;
     while(p_TrafficGen->globalTxStatistics.txFrames < totalNumFrames && !p_TrafficGen->isTxStopped)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);
         p_TrafficGenPort->getNextQueueHandle = TRUE;

         /* Transmit the frames */
         if (p_TrafficGenPort->inUse && p_TrafficGenPort->testPassed &&
         (p_TrafficGenPort->txStatistics.txFrames < p_TrafficGenPort->numOfFrames))
         {
             RunTest(p_TrafficGen, portId);
             if(!(p_TrafficGenPort->testPassed))
                 portsFailed++;

             if(p_TrafficGenPort->txStatistics.txFrames == p_TrafficGenPort->numOfFrames)
                 portsConfigured--;
         }

         /* If all ports failed, exit */
         if(portsFailed == portsConfigured)
             break;

         /* Get the next port id for transmission */
         portId = (uint16_t)((portId+1)%p_TrafficGen->numOfPorts);
     }

      /* Calculate bandwidth statistics */
     BandwidthStatistics(h_Tg);

     /* Display the status of transmission in each port */
     return (TG_ConfirmAllPorts(p_TrafficGen));
}

static bool RunContTest(t_Handle h_Tg)
{
     t_TrafficGen       *p_TrafficGen     = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort   *p_TrafficGenPort = NULL;
     uint16_t           portId          = 0;    /* Port identifier */
     uint32_t           portsFailed     = 0;    /* Holds the ports failed during transmission */
     uint32_t           portsConfigured = 0;    /* Holds the ports which are configured */

     portsConfigured = p_TrafficGen->numOfPorts;

     /* Transmit the frames among multiple ports in a round robin fashion */
     while (!p_TrafficGen->isTxStopped)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);
         p_TrafficGenPort->getNextQueueHandle = TRUE;

         /* Transmit the frames */
         if (p_TrafficGenPort->inUse && p_TrafficGenPort->testPassed)
         {
             RunTest(p_TrafficGen, portId);
             if(!(p_TrafficGenPort->testPassed))
                 portsFailed++;
         }

         /* If all ports failed, exit */
         if(portsFailed == portsConfigured)
             break;

         /* Get the next port id for transmission */
         portId = (uint16_t)((portId+1)%p_TrafficGen->numOfPorts);
     }

     /* Calculate bandwidth statistics */
     BandwidthStatistics(h_Tg);

     /* Display the status of transmission in each port */
     return (TG_ConfirmAllPorts(p_TrafficGen));
}


t_Handle TG_Init(t_TrafficGenParam  *p_TrafficGenParam)
{
     t_TrafficGen             *p_TrafficGen;

     p_TrafficGen = (t_TrafficGen *)XX_Malloc(sizeof(t_TrafficGen));
     if (!p_TrafficGen)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficGen"));
         return NULL;
     }
     memset(p_TrafficGen, 0, sizeof(t_TrafficGen));

     /* Init number of Ports and portLst */
     p_TrafficGen->numOfPorts = 0;

     INIT_LIST(&p_TrafficGen->portsLst);

     p_TrafficGen->queueIterator = p_TrafficGenParam->queueIterator;

     if(p_TrafficGenParam->tgSchedMechanism == e_PORTSLICE_RR_SCHED)
     {
        p_TrafficGen->f_SchedMech = GetQueueHandlePortSliceSched;
     }
     else if(p_TrafficGenParam->tgSchedMechanism == e_QUEUESLICE_RR_SCHED)
     {
        p_TrafficGen->f_SchedMech = GetQueueHandleQueueSliceSched;
     }

     return p_TrafficGen;
}

t_Error TG_Free(t_Handle h_Tg)
{
     t_TrafficGen         *p_TrafficGen     = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort     *p_TrafficGenPort = NULL;   /* Port handle */
     uint32_t             portId = 0;
     int                  i;

     if (!p_TrafficGen)
         RETURN_ERROR(MINOR, E_INVALID_HANDLE, NO_MSG);

     for(i = 0; i< p_TrafficGen->numOfPorts; i++)
     {
        p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);
        TG_PORT_Free(p_TrafficGenPort);
     }

     XX_Free(p_TrafficGen);
     return E_OK;
}

t_Handle TG_PORT_Config(t_Handle h_Tg, t_TrafficGenPortParam *p_TrafficGenPortParam)
{
     t_TrafficGen             *p_TrafficGen = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort         *p_TrafficGenPort;
     t_TrafficGenQueue        *p_TrafficGenQueue;
     t_TrafficGenQueueParam   *p_TrafficGenQueueParam;
     t_TrafficPort            *p_TrafficPort;
     uint32_t                 queueIndex;

     if (!IN_RANGE(TG_MIN_LINE_RATE, p_TrafficGenPortParam->lineRate, TG_MAX_LINE_RATE))
     {
         REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, ("Line rate"));
         return NULL;
     }

     p_TrafficGenPort = (t_TrafficGenPort *)XX_Malloc(sizeof(t_TrafficGenPort));
     if (!p_TrafficGenPort)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficGenPort"));
         return NULL;
     }
     memset(p_TrafficGenPort, 0, sizeof(t_TrafficGenPort));

     p_TrafficPort = ITG_CreateTrafficPort();
     if (!p_TrafficPort)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficPort"));
         XX_Free(p_TrafficGenPort);
         return NULL;
     }
     p_TrafficPort->h_Port = p_TrafficGenPort;

     p_TrafficGenPort->p_TrafficGen = (struct t_TrafficGen *)h_Tg;

     ITG_AddPortToList(p_TrafficPort, &p_TrafficGenPort->p_TrafficGen->portsLst);
     p_TrafficGen->numOfPorts++;

     p_TrafficGenPort->inUse = TRUE;
     p_TrafficGenPort->portId = p_TrafficGenPortParam->portId;
     p_TrafficGenPort->errorsThreshold = p_TrafficGenPortParam->errorsThreshold;
     p_TrafficGenPort->h_Dev = p_TrafficGenPortParam->h_Dev;
     p_TrafficGenPort->f_DevTx = p_TrafficGenPortParam->f_DevTx;
     p_TrafficGenPort->f_DevTxConf = p_TrafficGenPortParam->f_DevTxConf;
     p_TrafficGenPort->numOfFrames = p_TrafficGenPortParam->numOfFrames;
     p_TrafficGenPort->burstSize = p_TrafficGenPortParam->burstSize;
     p_TrafficGenPort->numOfQueues = p_TrafficGenPortParam->numOfQueues;
     p_TrafficGenPort->mtu = 0;
     p_TrafficGenPort->queueCounter = 0;
     p_TrafficGenPort->getNextQueueHandle = TRUE;
     p_TrafficGenPort->underSizeFrameValue = p_TrafficGenPortParam->underSizeFrameValue;
     p_TrafficGenPort->overSizeFrameValue = p_TrafficGenPortParam->overSizeFrameValue;
     p_TrafficGenPort->noConfirmationMode = p_TrafficGenPortParam->noConfirmationMode;

     p_TrafficGenPort->p_Queues = (t_TrafficGenQueue *)XX_Malloc(p_TrafficGenPort->numOfQueues * sizeof(t_TrafficGenQueue));
     if (!p_TrafficGenPort->p_Queues)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficGenPort->p_Queues"));
         XX_Free(p_TrafficGenPort);
         return NULL;
     }
     memset(p_TrafficGenPort->p_Queues, 0, p_TrafficGenPort->numOfQueues * sizeof(t_TrafficGenQueue));

     /* p_QueueIndexMap is used to associate 'queueId' with 'queueIndex'. p_QueueIndexMap will be used to retrieve
     'queueIndex' back and then retrieve the corresponding queue parameters when queues are configured out of order.
     E.g. When two queues are configured out of order (queueId = 7,5), queueIndex = 0 refers to 7th queue paremeters
     and queueIndex = 1 refers to 5th queue parameters.
     Allocate p_QueueIndexMap for maximum number of values the member variable 'queueId' can hold */
     p_TrafficGenPort->p_QueueIndexMap = (uint32_t *)XX_Malloc(p_TrafficGenPortParam->maxQueueIndex * sizeof(uint32_t));

     if (!p_TrafficGenPort->p_QueueIndexMap)
     {
         REPORT_ERROR(CRITICAL, E_NO_MEMORY, ("p_TrafficGenPort->p_QueueIndexMap"));
         XX_Free(p_TrafficGenPort);
         XX_Free(p_TrafficGenPort->p_Queues);
         return NULL;
     }
     memset(p_TrafficGenPort->p_QueueIndexMap, 0, p_TrafficGenPortParam->maxQueueIndex * sizeof(uint32_t));

     for (queueIndex = 0; queueIndex < p_TrafficGenPort->numOfQueues; queueIndex++)
     {
         p_TrafficGenQueueParam = &(p_TrafficGenPortParam->p_QueuesParam[queueIndex]);

         p_TrafficGenQueue = &(p_TrafficGenPort->p_Queues[queueIndex]);

         p_TrafficGenQueue->queueId = p_TrafficGenQueueParam->queueId;
         p_TrafficGenQueue->p_EmptyFramesLst = (t_List *)PG_GetList(p_TrafficGenQueueParam->h_FramesLst);
         /* Store the first element of Tx list for re-chaining in the next run */
         p_TrafficGenQueue->p_FirstNode = LIST_FIRST(p_TrafficGenQueue->p_EmptyFramesLst);
         /* Load Shuffle Option to Queue Param */
         p_TrafficGenQueue->txShuffleFrames = p_TrafficGenQueueParam->shuffleTxFrames;

         p_TrafficGenPort->p_QueueIndexMap[p_TrafficGenQueueParam->queueId] = queueIndex;

         p_TrafficGenQueue->retransmitThreshold = 0;

         INIT_LIST(&p_TrafficGenQueue->confFramesLst);

         if(p_TrafficGenQueue->txShuffleFrames)
         {
            /*  Update the txFramesInLst  */
             p_TrafficGenQueue->txFramesInLst = (uint32_t)LIST_NumOfObjs(p_TrafficGenQueue->p_EmptyFramesLst);
             p_TrafficGenQueue->txFrameCounter = p_TrafficGenQueue->txFramesInLst;
         }
     }

     /* The time needed for one byte to be sent - in nano-secs */
     p_TrafficGenPort->byteTime = (uint32_t)(TG_BPS_TO_NANO_SEC_FACTOR / p_TrafficGenPortParam->lineRate);

     return p_TrafficGenPort;
}

void TG_PORT_Free(t_Handle h_TgPort)
{
    t_TrafficGenPort    *p_TrafficGenPort  = (t_TrafficGenPort *)h_TgPort;
    t_TrafficGenQueue   *p_TrafficGenQueue = NULL;
    int                 j = 0;
    t_TrafficPort       *p_TrafficPort = NULL;
    t_PgFrame           *p_TrafficGenFrame;

    SANITY_CHECK_RETURN(h_TgPort, E_INVALID_HANDLE);

    if (p_TrafficGenPort->inUse)
    {
         p_TrafficGenPort->inUse = FALSE;

         for (j = 0; j < p_TrafficGenPort->numOfQueues; j++)
         {
             p_TrafficGenQueue = &p_TrafficGenPort->p_Queues[j];

             p_TrafficGenFrame = RemoveNextFrameFromConfLst(p_TrafficGenQueue);
             while (p_TrafficGenFrame)
             {
                 PG_DestroyFrame(p_TrafficGenFrame->h_Frame);
                 XX_Free(p_TrafficGenFrame);
                 p_TrafficGenFrame = RemoveNextFrameFromConfLst(p_TrafficGenQueue);
             }
         }
     }

     if(p_TrafficGenPort->p_Queues)
     {
        XX_Free(p_TrafficGenPort->p_Queues);
     }
     if(p_TrafficGenPort->p_QueueIndexMap)
     {
        XX_Free(p_TrafficGenPort->p_QueueIndexMap);
     }

     p_TrafficPort = ITG_RemovePortFromList(h_TgPort, &p_TrafficGenPort->p_TrafficGen->portsLst);
     if (p_TrafficPort)
     {
         p_TrafficGenPort->p_TrafficGen->numOfPorts--;

         XX_Free(p_TrafficPort->h_Port);
         XX_Free(p_TrafficPort);
     }
}

t_Error TG_PORT_SetIpMtu(t_Handle h_TgPort, uint32_t mtu)
{
    t_TrafficGenPort    *p_TrafficGenPort = (t_TrafficGenPort *)h_TgPort;
    t_Error             errCode = E_OK;
    uint32_t            queueId;

    ASSERT_COND(p_TrafficGenPort);

    if (p_TrafficGenPort->mtu != 0)
        REPORT_ERROR(WARNING, E_INVALID_VALUE,
            ("IP Fragmentation can be configured only once for TG port #%d",
            p_TrafficGenPort->portId));
    else
    {
        p_TrafficGenPort->mtu = mtu;
        /* Fragment the frame lists for all of this port's queues */
        for (queueId = 0; queueId < p_TrafficGenPort->numOfQueues; queueId++)
        {
            errCode = PG_FragmentFrameList(p_TrafficGenPort->p_Queues[queueId].p_EmptyFramesLst,
                p_TrafficGenPort->mtu);
            p_TrafficGenPort->p_Queues[queueId].p_FirstNode =
                LIST_FIRST(p_TrafficGenPort->p_Queues[queueId].p_EmptyFramesLst);
            if (errCode != E_OK)
                RETURN_ERROR(MAJOR, errCode, NO_MSG);
        }
    }

    return errCode;
}

bool TG_Run(t_Handle h_Tg, e_TgOperationMode mode)
{
     t_TrafficGen       *p_TrafficGen = (t_TrafficGen *)h_Tg;

     p_TrafficGen->isTxStopped = FALSE;

     switch (mode)
     {
         case (e_TG_MODE_BATCH):

             p_TrafficGen->txMode = e_TG_MODE_BATCH;
             /* Reset TG statistics */
             ResetGenStatistics(h_Tg);
             /* Re-chain Tx list for the successive run */
             RechainTxList(h_Tg);
             return RunBatchTest(h_Tg);

         case (e_TG_MODE_CONTINUOUS):

             p_TrafficGen->txMode = e_TG_MODE_CONTINUOUS;
             /* Reset TG statistics */
             ResetGenStatistics(h_Tg);
             /* Re-chain Tx list for the successive run */
             RechainTxList(h_Tg);
             return RunContTest(h_Tg);

         default:
         {
             REPORT_ERROR(MINOR, E_NOT_SUPPORTED, ("Invalid transmission mode"));
             return FALSE;
         }
     }
}

bool TG_RunSimple(t_Handle h_Tg, uint16_t portId, uint32_t queueId, uint32_t numOfFrames)
{
     t_TrafficGen         *p_TrafficGen      = (t_TrafficGen *)h_Tg;
     t_PgFrame            *p_TrafficGenFrame = NULL;
     t_TrafficGenPort     *p_TrafficGenPort  = NULL;
     t_TrafficGenQueue    *p_TrafficGenQueue = NULL;
     t_TrafficFrameInfo   *p_FrameInfo       = NULL;            /* Holds the frame informations */
     t_Error              rc = E_INVALID_STATE;
     bool                 localKeepGoing = TRUE;
     int                  loops      = 0;
     uint32_t             intFlags   = 0;
     uint16_t             portIndex  = 0, length;
     uint32_t             queueIndex = 0;

     if(!p_TrafficGen)
     {
        REPORT_ERROR(CRITICAL, E_INVALID_HANDLE, ("p_TrafficGen"));
        return FALSE;
     }

     p_TrafficGen->txMode = e_TG_MODE_SIMPLE;

     for(portIndex = 0; portIndex < p_TrafficGen->numOfPorts; portIndex++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portIndex);
         if(p_TrafficGenPort->portId == portId)
         {
             break;
         }
     }

     /* Check if port Found */
     if(!(portIndex < p_TrafficGen->numOfPorts))
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid Port Id"));
        return FALSE;
     }

     queueIndex = p_TrafficGenPort->p_QueueIndexMap[queueId];

     /* Determine if frame is transmitted in a configured queue */
     if(p_TrafficGenPort->p_Queues[queueIndex].queueId != queueId)
     {
         XX_Print("Frame cannot be transmitted in a non-configured queue\n");
         p_TrafficGenPort->testPassed = FALSE;
         return p_TrafficGenPort->testPassed;
     }

     /* Reset The specified port and queue statistics */
     ResetGenPortParam(FALSE, queueId, p_TrafficGenPort);

     p_TrafficGenQueue = &(p_TrafficGenPort->p_Queues[queueIndex]);

     if (numOfFrames == 0)
         numOfFrames = p_TrafficGenPort->numOfFrames;

     while (p_TrafficGenPort->keepGoing && localKeepGoing)
     {
         if (p_TrafficGenQueue->txFrames == numOfFrames)
            break;

         if(p_TrafficGenQueue->txFrames !=0 && p_TrafficGenQueue->txFrames%100 == 0)
             XX_UDelay(1000000);

         p_TrafficGenFrame = DequeueFrameFromReadyQ(p_TrafficGenQueue);
         if (!p_TrafficGenFrame)
         {
             if (p_TrafficGenPort->f_DevTxConf)
                 p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, queueId);

             p_TrafficGenFrame = DequeueFrameFromReadyQ(p_TrafficGenQueue);
             if (!p_TrafficGenFrame)
             {
                 localKeepGoing = FALSE;
                 rc = E_TIMEOUT;
                 continue;
             }
         }

         length = F_GetLength(p_TrafficGenFrame->h_Frame);

         intFlags = XX_DisableAllIntr();

         loops = 100;
         while (((rc = p_TrafficGenPort->f_DevTx(p_TrafficGenPort->h_Dev, queueId, p_TrafficGenFrame->h_Frame)) != E_OK) &&
                         --loops)
             XX_UDelay(1);

         if (loops == 0)
         {
            localKeepGoing = FALSE;
         }

         if (rc == E_OK)
         {
             /* Get all frame information */
             p_FrameInfo = F_GetFrameInfo(p_TrafficGenFrame->h_Frame);
             p_FrameInfo->h_App = (t_Handle)p_TrafficGenFrame;
             F_SetFrameInfo(p_TrafficGenFrame->h_Frame, p_FrameInfo);

             /* Calculate the transmit statistics */
             UpdateTxStats(p_TrafficGen, p_TrafficGenFrame->h_Frame, length, portId);

             if (p_TrafficGenPort->noConfirmationMode == FALSE)
             {
                AddFrameToConfLst(p_TrafficGenQueue, p_TrafficGenFrame);
             }
             else
             {
                PG_DestroyFrame(p_TrafficGenFrame->h_Frame);
                XX_Free(p_TrafficGenFrame);
             }

             p_TrafficGenQueue->txFrames++;

             /* Increment the transmit frame counter for the port and across all ports */
             p_TrafficGenPort->txStatistics.txFrames++;
             p_TrafficGen->globalTxStatistics.txFrames++;
         }
         else
         {
             XX_Print("Wasn't good transmition\n");
             AddFrameToHeadOfReadyQ(p_TrafficGenQueue, p_TrafficGenFrame);
         }

             XX_RestoreAllIntr(intFlags);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
         PRINT_TX_PORT_STATUS(p_TrafficGenPort, queueIndex, " T");
 #endif /* (defined(DEBUG_ERRORS) && ... */
     }

     if (p_TrafficGenPort->f_DevTxConf)
         p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, queueId);

     if (rc != E_OK)
         p_TrafficGenPort->testPassed = FALSE;

     PRINT_TX_PORT_STATUS(p_TrafficGenPort, queueIndex, " S");
     XX_Print("\n");

     return p_TrafficGenPort->testPassed;
}

void TG_Stop(t_Handle h_Tg)
{
     t_TrafficGen *p_TrafficGen = (t_TrafficGen *)h_Tg;

     p_TrafficGen->isTxStopped = TRUE;
}

bool TG_Resume(t_Handle h_Tg)
{
     t_TrafficGen *p_TrafficGen = (t_TrafficGen *)h_Tg;

     p_TrafficGen->isTxStopped = FALSE;

     switch (p_TrafficGen->txMode)
     {
         case (e_TG_MODE_BATCH):
             return RunBatchTest(h_Tg);
         case (e_TG_MODE_CONTINUOUS):
             return RunContTest(h_Tg);
         default:
         {
             REPORT_ERROR(MINOR, E_NOT_SUPPORTED, ("Invalid transmission mode"));
             return FALSE;
         }
     }
}

bool TG_ConfirmAllPorts(t_Handle h_Tg)
{
    t_TrafficGen        *p_TrafficGen = (t_TrafficGen *)h_Tg;
    t_TrafficGenPort    *p_TrafficGenPort;
    t_TrafficGenQueue   *p_TrafficGenQueue;
    uint16_t            portId;             /* Port identifier */
    bool                testPassed = TRUE;  /* Flag to indicate the status of the test */
    uint32_t            queue = 0, timeToWait;

     ASSERT_COND(p_TrafficGen);

     for (portId = 0; portId <p_TrafficGen->numOfPorts; portId++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portId);

         if (p_TrafficGenPort->inUse)
         {
             /* Wait for all frames to be sent (usec) - few times of burst time... */
             timeToWait = (uint32_t)
                (10 * DIV_CEIL(p_TrafficGenPort->byteTime * p_TrafficGenPort->burstSize, 1000));
             XX_UDelay(timeToWait);

             for (queue = 0; queue < p_TrafficGenPort->numOfQueues; queue++)
             {
                 if (p_TrafficGenPort->f_DevTxConf)
                     p_TrafficGenPort->f_DevTxConf (p_TrafficGenPort->h_Dev, (uint8_t)p_TrafficGenPort->p_Queues[queue].queueId);

                 p_TrafficGenQueue = &p_TrafficGenPort->p_Queues[queue];
                 if (!LIST_IsEmpty(&p_TrafficGenQueue->confFramesLst))
                 {
                    /* Not all frames were confirmed */
                    testPassed = FALSE;
                 }

                 PRINT_TX_PORT_STATUS(p_TrafficGenPort, queue, " S");
                 XX_Print("\n");
             }

             if (!p_TrafficGenPort->testPassed)
             {
                 testPassed = FALSE;
             }
         }
     }

     return testPassed;
}

t_Error TG_GetStatistics(t_Handle h_Tg, t_TrafficGenTxStatistics *p_GenStats)
{
    t_TrafficGen  *p_TrafficGen = (t_TrafficGen *)h_Tg;

    if(!p_TrafficGen || !p_GenStats)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
        return E_INVALID_HANDLE;
    }

    memcpy(p_GenStats, &(p_TrafficGen->globalTxStatistics), sizeof(t_TrafficGenTxStatistics));

    return E_OK;
}

t_Error TG_PrintTxStatistics(t_Handle h_Tg)
{
     t_TrafficGen  *p_TrafficGen = (t_TrafficGen *)h_Tg; /* Application handle */
     uint16_t      numPorts = 0;               /* Loop index to hold the port number */

     t_TrafficGenPort   *p_TrafficGenPort;

     /* Check the handle */
     SANITY_CHECK_RETURN_ERROR(p_TrafficGen, E_INVALID_HANDLE);

     /* Print the transmit statistics for each port */
     for(numPorts = 0; numPorts < p_TrafficGen->numOfPorts; numPorts++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,numPorts);
         XX_Print("\n===============================\n");
         XX_Print("Tx Stats : PORT %d\n", p_TrafficGenPort->portId);
         XX_Print("===============================\n");
         FrameStatistics(p_TrafficGenPort->txStatistics);
     }

     /* Print the transmit statistics across all ports */
     XX_Print("\n===============================\n");
     XX_Print("Tx Stats across all ports\n");
     XX_Print("===============================\n");

     FrameStatistics(p_TrafficGen->globalTxStatistics);

     return E_OK;
}

t_Handle TG_GetPortHandle(t_Handle h_Tg, uint16_t portId)
{
     t_TrafficGen         *p_TrafficGen     = (t_TrafficGen *)h_Tg;
     t_TrafficGenPort     *p_TrafficGenPort = NULL;
     uint16_t             portIndex = 0;

     for(portIndex = 0; portIndex < p_TrafficGen->numOfPorts; portIndex++)
     {
         p_TrafficGenPort = GET_NTH_GEN_PORT_HANDLE(&p_TrafficGen->portsLst,portIndex);
         if(p_TrafficGenPort->portId == portId)
         {
             break;
         }
     }

     /* Check if port Found */
     if(!(portIndex < p_TrafficGen->numOfPorts))
     {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid Port Id"));
        return NULL;
     }
     return p_TrafficGenPort;
}


void TG_PORT_Enable(t_Handle h_TgPort)
{
     t_TrafficGenPort     *p_TrafficGenPort = (t_TrafficGenPort *)h_TgPort;

     if(!p_TrafficGenPort)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid handle"));
         return;
     }

     if(p_TrafficGenPort->p_TrafficGen->isTxStopped == FALSE)
     {
         REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Cannot Enable Port while the test is Running"));
     }
     else
     {
         p_TrafficGenPort->inUse = TRUE;
     }
}

void TG_PORT_Disable(t_Handle h_TgPort)
{
     t_TrafficGenPort     *p_TrafficGenPort = (t_TrafficGenPort *)h_TgPort;

     if(!p_TrafficGenPort)
     {
         REPORT_ERROR(MAJOR, E_INVALID_HANDLE, ("Invalid handle"));
         return;
     }

     if(p_TrafficGenPort->p_TrafficGen->isTxStopped == FALSE)
     {
         REPORT_ERROR(MAJOR, E_INVALID_OPERATION, ("Cannot Disable Port while the test is Running"));
     }
     else
     {
         p_TrafficGenPort->inUse = FALSE;
     }
}

t_Error TG_PORT_GetStatistics(t_Handle h_TrafficPort, t_TrafficGenTxStatistics *p_GenStats)
{
    t_TrafficGenPort    *p_TrafficGenPort = (t_TrafficGenPort *)h_TrafficPort;   /* Port handle */

    if(!p_TrafficGenPort || !p_GenStats)
    {
        REPORT_ERROR(MAJOR, E_INVALID_HANDLE, NO_MSG);
        return E_INVALID_HANDLE;
    }

    memcpy(p_GenStats, &(p_TrafficGenPort->txStatistics), sizeof(t_TrafficGenTxStatistics));

    return E_OK;
}

t_Error TG_PORT_ConfirmFrame(t_Handle h_Port, uint32_t queueId, t_Handle h_Frame)
{
     t_TrafficGenPort    *p_TrafficGenPort = (t_TrafficGenPort *)h_Port;
     t_TrafficGen        *p_TrafficGen;
     t_PgFrame           *p_TrafficGenFrame;
     uint32_t            status;
     uint32_t            queueIndex = 0;
     t_TrafficFrameInfo  *p_FrameInfo = NULL;           /* Holds the frame informations */

     p_TrafficGen = (t_TrafficGen *)p_TrafficGenPort->p_TrafficGen;

     /* Get all frame information */
     p_FrameInfo = F_GetFrameInfo(h_Frame);

     p_TrafficGenFrame = (t_PgFrame *)p_FrameInfo->h_App;
     ASSERT_COND(p_TrafficGenFrame);

     RemoveFrameFromConfLst(p_TrafficGenFrame);
     /* To maintain 1024 frames in the Rx list, each frame that is dequeued
      * is enqueued immediately, so list wil not be empty and the check for
      * rxFrameNode being empty is henceforth not required.
      */
     queueIndex = p_TrafficGenPort->p_QueueIndexMap[queueId];

     EnqueueFrameToReadyQ(&(p_TrafficGenPort->p_Queues[queueIndex]), p_TrafficGenFrame);
     p_TrafficGenPort->p_Queues[queueIndex].txConfedFrames++;

     status = F_GetStatus(h_Frame);

     /* Increment the transmit error counter for the port and across all ports */
     if(status)
     {
         p_TrafficGenPort->txStatistics.txErrors++;
         p_TrafficGen->globalTxStatistics.txErrors++;

         if (++p_TrafficGenPort->p_Queues[queueIndex].txErrors >= p_TrafficGenPort->errorsThreshold)
         {
             p_TrafficGenPort->keepGoing = FALSE;
             p_TrafficGenPort->testPassed = FALSE;
         }
     }

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 1))
     PRINT_TX_PORT_STATUS(p_TrafficGenPort, queueIndex, "TC");
 #endif /* (defined(DEBUG_ERRORS) && ... */

     return E_OK;
}



