/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mem_mng.c

 @Description   Implementation of the memory allocation management module.
*//***************************************************************************/
#include "std_ext.h"
#include "string_ext.h"
#include "error_ext.h"
#include "mm_ext.h"
#include "mem_mng.h"




static void MemMngAddEarlyEntry(t_MemMng    *p_MemMng,
                                void        *p_Memory,
                                uint32_t    size,
                                char        *info,
                                char        *filename,
                                int         line);

static bool MemMngRemoveEarlyEntry(t_MemMng *p_MemMng, void *p_Memory);

static void MemMngAddEntry(t_MemMng             *p_MemMng,
                           t_MemMngPartition    *p_Partition,
                           void                 *p_Memory,
                           uint32_t             size,
                           char                 *info,
                           char                 *filename,
                           int                  line);

static bool MemMngRemoveEntry(t_MemMng          *p_MemMng,
                              t_MemMngPartition *p_Partition,
                              void              *p_Memory);

static bool MemMngGetPartitionIdByAddr(t_MemMng             *p_MemMng,
                                       uint32_t             addr,
                                       int                  *p_PartitionId,
                                       t_MemMngPartition    **p_Partition);

static void MemMngFreePartition(t_MemMng *p_MemMng, t_List *p_PartitionIterator);


/*****************************************************************************/
t_Handle MEM_MNG_Init(t_MemMngParam *p_MemMngParam)
{
    t_MemMng    *p_MemMng;

    SANITY_CHECK_RETURN_VALUE(p_MemMngParam, E_NULL_POINTER, NULL);

    if (!(p_MemMngParam->f_Malloc       && p_MemMngParam->f_Free &&
          p_MemMngParam->f_EarlyMalloc  && p_MemMngParam->f_EarlyFree))
    {
        REPORT_ERROR(MAJOR, E_NULL_POINTER, ("Malloc/Free callback routine"));
        return NULL;
    }

    p_MemMng = p_MemMngParam->f_Malloc(sizeof(t_MemMng));
    if (!p_MemMng)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Memory manager structure"));
        return NULL;
    }
    memset(p_MemMng, 0, sizeof(t_MemMng));

    p_MemMng->f_Malloc      = p_MemMngParam->f_Malloc;
    p_MemMng->f_Free        = p_MemMngParam->f_Free;
    p_MemMng->f_EarlyMalloc = p_MemMngParam->f_EarlyMalloc;
    p_MemMng->f_EarlyFree   = p_MemMngParam->f_EarlyFree;
    p_MemMng->h_Spinlock    = p_MemMngParam->h_Spinlock;

    /* Initialize internal partitions list */
    INIT_LIST(&(p_MemMng->memPartitionsList));
    /* Initialize the early allocations list */
    INIT_LIST(&(p_MemMng->earlyMemDebugList));

    return p_MemMng;
}


/*****************************************************************************/
void MEM_MNG_Free(t_Handle h_MemMng)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_List              *p_PartitionIterator, *p_TmpIterator;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN(p_MemMng, E_INVALID_HANDLE);

    if (p_MemMng->h_Spinlock)
    {
        intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
        LIST_FOR_EACH_SAFE(p_PartitionIterator,
                           p_TmpIterator,
                           &(p_MemMng->memPartitionsList))
        {
            MemMngFreePartition(p_MemMng, p_PartitionIterator);
        }
        XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
    }

    p_MemMng->f_Free(p_MemMng);
}


/*****************************************************************************/
int MEM_MNG_GetAvailablePartitionId(t_Handle h_MemMng)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    t_List              *p_PartitionIterator;
    int                 availableId;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_ERROR(p_MemMng, E_INVALID_HANDLE);

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    /* Return the next ID higher than any registered partition ID */
    LIST_FOR_EACH(p_PartitionIterator, &(p_MemMng->memPartitionsList))
    {
        p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        availableId = p_Partition->id + 1;
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    return availableId;
}


/*****************************************************************************/
t_Error MEM_MNG_RegisterPartition(t_Handle  h_MemMng,
                                  int       partitionId,
                                  uint32_t  baseAddress,
                                  uint32_t  size,
                                  uint32_t  attributes,
                                  char      name[],
                                  void *    (*f_UserMalloc)(uint32_t size, uint32_t alignment),
                                  void      (*f_UserFree)(void *p_Addr),
                                  bool      enableDebug)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition, *p_NewPartition;
    t_List              *p_PartitionIterator;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_ERROR(p_MemMng, E_INVALID_HANDLE);

    if ((f_UserMalloc || f_UserFree) && !(f_UserMalloc && f_UserFree))
    {
        RETURN_ERROR(MAJOR, E_CONFLICT,
                     ("f_UserMalloc and f_UserFree must be both NULL or not NULL"));
    }

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    LIST_FOR_EACH(p_PartitionIterator, &(p_MemMng->memPartitionsList))
    {
        p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        if (p_Partition->id == partitionId)
        {
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            RETURN_ERROR(MAJOR, E_ALREADY_EXISTS, ("Partition ID %d", partitionId));
        }
        else if (p_Partition->id > partitionId)
        {
            break;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    p_NewPartition = (t_MemMngPartition *)p_MemMng->f_Malloc(sizeof(t_MemMngPartition));
    if (!p_NewPartition)
    {
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Memory manager partition"));
    }
    memset(p_NewPartition, 0, sizeof(t_MemMngPartition));
    p_NewPartition->h_Spinlock = XX_InitSpinlock();
    if (!p_NewPartition->h_Spinlock)
    {
        p_MemMng->f_Free(p_NewPartition);
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("spinlock object for partition: %s", name));
    }

    if (!f_UserMalloc)
    {
        /* Prevent allocation of address 0x00000000 (reserved to NULL) */
        if (baseAddress == 0)
        {
            baseAddress += 4;
            size -= 4;
        }

        /* Initialize the memory manager handle for the new partition */
        if (E_OK != MM_Init(&(p_NewPartition->h_MemManager), baseAddress, size))
        {
            p_MemMng->f_Free(p_NewPartition);
            RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("MM object for partition: %s", name));
        }
    }

    /* Copy partition name */
    strncpy(p_NewPartition->info.name, name, MEM_MNG_MAX_PARTITION_NAME_LEN);

    /* Initialize debug entries list */
    INIT_LIST(&(p_NewPartition->memDebugList));

    /* Store other parameters */
    p_NewPartition->id = partitionId;
    p_NewPartition->info.baseAddress = baseAddress;
    p_NewPartition->info.size = size;
    p_NewPartition->info.attributes = attributes;
    p_NewPartition->f_UserMalloc = f_UserMalloc;
    p_NewPartition->f_UserFree = f_UserFree;
    p_NewPartition->enableDebug = enableDebug;

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    /* Add the new partition to the sorted position in the partitions list */
    if (LIST_IsEmpty(&(p_MemMng->memPartitionsList)))
    {
        LIST_Add(&(p_NewPartition->node), &(p_MemMng->memPartitionsList));
    }
    else
    {
        if (p_Partition->id < partitionId)
            LIST_Add(&(p_NewPartition->node), &(p_Partition->node));
        else
            LIST_Add(&(p_NewPartition->node), p_Partition->node.p_Prev);
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    return E_OK;
}


/*****************************************************************************/
t_Error MEM_MNG_UnregisterPartition(t_Handle h_MemMng, int partitionId)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    t_List              *p_PartitionIterator, *p_TmpIterator;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_ERROR(p_MemMng, E_INVALID_HANDLE);

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    /* Find the requested partition and release it */
    LIST_FOR_EACH_SAFE(p_PartitionIterator,
                       p_TmpIterator,
                       &(p_MemMng->memPartitionsList))
    {
        p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        if (p_Partition->id == partitionId)
        {
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            MemMngFreePartition(p_MemMng, p_PartitionIterator);
            return E_OK;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    RETURN_ERROR(MAJOR, E_NOT_SUPPORTED, ("Partition ID %d", partitionId));
}


/*****************************************************************************/
t_Error MEM_MNG_GetPartitionInfo(t_Handle               h_MemMng,
                                  int                   partitionId,
                                  t_MemMngPartitionInfo *p_PartitionInfo)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    t_List              *p_PartitionIterator;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_ERROR(p_MemMng, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_PartitionInfo, E_NULL_POINTER);

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    LIST_FOR_EACH(p_PartitionIterator, &(p_MemMng->memPartitionsList))
    {
        p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        if (p_Partition->id == partitionId)
        {
            *p_PartitionInfo = p_Partition->info;
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            return E_OK;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    RETURN_ERROR(MAJOR, E_NOT_FOUND, ("Partition ID %d", partitionId));
}


/*****************************************************************************/
t_Error MEM_MNG_GetPartitionIdByAddr(t_Handle   h_MemMng,
                                     uint32_t   addr,
                                     int        *p_PartitionId)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    bool                found;

    SANITY_CHECK_RETURN_ERROR(p_MemMng, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_PartitionId, E_NULL_POINTER);

    found = MemMngGetPartitionIdByAddr(p_MemMng, addr, p_PartitionId, &p_Partition);

    if (!found)
    {
        RETURN_ERROR(MAJOR, E_NOT_FOUND, ("Partition ID"));
    }

    return E_OK;
}


/*****************************************************************************/
uint32_t MEM_MNG_CheckLeaks(t_Handle                h_MemMng,
                            int                     partitionId,
                            t_MemMngLeakReportFunc  *f_ReportLeak)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    t_List              *p_PartitionIterator;
    t_List              *p_DebugIterator;
    uint32_t            count = 0;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_VALUE(p_MemMng, E_INVALID_HANDLE, 0);

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    if (partitionId == MEM_MNG_EARLY_PARTITION_ID)
    {
        if (f_ReportLeak)
        {
            LIST_FOR_EACH(p_DebugIterator, &(p_MemMng->earlyMemDebugList))
            {
                count++;

                p_MemMngDebugEntry = MEM_MNG_DBG_OBJECT(p_DebugIterator);

                f_ReportLeak((uint32_t)p_MemMngDebugEntry->p_Memory,
                             p_MemMngDebugEntry->size,
                             p_MemMngDebugEntry->info,
                             p_MemMngDebugEntry->filename,
                             p_MemMngDebugEntry->line);
            }
        }
        else
        {
            LIST_FOR_EACH(p_DebugIterator, &(p_MemMng->earlyMemDebugList))
            {
                count++;
            }
        }
        XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
        return count;
    }
    else
    {
        /* Search in registered partitions */
        LIST_FOR_EACH(p_PartitionIterator, &(p_MemMng->memPartitionsList))
        {
            p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

            if (p_Partition->id == partitionId)
            {
                XX_LockSpinlock(p_Partition->h_Spinlock);

                if (f_ReportLeak)
                {
                    LIST_FOR_EACH(p_DebugIterator, &(p_Partition->memDebugList))
                    {
                        count++;

                        p_MemMngDebugEntry = MEM_MNG_DBG_OBJECT(p_DebugIterator);

                        f_ReportLeak((uint32_t)p_MemMngDebugEntry->p_Memory,
                                     p_MemMngDebugEntry->size,
                                     p_MemMngDebugEntry->info,
                                     p_MemMngDebugEntry->filename,
                                     p_MemMngDebugEntry->line);
                    }
                }
                else
                    count = (uint32_t)LIST_NumOfObjs(&(p_Partition->memDebugList));

                XX_UnlockSpinlock(p_Partition->h_Spinlock);
                XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
                return count;
            }
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    REPORT_ERROR(MAJOR, E_NOT_FOUND, ("Partition ID %d", partitionId));
    return 0;
}


/*****************************************************************************/
void * MEM_MNG_AllocMem(t_Handle    h_MemMng,
                        int         partitionId,
                        uint32_t    size,
                        uint32_t    alignment,
                        char        *info,
                        char        *filename,
                        int         line)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    t_List              *p_PartitionIterator, *p_Temp;
    void                *p_Memory;
    uint32_t            intFlags;

    SANITY_CHECK_RETURN_VALUE(p_MemMng, E_INVALID_HANDLE, 0);

    if (size == 0)
    {
        REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("Allocation size must be positive"));
    }

    /* Check if this is an early allocation */
    if (partitionId == MEM_MNG_EARLY_PARTITION_ID)
    {
        /* Use early allocation routine */
        p_Memory = p_MemMng->f_EarlyMalloc(size, alignment);
        if (!p_Memory)
        {
            REPORT_ERROR(MINOR, E_NO_MEMORY, ("Early allocation"));
            return NULL;
        }

        MemMngAddEarlyEntry(p_MemMng,
                            p_Memory,
                            size,
                            info,
                            filename,
                            line);

        return p_Memory;
    }

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    /* Not early allocation - allocate from registered partitions */
    LIST_FOR_EACH_SAFE(p_PartitionIterator, p_Temp, &(p_MemMng->memPartitionsList))
    {
        p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        if (p_Partition->id == partitionId)
        {
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            if (p_Partition->f_UserMalloc)
            {
                /* User-defined malloc */
                p_Memory = p_Partition->f_UserMalloc(size, alignment);
                if (!p_Memory)
                    /* Do not report error - let the allocating entity report it */
                    return NULL;
            }
            else
            {
                /* Internal MM malloc */
                p_Memory = CAST_UINT64_TO_POINTER(
                    MM_Get(p_Partition->h_MemManager, size, (int32_t)alignment, ""));
                if ((uint32_t)p_Memory == ILLEGAL_BASE)
                    /* Do not report error - let the allocating entity report it */
                    return NULL;
            }
            intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);

            if (p_Partition->enableDebug)
            {
                MemMngAddEntry(p_MemMng,
                               p_Partition,
                               p_Memory,
                               size,
                               info,
                               filename,
                               line);
            }

            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            return p_Memory;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    REPORT_ERROR(MAJOR, E_NOT_FOUND, ("Partition ID %d", partitionId));
    return NULL;
}


/*****************************************************************************/
void MEM_MNG_FreeMem(t_Handle h_MemMng, void *p_Memory)
{
    t_MemMng            *p_MemMng = (t_MemMng *)h_MemMng;
    t_MemMngPartition   *p_Partition;
    uint32_t            addr = (uint32_t)p_Memory;
    int                 partitionId;
    bool                addressFound = TRUE;

    SANITY_CHECK_RETURN(p_MemMng, E_INVALID_HANDLE);

    /* Try to find the entry in the early allocations list */
    if (MemMngRemoveEarlyEntry(p_MemMng, p_Memory))
    {
        p_MemMng->f_EarlyFree(p_Memory);
    }
    else
    {
        if (MemMngGetPartitionIdByAddr(p_MemMng, addr, &partitionId, &p_Partition))
        {
            if (p_Partition->enableDebug &&
                !MemMngRemoveEntry(p_MemMng, p_Partition, p_Memory))
            {
                addressFound = FALSE;
            }

            if (addressFound)
            {
                if (p_Partition->f_UserFree)
                {
                    /* User-defined malloc */
                    p_Partition->f_UserFree(p_Memory);
                }
                else
                {
                    MM_Put(p_Partition->h_MemManager, CAST_POINTER_TO_UINT64(p_Memory));
                }
            }
        }
        else
        {
            addressFound = FALSE;
        }
    }

    if (!addressFound)
    {
        REPORT_ERROR(MAJOR, E_NOT_FOUND,
                     ("Attempt to free unallocated address (0x%08x)\n", p_Memory));
    }
}


/*****************************************************************************/
static bool MemMngGetPartitionIdByAddr(t_MemMng          *p_MemMng,
                                          uint32_t          addr,
                                          int               *p_PartitionId,
                                          t_MemMngPartition **p_Partition)
{
    t_MemMngPartition   *p_TmpPartition;
    t_List              *p_PartitionIterator;
    uint32_t            intFlags;

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    LIST_FOR_EACH(p_PartitionIterator, &(p_MemMng->memPartitionsList))
    {
        p_TmpPartition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

        if ((addr >= p_TmpPartition->info.baseAddress) &&
            (addr < (p_TmpPartition->info.baseAddress + p_TmpPartition->info.size)))
        {
            *p_PartitionId = p_TmpPartition->id;
            *p_Partition = p_TmpPartition;
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            return TRUE;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    return FALSE;
}


/*****************************************************************************/
static void MemMngFreePartition(t_MemMng *p_MemMng, t_List *p_PartitionIterator)
{
    t_MemMngPartition   *p_Partition;
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    t_List              *p_DebugIterator, *p_TmpIterator;
    uint32_t            intFlags;

    p_Partition = MEM_MNG_PARTITION_OBJECT(p_PartitionIterator);

    intFlags = XX_LockIntrSpinlock(p_Partition->h_Spinlock);
    /* Release the debug entries list */
    LIST_FOR_EACH_SAFE(p_DebugIterator, p_TmpIterator, &(p_Partition->memDebugList))
    {
        p_MemMngDebugEntry = MEM_MNG_DBG_OBJECT(p_DebugIterator);

        LIST_Del(p_DebugIterator);
        p_MemMng->f_Free(p_MemMngDebugEntry);
    }
    XX_UnlockIntrSpinlock(p_Partition->h_Spinlock, intFlags);

    if (!p_Partition->f_UserMalloc)
    {
        /* Release the memory manager object */
        MM_Free(p_Partition->h_MemManager);
    }

    /* Remove from partitions list and free the allocated memory */
    LIST_Del(p_PartitionIterator);
    if (p_Partition->h_Spinlock)
        XX_FreeSpinlock(p_Partition->h_Spinlock);
    p_MemMng->f_Free(p_Partition);
}


/*****************************************************************************/
static void MemMngAddEarlyEntry(t_MemMng    *p_MemMng,
                                void        *p_Memory,
                                uint32_t    size,
                                char        *info,
                                char        *filename,
                                int         line)
{
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    uint32_t            intFlags;

    p_MemMngDebugEntry =
        (t_MemMngDebugEntry *)p_MemMng->f_Malloc(sizeof(t_MemMngDebugEntry));

    if (p_MemMngDebugEntry != NULL)
    {
        INIT_LIST(&p_MemMngDebugEntry->node);
        p_MemMngDebugEntry->p_Memory = p_Memory;
        p_MemMngDebugEntry->filename = filename;
        p_MemMngDebugEntry->info = info;
        p_MemMngDebugEntry->line = line;
        p_MemMngDebugEntry->size = size;
        intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
        LIST_AddToTail(&p_MemMngDebugEntry->node, &(p_MemMng->earlyMemDebugList));
        XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
    }
    else
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Memory manager debug entry"));
    }
}


/*****************************************************************************/
static bool MemMngRemoveEarlyEntry(t_MemMng *p_MemMng, void *p_Memory)
{
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    t_List              *p_DebugIterator, *p_TmpIterator;
    uint32_t            intFlags;

    intFlags = XX_LockIntrSpinlock(p_MemMng->h_Spinlock);
    LIST_FOR_EACH_SAFE(p_DebugIterator, p_TmpIterator, &(p_MemMng->earlyMemDebugList))
    {
        p_MemMngDebugEntry = MEM_MNG_DBG_OBJECT(p_DebugIterator);

        if (p_MemMngDebugEntry->p_Memory == p_Memory)
        {
            LIST_Del(p_DebugIterator);
            XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);
            p_MemMng->f_Free(p_MemMngDebugEntry);
            return TRUE;
        }
    }
    XX_UnlockIntrSpinlock(p_MemMng->h_Spinlock, intFlags);

    return FALSE;
}


/*****************************************************************************/
static void MemMngAddEntry(t_MemMng             *p_MemMng,
                           t_MemMngPartition    *p_Partition,
                           void                 *p_Memory,
                           uint32_t             size,
                           char                 *info,
                           char                 *filename,
                           int                  line)
{
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    uint32_t            intFlags;

    p_MemMngDebugEntry =
        (t_MemMngDebugEntry *)p_MemMng->f_Malloc(sizeof(t_MemMngDebugEntry));

    if (p_MemMngDebugEntry != NULL)
    {
        INIT_LIST(&p_MemMngDebugEntry->node);
        p_MemMngDebugEntry->p_Memory = p_Memory;
        p_MemMngDebugEntry->filename = filename;
        p_MemMngDebugEntry->info = info;
        p_MemMngDebugEntry->line = line;
        p_MemMngDebugEntry->size = size;
        intFlags = XX_LockIntrSpinlock(p_Partition->h_Spinlock);
        LIST_AddToTail(&p_MemMngDebugEntry->node, &(p_Partition->memDebugList));
        XX_UnlockIntrSpinlock(p_Partition->h_Spinlock, intFlags);

        p_Partition->info.currentUsage += size;
        p_Partition->info.totalAllocations += 1;

        if (p_Partition->info.currentUsage > p_Partition->info.maximumUsage)
        {
            p_Partition->info.maximumUsage = p_Partition->info.currentUsage;
        }
    }
    else
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Memory manager debug entry"));
    }
}


/*****************************************************************************/
static bool MemMngRemoveEntry(t_MemMng          *p_MemMng,
                              t_MemMngPartition *p_Partition,
                              void              *p_Memory)
{
    t_MemMngDebugEntry  *p_MemMngDebugEntry;
    t_List              *p_DebugIterator, *p_TmpIterator;
    uint32_t            intFlags;

    intFlags = XX_LockIntrSpinlock(p_Partition->h_Spinlock);

    LIST_FOR_EACH_SAFE(p_DebugIterator, p_TmpIterator, &(p_Partition->memDebugList))
    {
        p_MemMngDebugEntry = MEM_MNG_DBG_OBJECT(p_DebugIterator);

        if (p_MemMngDebugEntry->p_Memory == p_Memory)
        {
            p_Partition->info.currentUsage -= p_MemMngDebugEntry->size;
            p_Partition->info.totalDeallocations += 1;

            LIST_Del(p_DebugIterator);
            XX_UnlockIntrSpinlock(p_Partition->h_Spinlock, intFlags);
            p_MemMng->f_Free(p_MemMngDebugEntry);
            return TRUE;
        }
    }

    XX_UnlockIntrSpinlock(p_Partition->h_Spinlock, intFlags);

    return FALSE;
}


