/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          shared_memory.c

 @Description   Prototypes, externals and typedefs for shared_memory resource
*//***************************************************************************/

#include "shared_memory.h"
#include "error_ext.h"
#include "list_ext.h"
#include "string_ext.h"
#include "sys_platform_ext.h"

/* Get a message from queue */
static t_Msg*   SharedMemoryDequeueMsg(t_SharedMemory *p_SharedMemory,
                                       uint32_t        queueId);

/* Put a message to a specific queue */
static void     SharedMemoryEnqueueMsg(t_SharedMemory *p_SharedMemory,
                                       t_Msg          *p_Msg,
                                       uint32_t        queueId);

static t_Error  SharedMemoryCheckEarlyInitParameters(t_SharedMemoryEarlyInitParam *p_Param);


/*****************************************************************************/
t_Error SHARED_MEMORY_EarlyInit(t_SharedMemoryEarlyInitParam *p_Param)
{
    t_SharedMemoryMap         *p_SharedMemoryMap;
    t_MsgQueue                *p_TempMessageQueue;
    t_Msg                     *p_Msg, *p_CurrMessage;
    uint64_t                  msgQueueAddress, msgOffset;
    uint32_t                  i;

    SANITY_CHECK_RETURN_VALUE(p_Param, E_NULL_POINTER, NULL);

    CHECK_INIT_PARAMETERS(p_Param, SharedMemoryCheckEarlyInitParameters);

    /**********************************************************/
    /* Shared memory map initialization                       */
    /**********************************************************/

    p_SharedMemoryMap = CAST_UINT64_TO_POINTER_TYPE(t_SharedMemoryMap, p_Param->baseAddress);
    memset((void *)p_SharedMemoryMap, 0, p_Param->sharedMemorySize);

    p_SharedMemoryMap->totalMessageQueueLength = sizeof(t_MsgQueue);
    while(p_SharedMemoryMap->totalMessageQueueLength & (p_Param->queueAlignment-1))
        p_SharedMemoryMap->totalMessageQueueLength++;

    /* Calculate the required message length */
    p_SharedMemoryMap->totalMessageLength = p_Param->msgBodyLength + sizeof(p_Msg->nextMsgOffset) + sizeof(p_Msg->msgLength);

    /* Initialize the message queue offset in the shared memory */
    msgQueueAddress = CAST_POINTER_TO_UINT64(p_SharedMemoryMap) + sizeof(t_SharedMemoryMap);
    while (msgQueueAddress & (p_Param->queueAlignment-1))
        msgQueueAddress++;

    p_SharedMemoryMap->msgQueueOffset = msgQueueAddress - CAST_POINTER_TO_UINT64(p_SharedMemoryMap);

    /* Initialize the available messages offset in the shared memory */
    p_SharedMemoryMap->availableMsgsOffset = p_SharedMemoryMap->msgQueueOffset + (p_SharedMemoryMap->totalMessageQueueLength * p_Param->numOfMessageQueues);

    /* Initialize the available messages list */
    msgOffset = p_SharedMemoryMap->availableMsgsOffset;
    for (i = 0; i < p_Param->numOfMessages; i++)
    {
        p_CurrMessage = CAST_UINT64_TO_POINTER_TYPE(t_Msg, CAST_POINTER_TO_UINT64(p_SharedMemoryMap) + msgOffset);
        msgOffset += p_SharedMemoryMap->totalMessageLength;
        p_CurrMessage->nextMsgOffset = msgOffset;
    }

    /* Fix the last message next pointer */
    p_CurrMessage->nextMsgOffset = 0;

    /* Initialize AVAILABLE_MSGS_QUEUE to point to all available messages */
    p_TempMessageQueue = CAST_UINT64_TO_POINTER_TYPE(t_MsgQueue,
        CAST_POINTER_TO_UINT64(p_SharedMemoryMap) + p_SharedMemoryMap->msgQueueOffset + AVAILABLE_MSGS_QUEUE * p_SharedMemoryMap->totalMessageQueueLength);

    p_TempMessageQueue->headOffset = p_SharedMemoryMap->availableMsgsOffset;
    p_TempMessageQueue->tailOffset = p_TempMessageQueue->headOffset + p_SharedMemoryMap->totalMessageLength * (p_Param->numOfMessages-1);

    /* Initialize all the other queues to be empty */
    for (i = (AVAILABLE_MSGS_QUEUE + 1); i < p_Param->numOfMessageQueues; i++)
    {
        p_TempMessageQueue = CAST_UINT64_TO_POINTER_TYPE(t_MsgQueue,
            CAST_POINTER_TO_UINT64(p_SharedMemoryMap) + p_SharedMemoryMap->msgQueueOffset + p_SharedMemoryMap->totalMessageQueueLength * i);

        p_TempMessageQueue->headOffset = 0;
        p_TempMessageQueue->tailOffset = 0;
    }

    return E_OK;
}

/*****************************************************************************/
t_Handle SHARED_MEMORY_Init(t_SharedMemoryParam *p_Param)
{
    t_SharedMemory  *p_SharedMemory;

    SANITY_CHECK_RETURN_VALUE(p_Param, E_NULL_POINTER, NULL);

    p_SharedMemory = (t_SharedMemory *)XX_Malloc(sizeof(t_SharedMemory));
    if (!p_SharedMemory)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Shared memory driver structure"));
        return NULL;
    }

    /* Save user parameters in the internal structure */
    p_SharedMemory->id                      = p_Param->id;
    p_SharedMemory->h_App                   = p_Param->h_App;
    p_SharedMemory->f_TxConf                = p_Param->f_TxConf;
    p_SharedMemory->f_RxStore               = p_Param->f_RxStore;
    p_SharedMemory->memoryMapAddress        = p_Param->baseAddress;
    p_SharedMemory->interrupts              = p_Param->interrupts;
    p_SharedMemory->irq                     = p_Param->irq;

    return p_SharedMemory;
}

/*****************************************************************************/
t_Error SHARED_MEMORY_Free(t_Handle h_SharedMemory)
{
    t_SharedMemory  *p_SharedMemory = (t_SharedMemory *)h_SharedMemory;

    SANITY_CHECK_RETURN_ERROR(p_SharedMemory, E_INVALID_HANDLE);

    XX_Free(p_SharedMemory);

    return E_OK;
}

/*****************************************************************************/
t_Error SHARED_MEMORY_Write(t_Handle   h_SharedMemory,
                            uint32_t   queueId,
                            uint8_t   *p_Data,
                            uint32_t   length)
{
    t_SharedMemory     *p_SharedMemory = (t_SharedMemory *)h_SharedMemory;
    t_SharedMemoryMap  *p_SharedMemoryMap = CAST_UINT64_TO_POINTER_TYPE(t_SharedMemoryMap, p_SharedMemory->memoryMapAddress);
    t_Msg              *p_Msg;
    uint32_t            i, allowedMsgLength;

    SANITY_CHECK_RETURN_ERROR(p_SharedMemory, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Data, E_NULL_POINTER);
    SANITY_CHECK_RETURN_ERROR(queueId, E_INVALID_VALUE);

    allowedMsgLength = p_SharedMemoryMap->totalMessageLength - sizeof(t_Msg) - sizeof(p_Msg->msgBody) ;
    if (length > allowedMsgLength )
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Shared memory write - message length exceeds the maximum (%d", allowedMsgLength));

    /* Get an available message */
    p_Msg = SharedMemoryDequeueMsg(p_SharedMemory, AVAILABLE_MSGS_QUEUE);
    if (p_Msg == NULL)
    {
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Shared memory write - no available message"));
    }

    /* Clean the message not to contain old content */
    memset(p_Msg, 0, p_SharedMemoryMap->totalMessageLength);

    /* Copy the p_Data to the shared memory (t_Msg) */
    for (i = 0; i < length; i++)
    {
        p_Msg->msgBody[i] = p_Data[i];
    }
    p_Msg->msgLength = length;

    /* Enqueue the new message to the required queue */
    SharedMemoryEnqueueMsg(p_SharedMemory, p_Msg, queueId);

    /* Call to f_TxConf if exists */
    if (p_SharedMemory->f_TxConf)
        p_SharedMemory->f_TxConf(p_SharedMemory->h_App, queueId, p_Data, length);

    return E_OK;
}

/*****************************************************************************/
t_Error SHARED_MEMORY_ReadConf(t_Handle  h_SharedMemory,
                               uint32_t  queueId,
                               uint8_t  *p_Data)
{
    t_SharedMemory      *p_SharedMemory = (t_SharedMemory *)h_SharedMemory;
    t_Msg               *p_Msg = NULL;

    SANITY_CHECK_RETURN_ERROR(p_Data, E_NULL_POINTER);

    UNUSED(queueId);

    p_Msg = (t_Msg *)(p_Data - sizeof(p_Msg->nextMsgOffset) - sizeof(p_Msg->msgLength));

    /* Return the t_Msg to the available message queue */
    SharedMemoryEnqueueMsg(p_SharedMemory, p_Msg, AVAILABLE_MSGS_QUEUE);

    return E_OK;
}


/*****************************************************************************/
t_Error SHARED_MEMORY_Read(t_Handle h_SharedMemory, uint32_t queueId)
{
    t_Msg               *p_Msg = NULL;
    t_SharedMemory      *p_SharedMemory = (t_SharedMemory *)h_SharedMemory;

    SANITY_CHECK_RETURN_ERROR(p_SharedMemory, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_VALUE(queueId, 0, E_INVALID_VALUE);

    /* Check the relevant queue to see if there is a new message */
    p_Msg = SharedMemoryDequeueMsg(p_SharedMemory, queueId);
    while (p_Msg)
    {
        if (p_SharedMemory->f_RxStore)
            p_SharedMemory->f_RxStore(p_SharedMemory->h_App, queueId, (uint8_t*)&(p_Msg->msgBody[0]), p_Msg->msgLength);

        p_Msg = SharedMemoryDequeueMsg(p_SharedMemory, queueId);
    }

    return E_OK;
}

/*****************************************************************************/
t_Error SHARED_MEMORY_Interrupt(t_Handle h_SharedMemory)
{
    t_SharedMemory     *p_SharedMemory = (t_SharedMemory*)h_SharedMemory;
    t_Error            errCode;
    uint32_t           i = 0;
    t_SharedMemoryMap  *p_SharedMemoryMap = CAST_UINT64_TO_POINTER_TYPE(t_SharedMemoryMap, p_SharedMemory->memoryMapAddress);
    uint64_t           msgQueueOffset;

    SANITY_CHECK_RETURN_ERROR(p_SharedMemory, E_INVALID_HANDLE);

    /* Check all the queues in the shared memory and call to
       SHARED_MEMORY_Rx for each one of them */
    msgQueueOffset = p_SharedMemoryMap->msgQueueOffset;
    while (msgQueueOffset < p_SharedMemoryMap->availableMsgsOffset)
    {
        errCode = SHARED_MEMORY_Read(h_SharedMemory, i);
        if (errCode != E_OK)
            RETURN_ERROR(MAJOR, errCode, NO_MSG);

        msgQueueOffset += p_SharedMemoryMap->totalMessageQueueLength;
        i++;
    }

    return E_OK;
}

/*****************************************************************************/
static t_Error SharedMemoryCheckEarlyInitParameters(t_SharedMemoryEarlyInitParam *p_Param)
{
    uint64_t        sharedMemorySize;
    uint32_t        totalMsgLength, totalMessageQueueLength;
    t_Msg          *p_Msg;

    if (!p_Param->sharedMemorySize)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Shared memory size"));
    }

     if (!p_Param->msgBodyLength)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Shared memory message body length"));
    }

    if (!p_Param->numOfMessages)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Shared memory number of messages"));
    }

    if (p_Param->numOfMessageQueues < 3)
    {
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Shared memory number of queues"));
    }

    totalMsgLength = p_Param->msgBodyLength + sizeof(p_Msg->nextMsgOffset) + sizeof(p_Msg->msgLength);

    totalMessageQueueLength = sizeof(t_MsgQueue);
    while(totalMessageQueueLength & (p_Param->queueAlignment-1))
        totalMessageQueueLength++;

    /* Calculate the total required size of the shared memory */
    sharedMemorySize = (totalMsgLength * p_Param->numOfMessages) +
                       (totalMessageQueueLength * p_Param->numOfMessageQueues) +
                       sizeof(t_SharedMemoryMap);

    /* Check that total shared memory size is enough for the number of required queues and messages */
    if (sharedMemorySize > p_Param->sharedMemorySize)
    {
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("Shared memory size is too small"));
    }

    return E_OK;
}

/*****************************************************************************/
/* Get a message from queue */
static t_Msg * SharedMemoryDequeueMsg(t_SharedMemory *p_SharedMemory, uint32_t queueId)
{
    uint64_t          memoryMapAddress;
    t_SharedMemoryMap *p_SharedMemoryMap;
    t_Msg             *p_CurrMsg = NULL;
    t_MsgQueue        *p_MsgQueue;

    memoryMapAddress = p_SharedMemory->memoryMapAddress;
    p_SharedMemoryMap = CAST_UINT64_TO_POINTER_TYPE(t_SharedMemoryMap, memoryMapAddress);

    p_MsgQueue = CAST_UINT64_TO_POINTER_TYPE(t_MsgQueue,
        memoryMapAddress + p_SharedMemoryMap->msgQueueOffset + queueId * p_SharedMemoryMap->totalMessageQueueLength);

    /* Lock message queue */
    while (CORE_TestAndSet(&p_MsgQueue->lock) == QUEUE_UNLOCK_FLAG)
    {
        while (p_MsgQueue->lock == QUEUE_UNLOCK_FLAG) ;
    }
    /* Wait for all previous instructions to complete */
    CORE_InstructionSync();

    /* Update the head field because we dequeue the first message */
    if (p_MsgQueue->headOffset != 0)
    {
        p_CurrMsg = CAST_UINT64_TO_POINTER_TYPE(t_Msg, memoryMapAddress + p_MsgQueue->headOffset);

        /* Check if it's the only message that exists in the queue */
        if (p_MsgQueue->headOffset == p_MsgQueue->tailOffset)
            p_MsgQueue->tailOffset = 0;

        p_MsgQueue->headOffset = p_CurrMsg->nextMsgOffset;
    }

    /* Unlock message queue */
    CORE_MemoryBarrier();
    p_MsgQueue->lock = QUEUE_UNLOCK_FLAG;

    return p_CurrMsg;
}

/*****************************************************************************/
/* Put a message to a specific queue */
static void SharedMemoryEnqueueMsg(t_SharedMemory *p_SharedMemory, t_Msg *p_Msg, uint32_t queueId)
{
    uint64_t          memoryMapAddress;
    t_SharedMemoryMap *p_SharedMemoryMap;
    t_Msg             *p_LastMsg;
    t_MsgQueue        *p_MsgQueue;

    memoryMapAddress = p_SharedMemory->memoryMapAddress;
    p_SharedMemoryMap = CAST_UINT64_TO_POINTER_TYPE(t_SharedMemoryMap, memoryMapAddress);

    p_MsgQueue = CAST_UINT64_TO_POINTER_TYPE(t_MsgQueue,
        memoryMapAddress + p_SharedMemoryMap->msgQueueOffset + queueId * p_SharedMemoryMap->totalMessageQueueLength);

    /* Lock message queue */
    while (CORE_TestAndSet(&p_MsgQueue->lock) == QUEUE_UNLOCK_FLAG)
    {
        while (p_MsgQueue->lock == QUEUE_UNLOCK_FLAG) ;
    }
    /* Wait for all previous instructions to complete */
    CORE_InstructionSync();

    if (p_MsgQueue->tailOffset != 0)
    {
        p_LastMsg = CAST_UINT64_TO_POINTER_TYPE(t_Msg, memoryMapAddress + p_MsgQueue->tailOffset);
        p_LastMsg->nextMsgOffset = CAST_POINTER_TO_UINT64(p_Msg) - memoryMapAddress;
    }
    else
        /* This is the first message in the queue */
        p_MsgQueue->headOffset = CAST_POINTER_TO_UINT64(p_Msg) - memoryMapAddress;


    p_Msg->nextMsgOffset = 0;
    p_MsgQueue->tailOffset = CAST_POINTER_TO_UINT64(p_Msg) - memoryMapAddress;

    /* Unlock message queue */
    CORE_MemoryBarrier();
    p_MsgQueue->lock = QUEUE_UNLOCK_FLAG;
}
