/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
#include "error_ext.h"
#include "std_ext.h"
#include "string_ext.h"
#include "xx_ext.h"
#include "sprint_ext.h"
#include "stop_watch.h"

#define __ERR_MODULE__  MODULE_UNKNOWN

t_Handle h_StopWatch;

t_Handle STW_Config( t_StwParam *p_StwParam )
{
    t_Stw             *p_Stw;

    SANITY_CHECK_RETURN_VALUE( p_StwParam, E_NULL_POINTER, NULL );

    p_Stw = (t_Stw*)XX_Malloc( sizeof(t_Stw) );
    if( !p_Stw )
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("Stop Watch structure"));
        return NULL;
    }
    /* Zero out the stop watch object.*/
    memset( p_Stw, 0, sizeof(t_Stw) );

    p_Stw->f_GetTime = p_StwParam->f_GetTime;
    p_Stw->h_TimeClk = p_StwParam->h_TimeClk;

    return p_Stw;
}

t_Error STW_Init( t_Handle h_Stw )
{
    uint32_t id;
    t_Error rc;
    t_Stw   *p_Stw = (t_Stw*)h_Stw;

    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );

    rc = STW_GetId( p_Stw, "Global", &id );
    SANITY_CHECK_RETURN_ERROR( rc==E_OK, rc );

    p_Stw->globalTimer = id;
    p_Stw->currentTimer = id;
    STW_Start( p_Stw, id );

    return E_OK;
}

t_Error STW_GetId( t_Handle h_Stw, char *sectionName, uint32_t* p_Id )
{
    uint32_t i;
    t_Stw               *p_Stw;
    t_StwSectionTimer   *p_StwSectionTimer;

    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );

    p_Stw = (t_Stw*)h_Stw;

    for( i=0; i<MAX_NUM_OF_SECTIONS; i++ )
    {
        if(!p_Stw->p_StwSectionTimers[i])
            break;
    }
    if( i >= MAX_NUM_OF_SECTIONS )
    {
        RETURN_ERROR( MAJOR, E_CONFLICT, ("the number of sections timed has exceeded"
                      "the maximum allowed number (maximum is: %d)", MAX_NUM_OF_SECTIONS ));
    }

    p_Stw->p_StwSectionTimers[i] = (t_StwSectionTimer*)XX_Malloc( sizeof(t_StwSectionTimer) );
    if( !p_Stw->p_StwSectionTimers[i] )
    {
        RETURN_ERROR( MAJOR, E_NO_MEMORY, NO_MSG );
    }

    p_StwSectionTimer = p_Stw->p_StwSectionTimers[i];

    p_StwSectionTimer->name = (char*)XX_Malloc( strlen(sectionName) + 1 );
    if( !p_StwSectionTimer->name )
    {
        RETURN_ERROR( MAJOR, E_NO_MEMORY, NO_MSG );
    }

    strcpy( p_StwSectionTimer->name, sectionName );

    STW_Clear( h_Stw, i );

    *p_Id = i;

    return E_OK;
}

t_Error STW_Start( t_Handle h_Stw, uint32_t id )
{
    t_Stw   *p_Stw = (t_Stw*)h_Stw;
    t_StwSectionTimer   *p_StwSectionTimer;
    t_StwSectionTimer   *p_StwCurrentTimer;
    uint32_t            intFlags;

    intFlags = XX_DisableAllIntr();
    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );

    p_StwSectionTimer = p_Stw->p_StwSectionTimers[id];
    SANITY_CHECK_RETURN_ERROR( p_StwSectionTimer, E_NULL_POINTER );

    p_StwCurrentTimer = p_Stw->p_StwSectionTimers[p_Stw->currentTimer];


    if ( id != p_Stw->globalTimer )
    {
        /* Pause the current running timer */
        if ( p_StwCurrentTimer->pause == 0 )
        {
            p_StwCurrentTimer->pause = p_Stw->f_GetTime( p_Stw->h_TimeClk );;
        }
        else
        {
            char message[1024];
            sprintf( message, "STW_Start: %s tried to pause %s. but %s is already on pause",
                       p_StwSectionTimer->name,
                       p_StwCurrentTimer->name,
                       p_StwCurrentTimer->name );
            RETURN_ERROR( MAJOR, E_ALREADY_EXISTS, (message) );
        }
    }

    p_StwSectionTimer->lastTimer = p_Stw->currentTimer;
    p_Stw->currentTimer = id;

    if ( p_StwSectionTimer->time == 0)
        p_StwSectionTimer->time = p_Stw->f_GetTime( p_Stw->h_TimeClk );
    else
    {
        char message[1024];
        sprintf(message,"STW_Start: %s has called STW_Start twice in a row (without STW_Stop)", p_StwSectionTimer->name );
        RETURN_ERROR( MAJOR, E_ALREADY_EXISTS, (message) );
    }
    XX_RestoreAllIntr(intFlags);
    return E_OK;
}

t_Error STW_Stop( t_Handle h_Stw, uint32_t id  )
{
    uint64_t time;
    t_Stw   *p_Stw = (t_Stw*)h_Stw;
    t_StwSectionTimer   *p_StwSectionTimer;
    t_StwSectionTimer   *p_StwCurrentTimer;
    uint32_t       intFlags;

    intFlags = XX_DisableAllIntr();
    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );
    time = p_Stw->f_GetTime( p_Stw->h_TimeClk );

    p_StwSectionTimer = p_Stw->p_StwSectionTimers[id];
    SANITY_CHECK_RETURN_ERROR( p_StwSectionTimer, E_NULL_POINTER );


    if( p_StwSectionTimer->time != 0)
    {
        uint64_t currTime = time - p_StwSectionTimer->time;
        p_StwSectionTimer->sumTime += currTime;
        p_StwSectionTimer->minTime = p_StwSectionTimer->minTime > currTime ? currTime: p_StwSectionTimer->minTime;
        p_StwSectionTimer->maxTime = p_StwSectionTimer->maxTime < currTime ? currTime: p_StwSectionTimer->maxTime;
        p_StwSectionTimer->numTime += 1;
        p_StwSectionTimer->time = 0;
    }
    else
    {
        char message[1024];
        sprintf(message,"STW_Stop: %s has called STW_Stop twice in a row (without STW_Start)",p_StwSectionTimer->name);
        RETURN_ERROR( MAJOR, E_ALREADY_EXISTS, (message) );
    }

    if ( id != p_Stw->globalTimer )
    {
        p_Stw->currentTimer = p_StwSectionTimer->lastTimer;
        p_StwCurrentTimer = p_Stw->p_StwSectionTimers[p_Stw->currentTimer];
        if ( p_StwCurrentTimer->pause != 0)
        {
            uint64_t pause = time - p_StwCurrentTimer->pause;
            p_StwCurrentTimer->sumPause += pause;
            p_StwCurrentTimer->minPause = p_StwCurrentTimer->minPause > pause ? pause: p_StwCurrentTimer->minPause;
            p_StwCurrentTimer->maxPause = p_StwCurrentTimer->maxPause < pause ? pause: p_StwCurrentTimer->maxPause;
            p_StwCurrentTimer->numPause += 1;
            p_StwCurrentTimer->pause = 0;
        }
        else
        {
            char message[1024];
            sprintf( message, "STW_Stop: %s tried to resume %s which is already running",
                       p_StwSectionTimer->name,
                       p_StwCurrentTimer->name);
            RETURN_ERROR( MAJOR, E_ALREADY_EXISTS, (message) );
        }
    }
    else
    {
        p_Stw->currentTimer = p_Stw->currentTimer;
    }
    XX_RestoreAllIntr(intFlags);
    return E_OK;
}

t_Error STW_Clear( t_Handle h_Stw, uint32_t id )
{
    t_StwSectionTimer   *p_StwSectionTimer;
    p_StwSectionTimer = ((t_Stw*)h_Stw)->p_StwSectionTimers[id];
    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );
    SANITY_CHECK_RETURN_ERROR( p_StwSectionTimer, E_NULL_POINTER );

    p_StwSectionTimer->sumTime = 0;
    p_StwSectionTimer->minTime = 0xFFFFFFFFFFFFFFFFLL;
    p_StwSectionTimer->maxTime = 0;
    p_StwSectionTimer->numTime = 0;
    p_StwSectionTimer->time = 0;
    p_StwSectionTimer->sumPause = 0;
    p_StwSectionTimer->minPause = 0xFFFFFFFFFFFFFFFFLL;
    p_StwSectionTimer->maxPause = 0;
    p_StwSectionTimer->numPause = 0;
    p_StwSectionTimer->pause = 0;

    return E_OK;
}


t_Error STW_Print( t_Handle h_Stw )
{
    uint32_t id;
    t_Stw   *p_Stw = (t_Stw*)h_Stw;
    uint64_t time = p_Stw->f_GetTime( p_Stw->h_TimeClk );
    uint64_t timeElapsed;

    SANITY_CHECK_RETURN_ERROR( h_Stw, E_NULL_POINTER );

    timeElapsed = time - p_Stw->p_StwSectionTimers[p_Stw->globalTimer]->time;

/*
    XX_Print("\nTime elapsed: %16llx,\n",timeElapsed);
    XX_Print("%12s, %16s, %16s, %16s, %11s"
             ", %16s, %16s, %16s, %11s\n"
             "=========================================================================="
             "==========================================================================\n",
             "Name", "total time", "min time", "max time", "num time",
             "total pause", "min pause", "max pause", "num pause" );

    XX_Print( "%12s, %16llx\n", p_Stw->p_StwSectionTimers[p_Stw->globalTimer]->name, timeElapsed );

    for ( id = 1; id <MAX_NUM_OF_SECTIONS && p_Stw->p_StwSectionTimers[id] != NULL; id++)
    {
        XX_Print( "%12s, %16llu, %16llu, %16llu, %11u, %16llu, %16llu, %16llu, %11u\n",
                p_Stw->p_StwSectionTimers[id]->name,
                p_Stw->p_StwSectionTimers[id]->sumTime,
                p_Stw->p_StwSectionTimers[id]->minTime,
                p_Stw->p_StwSectionTimers[id]->maxTime,
                p_Stw->p_StwSectionTimers[id]->numTime,
                p_Stw->p_StwSectionTimers[id]->sumPause,
                p_Stw->p_StwSectionTimers[id]->minPause,
                p_Stw->p_StwSectionTimers[id]->maxPause,
                p_Stw->p_StwSectionTimers[id]->numPause );

    }
*/

    XX_Print("\n%12s, %18s, %8s\n"
             "=========================================================\n",
             "Name", "AVG TB cycles", "Ratio" );

    for ( id = 1; id <MAX_NUM_OF_SECTIONS && p_Stw->p_StwSectionTimers[id] != NULL; id++)
    {
        uint64_t avgDurationInCyc;
        float ratio;

        if (!p_Stw->p_StwSectionTimers[id]->numTime)
        {
            XX_Print("%12s, not used\n",
                     p_Stw->p_StwSectionTimers[id]->name);
            continue;
        }
        avgDurationInCyc =  /* Time Base Cycles.*/
            (p_Stw->p_StwSectionTimers[id]->sumTime - p_Stw->p_StwSectionTimers[id]->sumPause)
            /p_Stw->p_StwSectionTimers[id]->numTime;
        ratio = (float)(p_Stw->p_StwSectionTimers[id]->sumTime - p_Stw->p_StwSectionTimers[id]->sumPause)
                      /timeElapsed;

        XX_Print("%12s, %18llu, %8.2f%s\n",
                 p_Stw->p_StwSectionTimers[id]->name,
                 avgDurationInCyc,
                 (ratio*100), "%");
    }

    return E_OK;
}
