/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          cli.c

 @Description   Command Line Interface module implementation
*//***************************************************************************/

#include "std_ext.h"
#include "stdlib_ext.h"
#include "string_ext.h"
#include "stdarg_ext.h"
#include "error_ext.h"
#include "xx_ext.h"

#include "cli.h"


static void PrintFSLLogo(t_Cli *p_Cli)
{
    p_Cli->f_Print("Welcome to Freescale (TM) CLI.\n");

}

static void Usage(t_Cli *p_Cli)
{
    p_Cli->f_Print("Usage: > <command> [options] [arg1 ... argN]\n");
    p_Cli->f_Print("Type ? for help.\n");
}

static void Help(t_Cli *p_Cli)
{
    t_CliCommand    *p_Command;
    t_List          *p_Pos;

    p_Cli->f_Print("Usage: > <command> [options] [arg1 ... argN]\n"
             "\n"
             "Commands:\n");

    LIST_FOR_EACH(p_Pos, &p_Cli->commands)
    {
        p_Command = CLI_COMMAND_OBJ(p_Pos);
        p_Cli->f_Print("\t- %s\n", p_Command->format);
        p_Cli->f_Print("\t\t%s\n", p_Command->desc);
    }

}

static void DoMemDisp(t_Cli *p_Cli, uint8_t *addr, int numOfBytes)
{
    uint8_t* i;
    int size16Aling = (numOfBytes>>4)<<4;
    int size4Aling = (numOfBytes>>2)<<2;
    int notAlign = FALSE;
    if(numOfBytes%16)
        notAlign = TRUE;

    for(i=addr;(uint32_t)i<(uint32_t)addr+size16Aling;i+=16)
        p_Cli->f_Print("0x%08x: %08lx %08lx %08lx %08lx\n",
                i,
                *((uint32_t*)(i)),
                *((uint32_t*)(i+4)),
                *((uint32_t*)(i+8)),
                *((uint32_t*)(i+12)));
    if(notAlign)
        p_Cli->f_Print("0x%08x: ", i);
    for(;(uint32_t)i<(uint32_t)addr+size4Aling;i+=4)
        p_Cli->f_Print("%08lx ", *((uint32_t*)(i)));
    for(;(uint32_t)i<(uint32_t)addr+numOfBytes;i++)
        p_Cli->f_Print("%02x", *((uint8_t*)(i)));
    if(notAlign)
        p_Cli->f_Print("\n");
}

static t_Error DispatchMemDisp(t_Handle h_Cli, int argc, char *argv[])
{
    t_Cli       *p_Cli = (t_Cli *)h_Cli;
    char        *token;
    uint32_t    addr;
    int         tokensCnt=1;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    if (argc<(tokensCnt+1))
        return ERROR_CODE(E_INVALID_VALUE);

    token = argv[tokensCnt++];
    sscanf(token,"%x",(unsigned int *)&addr);
    if (argc==2)
        DoMemDisp(p_Cli, (uint8_t *)addr,(int)0x100);
    else
    {
        uint32_t numOfBytes;
        token = argv[tokensCnt++];
        sscanf(token,"%x",(unsigned int *)&numOfBytes);
        DoMemDisp(p_Cli, (uint8_t *)addr,(int)numOfBytes);
    }

    return E_OK;
}

static t_Error DispatchMemMod(t_Handle h_Cli, int argc, char *argv[])
{
    t_Cli       *p_Cli = (t_Cli *)h_Cli;
    char        *token;
    int         i, tokensCnt=1;
    uint8_t     values[100];
    uint32_t    addr, numOfBytes, tmp;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    if (argc<(tokensCnt+2))
        return ERROR_CODE(E_INVALID_VALUE);

    token = argv[tokensCnt++];
    sscanf(token,"%x",(unsigned int *)&addr);
    token = argv[tokensCnt++];
    sscanf(token,"%x",(int*)&numOfBytes);
    if ((numOfBytes <= 0) || (numOfBytes > 100))
        return ERROR_CODE(E_INVALID_VALUE);
    for (i=0; i<numOfBytes; i++)
    {
        if (argc<(tokensCnt+numOfBytes-i))
            return ERROR_CODE(E_INVALID_VALUE);
        token = argv[tokensCnt++];
        sscanf(token,"%x",(unsigned int *)&tmp);
        values[i] = (uint8_t)tmp;
    }
    switch (numOfBytes)
    {
        case 2:
            *(uint16_t *)addr = ((uint16_t *)values)[0];
            break;
        case 4:
            *(uint32_t *)addr = ((uint32_t *)values)[0];
            break;
        case 8:
            *(uint64_t *)addr = ((uint64_t *)values)[0];
            break;
        default:
            memcpy((uint8_t *)addr, values, numOfBytes);
            break;
    }
    DoMemDisp(p_Cli, (uint8_t *)addr,(int)numOfBytes);

    return E_OK;
}

static t_Error DispatchTime(t_Handle h_Cli, int argc, char *argv[])
{
    t_Cli    *p_Cli = (t_Cli *)h_Cli;
    uint32_t abs_time = XX_CurrentTime()/1000;
    uint32_t hour = abs_time/3600;
    uint32_t min  = (abs_time-(hour*3600))/60;
    uint32_t sec  = abs_time-(hour*3600)-(min*60);

    UNUSED(argc);UNUSED(argv);

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    p_Cli->f_Print("Time: %d:%02d:%02d\r\n", hour,min,sec);

    return E_OK;
}

static t_Error DispatchHelp(t_Handle h_Cli, int argc, char *argv[])
{
    t_Cli       *p_Cli = (t_Cli *)h_Cli;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    UNUSED(argc);UNUSED(argv);

    Help(p_Cli);

    return E_OK;
}

static void GetHistory(t_Cli *p_Cli)
{
    t_CliHistoryLine *p_Line;

    if (p_Cli->line[p_Cli->currPos] == '[')
        return;
    if (p_Cli->line[p_Cli->currPos] == 'A')
    {
        if ((p_Cli->p_HistoryPos->p_Next != &p_Cli->history) &&
            (CLI_HISTORY_LINE_OBJ(p_Cli->p_HistoryPos->p_Next)->length != 0))
        {
            p_Cli->p_HistoryPos = p_Cli->p_HistoryPos->p_Next;
            p_Line = CLI_HISTORY_LINE_OBJ(p_Cli->p_HistoryPos);
            p_Cli->f_Print("\r%s> %s", p_Cli->prompt, p_Line->line);
            p_Cli->currPos = p_Line->length;
            memcpy(p_Cli->line, p_Line->line, (uint32_t)p_Cli->currPos);
        }
        p_Cli->historyReq = FALSE;
    }
    else if (p_Cli->line[p_Cli->currPos] == 'B')
    {
        if ((p_Cli->p_HistoryPos != &p_Cli->history) &&
            (CLI_HISTORY_LINE_OBJ(p_Cli->p_HistoryPos)->length != 0))
        {
            p_Line = CLI_HISTORY_LINE_OBJ(p_Cli->p_HistoryPos);
            p_Cli->f_Print("\r%s> %s", p_Cli->prompt, p_Line->line);
            p_Cli->currPos = p_Line->length;
            memcpy(p_Cli->line, p_Line->line, (uint32_t)p_Cli->currPos);
            p_Cli->p_HistoryPos = p_Cli->p_HistoryPos->p_Prev;
        }
        p_Cli->historyReq = FALSE;
    }
}

static t_Error GetLine(t_Cli *p_Cli, int *p_NumOfCharsRead)
{
    t_CliHistoryLine *p_Line;

    do
    {
        if ((p_Cli->line[p_Cli->currPos] = p_Cli->f_GetChar()) == '\0')
            return ERROR_CODE(E_EMPTY);

        if (p_Cli->line[p_Cli->currPos] == '\x1B')
        {
            p_Cli->historyReq = TRUE;
            continue;
        }

        if (p_Cli->historyReq)
        {
            GetHistory(p_Cli);
            continue;
        }

        if (p_Cli->line[p_Cli->currPos] == '\b')
        {
            if (p_Cli->currPos)
            {
                p_Cli->currPos--;
                p_Cli->f_Print("\b \b");
            }
            continue;
        }

        p_Cli->f_Print("%c", p_Cli->line[p_Cli->currPos]);
        if (p_Cli->line[p_Cli->currPos] != '\n')
        {
            if (p_Cli->line[p_Cli->currPos] == '\r')
            {
                p_Cli->f_Print("\n");
                break;
            }
            p_Cli->currPos++;
        }
        else
            break;
    }
    while (p_Cli->currPos < CLI_MAX_LINE_LENGTH);
    p_Cli->line[p_Cli->currPos] = '\0';

    *p_NumOfCharsRead = p_Cli->currPos;
    p_Cli->currPos = 0;
    p_Cli->historyReq = FALSE;
    p_Cli->p_HistoryPos = &p_Cli->history;

    /* Update the history lines */
    if (*p_NumOfCharsRead != 0)
    {
        p_Line = CLI_HISTORY_LINE_OBJ(p_Cli->history.p_Prev);
        LIST_DelAndInit(&p_Line->node);
        memcpy(p_Line->line, p_Cli->line, (uint32_t)(*p_NumOfCharsRead));
        p_Line->length = *p_NumOfCharsRead;
        LIST_Add(&p_Line->node, &p_Cli->history);
    }

    return E_OK;
}

static t_CliCommand * FindCommand(t_Cli *p_Cli, char *name)
{
    t_CliCommand    *p_Command;
    t_List          *p_Pos;

    LIST_FOR_EACH(p_Pos, &p_Cli->commands)
    {
        p_Command = CLI_COMMAND_OBJ(p_Pos);
        if (strcmp(p_Command->name, name) == 0)
            return p_Command;
    }

    return NULL;
}

static bool IsValidCmd(t_Cli *p_Cli, const char *cmd)
{
    if (strstr (cmd, "|"))
    {
        p_Cli->f_Print("character | (commands-pipeline) is not supported!\n");
        return FALSE;
    }
    if (strstr (cmd, ">") || strstr (cmd, "<"))
    {
        p_Cli->f_Print("characters < or > (redirections) are not supported!\n");
        return FALSE;
    }
    if (strstr (cmd, "\t"))
    {
        p_Cli->f_Print("character \\t (tab) is not supported! use space instead.\n");
        return FALSE;
    }
    return TRUE;
}

static void LoopTask(t_Handle h_Cli)
{
    t_Cli   *p_Cli = (t_Cli *)h_Cli;
    t_Error rc;
    int     numOfCharsRead;

    if ((rc = GetLine(p_Cli, &numOfCharsRead)) == E_OK)
    {
        char            *token;
        char            tmpLine[CLI_MAX_LINE_LENGTH+5];
        t_CliCommand    *p_Command;

        memcpy(tmpLine, p_Cli->line, CLI_MAX_LINE_LENGTH+5);

        if (IsValidCmd(p_Cli, tmpLine))
        {
            token = strtok(tmpLine, " ");

            if (token)
            {
                p_Command = FindCommand(p_Cli, token);
                if (!p_Command)
                {
                    Usage(p_Cli);
                }
                else
                {
                    char    *tokens[CLI_MAX_LINE_LENGTH];
                    int     tokensCnt = 0;

                    while ((tokensCnt<CLI_MAX_LINE_LENGTH) && token)
                    {
                        tokens[tokensCnt++] = token;
                        token = strtok(NULL, " ");
                    }
                    if (p_Command->f_DoCommand(p_Command->h_CommArg, tokensCnt, tokens) != E_OK)
                        p_Cli->f_Print("Illegal command usage!\n");
                }
            }
        }
        memset(p_Cli->line,0,(uint32_t)(CLI_MAX_LINE_LENGTH+5));
        p_Cli->f_Print("%s> ", p_Cli->prompt);
    }

    XX_ModTimer(p_Cli->h_Timer, CLI_TIMER_INTERVAL);
}

static void InitGeneralCommands(t_Cli *p_Cli)
{
    t_CliCommandParam commandParam;

    commandParam.name        = "md";
    commandParam.desc        = "Memory display n bytes (optional) from addr";
    commandParam.format      = "md <addr> [n]";
    commandParam.h_CommArg   = p_Cli;
    commandParam.f_DoCommand = DispatchMemDisp;
    CLI_RegisterCommand(p_Cli, &commandParam);

    commandParam.name        = "mm";
    commandParam.desc        = "Memory modified n bytes in addr with the values of arg1-argN";
    commandParam.format      = "mm <addr> <n> [arg1 argN]";
    commandParam.h_CommArg   = p_Cli;
    commandParam.f_DoCommand = DispatchMemMod;
    CLI_RegisterCommand(p_Cli, &commandParam);

    commandParam.name        = "time";
    commandParam.desc        = "Display current time";
    commandParam.format      = "time";
    commandParam.h_CommArg   = p_Cli;
    commandParam.f_DoCommand = DispatchTime;
    CLI_RegisterCommand(p_Cli, &commandParam);

    commandParam.name        = "?";
    commandParam.desc        = "Help";
    commandParam.format      = "?";
    commandParam.h_CommArg   = p_Cli;
    commandParam.f_DoCommand = DispatchHelp;
    CLI_RegisterCommand(p_Cli, &commandParam);
}

t_Handle CLI_ConfigAndInit(t_CliParam *p_CliParam)
{
    t_Cli   *p_Cli;
    int     i;

    p_Cli = XX_Malloc(sizeof(t_Cli));
    if (!p_Cli)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("CLI object"));
        return NULL;
    }
    memset(p_Cli, 0, sizeof(t_Cli));

    INIT_LIST(&p_Cli->commands);
    INIT_LIST(&p_Cli->history);
    if (p_CliParam->prompt != NULL)
        p_Cli->prompt = p_CliParam->prompt;
    else
        p_Cli->prompt = "NCSW";
    p_Cli->f_Print = p_CliParam->f_Print;
    p_Cli->f_GetChar = p_CliParam->f_GetChar;

    p_Cli->h_CommandsLock = XX_InitSpinlock();

    InitGeneralCommands(p_Cli);

    p_Cli->p_HistoryPos = &p_Cli->history;
    for (i=0; i<CLI_MAX_HISTORY; i++)
    {
        t_CliHistoryLine *p_Line = (t_CliHistoryLine *)XX_Malloc(sizeof(t_CliHistoryLine));
        if (!p_Line)
        {
            CLI_Free(p_Cli);
            REPORT_ERROR(MAJOR, E_NO_MEMORY, ("CLI history lines"));
            return NULL;
        }
        memset(p_Line, 0, sizeof(t_CliHistoryLine));
        INIT_LIST(&p_Line->node);
        LIST_AddToTail(&p_Line->node, &p_Cli->history);
    }

    return (t_Handle)p_Cli;
}

t_Error CLI_Free(t_Handle h_Cli)
{
    t_Cli               *p_Cli = (t_Cli *)h_Cli;
    t_CliCommand        *p_Command;
    t_CliHistoryLine    *p_Line;
    uint32_t            intFlags;

    if (!p_Cli)
        RETURN_ERROR(MINOR, E_INVALID_HANDLE, ("CLI object"));

    CLI_Stop(p_Cli);

    intFlags = XX_LockIntrSpinlock(p_Cli->h_CommandsLock);
    while (!LIST_IsEmpty(&p_Cli->commands))
    {
        p_Command = CLI_COMMAND_OBJ(p_Cli->commands.p_Next);
        LIST_DelAndInit(&p_Command->node);
        XX_Free(p_Command);
    }
    XX_UnlockIntrSpinlock(p_Cli->h_CommandsLock, intFlags);

    while (!LIST_IsEmpty(&p_Cli->history))
    {
        p_Line = CLI_HISTORY_LINE_OBJ(p_Cli->history.p_Next);
        LIST_DelAndInit(&p_Line->node);
        XX_Free(p_Line);
    }

    if (p_Cli->h_CommandsLock)
        XX_FreeSpinlock(p_Cli->h_CommandsLock);

    XX_Free(p_Cli);

    return E_OK;
}

t_Error CLI_Run(t_Handle h_Cli)
{
    t_Cli   *p_Cli = (t_Cli *)h_Cli;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    p_Cli->h_Timer = XX_CreateTimer();
    if (!p_Cli->h_Timer)
    {
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("CLI timer"));
    }

    XX_StartTimer(p_Cli->h_Timer, CLI_TIMER_INTERVAL, FALSE, (void (*)(t_Handle))LoopTask, p_Cli);

    PrintFSLLogo(p_Cli);

    Usage(p_Cli);

    p_Cli->f_Print("%s> ", p_Cli->prompt);

    return E_OK;
}

t_Error CLI_Stop(t_Handle h_Cli)
{
    t_Cli   *p_Cli = (t_Cli *)h_Cli;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    if (p_Cli->h_Timer)
    {
        XX_StopTimer(p_Cli->h_Timer);
        XX_FreeTimer(p_Cli->h_Timer);
        p_Cli->h_Timer = NULL;
    }

    return E_OK;
}

void CLI_Print(t_Handle h_Cli, char *format, ...)
{
    t_Cli   *p_Cli = (t_Cli *)h_Cli;
    char    tmpBuf[CLI_PRINT_BUF_SIZE];
    va_list args;

    SANITY_CHECK_RETURN(p_Cli, E_INVALID_HANDLE);

    va_start(args, format);
    vsprintf(tmpBuf, format, args);
    va_end(args);

    p_Cli->f_Print(tmpBuf);
}

char CLI_GetChar(t_Handle h_Cli)
{
    t_Cli   *p_Cli = (t_Cli *)h_Cli;

    SANITY_CHECK_RETURN_VALUE(p_Cli, E_INVALID_HANDLE, 0);

    return p_Cli->f_GetChar();
}

t_Error CLI_RegisterCommand(t_Handle h_Cli, t_CliCommandParam *p_CliCommandParam)
{
    t_Cli           *p_Cli = (t_Cli *)h_Cli;
    t_CliCommand    *p_Command;
    uint32_t        intFlags;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    p_Command = XX_Malloc(sizeof(t_CliCommand));
    if (!p_Command)
        RETURN_ERROR(MINOR, E_NO_MEMORY, ("Command object"));
    memset(p_Command, 0, sizeof(t_CliCommand));

    INIT_LIST(&p_Command->node);
    p_Command->name         = p_CliCommandParam->name;
    p_Command->desc         = p_CliCommandParam->desc;
    p_Command->format       = p_CliCommandParam->format;
    p_Command->h_CommArg    = p_CliCommandParam->h_CommArg;
    p_Command->f_DoCommand  = p_CliCommandParam->f_DoCommand;

    intFlags = XX_LockIntrSpinlock(p_Cli->h_CommandsLock);
    LIST_AddToTail(&p_Command->node, &p_Cli->commands);
    XX_UnlockIntrSpinlock(p_Cli->h_CommandsLock, intFlags);

    return E_OK;
}

t_Error CLI_UnregisterCommand(t_Handle h_Cli, char *name)
{
    t_Cli           *p_Cli = (t_Cli *)h_Cli;
    t_CliCommand    *p_Command;
    uint32_t        intFlags;

    SANITY_CHECK_RETURN_ERROR(p_Cli, E_INVALID_HANDLE);

    p_Command = FindCommand(p_Cli, name);

    if (p_Command)
    {
        intFlags = XX_LockIntrSpinlock(p_Cli->h_CommandsLock);
        LIST_Del(&(p_Command->node));
        XX_UnlockIntrSpinlock(p_Cli->h_CommandsLock, intFlags);

        XX_Free(p_Command);
    }

    return E_OK;
}

