/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          buf.c

 @Description   Implementation of buffer object.

*//***************************************************************************/
#include "std_ext.h"
#include "error_ext.h"
#include "memcpy_ext.h"
#include "mem_ext.h"
#include "xx_ext.h"
#include "buf_ext.h"


#define __ERR_MODULE__  MODULE_UNKNOWN


/* Global buffer objects pool handle */
t_Handle h_GlobalBufPool = NULL;


/*****************************************************************************/
t_Error B_InitPool(uint32_t numOfBuffers)
{
    t_Error errCode;

    if (h_GlobalBufPool)
    {
        RETURN_ERROR(MAJOR, E_ALREADY_EXISTS, ("Buffer objects pool"));
    }

    if (numOfBuffers > 0)
    {
        errCode = MEM_Init("BUF", &h_GlobalBufPool,
                           numOfBuffers, B_Sizeof(), 0, 0, 0);
        if (errCode != E_OK)
        {
            RETURN_ERROR(MAJOR, errCode, NO_MSG);
        }
    }

    return E_OK;
}


/*****************************************************************************/
void B_FreePool(void)
{
    if (h_GlobalBufPool)
    {
        MEM_Free(h_GlobalBufPool);
        h_GlobalBufPool = NULL;
    }
}


/*****************************************************************************/
t_Handle B_New(t_BufMemAccess    *p_MemAccess,
               uint16_t          tailLimit,
               uint16_t          headLimit,
               uint8_t           *p_Data,
               uint16_t          length)
{
    t_Buffer *p_Buf;

    ASSERT_COND(p_MemAccess);
    ASSERT_COND(tailLimit);
    /* Total buffer room (headLimit + tailLimit) must be less than 64KB */
    ASSERT_COND(headLimit < (0xFFFF - tailLimit));
    /* length must not exceed tailLimit */
    ASSERT_COND(length <= tailLimit);

    /* allocate buffer structure */
    p_Buf = (t_Buffer *)MEM_Get(h_GlobalBufPool);

    /* perform initializations */
    if (p_Buf)
    {
        if (!p_Data)
        {
            ASSERT_COND(p_MemAccess->f_GetMem);
            p_Buf->p_Data = p_MemAccess->f_GetMem(p_MemAccess->h_MemMng);
            if (!p_Buf->p_Data)
            {
                MEM_Put(h_GlobalBufPool, p_Buf);
                return NULL;
            }
        }
        else
        {
            p_Buf->p_Data = p_Data;
        }

        p_Buf->p_MemAccess = p_MemAccess;
        p_Buf->tailLimit = tailLimit;
        p_Buf->headLimit = headLimit;
        p_Buf->owners = 1;
        p_Buf->first = 0;
        p_Buf->last = length;
        p_Buf->p_Next = NULL;
        p_Buf->p_Prev = NULL;
    }

    return (t_Handle)p_Buf;
}


/*****************************************************************************/
t_Handle B_Clone(t_Handle h_Buffer)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    t_Buffer    *p_NewBuf;
    uint8_t     *p_Data = NULL;

    ASSERT_COND(h_Buffer);

    p_NewBuf = (t_Buffer *)MEM_Get(h_GlobalBufPool);

    if (p_NewBuf)
    {
        if (p_Buf->p_Data)
        {
            ASSERT_COND(p_Buf->p_MemAccess->f_GetMem);

            p_Data = p_Buf->p_MemAccess->f_GetMem(p_Buf->p_MemAccess->h_MemMng);
            if (!p_Data)
            {
                MEM_Put(h_GlobalBufPool, p_NewBuf);
                return NULL;
            }

            /* copy data from original buffer to new one */
            MemCpy32(p_Data + p_Buf->first,
                     p_Buf->p_Data + p_Buf->first,
                     (uint16_t)((int32_t)p_Buf->last - p_Buf->first));
        }

        /* identical fields */
        p_NewBuf->p_MemAccess = p_Buf->p_MemAccess;
        p_NewBuf->tailLimit = p_Buf->tailLimit;
        p_NewBuf->headLimit = p_Buf->headLimit;
        p_NewBuf->first = p_Buf->first;
        p_NewBuf->last = p_Buf->last;

        /* new settings */
        p_NewBuf->p_Next = NULL;
        p_NewBuf->p_Prev = NULL;
        p_NewBuf->p_Data = p_Data;
        p_NewBuf->owners = 1;
    }

    return p_NewBuf;
}


/*****************************************************************************/
uint8_t * B_Delete(t_Handle h_Buffer, bool releaseData)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    uint16_t    owners;
    uint8_t     *p_DataBlock;
    uint32_t    intFlags;

    ASSERT_COND(h_Buffer);

    p_DataBlock = p_Buf->p_Data;

    /* decrement owners count */
    intFlags = XX_DisableAllIntr();
    ASSERT_COND(p_Buf->owners);
    owners = --(p_Buf->owners);
    XX_RestoreAllIntr(intFlags);

    /* if the buffer is being used by someone else, leave it */
    if (owners == 0)
    {
        if (releaseData && p_Buf->p_MemAccess->f_PutMem)
        {
            p_Buf->p_MemAccess->f_PutMem(p_Buf->p_MemAccess->h_MemMng, p_Buf->p_Data);
            p_DataBlock = p_Buf->p_Data = NULL;
        }

        MEM_Put(h_GlobalBufPool, h_Buffer);
    }

    return p_DataBlock;
}


/*****************************************************************************/
void B_ReleaseData(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);
    ASSERT_COND(p_Buf->p_MemAccess->f_PutMem);

    p_Buf->p_MemAccess->f_PutMem(p_Buf->p_MemAccess->h_MemMng, p_Buf->p_Data);
    p_Buf->p_Data = NULL;
    p_Buf->first = 0;
    p_Buf->last = 0;
}


/*****************************************************************************/
uint8_t * B_SetDataBlock(t_Handle h_Buffer, uint8_t *p_DataBlock, uint16_t length)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    uint8_t     *p_OldDataBlock;

    ASSERT_COND(h_Buffer);
    /* length must not exceed tailLimit */
    ASSERT_COND(length <= p_Buf->tailLimit);
    /* length must be zero if p_DataBlock is NULL */
    ASSERT_COND((p_DataBlock || (length == 0)));

    p_OldDataBlock = p_Buf->p_Data;
    p_Buf->p_Data = p_DataBlock;
    p_Buf->first = 0;
    p_Buf->last = length;

    return p_OldDataBlock;
}


/*****************************************************************************/
void B_ReserveHeadRoom(t_Handle h_Buffer, uint16_t headRoom)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);
    /* headRoom cannot be reserved when data length is non-zero */
    ASSERT_COND(p_Buf->first == p_Buf->last);
    /* Head offset cannot exceed tailLimit */
    ASSERT_COND((int32_t)headRoom <= ((int32_t)p_Buf->tailLimit - p_Buf->first));

    p_Buf->first += headRoom;
    p_Buf->last += headRoom;
}


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))

/*****************************************************************************/
void B_Link(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    p_Buf->owners++;
}


/*****************************************************************************/
void B_SetLength(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    /* length must not exceed tailLimit */
    ASSERT_COND(length <= (uint16_t)((int32_t)p_Buf->tailLimit - p_Buf->first));

    p_Buf->last = (uint16_t)(p_Buf->first + length);
}


/*****************************************************************************/
uint16_t B_GetLength(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return (uint16_t)((int32_t)p_Buf->last - p_Buf->first);
}


/*****************************************************************************/
uint8_t * B_GetData(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return (uint8_t *)(p_Buf->p_Data + p_Buf->first);
}


/*****************************************************************************/
uint8_t * B_GetDataBlock(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return p_Buf->p_Data;
}


/*****************************************************************************/
void B_PullHead(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    /* Head offset cannot exceed data length */
    ASSERT_COND((int32_t)length <= ((int32_t)p_Buf->last - p_Buf->first));

    p_Buf->first += length;
}


/*****************************************************************************/
void B_PushHead(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    /* Head offset cannot exceed headLimit */
    ASSERT_COND((int32_t)length <= (p_Buf->first + (int32_t)p_Buf->headLimit));

    p_Buf->first -= length;
}


/*****************************************************************************/
void B_PullTail(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    /* Tail offset cannot exceed data length */
    ASSERT_COND((int32_t)length <= ((int32_t)p_Buf->last - p_Buf->first));

    p_Buf->last -= length;
}


/*****************************************************************************/
void B_PushTail(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);
    /* Tail offset cannot exceed tailLimit */
    ASSERT_COND(length <= (p_Buf->tailLimit - p_Buf->last));

    p_Buf->last += length;
}


/*****************************************************************************/
void B_ResetData(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    p_Buf->first = p_Buf->last = 0;
}


/*****************************************************************************/
t_Handle B_GetNext(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return (t_Handle)p_Buf->p_Next;
}


/*****************************************************************************/
t_Handle B_GetPrev(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return (t_Handle)p_Buf->p_Prev;
}


/*****************************************************************************/
void * B_GetUserPriv(t_Handle h_Buffer)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    return p_Buf->p_UserPriv;
}


/*****************************************************************************/
void B_SetNext(t_Handle h_Buffer, t_Handle nextBuf)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    p_Buf->p_Next = (t_Buffer *)nextBuf;
}


/*****************************************************************************/
void B_SetPrev(t_Handle h_Buffer, t_Handle prevBuf)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    p_Buf->p_Prev = (t_Buffer *)prevBuf;
}


/*****************************************************************************/
void B_SetUserPriv(t_Handle h_Buffer, void* p_UserPriv)
{
    t_Buffer *p_Buf = (t_Buffer *)h_Buffer;

    ASSERT_COND(h_Buffer);

    p_Buf->p_UserPriv = p_UserPriv;
}

#endif /* DEBUG_ERRORS */


/*****************************************************************************/
uint16_t B_AddToHead(t_Handle h_Buffer, uint8_t *p_Data, uint16_t length)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    uint16_t    count;
    uint16_t    headRoom;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);

    /* see how much will fit in */
    headRoom = (uint16_t)(p_Buf->first + (int32_t)p_Buf->headLimit);
    count = (uint16_t)MIN(headRoom, length);

    /* copy data to buffer */
    p_Buf->first -= count;
    MemCpy32((p_Buf->p_Data + p_Buf->first), (p_Data + length - count), count);

    return count;
}


/*****************************************************************************/
uint16_t B_AddToTail(t_Handle h_Buffer, uint8_t *p_Data, uint16_t length)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    uint16_t    count;
    uint16_t    tailRoom;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);

    /* see how much will fit in */
    tailRoom = (uint16_t)(p_Buf->tailLimit - p_Buf->last);
    count = (uint16_t)MIN(tailRoom, length);

    /* copy data to buffer */
    MemCpy32((p_Buf->p_Data + p_Buf->last), p_Data, (uint16_t)count);
    p_Buf->last += count;

    return count;
}


/*****************************************************************************/
uint16_t B_CutFromHead(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    uint16_t    count;
    uint16_t    dataLength;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);

    dataLength = B_GetLength(h_Buffer);

    /* see how much can be cut */
    count = (uint16_t)MIN(dataLength, length);

    p_Buf->first += count;
    return count;
}


/*****************************************************************************/
uint16_t B_CutFromTail(t_Handle h_Buffer, uint16_t length)
{
    t_Buffer   *p_Buf = (t_Buffer *)h_Buffer;
    uint16_t    count;

    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_Buf->p_Data);

    /* see how much can be cut */
    count = (uint16_t)MIN(p_Buf->last, length);

    p_Buf->last -= count;
    return count;
}


/*****************************************************************************/
void B_Print(t_Handle h_Buffer)
{
    t_Buffer    *p_Buf = (t_Buffer *)h_Buffer;
    int32_t     i;
    static int  loc = 1;

    ASSERT_COND(h_Buffer);

    XX_Print("length of buffer = %d\n", (p_Buf->last - p_Buf->first));

    for (i = p_Buf->first; i < p_Buf->last; i++)
    {
        if (!(loc++ % 24))
            XX_Print("\n");

        XX_Print("%x ", *(uint8_t *)(p_Buf->p_Data + i));

        if (i == (int32_t)(p_Buf->last - 1))
            XX_Print("end of buffer (%d)\n", (p_Buf->last - p_Buf->first));
    }
}
