/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/*

 @File          mb_frame.c

 @Description   Implementation of multi-buffer frame object.

*//***************************************************************************/

#include "error_ext.h"
#include "std_ext.h"
#include "mem_ext.h"
#include "memcpy_ext.h"
#include "buf_ext.h"
#include "mb_frame_ext.h"
#include "xx_ext.h"



#define __ERR_MODULE__  MODULE_UNKNOWN


/* Global multi-buffer frames pool handle */
t_Handle h_GlobalMbfPool = NULL;


/*****************************************************************************/
t_Error F_InitPool(uint32_t numOfFrames)
{
    t_Error errCode;

    if (h_GlobalMbfPool)
    {
        RETURN_ERROR(MAJOR, E_ALREADY_EXISTS, ("Multi-buffer frames pool"));
    }

    if (numOfFrames > 0)
    {
        errCode = MEM_Init("MBF", &h_GlobalMbfPool,
                           numOfFrames, F_Sizeof(), 0, 0, 0);
        if (errCode != E_OK)
        {
            RETURN_ERROR(MAJOR, errCode, NO_MSG);
        }
    }

    return E_OK;
}


/*****************************************************************************/
void F_FreePool(void)
{
    if (h_GlobalMbfPool)
    {
        MEM_Free(h_GlobalMbfPool);
        h_GlobalMbfPool = NULL;
    }
}


/*****************************************************************************/
t_Handle F_New(t_Handle h_Buffer)
{
    t_Frame *p_Frame = (t_Frame *)MEM_Get(h_GlobalMbfPool);

    if (p_Frame)
    {
        p_Frame->firstBuf = h_Buffer;
        p_Frame->lastBuf = h_Buffer;

        if (h_Buffer)
        {
            B_SetPrev(h_Buffer, NULL);
            B_SetNext(h_Buffer, NULL);
            p_Frame->length = (uint16_t)B_GetLength(h_Buffer);
        }
        else
        {
            p_Frame->length = 0;
        }

        p_Frame->p_Next = NULL;
        p_Frame->owners = 1;
        p_Frame->info1 = NULL;
        p_Frame->info2 = NULL;
        p_Frame->info3 = 0;
        p_Frame->info4 = 0;
        p_Frame->info5 = 0;
        p_Frame->status = 0;
    }

    return (t_Handle)p_Frame;
}


/*****************************************************************************/
void F_Delete(t_Handle h_Frame, bool releaseData)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer, BufNext;
    uint16_t    owners;
    uint32_t    intFlags;

    ASSERT_COND(h_Frame);

    /* decrement owners count */
    intFlags = XX_DisableAllIntr();
    ASSERT_COND(p_Frame->owners);
    owners = --(p_Frame->owners);
    XX_RestoreAllIntr(intFlags);

    if (owners == 0)
    {
        /* for each buffer in frame ... */
        h_Buffer = p_Frame->firstBuf;

        while (h_Buffer)
        {
            /* save next pointer */
            BufNext = B_GetNext(h_Buffer);

            /* delete buffer structure + data block */
            B_Delete(h_Buffer, releaseData);

            /* get next buffer (NULL if there's no more) */
            h_Buffer = BufNext;
        }

        /* finally, release the frame structure itself */
        MEM_Put(h_GlobalMbfPool, h_Frame);
    }
}


/*****************************************************************************/
t_Handle F_GetBuffer(t_Handle h_Frame, uint16_t bufIndex, bool *p_IsLast)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    index = 0;

    ASSERT_COND(h_Frame);
    ASSERT_COND(p_IsLast);

    h_Buffer = p_Frame->firstBuf;

    while (h_Buffer && (index++ < bufIndex))
    {
        h_Buffer = B_GetNext(h_Buffer);
    }

    if (h_Buffer)
    {
        *p_IsLast = (bool)((h_Buffer == p_Frame->lastBuf) ? TRUE : FALSE);
    }

    return h_Buffer;
}


/*****************************************************************************/
t_Handle F_GetNextBuffer(t_Handle h_Frame, t_Handle h_Buffer, bool *p_IsLast)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);
    ASSERT_COND(h_Buffer);
    ASSERT_COND(p_IsLast);

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
    {
        /* Verify that the given buffer belongs to the frame */
        t_Handle tempBuf = p_Frame->firstBuf;

        while (tempBuf && (tempBuf != h_Buffer))
        {
            tempBuf = B_GetNext(tempBuf);
        }

        /* Buffer does not belong to given frame */
        ASSERT_COND(tempBuf);
    }
#endif

    h_Buffer = B_GetNext(h_Buffer);

    if (h_Buffer)
    {
        *p_IsLast = (bool)((h_Buffer == p_Frame->lastBuf) ? TRUE : FALSE);
    }

    return h_Buffer;
}


/*****************************************************************************/
void F_AppendBuffer(t_Handle h_Frame, t_Handle h_Buffer)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);
    ASSERT_COND(h_Buffer);

    if (p_Frame->lastBuf)
    {
        B_SetNext(p_Frame->lastBuf, h_Buffer);
    }
    else
    {
        p_Frame->firstBuf = h_Buffer;
    }

    B_SetNext(h_Buffer, NULL);
    B_SetPrev(h_Buffer, p_Frame->lastBuf); /* NULL if no buffers in frame */
    p_Frame->lastBuf = h_Buffer;
    p_Frame->length += B_GetLength(h_Buffer);
}


/*****************************************************************************/
void F_PrependBuffer(t_Handle h_Frame, t_Handle h_Buffer)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);
    ASSERT_COND(h_Buffer);

    if (p_Frame->firstBuf)
    {
        B_SetPrev(p_Frame->firstBuf, h_Buffer);
    }
    else
    {
        p_Frame->lastBuf = h_Buffer;
    }

    B_SetNext(h_Buffer, p_Frame->firstBuf); /* NULL if no buffers in frame */
    B_SetPrev(h_Buffer, NULL);
    p_Frame->firstBuf = h_Buffer;
    p_Frame->length += B_GetLength(h_Buffer);
}


/*****************************************************************************/
uint16_t F_InsertBuffer(t_Handle h_Frame, t_Handle h_Buffer, uint16_t bufIndex)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    nextBuf, prevBuf;
    uint16_t    index = 0;

    ASSERT_COND(h_Frame);
    ASSERT_COND(h_Buffer);

    nextBuf = p_Frame->firstBuf;

    while (nextBuf && (index++ < bufIndex))
    {
        nextBuf = B_GetNext(nextBuf);
    }

    if (index == bufIndex)
    {
        if (nextBuf)
        {
            prevBuf = B_GetPrev(nextBuf);
        }
        else
        {
            /* Insert the buffer as the last buffer */
            prevBuf = p_Frame->lastBuf;
            p_Frame->lastBuf = h_Buffer;
        }

        if (prevBuf)
        {
            B_SetNext(prevBuf, h_Buffer);
        }
        else
        {
            /* New buffer is first in frame */
            p_Frame->firstBuf = h_Buffer;
        }

        B_SetNext(h_Buffer, nextBuf);
        B_SetPrev(h_Buffer, prevBuf);

        p_Frame->length += B_GetLength(h_Buffer);
    }

    /* return insertion index, or highest valid index for insertion */
    return index;
}


/*****************************************************************************/
t_Handle F_RemoveFirstBuffer(t_Handle h_Frame)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    firstBuf, nextBuf;

    ASSERT_COND(h_Frame);

    firstBuf = p_Frame->firstBuf;

    if (firstBuf)
    {
        nextBuf = B_GetNext(firstBuf);
        if (nextBuf)
        {
            B_SetPrev(nextBuf, NULL);
        }
        else
        {
            /* Frame is now empty */
            p_Frame->lastBuf = NULL;
        }

        p_Frame->firstBuf = nextBuf; /* Can be NULL */
        B_SetNext(firstBuf, NULL);

        p_Frame->length -= B_GetLength(firstBuf);
    }

    return firstBuf;
}


/*****************************************************************************/
t_Handle F_RemoveLastBuffer(t_Handle h_Frame)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    lastBuf, prevBuf;

    ASSERT_COND(h_Frame);

    lastBuf = p_Frame->lastBuf;

    if (lastBuf)
    {
        prevBuf = B_GetPrev(lastBuf);
        if (prevBuf)
        {
            B_SetNext(prevBuf, NULL);
        }
        else
        {
            /* Frame is now empty */
            p_Frame->firstBuf = NULL;
        }

        p_Frame->lastBuf = prevBuf; /* Can be NULL */
        B_SetPrev(lastBuf, NULL);

        p_Frame->length -= B_GetLength(lastBuf);
    }

    return lastBuf;
}


/*****************************************************************************/
t_Handle F_RemoveBuffer(t_Handle h_Frame, uint16_t bufIndex)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer, nextBuf, prevBuf;
    uint16_t    index = 0;

    ASSERT_COND(h_Frame);

    h_Buffer = p_Frame->firstBuf;

    while (h_Buffer && (index++ < bufIndex))
    {
        h_Buffer = B_GetNext(h_Buffer);
    }

    if (h_Buffer)
    {
        /* Both are NULL if the frame contains a single buffer */
        prevBuf = B_GetPrev(h_Buffer);
        nextBuf = B_GetNext(h_Buffer);

        if (prevBuf)
        {
            B_SetNext(prevBuf, nextBuf);
        }
        else
        {
            p_Frame->firstBuf = nextBuf;
        }

        if (nextBuf)
        {
            B_SetPrev(nextBuf, prevBuf);
        }
        else
        {
            p_Frame->lastBuf = prevBuf;
        }

        p_Frame->length -= B_GetLength(h_Buffer);
    }

    return h_Buffer;
}


/*****************************************************************************/
uint16_t F_AppendData(t_Handle h_Frame, uint8_t *p_Data, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    uint16_t    count = 0, increment;
    t_Handle    h_LastBuf, h_NewBuffer;

    ASSERT_COND(h_Frame);

    h_LastBuf = p_Frame->lastBuf;

    if (h_LastBuf)
    {
        /* Add data to last buffer's tail */
        increment = count = B_AddToTail(h_LastBuf, p_Data, length);

        while (count < length)
        {
            /* Create a new buffer with similar attributes to the last buffer */
            h_NewBuffer = B_New(B_GetMemAccessPtr(h_LastBuf),
                                B_GetTailLimit(h_LastBuf),
                                0,
                                NULL,
                                0);
            if (!h_NewBuffer)
            {
                break;
            }

            /* Copy as much data as possible into the new buffer */
            p_Data += increment;
            increment = B_AddToTail(h_NewBuffer, p_Data, (uint16_t)(length - count));
            count += increment;

            /* Append the new buffer to the frame */
            B_SetNext(p_Frame->lastBuf, h_NewBuffer);
            B_SetPrev(h_NewBuffer, p_Frame->lastBuf);
            p_Frame->lastBuf = h_NewBuffer;
        }

        /* Update length of frame */
        p_Frame->length += count;
    }

    return count;
}


/*****************************************************************************/
uint16_t F_PrependData(t_Handle h_Frame, uint8_t *p_Data, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    uint16_t    count = 0;

    ASSERT_COND(h_Frame);

    if (p_Frame->firstBuf)
    {
        /* Add data to first buffer's head */
        count = B_AddToHead(p_Frame->firstBuf, p_Data, length);

        /* Update length of frame */
        p_Frame->length += count;
    }

    return count;
}


/*****************************************************************************/
uint16_t F_CutFromHead(t_Handle h_Frame, uint16_t length, t_Handle *removedBuffer)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    count = 0;

    ASSERT_COND(h_Frame);

    h_Buffer = p_Frame->firstBuf;

    if (removedBuffer)
    {
        uint16_t bufLength;

        *removedBuffer = NULL;

        while (h_Buffer && length)
        {
            bufLength = B_GetLength(h_Buffer);

            /* Check if the whole buffer should be removed from the frame */
            if (bufLength <= length)
            {
                count += bufLength;
                length -= bufLength;

                B_ResetData(h_Buffer);
                *removedBuffer = h_Buffer;
                h_Buffer = B_GetNext(h_Buffer);
            }
            else
            {
                /* Cut the required number of bytes from this buffer */
                count += B_CutFromHead(h_Buffer, length);
                break;
            }
        }

        if (*removedBuffer)
        {
            B_SetNext(*removedBuffer, NULL);

            /* Update the frame's last buffer (can be NULL) */
            p_Frame->firstBuf = h_Buffer;

            if (h_Buffer)
            {
                B_SetPrev(h_Buffer, NULL);
            }
            else
            {
                /* Frame is now empty */
                p_Frame->lastBuf = NULL;
            }
        }
    }
    else
    {
        /* Do not remove buffers - cut as much as possible from first buffer */
        count = B_CutFromHead(h_Buffer, length);
    }

    p_Frame->length -= count;

    return count;
}


/*****************************************************************************/
uint16_t F_CutFromTail(t_Handle h_Frame, uint16_t length, t_Handle *removedBuffer)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    count = 0;

    ASSERT_COND(h_Frame);

    h_Buffer = p_Frame->lastBuf;

    if (removedBuffer)
    {
        uint16_t bufLength;

        *removedBuffer = NULL;

        while (h_Buffer && length)
        {
            bufLength = B_GetLength(h_Buffer);

            /* Check if the whole buffer should be removed from the frame */
            if (bufLength <= length)
            {
                count += bufLength;
                length -= bufLength;

                B_ResetData(h_Buffer);
                *removedBuffer = h_Buffer;
                h_Buffer = B_GetPrev(h_Buffer);
            }
            else
            {
                /* Cut the required number of bytes from this buffer */
                count += B_CutFromTail(h_Buffer, length);
                break;
            }
        }

        if (*removedBuffer)
        {
            B_SetPrev(*removedBuffer, NULL);

            /* Update the frame's last buffer (can be NULL) */
            p_Frame->lastBuf = h_Buffer;

            if (h_Buffer)
            {
                B_SetNext(h_Buffer, NULL);
            }
            else
            {
                /* Frame is now empty */
                p_Frame->firstBuf = NULL;
            }
        }
    }
    else
    {
        /* Do not remove buffers - cut as much as possible from last buffer */
        count = B_CutFromTail(h_Buffer, length);
    }

    p_Frame->length -= count;

    return count;
}


/*****************************************************************************/
void F_PullHead(t_Handle h_Frame, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->firstBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
        B_PullHead(h_Buffer, length);

    p_Frame->length -= length;
}


/*****************************************************************************/
void F_PushHead(t_Handle h_Frame, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->firstBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
        B_PushHead(h_Buffer, length);

    p_Frame->length += length;
}


/*****************************************************************************/
void F_PullTail(t_Handle h_Frame, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->lastBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
        B_PullTail(h_Buffer, length);

    p_Frame->length -= length;
}


/*****************************************************************************/
void F_PushTail(t_Handle h_Frame, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->lastBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
        B_PushTail(h_Buffer, length);

    p_Frame->length += length;
}


/*****************************************************************************/
uint16_t F_Read(t_Handle h_Frame, uint8_t *p_Dest, uint16_t offset, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    count = 0;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->firstBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
    {
        /* special case: data is all in first buffer */
        if ((offset + length) <= B_GetLength(h_Buffer))
        {
            MemCpy32(p_Dest, (uint8_t *)(B_GetData(h_Buffer) + offset), length);
            return length;
        }
        else
        {
            uint32_t bufLength;

            /* first check that the requested offset is valid for this frame */
            if (offset >= p_Frame->length)
                return 0;

            bufLength = B_GetLength(h_Buffer);

            while (offset > bufLength)
            {
                offset -= bufLength;
                h_Buffer = B_GetNext(h_Buffer);
                if (!h_Buffer)
                   return 0;
                bufLength = (uint16_t)B_GetLength(h_Buffer);
            }

            bufLength -= offset;

            while (count < length)
            {
                uint32_t i = (uint32_t)(length - count);

                if (i > bufLength)
                    i = bufLength;

                MemCpy32(p_Dest, (uint8_t *)(B_GetData(h_Buffer) + offset), i);
                count += i;     /* update byte count */

                /* see if we need to go to the next buffer for more data */
                if (count < length)
                {
                    offset = 0;
                    p_Dest += i;  /* advance data pointer */

                    h_Buffer = B_GetNext(h_Buffer);
                    if (!h_Buffer)
                        break;

                    /* prepare for next time round */
                    bufLength = B_GetLength(h_Buffer);
                }
            }
        }
    }

    /* return the number of bytes read */
    return count;
}


/*****************************************************************************/
uint16_t F_ReadByte(t_Handle h_Frame, uint8_t *p_Dest, uint16_t offset)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint32_t    length;

    ASSERT_COND(h_Frame);

    /* first check that the requested offset is valid for this frame */
    if (offset >= p_Frame->length)
        return 0;

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->firstBuf;

    while ((offset > 0) && h_Buffer)
    {
        length = B_GetLength(h_Buffer);
        if (offset < length)
            break;
        offset -= length;
        h_Buffer = B_GetNext(h_Buffer);
    }

    if (!h_Buffer)
    {
        return 0;
    }

    /* read the required data byte */
    *p_Dest = *(uint8_t *)(B_GetData(h_Buffer) + offset);
    return 1;
}


/*****************************************************************************/
uint16_t F_Replace(t_Handle h_Frame, uint8_t *p_Source, uint16_t offset, uint16_t length)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    count = 0;

    ASSERT_COND(h_Frame);

    /* now find buffer containing the start of the block at the required offset */
    h_Buffer = p_Frame->firstBuf;

    /* if no buffers attached, return */
    if (h_Buffer)
    {
        /* special case: data is all in first buffer */
        if ((offset + length) <= B_GetLength(h_Buffer))
        {
            MemCpy32((uint8_t *)(B_GetData(h_Buffer)+ offset), p_Source, length);
            return length;
        }
        else
        {
            uint32_t bufLength;

            /* first check that the requested offset is valid for this frame */
            if (offset >= p_Frame->length)
                return 0;

            bufLength = B_GetLength(h_Buffer);

            while (offset > bufLength)
            {
                offset -= bufLength;
                h_Buffer = B_GetNext(h_Buffer);
                if (!h_Buffer)
                    return 0;
                bufLength = B_GetLength(h_Buffer);
            }

            bufLength -= offset;

            while (count < length)
            {
                uint32_t i = (uint32_t)(length - count);

                if (i > bufLength)
                    i = bufLength;

                MemCpy32((uint8_t *)(B_GetData(h_Buffer) + offset), p_Source, i);
                count += i;     /* update byte count */

                /* see if we need to go to the next buffer for more data */
                if (count < length)
                {
                   offset = 0;
                   p_Source += i;   /* advance data pointer */

                   h_Buffer = B_GetNext(h_Buffer);
                   if (!h_Buffer)
                      break;

                   /* prepare for next time round */
                   bufLength = B_GetLength(h_Buffer);
                }
            }
        }
    }

    /* return the number of bytes replaced */
    return count;
}


/*****************************************************************************/
t_Handle F_Copy(t_Handle        h_Frame,
                t_BufMemAccess  *p_MemAccess,
                uint16_t        bufferTailLimit,
                uint16_t        bufferHeadLimit)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Frame     *p_NewFrame;
    t_Handle    h_NewBuffer;

    ASSERT_COND(h_Frame);
    ASSERT_COND(p_MemAccess);

    h_NewBuffer = B_New(p_MemAccess, bufferTailLimit, bufferHeadLimit, NULL, 0);
    if (!h_NewBuffer)
    {
        return NULL;
    }

    p_NewFrame = (t_Frame *)F_New(h_NewBuffer);

    if (p_NewFrame)
    {
        uint16_t    count, length;
        t_Handle    h_Buffer = p_Frame->firstBuf;

        p_NewFrame->info1   = p_Frame->info1;
        p_NewFrame->info2   = p_Frame->info2;
        p_NewFrame->info3   = p_Frame->info3;
        p_NewFrame->info4   = p_Frame->info4;
        p_NewFrame->info5   = p_Frame->info5;
        p_NewFrame->status  = p_Frame->status;

        /* For each buffer in the frame ... */
        while (h_Buffer)
        {
            length = B_GetLength(h_Buffer);

            /* Copy buffer to the new frame */
            count = F_AppendData(p_NewFrame, B_GetData(h_Buffer), length);
            if (count != length)
            {
                F_Delete(p_NewFrame, TRUE);
                return NULL;
            }
            h_Buffer = B_GetNext(h_Buffer);
        }
    }
    else
    {
        B_Delete(h_NewBuffer, TRUE);
    }

    return p_NewFrame;
}


/*****************************************************************************/
t_Handle F_Clone(t_Handle h_Frame, bool cloneBuffers)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Frame     *p_NewFrame;
    t_Handle    h_Buffer, h_NewBuffer;
    uint32_t    intFlags;

    ASSERT_COND(h_Frame);

    p_NewFrame = (t_Frame *)MEM_Get(h_GlobalMbfPool);

    if (p_NewFrame)
    {
        p_NewFrame->p_Next  = NULL;
        p_NewFrame->info1   = p_Frame->info1;
        p_NewFrame->info2   = p_Frame->info2;
        p_NewFrame->info3   = p_Frame->info3;
        p_NewFrame->info4   = p_Frame->info4;
        p_NewFrame->info5   = p_Frame->info5;
        p_NewFrame->status  = p_Frame->status;
        p_NewFrame->owners  = 1;
        p_NewFrame->firstBuf = NULL;
        p_NewFrame->lastBuf = NULL;
        p_NewFrame->length  = 0;

        h_Buffer = p_Frame->firstBuf;

        if (cloneBuffers)
        {
            /* Clone frame with new buffers */
            while (h_Buffer)
            {
                /* generate a copy of the buffer */
                h_NewBuffer = B_Clone(h_Buffer);
                if (!h_NewBuffer)
                {
                    F_Delete((t_Handle)p_NewFrame, TRUE);
                    p_NewFrame = NULL;
                    break;
                }

                /* append the new buffer to the frame */
                F_AppendBuffer((t_Handle)p_NewFrame, h_NewBuffer);
                h_Buffer = B_GetNext(h_Buffer);
            }
        }
        else
        {
            /* Clone frame by linking existing buffers */
            intFlags = XX_DisableAllIntr();

            while (h_Buffer)
            {
                B_Link(h_Buffer);
                h_Buffer = B_GetNext(h_Buffer);
            }

            XX_RestoreAllIntr(intFlags);

            p_NewFrame->firstBuf = p_Frame->firstBuf;
            p_NewFrame->lastBuf = p_Frame->lastBuf;
            p_NewFrame->length = p_Frame->length;
        }
    }

    return p_NewFrame;
}


/*****************************************************************************/
uint16_t F_NumOfBuffers(t_Handle h_Frame)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;
    uint16_t    numOfBuffers = 0;

    ASSERT_COND(h_Frame);

    h_Buffer = p_Frame->firstBuf;

    while (h_Buffer)
    {
        numOfBuffers++;
        h_Buffer = B_GetNext(h_Buffer);
    }

    return numOfBuffers;
}


#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))


/*****************************************************************************/
void F_Link(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->owners++;
}


/*****************************************************************************/
uint16_t F_GetLength(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->length;
}


/*****************************************************************************/
t_Handle F_GetFirstBuffer(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->firstBuf;
}


/*****************************************************************************/
t_Handle F_GetLastBuffer(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->lastBuf;
}


/*****************************************************************************/
void F_SetInfo1(t_Handle h_Frame, void *info1)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->info1 = info1;
}


/*****************************************************************************/
void F_SetInfo2(t_Handle h_Frame, void *info2)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->info2 = info2;
}


/*****************************************************************************/
void F_SetInfo3(t_Handle h_Frame, uint32_t info3)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->info3 = info3;
}


/*****************************************************************************/
void F_SetInfo4(t_Handle h_Frame, uint32_t info4)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->info4 = info4;
}


/*****************************************************************************/
void F_SetInfo5(t_Handle h_Frame, uint32_t info5)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->info5 = info5;
}


/*****************************************************************************/
void F_SetStatus(t_Handle h_Frame, uint32_t status)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    p_Frame->status = status;
}


/*****************************************************************************/
void *F_GetInfo1(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->info1;
}


/*****************************************************************************/
void *F_GetInfo2(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->info2;
}


/*****************************************************************************/
uint32_t F_GetInfo3(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->info3;
}


/*****************************************************************************/
uint32_t F_GetInfo4(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->info4;
}


/*****************************************************************************/
uint32_t F_GetInfo5(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->info5;
}


/*****************************************************************************/
uint32_t F_GetStatus(t_Handle h_Frame)
{
    t_Frame *p_Frame = (t_Frame *)h_Frame;

    ASSERT_COND(h_Frame);

    return p_Frame->status;
}


#endif /* DEBUG_ERRORS */


/*****************************************************************************/
void F_Print(t_Handle h_Frame)
{
    t_Frame     *p_Frame = (t_Frame *)h_Frame;
    t_Handle    h_Buffer;

    ASSERT_COND(h_Frame);

    h_Buffer = p_Frame->firstBuf;

    while (h_Buffer)
    {
        B_Print(h_Buffer);
        h_Buffer = B_GetNext(h_Buffer);
    }
}


