/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
 /**************************************************************************//**

 @File          gpio_ext.h

 @Description   GPIO Application Programming Interface
*//***************************************************************************/
#ifndef __GPIO_EXT_H
#define __GPIO_EXT_H


#include "std_ext.h"
#include "part_ext.h"
#include "io_port_ext.h"

/**************************************************************************//**
 @Group         gpio_grp GPIO Application Programming Interface

 @Description   GPIO API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         gpio_init_grp GPIO Initialization Unit

 @Description   GPIO initialization unit API functions, definitions and enums

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   GPIO Configuration Parameters.

                This structure contains the configuration parameters for the
                GPIO module. It is used with GPIO_ConfigAndInit() routine.
*//***************************************************************************/
typedef struct t_GpioParam
{
    uint32_t    baseAddr;               /**< Address of GPIO registers */
    int         irq[GPIO_NUM_OF_PORTS]; /**< Interrupt source ID of each port -
                                             pass (-1) if interrupts are not
                                             required */
} t_GpioParam;


/**************************************************************************//**
 @Function      GPIO_ConfigAndInit

 @Description   Configures and initializes the I/O Port manager.

                This routine must be called prior to any I/O ports pin access.

 @Param[in]     p_GpioParam - Pointer to GPIO configuration structure.

 @Return        A handle to the GPIO module, to be passed on subsequent
                API routines.
*//***************************************************************************/
t_Handle GPIO_ConfigAndInit(t_GpioParam *p_GpioParam);

/**************************************************************************//**
 @Function      GPIO_Free

 @Description   Frees resources allocated for managing the IO port.

 @Param[in]     h_Gpio - The handle for that port, as returned by the
                         GPIO_ConfigAndInit() routine.

 @Return        E_OK on success, other value otherwise.
*//***************************************************************************/
t_Error GPIO_Free(t_Handle h_Gpio);

/** @} */ /* end of gpio_init_grp group */


/**************************************************************************//**
 @Group         gpio_control_grp GPIO Control Unit

 @Description   GPIO control unit API functions, definitions and enums

 @{
*//***************************************************************************/

/* Values for internal usage */
#define IO_PIN_PORT_ID_MASK     0x0F000000
#define IO_PIN_ID_MASK          0x001F0000
#define IO_PIN_DIR_MASK         0x00000030
#define IO_PIN_SHARED_MASK      0x00000008
#define IO_PIN_ASSIGN_MASK      0x00000007
#define IO_PIN_PORT_ID_SHIFT    24
#define IO_PIN_ID_SHIFT         16
#define IO_PIN_DIR_SHIFT        4


/**************************************************************************//**
 @Function      IO_PIN

 @Description   Macro for creating an I/O pin number.

                The number generated by this macro contains all required
                information for pin configuration. This number must be provided
                in subsequent calls to any IO_PORT routine.

 @Param[in]     _portId - Port ID (interpreted by the specific I/O port module).
 @Param[in]     _pinId  - Pin ID inside the port.
 @Param[in]     _assign - Assigned pin functionality (see e_IoPinAssignment)..
 @Param[in]     _dir    - Pin direction (see e_IoPinDirection).
 @Param[in]     _shared - TRUE if this pin can be shared by several users;
                          FALSE if this pin is exclusive for a single user.

 @Return        The I/O pin number.
*//***************************************************************************/
#define IO_PIN(_portId, _pinId, _assign, _dir, _shared)                 \
    ((int)                                                              \
     ((((_portId) << IO_PIN_PORT_ID_SHIFT)  & IO_PIN_PORT_ID_MASK)  |   \
      (((_pinId)  << IO_PIN_ID_SHIFT)       & IO_PIN_ID_MASK)       |   \
      (((_dir)    << IO_PIN_DIR_SHIFT)      & IO_PIN_DIR_MASK)      |   \
      ((_assign)                            & IO_PIN_ASSIGN_MASK)   |   \
      ((_shared) ? IO_PIN_SHARED_MASK : 0)))


#define GPIO_PIN(_portId, _pinId, _assign, _dir, _shared) \
    IO_PIN((_portId), (_pinId), e_IO_PIN_ASSIGN_##_assign, e_IO_PIN_DIR_##_dir, (_shared))

#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      GPIO_DumpRegs

 @Description   Dumps GPIO registers.

 @Param[in]     h_Gpio - Handle to GPIO object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error GPIO_DumpRegs(t_Handle h_Gpio);

#endif /* DEBUG_ERRORS > 0 */


/** @} */ /* end of gpio_control_grp group */
/** @} */ /* end of gpio_grp  */

#endif /* __GPIO_EXT_H */

