/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          io_port_ext.h

 @Description   General Purpose I/O Ports Application Programming Interface
*//***************************************************************************/
#ifndef __IO_PORT_EXT_H
#define __IO_PORT_EXT_H

#include "std_ext.h"


/**************************************************************************//**
 @Group         io_port_grp Generic I/O Port Application Programming Interface

 @Description   Generic I/O Port API functions, definitions and enums

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         io_port_init_grp Generic I/O Port Initialization Unit

 @Description   Generic I/O Port initialization unit API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   I/O Pin Direction
*//*************************************************************************/
typedef enum e_IoPinDirection
{
    e_IO_PIN_DIR_OPEN_DRAIN_OUTPUT = 0, /**< Open-drain output pin */
    e_IO_PIN_DIR_OUTPUT,                /**< Direct output pin */
    e_IO_PIN_DIR_INPUT,                 /**< Input pin */
    e_IO_PIN_DIR_IN_OUT                 /**< Both input and output pin */
} e_IoPinDirection;

/**************************************************************************//**
 @Description   I/O Pin Interrupt Mode
*//*************************************************************************/
typedef enum e_IoPinIntrMode
{
    e_IO_PIN_INTR_DISABLED = 0, /**< No interrupt on input change */
    e_IO_PIN_INTR_ANY_CHANGE,   /**< Interrupt on any input change */
    e_IO_PIN_INTR_HIGH_TO_LOW,  /**< Interrupt on high-to-low input change */
    e_IO_PIN_INTR_LOW_TO_HIGH   /**< Interrupt on low-to-high input change */
} e_IoPinIntrMode;

/**************************************************************************//**
 @Description   I/O Pin Assignment
*//*************************************************************************/
typedef enum e_IoPinAssignment
{
    e_IO_PIN_ASSIGN_GPIO       = 0, /**< General Purpose I/O */
    e_IO_PIN_ASSIGN_OPTION_0   = 0, /**< Dedicated I/O option 0 (only where GPIO is unavailable) */
    e_IO_PIN_ASSIGN_OPTION_1   = 1, /**< Dedicated I/O option 1 */
    e_IO_PIN_ASSIGN_OPTION_2   = 2, /**< Dedicated I/O option 2 */
    e_IO_PIN_ASSIGN_OPTION_3   = 3, /**< Dedicated I/O option 3 */
    e_IO_PIN_ASSIGN_OPTION_4   = 4, /**< Dedicated I/O option 4 */
    e_IO_PIN_ASSIGN_OPTION_5   = 5, /**< Dedicated I/O option 5 */
    e_IO_PIN_ASSIGN_OPTION_6   = 6, /**< Dedicated I/O option 6 */
    e_IO_PIN_ASSIGN_OPTION_7   = 7  /**< Dedicated I/O option 7 */
} e_IoPinAssignment;


/**************************************************************************//**
 @Description   I/O pin interrupt callback routine type.

                The user can register callbacks of this type using the
                IO_PORT_SetIntrCallback() routine.

 @Param[in]     h_App   - The user's application context.
 @Param[in]     portId  - The port ID.
 @Param[in]     pinId   - The pin ID inside the port.

 @Return        None.
*//***************************************************************************/
typedef void (t_IoPinIntrCallback)(t_Handle h_App, uint8_t portId, uint8_t pinId);


#define IO_PIN_UNKNOWN          ((int)0xFFFFFFFF) /**< Invalid/unknown I/O pin number */


/* Values for internal usage */
#define IO_PIN_PORT_ID_MASK     0x0F000000
#define IO_PIN_ID_MASK          0x001F0000
#define IO_PIN_DIR_MASK         0x00000030
#define IO_PIN_SHARED_MASK      0x00000008
#define IO_PIN_ASSIGN_MASK      0x00000007
#define IO_PIN_PORT_ID_SHIFT    24
#define IO_PIN_ID_SHIFT         16
#define IO_PIN_DIR_SHIFT        4


/**************************************************************************//**
 @Function      IO_PIN

 @Description   Macro for creating an I/O pin number.

                The number generated by this macro contains all required
                information for pin configuration. This number must be provided
                in subsequent calls to any IO_PORT routine.

 @Param[in]     _portId - Port ID (interpreted by the specific I/O port module).
 @Param[in]     _pinId  - Pin ID inside the port.
 @Param[in]     _assign - Assigned pin functionality (see e_IoPinAssignment)..
 @Param[in]     _dir    - Pin direction (see e_IoPinDirection).
 @Param[in]     _shared - TRUE if this pin can be shared by several users;
                          FALSE if this pin is exclusive for a single user.

 @Return        The I/O pin number.
*//***************************************************************************/
#define IO_PIN(_portId, _pinId, _assign, _dir, _shared)                 \
    ((int)                                                              \
     ((((_portId) << IO_PIN_PORT_ID_SHIFT)  & IO_PIN_PORT_ID_MASK)  |   \
      (((_pinId)  << IO_PIN_ID_SHIFT)       & IO_PIN_ID_MASK)       |   \
      (((_dir)    << IO_PIN_DIR_SHIFT)      & IO_PIN_DIR_MASK)      |   \
      ((_assign)                            & IO_PIN_ASSIGN_MASK)   |   \
      ((_shared) ? IO_PIN_SHARED_MASK : 0)))


/**************************************************************************//**
 @Function      IO_PIN_PORT_ID

 @Description   Macro for extracting the port ID from an I/O pin number.

 @Param[in]     _ioPin - The I/O pin number.

 @Return        The I/O pin port ID.
*//***************************************************************************/
#define IO_PIN_PORT_ID(_ioPin) \
    (uint8_t)(((_ioPin) & IO_PIN_PORT_ID_MASK) >> IO_PIN_PORT_ID_SHIFT)

/**************************************************************************//**
 @Function      IO_PIN_ID

 @Description   Macro for extracting the pin ID from an I/O pin number.

 @Param[in]     _ioPin - The I/O pin number.

 @Return        The I/O pin ID.
*//***************************************************************************/
#define IO_PIN_ID(_ioPin) \
    (uint8_t)(((_ioPin) & IO_PIN_ID_MASK) >> IO_PIN_ID_SHIFT)

/**************************************************************************//**
 @Function      IO_PIN_DIRECTION

 @Description   Macro for extracting the pin direction from an I/O pin number.

 @Param[in]     _ioPin - The I/O pin number.

 @Return        The I/O pin direction.
*//***************************************************************************/
#define IO_PIN_DIRECTION(_ioPin) \
    (e_IoPinDirection)(((_ioPin) & IO_PIN_DIR_MASK) >> IO_PIN_DIR_SHIFT)

/**************************************************************************//**
 @Function      IO_PIN_ASSIGNMENT

 @Description   Macro for extracting the pin assignment from an I/O pin number.

 @Param[in]     _ioPin - The I/O pin number.

 @Return        The I/O pin assignment.
*//***************************************************************************/
#define IO_PIN_ASSIGNMENT(_ioPin) \
    (e_IoPinAssignment)((_ioPin) & IO_PIN_ASSIGN_MASK)

/** @} */ /* end of io_port_init_grp  */


/**************************************************************************//**
 @Group         io_port_ctrl_grp Generic I/O Port Control Unit

 @Description   Generic I/O Port control unit API.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      IO_PORT_ConfigurePin

 @Description   Configures the selected I/O pin.

                The given I/O pin must be defined using the IO_PIN() macro.
                The number generated by this macro contains all required
                information for pin configuration.

 @Param[in]     h_IoPortModule  - Handle to the specific I/O port module.
 @Param[in]     ioPin           - The I/O pin number.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error IO_PORT_ConfigurePin(t_Handle h_IoPortModule, int ioPin);

/**************************************************************************//**
 @Function      IO_PORT_ReleasePin

 @Description   Releases the selected I/O pin.

                The given I/O pin must be defined using the IO_PIN() macro.

 @Param[in]     h_IoPortModule  - Handle to the specific I/O port module.
 @Param[in]     ioPin           - The I/O pin number.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error IO_PORT_ReleasePin(t_Handle h_IoPortModule, int ioPin);

/**************************************************************************//**
 @Function      IO_PORT_ReadPinInput

 @Description   Reads the value of a given input pin.

                The given I/O pin must be defined using the IO_PIN() macro.

 @Param[in]     h_IoPortModule  - Handle to the specific I/O port module.
 @Param[in]     ioPin           - The input pin number.

 @Return        The value of a given input pin (0 or 1).
*//***************************************************************************/
uint8_t IO_PORT_ReadPinInput(t_Handle h_IoPortModule, int ioPin);

/**************************************************************************//**
 @Function      IO_PORT_SetPinOutput

 @Description   Sets the value of a given output pin.

                The given I/O pin must be defined using the IO_PIN() macro.

 @Param[in]     h_IoPortModule  - Handle to the specific I/O port module.
 @Param[in]     ioPin           - The output pin number.
 @Param[in]     output          - The value to set on the output pin (0 or 1).

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error IO_PORT_SetPinOutput(t_Handle h_IoPortModule, int ioPin, uint8_t output);

/**************************************************************************//**
 @Function      IO_PORT_SetIntrCallback

 @Description   Registers a user callback for the interrupt of a given input pin.

                The given I/O pin must be defined using the IO_PIN() macro.

                The user selects the interrupt mode for the pin, and provides a
                callback routine and its argument (user's context).

 @Param[in]     h_IoPortModule  - Handle to the specific I/O port module.
 @Param[in]     ioPin           - The input pin number.
 @Param[in]     intrMode        - Interrupt mode.
 @Param[in]     f_IntrCallback  - Interrupt callback routine.
 @Param[in]     h_App           - The user's application context passed as an
                                  argument to the interrupt callback routine.

 @Return        E_OK on success; error code otherwise.
*//***************************************************************************/
t_Error IO_PORT_SetIntrCallback(t_Handle            h_IoPortModule,
                                int                 ioPin,
                                e_IoPinIntrMode     intrMode,
                                t_IoPinIntrCallback *f_IntrCallback,
                                t_Handle            h_App);


/** @} */ /* end of io_port_ctrl_grp  */
/** @} */ /* end of io_port_grp  */


#endif /* __IO_PORT_EXT_H */

