/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**

 @File          sec_jq_ext.h

 @Description   SEC JQ external structures and definitions.
*//***************************************************************************/
#ifndef __SEC_JQ_EXT_H
#define __SEC_JQ_EXT_H


#include "sec_ext.h"


/**************************************************************************//**
 @Group         sec_jq_mdl_grp SEC JQ Application Programming Interface

 @Description   Functions, definitions and enumerators

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         sec_jq_mdl_initialization_grp SEC JQ Initialization Unit

 @Description   Initialization routines

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Prototype of the application routine called upon every job completion.

 @Param[in]     h_App     - Application parameters.
 @Param[in]     p_JobDesc - Job descriptor address.
 @Param[in]     h_Context - Job descriptor context.
 @Param[in]     jobStatus - Job status.

 @Return        None.
*//***************************************************************************/
typedef void (t_SecJqCbr)(t_Handle h_App,
                          void*    p_JobDesc,
                          t_Handle h_Context,
                          uint32_t jobStatus);


/**************************************************************************//**
 @Description   Configuration Parameters
*//***************************************************************************/
typedef struct t_SecJqParam
{
    t_Handle    h_Sec         ; /**< SEC object                 */
    uint64_t    baseAddr      ; /**< Base address               */
    uint8_t     jobQueueId    ; /**< Job queue ID               */
    uint16_t    seqLiodnVal   ; /**< Sequential LIODN value     */
    uint16_t    nonSeqLiodnVal; /**< Non-sequential LIODN value */
    t_SecJqCbr* f_Cbr         ; /**< Callback routine           */
    t_Handle    h_App         ; /**< Application                */
    int         irq           ; /**< IRQ                        */
}
t_SecJqParam;


/**************************************************************************//**
 @Function      SEC_JQ_Config

 @Description   Allocates a SEC JQ object.

 @Param[in]     p_Param - Pointer to SEC JQ Configuration parameters.

 @Return        Handle to a SEC JQ object on success; NULL otherwise.
*//***************************************************************************/
t_Handle SEC_JQ_Config(t_SecJqParam* p_Param);


/**************************************************************************//**
 @Function      SEC_JQ_Init

 @Description   Initializes a SEC JQ registers.

 @Param[in]     h_SecJq - SEC JQ object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_Init(t_Handle h_SecJq);


/**************************************************************************//**
 @Function      SEC_JQ_Free

 @Description   Deallocates a SEC JQ object.

 @Param[in]     h_SecJq - SEC JQ object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_Free(t_Handle h_SecJq);


/**************************************************************************//**
 @Group         sec_jq_mdl_advance_config_grp SEC JQ Advanced Configuration Unit

 @Description   Advanced configuration routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      SEC_JQ_ConfigAllowMtdJobs

 @Description   Configures a SEC JQ whether or not to allow MTD jobs.

 @Param[in]     h_SecJq      - SEC JQ object.
 @Param[in]     allowMtdJobs - Allow MTD jobs (default value = [FALSE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_ConfigAllowMtdJobs(t_Handle h_SecJq,
                                  bool     allowMtdJobs);


/**************************************************************************//**
 @Description   Ring Configuration Parameters
*//***************************************************************************/
typedef struct t_SecJqRingParam
{
    uint32_t len  ; /**< Number of entries   */
    int      memId; /**< Memory partition ID */
}
t_SecJqRingParam;


/**************************************************************************//**
 @Function      SEC_JQ_ConfigInputRing

 @Description   Configures a SEC JQ input ring.

 @Param[in]     h_SecJq     - SEC JQ object.
 @Param[in]     p_RingParam - Input ring configuration parameters:
                              - len   - Number of entries   (default value = [256  ]).
                              - memId - Memory partition ID (default value = [0]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      len must be between [MIN_VAL_inputRingLen] and [MAX_VAL_inputRingLen].
*//***************************************************************************/
t_Error SEC_JQ_ConfigInputRing(t_Handle          h_SecJq,
                               t_SecJqRingParam* p_RingParam);


/**************************************************************************//**
 @Function      SEC_JQ_ConfigOutputRing

 @Description   Configures a SEC JQ output ring.

 @Param[in]     h_SecJq     - SEC JQ object.
 @Param[in]     p_RingParam - Output ring configuration parameters:
                              - len   - Number of entries   (default value = [256  ]).
                              - memId - Memory partition ID (default value = [0]).

 @Return        E_OK on success; Error code otherwise.

 @Cautions      len must be between [MIN_VAL_outputRingLen] and [MAX_VAL_outputRingLen].
*//***************************************************************************/
t_Error SEC_JQ_ConfigOutputRing(t_Handle          h_SecJq,
                                t_SecJqRingParam* p_RingParam);


/**************************************************************************//**
 @Description   Interrupt Configuration Parameters
*//***************************************************************************/
typedef struct t_SecJqIntrParam
{
    uint32_t jobCountThr; /**< Job-count threshold */
    uint32_t ticCountThr; /**< Tic-count threshold */
}
t_SecJqIntrParam;


/**************************************************************************//**
 @Collection   Interrupt Definitions
*//***************************************************************************/
#define SEC_JQ_INTR_TIC_COUNT_THR_UNITS 64 /**< Interrupt tic-count threshold units */
/* @} */


/**************************************************************************//**
 @Function      SEC_JQ_ConfigInterrupts

 @Description   Configures a SEC JQ interrupts.

 @Param[in]     h_SecJq     - SEC JQ object.
 @Param[in]     p_IntrParam - Interrupt configuration parameters:
                              - jobCountThr - Job-count threshold (default value = [1]).
                              - ticCountThr - Tic-count threshold (default value = [0]).

                The job-count measures the number of jobs completed since the previous interrupt.
                The tic-count measures the number of tics elapsed after the completion of the first job.
                Therefore, if the job-count threshold is set to 1, then the tic-count threshold has no effect.

 @Return        E_OK on success; Error code otherwise.

 @Cautions      ticCountThr must be a multiple of SEC_JQ_INTR_TIC_COUNT_THR_UNITS.
                jobCountThr must be between [MIN_VAL_intrJobCountThr] and [MAX_VAL_intrJobCountThr].
                ticCountThr must be between [MIN_VAL_intrTicCountThr] and [MAX_VAL_intrTicCountThr].
*//***************************************************************************/
t_Error SEC_JQ_ConfigInterrupts(t_Handle          h_SecJq,
                                t_SecJqIntrParam* p_IntrParam);


/**************************************************************************//**
 @Description   Byte Swap Configuration Parameters
*//***************************************************************************/
typedef struct t_SecJqByteSwapParam
{
    e_SecByteSwap icd; /* Input control data  */
    e_SecByteSwap ocd; /* Output control data */
    e_SecByteSwap imd; /* Input message data  */
    e_SecByteSwap omd; /* Output message data */
}
t_SecJqByteSwapParam;


/**************************************************************************//**
 @Function      SEC_JQ_ConfigByteSwap

 @Description   Configures a SEC JQ Byte-Swap mode.

 @Param[in]     h_SecJq         - SEC JQ object.
 @Param[in]     p_ByteSwapParam - Byte swap configuration parameters:
                                  - icd - Input control data  (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - ocd - Output control data (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - imd - Input message data  (default value = [e_SEC_BYTE_SWAP_NONE]).
                                  - omd - Output message data (default value = [e_SEC_BYTE_SWAP_NONE]).

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_ConfigByteSwap(t_Handle              h_SecJq,
                              t_SecJqByteSwapParam* p_ByteSwapParam);


/** @} */ /* end of sec_jq_mdl_advance_config_grp */
/** @} */ /* end of sec_jq_mdl_initialization_grp */


/**************************************************************************//**
 @Group         sec_jq_mdl_runtime_control_grp SEC JQ Control Unit

 @Description   Runtime control routines

 @{
*//***************************************************************************/


/**************************************************************************//**
 @Function      SEC_JQ_Enqueue

 @Description   Enqueues one job onto a SEC JQ.

 @Param[in]     h_SecJq   - SEC JQ object.
 @Param[in]     p_JobDesc - Job descriptor address.
 @Param[in]     h_Context - Job descriptor context.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_Enqueue(t_Handle h_SecJq,
                       void*    p_JobDesc,
                       t_Handle h_Context);


/**************************************************************************//**
 @Function      SEC_JQ_Dequeue

 @Description   Dequeues all jobs from a SEC JQ.

 @Param[in]     h_SecJq - SEC JQ object.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SEC_JQ_Dequeue(t_Handle h_SecJq);


#if (DEBUG_ERRORS > 0)

/**************************************************************************//**
 @Function      SEC_JQ_DumpRegisters

 @Description   Dumps the contents of a SEC JQ registers.

 @Param[in]     h_SecJq - SEC JQ object.

 @Return        None.
*//***************************************************************************/
void SEC_JQ_DumpRegisters(t_Handle h_SecJq);

#endif /* (DEBUG_ERRORS > 0) */


/** @} */ /* end of sec_jq_mdl_runtime_control_grp */
/** @} */ /* end of sec_jq_mdl_grp */


#endif /* __SEC_JQ_EXT_H */
