/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          mem_mng_ext.h

 @Description   External definitions for memory allocation management module.
*//***************************************************************************/
#ifndef __MEM_MNG_EXT_H
#define __MEM_MNG_EXT_H


#include "std_ext.h"


/**************************************************************************//**
 @Group         mem_mng_grp     Memory Allocation Management module

 @Description   API for memory allocation management.

 @{
*//***************************************************************************/

#define MEM_MNG_MAX_PARTITION_NAME_LEN      32
        /**< Maximum length of memory partition name */


#define MEM_MNG_EARLY_PARTITION_ID          ((int)(-1))
        /**< Dummy partition ID for early memory allocations (before partitions
             are registered); allows memory leaks checking even for early
             memory allocations. */


/**************************************************************************//**
 @Description   Memory management module initialization parameters
 *//***************************************************************************/
typedef struct t_MemMngParam
{
    void *  (*f_Malloc)(uint32_t size);
            /**< Memory allocation routine for internal structures */
    void    (*f_Free)(void *p_Mem);
            /**< Memory deallocation routine for internal structures */

    void *  (*f_EarlyMalloc)(uint32_t size, uint32_t alignment);
            /**< Early allocation routine (before partitions are registered) */
    void    (*f_EarlyFree)(void *p_Addr);
            /**< Early deallocation routine (before partitions are registered) */

    t_Handle    h_Spinlock;
            /**< We might not have malloc at the begining, so we
                 need the spinlock object to be allocated outside of the manager */
} t_MemMngParam;


/**************************************************************************//**
 @Function      MEM_MNG_Init

 @Description   Initialize the memory allocation management module.

 @Param[in]     p_MemMngParam - MEM_MNG initialization parameters.

 @Return        Handle to intialized MEM_MNG object, or NULL on error.
*//***************************************************************************/
t_Handle MEM_MNG_Init(t_MemMngParam *p_MemMngParam);

/**************************************************************************//**
 @Function      MEM_MNG_Free

 @Description   Free the memory allocation management module.

 @Param[in]     h_MemMng - Handle to MEM_MNG object.

 @Return        None.
*//***************************************************************************/
void MEM_MNG_Free(t_Handle h_MemMng);


void * MEM_MNG_AllocMem(t_Handle    h_MemMng,
                        int         partitionId,
                        uint32_t    size,
                        uint32_t    alignment,
                        char        *info,
                        char        *filename,
                        int         line);

void MEM_MNG_FreeMem(t_Handle h_MemMng, void *p_Memory);


/**************************************************************************//**
 @Description   Memory partition information
 *//***************************************************************************/
typedef struct t_MemMngPartitionInfo
{
    char        name[MEM_MNG_MAX_PARTITION_NAME_LEN];
    uint32_t    baseAddress;
    uint32_t    size;
    uint32_t    attributes;
    uint32_t    currentUsage;
    uint32_t    maximumUsage;
    uint32_t    totalAllocations;
    uint32_t    totalDeallocations;

} t_MemMngPartitionInfo;


t_Error MEM_MNG_GetPartitionIdByAddr(t_Handle   h_MemMng,
                                     uint32_t   addr,
                                     int        *p_PartitionId);


int MEM_MNG_GetAvailablePartitionId(t_Handle h_MemMng);

t_Error MEM_MNG_RegisterPartition(t_Handle  h_MemMng,
                                  int       partitionId,
                                  uint32_t  baseAddress,
                                  uint32_t  size,
                                  uint32_t  attributes,
                                  char      name[],
                                  void *    (*f_UserMalloc)(uint32_t size, uint32_t alignment),
                                  void      (*f_UserFree)(void *p_Addr),
                                  bool      enableDebug);

t_Error MEM_MNG_UnregisterPartition(t_Handle h_MemMng, int partitionId);


/**************************************************************************//**
 @Function      MEM_MNG_GetPartitionInfo

 @Description   Get information and usage statistics of a selected partition.
*//***************************************************************************/
t_Error MEM_MNG_GetPartitionInfo(t_Handle               h_MemMng,
                                 int                    partitionId,
                                 t_MemMngPartitionInfo  *p_PartitionInfo);


typedef void (t_MemMngLeakReportFunc)(uint32_t    addr,
                                      uint32_t    size,
                                      char        *info,
                                      char        *filename,
                                      int         line);

uint32_t MEM_MNG_CheckLeaks(t_Handle                h_MemMng,
                            int                     partitionId,
                            t_MemMngLeakReportFunc  *f_ReportLeak);


/** @} */ /* end of mem_mng_grp */


#endif /* __MEM_MNG_EXT_H */
