/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**

 @File          mii.h

 @Description   Media Independent Interface (MII) API
*//***************************************************************************/

#ifndef __MII_H_
#define __MII_H_


/**************************************************************************//**
 @Group         mii Media Independent Interface API

 @Description   Media Independent Interface (MII) API

 @{
*//***************************************************************************/

/******************************************************************************

                   Media Independent Interface (MII) Module
                   ========================================

This module provides a MII-compliant API, that can aid Ethernet drivers in
negotiating the technology abilities of a local PHY with their link-partner,
and configuring the local PHY to a desired operating mode. The MII connects
the CSMA/CD Media Access Controller (MAC) with the Physical Layer Entity (PHY).
MII and its management interface are defined and detailed in the IEEE
802.3 standard.

A MII-compliant PHY is typically configured through the MII management
interface, which contains two lines: Management Data Clock (MDC) and
Management Data Input/Output (MDIO). MDC sets the required timing reference
for data transactions on the MDIO line, between the PHY and the MAC. MDIO data
contains both control and status information, encoded into a frame format.
This MII module is not familiar with the specific MAC-MII interface, so the
driver must implement and supply the read and write routines to access the
MII management interface. See "Required Callback Functions" below.


MII Module API:
===============

    int MII_PhyInitialize(t_MiiParam *p_MiiParam);

This routine configures the PHY for operation. It should be called from the
driver's initialization routine. The PHY is programmed through the PHY read
and PHY write routines that are provided by the driver in the MII information
structure (t_MiiParam). Before using this routine, the driver has to allocate
and initialize the MII information structure. See MII_PhyInitialize()
documentation for more details on the configuration methods.


    int MII_PhyCtrl(t_MiiParam *p_MiiParam, unsigned short command);

This routine controls the PHY after initialization. It takes a pointer to
a MII information structure that was previously supplied to the
MII_PhyInitialize() routine, and the command that should be carried out. See
MII_CTRL_xxx for available control commands.


Required Callback Functions:
============================

    int f_MiiPhyRead(void *h_Driver, unsigned char phyAddr,
                     unsigned char phyReg, unsigned short *p_Data);

This routine should implement a MII management interface read operation
of a 16-bit value from the <phyReg> register of the MII-compliant PHY
in address <phyAddr>, into the 16-bit address pointed by <p_Data>.


    int f_MiiPhyWrite(void *h_Driver, unsigned char phyAddr,
                      unsigned char phyReg, unsigned short data);

This routine should implement a MII management interface write operation
of the 16-bit <data> to the <phyReg> register of the MII-compliant PHY
in address <phyAddr>.

    void f_MiiWait(unsigned long waitParam);

This routine should implement a limited delay (either idle or busy) to the
calling task. It is called with the <waitParam> field of the MII
information structure as parameter, which represents the delay unit. The
<maxWaitCycles> field of the structure represents the maximum number of
cycles (calls to f_MiiWait()) that are allowed. When NULL is provided instead
of this function, no delay shall be performed (this is not recommended).
Note, that the MII auto-negotiation process may take 3-5 seconds to
complete, therefore the <waitParam> and <maxWaitCycles> fields must be
set correspondingly.


Usage Examples
==============

Typical usage, with auto-negotiation:

    t_MiiParam  miiParam;
    int         ret;

    miiParam.phyAddr = MII_DEF_PHY_ADDR;    // Start with the PHY in address 0
    miiParam.numOfPhys = 1;                 // Scan only one PHY
    miiParam.configFlags = MII_CFG_AUTO;    // Use auto-negotiation, no media limitations
    miiParam.waitParam = 1000;              // Delay unit = 1 second (example, system-specific)
    miiParam.maxWaitCycles = 6;             // Maximum allowed delay = 6 seconds
    miiParam.f_MiiPhyRead = DrvPhyRead;     // The driver's MII management read routine
    miiParam.f_MiiPhyWrite = DrvPhyWrite;   // The driver's MII management write routine
    miiParam.f_MiiWait = DrvWait;           // The driver's delay routine
    miiParam.h_Driver = NULL;               // Driver handle is not needed

    ret = MII_PhyInitialize(&miiParam);

    if (ret == MII_OK)
    {
        // check miiParam.configResults
    }


To use auto-negotiation, but allow only 1000 Mb/s Full-Duplex media, use:

    miiParam.configFlags = MII_CFG_AUTO | MII_CFG_ANY_1000 | MII_CFG_FD;


To set the PHY manually to any 100 Mb/s media in loopback mode, use:

    miiParam.configFlags = MII_CFG_ANY_100 | MII_CFG_LOOPBACK;


******************************************************************************/


typedef int (t_MiiPhyReadFunc)(void             *h_Driver,
                               unsigned char    phyAddr,
                               unsigned char    phyReg,
                               unsigned short   *p_Data);

typedef int (t_MiiPhyWriteFunc)(void            *h_Driver,
                                unsigned char   phyAddr,
                                unsigned char   phyReg,
                                unsigned short  data);

typedef void (t_MiiWaitFunc)(unsigned long waitParam);



/**************************************************************************//**
 @Description   MII Information Structure.

                This structure must be initialized by the driver before using
                the MII API routines.

*//***************************************************************************/
typedef struct
{
    unsigned char       phyAddr;
    /**< In:  Address of the first PHY to scan (0 to (MII_MAX_PHYS - 1));
         Out: Address of the first valid PHY in the requested range. */

    unsigned char       numOfPhys;
    /**< In:  Number of consecutive PHYs to scan (1 to (MII_MAX_PHYS - phyAddr)). */

    unsigned long       configFlags;
    /**< In:  Configuration flags (see MII_CFG_xxx);
         Out: Updated configuration flags, cleared from unsupported abilities. */

    unsigned long       configResults;
    /**< Out: Configuration result flags (see MII_CFG_xxx). */

    unsigned long       waitParam;
    /**< In:  The parameter of f_MiiWait() (passed on each call). */

    unsigned long       maxWaitCycles;
    /**< In:  Maximum wait cycles (calls to f_MiiWait()). */

    t_MiiWaitFunc       *f_MiiWait;
    /**< In:  Delay routine. */

    t_MiiPhyReadFunc    *f_MiiPhyRead;
    /**< In:  MII management PHY read routine. */

    t_MiiPhyWriteFunc   *f_MiiPhyWrite;
    /**< In:  MII management PHY write routine. */

    void                *h_Driver;
    /**< In:  (Optional) A handle (meaningful for the driver only) that will
              be passed as a parameter to the f_MiiPhyRead() and f_MiiPhyWrite()
              routines. */

} t_MiiParam;


/******************************************************************************
    MII API Functions
******************************************************************************/

/**************************************************************************//**
 @Function      MII_PhyInitialize

 @Description   Configures the PHY for operation.

                This function scans the requested range of PHY addresses
                (starting from p_MiiParam->phyAddr, up to a total of
                p_MiiParam->numOfPhys), until a valid MII-compliant PHY is
                found. Then the PHY is configured according to the given
                configuration flags. The address of the PHY is returned in the
                MII information structure, as well as the results of the PHY
                configuration. See the documentation of t_MiiParam for more
                details.

                A PHY can be configured using direct ("manual") mode selection
                or through auto-negotiation (as defined in the IEEE 802.3
                standard). In the auto-negotiation process, both link partners
                negotiate their abilities and the highest-priority common
                ability is selected. The user can mask some of the PHY's
                abilities, by specifying the requested operating mode in the
                configuration flags. Thus, it is always recommended to use the
                auto-negotiation method (specify MII_CFG_AUTO in the
                configuration flags). If auto-negotiation fails and MII_CFG_FORCE
                configuration flag is provided, manual mode configuration is
                immediately attempted with the existing configuration flags.

                In manual mode selection, the configuration flags are checked
                and the highest-priority ability is activated. If no media and
                no duplex mode are specified, operation mode defaults to
                10base-T, Full Duplex.

                When the MII_CFG_FORCE flag is specified in the configuration
                flags and the link was not established, the selected manual
                operating mode shall be written again to the PHY control register,
                without re-checking the link. In this case, MII_CFG_FORCE will
                be specified in the configuration result flags, but the PHY is
                not guaranteed to work properly.

 @Param         p_MiiParam    - (in) pointer to an initialized MII information
                              structure.

 @Return        See MII Return Codes.

*//****************************************************************************/
int MII_PhyInitialize(t_MiiParam *p_MiiParam);


/**************************************************************************//**
 @Function      MII_PhyCtrl

 @Description   Controls the PHY after initialization.

                Note, that this function does not scan the PHY addresses range.
                It assumes the given PHY address is valid (as returned from
                MII_PhyInitialize()). Also, the configuration flags and the
                configuration result flags are ignored and have no meaning
                within this function. The recommended usage for this function
                is to supply it with the same MII information structure that
                was previously supplied to MII_PhyInitialize().

 @Param         p_MiiParam    - (in) pointer to an initialized MII information
                              structure.
 @Param         command     - (in) requested command (see MII_CTRL_xxx).

 @Return        See MII Return Codes.

*//****************************************************************************/
int MII_PhyCtrl(t_MiiParam *p_MiiParam, unsigned short command);


/******************************************************************************
    MII Return Codes
******************************************************************************/
#define MII_OK                  0   /**< Successful operation */
#define MII_ERROR               1   /**< General error */

#define MII_ILLEGAL_PARAMS      10  /**< Illegal parameters */
#define MII_ILLEGAL_MODE        11  /**< Illegal configuration requested */
#define MII_WAIT_LIMIT          12  /**< Wait limit was reached */

#define MII_INVALID_PHY         20  /**< No valid PHY was found */
#define MII_LINK_DOWN           21  /**< Link is down */
#define MII_REMOTE_FAULT        22  /**< Link-partner's fault detected */


/******************************************************************************
    PHY Addresses Constants
******************************************************************************/
#define MII_DEF_PHY_ADDR        0   /**< Default PHY address */
#define MII_MAX_PHYS            32  /**< Maximum number of consecutive PHYs */


/******************************************************************************
    MII Configuration Flags
******************************************************************************/
#define MII_CFG_1000T           0x80000000  /**< Select 1000base-T FD */
#define MII_CFG_1000X           0x40000000  /**< Select 1000base-X FD */
#define MII_CFG_100T2           0x20000000  /**< Select 100base-T2 FD */
#define MII_CFG_100TX           0x10000000  /**< Select 100base-TX FD */
#define MII_CFG_100T4           0x08000000  /**< Select 100base-T4 (HD) */
#define MII_CFG_10T             0x04000000  /**< Select 10base-T FD */

#define MII_CFG_ANY_1000        (MII_CFG_1000X | MII_CFG_1000T)
#define MII_CFG_ANY_100         (MII_CFG_100T4 | MII_CFG_100TX | MII_CFG_100T2)
#define MII_CFG_ANY_10          (MII_CFG_10T)

#define MII_CFG_ANY_MEDIA       (MII_CFG_ANY_1000 | MII_CFG_ANY_100 | MII_CFG_ANY_10)

#define MII_CFG_FD              0x00800000  /**< Select Full Duplex only */
#define MII_CFG_HD              0x00400000  /**< Select Half Duplex only */
#define MII_CFG_LOOPBACK        0x00200000  /**< Set PHY to loopback mode */
#define MII_CFG_MASTER          0x00100000  /**< Set PHY as master (100base-T2/1000base-T) */
#define MII_CFG_PAUSE           0x00080000  /**< Set PHY to support PAUSE function (relevant for Full Duplex) */

#define MII_CFG_EXT_LOOPBACK    0x00000010  /**< External loopback mode
                                                 (ignore link-partner's auto-negotiation results) */
#define MII_CFG_NO_ABILITY_CLR  0x00000008  /**< Do not remove abilities that are missing from status registers */
#define MII_CFG_NO_RESET        0x00000004  /**< Do not reset the PHY */
#define MII_CFG_AUTO            0x00000002  /**< Configure PHY through auto-negotiation */
#define MII_CFG_FORCE           0x00000001  /**< Configure PHY manually and don't check link;
                                                 However, if MII_CFG_AUTO is also specified then
                                                 auto-negotiation configuration is applied first and only
                                                 if no link has been established manual configuration is
                                                 forced. */


/******************************************************************************
    MII Control Commands
******************************************************************************/
#define MII_CTRL_CHECK_LINK     0x8000      /**< Check the link status */
#define MII_CTRL_RESET          0x4000      /**< Reset the PHY */
#define MII_CTRL_ISOLATE        0x2000      /**< Electrically isolate the PHY */
#define MII_CTRL_POWER_DOWN     0x1000      /**< Set the PHY to power-down state */
#define MII_CTRL_RENEGOTIATE    0x0800      /**< Restart the auto-negotiation process */



/** @} *//* end of mii group */

#endif /* __MII_H_ */

