/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/******************************************************************************

 @File          platform_ext.h

 @Description   Prototypes, externals and typedefs for platform routines
****************************************************************************/
#ifndef __PLATFORM_EXT_H
#define __PLATFORM_EXT_H

#include "std_ext.h"
#include "sys_ext.h"


/**************************************************************************//**
 @Group         platform_grp PLATFORM Application Programming Interface

 @Description   Generic Platform API, that must be implemented by each
                specific platform.

 @{
*//***************************************************************************/

#define PLATFORM_IO_MODE_ANY    (-1)    /**< Unspecified I/O mode */

/**************************************************************************//**
 @Description   Cache Operation Mode
*//***************************************************************************/
typedef enum e_CacheMode
{
    e_CACHE_MODE_DISABLED       = 0x00000000,   /**< Cache is disabled */
    e_CACHE_MODE_DATA_ONLY      = 0x00000001,   /**< Cache is enabled for data only */
    e_CACHE_MODE_INST_ONLY      = 0x00000002,   /**< Cache is enabled for instructions only */
    e_CACHE_MODE_DATA_AND_INST  = 0x00000003    /**< Cache is enabled for data and instructions */

} e_CacheMode;

/**************************************************************************//**
 @Description   Memory Partition Identifiers

                Note that not all memory partitions are supported by all
                platforms. Every platform may select which memory partitions
                to support.
*//***************************************************************************/
typedef enum e_MemoryPartitionId
{
    e_MEM_1ST_DDR_CACHEABLE     = 1, /**< Primary DDR cacheable memory partition */
    e_MEM_1ST_DDR_NON_CACHEABLE = 2, /**< Primary DDR non-cacheable memory partition */
    e_MEM_2ND_DDR_CACHEABLE     = 3, /**< Secondary DDR cacheable memory partition */
    e_MEM_2ND_DDR_NON_CACHEABLE = 4, /**< Secondary DDR non-cacheable memory partition */
    e_MEM_SDRAM                 = 5, /**< SDRAM non-cacheable memory partition */
    e_MEM_L2_SRAM               = 6, /**< L2 SRAM cacheable memory partition */
    e_MEM_MURAM_LOW             = 7, /**< High-range MURAM partition */
    e_MEM_MURAM_HIGH            = 8, /**< Low-range MURAM partition */
    e_MEM_MURAM                 = 9  /**< Full-range MURAM partition */

} e_MemoryPartitionId;

/**************************************************************************//**
 @Description   LED Color
*//***************************************************************************/
typedef enum e_LedColor
{
   e_LED_COLOR_GREEN,   /**< Green LED */
   e_LED_COLOR_YELLOW,  /**< Yellow LED */
   e_LED_COLOR_RED      /**< Red LED */

} e_LedColor;



#define MAX_CHIP_NAME_LEN   10   /**< Maximum chip name length (including null character) */

/**************************************************************************//**
 @Description   Chip Type and Revision Information Structure
*//***************************************************************************/
typedef struct t_ChipRevInfo
{
    char        chipName[MAX_CHIP_NAME_LEN];
                /**< Chip name (e.g. "MPC8360E", "MPC8360") */
    uint16_t    revMajor;
                /**< Major chip revision */
    uint16_t    revMinor;
                /**< Minor chip revision */
} t_ChipRevInfo;


/**************************************************************************//**
 @Description   Platform Events
*//***************************************************************************/
typedef enum e_PlatformEvent
{
    e_PLATFORM_EVENT_LINK_DOWN, /**< Link-up event */
    e_PLATFORM_EVENT_LINK_UP    /**< Link-down event */

} e_PlatformEvent;


/**************************************************************************//**
 @Description   Interrupt Source Types
*//***************************************************************************/
typedef enum e_InterruptType
{
    e_INTR_TYPE_GENERAL,
    e_INTR_TYPE_IRQ,
    e_INTR_TYPE_ERR,
    e_INTR_TYPE_GTIMERS_TIMER,
    e_INTR_TYPE_RTC_COUNT,
    e_INTR_TYPE_RTC_ALARM,
    e_INTR_TYPE_PCI_IRQ,
    e_INTR_TYPE_PCI_PME,
    e_INTR_TYPE_PCI_MSI,
    e_INTR_TYPE_ETSEC_TX,
    e_INTR_TYPE_ETSEC_RX,
    e_INTR_TYPE_ETSEC_ERROR,
    e_INTR_TYPE_RIO_PW,
    e_INTR_TYPE_RIO_DB_OUT,
    e_INTR_TYPE_RIO_DB_IN,
    e_INTR_TYPE_RIO_MSG_OUT,
    e_INTR_TYPE_RIO_MSG_IN,
    e_INTR_TYPE_QE_LOW,
    e_INTR_TYPE_QE_HIGH,
    e_INTR_TYPE_QE_IO_PORTS,
    e_INTR_TYPE_QE_IRAM_ERR,
    e_INTR_TYPE_QE_MURAM_ERR,
    e_INTR_TYPE_QE_RTT,
    e_INTR_TYPE_QE_SDMA,
    e_INTR_TYPE_QE_VT,
    e_INTR_TYPE_QE_EXT_REQ
} e_InterruptType;


/**************************************************************************//**
 @Description   Callback Function Prototype for Link Events
*//***************************************************************************/
typedef void (t_PlatformLinkEventsCb)(t_Handle          h_Controller,
                                      uint8_t           linkId,
                                      e_PlatformEvent   event);


/**************************************************************************//**
 @Description   Descriptor of Board Connections
*//***************************************************************************/
typedef struct t_BoardConnectorDesc
{
    struct
    {
        e_SysModule module;
        uint32_t    id;
    } source;

    struct
    {
        e_SysModule module;
        uint32_t    id;
    } dest;

} t_BoardConnectorDesc;


/**************************************************************************//**
 @Description   Platform Configuration Parameters Structure.

                This structure must be specifically defined by each platform.
*//***************************************************************************/
struct t_PlatformParam;


/**************************************************************************//**
 @Function      PLATFORM_Init

 @Description   Initializes the platform object according to the user's
                parameters and the board design and constraints.

 @Param[in]     p_PlatformParam - Pointer to platform configuration parameters.

 @Return        Handle to initialized platform object; NULL on failure.
*//***************************************************************************/
t_Handle PLATFORM_Init(struct t_PlatformParam *p_PlatformParam);

/**************************************************************************//**
 @Function      PLATFORM_Free

 @Description   Releases the platform object and all its allocated resources.

 @Param[in]     h_Platform - Platform object handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_Free(t_Handle h_Platform);

/**************************************************************************//**
 @Function      PLATFORM_GetChipRevInfo

 @Description   Retrieves the revision information of the device.

 @Param[in]     h_Platform      - Platform object handle.
 @Param[out]    p_ChipRevInfo   - Returns the revision information structure.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_GetChipRevInfo(t_Handle h_Platform, t_ChipRevInfo *p_ChipRevInfo);

/**************************************************************************//**
 @Function      PLATFORM_GetMemoryMappedModuleBase

 @Description   Get the base address of a memory-mapped object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Object type (usually a sub-module type)
 @Param[in]     id          - Object identifier

 @Return        Base address of a memory-mapped object.
*//***************************************************************************/
uint32_t PLATFORM_GetMemoryMappedModuleBase(t_Handle    h_Platform,
                                            e_SysModule module,
                                            uint32_t    id);

/**************************************************************************//**
 @Function      PLATFORM_GetCoreClk

 @Description   Gets the core clock frequency of the device.

 @Param[in]     h_Platform - Platform object handle.

 @Return        The core clock frequency (Hz).
*//***************************************************************************/
uint32_t PLATFORM_GetCoreClk(t_Handle h_Platform);

/**************************************************************************//**
 @Function      PLATFORM_GetSystemBusClk

 @Description   Gets the system bus frequency of the device.

 @Param[in]     h_Platform - Platform object handle.

 @Return        The system bus frequency (Hz).
*//***************************************************************************/
uint32_t PLATFORM_GetSystemBusClk(t_Handle h_Platform);


uint32_t PLATFORM_GetDdrClk(t_Handle h_Platform);

/**************************************************************************//**
 @Function      PLATFORM_GetLocalBusClk

 @Description   Gets the local bus frequency of the device.

 @Param[in]     h_Platform - Platform object handle.

 @Return        The local bus frequency (Hz).
*//***************************************************************************/
uint32_t PLATFORM_GetLocalBusClk(t_Handle h_Platform);

/**************************************************************************//**
 @Function      PLATFORM_GetControllerClk

 @Description   Gets the input clock frequency of a given controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Controller object type (usually a sub-module type)
 @Param[in]     id          - Controller object identifier

 @Return        The input clock frequency of a given controller object (Hz).
*//***************************************************************************/
uint32_t PLATFORM_GetControllerClk(t_Handle     h_Platform,
                                   e_SysModule  module,
                                   uint32_t     id);

/**************************************************************************//**
 @Function      PLATFORM_GetInterruptId

 @Description   Gets the platform's interrupt ID of a given controller object.

 @Param[in]     h_Platform      - Platform object handle.
 @Param[in]     module          - Controller object type (usually a sub-module type)
 @Param[in]     id              - Controller object identifier
 @Param[in]     intrType        - Interrupt type selection
 @Param[in]     intrRelatedId   - Specific ID that may be related to some
                                  interrupt types

 @Return        The interrupt ID of a given controller object;
                Returns -1 if not found or on any error.
*//***************************************************************************/
int PLATFORM_GetInterruptId(t_Handle        h_Platform,
                            e_SysModule     module,
                            uint32_t        id,
                            e_InterruptType intrType,
                            uint32_t        intrRelatedId);

/**************************************************************************//**
 @Function      PLATFORM_GetTransactionSourceName

 @Description   Returns the name of a given bus transaction source.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     transSrc    - The requested transaction source.

 @Return        The name (string) of the given transaction source.
*//***************************************************************************/
char * PLATFORM_GetTransactionSourceName(t_Handle h_Platform, e_TransSrc transSrc);

/**************************************************************************//**
 @Function      PLATFORM_GetLocalAccessWindowInfo

 @Description   Retrieves the base address and size of a local access window that
                is associated with a given controller object.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Controller object type (usually a sub-module type)
 @Param[in]     id          - Controller object identifier
 @Param[out]    p_BaseAddr  - Returns the base address of the local access window
 @Param[out]    p_Size      - Returns the size of the local access window

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_GetLocalAccessWindowInfo(t_Handle      h_Platform,
                                          e_SysModule   module,
                                          uint32_t      id,
                                          uint32_t      *p_BaseAddr,
                                          uint32_t      *p_Size);

/**************************************************************************//**
 @Function      PLATFORM_SetLed

 @Description   Sets a selected LED on the board to on/off state.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     led         - LED color selection
 @Param[in]     ledOn       - TRUE to turn the LED on; FALSE to turn it off.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_SetLed(t_Handle h_Platform, e_LedColor led, bool ledOn);

/**************************************************************************//**
 @Function      PLATFORM_ConnectExternalRequest

 @Description   Connects a specific module of the chip to external request signal.
                Usually used when a Ucode is  using the exteranl request as an interrupt signal
                for controlling the block, without a CPU help.

 @Param[in]     h_Platform  - Platform object handle.
 @Param[in]     module      - Module to be connected.
 @Param[in]     extReqNum   - External request number.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_ConnectExternalRequest(t_Handle h_Platform, e_SysModule module,uint8_t extReqNum);

/**************************************************************************//**
 @Function      PLATFORM_EnableConsole

 @Description   Enables the platform's console.

 @Param[in]     h_Platform - Platform object handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_EnableConsole(t_Handle h_Platform);

/**************************************************************************//**
 @Function      PLATFORM_DisableConsole

 @Description   Disables the platform's console.

 @Param[in]     h_Platform - Platform object handle.

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error PLATFORM_DisableConsole(t_Handle h_Platform);


/** @} */ /* end of platform_grp */

#endif /* __PLATFORM_EXT_H */
