/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**

 @File          sys_ipc_ext.h

 @Description   Definitions and functions for initializing and operating
                inter-process communication objects.

*//***************************************************************************/
#ifndef __SYS_IPC_EXT_H
#define __SYS_IPC_EXT_H

#include "std_ext.h"
#include "core_ext.h"


/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_ipc_grp     System Inter-Process Communication Interfaces

 @Description   System interfaces for Inter-Process Communication.

                The system IPC interfaces include:
                - Spinlocks
                - Message passing

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection    Spinlock Application Programming Interface

 @Description   Spinlock definitions and functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Spinlock structure.
*//***************************************************************************/
typedef struct t_Spinlock
{
    atomic_t    lock;   /**< Lock indicator */
} t_Spinlock;


/**************************************************************************//**
 @Function      SYS_InitSpinlock

 @Description   Initialize spinlock structure.

                The routine initializes the given spinlock structure as unlocked.

 @Param         p_Spinlock  -  (In) Pointer to spinlock structure to be initialize.
*//***************************************************************************/
static __inline__ void SYS_InitSpinlock(t_Spinlock *p_Spinlock)
{
    p_Spinlock->lock = 0;
}

/**************************************************************************//**
 @Function      SYS_LockSpinlock

 @Description   Lock the spinlock.

 @Param[in]     p_Spinlock - Pointer to spinlock structure.

 @Return        None.
*//***************************************************************************/
static __inline__ void SYS_LockSpinlock(t_Spinlock *p_Spinlock)
{
#ifdef NCSW_SMP
    /* The implementation below avoids looping on atomic reservations.
       First try to acquire the lock. If the lock is taken, loop until
       the lock seems to be free, and try again. */
    while (!CORE_TestAndSet(&(p_Spinlock->lock)))
    {
    	while (p_Spinlock->lock != 0) ;
	}

	/* Wait for all previous instructions to complete */
	CORE_InstructionSync();
#else  /* not NCSW_SMP */
    /* Single core version: do nothing */
    UNUSED(p_Spinlock);
#endif /* not NCSW_SMP */
}

/**************************************************************************//**
 @Function      SYS_UnlockSpinlock

 @Description   Unlock the spinlock.

 @Param[in]     p_Spinlock - Pointer to spinlock structure.

 @Return        None.
*//***************************************************************************/
static __inline__ void SYS_UnlockSpinlock(t_Spinlock *p_Spinlock)
{
#ifdef NCSW_SMP
	/* Memory barrier is required before spinlock is released */
    CORE_MemoryBarrier();
    p_Spinlock->lock = 0;
#else  /* not NCSW_SMP */
    UNUSED(p_Spinlock);
#endif /* not NCSW_SMP */
}

/**************************************************************************//**
 @Function      SYS_LockIntrSpinlock

 @Description   Lock the spinlock and disable interrupts on local processor.

 @Param[in]     p_Spinlock - Pointer to spinlock structure.

 @Return        A value that represents the interrupts state before the
                operation, and should be passed to the matching
                SYS_UnlockIntrSpinlock() call.
*//***************************************************************************/
static __inline__ uint32_t SYS_LockIntrSpinlock(t_Spinlock *p_Spinlock)
{
#ifdef NCSW_SMP
    register uint32_t intrFlags;

    /* The implementation below avoids looping on atomic reservations while
       interrupts are disabled. First try to acquire the lock. If the lock is
       taken, loop with interrupts enabled until the lock seems to be free,
       and try again. */
    while (TRUE)
    {
        intrFlags = XX_DisableAllIntr();

        if (CORE_TestAndSet(&(p_Spinlock->lock)))
            break; /* Exit with interrupts disabled */

        XX_RestoreAllIntr(intrFlags);

    	while (p_Spinlock->lock != 0) ;
	}

	/* Wait for all previous instructions to complete */
	CORE_InstructionSync();
	return intrFlags;
#else  /* not NCSW_SMP */
    /* Single core version: simply disable interrupts */
    UNUSED(p_Spinlock);

    return XX_DisableAllIntr();
#endif /* not NCSW_SMP */
}

/**************************************************************************//**
 @Function      SYS_UnlockIntrSpinlock

 @Description   Unlock the spinlock and restore interrupts on local processor.

 @Param[in]     p_Spinlock  - Pointer to spinlock structure.
 @Param[in]     intrFlags   - A value that represents the interrupts state to
                              restore, as returned by the matching call for
                              SYS_LockIntrSpinlock().

 @Return        None.
*//***************************************************************************/
static __inline__ void SYS_UnlockIntrSpinlock(t_Spinlock *p_Spinlock, uint32_t intrFlags)
{
#ifdef NCSW_SMP
	/* Memory barrier is required before spinlock is released */
    CORE_MemoryBarrier();
    p_Spinlock->lock = 0;
#else  /* not NCSW_SMP */
    UNUSED(p_Spinlock);
#endif /* not NCSW_SMP */

    /* Restore interrupts */
    XX_RestoreAllIntr(intrFlags);
}

/* @} */ /* end of spinlock collection */


/** @} */ /* end of sys_ipc_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_IPC_EXT_H */
