/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
#ifndef __SYS_MEM_MNG_EXT_H
#define __SYS_MEM_MNG_EXT_H

#include "std_ext.h"
#include "mem_mng_ext.h"



/**************************************************************************//**
 @Group         sys_grp     System Interfaces

 @Description   Bare-board system programming interfaces.

 @{
*//***************************************************************************/
/**************************************************************************//**
 @Group         sys_mem_grp     System Memory Management Service

 @Description   Bare-board system interface for memory management.

 @{
*//***************************************************************************/

#define SYS_DEFAULT_HEAP_PARTITION  0   /**< Partition ID for default heap */


/**************************************************************************//**
 @Function      SYS_MemAlloc

 @Description   Allocate a memory block from a specific partition and with
                specific attributes.

 @Param[in]     partitionId - Requested memory partition ID
 @Param[in]     size        - Requested memory size
 @Param[in]     alignment   - Requested memory alignment
 @Param[in]     info        - Allocation information string (for debug)
 @Param[in]     filename    - Caller file name (for debug)
 @Param[in]     line        - Caller line number (for debug)

 @Return        Pointer to allocated memory; NULL on failure.
*//***************************************************************************/
void * SYS_MemAlloc(int         partitionId,
                    uint32_t    size,
                    uint32_t    alignment,
                    char        *info,
                    char        *filename,
                    int         line);

/**************************************************************************//**
 @Function      SYS_MemFree

 @Description   Free a memory block that was previously allocated using the
                SYS_MemAlloc() routine.

 @Param[in]     p_Memory - Pointer to the memory block

 @Return        None.
*//***************************************************************************/
void SYS_MemFree(void *p_Memory);

/**************************************************************************//**
 @Function      SYS_GetAvailableMemPartition

 @Description   Returns an available memory partition ID.

                The returned ID may be used for registration of a new memory
                partition.

 @Return        An available memory partition ID.
*//***************************************************************************/
int SYS_GetAvailableMemPartition(void);

/**************************************************************************//**
 @Function      SYS_RegisterMemPartition

 @Description   Register a new memory partition to the system's memory manager.

                Note that if \c f_UserMalloc and \c f_UserFree are not NULL,
                the system will not manage the partition, but only record
                allocations and de-allocations for debug purposes (providing
                that \c enableDebug is set to TRUE).

 @Param[in]     partitionId     - Memory partition ID
 @Param[in]     baseAddress     - Base address of memory partition
 @Param[in]     size            - Size (in bytes) of memory partition
 @Param[in]     attributes      - Memory attributes mask (a combination of MEMORY_ATTR_x flags)
 @Param[in]     name            - Memory partition name (up to 32 bytes).
 @Param[in]     f_UserMalloc    - User's memory allocation routine, for bypassing the
                                  default memory manager; Set to NULL for default operation.
 @Param[in]     f_UserFree      - User's memory freeing routine, for bypassing the
                                  default memory manager; Set to NULL for default operation.
 @Param[in]     enableDebug     - TRUE to enable memory leaks debug; FALSE to disable.

 @Return        Pointer to allocated memory; NULL on failure.
*//***************************************************************************/
t_Error SYS_RegisterMemPartition(int        partitionId,
                                 uint32_t   baseAddress,
                                 uint32_t   size,
                                 uint32_t   attributes,
                                 char       name[],
                                 void *     (*f_UserMalloc)(uint32_t size, uint32_t alignment),
                                 void       (*f_UserFree)(void *p_Addr),
                                 bool       enableDebug);

/**************************************************************************//**
 @Function      SYS_UnregisterMemPartition

 @Description   Unregister a specific memory partition.

                Note that if \c f_UserMalloc and \c f_UserMalloc are not NULL,
                the system will not manage the partition, but only record
                allocations and de-allocations for debug purposes (providing
                that \c enableDebug is set to TRUE).

 @Param[in]     partitionId     - Memory partition ID

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error SYS_UnregisterMemPartition(int partitionId);

/**************************************************************************//**
 @Function      SYS_GetMemPartitionBase

 @Description   Retrieves the base address of a given memory partition.

 @Param[in]     partitionId - Memory partition ID

 @Return        Base address of the memory partition.
*//***************************************************************************/
uint32_t SYS_GetMemPartitionBase(int partitionId);

/**************************************************************************//**
 @Function      SYS_GetMemPartitionAttributes

 @Description   Retrieves the memory attributes mask of a given memory partition.

 @Param[in]     partitionId - Memory partition ID

 @Return        Memory attributes of the memory partition.
*//***************************************************************************/
uint32_t SYS_GetMemPartitionAttributes(int partitionId);

/**************************************************************************//**
 @Function      SYS_PrintMemPartitionDebugInfo

 @Description   Prints information and statistics of the given memory partition,
                and possibly reports known memory leaks.

 @Param[in]     partitionId - Memory partition ID
 @Param[in]     reportLeaks - TRUE to report memory leaks; FALSE otherwise.

 @Return        None.
*//***************************************************************************/
void SYS_PrintMemPartitionDebugInfo(int partitionId, bool reportLeaks);


/** @} */ /* end of sys_mem_grp */
/** @} */ /* end of sys_grp */


#endif /* __SYS_MEM_MNG_EXT_H */
