/******************************************************************************

 Copyright  1995-2009 Freescale Semiconductor Inc.

 All Rights Reserved



 **************************************************************************/
/**

 @File          timer_ext.h

 @Description   structures and definitions for initialize and operate the
                Timer module.
                This module

*//***************************************************************************/
#ifndef __TIMER_EXT_H
#define __TIMER_EXT_H


#include "std_ext.h"
#include "list_ext.h"


/**************************************************************************//**
 @Group         Timer - Timers module

 @Description   Timer module functions,definitions and enums.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Physical timer operations

                These operations must be supported by any physical timer that
                is operated by the TIMER module.
*//***************************************************************************/
typedef struct t_TimerOps
{
    t_Handle    h_Timer;
    uint8_t     id;
    e_TimerMode mode;

    t_Error (*f_Start)   (t_Handle      h_Timer,
                          uint8_t       id,
                          void          (*f_ExpiredCallback)(t_Handle h_App, uint8_t id),
                          t_Handle      h_App,
                          uint64_t      period,
                          e_TimerMode   mode);

    t_Error (*f_Stop)    (t_Handle      h_Timer,
                          uint8_t       id);

    t_Error (*f_SetTime) (t_Handle      h_Timer,
                          uint8_t       id,
                          uint64_t      period);

    t_Error (*f_GetTime) (t_Handle      h_Timer,
                          uint8_t       id,
                          uint64_t      *p_Period);
} t_TimerOps;


/*
 * Timers may be dynamically created and destroyed, and should be initialized
 * by a call to InitTimer() upon creation.
 *
 * The "data" field enables use of a common timeout function for several
 * timeouts. You can use this field to distinguish between the different
 * invocations.
 */
typedef struct t_Timer
{
    uint64_t         expires;
    t_Handle         data;
    int              sysTimer;
    void             (*f_TimerExpired)(t_Handle);
    t_TaskletHandle  h_TimerTasklet;

    t_List           list;
} t_Timer;


/**************************************************************************//**
 @Function      TIMER_InitModule

 @Description   Initialize the timer module.

                This routine ....

*//***************************************************************************/
t_Error TIMER_InitModule(t_TimerOps *p_TimerOps, bool timersQueueEnable);

/**************************************************************************//**
 @Function      TIMER_FreeModule

 @Description   Exit the timer module.

                This routine ....

*//***************************************************************************/
void TIMER_FreeModule(void);

/**************************************************************************//**
 @Function      TIMER_Init

 @Description   Initialize timer structure.

                The routine initializing the given timer structure with the
                given parameters.

 @Param         p_Timer        -  (In) Pointer to timer structure to be initialize.
 @Param         expires        -  (In) The interval time of the expiration
                                       for this timer (in milli-seconds).
 @Param         f_TimerExpired -  (In) User callback routine that will be called
                                       when this timer will be expired.
 @Param         data           -  (In) The argument for the callback routine.
 @Param         sysTimer       -  (In) flag that indicate a system timer or
                                       user timer.

 @Return        E_OK - for succsses, E_FAIL\E_BADPARM - otherwise.
*//***************************************************************************/
t_Error TIMER_Init(t_Timer  *p_Timer,
                   uint32_t expires,
                   void     (*f_TimerExpired)(t_Handle),
                   t_Handle data,
                   int      sysTimer);

/**************************************************************************//**
 @Function      TIMER_Free

 @Description   Free timer structure.

 @Param         p_Timer        -  (In) Pointer to timer structure to be initialize.

 @Return        E_OK - for succsses, E_FAIL\E_BADPARM - otherwise.
*//***************************************************************************/
t_Error TIMER_Free(t_Timer *p_Timer);

/**************************************************************************//**
 @Function      TIMER_GetExpirationTime

 @Description   Get the expiration time of the timer.

                The routine returns the expiration time of the given timer.

 @Param         p_Timer        -  (In) Pointer to timer structure.

 @Return        uint32_t: current time - p_Timer->expires.
*//***************************************************************************/
uint32_t TIMER_GetExpirationTime(t_Timer *p_Timer);

/**************************************************************************//**
 @Function      TIMER_Add

 @Description   Activate a timer.

                The routine adds the given timer to the timers queue.

 @Param         p_Timer        -  (In) Pointer to timer structure to be added.
*//***************************************************************************/
void TIMER_Add(t_Timer *p_Timer);

/**************************************************************************//**
 @Function      TIMER_Del

 @Description   Initialize timer structure.

                The routine initializing the given timer structure with the
                given parameters.

 @Param         p_Timer        -  (In) Pointer to timer structure to be deleted.
*//***************************************************************************/
void TIMER_Del(t_Timer *p_Timer);

/**************************************************************************//**
 @Function      TIMER_Mod

 @Description   Update timer exparition.

                TIMER_ModTimer is a more efficient way to update the expire
                field of an active timer (if the timer is inactive it will be
                activated) TIMER_ModTimer(a,b) is equivalent to
                TIMER_DelTimer(a); a->expires = b; TIMER_AddTimer(a).
                If the timer is known to be not pending (ie, in the handler),
                TIMER_ModTimer is less efficient than
                a->expires = b; TIMER_AddTimer(a).

 @Param         p_Timer        -  (In) Pointer to timer structure to be modified.
 @Param         expires        -  (In) The interval time of the expiration
                                       for this timer (In milli-seconds).
 @Return        E_OK - for succsses, E_FAIL\E_BADPARM - otherwise.
*//***************************************************************************/
t_Error TIMER_Mod(t_Timer *p_Timer, uint32_t expires);

/**************************************************************************//**
 @Function      TIMER_IsPending

 @Description   Check if a timer is pending.

                The routine checks whether the given timer is queued to the
                timers list or not.

 @Param         p_Timer        -  (In) Pointer to timer structure.

 @Return        1 - If this timer is pending, 0 - otherwise.
*//***************************************************************************/
int TIMER_IsPending(const t_Timer *p_Timer);

/**************************************************************************//**
 @Function      TIMER_CurrentTime

 @Description   Get the current time in the system.

                The routine read the glabal time value.

 @Return        Current time (In milli-seconds).
*//***************************************************************************/
uint32_t TIMER_CurrentTime(void);

/**************************************************************************//**
 @Function      TIMER_UDelay

 @Description   This is busy wait until the desired time has passed.

                The routine ...

 @Param         usecs       -  (In) The time to wait (In micro seconds).

 @Return        None.

 @Cautions      This routine is BUSY WAITTING!
*//***************************************************************************/
void TIMER_UDelay(uint32_t usecs);

/** @} */ /* end of group */


#endif /* __TIMER_EXT_H */
