/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
 @File          traffic_analyzer_ext.h

 @Description   Traffic Analyzer data structures and definitions
*//***************************************************************************/

#ifndef __TRAFFIC_ANALYZER_EXT_H
#define __TRAFFIC_ANALYZER_EXT_H

#include "std_ext.h"
#include "itg_common_ext.h"

/**************************************************************************//**
 @Group         itg_package_grp Internal Traffic Generator

 @Description   Internal Traffic Generator routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         traffic_analyzer_grp Traffic Analyzer Application Programming Interface

 @Description   Traffic Analyzer defines, structures and functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         traffic_analyzer_init_grp Traffic Analyzer Initialization Unit

 @Description   Traffic Analyzer Initialization Unit

 @{
*//***************************************************************************/

#define TA_MIN_LINE_RATE 1          /**< Minimal line rate in bps */
#define TA_MAX_LINE_RATE 100000000  /**< Maximal line rate in bps */

/**************************************************************************//**
 @Description   Modes of Comparison
*//***************************************************************************/
typedef enum {
    e_FRAME_LENGTH = 0,             /**< Compare only frame length                  */
    e_FRAME_LENGTH_HEADER,          /**< Compare frame length and header            */
    e_FRAME_LENGTH_HEADER_PAYLOAD   /**< Compare the whole frame including payload  */
} e_FrameComparisonMode;

/**************************************************************************//**
 @Description   Traffic Analyzer Queues Parameters Structure
*//***************************************************************************/
typedef struct {
    uint32_t                        queueId;                    /**< Queue identifier           */
    bool                            enableFlexibleRxMode;       /**< Flexible receive mode, TRUE to enable;
                                                                     allows frames reception out of order
                                                                     specified by h_FramesLst */
    t_Handle                        h_FramesLst;                /**< Frame list to be received by the queue */
} t_TrafficAnalyzerQueueParam;

/**************************************************************************//**
 @Description   Traffic Analyzer Port Parameters Structure
*//***************************************************************************/
typedef struct t_TrafficAnalyzerPortParam
{
    uint16_t                        portId;                 /**< Port identifier                        */
    t_Handle                        h_Dev;                  /**< Device handle                          */
    t_Error                         (*f_DevRx) (t_Handle h_Dev, uint32_t queueId);
                                                            /**< Receive callback function              */

    t_Error                         (*f_DevRxConf) (t_Handle h_Dev, uint32_t queueId, t_Handle h_Frame);
                                                            /**< Receive confirmation callback function */

    uint32_t                        numOfFrames;            /**< Number of frames                       */
    uint32_t                        numOfQueues;            /**< Number of receive queues               */
    uint32_t                        maxQueueIndex;          /**< Maximal possible receive queue
                                                                 index number.

                                                                 This index should be equal to
                                                                 numOfQueues when all receive queues
                                                                 from 0 to (numOfQueues - 1) are
                                                                 utilized. Otherwise if part of
                                                                 queues is skipped, this index
                                                                 specifies upper limit for range of
                                                                 queues to be used and should be set
                                                                 to (maximal queue index in the driver
                                                                 that will be used + 1) while
                                                                 numOfQueues specifies actual number
                                                                 of queues to be used. */

    t_TrafficAnalyzerQueueParam     *p_QueuesParam;         /**< Receive queue parameters for every queue,
                                                                 array of size numOfQueues              */

    uint32_t                        lineRate;               /**< Line rate in bps                       */

    uint32_t                        underSizeFrameValue;    /**< Undersize frame value                  */
    uint32_t                        overSizeFrameValue;     /**< Oversize frame value                   */

    t_Error                         (*f_UserInspection) (t_Handle h_Frame);
                                                            /**< Callback function for User Inspection
                                                                 on the received packets                */
} t_TrafficAnalyzerPortParam;

/**************************************************************************//**
 @Description   Traffic Analyzer Parameters Structure
*//***************************************************************************/
typedef struct {
    e_FrameComparisonMode           frameComparisonMode;        /**< Frame comparison mode      */
    uint16_t                        comparisonDepth;            /**< Comparison depth           */

    bool                            pollingMode;                /**< Polling or Interrupt Mode,
                                                                     TRUE for polling           */
    uint32_t                        pollingPeriod;              /**< Polling interval period    */

} t_TrafficAnalyzerParam;

/**************************************************************************//**
 @Function      TA_Init

 @Description   Creates and initializes traffic analyzer object.

 @Param[in]     p_TrafficAnalyzerParam -  Pointer to the TA parameters.

 @Return        Traffic Analyzer handle on success, NULL on failure

 @Cautions      None
*//***************************************************************************/
t_Handle TA_Init(t_TrafficAnalyzerParam *p_TrafficAnalyzerParam);

/**************************************************************************//**
 @Function      TA_Free

 @Description   Frees TA and TA Port objects.

 @Param[in]     h_Ta -  Handle to the TA.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
t_Error TA_Free(t_Handle h_Ta);

/**************************************************************************//**
 @Function      TA_PORT_Config

 @Description   Creates and intializes the TA port object.

                Frames list for every port's queue should be created in advance
                through PG List API functions and is provided to this function
                via h_FramesLst field of t_TrafficAnalyzerQueueParam struct.


 @Param[in]     h_Ta                       -  Handle to TA object
 @Param[in]     p_TrafficAnalyzerPortParam -  Pointer to the traffic analyzer port parameters

 @Return        Handle to the Traffic Analyzer Port on success, NULL on failure

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
t_Handle TA_PORT_Config(t_Handle  h_Ta, t_TrafficAnalyzerPortParam  *p_TrafficAnalyzerPortParam);

/**************************************************************************//**
 @Function      TA_PORT_Free.

 @Description   Frees TA Port.

 @Param[in]     h_TaPort    -  Handle to the Port.

 @Return        None.

 @Cautions      TA Port must have been previously created using TA_PORT_Config().
*//***************************************************************************/
void TA_PORT_Free(t_Handle h_TaPort);

/** @} */ /* end of traffic_analyzer_init_grp */

/**************************************************************************//**
 @Group         traffic_analyzer_control_grp Traffic Analyzer Control Unit

 @Description   Traffic Analyzer Control Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Traffic Analyzer Statistics Structure
*//***************************************************************************/
typedef struct t_TrafficAnalyzerRxStatistics
{
    uint32_t                    rxFrames;            /**< Rx frames counter                 */
    uint32_t                    rxErrors;            /**< Rx errors counter                 */
    uint32_t                    rxOctets;            /**< Rx octets counter                 */

    uint32_t                    seqPayloadCnt;       /**< Rx sequential payload counter     */
    uint32_t                    fixedPayloadCnt;     /**< Rx fixed payload counter          */
    uint32_t                    patternTypeOneCnt;   /**< Fixed pattern (0x5A5A) frames     */
    uint32_t                    patternTypeTwoCnt;   /**< Fixed pattern (0x55AA) frames     */
    uint32_t                    patternTypeThreeCnt; /**< Fixed pattern (0x5555) frames     */
    uint32_t                    patternTypeFourCnt;  /**< Fixed pattern (0x00FF) frames     */
    uint32_t                    patternTypeFiveCnt;  /**< Fixed pattern (0xF0F0) frames     */
    uint32_t                    patternTypeSixCnt;   /**< Fixed pattern (0x0055AAFF) frames */

    uint32_t                    rxUndersizedFrames;  /**< Undersized frame counter          */
    uint32_t                    rxOversizedFrames;   /**< Oversized frame counter           */
    uint32_t                    rxJumboFrames;       /**< Jumbo frame counter               */
} t_TrafficAnalyzerRxStatistics;

/**************************************************************************//**
 @Function      TA_Run

 @Description   Runs traffic analyzer.

                This function performs the following operations:
                - Activates all configured ports and waits to receive
                traffic across all active TA ports.
                - When operating in polling mode, it also initializes
                polling timer.

                This function invocation should be completed by
                TA_WaitOnAllPorts call to start received frames
                comparison/analysis.

                When issued after previous TA_Stop invocation it
                re-initializes/zeroes Rx statistics and prepares TA
                to expect receiving frames from the beginning of
                original Rx list. TA_WaitOnAllPorts function should
                be called to continue TA operation.

 @Param[in]     h_Ta - Handle to the TA.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
t_Error TA_Run(t_Handle h_Ta);

/**************************************************************************//**
 @Function      TA_Stop

 @Description   Stops the TA Reception process across all active ports.

                This function halts current TA_WaitOnAllPorts operation
                and return control to the user. If TA_Run if issued at
                a later time then TA will re-initialize its state and zero
                TA statistics. When TA_WaitOnAllPorts will be issued after
                that, TA will expect to receive frame based on order of its
                original Rx list.

 @Param[in]     h_Ta - Handle to the TA.

 @Return        None.

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
void TA_Stop(t_Handle h_Ta);

/**************************************************************************//**
 @Function      TA_WaitOnAllPorts

 @Description   Confirms the reception/comparison of all transmitted frames.

                This function waits for reception of all expected frames and
                performs their comparison. If not all frames are received
                during optimal period of time this function exists upon
                timeout. Optimal time/timeout period is calculated based
                on lineRate and numOfFrames parameters of t_TrafficAnalyzerPortParam
                structure.

                This function operation can be stopped through TA_Stop
                invocation.

 @Param[in]     h_Ta - Handle to the TA

 @Return        TRUE on success, FALSE on failure

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
bool TA_WaitOnAllPorts(t_Handle h_Ta);

/**************************************************************************//**
 @Function      TA_GetStatistics

 @Description   Gets the Traffic Analyzer Statistics.

 @Param[in]     h_Ta             - Handle to the TA.
 @Param[out]    p_AnalyzerStats  - Pointer to the TA's receive statistics.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
t_Error TA_GetStatistics( t_Handle h_Ta, t_TrafficAnalyzerRxStatistics *p_AnalyzerStats);

/**************************************************************************//**
 @Function      TA_PrintRxStatistics

 @Description   Prints receive statistics.

 @Param[in]     h_Ta - Handle to the TA.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TA must have been previously created using TA_Init().
*//***************************************************************************/
t_Error TA_PrintRxStatistics(t_Handle h_Ta);

/**************************************************************************//**
 @Function      TA_GetPortHandle

 @Description   Gets the handle of portId's port.

 @Param[in]     h_Ta             - Handle to the TA.
 @Param[in]     portId           - Port identifier.

 @Return        Handle to the TA Port.

 @Cautions      None.
*//***************************************************************************/
t_Handle TA_GetPortHandle(t_Handle h_Ta, uint16_t portId);

/**************************************************************************//**
 @Function      TA_PORT_Enable

 @Description   Enables the reception on the specified TA Port.

 @Param[in]     h_TaPort - Handle to the Port.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
void TA_PORT_Enable(t_Handle h_TaPort);

/**************************************************************************//**
 @Function      TA_PORT_Disable

 @Description   Disables the reception on the specified TA Port.

 @Param[in]     h_TaPort - Handle to the Port.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
void TA_PORT_Disable(t_Handle h_TaPort);

/**************************************************************************//**
 @Function      TA_PORT_GetStatistics

 @Description   Gets the Statistics on specified TA Port.

 @Param[in]     h_TrafficPort         - Handle to the TA Port.
 @Param[in]     p_AnalyzerPortStats   - Pointer to TA Port statistics.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
t_Error TA_PORT_GetStatistics(t_Handle h_TrafficPort, t_TrafficAnalyzerRxStatistics *p_AnalyzerPortStats);

/** @} */ /* end of traffic_analyzer_control_grp */

/**************************************************************************//**
 @Group         traffic_analyzer_runtime_grp Traffic Analyzer Runtime Unit

 @Description   Traffic Analyzer Runtime Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      TA_PORT_ReceiveFrame

 @Description   Receive function for Traffic Analyzer port.

                This function should be issued upon frame reception by driver
                to forward received frame to TA for comparison.

 @Param[in]     h_Port  - Handle to the TA Port.
 @Param[in]     queueId - Queue identifier.
 @Param[in]     h_Frame - Handle to the frame.

 @Return        Rx callback response (continue/pause).

 @Cautions      TA Port must have been previously created using TA_PORT_Config().
*//***************************************************************************/
e_RxStoreResponse TA_PORT_ReceiveFrame(t_Handle h_Port, uint32_t  queueId, t_Handle h_Frame);

/** @} */ /* end of traffic_analyzer_runtime_grp */

/** @} */ /* end of traffic_analyzer_grp */

/** @} */ /* end of itg_package_grp */

#endif /* __TRAFFIC_ANALYZER_EXT_H */

