/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/**************************************************************************//**
@File          traffic_generator_ext.h

@Description   Traffic Generator data structures and definitions
*//***************************************************************************/

#ifndef __TRAFFIC_GENERATOR_EXT_H
#define __TRAFFIC_GENERATOR_EXT_H

#include "std_ext.h"
#include "itg_common_ext.h"

/**************************************************************************//**
 @Group         itg_package_grp Internal Traffic Generator

 @Description   Internal Traffic Generator routines.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         traffic_generator_grp Traffic Generator Application Programming Interface

 @Description   Traffic Generator defines, structures and functions.

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Group         traffic_generator_init_grp Traffic Generator Initialization Unit

 @Description   Traffic Generator Initialization Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Collection   TG Defines

               General Traffic Generator defines

 @{
*//***************************************************************************/
#define TG_RETRANSMIT_THRESHOLD 5        /**< Stop transmission after specified number of failures */

#define TG_MIN_LINE_RATE 1               /**< Minimal line rate in bps */
#define TG_MAX_LINE_RATE 100000000       /**< Maximal line rate in bps */

/** @} */

/**************************************************************************//**
 @Description   Enumeration for Transmit Operation Mode
*//***************************************************************************/
typedef enum
{
    e_TG_MODE_BATCH         = 0 /**< Batch transmission mode - the number of
                                     frames specified during TG Ports configuration
                                     will be transmitted over all active/configured
                                     TG ports; used as parameter to TG_Run routine */
  , e_TG_MODE_CONTINUOUS        /**< Continuous transmission mode - frames are
                                     transmitted continuously over all active
                                     TG ports; used as parameter to TG_Run routine */
  , e_TG_MODE_SIMPLE            /**< Simple transmission mode - sends specified
                                     number of frames on specific queue of
                                     specific port; for simple mode use TG_RunSimple
                                     routine */
} e_TgOperationMode;

/**************************************************************************//**
 @Description   TG Scheduling Mechanism
*//***************************************************************************/
typedef enum
{
    e_PORTSLICE_RR_SCHED    = 0 /**< Port Slicing in Round Robin - Transmit
                                     a frame on all queues of the first
                                     port then all queues of the second port etc.   */
  , e_QUEUESLICE_RR_SCHED       /**< Queue Slicing in Round Robin - Transmit
                                     frames on first queue of every
                                     port, after that second queue of every
                                     port and so on.                                */
} e_TrafficSchedMechanism;

/**************************************************************************//**
 @Description   Traffic Generator Queue Parameters Structure
*//***************************************************************************/
typedef struct t_TrafficGenQueueParam
{
    uint32_t                    queueId;            /**< Queue identifier */
    bool                        shuffleTxFrames;    /**< Flexible transmission mode, TRUE to enable;
                                                         In shuffle mode frames of h_FramesLst are sent
                                                         in random order  */
    t_Handle                    h_FramesLst;        /**< Frame list to be transmitted on every queue */
} t_TrafficGenQueueParam;

/**************************************************************************//**
 @Description   Traffic Generator Port Parameters Structure
*//***************************************************************************/
typedef struct t_TrafficGenPortParam
{
    uint16_t                        portId;             /**< Port identifier                */
    t_Handle                        h_Dev;              /**< Device handle                  */
    t_Error                         (*f_DevTx) (t_Handle h_Dev, uint32_t queueId, t_Handle h_Frame);
                                                        /**< Transmit callback function     */

    void                            (*f_DevTxConf) (t_Handle h_Dev, uint32_t queueId);
                                                        /**< Confirmation callback function */

    uint32_t                        errorsThreshold;    /**< Error threshold                */
    uint32_t                        numOfFrames;        /**< Number of frames               */
    uint32_t                        lineRate;           /**< Line rate in bps               */
    uint32_t                        burstSize;          /**< Burst size in bytes            */
    uint32_t                        numOfQueues;        /**< Number of transmit queues      */
    uint32_t                        maxQueueIndex;      /**< Maximal possible transmit queue
                                                             index number.

                                                             This index should be equal to
                                                             numOfQueues when all transmit queues
                                                             from 0 to (numOfQueues - 1) are
                                                             utilized. Otherwise if part of
                                                             queues is skipped, this index
                                                             specifies upper limit for range of
                                                             queues to be used and should be set
                                                             to (maximal queue index in the driver
                                                             that will be used + 1) while
                                                             numOfQueues specifies actual number
                                                             of queues to be used.          */

    t_TrafficGenQueueParam          *p_QueuesParam;     /**< Transmit queue parameters for every queue,
                                                             array of size numOfQueues      */

    uint32_t                        underSizeFrameValue; /**< Undersize frame value - frame shorter than
                                                              this value will be counted as undersized in
                                                              TG and TG port statistics     */
    uint32_t                        overSizeFrameValue;  /**< Oversize frame value - frame longer than
                                                              this value will be counted as oversized in
                                                              TG and TG port statistics      */
    bool                            noConfirmationMode;  /**< No Tx confirmation is required when TRUE,
                                                              frame object will be deleted upon successful transmission */
} t_TrafficGenPortParam;

/**************************************************************************//**
 @Description   Traffic Generator Parameters Structure
*//***************************************************************************/
typedef struct t_TrafficGenParam
{
    e_TrafficSchedMechanism         tgSchedMechanism;     /**< Scheduling mechanism     */
    uint32_t                        queueIterator;        /**< Number of frames transmitted per queue
                                                               prior to going on to next queue; next
                                                               queue is based on tgSchedMechanism
                                                               definition */
} t_TrafficGenParam;

/**************************************************************************//**
 @Function      TG_Init

 @Description   Creates and initializes Traffic Generator object.

 @Param[in]     p_TrafficGenParam - Pointer to the TG parameters.

 @Return        Traffic Generator handle on success, NULL on failure

 @Cautions      None
*//***************************************************************************/
t_Handle TG_Init(t_TrafficGenParam *p_TrafficGenParam);

/**************************************************************************//**
 @Function      TG_Free

 @Description   Frees TG and TG Port objects.

 @Param[in]     h_Tg - Handle to the TG.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
t_Error TG_Free(t_Handle h_Tg);

/**************************************************************************//**
 @Function      TG_PORT_Config

 @Description   Creates and intializes TG Port object.

                Frames list for every port's queue should be created in advance
                through PG List API functions and is provided to this function
                via h_FramesLst field of t_TrafficGenQueuesParam struct.

 @Param[in]     h_Tg                  - Handle to the TG.
 @Param[in]     p_TrafficGenPortParam - Handle to the TG port parameters.

 @Return        Handle to the Traffic Generator Port on success, NULL on failure

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
t_Handle TG_PORT_Config(t_Handle h_Tg, t_TrafficGenPortParam *p_TrafficGenPortParam);

/**************************************************************************//**
 @Function      TG_PORT_SetIpMtu

 @Description   Sets the TG Port IP MTU.

 @Param[in]     h_TgPort    - Handle to the TG Port to set the IP MTU on.
 @Param[in]     mtu         - New value of the MTU.

 @Return        E_OK on success, an error code on failure

 @Cautions      IP MTU can be set only ONCE for a specific port.
*//***************************************************************************/
t_Error TG_PORT_SetIpMtu(t_Handle h_TgPort, uint32_t mtu);

/**************************************************************************//**
 @Function      TG_PORT_Free

 @Description   Frees TG Port.

 @Param[in]     h_TgPort - Handle to the Port.

 @Return        None.

 @Cautions      TG Port must have been previously created using TG_PORT_Config().
*//***************************************************************************/
void TG_PORT_Free(t_Handle h_TgPort);

/** @} */ /* end of traffic_generator_init_grp */

/**************************************************************************//**
 @Group         traffic_generator_control_grp Traffic Generator Control Unit

 @Description   Traffic Generator Control Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Description   Traffic Generator Statistics Structure
*//***************************************************************************/
typedef struct t_TrafficGenTxStatistics
{
    uint32_t                    txEthernetFrames;       /**< Ethernet frame counter                 */
    uint32_t                    txEthernet802_3Frames;  /**< Ethernet 802.3 Frames              */
    uint32_t                    txPTPEthernetFrames;    /**< PTP Ethernet frame counter             */
    uint32_t                    txVLANFrames;           /**< VLAN frame counter                     */
    uint32_t                    txVTagged802_3Frames;   /**< Vtagged 802_3 frame counter            */
    uint32_t                    txMPLSFrames;           /**< MPLS frame counter                     */
    uint32_t                    txL2TPv2Frames;         /**< L2TPv2 frame counter                   */
    uint32_t                    txL2TPv3Frames;         /**< L2TPv3 frame counter                   */
    uint32_t                    txIPv4Frames;           /**< IPv4 frame counter                     */
    uint32_t                    txIPv6Frames;           /**< IPv6 frame counter                     */
    uint32_t                    txTCPFrames;            /**< TCP frame counter                  */
    uint32_t                    txUDPFrames ;           /**< UDP frame counter                  */
    uint32_t                    txSCTPFrames;           /**< SCTP frame counter                     */
    uint32_t                    txSCTPDataFrames;       /**< SCTP Data frame counter                */

    uint32_t                    txPTPV1GeneralUDPFrames;/**< PTP UDP General V1 frame counter       */
    uint32_t                    txPTPV2GeneralUDPFrames;/**< PTP UDP General V2 frame counter       */

    uint32_t                    txPTPV1EventUDPFrames;  /**< PTP UDP Event V1 frame counter         */
    uint32_t                    txPTPV2EventUDPFrames;  /**< PTP UDP Event V2 frame counter         */
    uint32_t                    txPTPV1MessageFrames;   /**< PTP V1 Message frame counter       */
    uint32_t                    txPTPV2MessageFrames;   /**< PTP V2 Message frame counter           */
    uint32_t                    txCompressedFrames;     /**< IP header compressed frame counter     */
    uint32_t                    txPPPoEFrames;          /**< PPPoE frame counter                    */
    uint32_t                    txPPPFrames;            /**< PPP frame counter                  */
    uint32_t                    txPPPMLMCFrames;        /**< PPP MLMC frame counter                 */
    uint32_t                    txHDLCFrames;           /**< HDLC frame counter                     */
    uint32_t                    txLLCFrames;            /**< LLC frame counter                  */
    uint32_t                    txNLPFrames;            /**< NLP frame counter                  */
    uint32_t                    txSNAPFrames;           /**< SNAP frame counter                     */
    uint32_t                    txAAL0Cells;            /**< AAL0 cell counter                  */
    uint32_t                    txATMRawFrames;         /**< Raw data frame counter             */
    uint32_t                    txRTPFrames;            /**< RTP frame counter                  */
    uint32_t                    txATMPwDemuxFrames;     /**< ATM PW frame counter               */
    uint32_t                    txATMPwEncapFrames;     /**< ATM PW frame counter               */
    uint32_t                    txSAToPPwDemuxFrames;   /**< SAToP PW frame counter                 */
    uint32_t                    txSAToPPwEncapFrames;   /**< SAToP PW frame counter                 */
    uint32_t                    txTunneledIPv4Frames;   /**< TunneledIPv4 frame counter             */
    uint32_t                    txTunneledIPv6Frames;   /**< TunneledIPv6 frame counter             */
    uint32_t                    txPaddingFrames;        /**< Padding Counter                        */

    uint32_t                    txIPChecksumErrors;     /**< IP checksum error counter          */
    uint32_t                    txUDPChecksumErrors;    /**< UDP checksum error counter             */
    uint32_t                    txTCPChecksumErrors;    /**< TCP checksum error counter             */
    uint32_t                    txIPv4VersionErrors;    /**< IPv4 version error counter             */
    uint32_t                    txIPv6VersionErrors;    /**< IPv6 version error counter             */
    uint32_t                    txAAL0HECErrors;        /**< AAL0 HEC error counter                 */
    uint32_t                    txRTPVersionErrors;     /**< RTP version error counter          */
    uint32_t                    txFCSErrors;            /**< FCS error counter                  */

    uint32_t                    txUndersizedFrames;     /**< Undersized frame counter           */
    uint32_t                    txOversizedFrames;      /**< Oversized frame counter                */
    uint32_t                    txJumboFrames;          /**< Jumbo frame counter                    */

    uint32_t                    txOctets;               /**< Transmit octet counter                 */
    uint32_t                    txFrames;               /**< Transmit frame counter                 */
    uint32_t                    txErrors;               /**< Transmit error counter                 */
    uint32_t                    txBandwidth;            /**< Transmit Bandwidth                     */
} t_TrafficGenTxStatistics;

/**************************************************************************//**
 @Function      TG_Run

 @Description   Triggers the transmission based on the transmission mode.

                When this routine follows TG_Stop() call it re-initializes/
                zeroes all previous Tx statistics and starts sending frames
                from the beginning of the original Tx list.

 @Param[in]     h_Tg  - Handle to the TG.
 @Param[in]     mode  - Transmission mode (Batch / Continuous).

 @Return        TRUE on Success, FALSE on Failure

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
bool TG_Run(t_Handle h_Tg, e_TgOperationMode mode);

/**************************************************************************//**
 @Function      TG_RunSimple

 @Description   Triggers frame transmission through the specified Port ID
                and Queue ID.

 @Param[in]     h_Tg        - Handle to the TG.
 @Param[in]     portId      - Port identifier.
 @Param[in]     queueId     - Queue identifier.
 @Param[in]     numOfFrames - Number of frames to transmit.

 @Return        TRUE on Success, FALSE on Failure

 @Cautions      TG must have been previously created using TG_Init().
                TG Port must have been previously created using TG_PORT_Config().
*//***************************************************************************/
bool TG_RunSimple(t_Handle h_Tg, uint16_t portId, uint32_t queueId, uint32_t numOfFrames);

/**************************************************************************//**
 @Function      TG_Stop

 @Description   Gracefully stops/suspends the transmission across all active
                TG ports.

                This function suspends transmission operation both in batch
                and continuous modes. All already transmitted frames are
                confirmed. Transmission can be resumed from the same place
                by invoking TG_Start() function or re-started from the
                beginning using TG_Run() function. In the latter case all
                statistics are re-initialized/zeroed during TG_Run()
                invocation and frames are transmitted from the beginning
                of the original list.

 @Param[in]     h_Tg - Handle to the TG.

 @Return        None.

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
void TG_Stop(t_Handle h_Tg);

/**************************************************************************//**
 @Function      TG_Resume

 @Description   Resumes the frame transmission across all active ports of TG.

                This function resumes transmission operation suspended by
                TG_Stop() function. Transmission starts from the same place
                it has been stopped by TG_Stop(). Can be used both in batch
                and continuous mode.

 @Param[in]     h_Tg - Handle to the TG.

 @Return        TRUE on Success, FALSE on Failure.

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
bool TG_Resume(t_Handle h_Tg);

/**************************************************************************//**
 @Function      TG_ConfirmAllPorts

 @Description   Confirms the frame transmission across all active ports.

                This function usage is optional. It is invoked automatically
                by TG at the end of transmission operation.

 @Param[in]     h_Tg - Handle to the TG.

 @Return        TRUE on Success, FALSE on Failure.

 @Cautions      TG must have been previously created using TG_Init().
*//***************************************************************************/
bool TG_ConfirmAllPorts(t_Handle h_Tg);

/**************************************************************************//**
 @Function      TG_GetStatistics

 @Description   Gets the Traffic Generator Statistics.

 @Param[in]     h_Tg          - Handle to the TG.
 @Param[out]    p_GenStats    - Pointer to TG's transmit statistics.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
 t_Error TG_GetStatistics(t_Handle h_Tg, t_TrafficGenTxStatistics *p_GenStats);

/**************************************************************************//**
 @Function      TG_PrintTxStatistics

 @Description   Prints transmit statistics.

 @Param[in]     h_Tg - Handle to the TG.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TG object must have been previously created using TG_Init().
*//***************************************************************************/
t_Error TG_PrintTxStatistics(t_Handle h_Tg);

/**************************************************************************//**
 @Function      TG_GetPortHandle

 @Description   Gets the handle of portId's port.

 @Param[in]     h_Tg             - Handle to the TG.
 @Param[in]     portId           - Port identifier.

 @Return        Handle to the Port.

 @Cautions      None.
*//***************************************************************************/
t_Handle TG_GetPortHandle(t_Handle h_Tg, uint16_t portId);

/**************************************************************************//**
 @Function      TG_PORT_Enable

 @Description   Enables the transmission on the specified TG Port.

 @Param[in]     h_TgPort - Handle to the Port.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
void TG_PORT_Enable(t_Handle h_TgPort);

/**************************************************************************//**
 @Function      TG_PORT_Disable

 @Description   Disables the transmission on the specified TG Port.

 @Param[in]     h_TgPort - Handle to the Port.

 @Return        None.

 @Cautions      None.
*//***************************************************************************/
void TG_PORT_Disable(t_Handle h_TgPort);

/**************************************************************************//**
 @Function      TG_PORT_GetStatistics

 @Description   Gets statistics on specified TG Port.

 @Param[in]     h_TrafficPort   - Handle to the TG Port.
 @Param[out]    p_GenStats      - Pointer to TG Port statistics.

 @Return        E_OK on success, Error code otherwise

 @Cautions      None.
*//***************************************************************************/
t_Error TG_PORT_GetStatistics(t_Handle h_TrafficPort, t_TrafficGenTxStatistics *p_GenStats);

/** @} */ /* end of traffic_generator_control_grp */

/**************************************************************************//**
 @Group         traffic_generator_runtime_grp Traffic Generator Runtime Unit

 @Description   Traffic Generator Runtime Unit

 @{
*//***************************************************************************/

/**************************************************************************//**
 @Function      TG_PORT_ConfirmFrame

 @Description   Confirms transmission on specified TG Port.

                This function should be issued during frame confirmation in the driver.
                It enqueues the confirmed frame to the Ready Queue so it can be used by
                TG for further transmission.

 @Param[in]     h_Port  - Handle to the TG Port.
 @Param[in]     queueId - Queue identifier.
 @Param[in]     h_Frame - Handle to the frame to be confirmed.

 @Return        E_OK on success, Error code otherwise

 @Cautions      TG Port must have been previously created using TG_PORT_Config().
*//***************************************************************************/
t_Error TG_PORT_ConfirmFrame(t_Handle h_Port, uint32_t queueId, t_Handle h_Frame);

/** @} */ /* end of traffic_generator_runtime_grp */

/** @} */ /* end of traffic_generator_grp */

/** @} */ /* end of itg_package_grp */

#endif /* __TRAFFIC_GENERATOR_EXT_H */


