/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>

static unsigned _resolve_synonyms(unsigned hwi_off);

/*******************************************************************************
* hwi_find
* 
* if the <unit> instance of <name> has been added to the hwinfo section
* of the syspage and belongs to parent <hwi_off> then return its hwi offset.
* 
* This function will allow the caller to find a specific item instance owned by
* a specific parent, resolving (all) synonym names (aliases) to a real device
* or bus.
* 
* If <hwi_off> == HWI_NULL_OFF, indicating no parent specified, then a system
* wide search will be done and the results returned regardless of the owning
* parent.
* If <unit> == -1, then the caller is indicating that they have no specific
* instance they are searching for and so the first occurrence of <name> will
* be returned. Setting <unit> == -1 is most useful when searching for a device
* on a specific bus (ie. <hwi_off> != HWI_NULL_OFF). 
* 
* Note that the search can be directed from a specific item <hwi_off>. Therefore,
* if a certain device is expected to be found attached to a certain bus, you can
* set <hwi_off> to the results of hwi_find_bus().
* For example, if you have multiple instances of the same device attached to
* 2 different buses, the <hwi_off> parameter allows you to find the instance
* connected to one of the buses only
* 
* Returns: the hwinfo section offset or HWI_NULL_OFF if the bus/device instance
* 			does not exist
* 
*/
unsigned hwi_find(const char *name, int unit, unsigned hwi_off)
{
	unsigned item_cnt = 0;
	unsigned parent = hwi_off;

	while ((hwi_off = hwi_find_item(hwi_off, name, NULL)) != HWI_NULL_OFF)
	{
		if ((unit < 0) || (item_cnt == unit))
		{
			hwi_off = _resolve_synonyms(hwi_off);
			break;
		}
		++item_cnt;
	}
	/* if a parent was specified, the found item must be owned by the parent */
	if ((hwi_off != HWI_NULL_OFF) && (parent != HWI_NULL_OFF))
	{
		hwi_tag *tag = hwi_off2tag(hwi_off);
		if (parent != tag->item.owner) hwi_off = HWI_NULL_OFF;
	}
	return hwi_off;
}

/*******************************************************************************
 * _resolve_synonyms
 * 
 * recursively resolve a synonym chain until we hit a 'real' device
*/
static unsigned _resolve_synonyms(unsigned hwi_off)
{
	unsigned count;

	count = 0;
	for( ;; ) {
		hwi_tag *tag;
		if (hwi_off == HWI_NULL_OFF) break;
		tag = hwi_off2tag(hwi_off);
		if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_synonym) != 0) break;
		/* if startup configures more than MAX_SYNONYM_COUNT synonyms to synonyms, fail */
		if (++count > MAX_SYNONYM_COUNT) {
		    return HWI_NULL_OFF;
		}
		hwi_off = tag->synonym.real;
	}
	return hwi_off;
}

__SRCVERSION("$URL$ $Rev$");
