/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */

#include <inttypes.h>
#include <string.h>
#include <stdarg.h>
#include <alloca.h>
#include <hw/sysinfo.h>
#include <sys/hwinfo.h>
#include <drvr/hwinfo.h>
#include <errno.h>


static int hwiattr_get_specific_bus(unsigned hwi_off, hwiattr_t *hwiattr);
static int hwiattr_get_specific_device(unsigned hwi_off, hwiattr_t *hwiattr);


/*******************************************************************************
 * hwiattr_get
 * 
 * this function will retrieve the attributes for the bus or device identified
 * by <hwi_off> and fill in the <hwiattr> structure.
 * 
 * Since this is a generic get function, the type of the bus or device must
 * be determined from <hwi_off>.
 * It is always faster to call the specific hwiattr_get_xxx() function directly
 * as all this routine (and the accompanying support functions) does is try to
 * figure out which specific routine to call.
 *  
 * Returns: the result from the specific hwiattr_get_*() routine or EINVAL
 * 
 * Implementation note
 * 
 * For speed, 2 variables will be used, 'bus_string_hwi_off' and
 * 'device_string_hwi_off' which are the indexes into the strings section of the
 * syspage for the HWI_TAG_NAME_bus and HWI_TAG_NAME_device tag strings
 * respectively. Once we have found these we can use them to quickly determine
 * whether we are dealing with a bus or a device
*/
int hwiattr_get(unsigned hwi_off, hwiattr_t *hwiattr)
{
	static unsigned bus_string_hwi_off = HWI_NULL_OFF;
	static unsigned device_string_hwi_off = HWI_NULL_OFF;

	int r = EINVAL;

	if ((hwi_off != HWI_NULL_OFF) && (hwiattr != NULL))
	{
		hwi_tag *tag = hwi_off2tag(hwi_off);

		if (tag->prefix.name == bus_string_hwi_off)
		{
			r = hwiattr_get_specific_bus(hwi_off, hwiattr);
		}
		else if (tag->prefix.name == device_string_hwi_off)
		{
			r = hwiattr_get_specific_device(hwi_off, hwiattr);
		}
		/* need to lookup so cache the hwi offsets for buses and devices for future */
		else if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_bus) == 0)
		{
			/* this path will only be run once (for known buses) */
			bus_string_hwi_off = tag->prefix.name;
			r = hwiattr_get_specific_bus(hwi_off, hwiattr);
		}
		else if (strcmp(__hwi_find_string(tag->prefix.name), HWI_TAG_NAME_device) == 0)
		{
			/* this path will only be run once (for known devices) */
			device_string_hwi_off = tag->prefix.name;
			r = hwiattr_get_specific_device(hwi_off, hwiattr);
		}
	}
	return r;
}

/*******************************************************************************
 * hwiattr_get_specific_bus
 * 
 * this function will determine the type of bus identified by <hwi_off> and call
 * the specific routine to fill in <hwiattr>
 *   
 * Returns: the result from the specific hwiattr_get_*() routine or EINVAL
 * 
 * Implementation note
 * 
 * For speed, we keep variables of the offset into the strings section of the
 * syspage for each of the bus type names. As a buses are looked up and the
 * indexes cached, we can then use them to more quickly determine which specific
 * bus we are dealing with in future references.
*/
static int hwiattr_get_specific_bus(unsigned hwi_off, hwiattr_t *hwiattr)
{
	static unsigned pci_bus_string_hwi_off = HWI_NULL_OFF;
	static unsigned usb_bus_string_hwi_off = HWI_NULL_OFF;
	static unsigned spi_bus_string_hwi_off = HWI_NULL_OFF;
	static unsigned i2c_bus_string_hwi_off = HWI_NULL_OFF;
	/* add more as more hwibus_add_*() functions are added */

	int r = EINVAL;
	hwi_tag *tag = hwi_off2tag(hwi_off);
	
	if (tag->item.itemname == pci_bus_string_hwi_off)
	{
		r = hwiattr_get_pci(hwi_off, &hwiattr->pci);
	}
	else if (tag->item.itemname == usb_bus_string_hwi_off)
	{
		r = hwiattr_get_usb(hwi_off, &hwiattr->usb);
	}
	else if (tag->item.itemname == i2c_bus_string_hwi_off)
	{
		r = hwiattr_get_i2c(hwi_off, &hwiattr->i2c);
	}
	else if (tag->item.itemname == spi_bus_string_hwi_off)
	{
		r = hwiattr_get_spi(hwi_off, &hwiattr->spi);
	}
	/* add more as more hwibus_add_*() functions are added */

	/* need to do a lookup so cache the hwi offsets for the specific bus for future */
	else if (strcmp(__hwi_find_string(tag->item.itemname), HWI_ITEM_BUS_PCI) == 0)
	{
		/* this path will only be run once */
		pci_bus_string_hwi_off = tag->item.itemname;
		r = hwiattr_get_pci(hwi_off, &hwiattr->pci);
	}
	else if (strcmp(__hwi_find_string(tag->item.itemname), HWI_ITEM_BUS_USB) == 0)
	{
		/* this path will only be run once */
		pci_bus_string_hwi_off = tag->item.itemname;
		r = hwiattr_get_usb(hwi_off, &hwiattr->usb);
	}
	else if (strcmp(__hwi_find_string(tag->item.itemname), HWI_ITEM_BUS_I2C) == 0)
	{
		/* this path will only be run once */
		pci_bus_string_hwi_off = tag->item.itemname;
		r = hwiattr_get_i2c(hwi_off, &hwiattr->i2c);
	}
	else if (strcmp(__hwi_find_string(tag->item.itemname), HWI_ITEM_BUS_SPI) == 0)
	{
		/* this path will only be run once */
		pci_bus_string_hwi_off = tag->item.itemname;
		r = hwiattr_get_spi(hwi_off, &hwiattr->spi);
	}
	return r;
}

/*******************************************************************************
 * hwiattr_get_specific_device
 * 
 * this function will determine the type of device identified by <hwi_off> and
 * call the specific routine to fill in <hwiattr>
 *   
 * Returns: the result from the specific hwiattr_get_*() routine
 * 
 * Implementation note
 * 
 * The pnpid field of the 'device' tag contains a value identifying the device
 * class and hence the specific 'hwiattr_get_*()' call that should be made
*/
static int hwiattr_get_specific_device(unsigned hwi_off, hwiattr_t *hwiattr)
{
	int r;
	hwi_tag *tag = hwi_off2tag(hwi_off);
 
	switch (tag->device.pnpid)
	{
		case hwi_devclass_UART:
			r = hwiattr_get_uart(hwi_off, &hwiattr->uart);
			break;

		case hwi_devclass_ENET:
			r = hwiattr_get_enet(hwi_off, &hwiattr->enet);
			break;

		case hwi_devclass_RTC:
			r = hwiattr_get_rtc(hwi_off, &hwiattr->rtc);
			break;
			
		case hwi_devclass_DMA:
			r = hwiattr_get_dma(hwi_off, &hwiattr->dma);
			break;
			
		/* add more as more hwidev_add_*() functions are added */
		default:
			r = EINVAL;
			break;
	}
	return r;
}


__SRCVERSION("$URL$ $Rev$");

