/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int f3s_array_alloc (f3s_array_t *array)
{
/*{{{1*/
	int32_t		i, phys;
	uint32_t	chip;

	/* Allocate memory for various vectors */
	array->chip_vect     = f3s_calloc (array->chip_total * sizeof(f3s_chip_t));
	array->unit_vect     = f3s_calloc (array->unit_total * sizeof(f3s_chip_t *));
	array->phys_vect     = f3s_calloc (array->unit_total * sizeof(*array->phys_vect));
	array->age_vect      = f3s_calloc (array->unit_total * sizeof(*array->age_vect));
	array->break_vect    = f3s_calloc (array->unit_total * sizeof(*array->break_vect));
	array->map_lock_vect = f3s_calloc (array->unit_total * sizeof(f3s_lock_t));
	array->map_vect      = f3s_malloc (array->unit_total * sizeof(f3s_map_t));

	if (!array->chip_vect || !array->unit_vect || !array->phys_vect ||
	    !array->age_vect || !array->break_vect || !array->map_lock_vect ||
	    !array->map_vect)
	{
		f3s_assert (0);

		f3s_free (array->chip_vect,
		          array->chip_total * sizeof(*array->chip_vect));
		f3s_free (array->unit_vect,
		          array->unit_total * sizeof(*array->unit_vect));
		f3s_free (array->phys_vect,
		          array->unit_total * sizeof(*array->phys_vect));
		f3s_free (array->age_vect,
		          array->unit_total * sizeof(*array->age_vect));
		f3s_free (array->break_vect,
		          array->unit_total * sizeof(*array->break_vect));
		f3s_free (array->map_lock_vect,
		          array->unit_total * sizeof(*array->map_lock_vect));
		f3s_free (array->map_vect,
		          array->unit_total * sizeof(*array->map_vect));

		array->chip_vect     = NULL;
		array->unit_vect     = NULL;
		array->phys_vect     = NULL;
		array->age_vect      = NULL;
		array->break_vect    = NULL;
		array->map_lock_vect = NULL;
		array->map_vect      = NULL;

		return (ENOMEM);
	}

	/* initialize map_lock_vect with recursive */
	for(i = 0; i < array->unit_total; i++)
	{
		f3s_lock_init(&array->map_lock_vect[i]);
	}

	array->resume = 0;

	/* Initialize all chips in partition */
	for (chip = 0; chip < array->chip_total; chip++) 
	{
		pthread_mutex_init (&array->chip_vect[chip].command_mutex, NULL);
		pthread_cond_init  (&array->chip_vect[chip].erase_cond,    NULL);
		array->chip_vect[chip].erase_unit = -1;
	}

	/* Set unit pointer to every chip in array vector */
	for (phys = 0; phys < array->unit_total; phys++) 
	{
		array->unit_vect[phys] = &array->chip_vect[phys >> (array->chip_pow2 - array->unit_pow2)];
	}

	return (EOK);
/*}}}1*/
}

int f3s_array_free (f3s_array_t *array)
{
/*{{{1*/
	uint32_t	i, chip;

	/* Free locks */
	for(i = 0; i < array->unit_total; i++)
	{
		f3s_lock_free(&array->map_lock_vect[i]);
	}

	/* Free memory for various vectors */
	f3s_free (array->unit_vect,
	          array->unit_total * sizeof(*array->unit_vect));
	array->unit_vect = NULL;
	f3s_free (array->phys_vect,
	          array->unit_total * sizeof(*array->phys_vect));
	array->phys_vect = NULL;
	f3s_free (array->age_vect,
	          array->unit_total * sizeof(*array->age_vect));
	array->age_vect = NULL;
	f3s_free (array->break_vect,
	          array->unit_total * sizeof(*array->break_vect));
	array->break_vect = NULL;
	f3s_free (array->map_lock_vect,
	          array->unit_total * sizeof(*array->map_lock_vect));
	array->map_lock_vect = NULL;
	f3s_free (array->map_vect,
	          array->unit_total * sizeof(*array->map_vect));
	array->map_vect = NULL;
	
	/* Free all chips in partition */
	for (chip = 0; chip < array->chip_total; chip++) 
	{
		pthread_mutex_destroy (&array->chip_vect[chip].command_mutex);
		pthread_cond_destroy  (&array->chip_vect[chip].erase_cond);
	}

	/* Free memory for chip vector */
	f3s_free (array->chip_vect,
	          array->chip_total * sizeof(*array->chip_vect));
	array->chip_vect = NULL;
	
	return (EOK);
/*}}}1*/
}

int f3s_array_attach (f3s_array_t *array, f3s_attr_t *skt_attr)
{
/*{{{1*/
	f3s_part_t *	part;
	f3s_pair_t *	pair;
	f3s_mount_t *	raw_mount;
	f3s_mount_t *	f3s_mount;
	uint32_t		unit_first = 0;
	uint32_t		index;
	int				error;

	/* Initialize index */
	index = f3s.part_index;

	/* Clear array break vector */
	memset (array->break_vect, 0,
	        array->unit_total * sizeof(*array->break_vect));

	/* for all units in the array, create the partitions containing those units */
	while (unit_first < array->unit_total) 
	{
		/* create partition pair list entry for mount */
		pair = f3s_malloc (sizeof(*pair));
		if (!pair) {f3s_assert (pair); return (ENOMEM);}

		/* create attributes for raw and file system mountpoints */
		pair->raw_attr = f3s_calloc (sizeof(*pair->raw_attr));
		pair->f3s_attr = f3s_calloc (sizeof(*pair->f3s_attr));

		/* create partition structure */
		part = f3s_calloc (sizeof(*part));

		/* create mount structure for raw and f3s mountpoints */
		raw_mount = f3s_calloc (sizeof(*raw_mount));
		f3s_mount = f3s_calloc (sizeof(*f3s_mount));

		/* check if alloc failed for anything */
		if (!pair->raw_attr || !pair->f3s_attr || !part || !raw_mount ||
			!f3s_mount) 
		{
			f3s_assert (0);

			f3s_free (pair->raw_attr, sizeof(*pair->raw_attr));
			f3s_free (pair->f3s_attr, sizeof(*pair->f3s_attr));
			f3s_free (pair,           sizeof(*pair));
			f3s_free (part,           sizeof(*part));
			f3s_free (raw_mount,      sizeof(*raw_mount));
			f3s_free (f3s_mount,      sizeof(*f3s_mount));

			return (ENOMEM);
		}

		/* initialize attributes for raw and file system mountpoints */
		f3s_attr_init (pair->raw_attr, S_IFBLK | S_IRUSR | S_IRGRP | S_IWUSR,
		               skt_attr, NULL, NULL, NULL);
		f3s_attr_init (pair->f3s_attr, S_IFDIR | S_IPERMS,
		               pair->raw_attr, NULL, NULL, NULL);

		/* initialize device number */
		pair->raw_attr->attr.rdev = rsrcdbmgr_devno_attach (F3S_BASE_TYPE, -1, 0);
		f3s_assert(pair->raw_attr->attr.rdev != (-1U));
	
		/* initialize ocb share list for raw and file system mountpoints */
		f3s_list_init (&pair->raw_attr->share_list);
		f3s_list_init (&pair->f3s_attr->share_list);

		/* initialize mount structures for raw and f3s mountpoints */
		f3s_mount_init (raw_mount, skt_attr->attr.rdev, NULL);
		f3s_mount_init (f3s_mount, pair->raw_attr->attr.rdev, &f3s.f3s_funcs);

		/* copy array structure pointer into mount structure */
		raw_mount->array = array;
		f3s_mount->array = array;

		/* initialize mounteds */
		raw_mount->mounted.skt_attr = skt_attr;
		f3s_mount->mounted.raw_attr = pair->raw_attr;

		/* initialize partition structure */
		part->unit_first = unit_first;
		part->array      = skt_attr->attr.mount->array;
		f3s_lock_init(&part->table_lock);

		pair->raw_attr->part       = part;
		pair->raw_attr->attr.mount = raw_mount;

		pair->f3s_attr->part       = part;
		pair->f3s_attr->attr.mount = f3s_mount;

		/* setup break flag for first unit of partition */
		array->break_vect[unit_first] = 1;

		/* attach f3s mountpoint */
		iofunc_attr_lock(&pair->raw_attr->attr);
		iofunc_attr_lock(&pair->f3s_attr->attr);
		F3S_TABLE_ENTER(part);
		error = f3s_part_attach (part, pair->raw_attr, pair->f3s_attr, index);
		F3S_TABLE_LEAVE (part);
		iofunc_attr_unlock(&pair->f3s_attr->attr);
		iofunc_attr_unlock(&pair->raw_attr->attr);

		/* XXX - Handle error return */

		/* add pair to list of array mountpoints */
		f3s_list_add (&skt_attr->attr.mount->mounted.pair_list,
		              (f3s_list_t *) pair);

		/* setup unit first for next partition */
		unit_first = part->unit_first + part->unit_num;

		/* increment partition name index */
		index++;
	}

	return (EOK);
/*}}}1*/
}

int f3s_array_detach (f3s_array_t *array, f3s_attr_t *skt_attr)
{
/*{{{1*/
	f3s_pair_t *pair;
	f3s_part_t *part;
	int			error;

	/* while there is a pair in array list */
	while (skt_attr->attr.mount->mounted.pair_list.next !=
	       &skt_attr->attr.mount->mounted.pair_list) 
	{
		f3s_lock_assert((skt_attr->attr.lock_tid == pthread_self()) && (skt_attr->attr.lock_count > 0));

		/* detach mountpoint pair */
		pair = (f3s_pair_t *) skt_attr->attr.mount->mounted.pair_list.next;

		iofunc_attr_lock(&pair->raw_attr->attr);
		iofunc_attr_lock(&pair->f3s_attr->attr);
		
		f3s_lock_assert((pair->raw_attr->attr.lock_tid == pthread_self()) && (pair->raw_attr->attr.lock_count > 0));
		f3s_lock_assert((pair->f3s_attr->attr.lock_tid == pthread_self()) && (pair->f3s_attr->attr.lock_count > 0));

		part = pair->raw_attr->part;
		F3S_TABLE_ENTER (part);
		error = f3s_part_detach (part, pair->raw_attr, pair->f3s_attr);
		F3S_TABLE_LEAVE (part);
		f3s_assert(error == EOK);

		/* XXX - Handle error return */

		/* remove mountpoint pair pointer from list */
		f3s_list_rem ((f3s_list_t *) pair);

		/* free partition memory */
		f3s_free (part, sizeof(*part));

		/* initialize device number */
		rsrcdbmgr_devno_detach (pair->raw_attr->attr.rdev, 0);

		/* free mount memory */
		f3s_free (pair->raw_attr->attr.mount,
		          sizeof(*pair->raw_attr->attr.mount));
		f3s_free (pair->f3s_attr->attr.mount,
		          sizeof(*pair->f3s_attr->attr.mount));

		iofunc_attr_unlock(&pair->f3s_attr->attr);
		iofunc_attr_unlock(&pair->raw_attr->attr);

		/* free raw and f3s attributes */
		f3s_free (pair->raw_attr, sizeof(*pair->raw_attr));
		f3s_free (pair->f3s_attr, sizeof(*pair->f3s_attr));

		/* free pair memory */
		f3s_free (pair, sizeof(*pair));
	}

	return (EOK);
/*}}}1*/
}

int f3s_array_calc (f3s_array_t *array)
{
/*{{{1*/
	uint32_t	chip_size;
	uint32_t	unit_size;
	uint32_t	geo_index;

	/* initialize unit and chip sizes */
	unit_size = 0;
	chip_size = 0;

	/* for all geometries in vector */
	for (geo_index = 0; geo_index < array->dbase.geo_num; geo_index++) 
	{
		/* setup unit size */
		unit_size = max(unit_size, 1 << array->dbase.geo_vect[geo_index].unit_pow2);

		/* calculate chip size */
		chip_size += array->dbase.geo_vect[geo_index].unit_num *
		             (1 << array->dbase.geo_vect[geo_index].unit_pow2);
	}

	/* put unit size in array structure */
	array->unit_size = unit_size;

	/* find power of two of sizes for shifts */
	array->unit_pow2 = 1;
	while ((array->unit_pow2 < 32) && ((1 << array->unit_pow2) != unit_size))
	{
		array->unit_pow2++;
	}

	/* check if unit power of two makes no sense */
	if (array->unit_pow2 == 32) 
	{
		fprintf (stderr, "(devf  t%d::%s:%d) Error, One of the sectors is not a power of 2\n",
					pthread_self(), __func__, __LINE__);
		return (ENOTSUP);
	}

	array->chip_pow2 = 1;
	while ((array->chip_pow2 < 32) && ((1 << array->chip_pow2) != chip_size))
	{
		array->chip_pow2++;
	}

	/* check if chip power of two makes no sense */
	if (array->chip_pow2 == 32) return (ENOTSUP);

	/* setup number of units */
	array->unit_total = array->chip_total << (array->chip_pow2 - array->unit_pow2);

	return (EOK);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/array.c $ $Rev: 580669 $" );
