/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

void f3s_attr_init (f3s_attr_t *attr, mode_t mode, f3s_attr_t *dattr,
                    f3s_desc_t *desc, f3s_stat_t *stat,
                    struct _client_info *info)
{
/*{{{1*/

	/* check if default attributes are different from attributes */
	if (dattr != attr) 
	{
		/* initalize with default attribute structure */
		/*
		 * Note:
		 * since making locking changes, a lock will be held on an f3s_attr_t (even though in
		 * some circumstances it may not need to be) and so if initializing, make sure that
		 * the lock state is preserved. This is the only call to iofunc_attr_init()
		*/
		uint_t  lock_tid = attr->attr.lock_tid;
		uint_t  lock_count = attr->attr.lock_count;
		iofunc_attr_init (&attr->attr, mode, &dattr->attr, info);
		attr->attr.lock_tid = lock_tid;
		attr->attr.lock_count = lock_count;
	}

	/* Clear dirty time flags */
	attr->attr.flags &= ~IOFUNC_ATTR_DIRTY_TIME;

	/* Use the descriptor's first extent as the inode number */

	if (desc){
		F3S_EXTPTR_TO_INODE(desc->first, attr->attr.inode);
	}

	/* check if stat is available */
	if (stat) 
	{
		/* copy stat info into attr */
		attr->attr.mode  = stat->mode;

		attr->attr.uid = stat->uid;
		attr->attr.gid = stat->gid;

		attr->attr.atime = stat->mtime;
		attr->attr.mtime = stat->mtime;
		attr->attr.ctime = stat->ctime;

		/* Directories have an extra link */
		if (S_ISDIR(stat->mode))
			attr->attr.nlink = 2;
		else
			attr->attr.nlink = 1;
	}
/*}}}1*/
}

void f3s_attr_inval (f3s_ocb_t *ocb, f3s_attr_t *attr, uint32_t last_flag,
                     int32_t offset, int32_t adjust, uint32_t rewind_forward)
{
/*{{{1*/
	f3s_share_t *file_share;

	iofunc_attr_lock(&attr->attr);
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
	
	/* setup share pointer at beginning of file share list */
	file_share = (f3s_share_t *)attr->share_list.next;

	/* while there is a share in the list */
	while (file_share != (f3s_share_t *)&attr->share_list) 
	{
		/* check if share ocb is different from current ocb */
		if (file_share->ocb != ocb) 
		{
			/* set no last status flag in share ocb file descriptor */
			file_share->ocb->file_desc.status |= last_flag;
			/* pr 60121 - Rewind the desc structure */
			f3s_desc_rewind(&file_share->ocb->file_desc);

			/*
			 * Other OCBs that are open to the same directory must have their
			 * offsets adjusted so that they will continue to properly iterate
			 * over files.
			 */
			if (file_share->ocb->ocb.offset >= offset) 
			{
				/* clear no rew. or for. status flag in share ocb file descriptor */
				file_share->ocb->file_desc.status &= ~rewind_forward;

				if ((adjust >= 0) ||
				    (file_share->ocb->ocb.offset > -adjust))
				{
					/* adjust ocb offset */
					file_share->ocb->ocb.offset += adjust;
				} else {
					/* prevent offsets from going negative */
					file_share->ocb->ocb.offset = 0;
				}
			}
		}
		/* go to next list entry */
		file_share = (f3s_share_t *)file_share->list.next;
	}
	iofunc_attr_unlock(&attr->attr);
/*}}}1*/
}

void f3s_attr_resv (f3s_attr_t *attr, int32_t size)
{
/*{{{1*/
	f3s_part_t *part;
	int32_t		alloc_size = 0;

	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));
	/* get partition info pointer */
	part = attr->part;

	/* make sure we have a resv ptr in the first place */
	if ((attr->resv_ptr) && (attr->resv_ptr->allocated == 0))
	{
		F3S_TABLE_ENTER(part);

		/* attempt to reserve flash space */
		alloc_size = f3s_resv_alloc (part, attr->resv_ptr, size);
		if (alloc_size < 0) 
		{
			/* free append buffer */
			f3s_free (attr->buffer, f3s.buffer_size);
			attr->buffer = NULL;
			attr->buffer_pos = 0;

			/* free reservation structure */
			f3s_free (attr->resv_ptr, sizeof(*attr->resv_ptr));
			attr->resv_ptr = NULL;
		}

		F3S_TABLE_LEAVE(part);
	}
/*}}}1*/
}

int f3s_attr_sync (f3s_ocb_t *ocb, f3s_attr_t *attr)
{
/*{{{1*/
	f3s_desc_t	desc;
	f3s_part_t *part;
	int32_t		previous_size;
	int			err_size;
	int			error;


	/* get partition info pointer */
	part = attr->part;

	/* If there is something in append buffer */
	if (attr->buffer_pos) 
	{
		/* get a copy of file descriptor */
		desc = ocb->file_desc;

		/* get previous file size */
		previous_size = f3s_file_forward (part, &desc, F3S_FILE_MAX);
		if (previous_size < 0)
			return (errno);

		/* flush the append buffer to the file */
		err_size = f3s_file_append (part, &desc, attr->resv_ptr,
		                            attr->buffer_pos, attr->buffer, 0);
		error = errno;

		/* Update the main file desc structure, so that we don't
		 * lose it if we error out.
		 */
		ocb->file_desc = desc;

		/* attempt to reserve flash space again (regardless of append success/failure) */
		f3s_attr_resv (attr, f3s.buffer_size);
		
		/* if the append failed, return the errno */
		if (err_size < 0)
			return (error);

		/* clear the append buffer */
		attr->buffer_pos = 0;

		/* invalidate last and forward in sharing ocb descriptors */
		f3s_attr_inval (NULL, attr, F3S_DESC_NO_LAST, previous_size, 0,
		                F3S_DESC_NO_FORWARD);
	}

	/* If file needs rewind */
	if (!(ocb->file_desc.status & F3S_DESC_NO_REWIND)) 
	{
		/* rewind file descriptor */
		f3s_desc_rewind (&ocb->file_desc);

		/* forward file descriptor to ocb offset */
		err_size = f3s_file_forward (part, &ocb->file_desc, ocb->ocb.offset);
		if (err_size < 0)
			return (errno);

		/* set no rewind flag in file descriptor */
		ocb->file_desc.status |= (F3S_DESC_NO_REWIND | F3S_DESC_NO_FORWARD);
		
	}	/* If file needs forward */
	else if (!(ocb->file_desc.status & F3S_DESC_NO_FORWARD)) 
	{
		/* forward file descriptor to ocb offset */
		err_size = f3s_file_forward (part, &ocb->file_desc,
		                             ocb->ocb.offset - ocb->file_desc.curr_offset);
		if (err_size < 0)
			return (errno);

		/* set no rewind flag in file descriptor */
		ocb->file_desc.status |= F3S_DESC_NO_FORWARD;
	}

	return (EOK);
/*}}}1*/
}
 

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/attr.c $ $Rev: 580669 $" );
