/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int f3s_boot_read (f3s_part_t *part, int32_t phys, f3s_boot_t *boot)
{
/*{{{1*/
	f3s_head_t	head;
	uint32_t	boot_offset;
	int32_t		boot_size;
	int			error;

	/* Read the extent header */
	boot_size = f3s_head_read (part, phys, F3S_BOOT_INDEX, &boot_offset, &head, 1);
	if (boot_size < 0) return (-1);

	/* Perform some sanity checks */
	if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS) return (ENOENT);
	if (boot_size   != sizeof(*boot)) return (ENOENT);
	if (boot_offset != sizeof(f3s_unit_info_t) + sizeof(f3s_unit_logi_t)) return (ENOENT);

	/* Read the boot header */
	error = f3s_flash_read (part->array, phys, boot_offset, sizeof(*boot),
	                        (uint8_t *)boot);
	if (error) return (error);

	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_boot_swap (boot);

	/* If struct size is not valid */
	if (boot->struct_size != sizeof(*boot)) return (ENOENT);

	/* If signature is not valid */
	if (memcmp (boot->sig, F3S_SIG_STRING, F3S_SIG_SIZE)) return (ENOENT);

	/* If partition size is not valid */
	if ((boot->unit_total + part->unit_first) > part->array->unit_total) return (EBADFSYS);

	/* If major revision number is not valid */
	if (boot->rev_major != F3S_REV_MAJOR) return (EBADFSYS);

	return (EOK);
/*}}}1*/
}

int f3s_boot_write (f3s_part_t *part, int32_t phys,
                    f3s_extptr_t root,
                    uint16_t unit_num, uint16_t unit_spare,
                    uint16_t align_pow2,
                    uint16_t index)
{
/*{{{1*/
	f3s_boot_t	boot;
	f3s_map_t *	map;
	int			err_size;

	/* write structure size */
	boot.struct_size = sizeof(boot);

	/* write parameters into boot info structure */
	boot.rev_major = F3S_REV_MAJOR;
	boot.rev_minor = F3S_REV_MINOR;

	memcpy (&boot.sig, F3S_SIG_STRING, F3S_SIG_SIZE);

	boot.unit_index = index;
	boot.unit_total = unit_num;
	boot.unit_spare = unit_spare;

	boot.align_pow2 = align_pow2;

	/* copy root extent pointer */
	boot.root = root;

	/* check if boot structure needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_boot_swap (&boot);

	/* Force clear the last header bit from the unit logi structure */
	err_size = f3s_head_status (part, &phys, F3S_UNIT_LOGI_INDEX, ~F3S_EXT_LAST, 0);
	if (err_size < 0) return (errno);

	/* write extent with boot info */
	err_size = f3s_ext_write (part, &phys, F3S_BOOT_INDEX,
	                          F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t) + sizeof(f3s_unit_logi_t), F3S_SYS_ALIGN),
	                          sizeof(boot), (uint8_t *)&boot,
	                          ~F3S_EXT_TYPE | F3S_EXT_SYS, F3S_SYS_POW2, 1, 0);
	if (err_size < 0) return (errno);

	/* Update the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];
	map->head_last  = F3S_BOOT_INDEX;
	map->free_first = F3S_BOOT_INDEX + 1;
	map->text_size += F3S_TEXT_ALIGN(part, sizeof(f3s_boot_t), F3S_SYS_ALIGN);;
	map->free_size -= F3S_TEXT_ALIGN(part, sizeof(f3s_boot_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);;

	return (EOK);
/*}}}1*/
}

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/boot.c $ $Rev: 580669 $" );
