/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int32_t f3s_entry_create (f3s_part_t *part, f3s_desc_t *desc, f3s_extptr_t first,
                          char *name, int16_t length, f3s_stat_t *stat,
                          uint32_t depth, uint8_t moves)
{
/*{{{1*/
	f3s_dirent_t *	dirent;
	f3s_stat_t *	stat_buffer;
	int32_t			dirent_size;

	/*
	 * NOTE: <length> is the length of <name> INCLUDING the terminating NULL. In
	 * order to resolve PR18519 (allow names upto F3S_NAME_MAX) some special
	 * handling has to be added because f3s_dirent_t.namelen has a max value of 255
	 * and the filesystem was originally written (incorrectly) to include the terminating
	 * NULL in the f3s_dirent_t.namelen calculation. Because of this and the need for
	 * backwards (and as much as possible forwards) compatibility, the terminating NULL
	 * must be kept as part of the f3s_dirent_t.namelen calculation (remember that this is
	 * an on chip data structure) except for the case when <length> == F3S_NAME_MAX
	 * (excluding terminating NULL). The good news is that there is room for the terminating
	 * NULL in the entry due to name alignment, it is just that it cannot be accounted for
	 * when the name is F3S_NAME_MAX + '\0'. In order to solve this, whenever
	 * f3s_dirent_t.namelen == F3S_NAME_MAX, an extra check will be made to see if
	 * the NULL character is present at position F3S_NAME_MAX and if not, it must be
	 * at the very next position (remember that there is space to store F3S_NAME_MAX + 1
	 * bytes).
	 *
	 * backwards compatibility - filesystems created with prior to this modification will
	 * 											  continue to function properly with the new code
	 * forwards compatibility - filesystems created with the library containing the modification
	 * 										  will work properly with previous flash libraries as long as no
	 * 										  files were created with names >= F3S_NAME_MAX (excluding
	 * 										  terminating NULL)
	 * 
	 * Upon entry to this routine, the following assumptions continue to be made
	 * 
	 * 	<length> is the length of <name> including terminating NULL
	*/

	/* Sanity checks */
	f3s_assert (length > 0);			// make sure length is valid but let normal error path handle case of too long
	if (length > (F3S_NAME_MAX + 1))
	{
		errno = ENAMETOOLONG;
		return (-1);
	}

	/* setup size of directory entry */
	dirent_size = sizeof(*dirent) + F3S_NAME_ALIGN(length) + sizeof(*stat);

	/* allocate memory for new directory entry */
	dirent = alloca (dirent_size);

	/* check if dirent pointer is not valid */
	if (!dirent) {
		errno = ENOMEM;
		return (-1);
	}

	/* setup new directory entry */
	dirent->struct_size = sizeof(*dirent);
	dirent->moves       = moves;
	dirent->namelen     = min(length, F3S_NAME_MAX);
	dirent->first       = first;

	/* copy entry name */
	memcpy (dirent + 1, name, length);

	/* set stat buffer pointer */
	stat_buffer = (f3s_stat_t *)((uint8_t *)(dirent + 1) + F3S_NAME_ALIGN(length));

	/* copy stat info */
	memcpy (stat_buffer, stat, sizeof(*stat));

	/* check if dirent and stat buffer need swapping */
	if (part->table.swap_flag) {
		f3s_dirent_swap (dirent);
		f3s_stat_swap (stat_buffer);
	}

	/* append directory entry to current directory */
	dirent_size = f3s_file_append (part, desc, NULL, dirent_size,
	                               (uint8_t *)dirent, 1);
	if (dirent_size < 0) return (dirent_size);

	/* DEBUG: Verify directory integrity */
	f3s_verify_dir (part, desc->first.logi_unit, desc->first.index);

	/* increment dirent counter */
	F3S_ALLOC_ENTER(part);
	part->table.dirent_num++;
	F3S_ALLOC_LEAVE(part);

	/* return size of new directory entry */
	return (dirent_size);
/*}}}1*/
}

int32_t f3s_entry_delete (f3s_part_t *part, f3s_desc_t *desc, uint32_t depth)
{
/*{{{1*/
	f3s_extptr_t	curr;
	f3s_desc_t		over_desc;
	f3s_head_t		head;
	f3s_cache_t *	cache;
	int32_t			text_size;
	int32_t			write_size;
	int32_t			curr_phys;

	/* check if cache is valid for depth */
	F3S_CACHE_ENTER(part);
	if (depth < F3S_CACHE_MAX) 
	{
		/* get cache pointer for depth */
		cache = &part->table.cache_vect[depth];

		/* invalidate all caches from level to deeper */
		for (; depth < F3S_CACHE_MAX; depth++) 
		{
			/* invalidate cache */
			cache->curr_offset = 0;

			/* increment cache pointer */
			cache++;
		}
	}
	F3S_CACHE_LEAVE(part);

	/* read current fresh header and lock the unit */
	text_size = f3s_extptr_fresh (part, desc->curr, NULL, NULL, &head, &curr);
	if (text_size < 0)
	{
		return (text_size);
	}

	/* get current physical unit index for fresh header */
	curr_phys = f3s_extptr_phys (part, curr);

	F3S_UNIT_LEAVE(part, curr_phys);
	F3S_LOGI_LEAVE(part, curr.logi_unit);

	/* check if header has a next */
	if (!(head.status[0] & F3S_EXT_NO_NEXT))
	{
		/* setup overwrite directory descriptor */
		f3s_desc_init (&over_desc, head.next, ~F3S_DESC_NO_DIR);

	/* else, header has no next */
	} else 
	{
		/* setup directory descriptor */
		f3s_desc_init (&over_desc, f3s_null_extptr, ~F3S_DESC_NO_DIR);

		/* overwrite file at current extent */
		write_size = f3s_file_create (part, &over_desc, &part->table.deletion_resv, 0, NULL, 1);
		if (write_size < 0) return (-1);
	}

	/* commit overwrite descriptor to file descriptor */
	write_size = f3s_file_commit (part, desc, over_desc.first, F3S_COMMIT_OVER);
	if (write_size < 0) return (-1);

	/* decrement dirent counter */
	F3S_ALLOC_ENTER(part);
	part->table.dirent_num--;
	F3S_ALLOC_LEAVE(part);

	/* check if header had no next */
	F3S_TABLE_ENTER (part);
	if (head.status[0] & F3S_EXT_NO_NEXT)
	{
		/* reserve space for a header to garantee next deletion */
		f3s_resv_alloc (part, &part->table.deletion_resv, 0);
	}
	F3S_TABLE_LEAVE (part);

	/* write size at this point has no relation to the number of bytes deleted */
	return (write_size);
/*}}}1*/
}

/* 
 * This function will return EOK if <name> is found. It will return ENOENT if the name is not found. It will return
 * other errnos for other failures and callers should not interpret these other returns as an indication that the
 * entry does not exist and can be made
*/
int f3s_entry_lookup (f3s_part_t *part, f3s_desc_t *desc, f3s_dirent_t *dirent,
                      char *name, int16_t length, f3s_stat_t *stat, uint32_t depth)
{
/*{{{1*/
	f3s_extptr_t curr;
	f3s_dirent_t dirent_buffer;
	f3s_cache_t *cache = NULL;
	uint8_t *compare;
	int32_t found_flag = 1;
	int32_t dirent_size;
	int32_t	name_size;
	int32_t stat_size;
	int32_t desc_offset = 0;
	int32_t reclen;
	int32_t loop_flag = 0;
	int32_t curr_offset;
	int32_t forward_size;


	f3s_assert (length > 0);			// make sure length is valid but let normal error path handle case of too long
	if (length > (F3S_NAME_MAX + 1))
		return(ENAMETOOLONG);
	
	/* check if length is bigger than one */
	if (length > 1) 
	{
		/* allocate stack space for compare */
		compare = alloca (F3S_NAME_ALIGN(length));

		/* check if compare pointer is not valid */
		if (!compare)
		{
			return (ENOMEM);
		}

		F3S_CACHE_ENTER(part);

		/* check if cache is valid for depth */
		if (depth < F3S_CACHE_MAX) 
		{
			/* get cache pointer for depth */
			cache = &part->table.cache_vect[depth];

			/* check if cache offset is valid */
			if (cache->curr_offset) 
			{
				/* copy cache into descriptor */
				desc->curr = cache->curr;
				desc->curr_offset = cache->curr_offset;

				/* set decriptor offset to cache offset */
				desc_offset = cache->curr_offset;
			}
		}

		/* find name loop */
		while (1) 
		{
			/* remember current extent pointer */
			curr = desc->curr;
			curr_offset = desc->curr_offset;

			/* read directory entry into buffer */
			dirent_size = f3s_dirent_read (part, desc, desc_offset - desc->curr_offset,
			                               &dirent_buffer);

			if (dirent_size < 0)
			{
				int error = errno;
				F3S_CACHE_LEAVE(part);
				return(error);
			}
			
			f3s_assert (dirent_size == 0 || 
			            dirent_buffer.struct_size == sizeof(dirent_buffer));

			/* check if we are at end of directory */
			if (dirent_size == 0) 
			{
				/* check if cache is valid and its offset is non zero */
				if (cache && cache->curr_offset) 
				{
					/* set loop flag */
					loop_flag = 1;

					/* rewind descriptor */
					f3s_desc_rewind (desc);

					/* reset descriptor offset */
					desc_offset = 0;

					/* continue from here */
					continue;

				} else {
					/* clear found flag */
					found_flag = 0;

					/* break of loop */
					break;
				}
			}

			/* calculate record length */
			reclen = sizeof(dirent_buffer) +
			         F3S_NAME_ALIGN(dirent_buffer.namelen) + sizeof(f3s_stat_t);

			/* adjust offset with directory entry record length */
			desc_offset += reclen;

			/* check if name lengths are equal */
			if ((length == dirent_buffer.namelen) ||
				((length == (F3S_NAME_MAX + 1)) && (length == F3S_NAME_ALIGN(dirent_buffer.namelen))))
			{
				/* read directory entry name */
				name_size = f3s_file_read (part, desc, sizeof(dirent_buffer),
														F3S_NAME_ALIGN(dirent_buffer.namelen), compare);
				if (name_size < 0)
				{
					/* clear found flag */
					found_flag = 0;

					/* break of loop */
					break;
				}

				f3s_assert(name_size == F3S_NAME_ALIGN(length));

				/* check if names compare right */
				if (!memcmp (name, compare, length)) 
				{
					/* check if current position is different from cache */
					if (cache && (cache->curr_offset != desc->curr_offset)) 
					{
						/* copy current position in cache */
						cache->curr = desc->curr;
						cache->curr_offset = desc->curr_offset;

						/* invalidate all deeper caches */
						for (depth++; depth < F3S_CACHE_MAX; depth++) 
						{
							/* increment cache pointer */
							cache++;

							/* invalidate cache */
							cache->curr_offset = 0;
						}
					}
					/* break of loop */
					break;
				}
				else	/* else, names are not equal */
				{
					/* forward file descriptor to next directory entry */
					forward_size = f3s_file_forward (part, desc, reclen);

					if (forward_size < 0) 
					{
						/* clear found flag */
						found_flag = 0;

						/* break of loop */
						break;
					}
				}


			}
			else	/* else, name lengths are not equal, no need to check further */
			{
				/* forward file descriptor to next directory entry */
				forward_size = f3s_file_forward (part, desc, reclen);

				if (forward_size < 0) 
				{
					/* clear found flag */
					found_flag = 0;

					/* break of loop */
					break;
				}
			}

			/* check if we have looped and we are back to cache offset */
			if (loop_flag && (cache->curr_offset == desc->curr_offset)) 
			{
				/* clear found flag */
				found_flag = 0;

				/* break of loop */
				break;
			}
		}	// while (1)
		F3S_CACHE_LEAVE(part);

	}
	else	/* else, current entry is wanted */
	{
		/* remember current extent pointer */
		curr        = desc->curr;
		curr_offset = desc->curr_offset;

		/* read directory entry into buffer */
		dirent_size = f3s_dirent_read (part, desc, 0, &dirent_buffer);
		if (dirent_size == 0)
			found_flag = 0;

		f3s_assert (dirent_size == 0 || 
		            dirent_buffer.struct_size == sizeof(dirent_buffer));
	}

	/* check if entry is found */
	if (found_flag) 
	{
		/* check if dirent is wanted */
		if (dirent) 
		{
			/* copy dirent buffer */
			*dirent = dirent_buffer;
		}

		/* check if stat is wanted */
		if (stat) 
		{
			/* read stat info */
			stat_size = f3s_file_read (part, desc, sizeof(dirent_buffer) +
			                           F3S_NAME_ALIGN(dirent_buffer.namelen),
			                           sizeof(*stat), (uint8_t *)stat);
			if (stat_size < 0)
				found_flag = 0;

			/* check if stat structure needs swapping */
			//swap_flag is set once, at mount time
			if (part->table.swap_flag) f3s_stat_swap (stat);
		}

		/* put back found extent pointer */
		desc->curr        = curr;
		desc->curr_offset = curr_offset;
	}

	if (found_flag)
		return (EOK);
	else
		return (ENOENT);
/*}}}1*/
}

int32_t f3s_entry_retrace (f3s_part_t *part, f3s_desc_t *desc, f3s_extptr_t first)
{
/*{{{1*/
	f3s_dirent_t	dirent;
	int32_t			found_flag = 1;
	int32_t			dirent_size;
	int32_t			desc_offset = 0;
	int32_t			reclen;

	/* rewind descriptor */
	f3s_desc_rewind (desc);

	/* find first extptr loop */
	while (1) 
	{
		/* read directory entry into buffer */
		dirent_size = f3s_dirent_read (part, desc, desc_offset - desc->curr_offset,
		                               &dirent);

		if (dirent_size < 0)
			return(0);	// not found

		f3s_assert (dirent_size == 0 ||
		            dirent.struct_size == sizeof(dirent));

		/* check if we are at end of directory */
		if (dirent_size == 0) 
		{
			/* clear found flag */
			found_flag = 0;
			break;
		}

		/* calculate record length */
		reclen = sizeof(dirent) + F3S_NAME_ALIGN(dirent.namelen) + sizeof(f3s_stat_t);

		/* check if first extent pointer is equal to the searched one */
		if (!memcmp (&dirent.first, &first, sizeof(f3s_extptr_t)))
		{
			/* break of loop */
			break;
		}

		/* adjust offset with directory entry record length */
		desc_offset += reclen;
	}

	/* return found flag */
	return (found_flag);
/*}}}1*/
}

int32_t f3s_entry_change (f3s_part_t *part, f3s_desc_t *desc, f3s_dirent_t *dirent,
                          char *name, int16_t length, f3s_stat_t *stat, uint8_t moves)
{
/*{{{1*/
	f3s_extptr_t	curr;
	f3s_head_t		head;
	f3s_desc_t		over_desc;
	f3s_dirent_t *	dirent_buffer;
	f3s_stat_t *	stat_buffer;
	int32_t			dirent_size;
	int32_t			text_size;
	int32_t			curr_phys;
	int32_t			write_size;

	f3s_assert (length > 0);			// make sure length is valid but let normal error path handle case of too long
	if (length > (F3S_NAME_MAX + 1))
	{
		errno = ENAMETOOLONG;
		return (-1);
	}
	
	/* read current fresh header and lock the unit */
	text_size = f3s_extptr_fresh (part, desc->curr, NULL, NULL, &head, &curr);
	if (text_size < 0)
	{
		return (text_size);
	}

	/* get current physical unit index for fresh header */
	curr_phys = f3s_extptr_phys (part, curr);

	F3S_UNIT_LEAVE (part, curr_phys);
	F3S_LOGI_LEAVE (part, curr.logi_unit);

	/* setup size of directory entry */
	dirent_size = sizeof(*dirent) + F3S_NAME_ALIGN(length) + sizeof(*stat);

	/* allocate memory for new directory entry */
	dirent_buffer = alloca (dirent_size);

	/* check if dirent buffer pointer is not valid */
	if (!dirent_buffer) 
	{
		f3s_assert (dirent_buffer);
		errno = ENOMEM;
		return (-1);
	}

	/* copy dirent */
	memcpy (dirent_buffer, dirent, sizeof(*dirent));

	/* setup name length */
	dirent_buffer->namelen = min(length, F3S_NAME_MAX);

	/* copy entry name */
	memcpy (dirent_buffer + 1, name, length);

	/* copy moves parameter */
	dirent_buffer->moves = moves;

	/* set stat buffer pointer */
	stat_buffer = (f3s_stat_t *)((uint8_t *)(dirent_buffer + 1) + F3S_NAME_ALIGN(length));

	/* copy stat info */
	memcpy (stat_buffer, stat, sizeof(*stat));

	/* check if dirent and stat buffers need swapping */
	if (part->table.swap_flag) 
	{
		f3s_dirent_swap (dirent_buffer);
		f3s_stat_swap (stat_buffer);
	}

	/* setup directory descriptor */
	f3s_desc_init (&over_desc, f3s_null_extptr, ~F3S_DESC_NO_DIR);

	/* create directory entry file */
	write_size = f3s_file_create (part, &over_desc, NULL, dirent_size,
	                              (uint8_t *)dirent_buffer, 1);
	if (write_size < 0) return (write_size);

	/* check if header has a next */
	if (!(head.status[0] & F3S_EXT_NO_NEXT)) 
	{
		/* commit head next to overwrite descriptor */
		write_size = f3s_file_commit (part, &over_desc, head.next, F3S_COMMIT_APPEND);
		if (write_size < 0) return (write_size);
	}

	/* commit overwrite descriptor to file descriptor */
	write_size = f3s_file_commit (part, desc, over_desc.first, F3S_COMMIT_OVER);
	
	/* DEBUG: Verify directory integrity */
	f3s_verify_dir (part, desc->first.logi_unit, desc->first.index);

	/* return size of new directory entry */
	return (dirent_size);
/*}}}1*/
}

int32_t f3s_entry_update (f3s_part_t *part, f3s_desc_t *desc, f3s_stat_t *stat)
{
/*{{{1*/
	f3s_dirent_t	dirent;
	char*			name;
	unsigned int  name_len;
	int32_t			update_size;
	int32_t			read_size;
	int				error;

	/* read directory entry */
	error = f3s_entry_lookup (part, desc, &dirent, "", 1, NULL, F3S_CACHE_MAX);
	if (error != EOK)
	{
		errno = error;
		return (-1);
	}

	/* allocate stack space for name */
	name = alloca(F3S_NAME_ALIGN(dirent.namelen));

	/* check if name pointer is not valid */
	if (!name) 
	{
		errno = ENOMEM;
		return (-1);
	}

	/* read directory entry name */
	read_size = f3s_file_read (part, desc, sizeof(dirent), F3S_NAME_ALIGN(dirent.namelen), (uint8_t*)name);
	if (read_size < 0) return (read_size);

	/* the real name length for the directory entry may be F3S_NAME_MAX or F3S_NAME_MAX + 1 */
	name_len = dirent.namelen;
	if ((dirent.namelen == F3S_NAME_MAX) && (name[F3S_NAME_MAX] == '\0'))
		++name_len;

	/* change directory entry */
	update_size = f3s_entry_change (part, desc, &dirent, name, name_len, stat, dirent.moves);
	return (update_size);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/entry.c $ $Rev: 580669 $" );
