/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int32_t
f3s_ext_alloc (f3s_part_t *part, int32_t *phys,
               uint32_t *text_offset, int32_t text_size,
               f3s_extptr_t *extptr, uint32_t *mask, uint16_t align_pow2)
{
/*{{{1*/
	f3s_map_t *	map;
	uint32_t	new_index;
	int32_t		alloc_size;
	uint32_t	status;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* get map from physical unit */
	f3s_lock_assert(part->array->map_lock_vect[*phys].thread == pthread_self());
	map = &part->array->map_vect[*phys];

	/* check if there is free headers before last */
	if (map->free_num)
	{
		/* get new index from free first in map */
		new_index = map->free_first;

		/* find the next free index if one exists (don't consider the one we are taking) */
		if (map->free_num > 1)
		{
			uint32_t  next_free_idx = map->free_first;
			/* find next free entry loop */
			do
			{
				/* note >= check because if we hit the last, free_num was wrong */
				if ((++next_free_idx >= map->head_last) ||
					(f3s_head_check (part, *phys, next_free_idx, &status) < 0))
					return (-1);
			} while ((status & F3S_EXT_MASK) != F3S_EXT_FREE);

			/* successfully found the new map->free_first. Update data structures and continue */
			*mask &= ~F3S_EXT_LAST;
			map->free_first = next_free_idx;
		}
		else
		{
			/* make map free first equal to map head last */
			map->free_first = map->head_last;
		}
		
		/* decrement number of free indexes */
		--map->free_num;
	}
	else		/* else there is no free header before last */
	{
		/* status last header as not last */
		if (f3s_head_status (part, phys, map->head_last, ~F3S_EXT_LAST, 1) < 0)
			return (-1);

		/* Renew the map pointer in case f3s_head_status changed phys */
		f3s_lock_assert(part->array->map_lock_vect[*phys].thread == pthread_self());
		map = &part->array->map_vect[*phys];

		/* increment last header index. It is the new first free */
		map->head_last++;
		map->free_first = map->head_last;

		/* increment global counter of headers */
		F3S_ALLOC_ENTER(part);
		part->table.head_num++;

		/* reduce space available */
		map->free_size         -= sizeof(f3s_head_t);
		part->table.free_total -= sizeof(f3s_head_t);
		F3S_ALLOC_LEAVE(part);

		/* get newly allocated index */
		new_index = map->head_last;
	}

	/* set text offset parameter with new offset */
	if (text_offset)
		*text_offset = map->text_size;

	/* set extent pointer parameter */
	if (extptr)
	{
		f3s_lock_assert(part->array->map_lock_vect[*phys].thread == pthread_self());
		extptr->logi_unit = part->array->phys_vect[*phys];
		extptr->index = new_index;
	}

	/* limit allocation to what is available in unit */
	alloc_size = F3S_TEXT_ALIGN (part, text_size, 1 << align_pow2);
	if (map->free_size < alloc_size)
	{
		alloc_size = map->free_size;

	} else
	{
		/* add padding size to map and table */
		map->pad_size         += (alloc_size - text_size);
		F3S_ALLOC_ENTER(part);
		part->table.pad_total += (alloc_size - text_size);
		F3S_ALLOC_LEAVE(part);
	}

	/* reduce space available */
	map->free_size         -= alloc_size;
	F3S_ALLOC_ENTER(part);
	part->table.free_total -= alloc_size;
	F3S_ALLOC_LEAVE(part);

	/* increase allocated space */
	map->text_size += alloc_size;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, *phys, alloc_size);
	return (min(alloc_size, text_size));
/*}}}1*/
}

int32_t f3s_ext_write (f3s_part_t *part, int32_t *phys, uint32_t index,
                       uint32_t text_offset, int32_t text_size, uint8_t *text,
                       uint32_t mask, uint16_t align_pow2, int meta, int recover)
{
/*{{{1*/
	f3s_head_t	head;
	int32_t		head_offset;
	uint32_t	status;
	int			err_size;
	int			error;

	if ((text_offset >= part->array->unit_size) || (text_size >= part->array->unit_size))
	{
		printf("(devf  t%d::%s) Failed attempt to write %d bytes at offset 0x%x on P[%02X], H[%02X]\n"
			"unit size is only 0x%x\n", pthread_self(), __func__, text_size, text_offset, *phys,
			index, part->array->unit_size);
		return(-1);
	}
	
	/* fill header structure */
	memset(&head, F3S_BYTE_ERASE, sizeof(head));
	head.text_size = text_size;
	head.text_offset_lo = (uint16_t)(text_offset >> align_pow2);
	head.text_offset_hi = (uint8_t)(text_offset >> (align_pow2 + F3S_OFFSET_HI_POW2));

	/* calculate header offset */
	head_offset = F3S_HEAD_OFFSET(part->array, index);
	
	/* flag header as written */
	status = (mask & (~F3S_EXT_MASK | F3S_EXT_ALLOC)) & ~F3S_EXT_NO_WRITE;

	/* write extent header */
	error = f3s_head_write (part, phys, index, &head, recover);
	if (error != EOK)
		goto bad_shortcut;

	/* Write extent text (do before clearing F3S_EXT_NO_WRITE) */
	error = f3s_flash_write (part->array, *phys, text_offset, (uint32_t)text_size, text,
	                         meta ? F3S_VERIFY_WRITE : 0);
	if (error)
	{
		errno = error;
		goto bad_shortcut;
	}

	/* apply mask */
	err_size = f3s_head_status (part, phys, index, status, recover);
	if (err_size < 0) {error = errno; goto bad_shortcut;}

	return (text_size);

bad_shortcut:
	/* log that bad event */
	f3s_log_bad (part->array->index, part->index, *phys, index, text_size, status);
	errno = error;
	return (-1);
/*}}}1*/
}

int32_t f3s_extptr_basic (f3s_part_t *part, f3s_extptr_t extptr,
                          uint32_t *text_offset, f3s_head_t *head)
{
/*{{{1*/
	f3s_head_t	basic_head;
	int32_t		text_size;
	int32_t		phys;

	f3s_assert (extptr.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE(part)));

	/* get physical unit index */
	F3S_LOGI_ENTER(part, extptr.logi_unit);
	phys = f3s_extptr_phys (part, extptr);
	F3S_UNIT_ENTER (part, phys);
	f3s_lock_assert (extptr.index <= part->array->map_vect[phys].head_last);

	/* read header info */
	text_size = f3s_head_read (part, phys, extptr.index, text_offset, &basic_head, 1);
	if (text_size < 0)
	{
		F3S_UNIT_LEAVE (part, phys);
		F3S_LOGI_LEAVE (part, extptr.logi_unit);
		return (-1);
	}

	/* check to make sure the extptr is not deleted (see assert below) */
	if 	((basic_head.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC)
	{
		if ((basic_head.status[0] & F3S_EXT_MASK) != F3S_EXT_DELETE){
			F3S_UNIT_LEAVE (part, phys);
			F3S_LOGI_LEAVE (part, extptr.logi_unit);
			/* This used to print 'extptr not allocated' */
			f3s_log_bad (part->array->index, part->index, phys, extptr.index, text_size, basic_head.status[0]);
			return (-1);
		}
	}

	/* copy basic header at header pointer */
	if (head) *head = basic_head;
	return (text_size);
/*}}}1*/
}

int32_t f3s_extptr_delete (f3s_part_t *part, f3s_extptr_t extptr, f3s_head_t *head)
{
/*{{{1*/
	f3s_head_t	delete_head;
	f3s_map_t *	map;
	int32_t		text_size;
	int32_t		phys;
	int			err_size;

	/* get physical unit index */
	F3S_LOGI_ENTER(part, extptr.logi_unit);
	phys = f3s_extptr_phys (part, extptr);
	F3S_UNIT_ENTER(part, phys);

	/* get map pointer for physical unit */
	f3s_lock_assert(part->array->map_lock_vect[phys].thread == pthread_self());
	map = &part->array->map_vect[phys];

	/* delete supersede chain loop */
	while (1)
	{
		/* DEBUG: Verify the partition table totals match */
		f3s_verify_counters (part, 0, 0);

		/* read header info */
		text_size = f3s_head_read (part, phys, extptr.index, NULL, &delete_head, 1);
		if (text_size < 0)
		{
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE (part, extptr.logi_unit);
			return (-1);
		}
		f3s_assert ((delete_head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC);

		/* delete the header */
		err_size = f3s_head_status (part, &phys, extptr.index,
		                            ~F3S_EXT_MASK | F3S_EXT_DELETE, 1);
		if (err_size < 0)
		{
			/* log sticky extent */
			f3s_log_sticky (part->array->index, part->index, extptr.logi_unit,
			                phys, extptr.index);
		}

		/* because phys may have changed in call to f3s_head_read() above ... */
		map = &part->array->map_vect[phys];
		
		/* increment delete number in map */
		map->del_num++;

		/* check if header has a supersede pointer */
		if (!(delete_head.status[0] & F3S_EXT_NO_SUPER))
		{
			/* check if supersede extent is on different unit */
			if (delete_head.super.logi_unit != extptr.logi_unit)
			{
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE (part, extptr.logi_unit);

				/* get physical unit index */
				F3S_LOGI_ENTER(part, delete_head.super.logi_unit);
				phys = f3s_extptr_phys (part, delete_head.super);
				F3S_UNIT_ENTER(part, phys);

				/* get map pointer for physical unit */
				f3s_lock_assert(part->array->map_lock_vect[phys].thread == pthread_self());
				map = &part->array->map_vect[phys];
			}

			/* get supersede extent pointer */
			extptr = delete_head.super;
			continue;
		}

		break;
	}

	/* stale text size */
	f3s_unit_stale (part, phys, text_size);
	F3S_UNIT_LEAVE(part, phys);
	F3S_LOGI_LEAVE(part, extptr.logi_unit);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* copy delete header at header pointer */
	if (head) *head = delete_head;
	return (text_size);
/*}}}1*/
}

int32_t f3s_extptr_fresh (f3s_part_t *part, f3s_extptr_t extptr,
                          uint32_t *super_count, uint32_t *text_offset,
                          f3s_head_t *head, f3s_extptr_t *fresh)
{
/*{{{1*/
	f3s_head_t	fresh_head;
	uint32_t	count = 0;
	int32_t		text_size;
	int32_t		phys;
	int			error;
	uint32_t	retry = 0;
	const f3s_extptr_t saved_extptr = extptr;

	f3s_assert (extptr.logi_unit <= (F3S_FIRST_LOGI + F3S_UNIT_ACTIVE(part)));

	/* get physical unit index */
	F3S_LOGI_ENTER(part, extptr.logi_unit);
	phys = f3s_extptr_phys (part, extptr);
	F3S_UNIT_ENTER(part, phys);

	/* find unsuperseded header loop */
	while (1)
	{
		f3s_assert (extptr.index <= part->array->map_vect[phys].head_last);

		/* read header info */
		text_size = f3s_head_read (part, phys, extptr.index, text_offset, &fresh_head, 1);
		if (text_size < 0)
		{
			error = errno;
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE (part, extptr.logi_unit);
			errno = error;
			return (-1);
		}

		/* check to make sure extptr is not deleted (see assert below) */
		if ((fresh_head.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC)
		{
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE (part, extptr.logi_unit);
			/* This used to print 'extptr not allocated' */
			f3s_log_bad (part->array->index, part->index, phys, extptr.index, text_size, fresh_head.status[0]);
			if(count > 0) { /* super extent may be changed by 'reclaim', retry */
				if (((fresh_head.status[0] & F3S_EXT_MASK) == F3S_EXT_FREE) ||
				    ((fresh_head.status[0] & F3S_EXT_MASK) == F3S_EXT_DELETE)) {
					if(retry < 3) {
						count = 0;
						extptr = saved_extptr;
						F3S_LOGI_ENTER(part, extptr.logi_unit);
						phys = f3s_extptr_phys (part, extptr);
						F3S_UNIT_ENTER(part, phys);
						retry++;
						continue;
					}
				}
			}
			return (-1);
		}

		/* check if header has a supersede pointer */
		if (!(fresh_head.status[0] & F3S_EXT_NO_SUPER))
		{
			/* increment supersede counter */
			count++;

			/* check if supersede extent is on different unit */
			if (fresh_head.super.logi_unit != extptr.logi_unit)
			{
				/* Follow the superseded pointer to another unit */
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE (part, extptr.logi_unit);

				F3S_LOGI_ENTER(part, fresh_head.super.logi_unit);
				phys = f3s_extptr_phys (part, fresh_head.super);
				F3S_UNIT_ENTER(part, phys);
			}

			/* get supersede extent pointer */
			extptr = fresh_head.super;
			continue;
		}

		f3s_assert ((fresh_head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC);
		break;
	}

	/* copy supersede count */
	if (super_count) *super_count = count;

	/* copy fresh header at header pointer */
	if (head) *head = fresh_head;

	/* copy fresh extent pointer */
	*fresh = extptr;
	return (text_size);
/*}}}1*/
}

#ifndef NDEBUG
/* function added so some lock checking can be done. Macro changed to call this function */
uint16_t  _f3s_extptr_phys(f3s_part_t *PART_PTR, f3s_extptr_t EXTPTR)
{
	//f3s_lock_assert(PART_PTR->table.logi_lock_vect[EXTPTR.logi_unit].thread == pthread_self());
	return(PART_PTR->table.logi_vect[EXTPTR.logi_unit]);
}
#endif	/* NDEBUG */

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/ext.c $ $Rev: 580669 $" );
