/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int f3s_create (resmgr_context_t *ctp, io_open_t *msg, f3s_attr_t *dir_attr,
                f3s_attr_t *file_attr, f3s_desc_t *dir_desc,
                f3s_desc_t *file_desc, f3s_stat_t *file_stat, uint8_t moves)
{
	f3s_desc_t		dirent_desc;
	f3s_part_t *	part;
	f3s_stat_t		dir_stat;
	f3s_share_t *	dirent_share;
	char*		name;
	int32_t			dirent_size;
	int32_t			update_size;
	int16_t			length;
	int				error;

	f3s_lock_assert((dir_attr->attr.lock_tid == pthread_self()) && (dir_attr->attr.lock_count > 0));
	f3s_lock_assert((file_attr->attr.lock_tid == pthread_self()) && (file_attr->attr.lock_count > 0));

	/* get partition pointer */
	part = dir_attr->part;

	/* update times in file attributes */
	iofunc_time_update (&file_attr->attr);

	/* setup file stat */
	file_stat->struct_size = sizeof(*file_stat);
	file_stat->mode = file_attr->attr.mode;

	file_stat->uid = file_attr->attr.uid;
	file_stat->gid = file_attr->attr.gid;

	file_stat->ctime = file_attr->attr.ctime;
	file_stat->mtime = file_attr->attr.ctime;

	/* setup depth in file attributes */
	file_attr->depth = dir_attr->depth + 1;

	/* initialize name pointer */
	name = f3s_path_last (msg->connect.path, &length);

	/* create the directory entry */
	dirent_size = f3s_entry_create (part, dir_desc, file_desc->first, name, length,
	                                file_stat, file_attr->depth, moves);
	if (dirent_size < 0) return (errno);

	/* find directory share in share list */
	f3s_lock_assert((dir_attr->part_attr->attr.lock_tid == pthread_self()) && (dir_attr->part_attr->attr.lock_count > 0));
	dirent_share = (f3s_share_t *)f3s_list_find (&dir_attr->part_attr->share_list,
	                                             sizeof(dir_desc->first),
	                                             (uint8_t *)&dir_desc->first);
	if (dirent_share)
	{
		/* invalidate sharing directories */
		f3s_attr_inval (NULL, dirent_share->ocb->ocb.attr, F3S_DESC_NO_LAST,
		                F3S_FILE_MAX, 0, F3S_DESC_KEEP);
	}

	/* knock down dirty time flag */
	file_attr->attr.flags &= ~IOFUNC_ATTR_DIRTY_TIME;

	/* if parent time update is wanted */
	if (f3s.update_level >= F3S_UPDATE_PARENT)
	{
		/* update times in parent attributes */
		iofunc_time_update (&dir_attr->attr);

		f3s_lock_assert((dir_attr->attr.lock_tid == pthread_self()) && (dir_attr->attr.lock_count > 0));
		/* initialize dirent descriptor */
		f3s_desc_init (&dirent_desc, dir_attr->parent_extptr, ~F3S_DESC_NO_DIR);

		/* read parent directory entry */
		error = f3s_entry_lookup (part, &dirent_desc, NULL, "", 1, &dir_stat,
		                          F3S_CACHE_MAX);
		if (error)
		{
			if ((file_desc->status & F3S_DESC_NO_DIR) == 0)
			{
				int ret = f3s_entry_delete(part, dir_desc, dir_attr->depth);
				if (ret < 0)
					printf("(devf  t%d::%s:%d) f3s_entry_delete() failed for extptr(%d, %d)\n", 
									pthread_self(), __func__, __LINE__,
									file_desc->first.logi_unit, file_desc->first.index);
			}
			return (error);
		}
		
		/* copy modification time */
		dir_stat.mtime = dir_attr->attr.mtime;

		/* update directory entry */
		update_size = f3s_entry_update (part, &dirent_desc, &dir_stat);
		if (update_size < 0)
		{
			int error = errno;
			if ((file_desc->status & F3S_DESC_NO_DIR) == 0)
			{
				int ret = f3s_entry_delete(part, dir_desc, dir_attr->depth);
				if (ret < 0)
					printf("(devf  t%d::%s:%d) f3s_entry_delete() failed for extptr(%d, %d)\n",
									pthread_self(), __func__, __LINE__,
									file_desc->first.logi_unit, file_desc->first.index);
			}
			return (error);
		}
	}

	return (EOK);
}

int f3s_append (resmgr_context_t *ctp, io_write_t *msg, f3s_ocb_t *ocb)
{
	f3s_attr_t *attr;
	f3s_part_t *part;
	uint8_t *buffer;
	int32_t buffer_size;
	int32_t chunk_size;
	int32_t write_size = 0;
	int32_t append_flag = 0;
	int32_t done_size;
	int32_t flush_flag = 0;
	int32_t sync_flags;
	int32_t offset;
	int32_t forward_size;
	int error;

	/* reject unknown xtypes */
	if ((msg->i.xtype & _IO_XTYPE_MASK) != _IO_XTYPE_NONE) return (EINVAL);

	f3s_lock_assert((ocb->ocb.attr->attr.lock_tid == pthread_self()) && (ocb->ocb.attr->attr.lock_count > 0));
	/* get attributes pointer */
	attr = ocb->ocb.attr;
	f3s_lock_assert((attr->attr.lock_tid == pthread_self()) && (attr->attr.lock_count > 0));

	/* remember ocb offset */
	offset = ocb->ocb.offset;

	/* do posix write stuff and checks */
	error = iofunc_write_verify (ctp, msg, &ocb->ocb, NULL);
	if (error) return (error);

	/* get partition information pointer */
	part = attr->part;

	/* check if file needs rewind */
	if (!(ocb->file_desc.status & F3S_DESC_NO_REWIND))
	{
		/* rewind file descriptor */
		f3s_desc_rewind (&ocb->file_desc);

		/* forward file descriptor to offset */
		forward_size = f3s_file_forward (part, &ocb->file_desc, offset);
		if (forward_size < 0) return (errno);

		/* set not rewind and no forward flag in file descriptor */
		ocb->file_desc.status |= F3S_DESC_NO_REWIND | F3S_DESC_NO_FORWARD;

	/* else, check if file needs forward */
	} else if (!(ocb->file_desc.status & F3S_DESC_NO_FORWARD))
	{
		/* forward file descriptor to ocb offset */
		forward_size = f3s_file_forward (part, &ocb->file_desc,
		                                 offset - ocb->file_desc.curr_offset);
		if (forward_size < 0) return (errno);

		/* set no forward flag in file descriptor */
		ocb->file_desc.status |= F3S_DESC_NO_FORWARD;
	}

	/* get sync flags */
	sync_flags = iofunc_sync (ctp, &ocb->ocb, O_DSYNC | _IO_FLAG_WR);

	/* get size of write */
	buffer_size = msg->i.nbytes;

	/* get size of chunk */
	chunk_size = min (buffer_size, F3S_WRITE_MAX);

	/* get buffer pointer */
	buffer = (uint8_t *) (&msg->i + 1);

	/* while write size is smaller than size */
	while (write_size < buffer_size)
	{
		/* if file flag is for append */
		if (ocb->ocb.ioflag & O_APPEND)
		{
			append_flag = 1;

		/* if end of file is known */
		} else if (f3s_desc_end (&ocb->file_desc))
		{
			/* check if we are at end of file */
			if (offset == (f3s_desc_size (&ocb->file_desc) + attr->buffer_pos))
			{
				append_flag = 1;
			}
		}

		/* if we are appending */
		if ((append_flag) && (f3s.buffer_size))
		{
			/* if buffer pointer is null */
			if (!attr->buffer)
			{
				/* initialize reservation pointer */
				attr->resv_ptr = f3s_malloc (sizeof(*attr->resv_ptr));
				if (attr->resv_ptr)
				{
					memset(attr->resv_ptr, 0, sizeof(*attr->resv_ptr));
					
					/* initialize append buffer */
					attr->buffer = f3s_malloc (f3s.buffer_size);
					attr->buffer_pos = 0;
					
					/* if buffer allcoation was successfull */
					if (attr->buffer)
					{
						/* attempt to reserve flash space */
						f3s_attr_resv (attr, f3s.buffer_size);

					} else
					{
						/* free reservation structure */
						f3s_free (attr->resv_ptr, sizeof(*attr->resv_ptr));
						attr->resv_ptr = NULL;
					}
				}
			}

			/* if buffer to write is bigger than append buffer space */
			if ((sync_flags) || !(attr->buffer) ||
				(chunk_size > (f3s.buffer_size - attr->buffer_pos)))
			{
				/* if there is something in append buffer */
				if (attr->buffer_pos)
				{
					/* flush the append buffer to the file */
					done_size = f3s_file_append (part, &ocb->file_desc,
					                             attr->resv_ptr, attr->buffer_pos,
					                             attr->buffer, 0);

					/* attempt to reserve flash space again */
					f3s_attr_resv (attr, f3s.buffer_size);

					/* if done size makes sense */
					if (done_size < 0)
					{
						error = errno;
						goto fail_shortcut;
					}

					/* set flush flag */
					flush_flag = 1;

					/* clear the append buffer position */
					attr->buffer_pos = 0;

					/* get current offset */
					offset = f3s_file_forward (part, &ocb->file_desc, F3S_FILE_MAX);
					if (offset < 0)
					{
						error = errno;
						goto fail_shortcut;
					}
				}

				/* if buffer could fit into append buffer */
				if (!(sync_flags) && (attr->buffer) &&
					(chunk_size <= f3s.buffer_size))
				{
					/* copy buffer to append buffer */
					memcpy (attr->buffer, buffer, chunk_size);

					/* set append buffer position */
					attr->buffer_pos += chunk_size;

					/* set done size to chunk size */
					done_size = chunk_size;

				/* else, buffer does not fit into append buffer */
				} else
				{
					/* write with file append function */
					done_size = f3s_file_append (part, &ocb->file_desc, NULL,
					                             chunk_size, buffer, 0);
					if (done_size < 0)
					{
						error = errno;
						goto fail_shortcut;
					}

					/* set flush flag */
					flush_flag = 1;

					/* get current offset */
					offset = f3s_file_forward (part, &ocb->file_desc, F3S_FILE_MAX) -
					         done_size;
					if (offset < 0)
					{
						error = errno;
						goto fail_shortcut;
					}
				}

			/* else, buffer is smaller than append buffer space */
			} else
			{
				/* copy buffer to append buffer */
				memcpy (attr->buffer + attr->buffer_pos, buffer, chunk_size);

				/* set append buffer position */
				attr->buffer_pos += chunk_size;

				/* set done size to chunk size */
				done_size = chunk_size;
			}

		/* else, we are not appending */
		} else
		{
			/* this is not supported */
			error = ENOTSUP;
			goto fail_shortcut;
		}

		/* update offset in file */
		offset += done_size;

		/* update write size */
		write_size += done_size;

		/* check if there is still something to write */
		if (write_size < buffer_size)
		{
			/* get size of chunk */
			chunk_size = min (buffer_size - write_size, F3S_WRITE_MAX);

			/* read message */
			chunk_size = resmgr_msgread (ctp, buffer, chunk_size,
			                             sizeof (msg->i) + write_size);
			if (chunk_size < 0) {error = errno; goto fail_shortcut;}
			if (chunk_size == 0) break;
		}
	}

fail_shortcut:
	/* check if flush was performed */
	if (flush_flag)
	{
		/* invalidate sharing ocb decriptors */
		f3s_attr_inval (ocb, attr, F3S_DESC_NO_LAST, 0, 0, F3S_DESC_NO_REWIND);
	}

	/* reset proper offset */
	ocb->ocb.offset = offset;

	/* check for error other than no space and nothing was written */
	if ((error) && ((error != ENOSPC) || !(write_size))) return (error);

	_IO_SET_WRITE_NBYTES (ctp, write_size);
	return (EOK);
}

int f3s_namei (resmgr_context_t *ctp, io_open_t *msg, f3s_attr_t *part_attr,
               f3s_attr_t *dir_attr, f3s_desc_t *dir_desc,
               struct _client_info *info)
{
	f3s_dirent_t	dirent;
	f3s_stat_t		stat;
	f3s_part_t *	part;
	char *			name;
	int32_t			end;
	int32_t			name_size;
	int32_t			resolve = 0;
	int32_t			unresolve;
	int16_t			length;
	int				error;

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));

	/* get partition pointer */
	part = part_attr->part;

	/* get client info */
	error = iofunc_client_info (ctp, msg->connect.ioflag, info);
	if (error)
		return (error);

	/* copy partition attributes to directory attributes */
	*dir_attr = *part_attr;

	f3s_lock_assert((dir_attr->attr.lock_tid == pthread_self()) && (dir_attr->attr.lock_count > 0));
	/* setup directory entry descriptor from extptr in partition attributes */
	f3s_desc_init (dir_desc, part_attr->parent_extptr, ~F3S_DESC_NO_DIR);

	/* check if pathname is too long */
	if (msg->connect.path_len > F3S_PATH_MAX)
		return (ENAMETOOLONG);

	/* allocate space for name */
	name = alloca (msg->connect.path_len);
	if (!name)
	{
		f3s_assert (name);
		return (ENOMEM);
	}

	/* copy name from message */
	memcpy (name, msg->connect.path, msg->connect.path_len);

	if (msg->connect.path_len == 1)
	{
		/******************************************************** 
		These next three lines are a patch to fix the permissions
		on the root directory, the problem is namei() and f3s_entry_lookup()
		return differnt attr's for the root directory, so the modes weren't right.
		*********************************************************/
		error = f3s_entry_lookup (part, dir_desc, &dirent, name,
		                          msg->connect.path_len, &stat, dir_attr->depth);
		if (error != EOK)
			return (error);
			
		dir_attr->attr.mode  = stat.mode;
		part_attr->attr.mode = stat.mode;

		return (EOK);
	}

	/* get to end of path loop */
	while (1)
	{
		/* read parent directory entry */
		error = f3s_entry_lookup (part, dir_desc, &dirent, "", 1, NULL,
		                          F3S_CACHE_MAX);
		if (error != EOK)
			return (error);

		/* setup directory entry descriptor from first in directory entry */
		f3s_desc_init (dir_desc, dirent.first, ~F3S_DESC_NO_DIR);

		/* get first name from path */
		end = f3s_path_first (name, &length);
		if (end) break;

		/* put null at length position minus one */
		name[length - 1] = '\0';

		/* set resolved length */
		resolve += length;

		f3s_lock_assert((dir_attr->attr.lock_tid == pthread_self()) && (dir_attr->attr.lock_count > 0));
		/* find the file entry */
		error = f3s_entry_lookup (part, dir_desc, &dirent, name, length, &stat,
		                          dir_attr->depth);
		if (error != EOK)
			return (error);

		/* increment depth */
		dir_attr->depth++;

		/* go to next element of path */
		name += length;

		/* check if entry is a symbolic link */
		if (S_ISLNK (stat.mode))
		{
			/* get length of unresolved part of path */
			unresolve = msg->connect.path_len - resolve;

			/* set connect reply for symbolic link */
			_IO_SET_CONNECT_RET(ctp, _IO_CONNECT_RET_LINK);

			/* resolve the link */
			error = f3s_link_resolve (ctp, (io_readlink_t *)msg, part_attr,
			                          dirent.first, resolve - length, NULL);
			if (error != EOK)
				return(error);

			/* get name size from reply (excluding terminating '\0' */
			name_size = msg->link_reply.path_len - 1;

			/* append a '/' to separate the remaining portion of the path  */
			((uint8_t *)(&msg->link_reply + 1))[name_size++] = '/';

			/* copy unresolved part of path (including terminating '\0') */
			memcpy ((uint8_t *)(&msg->link_reply + 1) + name_size,
			        name, unresolve);

			/* adjust path length (include terminating '\0') */
			name_size += unresolve;
			msg->link_reply.path_len = name_size;

			/* return the resolved link (number of bytes does include the terminating NULL) */
			return (_RESMGR_PTR(ctp, &msg->link_reply,
			                    sizeof(msg->link_reply) + name_size));
		}

		/* check if entry is not a directory */
		if (!S_ISDIR(stat.mode)) return (ENOTDIR);

		/* populate directory attributes with stat info */
		f3s_attr_init (dir_attr, 0, dir_attr, dir_desc, &stat, info);

		/* check for access to directory */
		error = iofunc_check_access (ctp, &dir_attr->attr, S_IEXEC, info);
		if (error) return (error);

		f3s_lock_assert((dir_attr->attr.lock_tid == pthread_self()) && (dir_attr->attr.lock_count > 0));
		/* setup parent in lookup attributes */
		dir_attr->parent_extptr = dir_desc->curr;

		/* setup directory entry descriptor from extptr in partition attributes */
		f3s_desc_init (dir_desc, dir_attr->parent_extptr, ~F3S_DESC_NO_DIR);
	}

	return (EOK);
}

int f3s_link_resolve (resmgr_context_t *ctp, io_readlink_t *msg,
                      f3s_attr_t *part_attr, f3s_extptr_t first,
                      int16_t length, int *ret_val)
{
	f3s_desc_t	file_desc;
	f3s_part_t *part;
	uint8_t *	name;
	int32_t		name_size;
	uint8_t		leading;
uint_t  eflag;
uint_t  ftype;

	f3s_lock_assert((part_attr->attr.lock_tid == pthread_self()) && (part_attr->attr.lock_count > 0));
	/* get partition pointer */
	part = part_attr->part;

	/* setup file descriptor */
	f3s_desc_init (&file_desc, first, F3S_DESC_INIT);

	eflag = msg->connect.eflag;
	ftype = msg->connect.file_type;
	memset(&msg->link_reply, 0, sizeof(msg->link_reply));
	msg->link_reply.file_type = ftype;
	msg->link_reply.eflag = eflag;

	/* clear nentries in link reply and copy eflags */
	msg->link_reply.nentries = 0;
	msg->link_reply.eflag = msg->connect.eflag;

	/* setup name pointer */
	name = (uint8_t *)(&msg->link_reply + 1);

	/* read the link pathname */
	name_size = f3s_file_read (part, &file_desc, 0, 1, &leading);
	if (name_size < 0)
	{
		return(EIO);
	}

	/* check for absolute path name link (ie. leading slash) */
	if (length && leading != '/')
	{
		/* move unresolve part of name into name */
		memmove (name, msg->connect.path, length);

		/* add prefix size to name pointer */
		name += length;

	/* else, do not care about prefix */
	} else
	{
		length = 0;
	}

	/* read the link pathname */
	name_size = f3s_file_read (part, &file_desc, 0, F3S_NAME_MAX, name);
	if (name_size < 0)
		return (EIO);

	/* terminate the name string and include termination in size */
	name[name_size] = '\0';
	name_size += (length + 1);

	/* copy name size in message reply. Include the terminating NULL */
	msg->link_reply.path_len = name_size;
	if (ret_val != NULL)
		*ret_val = _RESMGR_PTR(ctp, &msg->link_reply, sizeof(msg->link_reply) + name_size);

	return(EOK);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s.c $ $Rev: 580669 $" );
