/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 *
 * You must obtain a written license from and pay applicable license fees to QNX
 * Software Systems before you may reproduce, modify or distribute this software,
 * or any work that includes all or part of this software.   Free development
 * licenses are available for evaluation and non-commercial purposes.  For more
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *
 * This file may contain contributions from others.  Please review this entire
 * file for other proprietary rights or license notices, as well as the QNX
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/
 * for other information.
 * $
 */




/*
** File: f3s.h
**
** Description:
**
** This file contains the definitions for the f3s resource manager for
** Neutrino
**
** Ident: $Id: f3s.h 580669 2011-09-01 15:18:50Z builder@qnx.com $
*/

/*
** Include Loop Prevention
*/

#ifndef __F3S_H_INCLUDED
#define __F3S_H_INCLUDED

/*
** Pre-Includes
*/

#include <libc.h>
#include <dirent.h>
#include <devctl.h>
#include <paths.h>
#include <inttypes.h>
#include <pthread.h>
#include <gulliver.h>
#include <sys/rsrcdbmgr.h>
#include <sys/procmgr.h>
#include <sys/neutrino.h>
#include <sys/dcmd_blk.h>
#include <sys/dcmd_f3s.h>
#include <fs/f3s_api.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>
#include "f3s_trace.h"
#include "f3s_inline.h"
#include "f3s_log.h"
#include "f3s_macro.h"
#include <sys/neutrino.h>

/*
** Constant Definitions
*/


/* General Purpose Constants */

#define F3S_WEAR_SLOT    7		/* wear leveling slot, has to be pow2-1 */

#define F3S_DATA_MAX     0x4000	/* max data size for resmgr */
#define F3S_WRITE_MAX    0x1000	/* max data size for resmgr */
#define F3S_READ_MAX     F3S_DATA_MAX	/* max data size for resmgr */
#define F3S_MSG_MAX      (F3S_DATA_MAX+sizeof(resmgr_iomsgs_t)+sizeof(struct _xtype_offset))
									 /* max message size for resmgr */
#define F3S_INDEX_MAX    0x10	/* max number of sockets or partitions */
#define F3S_CACHE_MAX    0x10	/* max number of lookup extent caches */

#define F3S_BYTE_ERASE   0xff	/* set all bits to one in a byte */
#define F3S_WORD_ERASE   0xffff	/* set all bits to one in a word */
#define F3S_DWORD_ERASE  0xffffffff	/* set all bits to one in a dword */

#define F3S_LATENCY_SIZE 64		/* xip size for low latency */
#define F3S_BUFFER_SIZE  512	/* default size of append buffers */

#define F3S_THREAD_NUM   2		/* default number of threads */
#define F3S_THREAD_MAX   100	/* allow a lot of threads */	

#define F3S_ATTACH_SKT   1		/* skt attach id in vector */
#define F3S_ATTACH_RAW   0		/* raw attach id in vector */
#define F3S_ATTACH_F3S   0		/* f3s attach id in vector */
#define F3S_ATTACH_NUM   2		/* number of attach id in vector */

#define F3S_DIRTY        1		/* Dirty flag for blocks on the spare list */
#define F3S_NOT_DIRTY    0		/* Not Dirty flag for blocks on the spare list */

#define MAX_GEOINFO_GEOS 16		/* Max num of geos in dcmd_getgeo() structure */


/* Descriptor Status Constants */

#define F3S_DESC_KEEP        0x0000	/* descriptor keep status */
#define F3S_DESC_NO_LAST     0x0001	/* descriptor last extent not known */
#define F3S_DESC_NO_DIR      0x0002	/* descriptor is for file */
#define F3S_DESC_NO_REWIND   0x0004	/* descriptor needs no rewind */
#define F3S_DESC_NO_FORWARD  0x0008	/* descriptor needs no rewind */
#define F3S_DESC_XCACHE      0x0010	/* descriptor needs xcache */
#define F3S_DESC_INIT        0xffff	/* descriptor initial status */

/* Table Status Constants */

#define F3S_TABLE_NO_SORT    0x0001	/* partition table not sorted */
#define F3S_TABLE_NO_ATTACH  0x0002	/* partition file system not attached */

/* File Limits Constants */

#define F3S_FILE_MAX  INT_MAX	/* maximum size of a file */

/* Base Type String Constant */

#define F3S_BASE_TYPE  FLASH_FS_TYPE	/* base type string */

/*
** Forward Declarations
*/

struct f3s_mount_s;
struct f3s_attr_s;
struct f3s_ocb_s;

/*
** Enum Definitions
*/

/* Times Update Level Enumeration */

typedef enum
{
	F3S_UPDATE_NEVER,			/* never update times */
	F3S_UPDATE_FILE,			/* update files for times */
	F3S_UPDATE_PARENT,			/* update files and parent for times */
	F3S_UPDATE_NUM				/* number of update levels */
}
f3s_update_level_t;

/* Flash Verify Enumeration */

typedef enum
{
	F3S_VERIFY_NONE,			/* never verify times */
	F3S_VERIFY_WROP,			/* verify for writes */
	F3S_VERIFY_EROP,			/* verify for erases */
	F3S_VERIFY_WREROP,			/* verify all */
	F3S_VERIFY_NUM				/* number of verify levels */
}
f3s_verify_level_t;

/* Commit Action Enumeration */

typedef enum
{
	F3S_COMMIT_APPEND,			/* commit append action */
	F3S_COMMIT_OVER,			/* commit overwrite action */
	F3S_COMMIT_NUM				/* number of commit actions */
}
f3s_commit_action_t;

/* Fit Direction Enumeration */

typedef enum
{
	F3S_FIT_FORWARD,			/* fit forward direction */
	F3S_FIT_BACKWARD,			/* fit backward direction */
	F3S_FIT_NUM					/* number of fit directions */
}
f3s_fit_direction_t;

/*
** Structure Definitions
*/

/* General purpose list definition */

typedef struct f3s_list_s
{
	struct f3s_list_s *next;	/* next entry pointer */
	struct f3s_list_s *prev;	/* previous entry pointer */
}
f3s_list_t;

/* Spare list entry */

typedef struct
{
	f3s_list_t list;			/* list links */
	int32_t phys;				/* physical index */
	int dirty;					/* dirty flag */
	uint32_t erase_count;
}
f3s_spare_t;

/* Copy list entry */

typedef struct
{
	f3s_list_t list;			/* list links */
	uint32_t index;				/* index of copy entry */
}
f3s_copy_t;

/* OCB share list entry */

typedef struct
{
	f3s_list_t list;			/* list links */
	f3s_extptr_t first;			/* first extent pointer of file */
	struct f3s_ocb_s *ocb;		/* ocb pointer */
}
f3s_share_t;

/* ATTR mount pair list entry */

typedef struct
{
	f3s_list_t list;			/* list links */
	struct f3s_attr_s *f3s_attr;	/* file system attr pointer */
	struct f3s_attr_s *raw_attr;	/* raw attr pointer */
}
f3s_pair_t;

/* Lookup Cache Definition */

typedef struct
{
	f3s_extptr_t curr;			/* current extent pointer */
	int32_t curr_offset;		/* offset of current file extent */
}
f3s_cache_t;

/* Chip Resource Definition */

typedef struct
{
	pthread_mutex_t command_mutex;	/* chip command mutex */
	pthread_cond_t erase_cond;	/* chip erase condition variable */
	int32_t use_flag;			/* chip in use flag */
	int32_t erase_unit;			/* unit number getting erased (none == -1) */
}
f3s_chip_t;

/* Unit map definition */

typedef struct
{
	uint32_t text_size;			/* allocated text size for unit */
	uint32_t free_size;			/* free size for unit */
	uint32_t stale_size;		/* stale size for unit */
	uint32_t pad_size;			/* size lost for alignment padding */
	uint32_t resv_size;			/* size reserved for privileged operations */
	uint32_t erase_count;		/* erase count */
	uint16_t head_last;			/* last header index for unit */
	uint16_t del_num;			/* number of deleted headers */
	uint16_t free_first;		/* first free index before last for unit */
	uint16_t free_num;			/* number of free index before last for unit */
	uint16_t resv_last;			/* last free index reserved for unit */
	uint16_t resv_num;			/* number of free index reserved for unit */
}
f3s_map_t;

/* Sort vector element */

typedef struct
{
	int32_t logi;                           /* logical unit number */
	int32_t phys;				/* physical unit number */
	uint32_t score;				/* points in score */
}
f3s_sort_t;

/* Critical section lock */

typedef struct
{
	uint16_t thread;			/* thread owner of the lock */
	int16_t count;  			/* counter for lock recursion */
	pthread_mutex_t mutex;
}
f3s_lock_t;

/* Reservation definition */

typedef struct
{
	uint16_t logi;				/* logical unit of reservation */
	int16_t free_flag;			/* header served in unit of reservation */
	uint32_t size;				/* size of reservation (plus header if free_flag==0) */
	uint_t  allocated;			/* boolean TRUE == 1, FALSE == 0 */
}
f3s_resv_t;

/* structure for the reclaim-trigger-related data */

typedef struct{
	int8_t enable;				/* Use real reclaim triggers, instead of effective */
	int8_t padding;				/* pad out the structure */
	int16_t etrigger;	/* Effective reclaim trigger */
	int16_t ethreshold; /* Effective reclaim threshold */
	int16_t trigger;	/* reclaim trigger percent stale over free */
	int16_t threshold;	/* reclaim threshold supersede limit */
}f3s_reclaimcfg_t;


/* Partition table definition */

typedef struct
{
	/* table lock */
	uint32_t status;           	/* status for table */
	f3s_sort_t *sort_vect;     	/* vector of sorted units */
	uint32_t reclaim_count;    	/* number of reclaims done (used with wear leveling) */
	f3s_resv_t  deletion_resv; 	/* reservation to guarantee entry deletions */

	/* no lock */
	int16_t swap_flag;         	/* swap necessary flag */
	uint16_t root_size;        	/* root extent size */
	uint16_t unit_spare;       	/* number of spare units (includes bad units) */
	uint16_t unit_retire;      	/* number of retired units */
	uint16_t align_size;       	/* size used for alignement */
	uint16_t align_pow2;       	/* power used for alignement */
	f3s_extptr_t boot_extptr;  	/* boot info extent pointer */
	f3s_extptr_t root_extptr;  	/* root info extent pointer */

	/* alloc lock */
	f3s_lock_t alloc_lock;
	uint32_t head_num;         	/* number of headers in total */
	uint32_t dirent_num;       	/* number of dirent extents except zero sized */
	uint32_t free_total;       	/* total free space available */
	uint32_t stale_total;		/* total stale space available */
	uint32_t pad_total;        	/* total lost space for alignment padding */
	uint32_t resv_total;       	/* total reserved space */

	/* cache lock */
	f3s_lock_t cache_lock;
	f3s_cache_t *cache_vect;   	/* lookup cache vector */

	f3s_lock_t *logi_lock_vect;	/* logical unit lock vector */
	uint16_t *logi_vect;       	/* vector for logical to physical match */

	f3s_lock_t spare_lock;     	/* spare list lock */
	f3s_list_t spare_list;     	/* list of spare units */

	/* table lock */
	f3s_extptr_t *obsol_vect;  	/* obsolete vector for reclaims */
	uint8_t *text_buffer;      	/* text buffer for reclaims */
	uint16_t *copy_vect;       	/* vector of indexes to copy */
	uint16_t copy_max;         	/* max entries in copy vector */
	uint16_t obsol_max;        	/* max entries in obsolete vector */
	uint16_t text_size;        	/* size of text buffer for reclaims */
	f3s_reclaimcfg_t reclaim;	/* Reclaim configuration information */
}
f3s_table_t;

/* Partition Chip Array Definition */

typedef struct
{
	uint32_t index;				/* index of array for logs */
	uint16_t unit_pow2;			/* size shift of a unit */
	uint16_t chip_pow2;			/* size shift of a chip */
	uint16_t unit_total;		/* total number of units */
	uint16_t chip_total;		/* total number of chips */
	uint32_t unit_size;			/* size of a unit */
	uint32_t access_num;		/* socket access lock vector count */
	pthread_key_t window_key;	/* window access key for threads */
	f3s_access_t *access_vect;	/* access vector pointer */
	f3s_socket_t *socket;		/* socket structure pointer */
	f3s_flash_t *flash_vect;	/* flash services vector */
	f3s_flash_t *flash;			/* flash services for array */
	f3s_dbase_t dbase;			/* flash data base for array */
	f3s_chip_t *chip_vect;		/* chip resources vector */
	f3s_chip_t **unit_vect;		/* unit map to chip resources vector */
	uint16_t *phys_vect;		/* vector for physical to logical match */
	uint8_t *age_vect;			/* vector for logical with physical age */
	int8_t *break_vect;			/* vector for unit partition breaks */
	f3s_lock_t *map_lock_vect;	/* unit map lock vector */
	f3s_map_t *map_vect;		/* vector of unit maps */
	int	resume;					/* resume unit for lockdown suspend */
}
f3s_array_t;

/* Extended cache (xcache) to reduce seek time
 *
 * Cache the position of every block of an opened file. Each file system 
 * partition has a xcache table and each table can handle 64 files in default. 
 * To speed up cache lookup and save memory, a page scheme is adopted. That
 * means several cache entries are organized into a page. The page can be 
 * allocated from xcache memory pool. The pool size depends on the total size of
 * all file system partitions. The raw partitions will be ignored.
 *
 *  
 * The block size and maximum file number supported by xcache solution can be
 * changed via command line options. However, the number of cache entries per 
 * page should only be changed at compile time.
 *
 */
#define XCACHE_FILE_MAX                 64        //At maximum 64 files
#define XCACHE_BLOCK_SIZE               0x20000   //one cache entry coveres 128K file size
#define XCACHE_ENTRIES_PER_PAGE         16        //16 cache entries per page

typedef struct f3s_xcache_page
{
	f3s_cache_t cache[XCACHE_ENTRIES_PER_PAGE];
	struct f3s_xcache_page *next;
	
} f3s_xcache_page_t;

typedef struct 
{
	f3s_extptr_t inode;
	f3s_xcache_page_t *page;
#define XCACHE_STATUS_EMPTY		(0)
#define XCACHE_STATUS_INUSE		(1 << 0)
#define XCACHE_STATUS_EXPIRED           (1 << 1)
	uint16_t status;
	uint16_t page_num;
	uint32_t entries;
	f3s_cache_t *found;
	f3s_cache_t *last;
} f3s_xcache_t;

/* xcache memory pool structure definition */
typedef struct 
{
	void *baseaddr;         /* start address of the pool */
	void *buffer_list;      /* buffer list */
	f3s_lock_t lock;

	uint16_t buffer_numb;   /* total number of buffers in the pool*/
	uint16_t buffer_free;   /* current number of free buffers */
	uint16_t buffer_size;   /* size of the every pool's buffer */

} f3s_xpool_t;


/* Partition Structure Definition */

typedef struct
{
	uint32_t index;				/* index of partition for logs */
	uint32_t pulse_connect;		/* pulse connection */
	int sync_code;			/* sync pulse code */
	int bkgd_code;			/* background reclaim pulse code */
	uint32_t bkgd_count;		/* background reclaim pulse count */
	uint16_t unit_first;		/* first unit index */
	uint16_t unit_num;			/* number of units */
	f3s_lock_t table_lock;		/* partition table lock */
	f3s_table_t table;			/* partition allocation table */
	f3s_array_t *array;			/* partition chip array pointer */
	
	f3s_xcache_t *xcache; 
	f3s_lock_t  xlock;
	f3s_xpool_t xpool; 
}
f3s_part_t;

/* File Descriptor Definition */

typedef struct
{
	uint16_t status;			/* status of descriptor */
	uint16_t last_size;			/* size of last file extent */
	f3s_extptr_t first;			/* first file extent pointer */
	f3s_extptr_t curr;			/* current file extent pointer */
	f3s_extptr_t last;			/* last file extent pointer (may be unknown) */
	int32_t curr_offset;		/* offset of current file extent */
	int32_t last_offset;		/* offset of last file extent */
}
f3s_desc_t;

/*
** Function Prototypes
*/

int f3s_ebadfsys (uint32_t partnum,
				  const char *file,
				  uint32_t line);

void f3s_option (int argc,
				 char **argv,
				 f3s_flash_t * flash_vect);

void f3s_usage (int argc,
				char **argv,
				char* message);

void f3s_bkgd_init (f3s_part_t * part);

void f3s_bkgd_signal (f3s_part_t * part);

void f3s_bkgd_kill (f3s_part_t * part);

int f3s_part_attach (f3s_part_t * part,
					 struct f3s_attr_s *raw_attr,
					 struct f3s_attr_s *f3s_attr,
					 uint32_t index);

int f3s_part_detach (f3s_part_t *part,
					 struct f3s_attr_s *raw_attr,
					 struct f3s_attr_s *f3s_attr);

int f3s_table_find (f3s_part_t *part);

void f3s_table_sort (f3s_part_t * part,
					 int32_t local);

int f3s_array_attach (f3s_array_t *array,
					  struct f3s_attr_s *skt_attr);

int f3s_array_calc (f3s_array_t *array);

int f3s_array_alloc (f3s_array_t *array);

int f3s_array_free (f3s_array_t *array);

int f3s_array_detach (f3s_array_t *array,
					  struct f3s_attr_s *skt_attr);

int f3s_skt_attach (f3s_array_t * array,
						struct f3s_attr_s *skt_attr,
						f3s_flash_t * flash_vect);

int f3s_skt_detach (f3s_array_t * array,
						struct f3s_attr_s *skt_attr);

int f3s_recover_boot (f3s_part_t *part);

int f3s_recover_reclaim (f3s_part_t *part);

int f3s_recover_write (f3s_part_t *part,
					   int32_t *reclaim_vect);

int f3s_recover_link (f3s_part_t *part,
					  uint8_t **link_matrix);

int f3s_recover_delete (f3s_part_t *part,
						uint8_t **link_matrix);

int f3s_cleanup_multi (f3s_part_t *part,
					   f3s_extptr_t multi);

int32_t f3s_root_read (f3s_part_t * part,
					   f3s_dirent_t * dirent,
					   char* name,
					   f3s_stat_t * stat);

int f3s_root_write (f3s_part_t *part,
					int32_t phys,
					f3s_extptr_t first,
					char *name,
					int16_t length,
					uint32_t ctime,
					uint16_t align_pow2);

void f3s_boot_swap (f3s_boot_t * boot);

int f3s_boot_read (f3s_part_t *part,
				   int32_t phys,
				   f3s_boot_t *boot);

int f3s_boot_write (f3s_part_t * part,
					int32_t phys,
					f3s_extptr_t root,
					uint16_t unit_num,
					uint16_t unit_spare,
					uint16_t align_pow2,
					uint16_t index);

void f3s_unit_info_swap (f3s_unit_info_t * unit);

void f3s_unit_logi_swap (f3s_unit_logi_t * unit);

int f3s_unit_info_read (f3s_part_t *part,
					    int32_t phys,
					    f3s_unit_info_t *unit,
					    int fix);

int f3s_unit_logi_read (f3s_part_t *part,
					    int32_t phys,
					    f3s_unit_logi_t *logi);

int f3s_unit_write (f3s_part_t *part,
					int32_t phys,
					uint32_t erase_count,
					f3s_extptr_t boot);

int f3s_unit_logi1 (f3s_part_t *part,
				    int32_t *phys,
				    uint16_t logi,
				    uint8_t age,
				    int recover);

int f3s_unit_logi2 (f3s_part_t *part,
				    int32_t *phys,
				    uint16_t logi,
				    uint8_t age,
				    int recover);

int f3s_unit_logi (f3s_part_t *part,
				   int32_t *phys,
				   uint16_t logi,
				   uint8_t age,
				   int recover);

void f3s_unit_stale (f3s_part_t * part,
					 int32_t phys,
					 int32_t text_size);

int32_t f3s_unit_alloc (f3s_part_t * part,
						int32_t phys,
						int32_t text_size,
						int32_t strict);

int f3s_unit_reclaim (f3s_part_t *part,
					  int32_t reclaim_phys,
					  int32_t spare_phys,
					  uint32_t *obsol_num);

int f3s_unit_refresh (f3s_part_t *part,
					  int32_t phys);

int32_t f3s_spare_grab (f3s_part_t * part, int* dirty);

int f3s_unit_inval (f3s_part_t *part, int32_t phys);

void f3s_spare_drop (f3s_part_t * part,
					 int32_t phys, int dirty);

int32_t f3s_frgd_reclaim (f3s_part_t * part,
						  int32_t reclaim_phys);

void f3s_lock_init(f3s_lock_t *lock);

void f3s_lock_free(f3s_lock_t *lock);

void f3s_lock_enter (f3s_lock_t * lock);

void f3s_lock_leave (f3s_lock_t * lock);

int32_t f3s_alloc_fit (f3s_part_t * part,
					   int32_t text_size,
					   f3s_fit_direction_t direction);

int32_t f3s_ext_alloc (f3s_part_t * part,
					   int32_t *phys,
					   uint32_t * text_offset,
					   int32_t text_size,
					   f3s_extptr_t * extptr,
					   uint32_t * mask,
					   uint16_t align_pow2);

int32_t f3s_ext_write (f3s_part_t * part,
					   int32_t *phys,
					   uint32_t index,
					   uint32_t text_offset,
					   int32_t text_size,
					   uint8_t * text,
					   uint32_t mask,
					   uint16_t align_pow2,
					   int meta,
					   int recover);

int32_t f3s_extptr_fresh (f3s_part_t * part,
						  f3s_extptr_t extptr,
						  uint32_t * super_count,
						  uint32_t * text_offset,
						  f3s_head_t * head,
						  f3s_extptr_t * fresh);

int32_t f3s_extptr_basic (f3s_part_t * part,
						  f3s_extptr_t extptr,
						  uint32_t * text_offset,
						  f3s_head_t * head);

int32_t f3s_extptr_delete (f3s_part_t * part,
						   f3s_extptr_t extptr,
						   f3s_head_t * head);

void f3s_extptr_swap (f3s_extptr_t * extptr);

int32_t f3s_resv_alloc (f3s_part_t * part,
						f3s_resv_t * resv,
						int32_t size);

void f3s_resv_free (f3s_part_t * part,
					f3s_resv_t * resv);

void f3s_head_swap (f3s_head_t * head);

int32_t f3s_head_check (f3s_part_t * part,
						int32_t phys,
						uint32_t index,
						uint32_t * status);

int32_t f3s_head_read (f3s_part_t * part,
					   int32_t phys,
					   uint32_t index,
					   uint32_t * text_offset,
					   f3s_head_t * head,
					   int fix);

int32_t f3s_head_write (f3s_part_t *part, int32_t *phys, uint32_t index,
                        f3s_head_t *head, int recover);

int32_t f3s_head_next (f3s_part_t * part, int32_t *phys, uint32_t index,
					   f3s_extptr_t next, int recover);

int32_t f3s_head_super (f3s_part_t * part, int32_t *phys, uint32_t index,
						f3s_extptr_t super, int recover);

int32_t f3s_head_status (f3s_part_t *part, int32_t *phys, uint32_t index,
                         uint32_t status, int recover);

int f3s_head_repair (f3s_part_t *part, int32_t *phys, uint32_t index,
                     f3s_head_t *head);

int32_t f3s_path_first (char* path,
						int16_t * length);

char* f3s_path_last (char* path,
						int16_t * length);

void f3s_list_add (f3s_list_t * list,
				   f3s_list_t * new);

void f3s_list_rem (f3s_list_t * old);

int32_t f3s_list_drain (f3s_list_t * list,
						int32_t entry_size);

f3s_list_t *f3s_list_find (f3s_list_t * list,
						   int32_t entry_size,
						   uint8_t * buffer);

void f3s_reclaim_sort (f3s_part_t * part,
					   uint32_t * copy_num,
					   int32_t phys);

int f3s_reclaim_copy (f3s_part_t *part,
					  uint32_t copy_num,
					  uint32_t *obsol_num,
					  uint32_t free_size,
					  int32_t phys,
					  int32_t spare_phys,
					  uint32_t resv_num);

int f3s_reclaim_clean (f3s_part_t *part,
					   uint32_t obsol_num);

int32_t f3s_file_create (f3s_part_t * part,
						 f3s_desc_t * desc,
						 f3s_resv_t * resv,
						 int32_t buffer_size,
						 uint8_t * buffer,
						 int meta);

int32_t f3s_file_read (f3s_part_t * part,
					   f3s_desc_t * desc,
					   int32_t desc_offset,
					   int32_t buffer_size,
					   uint8_t * buffer);

int32_t f3s_file_write (f3s_part_t * part,
						f3s_desc_t * desc,
						int32_t desc_offset,
						int32_t buffer_size,
						uint8_t * buffer);

int32_t f3s_file_append (f3s_part_t * part,
						 f3s_desc_t * desc,
						 f3s_resv_t * resv,
						 int32_t buffer_size,
						 uint8_t * buffer,
						 int meta);

int32_t f3s_file_grow (f3s_part_t * part,
					   f3s_desc_t * desc,
					   int32_t desc_offset,
					   int32_t buffer_size,
					   uint8_t * buffer);

int32_t f3s_file_commit (f3s_part_t * part,
						 f3s_desc_t * desc,
						 f3s_extptr_t extptr,
						 f3s_commit_action_t commit_action);

int32_t f3s_file_trunc (f3s_part_t * part,
						f3s_desc_t * desc,
						int32_t desc_offset);

int32_t f3s_file_forward (f3s_part_t * part,
						  f3s_desc_t * desc,
						  int32_t forward_size);

void f3s_file_delete (f3s_part_t * part,
					  f3s_desc_t * desc,
					  int32_t delete_size);

int32_t f3s_dirent_read (f3s_part_t * part,
						 f3s_desc_t * desc,
						 int32_t desc_offset,
						 f3s_dirent_t * dirent);

void f3s_dirent_swap (f3s_dirent_t * dirent);

void f3s_stat_swap (f3s_stat_t * stat);

int f3s_entry_lookup (f3s_part_t * part,
					  f3s_desc_t * desc,
					  f3s_dirent_t * dirent,
					  char* name,
					  int16_t length,
					  f3s_stat_t * stat,
					  uint32_t depth);

int32_t f3s_entry_retrace (f3s_part_t * part,
						   f3s_desc_t * desc,
						   f3s_extptr_t first);

int32_t f3s_entry_create (f3s_part_t * part,
						  f3s_desc_t * desc,
						  f3s_extptr_t first,
						  char * name,
						  int16_t length,
						  f3s_stat_t * stat,
						  uint32_t depth,
						  uint8_t moves);

int32_t f3s_entry_change (f3s_part_t * part,
						  f3s_desc_t * desc,
						  f3s_dirent_t * dirent,
						  char * name,
						  int16_t length,
						  f3s_stat_t * stat,
						  uint8_t moves);

int32_t f3s_entry_update (f3s_part_t * part,
						  f3s_desc_t * desc,
						  f3s_stat_t * stat);

int32_t f3s_entry_delete (f3s_part_t * part,
						  f3s_desc_t * desc,
						  uint32_t depth);

int f3s_sort_comp (const f3s_sort_t * sort1,
				   const f3s_sort_t * sort2);

int f3s_flash_read (f3s_array_t * array,
						int32_t phys,
						int32_t text_offset,
						int32_t buffer_size,
						uint8_t * buffer);

int f3s_flash_map (f3s_array_t * array,
					   int32_t phys,
					   int32_t text_offset,
					   int32_t buffer_size,
					   uint8_t ** memory,
					   int32_t * resume_flag);

void f3s_flash_unmap (f3s_array_t * array,
					  int32_t phys,
					  int32_t text_offset,
					  int32_t resume_flag);

int f3s_flash_write (f3s_array_t * array,
						 int32_t phys,
						 uint32_t text_offset,
						 uint32_t buffer_size,
						 uint8_t * buffer,
						 uint32_t write_flags);

int f3s_flash_mask (f3s_array_t * array,
						int32_t phys,
						int32_t text_offset,
						uint32_t mask);

int f3s_flash_erase (f3s_array_t * array,
						 int32_t phys);

int f3s_flash_islock (f3s_array_t *array,
					  int32_t phys);

int f3s_flash_lock (f3s_array_t *array,
				    int32_t phys);

int f3s_flash_unlock (f3s_array_t *array,
				      int32_t phys);

int f3s_flash_unlockall (f3s_array_t *array);

int f3s_check_frgd_reclaim(f3s_part_t *part, int32_t text_size);

int f3s_check_bkgd_reclaim(f3s_part_t *part);

int f3s_check_stale_total(f3s_part_t *part);

int f3s_check_free_total(f3s_part_t *part, int32_t limit);

void _f3s_verify_counters (f3s_part_t *part, int32_t phys, uint32_t extra_text);

void _f3s_verify_dir (f3s_part_t *part, int32_t logi, uint32_t index);

void _f3s_verify_headers (f3s_part_t *part, int32_t inc_phys);

/* 
** xcache functions 
*/
f3s_xcache_t *f3s_xcache_lookup(f3s_part_t *part, f3s_desc_t *desc, int32_t forward_size);

f3s_xcache_t *f3s_xcache_new(f3s_part_t *part, f3s_desc_t *desc);

void f3s_xcache_update(f3s_part_t *part, f3s_xcache_t *xcache, f3s_desc_t *desc);

void f3s_xcache_expire(f3s_part_t *part, f3s_desc_t *desc);

void f3s_xcache_truncate(f3s_part_t *part, f3s_desc_t *desc, int32_t trunc_offset);

void f3s_xcache_free(f3s_part_t *part, f3s_desc_t *desc);

void f3s_xpool_delete (f3s_xpool_t *pool);

int  f3s_xpool_init(f3s_xpool_t *pool, uint16_t size, uint16_t num);

/*
** Override Defines
*/

#define IOFUNC_MOUNT_T       struct f3s_mount_s
#define IOFUNC_ATTR_T        struct f3s_attr_s
#define IOFUNC_OCB_T         struct f3s_ocb_s
#define THREAD_POOL_PARAM_T  dispatch_context_t

/*
** Post-Includes
*/

#include <sys/iofunc.h>
#include <sys/dispatch.h>
#include "f3s_iofunc.h"


int f3s_sigusr1_handler (f3s_array_t* array);
int f3s_sigusr2_handler (f3s_array_t* array, thread_pool_attr_t* pool_attr);

/* Background Pulse Callout */

void f3s_bkgd_reclaim (resmgr_context_t * ctp,
					   int code,
					   union sigval value,
					   f3s_part_t * part);

/* Sync Pulse Callout */

void f3s_sync_pulse (resmgr_context_t * ctp,
					 int code,
					 union sigval value,
					 f3s_attr_t * part_attr);

/* Global Structure Definition */

/*
 * f3s_mnt_options_t
 *  devf-xxx mount options. Currently, the -R option will cause any detected partitions
 * to be mount readonly. This could have been a boolean but to allow other options
 * in the future, I have coded it as an enum
 * f3s_mntopts_t_OPTION_MASK masks out the enumeration values. These values must
 * be tested for equality
 * f3s_mntopts_t_CTRL_MASK masks out enumeration value control flags. These flags
 * are single bit option flags (booleans) and are used as qualifiers for the option values
*/
#define f3s_mntopts_t_OPTION_MASK		0x00FFFFFFU
#define f3s_mntopts_t_CTRL_MASK			0xFF000000U
typedef enum
{
	f3s_mntopts_t_MOUNT_RW = (0 & f3s_mntopts_t_OPTION_MASK),			/* read/write mount (default) */
	f3s_mntopts_t_MOUNT_RO = (1 & f3s_mntopts_t_OPTION_MASK),			/* read only mount */
	f3s_mntopts_t_MOUNT_ENUM = (2 & f3s_mntopts_t_OPTION_MASK),			/* enumerate only */
		/*    :    */
	f3s_mntopts_t_INIT = ((1 << 24) & f3s_mntopts_t_CTRL_MASK),				/* initialization in progress */
	f3s_mntopts_t_AUTOMNT = ((1 << 25) & f3s_mntopts_t_CTRL_MASK)	/* automount partitions */
} f3s_mntopts_t;


typedef struct
{
	resmgr_attr_t resmgr_attr;	/* resmgr attributes structure */
	thread_pool_t *tpp;			/* resmgr thread pool pointer */
	dispatch_t *dpp;			/* resmgr dispatch pointer */

	resmgr_connect_funcs_t	mnt_connect_funcs;
	resmgr_connect_funcs_t	raw_connect_funcs;
	resmgr_io_funcs_t		raw_io_funcs;
	resmgr_connect_funcs_t	f3s_connect_funcs;
	resmgr_io_funcs_t		f3s_io_funcs;
	resmgr_io_funcs_t		res_io_funcs;
	resmgr_io_funcs_t		dir_io_funcs;
	iofunc_funcs_t			f3s_funcs;	/* functions vector for mount points */

	int8_t verbose;				/* print verbose information flag */
	int8_t recover;				/* recover dangling extents flag */
	int8_t bkgd_flag;			/* background reclaim flag */
	int8_t bkgd_priority;		/* background reclaim priority */
	int8_t flash_verify;		/* flash verify write/erase */
	int8_t update_level;		/* times update level */
	int8_t array_index;			/* start index of array special files */
	int8_t part_index;			/* start index of part special files */
	int8_t thread_num;			/* number of threads */
	int8_t exit_on_fault;		/* exit driver on a fault */
	int8_t log_method;			/* 0=stdout,1=slog, 2=both */
#ifdef ENABLE_MOUNTBEHIND
	int8_t mount_behind;		/* flag to indication resmgr mount after */
#else
	int8_t mount_pad;			/* pad the structure */
#endif
	int8_t enum_only;			/* don't mount the filesystems, only enum the partitions */
	int8_t force_mount;
	int8_t padding[2];				/* pad the structure */
	int32_t buffer_size;		/* write buffer information size */
	char *socket_option;		/* socket service option */
	char *mount_over;			/* mountpoint default override for f3s */
	uint32_t mount_format;		/* mountpoint override format for f3s */
	f3s_mntopts_t  mount_options;

	int32_t mounter_id;			/* mounter id */
	int32_t daemon_id;			/* daemon id */

	int32_t retry;				/* number of retries before failure */

	f3s_attr_t mounter_attr;	/* mounter attributes */
	f3s_reclaimcfg_t reclaim;	/* Reclaim configuration information */

	//xcache globals
	uint32_t xcache_blksz;
	uint16_t xcache_file_max;

	//command-line options for new -e
	char** mountexact_list;
	unsigned mountexact_num;
}
f3s_t;

/*
** External Definitions
*/

extern f3s_t f3s;

extern const f3s_extptr_t f3s_null_extptr, f3s_first_extptr, f3s_root_extptr,
	f3s_boot_extptr;

#endif /* __F3S_H_INCLUDED */

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/f3s.h $ $Rev: 580669 $" )
