/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

int32_t f3s_file_create (f3s_part_t *part, f3s_desc_t *desc, f3s_resv_t *resv,
                         int32_t buffer_size, uint8_t *buffer, int meta)
{
/*{{{1*/
	f3s_extptr_t	new;
	uint16_t		alloc_size;
	int32_t			strict;
	int32_t			append_size;
	int32_t			phys;
	int32_t			write_size;
	uint32_t		text_offset;
	uint32_t		mask;
	int				error;

	/* check if descriptor type is not for directory */
	if (desc->status & F3S_DESC_NO_DIR)
	{
		/* setup mask for file type */
		mask = ~F3S_EXT_TYPE | F3S_EXT_FILE;
		strict = 0;

	/* else, descriptor is for directory */
	} else
	{
		/* setup mask for directory type */
		mask = ~F3S_EXT_TYPE | F3S_EXT_DIR;
		strict = 1;
	}

	F3S_TABLE_ENTER(part);

	/* check if we have a reservation and free it up if so */
	if (resv)
		f3s_resv_free (part, resv);

	/* buffer write loop */
	while (1)
	{
		/* allocate unit with free space; The allocated unit will 
		 * be locked by f3s_alloc_fit() in f3s_unit_alloc().
		 */
		phys = f3s_unit_alloc (part, -1, buffer_size, strict);
		if (phys < 0)
		{
			error = errno;
			F3S_TABLE_LEAVE(part);
			errno = error;
			return (-1);
		}

		/* allocate new extent */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		error = f3s_ext_alloc (part, &phys, &text_offset, buffer_size,
		                            &new, &mask, part->table.align_pow2);
		if (error < 0)
		{
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, part->array->phys_vect[phys]);
			continue;
		}else{
			alloc_size = (uint16_t)error;
		}

		/* check if extent is split */
		if (alloc_size != buffer_size)
		{
			/* clear no split bit in mask */
			mask &= ~F3S_EXT_NO_SPLIT;

			/* log that split event */
			f3s_log_split (part->array->index, part->index, new.logi_unit,
			               new.index, buffer_size, alloc_size);
		}

		/* write new extent with buffer data */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		write_size = f3s_ext_write (part, &phys, new.index, text_offset, alloc_size,
		                            buffer, mask, part->table.align_pow2, meta, 1);

		/* fix mask for further writes */
		mask |= (F3S_EXT_NO_SPLIT | F3S_EXT_LAST);
		if (write_size < 0)
		{
			/* stale the bad extent size for unit */
			f3s_unit_stale (part, phys, alloc_size);
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, part->array->phys_vect[phys]);
			continue;
		}

		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, part->array->phys_vect[phys]);

		/* adjust buffer pointer and size */
		buffer      += write_size;
		buffer_size -= write_size;
		break;
	}

	F3S_TABLE_LEAVE(part);
	
	/* setup first extent pointer in descriptor */
	desc->first = new;
	f3s_desc_rewind (desc);

	/* set end of file in descriptor */
	f3s_desc_last (desc, alloc_size);

	/* check if there is something left to write */
	if (buffer_size)
	{
		/* append rest of buffer to file */
		append_size = f3s_file_append (part, desc, NULL, buffer_size, buffer, meta);

		/* check if append size makes sense, create guarantees size */
		if (append_size < buffer_size)
		{
			/* rewind descriptor */
			f3s_desc_rewind (desc);

			/* roll back previous alloc */
			f3s_file_delete (part, desc, F3S_FILE_MAX);

			/* return invalid create */
			return (-1);
		}
	}
	else	/* else, there is nothing left to write */
	{
		/* clear append size */
		append_size = 0;
	}

	/* return size of data written */
	return (write_size + append_size);
/*}}}1*/
}

void f3s_file_delete (f3s_part_t *part, f3s_desc_t *desc, int32_t delete_size)
{
/*{{{1*/
	f3s_head_t	head;
	int32_t		text_size;

	/* free xcache */
	if (delete_size == F3S_FILE_MAX) 
	{
		if(*(uint32_t *)&desc->curr == *(uint32_t *)&desc->first) 
		{
			f3s_xcache_free(part, desc);
		}
	}


	/* delete extents loop */
	F3S_TABLE_ENTER(part);
	while (1)
	{
		/* delete current extent header */
		text_size = f3s_extptr_delete (part, desc->curr, &head);
		if(text_size < 0) {
			F3S_TABLE_LEAVE(part);
			return;
		}

		/* check if we have deleted enough */
		if (text_size >= delete_size) break;

		/* check if header has no next */
		if (head.status[0] & F3S_EXT_NO_NEXT) break;

		/* remove text size from size to delete */
		delete_size -= text_size;

		/* set current to head next */
		f3s_desc_step (desc, head.next, text_size);
	}
	F3S_TABLE_LEAVE(part);
/*}}}1*/
}

int32_t f3s_file_grow (f3s_part_t *part, f3s_desc_t *desc,
                       int32_t desc_offset, int32_t buffer_size,
                       uint8_t *buffer)
{
/*{{{1*/
	f3s_desc_t	grow_desc;
	uint8_t *	grow_data;
	int32_t		grow_size;
	int32_t		write_size = 0;
	int32_t		create_size;
	int32_t		append_size;
	int32_t		alloc_size;
	int			err_size;
	int			error;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* shortcut growing if there is not enough space */
	F3S_ALLOC_ENTER(part);
	if (desc_offset > (part->table.free_total + part->table.stale_total))
	{
		F3S_ALLOC_LEAVE(part);
		errno = ENOSPC;
		return (-1);
	}
	F3S_ALLOC_LEAVE(part);

	/* set reasonable grow size */
	grow_size = min(desc_offset, F3S_WRITE_MAX);
	alloc_size = grow_size;

	/* allocate and clear space for grow */
	grow_data = f3s_calloc (alloc_size * sizeof(*grow_data));
	if (grow_data == NULL)
	{
		f3s_assert (grow_data);
		errno = ENOMEM;
		return (-1);
	}

	/* initialize grow descriptor */
	f3s_desc_init (&grow_desc, f3s_null_extptr, ~F3S_DESC_XCACHE);

	/* create grow file with clear data */
	create_size = f3s_file_create (part, &grow_desc, NULL, grow_size, grow_data, 0);
	if (create_size < 0)
	{
		error = errno;

		/* free grow data buffer */
		f3s_free (grow_data, alloc_size * sizeof(*grow_data));

		errno = error;
		return (-1);
	}

	/* adjust write size */
	write_size = create_size;

	/* file grow/append loop */
	while (write_size < desc_offset)
	{
		/* set grow size to what is left to write, reasonably */
		grow_size = min(desc_offset - write_size, F3S_WRITE_MAX);

		/* append that data to file */
		append_size = f3s_file_append (part, &grow_desc, NULL, grow_size, grow_data, 0);
		if (append_size < grow_size)
		{
				error = errno;

				/* rewind grow descriptor */
				f3s_desc_rewind (&grow_desc);

				/* delete grow descriptor */
				f3s_file_delete (part, &grow_desc, F3S_FILE_MAX);

				/* free grow data buffer */
				f3s_free (grow_data, alloc_size * sizeof(*grow_data));

				errno = error;
				return (-1);
		}

		/* add append size to write size */
		write_size += append_size;
	}

	/* check if there is a buffer */
	if (buffer_size) {
		/* append buffer to grow file */
		write_size = f3s_file_append (part, &grow_desc, NULL,
		                              buffer_size, buffer, 0);
	}
	if (write_size > 0)
	{
		/* commit the grow descriptor to file descriptor */
		err_size = f3s_file_commit (part, desc, grow_desc.first,
		                            F3S_COMMIT_APPEND);
		if (err_size < 0)
		{
			error = errno;

			/* rewind grow descriptor */
			f3s_desc_rewind (&grow_desc);

			/* delete grow descriptor */
			f3s_file_delete (part, &grow_desc, F3S_FILE_MAX);

			/* free grow data buffer */
			f3s_free (grow_data, alloc_size * sizeof(*grow_data));

			errno = error;
			return (-1);
		}

		/* set and move current to grow desc last */
		error = f3s_file_forward (part, desc, grow_desc.last_offset);
		if (error < 0) return (error);

		/* set last in descriptor */
		f3s_desc_last (desc, grow_desc.last_size);

	} else
	{
		/* rewind grow descriptor */
		f3s_desc_rewind (&grow_desc);

		/* delete grow descriptor */
		f3s_file_delete (part, &grow_desc, F3S_FILE_MAX);
	}

	/* free grow data buffer */
	f3s_free (grow_data, alloc_size * sizeof(*grow_data));
	return (write_size);
/*}}}1*/
}

int32_t f3s_file_trunc (f3s_part_t *part, f3s_desc_t *desc, int32_t desc_offset)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_extptr_t	curr;
	f3s_desc_t		over_desc;
	uint8_t *		before_data = NULL;
	int32_t			text_size;
	int32_t			write_size;
	int32_t			phys;
	uint32_t		text_offset;
	int32_t			curr_offset;
	int32_t			trunc_offset;
	int				error;

	/* check if we need a before data buffer */
	if (desc_offset)
	{
		/* allocate space for size of data before truncate */
		before_data = f3s_malloc (desc_offset);
		if (before_data == NULL)
		{
			errno = ENOMEM;
			return (-1);
		}
	}

	trunc_offset = desc->curr_offset;

	/* read fresh current header and lock the unit */
	text_size = f3s_extptr_fresh (part, desc->curr, NULL, &text_offset, &head, &curr);
	if (text_size < 0)
	{
		if (before_data)
			f3s_free(before_data, desc_offset);
		return (text_size);
	}

	/* get physical unit index for fresh header */
	phys = f3s_extptr_phys (part, curr);

	/* read data before truncate */
	error = f3s_flash_read (part->array, phys, text_offset, desc_offset, before_data);
	if (error)
	{
		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, curr.logi_unit);
		if (before_data)
			f3s_free (before_data, desc_offset);
		errno = error;
		return (-1);
	}

	F3S_UNIT_LEAVE(part, phys);
	F3S_LOGI_LEAVE(part, curr.logi_unit);
	
	/* setup file descriptor */
	f3s_desc_init (&over_desc, f3s_null_extptr, ~F3S_DESC_XCACHE);

	/* overwrite file at current extent */
	write_size = f3s_file_create (part, &over_desc, NULL, desc_offset, before_data, 0);
	error = errno;
	if (before_data)
		f3s_free (before_data, desc_offset);
	if (write_size < desc_offset)
	{
		errno = error;
		return (-1);
	}

	/* commit overwrite descriptor to file descriptor */
	if (f3s_file_commit (part, desc, over_desc.first, F3S_COMMIT_OVER) < 0)
		return (-1);

	/* discard xcache entries beyond new EOF */
	if (trunc_offset == 0) 
		f3s_xcache_free(part, desc);
	else 
		f3s_xcache_truncate(part, desc, trunc_offset);

	/* check if header has a next */
	if (!(head.status[0] & F3S_EXT_NO_NEXT))
	{
		/* remember current position */
		curr        = desc->curr;
		curr_offset = desc->curr_offset;

		/* set descriptor to next step */
		f3s_desc_step (desc, head.next, text_size);

		/* delete the extents to end of file */
		f3s_file_delete (part, desc, F3S_FILE_MAX);

		/* restore current position */
		desc->curr        = curr;
		desc->curr_offset = curr_offset;
	}

	/* return size of truncated file */
	return (text_size);
/*}}}1*/
}

int32_t f3s_file_append (f3s_part_t *part, f3s_desc_t *desc, f3s_resv_t *resv,
                         int32_t buffer_size, uint8_t *buffer, int meta)
{
/*{{{1*/
	f3s_extptr_t	curr;
	f3s_extptr_t	new;
	uint32_t		alloc_size;
	int32_t			append_size;
	int32_t			strict;
	int32_t			text_size = 0;
	int32_t			curr_phys;
	int32_t			new_phys;
	int32_t			new_logi;
	int32_t			write_size;
	uint32_t		text_offset;
	int32_t			done_size = -1;
	int32_t			forward_size;
	uint32_t		mask;
	int				error;

	/* check if descriptor type is not for directory */
	if (desc->status & F3S_DESC_NO_DIR)
	{
		/* setup mask for file type */
		mask = ~F3S_EXT_TYPE | F3S_EXT_FILE;
		strict = 0;

	/* else, descriptor is for directory */
	} else
	{
		/* setup mask for directory type */
		mask = ~F3S_EXT_TYPE | F3S_EXT_DIR;
		strict = 1;
	}

	/* make sure current extent pointer is at end of file */
	forward_size = f3s_file_forward (part, desc, F3S_FILE_MAX);
	if (forward_size < 0)
		return (forward_size);

	/* setup current extent pointer from descriptor */
	curr = desc->curr;

	/* set write size with buffer size to write */
	append_size = buffer_size;

	F3S_TABLE_ENTER(part);

	/* check if we have a reservation */
	/* free that reservation */
	if (resv)
		f3s_resv_free (part, resv);

	/* buffer write loop */
	while (1)
	{
		/* get physical for current (used for local alloc) */
		F3S_LOGI_ENTER(part, curr.logi_unit);
		curr_phys = f3s_extptr_phys (part, curr);
		F3S_LOGI_LEAVE(part, curr.logi_unit);

		/* allocate unit with free space; The allocated unit will 
		 * be locked by f3s_alloc_fit() in f3s_unit_alloc().
		 */
		new_phys = f3s_unit_alloc (part, curr_phys, append_size, strict);
		if (new_phys < 0)
		{
			F3S_TABLE_LEAVE(part);
			return (done_size);
		}

		/* allocate new extent from unit */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		new_logi = part->array->phys_vect[new_phys];
		error = f3s_ext_alloc (part, &new_phys, &text_offset, append_size,
		                            &new, &mask, part->table.align_pow2);
		if (error < 0)
		{
			F3S_UNIT_LEAVE(part, new_phys);
			F3S_LOGI_LEAVE(part, new_logi);
			continue;
		}else{
			alloc_size = (uint32_t)error;
		}

		/* check if extent is split */
		if (alloc_size != append_size)
		{
			/* clear no split bit in mask */
			mask &= ~F3S_EXT_NO_SPLIT;

			/* log that split event */
			f3s_log_split (part->array->index, part->index, new.logi_unit,
			               new.index, append_size, alloc_size);
		}

		/* write new extent */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		write_size = f3s_ext_write (part, &new_phys, new.index, text_offset, alloc_size, 
		                            buffer, mask, part->table.align_pow2, meta, 1);

		/* fix mask for further writes */
		mask |= (F3S_EXT_NO_SPLIT | F3S_EXT_LAST);
		if (write_size < 0)
		{
			/* stale the bad extent size for unit */
			f3s_unit_stale (part, new_phys, alloc_size);
			F3S_UNIT_LEAVE(part, new_phys);
			F3S_LOGI_LEAVE(part, new_logi);
			continue;
		}

		F3S_UNIT_LEAVE(part, new_phys);
		F3S_LOGI_LEAVE(part, new_logi);

		/* next current header to new header */
		if (f3s_file_commit (part, desc, new, F3S_COMMIT_APPEND) < 0)
		{
			/* get rid of new extent pointer */
			F3S_TABLE_ENTER(part);
			f3s_extptr_delete (part, new, NULL);
			F3S_TABLE_LEAVE(part);
			return (done_size);
		}

		/* set current extent pointer to new extent pointer */
		curr = new;

		/* set text size to alloc size */
		text_size = alloc_size;

		/* adjust buffer pointer and write size */
		buffer      += write_size;
		append_size -= write_size;
		done_size    = buffer_size - append_size;

		/* move current extent pointer to end of file */
		forward_size = f3s_file_forward (part, desc, text_size);
		if (forward_size < 0) 
		{
			F3S_TABLE_LEAVE(part);
			return (forward_size);
		}

		/* setup last extent pointer */
		f3s_desc_last (desc, text_size);

		/* check if there is still something to write */
		if (append_size)
		{
			continue;
		}
		break;
	}

	F3S_TABLE_LEAVE(part);
	return (done_size);
/*}}}1*/
}

int32_t f3s_file_write (f3s_part_t *part, f3s_desc_t *desc,
                        int32_t desc_offset, int32_t buffer_size,
                        uint8_t *buffer)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_extptr_t	curr;
	f3s_extptr_t	delete;
	f3s_extptr_t	link;
	f3s_desc_t		old_desc;
	f3s_desc_t		over_desc;
	uint8_t *		before_data = NULL;
	uint8_t*		after_data = NULL;
	int32_t			text_size;
	int32_t			write_size;
	int32_t			delete_size;
	int32_t			after_size;
	int32_t			link_flag;
	int32_t			create_size;
	int32_t			append_size;
	int32_t			phys;
	int32_t			commit_size;
	uint32_t		text_offset;
	int32_t			delete_offset;
	int32_t			after_offset;
	int32_t			forward_size;
	int				error;

	/* 
	 * XXX - Why do we malloc() two buffers? We could malloc just one and
	 * change in place.
	 */

	/* read fresh current header and lock the unit */
	text_size = f3s_extptr_fresh (part, desc->curr, NULL, &text_offset,
	                              &head, &curr);
	if (text_size < 0)
	{
		return (text_size);
	}

	/* get physical unit index for fresh header */
	phys = f3s_extptr_phys (part, curr);

	/* check if before size is smaller than text size */
	if (desc_offset < text_size)
	{
		/* keep a copy of old descriptor */
		old_desc = *desc;

		/* check if there is something before */
		if (desc_offset)
		{
			/* allocate space for size of data before truncate */
			before_data = f3s_malloc (desc_offset);
			if (before_data == NULL)
			{
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);
				errno = ENOMEM;
				return (-1);
			}

			/* read data before write */
			error = f3s_flash_read (part->array, phys, text_offset,
			                        desc_offset, before_data);
			if (error)
			{
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);
				f3s_free (before_data, desc_offset);
				errno = error;
				return (-1);
			}
		}

		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, curr.logi_unit);

		/* remember header next for possible delete */
		delete = head.next;

		/* calculate delete offset */
		delete_offset = f3s_desc_curr (desc) + text_size;

		/* forward file descriptor to extent after */
		after_offset = f3s_file_forward (part, desc, desc_offset + buffer_size);
		if (after_offset < 0)
		{
			error = errno;
			f3s_free (before_data, desc_offset);
			errno = error;
			return (-1);
		}

		/* check if end of file is known and after offset is equal to size */
		if (f3s_desc_end (desc) && (after_offset == f3s_desc_size (desc)))
		{
			/* clear size to delete */
			delete_size = 0;

		} else
		{
			/* get size to delete in current extent */
			delete_size = after_offset - f3s_desc_curr (desc);
		}

		/* check if there is something to delete in current extent */
		if (delete_size)
		{
			/* read fresh current header and lock the unit */
			text_size = f3s_extptr_fresh (part, desc->curr, NULL, &text_offset,
			                              &head, &curr);
			if (text_size < 0)
			{
				error = errno;
				f3s_free (before_data, desc_offset);
				errno = error;
				return (-1);
			}

			/* get physical unit number */
			phys = f3s_extptr_phys (part, curr);

			/* adjust text offset with delete size in extent */
			text_offset += delete_size;

			/* set after size to reflect text size */
			after_size = text_size - delete_size;

			/* allocate space for buffer after write */
			after_data = f3s_malloc (after_size);
			if (after_data == NULL)
			{
				f3s_assert (after_data);
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);
				f3s_free (before_data, desc_offset);
				errno = ENOMEM;
				return (-1);
			}
			/* read data after write */
			error = f3s_flash_read (part->array, phys, text_offset, after_size,
			                        after_data);

			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, curr.logi_unit);

			if (error)
			{
				f3s_free (before_data, desc_offset);
				f3s_free (after_data,  after_size);
				errno = error;
				return (-1);
			}

			/* add size after to offset after */
			after_offset += after_size;

			/* check if header has a next */
			if (!(head.status[0] & F3S_EXT_NO_NEXT))
			{
				/* set descriptor to next extent */
				f3s_desc_step (desc, head.next, text_size);
			}

		/* else, there is nothing to delete in current extent */
		} else
		{
			/* clear after size */
			after_size = 0;
		}

		/* set link flag */
		link_flag = (after_offset == f3s_desc_curr (desc));

		/* setup link extent pointer for commit */
		link = desc->curr;

		/* calculate total delete size */
		delete_size = after_offset - delete_offset;

		/* put descriptor back to its previous current extent pointer */
		desc->curr        = old_desc.curr;
		desc->curr_offset = old_desc.curr_offset;

		f3s_xcache_truncate(part, desc, desc->curr_offset);

		/* initialize overwrite descriptor */
		f3s_desc_init (&over_desc, f3s_null_extptr, ~F3S_DESC_XCACHE);

		/* check if data before has a size */
		if (desc_offset)
		{
			/* overwrite file with before data */
			create_size = f3s_file_create (part, &over_desc, NULL,
			                               desc_offset, before_data, 0);
			error = errno;

			/* free before data buffer */
			f3s_free (before_data, desc_offset);

			if (create_size < 0)
			{
				f3s_free (after_data, after_size);
				errno = error;
				return (-1);
			}

			/* append buffer to file */
			write_size = f3s_file_append (part, &over_desc, NULL, buffer_size, buffer, 0);
			if (write_size < 0)
			{
				error = errno;

				/* rewind overwrite descriptor */
				f3s_desc_rewind (&over_desc);

				/* roll back previous create */
				f3s_file_delete (part, &over_desc, create_size);

				/* free after data buffer */
				f3s_free (after_data, after_size);

				errno = error;
				return (-1);
			}

		/* there is no data before */
		} else
		{
			/* clear create size */
			create_size = 0;

			/* overwrite file with buffer */
			write_size = f3s_file_create (part, &over_desc, NULL, buffer_size, buffer, 0);
			if (write_size < 0)
			{
				error = errno;
				f3s_free (after_data, after_size);
				errno = error;
				return (-1);
			}
		}

		/* check if data after has a size */
		if (after_size)
		{
			/* append buffer after write to file */
			append_size = f3s_file_append (part, &over_desc, NULL,
			                               after_size, after_data, 0);
			error = errno;

			/* free after data buffer */
			f3s_free (after_data, after_size);
			if (append_size < 0)
			{
				/* rewind overwrite descriptor */
				f3s_desc_rewind (&over_desc);

				/* roll back previous create and append */
				f3s_file_delete (part, &over_desc, create_size + write_size);

				errno = error;
				return (-1);
			}
		}

		/* check if current header should be linked to a next */
		if (link_flag)
		{
			/* commit the link extent pointer to overwrite descriptor */
			commit_size = f3s_file_commit (part, &over_desc, link,
			                               F3S_COMMIT_APPEND);
			if (commit_size < 0) return (-1);
		}

		/* commit the overwrite descriptor to file descriptor */
		commit_size = f3s_file_commit (part, desc, over_desc.first,
		                               F3S_COMMIT_OVER);
		if (commit_size < 0) return (-1);

		/* check if there is something to delete */
		if (delete_size)
		{
			/* set current in descriptor to delete extent and offset */
			desc->curr = delete;
			desc->curr_offset = delete_offset;

			/* delete this sub-file */
			f3s_file_delete (part, desc, delete_size);
		}

		/* set descriptor to old descriptor values */
		desc->curr = old_desc.curr;
		desc->curr_offset = old_desc.curr_offset;

		/* set descriptor to forward extent */
		forward_size = f3s_file_forward (part, desc, create_size + write_size);
		if (forward_size < 0) return (-1);
	}
	else if (desc_offset == text_size)		/* else check if size before is equal to text size */
	{
		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, curr.logi_unit);

		/* append buffer to file */
		write_size = f3s_file_append (part, desc, NULL, buffer_size, buffer, 0);
	}
	else	/* else size is bigger than text size */
	{
		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, curr.logi_unit);

/*
 * PR 56261. This assert is here to ensure that the extent header which we are
 * about to "next" doesn't already have a next pointer.  It is valid.
*/

		f3s_assert (head.status[0] & F3S_EXT_NO_NEXT);
		/* grow file to difference with size before and text size with buffer */
		write_size = f3s_file_grow (part, desc, desc_offset - text_size,
		                            buffer_size, buffer);
	}

	return (write_size);
/*}}}1*/
}

int32_t f3s_file_commit (f3s_part_t *part, f3s_desc_t *desc, f3s_extptr_t extptr,
                         f3s_commit_action_t commit_action)
{
/*{{{1*/
	f3s_extptr_t	curr;
	int32_t			text_size;
	int32_t			curr_phys;
	int32_t			reclaim_phys;
	int32_t			write_size;
	uint32_t		super_count;
	int				error;

	F3S_TABLE_ENTER(part);

	/* loop */
	do
	{
		/* read fresh current header and lock the unit */
		text_size = f3s_extptr_fresh (part, desc->curr, &super_count, NULL,
		                              NULL, &curr);
		if (text_size < 0)
		{
			error = errno;
			F3S_TABLE_LEAVE(part);
			errno = error;
			return (-1);
		}

		/* get current physical unit number */
		curr_phys = f3s_extptr_phys (part, curr);

		/* check if supersede is wanted */
		if ((commit_action == F3S_COMMIT_OVER) || (text_size == 0))
		{
			/* if super count is too big */
			if (super_count > 
					((part->table.reclaim.enable==1) ? part->table.reclaim.ethreshold:part->table.reclaim.threshold))
			{
				F3S_UNIT_LEAVE(part, curr_phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);

				/* get current physical unit number */
				F3S_LOGI_ENTER(part, desc->curr.logi_unit);
				curr_phys = f3s_extptr_phys (part, desc->curr);
				F3S_UNIT_ENTER(part, curr_phys);

				/* reclaim current unit */
				reclaim_phys = f3s_frgd_reclaim (part, curr_phys);

				/* check if reclaim was successfull */
				if (reclaim_phys < 0) F3S_UNIT_LEAVE(part, curr_phys);
				else                  F3S_UNIT_LEAVE(part, reclaim_phys);

				F3S_LOGI_LEAVE(part, desc->curr.logi_unit);

				/* read fresh current header and lock the unit */
				text_size = f3s_extptr_fresh (part, desc->curr, NULL, NULL,
				                              NULL, &curr);
				if (text_size < 0) {error = errno; goto error;}

				/* get current physical unit number */
				curr_phys = f3s_extptr_phys (part, curr);
			}

			F3S_UNIT_LEAVE(part, curr_phys);
			F3S_LOGI_LEAVE(part, curr.logi_unit);

			/* get physical unit index */
			F3S_LOGI_ENTER(part, extptr.logi_unit);
			curr_phys = f3s_extptr_phys (part, extptr);
			F3S_UNIT_ENTER(part, curr_phys);

			/* make the header non-basic */
			write_size = f3s_head_status (part, &curr_phys, extptr.index, ~F3S_EXT_BASIC, 1);
			error = errno;
			F3S_UNIT_LEAVE(part, curr_phys);
			F3S_LOGI_LEAVE(part, extptr.logi_unit);
			if (write_size < 0) goto error;

			/* get physical unit index */
			F3S_LOGI_ENTER(part, curr.logi_unit);
			curr_phys = f3s_extptr_phys (part, curr);
			F3S_UNIT_ENTER(part, curr_phys);

			/* Supersede current header with extent pointer */
			write_size = f3s_head_super (part, &curr_phys, curr.index, extptr, 1);
			if (write_size < 0)
			{
				error = errno;
				F3S_UNIT_LEAVE(part, curr_phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);
				goto error;
			}

			/* stale current extent */
			f3s_unit_stale (part, curr_phys, text_size);

		/* else, next is wanted */
		}
		else
		{
			/* next current header by extent pointer */
			write_size = f3s_head_next (part, &curr_phys, curr.index, extptr, 1);
			if (write_size < 0)
			{
				error = errno;
				F3S_UNIT_LEAVE(part, curr_phys);
				F3S_LOGI_LEAVE(part, curr.logi_unit);
				goto error;
			}

			/* set descriptor to next step */
			f3s_desc_step (desc, extptr, text_size);
		}
		F3S_UNIT_LEAVE(part, curr_phys);
		F3S_LOGI_LEAVE(part, curr.logi_unit);

		/* setup last unknown in descriptor */
		desc->status |= F3S_DESC_NO_LAST;
		break;

	/* while there are available spare units */
	} while (part->table.unit_spare > part->table.unit_retire);

	F3S_TABLE_LEAVE(part);
	return (write_size);

error:
	F3S_TABLE_LEAVE(part);
	f3s_ebadfsys (part->index, __func__, __LINE__);
	errno = EBADFSYS;
	return (-1);
/*}}}1*/
}

int32_t f3s_file_read (f3s_part_t *part, f3s_desc_t *desc,
                       int32_t desc_offset, int32_t buffer_size, 
                       uint8_t *buffer)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_extptr_t	curr;
	int32_t			done_size = 0;
	int32_t			text_size;
	int32_t			phys;
	int32_t			read_size;
	uint32_t		text_offset;
	int				error;

	/* buffer read loop */
	while (buffer_size)
	{
		/* read fresh current extent header and lock the unit */
		text_size = f3s_extptr_fresh (part, desc->curr, NULL, &text_offset,
		                              &head, &curr);
		if (text_size < 0)
		{
			return (text_size);
		}

		/* get physical unit index */
		phys = f3s_extptr_phys (part, curr);

		/* set read size from this extent */
		read_size = min(text_size - desc_offset, buffer_size);
		if (read_size >= 0)
		{
			/* read extent text at offset into buffer */
			error = f3s_flash_read (part->array, phys, text_offset + desc_offset,
			                        read_size, buffer);
			                        
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, curr.logi_unit);

			if (error)
			{
				/* set done size with read size */
				errno = error;
				done_size = -1;
				break;
			}

			/* adjust buffer pointer and size done */
			buffer    += read_size;
			done_size += read_size;
		}
		else		/* else, nothing left to read */
		{
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, curr.logi_unit);
			break;
		}

		/* check if size is grater or equal to text size less offset */
		if (buffer_size >= (text_size - desc_offset))
		{
			/* check if current extent pointer has a next */
			if (!(head.status[0] & F3S_EXT_NO_NEXT))
			{
				/* set descriptor to next step */
				f3s_desc_step (desc, head.next, text_size);

			/* else, the end of file was reached */
			}
			else
			{
				/* setup last extent pointer */
				f3s_desc_last (desc, text_size);
				break;
			}
			/* clear offset */
			desc_offset = 0;
		}
		/* subtract size to read with size read */
		buffer_size -= read_size;
	}

	/* return size done */
	return (done_size);
/*}}}1*/
}

int32_t f3s_file_forward (f3s_part_t *part, f3s_desc_t *desc,
                          int32_t forward_size)
{
/*{{{1*/
	f3s_xcache_t	*xcache = NULL;
	f3s_extptr_t	curr;
	f3s_head_t		head;
	int32_t			text_size;
	int32_t			phys;

	/* check if last extent pointer is valid and size is bigger */
	if (f3s_desc_end (desc) && forward_size >= f3s_desc_size (desc))
	{
		/* set current extent pointer to last */
		desc->curr        = desc->last;
		desc->curr_offset = desc->last_offset;

		/* limit size to last extent text size */
		forward_size = desc->last_size;

	/* else, current extent forward loop */
	}
	else
	{
		// lookup xcache entry for inode
		xcache = f3s_xcache_lookup(part, desc, forward_size);
		if((xcache != NULL) && (xcache->found != NULL)) 
		{
			forward_size = desc->curr_offset + (forward_size - xcache->found->curr_offset);
			desc->curr = xcache->found->curr;
			desc->curr_offset = xcache->found->curr_offset;
		}

		while (1)
		{
			/* read fresh current extent header and lock the unit */
			text_size = f3s_extptr_fresh (part, desc->curr, NULL, NULL,
			                              &head, &curr);
			if (text_size < 0)
			{
				return (text_size);
			}

			/* add xcache entry */
			if(xcache == NULL)
				xcache = f3s_xcache_new(part, desc);
			else 
				f3s_xcache_update(part, xcache, desc);

			/* get physical unit number */
			phys = f3s_extptr_phys (part, curr);

			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, curr.logi_unit);

			/* check if size is bigger or equal to text size */
			if (forward_size >= text_size)
			{
				/* check if header has a next */
				if (!(head.status[0] & F3S_EXT_NO_NEXT))
				{
					/* set descriptor to next step */
					f3s_desc_step (desc, head.next, text_size);

					/* subtract text size from size */
					forward_size -= text_size;

				/* else, end of file was reached */
				}
				else
				{
					/* setup last extent pointer */
					f3s_desc_last (desc, text_size);

					/* setup size to last extent text size */
					forward_size = text_size;
					break;
				}

			/* else, size is smaller that text size */
			}
			else
			{
				break;
			}
		}
	}

	/* return current offset with size left */
	return (desc->curr_offset + forward_size);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/file.c $ $Rev: 580669 $" );
