/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

static uint32_t f3s_mtd_geo (f3s_dbase_t *dbase, f3s_array_t *array, int32_t offset)
{
/*{{{1*/
	f3s_geo_t *	geo;
	uint32_t	geo_lo;
	uint32_t	geo_hi;
	uint32_t	geo_index;
	uint32_t	chip_index;

	/* Find size of current block */
	geo_hi = 0;
	for (chip_index = 0; chip_index < array->chip_total; chip_index++)
	{
		for (geo_index = 0; geo_index < dbase->geo_num; geo_index++)
		{
			geo = &dbase->geo_vect[geo_index];
			geo_lo  = geo_hi;
			geo_hi += geo->unit_num << geo->unit_pow2;

			if ((offset >= geo_lo) && (offset < geo_hi))
			{
				return (1 << geo->unit_pow2);
			}
		}
	}

	errno = ERANGE;
	return (0);
/*}}}1*/
}

static int f3s_erase_v1 (f3s_array_t *array, f3s_chip_t *chip,
                         f3s_access_t *access, int32_t phys)
{
/*{{{1*/
	f3s_flash_t *	flash = array->flash;
	int32_t			erase_size;
	int32_t			erase_offset;
	int32_t			done_size;
	int				error;

	/* Wait for active erases to finish */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	while (chip->erase_unit >= 0) pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
	chip->erase_unit = phys;

	/* Main erase loop */
	erase_size   = array->unit_size;
	erase_offset = F3S_TEXT_OFFSET(array, phys, 0);
	while (erase_size)
	{
		/* Initiate erase */
		flash->erase (&array->dbase, access, 0, erase_offset, erase_size);

		/* Wait for erase to finish */
		while (1)
		{
			/* Release chip command mutex */
			chip->use_flag = 0;
			pthread_mutex_unlock (&chip->command_mutex);

			/* Yield for 10 milliseconds */
			delay (10);

			/* Re-acquire chip command mutex */
			f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
			pthread_mutex_lock (&chip->command_mutex);
			chip->use_flag = 1;

			/* See if we're done */
			done_size = flash->sync (&array->dbase, access, 0, erase_offset, erase_size);
			if (done_size > 0) break;
			if (done_size == 0) continue;

			/* Something bad happened, don't know why */
			error = EIO;
			goto error;
		}

		/* Adjust erase size and offset */
		erase_size   -= done_size;
		erase_offset += done_size;
	}

	return (EOK);

error:
	flash->reset (&array->dbase, access, 0, erase_offset);
	return (error);
/*}}}1*/
}

static int f3s_erase_v2 (f3s_array_t *array, f3s_chip_t *chip,
                         f3s_access_t *access, int32_t phys)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	int32_t			erase_size;
	int32_t			erase_offset;
	int32_t			done_size;
	int				error;

	/* Wait for active erases to finish */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	while (chip->erase_unit >= 0) pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
	chip->erase_unit = phys;

	/* Main erase loop */
	erase_size   = array->unit_size;
	erase_offset = F3S_TEXT_OFFSET(array, phys, 0);
	while (1)
	{
		/* Initiate erase */
		error = flash->v2erase (&array->dbase, access, 0, erase_offset);
		if (error) goto error;

		/* Wait for erase to finish */
		while (1)
		{
			/* Release chip command mutex */
			chip->use_flag = 0;
			pthread_mutex_unlock (&chip->command_mutex);

			/* Yield for 10 milliseconds */
			delay (10);

			/* Re-acquire chip command mutex */
			f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
			pthread_mutex_lock (&chip->command_mutex);
			chip->use_flag = 1;

			/* See if we're done */
			error = flash->v2sync (&array->dbase, access, 0, erase_offset);
			if (!error) break;
			if (error == EAGAIN) continue;
			goto error;
		}

		/* Obtain the unit size */
		done_size = f3s_mtd_geo (&array->dbase, array, erase_offset);
		if (done_size == 0) return (errno);

		if (done_size >= erase_size) break;
		erase_size   -= done_size;
		erase_offset += done_size;
	}

	return (EOK);

error:
	flash->reset (&array->dbase, access, 0, erase_offset);
	return (error);
/*}}}1*/
}

int f3s_flash_erase (f3s_array_t *array, int32_t phys)
{
/*{{{1*/
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int				error;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* Perform MTD version specific erase */
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
	{
		error = f3s_erase_v2 (array, chip, access, phys);
	} else
	{
		error = f3s_erase_v1 (array, chip, access, phys);
	}

	/* Signal erase is finished */
	chip->erase_unit = -1;
	pthread_cond_broadcast (&chip->erase_cond);

	/* Release chip command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

static int f3s_read_v1 (f3s_array_t *array, f3s_chip_t *chip,
                        f3s_access_t *access, int32_t phys,
                        int32_t offset, int32_t size, uint8_t *buf)
{
/*{{{1*/
	f3s_flash_t *	flash = (f3s_flash_t *)array->flash;
	uint8_t *		memory;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			togo;
	int32_t			xfer;
	int				err_size;
	int				error;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
		    (array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			err_size = flash->suspend (&array->dbase, access, 0, resume_offset);
			if (err_size < 0) {error = EIO; resume_offset = -1; goto error;}
			break;
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/*
	 * KLOCWORKS (PR29539)
	 * there is a situation whereby 'buf' can be NULL upon entry to this
	 * routine. The way the code is currently written, this is only a problem
	 * if 'size' > 0. We are not seeing any problem associated with this
	 * condition however in order to be sure, the following check is
	 * being added
	*/
	if (size != 0)
	{
		f3s_assert(buf != NULL);		/* trap on a debug load */
		if (buf == NULL)
		{
			error = EINVAL;
			goto error;
		}
	}
	
	/* Perform the read */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	togo        = size;
	if (flash->read == NULL)
	{
		while (togo)
		{
			xfer = togo;
			memory = access->service->page (&access->socket, F3S_POWER_VCC,
			                                text_offset, &xfer);
			if (memory == NULL) {error = EIO; goto error;}

			f3s_assert(xfer <= togo);
			xfer = min(xfer, togo);			// avoid a buffer overflow
			memcpy (buf, memory, xfer);

			text_offset += xfer;
			togo        -= xfer;
			buf         += xfer;
		}

	} else
	{
		while (togo)
		{
			xfer = flash->read (&array->dbase, access, 0, text_offset, togo, buf);
			if (xfer < 0) {error = EIO; goto error;}

			f3s_assert(xfer <= togo);
			xfer = min(xfer, togo);			// avoid a buffer overflow
			
			text_offset += xfer;
			togo        -= xfer;
			buf         += xfer;
		}

	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0) flash->resume (&array->dbase, access, 0, resume_offset);
	return (error);
/*}}}1*/
}

/* command_mutex assumed held */
static int f3s_read_v2 (f3s_array_t *array, f3s_chip_t *chip,
                        f3s_access_t *access, int32_t phys,
                        int32_t offset, int32_t size, uint8_t *buf)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	uint8_t *		memory;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			togo;
	int32_t			xfer;
	int				error;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
		    (array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}
			
				/*
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
				*/
			}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/*
	 * KLOCWORKS (PR29539)
	 * there is a situation whereby 'buf' can be NULL upon entry to this
	 * routine. The way the code is currently written, this is only a problem
	 * if 'size' > 0. We are not seeing any problem associated with this
	 * condition however in order to be sure, the following check is
	 * being added
	*/
	if (size != 0)
	{
		f3s_assert(buf != NULL);		/* trap on a debug load */
		if (buf == NULL)
		{
			error = EINVAL;
			goto error;
		}
	}
	
	/* Perform the read */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	togo        = size;
	if (flash->v2read == NULL)
	{
		while (togo)
		{
			xfer = togo;
			memory = access->service->page (&access->socket, F3S_POWER_VCC,
			                                text_offset, &xfer);
			if (memory == NULL) {error = EIO; goto error;}

			f3s_assert(xfer <= togo);
			xfer = min(xfer, togo);			// avoid a buffer overflow
			memcpy (buf, memory, xfer);

			text_offset += xfer;
			togo        -= xfer;
			buf         += xfer;
		}

	} else
	{
		while (togo)
		{
			xfer = flash->v2read (&array->dbase, access, 0, text_offset, togo, buf);
			if (xfer < 0) {error = errno; goto error;}

			f3s_assert(xfer <= togo);
			xfer = min(xfer, togo);			// avoid a buffer overflow
			text_offset += xfer;
			togo        -= xfer;
			buf         += xfer;
		}

	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) return (error2);
	}
	return (error);
/*}}}1*/
}

int f3s_flash_read (f3s_array_t *array, int32_t phys,
                    int32_t text_offset, int32_t buffer_size,
                    uint8_t *buffer)
{
/*{{{1*/
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int				error;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);
	if ((text_offset < 0) || (text_offset >= array->unit_size)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* Perform MTD version specific read */
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
	{
		error = f3s_read_v2 (array, chip, access, phys, text_offset,
		                     buffer_size, buffer);
	} else
	{
		error = f3s_read_v1 (array, chip, access, phys, text_offset,
		                     buffer_size, buffer);
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

static int f3s_write_v1 (f3s_array_t *array, f3s_chip_t *chip,
                         f3s_access_t *access, int32_t phys,
                         uint32_t offset, uint32_t size, uint8_t *buf,
                         uint32_t write_flags)
{
/*{{{1*/
	f3s_flash_t *   flash = array->flash;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			togo;
	int32_t			xfer;
	int				err_size;
	int				error;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
			(array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			err_size = flash->suspend (&array->dbase, access, 0, resume_offset);
			if (err_size < 0) {error = EIO; resume_offset = -1; goto error;}
			break;
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Override write flags */
	if (f3s.flash_verify >= F3S_VERIFY_WROP) write_flags |= F3S_VERIFY_WRITE;

	/* Main write loop */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	togo        = size;
	while (togo)
	{
		xfer = flash->write (&array->dbase, access, write_flags,
		                     text_offset, togo, buf);
		if (xfer < 0)
		{
			error = EIO;
			flash->reset (&array->dbase, access, 0, text_offset);
			goto error;
		}

		/* ajust offset, left, and buffer */
		text_offset += xfer;
		togo        -= xfer;
		buf         += xfer;
	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0) flash->resume (&array->dbase, access, 0, resume_offset);
	return (error);
/*}}}1*/
}

static int f3s_write_v2 (f3s_array_t *array, f3s_chip_t *chip,
                         f3s_access_t *access, int32_t phys,
                         uint32_t offset, uint32_t size, uint8_t *buf,
                         uint32_t write_flags)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			togo;
	int32_t			xfer;
	int				error;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
			(array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}

			/* 
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
			 */
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Override write flags */
	if (f3s.flash_verify >= F3S_VERIFY_WROP) write_flags |= F3S_VERIFY_WRITE;

	/* Main write loop */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	togo        = size;
	while (togo)
	{
		xfer = flash->v2write (&array->dbase, access, write_flags,
		                       text_offset, togo, buf);
		if (xfer < 0)
		{
			error = errno;
			flash->reset (&array->dbase, access, 0, text_offset);
			goto error;
		}

		/* ajust offset, left, and buffer */
		text_offset += xfer;
		togo        -= xfer;
		buf         += xfer;
	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) return (error2);
	}
	return (error);
/*}}}1*/
}

int f3s_flash_write (f3s_array_t *array, int32_t phys,
                         uint32_t text_offset, uint32_t buffer_size,
                         uint8_t *buffer, uint32_t write_flags)
{
/*{{{1*/
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int				error;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);
	if (text_offset >= array->unit_size) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* Determine MTD version */
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
	{
		error = f3s_write_v2 (array, chip, access, phys, text_offset,
		                      buffer_size, buffer, write_flags);
	} else
	{
		error = f3s_write_v1 (array, chip, access, phys, text_offset,
		                      buffer_size, buffer, write_flags);
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

static int f3s_mask_v1 (f3s_array_t *array, f3s_chip_t *chip,
                        f3s_access_t *access, int32_t phys,
                        int32_t offset, uint32_t mask)
{
/*{{{1*/
	f3s_flash_t *	flash = array->flash;
	uint8_t *		memory;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			xfer;
	uint32_t		status;
	int				err_size;
	int				error = EOK;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
			(array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			err_size = flash->suspend (&array->dbase, access, 0, resume_offset);
			if (err_size < 0) {error = EIO; resume_offset = -1; goto error;}
			break;
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Read the old value */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	if (flash->read == NULL)
	{
		xfer = sizeof(status);
		memory = access->service->page (&access->socket, F3S_POWER_VCC,
		                                text_offset, &xfer);
		if (memory == NULL) {error = EIO; goto error;}
		
		f3s_assert(xfer <= sizeof(status));
		xfer = min(xfer, sizeof(status));			// avoid an overflow
		memcpy (&status, memory, xfer);

	} else
	{
		err_size = flash->read (&array->dbase, access, 0, text_offset,
		                        sizeof(status), (uint8_t *)&status);
		if (err_size < 0) {error = EIO; goto error;}
	}

	/* Write the new value */
	status &= mask;
	err_size = flash->write (&array->dbase, access, F3S_VERIFY_WRITE,
	                         text_offset, sizeof(status), (uint8_t *)&status);
	if (err_size < 0)
	{
		error = EIO;
		flash->reset (&array->dbase, access, 0, text_offset);
		goto error;
	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0) flash->resume (&array->dbase, access, 0, resume_offset);
	return (error);
/*}}}1*/
}

static int f3s_mask_v2 (f3s_array_t *array, f3s_chip_t *chip,
                        f3s_access_t *access, int32_t phys,
                        int32_t offset, uint32_t mask)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	uint8_t *		memory;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			xfer;
	uint32_t		status;
	int				err_size;
	int				error = EOK;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
			(array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}

			/* 
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
			 */
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Read the old value */
	text_offset = F3S_TEXT_OFFSET(array, phys, offset);
	if (flash->v2read == NULL)
	{
		xfer = sizeof(status);
		memory = access->service->page (&access->socket, F3S_POWER_VCC,
		                                text_offset, &xfer);
		if (memory == NULL) {error = EIO; goto error;}

		f3s_assert(xfer <= sizeof(status));
		xfer = min(xfer, sizeof(status));			// avoid an overflow
		memcpy (&status, memory, xfer);

	} else
	{
		err_size = flash->v2read (&array->dbase, access, 0, text_offset,
		                          sizeof(status), (uint8_t *)&status);
		if (err_size < 0) {error = errno; goto error;}
	}

	/* Write the new value */
	status &= mask;
	err_size = flash->v2write (&array->dbase, access, F3S_VERIFY_WRITE,
	                           text_offset, sizeof(status), (uint8_t *)&status);
	if (err_size < 0)
	{
		error = errno;
		flash->reset (&array->dbase, access, 0, text_offset);
		goto error;
	}

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) return (error2);
	}
	return (error);
/*}}}1*/
}

int f3s_flash_mask (f3s_array_t *array, int32_t phys,
                    int32_t text_offset, uint32_t mask)
{
/*{{{1*/
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int				error = EOK;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);
	if ((text_offset < 0) || (text_offset >= array->unit_size)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* Determine MTD version */
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
	{
		error = f3s_mask_v2 (array, chip, access, phys, text_offset, mask);
	} else
	{
		error = f3s_mask_v1 (array, chip, access, phys, text_offset, mask);
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

int f3s_flash_map (f3s_array_t *array, int32_t phys,
                   int32_t text_offset, int32_t buffer_size,
                   uint8_t **memory, int32_t *resume_flag)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int32_t			resume_offset;
	int				err_size;
	int				error;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	*resume_flag = -1;		// ensure that this value cannot be used unless an erase was actually suspended
	
	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);
	if ((text_offset < 0) || (text_offset >= array->unit_size)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* Deal with an erasing chip */
	resume_offset = -1;
	if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
	{
		while (chip->erase_unit >= 0)
		{
			/* If chip can be suspended */
			if ((chip->erase_unit != phys) &&
				(array->dbase.flags & F3S_ERASE_FOR_READ))
			{
				resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
				error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
				if (!error) break;
				if (error != ECANCELED) {resume_offset = -1; goto error;}

				/* 
				 * The erase just finished, we need to preserve status until the
				 * erase routine can interpret it. So, we fall through and block.
				 */
			}

			/* Wait for erase to finish */
			f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
			pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
			resume_offset = -1;
		}

	} else
	{
		while (chip->erase_unit >= 0)
		{
			/* If chip can be suspended */
			if ((chip->erase_unit != phys) &&
				(array->dbase.flags & F3S_ERASE_FOR_READ))
			{
				resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
				err_size = flash->suspend (&array->dbase, access, 0, resume_offset);
				if (err_size < 0) {error = EIO; resume_offset = -1; goto error;}
				break;
			}

			/* Wait for erase to finish */
			f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
			pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
			resume_offset = -1;
		}
	}
	*resume_flag = resume_offset;

	/* Set proper page on socket */
	text_offset = F3S_TEXT_OFFSET(array, phys, text_offset);
	*memory = access->service->page (&access->socket, F3S_POWER_VCC,
	                                 text_offset, &buffer_size);
	if ((*memory) == NULL) {
		if (resume_offset >= 0)
		{
			if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
			{
				error = flash->v2resume (&array->dbase, access, 0, resume_offset);
				if (error) return (error);
			} else
			{
				flash->resume (&array->dbase, access, 0, resume_offset);
			}
			
			*resume_flag = -1;	// no need for the caller to resume the erase
		}
		return (EIO);
	}

	return (EOK);
error:
	return (error);
/*}}}1*/
}

void f3s_flash_unmap (f3s_array_t *array, int32_t phys,
                      int32_t text_offset, int32_t resume_flag)
{
/*{{{1*/
	f3s_flash_v2_t *flash = (f3s_flash_v2_t *)array->flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* If chip needs to resume */
	if (resume_flag >= 0)
	{
		if (array->flash->struct_size == sizeof(f3s_flash_v2_t))
		{
			flash->v2resume (&array->dbase, access, 0, resume_flag);
		} else
		{
			flash->resume (&array->dbase, access, 0, resume_flag);
		}
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);
/*}}}1*/
}

int f3s_flash_islock (f3s_array_t *array, int32_t phys)
{
/*{{{1*/
	f3s_flash_v2_t *flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int32_t			resume_offset;
	int32_t			text_offset;
	int				error;

	/* This is a MTD v2 only feature */
	if (array->flash->struct_size < sizeof(f3s_flash_v2_t)) return (ENOTSUP);
	flash = (f3s_flash_v2_t *)array->flash;
	if (flash->v2islock == NULL) return (ENOTSUP);

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
		    (array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}

			/* 
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
			 */
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Lock with partition's flash lock function */
	text_offset = F3S_TEXT_OFFSET(array, phys, 0);
	error = flash->v2islock (&array->dbase, access, F3S_VERIFY_WRITE, text_offset);
	if (error) goto error;

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) error = error2;
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

int f3s_flash_lock (f3s_array_t *array, int32_t phys)
{
/*{{{1*/
	f3s_flash_v2_t *flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			curr_size, lock_size;
	int				error;

	/* This is a MTD v2 only feature */
	if (array->flash->struct_size < sizeof(f3s_flash_v2_t)) return (ENOTSUP);
	flash = (f3s_flash_v2_t *)array->flash;
	if (flash->v2lock == NULL) return (ENOTSUP);

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
		    (array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}

			/* 
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
			 */
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Lock with partition's flash lock function */
	text_offset = F3S_TEXT_OFFSET(array, phys, 0);
	lock_size = array->unit_size; 
	do{
		error = flash->v2lock (&array->dbase, access, F3S_VERIFY_WRITE, text_offset);
		if (error)
		{
			flash->reset (&array->dbase, access, 0, text_offset);
			goto error;
		}
		curr_size = f3s_mtd_geo(&array->dbase, array, text_offset);
		f3s_assert(curr_size >= 0);
		text_offset += curr_size;
		lock_size -= curr_size;
	}while ((lock_size > 0) && curr_size);

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) error = error2;
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

int f3s_flash_unlock (f3s_array_t *array, int32_t phys)
{
/*{{{1*/
	f3s_flash_v2_t *flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			curr_size, unlock_size;
	int				error;

	/* This is a MTD v2 only feature */
	if (array->flash->struct_size < sizeof(f3s_flash_v2_t)) return (ENOTSUP);
	flash = (f3s_flash_v2_t *)array->flash;
	if (flash->v2unlock == NULL) return (ENOTSUP);

	f3s_assert (phys >= 0 && phys < array->unit_total);
	f3s_assert (array->map_lock_vect[phys].thread == pthread_self ());
	f3s_assert (array->map_lock_vect[phys].count);

	if ((phys < 0) || (phys >= array->unit_total)) return (EINVAL);

	/* Get chip resource pointer */
	f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
	chip = array->unit_vect[phys];

	/* Get access window through thread key */
	access = pthread_getspecific (array->window_key);
	if (access == NULL)
	{
		/* Set access pointer */
		access = &array->access_vect[pthread_self () % array->access_num];
		pthread_setspecific (array->window_key, access);
	}

	/* Acquire chip command mutex */
	pthread_mutex_lock (&chip->command_mutex);
	chip->use_flag = 1;

	/* If chip is erasing */
	resume_offset = -1;
	while (chip->erase_unit >= 0)
	{
		/* If chip can be suspended */
		if ((chip->erase_unit != phys) &&
		    (array->dbase.flags & F3S_ERASE_FOR_READ))
		{
			resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
			error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
			if (!error) break;
			if (error != ECANCELED) {resume_offset = -1; goto error;}

			/* 
			 * The erase just finished, we need to preserve status until the
			 * erase routine can interpret it. So, we fall through and block.
			 */
		}

		/* Wait for erase to finish */
		f3s_lock_assert(array->map_lock_vect[phys].thread == pthread_self());
		pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
		resume_offset = -1;
	}

	/* Unlock with partitions flash unlock function */
	text_offset = F3S_TEXT_OFFSET(array, phys, 0);
	unlock_size = array->unit_size; 
	do{
		error = flash->v2unlock (&array->dbase, access, F3S_VERIFY_WRITE, text_offset);
		if (error)
		{
			flash->reset (&array->dbase, access, 0, text_offset);
			goto error;
		}
		curr_size = f3s_mtd_geo(&array->dbase, array, text_offset);
		f3s_assert(curr_size >= 0);
		text_offset += curr_size;
		unlock_size -= curr_size;
	}while ((unlock_size > 0) && curr_size);

	error = EOK;
error:
	/* Resume suspended erase */
	if (resume_offset >= 0)
	{
		int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
		if (error2) error = error2;
	}

	/* Release command mutex */
	chip->use_flag = 0;
	pthread_mutex_unlock (&chip->command_mutex);

	return (error);
/*}}}1*/
}

int f3s_flash_unlockall (f3s_array_t *array)
{
/*{{{1*/
	f3s_flash_v2_t *flash;
	f3s_chip_t *	chip;
	f3s_access_t *	access;
	int32_t			resume_offset;
	int32_t			text_offset;
	int32_t			curr_phys;
	int				error;

	/* This is a MTD v2 only feature */
	if (array->flash->struct_size < sizeof(f3s_flash_v2_t)) return (ENOTSUP);
	flash = (f3s_flash_v2_t *)array->flash;
	if (flash->v2unlockall == NULL) return (ENOTSUP);

	/* Loop to get each chip and starting its starting physical unit */
	chip = NULL;
	for (curr_phys = 0; curr_phys < array->unit_total; curr_phys++)
	{
		/* Only hit each chip once */
		f3s_lock_assert(array->map_lock_vect[curr_phys].thread == pthread_self());
		if (chip == array->unit_vect[curr_phys]) continue;

		/* Get chip resource pointer */
		chip = array->unit_vect[curr_phys];

		/* Get access window through thread key */
		access = pthread_getspecific (array->window_key);
		if (access == NULL)
		{
			/* Set access pointer */
			access = &array->access_vect[pthread_self () % array->access_num];
			pthread_setspecific (array->window_key, access);
		}

		/* Acquire chip command mutex */
		pthread_mutex_lock (&chip->command_mutex);
		chip->use_flag = 1;

		/* If chip is erasing */
		resume_offset = -1;
		while (chip->erase_unit >= 0)
		{
			/* If chip can be suspended */
			if (array->dbase.flags & F3S_ERASE_FOR_READ)
			{
				resume_offset = F3S_TEXT_OFFSET(array, chip->erase_unit, 0);
				error = flash->v2suspend (&array->dbase, access, 0, resume_offset);
				if (!error) break;
				if (error != ECANCELED) {resume_offset = -1; goto error;}

				/* 
				 * The erase just finished, we need to preserve status until the
				 * erase routine can interpret it. So, we fall through and block.
				 */
			}

			/* Wait for erase to finish */
			f3s_lock_assert(array->map_lock_vect[curr_phys].thread == pthread_self());
			pthread_cond_wait (&chip->erase_cond, &chip->command_mutex);
			resume_offset = -1;
		}

		/* Unlock with partition's flash unlock function */
		text_offset = F3S_TEXT_OFFSET(array, curr_phys, 0);
		error = flash->v2unlockall (&array->dbase, access, F3S_VERIFY_WRITE, text_offset);
		if (error)
		{
			flash->reset (&array->dbase, access, 0, text_offset);
			goto error;
		}

		error = EOK;
error:
		/* Resume suspended erase */
		if (resume_offset >= 0)
		{
			int error2 = flash->v2resume (&array->dbase, access, 0, resume_offset);
			if (error2) error = error2;
		}

		/* Release command mutex */
		chip->use_flag = 0;
		pthread_mutex_unlock (&chip->command_mutex);

		if (error) return (error);
	}

	return (EOK);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/flash.c $ $Rev: 580669 $" );
