/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */



#include "f3s.h"



#ifdef ENABLE_LOCKDOWN

static int f3s_lockdown_sig_handler (f3s_array_t* array, int32_t phys);
static int f3s_unlockdown_sig_handler (f3s_array_t* array, int32_t phys);
static int f3s_lockdown_handler(f3s_array_t* array, int32_t phys);
static int f3s_unlockdown_handler(f3s_array_t* array, int32_t phys);

static pthread_mutex_t lockdown_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_barrier_t lockdown_barrier = PTHREAD_BARRIER_INITIALIZER (2);

static int lockdown_pending = 0;
static int lockdown_b_waiting = 0;

static int f3s_lockdown_handler(f3s_array_t* array, int32_t phys)
{
/*{{{1 Sets the lockdown pending, and suspends any erases */
    uint8_t* memory_storage;
	f3s_part_t *part = NULL; //to quiet a check in F3S_RAW_ENTER
	f3s_attr_t *attr = NULL;

    pthread_mutex_lock(&lockdown_mutex);
    if (lockdown_pending)
    {
        pthread_mutex_unlock(&lockdown_mutex);
        errno = EBUSY;
        return errno;
	}

	lockdown_pending = 1;
	pthread_mutex_unlock(&lockdown_mutex);

    if (f3s.verbose >2)
		fprintf(stderr, "Got the lockdown message\n");

	/* Suspend any active erases */
	F3S_RAW_ENTER(array, phys);

    f3s_flash_map(array, 0, 0, 0, &memory_storage, &(array->resume));

    errno = EOK;
    return 0;
/*}}}1*/
}

static int f3s_unlockdown_handler(f3s_array_t* array, int32_t phys)
{
/*{{{1 Clears the lockdown pending.  Leaves lockdown state. */

    pthread_mutex_lock(&lockdown_mutex);
    if (!lockdown_pending){
        pthread_mutex_unlock(&lockdown_mutex);
        errno = EOK;
        return errno;
    }

    if (f3s.verbose >2){
		fprintf(stdout, "Got the unlockdown message\n");
	}

    pthread_mutex_unlock(&lockdown_mutex);

    f3s_flash_unmap(array, 0,0, array->resume);
    F3S_RAW_LEAVE(array, phys);

	pthread_mutex_lock(&lockdown_mutex);
	lockdown_pending = 0;
	pthread_mutex_unlock(&lockdown_mutex);
   return 0;
/*}}}1*/
}

static int f3s_lockdown_sig_handler (f3s_array_t* array, int32_t phys){
	f3s_lockdown_handler(array, 0);
	/* check to see if the other thread is going
	 * to wait on the barrier, we don't want to block
	 * forever
	 */
	pthread_mutex_lock(&lockdown_mutex);
	if (lockdown_b_waiting){
		pthread_mutex_unlock(&lockdown_mutex);
		pthread_barrier_wait(&lockdown_barrier);
	}else{
		pthread_mutex_unlock(&lockdown_mutex);
	}
	return 0;
}

static int f3s_unlockdown_sig_handler (f3s_array_t* array, int32_t phys){
	f3s_unlockdown_handler(array, 0);
	return 0;
}
#endif /* ENABLE_LOCKDOWN */



int f3s_raw_lockdown (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
#ifdef ENABLE_LOCKDOWN
	pthread_mutex_lock(&lockdown_mutex);
	if (lockdown_pending){
		pthread_mutex_unlock(&lockdown_mutex);
		return EBUSY;
	}
	lockdown_b_waiting = 1;
	pthread_mutex_unlock(&lockdown_mutex);

	kill (getpid(), SIGUSR1);

	pthread_barrier_wait(&lockdown_barrier);

	pthread_mutex_lock(&lockdown_mutex);
	lockdown_b_waiting = 0;
	pthread_mutex_unlock(&lockdown_mutex);

    return EOK;
#else
	return ENOSYS;
#endif
/*}}}1*/
}

int f3s_raw_unlockdown (resmgr_context_t *ctp, io_devctl_t *msg, f3s_ocb_t *ocb)
{
/*{{{1*/
#ifdef ENABLE_LOCKDOWN
	errno = EOK;
	kill (getpid(), SIGUSR2);
    return errno;
#else
	return ENOSYS;
#endif
/*}}}1*/	
}




int f3s_sigusr1_handler (f3s_array_t* array){
#ifdef ENABLE_LOCKDOWN
	f3s_lockdown_sig_handler (array, 0);
#else
	if (f3s.tpp != NULL)
	{
		/* Stop the thread pool */
		thread_pool_destroy (f3s.tpp);
		f3s.tpp = NULL;
	}
#endif
	return 0;
}

int f3s_sigusr2_handler (f3s_array_t* array, thread_pool_attr_t *pool_attr){
#ifdef ENABLE_LOCKDOWN
	f3s_unlockdown_sig_handler (array, 0);
#else
	if (f3s.tpp == NULL)
	{
		/* Re-create the thread pool */
		f3s.tpp = thread_pool_create (pool_attr, 0);
		if (f3s.tpp == NULL) return (errno);

		/*Restart the thread pool */
		thread_pool_start(f3s.tpp);
	}
#endif
	return 0;
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/lockdown.c $ $Rev: 580669 $" );
