/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */


/*!\file reclaim.c
 * \brief Implementation routines for reclaims.
 */
#include "f3s.h"

//#define	INCLUDE_DUMP_FUNCS		/* uncomment if desired */

#ifdef INCLUDE_DUMP_FUNCS
static void _dump_copy_vect (uint16_t *copy_vect, uint32_t copy_num, uint32_t cur_idx);
static void _dump_obsol_vect (f3s_part_t *part, f3s_extptr_t *obsol_vect, uint32_t obsol_num);

#define dump_copy_vect(V, N, I)			_dump_copy_vect((V), (N), (I))
#define dump_obsol_vect(P, V, N)		_dump_obsol_vect((P), (V), (N))

#else	/* INCLUDE_DUMP_FUNCS */

#define dump_copy_vect(V, N, I)
#define dump_obsol_vect(P, V, N)

#endif	/* INCLUDE_DUMP_FUNCS */

/*!
 *\brief f3s_frgd_reclaim.
 *	
 *This function performs foreground reclaims, which are designed to be
 *minimalist and fast.  Foreground reclaims occur while the
 *client process is blocked in the library call.  This function
 *actually performs two reclaims, the first swaps the current spare
 *with the lowest-erase-count block in the partition.  This provides
 *near 100% even wear-level, assuming only foreground reclaims.
 *Foreground reclaims are most commonly triggered by overwriting the same
 *offset in a file more than "supersede count" (-p cmd-line option)
 *number of times.
 *	
 * \param	part A pointer to the partition's data structure.
 * \param	reclaim_phys The physical block that needs to be reclaimed.
 *
 * \pre		Lock Required - UNIT lock for reclaim_phys
 * \post	Lock Held	- UNIT lock for spare_phys 
 * \post	Lock Released - UNIT lock for reclaim_phys
 * \returns Success indication.
 * \retval	 EOK on success.
 */
int32_t f3s_frgd_reclaim (f3s_part_t *part, int32_t reclaim_phys)
{
/*{{{1*/
	int32_t		spare_phys, reclaim_logi;
	uint32_t	obsol_num;
	int			error;
	unsigned	retry_cnt = 2 * part->table.unit_spare;
	uint32_t	erase_min = ~0; //specifying the max value
	uint32_t	erase_count;
	int32_t		erase_min_phys = -1; //specifying -1 because I want -1
	int32_t		erase_min_logi = -1; //specifying -1 because I want -1
	int32_t		tmp_phys;
	unsigned	loop_ctrl;
	int			dirty;  //true/false flag to hold if the spare is dirty or not
	
	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);
	f3s_verify_headers (part, -1);

	if (part->unit_num > 2){
		//Only do the double-reclaim if there are more than 2 blocks,
		//otherwise we are just burning out the flash block unnecessarily.

		/* Search for the unit with the lowest erase count, skip the
		 * unit with the boot block.  We will reclaim the unit with the
		 * lowest erase count, then perform the reclaim of reclaim_phys.
		 *
		 * FOR each entry in unit_vect
		 * 		IF entry's erase count is the lowest found THEN
		 * 			IF the entry isn't the boot block THEN
		 * 				save the current entry
		 * 			END IF
		 * 		END IF
		 * END FOR
		 * 
		 * LOCK the saved entry
		 * GRAB a spare
		 * RECLAIM the saved entry
		 * UNLOCK the saved entry
		 * REFRESH the saved entry
		 * RECLAIM the reclaim_phys
		 * REFRESH the reclaim_phys
		*/	
	
	    /* Logical numbers start counting at 1.  Since we want to skip the
	     * block with the boot header, we start looking at logical numbers
	     * of F3S_FIRST_LOGI.
	     * 
	     * Because the spare blocks have invalid entries in the Logi to Phys
	     * array, we only loop through F3S_UNIT_ACTIVE number of blocks.
	    */

		reclaim_logi = part->array->phys_vect[reclaim_phys];
		for (loop_ctrl = min (F3S_FIRST_LOGI + 1, F3S_UNIT_ACTIVE(part));
				loop_ctrl <= F3S_UNIT_ACTIVE(part); loop_ctrl++)
		{
			if(loop_ctrl == reclaim_logi)
			{
				continue;
			}

			F3S_LOGI_ENTER(part, loop_ctrl);
			tmp_phys = part->table.logi_vect[loop_ctrl];
			F3S_UNIT_ENTER(part, tmp_phys);

			erase_count = part->array->map_vect[tmp_phys].erase_count;
			if (erase_min > erase_count){
				erase_min = erase_count;
				erase_min_phys = tmp_phys;
				erase_min_logi = loop_ctrl;
			}

			F3S_UNIT_LEAVE(part, tmp_phys);
			F3S_LOGI_LEAVE(part, loop_ctrl);
		}
			
		/* We now have the non-spare physical with the lowest erase_count.  Since
	 	 * the reclaim we're about to do will swap a spare block and this
		 * non-spare block, we'll hit the spare block on the next pass, and we
		 * don't need to scan the spare blocks specifically.
		*/
		if (erase_min_phys != -1)
		{
			F3S_LOGI_ENTER(part, erase_min_logi);
			F3S_UNIT_ENTER(part, erase_min_phys);
			/* loop until physical unit is properly reclaimed */
			while (1)
			{
				/* grab a spare unit */
				spare_phys = f3s_spare_grab (part, &dirty);
				if (spare_phys < 0){
					F3S_UNIT_LEAVE (part, erase_min_phys);
					F3S_LOGI_LEAVE (part, erase_min_logi);
					return (spare_phys);
				}

				/* Clean up the spare if it's dirty  */
				if (F3S_DIRTY == dirty){
					error = f3s_unit_refresh (part, spare_phys);
					if (error){
						f3s_spare_drop (part, spare_phys, F3S_DIRTY);
						continue;
					}
				}

				/* reclaim unit */
				error = f3s_unit_reclaim (part, erase_min_phys, spare_phys, &obsol_num);
				if (error)
				{
					fprintf (stderr, "(devf  t%d::%s:%d) reclaim failure\n", pthread_self(), __func__, __LINE__);
					/*
				 	 * potential infinite loop of grab and drop. Limit attempts by trying (at most) twice the number
				 	 * of configured spare units. Should actually be based on the number of units in the spare list
				 	 * (since spares can be come bad) but this is a more lengthy calculation and potentially inaccurate
				 	 * as during a reclaim a spare might not exist while it is being erased.
					*/
					if (retry_cnt == 0)
					{
						printf("(devf  t%d::%s:%d) Unable to find an suitable spare after %d attempts\n",
									pthread_self(), __func__, __LINE__, 2 * part->table.unit_spare);
						F3S_UNIT_LEAVE(part, erase_min_phys);
						F3S_LOGI_LEAVE(part, erase_min_logi);
						F3S_UNIT_LEAVE(part, spare_phys);
						errno = EIO;
						return (-1); 
					}
					else
					{
						/* put the spare back */
						error = f3s_unit_refresh (part, spare_phys);
						/*
					 	 * If the spare could be erased, put it back on the spare list.
					 	 * If it could not, it is currently left dangling until the
					 	 * resource manager is restarted. Might need to reflect this
					 	 * loss in the bad blocks and/or the unit_spare counts
					 	*/
						f3s_spare_drop (part, spare_phys, (error)?F3S_DIRTY:F3S_NOT_DIRTY);
				
						--retry_cnt;
						continue;
					}
				}

				/* successfully reclaimed to the spare_phys. Done with its lock */
				F3S_UNIT_LEAVE(part, spare_phys);
				F3S_LOGI_LEAVE(part, erase_min_logi);
			
				/* refresh new spare unit */
				error = f3s_unit_refresh (part, erase_min_phys);
				f3s_spare_drop (part, erase_min_phys, (error)?F3S_DIRTY:F3S_NOT_DIRTY);

				/* Clean up obsolete list */
				error = f3s_reclaim_clean (part, obsol_num);
				if (error) 
				{
					errno = error;
					return (-1);
				}
				break;
			}
		}
	}
	/* Now that we have done the above reclaim of the block with the lowest erase count,
	 * one of the spares in the spare pool has the lowest erase count.  We will now
	 * perform the standard reclaim code, grabbing a spare from the pool.  The 
	 * code above will cycle the blocks in the spare pool, so that we don't get the
	 * "ping-pong" effect of some usage patterns, where one or two blocks get very
	 * high erase counts.
	 */

	/* In case we changed retry_cnt above, reset it*/
	retry_cnt = 2 * part->table.unit_spare;
	
	/* loop until physical unit is properly reclaimed */
	while (1)
	{
		/* grab a spare unit */
		spare_phys = f3s_spare_grab (part, &dirty);
		if (spare_phys < 0) return (spare_phys);

		/* Clean up the spare if it's dirty  */
		if (F3S_DIRTY == dirty){
			error = f3s_unit_refresh (part, spare_phys);
			if (error){
				f3s_spare_drop (part, spare_phys, F3S_DIRTY);
				continue;
			}
		}

		/* reclaim unit */
		f3s_log_reclaim ("foreground", part->array->index, part->index, part->array->phys_vect[reclaim_phys],
									reclaim_phys, spare_phys, ((f3s_map_t *)&part->array->map_vect[reclaim_phys])->stale_size);
		error = f3s_unit_reclaim (part, reclaim_phys, spare_phys, &obsol_num);
		if (error)
		{
			
			fprintf (stderr, "(devf  t%d::%s:%d) reclaim failure\n", pthread_self(), __func__, __LINE__);

			/*
			 * potential infinite loop of grab and drop. Limit attempts by trying (at most) twice the number
			 * of configured spare units. Should actually be based on the number of units in the spare list
			 * (since spares can be come bad) but this is a more lengthy calculation and potentially inaccurate
			 * as during a reclaim a spare might not exist while it is being erased.
			*/
			if (retry_cnt == 0)
			{
				printf("(devf  t%d::%s:%d) Unable to find an suitable spare after %d attempts\n",
							pthread_self(), __func__, __LINE__, 2 * part->table.unit_spare);
				F3S_UNIT_LEAVE(part, spare_phys);
				errno = EIO;
				return (-1); 
			}
			else
			{
				/* put the spare back */
				error = f3s_unit_refresh (part, spare_phys);
				/*
				 * is the spare could be erased, put it back on the spare list. If it could not, it is currently left dangling
				 * until the resource manager is restarted. Might need to reflect this loss in the bad blocks and/or
				 * the unit_spare counts
				*/
				f3s_spare_drop (part, spare_phys, error?F3S_DIRTY:F3S_NOT_DIRTY);
				
				--retry_cnt;
				continue;
			}
		}

		/* refresh new spare unit */
		error = f3s_unit_refresh (part, reclaim_phys);
		f3s_spare_drop (part, reclaim_phys, error?F3S_DIRTY:F3S_NOT_DIRTY);

		/* Clean up obsolete list */
		error = f3s_reclaim_clean (part, obsol_num);
		if (error) {errno = error; return (-1);}

		break;
	}
	
	/* set flag for units not sorted */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	part->table.status |= F3S_TABLE_NO_SORT;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);
	f3s_verify_headers (part, -1);

	/* return the new unit */
	return (spare_phys);
/*}}}1*/
}

/* XXX - Make a typedef for the callback */
/*!
 *\brief f3s_bkgd_init.
 *
 * Attaches a pulse handler for background reclaims, if necessary.
 * \param	part A pointer to the partition to enable background reclaims on.
 * \pre		Lock Required  - Table lock
 * \post	Lock Held - None
 * \post	Lock Released - None
 * \returns Nothing.
 */
void f3s_bkgd_init (f3s_part_t *part)
{
/*{{{1*/
	/* check if background reclaim is wanted */
	if ((f3s.bkgd_flag) && (part->bkgd_code == -1))
	{
		/* initialize background reclaim pulse for partition */
		part->bkgd_code = pulse_attach (f3s.dpp, MSG_FLAG_ALLOC_PULSE, 0,
							(int (*) (message_context_t *, int, unsigned, void *))
							f3s_bkgd_reclaim, part);

		if (part->bkgd_code  == -1) 
		{
			part->bkgd_count = 0;
			perror ("unable to pulse attach");
			return;
		}

		/* initialize background reclaim pulse counter */
		part->bkgd_count = 1;
	}
/*}}}1*/
}
/*!
 * \brief f3s_bkgd_kill.
 *
 * Disables the background reclaim pulse, and therefore the
 * feature.
 * \pre Lock required - Table lock
 * \post Lock Held - None
 * \post Lock Released - None
 * \param part A pointer to the parition to perform this command on.
 * \returns Nothing.
 */
void f3s_bkgd_kill (f3s_part_t *part)
{
/*{{{1*/
	f3s_lock_assert(part->table_lock.thread == pthread_self());

	/* detach background reclaim pulse for partition */
	if ((f3s.bkgd_flag) && (part->bkgd_code != -1))
	{
		pulse_detach (f3s.dpp, part->bkgd_code, 0);
		part->bkgd_code = -1;
	}
/*}}}1*/
}

/*!
 * \brief f3s_bkgd_signal.
 *
 * Used to trigger the background reclaim feature.
 * \pre	Lock required - Table lock
 * \post Lock Held - None
 * \post Lock Released - None
 * \param part The partition to signal the reclaim on.
 * \returns Nothing
 */
void f3s_bkgd_signal (f3s_part_t *part)
{
/*{{{1*/
	f3s_lock_assert(part->table_lock.thread == pthread_self());

	/* check if background reclaim is wanted, and reclaims have not been throttled */
	if ((part->bkgd_code != -1) && (part->bkgd_count > 0))
	{
		/* send pulse for background reclaim */
		if (MsgSendPulse (part->pulse_connect, f3s.bkgd_priority, part->bkgd_code, 0) != -1)
		{
			/* decrement number of pulses sendable */
				part->bkgd_count--;
			}
		}
/*}}}1*/
}

/*!
 * \brief f3s_bkgd_reclaim.
 *
 * Performs a reclaim in the background, at the priority specified by
 * the -b command-line option.  This function will sort all units
 * in the partition and reclaim the one that contains the most stale space.
 * 
 * \param ctp The resmgr context
 * \param code pulse code, unused 
 * \param value pulse value, unused
 * \param part Partition to perform reclaim on
 * \pre Lock Required    None
 * \post Lock Held    None
 * \returns Nothing
 */
void f3s_bkgd_reclaim (resmgr_context_t *ctp,
                       int code, union sigval value, f3s_part_t *part)
{
/*{{{1*/
	f3s_map_t *	map;
	f3s_sort_t *sort;
	int32_t		reclaim_phys;
	int32_t		spare_phys;
	uint32_t	obsol_num;
	int			error;
	int			dirty;

	F3S_TABLE_ENTER(part);

	/* make sure that the background reclaim was not killed waiting for the lock */
	if (part->bkgd_code == -1)
	{
		F3S_TABLE_LEAVE(part);
		return;
	}

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* check if there is stale space */
	if(f3s_check_stale_total(part))
	{
		/* grab a spare unit if one is available */
		if ((spare_phys = f3s_spare_grab (part, &dirty)) >= 0)
		{
			if (dirty){
				f3s_unit_refresh (part, spare_phys);
			}
			/* Sort units by best allocation and worst reclaim */
			if (part->table.status & F3S_TABLE_NO_SORT)
				f3s_table_sort (part, -1);

			/* get sort pointer */
			/* sort will start with the highest "score" */
			sort = &part->table.sort_vect[F3S_UNIT_ACTIVE(part) - 1];

			/* sort currently points to the entry with the lowest
			 * score, but that doesn't mean that it has any stale
			 * space.  If score is zero, then the block is all stale,
			 * and we can reclaim it.  Otherwise, we need to scan
			 * until we find one with stale, that will also have
			 * the most stale space.
			*/
			if (sort->score == 0)
			{
				F3S_LOGI_ENTER(part, sort->logi);	
				F3S_UNIT_ENTER(part, sort->phys);		// no score, this is the unit we are looking for
			}
			else
			{
				/* find a unit with some stale space */
				while (sort >= part->table.sort_vect) 
				{
					F3S_LOGI_ENTER(part, sort->logi);	
					F3S_UNIT_ENTER(part, sort->phys);

					/* get unit map pointer */
					f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[sort->phys].thread == pthread_self()));
					map = &part->array->map_vect[sort->phys];

					/* Stop looking if there is stale space on unit */
					if (map->stale_size > 0)
						break;	// got it
					else
					{
						// try the next unit
						F3S_UNIT_LEAVE(part, sort->phys);
						F3S_LOGI_LEAVE(part, sort->logi);
						--sort;
					}
				}
				/*
				 * if sort < part->table.sort_vect, we did not find a unit with stale space.
				 * Note that the old version of this code just went ahead and used the 'first'
				 * entry in the sort_vect
				 */
				f3s_assert(sort >= part->table.sort_vect);
				if(sort < part->table.sort_vect) { //not found 
					F3S_TABLE_LEAVE(part); 
					return;
				}

			}

			/* At this point the unit lock is held on the 'reclaim_phys = sort->phys' */
			reclaim_phys = sort->phys;

			/* reclaim unit */
			f3s_log_reclaim ("background", part->array->index, part->index, part->array->phys_vect[reclaim_phys],
										reclaim_phys, spare_phys, ((f3s_map_t *)&part->array->map_vect[reclaim_phys])->stale_size);
			error = f3s_unit_reclaim (part, reclaim_phys, spare_phys, &obsol_num);

			// if the reclaim fails, try and return the spare, clean up the locks and leave
			if (error) 
			{
				fprintf (stderr, "(devf  t%d::%s:%d) background reclaim failure\n",
							pthread_self(), __func__, __LINE__);

				/* erase the spare and if successful put it back. If unable to erase, it is left dangling */
				error = f3s_unit_refresh (part, spare_phys);
				f3s_spare_drop (part, spare_phys, error?F3S_DIRTY:F3S_NOT_DIRTY);

				F3S_UNIT_LEAVE (part, reclaim_phys);
				F3S_LOGI_LEAVE (part, sort->logi);
			}
			else	// reclaim ok
			{
				/* reflect physical unit swap in sort vector */
				sort->phys = spare_phys;
				F3S_UNIT_LEAVE(part, spare_phys);
				F3S_LOGI_LEAVE(part, sort->logi);

				/* set table not sorted flag */
				part->table.status |= F3S_TABLE_NO_SORT;

				/* refresh new spare unit */
				error = f3s_unit_refresh (part, reclaim_phys);
				f3s_spare_drop (part, reclaim_phys, error?F3S_DIRTY:F3S_NOT_DIRTY);
				
				/* Clean up obsolete list */
				error = f3s_reclaim_clean (part, obsol_num);
				if (error)
					fprintf (stderr, "(devf  t%d::%s:%d)background reclaim failure\n",
								pthread_self(), __func__, __LINE__);

				/* DEBUG: Verify the partition table totals match */
				f3s_verify_counters (part, 0, 0);
			}
		}
	}
	/* increment number of pulses sendable */
	part->bkgd_count++;

	F3S_TABLE_LEAVE(part);
/*}}}1*/
}


void f3s_reclaim_sort (f3s_part_t *part, uint32_t *copy_num, int32_t phys)
{
/*{{{1*/
	f3s_head_t	head;
	f3s_map_t *	map;
	uint16_t *	move_entry;
	uint16_t *	copy_vect;
	int32_t		super_phys;
	int32_t		read_size;
	int32_t		realloc_count;
	uint32_t	sort_index;
	uint32_t	move_index;
	uint32_t	copy_count = 0;

	/* get map pointer for unit */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	f3s_lock_assert(part->table.logi_lock_vect[part->array->phys_vect[phys]].thread == pthread_self());
	map = &part->array->map_vect[phys];

	/* initialize copy vector */
	if (map->head_last + 1 > part->table.copy_max)
	{
		/* We grow the reallocation count by multiples of F3S_CACHE_MAX
		 * so we need to round map->head_last +1 up to the next multiple
		 * of F3S_CACHE_MAX.  This just reduces the number of calls to
		 * realloc.
		*/
		realloc_count = ((map->head_last + 1) / F3S_CACHE_MAX + 1) * F3S_CACHE_MAX;

		/* reallocate copy vector */
		copy_vect = f3s_realloc (part->table.copy_vect, part->table.copy_max * sizeof (*copy_vect), realloc_count * sizeof (*copy_vect));
		if (!copy_vect) goto failure_shortcut;
		
		part->table.copy_vect = copy_vect;
		part->table.copy_max  = realloc_count;

	} else
	{
		copy_vect = part->table.copy_vect;
	}

	/* initialize sort index */
	sort_index = F3S_UNIT_LOGI_INDEX + 1;

	/* check if unit has boot */
	f3s_assert(part->array->map_lock_vect[phys].thread == pthread_self());
	if (part->array->phys_vect[phys] == part->table.boot_extptr.logi_unit)
	{
		/* skip over the boot head_t */
		sort_index++;
	}

	/* for all indexes on unit except unit info and unit logi */
	for (; sort_index <= map->head_last; sort_index++)
	{
		/* read extent header from physical unit */
		read_size = f3s_head_read (part, phys, sort_index, NULL, &head, 1);
		if ((read_size < 0) && (errno != ENODATA))
			goto failure_shortcut;

		/* check if extent header is allocated */
		if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC)
		{
			/* put extent index into copy vector */
			copy_vect[copy_count] = sort_index;

			/* increment number of copy indexes */
			copy_count++;

			/* DEBUG: Dump the current state of the copy vector */
			dump_copy_vect (copy_vect, copy_count, 0xFFFF);

			/* while header is superseded */
			while (!(head.status[0] & F3S_EXT_NO_SUPER))
			{
				/* get physical unit of supersede header */
				super_phys = f3s_extptr_phys (part, head.super);

				/* check if header is on same unit */
				if (super_phys == phys)
				{
					/* check if superseder index is smaller than base index */
					if (head.super.index < sort_index)
					{
						/* find index in copy vect */
						for (move_entry = copy_vect, move_index = 1;
						     move_index <= copy_count && *move_entry != head.super.index;
						     move_entry++, move_index++)
						{
							;
						}

						if (move_index <= copy_count)
						{
							/* remove index from copy vect */
							memmove (move_entry, move_entry + 1, (copy_count - move_index) * sizeof (*move_entry));
						}

						/* put index back at end of copy vect to ensure proper sort */
						copy_vect[copy_count - 1] = head.super.index;
						//copy_count--;

						/* DEBUG: Dump the current state of the copy vector */
						dump_copy_vect (copy_vect, copy_count, 0xFFFF);
					}

					/* read supersede header */
					read_size = f3s_head_read (part, phys, head.super.index, NULL, &head, 1);
					if ((read_size < 0) && (errno != ENODATA)){
						goto failure_shortcut;
					}

				/* else, supersede header is not on same unit */
				} else
				{
					/* read supersede header */
					F3S_UNIT_ENTER(part, super_phys);
					read_size = f3s_head_read (part, super_phys, head.super.index, NULL, &head, 1);
					F3S_UNIT_LEAVE(part, super_phys);
					if (read_size < 0) goto failure_shortcut;
				}
			}
		}
	}

failure_shortcut:
	*copy_num = copy_count;
/*}}}1*/
}

int f3s_reclaim_copy (f3s_part_t *part, uint32_t copy_num,
                      uint32_t *obsol_num, uint32_t free_size,
                      int32_t phys, int32_t spare_phys, uint32_t resv_num)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_extptr_t	extptr;
	f3s_extptr_t *	obsol_vect;
	f3s_map_t *		map;
	f3s_extptr_t *	realloc_vect;
	uint16_t *		move_entry;
	uint16_t *		copy_vect;
	uint8_t *		realloc_buffer;
	int32_t			text_size = 0;
	int32_t			fit_flag;
	int32_t			super_phys;
	uint32_t		text_offset;
	int32_t			align_size;
	int32_t			obsol_count;
	int32_t			realloc_size;
	int32_t			realloc_count;
	uint32_t		copy_index;
	uint32_t		move_index;
	int				err_size;
	int				error;

	uint16_t *		old_vect;
	int32_t			old_num;

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* get map pointer for spare */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	f3s_lock_assert(part->table.logi_lock_vect[part->array->phys_vect[phys]].thread == pthread_self());
	f3s_lock_assert((part->array->map_lock_vect[phys].thread == pthread_self()) && (part->array->map_lock_vect[spare_phys].thread == pthread_self()));
	map = &part->array->map_vect[spare_phys];

	/* initialize copy vector */
	copy_vect = part->table.copy_vect;

	old_vect = copy_vect;
	old_num  = copy_num;

	/* initialize obsolete counter and vector */
	obsol_count = 0;
	obsol_vect = part->table.obsol_vect;

	/* while there is something in copy vect */
	while (copy_num)
	{
		/* get index from copy vector */
		copy_index = *copy_vect;

		/* remove entry from copy vect */
		copy_vect++;
		copy_num--;

		/* DEBUG: Dump the current state of the copy vector */
		dump_copy_vect (old_vect, old_num, copy_index);

		/* read extent header from physical unit */
		text_size = f3s_head_read (part, phys, copy_index, &text_offset, &head, 1);
		if (text_size < 0) {error = errno; goto fail_shortcut;}

		/* if extent header is superseded */
		if (!(head.status[0] & F3S_EXT_NO_SUPER))
		{
			/* extract supersede extent pointer from supersede header */
			extptr = head.super;

			/* get physical unit of supersede header */
			super_phys = f3s_extptr_phys (part, extptr);

			/* check if we need to enter supersede physical unit */
			if (super_phys != phys)
				F3S_UNIT_ENTER(part, super_phys);

			/* read supersede header */
			text_size = f3s_head_read (part, super_phys, extptr.index, &text_offset, &head, 1);
			if (text_size < 0)
			{
				if (super_phys != phys)
					F3S_UNIT_LEAVE(part, super_phys);
				error = errno;
				goto fail_shortcut;
			}

			/* read supersede header loop */
			while (!(head.status[0] & F3S_EXT_NO_SUPER))
			{
				/* check if header is on same unit */
				if (super_phys == phys)
				{
					// unit lock on super_phys NOT held
					
					/* find index in copy vect */
					for (move_entry = copy_vect, move_index = 1;
					     move_index <= copy_num && *move_entry != extptr.index;
					     move_entry++, move_index++)
					{
						;
					}

					if (move_index <= copy_num)
					{
						/* remove index from copy vect */
						memmove (move_entry, move_entry + 1, (copy_num - move_index) * sizeof (*move_entry));
						copy_num--;

						old_num--;

						/* DEBUG: Dump the current state of the copy vector */
						dump_copy_vect (old_vect, old_num, copy_index);
					}
					else
					{
						/* did not find index match, this chain is bad */
						printf("(devf  t%d::%s:%d) supersede chain is bad? \n", pthread_self(), __func__, __LINE__);
						break; 	// supersede header loop
					}
				}
				else	// supersede is not on the same unit, so superseded head_t is obsolete
				{
					F3S_UNIT_LEAVE(part, super_phys);

					/* increment number of obsolete extents */
					obsol_count++;

					/* check if maximum of obsolete entries was reached */
					f3s_lock_assert(part->table_lock.thread == pthread_self());
					if (obsol_count > part->table.obsol_max)
					{
						/* setup realloc count */
						realloc_count = (obsol_count / F3S_CACHE_MAX + 1) * F3S_CACHE_MAX;

						/* reallocate obsolete vector */
						realloc_vect = f3s_realloc (obsol_vect, part->table.obsol_max * sizeof (extptr), realloc_count * sizeof (extptr));
						if (!realloc_vect) {error = ENOMEM; goto fail_shortcut;}

						obsol_vect = realloc_vect;

						/* adjust obsolete maximum */
						part->table.obsol_max = realloc_count;
					}

					/* put supersede extent pointer in obsolete vector */
					obsol_vect[obsol_count - 1] = extptr;
				}

				/* extract supersede extent pointer from supersede header */
				extptr = head.super;

				/* get physical unit of supersede header */
				super_phys = f3s_extptr_phys (part, extptr);

				/* check if we need to enter supersede physical unit */
				if (super_phys != phys)
					F3S_UNIT_ENTER(part, super_phys);

				/* read supersede header */
				text_size = f3s_head_read (part, super_phys, extptr.index, &text_offset, &head, 1);
				if (text_size < 0)
				{
					if (super_phys != phys)
						F3S_UNIT_LEAVE(part, super_phys);
					error = errno;
					goto fail_shortcut;
				}
			}	// end while (!(head.status[0] & F3S_EXT_NO_SUPER))

			// at this point, we may hold a unit lock on super_phys if it is different that phys

			/* get aligned size */
			align_size = F3S_TEXT_ALIGN (part, text_size, part->table.align_size);

			/* check if superseding text is on same unit and if so set the fit flag */
			if (super_phys == phys)
			{
				fit_flag = 1;
			}
			/* following check not required since second else if compiled out */
			else if (!(head.status[0] & F3S_EXT_BASIC))		/* if header is non-basic, clear the fit flag */
			{
				fit_flag = 0;
#if 0
/* Somehow the free_size is too big */
			/* else, check if superseding text size would fit on unit */
			} else if (align_size <= free_size)
			{
				/* adjust free size */
				free_size -= align_size;

				/* set fit flag */
				fit_flag = 1;
#endif
			}
			else
			{
				fit_flag = 0;
			}

			/* check if fit flag is set */
			if (fit_flag)
			{
				/* check if aligned text size fits in current buffer */
				if (align_size > part->table.text_size)
				{
					/* Grow the text buffer */
					realloc_size = (align_size / F3S_WRITE_MAX + 1) * F3S_WRITE_MAX;

					realloc_buffer = f3s_realloc (part->table.text_buffer, part->table.text_size, realloc_size);
					if (!realloc_buffer)
					{
						if (super_phys != phys)
							F3S_UNIT_LEAVE(part, super_phys);
						error = ENOMEM;
						goto fail_shortcut;
					}

					part->table.text_buffer = realloc_buffer;
					part->table.text_size   = realloc_size;
				}

				/* read superseding text */
				error = f3s_flash_read (part->array, super_phys, text_offset, text_size, part->table.text_buffer);
				if (error)
				{
					if (super_phys != phys)
						F3S_UNIT_LEAVE(part, super_phys);
					goto fail_shortcut;
				}

				/* XXX fit_flag is 1 iff super_phys == phys, so this branch
				 * will ALWAYS be taken
				 */
				/* check if header is on same unit */
				if (super_phys == phys)
				{
					/* find index in copy vect */
					for (move_entry = copy_vect, move_index = 1;
					     move_index <= copy_num && *move_entry != extptr.index;
					     move_entry++, move_index++)
					{
						;
					}

					if (move_index <= copy_num)
					{
						/* remove index from copy vect */
						memmove (move_entry, move_entry + 1, (copy_num - move_index) * sizeof (*move_entry));
						copy_num--;

					} else
					{
						/* did not find index match, this chain is bad */
						printf("(devf  t%d::%s:%d) supersede chain is bad? \n", pthread_self(), __func__, __LINE__);
						break;	// while (copy_num)
					}
				}
				else	// super_phys != phys
				{
					F3S_UNIT_LEAVE(part, super_phys);

					/* increment number of obsolete extents */
					obsol_count++;

					/* check if maximum of obsolete entries was reached */
					if (obsol_count > part->table.obsol_max)
					{
						/* setup realloc count */
						realloc_count = (obsol_count / F3S_CACHE_MAX + 1) * F3S_CACHE_MAX;

						/* reallocate obsolete vector */
						realloc_vect = f3s_realloc (obsol_vect, part->table.obsol_max * sizeof (extptr), realloc_count * sizeof (extptr));
						if (!realloc_vect)
						{
							error = ENOMEM;
							goto fail_shortcut;
						}

						obsol_vect = realloc_vect;

						/* adjust obsolete maximum */
						part->table.obsol_max = realloc_count;
					}

					/* put supersede extent pointer in obsolete vector */
					obsol_vect[obsol_count - 1] = extptr;
				}
			}
			else	/* else, fit flag is not set */
			{
				if (super_phys != phys)
				F3S_UNIT_LEAVE(part, super_phys);
			}
		}
		else	/* else, head is not superseded */
		{
			/* set fit flag */
			fit_flag = 1;

			/* get aligned size */
			align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

			/* check if aligned text size fits in current buffer */
			if (align_size > part->table.text_size)
			{
				/* setup realloc size */
				realloc_size = (align_size / F3S_WRITE_MAX + 1) * F3S_WRITE_MAX;

				/* allocate space for superseding text */
				realloc_buffer = f3s_realloc (part->table.text_buffer, part->table.text_size, realloc_size);
				if (!realloc_buffer) {error = ENOMEM; goto fail_shortcut;}

				/* set new value for text max */
				part->table.text_buffer = realloc_buffer;
				part->table.text_size   = realloc_size;
			}

			/* read text */
			error = f3s_flash_read (part->array, phys, text_offset, text_size, part->table.text_buffer);
			if (error) goto fail_shortcut;
		}

		// at this point, no locks are held which were acquired within this function
		
		/* check if fit flag is set */
		if (fit_flag)
		{
			/* Don't overwrite extent headers */
			f3s_assert ((map->text_size + text_size) <= (part->array->unit_size - (map->head_last * sizeof(f3s_head_t))));

			/* write the extent in spare unit with read text */
			err_size = f3s_ext_write (part, &spare_phys, copy_index, map->text_size,
			                          text_size, part->table.text_buffer,
			                          (~F3S_EXT_TYPE & ~F3S_EXT_NO_SPLIT) | head.status[0],
			                          part->table.align_pow2, 1, 0);
			if (err_size < 0) {error = errno; goto fail_shortcut;}

			/* check if header has a next */
			if (!(head.status[0] & F3S_EXT_NO_NEXT))
			{
				/* next new header */
				err_size = f3s_head_next (part, &spare_phys, copy_index, head.next, 0);
				if (err_size < 0) {error = errno; goto fail_shortcut;}
			}
			/* adjust text size and free size for map */
			map->text_size += align_size;
			map->free_size -= align_size;

			/* claculate padding size for map */
			map->pad_size += align_size - text_size;
		}
		else		/* else, fit flag is cleared */
		{
			/* write a zero sized extent in spare unit */
			f3s_lock_assert(part->table_lock.thread == pthread_self());
			err_size = f3s_ext_write (part, &spare_phys, copy_index, map->text_size,
			                          0, NULL,
			                          (~F3S_EXT_TYPE & ~F3S_EXT_BASIC) | head.status[0],
			                          part->table.align_pow2, 1, 0);
			if (err_size < 0) {error = errno; goto fail_shortcut;}

			/* supersede header by unfitting extent */
			err_size = f3s_head_super (part, &spare_phys, copy_index, extptr, 0);
			if (err_size < 0) {error = errno; goto fail_shortcut;}
		}

		/* clear last bit on previous last header */
		err_size = f3s_head_status (part, &spare_phys, map->head_last, ~F3S_EXT_LAST, 0);
		if (err_size < 0) {error = errno; goto fail_shortcut;}

		/* Count the extent header */
		map->free_size -= sizeof(f3s_head_t);

		/* for all headers before last starting at previous last */
		for (map->head_last++; map->head_last < copy_index; map->head_last++)
		{
			/* Count the extent header */
			map->free_size -= sizeof(f3s_head_t);

			/* If reserved entries are wanted */
			if (resv_num)
			{
				/* Reserve an extent header */
				resv_num--;
				map->resv_last = map->head_last;
				map->resv_num++;

			} else
			{
				/* Mark the first gap in extent headers */
				if (!map->free_num) map->free_first = map->head_last;

				/* Count the gap in extent headers */
				map->free_num++;
			}

			/* clear last bit on header */
			err_size = f3s_head_status (part, &spare_phys, map->head_last, ~F3S_EXT_LAST, 0);
			if (err_size < 0) {error = errno; goto fail_shortcut;}
		}
	}	// while(copy_num)

	/* Allocate remaining extent headers that didn't fit into gaps */
	while (resv_num)
	{
		/* Clear last bit on header */
		err_size = f3s_head_status (part, &spare_phys, map->head_last, ~F3S_EXT_LAST, 0);
		if (err_size < 0) {error = errno; goto fail_shortcut;}

		/* Allocate a reserved header */
		resv_num--;
		map->resv_num++;
		map->head_last++;
		map->resv_last  = map->head_last;
		map->free_size -= sizeof(f3s_head_t);
		map->free_first = map->head_last;
	}

	error = EOK;
fail_shortcut:
	/* setup obsolete vector with is potentially new value */
	part->table.obsol_vect = obsol_vect;
	*obsol_num = obsol_count;

	/* 
	 * DEBUG: Dump the obsolete vector to see if we're causing deleted extents
	 *        in the middle of good chains.
	 */
	dump_obsol_vect (part, obsol_vect, obsol_count);

	/* DEBUG: Verify the partition table totals match */
	if (error == EOK)
		f3s_verify_counters (part, 0, 0);

	return (error);
/*}}}1*/
}

int f3s_reclaim_clean (f3s_part_t *part, uint32_t obsol_num)
{
/*{{{1*/
	f3s_extptr_t	extptr;
	f3s_head_t		head;
	f3s_extptr_t *	obsol_vect;
	f3s_map_t *		map;
	int32_t			text_size;
	int32_t			phys;
	int				err_size;
	int				error;

	/* initialize obsolete vector */
	obsol_vect = part->table.obsol_vect;

	/* while there is an entry in obsolete vector */
	while (obsol_num)
	{
		/* read the extent pointer from entry */
		extptr = *obsol_vect;

		/* remove entry from vector */
		obsol_vect++;
		obsol_num--;

		/* get the physical unit */
		F3S_LOGI_ENTER(part, extptr.logi_unit);
		phys = f3s_extptr_phys (part, extptr);
		F3S_UNIT_ENTER(part, phys);

		/* get map pointer for physical unit */
		f3s_lock_assert(part->array->map_lock_vect[phys].thread == pthread_self());
		map = &part->array->map_vect[phys];

		/* read the header */
		text_size = f3s_head_read (part, phys, extptr.index, NULL, &head, 1);
		if (text_size < 0)
		{
			error = errno;
			F3S_UNIT_LEAVE(part, phys);
			F3S_LOGI_LEAVE(part, extptr.logi_unit);
			return (error);
		}

		/* if the header has not already been deleted */
		if ((head.status[0] & F3S_EXT_MASK) != F3S_EXT_DELETE)
		{
			/* delete the header */
			err_size = f3s_head_status (part, &phys, extptr.index,
			                            ~F3S_EXT_MASK | F3S_EXT_DELETE, 1);
			if (err_size < 0)
			{
				F3S_UNIT_LEAVE(part, phys);
				F3S_LOGI_LEAVE(part, extptr.logi_unit);
				error = errno;
				f3s_log_sticky (part->array->index, part->index, extptr.logi_unit, phys, extptr.index);
				return (error);
			}

			f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
			map = &part->array->map_vect[phys];

			/* increment deleted header in map */
			map->del_num++;

			/* make stale any non-superseded extents */
			if (head.status[0] & F3S_EXT_NO_SUPER)
				f3s_unit_stale (part, phys, text_size);
		}

		F3S_UNIT_LEAVE(part, phys);
		F3S_LOGI_LEAVE(part, extptr.logi_unit);
	}

	f3s_verify_headers (part, -1);
	return (EOK);
/*}}}1*/
}

#ifdef INCLUDE_DUMP_FUNCS
static void _dump_copy_vect (uint16_t *copy_vect, uint32_t copy_num, uint32_t cur_idx)
{
	uint32_t	i;

	printf("[");
	for (i = 0; i < copy_num; i++) {
		if (copy_vect[i] == cur_idx) printf (" *%2.2x", copy_vect[i]);
		else                         printf (" %2.2x",  copy_vect[i]);
	}
	printf(" ]\n");
}

static void _dump_obsol_vect (f3s_part_t *part, f3s_extptr_t *obsol_vect, uint32_t obsol_num)
{
	uint32_t	i;

	printf("[");
	for (i = 0; i < obsol_num; i++) {
		printf (" [%2.2x,%2.2x,%2.2x]",
		        obsol_vect[i].logi_unit, part->table.logi_vect[obsol_vect[i].logi_unit], obsol_vect[i].index);
	}
	printf(" ]\n");
}
#endif	/* INCLUDE_DUMP_FUNCS */

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/reclaim.c $ $Rev: 580669 $" );
