/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_resv_alloc.c
**
** Description:
**
** This file contains the file create function for the f3s resource manager
** for Neutrino
**
** Ident: $Id: resv_alloc.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

int32_t
f3s_resv_alloc (f3s_part_t * part,
				f3s_resv_t * resv,
				int32_t size)
{
	uint32_t status;
	int32_t phys;
	f3s_map_t *map;

	if ((resv->allocated) && (f3s.verbose >= 1))
		printf("(devf  t%d::%s:%d) WARNING: reallocating reservation over an existing %s reservation\n",
					pthread_self(), __func__, __LINE__,
					(resv == &part->table.deletion_resv) ? "deletion" : "attr or append buffer");
	f3s_assert(resv->allocated == 0);
	
	/* allocate unit with free space */
	phys = f3s_unit_alloc (part, -1, size, 1);
	if (phys < 0)
		return (phys);

	/* set logical unit number into reservation structure */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	resv->logi = part->array->phys_vect[phys];

	/* get map of physical unit */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];

	/* check if there are free headers before the last */
	if (map->free_num)
	{
		/* get new index from free first in map */
		uint32_t  first_free_idx = map->free_first;

		/* find the next free index if one exists (don't consider the one we are taking) */
		if (map->free_num > 1)
		{
			uint32_t  next_free_idx = map->free_first;
			/* find next free entry loop */
			do
			{
				/* note >= check because if we hit the last, free_num was wrong */
				if ((++next_free_idx >= map->head_last) ||
					(f3s_head_check (part, phys, next_free_idx, &status) < 0))
				{
					F3S_UNIT_LEAVE (part, phys);
					F3S_LOGI_LEAVE (part, part->array->phys_vect[phys]);
					return (-1);
				}
			} while ((status & F3S_EXT_MASK) != F3S_EXT_FREE);

			/* successfully found the new map->free_first. Update data structures and continue */
			map->free_first = next_free_idx;
		}
		else
		{
			/* make map free first equal to map head last */
			map->free_first = map->head_last;
		}

		/* set free flag into reservation structure */
		resv->free_flag = 1;

		/* setup map resv last to map free first */
		map->resv_last = first_free_idx;

		/* increment number of reserved in map */
		++map->resv_num;

		/* decrement number of free indexes */
		--map->free_num;
	}
	else		/* else, there are no free headers */
	{
		/* clear free flag into reservation structure */
		resv->free_flag = 0;

		/* add header size to reservation size */
		size += sizeof(f3s_head_t);
	}

	/* set size into reservation structure */
	resv->size = size;

	/* reflect reservation into physical unit map */
	map->free_size -= size;
	map->resv_size += size;

	/* reflect reservation into partition table */
	F3S_ALLOC_ENTER(part);
	part->table.free_total -= size;
	part->table.resv_total += size;
	F3S_ALLOC_LEAVE(part);

	resv->allocated = 1;	// mark the reservation as allocated
	F3S_UNIT_LEAVE (part, phys);
	F3S_LOGI_LEAVE (part, part->array->phys_vect[phys]);

	/* return size of data reserved */
	return (size);
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/resv_alloc.c $ $Rev: 580669 $" );
