/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_resv_free.c
**
** Description:
**
** This file contains the file create function for the f3s resource manager
** for Neutrino
**
** Ident: $Id: resv_free.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_resv_free
*/

void
f3s_resv_free (f3s_part_t * part,
			   f3s_resv_t * resv)
{
	uint32_t status;
	int32_t phys, read_size;
	f3s_map_t *map;

	/* Calling code doesn't always know if the reservation is free'd or not.
	 * For instance, the partition wide "deletion" reservation always
	 * exists, it just might not be allocated.
	 */
	if (resv->allocated == 0){
		return;
	}

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* translate logi from reservation structure into physical index */
	f3s_lock_assert((part->table_lock.thread == pthread_self()));
	F3S_LOGI_ENTER (part, resv->logi);
	phys = part->table.logi_vect[resv->logi];
	F3S_UNIT_ENTER (part, phys);

	/* get map of physical unit */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];

	f3s_assert(map->free_first <= (map->head_last + 1));
	f3s_assert((resv->size + map->text_size + map->free_size + ((map->head_last + 1) * sizeof(f3s_head_t))) <=
							part->array->unit_size);

	/* check if a free header was reserved */
	if (resv->free_flag)
	{
		/* setup map free first to map resv last */
		map->free_first = map->resv_last;

		/* increment map free num in map */
		map->free_num++;

		/* decrement number of reserved in map */
		map->resv_num--;

		/* if there are still reserved indexes */
		if (map->resv_num)
		{
			/* find previous free entry loop */
			do
			{
				/* decrement resv last in map */
				map->resv_last--;

				/* read extent header from physical unit */
				read_size = f3s_head_check (part, phys, map->resv_last, &status);
				if (read_size < 0)
				{
					F3S_UNIT_LEAVE (part, phys);
					F3S_LOGI_LEAVE (part, resv->logi);
					return;
				}

			/* check if extent header is free */
			} while ((status & F3S_EXT_MASK) != F3S_EXT_FREE);
		} else
		{
			/* clear map resv last */
			map->resv_last = 0;
		}
	}

	/* reflect reservation into physical unit map */
	map->free_size += resv->size;
	map->resv_size -= resv->size;

	/* reflect reservation into partition table */
	F3S_ALLOC_ENTER(part);
	part->table.free_total += resv->size;
	part->table.resv_total -= resv->size;
	F3S_ALLOC_LEAVE(part);

	resv->allocated = 0;	// mark the reservation as unallocated

	F3S_UNIT_LEAVE (part, phys);
	F3S_LOGI_LEAVE (part, resv->logi);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/resv_free.c $ $Rev: 580669 $" );
