/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

static int f3s_skt_version (f3s_flash_t *flash_vect)
{
/*{{{1*/
	f3s_flash_v2_t *v2flash;

	/* Check common callouts */
	if (flash_vect->ident == NULL) return (EINVAL);
	if (flash_vect->reset == NULL) return (EINVAL);

	/* Validate v1 callouts */
	if (flash_vect->struct_size == sizeof(f3s_flash_t))
	{
		if (flash_vect->write   == NULL) return (EINVAL);
		if (flash_vect->sync    == NULL) return (EINVAL);

		/* Can't have one without the other */
		if ((flash_vect->erase != NULL) &&
		    ((flash_vect->suspend == NULL) ^
		     (flash_vect->resume  == NULL))) return (EINVAL);

		return (EOK);

	/* Validate v2 callouts */
	} else if (flash_vect->struct_size == sizeof(f3s_flash_v2_t))
	{
		v2flash = (f3s_flash_v2_t *)flash_vect;

		if (v2flash->v2write   == NULL) return (EINVAL);
		if (v2flash->v2erase   == NULL) return (EINVAL);
		if (v2flash->v2sync    == NULL) return (EINVAL);

		/* Can't have one without the other */
		if ((v2flash->v2erase != NULL) &&
		    ((v2flash->v2suspend == NULL) ^
		     (v2flash->v2resume  == NULL))) return (EINVAL);

		return (EOK);
	}

	return (EINVAL);
/*}}}1*/
}

int f3s_skt_attach (f3s_array_t *array, f3s_attr_t *skt_attr,
                    f3s_flash_t *flash_vect)
{
/*{{{1*/
	char			name[16];
	f3s_access_t *	access;
	int32_t			chip_total;
	int				error;
	unsigned		rflag = 0|_RESMGR_FLAG_SELF;

	/* Sanity check callout version */
	error = f3s_skt_version (flash_vect);
	if (error) return (error);

	/* setup array index */
	array->index = array->access_vect[0].socket.socket_index;

	/* build pathname for skt mountpoint */
	sprintf (name, _PATH_DEV "fs%X", array->index);

	/* attach pathname for skt mountpoint */
#ifdef ENABLE_MOUNTBEHIND
	if (f3s.mount_behind){
		rflag |= _RESMGR_FLAG_AFTER;
	}
#endif
	skt_attr->attr.mount->attach_id[F3S_ATTACH_SKT] =
		resmgr_attach (f3s.dpp, &f3s.resmgr_attr, name,
		               _FTYPE_FILE, rflag,
		               &f3s.raw_connect_funcs, &f3s.raw_io_funcs, skt_attr);
	if (skt_attr->attr.mount->attach_id[F3S_ATTACH_SKT] < 0) return (errno);

	/* set inode number in attributes */
	error = resmgr_devino (skt_attr->attr.mount->attach_id[F3S_ATTACH_SKT], &skt_attr->attr.mount->mount.dev, &skt_attr->attr.inode);
	if (error) return (error);

	/* initialize mount list of mountpoint pairs */
	f3s_list_init (&skt_attr->attr.mount->mounted.pair_list);

	/* setup flash vector in mount structure */
	skt_attr->attr.mount->array->flash_vect = flash_vect;

	/* log socket attachment */
	f3s_log_socket (array->index, array->access_vect[0].socket.name);

	/* initialize flash pointer to flash vector */
	array->flash = array->flash_vect;

	/* probe for type and size of flash array */

	/*
	 * Note: the current implementation has only 1 access_vect structure. If this changes, this code
	 * or f3s_flash_probe() will have to change to deal with that
	*/
	chip_total = f3s_flash_probe (&array->access_vect[0], &array->flash, &array->dbase);
	if (chip_total < 0) return (ENOENT);

	/* copy chip total to array */
	array->chip_total = chip_total;

	/* call array calculation function */
	error = f3s_array_calc (array);
	if (error) return (error);

	/* allocate partition mapping array */
	error = f3s_array_alloc (array);
	if (error) return (error);

	/* log array detection */
	f3s_log_array (array->index, array->unit_total, array->unit_size, array->dbase.name);

	/* setup size for attributes */
	skt_attr->attr.nbytes = array->chip_total << array->chip_pow2;

	/* get access pointer */
	access = &array->access_vect[0];

	/* check if array is inserted in socket */
	error = access->service->status (&access->socket, F3S_OPER_SOCKET);
	if (!error && (f3s.mount_options & f3s_mntopts_t_AUTOMNT))
	{
		/* call array attach function */
		iofunc_attr_lock(&skt_attr->attr);
		error = f3s_array_attach (array, skt_attr);
		iofunc_attr_unlock(&skt_attr->attr);
	}

	/*Clear the mountexact list, so that once the initial auto-mount is done, mounts behave normally. */
	if (f3s.mountexact_num > 0){
		for (; f3s.mountexact_num; f3s.mountexact_num--){
			free(f3s.mountexact_list[f3s.mountexact_num-1]);
		}
		free(f3s.mountexact_list);
	}

	/* return error code */
	return (error);
/*}}}1*/
}

int f3s_skt_detach (f3s_array_t *array, f3s_attr_t *skt_attr)
{
/*{{{1*/
	int32_t	flag;
	int		error;

	/* Detach partition attributes from socket */
	error = f3s_array_detach (array, skt_attr);
	if (error) return (error);

	/* Detach raw mountpoint name */
	flag = resmgr_detach (f3s.dpp,
	                      skt_attr->attr.mount->attach_id[F3S_ATTACH_SKT],
	                      _RESMGR_DETACH_ALL);
	if (flag < 0) return (errno);

	/* Free array vectors */
	f3s_array_free (array);

	/* Clear size for attributes */
	skt_attr->attr.nbytes = 0;
	return (EOK);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/socket.c $ $Rev: 580669 $" );
