/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_spare_grab.c
**
** Description:
**
** This file contains the spare grab function for the f3s resource manager for
** Neutrino
**
** Ident: $Id: spare_grab.c 580669 2011-09-01 15:18:50Z builder@qnx.com $
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_spare_grab
*/


/* Returns the physical block.  If an error occurs, returns -1 and
 * sets errno.  The "dirty" arg is a true/false flag to indicate
 * whether the spare requires a refresh (erase) prior to be
 * written in to.
 *
 * Required:  The caller must either pass a valid variable
 * for the "dirty" argument, or assume that the returned
 * phys is dirty.
 *
 * 
 */
int32_t f3s_spare_grab (f3s_part_t * part, int* dirty)
{
	int32_t phys;
	f3s_spare_t *spare_entry;
	f3s_spare_t *spare_entry_iter;

	f3s_assert(NULL != dirty);
	
	// should be a spare on the list, grab it
	F3S_SPARE_ENTER (part);

	/* check if there is an entry in spare list */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.spare_list.next != &part->table.spare_list) {

		/* get a spare unit entry from list */
		spare_entry = (f3s_spare_t *) part->table.spare_list.next;

		spare_entry_iter = spare_entry;
	
		/* Search for the entry with the lowest erase count.
		 * The list constructs are a little odd, in that they are a ring,
		 * with the handle into the ring being a fake entry.  This means
		 * that we start searching at part->table.spare_list.next, and loop
		 * we reach part->table.spare_list.
		 */
		while (spare_entry_iter != (f3s_spare_t*)&(part->table.spare_list)) {
			spare_entry = (spare_entry_iter->erase_count < spare_entry->erase_count)?spare_entry_iter:spare_entry;	
			spare_entry_iter = (f3s_spare_t*)spare_entry_iter->list.next;
		}
		
		/* get spare unit index from spare entry */
		phys = spare_entry->phys;
		if (dirty){
			*dirty = spare_entry->dirty;
		}

		/* remove spare unit from spare list */
		f3s_list_rem ((f3s_list_t *) spare_entry);

		/* free spare entry */
		f3s_free (spare_entry, sizeof (*spare_entry));

		F3S_UNIT_ENTER (part, phys);
	}
	else	/* else, there is no spare unit available */
	{
		errno = ENOSPC;
		phys = -1;
	}

	F3S_SPARE_LEAVE (part);
	/* return spare physical number */
	return phys;
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/spare_grab.c $ $Rev: 580669 $" );
