/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */


#include "f3s.h"

static uint32_t FF32[]={0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
				0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF};

static int f3s_head_fix    (f3s_part_t *part, f3s_head_t *head, uint32_t phys);
static int f3s_head_verify (f3s_part_t *part, f3s_head_t *head, uint32_t phys);

int32_t
f3s_dirent_read (f3s_part_t * part,
				 f3s_desc_t * desc,
				 int32_t desc_offset,
				 f3s_dirent_t * dirent)
{
/*{{{1*/
	int32_t read_size;

	/* read directory entry */
	read_size = f3s_file_read (part, desc, desc_offset, sizeof(*dirent), (uint8_t *)dirent);
	if (read_size <= 0) return (read_size);

	/* check if dirent structure needs swapping */
	if (part->table.swap_flag) f3s_dirent_swap (dirent);

	/* Sanity checks */
	if (dirent->struct_size != sizeof(*dirent)) {
		errno = EBADFSYS;
		return (-1);
	}

	/* return size read */
	return (read_size);
/*}}}1*/
}

int32_t f3s_head_read (f3s_part_t *part, int32_t phys, uint32_t index,
                       uint32_t *text_offset, f3s_head_t *head, int fix)
{
/*{{{1*/
	int		error;
	int		align_pow2, swap_flag;
 
	//F3S_TABLE_ENTER(part);
	align_pow2 = part->table.align_pow2;
	swap_flag = part->table.swap_flag;
	//F3S_TABLE_LEAVE(part);

	/* read head */
	error = f3s_flash_read (part->array, phys,
	                        F3S_HEAD_OFFSET(part->array, index),
	                        sizeof(*head), (uint8_t *)head);
	if (error) {errno = error; return (-1);}

	/* check if head structure needs swapping */
	//f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (swap_flag) f3s_head_swap (head);

	/* Fix or verify the header */
	if (fix)
	{
		if (((head->status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR) && (fix == 1))
		{
			if(*(uint32_t *)&head->super == 0 && *(uint32_t *)&head->next == 0)
			{
				head->text_size = 0;
			}
		}
		error = f3s_head_fix (part, head, phys);
		if (error) {errno = error; return (-1);}
	} else
	{
		error = f3s_head_verify (part, head, phys);
		if (error) {errno = error; return (-1);}
	}

	if (error){
		/* Check if this is a bad head, or an ENODATA */
		if (memcmp(head, FF32, sizeof (f3s_head_t))){
			errno = error;
		}else{
			errno = ENODATA;
		}
		return -1;
	}
				

	/* check if text offset is wanted */
	if (text_offset)
	{
		/* check type of header for alignment */
		if ((head->status[0] & F3S_EXT_TYPE) == F3S_EXT_SYS)
		{
			/* extract text offset */
			*text_offset  = head->text_offset_lo << F3S_SYS_POW2;
			*text_offset += head->text_offset_hi << (F3S_SYS_POW2 + F3S_OFFSET_HI_POW2);

		} else
		{
			/* extract text offset */
			*text_offset  = head->text_offset_lo << align_pow2;
			*text_offset += head->text_offset_hi << (align_pow2 + F3S_OFFSET_HI_POW2);
		}
	}

	if (head->status[0] & F3S_EXT_NO_WRITE) return (0);
	else                                    return (head->text_size);
/*}}}1*/
}

int32_t f3s_head_check (f3s_part_t *part, int32_t phys, uint32_t index,
                        uint32_t *status)
{
	f3s_head_t	head;

	if (f3s_head_read(part, phys, index, NULL, &head, 1) < 0)
		return(-1);
	else
	{
		*status = head.status[0];
		return (sizeof(*status));
	}
}

int f3s_head_write (f3s_part_t *part, int32_t *phys, uint32_t index,
                    f3s_head_t *head, int recover)
{
/*{{{1*/
	f3s_head_t	thead;
	uint32_t	head_offset;
	int			error;

	memset (&thead, 0xFF, sizeof(f3s_head_t));
	memcpy (((uint8_t *)&thead) + offsetof(f3s_head_t, pad[2]),
	        ((uint8_t *)head)  + offsetof(f3s_head_t, pad[2]),
	        sizeof(f3s_head_t) - offsetof(f3s_head_t, pad[2]));

	/* Clear F3S_EXT_LAST flag if necessary */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[*phys].thread == pthread_self()));
	if (index < part->array->map_vect[*phys].head_last) thead.status[0] &= ~F3S_EXT_LAST;

	/* Prepare to mark the header as allocated, but not written */
	thead.status[0] &= ~F3S_EXT_MASK;
	thead.status[0] |= F3S_EXT_ALLOC;

	thead.status[1] = thead.status[0];
	thead.status[2] = thead.status[0];

	/* if next needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_head_swap (&thead);

	/* calculate header offset */
	head_offset = F3S_HEAD_OFFSET(part->array, index);

	/* write the extent header */
	error = f3s_flash_write (part->array, *phys,
	                         head_offset + offsetof(f3s_head_t, pad[2]),
	                         (uint32_t)(offsetof(f3s_head_t, next) - offsetof(f3s_head_t, pad[2])),
	                         ((uint8_t *)&thead) + offsetof(f3s_head_t, pad[2]), F3S_VERIFY_WRITE);
	if (error) goto recover;

	/* Mark the header as allocated, but not written */
	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[0]),
	                       thead.status[0]);
	if (error) goto recover;
	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[1]),
	                       thead.status[1]);
	if (error) goto recover;
	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[2]),
	                       thead.status[2]);
	if (error) goto recover;

	return (EOK);
recover:
	if (!(recover) || (error != EIO)) return (error);

	/* Swap back thead into processor endian */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_head_swap (&thead);


	/* Repair the damage by copying to a new unit */
	error = f3s_head_repair (part, phys, index, &thead);
	return (error);
/*}}}1*/
}

/* Potentially factor f3s_head_next() and f3s_head_super() */
int32_t f3s_head_next (f3s_part_t *part, int32_t *phys, uint32_t index,
                       f3s_extptr_t next, int recover)
{
/*{{{1*/
	f3s_head_t	head;
	uint32_t	head_offset;
	uint32_t	text_offset;
	uint32_t	status;
	int			err_size;
	int			error;

	/* Sanity check */
	if (next.logi_unit == 0 && next.index != 0) return (EINVAL);

	/* Read the old header */
	err_size = f3s_head_read (part, *phys, index, &text_offset, &head, 1);
	if (err_size < 0) return (-1);

	/* check if next needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_extptr_swap (&next);

	/* calculate header offset */
	head_offset = F3S_HEAD_OFFSET(part->array, index);

	/* write next pointer in head info */
	error = f3s_flash_write (part->array, *phys,
	                         head_offset + offsetof (f3s_head_t, next),
	                         sizeof(next), (uint8_t *)&next, F3S_VERIFY_WRITE);

	/* flag header as nexted */
	status = ~F3S_EXT_NO_NEXT;

	/* check if status needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) ENDIAN_SWAP32(&status);
	if (error) goto recover;

	/* apply mask */
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[0]), status);
	if (error) goto recover;
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[1]), status);
	if (error) goto recover;
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[2]), status);
	if (error) goto recover;
	return (sizeof(next));

recover:
	if (!(recover) || (error != EIO)) {errno = error; return (-1);}

	/* Patch the header */
	head.status[0] &= ~F3S_EXT_NO_NEXT;
	head.status[1] &= ~F3S_EXT_NO_NEXT;
	head.status[2] &= ~F3S_EXT_NO_NEXT;
	if (part->table.swap_flag){f3s_extptr_swap(&next);}
	head.next       = next;

	/* Repair the damage by copying to a new unit */
	error = f3s_head_repair (part, phys, index, &head);
	if (error) {errno = error; return (-1);} 

	return (sizeof(next));
/*}}}1*/
}

int32_t f3s_head_super (f3s_part_t *part, int32_t *phys, uint32_t index,
                        f3s_extptr_t super, int recover)
{
/*{{{1*/
	f3s_head_t	head;
	uint32_t	head_offset;
	uint32_t	text_offset;
	uint32_t	status;
	int			err_size;
	int			error;

	/* Sanity check */
	if (super.logi_unit == 0 && super.index != 0) return (EINVAL);

	/* Read the old header */
	err_size = f3s_head_read (part, *phys, index, &text_offset, &head, 1);
	if (err_size < 0) return (-1);

	/* check if super needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_extptr_swap (&super);

	/* calculate header offset */
	head_offset = F3S_HEAD_OFFSET (part->array, index);

	/* write supersede pointer in head info */
	error = f3s_flash_write (part->array, *phys,
	                         head_offset + offsetof (f3s_head_t, super),
	                         sizeof(super), (uint8_t *)&super, F3S_VERIFY_WRITE);

	/* flag header as superseded */
	status = ~F3S_EXT_NO_SUPER;

	/* swap the status */
	if (part->table.swap_flag) ENDIAN_SWAP32(&status);
	if (error) goto recover;

	/* apply mask */
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[0]), status);
	if (error) goto recover;
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[1]), status);
	if (error) goto recover;
	error = f3s_flash_mask (part->array, *phys,
	                        head_offset + offsetof(f3s_head_t, status[2]), status);
	if (error) goto recover;

	f3s_verify_headers(part, -1);
	return (sizeof(super));

recover:
	if (!(recover) || (error != EIO)) {errno = error; return (-1);}

	/* Patch the header */
	head.status[0] &= ~F3S_EXT_NO_SUPER;
	head.status[1] &= ~F3S_EXT_NO_SUPER;
	head.status[2] &= ~F3S_EXT_NO_SUPER;
	if (part->table.swap_flag){f3s_extptr_swap(&super);}
	head.super      = super;

	/* Repair the damage by copying to a new unit */
	error = f3s_head_repair (part, phys, index, &head);
	if (error) {errno = error; return (-1);} 

	f3s_verify_headers(part, -1);
	return (sizeof(super));
/*}}}1*/
}

int32_t f3s_head_status (f3s_part_t *part, int32_t *phys, uint32_t index,
                         uint32_t status, int recover)
{
/*{{{1*/
	f3s_head_t	head;
	uint32_t	head_offset;
	uint32_t	text_offset;
	int			err_size;
	int			error;

	/* Read the old header */
	err_size = f3s_head_read (part, *phys, index, &text_offset, &head, 1);
	if (err_size < 0) return (-1);

	head_offset = F3S_HEAD_OFFSET(part->array, index);
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) ENDIAN_SWAP32(&status);

	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[0]),
	                       status);
	if (error) goto recover;
	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[1]),
	                       status);
	if (error) goto recover;
	error = f3s_flash_mask(part->array, *phys,
	                       head_offset + offsetof(f3s_head_t, status[2]),
	                       status);
	if (error) goto recover;

	return (sizeof(status));

recover:
	if (!(recover) || (error != EIO)) {errno = error; return (-1);}

	/* Patch the header (return status to its pre-swapped state for call to head repair) */
	if (part->table.swap_flag) ENDIAN_SWAP32 (&status);
	head.status[0] &= status;
	head.status[1] &= status;
	head.status[2] &= status;

	/* Repair the damage by copying to a new unit */
	error = f3s_head_repair (part, phys, index, &head);
	if (error) {errno = error; return (-1);} 

	return (sizeof(status));
/*}}}1*/
}

int f3s_head_repair (f3s_part_t *part, int32_t *phys, uint32_t index,
                     f3s_head_t *head)
{
/*{{{1*/
	uint8_t			buf[64];
	f3s_head_t		thead;
	f3s_map_t *		map;
	uint32_t		age;
	int32_t			logi;
	int32_t			spare_phys;
	int32_t			reclaim_phys = *phys;
	uint32_t		text_offset;
	int32_t			text_size;
	int32_t			togo;
	int32_t			xfer;
	int32_t			i;
	int				err_size;
	int				error;
	unsigned   retry_cnt = 2 * part->table.unit_spare;
	int				dirty;
	
	/* DEBUG: Double check header integrity */
	f3s_verify_headers (part, -1);

	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[reclaim_phys].thread == pthread_self()));
	map  = &part->array->map_vect[reclaim_phys];
	logi = part->array->phys_vect[reclaim_phys];
	f3s_lock_assert(part->table.logi_lock_vect[logi].thread == pthread_self());
	age  = part->array->age_vect[reclaim_phys];

	// caution: because of the way this code was written, do not break this loop without an explicit goto
	while (1)
	{
		/* Grab a spare block */
		spare_phys = f3s_spare_grab (part, &dirty);
		if (spare_phys < 0){
			return (errno);
		}

		f3s_log_head_repair (part->array->index, part->index, logi,
		                     reclaim_phys, spare_phys, map->stale_size);
		/* if the spare is dirty, clean it up */
		if (dirty){
			error = f3s_unit_refresh(part, spare_phys);
			if (error){
				f3s_spare_drop (part, spare_phys, dirty);//put it back, still marked as dirty
				if (retry_cnt == 0)
				{
					printf("(devf  t%d::%s:%d) Unable to find an suitable spare after %d attempts\n",
								pthread_self(), __func__, __LINE__, 2 * part->table.unit_spare);
					errno = EIO;
					return (-1); 
				}
				else
				{
					--retry_cnt;
					continue;
				}
			}
		}


		/* Write the initial logical unit header (without MD5) */
		error = f3s_unit_logi1 (part, &spare_phys, logi, age + 1, 0);
		if (error) 
		{
			/*
			 * at this point, the current spare_phys does not appear to be usable (f3s_unit_logi1() does not
			 * provide an indication as to what went wrong, so for now we will assume it is not usable. May
			 * want to consider trying to determine whether or not the spare_phys is recoverable and if so put it
			 * back on the spare list). The unit lock on spare_phys will be kept and the unit will be left dangling
			 * for now (as it was in the previous version of the code) until it is decided whether or not an attempt to
			 * recover it should be made.
			 * In summary, the only change made was to try and use a different spare (if one should exist) but limit
			 * the number of attempts to twice the number of spares for the partition
			*/
			if (retry_cnt == 0)
			{
				printf("(devf  t%d::%s:%d) Unable to find an suitable spare after %d attempts\n",
							pthread_self(), __func__, __LINE__, 2 * part->table.unit_spare);
				errno = EIO;
				return (-1); 
			}
			else
			{
				--retry_cnt;
				continue;
			}
		}

		/* Copy all of the extents */
		for (i = F3S_UNIT_LOGI_INDEX + 1; i <= map->head_last; i++)
		{
			/* If we're writing the boot header */
			if ((logi == 1) && (i == F3S_BOOT_INDEX))
			{
				/* Update the boot header */
				f3s_lock_assert(part->table_lock.thread == pthread_self());
				error = f3s_boot_write (part, spare_phys,
				                        part->table.root_extptr, part->unit_num,
				                        part->table.unit_spare,
				                        part->table.align_pow2,
				                        spare_phys - part->unit_first);
				if (error) goto recover;

				/* 
				 * Since we re-wrote the boot header, it didn't carry over
				 * it's F3S_EXT_LAST state (which would have been cleared to
				 * make room for the root dirent.
				 */

				/* We know the root directory must follow, so never last */
				err_size = f3s_head_status (part, &spare_phys, F3S_BOOT_INDEX,
				                            ~F3S_EXT_LAST, 0);
				if (err_size < 0) goto recover;

			/* Otherwise, it involves copying data */
			} else
			{
				/* If this is the index we're rewriting */
				if ((head != NULL) && (i == index))
				{
					memcpy(&thead, head, sizeof(thead));
					f3s_lock_assert(part->table_lock.thread == pthread_self());

					text_offset  = head->text_offset_lo << part->table.align_pow2;
					text_offset += head->text_offset_hi << (part->table.align_pow2 + F3S_OFFSET_HI_POW2);
					text_size    = head->text_size;

				} else
				{
					/* Read the header */
					text_size = f3s_head_read (part, reclaim_phys, i, &text_offset, &thead, 1);
					if (text_size < 0) goto recover;
				}

				if ((thead.status[0] & F3S_EXT_MASK) != F3S_EXT_ALLOC ||
				    !(thead.status[0] & F3S_EXT_NO_SUPER)){
					/* The data of this extent isn't valid, so don't copy it. */
				}else{
					/* Copy the data */
					togo = text_size;
					while (togo)
					{
						/* Compute the xfer size */
						xfer = min(togo, sizeof(buf));

						/* Copy the data */
						error = f3s_flash_read (part->array, reclaim_phys, text_offset,
					                        xfer, buf);
						if (error) goto recover;
	
						/* The reclaim must be exact, every write must be verified */
						error = f3s_flash_write (part->array, spare_phys, text_offset,
					                         xfer, buf, F3S_VERIFY_WRITE);
						if (error) goto recover;

						/* Update counters */
						togo        -= xfer;
						text_offset += xfer;
					}
				}

				/*
				 * Must clear the last bit here, not earlier, in case there
				 * are no more extents after the logical header.
				 */
				if (i == F3S_UNIT_LOGI_INDEX + 1)
				{
					/* Force clear the last header bit on the info structure */
					err_size = f3s_head_status (part, &spare_phys, F3S_UNIT_LOGI_INDEX,
					                            ~F3S_EXT_LAST, 0);
					if (err_size < 0) goto recover;
				}
				
				/* Be sure to Swap back the header */
				f3s_lock_assert(part->table_lock.thread == pthread_self());
				if (part->table.swap_flag){
					f3s_head_swap(&thead);
				}
					
				/* Write the extent header */
				error = f3s_flash_write (part->array, spare_phys,
				                         F3S_HEAD_OFFSET(part->array, i),
				                         sizeof(thead), (uint8_t *)&thead,
				                         F3S_VERIFY_WRITE);
				if (error) goto recover;
			}
		}
		goto done;
	}

	// if we get here, it needs to be because of an unconditional branch. See comments above at while(1)
recover:
	/* Try and put the spare back, return the error that got us here */
	f3s_log_error(part->array->index, part->index, error);
	if (f3s_unit_refresh (part, spare_phys) == EOK)
		f3s_spare_drop (part, spare_phys, F3S_NOT_DIRTY);
	else
		f3s_spare_drop (part, spare_phys, F3S_DIRTY);
	return(error);
			
done:
	/* Make the logical unit header valid */
	error = f3s_unit_logi2 (part, &spare_phys, logi, age + 1, 0);
	if (error) return (error);

	/* Switch to the new block */
	f3s_lock_assert((part->table_lock.thread == pthread_self()));
	part->table.logi_vect[logi]          = spare_phys;
	f3s_lock_assert((part->array->map_lock_vect[spare_phys].thread == pthread_self()) && (part->array->map_lock_vect[reclaim_phys].thread == pthread_self()));
	part->array->phys_vect[reclaim_phys] = 0;
	part->array->phys_vect[spare_phys]   = logi;
	part->array->age_vect[reclaim_phys]  = 0xFF;
	part->array->age_vect[spare_phys]    = age + 1;
	part->array->map_vect[spare_phys]    = *map;
	*phys = spare_phys;

	/* Transfer the lock count from reclaim_phys to spare_phys */
	while (part->array->map_lock_vect[spare_phys].count <
	       part->array->map_lock_vect[reclaim_phys].count)
	{
		F3S_UNIT_ENTER(part, spare_phys);
	}

	/* Reduce lock count of spare_phys to 1 */
	while (part->array->map_lock_vect[reclaim_phys].count > 1)
	{
		F3S_UNIT_LEAVE(part, reclaim_phys);
	}

	/* Erase the original block */
	error = f3s_unit_refresh (part, reclaim_phys);
	f3s_spare_drop (part, reclaim_phys, error?F3S_DIRTY:F3S_NOT_DIRTY);

	/* Force space allocation tables to re-sort */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	part->table.status |= F3S_TABLE_NO_SORT;

	/* DEBUG: Double check header integrity */
	f3s_verify_headers (part, -1);
	return (EOK);
/*}}}1*/
}

static int f3s_head_fix (f3s_part_t *part, f3s_head_t *head, uint32_t phys)
{
/*{{{1*/
	uint32_t lastindex = part->array->unit_size / sizeof (f3s_head_t);
	uint32_t lastunit  = part->unit_num;
#ifndef NDEBUG
	char m[1024];
	unsigned n = 0;
	unsigned p;
	n += sprintf(&m[n], "%s(0x%.8x, 0x%.8x, %d)\nlast index = %d, last unit = %d\n",
					__func__, (unsigned int)part, (unsigned int)head, phys, lastindex, lastunit);
	p = n;
#endif	/* NDEBUG */

	/* Sanity check the status bits */
	if (head->status[0] != head->status[1])
	{
		/* All status field copies are different, use the last one */
		if (head->status[1] != head->status[2])
		{
#ifndef NDEBUG
			n += sprintf(&m[n], "\tAll 3 status words are different, [0] == 0x%.8x, [1] == 0x%.8x, [2] == 0x%.8x, choosing [2]\n",
			head->status[0], head->status[1], head->status[2]);
#endif	/* NDEBUG */

			head->status[0] = head->status[2];
			head->status[1] = head->status[2];
		} else
		{
#ifndef NDEBUG
			n += sprintf(&m[n], "\tstatus words [1] == [2] == 0x%.8x, status [0] == 0x%.8x, choosing [1]/[2]\n", head->status[1], head->status[0]);
#endif	/* NDEBUG */
			head->status[0] = head->status[1];
		}

	} else if (head->status[1] != head->status[2])
	{
#ifndef NDEBUG
		n += sprintf(&m[n], "\tstatus words [0] == [1] == 0x%.8x, status [2] == 0x%.8x, choosing [0]/[1]\n", head->status[1], head->status[2]);
#endif	/* NDEBUG */
		/* We know the first two copies are the same */
		head->status[2] = head->status[1];
	}

	/* If there is no next entry */
	if (head->status[0] & F3S_EXT_NO_NEXT)
	{
#ifndef NDEBUG
		if ((head->next.logi_unit != 0xFFFF) || (head->next.index != 0xFFFF))
		{
			n += sprintf(&m[n], "\tF3s_EXT_NO_NEXT is set\n"
											"\t\tchanging head->next.logi_unit from 0x%.4x to 0xFFFF\n"
											"\t\tchanging head->next.index from 0x%.4x to 0xFFFF\n",
											head->next.logi_unit, head->next.index);
		}
#endif	/* NDEBUG */
		head->next.logi_unit = 0xFFFF;
		head->next.index     = 0xFFFF;

	/* If there is a next entry but its invalid */
	} else if ((head->next.logi_unit > lastunit)  ||
	           (head->next.index     > lastindex) ||
		   (head->next.logi_unit == 0 && head->next.index == 0))
	{
#ifndef NDEBUG
		if ((head->next.logi_unit != 0xFFFF) || (head->next.index != 0xFFFF))
		{
			n += sprintf(&m[n], "\tnext.logi_unit > lastunit OR next.index > lastindex\n"
												"\t\tchanging head->next.logi_unit from 0x%.4x to 0xFFFF\n"
												"\t\tchanging head->next.index from 0x%.4x to 0xFFFF\n"
												"\tchanging head->status[0] from 0x%.8x to 0x%.8x\n",
												head->next.logi_unit, head->next.index, head->status[0],
												head->status[0] | F3S_EXT_NO_NEXT);
		}
#endif	/* NDEBUG */
		head->next.logi_unit = 0xFFFF;
		head->next.index     = 0xFFFF;
		head->status[0]     |= F3S_EXT_NO_NEXT;
		head->status[1] = head->status[2] = head->status[0];
	}

	/* Check the supersede stuff */
	if (head->status[0] & F3S_EXT_NO_SUPER)
	{
#ifndef NDEBUG
		if ((head->super.logi_unit != 0xFFFF) || (head->super.index != 0xFFFF))
		{
			n += sprintf(&m[n], "\tF3s_EXT_NO_SUPER is set\n"
											"\t\tchanging head->super.logi_unit from 0x%.4x to 0xFFFF\n"
											"\t\tchanging head->super.index from 0x%.4x to 0xFFFF\n",
											head->super.logi_unit, head->super.index);
		}
#endif	/* #ifndef NDEBUG */
		head->super.logi_unit = 0xFFFF;
		head->super.index     = 0xFFFF;

	/* If there is a next entry but its invalid */
	} else if ((head->super.logi_unit > lastunit)  ||
	           (head->super.index     > lastindex) || 
		   (head->super.logi_unit == 0 && head->super.index == 0))
	{
#ifndef NDEBUG
		if ((head->super.logi_unit != 0xFFFF) || (head->super.index != 0xFFFF))
		{
			n += sprintf(&m[n], "\tsuper.logi_unit > lastunit OR super.index > lastindex\n"
											"\t\tchanging head->super.logi_unit from 0x%.4x to 0xFFFF\n"
											"\t\tchanging head->super.index from 0x%.4x to 0xFFFF\n"
											"\t\tchanging head->status[0] from 0x%.8x to 0x%.8x\n",
											head->super.logi_unit, head->super.index, head->status[0],
											head->status[0] | F3S_EXT_NO_SUPER);
}
#endif	/* NDEBUG */
		head->super.logi_unit = 0xFFFF;
		head->super.index     = 0xFFFF;
		head->status[0]      |= F3S_EXT_NO_SUPER;
		head->status[1] = head->status[2] = head->status[0];
	}

	

#ifndef NDEBUG
	if (f3s.verbose > 1){
		if (n > p)		// something interesting to print ??
			printf("(devf  t%d::%s:%d) %s\n", pthread_self(), __func__, __LINE__, m);
	}
#endif	/* NDEBUG */

	return 0;
/*}}}1*/
}

static int f3s_head_verify (f3s_part_t *part, f3s_head_t *head, uint32_t phys)
{
/*{{{1*/
#if 0
	uint32_t lastindex = part->array->unit_size / sizeof (f3s_head_t);
	uint32_t lastunit  = part->unit_first + part->unit_num - 1;
#endif

	/* Sanity check the status bits */
	if (head->status[0] != head->status[1]) return (EBADFSYS);
	if (head->status[1] != head->status[2]) return (EBADFSYS);

	/* Deal with empty last extent headers */
	if (head->status[0] == (uint_t)-1)	// (uint_t)-1 gives a 0xFF..FF compare the size of status[0]
		return (EBADFSYS);

	/* Too agressive? We have to trust the status bits */
#if 0
	/* If there is no next entry */
	if (head->status[0] & F3S_EXT_NO_NEXT)
	{
		if (head->next.logi_unit != 0xFFFF) return (EBADFSYS);
		if (head->next.index     != 0xFFFF) return (EBADFSYS);

	/* If there is a next entry but its invalid */
	} else if ((head->next.logi_unit > lastunit) ||
	           (head->next.index     > lastindex))
	{
		return (EBADFSYS);
	}

	/* If there is no supersede entry */
	if (head->status[0] & F3S_EXT_NO_SUPER)
	{
		if (head->super.logi_unit != 0xFFFF) return (EBADFSYS);
		if (head->super.index     != 0xFFFF) return (EBADFSYS);

	/* If there is a next entry but its invalid */
	} else if ((head->super.logi_unit > lastunit) ||
	           (head->super.index     > lastindex))
	{
		return (EBADFSYS);
	}
#endif

	return (EOK);
/*}}}1*/
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/struct.c $ $Rev: 580669 $" );
