/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




/*
** File: f3s_table_sort.c
**
** Description:
**
** This file contains the table sort function for the f3s flash file
** system
**
** Ident: $id$
**
*/

/*
** Includes
*/

#include "f3s.h"

/*
** Function: f3s_table_sort
*/

void f3s_table_sort (f3s_part_t *part, int32_t local)
{
	f3s_sort_t *	sort;
	f3s_sort_t *	sort_min;
	f3s_map_t *		map;
	uint32_t		logi;
	int32_t			phys;
	int32_t			erase_min = INT_MAX;
	int32_t			erase_max = 0;

	/* get sort pointer */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	sort = part->table.sort_vect;

	/* initialize sort_min to avoid compiler warning */
	sort_min = sort;

	/* for all units except spares */
	for (logi = F3S_FIRST_LOGI; logi <= F3S_UNIT_ACTIVE (part); logi++)
	{
		/* get physical unit number */
		F3S_LOGI_ENTER (part, logi);
		phys = part->table.logi_vect[logi];
		F3S_UNIT_ENTER (part, phys);

		/* get map pointer */
		map = &part->array->map_vect[phys];

		/* put physical unit number in sort vector */
		sort->phys = phys;
		sort->logi = logi;

		/* initialize unit scores */
		sort->score = part->array->unit_size + part->array->unit_size;

		/* check if free space is not zero */
		if (map->free_size)
		{
			/* add unit size to default score */
			sort->score += part->array->unit_size;

			/* add available space score for unit */
			sort->score += map->free_size;
		}

		/* check if stale space is not zero */
		if (map->stale_size)
		{
			/* subtract unit size to default score */
			sort->score -= part->array->unit_size;

			/* subtract stale space score for unit */
			sort->score -= map->stale_size;
		}

		/* check if erase count is smaller than min */
		if (map->erase_count < erase_min)
		{
			/* remember smallest sort entry and erase count */
			sort_min = sort;
			erase_min = map->erase_count;
		}

		/* check if erase count is bigger than max */
		if (map->erase_count > erase_max)
		{
			/* remember biggest erase count */
			erase_max = map->erase_count;
		}

		/* check if chip is idle */
		f3s_lock_assert(part->array->map_lock_vect[phys].thread == pthread_self());
#if 0
		/*
		 * this flag is protected by the command_mutex in all other cases except here. This is a bug however it
		 * is likely working since the map_lock_vect[phys] lock is held and this is also the case for the point in the
		 * code where the the command_mutex is manipulated. Will put in an assert anyway just to check
		 * On top of that, it is not clear why this flag even exists since the predicate used in condition waits, checks
		 * for erase_unit >= 0
		*/
		if ((part->array->unit_vect[phys]->command_mutex.__count > 0) &&
 			((part->array->unit_vect[phys]->command_mutex.__owner & _NTO_SYNC_OWNER_MASK) != pthread_self()))
		 	f3s_assert(0);
#endif	
		if (!part->array->unit_vect[phys]->use_flag)
		{
			/* add idle chip bonus for unit */
			sort->score += map->free_size;
		}

		/* check if unit is local */
		if (phys == local)
		{
			/* add bonus for local unit */
			sort->score += map->free_size;
		}

		/* go to next sort entry */
		F3S_UNIT_LEAVE (part, phys);
		F3S_LOGI_LEAVE (part, logi);
		sort++;
	}

	/* check if in wear slot and diff. with erase min and max is bigger than unit num */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (((part->table.reclaim_count & F3S_WEAR_SLOT) == F3S_WEAR_SLOT) &&
	    ((erase_max - erase_min) > F3S_UNIT_ACTIVE (part)))
	{
		/* reduce the score of minimum erase unit */
		sort_min->score = 0;
	}

	/* sort units for best allocation and worst reclaim order */
	qsort (part->table.sort_vect, F3S_UNIT_ACTIVE (part), sizeof (f3s_sort_t),
		   (int (*)(const void *, const void *)) f3s_sort_comp);

	/* set table sorted flag */
	part->table.status &= ~F3S_TABLE_NO_SORT;

	/* increment reclaim counter */
/* why is this counter not being incremented after the reclaim actually happens */
	part->table.reclaim_count++;
}

/*
** End
*/

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/table_sort.c $ $Rev: 580669 $" );
