/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"

static void f3s_unit_logi_md5 (f3s_unit_logi_t *unit_logi, uint32_t *md5, int swap);

int32_t f3s_unit_alloc (f3s_part_t *part, int32_t phys, int32_t text_size,
                        int32_t strict)
{
/*{{{1*/
	f3s_sort_t *	sort;
	int32_t			reclaim_phys;

	/* check if units are not sorted */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.status & F3S_TABLE_NO_SORT)
	{
		/* sort units for best allocation and worst reclaim order */
		f3s_table_sort (part, phys);
	}

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* if there is not enough free space but there would be with a reclaim, do it now */
	if(f3s_check_frgd_reclaim(part, text_size)) 
	{
		/* get sort pointer */
		sort = &part->table.sort_vect[F3S_UNIT_ACTIVE (part) - 1];

		/* foreground reclaim a unit */
		F3S_LOGI_ENTER(part, sort->logi);
		F3S_UNIT_ENTER(part, sort->phys);
		reclaim_phys = f3s_frgd_reclaim (part, sort->phys);
		if (reclaim_phys < 0)
		{
			F3S_UNIT_LEAVE(part, sort->phys);
			F3S_LOGI_LEAVE(part, sort->logi);
			return (reclaim_phys);
		}
		F3S_UNIT_LEAVE(part, reclaim_phys);
		F3S_LOGI_LEAVE(part, sort->logi);

		/* reflect physical unit swap in sort vector */
		sort->phys = reclaim_phys;

		/* sort units for best allocation and worst reclaim order */
		f3s_table_sort (part, reclaim_phys);
	}

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* check if there is more stale space than free space trigger */
	if(f3s_check_bkgd_reclaim(part))
	{
		/* signal reclaim background */
		f3s_bkgd_signal (part);
	}

	/* try to do a strict forward fit */
	phys = f3s_alloc_fit (part, text_size, F3S_FIT_FORWARD);
	if (phys >= 0)
	{
		/* check if table needs to be re-sorted */
		if (part->table.sort_vect->phys != phys)
		{
			/* set table not sorted flag */
			part->table.status |= F3S_TABLE_NO_SORT;
		}

	/* else check if strict size alloc is wanted */
	} else if (strict)
	{
		/* try to do a strict backward fit */
		phys = f3s_alloc_fit (part, text_size, F3S_FIT_BACKWARD);
		if (phys >= 0)
		{
			/* set table not sorted flag */
			part->table.status |= F3S_TABLE_NO_SORT;
		}

	/* else, strict alloc is not needed */
	} else
	{
		/* try to do a minimal fit */
		phys = f3s_alloc_fit (part, 1, F3S_FIT_FORWARD);
		if (phys >= 0)
		{
			/* check if table needs to be re-sorted */
			if (part->table.sort_vect->phys != phys)
			{
				/* set table not sorted flag */
				part->table.status |= F3S_TABLE_NO_SORT;
			}

		/* else, forward fit was not successfull */
		} else
		{
			/* try to do a minimal backward fit */
			phys = f3s_alloc_fit (part, 1, F3S_FIT_BACKWARD);
			if (phys >= 0)
			{
				/* set table not sorted flag */
				part->table.status |= F3S_TABLE_NO_SORT;
			}
		}
	}

	return (phys);
/*}}}1*/
}

int f3s_unit_info_read (f3s_part_t *part, int32_t phys, f3s_unit_info_t *unit_info,
                        int fix)
{
/*{{{1*/
	f3s_head_t	head;
	uint32_t	unit_offset;
	int32_t		unit_size;
	int			error;

	/* Read the extent header */
	unit_size = f3s_head_read (part, phys, F3S_UNIT_INFO_INDEX, &unit_offset, &head, fix);
	if (unit_size < 0) return (errno);

	/* Perform some sanity checks */
	if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS) return (ENOENT);
	if (unit_size   != sizeof(*unit_info)) return (ENOENT);
	if (unit_offset != 0) return (ENOENT);

	/* Stop on existance check */
	if (unit_info == NULL) return (EOK);

	/* Read the unit header */
	error = f3s_flash_read (part->array, phys, unit_offset, sizeof(*unit_info),
	                        (uint8_t *)unit_info);
	if (error) return (error);

	/* Determine endian */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
#if defined(__BIGENDIAN__)
	if (unit_info->endian == 'B')
	{
		part->table.swap_flag = 0;

	} else if (unit_info->endian == 'L')
	{
		part->table.swap_flag = 1;
		f3s_unit_info_swap (unit_info);

	} else
	{
		return (ENOENT);
	}
#else
	if (unit_info->endian == 'B')
	{
		part->table.swap_flag = 1;
		f3s_unit_info_swap (unit_info);

	} else if (unit_info->endian == 'L')
	{
		part->table.swap_flag = 0;

	} else
	{
		return (ENOENT);
	}
#endif

	/* More sanity checks */
	if (unit_info->struct_size != sizeof(*unit_info)) return (ENOENT);
	if (unit_info->unit_pow2   != part->array->unit_pow2) return (ENOENT);

	return (EOK);
/*}}}1*/
}

int f3s_unit_logi_read (f3s_part_t *part, int32_t phys, f3s_unit_logi_t *unit_logi)
{
/*{{{1*/
	uint32_t	md5[4];
	f3s_head_t	head;
	uint32_t	unit_offset;
	int32_t		unit_size;
	uint32_t	i;
	int			error;

	/* Read the extent header */
	unit_size = f3s_head_read (part, phys, F3S_UNIT_LOGI_INDEX, &unit_offset, &head, 1);
	if (unit_size < 0) return (errno);

	/* Perform some sanity checks */
	if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS) {
		/* Possibly not allocated */
		for (i = 0; i < sizeof(f3s_head_t); i++) {
			if (((uint8_t *)&head)[i] != 0xFF) return (EBADFSYS);
		}
		return (ENODATA);
	}
	if (unit_size   != sizeof(*unit_logi)) return (EBADFSYS);
	if (unit_offset != sizeof(f3s_unit_info_t)) return (EBADFSYS);

	/* Stop on existance check */
	if (unit_logi == NULL) return (EOK);

	/* Read the unit header */
	error = f3s_flash_read (part->array, phys, unit_offset, sizeof(*unit_logi),
	                        (uint8_t *)unit_logi);
	if (error) return (error);

	/* Compute the checksum in on-flash endian */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	f3s_unit_logi_md5 (unit_logi, md5, part->table.swap_flag);

	/* Endian swap */
	if (part->table.swap_flag) f3s_unit_logi_swap (unit_logi);

	/* More sanity checks */
	if (unit_logi->struct_size != sizeof(*unit_logi)) return (EBADFSYS);

	/* Verify MD5 checksum */
	if (memcmp (unit_logi->md5, md5, sizeof(uint32_t) * 4)) return (EBADFSYS);

	return (EOK);
/*}}}1*/
}

int f3s_unit_write (f3s_part_t *part, int32_t phys,
                    uint32_t erase_count, f3s_extptr_t boot)
{
/*{{{1*/
	f3s_unit_info_t	unit_info;
	f3s_map_t *		map;
	int				err_size;

	/* write parameters into unit info structure */
	unit_info.struct_size = sizeof(unit_info);

	f3s_lock_assert(part->table_lock.thread == pthread_self());
#if defined(__BIGENDIAN__)
	if (part->table.swap_flag) unit_info.endian = 'L';
	else                       unit_info.endian = 'B';
#else
	if (part->table.swap_flag) unit_info.endian = 'B';
	else                       unit_info.endian = 'L';
#endif

	unit_info.pad         = 0xFF;
	unit_info.unit_pow2   = part->array->unit_pow2;
	unit_info.reserve     = F3S_WORD_ERASE;
	unit_info.erase_count = erase_count;
	unit_info.boot        = boot;

	/* if unit structure needs swapping */
	if (part->table.swap_flag) f3s_unit_info_swap (&unit_info);

	/* write extent with unit info */
	err_size = f3s_ext_write (part, &phys, F3S_UNIT_INFO_INDEX, 0,
	                          sizeof(unit_info), ((uint8_t *)&unit_info),
	                          ~F3S_EXT_TYPE | F3S_EXT_SYS, F3S_SYS_POW2, 1, 0);
	if (err_size < 0) return (errno);

	/* Initialize the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];
	memset (map, 0, sizeof (*map));
	map->text_size    = F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t), F3S_SYS_ALIGN);
	map->free_size    = part->array->unit_size;
	map->free_size   -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_info_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);
	map->erase_count  = erase_count;
	map->head_last    = F3S_UNIT_INFO_INDEX;
	map->free_first   = F3S_UNIT_INFO_INDEX + 1;

	return (EOK);
/*}}}1*/
}

int f3s_unit_logi1 (f3s_part_t *part, int32_t *phys, uint16_t logi, uint8_t age,
                    int recover)
{
/*{{{1*/
	f3s_unit_logi_t	unit_logi;
	f3s_map_t *		map;
	int				err_size;

	/* Write parameters into unit info structure */
	memset(&unit_logi, 0xFF, sizeof(f3s_unit_logi_t));
	unit_logi.struct_size = sizeof(unit_logi);
	unit_logi.logi        = logi;
	unit_logi.age         = age;

	/* If unit structure needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_unit_logi_swap (&unit_logi);

	/* Force clear the last header bit on the info structure */
	err_size = f3s_head_status (part, phys, F3S_UNIT_INFO_INDEX,
	                            ~F3S_EXT_LAST, recover);
	if (err_size < 0) return (errno);

	/* write extent with unit info */
	err_size = f3s_ext_write (part, phys, F3S_UNIT_LOGI_INDEX, sizeof(f3s_unit_info_t),
	                          sizeof(unit_logi), ((uint8_t *)&unit_logi),
	                          ~F3S_EXT_TYPE | F3S_EXT_SYS, F3S_SYS_POW2, 1, recover);
	if (err_size < 0) return (errno);

	/* Update the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[*phys].thread == pthread_self()));
	map = &part->array->map_vect[*phys];
	map->head_last  = F3S_UNIT_LOGI_INDEX;
	map->free_first = F3S_UNIT_LOGI_INDEX + 1;
	map->text_size += F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t), F3S_SYS_ALIGN);;
	map->free_size -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);;

	return (EOK);
/*}}}1*/
}

int f3s_unit_logi2 (f3s_part_t *part, int32_t *phys, uint16_t logi, uint8_t age,
                    int recover)
{
/*{{{1*/
	f3s_unit_logi_t	unit_logi;
	int				error;

	/* Write parameters into unit info structure */
	memset(&unit_logi, 0xFF, sizeof(f3s_unit_logi_t));
	unit_logi.struct_size = sizeof(unit_logi);
	unit_logi.logi        = logi;
	unit_logi.age         = age;

	/* If unit structure needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_unit_logi_swap (&unit_logi);

	/* Compute the checksum in on-flash endian */
	f3s_unit_logi_md5 (&unit_logi, unit_logi.md5, part->table.swap_flag);
	if (part->table.swap_flag)
	{
		ENDIAN_SWAP32 (&unit_logi.md5[0]);
		ENDIAN_SWAP32 (&unit_logi.md5[1]);
		ENDIAN_SWAP32 (&unit_logi.md5[2]);
		ENDIAN_SWAP32 (&unit_logi.md5[3]);
	}

	/* Make the logical unit header valid */
	error = f3s_flash_write (part->array, *phys,
	                         sizeof(f3s_unit_info_t) + offsetof(f3s_unit_logi_t, md5),
	                         sizeof(unit_logi.md5), (uint8_t *)unit_logi.md5, F3S_VERIFY_WRITE);
	if (error) return (error);

	return (EOK);
/*}}}1*/
}

/* Nobody but the format code calls this anyore, maybe we should get rid of it */
int f3s_unit_logi (f3s_part_t *part, int32_t *phys, uint16_t logi, uint8_t age,
                   int recover)
{
/*{{{1*/
	f3s_unit_logi_t	unit_logi;
	f3s_map_t *		map;
	int				err_size;

	/* Write parameters into unit info structure */
	unit_logi.struct_size = sizeof(unit_logi);
	unit_logi.logi        = logi;
	unit_logi.age         = age;

	/* If unit structure needs swapping */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (part->table.swap_flag) f3s_unit_logi_swap (&unit_logi);

	/* Compute the checksum in on-flash endian */
	f3s_unit_logi_md5 (&unit_logi, unit_logi.md5, part->table.swap_flag);
	if (part->table.swap_flag)
	{
		ENDIAN_SWAP32 (&unit_logi.md5[0]);
		ENDIAN_SWAP32 (&unit_logi.md5[1]);
		ENDIAN_SWAP32 (&unit_logi.md5[2]);
		ENDIAN_SWAP32 (&unit_logi.md5[3]);
	}

	/* Force clear the last header bit on the info structure */
	err_size = f3s_head_status (part, phys, F3S_UNIT_INFO_INDEX,
	                            ~F3S_EXT_LAST, recover);
	if (err_size < 0) return (errno);

	/* write extent with unit info */
	err_size = f3s_ext_write (part, phys, F3S_UNIT_LOGI_INDEX, sizeof(f3s_unit_info_t),
	                          sizeof(unit_logi), ((uint8_t *)&unit_logi),
	                          ~F3S_EXT_TYPE | F3S_EXT_SYS, F3S_SYS_POW2, 1, recover);
	if (err_size < 0) return (errno);

	/* Update the map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[*phys].thread == pthread_self()));
	map = &part->array->map_vect[*phys];
	map->head_last  = F3S_UNIT_LOGI_INDEX;
	map->free_first = F3S_UNIT_LOGI_INDEX + 1;
	map->text_size += F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t), F3S_SYS_ALIGN);;
	map->free_size -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);;

	return (EOK);
/*}}}1*/
}

int f3s_unit_inval (f3s_part_t *part, int32_t phys)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_unit_logi_t	unit_logi;
	uint32_t		unit_offset;
	int32_t			unit_size;
	int				error;

	/* Read the extent header */
	unit_size = f3s_head_read (part, phys, F3S_UNIT_LOGI_INDEX, &unit_offset, &head, 1);
	if (unit_size < 0) return (-1);

	/* Perform some sanity checks */
	if ((head.status[0] & F3S_EXT_TYPE) != F3S_EXT_SYS) return (ENOENT);
	if (unit_size != sizeof(unit_logi)) return (ENOENT);

	/* Zero the unit header */
	memset (&unit_logi, 0x00, sizeof(unit_logi));
	error = f3s_flash_write (part->array, phys, unit_offset, sizeof(unit_logi),
	                         (uint8_t *)&unit_logi, F3S_VERIFY_WRITE);
	if (error) return (error);

	return (EOK);
/*}}}1*/
}

int f3s_unit_retire (f3s_part_t *part, int32_t phys)
{
/*{{{1*/
	/* Ensure retired units are never mistaken as good */
	printf("(devf  t%d) phys %d is retired. \n", pthread_self(), phys);
	return (f3s_unit_inval (part, phys));
/*}}}1*/
}

int f3s_unit_reclaim (f3s_part_t *part, int32_t reclaim_phys, int32_t spare_phys,
                      uint32_t *obsol_num)
{
/*{{{1*/
	f3s_map_t *		old_map;
	f3s_map_t *		new_map;
#if 0
	int32_t			align_size;
#endif
	uint32_t		logi;
	uint32_t		age;
	uint32_t		copy_num;
#if 0
	int				err_size;
#endif
	int				error;

	//printf("f3s_unit_reclaim: [%d -> %d]\n", reclaim_phys, spare_phys);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* Create a sorted copy vect for reclaim */
	f3s_reclaim_sort (part, &copy_num, reclaim_phys);

	/* Get logical unit number */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) && (part->array->map_lock_vect[reclaim_phys].thread == pthread_self()));
	logi = part->array->phys_vect[reclaim_phys];
	f3s_lock_assert(part->table.logi_lock_vect[logi].thread == pthread_self());
	age  = part->array->age_vect[reclaim_phys];

	/* Get map pointer for physical unit */
	old_map = &part->array->map_vect[reclaim_phys];
	f3s_lock_assert((part->array->map_lock_vect[spare_phys].thread == pthread_self()));
	new_map = &part->array->map_vect[spare_phys];

	/* Log unit reclaim event */
	f3s_log_reclaim ("", part->array->index, part->index, logi,
	                 reclaim_phys, spare_phys, old_map->stale_size);
	f3s_assert (old_map->stale_size < part->array->unit_size);

	/* Write the initial logical unit header (without MD5) */
	error = f3s_unit_logi1 (part, &spare_phys, logi, age + 1, 0);
	if (error) return (error);

	/* Update the map counters */
//	new_map->head_last  = F3S_UNIT_LOGI_INDEX;
//	new_map->free_first = F3S_UNIT_LOGI_INDEX + 1;
//	new_map->text_size += F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t), F3S_SYS_ALIGN);;
//	new_map->free_size -= F3S_TEXT_ALIGN(part, sizeof(f3s_unit_logi_t) + sizeof(f3s_head_t), F3S_SYS_ALIGN);;

	/* We better have enough space for the reclaim */
	f3s_assert (new_map->free_size >= old_map->text_size);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);

	/* No need to clear the "last" flag, it will be done as necessary */
#if 0
	/* Force clear the last header bit on the info structure */
	err_size = f3s_head_status (part, &spare_phys, F3S_UNIT_INFO_INDEX,
	                            ~F3S_EXT_LAST, 0);
	if (err_size < 0) return (errno);
#endif

	/* check if unit has boot */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	if (logi == part->table.boot_extptr.logi_unit)
	{
		/* write boot extent on spare unit */
		error = f3s_boot_write (part, spare_phys,
		                        part->table.root_extptr, part->unit_num,
		                        part->table.unit_spare,
		                        part->table.align_pow2,
		                        spare_phys - part->unit_first);
		if (error) return (error);
	}

	/* Copy extents from physical unit to spare unit */
	error = f3s_reclaim_copy (part, copy_num, obsol_num, 
	                          old_map->stale_size + old_map->free_size,
	                          reclaim_phys, spare_phys, old_map->resv_num);
	if (error) return (error);

	/* Make the logical unit header valid */
	error = f3s_unit_logi2 (part, &spare_phys, logi, age + 1, 0);
	if (error) return (error);

	/*
	 * Prevent roll-back to a block that may be partially erased.
	 */
	/* Schedule the old unit for erase */
	f3s_unit_inval (part, reclaim_phys);

	//if (new_map->free_size < (old_map->stale_size + old_map->free_size))
	//	printf ("Reclaim [%d->%d] exceeded stale by %d bytes\n",
	//			reclaim_phys, spare_phys, (old_map->stale_size + old_map->free_size) - new_map->free_size);

	/* Transfer any reserved space */
	new_map->free_size -= old_map->resv_size;
	new_map->resv_size  = old_map->resv_size;

	/* Swap in the new unit */
	f3s_lock_assert((part->table_lock.thread == pthread_self()));
	part->table.logi_vect[logi]          = spare_phys;
	f3s_lock_assert((part->array->map_lock_vect[spare_phys].thread == pthread_self()) && (part->array->map_lock_vect[reclaim_phys].thread == pthread_self()));part->array->phys_vect[reclaim_phys] = 0;
	part->array->phys_vect[spare_phys]   = logi;
	part->array->age_vect[reclaim_phys]  = 0xFF;
	part->array->age_vect[spare_phys]    = age + 1;

	F3S_ALLOC_ENTER(part);
	part->table.free_total  -= old_map->free_size;
	part->table.stale_total -= old_map->stale_size;
	part->table.pad_total   -= old_map->pad_size;
	part->table.head_num    -= old_map->head_last + 1;

	part->table.free_total += new_map->free_size;
	part->table.pad_total  += new_map->pad_size;
	part->table.head_num   += new_map->head_last + 1;
	F3S_ALLOC_LEAVE(part);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, 0, 0);
	return (EOK);
/*}}}1*/
}

int f3s_unit_refresh (f3s_part_t *part, int32_t phys)
{
/*{{{1*/
	f3s_map_t *	map;
	int			error;
	int			retry_count = 20;

	/* get map pointer for physical unit */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) || (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];

	/* Keep erasing and formating the unit until success or erase fails */
	while (1)
	{
		/* count the erase */
/* we are counting the erase before its is actually erased? This then is actually erase
 * attempts and could lead to inaccurate decisions re wear leveling
*/
		map->erase_count++;

		/* erase physical unit */
		error = f3s_flash_erase (part->array, phys);
		if (error)
		{
			if ((error == EBUSY) || (error == EIO)){
				if (retry_count--){
					continue;
				}else{
					goto retire_shortcut;
				}
			}else	// some other error, so retire the unit
				goto retire_shortcut;
		}

		/* write physical unit with incremented erase count */
		error = f3s_unit_write (part, phys,
		                        map->erase_count, part->table.boot_extptr);
		if (error) continue;

		/* refresh was successfull */
		return (EOK);
	}

/* this is for retire write sizes */
retire_shortcut:
	/* status physical unit as retire */
	f3s_unit_retire (part, phys);

	/* log that retire event */
	f3s_log_retire (part->array->index, part->index, phys);
	return (error);
/*}}}1*/
}

void f3s_unit_stale (f3s_part_t *part, int32_t phys, int32_t text_size)
{
/*{{{1*/
	f3s_map_t *map;
	int32_t align_size;

	f3s_assert (text_size >= 0);
	f3s_assert (text_size != 0xffff);

	/* DEBUG: Verify the partition table totals match */
	f3s_verify_counters (part, phys, text_size);

	/* get aligned size */
	f3s_lock_assert(part->table_lock.thread == pthread_self());
	align_size = F3S_TEXT_ALIGN(part, text_size, part->table.align_size);

	/* Update global counters */
	F3S_ALLOC_ENTER(part);
	part->table.stale_total += align_size;
	part->table.pad_total   -= align_size - text_size;
	F3S_ALLOC_LEAVE(part);

	/* Update map counters */
	f3s_lock_assert((part->table_lock.thread == pthread_self()) || (part->array->map_lock_vect[phys].thread == pthread_self()));
	map = &part->array->map_vect[phys];
	map->stale_size += align_size;
	map->pad_size   -= align_size - text_size;

	/* DEBUG: Verify the partition table totals match */
	f3s_assert (map->stale_size < map->text_size);
	f3s_verify_counters (part, 0, 0);

	f3s_assert (map->pad_size < part->array->unit_size);
/*}}}1*/
}

int32_t f3s_alloc_fit (f3s_part_t *part, int32_t text_size, f3s_fit_direction_t direction)
{
/*{{{1*/
	f3s_sort_t *sort;
	f3s_map_t *	map;
	int32_t		enough_size;
	int32_t		reclaim_phys;
	int32_t		alloc_phys;
	int32_t		sort_phys;
	int32_t		sort_logi;

	/* check if search is backward */
	if (direction == F3S_FIT_BACKWARD)
	{
		/* get sort pointer */
		f3s_lock_assert(part->table_lock.thread == pthread_self());
		sort = &part->table.sort_vect[F3S_UNIT_ACTIVE(part) - 1];
	} else
	{
		/* get sort pointer */
		sort = part->table.sort_vect;
	}

	/* for all units in sorted list */
	for (alloc_phys = 0; alloc_phys < F3S_UNIT_ACTIVE(part); alloc_phys++)
	{
		/* get sort logical unit */
		sort_logi = sort->logi;

		/* get sort physical unit */
		sort_phys = sort->phys;
		
		F3S_LOGI_ENTER(part, sort_logi);
		F3S_UNIT_ENTER(part, sort_phys);

		/* get unit map */
		f3s_lock_assert((part->table_lock.thread == pthread_self()) || (part->array->map_lock_vect[sort_phys].thread == pthread_self()));
		map = &part->array->map_vect[sort_phys];

		/* set requested size to enough room */
		enough_size = text_size;

		/* check if there are no free headers */
		if (!map->free_num)
		{
			/* add size of a header to enough room */
			enough_size += sizeof(f3s_head_t);
		}

		/* check if search is backward (with reclaim) */
		if (direction == F3S_FIT_BACKWARD)
		{
			/* check if unit would have enough room with a reclaim */
			if ((map->free_size + map->stale_size) >= enough_size)
			{
				/* foreground reclaim a unit */
				reclaim_phys = f3s_frgd_reclaim (part, sort_phys);
				if (reclaim_phys < 0)
				{
					F3S_UNIT_LEAVE(part, sort_phys);
					F3S_LOGI_LEAVE(part, sort_logi);
					return (reclaim_phys);
				}

				/* reflect physical unit swap in sort vector */
				sort->phys = reclaim_phys;

				/* set sort phys for proper leave */
				sort_phys = reclaim_phys;

				/* get unit map */
				f3s_lock_assert((part->table_lock.thread == pthread_self()) || (part->array->map_lock_vect[sort_phys].thread == pthread_self()));
				map = &part->array->map_vect[sort_phys];

				/* set size of enough room without header size */
				enough_size = text_size;

				/* check if there are no free headers */
				if (!map->free_num)
				{
					/* set size of a header to enough room with header size */
					enough_size += sizeof(f3s_head_t);
				}

				/* check if unit has enough room */
				if (map->free_size >= enough_size)
					return (sort_phys);
			}

			/* go to next sort list entry */
			sort--;

		/* else, search is forward (no reclaim) */
		}
		else {

			/* check if unit has enough room */
			if (map->free_size >= enough_size)
				return (sort_phys);

			/* go to next sort list entry */
			sort++;
		}
		F3S_UNIT_LEAVE(part, sort_phys);
		F3S_LOGI_LEAVE(part, sort_logi);
	}

	/* set errno to no space left error */
	errno = ENOSPC;
	return (-1);
/*}}}1*/
}

/*
 * f3s_unit_logi_md5
 *
 * This function computes the MD5 checksum for the f3s_unit_logi_t data
 * structure. It is heavily customized for this computation. In particular,
 * the required padding and the length of the message are all hardwired into a
 * single round of the algorithm.
 *
 * NOTE: This function must be updated if the structure of f3s_unit_logi_t
 *       ever changes!!!!!!
 *
 * NOTE: Code adopted (but not copied) from: 
 *        "Applied Cryptography 2nd Edition" by Bruce Schneier
 */
/* XXX - This function may need some endianization work */
static void f3s_unit_logi_md5 (f3s_unit_logi_t *unit_logi, uint32_t *md5, int swap)
{
/*{{{1*/
#define ROTL(__a, __r) (((__a) << (__r)) | ((__a) >> (32 - (__r))))

#define F(X, Y, Z) ((X & Y) | ((~X) & Z))
#define G(X, Y, Z) ((X ^ Z) | (Y & (~Z)))
#define H(X, Y, Z) (X ^ Y ^ Z)
#define I(X, Y, Z) (Y ^ (X | (~Z)))

#define FF(a, b, c, d, M, s, t) a = b + ROTL((a + F(b, c, d) + M + t), s)
#define GG(a, b, c, d, M, s, t) a = b + ROTL((a + G(b, c, d) + M + t), s)
#define HH(a, b, c, d, M, s, t) a = b + ROTL((a + H(b, c, d) + M + t), s)
#define II(a, b, c, d, M, s, t) a = b + ROTL((a + I(b, c, d) + M + t), s)

	uint32_t	logi[2];
	uint8_t		bpad[4] = {0x80,                    0, 0, 0};
	uint8_t		blen[4] = {sizeof(f3s_unit_logi_t), 0, 0, 0};
	uint32_t *	wpad    = (uint32_t *)bpad;
	uint32_t *	wlen    = (uint32_t *)blen;
	uint32_t	a, b, c, d;

	/* Copy the logical unit header */
	memcpy (logi, unit_logi, sizeof(uint32_t) * 2);

	/* Perform optional endian swapping of input */
	if (swap)
	{
		ENDIAN_SWAP32 (&logi[0]);
		ENDIAN_SWAP32 (&logi[1]);
		ENDIAN_SWAP32 (&bpad[0]);
		ENDIAN_SWAP32 (&blen[0]);
	}

	/* Initialize accumulators */
	a = 0x01234567;
	b = 0x89ABCDEF;
	c = 0xFEDCBA89;
	d = 0x76543210;

	/* Round 1 */
	FF(a, b, c, d, logi[0],  7, 0xD76AA478); /* Buf[ 0] */
	FF(d, a, b, c, logi[1], 12, 0xE8C7B756); /* Buf[ 1] */
	FF(c, d, a, b, wpad[0], 17, 0x242070DB); /* Buf[ 2] */
	FF(b, c, d, a, 0,       22, 0xC1BDCEEE); /* Buf[ 3] */
	FF(a, b, c, d, 0,        7, 0xF57C0FAF); /* Buf[ 4] */
	FF(d, a, b, c, 0,       12, 0x4787C62A); /* Buf[ 5] */
	FF(c, d, a, b, 0,       17, 0xA8304613); /* Buf[ 6] */
	FF(b, c, d, a, 0,       22, 0xFD469501); /* Buf[ 7] */
	FF(a, b, c, d, 0,        7, 0x698098D8); /* Buf[ 8] */
	FF(d, a, b, c, 0,       12, 0x8B44F7AF); /* Buf[ 9] */
	FF(c, d, a, b, 0,       17, 0xFFFF5BB1); /* Buf[10] */
	FF(b, c, d, a, 0,       22, 0x895CD7BE); /* Buf[11] */
	FF(a, b, c, d, 0,        7, 0x6B901122); /* Buf[12] */
	FF(d, a, b, c, 0,       12, 0xFD987193); /* Buf[13] */
	FF(c, d, a, b, wlen[0], 17, 0xA679438E); /* Buf[14] */
	FF(b, c, d, a, 0,       22, 0x49B40821); /* Buf[15] */

	/* Round 2 */
	GG(a, b, c, d, logi[1],  5, 0xF61E2562); /* Buf[ 1] */
	GG(d, a, b, c, 0,        9, 0xC040B340); /* Buf[ 6] */
	GG(c, d, a, b, 0,       14, 0x265E5A51); /* Buf[11] */
	GG(b, c, d, a, logi[0], 20, 0xE9B6C7AA); /* Buf[ 0] */
	GG(a, b, c, d, 0,        5, 0xD62F105D); /* Buf[ 5] */
	GG(d, a, b, c, 0,        9, 0x02441453); /* Buf[10] */
	GG(c, d, a, b, 0,       14, 0xD8A1E681); /* Buf[15] */
	GG(b, c, d, a, 0,       20, 0xE7D3FBC8); /* Buf[ 4] */
	GG(a, b, c, d, 0,        5, 0x21E1CDE6); /* Buf[ 9] */
	GG(d, a, b, c, wlen[0],  9, 0xC33707D6); /* Buf[14] */
	GG(c, d, a, b, 0,       14, 0xF4D50D87); /* Buf[ 3] */
	GG(b, c, d, a, 0,       20, 0x455A14ED); /* Buf[ 8] */
	GG(a, b, c, d, 0,        5, 0xA9E3E905); /* Buf[13] */
	GG(d, a, b, c, wpad[0],  9, 0xFCEFA3F8); /* Buf[ 2] */
	GG(c, d, a, b, 0,       14, 0x676F02D9); /* Buf[ 7] */
	GG(b, c, d, a, 0,       20, 0x8D2A4C8A); /* Buf[12] */
                                                    
	/* Round 3 */                                   
	HH(a, b, c, d, 0,        4, 0xFFFA3942); /* Buf[ 5] */
	HH(d, a, b, c, 0,       11, 0x8771F681); /* Buf[ 8] */
	HH(c, d, a, b, 0,       16, 0x6D9D6122); /* Buf[11] */
	HH(b, c, d, a, wlen[0], 23, 0xFDE5380C); /* Buf[14] */
	HH(a, b, c, d, logi[1],  4, 0xA4BEEA44); /* Buf[ 1] */
	HH(d, a, b, c, 0,       11, 0x4BDECFA9); /* Buf[ 4] */
	HH(c, d, a, b, 0,       16, 0xF6BB4B60); /* Buf[ 7] */
	HH(b, c, d, a, 0,       23, 0xBEBFBC70); /* Buf[10] */
	HH(a, b, c, d, 0,        4, 0x289B7EC6); /* Buf[13] */
	HH(d, a, b, c, logi[0], 11, 0xEAA127FA); /* Buf[ 0] */
	HH(c, d, a, b, 0,       16, 0xD4EF3085); /* Buf[ 3] */
	HH(b, c, d, a, 0,       23, 0x04881D05); /* Buf[ 6] */
	HH(a, b, c, d, 0,        4, 0xD9D4D039); /* Buf[ 9] */
	HH(d, a, b, c, 0,       11, 0xE6DB99E5); /* Buf[12] */
	HH(c, d, a, b, 0,       16, 0x1FA27CF8); /* Buf[15] */
	HH(b, c, d, a, wpad[0], 23, 0xC4AC5665); /* Buf[ 2] */
                                                    
	/* Round 4 */                                   
	II(a, b, c, d, logi[0],  6, 0xF4292244); /* Buf[ 0] */
	II(d, a, b, c, 0,       10, 0x432AFF97); /* Buf[ 7] */
	II(c, d, a, b, wlen[0], 15, 0xAB9423A7); /* Buf[14] */
	II(b, c, d, a, 0,       21, 0xFC93A039); /* Buf[ 5] */
	II(a, b, c, d, 0,        6, 0x655B59C3); /* Buf[12] */
	II(d, a, b, c, 0,       10, 0x8F0CCC92); /* Buf[ 3] */
	II(c, d, a, b, 0,       15, 0xFFEFF47D); /* Buf[10] */
	II(b, c, d, a, logi[1], 21, 0x85845DD1); /* Buf[ 1] */
	II(a, b, c, d, 0,        6, 0x6FA87E4F); /* Buf[ 8] */
	II(d, a, b, c, 0,       10, 0xFE2CE6E0); /* Buf[15] */
	II(c, d, a, b, 0,       15, 0xA3014314); /* Buf[ 6] */
	II(b, c, d, a, 0,       21, 0x4E0811A1); /* Buf[13] */
	II(a, b, c, d, 0,        6, 0xF7537E82); /* Buf[ 4] */
	II(d, a, b, c, 0,       10, 0xBD3AF235); /* Buf[11] */
	II(c, d, a, b, wpad[0], 15, 0x2AD7D2BB); /* Buf[ 2] */
	II(b, c, d, a, 0,       21, 0xEB86D391); /* Buf[ 9] */

	/* Store results */
	md5[0] = 0x01234567 + a;
	md5[1] = 0x89ABCDEF + b;
	md5[2] = 0xFEDCBA89 + c;
	md5[3] = 0x76543210 + d;
/*}}}1*/
}

#ifndef NDEBUG
uint16_t _f3s_unit_active(f3s_part_t  *PART_PTR)
{
	return(PART_PTR->unit_num - PART_PTR->table.unit_spare);
}
#endif	/* NDEBUG */

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/unit.c $ $Rev: 580669 $" );
