/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 * 
 * You must obtain a written license from and pay applicable license fees to QNX 
 * Software Systems before you may reproduce, modify or distribute this software, 
 * or any work that includes all or part of this software.   Free development 
 * licenses are available for evaluation and non-commercial purposes.  For more 
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *  
 * This file may contain contributions from others.  Please review this entire 
 * file for other proprietary rights or license notices, as well as the QNX 
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/ 
 * for other information.
 * $
 */




#include "f3s.h"
#include <ctype.h>

unsigned verbose = 0;

/* Avoid weak-symbol magic for now */
#if 0
const char * __attribute__ ((weak)) MTD_BRANCH = "?";
const char * __attribute__ ((weak)) MTD_DATE   = "?";
const char * __attribute__ ((weak)) MTD_TIME   = "?";
#endif

#define XSTR(S) STR(S)
#define STR(S) #S
#ifndef FFS_BUILD
#define FFS_BUILD ?
#endif

void f3s_list_add (f3s_list_t *list, f3s_list_t *new)
{
/*{{{1*/
	new->prev = list->prev;
	new->next = list;
	list->prev->next = new;
	list->prev = new;
/*}}}1*/
}

void f3s_list_rem (f3s_list_t *old)
{
/*{{{1*/
	old->prev->next = old->next;
	old->next->prev = old->prev;
/*}}}1*/
}

f3s_list_t * f3s_list_find (f3s_list_t *list, int32_t key_size, uint8_t *key)
{
/*{{{1*/
	f3s_list_t *entry;

	/* Search for the matching key */
	entry = list->next;
	while (entry != list)
	{
		/* If we have a match */
		if (!memcmp (&entry[1], key, key_size)) return (entry);

		entry = entry->next;
	}

	return (NULL);
/*}}}1*/
}

int32_t f3s_list_drain (f3s_list_t *list, int32_t entry_size)
{
/*{{{1*/
	f3s_list_t *entry;
	int32_t		entry_num = 0;

	/* while there is a free entry */
	while (list->next != (f3s_list_t *)&list->next)
	{
		/* Count the entry */
		entry_num++;

		entry = list->next;
		f3s_list_rem (entry);

		/* XXX - We must trust that entry_size is correct! */
		f3s_free (entry, sizeof(*entry) + entry_size);
	}

	/* return number of entries */
	return (entry_num);
/*}}}1*/
}

void f3s_lock_init(f3s_lock_t *lock)
{
	pthread_mutexattr_t mattr;

	pthread_mutexattr_init(&mattr);
	pthread_mutexattr_setrecursive(&mattr, PTHREAD_RECURSIVE_ENABLE);
	pthread_mutex_init(&lock->mutex, &mattr);
}

void f3s_lock_free(f3s_lock_t *lock)
{
	pthread_mutex_destroy(&lock->mutex);
	f3s_assert(lock->count == 0);
}

void f3s_lock_enter (f3s_lock_t *lock)
{
/*{{{1*/
	pthread_t thread = pthread_self ();

	/* Acquire lock */
	if (pthread_mutex_lock(&lock->mutex) == EOK) {
		lock->thread = thread;
		lock->count++;
	} else {
		printf("(devf  t%d::%s) pthread_mutex lock failed! \n", pthread_self(), __func__);
	}
/*}}}1*/
}

void f3s_lock_leave (f3s_lock_t *lock)
{
/*{{{1*/
#ifndef NDEBUG
	/* If lock is not owned */
	if (lock->thread != pthread_self ()) DebugBreak();

	/* If lock count is 0 */
	if (!lock->count) DebugBreak();

#endif // NDEBUG

	lock->count--;
	if (lock->count == 0)
		lock->thread = 0xfeeb;

	pthread_mutex_unlock(&lock->mutex);

/*}}}1*/
}

void f3s_option (int argc, char **argv, f3s_flash_t *flash_vect)
{
/*{{{1*/
	int percent = 0;
	int32_t result;
	uint32_t option;
	extern char *optarg;

	/* initialize default options */
	f3s.mount_options = (f3s_mntopts_t_INIT | f3s_mntopts_t_AUTOMNT | f3s_mntopts_t_MOUNT_RW);
	f3s.buffer_size = F3S_BUFFER_SIZE;
	f3s.thread_num = F3S_THREAD_NUM;
	f3s.update_level = F3S_UPDATE_NEVER;
	f3s.reclaim.etrigger = f3s.reclaim.trigger = 100;
	f3s.reclaim.ethreshold = f3s.reclaim.threshold = 16;
	f3s.retry = 0;
	f3s.enum_only = -1;
	f3s.force_mount = 0;
	f3s.log_method = 0;  /* log to stdout, which is the legacy behaviour */
	f3s.xcache_file_max = XCACHE_FILE_MAX;
	f3s.xcache_blksz = XCACHE_BLOCK_SIZE;
	f3s.mountexact_num = 0;

	/* while there is an option to process */
	while (1)
	{
		/* get current option from arguments */
#if defined (ENABLE_MOUNTBEHIND)
		option = getopt (argc, argv, ":alrvxb:f:i:L:m:p:s:t:u:w:EVRAe:d:W:o:k:");
#else
		option = getopt (argc, argv, ":alrvxb:f:i:L:m:p:s:t:u:w:EVRe:d:W:o:k:");
#endif

		/* check if it was last option */
		if (option == -1) break;

		/* check what option needs processing */
		switch (option)
		{
#ifdef ENABLE_MOUNTBEHIND
			case 'A':
				f3s.mount_behind = 1;	/* set mount after */
				break;
#endif
			case 'v':			/* verbose */
				f3s.verbose++;
				verbose++;
				break;

			case 'a':			/* clear automount flag */
				f3s.mount_options &= ~f3s_mntopts_t_AUTOMNT;
				break;

			case 'l':			/* list all flash dbases */
				f3s_flash_list (flash_vect);
				break;

			case 'L':			/* set retry limit */
				f3s.retry = strtol (optarg, NULL, 0);
				break;

			case 'r':			/* set recover flag */
				f3s.recover = 1;
				break;

			case 'i':			/* set array index */
				if (*optarg != ',')
				{
					result = strtol (optarg, NULL, 0);

					/* validate the array index */
					if (result >= F3S_INDEX_MAX || result < 0) f3s_usage (argc, argv, "array index");
					else                         f3s.array_index = result;

					/* skip to next sub-option */
					do {optarg++;} while (*optarg != ',' && *optarg);
				}

				if (*optarg == ',')
				{
					optarg++;
					result = strtol (optarg, NULL, 0);

					/* validate the part index */
					if (result >= F3S_INDEX_MAX || result < 0) f3s_usage (argc, argv, "part index");
					else                         f3s.part_index = result;
				}
				break;

			case 'b':			/* set background reclaim flag and priority */
				result = strtol (optarg, NULL, 0);

				/* validate the background priority */
				if (result > sched_get_priority_max (SCHED_RR) ||
					result < sched_get_priority_min (SCHED_RR))
				{
					/* display usage message */
					f3s_usage (argc, argv, "background reclaim priority");
				} else
				{
					f3s.bkgd_priority = result;
				}

				f3s.bkgd_flag = 1;
				break;

			case 'f':			/* set options for flash services */
				result = strtol (optarg, NULL, 0);

				/* validate the background priority */
				if (result >= F3S_VERIFY_NUM) f3s_usage (argc, argv, "flash verify");
				else                          f3s.flash_verify = result;
				break;

			case 'm':			/* set mountpoint override */
				/* check if there is an override for regular mountpoint */
				if (*optarg != ',')
				{
					/* setup mountpoint default override */
					f3s.mount_over = optarg;

					/* scan for a null or coma */
					do
					{
						/* point to next character */
						optarg++;

						/* check for format specifier */
						if (*optarg == '%')
						{
							f3s.mount_format++;
							percent = 1;

						} else if (percent)
						{
							if (*optarg != 'X') f3s_usage (argc, argv, "mount format");
							percent = 0;
						}
					} while (*optarg != ',' && *optarg);

					/* check if there are too many format specifiers */
					if (f3s.mount_format > 2) f3s_usage (argc, argv, "mount format");
				}
				break;

			case 'p':			/* set parameters */
				if (*optarg != ',')
				{
					result = strtol (optarg, NULL, 0);

					/* validate the reclaim trigger */
					if (result > SHRT_MAX) f3s_usage (argc, argv, "background percent");
					else                   f3s.reclaim.etrigger = f3s.reclaim.trigger = result;

					/* skip to next sub-option */
					do {optarg++;} while (*optarg != ',' && *optarg);
				}

				if (*optarg == ',')
				{
					optarg++;
					result = strtol (optarg, NULL, 0);

					/* validate the reclaim threshold */
					if (result > SHRT_MAX) f3s_usage (argc, argv, "super limit");
					else                   f3s.reclaim.ethreshold = f3s.reclaim.threshold = result;
				}
				break;

			case 's':			/* set options for socket services */
				f3s.socket_option = optarg;
				break;

			case 'w':			/* write buffer size */
				result = strtol (optarg, NULL, 0);

				/* validate the background priority */
				if (result > F3S_WRITE_MAX) f3s_usage (argc, argv, "buffer size");
				else                       f3s.buffer_size = result;
				break;

			case 't':			/* number of threads */
				result = strtol (optarg, NULL, 0);

				/* validate the number of threads */
				if (result <= 0 || result > F3S_THREAD_MAX)
					f3s_usage (argc, argv, "number of threads");
				else
					f3s.thread_num = result;
				break;

			case 'u':			/* update level */
				result = strtol (optarg, NULL, 0);

				/* validate the update level */
				if (result >= F3S_UPDATE_NUM) f3s_usage (argc, argv, "time update level");
				else                          f3s.update_level = result;
				break;

			case 'R':			/* mount all partitions readonly */
				f3s.mount_options |= f3s_mntopts_t_MOUNT_RO;
				break;

			case 'E':			/* exit on fault */
				f3s.exit_on_fault = 1;
				break;

			case 'e':
				if(isdigit(optarg[0])){
						/* optarg is the number of partitions to auto-mount */
						f3s.enum_only = (int8_t)strtol(optarg, NULL, 10);
				}else{
					unsigned i = 0;
					char* targ = optarg;
					/* Optarg is a comma-separated list of exact mount strings */
					f3s.mountexact_num = 1;
					for (targ = optarg; *targ; targ++){
						if (*targ == ':'){
							f3s.mountexact_num++;
						}
					}
					f3s.mountexact_list = calloc(sizeof (char*), f3s.mountexact_num);
					for(i = 0; i< f3s.mountexact_num; i++){
						f3s.mountexact_list[i] = calloc(F3S_NAME_MAX+1U, 1U);
						if ((targ = memccpy(f3s.mountexact_list[i], optarg, ':', F3S_NAME_MAX))){
							/* Overwrite the ':' with a NULL */
							*(targ-1) = '\0';
							optarg += (targ - f3s.mountexact_list[i]);
						}
					}
				}
				break;
			case 'V':			/* print version and exit */
/* Avoid weak symbol magic for now */
#if 0
				if (flash_vect->struct_size == sizeof(f3s_flash_t))
				{
					printf ("F3S_VER=3 (" "$Name$" "," __DATE__ " " __TIME__ ") MTD_VER=%c (%s,%s,%s)\n",
					        '1', MTD_BRANCH, MTD_DATE, MTD_TIME);
				}
				else if (flash_vect->struct_size == sizeof(f3s_flash_v2_t))
				{
					printf ("F3S_VER=3 (" "$Name$" "," __DATE__ " " __TIME__ ") MTD_VER=%c (%s,%s,%s)\n",
					        '2', MTD_BRANCH, MTD_DATE, MTD_TIME);
				}
				else
				{
					printf ("F3S_VER=3 (" "$Name$" "," __DATE__ " " __TIME__ ") MTD_VER=%c (%s,%s,%s)\n",
					        '?', "?", "?", "?");
				}
#else
				if (flash_vect->struct_size == sizeof(f3s_flash_t))
				{
					printf ("F3S_VER=3 (" XSTR(FFS_BUILD) "," __DATE__ " " __TIME__ ") MTD_VER=%c\n",
					        '1');
				}
				else if (flash_vect->struct_size == sizeof(f3s_flash_v2_t))
				{
					printf ("F3S_VER=3 (" XSTR(FFS_BUILD) "," __DATE__ " " __TIME__ ") MTD_VER=%c\n",
					        '2');
				}
				else
				{
					printf ("F3S_VER=3 (" XSTR(FFS_BUILD) "," __DATE__ " " __TIME__ ") MTD_VER=%c\n",
					        '?');
				}
#endif
				exit(0);
				break;
			case 'd':			/* Log method */
				f3s.log_method = (int8_t)strtol (optarg, NULL, 10);
				break;
			case 'W':
				/* no op, just here to quell the warnings, since this is handled
				 * in the boards/main() function.
				 */
				 break;
			case 'o':
				f3s.xcache_file_max = atoi(optarg);
				break;
			case 'k':
				f3s.xcache_blksz = atoi(optarg)*1024;
				break;
			case ':':		/* missing argument */
				{
					char error_buffer[80];
					sprintf(error_buffer, "argument for option -%c\n", optopt);
					f3s_usage(argc, argv, error_buffer); 
				}
				break;
			case '?':		/* unknown option */
				{
					char error_buffer [20];
					sprintf(error_buffer, "option '-%c'\n", optopt);
					f3s_usage (argc, argv, error_buffer);
				}
				break;
			default :
				f3s_usage(argc, argv, "getopt error");
		}
	}
/*}}}1*/
}

int f3s_ebadfsys (uint32_t partnum, const char *file, uint32_t line)
{
/*{{{1*/
	if (f3s.exit_on_fault)
	{
		printf ("Corrupt file system detected. %s: %d\n", file, line);
		exit (partnum + 1);
	}

	return (EBADFSYS);
/*}}}1*/
}

int f3s_check_frgd_reclaim(f3s_part_t *part, int32_t text_size) 
{
	int ret = 0;

	F3S_ALLOC_ENTER(part);
	if ((part->table.free_total < text_size) && 
	     ((part->table.stale_total + part->table.free_total) >= text_size)) 
	{
		ret = 1;
	}	
	F3S_ALLOC_LEAVE(part); 
	return ret;
}

int f3s_check_bkgd_reclaim(f3s_part_t *part)
{
	int ret = 0;

	F3S_ALLOC_ENTER(part); 

	if (part->table.free_total && ((uint64_t)part->table.stale_total * 100 / (uint64_t)part->table.free_total) > 
	     ((part->table.reclaim.enable) ? part->table.reclaim.etrigger:part->table.reclaim.trigger)) 
	{
		ret = 1;
	}	
	F3S_ALLOC_LEAVE(part);
	return ret;
}

int f3s_check_stale_total(f3s_part_t *part)
{
	int ret = 0;

	F3S_ALLOC_ENTER(part);
	if(part->table.stale_total)
	{
		ret = 1;
	}
	F3S_ALLOC_LEAVE(part);
	return ret;
}

int f3s_check_free_total(f3s_part_t *part, int32_t limit) 
{
	int ret = 0;

	F3S_ALLOC_ENTER(part);
	if(part->table.free_total < limit)
	{
		ret = 1;
	}
	F3S_ALLOC_LEAVE(part);
	return ret;
}

#if (defined(INCLUDE_VERIFY_FUNCS) && !defined(NDEBUG))

void _f3s_verify_counters (f3s_part_t *part, int32_t phys, uint32_t extra_text)
{
/*{{{1*/
	f3s_head_t	head;
	f3s_map_t *	map;
	f3s_extptr_t temp_extptr;
	uint32_t	text_total  = 0;
	uint32_t	active_size = 0;
	uint32_t	free_total  = 0;
	uint32_t	stale_total = 0;
	uint32_t	pad_total   = 0;
	uint32_t	resv_total  = 0;
	uint32_t	head_num    = 0;
	uint32_t	spare_num   = 0;
	uint32_t	i, j, logi, logi_max;
	uint32_t	align_pow2;
	int32_t		text_size;

//printf ("verify_counters: %d\n", phys);
	F3S_TABLE_ENTER(part);
	logi_max = F3S_UNIT_ACTIVE(part);
	spare_num = part->unit_num - logi_max;
	align_pow2 = part->table.align_pow2;
	F3S_TABLE_LEAVE(part);

	for (logi = F3S_FIRST_LOGI; logi <= logi_max; i++)
	{
		temp_extptr.logi_unit = logi;
		temp_extptr.index = 0;

		F3S_LOGI_ENTER(part, logi);
		i = f3s_extptr_phys(part, temp_extptr);

		F3S_UNIT_ENTER(part, i);
		map = &part->array->map_vect[i];

		/* Verify unit integrity */
		f3s_assert ((map->text_size + map->free_size +
		             map->resv_size + ((map->head_last + 1) * sizeof(f3s_head_t))) ==
		            (part->array->unit_size));

		text_total  += map->text_size;
		free_total  += map->free_size;
		stale_total += map->stale_size;
		pad_total   += map->pad_size;
		resv_total  += map->resv_size;
		head_num    += map->head_last + 1;

		active_size = 0;
		for (j = 0; j <= map->head_last; j++)
		{
			text_size = f3s_head_read (part, i, j, NULL, &head, 1);
			if (text_size < 0){
				F3S_UNIT_LEAVE(part, i);
				F3S_LOGI_LEAVE(part, logi);   
				continue;
			}

			if ((head.status[0] & F3S_EXT_NO_SUPER) &&
			    ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_ALLOC))
			{
				if (!(head.status[0] & F3S_EXT_NO_WRITE))
				{
					active_size += F3S_TEXT_ALIGN(part, head.text_size, align_pow2);

				} else
				{
					active_size += F3S_TEXT_ALIGN(part, text_size, align_pow2);
				}
			}
		}
		if (i == phys) f3s_assert((active_size + map->stale_size + F3S_TEXT_ALIGN(part, extra_text, align_pow2)) == map->text_size);
		else           f3s_assert((active_size + map->stale_size) == map->text_size);
		active_size += (map->head_last + 1) * sizeof(f3s_head_t);

		f3s_assert((active_size + map->stale_size + map->free_size) <= part->array->unit_size);
		F3S_UNIT_LEAVE(part, i);
		F3S_LOGI_LEAVE(part, logi);
	}
	F3S_ALLOC_ENTER(part);
	f3s_assert (part->table.free_total  == free_total);
	f3s_assert (part->table.stale_total == stale_total);
	f3s_assert (part->table.pad_total   == pad_total);
	f3s_assert (part->table.resv_total  == resv_total);
	f3s_assert (part->table.head_num    == head_num);
	f3s_assert (text_total + free_total + resv_total + (head_num * sizeof(f3s_head_t)) + (spare_num * part->array->unit_size) ==
	            (part->array->unit_size * part->unit_num));
	F3S_ALLOC_LEAVE(part);
/*}}}1*/
}

void _f3s_verify_dir (f3s_part_t *part, int32_t logi, uint32_t index)
{
/*{{{1*/
	f3s_head_t		head;
	f3s_dirent_t	dirent;
	int32_t			tphys;
	int32_t			tindex = index;
	int32_t			text_size;
	int32_t			text_offset;
	int				error;
	int				swap;

	F3S_TABLE_ENTER(part);
	swap = part->table.swap_flag;
	F3S_TABLE_LEAVE(part);


	F3S_LOGI_ENTER(part, logi);
	tphys = part->table.logi_vect[logi];
	F3S_UNIT_ENTER(part, tphys);

//printf ("verify_dir: [%d,%d,%d]\n", logi, tphys, index);

	while (1)
	{
		/* Read the header */
		text_size = f3s_head_read (part, tphys, tindex, &text_offset, &head, 1);
		f3s_assert (text_size >= 0);

		/* Extent header level sanity checks */
		f3s_assert ((head.status[0] & F3S_EXT_TYPE) == F3S_EXT_DIR);

		/* Handle regular extents */
		if (head.status[0] & F3S_EXT_NO_SUPER)
		{
			if (text_size)
			{
				/* Read the directory entry */
				error = f3s_flash_read (part->array, tphys, text_offset,
				                        sizeof(f3s_dirent_t), (uint8_t *)&dirent);
				f3s_assert (!error);

				/* Endian swapping */
				if (swap) f3s_dirent_swap (&dirent);

				/* Directory entry level sanity checks */
				f3s_assert (dirent.struct_size == sizeof(f3s_dirent_t));
			}

			/* Stop on last extent */
			if (head.status[0] & F3S_EXT_NO_NEXT) break;

			/* Advance to the next entry */
			F3S_UNIT_LEAVE(part, tphys);
			F3S_LOGI_LEAVE(part, logi);
			logi = head.next.logi_unit;
			F3S_LOGI_ENTER(part, logi);
			tphys  = part->table.logi_vect[logi];
			tindex = head.next.index;

		/* Handle superseded extets */
		} else
		{
			/* Advance to the linked (super or next) entry */
			F3S_UNIT_LEAVE(part, tphys);
			F3S_LOGI_LEAVE(part, logi);
			logi = head.super.logi_unit;
			F3S_LOGI_ENTER(part, logi);
			tphys  = part->table.logi_vect[logi];
			tindex = head.super.index;
		}

		F3S_UNIT_ENTER(part, tphys);
	}

	F3S_UNIT_LEAVE(part, tphys);
	F3S_LOGI_LEAVE(part, logi);
/*}}}1*/
}

void _f3s_verify_headers (f3s_part_t *part, int32_t inc_phys)
{
/*{{{1*/
	f3s_head_t	head;
	f3s_head_t	lhead;
	f3s_map_t *	map;
	f3s_extptr_t temp_extptr;
	int32_t		lphys;
	int32_t		lindex;
	int32_t		text_size;
	int32_t		i, j, logi, max_logi, llogi;

	F3S_TABLE_ENTER(part);
	max_logi = F3S_UNIT_ACTIVE(part);
	F3S_TABLE_LEAVE(part);

	for (logi = F3S_FIRST_LOGI; logi <= max_logi; logi++)
	{
		F3S_LOGI_ENTER(part, logi);
		i = f3s_extptr_phys(part, temp_extptr);

		F3S_UNIT_ENTER(part, i);
		map = &part->array->map_vect[i];

		/* Verify each extent header */
		for (j = 0; j <= map->head_last; j++)
		{
			/* Read the header */
			text_size = f3s_head_read (part, i, j, NULL, &head, 1);
			f3s_assert (text_size >= 0);

			/* Ignore deleted extents */
			if ((head.status[0] & F3S_EXT_MASK) == F3S_EXT_DELETE) continue;

			/* Verify link */
			if (!(head.status[0] & F3S_EXT_NO_SUPER))
			{
				llogi = head.super.logi_unit;
				lindex = head.super.index;

			} else if (!(head.status[0] & F3S_EXT_NO_NEXT))
			{
				llogi = head.next.logi_unit;
				lindex = head.next.index;

			} else
			{
				F3S_UNIT_LEAVE(part, i);
				F3S_LOGI_LEAVE(part, logi);
				continue;
			}
			/* Read the linked header */
			F3S_LOGI_ENTER(part, llogi);
			lphys  = part->table.logi_vect[llogi];
			F3S_UNIT_ENTER(part, lphys);
			
			text_size = f3s_head_read (part, lphys, lindex, NULL, &lhead, 1);
			f3s_assert (text_size >= 0);

			F3S_UNIT_LEAVE(part, lphys);
			F3S_LOGI_LEAVE(part, llogi);

			/* Extent header level sanity checks */
			f3s_assert (!(lhead.status[0] & F3S_EXT_NO_WRITE));

			/* Ensure that they both have the same state and type */
			f3s_assert ((head.status[0]  & F3S_EXT_MASK) ==
			            (lhead.status[0] & F3S_EXT_MASK));
			f3s_assert ((head.status[0]  & F3S_EXT_TYPE) ==
			            (lhead.status[0] & F3S_EXT_TYPE));
		}

		F3S_UNIT_LEAVE(part, i);
		F3S_LOGI_LEAVE(part, logi);
	}
/*}}}1*/
}
#endif	/* (defined(INCLUDE_VERIFY_FUNCS) && !defined(NDEBUG)) */

__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/util.c $ $Rev: 580669 $" );
