/*
 * $QNXLicenseC:
 * Copyright 2007, QNX Software Systems. All Rights Reserved.
 *
 * You must obtain a written license from and pay applicable license fees to QNX
 * Software Systems before you may reproduce, modify or distribute this software,
 * or any work that includes all or part of this software.   Free development
 * licenses are available for evaluation and non-commercial purposes.  For more
 * information visit http://licensing.qnx.com or email licensing@qnx.com.
 *
 * This file may contain contributions from others.  Please review this entire
 * file for other proprietary rights or license notices, as well as the QNX
 * Development Suite License Guide at http://licensing.qnx.com/license-guide/
 * for other information.
 * $
 */

#include "f3s.h"

typedef struct data_buffer
{
	struct data_buffer *next;
	uint8_t data[1];

} dat_buf_t;
#define DATEXTRA	offsetof(dat_buf_t, data) 


#define CHECK_DESC_STAT(desc, stat)	(((desc)->status & (stat)) == (stat))


static void *xcache_page_get(f3s_xpool_t *pool);
static void xcache_page_put(f3s_xpool_t *pool, void *pbuf);
static int  xcache_page_recycle(f3s_part_t *part);
static inline f3s_xcache_t *xcache_entry_find(f3s_part_t *part, f3s_desc_t *desc);

/* xcache operations */

static inline f3s_xcache_t *xcache_entry_find(f3s_part_t *part, f3s_desc_t *desc)
{	
	int i;
	f3s_xcache_t *xcache = NULL;

	F3S_XCACHE_ENTER(part);
	for(i = 0; i < f3s.xcache_file_max; i++) {
		if(*(uint32_t *)&part->xcache[i].inode == *(uint32_t *)&desc->first) {
 			xcache = &part->xcache[i];
			xcache->status = XCACHE_STATUS_INUSE;
			break;
		}
	}
	F3S_XCACHE_LEAVE(part);
	return xcache;
}

f3s_xcache_t *f3s_xcache_lookup(f3s_part_t *part, f3s_desc_t *desc, int32_t forward_size) 
{
	int blks, off, page_off;

	f3s_xcache_page_t *page, *old_page = NULL;
	f3s_xcache_t *xcache = NULL;

	if(!CHECK_DESC_STAT(desc, F3S_DESC_NO_DIR|F3S_DESC_XCACHE) || (part->xcache == NULL)) {
		return NULL;
	}

	if((xcache = xcache_entry_find(part, desc)) == NULL) {
		return NULL;
	}

	/* check if it is neccessary to look for a cache entry */
	if(forward_size < f3s.xcache_blksz) {
		xcache->found = NULL;
		return xcache;
	}
	if(forward_size == F3S_FILE_MAX) {
		if(xcache->last->curr_offset > desc->curr_offset) {
			xcache->found = xcache->last;
		}
		else {
			xcache->found = NULL;
		}
		return xcache;
	}
 
	blks = (desc->curr_offset + forward_size)/f3s.xcache_blksz;
	if(blks > xcache->entries) {
		xcache->found = xcache->last;
		return xcache;
	}
	blks--;
	f3s_assert(blks >=0);
	/* find the closest page */
	page = xcache->page;
	page_off = blks/XCACHE_ENTRIES_PER_PAGE;
	if((page_off+1) > xcache->page_num) {
		page_off = xcache->page_num - 1;
		off = XCACHE_ENTRIES_PER_PAGE - 1;
	} else {
		off = blks%XCACHE_ENTRIES_PER_PAGE;
	}

	while(page_off > 0) {
		old_page = page;
		page = page->next;
		page_off--;
	}

RETRY:  /* find the closest offset */
	if(page->cache[off].curr_offset <= desc->curr_offset) {
		xcache->found = NULL;
	} else {
		if(page->cache[off].curr_offset > (desc->curr_offset + forward_size)) {
			if(off == 0) {
				if(old_page == NULL) {
					xcache->found = NULL;
				} else {
					page = old_page;
					off  = XCACHE_ENTRIES_PER_PAGE - 1;
					goto RETRY;
				}
			} else {
				off--;
				goto RETRY;
			}
		} else {
			xcache->found = &page->cache[off];
		}
	}
	f3s_assert ((off >=0) && (off < XCACHE_ENTRIES_PER_PAGE));

        return xcache;
}

/* create an record in xcache table for a file and setup first page */
f3s_xcache_t *f3s_xcache_new(f3s_part_t *part, f3s_desc_t *desc)
{
	int i, expired = f3s.xcache_file_max;
	f3s_xcache_t *xcache;
	f3s_xcache_page_t *page, *temp;

	if(!CHECK_DESC_STAT(desc, F3S_DESC_NO_DIR|F3S_DESC_XCACHE) || (part->xcache == NULL)) {
		return NULL;
	}

	if(desc->curr_offset < f3s.xcache_blksz) {
		return NULL;
	}

	if(desc->curr_offset/f3s.xcache_blksz > 1) {
		return NULL;
	}

	/* find an empty entry in xcache table */
	F3S_XCACHE_ENTER(part);
	for(i=0; i<f3s.xcache_file_max; i++) {
		if(part->xcache[i].status == XCACHE_STATUS_EMPTY) {
			break;
		}
		if(part->xcache[i].status == XCACHE_STATUS_EXPIRED) {
			expired = i;
		}
	}

	if(i == expired) {
		f3s_log_xcache(part->array->index, "table full");
		F3S_XCACHE_LEAVE(part);
		return NULL;
	} 

	xcache = &part->xcache[i < f3s.xcache_file_max ? i : expired];
	xcache->inode = desc->first;
	xcache->status = XCACHE_STATUS_INUSE;
	F3S_XCACHE_LEAVE(part);

	/* recycle unused page or allocate first page */
	if(i == f3s.xcache_file_max) { 
		page = xcache->page->next;
		while(page != NULL) {
			temp = page;
			page = page->next;
			xcache_page_put(&part->xpool, temp);
		}
	} else {
		while ((xcache->page = xcache_page_get(&part->xpool)) == NULL) {
			if(xcache_page_recycle(part)) {
				f3s_log_xcache(part->array->index, "no free page");
				F3S_XCACHE_ENTER(part);
				*(uint32_t *)&xcache->inode = 0;
				xcache->status = XCACHE_STATUS_EMPTY;
				F3S_XCACHE_LEAVE(part);
				return NULL;
			}
		}
	}

	/* init the xcache and first page */
	memset(xcache->page->cache, 0, sizeof(f3s_cache_t)*XCACHE_ENTRIES_PER_PAGE);
	xcache->page->cache[0].curr = desc->curr;
	xcache->page->cache[0].curr_offset = desc->curr_offset;
	xcache->last = &xcache->page->cache[0];
	xcache->page_num = 1;
	xcache->page->next = NULL;
	xcache->found = NULL;
	xcache->entries = 1;

	return xcache;
}

/* add entries for existing xcache record */
void f3s_xcache_update(f3s_part_t *part, f3s_xcache_t *xcache, f3s_desc_t *desc)
{
	int blks, loops;
	f3s_cache_t *cache, *old = NULL;
	f3s_xcache_page_t *page, *newpage = NULL;

	blks = desc->curr_offset/f3s.xcache_blksz;
	if (blks <= xcache->entries) {
		return;
	}

	loops = blks - xcache->entries - 1;
	if(loops > 0) {
		blks = xcache->entries + 1;
		old = xcache->last;
	}

LOOP:
	if(((blks-1)/XCACHE_ENTRIES_PER_PAGE + 1) > xcache->page_num) {
		while ((newpage = xcache_page_get(&part->xpool)) == NULL) {
			if(xcache_page_recycle(part)) {
				f3s_log_xcache(part->array->index, "no free page");
				return;
			}
		}
		newpage->next = NULL;
		memset(newpage->cache, 0, sizeof(f3s_cache_t)*XCACHE_ENTRIES_PER_PAGE);
		cache = &newpage->cache[0];
		page = (f3s_xcache_page_t *)(xcache->last - (XCACHE_ENTRIES_PER_PAGE - 1));
		page->next = newpage;
		xcache->last = cache;
		xcache->page_num++;

	} else {
		cache = ++xcache->last;
	}

	/* add a new cache entry */
	if(loops-- > 0) {
		memcpy(cache, old, sizeof(f3s_cache_t));
		old = xcache->last;
		xcache->entries++;
		blks++;
		goto LOOP;
	} else {
		cache->curr = desc->curr;
		cache->curr_offset = desc->curr_offset;
		xcache->entries++;
	}
}

/* set xcache state to expired when all file descriptors closed */
void f3s_xcache_expire(f3s_part_t *part, f3s_desc_t *desc)
{
	f3s_xcache_t *xcache = NULL;

	if(!CHECK_DESC_STAT(desc, F3S_DESC_NO_DIR|F3S_DESC_XCACHE) || (part->xcache == NULL)) {
		return;
	}

	if((xcache = xcache_entry_find(part, desc)) == NULL) {
		return;
	}

	F3S_XCACHE_ENTER(part);
	xcache->status = XCACHE_STATUS_EXPIRED;
	F3S_XCACHE_LEAVE(part);
}

/* discard cache entries beyond new EOF when file is growing smaller */
void f3s_xcache_truncate(f3s_part_t *part, f3s_desc_t *desc, int32_t trunc_offset)
{
	int blks, off, page_off;

	f3s_xcache_page_t *page, *temp, *prev_page;
	f3s_xcache_t *xcache = NULL;
	f3s_cache_t *cache;

	if(!CHECK_DESC_STAT(desc, F3S_DESC_NO_DIR|F3S_DESC_XCACHE) || (part->xcache == NULL)) {
		return;
	}

	if((xcache = xcache_entry_find(part, desc)) == NULL) {
		return;
	}

	/* Find what block the trunc_offset is within */
	blks = trunc_offset/f3s.xcache_blksz;
	if(xcache->entries < blks) {
		return;
	}

	/* get a pointer to the page that holds the block */	
	page_off = blks > 0 ? (blks-1)/XCACHE_ENTRIES_PER_PAGE : 0;
	page = xcache->page;
	prev_page = page;
	while(page_off--) {
		prev_page = page;
		page = page->next;
	}

	/* free any more pages beyond trunc_offset */
	while(page->next != NULL) {
		temp = page->next;
		page->next = temp->next;
		xcache_page_put(&part->xpool, temp);
		xcache->page_num--;
	}

	/* We're truncating to the first block, free it all */
	if(blks == 0) {
		xcache_page_put(&part->xpool, xcache->page);
		F3S_XCACHE_ENTER(part);
		memset(xcache, 0, sizeof(f3s_xcache_t));
		F3S_XCACHE_LEAVE(part);
		return;
	}

	/* invalidate unused caches */
	off = (blks-1)%XCACHE_ENTRIES_PER_PAGE;
	cache = &page->cache[off];
	if(cache->curr_offset >= trunc_offset) {
		if(blks == 1) {
			xcache_page_put(&part->xpool, xcache->page);
			F3S_XCACHE_ENTER(part);
			memset(xcache, 0, sizeof(f3s_xcache_t));
			F3S_XCACHE_LEAVE(part);
			return;
		}
		xcache->entries = blks - 1;
		if (off == 0){
			/* We're at the first block of the page.  Have to move to the last
			 * block of the previous page.  Then put the page back.
			 */
			off = XCACHE_ENTRIES_PER_PAGE - 1;

			f3s_assert(page->next == NULL);
			xcache_page_put(&part->xpool, page);
			page = prev_page;
			xcache->page_num--;
		}else{
			off--;
		}
	} else {
		xcache->entries = blks;
	}
	f3s_assert((off >= 0) && (off < XCACHE_ENTRIES_PER_PAGE));
	xcache->last = &page->cache[off];
	while(++off < XCACHE_ENTRIES_PER_PAGE) {
		memset(&page->cache[off], 0, sizeof(f3s_cache_t));
	}
}

/* free a xcache and associated pages when the file is removed */
void f3s_xcache_free(f3s_part_t *part, f3s_desc_t *desc)
{	
	f3s_xcache_page_t *page, *temp;
	f3s_xcache_t *xcache = NULL;

	if(!CHECK_DESC_STAT(desc, F3S_DESC_NO_DIR|F3S_DESC_XCACHE) || (part->xcache == NULL)) {
		return;
	}

	if((xcache = xcache_entry_find(part, desc)) == NULL) {
		return;
	}

	page = xcache->page;
	F3S_XCACHE_ENTER(part);
	memset(xcache, 0, sizeof(f3s_xcache_t));
	F3S_XCACHE_LEAVE(part);

	while(page != NULL) {
		temp = page->next;
		xcache_page_put(&part->xpool, page);
		page = temp;
	}
}

/* recycle pages of expired xcache when neccessary */
static int xcache_page_recycle(f3s_part_t *part)
{
	int i;
	f3s_xcache_t *xcache = NULL;
	f3s_xcache_page_t *page, *temp;

	F3S_XCACHE_ENTER(part);
	for(i=0; i<f3s.xcache_file_max; i++) {
		if(part->xcache[i].status == XCACHE_STATUS_EXPIRED) {
			xcache = &part->xcache[i];
			break;
		}
	}
	if(xcache == NULL) {
		F3S_XCACHE_LEAVE(part);
		return 1;
	}

	page = xcache->page;
	memset(xcache, 0, sizeof(f3s_xcache_t));
	F3S_XCACHE_LEAVE(part);

	while(page != NULL) {
		temp = page->next;
		xcache_page_put(&part->xpool, page);
		page = temp;
	}
	return 0;
}


/* xcache memory pool interfaces */

int f3s_xpool_init(f3s_xpool_t *pool, uint16_t size, uint16_t num)
{
	int i;
	uint32_t memsize, memaddr;

	/* init pool */
	pool->buffer_numb = num;
	pool->buffer_free = num;
	pool->buffer_size = size;
	pool->buffer_list = NULL;

	/* allocate memory */
	memsize = pool->buffer_size + DATEXTRA;
	pool->baseaddr = malloc(memsize*pool->buffer_numb);
	if(pool->baseaddr == NULL) {
		return (-1);
	}
	f3s_lock_init(&pool->lock);

	/* generate list of buffers */
	memaddr = (uint32_t)pool->baseaddr;

	for(i=0; i<pool->buffer_numb; i++) {
		dat_buf_t *b;
		b = (dat_buf_t *)(memaddr + i*memsize);
		b->next = pool->buffer_list;
		pool->buffer_list = b;
	}

	return 0;
}

void f3s_xpool_delete(f3s_xpool_t *pool)
{
	if(pool->baseaddr != NULL) {
		free(pool->baseaddr);
		pool->baseaddr = NULL;
		f3s_lock_free(&pool->lock);
	}
}

static void *xcache_page_get(f3s_xpool_t *pool)
{
	dat_buf_t *buffer;

	F3S_XPOOL_ENTER(pool);
	if(!pool->buffer_free) {
		F3S_XPOOL_LEAVE(pool);
		return NULL; 
	}
	pool->buffer_free--;
	buffer = pool->buffer_list;
	pool->buffer_list = buffer->next;
	F3S_XPOOL_LEAVE(pool);
	return ((void *)buffer->data);
}

static void xcache_page_put(f3s_xpool_t *pool, void *pbuf)
{
	dat_buf_t *buffer;

	buffer = (dat_buf_t *)((unsigned char *)pbuf - DATEXTRA);
	F3S_XPOOL_ENTER(pool);
	pool->buffer_free++;
	buffer->next = pool->buffer_list;
	pool->buffer_list = buffer;
	F3S_XPOOL_LEAVE(pool);
}


__SRCVERSION( "$URL: http://svn/product/tags/restricted/bsp/nto650/freescale-p4080-ds/latest/src/lib/fs-flash3/xcache.c $ $Rev: 580669 $" );
