/*
 * $QNXLicenseC: 
 * Copyright 2011, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $ 
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include <errno.h>
#include <hw/inout.h>
#include <time.h>
#include <string.h>
#include <sys/neutrino.h>

#define MY_PULSE_CODE 5
#define MAX_INTERRUPT_COUNT 1024

time_t last_output_time = 0;
uint32_t interrupt_count[MAX_INTERRUPT_COUNT];
uint32_t last_interrupt_count[MAX_INTERRUPT_COUNT];
uint32_t id_to_interrupt_map[MAX_INTERRUPT_COUNT];

void print_usage(char* command)
{
	fprintf(stderr, "%s [ -h -s sample_seconds ] interrupt_list\n"
	                "Attaches to all interrupts in interrupt_list and prints out a count of each interrupt\n"
	                "every sample_seconds (default of 1).\n", command);
}

const struct sigevent *hdlr(void *blah, int id )
{
	struct sigevent *int_event; 
	int_event = (struct sigevent *)blah;

	interrupt_count[id_to_interrupt_map[id]] += 1;
	return NULL;
}

int main(int argc, char *argv[])
{
	int c;
	int argCount;
	
	int id;
	int  self_coid, chid;
	struct sigevent int_event;
	
	uint32_t temp_interrupt_count[MAX_INTERRUPT_COUNT];
	uint32_t* attached_interrupts;
	int i;
	int interrupt;
	unsigned sample_seconds = 1;

	if( ThreadCtl( _NTO_TCTL_IO, NULL ) == -1 ) {
		fprintf( stderr, "netio:  ThreadCtl\n" );
		return( 0 );
	}
	
	while ((c = getopt(argc, argv, "hs:")) != -1) {
		switch (c) {
			case 'h':
				print_usage(argv[0]);
				goto success;
			case 's':
				sample_seconds = strtoul(optarg, NULL, 0);
				if (sample_seconds == 0) {
					fprintf(stderr, "Invalid sample_seconds\n");
					print_usage(argv[0]);
					goto fail;
				}
				break;
			default:
				fprintf(stderr, "Unknown option: %c\n", c);
				print_usage(argv[0]);
				goto fail;
		}
	}
	
	argCount = argc - optind;
	if (argCount < 1) {
		fprintf(stderr, "Must specify at least 1 interrupt to attach to\n");
		print_usage(argv[0]);
		goto fail;
	}
	
	attached_interrupts = calloc(argCount, sizeof(uint32_t));
	memset(interrupt_count, 0, MAX_INTERRUPT_COUNT*sizeof(uint32_t));
	memset(last_interrupt_count, 0, MAX_INTERRUPT_COUNT*sizeof(uint32_t));
	memset(temp_interrupt_count, 0, MAX_INTERRUPT_COUNT*sizeof(uint32_t));
	memset(id_to_interrupt_map, 0xff, MAX_INTERRUPT_COUNT*sizeof(uint32_t));
	
	for (i = 0; i < argCount; i++) {
		attached_interrupts[i] = strtoul(argv[optind+i], NULL, 0);
		if (attached_interrupts[i] == 0 && errno) {
			perror("strtoul");
			print_usage(argv[0]);
			goto fail;
		} else if (attached_interrupts[i] >= MAX_INTERRUPT_COUNT) {
			fprintf(stderr, "Specified interrupt %d too large, must be < %d\n", attached_interrupts[i], MAX_INTERRUPT_COUNT);
			goto fail;
		}
	}
	
	chid = ChannelCreate(_NTO_CHF_COID_DISCONNECT);
	if( -1 == chid) {
		perror(  "ChannelCreate");
		exit( EXIT_FAILURE );
	}
	self_coid = ConnectAttach( 0, 0, chid, _NTO_SIDE_CHANNEL, 0 );
	if( -1 == self_coid ) {
		perror( "ConnectAttach");
		exit( EXIT_FAILURE );
	}
	SIGEV_PULSE_INIT( &int_event, self_coid, 11, MY_PULSE_CODE, 0 );
	
	for (i = 0; i < argCount; i++) {
		id = InterruptAttach( attached_interrupts[i], hdlr, &int_event, sizeof(int_event), _NTO_INTR_FLAGS_TRK_MSK );
		if (id >= MAX_INTERRUPT_COUNT) {
			fprintf(stderr, "Unlucky bug, id handed out by OS too big, this is a consequence of the interrupt\n"
			        "to ID map implementation of this tool.\n");
			goto fail;
		}
		id_to_interrupt_map[id] = attached_interrupts[i];
	}

	while (1) {
		sleep(sample_seconds);
		/* "Sample" the interrupt counts */
		for (i = 0; i < argCount; i++) {
			interrupt = attached_interrupts[i];
			temp_interrupt_count[interrupt] = interrupt_count[interrupt];
		}
		
		for (i = 0; i < argCount; i++) {
			interrupt = attached_interrupts[i];
			if (temp_interrupt_count[interrupt] != last_interrupt_count[interrupt]) {
				printf("Interrupt #%d: %d (%d)\n", interrupt, temp_interrupt_count[interrupt], (temp_interrupt_count[interrupt] - last_interrupt_count[interrupt]));
				last_interrupt_count[interrupt] = temp_interrupt_count[interrupt];
			}
		}
	}
	
//	while( 1 ) {
//		rcvid = MsgReceive( chid, &pmsg, sizeof(pmsg), NULL );
//		if( -1 == rcvid ) {
//			perror("MsgReceive");
//			continue;
//		}
//		if ( 0 == rcvid ) {
////			unsigned char byte;
////			if( pmsg.code != MY_PULSE_CODE ) {
////				printf("unexpected pulse code, oops\n");
////				continue;
////			}
////			byte = pmsg.value.sival_int;
////			printf("got byte %x\n", byte );
////			if ( byte == 0x12 && !eflag )
////				exit(0);
////			if ( byte == 0x98 && !eflag )
////				exit(0);
////			if ( byte == 0xd1 && eflag )
////				exit( 0 );
////			eflag = 0;
////			if (byte == 0xe0 ) eflag = 1;
//
//			cur_time = time(NULL);
//			if (cur_time != last_output_time) {
//				last_output_time = cur_time;
//				printf("Now at %d interrupts\n", count);
//			}
//			continue;
//		}
//		printf( "got an unexpected message\n" );
//		MsgError( rcvid, ENOSYS );
//	}
	
success:
	return 0;
	
fail:
	return -1;
}
