/******************************************************************************

  1995-2003, 2004, 2005-2010 Freescale Semiconductor, Inc.
 All rights reserved.

 This is proprietary source code of Freescale Semiconductor Inc.,
 and its use is subject to the NetComm Device Drivers EULA.
 The copyright notice above does not evidence any actual or intended
 publication of such source code.

 **************************************************************************/
/******************************************************************************
 @File          fm.c

 @Description   FM driver routines implementation.
*//***************************************************************************/
#include "std_ext.h"
#include "error_ext.h"
#include "xx_ext.h"
#include "string_ext.h"
#include "sprint_ext.h"
#include "debug_ext.h"
#include "fm_muram_ext.h"

#include "fm_common.h"
#include "fm_ipc.h"
#include "fm.h"


/****************************************/
/*       static functions               */
/****************************************/

static volatile bool blockingFlag = FALSE;
static void IpcMsgCompletionCB(t_Handle   h_Fm,
                               uint8_t    *p_Msg,
                               uint8_t    *p_Reply,
                               uint32_t   replyLength,
                               t_Error    status)
{
    UNUSED(h_Fm);UNUSED(p_Msg);UNUSED(p_Reply);UNUSED(replyLength);UNUSED(status);
    blockingFlag = FALSE;
}

static t_Error CheckFmParameters(t_Fm *p_Fm)
{

    if(!p_Fm->p_FmDriverParam->dmaAxiDbgNumOfBeats || (p_Fm->p_FmDriverParam->dmaAxiDbgNumOfBeats > DMA_MODE_MAX_AXI_DBG_NUM_OF_BEATS))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("axiDbgNumOfBeats has to be in the range 1 - %d", DMA_MODE_MAX_AXI_DBG_NUM_OF_BEATS));
    if(p_Fm->p_FmDriverParam->dmaCamNumOfEntries % DMA_CAM_UNITS)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaCamNumOfEntries has to be divisble by %d", DMA_CAM_UNITS));
    if(!p_Fm->p_FmDriverParam->dmaCamNumOfEntries || (p_Fm->p_FmDriverParam->dmaCamNumOfEntries > DMA_MODE_MAX_CAM_NUM_OF_ENTRIES))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaCamNumOfEntries has to be in the range 1 - %d", DMA_MODE_MAX_CAM_NUM_OF_ENTRIES));
    if(p_Fm->p_FmDriverParam->dmaCommQThresholds.assertEmergency > DMA_THRESH_MAX_COMMQ)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaCommQThresholds.assertEmergency can not be larger than %d", DMA_THRESH_MAX_COMMQ));
    if(p_Fm->p_FmDriverParam->dmaCommQThresholds.clearEmergency > DMA_THRESH_MAX_COMMQ)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaCommQThresholds.clearEmergency can not be larger than %d", DMA_THRESH_MAX_COMMQ));
    if(p_Fm->p_FmDriverParam->dmaCommQThresholds.clearEmergency >= p_Fm->p_FmDriverParam->dmaCommQThresholds.assertEmergency)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaCommQThresholds.clearEmergency must be smaller than dmaCommQThresholds.assertEmergency"));
    if(p_Fm->p_FmDriverParam->dmaReadBufThresholds.assertEmergency > DMA_THRESH_MAX_BUF)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaReadBufThresholds.assertEmergency can not be larger than %d", DMA_THRESH_MAX_BUF));
    if(p_Fm->p_FmDriverParam->dmaReadBufThresholds.clearEmergency > DMA_THRESH_MAX_BUF)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaReadBufThresholds.clearEmergency can not be larger than %d", DMA_THRESH_MAX_BUF));
    if(p_Fm->p_FmDriverParam->dmaReadBufThresholds.clearEmergency >= p_Fm->p_FmDriverParam->dmaReadBufThresholds.assertEmergency)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaReadBufThresholds.clearEmergency must be smaller than dmaReadBufThresholds.assertEmergency"));
    if(p_Fm->p_FmDriverParam->dmaWriteBufThresholds.assertEmergency > DMA_THRESH_MAX_BUF)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaWriteBufThresholds.assertEmergency can not be larger than %d", DMA_THRESH_MAX_BUF));
    if(p_Fm->p_FmDriverParam->dmaWriteBufThresholds.clearEmergency > DMA_THRESH_MAX_BUF)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaWriteBufThresholds.clearEmergency can not be larger than %d", DMA_THRESH_MAX_BUF));
    if(p_Fm->p_FmDriverParam->dmaWriteBufThresholds.clearEmergency >= p_Fm->p_FmDriverParam->dmaWriteBufThresholds.assertEmergency)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("dmaWriteBufThresholds.clearEmergency must be smaller than dmaWriteBufThresholds.assertEmergency"));

    if(!p_Fm->fmClkFreq)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("fmClkFreq must be set."));
    if (USEC_TO_CLK(p_Fm->p_FmDriverParam->dmaWatchdog, p_Fm->fmClkFreq) > DMA_MAX_WATCHDOG)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE,
                     ("dmaWatchdog depends on FM clock. dmaWatchdog(in microseconds) * clk (in Mhz), may not exceed 0x08x", DMA_MAX_WATCHDOG));

#ifdef FM_PARTITION_ARRAY
    {
        t_FmRevisionInfo revInfo;
        uint8_t     i;

        FM_GetRevision(p_Fm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            for (i=0; i<FM_SIZE_OF_LIODN_TABLE; i++)
                if (p_Fm->p_FmDriverParam->liodnBasePerPort[i] & ~FM_LIODN_BASE_MASK)
                    RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("liodn number is out of range"));
    }
#endif /* FM_PARTITION_ARRAY */

    if(p_Fm->totalFifoSize % BMI_FIFO_UNITS)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("totalFifoSize number has to be divisible by %d", BMI_FIFO_UNITS));
    if(!p_Fm->totalFifoSize || (p_Fm->totalFifoSize > BMI_MAX_FIFO_SIZE))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("totalFifoSize number has to be in the range 256 - %d", BMI_MAX_FIFO_SIZE));
    if(!p_Fm->totalNumOfTasks || (p_Fm->totalNumOfTasks > BMI_MAX_NUM_OF_TASKS))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("totalNumOfTasks number has to be in the range 1 - %d", BMI_MAX_NUM_OF_TASKS));
    if(!p_Fm->maxNumOfOpenDmas || (p_Fm->maxNumOfOpenDmas > BMI_MAX_NUM_OF_DMAS))
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("maxNumOfOpenDmas number has to be in the range 1 - %d", BMI_MAX_NUM_OF_DMAS));

    if(p_Fm->p_FmDriverParam->thresholds.dispLimit > FPM_MAX_DISP_LIMIT)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("thresholds.dispLimit can't be greater than %d", FPM_MAX_DISP_LIMIT));

    if(!p_Fm->f_Exception)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Exceptions callback not provided"));
    if(!p_Fm->f_BusError)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Exceptions callback not provided"));

    return E_OK;
}

static void SendIpcIsr(t_Fm *p_Fm, uint32_t macEvent, uint32_t pendingReg)
{
    t_Error     err;
    t_FmIpcIsr  fmIpcIsr;
    t_FmIpcMsg  msg;

    ASSERT_COND(p_Fm->guestId == NCSW_MASTER_ID);
    if (p_Fm->intrMng[macEvent].guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GUEST_ISR;
        fmIpcIsr.pendingReg = pendingReg;
        fmIpcIsr.boolErr = FALSE;
        memcpy(msg.msgBody, &fmIpcIsr, sizeof(fmIpcIsr));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[p_Fm->intrMng[macEvent].guestId],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) + sizeof(fmIpcIsr),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            REPORT_ERROR(MINOR, err, NO_MSG);
        return;
    }
    else
        p_Fm->intrMng[macEvent].f_Isr(p_Fm->intrMng[macEvent].h_SrcHandle);
}


static void    BmiErrEvent(t_Fm *p_Fm)
{
    uint32_t    event, mask;

    event = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ievr);
    mask = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier);

    /* clear the forced events */
    if(GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr)& event)
        WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr, event & ~(event & mask));

    event &= mask;

    /* clear the acknowledged events */
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ievr, event);

    if(event & BMI_ERR_INTR_EN_PIPELINE_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_BMI_PIPELINE_ECC);
    if(event & BMI_ERR_INTR_EN_LIST_RAM_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_BMI_LIST_RAM_ECC);
    if(event & BMI_ERR_INTR_EN_STATISTICS_RAM_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_BMI_STATISTICS_RAM_ECC);
    if(event & BMI_ERR_INTR_EN_DISPATCH_RAM_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_BMI_DISPATCH_RAM_ECC);
}

static void    QmiErrEvent(t_Fm *p_Fm)
{
    uint32_t    event, mask;

    event = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_eie);
    mask = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien);
    /* clear the forced events */
    if(GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_eif)& event)
        WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eif, event & ~(event & mask));

    event &= mask;

    /* clear the acknowledged events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eie, event);

    if(event & QMI_ERR_INTR_EN_DOUBLE_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_QMI_DOUBLE_ECC);
    if(event & QMI_ERR_INTR_EN_DEQ_FROM_DEF)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID);
}

static void    DmaErrEvent(t_Fm *p_Fm)
{
    uint64_t            addr=0;
    uint32_t            status, mask, tmpReg=0;
    uint8_t             tnum;
    uint8_t             hardwarePortId;
    uint8_t             relativePortId;
    uint16_t            liodn;

    status = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmsr);
    mask = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr);

    /* get bus error regs befor clearing BER */
    if ((status & DMA_STATUS_BUS_ERR) && (mask & DMA_MODE_BER))
    {
        addr = (uint64_t)GET_UINT32(p_Fm->p_FmDmaRegs->fmdmtal);
        addr |= (uint64_t)GET_UINT32(p_Fm->p_FmDmaRegs->fmdmtah) << 32;

        /* get information about the owner of that bus error */
        tmpReg = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmtcid);
    }

    /* clear set events */
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmsr, status);

    if ((status & DMA_STATUS_BUS_ERR) && (mask & DMA_MODE_BER))
    {
        hardwarePortId = (uint8_t)(((tmpReg & DMA_TRANSFER_PORTID_MASK) >> DMA_TRANSFER_PORTID_SHIFT));
        HW_PORT_ID_TO_SW_PORT_ID(relativePortId, hardwarePortId);
        tnum = (uint8_t)((tmpReg & DMA_TRANSFER_TNUM_MASK) >> DMA_TRANSFER_TNUM_SHIFT);
        liodn = (uint16_t)(tmpReg & DMA_TRANSFER_LIODN_MASK);
        ASSERT_COND(p_Fm->portsTypes[hardwarePortId] != e_FM_PORT_TYPE_DUMMY);
        p_Fm->f_BusError(p_Fm->h_App, p_Fm->portsTypes[hardwarePortId], relativePortId, addr, tnum, liodn);
    }
    if(mask & DMA_MODE_ECC)
    {
        if (status & DMA_STATUS_READ_ECC)
            p_Fm->f_Exception(p_Fm->h_App, e_FM_EX_DMA_READ_ECC);
        if (status & DMA_STATUS_SYSTEM_WRITE_ECC)
            p_Fm->f_Exception(p_Fm->h_App, e_FM_EX_DMA_SYSTEM_WRITE_ECC);
        if (status & DMA_STATUS_FM_WRITE_ECC)
            p_Fm->f_Exception(p_Fm->h_App, e_FM_EX_DMA_FM_WRITE_ECC);
    }
}

static void    FpmErrEvent(t_Fm *p_Fm)
{
    uint32_t    event;

    event = GET_UINT32(p_Fm->p_FmFpmRegs->fpmem);

    /* clear the all occured events */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, event);

    if((event  & FPM_EV_MASK_DOUBLE_ECC) && (event & FPM_EV_MASK_DOUBLE_ECC_EN))
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_FPM_DOUBLE_ECC);
    if((event  & FPM_EV_MASK_STALL) && (event & FPM_EV_MASK_STALL_EN))
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_FPM_STALL_ON_TASKS);
    if((event  & FPM_EV_MASK_SINGLE_ECC) && (event & FPM_EV_MASK_SINGLE_ECC_EN))
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_FPM_SINGLE_ECC);
}

static void    MuramErrIntr(t_Fm *p_Fm)
{
    uint32_t    event, mask;

    event = GET_UINT32(p_Fm->p_FmFpmRegs->fmrcr);
    mask = GET_UINT32(p_Fm->p_FmFpmRegs->fmrie);

    /* clear MURAM event bit */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, event & ~FPM_RAM_CTL_IRAM_ECC);

    ASSERT_COND(event & FPM_RAM_CTL_MURAM_ECC);
    ASSERT_COND(event & FPM_RAM_CTL_RAMS_ECC_EN);

    if ((mask & FPM_MURAM_ECC_ERR_EX_EN))
        p_Fm->f_Exception(p_Fm->h_App, e_FM_EX_MURAM_ECC);
}

static void IramErrIntr(t_Fm *p_Fm)
{
    uint32_t    event, mask;

    event = GET_UINT32(p_Fm->p_FmFpmRegs->fmrcr) ;
    mask = GET_UINT32(p_Fm->p_FmFpmRegs->fmrie);
    /* clear the acknowledged events (do not clear IRAM event) */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, event & ~FPM_RAM_CTL_MURAM_ECC);

    ASSERT_COND(event & FPM_RAM_CTL_IRAM_ECC);
    ASSERT_COND(event & FPM_RAM_CTL_IRAM_ECC_EN);

    if ((mask & FPM_IRAM_ECC_ERR_EX_EN))
        p_Fm->f_Exception(p_Fm->h_App, e_FM_EX_IRAM_ECC);
}

static void QmiEvent(t_Fm *p_Fm)
{
    uint32_t    event, mask;

    event = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_ie);
    mask = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_ien);
    /* clear the forced events */
    if(GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_if)& event)
        WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_if,  event & ~(event & mask));

    event &= mask;

    /* clear the acknowledged events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_ie, event);

    if(event & QMI_INTR_EN_SINGLE_ECC)
        p_Fm->f_Exception(p_Fm->h_App,e_FM_EX_QMI_SINGLE_ECC);
}

static void UnimplementedIsr(t_Handle h_Arg)
{
    UNUSED(h_Arg);

    REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, ("Unimplemented Isr!"));
}

static void UnimplementedFmanCtrlIsr(t_Handle h_Arg, uint32_t event)
{
    UNUSED(h_Arg); UNUSED(event);

    REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, ("Unimplemented FmCtl Isr!"));
}

static void FmEnableTimeStamp(t_Fm *p_Fm)
{
    uint32_t                tmpReg;
    uint64_t                fraction;
    uint32_t                integer;
    uint8_t                 count1MicroBit = 8;
    uint32_t                tsFrequency = (uint32_t)(1<<count1MicroBit); /* in Mhz */

    /* configure timestamp so that bit 8 will count 1 microsecond */
    /* Find effective count rate at TIMESTAMP least significant bits:
       Effective_Count_Rate = 1MHz x 2^8 = 256MHz
       Find frequency ratio between effective count rate and the clock:
       Effective_Count_Rate / CLK e.g. for 600 MHz clock:
       256/600 = 0.4266666... */
    integer = tsFrequency/p_Fm->fmClkFreq;
    /* we multiply by 2^16 to keep the fraction of the division */
    /* we do not divid back, since we write this value as fraction - see spec */
    fraction = ((tsFrequency << 16) - (integer << 16)*p_Fm->fmClkFreq)/p_Fm->fmClkFreq;
    /* we check remainder of the division in order to round up if not integer */
    if(((tsFrequency << 16) - (integer << 16)*p_Fm->fmClkFreq) % p_Fm->fmClkFreq)
        fraction++;

    tmpReg = (integer << FPM_TS_INT_SHIFT) | (uint16_t)fraction;
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmtsc2, tmpReg);

    /* enable timestamp with original clock */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmtsc1, FPM_TS_CTL_EN);

    p_Fm->count1MicroBit = count1MicroBit;
    p_Fm->enabledTimeStamp = TRUE;
}


static void FreeInitResources(t_Fm *p_Fm)
{
    if (p_Fm->camBaseAddr)
       FM_MURAM_FreeMem(p_Fm->h_FmMuram, CAST_UINT64_TO_POINTER(p_Fm->camBaseAddr));
    if (p_Fm->fifoBaseAddr)
       FM_MURAM_FreeMem(p_Fm->h_FmMuram, CAST_UINT64_TO_POINTER(p_Fm->fifoBaseAddr));
    if (p_Fm->resAddr)
       FM_MURAM_FreeMem(p_Fm->h_FmMuram, CAST_UINT64_TO_POINTER(p_Fm->resAddr));
}

static t_Error ClearIRam(t_Fm *p_Fm)
{
    t_FMIramRegs    *p_Iram = CAST_UINT64_TO_POINTER_TYPE(t_FMIramRegs, (p_Fm->baseAddr + FM_MM_IMEM));
    int             i;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    /* Enable the auto-increment */
    WRITE_UINT32(p_Iram->iadd, IRAM_IADD_AIE);
    while (GET_UINT32(p_Iram->iadd) != IRAM_IADD_AIE) ;

    for (i=0; i < (FM_IRAM_SIZE/4); i++)
        WRITE_UINT32(p_Iram->idata, 0xffffffff);


    WRITE_UINT32(p_Iram->iadd, FM_IRAM_SIZE - 4);
    while (GET_UINT32(p_Iram->idata) != 0xffffffff) ;

    return E_OK;
}

static t_Error LoadFmanCtrlCode(t_Fm *p_Fm)
{
    t_FMIramRegs    *p_Iram = CAST_UINT64_TO_POINTER_TYPE(t_FMIramRegs, (p_Fm->baseAddr + FM_MM_IMEM));
    int             i;
    uint32_t        tmp;
    uint8_t         compTo16;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    /* Enable the auto-increment */
    WRITE_UINT32(p_Iram->iadd, IRAM_IADD_AIE);
    while (GET_UINT32(p_Iram->iadd) != IRAM_IADD_AIE) ;

    for (i=0; i < (p_Fm->p_FmDriverParam->firmware.size / 4); i++)
        WRITE_UINT32(p_Iram->idata, p_Fm->p_FmDriverParam->firmware.p_Code[i]);

    compTo16 = (uint8_t)(p_Fm->p_FmDriverParam->firmware.size % 16);
    if(compTo16)
        for (i=0; i < ((16-compTo16) / 4); i++)
            WRITE_UINT32(p_Iram->idata, 0xffffffff);

    WRITE_UINT32(p_Iram->iadd,p_Fm->p_FmDriverParam->firmware.size-4);
    /* verify that writing has completed */
    while (GET_UINT32(p_Iram->idata) != p_Fm->p_FmDriverParam->firmware.p_Code[(p_Fm->p_FmDriverParam->firmware.size / 4)-1]) ;

    if (p_Fm->p_FmDriverParam->fwVerify)
    {
        WRITE_UINT32(p_Iram->iadd, IRAM_IADD_AIE);
        while (GET_UINT32(p_Iram->iadd) != IRAM_IADD_AIE) ;
        for (i=0; i < (p_Fm->p_FmDriverParam->firmware.size / 4); i++)
            if ((tmp=GET_UINT32(p_Iram->idata)) != p_Fm->p_FmDriverParam->firmware.p_Code[i])
                RETURN_ERROR(MAJOR, E_WRITE_FAILED,
                             ("UCode write error : write 0x%x, read 0x%x",
                              p_Fm->p_FmDriverParam->firmware.p_Code[i],tmp));
        WRITE_UINT32(p_Iram->iadd, 0x0);
    }
    /* Enable patch from IRAM */
    WRITE_UINT32(p_Iram->iready, IRAM_READY);

    DBG(INFO, ("FMan-Controller code (ver %d.%d) loaded to IRAM.",
               ((uint8_t *)p_Fm->p_FmDriverParam->firmware.p_Code)[5],
               ((uint8_t *)p_Fm->p_FmDriverParam->firmware.p_Code)[7]));
    XX_Print("FMan%d-Controller code (ver %d.%d) loaded to IRAM.", p_Fm->fmId,
               ((uint8_t *)p_Fm->p_FmDriverParam->firmware.p_Code)[5],
               ((uint8_t *)p_Fm->p_FmDriverParam->firmware.p_Code)[7]);

    return E_OK;
}


/****************************************/
/*       Inter-Module functions        */
/****************************************/
static t_Error    FmGuestHandleIpcMsgCB(t_Handle  h_Fm,
                                        uint8_t   *p_Msg,
                                        uint32_t  msgLength,
                                        uint8_t   *p_Reply,
                                        uint32_t  *p_ReplyLength)
{
    uint32_t    msgId;
    uint8_t     *p_MsgBody;

UNUSED(p_Reply);
    SANITY_CHECK_RETURN_ERROR(h_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((msgLength > sizeof(uint32_t)), E_INVALID_VALUE);

    ASSERT_COND(p_Msg);
    msgId = ((t_FmIpcMsg*)p_Msg)->msgId;
    p_MsgBody = ((t_FmIpcMsg*)p_Msg)->msgBody;

    switch(msgId)
    {
        case (FM_GUEST_ISR):
            *p_ReplyLength = 0;
            if(((t_FmIpcIsr*)p_MsgBody)->boolErr)
                FM_GuestErrorIsr(h_Fm,  ((t_FmIpcIsr*)p_MsgBody)->pendingReg);
            else
                FM_GuestEventIsr(h_Fm,  ((t_FmIpcIsr*)p_MsgBody)->pendingReg);
            break;
        default:
            *p_ReplyLength = 0;
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("command not found!!!"));
    }
    return E_OK;
}

static t_Error     FmHandleIpcMsgCB(t_Handle  h_Fm,
                                    uint8_t   *p_Msg,
                                    uint32_t  msgLength,
                                    uint8_t   *p_Reply,
                                    uint32_t  *p_ReplyLength)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    msgId;
    uint8_t     *p_MsgBody;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR((msgLength >= sizeof(uint32_t)), E_INVALID_VALUE);

    ASSERT_COND(p_Msg);
    msgId = ((t_FmIpcMsg*)p_Msg)->msgId;
    p_MsgBody = ((t_FmIpcMsg*)p_Msg)->msgBody;

    memset(p_Reply, 0, FM_IPC_MAX_REPLY_SIZE);

    switch(msgId)
    {
        case (FM_GET_SET_PORT_PARAMS):
        {
            t_FmInterModulePortInitParams     initParams;
            t_Error                             err;

            *p_ReplyLength = 0;
            initParams.hardwarePortId = ((t_FmIpcPortInInitParams*)p_MsgBody)->hardwarePortId;
            initParams.portType = (e_FmPortType)((t_FmIpcPortInInitParams*)p_MsgBody)->enumPortType;
            initParams.independentMode = (bool)(((t_FmIpcPortInInitParams*)p_MsgBody)->boolIndependentMode);
            initParams.liodnOffset = ((t_FmIpcPortInInitParams*)p_MsgBody)->liodnOffset;
            initParams.numOfTasks = ((t_FmIpcPortInInitParams*)p_MsgBody)->numOfTasks;
            initParams.numOfExtraTasks = ((t_FmIpcPortInInitParams*)p_MsgBody)->numOfExtraTasks;
            initParams.numOfOpenDmas = ((t_FmIpcPortInInitParams*)p_MsgBody)->numOfOpenDmas;
            initParams.numOfExtraOpenDmas = ((t_FmIpcPortInInitParams*)p_MsgBody)->numOfExtraOpenDmas;
            initParams.sizeOfFifo = ((t_FmIpcPortInInitParams*)p_MsgBody)->sizeOfFifo;
            initParams.extraSizeOfFifo = ((t_FmIpcPortInInitParams*)p_MsgBody)->extraSizeOfFifo;
            initParams.deqPipelineDepth = ((t_FmIpcPortInInitParams*)p_MsgBody)->deqPipelineDepth;
            initParams.liodnBase = ((t_FmIpcPortInInitParams*)p_MsgBody)->liodnBase;
            err = FmGetSetPortParams(h_Fm, &initParams);
            if(err)
                return err;
            else
            {
                *p_ReplyLength = sizeof(t_FmIpcPhysAddr);
                ((t_FmIpcPhysAddr*)p_Reply)->high = initParams.fmMuramPhysBaseAddr.high;
                ((t_FmIpcPhysAddr*)p_Reply)->low = initParams.fmMuramPhysBaseAddr.low;
            }
            break;
        }
        case (FM_RESUME_STALLED_PORT):
            *p_ReplyLength = 0;
            return FmResumeStalledPort(h_Fm, p_MsgBody[0]);
        case (FM_MASTER_IS_ALIVE):
        {
            uint8_t guestId = p_MsgBody[0];
            /* build the FM master partition IPC address */
            memset(p_Fm->fmIpcHandlerModuleName[guestId], 0, MODULE_NAME_SIZE);
            if(Sprint (p_Fm->fmIpcHandlerModuleName[guestId], "FM_%d_%d",p_Fm->fmId, guestId) != (guestId<10 ? 6:7))
                RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));
            p_Fm->h_IpcSessions[guestId] = XX_IpcInitSession(p_Fm->fmIpcHandlerModuleName[guestId], p_Fm->fmModuleName);
            if (p_Fm->h_IpcSessions[guestId] == NULL)
                RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("FM Master IPC session for guest %d", guestId));
            ((uint8_t*)p_Reply)[0] = 1;
            *p_ReplyLength = sizeof(uint8_t);
            break;
        }

        case (FM_IS_PORT_STALLED):
        {
            bool tmp;
            *p_ReplyLength = sizeof(uint8_t);
            if (FmIsPortStalled(h_Fm, p_MsgBody[0], &tmp) != E_OK)
                RETURN_ERROR(MINOR, E_INVALID_STATE, NO_MSG);
            p_Reply[0] = (uint8_t)tmp;
            break;
        }
        case (FM_RESET_MAC):
            *p_ReplyLength = 0;
            return FmResetMac(p_Fm, (e_FmMacType)(((t_FmIpcMacParams*)p_MsgBody)->enumType), ((t_FmIpcMacParams*)p_MsgBody)->id);
        case (FM_SET_MAC_MAX_FRAME):
            *p_ReplyLength = 0;
            return FmSetMacMaxFrame(p_Fm, (e_FmMacType)(((t_FmIpcMacMaxFrameParams*)p_MsgBody)->macParams.enumType),
                                    ((t_FmIpcMacMaxFrameParams*)p_MsgBody)->macParams.id,
                                    ((t_FmIpcMacMaxFrameParams*)p_MsgBody)->maxFrameLength);
        case (FM_GET_CLK_FREQ):
            *(uint16_t*)p_Reply = p_Fm->fmClkFreq;
            *p_ReplyLength = sizeof(uint16_t);
            break;
        case (FM_FREE_PORT):
        {
            t_FmInterModulePortFreeParams   portParams;

            *p_ReplyLength = 0;
            portParams.hardwarePortId = ((t_FmIpcPortFreeParams*)p_MsgBody)->hardwarePortId;
            portParams.portType = (e_FmPortType)(((t_FmIpcPortFreeParams*)p_MsgBody)->enumPortType);
#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
            portParams.deqPipelineDepth = ((t_FmIpcPortFreeParams*)p_MsgBody)->deqPipelineDepth;
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */
            FmFreePortParams(h_Fm, &portParams);
        }
            break;
        case (FM_REGISTER_INTR):
            *p_ReplyLength = 0;
            p_Fm->intrMng[((t_FmIpcRegisterIntr*)p_MsgBody)->event].guestId = ((t_FmIpcRegisterIntr*)p_MsgBody)->guestId;
            break;
#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
        case (FM_DUMP_REGS):
            *p_ReplyLength = 0;
            return FM_DumpRegs(h_Fm);
        case (FM_DUMP_PORT_REGS):
            *p_ReplyLength = 0;
            return FmDumpPortRegs(h_Fm, p_MsgBody[0]);
#endif /* (defined(DEBUG_ERRORS) && ... */
        case (FM_GET_REV):
        {
            t_FmRevisionInfo fmRevInfo;

            FM_GetRevision(h_Fm, &fmRevInfo);
            ((t_FmIpcRevisionInfo*)p_Reply)->majorRev = fmRevInfo.majorRev;
            ((t_FmIpcRevisionInfo*)p_Reply)->minorRev = fmRevInfo.minorRev;
            *p_ReplyLength = sizeof(t_FmIpcRevisionInfo);
        }
            break;
        case (FM_DMA_STAT):
        {
            t_FmDmaStatus dmaStatus;

            FM_GetDmaStatus(h_Fm, &dmaStatus);
            ((t_FmIpcDmaStatus*)p_Reply)->boolCmqNotEmpty = (uint8_t)dmaStatus.cmqNotEmpty;
            ((t_FmIpcDmaStatus*)p_Reply)->boolBusError = (uint8_t)dmaStatus.busError;
            ((t_FmIpcDmaStatus*)p_Reply)->boolReadBufEccError = (uint8_t)dmaStatus.readBufEccError;
            ((t_FmIpcDmaStatus*)p_Reply)->boolWriteBufEccSysError = (uint8_t)dmaStatus.writeBufEccSysError;
            ((t_FmIpcDmaStatus*)p_Reply)->boolWriteBufEccFmError = (uint8_t)dmaStatus.writeBufEccFmError;
            *p_ReplyLength = sizeof(t_FmIpcDmaStatus);
        }
            break;
        case (FM_ALLOC_FMAN_CTRL_EVENT_REG):
            *p_ReplyLength = sizeof(uint8_t);
             return FmAllocFmanCtrlEventReg(h_Fm, (uint8_t*)p_Reply);
        case (FM_FREE_FMAN_CTRL_EVENT_REG):
            *p_ReplyLength = 0;
            FmFreeFmanCtrlEventReg(h_Fm, p_MsgBody[0]);
            break;
        case (FM_GET_TIMESTAMP_SCALE):
            *(uint32_t*)p_Reply = FmGetTimeStampScale(h_Fm);
            *p_ReplyLength = sizeof(uint32_t);
            break;
        case (FM_GET_COUNTER):
           *(uint32_t*)p_Reply = FM_GetCounter(h_Fm, (e_FmCounters)*(uint32_t*)p_MsgBody);
            *p_ReplyLength = sizeof(uint32_t);
            break;
        case (FM_SET_FMAN_CTRL_EVENTS_ENABLE):
            *p_ReplyLength = 0;
            FmSetFmanCtrlIntr(h_Fm, ((t_FmIpcFmanEvents*)p_MsgBody)->eventRegId, ((t_FmIpcFmanEvents*)p_MsgBody)->enableEvents);
            break;
        case (FM_GET_FMAN_CTRL_EVENTS_ENABLE):
            *(uint32_t*)p_Reply = FmGetFmanCtrlIntr(h_Fm, p_MsgBody[0]);
            *p_ReplyLength = sizeof(uint32_t);
            break;
        case (FM_GET_PHYS_MURAM_BASE):
        {
            t_FmPhysAddr physAddr;

            FmGetPhysicalMuramBase(h_Fm, &physAddr);
            ((t_FmIpcPhysAddr*)p_Reply)->high    = physAddr.high;
            ((t_FmIpcPhysAddr*)p_Reply)->low     = physAddr.low;
            *p_ReplyLength = sizeof(t_FmIpcPhysAddr);
            break;
        }
        case (FM_ENABLE_RAM_ECC):
            *p_ReplyLength = 0;
            FM_EnableRamsEcc(h_Fm);
            FM_SetException(h_Fm, e_FM_EX_IRAM_ECC, TRUE);
            FM_SetException(h_Fm, e_FM_EX_MURAM_ECC, TRUE);
            break;
        case (FM_DISABLE_RAM_ECC):
            *p_ReplyLength = 0;
            FM_SetException(h_Fm, e_FM_EX_IRAM_ECC, FALSE);
            FM_SetException(h_Fm, e_FM_EX_MURAM_ECC, FALSE);
            FM_DisableRamsEcc(h_Fm);
            break;
        case (FM_SET_NUM_OF_FMAN_CTRL):
            *p_ReplyLength = 0;
            return FmSetNumOfRiscsPerPort(h_Fm,
                                    ((t_FmIpcPortNumOfFmanCtrls*)p_MsgBody)->hardwarePortId,
                                    ((t_FmIpcPortNumOfFmanCtrls*)p_MsgBody)->numOfFmanCtrls);
        default:
            *p_ReplyLength = 0;
            RETURN_ERROR(MINOR, E_INVALID_SELECTION, ("command not found!!!"));
    }
    return E_OK;
}

static t_Error CheckSavedFmValidity(t_Fm *p_SavedFm)
{
    uint32_t    tmpReg, exceptions;

    if(p_SavedFm->h_Pcd)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, ("May not recover if PCD is used"));

    /* driver's initialization */
    if(p_SavedFm->p_FmFpmRegs != CAST_UINT64_TO_POINTER_TYPE(t_FmFpmRegs, (p_SavedFm->baseAddr + FM_MM_FPM)))
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (p_FmFpmRegs)"));
    if(p_SavedFm->p_FmBmiRegs != CAST_UINT64_TO_POINTER_TYPE(t_FmBmiRegs, (p_SavedFm->baseAddr + FM_MM_BMI)))
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (p_FmBmiRegs)"));
    if(p_SavedFm->p_FmQmiRegs != CAST_UINT64_TO_POINTER_TYPE(t_FmQmiRegs, (p_SavedFm->baseAddr + FM_MM_QMI)))
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (p_FmQmiRegs)"));
    if(p_SavedFm->p_FmDmaRegs != CAST_UINT64_TO_POINTER_TYPE(t_FmDmaRegs, (p_SavedFm->baseAddr + FM_MM_DMA)))
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (p_FmDmaRegs)"));

    /* Get from HW */
    tmpReg = GET_UINT32(p_SavedFm->p_FmFpmRegs->fpmtsc1);
    if(!!p_SavedFm->enabledTimeStamp != !!(tmpReg & FPM_TS_CTL_EN))
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (enabledTimeStamp)"));

    tmpReg = GET_UINT32(p_SavedFm->p_FmBmiRegs->fmbm_cfg1) >> BMI_CFG1_FIFO_SIZE_SHIFT;
    tmpReg = ((tmpReg & BMI_FIFO_SIZE_MASK) + 1)*BMI_FIFO_UNITS;
    if(p_SavedFm->totalFifoSize != tmpReg)
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (totalFifoSize)"));

    tmpReg = GET_UINT32(p_SavedFm->p_FmBmiRegs->fmbm_cfg2) >> BMI_CFG2_TASKS_SHIFT;
    tmpReg = (tmpReg & BMI_FIFO_SIZE_MASK) + 1;
    if(p_SavedFm->totalNumOfTasks != tmpReg)
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (totalNumOfTasks)"));


    /* collect all FM enabled exceptions */
    tmpReg = 0;
    exceptions = 0;
    tmpReg = GET_UINT32(p_SavedFm->p_FmDmaRegs->fmdmmr);
    if(tmpReg & DMA_MODE_BER)
        exceptions |= FM_EX_DMA_BUS_ERROR;
    if(tmpReg & DMA_MODE_ECC)
    {
        /* this bit enables one of 3 exeptions, check if either is set */
        if (p_SavedFm->exceptions & (FM_EX_DMA_SYSTEM_WRITE_ECC | FM_EX_DMA_READ_ECC | FM_EX_DMA_FM_WRITE_ECC))
            exceptions |= (p_SavedFm->exceptions & (FM_EX_DMA_SYSTEM_WRITE_ECC | FM_EX_DMA_READ_ECC | FM_EX_DMA_FM_WRITE_ECC));
        else
            RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (exceptions)"));
    }
    tmpReg = GET_UINT32(p_SavedFm->p_FmFpmRegs->fpmem);
    if(tmpReg & FPM_EV_MASK_STALL_EN)
        exceptions |= FM_EX_FPM_STALL_ON_TASKS;
    if(tmpReg & FPM_EV_MASK_SINGLE_ECC_EN)
        exceptions |= FM_EX_FPM_SINGLE_ECC;
    if(tmpReg & FPM_EV_MASK_DOUBLE_ECC_EN)
        exceptions |= FM_EX_FPM_DOUBLE_ECC;

    tmpReg = GET_UINT32(p_SavedFm->p_FmFpmRegs->fmrie);
    if(tmpReg & FPM_IRAM_ECC_ERR_EX_EN)
        exceptions |= FM_EX_IRAM_ECC;
    if(tmpReg & FPM_MURAM_ECC_ERR_EX_EN)
        exceptions |= FM_EX_NURAM_ECC;

    tmpReg = GET_UINT32(p_SavedFm->p_FmBmiRegs->fmbm_ier);
    if(tmpReg & BMI_ERR_INTR_EN_LIST_RAM_ECC)
        exceptions |= FM_EX_BMI_LIST_RAM_ECC;
    if(tmpReg & BMI_ERR_INTR_EN_PIPELINE_ECC)
        exceptions |= FM_EX_BMI_PIPELINE_ECC;
    if(tmpReg & BMI_ERR_INTR_EN_STATISTICS_RAM_ECC)
        exceptions |= FM_EX_BMI_STATISTICS_RAM_ECC;
    if(tmpReg & BMI_ERR_INTR_EN_DISPATCH_RAM_ECC)
        exceptions |= FM_EX_BMI_DISPATCH_RAM_ECC;

    tmpReg = GET_UINT32(p_SavedFm->p_FmQmiRegs->fmqm_eien);
    if(tmpReg & QMI_ERR_INTR_EN_DEQ_FROM_DEF)
        exceptions |= FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID;
    if(tmpReg & QMI_ERR_INTR_EN_DOUBLE_ECC)
        exceptions |= FM_EX_QMI_DOUBLE_ECC;

    tmpReg = GET_UINT32(p_SavedFm->p_FmQmiRegs->fmqm_ien);
    if(tmpReg & QMI_INTR_EN_SINGLE_ECC)
        exceptions |= FM_EX_QMI_SINGLE_ECC;

    if (p_SavedFm->exceptions != exceptions)
        RETURN_ERROR(MAJOR, E_CONFLICT, ("Conflict between saved FM structure and memory (exceptions)"));

    return E_OK;
}

#ifdef FM_10G_TX_ECC_FRMS_ERRATA_10GMAC_A004
t_Error Fm10GTxEccWorkaround(t_Handle h_Fm)
{
    t_Fm            *p_Fm = (t_Fm*)h_Fm;
    t_FMIramRegs    *p_Iram = CAST_UINT64_TO_POINTER_TYPE(t_FMIramRegs, (p_Fm->baseAddr + FM_MM_IMEM));
    int             timeout = 1000;

    WRITE_UINT32(p_Iram->iready, 0x80000000);//iram_ready
    XX_UDelay(5000);
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmextc, 0x40000000);
    while ((GET_UINT32(p_Fm->p_FmFpmRegs->fpmextc) & 0x40000000) &&
           --timeout) ;
    if (!timeout)
        return ERROR_CODE(E_TIMEOUT);
    return E_OK;
}
#endif /* FM_10G_TX_ECC_FRMS_ERRATA_10GMAC_A004 */

uint64_t FmGetPcdPrsBaseAddr(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, 0);

    if(p_Fm->guestId != NCSW_MASTER_ID)
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Guset"));

    return (p_Fm->baseAddr + FM_MM_PRS);
}

uint64_t FmGetPcdKgBaseAddr(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, 0);

    if(p_Fm->guestId != NCSW_MASTER_ID)
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Guset"));

    return (p_Fm->baseAddr + FM_MM_KG);
}

uint64_t FmGetPcdPlcrBaseAddr(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, 0);

    if(p_Fm->guestId != NCSW_MASTER_ID)
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("Guset"));

    return (p_Fm->baseAddr + FM_MM_PLCR);
}

t_Handle FmGetMuramHandle(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, NULL);

    return (p_Fm->h_FmMuram);
}

t_Error FmGetPhysicalMuramBase(t_Handle h_Fm, t_FmPhysAddr *p_FmPhysAddr)
{
    t_Fm            *p_Fm = (t_Fm*)h_Fm;
    t_FmIpcPhysAddr fmIpcPhysAddr;
    t_Error         err;
    uint32_t        replyLength;
    t_FmIpcMsg      msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_PHYS_MURAM_BASE;
        replyLength = sizeof(fmIpcPhysAddr);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     (uint8_t*)&fmIpcPhysAddr,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if(replyLength != sizeof(fmIpcPhysAddr))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        p_FmPhysAddr->low     = fmIpcPhysAddr.low;
        p_FmPhysAddr->high    = fmIpcPhysAddr.high;

        return E_OK;
    }

    /* General FM driver initialization */
    p_FmPhysAddr->low = (uint32_t)p_Fm->fmMuramPhysBaseAddr;
    p_FmPhysAddr->high = (uint8_t)((p_Fm->fmMuramPhysBaseAddr & 0x000000ff00000000LL) >> 32);

    return E_OK;
}

t_Error FmAllocFmanCtrlEventReg(t_Handle h_Fm, uint8_t *p_EventId)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint8_t     i;
    uint32_t    replyLength;
    t_Error     err;
    t_FmIpcMsg   msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_ALLOC_FMAN_CTRL_EVENT_REG;
        replyLength = sizeof(uint8_t);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     p_EventId,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if(replyLength != sizeof(uint8_t))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        return E_OK;
    }

    for(i=0;i<FM_NUM_OF_FMAN_CTRL_EVENT_REGS;i++)
        if (!p_Fm->usedEventRegs[i])
        {
            p_Fm->usedEventRegs[i] = TRUE;
            *p_EventId = i;
            break;
        }

    if (i==FM_NUM_OF_FMAN_CTRL_EVENT_REGS)
        RETURN_ERROR(MAJOR, E_BUSY, ("No resource - Fman controller event register."));

    return E_OK;
}

void FmFreeFmanCtrlEventReg(t_Handle h_Fm, uint8_t eventId)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    t_Error     err;
    t_FmIpcMsg  msg;

    if(((t_Fm *)h_Fm)->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_FREE_FMAN_CTRL_EVENT_REG;
        msg.msgBody[0] = eventId;
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(eventId),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            REPORT_ERROR(MINOR, err, NO_MSG);
        return;
    }

    ((t_Fm*)h_Fm)->usedEventRegs[eventId] = FALSE;
}

void FmRegisterIntr(t_Handle h_Fm,
                        e_FmEventModules        module,
                        uint8_t                 modId,
                        e_FmIntrType            intrType,
                        void (*f_Isr) (t_Handle h_Arg),
                        t_Handle    h_Arg)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    uint8_t             event= 0;
    t_FmIpcRegisterIntr fmIpcRegisterIntr;
    t_Error             err;
    t_FmIpcMsg          msg;

    GET_FM_MODULE_EVENT(module, modId,intrType, event);

    /* register in local FM structure */
    ASSERT_COND(event != e_FM_EV_DUMMY_LAST);
    p_Fm->intrMng[event].f_Isr = f_Isr;
    p_Fm->intrMng[event].h_SrcHandle = h_Arg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        /* register in Master FM structure */
        fmIpcRegisterIntr.event = event;
        fmIpcRegisterIntr.guestId = p_Fm->guestId;
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_REGISTER_INTR;
        memcpy(msg.msgBody, &fmIpcRegisterIntr, sizeof(fmIpcRegisterIntr));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) + sizeof(fmIpcRegisterIntr),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            REPORT_ERROR(MINOR, err, NO_MSG);
    }

}

void FmUnregisterIntr(t_Handle h_Fm,
                        e_FmEventModules        module,
                        uint8_t                 modId,
                        e_FmIntrType            intrType)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;
    uint8_t     event= 0;

    GET_FM_MODULE_EVENT(module, modId,intrType, event);

    ASSERT_COND(event != e_FM_EV_DUMMY_LAST);
    p_Fm->intrMng[event].f_Isr = UnimplementedIsr;
    p_Fm->intrMng[event].h_SrcHandle = NULL;
}

void FmSetFmanCtrlIntr(t_Handle h_Fm, uint8_t   eventRegId, uint32_t enableEvents)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    t_FmIpcFmanEvents   fmanCtrl;
    t_Error             err;
    t_FmIpcMsg          msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        fmanCtrl.eventRegId = eventRegId;
        fmanCtrl.enableEvents = enableEvents;
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_SET_FMAN_CTRL_EVENTS_ENABLE;
        memcpy(msg.msgBody, &fmanCtrl, sizeof(fmanCtrl));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(fmanCtrl),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            REPORT_ERROR(MINOR, err, NO_MSG);
        return;
    }

    ASSERT_COND(eventRegId < FM_NUM_OF_FMAN_CTRL_EVENT_REGS);
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[eventRegId], enableEvents);
}

uint32_t FmGetFmanCtrlIntr(t_Handle h_Fm, uint8_t   eventRegId)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    t_Error     err;
    uint32_t    events;
    t_FmIpcMsg  msg;
    uint32_t    replyLength;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_FMAN_CTRL_EVENTS_ENABLE;
        msg.msgBody[0] = eventRegId;
        replyLength = sizeof(events);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(eventRegId),
                                     (uint8_t*)&events,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if (replyLength != sizeof(events))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        return events;
    }

    return GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[eventRegId]);
}

void  FmRegisterFmanCtrlIntr(t_Handle h_Fm, uint8_t eventRegId, void (*f_Isr) (t_Handle h_Arg, uint32_t event), t_Handle    h_Arg)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;

    ASSERT_COND(eventRegId<FM_NUM_OF_FMAN_CTRL_EVENT_REGS);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        ASSERT_COND(0);
        /* TODO */
    }

    p_Fm->fmanCtrlIntr[eventRegId].f_Isr = f_Isr;
    p_Fm->fmanCtrlIntr[eventRegId].h_SrcHandle = h_Arg;
}

void  FmUnregisterFmanCtrlIntr(t_Handle h_Fm, uint8_t eventRegId)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;

    ASSERT_COND(eventRegId<FM_NUM_OF_FMAN_CTRL_EVENT_REGS);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        ASSERT_COND(0);
        /* TODO */
    }

    p_Fm->fmanCtrlIntr[eventRegId].f_Isr = UnimplementedFmanCtrlIsr;
    p_Fm->fmanCtrlIntr[eventRegId].h_SrcHandle = NULL;
}

void  FmRegisterPcd(t_Handle h_Fm, t_Handle h_FmPcd)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;

    if(p_Fm->h_Pcd)
        REPORT_ERROR(MAJOR, E_ALREADY_EXISTS, ("PCD already set"));

    p_Fm->h_Pcd = h_FmPcd;

}

void  FmUnregisterPcd(t_Handle h_Fm)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;

    if(!p_Fm->h_Pcd)
        REPORT_ERROR(MAJOR, E_ALREADY_EXISTS, ("No PCD"));

    p_Fm->h_Pcd = NULL;

}

t_Handle  FmGetPcdHandle(t_Handle h_Fm)
{
    t_Fm       *p_Fm = (t_Fm*)h_Fm;

    return p_Fm->h_Pcd;
}

uint8_t FmGetId(t_Handle h_Fm)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, 0xff);

    return p_Fm->fmId;
}

t_Error FmSetNumOfRiscsPerPort(t_Handle h_Fm, uint8_t hardwarePortId, uint8_t numOfFmanCtrls)
{

    t_Fm                        *p_Fm = (t_Fm*)h_Fm;
    uint32_t                    tmpReg = 0;
    t_FmIpcPortNumOfFmanCtrls   params;
    t_FmIpcMsg                  msg;
    t_Error                     err;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(((numOfFmanCtrls > 0) && (numOfFmanCtrls < 3)) , E_INVALID_HANDLE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        params.hardwarePortId = hardwarePortId;
        params.numOfFmanCtrls = numOfFmanCtrls;
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_SET_NUM_OF_FMAN_CTRL;
        memcpy(msg.msgBody, &params, sizeof(params));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) +sizeof(params),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);

        return E_OK;
    }

    XX_LockSpinlock(p_Fm->h_Spinlock);

    tmpReg = (uint32_t)(hardwarePortId << FPM_PORT_FM_CTL_PORTID_SHIFT);

    /*TODO - maybe to put CTL# according to another criteria*/

    if(numOfFmanCtrls == 2)
        tmpReg = FPM_PORT_FM_CTL2 | FPM_PORT_FM_CTL1;

    /* order restoration */
    if(hardwarePortId%2)
        tmpReg |= (FPM_PORT_FM_CTL1 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT) | FPM_PORT_FM_CTL1;
    else
        tmpReg |= (FPM_PORT_FM_CTL2 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT) | FPM_PORT_FM_CTL2;

    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmpr, tmpReg);
    XX_UnlockSpinlock(p_Fm->h_Spinlock);

    return E_OK;
}

t_Error FmGetSetPortParams(t_Handle h_Fm,t_FmInterModulePortInitParams *p_PortParams)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    uint32_t                tmpReg;
    uint8_t                 hardwarePortId = p_PortParams->hardwarePortId, relativePortId;
    uint32_t                minFifoSizeRequired = 0;
    uint16_t                macMaxFrameLength = 0;
    t_FmIpcPortInInitParams portInParams;
    t_FmIpcPhysAddr         fmMuramPhysBaseAddr;
    t_Error                 err;
    t_FmIpcMsg              msg;
    uint32_t                replyLength;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        portInParams.hardwarePortId = p_PortParams->hardwarePortId;
        portInParams.enumPortType = (uint32_t)p_PortParams->portType;
        portInParams.boolIndependentMode = (uint8_t)p_PortParams->independentMode;
        portInParams.liodnOffset = p_PortParams->liodnOffset;
        portInParams.numOfTasks = p_PortParams->numOfTasks;
        portInParams.numOfExtraTasks = p_PortParams->numOfExtraTasks;
        portInParams.numOfOpenDmas = p_PortParams->numOfOpenDmas;
        portInParams.numOfExtraOpenDmas = p_PortParams->numOfExtraOpenDmas;
        portInParams.sizeOfFifo = p_PortParams->sizeOfFifo;
        portInParams.extraSizeOfFifo = p_PortParams->extraSizeOfFifo;
        portInParams.deqPipelineDepth = p_PortParams->deqPipelineDepth;
        portInParams.liodnBase = p_PortParams->liodnBase;

        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_SET_PORT_PARAMS;
        memcpy(msg.msgBody, &portInParams, sizeof(portInParams));
        replyLength = sizeof(fmMuramPhysBaseAddr);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) +sizeof(portInParams),
                                     (uint8_t*)&fmMuramPhysBaseAddr,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if (replyLength != sizeof(fmMuramPhysBaseAddr))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        p_PortParams->fmMuramPhysBaseAddr.high = fmMuramPhysBaseAddr.high;
        p_PortParams->fmMuramPhysBaseAddr.low = fmMuramPhysBaseAddr.low;

        return E_OK;
    }

    XX_LockSpinlock(p_Fm->h_Spinlock);

    if(p_PortParams->independentMode)
    {
        /* set port parameters */
        p_Fm->independentMode = p_PortParams->independentMode;
        /* disable dispatch limit */
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmflc, 0);
    }

    if(p_PortParams->portType == e_FM_PORT_TYPE_OH_HOST_COMMAND)
    {
        if(p_Fm->hcPortInitialized)
        {
            XX_UnlockSpinlock(p_Fm->h_Spinlock);
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Only one host command port is allowed."));
        }
        else
            p_Fm->hcPortInitialized = TRUE;
    }
    p_Fm->portsTypes[hardwarePortId] = p_PortParams->portType;

    p_Fm->extraTasksPoolSize = (uint8_t)MAX(p_Fm->extraTasksPoolSize, p_PortParams->numOfExtraTasks);
    /* check that there are enough uncommitted tasks */
    if((p_Fm->accumulatedNumOfTasks + p_PortParams->numOfTasks) >
       (p_Fm->totalNumOfTasks - p_Fm->extraTasksPoolSize))
    {
        XX_UnlockSpinlock(p_Fm->h_Spinlock);
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Requested numOfTasks and extra tasks pool exceed total numOfTasks."));
    }
    else
    {
        p_Fm->accumulatedNumOfTasks += p_PortParams->numOfTasks;
        tmpReg = (uint32_t)(((p_PortParams->numOfTasks-1) << BMI_NUM_OF_TASKS_SHIFT) |
                    (p_PortParams->numOfExtraTasks << BMI_EXTRA_NUM_OF_TASKS_SHIFT));
        WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1],tmpReg);
    }
#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
    if((p_PortParams->portType != e_FM_PORT_TYPE_RX) && (p_PortParams->portType != e_FM_PORT_TYPE_RX_10G))
    /* for transmit & O/H ports */
    {
        uint8_t     enqTh;
        uint8_t     deqTh;
        bool        update = FALSE;

        /* update qmi ENQ/DEQ threshold */
        p_Fm->accumulatedNumOfDeqTnums += p_PortParams->deqPipelineDepth;
        tmpReg = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc);
        enqTh = (uint8_t)(tmpReg>>8);
        /* if enqTh is too big, we reduce it to the max value that is still OK */
        if(enqTh >= (QMI_MAX_NUM_OF_TNUMS - p_Fm->accumulatedNumOfDeqTnums))
        {
            enqTh = (uint8_t)(QMI_MAX_NUM_OF_TNUMS - p_Fm->accumulatedNumOfDeqTnums - 1);
            tmpReg &= ~QMI_CFG_ENQ_MASK;
            tmpReg |= ((uint32_t)enqTh << 8);
            update = TRUE;
        }

        deqTh = (uint8_t)tmpReg;
        /* if deqTh is too small, we enlarge it to the min value that is still OK.
         deqTh may not be larger than 63 (QMI_MAX_NUM_OF_TNUMS-1). */
        if((deqTh <= p_Fm->accumulatedNumOfDeqTnums)  && (deqTh < QMI_MAX_NUM_OF_TNUMS-1))
        {
            deqTh = (uint8_t)(p_Fm->accumulatedNumOfDeqTnums + 1);
            tmpReg &= ~QMI_CFG_DEQ_MASK;
            tmpReg |= (uint32_t)deqTh;
            update = TRUE;
        }
        if(update)
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc, tmpReg);
    }
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */
#ifdef FM_LOW_END_RESTRICTION
    if((hardwarePortId==0x1) || (hardwarePortId==0x29))
    {
        if(p_Fm->lowEndRestriction)
        {
            XX_UnlockSpinlock(p_Fm->h_Spinlock);
            RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("OP #0 cannot work with Tx Port #1."));
        }
        else
            p_Fm->lowEndRestriction = TRUE;
    }
#endif /* FM_LOW_END_RESTRICTION */

    p_Fm->extraOpenDmasPoolSize = (uint8_t)MAX(p_Fm->extraOpenDmasPoolSize, p_PortParams->numOfExtraOpenDmas);
    /* check that there are enough uncommited open DMA's */
    if((p_Fm->accumulatedNumOfOpenDmas + p_PortParams->numOfOpenDmas) >
       p_Fm->maxNumOfOpenDmas)
    {
        XX_UnlockSpinlock(p_Fm->h_Spinlock);
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Requested numOfOpenDmas exceeds total numOfOpenDmas."));
    }
    else
    {
        p_Fm->accumulatedNumOfOpenDmas += p_PortParams->numOfOpenDmas;
        tmpReg = (uint32_t)(((p_PortParams->numOfOpenDmas-1) << BMI_NUM_OF_DMAS_SHIFT) |
                    (p_PortParams->numOfExtraOpenDmas << BMI_EXTRA_NUM_OF_DMAS_SHIFT));
        WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1],
                GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1]) | tmpReg);
        /* update total num of DMA's with committed number of open DMAS, and max uncommitted pool. */
        tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg2) & ~BMI_CFG2_DMAS_MASK;
        tmpReg |= (uint32_t)(p_Fm->accumulatedNumOfOpenDmas + p_Fm->extraOpenDmasPoolSize - 1) << BMI_CFG2_DMAS_SHIFT;
        WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg2,  tmpReg);
    }

    /* if nessaccary, update sizeOfFifo field. */
    if((p_PortParams->portType == e_FM_PORT_TYPE_TX) || (p_PortParams->portType == e_FM_PORT_TYPE_RX))
    {
        HW_PORT_ID_TO_SW_PORT_ID(relativePortId, hardwarePortId);
        macMaxFrameLength = p_Fm->macMaxFrameLengths1G[relativePortId];
    }
#if (defined(FM_MAX_NUM_OF_10G_MACS) && (FM_MAX_NUM_OF_10G_MACS))
    if((p_PortParams->portType == e_FM_PORT_TYPE_TX_10G) || (p_PortParams->portType == e_FM_PORT_TYPE_RX_10G))
    {
        HW_PORT_ID_TO_SW_PORT_ID(relativePortId, hardwarePortId);
        macMaxFrameLength = p_Fm->macMaxFrameLengths10G[relativePortId];
    }
#endif /* (defined(FM_MAX_NUM_OF_10G_MACS) && ... */

    if((p_PortParams->portType == e_FM_PORT_TYPE_TX) || (p_PortParams->portType == e_FM_PORT_TYPE_TX_10G))
    {
        if(p_PortParams->independentMode)
            minFifoSizeRequired = (uint32_t)((macMaxFrameLength % BMI_FIFO_UNITS ?
                                (macMaxFrameLength/BMI_FIFO_UNITS + 1) * BMI_FIFO_UNITS :
                                macMaxFrameLength) +
                                (3*BMI_FIFO_UNITS));
        else
            minFifoSizeRequired = (uint32_t)((macMaxFrameLength % BMI_FIFO_UNITS ?
                                   (macMaxFrameLength/BMI_FIFO_UNITS + 1) * BMI_FIFO_UNITS :
                                   macMaxFrameLength) +
                                   (p_PortParams->deqPipelineDepth+3)*BMI_FIFO_UNITS);
    }
    else
            if(((p_PortParams->portType == e_FM_PORT_TYPE_RX) || (p_PortParams->portType == e_FM_PORT_TYPE_RX_10G))
                && p_PortParams->independentMode)
                minFifoSizeRequired = (uint32_t)(((macMaxFrameLength % BMI_FIFO_UNITS) ?
                                                 ((macMaxFrameLength/BMI_FIFO_UNITS + 1) * BMI_FIFO_UNITS) :
                                                 macMaxFrameLength) +
                                                 (4*BMI_FIFO_UNITS));
    if (minFifoSizeRequired && (p_PortParams->sizeOfFifo < minFifoSizeRequired))
    {
        p_PortParams->sizeOfFifo = minFifoSizeRequired;
        DBG(WARNING, ("FIFO size enlarged to %d", minFifoSizeRequired));
    }

    /* we leave FM_MAX_NUM_OF_RX_PORTS spare buffers for excessive buffers
       check that there are enough uncommitted FIFO buffs */
    p_Fm->extraFifoPoolSize = MAX(p_Fm->extraFifoPoolSize, p_PortParams->extraSizeOfFifo);
    if((p_Fm->accumulatedFifoSize + p_PortParams->sizeOfFifo) >
       (p_Fm->totalFifoSize - p_Fm->extraFifoPoolSize))
    {
        XX_UnlockSpinlock(p_Fm->h_Spinlock);
        RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("Requested fifo size and extra size exceed total FIFO size."));
    }
    else
    {
        p_Fm->accumulatedFifoSize += p_PortParams->sizeOfFifo;
        tmpReg = (uint32_t)((p_PortParams->sizeOfFifo/BMI_FIFO_UNITS - 1) | ((p_PortParams->extraSizeOfFifo/BMI_FIFO_UNITS) << BMI_FIFO_SIZE_SHIFT));
        WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_pfs[hardwarePortId-1], tmpReg);
    }
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ppid[hardwarePortId-1], (uint32_t)p_PortParams->liodnOffset);

    tmpReg = (uint32_t)(hardwarePortId << FPM_PORT_FM_CTL_PORTID_SHIFT);
    if(p_PortParams->independentMode)
    {
        if((p_PortParams->portType==e_FM_PORT_TYPE_RX) || (p_PortParams->portType==e_FM_PORT_TYPE_RX_10G))
            tmpReg |= (FPM_PORT_FM_CTL1 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT) |FPM_PORT_FM_CTL1;
        else
            tmpReg |= (FPM_PORT_FM_CTL2 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT) |FPM_PORT_FM_CTL2;
    }
    else
    {
        tmpReg |= (FPM_PORT_FM_CTL2|FPM_PORT_FM_CTL1);

        /* order restoration */
        if(hardwarePortId%2)
            tmpReg |= (FPM_PORT_FM_CTL1 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT);
        else
            tmpReg |= (FPM_PORT_FM_CTL2 << FPM_PRC_ORA_FM_CTL_SEL_SHIFT);
    }
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmpr, tmpReg);

    {
#ifdef FM_PARTITION_ARRAY
        t_FmRevisionInfo revInfo;
        FM_GetRevision(p_Fm, &revInfo);
        if ((revInfo.majorRev == 2) && (revInfo.minorRev == 0))
#endif /* FM_PARTITION_ARRAY */
        {
            /* set LIODN base for this port */
            tmpReg = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmplr[hardwarePortId/2]);
            if(hardwarePortId%2)
            {
                tmpReg &= ~FM_LIODN_BASE_MASK;
                tmpReg |= (uint32_t)p_PortParams->liodnBase;
            }
            else
            {
                tmpReg &= ~(FM_LIODN_BASE_MASK<< DMA_LIODN_SHIFT);
                tmpReg |= (uint32_t)p_PortParams->liodnBase << DMA_LIODN_SHIFT;
            }
            WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmplr[hardwarePortId/2], tmpReg);
        }
    }

    FmGetPhysicalMuramBase(p_Fm, &p_PortParams->fmMuramPhysBaseAddr);
    XX_UnlockSpinlock(p_Fm->h_Spinlock);

    return E_OK;
}

void FmFreePortParams(t_Handle h_Fm,t_FmInterModulePortFreeParams *p_PortParams)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    uint32_t                tmpReg;
    uint8_t                 hardwarePortId = p_PortParams->hardwarePortId;
    uint8_t                 numOfTasks;
    t_Error                 err;
    t_FmIpcPortFreeParams   portParams;
    t_FmIpcMsg              msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        portParams.hardwarePortId = p_PortParams->hardwarePortId;
        portParams.enumPortType = (uint32_t)p_PortParams->portType;
#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
        portParams.deqPipelineDepth = p_PortParams->deqPipelineDepth;
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_FREE_PORT;
        memcpy(msg.msgBody, &portParams, sizeof(portParams));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(portParams),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            REPORT_ERROR(MINOR, err, NO_MSG);
        return;
    }

    if(p_PortParams->portType == e_FM_PORT_TYPE_OH_HOST_COMMAND)
    {
        ASSERT_COND(p_Fm->hcPortInitialized);
        p_Fm->hcPortInitialized = FALSE;
    }

    p_Fm->portsTypes[hardwarePortId] = e_FM_PORT_TYPE_DUMMY;

    tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1]);
    /* free numOfTasks */
    numOfTasks = (uint8_t)(((tmpReg & BMI_NUM_OF_TASKS_MASK) >> BMI_NUM_OF_TASKS_SHIFT) + 1);
    ASSERT_COND(p_Fm->accumulatedNumOfTasks >= numOfTasks);
    p_Fm->accumulatedNumOfTasks -= numOfTasks;

    /* free numOfOpenDmas */
    ASSERT_COND(p_Fm->accumulatedNumOfOpenDmas >= ((tmpReg & BMI_NUM_OF_DMAS_MASK) >> BMI_NUM_OF_DMAS_SHIFT));
    p_Fm->accumulatedNumOfOpenDmas -= ((tmpReg & BMI_NUM_OF_DMAS_MASK) >> BMI_NUM_OF_DMAS_SHIFT);
    tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_pfs[hardwarePortId-1]);
    /* free sizeOfFifo */
    ASSERT_COND(p_Fm->accumulatedFifoSize >= ((tmpReg & BMI_FIFO_SIZE_MASK) + 1)*BMI_FIFO_UNITS);
    p_Fm->accumulatedFifoSize -= ((tmpReg & BMI_FIFO_SIZE_MASK) + 1)*BMI_FIFO_UNITS;

    /* clear registers */
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1], 0);
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_pfs[hardwarePortId-1], 0);
    //WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ppid[hardwarePortId-1], 0);

#ifdef FM_PORT_DISABLED_ERRATA_FMANx9
    /* this errata means that when a port is taken down, other port may not use its
     * resources for a while as it may still be using it (in case of reject).
     */
        {
            t_FmRevisionInfo revInfo;
            FM_GetRevision(p_Fm, &revInfo);
            if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
                XX_UDelay(100000);
        }
#endif /* FM_PORT_DISABLED_ERRATA_FMANx9 */

#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
    if((p_PortParams->portType != e_FM_PORT_TYPE_RX) && (p_PortParams->portType != e_FM_PORT_TYPE_RX_10G))
    /* for transmit & O/H ports */
    {
        uint8_t     enqTh;
        uint8_t     deqTh;

        tmpReg = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc);
        /* update qmi ENQ/DEQ threshold */
        p_Fm->accumulatedNumOfDeqTnums -= p_PortParams->deqPipelineDepth;

        /* p_Fm->accumulatedNumOfDeqTnums is now smaller,
           so we can enlarge enqTh */
        enqTh = (uint8_t)(QMI_MAX_NUM_OF_TNUMS - p_Fm->accumulatedNumOfDeqTnums - 1);
        tmpReg &= ~QMI_CFG_ENQ_MASK;
        tmpReg |= ((uint32_t)enqTh << QMI_CFG_ENQ_SHIFT);

         /* p_Fm->accumulatedNumOfDeqTnums is now smaller,
           so we can reduce deqTh */
        deqTh = (uint8_t)(p_Fm->accumulatedNumOfDeqTnums + 1);
        tmpReg &= ~QMI_CFG_DEQ_MASK;
        tmpReg |= (uint32_t)deqTh;

        WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc, tmpReg);
    }
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */

#ifdef FM_LOW_END_RESTRICTION
    if((hardwarePortId==0x1) || (hardwarePortId==0x29))
        p_Fm->lowEndRestriction = FALSE;
#endif /* FM_LOW_END_RESTRICTION */
}

t_Error FmIsPortStalled(t_Handle h_Fm, uint8_t hardwarePortId, bool *p_IsStalled)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    tmpReg;
    uint8_t     isStalled;
    t_Error     err;
    t_FmIpcMsg  msg;
    uint32_t    replyLength;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_IS_PORT_STALLED;
        msg.msgBody[0] = hardwarePortId;
        replyLength = sizeof(isStalled);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(hardwarePortId),
                                     (uint8_t*)&isStalled,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if (replyLength != sizeof(isStalled))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));
        *p_IsStalled = (bool)!!isStalled;

        return E_OK;
    }

    tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmfp_ps[hardwarePortId]);
    *p_IsStalled = (bool)!!(tmpReg & FPM_PS_STALLED);

    return E_OK;
}

t_Error FmResumeStalledPort(t_Handle h_Fm, uint8_t hardwarePortId)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    uint32_t                tmpReg;
    t_FmIpcMsg              msg;
    t_Error                 err;
    bool                    isStalled;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_RESUME_STALLED_PORT;
        msg.msgBody[0] = hardwarePortId;
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) + sizeof(hardwarePortId),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        return E_OK;
    }

    /* Get port status */
    err = FmIsPortStalled(h_Fm, hardwarePortId, &isStalled);
    if(err)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Can't get port status"));
    if (!isStalled)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Port is not stalled"));

    tmpReg = (uint32_t)((hardwarePortId << FPM_PORT_FM_CTL_PORTID_SHIFT) | FPM_PRC_REALSE_STALLED);
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmpr, tmpReg);

    return E_OK;
}

t_Error FmResetMac(t_Handle h_Fm, e_FmMacType type, uint8_t macId)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    uint32_t            bitMask;
    t_FmIpcMacParams    macParams;
    t_Error             err;
    t_FmIpcMsg          msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        macParams.id = macId;
        macParams.enumType = (uint32_t)type;
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_RESET_MAC;
        memcpy(msg.msgBody,  &macParams, sizeof(macParams));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(macParams),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        return E_OK;
    }

    /* Get the relevant bit mask */
    if (type == e_FM_MAC_10G)
    {
        switch(macId)
        {
            case(0):
                bitMask = FPM_RSTC_10G0_RESET;
                break;
            default:
                RETURN_ERROR(MINOR, E_INVALID_VALUE, ("Illegal MAC Id"));
        }
    }
    else
    {
        switch(macId)
        {
            case(0):
                bitMask = FPM_RSTC_1G0_RESET;
                break;
            case(1):
                bitMask = FPM_RSTC_1G1_RESET;
                break;
            case(2):
                bitMask = FPM_RSTC_1G2_RESET;
                break;
            case(3):
                bitMask = FPM_RSTC_1G3_RESET;
                break;
            default:
                RETURN_ERROR(MINOR, E_INVALID_VALUE, ("Illegal MAC Id"));
        }
    }

    /* reset */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrstc, bitMask);

    return E_OK;
}

t_Error FmSetMacMaxFrame(t_Handle h_Fm, e_FmMacType type, uint8_t macId, uint16_t mtu)
{
    t_Fm                        *p_Fm = (t_Fm*)h_Fm;
    t_FmIpcMacMaxFrameParams    macMaxFrameLengthParams;
    t_Error                     err;
    t_FmIpcMsg                  msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        macMaxFrameLengthParams.macParams.id = macId;
        macMaxFrameLengthParams.macParams.enumType = (uint32_t)type;
        macMaxFrameLengthParams.maxFrameLength = (uint16_t)mtu;
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_SET_MAC_MAX_FRAME;
        memcpy(msg.msgBody,  &macMaxFrameLengthParams, sizeof(macMaxFrameLengthParams));
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId)+sizeof(macMaxFrameLengthParams),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        return E_OK;
    }

#if (defined(FM_MAX_NUM_OF_10G_MACS) && (FM_MAX_NUM_OF_10G_MACS))
    if (type == e_FM_MAC_10G)
        p_Fm->macMaxFrameLengths10G[macId] = mtu;
    else
#else
    UNUSED(type);
#endif /* (defined(FM_MAX_NUM_OF_10G_MACS) && ... */
        p_Fm->macMaxFrameLengths1G[macId] = mtu;

    return E_OK;

}

uint16_t FmGetClockFreq(t_Handle h_Fm)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;
    /* for MC environment: this depends on the
     * fact that fmClkFreq was properly initialized at "init". */
    return p_Fm->fmClkFreq;
}

uint32_t    FmGetTimeStampScale(t_Handle h_Fm)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    uint32_t            timeStampScale;
    t_Error             err;
    uint32_t            replyLength;
    t_FmIpcMsg          msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_TIMESTAMP_SCALE;
        replyLength = sizeof(timeStampScale);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     (uint8_t*)&timeStampScale,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if(replyLength != sizeof(timeStampScale))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        return timeStampScale;
    }

    if(!p_Fm->enabledTimeStamp)
        FmEnableTimeStamp(p_Fm);

    return p_Fm->count1MicroBit;
}

bool FmRamsEccIsExternalCtl(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    tmpReg;

    tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmrcr);
    if(tmpReg & FPM_RAM_CTL_RAMS_ECC_EN_SRC_SEL)
        return TRUE;
    else
        return FALSE;
}

t_Error FmEnableRamsEcc(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    p_Fm->ramsEccOwners++;
    p_Fm->internalCall = TRUE;

    return FM_EnableRamsEcc(p_Fm);
}

t_Error FmDisableRamsEcc(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    ASSERT_COND(p_Fm->ramsEccOwners);
    p_Fm->ramsEccOwners--;

    if(p_Fm->ramsEccOwners==0)
    {
        p_Fm->internalCall = TRUE;
        return FM_DisableRamsEcc(p_Fm);
    }
    return E_OK;
}

uint8_t FmGetGuestId(t_Handle h_Fm)
{
    t_Fm     *p_Fm = (t_Fm*)h_Fm;

    return p_Fm->guestId;
}

bool FmIsMaster(t_Handle h_Fm)
{
    t_Fm     *p_Fm = (t_Fm*)h_Fm;

    return (p_Fm->guestId == NCSW_MASTER_ID);
}

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
t_Error FmDumpPortRegs (t_Handle h_Fm,uint8_t hardwarePortId)
{
    t_Fm        *p_Fm = (t_Fm *)h_Fm;
    t_FmIpcMsg  msg;
    DECLARE_DUMP;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_DUMP_PORT_REGS;
        msg.msgBody[0] = hardwarePortId;
        return XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                    (uint8_t*)&msg,
                                    sizeof(msg.msgId)+sizeof(hardwarePortId),
                                    NULL,
                                    NULL,
                                    NULL,
                                    NULL);
    }

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    DUMP_TITLE(&p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1], ("fmbm_pp for port %d", (hardwarePortId)));
    DUMP_MEMORY(&p_Fm->p_FmBmiRegs->fmbm_pp[hardwarePortId-1], sizeof(uint32_t));

    DUMP_TITLE(&p_Fm->p_FmBmiRegs->fmbm_pfs[hardwarePortId-1], ("fmbm_pfs for port %d", (hardwarePortId )));
    DUMP_MEMORY(&p_Fm->p_FmBmiRegs->fmbm_pfs[hardwarePortId-1], sizeof(uint32_t));

    DUMP_TITLE(&p_Fm->p_FmBmiRegs->fmbm_ppid[hardwarePortId-1], ("bm_ppid for port %d", (hardwarePortId)));
    DUMP_MEMORY(&p_Fm->p_FmBmiRegs->fmbm_ppid[hardwarePortId-1], sizeof(uint32_t));

    return E_OK;
}
#endif /* (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0)) */


/*****************************************************************************/
/*                      API Init unit functions                              */
/*****************************************************************************/
t_Handle FM_Config(t_FmParams *p_FmParam)
{
    t_Fm                *p_Fm;
    uint8_t             i;
    uint64_t            baseAddr = p_FmParam->baseAddr;

    SANITY_CHECK_RETURN_VALUE(p_FmParam, E_NULL_POINTER, NULL);
    SANITY_CHECK_RETURN_VALUE(((p_FmParam->firmware.p_Code && p_FmParam->firmware.size) ||
                               (!p_FmParam->firmware.p_Code && !p_FmParam->firmware.size)),
                              E_INVALID_VALUE, NULL);

    /* Allocate FM structure */
    p_Fm = (t_Fm *) XX_Malloc(sizeof(t_Fm));
    if (!p_Fm)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("FM driver structure"));
        return NULL;
    }
    memset(p_Fm, 0, sizeof(t_Fm));

    /* Initialize FM parameters which will be kept by the driver */
    p_Fm->fmId              = p_FmParam->fmId;
    p_Fm->guestId           = p_FmParam->guestId;

    for(i=0; i<FM_MAX_NUM_OF_HW_PORT_IDS; i++)
        p_Fm->portsTypes[i] = e_FM_PORT_TYPE_DUMMY;

    /* Allocate the FM driver's parameters structure */
    p_Fm->p_FmDriverParam = (t_FmDriverParam *)XX_Malloc(sizeof(t_FmDriverParam));
    if (!p_Fm->p_FmDriverParam)
    {
        XX_Free(p_Fm);
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("FM driver parameters"));
        return NULL;
    }
    memset(p_Fm->p_FmDriverParam, 0, sizeof(t_FmDriverParam));

    /* Initialize FM parameters which will be kept by the driver */
    p_Fm->fmId              = p_FmParam->fmId;
    p_Fm->h_FmMuram         = p_FmParam->h_FmMuram;
    p_Fm->h_App             = p_FmParam->h_App;
    p_Fm->fmClkFreq         = p_FmParam->fmClkFreq;
    p_Fm->f_Exception       = p_FmParam->f_Exception;
    p_Fm->f_BusError        = p_FmParam->f_BusError;
    p_Fm->p_FmFpmRegs       = CAST_UINT64_TO_POINTER_TYPE(t_FmFpmRegs, (baseAddr + FM_MM_FPM));
    p_Fm->p_FmBmiRegs       = CAST_UINT64_TO_POINTER_TYPE(t_FmBmiRegs, (baseAddr + FM_MM_BMI));
    p_Fm->p_FmQmiRegs       = CAST_UINT64_TO_POINTER_TYPE(t_FmQmiRegs, (baseAddr + FM_MM_QMI));
    p_Fm->p_FmDmaRegs       = CAST_UINT64_TO_POINTER_TYPE(t_FmDmaRegs, (baseAddr + FM_MM_DMA));
    p_Fm->baseAddr          = baseAddr;
    p_Fm->irq               = p_FmParam->irq;
    p_Fm->errIrq            = p_FmParam->errIrq;
    p_Fm->hcPortInitialized = FALSE;
    p_Fm->independentMode   = FALSE;
    p_Fm->ramsEccEnable     = FALSE;
    p_Fm->totalNumOfTasks   = DEFAULT_totalNumOfTasks;
    p_Fm->totalFifoSize     = DEFAULT_totalFifoSize;
    p_Fm->maxNumOfOpenDmas  = DEFAULT_maxNumOfOpenDmas;
    p_Fm->extraFifoPoolSize = FM_MAX_NUM_OF_RX_PORTS*BMI_FIFO_UNITS;

    p_Fm->exceptions        = DEFAULT_exceptions;

    p_Fm->h_Spinlock = XX_InitSpinlock();
    if (!p_Fm->h_Spinlock)
    {
        XX_Free(p_Fm->p_FmDriverParam);
        XX_Free(p_Fm);
        REPORT_ERROR(MAJOR, E_INVALID_STATE, ("cant allocate spinlock!"));
        return NULL;
    }

#ifdef FM_PARTITION_ARRAY
    /* Initialize FM driver parameters parameters (for initialization phase only) */
    memcpy(p_Fm->p_FmDriverParam->liodnBasePerPort, p_FmParam->liodnBasePerPort, FM_SIZE_OF_LIODN_TABLE*sizeof(uint16_t));
#endif /* FM_PARTITION_ARRAY */

    /*p_Fm->p_FmDriverParam->numOfPartitions                      = p_FmParam->numOfPartitions;    */
    p_Fm->p_FmDriverParam->enCounters                           = FALSE;

    p_Fm->p_FmDriverParam->resetOnInit                          = DEFAULT_resetOnInit;

    p_Fm->p_FmDriverParam->thresholds.dispLimit                 = DEFAULT_dispLimit;
    p_Fm->p_FmDriverParam->thresholds.prsDispTh                 = DEFAULT_prsDispTh;
    p_Fm->p_FmDriverParam->thresholds.plcrDispTh                = DEFAULT_plcrDispTh;
    p_Fm->p_FmDriverParam->thresholds.kgDispTh                  = DEFAULT_kgDispTh;
    p_Fm->p_FmDriverParam->thresholds.bmiDispTh                 = DEFAULT_bmiDispTh;
    p_Fm->p_FmDriverParam->thresholds.qmiEnqDispTh              = DEFAULT_qmiEnqDispTh;
    p_Fm->p_FmDriverParam->thresholds.qmiDeqDispTh              = DEFAULT_qmiDeqDispTh;
    p_Fm->p_FmDriverParam->thresholds.fmCtl1DispTh              = DEFAULT_fmCtl1DispTh;
    p_Fm->p_FmDriverParam->thresholds.fmCtl2DispTh              = DEFAULT_fmCtl2DispTh;

    p_Fm->p_FmDriverParam->dmaStopOnBusError                    = DEFAULT_dmaStopOnBusError;
    p_Fm->p_FmDriverParam->dmaBusProtect.privilegeBusProtect    = DEFAULT_privilegeBusProtect;
    p_Fm->p_FmDriverParam->dmaBusProtect.busProtectType         = DEFAULT_busProtectionType;

    p_Fm->p_FmDriverParam->dmaCacheOverride                     = DEFAULT_cacheOverride;
    p_Fm->p_FmDriverParam->dmaAidMode                           = DEFAULT_aidMode;
    p_Fm->p_FmDriverParam->dmaAidOverride                       = DEFAULT_aidOverride;
    p_Fm->p_FmDriverParam->dmaAxiDbgNumOfBeats                  = DEFAULT_axiDbgNumOfBeats;
    p_Fm->p_FmDriverParam->dmaCamNumOfEntries                   = DEFAULT_dmaCamNumOfEntries;
    p_Fm->p_FmDriverParam->dmaWatchdog                          = DEFAULT_dmaWatchdog;

    p_Fm->p_FmDriverParam->dmaCommQThresholds.clearEmergency               = DEFAULT_dmaCommQLow;
    p_Fm->p_FmDriverParam->dmaCommQThresholds.assertEmergency              = DEFAULT_dmaCommQHigh;
    p_Fm->p_FmDriverParam->dmaReadBufThresholds.clearEmergency             = DEFAULT_dmaReadIntBufLow;
    p_Fm->p_FmDriverParam->dmaReadBufThresholds.assertEmergency            = DEFAULT_dmaReadIntBufHigh;
    p_Fm->p_FmDriverParam->dmaWriteBufThresholds.clearEmergency            = DEFAULT_dmaWriteIntBufLow;
    p_Fm->p_FmDriverParam->dmaWriteBufThresholds.assertEmergency           = DEFAULT_dmaWriteIntBufHigh;
    p_Fm->p_FmDriverParam->dmaSosEmergency                                 = DEFAULT_dmaSosEmergency;

    p_Fm->p_FmDriverParam->dmaDbgCntMode                        = DEFAULT_dmaDbgCntMode;

    p_Fm->p_FmDriverParam->dmaEnEmergency                       = FALSE;
    p_Fm->p_FmDriverParam->dmaEnEmergencySmoother               = FALSE;
    p_Fm->p_FmDriverParam->catastrophicErr                      = DEFAULT_catastrophicErr;
    p_Fm->p_FmDriverParam->dmaErr                               = DEFAULT_dmaErr;
    p_Fm->p_FmDriverParam->haltOnExternalActivation             = DEFAULT_haltOnExternalActivation;
    p_Fm->p_FmDriverParam->haltOnUnrecoverableEccError          = DEFAULT_haltOnUnrecoverableEccError;
    p_Fm->p_FmDriverParam->enIramTestMode                       = FALSE;
    p_Fm->p_FmDriverParam->enMuramTestMode                      = FALSE;
    p_Fm->p_FmDriverParam->externalEccRamsEnable                = DEFAULT_externalEccRamsEnable;

    p_Fm->p_FmDriverParam->fwVerify                             = DEFAULT_VerifyUcode;
    p_Fm->p_FmDriverParam->firmware.size                        = p_FmParam->firmware.size;
    if (p_Fm->p_FmDriverParam->firmware.size)
    {
        p_Fm->p_FmDriverParam->firmware.p_Code = (uint32_t *)XX_Malloc(p_Fm->p_FmDriverParam->firmware.size);
        if (!p_Fm->p_FmDriverParam->firmware.p_Code)
        {
            XX_Free(p_Fm->p_FmDriverParam);
            XX_Free(p_Fm);
            REPORT_ERROR(MAJOR, E_NO_MEMORY, ("FM firmware code"));
            return NULL;
        }
        memcpy(p_Fm->p_FmDriverParam->firmware.p_Code ,p_FmParam->firmware.p_Code ,p_Fm->p_FmDriverParam->firmware.size);
    }

    return p_Fm;
}

/**************************************************************************//**
 @Function      FM_Init

 @Description   Initializes the FM module

 @Param[in]     h_Fm - FM module descriptor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error FM_Init(t_Handle h_Fm)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    t_FmDriverParam         *p_FmDriverParam = NULL;
    t_FmRevisionInfo        revInfo;
    t_Error                 err = E_OK;
    uint32_t                tmpReg, cfgReg = 0;
    int                     i;
    uint16_t                periodInFmClocks;
    uint8_t                 remainder;

    SANITY_CHECK_RETURN_ERROR(h_Fm, E_INVALID_HANDLE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        t_FmIpcMsg              msg;
        uint8_t                 isMasterAlive = 0;
        uint32_t                replyLength;

        /* build the FM guest partition IPC address */
        if(Sprint (p_Fm->fmModuleName, "FM_%d_%d",p_Fm->fmId, p_Fm->guestId) != (p_Fm->guestId<10 ? 6:7))
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));

        /* build the FM master partition IPC address */
        memset(p_Fm->fmIpcHandlerModuleName, 0, MODULE_NAME_SIZE);
        if(Sprint (p_Fm->fmIpcHandlerModuleName[0], "FM_%d_%d",p_Fm->fmId, NCSW_MASTER_ID) != 6)
            RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));

        for(i=0;i<e_FM_EV_DUMMY_LAST;i++)
            p_Fm->intrMng[i].f_Isr = UnimplementedIsr;

        p_Fm->h_IpcSessions[0] = XX_IpcInitSession(p_Fm->fmIpcHandlerModuleName[0], p_Fm->fmModuleName);
        if (p_Fm->h_IpcSessions[0] == NULL)
            RETURN_ERROR(MAJOR, E_NOT_AVAILABLE, ("FM Guest %d IPC session", p_Fm->guestId));

        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_MASTER_IS_ALIVE;
        msg.msgBody[0] = p_Fm->guestId;

        do
        {
            blockingFlag = TRUE;
            replyLength = sizeof(isMasterAlive);
            if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                         (uint8_t*)&msg,
                                         sizeof(msg.msgId)+sizeof(p_Fm->guestId),
                                         &isMasterAlive,
                                         &replyLength,
                                         IpcMsgCompletionCB,
                                         h_Fm)) != E_OK)
                REPORT_ERROR(MINOR, err, NO_MSG);
            while(blockingFlag) ;
        } while (!isMasterAlive);

        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_CLK_FREQ;
        replyLength = sizeof(p_Fm->fmClkFreq);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     (uint8_t*)&(p_Fm->fmClkFreq),
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if(replyLength != sizeof(p_Fm->fmClkFreq))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        err = XX_IpcRegisterMsgHandler(p_Fm->fmModuleName, FmGuestHandleIpcMsgCB, p_Fm, FM_IPC_MAX_REPLY_SIZE);
        if(err)
            RETURN_ERROR(MAJOR, err, NO_MSG);

        XX_Free(p_Fm->p_FmDriverParam);
        p_Fm->p_FmDriverParam = NULL;

        FM_DisableRamsEcc(p_Fm);
        FmMuramClear(p_Fm->h_FmMuram);
        FM_EnableRamsEcc(p_Fm);

        return E_OK;
    }

    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    FmMuramClear(p_Fm->h_FmMuram);

    FM_GetRevision(p_Fm, &revInfo);

    if (revInfo.majorRev < 4)
        p_Fm->exceptions &= ~FM_EX_BMI_DISPATCH_RAM_ECC;

#ifdef FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8
    if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
        p_Fm->exceptions  &= ~FM_EX_BMI_LIST_RAM_ECC;
#endif   /* FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8 */

#ifdef FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9
    if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
        p_Fm->exceptions  &= ~FM_EX_BMI_PIPELINE_ECC;
#endif /* FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9 */

    CHECK_INIT_PARAMETERS(p_Fm, CheckFmParameters);

    p_FmDriverParam = p_Fm->p_FmDriverParam;

    if(p_FmDriverParam->resetOnInit)
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrstc, FPM_RSTC_FM_RESET);

    /*************************************/
    /* Load FMan-Controller code to Iram */
    /*************************************/
    if (ClearIRam(p_Fm) != E_OK)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);
    if (p_Fm->p_FmDriverParam->firmware.p_Code &&
        (LoadFmanCtrlCode(p_Fm) != E_OK))
        RETURN_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);


    /* General FM driver initialization */
    p_Fm->fmMuramPhysBaseAddr = (uint64_t)(XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_Fm->baseAddr + FM_MM_MURAM)));
    for(i=0;i<e_FM_EV_DUMMY_LAST;i++)
        p_Fm->intrMng[i].f_Isr = UnimplementedIsr;
    for(i=0;i<FM_NUM_OF_FMAN_CTRL_EVENT_REGS;i++)
        p_Fm->fmanCtrlIntr[i].f_Isr = UnimplementedFmanCtrlIsr;

    /**********************/
    /* Init DMA Registers */
    /**********************/
    /* clear status reg events */
    tmpReg = (DMA_STATUS_BUS_ERR | DMA_STATUS_READ_ECC | DMA_STATUS_SYSTEM_WRITE_ECC | DMA_STATUS_FM_WRITE_ECC);
  //  tmpReg |= (DMA_STATUS_SYSTEM_DPEXT_ECC | DMA_STATUS_FM_DPEXT_ECC | DMA_STATUS_SYSTEM_DPDAT_ECC | DMA_STATUS_FM_DPDAT_ECC | DMA_STATUS_FM_SPDAT_ECC);
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmsr, GET_UINT32(p_Fm->p_FmDmaRegs->fmdmsr) | tmpReg);

    /* configure mode register */
    tmpReg = 0;
    tmpReg |= p_FmDriverParam->dmaCacheOverride << DMA_MODE_CACHE_OR_SHIFT;
    if(p_FmDriverParam->dmaAidOverride)
        tmpReg |= DMA_MODE_AID_OR;
    if (p_Fm->exceptions & FM_EX_DMA_BUS_ERROR)
        tmpReg |= DMA_MODE_BER;
    if ((p_Fm->exceptions & FM_EX_DMA_SYSTEM_WRITE_ECC) | (p_Fm->exceptions & FM_EX_DMA_READ_ECC) | (p_Fm->exceptions & FM_EX_DMA_FM_WRITE_ECC))
        tmpReg |= DMA_MODE_ECC;
    if(p_FmDriverParam->dmaStopOnBusError)
        tmpReg |= DMA_MODE_SBER;
    tmpReg |= (uint32_t)(p_FmDriverParam->dmaAxiDbgNumOfBeats - 1) << DMA_MODE_AXI_DBG_SHIFT;
    if (p_FmDriverParam->dmaEnEmergency)
    {
        tmpReg |= p_FmDriverParam->dmaEmergency.emergencyBusSelect;
        tmpReg |= p_FmDriverParam->dmaEmergency.emergencyLevel << DMA_MODE_EMERGENCY_LEVEL_SHIFT;
        if(p_FmDriverParam->dmaEnEmergencySmoother)
            WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmemsr, p_FmDriverParam->dmaEmergencySwitchCounter);
     }
    tmpReg |= ((p_FmDriverParam->dmaCamNumOfEntries/DMA_CAM_UNITS) - 1) << DMA_MODE_CEN_SHIFT;

    if(p_FmDriverParam->dmaBusProtect.privilegeBusProtect)
        tmpReg |= DMA_MODE_PRIVILEGE_PROT;
    tmpReg |= DMA_MODE_SECURE_PROT;
    tmpReg |= p_FmDriverParam->dmaBusProtect.busProtectType << DMA_MODE_BUS_PROT_SHIFT;
    tmpReg |= p_FmDriverParam->dmaDbgCntMode << DMA_MODE_DBG_SHIFT;
    tmpReg |= p_FmDriverParam->dmaAidMode << DMA_MODE_AID_MODE_SHIFT;

    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, tmpReg);

    /* configure thresholds register */
    tmpReg = ((uint32_t)p_FmDriverParam->dmaCommQThresholds.assertEmergency << DMA_THRESH_COMMQ_SHIFT) |
                ((uint32_t)p_FmDriverParam->dmaReadBufThresholds.assertEmergency << DMA_THRESH_READ_INT_BUF_SHIFT) |
                ((uint32_t)p_FmDriverParam->dmaWriteBufThresholds.assertEmergency);
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmtr, tmpReg);

    /* configure hysteresis register */
    tmpReg = ((uint32_t)p_FmDriverParam->dmaCommQThresholds.clearEmergency << DMA_THRESH_COMMQ_SHIFT) |
                ((uint32_t)p_FmDriverParam->dmaReadBufThresholds.clearEmergency << DMA_THRESH_READ_INT_BUF_SHIFT) |
                ((uint32_t)p_FmDriverParam->dmaWriteBufThresholds.clearEmergency);
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmhy, tmpReg);

    /* configure emergency threshold */
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmsetr, p_FmDriverParam->dmaSosEmergency);

    /* configure Watchdog */
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmwcr, USEC_TO_CLK(p_FmDriverParam->dmaWatchdog, p_Fm->fmClkFreq));

    /* Allocate MURAM for CAM */
    p_Fm->camBaseAddr = CAST_POINTER_TO_UINT64(FM_MURAM_AllocMem(p_Fm->h_FmMuram,
                                                                 (uint32_t)(p_FmDriverParam->dmaCamNumOfEntries*DMA_CAM_SIZEOF_ENTRY),
                                                                 DMA_CAM_ALIGN));
    if (!p_Fm->camBaseAddr )
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("MURAM alloc for DMA CAM failed"));

    WRITE_BLOCK(CAST_UINT64_TO_POINTER_TYPE(uint8_t, p_Fm->camBaseAddr), 0, (uint32_t)(p_FmDriverParam->dmaCamNumOfEntries*DMA_CAM_SIZEOF_ENTRY));

    /* VirtToPhys */
    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmebcr,
                 (uint32_t)(XX_VirtToPhys(CAST_UINT64_TO_POINTER(p_Fm->camBaseAddr)) -
                            p_Fm->fmMuramPhysBaseAddr));

#ifdef FM_PARTITION_ARRAY
    {
        t_FmRevisionInfo revInfo;
        FM_GetRevision(p_Fm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            /* liodn-partitions */
            for (i=0 ; i<FM_SIZE_OF_LIODN_TABLE ; i+=2)
            {
                tmpReg = (((uint32_t)p_FmDriverParam->liodnBasePerPort[i] << DMA_LIODN_SHIFT) |
                            (uint32_t)p_FmDriverParam->liodnBasePerPort[i+1]);
                WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmplr[i/2], tmpReg);
            }
    }
#endif /* FM_PARTITION_ARRAY */

    /**********************/
    /* Init FPM Registers */
    /**********************/
    tmpReg = (uint32_t)(p_FmDriverParam->thresholds.dispLimit << FPM_DISP_LIMIT_SHIFT);
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmflc, tmpReg);

    tmpReg =   (((uint32_t)p_FmDriverParam->thresholds.prsDispTh  << FPM_THR1_PRS_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.kgDispTh  << FPM_THR1_KG_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.plcrDispTh  << FPM_THR1_PLCR_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.bmiDispTh  << FPM_THR1_BMI_SHIFT));
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmdis1, tmpReg);

    tmpReg =   (((uint32_t)p_FmDriverParam->thresholds.qmiEnqDispTh  << FPM_THR2_QMI_ENQ_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.qmiDeqDispTh  << FPM_THR2_QMI_DEQ_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.fmCtl1DispTh  << FPM_THR2_FM_CTL1_SHIFT) |
                ((uint32_t)p_FmDriverParam->thresholds.fmCtl2DispTh  << FPM_THR2_FM_CTL2_SHIFT));
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmdis2, tmpReg);

    /* define exceptions and error behavior */
    tmpReg = 0;
    /* Clear events */
    tmpReg |= (FPM_EV_MASK_STALL | FPM_EV_MASK_DOUBLE_ECC | FPM_EV_MASK_SINGLE_ECC);
    /* enable interrupts */
    if(p_Fm->exceptions & FM_EX_FPM_STALL_ON_TASKS)
        tmpReg |= FPM_EV_MASK_STALL_EN;
    if(p_Fm->exceptions & FM_EX_FPM_SINGLE_ECC)
        tmpReg |= FPM_EV_MASK_SINGLE_ECC_EN;
    if(p_Fm->exceptions & FM_EX_FPM_DOUBLE_ECC)
        tmpReg |= FPM_EV_MASK_DOUBLE_ECC_EN;
    tmpReg |= (p_Fm->p_FmDriverParam->catastrophicErr  << FPM_EV_MASK_CAT_ERR_SHIFT);
    tmpReg |= (p_Fm->p_FmDriverParam->dmaErr << FPM_EV_MASK_DMA_ERR_SHIFT);
    if(!p_Fm->p_FmDriverParam->haltOnExternalActivation)
        tmpReg |= FPM_EV_MASK_EXTERNAL_HALT;
    if(!p_Fm->p_FmDriverParam->haltOnUnrecoverableEccError)
        tmpReg |= FPM_EV_MASK_ECC_ERR_HALT;
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, tmpReg);

    /* clear all fmCtls event registers */
    for(i=0;i<FM_NUM_OF_FMAN_CTRL_EVENT_REGS;i++)
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmcev[i], 0xFFFFFFFF);

    /* RAM ECC -  enable and clear events*/
    /* first we need to clear all parser memory, as it is uninitialized and
    may cause ECC errors */
    tmpReg = 0;
    /* event bits */
    tmpReg = (FPM_RAM_CTL_MURAM_ECC | FPM_RAM_CTL_IRAM_ECC);
    /* Rams enable is not effected by the RCR bit, but by a COP configuration */
    if(p_Fm->p_FmDriverParam->externalEccRamsEnable)
        tmpReg |= FPM_RAM_CTL_RAMS_ECC_EN_SRC_SEL;

    /* enable test mode */
    if(p_FmDriverParam->enMuramTestMode)
        tmpReg |= FPM_RAM_CTL_MURAM_TEST_ECC;
    if(p_FmDriverParam->enIramTestMode)
        tmpReg |= FPM_RAM_CTL_IRAM_TEST_ECC;
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, tmpReg);

    tmpReg = 0;
    if(p_Fm->exceptions & FM_EX_IRAM_ECC)
    {
        tmpReg |= FPM_IRAM_ECC_ERR_EX_EN;
        FmEnableRamsEcc(p_Fm);
    }
    if(p_Fm->exceptions & FM_EX_NURAM_ECC)
    {
        tmpReg |= FPM_MURAM_ECC_ERR_EX_EN;
        FmEnableRamsEcc(p_Fm);
    }
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrie, tmpReg);

    /**********************/
    /* Init BMI Registers */
    /**********************/

    /* define common resources */
    /* allocate MURAM for FIFO according to total size */
    p_Fm->fifoBaseAddr = CAST_POINTER_TO_UINT64(FM_MURAM_AllocMem(p_Fm->h_FmMuram,
                                                                  p_Fm->totalFifoSize,
                                                                  BMI_FIFO_ALIGN));
    if (!p_Fm->fifoBaseAddr)
    {
        FreeInitResources(p_Fm);
        RETURN_ERROR(MAJOR, E_NO_MEMORY, ("MURAM alloc for FIFO failed"));
    }

    tmpReg = (uint32_t)(XX_VirtToPhys(CAST_UINT32_TO_POINTER(p_Fm->fifoBaseAddr)) - p_Fm->fmMuramPhysBaseAddr);
    tmpReg = tmpReg / BMI_FIFO_ALIGN;

    tmpReg |= ((p_Fm->totalFifoSize/BMI_FIFO_UNITS - 1) << BMI_CFG1_FIFO_SIZE_SHIFT);
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg1, tmpReg);

    tmpReg =  ((uint32_t)(p_Fm->totalNumOfTasks - 1) << BMI_CFG2_TASKS_SHIFT );
    /* num of DMA's will be dynamically updated when each port is set */
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg2, tmpReg);

    /* define unmaskable exceptions, enable and clear events */
    tmpReg = 0;
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ievr, (BMI_ERR_INTR_EN_LIST_RAM_ECC |
                                                BMI_ERR_INTR_EN_PIPELINE_ECC |
                                                BMI_ERR_INTR_EN_STATISTICS_RAM_ECC |
                                                BMI_ERR_INTR_EN_DISPATCH_RAM_ECC));
    if(p_Fm->exceptions & FM_EX_BMI_LIST_RAM_ECC)
        tmpReg |= BMI_ERR_INTR_EN_LIST_RAM_ECC;
    if(p_Fm->exceptions & FM_EX_BMI_PIPELINE_ECC)
        tmpReg |= BMI_ERR_INTR_EN_PIPELINE_ECC;
    if(p_Fm->exceptions & FM_EX_BMI_STATISTICS_RAM_ECC)
        tmpReg |= BMI_ERR_INTR_EN_STATISTICS_RAM_ECC;
    if(p_Fm->exceptions & FM_EX_BMI_DISPATCH_RAM_ECC)
        tmpReg |= BMI_ERR_INTR_EN_DISPATCH_RAM_ECC;
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier, tmpReg);

    /**********************/
    /* Init QMI Registers */
    /**********************/
     /* Clear error interrupt events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eie, (QMI_ERR_INTR_EN_DOUBLE_ECC | QMI_ERR_INTR_EN_DEQ_FROM_DEF));
    tmpReg = 0;
    if(p_Fm->exceptions & FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID)
        tmpReg |= QMI_ERR_INTR_EN_DEQ_FROM_DEF;
    if(p_Fm->exceptions & FM_EX_QMI_DOUBLE_ECC)
        tmpReg |= QMI_ERR_INTR_EN_DOUBLE_ECC;
    /* enable events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien, tmpReg);

    if(p_Fm->p_FmDriverParam->tnumAgingPeriod)
    {
        /* tnumAgingPeriod is in units of microseconds, p_FmClockFreq is in Mhz */
        periodInFmClocks = (uint16_t)(p_Fm->p_FmDriverParam->tnumAgingPeriod*p_Fm->fmClkFreq);
        /* periodInFmClocks must be a 64 multiply */
        remainder = (uint8_t)(periodInFmClocks % 64);
        if (remainder > 64)
            tmpReg = (uint32_t)((periodInFmClocks/64) + 1);
        else
        {
            tmpReg = (uint32_t)(periodInFmClocks/64);
            if(!tmpReg)
                tmpReg = 1;
        }
        tmpReg <<= QMI_TAPC_TAP;
        WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_tapc, tmpReg);

    }
    tmpReg = 0;
    /* Clear interrupt events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_ie, QMI_INTR_EN_SINGLE_ECC);
    if(p_Fm->exceptions & FM_EX_QMI_SINGLE_ECC)
        tmpReg |= QMI_INTR_EN_SINGLE_ECC;
    /* enable events */
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_ien, tmpReg);

    /* clear & enable global counters  - calculate reg and save for later,
       because it's the same reg for QMI enable */
    if(p_Fm->p_FmDriverParam->enCounters)
        cfgReg = QMI_CFG_EN_COUNTERS;
#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
    cfgReg |= (uint32_t)(((QMI_DEF_TNUMS_THRESH) << 8) |  (uint32_t)QMI_DEF_TNUMS_THRESH);
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */

    if (p_Fm->irq != NO_IRQ)
    {
        XX_SetIntr(p_Fm->irq, FM_EventIsr, p_Fm);
        XX_EnableIntr(p_Fm->irq);
    }

    if (p_Fm->errIrq != NO_IRQ)
    {
        XX_SetIntr(p_Fm->errIrq, FM_ErrorIsr, p_Fm);
        XX_EnableIntr(p_Fm->errIrq);
    }

    /* build the FM master partition IPC address */
    if (Sprint (p_Fm->fmModuleName, "FM_%d_%d",p_Fm->fmId, NCSW_MASTER_ID) != 6)
        RETURN_ERROR(MAJOR, E_INVALID_STATE, ("Sprint failed"));

    err = XX_IpcRegisterMsgHandler(p_Fm->fmModuleName, FmHandleIpcMsgCB, p_Fm, FM_IPC_MAX_REPLY_SIZE);
    if(err)
        RETURN_ERROR(MAJOR, err, NO_MSG);

    /**********************/
    /* Enable all modules */
    /**********************/
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_init, BMI_INIT_START);
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc, cfgReg | QMI_CFG_ENQ_EN | QMI_CFG_DEQ_EN);

    if (p_Fm->p_FmDriverParam->firmware.p_Code)
    {
        XX_Free(p_Fm->p_FmDriverParam->firmware.p_Code);
        p_Fm->p_FmDriverParam->firmware.p_Code = NULL;
    }

    XX_Free(p_Fm->p_FmDriverParam);
    p_Fm->p_FmDriverParam = NULL;

    return E_OK;
}

/**************************************************************************//**
 @Function      FM_Free

 @Description   Frees all resources that were assigned to FM module.

                Calling this routine invalidates the descriptor.

 @Param[in]     h_Fm - FM module descriptor

 @Return        E_OK on success; Error code otherwise.
*//***************************************************************************/
t_Error FM_Free(t_Handle h_Fm)
{
   t_Fm        *p_Fm = (t_Fm*)h_Fm;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

        XX_IpcUnregisterMsgHandler(p_Fm->fmModuleName);

        if(p_Fm->recoveryMode)
            XX_FreeSmart(p_Fm);
        else
            XX_Free(p_Fm);

        return E_OK;
    }

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    /* disable BMI and QMI */
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_init, 0);
    WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc, 0);

    /* release BMI resources */
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg2, 0);
    WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_cfg1, 0);

    /* disable ECC */
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, 0);

    if ((p_Fm->guestId == NCSW_MASTER_ID) &&
        (p_Fm->fmModuleName[0] != 0))
        XX_IpcUnregisterMsgHandler(p_Fm->fmModuleName);

    if (p_Fm->irq != NO_IRQ)
    {
        XX_DisableIntr(p_Fm->irq);
        XX_FreeIntr(p_Fm->irq);
    }

    if (p_Fm->errIrq != NO_IRQ)
    {
        XX_DisableIntr(p_Fm->errIrq);
        XX_FreeIntr(p_Fm->errIrq);
    }

    if (p_Fm->h_Spinlock)
        XX_FreeSpinlock(p_Fm->h_Spinlock);

    if(p_Fm->p_FmDriverParam)
    {
        XX_Free(p_Fm->p_FmDriverParam);
        p_Fm->p_FmDriverParam = NULL;
    }

    FreeInitResources(p_Fm);

    if(p_Fm->recoveryMode)
        XX_FreeSmart(p_Fm);
    else
        XX_Free(p_Fm);

    return E_OK;
}

/*************************************************/
/*       API Advanced Init unit functions        */
/*************************************************/

t_Error FM_ConfigResetOnInit(t_Handle h_Fm, bool enable)
{

    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->resetOnInit = enable;

    return E_OK;
}


t_Error FM_ConfigTotalNumOfTasks(t_Handle h_Fm, uint8_t totalNumOfTasks)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->totalNumOfTasks = totalNumOfTasks;

    return E_OK;
}

t_Error FM_ConfigTotalFifoSize(t_Handle h_Fm, uint32_t totalFifoSize)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->totalFifoSize = totalFifoSize;

    return E_OK;
}

t_Error FM_ConfigMaxNumOfOpenDmas(t_Handle h_Fm, uint8_t maxNumOfOpenDmas)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->maxNumOfOpenDmas = maxNumOfOpenDmas;

    return E_OK;
}

t_Error FM_ConfigThresholds(t_Handle h_Fm, t_FmThresholds *p_FmThresholds)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    memcpy(&p_Fm->p_FmDriverParam->thresholds, p_FmThresholds, sizeof(t_FmThresholds));

    return E_OK;
}

t_Error FM_ConfigDmaBusProtect(t_Handle h_Fm, t_FmDmaBusProtect *p_FmDmaBusProtect)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    memcpy(&p_Fm->p_FmDriverParam->dmaBusProtect, p_FmDmaBusProtect, sizeof(t_FmDmaBusProtect));

    return E_OK;
}

t_Error FM_ConfigDmaCacheOverride(t_Handle h_Fm, e_FmDmaCacheOverride cacheOverride)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaCacheOverride = cacheOverride;

    return E_OK;
}

t_Error FM_ConfigDmaAidOverride(t_Handle h_Fm, bool aidOverride)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaAidOverride = aidOverride;

    return E_OK;
}

t_Error FM_ConfigDmaAidMode(t_Handle h_Fm, e_FmDmaAidMode aidMode)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaAidMode = aidMode;

    return E_OK;
}

t_Error FM_ConfigDmaAxiDbgNumOfBeats(t_Handle h_Fm, uint8_t axiDbgNumOfBeats)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaAxiDbgNumOfBeats = axiDbgNumOfBeats;

    return E_OK;
}

t_Error FM_ConfigDmaCamNumOfEntries(t_Handle h_Fm, uint8_t numOfEntries)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaCamNumOfEntries = numOfEntries;

    return E_OK;
}

t_Error FM_ConfigDmaWatchdog(t_Handle h_Fm, uint32_t watchdogValue)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    t_FmRevisionInfo    revInfo;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    FM_GetRevision(h_Fm, &revInfo);
    if (revInfo.majorRev < 4)
        RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("watchdog!"));

    p_Fm->p_FmDriverParam->dmaWatchdog = watchdogValue;

    return E_OK;
}

t_Error FM_ConfigDmaWriteBufThresholds(t_Handle h_Fm, t_FmDmaThresholds *p_FmDmaThresholds)

{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    memcpy(&p_Fm->p_FmDriverParam->dmaWriteBufThresholds, p_FmDmaThresholds, sizeof(t_FmDmaThresholds));

    return E_OK;
}

t_Error FM_ConfigDmaCommQThresholds(t_Handle h_Fm, t_FmDmaThresholds *p_FmDmaThresholds)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    memcpy(&p_Fm->p_FmDriverParam->dmaCommQThresholds, p_FmDmaThresholds, sizeof(t_FmDmaThresholds));

    return E_OK;
}

t_Error FM_ConfigDmaReadBufThresholds(t_Handle h_Fm, t_FmDmaThresholds *p_FmDmaThresholds)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    memcpy(&p_Fm->p_FmDriverParam->dmaReadBufThresholds, p_FmDmaThresholds, sizeof(t_FmDmaThresholds));

    return E_OK;
}

t_Error FM_ConfigDmaEmergency(t_Handle h_Fm, t_FmDmaEmergency *p_Emergency)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaEnEmergency = TRUE;
    memcpy(&p_Fm->p_FmDriverParam->dmaEmergency, p_Emergency, sizeof(t_FmDmaEmergency));

    return E_OK;
}

t_Error FM_ConfigDmaEmergencySmoother(t_Handle h_Fm, uint32_t emergencyCnt)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    if(!p_Fm->p_FmDriverParam->dmaEnEmergency)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("FM_ConfigEnDmaEmergencySmoother may be called only after FM_ConfigEnDmaEmergency"));

    p_Fm->p_FmDriverParam->dmaEnEmergencySmoother = TRUE;
    p_Fm->p_FmDriverParam->dmaEmergencySwitchCounter = emergencyCnt;

    return E_OK;
}

t_Error FM_ConfigDmaDbgCounter(t_Handle h_Fm, e_FmDmaDbgCntMode fmDmaDbgCntMode)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaDbgCntMode = fmDmaDbgCntMode;

    return E_OK;
}

t_Error FM_ConfigDmaStopOnBusErr(t_Handle h_Fm, bool stop)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaStopOnBusError = stop;

    return E_OK;
}

t_Error FM_ConfigDmaSosEmergencyThreshold(t_Handle h_Fm, uint32_t dmaSosEmergency)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaSosEmergency = dmaSosEmergency;

    return E_OK;
}

t_Error FM_ConfigEnableCounters(t_Handle h_Fm)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->enCounters = TRUE;

    return E_OK;
}

t_Error FM_ConfigDmaErr(t_Handle h_Fm, e_FmDmaErr dmaErr)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->dmaErr = dmaErr;

    return E_OK;
}

t_Error FM_ConfigCatastrophicErr(t_Handle h_Fm, e_FmCatastrophicErr catastrophicErr)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->catastrophicErr = catastrophicErr;

    return E_OK;
}

t_Error FM_ConfigEnableMuramTestMode(t_Handle h_Fm)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->enMuramTestMode = TRUE;

    return E_OK;
}

t_Error FM_ConfigEnableIramTestMode(t_Handle h_Fm)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->enIramTestMode = TRUE;

    return E_OK;
}

t_Error FM_ConfigHaltOnExternalActivation(t_Handle h_Fm, bool enable)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

#ifdef FM_HALT_SIG_ERRATA_GEN12
    {
        t_FmRevisionInfo revInfo;
        FM_GetRevision(h_Fm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("HaltOnExternalActivation!"));
    }
#endif /* FM_HALT_SIG_ERRATA_GEN12 */

    p_Fm->p_FmDriverParam->haltOnExternalActivation = enable;

    return E_OK;
}

t_Error FM_ConfigHaltOnUnrecoverableEccError(t_Handle h_Fm, bool enable)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

#ifdef FM_ECC_HALT_NO_SYNC_TKT028214
    {
        t_FmRevisionInfo revInfo;
        FM_GetRevision(h_Fm, &revInfo);
        if ((revInfo.majorRev == 2) && (revInfo.minorRev == 0))
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("HaltOnEccError!"));
    }
#endif /* FM_ECC_HALT_NO_SYNC_TKT028214 */

    p_Fm->p_FmDriverParam->haltOnUnrecoverableEccError = enable;

    return E_OK;
}

t_Error FM_ConfigException(t_Handle h_Fm, e_FmExceptions exception, bool enable)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    uint32_t            bitMask = 0;
    t_FmRevisionInfo    revInfo;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

#ifdef FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9
    if((exception == e_FM_EX_BMI_PIPELINE_ECC) && (enable))
    {
        FM_GetRevision(p_Fm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_PIPELINE_ECC!"));
    }
#endif /* FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9 */
#ifdef FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8
    if((exception == e_FM_EX_BMI_LIST_RAM_ECC) && (enable))
    {
        FM_GetRevision(p_Fm, &revInfo);
        if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_LIST_RAM_ECC!"));
    }
#endif   /* FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8 */
    if((exception == e_FM_EX_BMI_DISPATCH_RAM_ECC) && (enable))
    {
        FM_GetRevision(p_Fm, &revInfo);
        if (revInfo.majorRev > 3)
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_DISPATCH_RAM_ECC!"));
    }

    GET_EXCEPTION_FLAG(bitMask, exception);
    if(bitMask)
    {
        if (enable)
            p_Fm->exceptions |= bitMask;
        else
            p_Fm->exceptions &= ~bitMask;
   }
    else
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Undefined exception"));

    return E_OK;
}

t_Error FM_ConfigExternalEccRamsEnable(t_Handle h_Fm, bool enable)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    p_Fm->p_FmDriverParam->externalEccRamsEnable = enable;

    return E_OK;
}

t_Error FM_ConfigTnumAgingPeriod(t_Handle h_Fm, uint16_t tnumAgingPeriod)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    t_FmRevisionInfo revInfo;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);


    FM_GetRevision(h_Fm, &revInfo);
    if (revInfo.majorRev < 3)
        RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("FM_ConfigTnumAgingPeriod!"));

    p_Fm->p_FmDriverParam->tnumAgingPeriod = tnumAgingPeriod;

    return E_OK;

}

/****************************************************/
/*       API Run-time Control uint functions        */
/****************************************************/
t_Handle FM_GetPcdHandle(t_Handle h_Fm)
{
    SANITY_CHECK_RETURN_VALUE(h_Fm, E_INVALID_HANDLE, NULL);
    SANITY_CHECK_RETURN_VALUE(!((t_Fm*)h_Fm)->p_FmDriverParam, E_INVALID_STATE, NULL);

    return ((t_Fm*)h_Fm)->h_Pcd;
}

void FM_EventIsr(t_Handle h_Fm)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    uint32_t                pending, event;

    SANITY_CHECK_RETURN(h_Fm, E_INVALID_HANDLE);

    /* normal interrupts */
    pending = GET_UINT32(p_Fm->p_FmFpmRegs->fmnpi);
	if (!pending)
		return;

    if(pending & INTR_EN_BMI)
        REPORT_ERROR(MAJOR, E_NOT_SUPPORTED, ("BMI Event - undefined!"));
    if(pending & INTR_EN_QMI)
        QmiEvent(p_Fm);
    if(pending & INTR_EN_PRS)
        p_Fm->intrMng[e_FM_EV_PRS].f_Isr(p_Fm->intrMng[e_FM_EV_PRS].h_SrcHandle);
    if(pending & INTR_EN_PLCR)
        p_Fm->intrMng[e_FM_EV_PLCR].f_Isr(p_Fm->intrMng[e_FM_EV_PLCR].h_SrcHandle);
    if(pending & INTR_EN_KG)
        p_Fm->intrMng[e_FM_EV_KG].f_Isr(p_Fm->intrMng[e_FM_EV_KG].h_SrcHandle);
    if(pending & INTR_EN_TMR)
            p_Fm->intrMng[e_FM_EV_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_TMR].h_SrcHandle);

    /* MAC events may belong to different partitions */
    if(pending & INTR_EN_1G_MAC0_TMR)
    {
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_1G_MAC0_TMR].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_1G_MAC0_TMR, pending);
        else
            p_Fm->intrMng[e_FM_EV_1G_MAC0_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC0_TMR].h_SrcHandle);
    }
    if(pending & INTR_EN_1G_MAC1_TMR)
    {
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_1G_MAC1_TMR].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_1G_MAC1_TMR, pending);
        else
            p_Fm->intrMng[e_FM_EV_1G_MAC1_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC1_TMR].h_SrcHandle);
    }
    if(pending & INTR_EN_1G_MAC2_TMR)
    {
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_1G_MAC2_TMR].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_1G_MAC2_TMR, pending);
        else
            p_Fm->intrMng[e_FM_EV_1G_MAC2_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC2_TMR].h_SrcHandle);
    }
    if(pending & INTR_EN_1G_MAC3_TMR)
    {
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_1G_MAC3_TMR].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_1G_MAC3_TMR, pending);
        else
           p_Fm->intrMng[e_FM_EV_1G_MAC3_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC3_TMR].h_SrcHandle);
    }
    /* IM port events may belong to different partitions */
    if(pending & INTR_EN_REV0)
    {
        event = GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcev[0]) & GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[0]);
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmcev[0], event);
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_FMAN_CTRL_0].guestId)
            /*TODO IPC ISR For Fman Ctrl */
            ASSERT_COND(0);
            //SendIpcIsr(p_Fm, e_FM_EV_FMAN_CTRL_0, pending);
        else
            p_Fm->fmanCtrlIntr[0].f_Isr(p_Fm->fmanCtrlIntr[0].h_SrcHandle, event);

    }
    if(pending & INTR_EN_REV1)
    {
        event = GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcev[1]) & GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[1]);
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmcev[1], event);
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_FMAN_CTRL_1].guestId)
            /*TODO IPC ISR For Fman Ctrl */
            ASSERT_COND(0);
            //SendIpcIsr(p_Fm, e_FM_EV_FMAN_CTRL_1, pending);
        else
            p_Fm->fmanCtrlIntr[1].f_Isr(p_Fm->fmanCtrlIntr[1].h_SrcHandle, event);

    }
    if(pending & INTR_EN_REV2)
    {
        event = GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcev[2]) & GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[2]);
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmcev[2], event);
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_FMAN_CTRL_2].guestId)
            /*TODO IPC ISR For Fman Ctrl */
            ASSERT_COND(0);
            //SendIpcIsr(p_Fm, e_FM_EV_FMAN_CTRL_2, pending);
        else
           p_Fm->fmanCtrlIntr[2].f_Isr(p_Fm->fmanCtrlIntr[2].h_SrcHandle, event);
    }
    if(pending & INTR_EN_REV3)
    {
        event = GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcev[3]) & GET_UINT32(p_Fm->p_FmFpmRegs->fmfpfcee[3]);
        WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmcev[3], event);
        if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_FMAN_CTRL_3].guestId)
            /*TODO IPC ISR For Fman Ctrl */
            ASSERT_COND(0);
            //SendIpcIsr(p_Fm, e_FM_EV_FMAN_CTRL_2, pendin3);
        else
            p_Fm->fmanCtrlIntr[3].f_Isr(p_Fm->fmanCtrlIntr[3].h_SrcHandle, event);
    }
}

void FM_ErrorIsr(t_Handle h_Fm)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;
    uint32_t                pending;

    SANITY_CHECK_RETURN(h_Fm, E_INVALID_HANDLE);

    /* error interrupts */
    pending = GET_UINT32(p_Fm->p_FmFpmRegs->fmepi);
    if (!pending)
        return;

    if(pending & ERR_INTR_EN_BMI)
        BmiErrEvent(p_Fm);
    if(pending & ERR_INTR_EN_QMI)
        QmiErrEvent(p_Fm);
    if(pending & ERR_INTR_EN_FPM)
        FpmErrEvent(p_Fm);
    if(pending & ERR_INTR_EN_DMA)
        DmaErrEvent(p_Fm);
    if(pending & ERR_INTR_EN_IRAM)
        IramErrIntr(p_Fm);
    if(pending & ERR_INTR_EN_MURAM)
        MuramErrIntr(p_Fm);
    if(pending & ERR_INTR_EN_PRS)
        p_Fm->intrMng[e_FM_EV_ERR_PRS].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_PRS].h_SrcHandle);
    if(pending & ERR_INTR_EN_PLCR)
        p_Fm->intrMng[e_FM_EV_ERR_PLCR].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_PLCR].h_SrcHandle);
    if(pending & ERR_INTR_EN_KG)
        p_Fm->intrMng[e_FM_EV_ERR_KG].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_KG].h_SrcHandle);

    /* MAC events may belong to different partitions */
    if(pending & ERR_INTR_EN_1G_MAC0)
    {
       if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_ERR_1G_MAC0].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_ERR_1G_MAC0, pending);
        else
            p_Fm->intrMng[e_FM_EV_ERR_1G_MAC0].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC0].h_SrcHandle);
    }
    if(pending & ERR_INTR_EN_1G_MAC1)
    {
       if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_ERR_1G_MAC1].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_ERR_1G_MAC1, pending);
        else
            p_Fm->intrMng[e_FM_EV_ERR_1G_MAC1].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC1].h_SrcHandle);
    }
    if(pending & ERR_INTR_EN_1G_MAC2)
    {
       if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_ERR_1G_MAC2].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_ERR_1G_MAC2, pending);
        else
            p_Fm->intrMng[e_FM_EV_ERR_1G_MAC2].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC2].h_SrcHandle);
    }
    if(pending & ERR_INTR_EN_1G_MAC3)
    {
       if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_ERR_1G_MAC3].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_ERR_1G_MAC3, pending);
        else
            p_Fm->intrMng[e_FM_EV_ERR_1G_MAC3].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC3].h_SrcHandle);
    }
    if(pending & ERR_INTR_EN_10G_MAC0)
    {
       if (p_Fm->guestId != p_Fm->intrMng[e_FM_EV_ERR_10G_MAC0].guestId)
            SendIpcIsr(p_Fm, e_FM_EV_ERR_10G_MAC0, pending);
        else
            p_Fm->intrMng[e_FM_EV_ERR_10G_MAC0].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_10G_MAC0].h_SrcHandle);
    }
}

void FM_GuestErrorIsr(t_Handle h_Fm, uint32_t pending)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN(h_Fm, E_INVALID_HANDLE);

    /* error interrupts */
    if(pending & ERR_INTR_EN_1G_MAC0)
        p_Fm->intrMng[e_FM_EV_ERR_1G_MAC0].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC0].h_SrcHandle);
    if(pending & ERR_INTR_EN_1G_MAC1)
        p_Fm->intrMng[e_FM_EV_ERR_1G_MAC1].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC1].h_SrcHandle);
    if(pending & ERR_INTR_EN_1G_MAC2)
        p_Fm->intrMng[e_FM_EV_ERR_1G_MAC2].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC2].h_SrcHandle);
    if(pending & ERR_INTR_EN_1G_MAC3)
        p_Fm->intrMng[e_FM_EV_ERR_1G_MAC3].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_1G_MAC3].h_SrcHandle);
    if(pending & ERR_INTR_EN_10G_MAC0)
        p_Fm->intrMng[e_FM_EV_ERR_10G_MAC0].f_Isr(p_Fm->intrMng[e_FM_EV_ERR_10G_MAC0].h_SrcHandle);
}

void FM_GuestEventIsr(t_Handle h_Fm, uint32_t pending)
{
    t_Fm                    *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN(h_Fm, E_INVALID_HANDLE);

    if(pending & INTR_EN_1G_MAC0_TMR)
        p_Fm->intrMng[e_FM_EV_1G_MAC0_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC0_TMR].h_SrcHandle);
    if(pending & INTR_EN_1G_MAC1_TMR)
        p_Fm->intrMng[e_FM_EV_1G_MAC1_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC1_TMR].h_SrcHandle);
    if(pending & INTR_EN_1G_MAC2_TMR)
        p_Fm->intrMng[e_FM_EV_1G_MAC2_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC2_TMR].h_SrcHandle);
    if(pending & INTR_EN_1G_MAC3_TMR)
        p_Fm->intrMng[e_FM_EV_1G_MAC3_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_1G_MAC3_TMR].h_SrcHandle);
    if(pending & INTR_EN_TMR)
        p_Fm->intrMng[e_FM_EV_TMR].f_Isr(p_Fm->intrMng[e_FM_EV_TMR].h_SrcHandle);
}

t_Error FM_SetPortsBandwidth(t_Handle h_Fm, t_FmPortsBandwidthParams *p_PortsBandwidth)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    int         i;
    uint8_t     sum;
    uint8_t     hardwarePortId=0;
    uint32_t    tmpRegs[8] = {0,0,0,0,0,0,0,0};
    uint8_t     relativePortId, shift, weight, maxPercent = 0;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    /* check that all ports add up to 100% */
    sum = 0;
    for (i=0;i<p_PortsBandwidth->numOfPorts;i++)
        sum +=p_PortsBandwidth->portsBandwidths[i].bandwidth;
    if (sum != 100)
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Sum of ports bandwidth differ from 100%"));

    /* find highest precent */
    for (i=0;i<p_PortsBandwidth->numOfPorts;i++)
    {
        if (p_PortsBandwidth->portsBandwidths[i].bandwidth > maxPercent)
            maxPercent = p_PortsBandwidth->portsBandwidths[i].bandwidth;
    }

    /* calculate weight for each port */
    for (i=0;i<p_PortsBandwidth->numOfPorts;i++)
    {
        weight = (uint8_t)((p_PortsBandwidth->portsBandwidths[i].bandwidth * PORT_MAX_WEIGHT )/maxPercent);
        /* we want even division between 1-to-PORT_MAX_WEIGHT. so if exect division
           is not reached, we round up so that:
           0 until maxPercent/PORT_MAX_WEIGHT get "1"
           maxPercent/PORT_MAX_WEIGHT+1 until (maxPercent/PORT_MAX_WEIGHT)*2 get "2"
           ...
           maxPercent - maxPercent/PORT_MAX_WEIGHT until maxPercent get "PORT_MAX_WEIGHT: */
        if((uint8_t)((p_PortsBandwidth->portsBandwidths[i].bandwidth * PORT_MAX_WEIGHT ) % maxPercent))
            weight++;

        /* find the location of this port within the register */
        SW_PORT_ID_TO_HW_PORT_ID(hardwarePortId, p_PortsBandwidth->portsBandwidths[i].type,p_PortsBandwidth->portsBandwidths[i].relativePortId);
        relativePortId = (uint8_t)(hardwarePortId % 8);
        shift = (uint8_t)(32-4*(relativePortId+1));


        if(weight > 1)
            /* Add this port to tmpReg */
            /* (each 8 ports result in one register)*/
            tmpRegs[hardwarePortId/8] |= ((weight-1) << shift);
    }

    for(i=0;i<8;i++)
        if(tmpRegs[i])
            WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_arb[i], tmpRegs[i]);

    return E_OK;
}

t_Error FM_EnableRamsEcc(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        t_FmIpcMsg          msg;
        t_Error             err;

        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_ENABLE_RAM_ECC;
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        return E_OK;
    }

    if(!p_Fm->internalCall)
        p_Fm->explicitEnable = TRUE;
    p_Fm->internalCall = FALSE;

    if(p_Fm->ramsEccEnable)
        return E_OK;
    else
    {
        tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmrcr);
        if(tmpReg & FPM_RAM_CTL_RAMS_ECC_EN_SRC_SEL)
        {
            DBG(WARNING, ("Rams ECC is configured to be controlled through JTAG"));
            WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, tmpReg | FPM_RAM_CTL_IRAM_ECC_EN);
        }
        else
            WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, tmpReg | (FPM_RAM_CTL_RAMS_ECC_EN | FPM_RAM_CTL_IRAM_ECC_EN));
        p_Fm->ramsEccEnable = TRUE;
    }

    return E_OK;
}

t_Error FM_DisableRamsEcc(t_Handle h_Fm)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    tmpReg;
    bool        explicitDisable = FALSE;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_HANDLE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        t_FmIpcMsg          msg;
        t_Error             err;

        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_DISABLE_RAM_ECC;
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     NULL,
                                     NULL,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        return E_OK;
    }

    if(!p_Fm->internalCall)
        explicitDisable = TRUE;
    p_Fm->internalCall = FALSE;

    /* if rams are already disabled, or if rams were explicitly enabled and are
       currently called indirectly (not explicitly), ignore this call. */
    if(!p_Fm->ramsEccEnable || (p_Fm->explicitEnable && !explicitDisable))
        return E_OK;
    else
    {
        if(p_Fm->explicitEnable)
            /* This is the case were both explicit are TRUE.
               Turn off this flag for cases were following ramsEnable
               routines are called */
            p_Fm->explicitEnable = FALSE;

        tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmrcr);
        if(tmpReg & FPM_RAM_CTL_RAMS_ECC_EN_SRC_SEL)
        {
            DBG(WARNING, ("Rams ECC is configured to be controlled through JTAG"));
            WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, tmpReg & ~FPM_RAM_CTL_IRAM_ECC_EN);
        }
        else
            WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrcr, tmpReg & ~(FPM_RAM_CTL_RAMS_ECC_EN | FPM_RAM_CTL_IRAM_ECC_EN));
        p_Fm->ramsEccEnable = FALSE;
    }

    return E_OK;
}

t_Error FM_SetException(t_Handle h_Fm, e_FmExceptions exception, bool enable)
{
    t_Fm        *p_Fm = (t_Fm*)h_Fm;
    uint32_t    bitMask = 0;
    uint32_t    tmpReg;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    GET_EXCEPTION_FLAG(bitMask, exception);
    if(bitMask)
    {
        if (enable)
            p_Fm->exceptions |= bitMask;
        else
            p_Fm->exceptions &= ~bitMask;

        switch(exception)
        {
             case(e_FM_EX_DMA_BUS_ERROR):
                tmpReg = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr);
                if(enable)
                    tmpReg |= DMA_MODE_BER;
                else
                    tmpReg &= ~DMA_MODE_BER;
                /* disable bus error */
                WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, tmpReg);
                break;
             case(e_FM_EX_DMA_READ_ECC):
             case(e_FM_EX_DMA_SYSTEM_WRITE_ECC):
             case(e_FM_EX_DMA_FM_WRITE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr);
                if(enable)
                    tmpReg |= DMA_MODE_ECC;
                else
                    tmpReg &= ~DMA_MODE_ECC;
                WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, tmpReg);
                break;
             case(e_FM_EX_FPM_STALL_ON_TASKS):
                tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fpmem);
                if(enable)
                    tmpReg |= FPM_EV_MASK_STALL_EN;
                else
                    tmpReg &= ~FPM_EV_MASK_STALL_EN;
                WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, tmpReg);
                break;
             case(e_FM_EX_FPM_SINGLE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fpmem);
                if(enable)
                    tmpReg |= FPM_EV_MASK_SINGLE_ECC_EN;
                else
                    tmpReg &= ~FPM_EV_MASK_SINGLE_ECC_EN;
                WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, tmpReg);
                break;
            case( e_FM_EX_FPM_DOUBLE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fpmem);
                if(enable)
                    tmpReg |= FPM_EV_MASK_DOUBLE_ECC_EN;
                else
                    tmpReg &= ~FPM_EV_MASK_DOUBLE_ECC_EN;
                WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, tmpReg);
                break;
            case( e_FM_EX_QMI_SINGLE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_ien);
                if(enable)
                    tmpReg |= QMI_INTR_EN_SINGLE_ECC;
                else
                    tmpReg &= ~QMI_INTR_EN_SINGLE_ECC;
                WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_ien, tmpReg);
                break;
             case(e_FM_EX_QMI_DOUBLE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien);
                if(enable)
                    tmpReg |= QMI_ERR_INTR_EN_DOUBLE_ECC;
                else
                    tmpReg &= ~QMI_ERR_INTR_EN_DOUBLE_ECC;
                WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien, tmpReg);
                break;
             case(e_FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID):
                tmpReg = GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien);
                if(enable)
                    tmpReg |= QMI_ERR_INTR_EN_DEQ_FROM_DEF;
                else
                    tmpReg &= ~QMI_ERR_INTR_EN_DEQ_FROM_DEF;
                WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eien, tmpReg);
                break;
             case(e_FM_EX_BMI_LIST_RAM_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier);
                if(enable)
                {
#ifdef FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8
                    t_FmRevisionInfo revInfo;
                    FM_GetRevision(p_Fm, &revInfo);
                    if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
                        RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_LIST_RAM_ECC!"));
#endif   /* FM_RAM_LIST_ERR_IRQ_ERRATA_FMAN8 */
                    tmpReg |= BMI_ERR_INTR_EN_LIST_RAM_ECC;
                }
                else
                    tmpReg &= ~BMI_ERR_INTR_EN_LIST_RAM_ECC;
                WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier, tmpReg);
                break;
             case(e_FM_EX_BMI_PIPELINE_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier);
                if(enable)
                {
#ifdef FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9
                    t_FmRevisionInfo revInfo;
                    FM_GetRevision(p_Fm, &revInfo);
                    if ((revInfo.majorRev == 1) && (revInfo.minorRev == 0))
                        RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_PIPELINE_ECC!"));
#endif /* FM_BMI_PIPELINE_ERR_IRQ_ERRATA_FMAN9 */
                    tmpReg |= BMI_ERR_INTR_EN_PIPELINE_ECC;
                }
                else
                    tmpReg &= ~BMI_ERR_INTR_EN_PIPELINE_ECC;
                WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier, tmpReg);
                break;
             case(e_FM_EX_BMI_STATISTICS_RAM_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier);
                if(enable)
                    tmpReg |= BMI_ERR_INTR_EN_STATISTICS_RAM_ECC;
                else
                    tmpReg &= ~BMI_ERR_INTR_EN_STATISTICS_RAM_ECC;
                WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier, tmpReg);
                break;
             case(e_FM_EX_BMI_DISPATCH_RAM_ECC):
                 {
                   t_FmRevisionInfo     revInfo;
                   FM_GetRevision(p_Fm, &revInfo);
                   if ((revInfo.majorRev > 3) && (revInfo.minorRev == 0))
                   {
                       tmpReg = GET_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier);
                       if(enable)
                           tmpReg |= BMI_ERR_INTR_EN_DISPATCH_RAM_ECC;
                       else
                           tmpReg &= ~BMI_ERR_INTR_EN_DISPATCH_RAM_ECC;
                       WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ier, tmpReg);
                   }
                   else
                       RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("e_FM_EX_BMI_DISPATCH_RAM_ECC"));
                 }
                 break;
             case(e_FM_EX_IRAM_ECC):
                 tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmrie);
                if(enable)
                {
                    /* enable ECC if not enabled */
                    FmEnableRamsEcc(p_Fm);
                    /* enable ECC interrupts */
                    tmpReg |= FPM_IRAM_ECC_ERR_EX_EN;
                }
                else
                {
                    /* ECC mechanism may be disabled, depending on driver status  */
                    FmDisableRamsEcc(p_Fm);
                    tmpReg &= ~FPM_IRAM_ECC_ERR_EX_EN;
                }
                WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrie, tmpReg);
                break;

             case(e_FM_EX_MURAM_ECC):
                tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fmrie);
                if(enable)
                {
                    /* enable ECC if not enabled */
                    FmEnableRamsEcc(p_Fm);
                    /* enable ECC interrupts */
                    tmpReg |= FPM_MURAM_ECC_ERR_EX_EN;
                }
                else
                {
                    /* ECC mechanism may be disabled, depending on driver status  */
                    FmDisableRamsEcc(p_Fm);
                    tmpReg &= ~FPM_MURAM_ECC_ERR_EX_EN;
                }

                WRITE_UINT32(p_Fm->p_FmFpmRegs->fmrie, tmpReg);
                break;
            default:
                RETURN_ERROR(MINOR, E_INVALID_SELECTION, NO_MSG);
        }
    }
    else
        RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("Undefined exception"));

    return E_OK;
}

t_Error FM_GetRevision(t_Handle h_Fm, t_FmRevisionInfo *p_FmRevisionInfo)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    uint32_t            tmpReg;
    t_Error             err;
    t_FmIpcRevisionInfo fmIpcRevisionInfo;
    uint32_t            replyLength;
    t_FmIpcMsg          msg;

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_REV;
        replyLength = sizeof(fmIpcRevisionInfo);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     (uint8_t*)&fmIpcRevisionInfo,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MINOR, err, NO_MSG);
        if (replyLength != sizeof(fmIpcRevisionInfo))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        p_FmRevisionInfo->majorRev = fmIpcRevisionInfo.majorRev;
        p_FmRevisionInfo->minorRev = fmIpcRevisionInfo.minorRev;
        return E_OK;
    }

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);

    /* read revision register 1 */
    tmpReg = GET_UINT32(p_Fm->p_FmFpmRegs->fm_ip_rev_1);
    p_FmRevisionInfo->majorRev = (uint8_t)((tmpReg & FPM_REV1_MAJOR_MASK) >> FPM_REV1_MAJOR_SHIFT);
    p_FmRevisionInfo->minorRev = (uint8_t)((tmpReg & FPM_REV1_MINOR_MASK) >> FPM_REV1_MINOR_SHIFT);

    return E_OK;
}

uint32_t  FM_GetCounter(t_Handle h_Fm, e_FmCounters counter)
{
    t_Fm                *p_Fm = (t_Fm*)h_Fm;
    t_Error             err;
    uint32_t            counterValue;
    uint32_t            replyLength;
    t_FmIpcMsg          msg;

    SANITY_CHECK_RETURN_VALUE(p_Fm, E_INVALID_HANDLE, 0);
    SANITY_CHECK_RETURN_VALUE(!p_Fm->p_FmDriverParam, E_INVALID_STATE, 0);


    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_GET_COUNTER;
        *(uint32_t*)msg.msgBody = (uint32_t)counter;
        replyLength = sizeof(counterValue);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId) +sizeof(counterValue),
                                     (uint8_t*)&(counterValue),
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
            RETURN_ERROR(MAJOR, err, NO_MSG);
        if(replyLength != sizeof(counterValue))
            RETURN_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));

        return counterValue;
     }

	if(!(GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc) & QMI_CFG_EN_COUNTERS))
		REPORT_ERROR(MINOR, E_INVALID_STATE, ("FM%d: Requested counter (%d) was not enabled", p_Fm->fmId, counter));

    switch(counter)
    {
        case(e_FM_COUNTERS_ENQ_TOTAL_FRAME):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_etfc);
        case(e_FM_COUNTERS_DEQ_TOTAL_FRAME):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dtfc);
        case(e_FM_COUNTERS_DEQ_0):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc0);
        case(e_FM_COUNTERS_DEQ_1):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc1);
        case(e_FM_COUNTERS_DEQ_2):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc2);
        case(e_FM_COUNTERS_DEQ_3):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc3);
        case(e_FM_COUNTERS_DEQ_FROM_DEFAULT):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dfdc);
        case(e_FM_COUNTERS_DEQ_FROM_CONTEXT):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dfcc);
        case(e_FM_COUNTERS_DEQ_FROM_FD):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dffc);
        case(e_FM_COUNTERS_DEQ_CONFIRM):
            return GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_dcc);
        case(e_FM_COUNTERS_SEMAPHOR_ENTRY_FULL_REJECT):
            return GET_UINT32(p_Fm->p_FmDmaRegs->fmdmsefrc);
        case(e_FM_COUNTERS_SEMAPHOR_QUEUE_FULL_REJECT):
            return GET_UINT32(p_Fm->p_FmDmaRegs->fmdmsqfrc);
        case(e_FM_COUNTERS_SEMAPHOR_SYNC_REJECT):
            return GET_UINT32(p_Fm->p_FmDmaRegs->fmdmssrc);
        default:
            break;
    }
    /* should never get here */
    ASSERT_COND(FALSE);

    return 0;
}

t_Error  FM_ModifyCounter(t_Handle h_Fm, e_FmCounters counter, uint32_t val)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

   SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
   SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    /* When applicable (when there is an 'enable counters' bit,
    check that counters are enabled */
    switch(counter)
    {
        case(e_FM_COUNTERS_ENQ_TOTAL_FRAME):
        case(e_FM_COUNTERS_DEQ_TOTAL_FRAME):
        case(e_FM_COUNTERS_DEQ_0):
        case(e_FM_COUNTERS_DEQ_1):
        case(e_FM_COUNTERS_DEQ_2):
        case(e_FM_COUNTERS_DEQ_3):
        case(e_FM_COUNTERS_DEQ_FROM_DEFAULT):
        case(e_FM_COUNTERS_DEQ_FROM_CONTEXT):
        case(e_FM_COUNTERS_DEQ_FROM_FD):
        case(e_FM_COUNTERS_DEQ_CONFIRM):
            if(!(GET_UINT32(p_Fm->p_FmQmiRegs->fmqm_gc) & QMI_CFG_EN_COUNTERS))
                RETURN_ERROR(MINOR, E_INVALID_STATE, ("Requested counter was not enabled"));
            break;
        default:
            break;
    }

    /* Set counter */
    switch(counter)
    {
        case(e_FM_COUNTERS_ENQ_TOTAL_FRAME):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_etfc, val);
            break;
        case(e_FM_COUNTERS_DEQ_TOTAL_FRAME):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dtfc, val);
            break;
        case(e_FM_COUNTERS_DEQ_0):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc0, val);
            break;
        case(e_FM_COUNTERS_DEQ_1):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc1, val);
            break;
        case(e_FM_COUNTERS_DEQ_2):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc2, val);
            break;
        case(e_FM_COUNTERS_DEQ_3):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dc3, val);
            break;
        case(e_FM_COUNTERS_DEQ_FROM_DEFAULT):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dfdc, val);
            break;
        case(e_FM_COUNTERS_DEQ_FROM_CONTEXT):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dfcc, val);
            break;
        case(e_FM_COUNTERS_DEQ_FROM_FD):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dffc, val);
            break;
        case(e_FM_COUNTERS_DEQ_CONFIRM):
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_dcc, val);
            break;
        case(e_FM_COUNTERS_SEMAPHOR_ENTRY_FULL_REJECT):
            WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmsefrc, val);
            break;
        case(e_FM_COUNTERS_SEMAPHOR_QUEUE_FULL_REJECT):
            WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmsqfrc, val);
            break;
        case(e_FM_COUNTERS_SEMAPHOR_SYNC_REJECT):
            WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmssrc, val);
            break;
        default:
            break;
    }

    return E_OK;
}

void FM_SetDmaEmergency(t_Handle h_Fm, e_FmDmaMuramPort muramPort, bool enable)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;
    uint32_t    bitMask;

    SANITY_CHECK_RETURN(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    bitMask = (uint32_t)((muramPort==e_FM_DMA_MURAM_PORT_WRITE) ? DMA_MODE_EMERGENCY_WRITE : DMA_MODE_EMERGENCY_READ);

    if(enable)
        WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr) | bitMask);
    else /* disable */
        WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr) & ~bitMask);

    return;
}

void FM_SetDmaExtBusPri(t_Handle h_Fm, e_FmDmaExtBusPri pri)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    WRITE_UINT32(p_Fm->p_FmDmaRegs->fmdmmr, GET_UINT32(p_Fm->p_FmDmaRegs->fmdmmr) | ((uint32_t)pri << DMA_MODE_BUS_PRI_SHIFT) );

    return;
}

void FM_GetDmaStatus(t_Handle h_Fm, t_FmDmaStatus *p_FmDmaStatus)
{
    t_Fm            *p_Fm = (t_Fm*)h_Fm;
    uint32_t            tmpReg;
    t_Error             err;
    t_FmIpcDmaStatus    ipcDmaStats;
    uint32_t            replyLength;
    t_FmIpcMsg          msg;

    SANITY_CHECK_RETURN(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        replyLength = sizeof(ipcDmaStats);
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_DMA_STAT;
        replyLength = sizeof(ipcDmaStats);
        if ((err = XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                     (uint8_t*)&msg,
                                     sizeof(msg.msgId),
                                     (uint8_t*)&ipcDmaStats,
                                     &replyLength,
                                     NULL,
                                     NULL)) != E_OK)
        {
            REPORT_ERROR(MINOR, err, NO_MSG);
            return;
        }
        if(replyLength != sizeof(ipcDmaStats))
        {
            REPORT_ERROR(MAJOR, E_INVALID_VALUE, ("IPC reply length mismatch"));
            return;
        }

        p_FmDmaStatus->cmqNotEmpty = (bool)ipcDmaStats.boolCmqNotEmpty;            /**< Command queue is not empty */
        p_FmDmaStatus->busError = (bool)ipcDmaStats.boolBusError;                  /**< Bus error occured */
        p_FmDmaStatus->readBufEccError = (bool)ipcDmaStats.boolReadBufEccError;        /**< Double ECC error on buffer Read */
        p_FmDmaStatus->writeBufEccSysError =(bool)ipcDmaStats.boolWriteBufEccSysError;    /**< Double ECC error on buffer write from system side */
        p_FmDmaStatus->writeBufEccFmError = (bool)ipcDmaStats.boolWriteBufEccFmError;     /**< Double ECC error on buffer write from FM side */
        return;
    }

    tmpReg = GET_UINT32(p_Fm->p_FmDmaRegs->fmdmsr);

    p_FmDmaStatus->cmqNotEmpty = (bool)(tmpReg & DMA_STATUS_CMD_QUEUE_NOT_EMPTY);
    p_FmDmaStatus->busError = (bool)(tmpReg & DMA_STATUS_BUS_ERR);
    p_FmDmaStatus->readBufEccError = (bool)(tmpReg & DMA_STATUS_READ_ECC);
    p_FmDmaStatus->writeBufEccSysError = (bool)(tmpReg & DMA_STATUS_SYSTEM_WRITE_ECC);
    p_FmDmaStatus->writeBufEccFmError = (bool)(tmpReg & DMA_STATUS_FM_WRITE_ECC);
    return;
}

t_Error FM_ForceIntr (t_Handle h_Fm, e_FmExceptions exception)
{
    t_Fm *p_Fm = (t_Fm*)h_Fm;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    switch(exception)
    {
        case e_FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID:
            if (!(p_Fm->exceptions & FM_EX_QMI_DEQ_FROM_UNKNOWN_PORTID))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eif, QMI_ERR_INTR_EN_DEQ_FROM_DEF);
            break;
        case e_FM_EX_QMI_SINGLE_ECC:
            if (!(p_Fm->exceptions & FM_EX_QMI_SINGLE_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_if, QMI_INTR_EN_SINGLE_ECC);
            break;
        case e_FM_EX_QMI_DOUBLE_ECC:
            if (!(p_Fm->exceptions & FM_EX_QMI_DOUBLE_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmQmiRegs->fmqm_eif, QMI_ERR_INTR_EN_DOUBLE_ECC);
            break;
        case e_FM_EX_BMI_LIST_RAM_ECC:
            if (!(p_Fm->exceptions & FM_EX_BMI_LIST_RAM_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr, BMI_ERR_INTR_EN_LIST_RAM_ECC);
            break;
        case e_FM_EX_BMI_PIPELINE_ECC:
            if (!(p_Fm->exceptions & FM_EX_BMI_PIPELINE_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr, BMI_ERR_INTR_EN_PIPELINE_ECC);
            break;
        case e_FM_EX_BMI_STATISTICS_RAM_ECC:
            if (!(p_Fm->exceptions & FM_EX_BMI_STATISTICS_RAM_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr, BMI_ERR_INTR_EN_STATISTICS_RAM_ECC);
            break;
        case e_FM_EX_BMI_DISPATCH_RAM_ECC:
            if (!(p_Fm->exceptions & FM_EX_BMI_DISPATCH_RAM_ECC))
                RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception is masked"));
            WRITE_UINT32(p_Fm->p_FmBmiRegs->fmbm_ifr, FM_EX_BMI_DISPATCH_RAM_ECC);
            break;
        default:
            RETURN_ERROR(MINOR, E_NOT_SUPPORTED, ("The selected exception may not be forced"));
    }

    return E_OK;
}

void FM_Resume(t_Handle h_Fm)
{
    t_Fm            *p_Fm = (t_Fm*)h_Fm;
    uint32_t        tmpReg;

    SANITY_CHECK_RETURN(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN(!p_Fm->p_FmDriverParam, E_INVALID_STATE);

    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        /* TODO */
        ASSERT_COND(0);
        return;
    }

    tmpReg  = GET_UINT32(p_Fm->p_FmFpmRegs->fpmem);
    /* clear tmpReg event bits in order not to clear standing events */
    tmpReg &= ~(FPM_EV_MASK_DOUBLE_ECC | FPM_EV_MASK_STALL | FPM_EV_MASK_SINGLE_ECC);
    WRITE_UINT32(p_Fm->p_FmFpmRegs->fpmem, tmpReg | FPM_EV_MASK_RELEASE_FM);
}

t_Handle FM_NoPCDRecovery(t_FmNoPCDRecoveryParams *p_FmParam)
{
    t_Fm                *p_Fm;

    /* Allocate FM structure */
    p_Fm = (t_Fm *) XX_MallocSmart(sizeof(t_Fm), p_FmParam->recoveryMemId, 4);
    if (!p_Fm)
    {
        REPORT_ERROR(MAJOR, E_NO_MEMORY, ("FM driver structure"));
        return NULL;
    }
    memset(p_Fm, 0, sizeof(t_Fm));
    p_Fm->recoveryMode = TRUE;
    p_Fm->recoveryMemId = p_FmParam->recoveryMemId;

    p_Fm->h_App             = p_FmParam->h_App;

    memcpy(p_Fm, p_FmParam->savedFmStructureAddress, sizeof(t_Fm));

    if (CheckSavedFmValidity((t_Fm *)p_FmParam->savedFmStructureAddress) != E_OK)
    {
        XX_FreeSmart(p_Fm);
        REPORT_ERROR(MAJOR, E_INVALID_STATE, NO_MSG);
        return NULL;
    }

    /* MURAM is unavailable after recovery */
    p_Fm->h_FmMuram = 0;
    p_Fm->camBaseAddr = 0;
    p_Fm->fifoBaseAddr = 0;
    p_Fm->resAddr = 0;

    /**********************************************************************/
    /* clear all ports/MACs state parameters - they will be re-initialized*/
    /**********************************************************************/

#if defined(FM_MAX_NUM_OF_10G_MACS) && (FM_MAX_NUM_OF_10G_MACS)
    memset(p_Fm->macMaxFrameLengths10G, 0, sizeof(uint16_t)*FM_MAX_NUM_OF_10G_MACS);
#endif /* defined(FM_MAX_NUM_OF_10G_MACS) && (FM_MAX_NUM_OF_10G_MACS) */
    memset(p_Fm->macMaxFrameLengths1G, 0, sizeof(uint16_t)*FM_MAX_NUM_OF_1G_MACS);
    /* clear Fman ctrl's usage */
    memset(p_Fm->fmanCtrlIntr, 0, FM_NUM_OF_FMAN_CTRL_EVENT_REGS*sizeof(t_FmanCtrlIntrSrc));
    memset(p_Fm->usedEventRegs, 0, FM_NUM_OF_FMAN_CTRL_EVENT_REGS*sizeof(bool));
    /* clear FM port's resources state parameters */
    p_Fm->extraTasksPoolSize = 0;
    p_Fm->extraOpenDmasPoolSize = 0;
    p_Fm->extraFifoPoolSize = FM_MAX_NUM_OF_RX_PORTS*BMI_FIFO_UNITS;
    p_Fm->maxNumOfOpenDmas  = DEFAULT_maxNumOfOpenDmas;
    p_Fm->accumulatedNumOfTasks = 0;
    p_Fm->accumulatedFifoSize = 0;
    p_Fm->accumulatedNumOfOpenDmas = 0;
#ifdef FM_QMI_DEQ_OPTIONS_SUPPORT
    p_Fm->accumulatedNumOfDeqTnums = 0;
#endif /* FM_QMI_DEQ_OPTIONS_SUPPORT */
#ifdef FM_LOW_END_RESTRICTION
    p_Fm->lowEndRestriction = FALSE;
#endif /* FM_LOW_END_RESTRICTION */

    /* set new spinlock */
    if (p_Fm->h_Spinlock)
         XX_FreeSpinlock(p_Fm->h_Spinlock);
    p_Fm->h_Spinlock = XX_InitSpinlock();
     if (!p_Fm->h_Spinlock)
     {
         XX_FreeSmart(p_Fm);
         REPORT_ERROR(MAJOR, E_INVALID_STATE, ("cant allocate spinlock!"));
         return NULL;
     }

     return p_Fm;
}

#if (defined(DEBUG_ERRORS) && (DEBUG_ERRORS > 0))
t_Error FM_DumpRegs(t_Handle h_Fm)
{
    t_Fm            *p_Fm = (t_Fm *)h_Fm;
    uint8_t         i = 0;
    t_FmIpcMsg      msg;

    DECLARE_DUMP;

    SANITY_CHECK_RETURN_ERROR(p_Fm, E_INVALID_HANDLE);
    SANITY_CHECK_RETURN_ERROR(!p_Fm->p_FmDriverParam, E_INVALID_STATE);


    if(p_Fm->guestId != NCSW_MASTER_ID)
    {
        memset(&msg, 0, sizeof(msg));
        msg.msgId = FM_DUMP_REGS;
        return XX_IpcSendMessage(p_Fm->h_IpcSessions[0],
                                    (uint8_t*)&msg,
                                    sizeof(msg.msgId),
                                    NULL,
                                    NULL,
                                    NULL,
                                    NULL);
    }


    DUMP_SUBTITLE(("\n"));

    DUMP_TITLE(p_Fm->p_FmFpmRegs, ("FmFpmRegs Regs"));

    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmtnc);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmpr);
    DUMP_VAR(p_Fm->p_FmFpmRegs,brkc);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmflc);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmdis1);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmdis2);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmepi);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmrie);

    DUMP_TITLE(&p_Fm->p_FmFpmRegs->fmfpfcev, ("fmfpfcev"));
    DUMP_SUBSTRUCT_ARRAY(i, 4)
    {
        DUMP_MEMORY(&p_Fm->p_FmFpmRegs->fmfpfcev[i], sizeof(uint32_t));
    }

    DUMP_TITLE(&p_Fm->p_FmFpmRegs->fmfpfcee, ("fmfpfcee"));
    DUMP_SUBSTRUCT_ARRAY(i, 4)
    {
        DUMP_MEMORY(&p_Fm->p_FmFpmRegs->fmfpfcee[i], sizeof(uint32_t));
    }

    DUMP_SUBTITLE(("\n"));
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmtsc1);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmtsc2);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmtsp);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmtsf);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmrcr);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmextc);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmext1);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmext2);

    DUMP_TITLE(&p_Fm->p_FmFpmRegs->fpmdrd, ("fpmdrd"));
    DUMP_SUBSTRUCT_ARRAY(i, 16)
    {
        DUMP_MEMORY(&p_Fm->p_FmFpmRegs->fpmdrd[i], sizeof(uint32_t));
    }

    DUMP_SUBTITLE(("\n"));
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmdra);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fm_ip_rev_1);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fm_ip_rev_2);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmrstc);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmcld);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fmnpi);
    DUMP_VAR(p_Fm->p_FmFpmRegs,fpmem);

    DUMP_TITLE(&p_Fm->p_FmFpmRegs->fpmcev, ("fpmcev"));
    DUMP_SUBSTRUCT_ARRAY(i, 4)
    {
        DUMP_MEMORY(&p_Fm->p_FmFpmRegs->fpmcev[i], sizeof(uint32_t));
    }

    DUMP_TITLE(&p_Fm->p_FmFpmRegs->fmfp_ps, ("fmfp_ps"));
    DUMP_SUBSTRUCT_ARRAY(i, 64)
    {
        DUMP_MEMORY(&p_Fm->p_FmFpmRegs->fmfp_ps[i], sizeof(uint32_t));
    }


    DUMP_TITLE(p_Fm->p_FmDmaRegs, ("p_FmDmaRegs Regs"));
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmsr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmmr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmtr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmhy);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmsetr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmtah);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmtal);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmtcid);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmra);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmrd);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmwcr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmebcr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmccqdr);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmccqvr1);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmccqvr2);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmcqvr3);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmcqvr4);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmcqvr5);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmsefrc);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmsqfrc);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmssrc);
    DUMP_VAR(p_Fm->p_FmDmaRegs,fmdmdcr);

    DUMP_TITLE(&p_Fm->p_FmDmaRegs->fmdmplr, ("fmdmplr"));

    DUMP_SUBSTRUCT_ARRAY(i, FM_SIZE_OF_LIODN_TABLE/2)
    {
        DUMP_MEMORY(&p_Fm->p_FmDmaRegs->fmdmplr[i], sizeof(uint32_t));
    }

    DUMP_TITLE(p_Fm->p_FmBmiRegs, ("p_FmBmiRegs COMMON Regs"));
    DUMP_VAR(p_Fm->p_FmBmiRegs,fmbm_init);
    DUMP_VAR(p_Fm->p_FmBmiRegs,fmbm_cfg1);
    DUMP_VAR(p_Fm->p_FmBmiRegs,fmbm_cfg2);
    DUMP_VAR(p_Fm->p_FmBmiRegs,fmbm_ievr);
    DUMP_VAR(p_Fm->p_FmBmiRegs,fmbm_ier);

    DUMP_TITLE(&p_Fm->p_FmBmiRegs->fmbm_arb, ("fmbm_arb"));
    DUMP_SUBSTRUCT_ARRAY(i, 8)
    {
        DUMP_MEMORY(&p_Fm->p_FmBmiRegs->fmbm_arb[i], sizeof(uint32_t));
    }


    DUMP_TITLE(p_Fm->p_FmQmiRegs, ("p_FmQmiRegs COMMON Regs"));
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_gc);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_eie);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_eien);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_eif);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_ie);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_ien);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_if);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_gs);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_ts);
    DUMP_VAR(p_Fm->p_FmQmiRegs,fmqm_etfc);

    return E_OK;
}
#endif /* (defined(DEBUG_ERRORS) && ... */



